"""Integration functions that integrates a sympy expression.

In [3]: integrate(1/x,x)
Out[3]: log(x)
In [4]: integrate(sin(x),x)
Out[4]: -cos(x)
"""
from integrals import integrate, Integral, line_integrate
