"""A module that handles matrices.

Includes functions for fast creating matrices like zero, one/eye, random matrix etc.
"""
from matrices import Matrix, SMatrix, zero, zeronm, zeros, one, ones, eye, \
     hessian, randMatrix, GramSchmidt, wronskian, casoratian, \
     list2numpy, matrix2numpy, DeferredVector
