# ----------------------------------------------------------------------------
# pyglet
# Copyright (c) 2006-2007 Alex Holkner
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions 
# are met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above copyright 
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#  * Neither the name of the pyglet nor the names of its
#    contributors may be used to endorse or promote products
#    derived from this software without specific prior written
#    permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# ----------------------------------------------------------------------------
'''Wrapper for http://oss.sgi.com/projects/ogl-sample/ABI/glext.h

Generated by tools/gengl.py.
Do not modify this file.
'''

__docformat__ = 'restructuredtext'
__version__ = '$Id: glext_arb.py 935 2007-06-25 08:22:58Z Alex.Holkner $'

from ctypes import *
from pyglet.gl.lib import link_GL as _link_function
from pyglet.gl.lib import c_ptrdiff_t

# BEGIN GENERATED CONTENT (do not edit below this line)

# This content is generated by tools/gengl.py.
# Wrapper for http://oss.sgi.com/projects/ogl-sample/ABI/glext.h


# GLEXT_LEGACY (/usr/include/GL/gl.h:70)
GL_GLEXT_VERSION = 29 	# GL/glext.h:57
# VERSION_1_2 (GL/glext.h:59)
GL_UNSIGNED_BYTE_3_3_2 = 32818 	# GL/glext.h:60
GL_UNSIGNED_SHORT_4_4_4_4 = 32819 	# GL/glext.h:61
GL_UNSIGNED_SHORT_5_5_5_1 = 32820 	# GL/glext.h:62
GL_UNSIGNED_INT_8_8_8_8 = 32821 	# GL/glext.h:63
GL_UNSIGNED_INT_10_10_10_2 = 32822 	# GL/glext.h:64
GL_RESCALE_NORMAL = 32826 	# GL/glext.h:65
GL_TEXTURE_BINDING_3D = 32874 	# GL/glext.h:66
GL_PACK_SKIP_IMAGES = 32875 	# GL/glext.h:67
GL_PACK_IMAGE_HEIGHT = 32876 	# GL/glext.h:68
GL_UNPACK_SKIP_IMAGES = 32877 	# GL/glext.h:69
GL_UNPACK_IMAGE_HEIGHT = 32878 	# GL/glext.h:70
GL_TEXTURE_3D = 32879 	# GL/glext.h:71
GL_PROXY_TEXTURE_3D = 32880 	# GL/glext.h:72
GL_TEXTURE_DEPTH = 32881 	# GL/glext.h:73
GL_TEXTURE_WRAP_R = 32882 	# GL/glext.h:74
GL_MAX_3D_TEXTURE_SIZE = 32883 	# GL/glext.h:75
GL_UNSIGNED_BYTE_2_3_3_REV = 33634 	# GL/glext.h:76
GL_UNSIGNED_SHORT_5_6_5 = 33635 	# GL/glext.h:77
GL_UNSIGNED_SHORT_5_6_5_REV = 33636 	# GL/glext.h:78
GL_UNSIGNED_SHORT_4_4_4_4_REV = 33637 	# GL/glext.h:79
GL_UNSIGNED_SHORT_1_5_5_5_REV = 33638 	# GL/glext.h:80
GL_UNSIGNED_INT_8_8_8_8_REV = 33639 	# GL/glext.h:81
GL_UNSIGNED_INT_2_10_10_10_REV = 33640 	# GL/glext.h:82
GL_BGR = 32992 	# GL/glext.h:83
GL_BGRA = 32993 	# GL/glext.h:84
GL_MAX_ELEMENTS_VERTICES = 33000 	# GL/glext.h:85
GL_MAX_ELEMENTS_INDICES = 33001 	# GL/glext.h:86
GL_CLAMP_TO_EDGE = 33071 	# GL/glext.h:87
GL_TEXTURE_MIN_LOD = 33082 	# GL/glext.h:88
GL_TEXTURE_MAX_LOD = 33083 	# GL/glext.h:89
GL_TEXTURE_BASE_LEVEL = 33084 	# GL/glext.h:90
GL_TEXTURE_MAX_LEVEL = 33085 	# GL/glext.h:91
GL_LIGHT_MODEL_COLOR_CONTROL = 33272 	# GL/glext.h:92
GL_SINGLE_COLOR = 33273 	# GL/glext.h:93
GL_SEPARATE_SPECULAR_COLOR = 33274 	# GL/glext.h:94
GL_SMOOTH_POINT_SIZE_RANGE = 2834 	# GL/glext.h:95
GL_SMOOTH_POINT_SIZE_GRANULARITY = 2835 	# GL/glext.h:96
GL_SMOOTH_LINE_WIDTH_RANGE = 2850 	# GL/glext.h:97
GL_SMOOTH_LINE_WIDTH_GRANULARITY = 2851 	# GL/glext.h:98
GL_ALIASED_POINT_SIZE_RANGE = 33901 	# GL/glext.h:99
GL_ALIASED_LINE_WIDTH_RANGE = 33902 	# GL/glext.h:100
# ARB_imaging (GL/glext.h:103)
GL_CONSTANT_COLOR = 32769 	# GL/glext.h:104
GL_ONE_MINUS_CONSTANT_COLOR = 32770 	# GL/glext.h:105
GL_CONSTANT_ALPHA = 32771 	# GL/glext.h:106
GL_ONE_MINUS_CONSTANT_ALPHA = 32772 	# GL/glext.h:107
GL_BLEND_COLOR = 32773 	# GL/glext.h:108
GL_FUNC_ADD = 32774 	# GL/glext.h:109
GL_MIN = 32775 	# GL/glext.h:110
GL_MAX = 32776 	# GL/glext.h:111
GL_BLEND_EQUATION = 32777 	# GL/glext.h:112
GL_FUNC_SUBTRACT = 32778 	# GL/glext.h:113
GL_FUNC_REVERSE_SUBTRACT = 32779 	# GL/glext.h:114
GL_CONVOLUTION_1D = 32784 	# GL/glext.h:115
GL_CONVOLUTION_2D = 32785 	# GL/glext.h:116
GL_SEPARABLE_2D = 32786 	# GL/glext.h:117
GL_CONVOLUTION_BORDER_MODE = 32787 	# GL/glext.h:118
GL_CONVOLUTION_FILTER_SCALE = 32788 	# GL/glext.h:119
GL_CONVOLUTION_FILTER_BIAS = 32789 	# GL/glext.h:120
GL_REDUCE = 32790 	# GL/glext.h:121
GL_CONVOLUTION_FORMAT = 32791 	# GL/glext.h:122
GL_CONVOLUTION_WIDTH = 32792 	# GL/glext.h:123
GL_CONVOLUTION_HEIGHT = 32793 	# GL/glext.h:124
GL_MAX_CONVOLUTION_WIDTH = 32794 	# GL/glext.h:125
GL_MAX_CONVOLUTION_HEIGHT = 32795 	# GL/glext.h:126
GL_POST_CONVOLUTION_RED_SCALE = 32796 	# GL/glext.h:127
GL_POST_CONVOLUTION_GREEN_SCALE = 32797 	# GL/glext.h:128
GL_POST_CONVOLUTION_BLUE_SCALE = 32798 	# GL/glext.h:129
GL_POST_CONVOLUTION_ALPHA_SCALE = 32799 	# GL/glext.h:130
GL_POST_CONVOLUTION_RED_BIAS = 32800 	# GL/glext.h:131
GL_POST_CONVOLUTION_GREEN_BIAS = 32801 	# GL/glext.h:132
GL_POST_CONVOLUTION_BLUE_BIAS = 32802 	# GL/glext.h:133
GL_POST_CONVOLUTION_ALPHA_BIAS = 32803 	# GL/glext.h:134
GL_HISTOGRAM = 32804 	# GL/glext.h:135
GL_PROXY_HISTOGRAM = 32805 	# GL/glext.h:136
GL_HISTOGRAM_WIDTH = 32806 	# GL/glext.h:137
GL_HISTOGRAM_FORMAT = 32807 	# GL/glext.h:138
GL_HISTOGRAM_RED_SIZE = 32808 	# GL/glext.h:139
GL_HISTOGRAM_GREEN_SIZE = 32809 	# GL/glext.h:140
GL_HISTOGRAM_BLUE_SIZE = 32810 	# GL/glext.h:141
GL_HISTOGRAM_ALPHA_SIZE = 32811 	# GL/glext.h:142
GL_HISTOGRAM_LUMINANCE_SIZE = 32812 	# GL/glext.h:143
GL_HISTOGRAM_SINK = 32813 	# GL/glext.h:144
GL_MINMAX = 32814 	# GL/glext.h:145
GL_MINMAX_FORMAT = 32815 	# GL/glext.h:146
GL_MINMAX_SINK = 32816 	# GL/glext.h:147
GL_TABLE_TOO_LARGE = 32817 	# GL/glext.h:148
GL_COLOR_MATRIX = 32945 	# GL/glext.h:149
GL_COLOR_MATRIX_STACK_DEPTH = 32946 	# GL/glext.h:150
GL_MAX_COLOR_MATRIX_STACK_DEPTH = 32947 	# GL/glext.h:151
GL_POST_COLOR_MATRIX_RED_SCALE = 32948 	# GL/glext.h:152
GL_POST_COLOR_MATRIX_GREEN_SCALE = 32949 	# GL/glext.h:153
GL_POST_COLOR_MATRIX_BLUE_SCALE = 32950 	# GL/glext.h:154
GL_POST_COLOR_MATRIX_ALPHA_SCALE = 32951 	# GL/glext.h:155
GL_POST_COLOR_MATRIX_RED_BIAS = 32952 	# GL/glext.h:156
GL_POST_COLOR_MATRIX_GREEN_BIAS = 32953 	# GL/glext.h:157
GL_POST_COLOR_MATRIX_BLUE_BIAS = 32954 	# GL/glext.h:158
GL_POST_COLOR_MATRIX_ALPHA_BIAS = 32955 	# GL/glext.h:159
GL_COLOR_TABLE = 32976 	# GL/glext.h:160
GL_POST_CONVOLUTION_COLOR_TABLE = 32977 	# GL/glext.h:161
GL_POST_COLOR_MATRIX_COLOR_TABLE = 32978 	# GL/glext.h:162
GL_PROXY_COLOR_TABLE = 32979 	# GL/glext.h:163
GL_PROXY_POST_CONVOLUTION_COLOR_TABLE = 32980 	# GL/glext.h:164
GL_PROXY_POST_COLOR_MATRIX_COLOR_TABLE = 32981 	# GL/glext.h:165
GL_COLOR_TABLE_SCALE = 32982 	# GL/glext.h:166
GL_COLOR_TABLE_BIAS = 32983 	# GL/glext.h:167
GL_COLOR_TABLE_FORMAT = 32984 	# GL/glext.h:168
GL_COLOR_TABLE_WIDTH = 32985 	# GL/glext.h:169
GL_COLOR_TABLE_RED_SIZE = 32986 	# GL/glext.h:170
GL_COLOR_TABLE_GREEN_SIZE = 32987 	# GL/glext.h:171
GL_COLOR_TABLE_BLUE_SIZE = 32988 	# GL/glext.h:172
GL_COLOR_TABLE_ALPHA_SIZE = 32989 	# GL/glext.h:173
GL_COLOR_TABLE_LUMINANCE_SIZE = 32990 	# GL/glext.h:174
GL_COLOR_TABLE_INTENSITY_SIZE = 32991 	# GL/glext.h:175
GL_CONSTANT_BORDER = 33105 	# GL/glext.h:176
GL_REPLICATE_BORDER = 33107 	# GL/glext.h:177
GL_CONVOLUTION_BORDER_COLOR = 33108 	# GL/glext.h:178
# VERSION_1_3 (GL/glext.h:181)
GL_TEXTURE0 = 33984 	# GL/glext.h:182
GL_TEXTURE1 = 33985 	# GL/glext.h:183
GL_TEXTURE2 = 33986 	# GL/glext.h:184
GL_TEXTURE3 = 33987 	# GL/glext.h:185
GL_TEXTURE4 = 33988 	# GL/glext.h:186
GL_TEXTURE5 = 33989 	# GL/glext.h:187
GL_TEXTURE6 = 33990 	# GL/glext.h:188
GL_TEXTURE7 = 33991 	# GL/glext.h:189
GL_TEXTURE8 = 33992 	# GL/glext.h:190
GL_TEXTURE9 = 33993 	# GL/glext.h:191
GL_TEXTURE10 = 33994 	# GL/glext.h:192
GL_TEXTURE11 = 33995 	# GL/glext.h:193
GL_TEXTURE12 = 33996 	# GL/glext.h:194
GL_TEXTURE13 = 33997 	# GL/glext.h:195
GL_TEXTURE14 = 33998 	# GL/glext.h:196
GL_TEXTURE15 = 33999 	# GL/glext.h:197
GL_TEXTURE16 = 34000 	# GL/glext.h:198
GL_TEXTURE17 = 34001 	# GL/glext.h:199
GL_TEXTURE18 = 34002 	# GL/glext.h:200
GL_TEXTURE19 = 34003 	# GL/glext.h:201
GL_TEXTURE20 = 34004 	# GL/glext.h:202
GL_TEXTURE21 = 34005 	# GL/glext.h:203
GL_TEXTURE22 = 34006 	# GL/glext.h:204
GL_TEXTURE23 = 34007 	# GL/glext.h:205
GL_TEXTURE24 = 34008 	# GL/glext.h:206
GL_TEXTURE25 = 34009 	# GL/glext.h:207
GL_TEXTURE26 = 34010 	# GL/glext.h:208
GL_TEXTURE27 = 34011 	# GL/glext.h:209
GL_TEXTURE28 = 34012 	# GL/glext.h:210
GL_TEXTURE29 = 34013 	# GL/glext.h:211
GL_TEXTURE30 = 34014 	# GL/glext.h:212
GL_TEXTURE31 = 34015 	# GL/glext.h:213
GL_ACTIVE_TEXTURE = 34016 	# GL/glext.h:214
GL_CLIENT_ACTIVE_TEXTURE = 34017 	# GL/glext.h:215
GL_MAX_TEXTURE_UNITS = 34018 	# GL/glext.h:216
GL_TRANSPOSE_MODELVIEW_MATRIX = 34019 	# GL/glext.h:217
GL_TRANSPOSE_PROJECTION_MATRIX = 34020 	# GL/glext.h:218
GL_TRANSPOSE_TEXTURE_MATRIX = 34021 	# GL/glext.h:219
GL_TRANSPOSE_COLOR_MATRIX = 34022 	# GL/glext.h:220
GL_MULTISAMPLE = 32925 	# GL/glext.h:221
GL_SAMPLE_ALPHA_TO_COVERAGE = 32926 	# GL/glext.h:222
GL_SAMPLE_ALPHA_TO_ONE = 32927 	# GL/glext.h:223
GL_SAMPLE_COVERAGE = 32928 	# GL/glext.h:224
GL_SAMPLE_BUFFERS = 32936 	# GL/glext.h:225
GL_SAMPLES = 32937 	# GL/glext.h:226
GL_SAMPLE_COVERAGE_VALUE = 32938 	# GL/glext.h:227
GL_SAMPLE_COVERAGE_INVERT = 32939 	# GL/glext.h:228
GL_MULTISAMPLE_BIT = 536870912 	# GL/glext.h:229
GL_NORMAL_MAP = 34065 	# GL/glext.h:230
GL_REFLECTION_MAP = 34066 	# GL/glext.h:231
GL_TEXTURE_CUBE_MAP = 34067 	# GL/glext.h:232
GL_TEXTURE_BINDING_CUBE_MAP = 34068 	# GL/glext.h:233
GL_TEXTURE_CUBE_MAP_POSITIVE_X = 34069 	# GL/glext.h:234
GL_TEXTURE_CUBE_MAP_NEGATIVE_X = 34070 	# GL/glext.h:235
GL_TEXTURE_CUBE_MAP_POSITIVE_Y = 34071 	# GL/glext.h:236
GL_TEXTURE_CUBE_MAP_NEGATIVE_Y = 34072 	# GL/glext.h:237
GL_TEXTURE_CUBE_MAP_POSITIVE_Z = 34073 	# GL/glext.h:238
GL_TEXTURE_CUBE_MAP_NEGATIVE_Z = 34074 	# GL/glext.h:239
GL_PROXY_TEXTURE_CUBE_MAP = 34075 	# GL/glext.h:240
GL_MAX_CUBE_MAP_TEXTURE_SIZE = 34076 	# GL/glext.h:241
GL_COMPRESSED_ALPHA = 34025 	# GL/glext.h:242
GL_COMPRESSED_LUMINANCE = 34026 	# GL/glext.h:243
GL_COMPRESSED_LUMINANCE_ALPHA = 34027 	# GL/glext.h:244
GL_COMPRESSED_INTENSITY = 34028 	# GL/glext.h:245
GL_COMPRESSED_RGB = 34029 	# GL/glext.h:246
GL_COMPRESSED_RGBA = 34030 	# GL/glext.h:247
GL_TEXTURE_COMPRESSION_HINT = 34031 	# GL/glext.h:248
GL_TEXTURE_COMPRESSED_IMAGE_SIZE = 34464 	# GL/glext.h:249
GL_TEXTURE_COMPRESSED = 34465 	# GL/glext.h:250
GL_NUM_COMPRESSED_TEXTURE_FORMATS = 34466 	# GL/glext.h:251
GL_COMPRESSED_TEXTURE_FORMATS = 34467 	# GL/glext.h:252
GL_CLAMP_TO_BORDER = 33069 	# GL/glext.h:253
GL_COMBINE = 34160 	# GL/glext.h:254
GL_COMBINE_RGB = 34161 	# GL/glext.h:255
GL_COMBINE_ALPHA = 34162 	# GL/glext.h:256
GL_SOURCE0_RGB = 34176 	# GL/glext.h:257
GL_SOURCE1_RGB = 34177 	# GL/glext.h:258
GL_SOURCE2_RGB = 34178 	# GL/glext.h:259
GL_SOURCE0_ALPHA = 34184 	# GL/glext.h:260
GL_SOURCE1_ALPHA = 34185 	# GL/glext.h:261
GL_SOURCE2_ALPHA = 34186 	# GL/glext.h:262
GL_OPERAND0_RGB = 34192 	# GL/glext.h:263
GL_OPERAND1_RGB = 34193 	# GL/glext.h:264
GL_OPERAND2_RGB = 34194 	# GL/glext.h:265
GL_OPERAND0_ALPHA = 34200 	# GL/glext.h:266
GL_OPERAND1_ALPHA = 34201 	# GL/glext.h:267
GL_OPERAND2_ALPHA = 34202 	# GL/glext.h:268
GL_RGB_SCALE = 34163 	# GL/glext.h:269
GL_ADD_SIGNED = 34164 	# GL/glext.h:270
GL_INTERPOLATE = 34165 	# GL/glext.h:271
GL_SUBTRACT = 34023 	# GL/glext.h:272
GL_CONSTANT = 34166 	# GL/glext.h:273
GL_PRIMARY_COLOR = 34167 	# GL/glext.h:274
GL_PREVIOUS = 34168 	# GL/glext.h:275
GL_DOT3_RGB = 34478 	# GL/glext.h:276
GL_DOT3_RGBA = 34479 	# GL/glext.h:277
# VERSION_1_4 (GL/glext.h:280)
GL_BLEND_DST_RGB = 32968 	# GL/glext.h:281
GL_BLEND_SRC_RGB = 32969 	# GL/glext.h:282
GL_BLEND_DST_ALPHA = 32970 	# GL/glext.h:283
GL_BLEND_SRC_ALPHA = 32971 	# GL/glext.h:284
GL_POINT_SIZE_MIN = 33062 	# GL/glext.h:285
GL_POINT_SIZE_MAX = 33063 	# GL/glext.h:286
GL_POINT_FADE_THRESHOLD_SIZE = 33064 	# GL/glext.h:287
GL_POINT_DISTANCE_ATTENUATION = 33065 	# GL/glext.h:288
GL_GENERATE_MIPMAP = 33169 	# GL/glext.h:289
GL_GENERATE_MIPMAP_HINT = 33170 	# GL/glext.h:290
GL_DEPTH_COMPONENT16 = 33189 	# GL/glext.h:291
GL_DEPTH_COMPONENT24 = 33190 	# GL/glext.h:292
GL_DEPTH_COMPONENT32 = 33191 	# GL/glext.h:293
GL_MIRRORED_REPEAT = 33648 	# GL/glext.h:294
GL_FOG_COORDINATE_SOURCE = 33872 	# GL/glext.h:295
GL_FOG_COORDINATE = 33873 	# GL/glext.h:296
GL_FRAGMENT_DEPTH = 33874 	# GL/glext.h:297
GL_CURRENT_FOG_COORDINATE = 33875 	# GL/glext.h:298
GL_FOG_COORDINATE_ARRAY_TYPE = 33876 	# GL/glext.h:299
GL_FOG_COORDINATE_ARRAY_STRIDE = 33877 	# GL/glext.h:300
GL_FOG_COORDINATE_ARRAY_POINTER = 33878 	# GL/glext.h:301
GL_FOG_COORDINATE_ARRAY = 33879 	# GL/glext.h:302
GL_COLOR_SUM = 33880 	# GL/glext.h:303
GL_CURRENT_SECONDARY_COLOR = 33881 	# GL/glext.h:304
GL_SECONDARY_COLOR_ARRAY_SIZE = 33882 	# GL/glext.h:305
GL_SECONDARY_COLOR_ARRAY_TYPE = 33883 	# GL/glext.h:306
GL_SECONDARY_COLOR_ARRAY_STRIDE = 33884 	# GL/glext.h:307
GL_SECONDARY_COLOR_ARRAY_POINTER = 33885 	# GL/glext.h:308
GL_SECONDARY_COLOR_ARRAY = 33886 	# GL/glext.h:309
GL_MAX_TEXTURE_LOD_BIAS = 34045 	# GL/glext.h:310
GL_TEXTURE_FILTER_CONTROL = 34048 	# GL/glext.h:311
GL_TEXTURE_LOD_BIAS = 34049 	# GL/glext.h:312
GL_INCR_WRAP = 34055 	# GL/glext.h:313
GL_DECR_WRAP = 34056 	# GL/glext.h:314
GL_TEXTURE_DEPTH_SIZE = 34890 	# GL/glext.h:315
GL_DEPTH_TEXTURE_MODE = 34891 	# GL/glext.h:316
GL_TEXTURE_COMPARE_MODE = 34892 	# GL/glext.h:317
GL_TEXTURE_COMPARE_FUNC = 34893 	# GL/glext.h:318
GL_COMPARE_R_TO_TEXTURE = 34894 	# GL/glext.h:319
# VERSION_1_5 (GL/glext.h:322)
GL_BUFFER_SIZE = 34660 	# GL/glext.h:323
GL_BUFFER_USAGE = 34661 	# GL/glext.h:324
GL_QUERY_COUNTER_BITS = 34916 	# GL/glext.h:325
GL_CURRENT_QUERY = 34917 	# GL/glext.h:326
GL_QUERY_RESULT = 34918 	# GL/glext.h:327
GL_QUERY_RESULT_AVAILABLE = 34919 	# GL/glext.h:328
GL_ARRAY_BUFFER = 34962 	# GL/glext.h:329
GL_ELEMENT_ARRAY_BUFFER = 34963 	# GL/glext.h:330
GL_ARRAY_BUFFER_BINDING = 34964 	# GL/glext.h:331
GL_ELEMENT_ARRAY_BUFFER_BINDING = 34965 	# GL/glext.h:332
GL_VERTEX_ARRAY_BUFFER_BINDING = 34966 	# GL/glext.h:333
GL_NORMAL_ARRAY_BUFFER_BINDING = 34967 	# GL/glext.h:334
GL_COLOR_ARRAY_BUFFER_BINDING = 34968 	# GL/glext.h:335
GL_INDEX_ARRAY_BUFFER_BINDING = 34969 	# GL/glext.h:336
GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING = 34970 	# GL/glext.h:337
GL_EDGE_FLAG_ARRAY_BUFFER_BINDING = 34971 	# GL/glext.h:338
GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING = 34972 	# GL/glext.h:339
GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING = 34973 	# GL/glext.h:340
GL_WEIGHT_ARRAY_BUFFER_BINDING = 34974 	# GL/glext.h:341
GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING = 34975 	# GL/glext.h:342
GL_READ_ONLY = 35000 	# GL/glext.h:343
GL_WRITE_ONLY = 35001 	# GL/glext.h:344
GL_READ_WRITE = 35002 	# GL/glext.h:345
GL_BUFFER_ACCESS = 35003 	# GL/glext.h:346
GL_BUFFER_MAPPED = 35004 	# GL/glext.h:347
GL_BUFFER_MAP_POINTER = 35005 	# GL/glext.h:348
GL_STREAM_DRAW = 35040 	# GL/glext.h:349
GL_STREAM_READ = 35041 	# GL/glext.h:350
GL_STREAM_COPY = 35042 	# GL/glext.h:351
GL_STATIC_DRAW = 35044 	# GL/glext.h:352
GL_STATIC_READ = 35045 	# GL/glext.h:353
GL_STATIC_COPY = 35046 	# GL/glext.h:354
GL_DYNAMIC_DRAW = 35048 	# GL/glext.h:355
GL_DYNAMIC_READ = 35049 	# GL/glext.h:356
GL_DYNAMIC_COPY = 35050 	# GL/glext.h:357
GL_SAMPLES_PASSED = 35092 	# GL/glext.h:358
GL_FOG_COORD_SRC = 33872 	# GL/glext.h:359
GL_FOG_COORD = 33873 	# GL/glext.h:360
GL_CURRENT_FOG_COORD = 33875 	# GL/glext.h:361
GL_FOG_COORD_ARRAY_TYPE = 33876 	# GL/glext.h:362
GL_FOG_COORD_ARRAY_STRIDE = 33877 	# GL/glext.h:363
GL_FOG_COORD_ARRAY_POINTER = 33878 	# GL/glext.h:364
GL_FOG_COORD_ARRAY = 33879 	# GL/glext.h:365
GL_FOG_COORD_ARRAY_BUFFER_BINDING = 34973 	# GL/glext.h:366
GL_SRC0_RGB = 34176 	# GL/glext.h:367
GL_SRC1_RGB = 34177 	# GL/glext.h:368
GL_SRC2_RGB = 34178 	# GL/glext.h:369
GL_SRC0_ALPHA = 34184 	# GL/glext.h:370
GL_SRC1_ALPHA = 34185 	# GL/glext.h:371
GL_SRC2_ALPHA = 34186 	# GL/glext.h:372
# VERSION_2_0 (GL/glext.h:375)
GL_BLEND_EQUATION_RGB = 32777 	# GL/glext.h:376
GL_VERTEX_ATTRIB_ARRAY_ENABLED = 34338 	# GL/glext.h:377
GL_VERTEX_ATTRIB_ARRAY_SIZE = 34339 	# GL/glext.h:378
GL_VERTEX_ATTRIB_ARRAY_STRIDE = 34340 	# GL/glext.h:379
GL_VERTEX_ATTRIB_ARRAY_TYPE = 34341 	# GL/glext.h:380
GL_CURRENT_VERTEX_ATTRIB = 34342 	# GL/glext.h:381
GL_VERTEX_PROGRAM_POINT_SIZE = 34370 	# GL/glext.h:382
GL_VERTEX_PROGRAM_TWO_SIDE = 34371 	# GL/glext.h:383
GL_VERTEX_ATTRIB_ARRAY_POINTER = 34373 	# GL/glext.h:384
GL_STENCIL_BACK_FUNC = 34816 	# GL/glext.h:385
GL_STENCIL_BACK_FAIL = 34817 	# GL/glext.h:386
GL_STENCIL_BACK_PASS_DEPTH_FAIL = 34818 	# GL/glext.h:387
GL_STENCIL_BACK_PASS_DEPTH_PASS = 34819 	# GL/glext.h:388
GL_MAX_DRAW_BUFFERS = 34852 	# GL/glext.h:389
GL_DRAW_BUFFER0 = 34853 	# GL/glext.h:390
GL_DRAW_BUFFER1 = 34854 	# GL/glext.h:391
GL_DRAW_BUFFER2 = 34855 	# GL/glext.h:392
GL_DRAW_BUFFER3 = 34856 	# GL/glext.h:393
GL_DRAW_BUFFER4 = 34857 	# GL/glext.h:394
GL_DRAW_BUFFER5 = 34858 	# GL/glext.h:395
GL_DRAW_BUFFER6 = 34859 	# GL/glext.h:396
GL_DRAW_BUFFER7 = 34860 	# GL/glext.h:397
GL_DRAW_BUFFER8 = 34861 	# GL/glext.h:398
GL_DRAW_BUFFER9 = 34862 	# GL/glext.h:399
GL_DRAW_BUFFER10 = 34863 	# GL/glext.h:400
GL_DRAW_BUFFER11 = 34864 	# GL/glext.h:401
GL_DRAW_BUFFER12 = 34865 	# GL/glext.h:402
GL_DRAW_BUFFER13 = 34866 	# GL/glext.h:403
GL_DRAW_BUFFER14 = 34867 	# GL/glext.h:404
GL_DRAW_BUFFER15 = 34868 	# GL/glext.h:405
GL_BLEND_EQUATION_ALPHA = 34877 	# GL/glext.h:406
GL_POINT_SPRITE = 34913 	# GL/glext.h:407
GL_COORD_REPLACE = 34914 	# GL/glext.h:408
GL_MAX_VERTEX_ATTRIBS = 34921 	# GL/glext.h:409
GL_VERTEX_ATTRIB_ARRAY_NORMALIZED = 34922 	# GL/glext.h:410
GL_MAX_TEXTURE_COORDS = 34929 	# GL/glext.h:411
GL_MAX_TEXTURE_IMAGE_UNITS = 34930 	# GL/glext.h:412
GL_FRAGMENT_SHADER = 35632 	# GL/glext.h:413
GL_VERTEX_SHADER = 35633 	# GL/glext.h:414
GL_MAX_FRAGMENT_UNIFORM_COMPONENTS = 35657 	# GL/glext.h:415
GL_MAX_VERTEX_UNIFORM_COMPONENTS = 35658 	# GL/glext.h:416
GL_MAX_VARYING_FLOATS = 35659 	# GL/glext.h:417
GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS = 35660 	# GL/glext.h:418
GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS = 35661 	# GL/glext.h:419
GL_SHADER_TYPE = 35663 	# GL/glext.h:420
GL_FLOAT_VEC2 = 35664 	# GL/glext.h:421
GL_FLOAT_VEC3 = 35665 	# GL/glext.h:422
GL_FLOAT_VEC4 = 35666 	# GL/glext.h:423
GL_INT_VEC2 = 35667 	# GL/glext.h:424
GL_INT_VEC3 = 35668 	# GL/glext.h:425
GL_INT_VEC4 = 35669 	# GL/glext.h:426
GL_BOOL = 35670 	# GL/glext.h:427
GL_BOOL_VEC2 = 35671 	# GL/glext.h:428
GL_BOOL_VEC3 = 35672 	# GL/glext.h:429
GL_BOOL_VEC4 = 35673 	# GL/glext.h:430
GL_FLOAT_MAT2 = 35674 	# GL/glext.h:431
GL_FLOAT_MAT3 = 35675 	# GL/glext.h:432
GL_FLOAT_MAT4 = 35676 	# GL/glext.h:433
GL_SAMPLER_1D = 35677 	# GL/glext.h:434
GL_SAMPLER_2D = 35678 	# GL/glext.h:435
GL_SAMPLER_3D = 35679 	# GL/glext.h:436
GL_SAMPLER_CUBE = 35680 	# GL/glext.h:437
GL_SAMPLER_1D_SHADOW = 35681 	# GL/glext.h:438
GL_SAMPLER_2D_SHADOW = 35682 	# GL/glext.h:439
GL_DELETE_STATUS = 35712 	# GL/glext.h:440
GL_COMPILE_STATUS = 35713 	# GL/glext.h:441
GL_LINK_STATUS = 35714 	# GL/glext.h:442
GL_VALIDATE_STATUS = 35715 	# GL/glext.h:443
GL_INFO_LOG_LENGTH = 35716 	# GL/glext.h:444
GL_ATTACHED_SHADERS = 35717 	# GL/glext.h:445
GL_ACTIVE_UNIFORMS = 35718 	# GL/glext.h:446
GL_ACTIVE_UNIFORM_MAX_LENGTH = 35719 	# GL/glext.h:447
GL_SHADER_SOURCE_LENGTH = 35720 	# GL/glext.h:448
GL_ACTIVE_ATTRIBUTES = 35721 	# GL/glext.h:449
GL_ACTIVE_ATTRIBUTE_MAX_LENGTH = 35722 	# GL/glext.h:450
GL_FRAGMENT_SHADER_DERIVATIVE_HINT = 35723 	# GL/glext.h:451
GL_SHADING_LANGUAGE_VERSION = 35724 	# GL/glext.h:452
GL_CURRENT_PROGRAM = 35725 	# GL/glext.h:453
GL_POINT_SPRITE_COORD_ORIGIN = 36000 	# GL/glext.h:454
GL_LOWER_LEFT = 36001 	# GL/glext.h:455
GL_UPPER_LEFT = 36002 	# GL/glext.h:456
GL_STENCIL_BACK_REF = 36003 	# GL/glext.h:457
GL_STENCIL_BACK_VALUE_MASK = 36004 	# GL/glext.h:458
GL_STENCIL_BACK_WRITEMASK = 36005 	# GL/glext.h:459
# ARB_multitexture (GL/glext.h:462)
GL_TEXTURE0_ARB = 33984 	# GL/glext.h:463
GL_TEXTURE1_ARB = 33985 	# GL/glext.h:464
GL_TEXTURE2_ARB = 33986 	# GL/glext.h:465
GL_TEXTURE3_ARB = 33987 	# GL/glext.h:466
GL_TEXTURE4_ARB = 33988 	# GL/glext.h:467
GL_TEXTURE5_ARB = 33989 	# GL/glext.h:468
GL_TEXTURE6_ARB = 33990 	# GL/glext.h:469
GL_TEXTURE7_ARB = 33991 	# GL/glext.h:470
GL_TEXTURE8_ARB = 33992 	# GL/glext.h:471
GL_TEXTURE9_ARB = 33993 	# GL/glext.h:472
GL_TEXTURE10_ARB = 33994 	# GL/glext.h:473
GL_TEXTURE11_ARB = 33995 	# GL/glext.h:474
GL_TEXTURE12_ARB = 33996 	# GL/glext.h:475
GL_TEXTURE13_ARB = 33997 	# GL/glext.h:476
GL_TEXTURE14_ARB = 33998 	# GL/glext.h:477
GL_TEXTURE15_ARB = 33999 	# GL/glext.h:478
GL_TEXTURE16_ARB = 34000 	# GL/glext.h:479
GL_TEXTURE17_ARB = 34001 	# GL/glext.h:480
GL_TEXTURE18_ARB = 34002 	# GL/glext.h:481
GL_TEXTURE19_ARB = 34003 	# GL/glext.h:482
GL_TEXTURE20_ARB = 34004 	# GL/glext.h:483
GL_TEXTURE21_ARB = 34005 	# GL/glext.h:484
GL_TEXTURE22_ARB = 34006 	# GL/glext.h:485
GL_TEXTURE23_ARB = 34007 	# GL/glext.h:486
GL_TEXTURE24_ARB = 34008 	# GL/glext.h:487
GL_TEXTURE25_ARB = 34009 	# GL/glext.h:488
GL_TEXTURE26_ARB = 34010 	# GL/glext.h:489
GL_TEXTURE27_ARB = 34011 	# GL/glext.h:490
GL_TEXTURE28_ARB = 34012 	# GL/glext.h:491
GL_TEXTURE29_ARB = 34013 	# GL/glext.h:492
GL_TEXTURE30_ARB = 34014 	# GL/glext.h:493
GL_TEXTURE31_ARB = 34015 	# GL/glext.h:494
GL_ACTIVE_TEXTURE_ARB = 34016 	# GL/glext.h:495
GL_CLIENT_ACTIVE_TEXTURE_ARB = 34017 	# GL/glext.h:496
GL_MAX_TEXTURE_UNITS_ARB = 34018 	# GL/glext.h:497
# ARB_transpose_matrix (GL/glext.h:500)
GL_TRANSPOSE_MODELVIEW_MATRIX_ARB = 34019 	# GL/glext.h:501
GL_TRANSPOSE_PROJECTION_MATRIX_ARB = 34020 	# GL/glext.h:502
GL_TRANSPOSE_TEXTURE_MATRIX_ARB = 34021 	# GL/glext.h:503
GL_TRANSPOSE_COLOR_MATRIX_ARB = 34022 	# GL/glext.h:504
# ARB_multisample (GL/glext.h:507)
GL_MULTISAMPLE_ARB = 32925 	# GL/glext.h:508
GL_SAMPLE_ALPHA_TO_COVERAGE_ARB = 32926 	# GL/glext.h:509
GL_SAMPLE_ALPHA_TO_ONE_ARB = 32927 	# GL/glext.h:510
GL_SAMPLE_COVERAGE_ARB = 32928 	# GL/glext.h:511
GL_SAMPLE_BUFFERS_ARB = 32936 	# GL/glext.h:512
GL_SAMPLES_ARB = 32937 	# GL/glext.h:513
GL_SAMPLE_COVERAGE_VALUE_ARB = 32938 	# GL/glext.h:514
GL_SAMPLE_COVERAGE_INVERT_ARB = 32939 	# GL/glext.h:515
GL_MULTISAMPLE_BIT_ARB = 536870912 	# GL/glext.h:516
# ARB_texture_env_add (GL/glext.h:519)
# ARB_texture_cube_map (GL/glext.h:522)
GL_NORMAL_MAP_ARB = 34065 	# GL/glext.h:523
GL_REFLECTION_MAP_ARB = 34066 	# GL/glext.h:524
GL_TEXTURE_CUBE_MAP_ARB = 34067 	# GL/glext.h:525
GL_TEXTURE_BINDING_CUBE_MAP_ARB = 34068 	# GL/glext.h:526
GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB = 34069 	# GL/glext.h:527
GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB = 34070 	# GL/glext.h:528
GL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB = 34071 	# GL/glext.h:529
GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB = 34072 	# GL/glext.h:530
GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB = 34073 	# GL/glext.h:531
GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB = 34074 	# GL/glext.h:532
GL_PROXY_TEXTURE_CUBE_MAP_ARB = 34075 	# GL/glext.h:533
GL_MAX_CUBE_MAP_TEXTURE_SIZE_ARB = 34076 	# GL/glext.h:534
# ARB_texture_compression (GL/glext.h:537)
GL_COMPRESSED_ALPHA_ARB = 34025 	# GL/glext.h:538
GL_COMPRESSED_LUMINANCE_ARB = 34026 	# GL/glext.h:539
GL_COMPRESSED_LUMINANCE_ALPHA_ARB = 34027 	# GL/glext.h:540
GL_COMPRESSED_INTENSITY_ARB = 34028 	# GL/glext.h:541
GL_COMPRESSED_RGB_ARB = 34029 	# GL/glext.h:542
GL_COMPRESSED_RGBA_ARB = 34030 	# GL/glext.h:543
GL_TEXTURE_COMPRESSION_HINT_ARB = 34031 	# GL/glext.h:544
GL_TEXTURE_COMPRESSED_IMAGE_SIZE_ARB = 34464 	# GL/glext.h:545
GL_TEXTURE_COMPRESSED_ARB = 34465 	# GL/glext.h:546
GL_NUM_COMPRESSED_TEXTURE_FORMATS_ARB = 34466 	# GL/glext.h:547
GL_COMPRESSED_TEXTURE_FORMATS_ARB = 34467 	# GL/glext.h:548
# ARB_texture_border_clamp (GL/glext.h:551)
GL_CLAMP_TO_BORDER_ARB = 33069 	# GL/glext.h:552
# ARB_point_parameters (GL/glext.h:555)
GL_POINT_SIZE_MIN_ARB = 33062 	# GL/glext.h:556
GL_POINT_SIZE_MAX_ARB = 33063 	# GL/glext.h:557
GL_POINT_FADE_THRESHOLD_SIZE_ARB = 33064 	# GL/glext.h:558
GL_POINT_DISTANCE_ATTENUATION_ARB = 33065 	# GL/glext.h:559
# ARB_vertex_blend (GL/glext.h:562)
GL_MAX_VERTEX_UNITS_ARB = 34468 	# GL/glext.h:563
GL_ACTIVE_VERTEX_UNITS_ARB = 34469 	# GL/glext.h:564
GL_WEIGHT_SUM_UNITY_ARB = 34470 	# GL/glext.h:565
GL_VERTEX_BLEND_ARB = 34471 	# GL/glext.h:566
GL_CURRENT_WEIGHT_ARB = 34472 	# GL/glext.h:567
GL_WEIGHT_ARRAY_TYPE_ARB = 34473 	# GL/glext.h:568
GL_WEIGHT_ARRAY_STRIDE_ARB = 34474 	# GL/glext.h:569
GL_WEIGHT_ARRAY_SIZE_ARB = 34475 	# GL/glext.h:570
GL_WEIGHT_ARRAY_POINTER_ARB = 34476 	# GL/glext.h:571
GL_WEIGHT_ARRAY_ARB = 34477 	# GL/glext.h:572
GL_MODELVIEW0_ARB = 5888 	# GL/glext.h:573
GL_MODELVIEW1_ARB = 34058 	# GL/glext.h:574
GL_MODELVIEW2_ARB = 34594 	# GL/glext.h:575
GL_MODELVIEW3_ARB = 34595 	# GL/glext.h:576
GL_MODELVIEW4_ARB = 34596 	# GL/glext.h:577
GL_MODELVIEW5_ARB = 34597 	# GL/glext.h:578
GL_MODELVIEW6_ARB = 34598 	# GL/glext.h:579
GL_MODELVIEW7_ARB = 34599 	# GL/glext.h:580
GL_MODELVIEW8_ARB = 34600 	# GL/glext.h:581
GL_MODELVIEW9_ARB = 34601 	# GL/glext.h:582
GL_MODELVIEW10_ARB = 34602 	# GL/glext.h:583
GL_MODELVIEW11_ARB = 34603 	# GL/glext.h:584
GL_MODELVIEW12_ARB = 34604 	# GL/glext.h:585
GL_MODELVIEW13_ARB = 34605 	# GL/glext.h:586
GL_MODELVIEW14_ARB = 34606 	# GL/glext.h:587
GL_MODELVIEW15_ARB = 34607 	# GL/glext.h:588
GL_MODELVIEW16_ARB = 34608 	# GL/glext.h:589
GL_MODELVIEW17_ARB = 34609 	# GL/glext.h:590
GL_MODELVIEW18_ARB = 34610 	# GL/glext.h:591
GL_MODELVIEW19_ARB = 34611 	# GL/glext.h:592
GL_MODELVIEW20_ARB = 34612 	# GL/glext.h:593
GL_MODELVIEW21_ARB = 34613 	# GL/glext.h:594
GL_MODELVIEW22_ARB = 34614 	# GL/glext.h:595
GL_MODELVIEW23_ARB = 34615 	# GL/glext.h:596
GL_MODELVIEW24_ARB = 34616 	# GL/glext.h:597
GL_MODELVIEW25_ARB = 34617 	# GL/glext.h:598
GL_MODELVIEW26_ARB = 34618 	# GL/glext.h:599
GL_MODELVIEW27_ARB = 34619 	# GL/glext.h:600
GL_MODELVIEW28_ARB = 34620 	# GL/glext.h:601
GL_MODELVIEW29_ARB = 34621 	# GL/glext.h:602
GL_MODELVIEW30_ARB = 34622 	# GL/glext.h:603
GL_MODELVIEW31_ARB = 34623 	# GL/glext.h:604
# ARB_matrix_palette (GL/glext.h:607)
GL_MATRIX_PALETTE_ARB = 34880 	# GL/glext.h:608
GL_MAX_MATRIX_PALETTE_STACK_DEPTH_ARB = 34881 	# GL/glext.h:609
GL_MAX_PALETTE_MATRICES_ARB = 34882 	# GL/glext.h:610
GL_CURRENT_PALETTE_MATRIX_ARB = 34883 	# GL/glext.h:611
GL_MATRIX_INDEX_ARRAY_ARB = 34884 	# GL/glext.h:612
GL_CURRENT_MATRIX_INDEX_ARB = 34885 	# GL/glext.h:613
GL_MATRIX_INDEX_ARRAY_SIZE_ARB = 34886 	# GL/glext.h:614
GL_MATRIX_INDEX_ARRAY_TYPE_ARB = 34887 	# GL/glext.h:615
GL_MATRIX_INDEX_ARRAY_STRIDE_ARB = 34888 	# GL/glext.h:616
GL_MATRIX_INDEX_ARRAY_POINTER_ARB = 34889 	# GL/glext.h:617
# ARB_texture_env_combine (GL/glext.h:620)
GL_COMBINE_ARB = 34160 	# GL/glext.h:621
GL_COMBINE_RGB_ARB = 34161 	# GL/glext.h:622
GL_COMBINE_ALPHA_ARB = 34162 	# GL/glext.h:623
GL_SOURCE0_RGB_ARB = 34176 	# GL/glext.h:624
GL_SOURCE1_RGB_ARB = 34177 	# GL/glext.h:625
GL_SOURCE2_RGB_ARB = 34178 	# GL/glext.h:626
GL_SOURCE0_ALPHA_ARB = 34184 	# GL/glext.h:627
GL_SOURCE1_ALPHA_ARB = 34185 	# GL/glext.h:628
GL_SOURCE2_ALPHA_ARB = 34186 	# GL/glext.h:629
GL_OPERAND0_RGB_ARB = 34192 	# GL/glext.h:630
GL_OPERAND1_RGB_ARB = 34193 	# GL/glext.h:631
GL_OPERAND2_RGB_ARB = 34194 	# GL/glext.h:632
GL_OPERAND0_ALPHA_ARB = 34200 	# GL/glext.h:633
GL_OPERAND1_ALPHA_ARB = 34201 	# GL/glext.h:634
GL_OPERAND2_ALPHA_ARB = 34202 	# GL/glext.h:635
GL_RGB_SCALE_ARB = 34163 	# GL/glext.h:636
GL_ADD_SIGNED_ARB = 34164 	# GL/glext.h:637
GL_INTERPOLATE_ARB = 34165 	# GL/glext.h:638
GL_SUBTRACT_ARB = 34023 	# GL/glext.h:639
GL_CONSTANT_ARB = 34166 	# GL/glext.h:640
GL_PRIMARY_COLOR_ARB = 34167 	# GL/glext.h:641
GL_PREVIOUS_ARB = 34168 	# GL/glext.h:642
# ARB_texture_env_crossbar (GL/glext.h:645)
# ARB_texture_env_dot3 (GL/glext.h:648)
GL_DOT3_RGB_ARB = 34478 	# GL/glext.h:649
GL_DOT3_RGBA_ARB = 34479 	# GL/glext.h:650
# ARB_texture_mirrored_repeat (GL/glext.h:653)
GL_MIRRORED_REPEAT_ARB = 33648 	# GL/glext.h:654
# ARB_depth_texture (GL/glext.h:657)
GL_DEPTH_COMPONENT16_ARB = 33189 	# GL/glext.h:658
GL_DEPTH_COMPONENT24_ARB = 33190 	# GL/glext.h:659
GL_DEPTH_COMPONENT32_ARB = 33191 	# GL/glext.h:660
GL_TEXTURE_DEPTH_SIZE_ARB = 34890 	# GL/glext.h:661
GL_DEPTH_TEXTURE_MODE_ARB = 34891 	# GL/glext.h:662
# ARB_shadow (GL/glext.h:665)
GL_TEXTURE_COMPARE_MODE_ARB = 34892 	# GL/glext.h:666
GL_TEXTURE_COMPARE_FUNC_ARB = 34893 	# GL/glext.h:667
GL_COMPARE_R_TO_TEXTURE_ARB = 34894 	# GL/glext.h:668
# ARB_shadow_ambient (GL/glext.h:671)
GL_TEXTURE_COMPARE_FAIL_VALUE_ARB = 32959 	# GL/glext.h:672
# ARB_window_pos (GL/glext.h:675)
# ARB_vertex_program (GL/glext.h:678)
GL_COLOR_SUM_ARB = 33880 	# GL/glext.h:679
GL_VERTEX_PROGRAM_ARB = 34336 	# GL/glext.h:680
GL_VERTEX_ATTRIB_ARRAY_ENABLED_ARB = 34338 	# GL/glext.h:681
GL_VERTEX_ATTRIB_ARRAY_SIZE_ARB = 34339 	# GL/glext.h:682
GL_VERTEX_ATTRIB_ARRAY_STRIDE_ARB = 34340 	# GL/glext.h:683
GL_VERTEX_ATTRIB_ARRAY_TYPE_ARB = 34341 	# GL/glext.h:684
GL_CURRENT_VERTEX_ATTRIB_ARB = 34342 	# GL/glext.h:685
GL_PROGRAM_LENGTH_ARB = 34343 	# GL/glext.h:686
GL_PROGRAM_STRING_ARB = 34344 	# GL/glext.h:687
GL_MAX_PROGRAM_MATRIX_STACK_DEPTH_ARB = 34350 	# GL/glext.h:688
GL_MAX_PROGRAM_MATRICES_ARB = 34351 	# GL/glext.h:689
GL_CURRENT_MATRIX_STACK_DEPTH_ARB = 34368 	# GL/glext.h:690
GL_CURRENT_MATRIX_ARB = 34369 	# GL/glext.h:691
GL_VERTEX_PROGRAM_POINT_SIZE_ARB = 34370 	# GL/glext.h:692
GL_VERTEX_PROGRAM_TWO_SIDE_ARB = 34371 	# GL/glext.h:693
GL_VERTEX_ATTRIB_ARRAY_POINTER_ARB = 34373 	# GL/glext.h:694
GL_PROGRAM_ERROR_POSITION_ARB = 34379 	# GL/glext.h:695
GL_PROGRAM_BINDING_ARB = 34423 	# GL/glext.h:696
GL_MAX_VERTEX_ATTRIBS_ARB = 34921 	# GL/glext.h:697
GL_VERTEX_ATTRIB_ARRAY_NORMALIZED_ARB = 34922 	# GL/glext.h:698
GL_PROGRAM_ERROR_STRING_ARB = 34932 	# GL/glext.h:699
GL_PROGRAM_FORMAT_ASCII_ARB = 34933 	# GL/glext.h:700
GL_PROGRAM_FORMAT_ARB = 34934 	# GL/glext.h:701
GL_PROGRAM_INSTRUCTIONS_ARB = 34976 	# GL/glext.h:702
GL_MAX_PROGRAM_INSTRUCTIONS_ARB = 34977 	# GL/glext.h:703
GL_PROGRAM_NATIVE_INSTRUCTIONS_ARB = 34978 	# GL/glext.h:704
GL_MAX_PROGRAM_NATIVE_INSTRUCTIONS_ARB = 34979 	# GL/glext.h:705
GL_PROGRAM_TEMPORARIES_ARB = 34980 	# GL/glext.h:706
GL_MAX_PROGRAM_TEMPORARIES_ARB = 34981 	# GL/glext.h:707
GL_PROGRAM_NATIVE_TEMPORARIES_ARB = 34982 	# GL/glext.h:708
GL_MAX_PROGRAM_NATIVE_TEMPORARIES_ARB = 34983 	# GL/glext.h:709
GL_PROGRAM_PARAMETERS_ARB = 34984 	# GL/glext.h:710
GL_MAX_PROGRAM_PARAMETERS_ARB = 34985 	# GL/glext.h:711
GL_PROGRAM_NATIVE_PARAMETERS_ARB = 34986 	# GL/glext.h:712
GL_MAX_PROGRAM_NATIVE_PARAMETERS_ARB = 34987 	# GL/glext.h:713
GL_PROGRAM_ATTRIBS_ARB = 34988 	# GL/glext.h:714
GL_MAX_PROGRAM_ATTRIBS_ARB = 34989 	# GL/glext.h:715
GL_PROGRAM_NATIVE_ATTRIBS_ARB = 34990 	# GL/glext.h:716
GL_MAX_PROGRAM_NATIVE_ATTRIBS_ARB = 34991 	# GL/glext.h:717
GL_PROGRAM_ADDRESS_REGISTERS_ARB = 34992 	# GL/glext.h:718
GL_MAX_PROGRAM_ADDRESS_REGISTERS_ARB = 34993 	# GL/glext.h:719
GL_PROGRAM_NATIVE_ADDRESS_REGISTERS_ARB = 34994 	# GL/glext.h:720
GL_MAX_PROGRAM_NATIVE_ADDRESS_REGISTERS_ARB = 34995 	# GL/glext.h:721
GL_MAX_PROGRAM_LOCAL_PARAMETERS_ARB = 34996 	# GL/glext.h:722
GL_MAX_PROGRAM_ENV_PARAMETERS_ARB = 34997 	# GL/glext.h:723
GL_PROGRAM_UNDER_NATIVE_LIMITS_ARB = 34998 	# GL/glext.h:724
GL_TRANSPOSE_CURRENT_MATRIX_ARB = 34999 	# GL/glext.h:725
GL_MATRIX0_ARB = 35008 	# GL/glext.h:726
GL_MATRIX1_ARB = 35009 	# GL/glext.h:727
GL_MATRIX2_ARB = 35010 	# GL/glext.h:728
GL_MATRIX3_ARB = 35011 	# GL/glext.h:729
GL_MATRIX4_ARB = 35012 	# GL/glext.h:730
GL_MATRIX5_ARB = 35013 	# GL/glext.h:731
GL_MATRIX6_ARB = 35014 	# GL/glext.h:732
GL_MATRIX7_ARB = 35015 	# GL/glext.h:733
GL_MATRIX8_ARB = 35016 	# GL/glext.h:734
GL_MATRIX9_ARB = 35017 	# GL/glext.h:735
GL_MATRIX10_ARB = 35018 	# GL/glext.h:736
GL_MATRIX11_ARB = 35019 	# GL/glext.h:737
GL_MATRIX12_ARB = 35020 	# GL/glext.h:738
GL_MATRIX13_ARB = 35021 	# GL/glext.h:739
GL_MATRIX14_ARB = 35022 	# GL/glext.h:740
GL_MATRIX15_ARB = 35023 	# GL/glext.h:741
GL_MATRIX16_ARB = 35024 	# GL/glext.h:742
GL_MATRIX17_ARB = 35025 	# GL/glext.h:743
GL_MATRIX18_ARB = 35026 	# GL/glext.h:744
GL_MATRIX19_ARB = 35027 	# GL/glext.h:745
GL_MATRIX20_ARB = 35028 	# GL/glext.h:746
GL_MATRIX21_ARB = 35029 	# GL/glext.h:747
GL_MATRIX22_ARB = 35030 	# GL/glext.h:748
GL_MATRIX23_ARB = 35031 	# GL/glext.h:749
GL_MATRIX24_ARB = 35032 	# GL/glext.h:750
GL_MATRIX25_ARB = 35033 	# GL/glext.h:751
GL_MATRIX26_ARB = 35034 	# GL/glext.h:752
GL_MATRIX27_ARB = 35035 	# GL/glext.h:753
GL_MATRIX28_ARB = 35036 	# GL/glext.h:754
GL_MATRIX29_ARB = 35037 	# GL/glext.h:755
GL_MATRIX30_ARB = 35038 	# GL/glext.h:756
GL_MATRIX31_ARB = 35039 	# GL/glext.h:757
# ARB_fragment_program (GL/glext.h:760)
GL_FRAGMENT_PROGRAM_ARB = 34820 	# GL/glext.h:761
GL_PROGRAM_ALU_INSTRUCTIONS_ARB = 34821 	# GL/glext.h:762
GL_PROGRAM_TEX_INSTRUCTIONS_ARB = 34822 	# GL/glext.h:763
GL_PROGRAM_TEX_INDIRECTIONS_ARB = 34823 	# GL/glext.h:764
GL_PROGRAM_NATIVE_ALU_INSTRUCTIONS_ARB = 34824 	# GL/glext.h:765
GL_PROGRAM_NATIVE_TEX_INSTRUCTIONS_ARB = 34825 	# GL/glext.h:766
GL_PROGRAM_NATIVE_TEX_INDIRECTIONS_ARB = 34826 	# GL/glext.h:767
GL_MAX_PROGRAM_ALU_INSTRUCTIONS_ARB = 34827 	# GL/glext.h:768
GL_MAX_PROGRAM_TEX_INSTRUCTIONS_ARB = 34828 	# GL/glext.h:769
GL_MAX_PROGRAM_TEX_INDIRECTIONS_ARB = 34829 	# GL/glext.h:770
GL_MAX_PROGRAM_NATIVE_ALU_INSTRUCTIONS_ARB = 34830 	# GL/glext.h:771
GL_MAX_PROGRAM_NATIVE_TEX_INSTRUCTIONS_ARB = 34831 	# GL/glext.h:772
GL_MAX_PROGRAM_NATIVE_TEX_INDIRECTIONS_ARB = 34832 	# GL/glext.h:773
GL_MAX_TEXTURE_COORDS_ARB = 34929 	# GL/glext.h:774
GL_MAX_TEXTURE_IMAGE_UNITS_ARB = 34930 	# GL/glext.h:775
# ARB_vertex_buffer_object (GL/glext.h:778)
GL_BUFFER_SIZE_ARB = 34660 	# GL/glext.h:779
GL_BUFFER_USAGE_ARB = 34661 	# GL/glext.h:780
GL_ARRAY_BUFFER_ARB = 34962 	# GL/glext.h:781
GL_ELEMENT_ARRAY_BUFFER_ARB = 34963 	# GL/glext.h:782
GL_ARRAY_BUFFER_BINDING_ARB = 34964 	# GL/glext.h:783
GL_ELEMENT_ARRAY_BUFFER_BINDING_ARB = 34965 	# GL/glext.h:784
GL_VERTEX_ARRAY_BUFFER_BINDING_ARB = 34966 	# GL/glext.h:785
GL_NORMAL_ARRAY_BUFFER_BINDING_ARB = 34967 	# GL/glext.h:786
GL_COLOR_ARRAY_BUFFER_BINDING_ARB = 34968 	# GL/glext.h:787
GL_INDEX_ARRAY_BUFFER_BINDING_ARB = 34969 	# GL/glext.h:788
GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING_ARB = 34970 	# GL/glext.h:789
GL_EDGE_FLAG_ARRAY_BUFFER_BINDING_ARB = 34971 	# GL/glext.h:790
GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING_ARB = 34972 	# GL/glext.h:791
GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING_ARB = 34973 	# GL/glext.h:792
GL_WEIGHT_ARRAY_BUFFER_BINDING_ARB = 34974 	# GL/glext.h:793
GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING_ARB = 34975 	# GL/glext.h:794
GL_READ_ONLY_ARB = 35000 	# GL/glext.h:795
GL_WRITE_ONLY_ARB = 35001 	# GL/glext.h:796
GL_READ_WRITE_ARB = 35002 	# GL/glext.h:797
GL_BUFFER_ACCESS_ARB = 35003 	# GL/glext.h:798
GL_BUFFER_MAPPED_ARB = 35004 	# GL/glext.h:799
GL_BUFFER_MAP_POINTER_ARB = 35005 	# GL/glext.h:800
GL_STREAM_DRAW_ARB = 35040 	# GL/glext.h:801
GL_STREAM_READ_ARB = 35041 	# GL/glext.h:802
GL_STREAM_COPY_ARB = 35042 	# GL/glext.h:803
GL_STATIC_DRAW_ARB = 35044 	# GL/glext.h:804
GL_STATIC_READ_ARB = 35045 	# GL/glext.h:805
GL_STATIC_COPY_ARB = 35046 	# GL/glext.h:806
GL_DYNAMIC_DRAW_ARB = 35048 	# GL/glext.h:807
GL_DYNAMIC_READ_ARB = 35049 	# GL/glext.h:808
GL_DYNAMIC_COPY_ARB = 35050 	# GL/glext.h:809
# ARB_occlusion_query (GL/glext.h:812)
GL_QUERY_COUNTER_BITS_ARB = 34916 	# GL/glext.h:813
GL_CURRENT_QUERY_ARB = 34917 	# GL/glext.h:814
GL_QUERY_RESULT_ARB = 34918 	# GL/glext.h:815
GL_QUERY_RESULT_AVAILABLE_ARB = 34919 	# GL/glext.h:816
GL_SAMPLES_PASSED_ARB = 35092 	# GL/glext.h:817
# ARB_shader_objects (GL/glext.h:820)
GL_PROGRAM_OBJECT_ARB = 35648 	# GL/glext.h:821
GL_SHADER_OBJECT_ARB = 35656 	# GL/glext.h:822
GL_OBJECT_TYPE_ARB = 35662 	# GL/glext.h:823
GL_OBJECT_SUBTYPE_ARB = 35663 	# GL/glext.h:824
GL_FLOAT_VEC2_ARB = 35664 	# GL/glext.h:825
GL_FLOAT_VEC3_ARB = 35665 	# GL/glext.h:826
GL_FLOAT_VEC4_ARB = 35666 	# GL/glext.h:827
GL_INT_VEC2_ARB = 35667 	# GL/glext.h:828
GL_INT_VEC3_ARB = 35668 	# GL/glext.h:829
GL_INT_VEC4_ARB = 35669 	# GL/glext.h:830
GL_BOOL_ARB = 35670 	# GL/glext.h:831
GL_BOOL_VEC2_ARB = 35671 	# GL/glext.h:832
GL_BOOL_VEC3_ARB = 35672 	# GL/glext.h:833
GL_BOOL_VEC4_ARB = 35673 	# GL/glext.h:834
GL_FLOAT_MAT2_ARB = 35674 	# GL/glext.h:835
GL_FLOAT_MAT3_ARB = 35675 	# GL/glext.h:836
GL_FLOAT_MAT4_ARB = 35676 	# GL/glext.h:837
GL_SAMPLER_1D_ARB = 35677 	# GL/glext.h:838
GL_SAMPLER_2D_ARB = 35678 	# GL/glext.h:839
GL_SAMPLER_3D_ARB = 35679 	# GL/glext.h:840
GL_SAMPLER_CUBE_ARB = 35680 	# GL/glext.h:841
GL_SAMPLER_1D_SHADOW_ARB = 35681 	# GL/glext.h:842
GL_SAMPLER_2D_SHADOW_ARB = 35682 	# GL/glext.h:843
GL_SAMPLER_2D_RECT_ARB = 35683 	# GL/glext.h:844
GL_SAMPLER_2D_RECT_SHADOW_ARB = 35684 	# GL/glext.h:845
GL_OBJECT_DELETE_STATUS_ARB = 35712 	# GL/glext.h:846
GL_OBJECT_COMPILE_STATUS_ARB = 35713 	# GL/glext.h:847
GL_OBJECT_LINK_STATUS_ARB = 35714 	# GL/glext.h:848
GL_OBJECT_VALIDATE_STATUS_ARB = 35715 	# GL/glext.h:849
GL_OBJECT_INFO_LOG_LENGTH_ARB = 35716 	# GL/glext.h:850
GL_OBJECT_ATTACHED_OBJECTS_ARB = 35717 	# GL/glext.h:851
GL_OBJECT_ACTIVE_UNIFORMS_ARB = 35718 	# GL/glext.h:852
GL_OBJECT_ACTIVE_UNIFORM_MAX_LENGTH_ARB = 35719 	# GL/glext.h:853
GL_OBJECT_SHADER_SOURCE_LENGTH_ARB = 35720 	# GL/glext.h:854
# ARB_vertex_shader (GL/glext.h:857)
GL_VERTEX_SHADER_ARB = 35633 	# GL/glext.h:858
GL_MAX_VERTEX_UNIFORM_COMPONENTS_ARB = 35658 	# GL/glext.h:859
GL_MAX_VARYING_FLOATS_ARB = 35659 	# GL/glext.h:860
GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS_ARB = 35660 	# GL/glext.h:861
GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS_ARB = 35661 	# GL/glext.h:862
GL_OBJECT_ACTIVE_ATTRIBUTES_ARB = 35721 	# GL/glext.h:863
GL_OBJECT_ACTIVE_ATTRIBUTE_MAX_LENGTH_ARB = 35722 	# GL/glext.h:864
# ARB_fragment_shader (GL/glext.h:867)
GL_FRAGMENT_SHADER_ARB = 35632 	# GL/glext.h:868
GL_MAX_FRAGMENT_UNIFORM_COMPONENTS_ARB = 35657 	# GL/glext.h:869
GL_FRAGMENT_SHADER_DERIVATIVE_HINT_ARB = 35723 	# GL/glext.h:870
# ARB_shading_language_100 (GL/glext.h:873)
GL_SHADING_LANGUAGE_VERSION_ARB = 35724 	# GL/glext.h:874
# ARB_texture_non_power_of_two (GL/glext.h:877)
# ARB_point_sprite (GL/glext.h:880)
GL_POINT_SPRITE_ARB = 34913 	# GL/glext.h:881
GL_COORD_REPLACE_ARB = 34914 	# GL/glext.h:882
# ARB_fragment_program_shadow (GL/glext.h:885)
# ARB_draw_buffers (GL/glext.h:888)
GL_MAX_DRAW_BUFFERS_ARB = 34852 	# GL/glext.h:889
GL_DRAW_BUFFER0_ARB = 34853 	# GL/glext.h:890
GL_DRAW_BUFFER1_ARB = 34854 	# GL/glext.h:891
GL_DRAW_BUFFER2_ARB = 34855 	# GL/glext.h:892
GL_DRAW_BUFFER3_ARB = 34856 	# GL/glext.h:893
GL_DRAW_BUFFER4_ARB = 34857 	# GL/glext.h:894
GL_DRAW_BUFFER5_ARB = 34858 	# GL/glext.h:895
GL_DRAW_BUFFER6_ARB = 34859 	# GL/glext.h:896
GL_DRAW_BUFFER7_ARB = 34860 	# GL/glext.h:897
GL_DRAW_BUFFER8_ARB = 34861 	# GL/glext.h:898
GL_DRAW_BUFFER9_ARB = 34862 	# GL/glext.h:899
GL_DRAW_BUFFER10_ARB = 34863 	# GL/glext.h:900
GL_DRAW_BUFFER11_ARB = 34864 	# GL/glext.h:901
GL_DRAW_BUFFER12_ARB = 34865 	# GL/glext.h:902
GL_DRAW_BUFFER13_ARB = 34866 	# GL/glext.h:903
GL_DRAW_BUFFER14_ARB = 34867 	# GL/glext.h:904
GL_DRAW_BUFFER15_ARB = 34868 	# GL/glext.h:905
# ARB_texture_rectangle (GL/glext.h:908)
GL_TEXTURE_RECTANGLE_ARB = 34037 	# GL/glext.h:909
GL_TEXTURE_BINDING_RECTANGLE_ARB = 34038 	# GL/glext.h:910
GL_PROXY_TEXTURE_RECTANGLE_ARB = 34039 	# GL/glext.h:911
GL_MAX_RECTANGLE_TEXTURE_SIZE_ARB = 34040 	# GL/glext.h:912
# ARB_color_buffer_float (GL/glext.h:915)
GL_RGBA_FLOAT_MODE_ARB = 34848 	# GL/glext.h:916
GL_CLAMP_VERTEX_COLOR_ARB = 35098 	# GL/glext.h:917
GL_CLAMP_FRAGMENT_COLOR_ARB = 35099 	# GL/glext.h:918
GL_CLAMP_READ_COLOR_ARB = 35100 	# GL/glext.h:919
GL_FIXED_ONLY_ARB = 35101 	# GL/glext.h:920
# ARB_half_float_pixel (GL/glext.h:923)
GL_HALF_FLOAT_ARB = 5131 	# GL/glext.h:924
# ARB_texture_float (GL/glext.h:927)
GL_TEXTURE_RED_TYPE_ARB = 35856 	# GL/glext.h:928
GL_TEXTURE_GREEN_TYPE_ARB = 35857 	# GL/glext.h:929
GL_TEXTURE_BLUE_TYPE_ARB = 35858 	# GL/glext.h:930
GL_TEXTURE_ALPHA_TYPE_ARB = 35859 	# GL/glext.h:931
GL_TEXTURE_LUMINANCE_TYPE_ARB = 35860 	# GL/glext.h:932
GL_TEXTURE_INTENSITY_TYPE_ARB = 35861 	# GL/glext.h:933
GL_TEXTURE_DEPTH_TYPE_ARB = 35862 	# GL/glext.h:934
GL_UNSIGNED_NORMALIZED_ARB = 35863 	# GL/glext.h:935
GL_RGBA32F_ARB = 34836 	# GL/glext.h:936
GL_RGB32F_ARB = 34837 	# GL/glext.h:937
GL_ALPHA32F_ARB = 34838 	# GL/glext.h:938
GL_INTENSITY32F_ARB = 34839 	# GL/glext.h:939
GL_LUMINANCE32F_ARB = 34840 	# GL/glext.h:940
GL_LUMINANCE_ALPHA32F_ARB = 34841 	# GL/glext.h:941
GL_RGBA16F_ARB = 34842 	# GL/glext.h:942
GL_RGB16F_ARB = 34843 	# GL/glext.h:943
GL_ALPHA16F_ARB = 34844 	# GL/glext.h:944
GL_INTENSITY16F_ARB = 34845 	# GL/glext.h:945
GL_LUMINANCE16F_ARB = 34846 	# GL/glext.h:946
GL_LUMINANCE_ALPHA16F_ARB = 34847 	# GL/glext.h:947
# ARB_pixel_buffer_object (GL/glext.h:950)
GL_PIXEL_PACK_BUFFER_ARB = 35051 	# GL/glext.h:951
GL_PIXEL_UNPACK_BUFFER_ARB = 35052 	# GL/glext.h:952
GL_PIXEL_PACK_BUFFER_BINDING_ARB = 35053 	# GL/glext.h:953
GL_PIXEL_UNPACK_BUFFER_BINDING_ARB = 35055 	# GL/glext.h:954
# EXT_abgr (GL/glext.h:957)
GL_ABGR_EXT = 32768 	# GL/glext.h:958
# EXT_blend_color (GL/glext.h:961)
GL_CONSTANT_COLOR_EXT = 32769 	# GL/glext.h:962
GL_ONE_MINUS_CONSTANT_COLOR_EXT = 32770 	# GL/glext.h:963
GL_CONSTANT_ALPHA_EXT = 32771 	# GL/glext.h:964
GL_ONE_MINUS_CONSTANT_ALPHA_EXT = 32772 	# GL/glext.h:965
GL_BLEND_COLOR_EXT = 32773 	# GL/glext.h:966
# EXT_polygon_offset (GL/glext.h:969)
GL_POLYGON_OFFSET_EXT = 32823 	# GL/glext.h:970
GL_POLYGON_OFFSET_FACTOR_EXT = 32824 	# GL/glext.h:971
GL_POLYGON_OFFSET_BIAS_EXT = 32825 	# GL/glext.h:972
# EXT_texture (GL/glext.h:975)
GL_ALPHA4_EXT = 32827 	# GL/glext.h:976
GL_ALPHA8_EXT = 32828 	# GL/glext.h:977
GL_ALPHA12_EXT = 32829 	# GL/glext.h:978
GL_ALPHA16_EXT = 32830 	# GL/glext.h:979
GL_LUMINANCE4_EXT = 32831 	# GL/glext.h:980
GL_LUMINANCE8_EXT = 32832 	# GL/glext.h:981
GL_LUMINANCE12_EXT = 32833 	# GL/glext.h:982
GL_LUMINANCE16_EXT = 32834 	# GL/glext.h:983
GL_LUMINANCE4_ALPHA4_EXT = 32835 	# GL/glext.h:984
GL_LUMINANCE6_ALPHA2_EXT = 32836 	# GL/glext.h:985
GL_LUMINANCE8_ALPHA8_EXT = 32837 	# GL/glext.h:986
GL_LUMINANCE12_ALPHA4_EXT = 32838 	# GL/glext.h:987
GL_LUMINANCE12_ALPHA12_EXT = 32839 	# GL/glext.h:988
GL_LUMINANCE16_ALPHA16_EXT = 32840 	# GL/glext.h:989
GL_INTENSITY_EXT = 32841 	# GL/glext.h:990
GL_INTENSITY4_EXT = 32842 	# GL/glext.h:991
GL_INTENSITY8_EXT = 32843 	# GL/glext.h:992
GL_INTENSITY12_EXT = 32844 	# GL/glext.h:993
GL_INTENSITY16_EXT = 32845 	# GL/glext.h:994
GL_RGB2_EXT = 32846 	# GL/glext.h:995
GL_RGB4_EXT = 32847 	# GL/glext.h:996
GL_RGB5_EXT = 32848 	# GL/glext.h:997
GL_RGB8_EXT = 32849 	# GL/glext.h:998
GL_RGB10_EXT = 32850 	# GL/glext.h:999
GL_RGB12_EXT = 32851 	# GL/glext.h:1000
GL_RGB16_EXT = 32852 	# GL/glext.h:1001
GL_RGBA2_EXT = 32853 	# GL/glext.h:1002
GL_RGBA4_EXT = 32854 	# GL/glext.h:1003
GL_RGB5_A1_EXT = 32855 	# GL/glext.h:1004
GL_RGBA8_EXT = 32856 	# GL/glext.h:1005
GL_RGB10_A2_EXT = 32857 	# GL/glext.h:1006
GL_RGBA12_EXT = 32858 	# GL/glext.h:1007
GL_RGBA16_EXT = 32859 	# GL/glext.h:1008
GL_TEXTURE_RED_SIZE_EXT = 32860 	# GL/glext.h:1009
GL_TEXTURE_GREEN_SIZE_EXT = 32861 	# GL/glext.h:1010
GL_TEXTURE_BLUE_SIZE_EXT = 32862 	# GL/glext.h:1011
GL_TEXTURE_ALPHA_SIZE_EXT = 32863 	# GL/glext.h:1012
GL_TEXTURE_LUMINANCE_SIZE_EXT = 32864 	# GL/glext.h:1013
GL_TEXTURE_INTENSITY_SIZE_EXT = 32865 	# GL/glext.h:1014
GL_REPLACE_EXT = 32866 	# GL/glext.h:1015
GL_PROXY_TEXTURE_1D_EXT = 32867 	# GL/glext.h:1016
GL_PROXY_TEXTURE_2D_EXT = 32868 	# GL/glext.h:1017
GL_TEXTURE_TOO_LARGE_EXT = 32869 	# GL/glext.h:1018
# EXT_texture3D (GL/glext.h:1021)
GL_PACK_SKIP_IMAGES_EXT = 32875 	# GL/glext.h:1022
GL_PACK_IMAGE_HEIGHT_EXT = 32876 	# GL/glext.h:1023
GL_UNPACK_SKIP_IMAGES_EXT = 32877 	# GL/glext.h:1024
GL_UNPACK_IMAGE_HEIGHT_EXT = 32878 	# GL/glext.h:1025
GL_TEXTURE_3D_EXT = 32879 	# GL/glext.h:1026
GL_PROXY_TEXTURE_3D_EXT = 32880 	# GL/glext.h:1027
GL_TEXTURE_DEPTH_EXT = 32881 	# GL/glext.h:1028
GL_TEXTURE_WRAP_R_EXT = 32882 	# GL/glext.h:1029
GL_MAX_3D_TEXTURE_SIZE_EXT = 32883 	# GL/glext.h:1030
# SGIS_texture_filter4 (GL/glext.h:1033)
GL_FILTER4_SGIS = 33094 	# GL/glext.h:1034
GL_TEXTURE_FILTER4_SIZE_SGIS = 33095 	# GL/glext.h:1035
# EXT_subtexture (GL/glext.h:1038)
# EXT_copy_texture (GL/glext.h:1041)
# EXT_histogram (GL/glext.h:1044)
GL_HISTOGRAM_EXT = 32804 	# GL/glext.h:1045
GL_PROXY_HISTOGRAM_EXT = 32805 	# GL/glext.h:1046
GL_HISTOGRAM_WIDTH_EXT = 32806 	# GL/glext.h:1047
GL_HISTOGRAM_FORMAT_EXT = 32807 	# GL/glext.h:1048
GL_HISTOGRAM_RED_SIZE_EXT = 32808 	# GL/glext.h:1049
GL_HISTOGRAM_GREEN_SIZE_EXT = 32809 	# GL/glext.h:1050
GL_HISTOGRAM_BLUE_SIZE_EXT = 32810 	# GL/glext.h:1051
GL_HISTOGRAM_ALPHA_SIZE_EXT = 32811 	# GL/glext.h:1052
GL_HISTOGRAM_LUMINANCE_SIZE_EXT = 32812 	# GL/glext.h:1053
GL_HISTOGRAM_SINK_EXT = 32813 	# GL/glext.h:1054
GL_MINMAX_EXT = 32814 	# GL/glext.h:1055
GL_MINMAX_FORMAT_EXT = 32815 	# GL/glext.h:1056
GL_MINMAX_SINK_EXT = 32816 	# GL/glext.h:1057
GL_TABLE_TOO_LARGE_EXT = 32817 	# GL/glext.h:1058
# EXT_convolution (GL/glext.h:1061)
GL_CONVOLUTION_1D_EXT = 32784 	# GL/glext.h:1062
GL_CONVOLUTION_2D_EXT = 32785 	# GL/glext.h:1063
GL_SEPARABLE_2D_EXT = 32786 	# GL/glext.h:1064
GL_CONVOLUTION_BORDER_MODE_EXT = 32787 	# GL/glext.h:1065
GL_CONVOLUTION_FILTER_SCALE_EXT = 32788 	# GL/glext.h:1066
GL_CONVOLUTION_FILTER_BIAS_EXT = 32789 	# GL/glext.h:1067
GL_REDUCE_EXT = 32790 	# GL/glext.h:1068
GL_CONVOLUTION_FORMAT_EXT = 32791 	# GL/glext.h:1069
GL_CONVOLUTION_WIDTH_EXT = 32792 	# GL/glext.h:1070
GL_CONVOLUTION_HEIGHT_EXT = 32793 	# GL/glext.h:1071
GL_MAX_CONVOLUTION_WIDTH_EXT = 32794 	# GL/glext.h:1072
GL_MAX_CONVOLUTION_HEIGHT_EXT = 32795 	# GL/glext.h:1073
GL_POST_CONVOLUTION_RED_SCALE_EXT = 32796 	# GL/glext.h:1074
GL_POST_CONVOLUTION_GREEN_SCALE_EXT = 32797 	# GL/glext.h:1075
GL_POST_CONVOLUTION_BLUE_SCALE_EXT = 32798 	# GL/glext.h:1076
GL_POST_CONVOLUTION_ALPHA_SCALE_EXT = 32799 	# GL/glext.h:1077
GL_POST_CONVOLUTION_RED_BIAS_EXT = 32800 	# GL/glext.h:1078
GL_POST_CONVOLUTION_GREEN_BIAS_EXT = 32801 	# GL/glext.h:1079
GL_POST_CONVOLUTION_BLUE_BIAS_EXT = 32802 	# GL/glext.h:1080
GL_POST_CONVOLUTION_ALPHA_BIAS_EXT = 32803 	# GL/glext.h:1081
# SGI_color_matrix (GL/glext.h:1084)
GL_COLOR_MATRIX_SGI = 32945 	# GL/glext.h:1085
GL_COLOR_MATRIX_STACK_DEPTH_SGI = 32946 	# GL/glext.h:1086
GL_MAX_COLOR_MATRIX_STACK_DEPTH_SGI = 32947 	# GL/glext.h:1087
GL_POST_COLOR_MATRIX_RED_SCALE_SGI = 32948 	# GL/glext.h:1088
GL_POST_COLOR_MATRIX_GREEN_SCALE_SGI = 32949 	# GL/glext.h:1089
GL_POST_COLOR_MATRIX_BLUE_SCALE_SGI = 32950 	# GL/glext.h:1090
GL_POST_COLOR_MATRIX_ALPHA_SCALE_SGI = 32951 	# GL/glext.h:1091
GL_POST_COLOR_MATRIX_RED_BIAS_SGI = 32952 	# GL/glext.h:1092
GL_POST_COLOR_MATRIX_GREEN_BIAS_SGI = 32953 	# GL/glext.h:1093
GL_POST_COLOR_MATRIX_BLUE_BIAS_SGI = 32954 	# GL/glext.h:1094
GL_POST_COLOR_MATRIX_ALPHA_BIAS_SGI = 32955 	# GL/glext.h:1095
# SGI_color_table (GL/glext.h:1098)
GL_COLOR_TABLE_SGI = 32976 	# GL/glext.h:1099
GL_POST_CONVOLUTION_COLOR_TABLE_SGI = 32977 	# GL/glext.h:1100
GL_POST_COLOR_MATRIX_COLOR_TABLE_SGI = 32978 	# GL/glext.h:1101
GL_PROXY_COLOR_TABLE_SGI = 32979 	# GL/glext.h:1102
GL_PROXY_POST_CONVOLUTION_COLOR_TABLE_SGI = 32980 	# GL/glext.h:1103
GL_PROXY_POST_COLOR_MATRIX_COLOR_TABLE_SGI = 32981 	# GL/glext.h:1104
GL_COLOR_TABLE_SCALE_SGI = 32982 	# GL/glext.h:1105
GL_COLOR_TABLE_BIAS_SGI = 32983 	# GL/glext.h:1106
GL_COLOR_TABLE_FORMAT_SGI = 32984 	# GL/glext.h:1107
GL_COLOR_TABLE_WIDTH_SGI = 32985 	# GL/glext.h:1108
GL_COLOR_TABLE_RED_SIZE_SGI = 32986 	# GL/glext.h:1109
GL_COLOR_TABLE_GREEN_SIZE_SGI = 32987 	# GL/glext.h:1110
GL_COLOR_TABLE_BLUE_SIZE_SGI = 32988 	# GL/glext.h:1111
GL_COLOR_TABLE_ALPHA_SIZE_SGI = 32989 	# GL/glext.h:1112
GL_COLOR_TABLE_LUMINANCE_SIZE_SGI = 32990 	# GL/glext.h:1113
GL_COLOR_TABLE_INTENSITY_SIZE_SGI = 32991 	# GL/glext.h:1114
# SGIS_pixel_texture (GL/glext.h:1117)
GL_PIXEL_TEXTURE_SGIS = 33619 	# GL/glext.h:1118
GL_PIXEL_FRAGMENT_RGB_SOURCE_SGIS = 33620 	# GL/glext.h:1119
GL_PIXEL_FRAGMENT_ALPHA_SOURCE_SGIS = 33621 	# GL/glext.h:1120
GL_PIXEL_GROUP_COLOR_SGIS = 33622 	# GL/glext.h:1121
# SGIX_pixel_texture (GL/glext.h:1124)
GL_PIXEL_TEX_GEN_SGIX = 33081 	# GL/glext.h:1125
GL_PIXEL_TEX_GEN_MODE_SGIX = 33579 	# GL/glext.h:1126
# SGIS_texture4D (GL/glext.h:1129)
GL_PACK_SKIP_VOLUMES_SGIS = 33072 	# GL/glext.h:1130
GL_PACK_IMAGE_DEPTH_SGIS = 33073 	# GL/glext.h:1131
GL_UNPACK_SKIP_VOLUMES_SGIS = 33074 	# GL/glext.h:1132
GL_UNPACK_IMAGE_DEPTH_SGIS = 33075 	# GL/glext.h:1133
GL_TEXTURE_4D_SGIS = 33076 	# GL/glext.h:1134
GL_PROXY_TEXTURE_4D_SGIS = 33077 	# GL/glext.h:1135
GL_TEXTURE_4DSIZE_SGIS = 33078 	# GL/glext.h:1136
GL_TEXTURE_WRAP_Q_SGIS = 33079 	# GL/glext.h:1137
GL_MAX_4D_TEXTURE_SIZE_SGIS = 33080 	# GL/glext.h:1138
GL_TEXTURE_4D_BINDING_SGIS = 33103 	# GL/glext.h:1139
# SGI_texture_color_table (GL/glext.h:1142)
GL_TEXTURE_COLOR_TABLE_SGI = 32956 	# GL/glext.h:1143
GL_PROXY_TEXTURE_COLOR_TABLE_SGI = 32957 	# GL/glext.h:1144
# EXT_cmyka (GL/glext.h:1147)
GL_CMYK_EXT = 32780 	# GL/glext.h:1148
GL_CMYKA_EXT = 32781 	# GL/glext.h:1149
GL_PACK_CMYK_HINT_EXT = 32782 	# GL/glext.h:1150
GL_UNPACK_CMYK_HINT_EXT = 32783 	# GL/glext.h:1151
# EXT_texture_object (GL/glext.h:1154)
GL_TEXTURE_PRIORITY_EXT = 32870 	# GL/glext.h:1155
GL_TEXTURE_RESIDENT_EXT = 32871 	# GL/glext.h:1156
GL_TEXTURE_1D_BINDING_EXT = 32872 	# GL/glext.h:1157
GL_TEXTURE_2D_BINDING_EXT = 32873 	# GL/glext.h:1158
GL_TEXTURE_3D_BINDING_EXT = 32874 	# GL/glext.h:1159
# SGIS_detail_texture (GL/glext.h:1162)
GL_DETAIL_TEXTURE_2D_SGIS = 32917 	# GL/glext.h:1163
GL_DETAIL_TEXTURE_2D_BINDING_SGIS = 32918 	# GL/glext.h:1164
GL_LINEAR_DETAIL_SGIS = 32919 	# GL/glext.h:1165
GL_LINEAR_DETAIL_ALPHA_SGIS = 32920 	# GL/glext.h:1166
GL_LINEAR_DETAIL_COLOR_SGIS = 32921 	# GL/glext.h:1167
GL_DETAIL_TEXTURE_LEVEL_SGIS = 32922 	# GL/glext.h:1168
GL_DETAIL_TEXTURE_MODE_SGIS = 32923 	# GL/glext.h:1169
GL_DETAIL_TEXTURE_FUNC_POINTS_SGIS = 32924 	# GL/glext.h:1170
# SGIS_sharpen_texture (GL/glext.h:1173)
GL_LINEAR_SHARPEN_SGIS = 32941 	# GL/glext.h:1174
GL_LINEAR_SHARPEN_ALPHA_SGIS = 32942 	# GL/glext.h:1175
GL_LINEAR_SHARPEN_COLOR_SGIS = 32943 	# GL/glext.h:1176
GL_SHARPEN_TEXTURE_FUNC_POINTS_SGIS = 32944 	# GL/glext.h:1177
# EXT_packed_pixels (GL/glext.h:1180)
GL_UNSIGNED_BYTE_3_3_2_EXT = 32818 	# GL/glext.h:1181
GL_UNSIGNED_SHORT_4_4_4_4_EXT = 32819 	# GL/glext.h:1182
GL_UNSIGNED_SHORT_5_5_5_1_EXT = 32820 	# GL/glext.h:1183
GL_UNSIGNED_INT_8_8_8_8_EXT = 32821 	# GL/glext.h:1184
GL_UNSIGNED_INT_10_10_10_2_EXT = 32822 	# GL/glext.h:1185
# SGIS_texture_lod (GL/glext.h:1188)
GL_TEXTURE_MIN_LOD_SGIS = 33082 	# GL/glext.h:1189
GL_TEXTURE_MAX_LOD_SGIS = 33083 	# GL/glext.h:1190
GL_TEXTURE_BASE_LEVEL_SGIS = 33084 	# GL/glext.h:1191
GL_TEXTURE_MAX_LEVEL_SGIS = 33085 	# GL/glext.h:1192
# SGIS_multisample (GL/glext.h:1195)
GL_MULTISAMPLE_SGIS = 32925 	# GL/glext.h:1196
GL_SAMPLE_ALPHA_TO_MASK_SGIS = 32926 	# GL/glext.h:1197
GL_SAMPLE_ALPHA_TO_ONE_SGIS = 32927 	# GL/glext.h:1198
GL_SAMPLE_MASK_SGIS = 32928 	# GL/glext.h:1199
GL_1PASS_SGIS = 32929 	# GL/glext.h:1200
GL_2PASS_0_SGIS = 32930 	# GL/glext.h:1201
GL_2PASS_1_SGIS = 32931 	# GL/glext.h:1202
GL_4PASS_0_SGIS = 32932 	# GL/glext.h:1203
GL_4PASS_1_SGIS = 32933 	# GL/glext.h:1204
GL_4PASS_2_SGIS = 32934 	# GL/glext.h:1205
GL_4PASS_3_SGIS = 32935 	# GL/glext.h:1206
GL_SAMPLE_BUFFERS_SGIS = 32936 	# GL/glext.h:1207
GL_SAMPLES_SGIS = 32937 	# GL/glext.h:1208
GL_SAMPLE_MASK_VALUE_SGIS = 32938 	# GL/glext.h:1209
GL_SAMPLE_MASK_INVERT_SGIS = 32939 	# GL/glext.h:1210
GL_SAMPLE_PATTERN_SGIS = 32940 	# GL/glext.h:1211
# EXT_rescale_normal (GL/glext.h:1214)
GL_RESCALE_NORMAL_EXT = 32826 	# GL/glext.h:1215
# EXT_vertex_array (GL/glext.h:1218)
GL_VERTEX_ARRAY_EXT = 32884 	# GL/glext.h:1219
GL_NORMAL_ARRAY_EXT = 32885 	# GL/glext.h:1220
GL_COLOR_ARRAY_EXT = 32886 	# GL/glext.h:1221
GL_INDEX_ARRAY_EXT = 32887 	# GL/glext.h:1222
GL_TEXTURE_COORD_ARRAY_EXT = 32888 	# GL/glext.h:1223
GL_EDGE_FLAG_ARRAY_EXT = 32889 	# GL/glext.h:1224
GL_VERTEX_ARRAY_SIZE_EXT = 32890 	# GL/glext.h:1225
GL_VERTEX_ARRAY_TYPE_EXT = 32891 	# GL/glext.h:1226
GL_VERTEX_ARRAY_STRIDE_EXT = 32892 	# GL/glext.h:1227
GL_VERTEX_ARRAY_COUNT_EXT = 32893 	# GL/glext.h:1228
GL_NORMAL_ARRAY_TYPE_EXT = 32894 	# GL/glext.h:1229
GL_NORMAL_ARRAY_STRIDE_EXT = 32895 	# GL/glext.h:1230
GL_NORMAL_ARRAY_COUNT_EXT = 32896 	# GL/glext.h:1231
GL_COLOR_ARRAY_SIZE_EXT = 32897 	# GL/glext.h:1232
GL_COLOR_ARRAY_TYPE_EXT = 32898 	# GL/glext.h:1233
GL_COLOR_ARRAY_STRIDE_EXT = 32899 	# GL/glext.h:1234
GL_COLOR_ARRAY_COUNT_EXT = 32900 	# GL/glext.h:1235
GL_INDEX_ARRAY_TYPE_EXT = 32901 	# GL/glext.h:1236
GL_INDEX_ARRAY_STRIDE_EXT = 32902 	# GL/glext.h:1237
GL_INDEX_ARRAY_COUNT_EXT = 32903 	# GL/glext.h:1238
GL_TEXTURE_COORD_ARRAY_SIZE_EXT = 32904 	# GL/glext.h:1239
GL_TEXTURE_COORD_ARRAY_TYPE_EXT = 32905 	# GL/glext.h:1240
GL_TEXTURE_COORD_ARRAY_STRIDE_EXT = 32906 	# GL/glext.h:1241
GL_TEXTURE_COORD_ARRAY_COUNT_EXT = 32907 	# GL/glext.h:1242
GL_EDGE_FLAG_ARRAY_STRIDE_EXT = 32908 	# GL/glext.h:1243
GL_EDGE_FLAG_ARRAY_COUNT_EXT = 32909 	# GL/glext.h:1244
GL_VERTEX_ARRAY_POINTER_EXT = 32910 	# GL/glext.h:1245
GL_NORMAL_ARRAY_POINTER_EXT = 32911 	# GL/glext.h:1246
GL_COLOR_ARRAY_POINTER_EXT = 32912 	# GL/glext.h:1247
GL_INDEX_ARRAY_POINTER_EXT = 32913 	# GL/glext.h:1248
GL_TEXTURE_COORD_ARRAY_POINTER_EXT = 32914 	# GL/glext.h:1249
GL_EDGE_FLAG_ARRAY_POINTER_EXT = 32915 	# GL/glext.h:1250
# EXT_misc_attribute (GL/glext.h:1253)
# SGIS_generate_mipmap (GL/glext.h:1256)
GL_GENERATE_MIPMAP_SGIS = 33169 	# GL/glext.h:1257
GL_GENERATE_MIPMAP_HINT_SGIS = 33170 	# GL/glext.h:1258
# SGIX_clipmap (GL/glext.h:1261)
GL_LINEAR_CLIPMAP_LINEAR_SGIX = 33136 	# GL/glext.h:1262
GL_TEXTURE_CLIPMAP_CENTER_SGIX = 33137 	# GL/glext.h:1263
GL_TEXTURE_CLIPMAP_FRAME_SGIX = 33138 	# GL/glext.h:1264
GL_TEXTURE_CLIPMAP_OFFSET_SGIX = 33139 	# GL/glext.h:1265
GL_TEXTURE_CLIPMAP_VIRTUAL_DEPTH_SGIX = 33140 	# GL/glext.h:1266
GL_TEXTURE_CLIPMAP_LOD_OFFSET_SGIX = 33141 	# GL/glext.h:1267
GL_TEXTURE_CLIPMAP_DEPTH_SGIX = 33142 	# GL/glext.h:1268
GL_MAX_CLIPMAP_DEPTH_SGIX = 33143 	# GL/glext.h:1269
GL_MAX_CLIPMAP_VIRTUAL_DEPTH_SGIX = 33144 	# GL/glext.h:1270
GL_NEAREST_CLIPMAP_NEAREST_SGIX = 33869 	# GL/glext.h:1271
GL_NEAREST_CLIPMAP_LINEAR_SGIX = 33870 	# GL/glext.h:1272
GL_LINEAR_CLIPMAP_NEAREST_SGIX = 33871 	# GL/glext.h:1273
# SGIX_shadow (GL/glext.h:1276)
GL_TEXTURE_COMPARE_SGIX = 33178 	# GL/glext.h:1277
GL_TEXTURE_COMPARE_OPERATOR_SGIX = 33179 	# GL/glext.h:1278
GL_TEXTURE_LEQUAL_R_SGIX = 33180 	# GL/glext.h:1279
GL_TEXTURE_GEQUAL_R_SGIX = 33181 	# GL/glext.h:1280
# SGIS_texture_edge_clamp (GL/glext.h:1283)
GL_CLAMP_TO_EDGE_SGIS = 33071 	# GL/glext.h:1284
# SGIS_texture_border_clamp (GL/glext.h:1287)
GL_CLAMP_TO_BORDER_SGIS = 33069 	# GL/glext.h:1288
# EXT_blend_minmax (GL/glext.h:1291)
GL_FUNC_ADD_EXT = 32774 	# GL/glext.h:1292
GL_MIN_EXT = 32775 	# GL/glext.h:1293
GL_MAX_EXT = 32776 	# GL/glext.h:1294
GL_BLEND_EQUATION_EXT = 32777 	# GL/glext.h:1295
# EXT_blend_subtract (GL/glext.h:1298)
GL_FUNC_SUBTRACT_EXT = 32778 	# GL/glext.h:1299
GL_FUNC_REVERSE_SUBTRACT_EXT = 32779 	# GL/glext.h:1300
# EXT_blend_logic_op (GL/glext.h:1303)
# SGIX_interlace (GL/glext.h:1306)
GL_INTERLACE_SGIX = 32916 	# GL/glext.h:1307
# SGIX_pixel_tiles (GL/glext.h:1310)
GL_PIXEL_TILE_BEST_ALIGNMENT_SGIX = 33086 	# GL/glext.h:1311
GL_PIXEL_TILE_CACHE_INCREMENT_SGIX = 33087 	# GL/glext.h:1312
GL_PIXEL_TILE_WIDTH_SGIX = 33088 	# GL/glext.h:1313
GL_PIXEL_TILE_HEIGHT_SGIX = 33089 	# GL/glext.h:1314
GL_PIXEL_TILE_GRID_WIDTH_SGIX = 33090 	# GL/glext.h:1315
GL_PIXEL_TILE_GRID_HEIGHT_SGIX = 33091 	# GL/glext.h:1316
GL_PIXEL_TILE_GRID_DEPTH_SGIX = 33092 	# GL/glext.h:1317
GL_PIXEL_TILE_CACHE_SIZE_SGIX = 33093 	# GL/glext.h:1318
# SGIS_texture_select (GL/glext.h:1321)
GL_DUAL_ALPHA4_SGIS = 33040 	# GL/glext.h:1322
GL_DUAL_ALPHA8_SGIS = 33041 	# GL/glext.h:1323
GL_DUAL_ALPHA12_SGIS = 33042 	# GL/glext.h:1324
GL_DUAL_ALPHA16_SGIS = 33043 	# GL/glext.h:1325
GL_DUAL_LUMINANCE4_SGIS = 33044 	# GL/glext.h:1326
GL_DUAL_LUMINANCE8_SGIS = 33045 	# GL/glext.h:1327
GL_DUAL_LUMINANCE12_SGIS = 33046 	# GL/glext.h:1328
GL_DUAL_LUMINANCE16_SGIS = 33047 	# GL/glext.h:1329
GL_DUAL_INTENSITY4_SGIS = 33048 	# GL/glext.h:1330
GL_DUAL_INTENSITY8_SGIS = 33049 	# GL/glext.h:1331
GL_DUAL_INTENSITY12_SGIS = 33050 	# GL/glext.h:1332
GL_DUAL_INTENSITY16_SGIS = 33051 	# GL/glext.h:1333
GL_DUAL_LUMINANCE_ALPHA4_SGIS = 33052 	# GL/glext.h:1334
GL_DUAL_LUMINANCE_ALPHA8_SGIS = 33053 	# GL/glext.h:1335
GL_QUAD_ALPHA4_SGIS = 33054 	# GL/glext.h:1336
GL_QUAD_ALPHA8_SGIS = 33055 	# GL/glext.h:1337
GL_QUAD_LUMINANCE4_SGIS = 33056 	# GL/glext.h:1338
GL_QUAD_LUMINANCE8_SGIS = 33057 	# GL/glext.h:1339
GL_QUAD_INTENSITY4_SGIS = 33058 	# GL/glext.h:1340
GL_QUAD_INTENSITY8_SGIS = 33059 	# GL/glext.h:1341
GL_DUAL_TEXTURE_SELECT_SGIS = 33060 	# GL/glext.h:1342
GL_QUAD_TEXTURE_SELECT_SGIS = 33061 	# GL/glext.h:1343
# SGIX_sprite (GL/glext.h:1346)
GL_SPRITE_SGIX = 33096 	# GL/glext.h:1347
GL_SPRITE_MODE_SGIX = 33097 	# GL/glext.h:1348
GL_SPRITE_AXIS_SGIX = 33098 	# GL/glext.h:1349
GL_SPRITE_TRANSLATION_SGIX = 33099 	# GL/glext.h:1350
GL_SPRITE_AXIAL_SGIX = 33100 	# GL/glext.h:1351
GL_SPRITE_OBJECT_ALIGNED_SGIX = 33101 	# GL/glext.h:1352
GL_SPRITE_EYE_ALIGNED_SGIX = 33102 	# GL/glext.h:1353
# SGIX_texture_multi_buffer (GL/glext.h:1356)
GL_TEXTURE_MULTI_BUFFER_HINT_SGIX = 33070 	# GL/glext.h:1357
# EXT_point_parameters (GL/glext.h:1360)
GL_POINT_SIZE_MIN_EXT = 33062 	# GL/glext.h:1361
GL_POINT_SIZE_MAX_EXT = 33063 	# GL/glext.h:1362
GL_POINT_FADE_THRESHOLD_SIZE_EXT = 33064 	# GL/glext.h:1363
GL_DISTANCE_ATTENUATION_EXT = 33065 	# GL/glext.h:1364
# SGIS_point_parameters (GL/glext.h:1367)
GL_POINT_SIZE_MIN_SGIS = 33062 	# GL/glext.h:1368
GL_POINT_SIZE_MAX_SGIS = 33063 	# GL/glext.h:1369
GL_POINT_FADE_THRESHOLD_SIZE_SGIS = 33064 	# GL/glext.h:1370
GL_DISTANCE_ATTENUATION_SGIS = 33065 	# GL/glext.h:1371
# SGIX_instruments (GL/glext.h:1374)
GL_INSTRUMENT_BUFFER_POINTER_SGIX = 33152 	# GL/glext.h:1375
GL_INSTRUMENT_MEASUREMENTS_SGIX = 33153 	# GL/glext.h:1376
# SGIX_texture_scale_bias (GL/glext.h:1379)
GL_POST_TEXTURE_FILTER_BIAS_SGIX = 33145 	# GL/glext.h:1380
GL_POST_TEXTURE_FILTER_SCALE_SGIX = 33146 	# GL/glext.h:1381
GL_POST_TEXTURE_FILTER_BIAS_RANGE_SGIX = 33147 	# GL/glext.h:1382
GL_POST_TEXTURE_FILTER_SCALE_RANGE_SGIX = 33148 	# GL/glext.h:1383
# SGIX_framezoom (GL/glext.h:1386)
GL_FRAMEZOOM_SGIX = 33163 	# GL/glext.h:1387
GL_FRAMEZOOM_FACTOR_SGIX = 33164 	# GL/glext.h:1388
GL_MAX_FRAMEZOOM_FACTOR_SGIX = 33165 	# GL/glext.h:1389
# SGIX_tag_sample_buffer (GL/glext.h:1392)
# FfdMaskSGIX (GL/glext.h:1395)
GL_TEXTURE_DEFORMATION_BIT_SGIX = 1 	# GL/glext.h:1396
GL_GEOMETRY_DEFORMATION_BIT_SGIX = 2 	# GL/glext.h:1397
# SGIX_polynomial_ffd (GL/glext.h:1400)
GL_GEOMETRY_DEFORMATION_SGIX = 33172 	# GL/glext.h:1401
GL_TEXTURE_DEFORMATION_SGIX = 33173 	# GL/glext.h:1402
GL_DEFORMATIONS_MASK_SGIX = 33174 	# GL/glext.h:1403
GL_MAX_DEFORMATION_ORDER_SGIX = 33175 	# GL/glext.h:1404
# SGIX_reference_plane (GL/glext.h:1407)
GL_REFERENCE_PLANE_SGIX = 33149 	# GL/glext.h:1408
GL_REFERENCE_PLANE_EQUATION_SGIX = 33150 	# GL/glext.h:1409
# SGIX_flush_raster (GL/glext.h:1412)
# SGIX_depth_texture (GL/glext.h:1415)
GL_DEPTH_COMPONENT16_SGIX = 33189 	# GL/glext.h:1416
GL_DEPTH_COMPONENT24_SGIX = 33190 	# GL/glext.h:1417
GL_DEPTH_COMPONENT32_SGIX = 33191 	# GL/glext.h:1418
# SGIS_fog_function (GL/glext.h:1421)
GL_FOG_FUNC_SGIS = 33066 	# GL/glext.h:1422
GL_FOG_FUNC_POINTS_SGIS = 33067 	# GL/glext.h:1423
GL_MAX_FOG_FUNC_POINTS_SGIS = 33068 	# GL/glext.h:1424
# SGIX_fog_offset (GL/glext.h:1427)
GL_FOG_OFFSET_SGIX = 33176 	# GL/glext.h:1428
GL_FOG_OFFSET_VALUE_SGIX = 33177 	# GL/glext.h:1429
# HP_image_transform (GL/glext.h:1432)
GL_IMAGE_SCALE_X_HP = 33109 	# GL/glext.h:1433
GL_IMAGE_SCALE_Y_HP = 33110 	# GL/glext.h:1434
GL_IMAGE_TRANSLATE_X_HP = 33111 	# GL/glext.h:1435
GL_IMAGE_TRANSLATE_Y_HP = 33112 	# GL/glext.h:1436
GL_IMAGE_ROTATE_ANGLE_HP = 33113 	# GL/glext.h:1437
GL_IMAGE_ROTATE_ORIGIN_X_HP = 33114 	# GL/glext.h:1438
GL_IMAGE_ROTATE_ORIGIN_Y_HP = 33115 	# GL/glext.h:1439
GL_IMAGE_MAG_FILTER_HP = 33116 	# GL/glext.h:1440
GL_IMAGE_MIN_FILTER_HP = 33117 	# GL/glext.h:1441
GL_IMAGE_CUBIC_WEIGHT_HP = 33118 	# GL/glext.h:1442
GL_CUBIC_HP = 33119 	# GL/glext.h:1443
GL_AVERAGE_HP = 33120 	# GL/glext.h:1444
GL_IMAGE_TRANSFORM_2D_HP = 33121 	# GL/glext.h:1445
GL_POST_IMAGE_TRANSFORM_COLOR_TABLE_HP = 33122 	# GL/glext.h:1446
GL_PROXY_POST_IMAGE_TRANSFORM_COLOR_TABLE_HP = 33123 	# GL/glext.h:1447
# HP_convolution_border_modes (GL/glext.h:1450)
GL_IGNORE_BORDER_HP = 33104 	# GL/glext.h:1451
GL_CONSTANT_BORDER_HP = 33105 	# GL/glext.h:1452
GL_REPLICATE_BORDER_HP = 33107 	# GL/glext.h:1453
GL_CONVOLUTION_BORDER_COLOR_HP = 33108 	# GL/glext.h:1454
# INGR_palette_buffer (GL/glext.h:1457)
# SGIX_texture_add_env (GL/glext.h:1460)
GL_TEXTURE_ENV_BIAS_SGIX = 32958 	# GL/glext.h:1461
# EXT_color_subtable (GL/glext.h:1464)
# PGI_vertex_hints (GL/glext.h:1467)
GL_VERTEX_DATA_HINT_PGI = 107050 	# GL/glext.h:1468
GL_VERTEX_CONSISTENT_HINT_PGI = 107051 	# GL/glext.h:1469
GL_MATERIAL_SIDE_HINT_PGI = 107052 	# GL/glext.h:1470
GL_MAX_VERTEX_HINT_PGI = 107053 	# GL/glext.h:1471
GL_COLOR3_BIT_PGI = 65536 	# GL/glext.h:1472
GL_COLOR4_BIT_PGI = 131072 	# GL/glext.h:1473
GL_EDGEFLAG_BIT_PGI = 262144 	# GL/glext.h:1474
GL_INDEX_BIT_PGI = 524288 	# GL/glext.h:1475
GL_MAT_AMBIENT_BIT_PGI = 1048576 	# GL/glext.h:1476
GL_MAT_AMBIENT_AND_DIFFUSE_BIT_PGI = 2097152 	# GL/glext.h:1477
GL_MAT_DIFFUSE_BIT_PGI = 4194304 	# GL/glext.h:1478
GL_MAT_EMISSION_BIT_PGI = 8388608 	# GL/glext.h:1479
GL_MAT_COLOR_INDEXES_BIT_PGI = 16777216 	# GL/glext.h:1480
GL_MAT_SHININESS_BIT_PGI = 33554432 	# GL/glext.h:1481
GL_MAT_SPECULAR_BIT_PGI = 67108864 	# GL/glext.h:1482
GL_NORMAL_BIT_PGI = 134217728 	# GL/glext.h:1483
GL_TEXCOORD1_BIT_PGI = 268435456 	# GL/glext.h:1484
GL_TEXCOORD2_BIT_PGI = 536870912 	# GL/glext.h:1485
GL_TEXCOORD3_BIT_PGI = 1073741824 	# GL/glext.h:1486
GL_TEXCOORD4_BIT_PGI = 2147483648 	# GL/glext.h:1487
GL_VERTEX23_BIT_PGI = 4 	# GL/glext.h:1488
GL_VERTEX4_BIT_PGI = 8 	# GL/glext.h:1489
# PGI_misc_hints (GL/glext.h:1492)
GL_PREFER_DOUBLEBUFFER_HINT_PGI = 107000 	# GL/glext.h:1493
GL_CONSERVE_MEMORY_HINT_PGI = 107005 	# GL/glext.h:1494
GL_RECLAIM_MEMORY_HINT_PGI = 107006 	# GL/glext.h:1495
GL_NATIVE_GRAPHICS_HANDLE_PGI = 107010 	# GL/glext.h:1496
GL_NATIVE_GRAPHICS_BEGIN_HINT_PGI = 107011 	# GL/glext.h:1497
GL_NATIVE_GRAPHICS_END_HINT_PGI = 107012 	# GL/glext.h:1498
GL_ALWAYS_FAST_HINT_PGI = 107020 	# GL/glext.h:1499
GL_ALWAYS_SOFT_HINT_PGI = 107021 	# GL/glext.h:1500
GL_ALLOW_DRAW_OBJ_HINT_PGI = 107022 	# GL/glext.h:1501
GL_ALLOW_DRAW_WIN_HINT_PGI = 107023 	# GL/glext.h:1502
GL_ALLOW_DRAW_FRG_HINT_PGI = 107024 	# GL/glext.h:1503
GL_ALLOW_DRAW_MEM_HINT_PGI = 107025 	# GL/glext.h:1504
GL_STRICT_DEPTHFUNC_HINT_PGI = 107030 	# GL/glext.h:1505
GL_STRICT_LIGHTING_HINT_PGI = 107031 	# GL/glext.h:1506
GL_STRICT_SCISSOR_HINT_PGI = 107032 	# GL/glext.h:1507
GL_FULL_STIPPLE_HINT_PGI = 107033 	# GL/glext.h:1508
GL_CLIP_NEAR_HINT_PGI = 107040 	# GL/glext.h:1509
GL_CLIP_FAR_HINT_PGI = 107041 	# GL/glext.h:1510
GL_WIDE_LINE_HINT_PGI = 107042 	# GL/glext.h:1511
GL_BACK_NORMALS_HINT_PGI = 107043 	# GL/glext.h:1512
# EXT_paletted_texture (GL/glext.h:1515)
GL_COLOR_INDEX1_EXT = 32994 	# GL/glext.h:1516
GL_COLOR_INDEX2_EXT = 32995 	# GL/glext.h:1517
GL_COLOR_INDEX4_EXT = 32996 	# GL/glext.h:1518
GL_COLOR_INDEX8_EXT = 32997 	# GL/glext.h:1519
GL_COLOR_INDEX12_EXT = 32998 	# GL/glext.h:1520
GL_COLOR_INDEX16_EXT = 32999 	# GL/glext.h:1521
GL_TEXTURE_INDEX_SIZE_EXT = 33005 	# GL/glext.h:1522
# EXT_clip_volume_hint (GL/glext.h:1525)
GL_CLIP_VOLUME_CLIPPING_HINT_EXT = 33008 	# GL/glext.h:1526
# SGIX_list_priority (GL/glext.h:1529)
GL_LIST_PRIORITY_SGIX = 33154 	# GL/glext.h:1530
# SGIX_ir_instrument1 (GL/glext.h:1533)
GL_IR_INSTRUMENT1_SGIX = 33151 	# GL/glext.h:1534
# SGIX_calligraphic_fragment (GL/glext.h:1537)
GL_CALLIGRAPHIC_FRAGMENT_SGIX = 33155 	# GL/glext.h:1538
# SGIX_texture_lod_bias (GL/glext.h:1541)
GL_TEXTURE_LOD_BIAS_S_SGIX = 33166 	# GL/glext.h:1542
GL_TEXTURE_LOD_BIAS_T_SGIX = 33167 	# GL/glext.h:1543
GL_TEXTURE_LOD_BIAS_R_SGIX = 33168 	# GL/glext.h:1544
# SGIX_shadow_ambient (GL/glext.h:1547)
GL_SHADOW_AMBIENT_SGIX = 32959 	# GL/glext.h:1548
# EXT_index_texture (GL/glext.h:1551)
# EXT_index_material (GL/glext.h:1554)
GL_INDEX_MATERIAL_EXT = 33208 	# GL/glext.h:1555
GL_INDEX_MATERIAL_PARAMETER_EXT = 33209 	# GL/glext.h:1556
GL_INDEX_MATERIAL_FACE_EXT = 33210 	# GL/glext.h:1557
# EXT_index_func (GL/glext.h:1560)
GL_INDEX_TEST_EXT = 33205 	# GL/glext.h:1561
GL_INDEX_TEST_FUNC_EXT = 33206 	# GL/glext.h:1562
GL_INDEX_TEST_REF_EXT = 33207 	# GL/glext.h:1563
# EXT_index_array_formats (GL/glext.h:1566)
GL_IUI_V2F_EXT = 33197 	# GL/glext.h:1567
GL_IUI_V3F_EXT = 33198 	# GL/glext.h:1568
GL_IUI_N3F_V2F_EXT = 33199 	# GL/glext.h:1569
GL_IUI_N3F_V3F_EXT = 33200 	# GL/glext.h:1570
GL_T2F_IUI_V2F_EXT = 33201 	# GL/glext.h:1571
GL_T2F_IUI_V3F_EXT = 33202 	# GL/glext.h:1572
GL_T2F_IUI_N3F_V2F_EXT = 33203 	# GL/glext.h:1573
GL_T2F_IUI_N3F_V3F_EXT = 33204 	# GL/glext.h:1574
# EXT_compiled_vertex_array (GL/glext.h:1577)
GL_ARRAY_ELEMENT_LOCK_FIRST_EXT = 33192 	# GL/glext.h:1578
GL_ARRAY_ELEMENT_LOCK_COUNT_EXT = 33193 	# GL/glext.h:1579
# EXT_cull_vertex (GL/glext.h:1582)
GL_CULL_VERTEX_EXT = 33194 	# GL/glext.h:1583
GL_CULL_VERTEX_EYE_POSITION_EXT = 33195 	# GL/glext.h:1584
GL_CULL_VERTEX_OBJECT_POSITION_EXT = 33196 	# GL/glext.h:1585
# SGIX_ycrcb (GL/glext.h:1588)
GL_YCRCB_422_SGIX = 33211 	# GL/glext.h:1589
GL_YCRCB_444_SGIX = 33212 	# GL/glext.h:1590
# SGIX_fragment_lighting (GL/glext.h:1593)
GL_FRAGMENT_LIGHTING_SGIX = 33792 	# GL/glext.h:1594
GL_FRAGMENT_COLOR_MATERIAL_SGIX = 33793 	# GL/glext.h:1595
GL_FRAGMENT_COLOR_MATERIAL_FACE_SGIX = 33794 	# GL/glext.h:1596
GL_FRAGMENT_COLOR_MATERIAL_PARAMETER_SGIX = 33795 	# GL/glext.h:1597
GL_MAX_FRAGMENT_LIGHTS_SGIX = 33796 	# GL/glext.h:1598
GL_MAX_ACTIVE_LIGHTS_SGIX = 33797 	# GL/glext.h:1599
GL_CURRENT_RASTER_NORMAL_SGIX = 33798 	# GL/glext.h:1600
GL_LIGHT_ENV_MODE_SGIX = 33799 	# GL/glext.h:1601
GL_FRAGMENT_LIGHT_MODEL_LOCAL_VIEWER_SGIX = 33800 	# GL/glext.h:1602
GL_FRAGMENT_LIGHT_MODEL_TWO_SIDE_SGIX = 33801 	# GL/glext.h:1603
GL_FRAGMENT_LIGHT_MODEL_AMBIENT_SGIX = 33802 	# GL/glext.h:1604
GL_FRAGMENT_LIGHT_MODEL_NORMAL_INTERPOLATION_SGIX = 33803 	# GL/glext.h:1605
GL_FRAGMENT_LIGHT0_SGIX = 33804 	# GL/glext.h:1606
GL_FRAGMENT_LIGHT1_SGIX = 33805 	# GL/glext.h:1607
GL_FRAGMENT_LIGHT2_SGIX = 33806 	# GL/glext.h:1608
GL_FRAGMENT_LIGHT3_SGIX = 33807 	# GL/glext.h:1609
GL_FRAGMENT_LIGHT4_SGIX = 33808 	# GL/glext.h:1610
GL_FRAGMENT_LIGHT5_SGIX = 33809 	# GL/glext.h:1611
GL_FRAGMENT_LIGHT6_SGIX = 33810 	# GL/glext.h:1612
GL_FRAGMENT_LIGHT7_SGIX = 33811 	# GL/glext.h:1613
# IBM_rasterpos_clip (GL/glext.h:1616)
GL_RASTER_POSITION_UNCLIPPED_IBM = 103010 	# GL/glext.h:1617
# HP_texture_lighting (GL/glext.h:1620)
GL_TEXTURE_LIGHTING_MODE_HP = 33127 	# GL/glext.h:1621
GL_TEXTURE_POST_SPECULAR_HP = 33128 	# GL/glext.h:1622
GL_TEXTURE_PRE_SPECULAR_HP = 33129 	# GL/glext.h:1623
# EXT_draw_range_elements (GL/glext.h:1626)
GL_MAX_ELEMENTS_VERTICES_EXT = 33000 	# GL/glext.h:1627
GL_MAX_ELEMENTS_INDICES_EXT = 33001 	# GL/glext.h:1628
# WIN_phong_shading (GL/glext.h:1631)
GL_PHONG_WIN = 33002 	# GL/glext.h:1632
GL_PHONG_HINT_WIN = 33003 	# GL/glext.h:1633
# WIN_specular_fog (GL/glext.h:1636)
GL_FOG_SPECULAR_TEXTURE_WIN = 33004 	# GL/glext.h:1637
# EXT_light_texture (GL/glext.h:1640)
GL_FRAGMENT_MATERIAL_EXT = 33609 	# GL/glext.h:1641
GL_FRAGMENT_NORMAL_EXT = 33610 	# GL/glext.h:1642
GL_FRAGMENT_COLOR_EXT = 33612 	# GL/glext.h:1643
GL_ATTENUATION_EXT = 33613 	# GL/glext.h:1644
GL_SHADOW_ATTENUATION_EXT = 33614 	# GL/glext.h:1645
GL_TEXTURE_APPLICATION_MODE_EXT = 33615 	# GL/glext.h:1646
GL_TEXTURE_LIGHT_EXT = 33616 	# GL/glext.h:1647
GL_TEXTURE_MATERIAL_FACE_EXT = 33617 	# GL/glext.h:1648
GL_TEXTURE_MATERIAL_PARAMETER_EXT = 33618 	# GL/glext.h:1649
# SGIX_blend_alpha_minmax (GL/glext.h:1653)
GL_ALPHA_MIN_SGIX = 33568 	# GL/glext.h:1654
GL_ALPHA_MAX_SGIX = 33569 	# GL/glext.h:1655
# SGIX_impact_pixel_texture (GL/glext.h:1658)
GL_PIXEL_TEX_GEN_Q_CEILING_SGIX = 33156 	# GL/glext.h:1659
GL_PIXEL_TEX_GEN_Q_ROUND_SGIX = 33157 	# GL/glext.h:1660
GL_PIXEL_TEX_GEN_Q_FLOOR_SGIX = 33158 	# GL/glext.h:1661
GL_PIXEL_TEX_GEN_ALPHA_REPLACE_SGIX = 33159 	# GL/glext.h:1662
GL_PIXEL_TEX_GEN_ALPHA_NO_REPLACE_SGIX = 33160 	# GL/glext.h:1663
GL_PIXEL_TEX_GEN_ALPHA_LS_SGIX = 33161 	# GL/glext.h:1664
GL_PIXEL_TEX_GEN_ALPHA_MS_SGIX = 33162 	# GL/glext.h:1665
# EXT_bgra (GL/glext.h:1668)
GL_BGR_EXT = 32992 	# GL/glext.h:1669
GL_BGRA_EXT = 32993 	# GL/glext.h:1670
# SGIX_async (GL/glext.h:1673)
GL_ASYNC_MARKER_SGIX = 33577 	# GL/glext.h:1674
# SGIX_async_pixel (GL/glext.h:1677)
GL_ASYNC_TEX_IMAGE_SGIX = 33628 	# GL/glext.h:1678
GL_ASYNC_DRAW_PIXELS_SGIX = 33629 	# GL/glext.h:1679
GL_ASYNC_READ_PIXELS_SGIX = 33630 	# GL/glext.h:1680
GL_MAX_ASYNC_TEX_IMAGE_SGIX = 33631 	# GL/glext.h:1681
GL_MAX_ASYNC_DRAW_PIXELS_SGIX = 33632 	# GL/glext.h:1682
GL_MAX_ASYNC_READ_PIXELS_SGIX = 33633 	# GL/glext.h:1683
# SGIX_async_histogram (GL/glext.h:1686)
GL_ASYNC_HISTOGRAM_SGIX = 33580 	# GL/glext.h:1687
GL_MAX_ASYNC_HISTOGRAM_SGIX = 33581 	# GL/glext.h:1688
# INTEL_texture_scissor (GL/glext.h:1691)
# INTEL_parallel_arrays (GL/glext.h:1694)
GL_PARALLEL_ARRAYS_INTEL = 33780 	# GL/glext.h:1695
GL_VERTEX_ARRAY_PARALLEL_POINTERS_INTEL = 33781 	# GL/glext.h:1696
GL_NORMAL_ARRAY_PARALLEL_POINTERS_INTEL = 33782 	# GL/glext.h:1697
GL_COLOR_ARRAY_PARALLEL_POINTERS_INTEL = 33783 	# GL/glext.h:1698
GL_TEXTURE_COORD_ARRAY_PARALLEL_POINTERS_INTEL = 33784 	# GL/glext.h:1699
# HP_occlusion_test (GL/glext.h:1702)
GL_OCCLUSION_TEST_HP = 33125 	# GL/glext.h:1703
GL_OCCLUSION_TEST_RESULT_HP = 33126 	# GL/glext.h:1704
# EXT_pixel_transform (GL/glext.h:1707)
GL_PIXEL_TRANSFORM_2D_EXT = 33584 	# GL/glext.h:1708
GL_PIXEL_MAG_FILTER_EXT = 33585 	# GL/glext.h:1709
GL_PIXEL_MIN_FILTER_EXT = 33586 	# GL/glext.h:1710
GL_PIXEL_CUBIC_WEIGHT_EXT = 33587 	# GL/glext.h:1711
GL_CUBIC_EXT = 33588 	# GL/glext.h:1712
GL_AVERAGE_EXT = 33589 	# GL/glext.h:1713
GL_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT = 33590 	# GL/glext.h:1714
GL_MAX_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT = 33591 	# GL/glext.h:1715
GL_PIXEL_TRANSFORM_2D_MATRIX_EXT = 33592 	# GL/glext.h:1716
# EXT_pixel_transform_color_table (GL/glext.h:1719)
# EXT_shared_texture_palette (GL/glext.h:1722)
GL_SHARED_TEXTURE_PALETTE_EXT = 33275 	# GL/glext.h:1723
# EXT_separate_specular_color (GL/glext.h:1726)
GL_LIGHT_MODEL_COLOR_CONTROL_EXT = 33272 	# GL/glext.h:1727
GL_SINGLE_COLOR_EXT = 33273 	# GL/glext.h:1728
GL_SEPARATE_SPECULAR_COLOR_EXT = 33274 	# GL/glext.h:1729
# EXT_secondary_color (GL/glext.h:1732)
GL_COLOR_SUM_EXT = 33880 	# GL/glext.h:1733
GL_CURRENT_SECONDARY_COLOR_EXT = 33881 	# GL/glext.h:1734
GL_SECONDARY_COLOR_ARRAY_SIZE_EXT = 33882 	# GL/glext.h:1735
GL_SECONDARY_COLOR_ARRAY_TYPE_EXT = 33883 	# GL/glext.h:1736
GL_SECONDARY_COLOR_ARRAY_STRIDE_EXT = 33884 	# GL/glext.h:1737
GL_SECONDARY_COLOR_ARRAY_POINTER_EXT = 33885 	# GL/glext.h:1738
GL_SECONDARY_COLOR_ARRAY_EXT = 33886 	# GL/glext.h:1739
# EXT_texture_perturb_normal (GL/glext.h:1742)
GL_PERTURB_EXT = 34222 	# GL/glext.h:1743
GL_TEXTURE_NORMAL_EXT = 34223 	# GL/glext.h:1744
# EXT_multi_draw_arrays (GL/glext.h:1747)
# EXT_fog_coord (GL/glext.h:1750)
GL_FOG_COORDINATE_SOURCE_EXT = 33872 	# GL/glext.h:1751
GL_FOG_COORDINATE_EXT = 33873 	# GL/glext.h:1752
GL_FRAGMENT_DEPTH_EXT = 33874 	# GL/glext.h:1753
GL_CURRENT_FOG_COORDINATE_EXT = 33875 	# GL/glext.h:1754
GL_FOG_COORDINATE_ARRAY_TYPE_EXT = 33876 	# GL/glext.h:1755
GL_FOG_COORDINATE_ARRAY_STRIDE_EXT = 33877 	# GL/glext.h:1756
GL_FOG_COORDINATE_ARRAY_POINTER_EXT = 33878 	# GL/glext.h:1757
GL_FOG_COORDINATE_ARRAY_EXT = 33879 	# GL/glext.h:1758
# REND_screen_coordinates (GL/glext.h:1761)
GL_SCREEN_COORDINATES_REND = 33936 	# GL/glext.h:1762
GL_INVERTED_SCREEN_W_REND = 33937 	# GL/glext.h:1763
# EXT_coordinate_frame (GL/glext.h:1766)
GL_TANGENT_ARRAY_EXT = 33849 	# GL/glext.h:1767
GL_BINORMAL_ARRAY_EXT = 33850 	# GL/glext.h:1768
GL_CURRENT_TANGENT_EXT = 33851 	# GL/glext.h:1769
GL_CURRENT_BINORMAL_EXT = 33852 	# GL/glext.h:1770
GL_TANGENT_ARRAY_TYPE_EXT = 33854 	# GL/glext.h:1771
GL_TANGENT_ARRAY_STRIDE_EXT = 33855 	# GL/glext.h:1772
GL_BINORMAL_ARRAY_TYPE_EXT = 33856 	# GL/glext.h:1773
GL_BINORMAL_ARRAY_STRIDE_EXT = 33857 	# GL/glext.h:1774
GL_TANGENT_ARRAY_POINTER_EXT = 33858 	# GL/glext.h:1775
GL_BINORMAL_ARRAY_POINTER_EXT = 33859 	# GL/glext.h:1776
GL_MAP1_TANGENT_EXT = 33860 	# GL/glext.h:1777
GL_MAP2_TANGENT_EXT = 33861 	# GL/glext.h:1778
GL_MAP1_BINORMAL_EXT = 33862 	# GL/glext.h:1779
GL_MAP2_BINORMAL_EXT = 33863 	# GL/glext.h:1780
# EXT_texture_env_combine (GL/glext.h:1783)
GL_COMBINE_EXT = 34160 	# GL/glext.h:1784
GL_COMBINE_RGB_EXT = 34161 	# GL/glext.h:1785
GL_COMBINE_ALPHA_EXT = 34162 	# GL/glext.h:1786
GL_RGB_SCALE_EXT = 34163 	# GL/glext.h:1787
GL_ADD_SIGNED_EXT = 34164 	# GL/glext.h:1788
GL_INTERPOLATE_EXT = 34165 	# GL/glext.h:1789
GL_CONSTANT_EXT = 34166 	# GL/glext.h:1790
GL_PRIMARY_COLOR_EXT = 34167 	# GL/glext.h:1791
GL_PREVIOUS_EXT = 34168 	# GL/glext.h:1792
GL_SOURCE0_RGB_EXT = 34176 	# GL/glext.h:1793
GL_SOURCE1_RGB_EXT = 34177 	# GL/glext.h:1794
GL_SOURCE2_RGB_EXT = 34178 	# GL/glext.h:1795
GL_SOURCE0_ALPHA_EXT = 34184 	# GL/glext.h:1796
GL_SOURCE1_ALPHA_EXT = 34185 	# GL/glext.h:1797
GL_SOURCE2_ALPHA_EXT = 34186 	# GL/glext.h:1798
GL_OPERAND0_RGB_EXT = 34192 	# GL/glext.h:1799
GL_OPERAND1_RGB_EXT = 34193 	# GL/glext.h:1800
GL_OPERAND2_RGB_EXT = 34194 	# GL/glext.h:1801
GL_OPERAND0_ALPHA_EXT = 34200 	# GL/glext.h:1802
GL_OPERAND1_ALPHA_EXT = 34201 	# GL/glext.h:1803
GL_OPERAND2_ALPHA_EXT = 34202 	# GL/glext.h:1804
# APPLE_specular_vector (GL/glext.h:1807)
GL_LIGHT_MODEL_SPECULAR_VECTOR_APPLE = 34224 	# GL/glext.h:1808
# APPLE_transform_hint (GL/glext.h:1811)
GL_TRANSFORM_HINT_APPLE = 34225 	# GL/glext.h:1812
# SGIX_fog_scale (GL/glext.h:1815)
GL_FOG_SCALE_SGIX = 33276 	# GL/glext.h:1816
GL_FOG_SCALE_VALUE_SGIX = 33277 	# GL/glext.h:1817
# SUNX_constant_data (GL/glext.h:1820)
GL_UNPACK_CONSTANT_DATA_SUNX = 33237 	# GL/glext.h:1821
GL_TEXTURE_CONSTANT_DATA_SUNX = 33238 	# GL/glext.h:1822
# SUN_global_alpha (GL/glext.h:1825)
GL_GLOBAL_ALPHA_SUN = 33241 	# GL/glext.h:1826
GL_GLOBAL_ALPHA_FACTOR_SUN = 33242 	# GL/glext.h:1827
# SUN_triangle_list (GL/glext.h:1830)
GL_RESTART_SUN = 1 	# GL/glext.h:1831
GL_REPLACE_MIDDLE_SUN = 2 	# GL/glext.h:1832
GL_REPLACE_OLDEST_SUN = 3 	# GL/glext.h:1833
GL_TRIANGLE_LIST_SUN = 33239 	# GL/glext.h:1834
GL_REPLACEMENT_CODE_SUN = 33240 	# GL/glext.h:1835
GL_REPLACEMENT_CODE_ARRAY_SUN = 34240 	# GL/glext.h:1836
GL_REPLACEMENT_CODE_ARRAY_TYPE_SUN = 34241 	# GL/glext.h:1837
GL_REPLACEMENT_CODE_ARRAY_STRIDE_SUN = 34242 	# GL/glext.h:1838
GL_REPLACEMENT_CODE_ARRAY_POINTER_SUN = 34243 	# GL/glext.h:1839
GL_R1UI_V3F_SUN = 34244 	# GL/glext.h:1840
GL_R1UI_C4UB_V3F_SUN = 34245 	# GL/glext.h:1841
GL_R1UI_C3F_V3F_SUN = 34246 	# GL/glext.h:1842
GL_R1UI_N3F_V3F_SUN = 34247 	# GL/glext.h:1843
GL_R1UI_C4F_N3F_V3F_SUN = 34248 	# GL/glext.h:1844
GL_R1UI_T2F_V3F_SUN = 34249 	# GL/glext.h:1845
GL_R1UI_T2F_N3F_V3F_SUN = 34250 	# GL/glext.h:1846
GL_R1UI_T2F_C4F_N3F_V3F_SUN = 34251 	# GL/glext.h:1847
# SUN_vertex (GL/glext.h:1850)
# EXT_blend_func_separate (GL/glext.h:1853)
GL_BLEND_DST_RGB_EXT = 32968 	# GL/glext.h:1854
GL_BLEND_SRC_RGB_EXT = 32969 	# GL/glext.h:1855
GL_BLEND_DST_ALPHA_EXT = 32970 	# GL/glext.h:1856
GL_BLEND_SRC_ALPHA_EXT = 32971 	# GL/glext.h:1857
# INGR_color_clamp (GL/glext.h:1860)
GL_RED_MIN_CLAMP_INGR = 34144 	# GL/glext.h:1861
GL_GREEN_MIN_CLAMP_INGR = 34145 	# GL/glext.h:1862
GL_BLUE_MIN_CLAMP_INGR = 34146 	# GL/glext.h:1863
GL_ALPHA_MIN_CLAMP_INGR = 34147 	# GL/glext.h:1864
GL_RED_MAX_CLAMP_INGR = 34148 	# GL/glext.h:1865
GL_GREEN_MAX_CLAMP_INGR = 34149 	# GL/glext.h:1866
GL_BLUE_MAX_CLAMP_INGR = 34150 	# GL/glext.h:1867
GL_ALPHA_MAX_CLAMP_INGR = 34151 	# GL/glext.h:1868
# INGR_interlace_read (GL/glext.h:1871)
GL_INTERLACE_READ_INGR = 34152 	# GL/glext.h:1872
# EXT_stencil_wrap (GL/glext.h:1875)
GL_INCR_WRAP_EXT = 34055 	# GL/glext.h:1876
GL_DECR_WRAP_EXT = 34056 	# GL/glext.h:1877
# EXT_422_pixels (GL/glext.h:1880)
GL_422_EXT = 32972 	# GL/glext.h:1881
GL_422_REV_EXT = 32973 	# GL/glext.h:1882
GL_422_AVERAGE_EXT = 32974 	# GL/glext.h:1883
GL_422_REV_AVERAGE_EXT = 32975 	# GL/glext.h:1884
# NV_texgen_reflection (GL/glext.h:1887)
GL_NORMAL_MAP_NV = 34065 	# GL/glext.h:1888
GL_REFLECTION_MAP_NV = 34066 	# GL/glext.h:1889
# EXT_texture_cube_map (GL/glext.h:1892)
GL_NORMAL_MAP_EXT = 34065 	# GL/glext.h:1893
GL_REFLECTION_MAP_EXT = 34066 	# GL/glext.h:1894
GL_TEXTURE_CUBE_MAP_EXT = 34067 	# GL/glext.h:1895
GL_TEXTURE_BINDING_CUBE_MAP_EXT = 34068 	# GL/glext.h:1896
GL_TEXTURE_CUBE_MAP_POSITIVE_X_EXT = 34069 	# GL/glext.h:1897
GL_TEXTURE_CUBE_MAP_NEGATIVE_X_EXT = 34070 	# GL/glext.h:1898
GL_TEXTURE_CUBE_MAP_POSITIVE_Y_EXT = 34071 	# GL/glext.h:1899
GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_EXT = 34072 	# GL/glext.h:1900
GL_TEXTURE_CUBE_MAP_POSITIVE_Z_EXT = 34073 	# GL/glext.h:1901
GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_EXT = 34074 	# GL/glext.h:1902
GL_PROXY_TEXTURE_CUBE_MAP_EXT = 34075 	# GL/glext.h:1903
GL_MAX_CUBE_MAP_TEXTURE_SIZE_EXT = 34076 	# GL/glext.h:1904
# SUN_convolution_border_modes (GL/glext.h:1907)
GL_WRAP_BORDER_SUN = 33236 	# GL/glext.h:1908
# EXT_texture_env_add (GL/glext.h:1911)
# EXT_texture_lod_bias (GL/glext.h:1914)
GL_MAX_TEXTURE_LOD_BIAS_EXT = 34045 	# GL/glext.h:1915
GL_TEXTURE_FILTER_CONTROL_EXT = 34048 	# GL/glext.h:1916
GL_TEXTURE_LOD_BIAS_EXT = 34049 	# GL/glext.h:1917
# EXT_texture_filter_anisotropic (GL/glext.h:1920)
GL_TEXTURE_MAX_ANISOTROPY_EXT = 34046 	# GL/glext.h:1921
GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT = 34047 	# GL/glext.h:1922
# EXT_vertex_weighting (GL/glext.h:1925)
GL_MODELVIEW0_STACK_DEPTH_EXT = 0 	# GL/glext.h:1926
GL_MODELVIEW1_STACK_DEPTH_EXT = 34050 	# GL/glext.h:1927
GL_MODELVIEW0_MATRIX_EXT = 0 	# GL/glext.h:1928
GL_MODELVIEW1_MATRIX_EXT = 34054 	# GL/glext.h:1929
GL_VERTEX_WEIGHTING_EXT = 34057 	# GL/glext.h:1930
GL_MODELVIEW0_EXT = 0 	# GL/glext.h:1931
GL_MODELVIEW1_EXT = 34058 	# GL/glext.h:1932
GL_CURRENT_VERTEX_WEIGHT_EXT = 34059 	# GL/glext.h:1933
GL_VERTEX_WEIGHT_ARRAY_EXT = 34060 	# GL/glext.h:1934
GL_VERTEX_WEIGHT_ARRAY_SIZE_EXT = 34061 	# GL/glext.h:1935
GL_VERTEX_WEIGHT_ARRAY_TYPE_EXT = 34062 	# GL/glext.h:1936
GL_VERTEX_WEIGHT_ARRAY_STRIDE_EXT = 34063 	# GL/glext.h:1937
GL_VERTEX_WEIGHT_ARRAY_POINTER_EXT = 34064 	# GL/glext.h:1938
# NV_light_max_exponent (GL/glext.h:1941)
GL_MAX_SHININESS_NV = 34052 	# GL/glext.h:1942
GL_MAX_SPOT_EXPONENT_NV = 34053 	# GL/glext.h:1943
# NV_vertex_array_range (GL/glext.h:1946)
GL_VERTEX_ARRAY_RANGE_NV = 34077 	# GL/glext.h:1947
GL_VERTEX_ARRAY_RANGE_LENGTH_NV = 34078 	# GL/glext.h:1948
GL_VERTEX_ARRAY_RANGE_VALID_NV = 34079 	# GL/glext.h:1949
GL_MAX_VERTEX_ARRAY_RANGE_ELEMENT_NV = 34080 	# GL/glext.h:1950
GL_VERTEX_ARRAY_RANGE_POINTER_NV = 34081 	# GL/glext.h:1951
# NV_register_combiners (GL/glext.h:1954)
GL_REGISTER_COMBINERS_NV = 34082 	# GL/glext.h:1955
GL_VARIABLE_A_NV = 34083 	# GL/glext.h:1956
GL_VARIABLE_B_NV = 34084 	# GL/glext.h:1957
GL_VARIABLE_C_NV = 34085 	# GL/glext.h:1958
GL_VARIABLE_D_NV = 34086 	# GL/glext.h:1959
GL_VARIABLE_E_NV = 34087 	# GL/glext.h:1960
GL_VARIABLE_F_NV = 34088 	# GL/glext.h:1961
GL_VARIABLE_G_NV = 34089 	# GL/glext.h:1962
GL_CONSTANT_COLOR0_NV = 34090 	# GL/glext.h:1963
GL_CONSTANT_COLOR1_NV = 34091 	# GL/glext.h:1964
GL_PRIMARY_COLOR_NV = 34092 	# GL/glext.h:1965
GL_SECONDARY_COLOR_NV = 34093 	# GL/glext.h:1966
GL_SPARE0_NV = 34094 	# GL/glext.h:1967
GL_SPARE1_NV = 34095 	# GL/glext.h:1968
GL_DISCARD_NV = 34096 	# GL/glext.h:1969
GL_E_TIMES_F_NV = 34097 	# GL/glext.h:1970
GL_SPARE0_PLUS_SECONDARY_COLOR_NV = 34098 	# GL/glext.h:1971
GL_UNSIGNED_IDENTITY_NV = 34102 	# GL/glext.h:1972
GL_UNSIGNED_INVERT_NV = 34103 	# GL/glext.h:1973
GL_EXPAND_NORMAL_NV = 34104 	# GL/glext.h:1974
GL_EXPAND_NEGATE_NV = 34105 	# GL/glext.h:1975
GL_HALF_BIAS_NORMAL_NV = 34106 	# GL/glext.h:1976
GL_HALF_BIAS_NEGATE_NV = 34107 	# GL/glext.h:1977
GL_SIGNED_IDENTITY_NV = 34108 	# GL/glext.h:1978
GL_SIGNED_NEGATE_NV = 34109 	# GL/glext.h:1979
GL_SCALE_BY_TWO_NV = 34110 	# GL/glext.h:1980
GL_SCALE_BY_FOUR_NV = 34111 	# GL/glext.h:1981
GL_SCALE_BY_ONE_HALF_NV = 34112 	# GL/glext.h:1982
GL_BIAS_BY_NEGATIVE_ONE_HALF_NV = 34113 	# GL/glext.h:1983
GL_COMBINER_INPUT_NV = 34114 	# GL/glext.h:1984
GL_COMBINER_MAPPING_NV = 34115 	# GL/glext.h:1985
GL_COMBINER_COMPONENT_USAGE_NV = 34116 	# GL/glext.h:1986
GL_COMBINER_AB_DOT_PRODUCT_NV = 34117 	# GL/glext.h:1987
GL_COMBINER_CD_DOT_PRODUCT_NV = 34118 	# GL/glext.h:1988
GL_COMBINER_MUX_SUM_NV = 34119 	# GL/glext.h:1989
GL_COMBINER_SCALE_NV = 34120 	# GL/glext.h:1990
GL_COMBINER_BIAS_NV = 34121 	# GL/glext.h:1991
GL_COMBINER_AB_OUTPUT_NV = 34122 	# GL/glext.h:1992
GL_COMBINER_CD_OUTPUT_NV = 34123 	# GL/glext.h:1993
GL_COMBINER_SUM_OUTPUT_NV = 34124 	# GL/glext.h:1994
GL_MAX_GENERAL_COMBINERS_NV = 34125 	# GL/glext.h:1995
GL_NUM_GENERAL_COMBINERS_NV = 34126 	# GL/glext.h:1996
GL_COLOR_SUM_CLAMP_NV = 34127 	# GL/glext.h:1997
GL_COMBINER0_NV = 34128 	# GL/glext.h:1998
GL_COMBINER1_NV = 34129 	# GL/glext.h:1999
GL_COMBINER2_NV = 34130 	# GL/glext.h:2000
GL_COMBINER3_NV = 34131 	# GL/glext.h:2001
GL_COMBINER4_NV = 34132 	# GL/glext.h:2002
GL_COMBINER5_NV = 34133 	# GL/glext.h:2003
GL_COMBINER6_NV = 34134 	# GL/glext.h:2004
GL_COMBINER7_NV = 34135 	# GL/glext.h:2005
# NV_fog_distance (GL/glext.h:2013)
GL_FOG_DISTANCE_MODE_NV = 34138 	# GL/glext.h:2014
GL_EYE_RADIAL_NV = 34139 	# GL/glext.h:2015
GL_EYE_PLANE_ABSOLUTE_NV = 34140 	# GL/glext.h:2016
# NV_texgen_emboss (GL/glext.h:2020)
GL_EMBOSS_LIGHT_NV = 34141 	# GL/glext.h:2021
GL_EMBOSS_CONSTANT_NV = 34142 	# GL/glext.h:2022
GL_EMBOSS_MAP_NV = 34143 	# GL/glext.h:2023
# NV_blend_square (GL/glext.h:2026)
# NV_texture_env_combine4 (GL/glext.h:2029)
GL_COMBINE4_NV = 34051 	# GL/glext.h:2030
GL_SOURCE3_RGB_NV = 34179 	# GL/glext.h:2031
GL_SOURCE3_ALPHA_NV = 34187 	# GL/glext.h:2032
GL_OPERAND3_RGB_NV = 34195 	# GL/glext.h:2033
GL_OPERAND3_ALPHA_NV = 34203 	# GL/glext.h:2034
# MESA_resize_buffers (GL/glext.h:2037)
# MESA_window_pos (GL/glext.h:2040)
# EXT_texture_compression_s3tc (GL/glext.h:2043)
GL_COMPRESSED_RGB_S3TC_DXT1_EXT = 33776 	# GL/glext.h:2044
GL_COMPRESSED_RGBA_S3TC_DXT1_EXT = 33777 	# GL/glext.h:2045
GL_COMPRESSED_RGBA_S3TC_DXT3_EXT = 33778 	# GL/glext.h:2046
GL_COMPRESSED_RGBA_S3TC_DXT5_EXT = 33779 	# GL/glext.h:2047
# IBM_cull_vertex (GL/glext.h:2050)
GL_CULL_VERTEX_IBM = 103050 	# GL/glext.h:2051
# IBM_multimode_draw_arrays (GL/glext.h:2054)
# IBM_vertex_array_lists (GL/glext.h:2057)
GL_VERTEX_ARRAY_LIST_IBM = 103070 	# GL/glext.h:2058
GL_NORMAL_ARRAY_LIST_IBM = 103071 	# GL/glext.h:2059
GL_COLOR_ARRAY_LIST_IBM = 103072 	# GL/glext.h:2060
GL_INDEX_ARRAY_LIST_IBM = 103073 	# GL/glext.h:2061
GL_TEXTURE_COORD_ARRAY_LIST_IBM = 103074 	# GL/glext.h:2062
GL_EDGE_FLAG_ARRAY_LIST_IBM = 103075 	# GL/glext.h:2063
GL_FOG_COORDINATE_ARRAY_LIST_IBM = 103076 	# GL/glext.h:2064
GL_SECONDARY_COLOR_ARRAY_LIST_IBM = 103077 	# GL/glext.h:2065
GL_VERTEX_ARRAY_LIST_STRIDE_IBM = 103080 	# GL/glext.h:2066
GL_NORMAL_ARRAY_LIST_STRIDE_IBM = 103081 	# GL/glext.h:2067
GL_COLOR_ARRAY_LIST_STRIDE_IBM = 103082 	# GL/glext.h:2068
GL_INDEX_ARRAY_LIST_STRIDE_IBM = 103083 	# GL/glext.h:2069
GL_TEXTURE_COORD_ARRAY_LIST_STRIDE_IBM = 103084 	# GL/glext.h:2070
GL_EDGE_FLAG_ARRAY_LIST_STRIDE_IBM = 103085 	# GL/glext.h:2071
GL_FOG_COORDINATE_ARRAY_LIST_STRIDE_IBM = 103086 	# GL/glext.h:2072
GL_SECONDARY_COLOR_ARRAY_LIST_STRIDE_IBM = 103087 	# GL/glext.h:2073
# SGIX_subsample (GL/glext.h:2076)
GL_PACK_SUBSAMPLE_RATE_SGIX = 34208 	# GL/glext.h:2077
GL_UNPACK_SUBSAMPLE_RATE_SGIX = 34209 	# GL/glext.h:2078
GL_PIXEL_SUBSAMPLE_4444_SGIX = 34210 	# GL/glext.h:2079
GL_PIXEL_SUBSAMPLE_2424_SGIX = 34211 	# GL/glext.h:2080
GL_PIXEL_SUBSAMPLE_4242_SGIX = 34212 	# GL/glext.h:2081
# SGIX_ycrcb_subsample (GL/glext.h:2084)
# SGIX_ycrcba (GL/glext.h:2087)
GL_YCRCB_SGIX = 33560 	# GL/glext.h:2088
GL_YCRCBA_SGIX = 33561 	# GL/glext.h:2089
# SGI_depth_pass_instrument (GL/glext.h:2092)
GL_DEPTH_PASS_INSTRUMENT_SGIX = 33552 	# GL/glext.h:2093
GL_DEPTH_PASS_INSTRUMENT_COUNTERS_SGIX = 33553 	# GL/glext.h:2094
GL_DEPTH_PASS_INSTRUMENT_MAX_SGIX = 33554 	# GL/glext.h:2095
# 3DFX_texture_compression_FXT1 (GL/glext.h:2098)
GL_COMPRESSED_RGB_FXT1_3DFX = 34480 	# GL/glext.h:2099
GL_COMPRESSED_RGBA_FXT1_3DFX = 34481 	# GL/glext.h:2100
# 3DFX_multisample (GL/glext.h:2103)
GL_MULTISAMPLE_3DFX = 34482 	# GL/glext.h:2104
GL_SAMPLE_BUFFERS_3DFX = 34483 	# GL/glext.h:2105
GL_SAMPLES_3DFX = 34484 	# GL/glext.h:2106
GL_MULTISAMPLE_BIT_3DFX = 536870912 	# GL/glext.h:2107
# 3DFX_tbuffer (GL/glext.h:2110)
# EXT_multisample (GL/glext.h:2113)
GL_MULTISAMPLE_EXT = 32925 	# GL/glext.h:2114
GL_SAMPLE_ALPHA_TO_MASK_EXT = 32926 	# GL/glext.h:2115
GL_SAMPLE_ALPHA_TO_ONE_EXT = 32927 	# GL/glext.h:2116
GL_SAMPLE_MASK_EXT = 32928 	# GL/glext.h:2117
GL_1PASS_EXT = 32929 	# GL/glext.h:2118
GL_2PASS_0_EXT = 32930 	# GL/glext.h:2119
GL_2PASS_1_EXT = 32931 	# GL/glext.h:2120
GL_4PASS_0_EXT = 32932 	# GL/glext.h:2121
GL_4PASS_1_EXT = 32933 	# GL/glext.h:2122
GL_4PASS_2_EXT = 32934 	# GL/glext.h:2123
GL_4PASS_3_EXT = 32935 	# GL/glext.h:2124
GL_SAMPLE_BUFFERS_EXT = 32936 	# GL/glext.h:2125
GL_SAMPLES_EXT = 32937 	# GL/glext.h:2126
GL_SAMPLE_MASK_VALUE_EXT = 32938 	# GL/glext.h:2127
GL_SAMPLE_MASK_INVERT_EXT = 32939 	# GL/glext.h:2128
GL_SAMPLE_PATTERN_EXT = 32940 	# GL/glext.h:2129
GL_MULTISAMPLE_BIT_EXT = 536870912 	# GL/glext.h:2130
# SGIX_vertex_preclip (GL/glext.h:2133)
GL_VERTEX_PRECLIP_SGIX = 33774 	# GL/glext.h:2134
GL_VERTEX_PRECLIP_HINT_SGIX = 33775 	# GL/glext.h:2135
# SGIX_convolution_accuracy (GL/glext.h:2138)
GL_CONVOLUTION_HINT_SGIX = 33558 	# GL/glext.h:2139
# SGIX_resample (GL/glext.h:2142)
GL_PACK_RESAMPLE_SGIX = 33836 	# GL/glext.h:2143
GL_UNPACK_RESAMPLE_SGIX = 33837 	# GL/glext.h:2144
GL_RESAMPLE_REPLICATE_SGIX = 33838 	# GL/glext.h:2145
GL_RESAMPLE_ZERO_FILL_SGIX = 33839 	# GL/glext.h:2146
GL_RESAMPLE_DECIMATE_SGIX = 33840 	# GL/glext.h:2147
# SGIS_point_line_texgen (GL/glext.h:2150)
GL_EYE_DISTANCE_TO_POINT_SGIS = 33264 	# GL/glext.h:2151
GL_OBJECT_DISTANCE_TO_POINT_SGIS = 33265 	# GL/glext.h:2152
GL_EYE_DISTANCE_TO_LINE_SGIS = 33266 	# GL/glext.h:2153
GL_OBJECT_DISTANCE_TO_LINE_SGIS = 33267 	# GL/glext.h:2154
GL_EYE_POINT_SGIS = 33268 	# GL/glext.h:2155
GL_OBJECT_POINT_SGIS = 33269 	# GL/glext.h:2156
GL_EYE_LINE_SGIS = 33270 	# GL/glext.h:2157
GL_OBJECT_LINE_SGIS = 33271 	# GL/glext.h:2158
# SGIS_texture_color_mask (GL/glext.h:2161)
GL_TEXTURE_COLOR_WRITEMASK_SGIS = 33263 	# GL/glext.h:2162
# EXT_texture_env_dot3 (GL/glext.h:2165)
GL_DOT3_RGB_EXT = 34624 	# GL/glext.h:2166
GL_DOT3_RGBA_EXT = 34625 	# GL/glext.h:2167
# ATI_texture_mirror_once (GL/glext.h:2170)
GL_MIRROR_CLAMP_ATI = 34626 	# GL/glext.h:2171
GL_MIRROR_CLAMP_TO_EDGE_ATI = 34627 	# GL/glext.h:2172
# NV_fence (GL/glext.h:2175)
GL_ALL_COMPLETED_NV = 34034 	# GL/glext.h:2176
GL_FENCE_STATUS_NV = 34035 	# GL/glext.h:2177
GL_FENCE_CONDITION_NV = 34036 	# GL/glext.h:2178
# IBM_texture_mirrored_repeat (GL/glext.h:2181)
GL_MIRRORED_REPEAT_IBM = 33648 	# GL/glext.h:2182
# NV_evaluators (GL/glext.h:2185)
GL_EVAL_2D_NV = 34496 	# GL/glext.h:2186
GL_EVAL_TRIANGULAR_2D_NV = 34497 	# GL/glext.h:2187
GL_MAP_TESSELLATION_NV = 34498 	# GL/glext.h:2188
GL_MAP_ATTRIB_U_ORDER_NV = 34499 	# GL/glext.h:2189
GL_MAP_ATTRIB_V_ORDER_NV = 34500 	# GL/glext.h:2190
GL_EVAL_FRACTIONAL_TESSELLATION_NV = 34501 	# GL/glext.h:2191
GL_EVAL_VERTEX_ATTRIB0_NV = 34502 	# GL/glext.h:2192
GL_EVAL_VERTEX_ATTRIB1_NV = 34503 	# GL/glext.h:2193
GL_EVAL_VERTEX_ATTRIB2_NV = 34504 	# GL/glext.h:2194
GL_EVAL_VERTEX_ATTRIB3_NV = 34505 	# GL/glext.h:2195
GL_EVAL_VERTEX_ATTRIB4_NV = 34506 	# GL/glext.h:2196
GL_EVAL_VERTEX_ATTRIB5_NV = 34507 	# GL/glext.h:2197
GL_EVAL_VERTEX_ATTRIB6_NV = 34508 	# GL/glext.h:2198
GL_EVAL_VERTEX_ATTRIB7_NV = 34509 	# GL/glext.h:2199
GL_EVAL_VERTEX_ATTRIB8_NV = 34510 	# GL/glext.h:2200
GL_EVAL_VERTEX_ATTRIB9_NV = 34511 	# GL/glext.h:2201
GL_EVAL_VERTEX_ATTRIB10_NV = 34512 	# GL/glext.h:2202
GL_EVAL_VERTEX_ATTRIB11_NV = 34513 	# GL/glext.h:2203
GL_EVAL_VERTEX_ATTRIB12_NV = 34514 	# GL/glext.h:2204
GL_EVAL_VERTEX_ATTRIB13_NV = 34515 	# GL/glext.h:2205
GL_EVAL_VERTEX_ATTRIB14_NV = 34516 	# GL/glext.h:2206
GL_EVAL_VERTEX_ATTRIB15_NV = 34517 	# GL/glext.h:2207
GL_MAX_MAP_TESSELLATION_NV = 34518 	# GL/glext.h:2208
GL_MAX_RATIONAL_EVAL_ORDER_NV = 34519 	# GL/glext.h:2209
# NV_packed_depth_stencil (GL/glext.h:2212)
GL_DEPTH_STENCIL_NV = 34041 	# GL/glext.h:2213
GL_UNSIGNED_INT_24_8_NV = 34042 	# GL/glext.h:2214
# NV_register_combiners2 (GL/glext.h:2217)
GL_PER_STAGE_CONSTANTS_NV = 34101 	# GL/glext.h:2218
# NV_texture_compression_vtc (GL/glext.h:2221)
# NV_texture_rectangle (GL/glext.h:2224)
GL_TEXTURE_RECTANGLE_NV = 34037 	# GL/glext.h:2225
GL_TEXTURE_BINDING_RECTANGLE_NV = 34038 	# GL/glext.h:2226
GL_PROXY_TEXTURE_RECTANGLE_NV = 34039 	# GL/glext.h:2227
GL_MAX_RECTANGLE_TEXTURE_SIZE_NV = 34040 	# GL/glext.h:2228
# NV_texture_shader (GL/glext.h:2231)
GL_OFFSET_TEXTURE_RECTANGLE_NV = 34380 	# GL/glext.h:2232
GL_OFFSET_TEXTURE_RECTANGLE_SCALE_NV = 34381 	# GL/glext.h:2233
GL_DOT_PRODUCT_TEXTURE_RECTANGLE_NV = 34382 	# GL/glext.h:2234
GL_RGBA_UNSIGNED_DOT_PRODUCT_MAPPING_NV = 34521 	# GL/glext.h:2235
GL_UNSIGNED_INT_S8_S8_8_8_NV = 34522 	# GL/glext.h:2236
GL_UNSIGNED_INT_8_8_S8_S8_REV_NV = 34523 	# GL/glext.h:2237
GL_DSDT_MAG_INTENSITY_NV = 34524 	# GL/glext.h:2238
GL_SHADER_CONSISTENT_NV = 34525 	# GL/glext.h:2239
GL_TEXTURE_SHADER_NV = 34526 	# GL/glext.h:2240
GL_SHADER_OPERATION_NV = 34527 	# GL/glext.h:2241
GL_CULL_MODES_NV = 34528 	# GL/glext.h:2242
GL_OFFSET_TEXTURE_MATRIX_NV = 34529 	# GL/glext.h:2243
GL_OFFSET_TEXTURE_SCALE_NV = 34530 	# GL/glext.h:2244
GL_OFFSET_TEXTURE_BIAS_NV = 34531 	# GL/glext.h:2245
GL_OFFSET_TEXTURE_2D_MATRIX_NV = 34529 	# GL/glext.h:2246
GL_OFFSET_TEXTURE_2D_SCALE_NV = 34530 	# GL/glext.h:2247
GL_OFFSET_TEXTURE_2D_BIAS_NV = 34531 	# GL/glext.h:2248
GL_PREVIOUS_TEXTURE_INPUT_NV = 34532 	# GL/glext.h:2249
GL_CONST_EYE_NV = 34533 	# GL/glext.h:2250
GL_PASS_THROUGH_NV = 34534 	# GL/glext.h:2251
GL_CULL_FRAGMENT_NV = 34535 	# GL/glext.h:2252
GL_OFFSET_TEXTURE_2D_NV = 34536 	# GL/glext.h:2253
GL_DEPENDENT_AR_TEXTURE_2D_NV = 34537 	# GL/glext.h:2254
GL_DEPENDENT_GB_TEXTURE_2D_NV = 34538 	# GL/glext.h:2255
GL_DOT_PRODUCT_NV = 34540 	# GL/glext.h:2256
GL_DOT_PRODUCT_DEPTH_REPLACE_NV = 34541 	# GL/glext.h:2257
GL_DOT_PRODUCT_TEXTURE_2D_NV = 34542 	# GL/glext.h:2258
GL_DOT_PRODUCT_TEXTURE_CUBE_MAP_NV = 34544 	# GL/glext.h:2259
GL_DOT_PRODUCT_DIFFUSE_CUBE_MAP_NV = 34545 	# GL/glext.h:2260
GL_DOT_PRODUCT_REFLECT_CUBE_MAP_NV = 34546 	# GL/glext.h:2261
GL_DOT_PRODUCT_CONST_EYE_REFLECT_CUBE_MAP_NV = 34547 	# GL/glext.h:2262
GL_HILO_NV = 34548 	# GL/glext.h:2263
GL_DSDT_NV = 34549 	# GL/glext.h:2264
GL_DSDT_MAG_NV = 34550 	# GL/glext.h:2265
GL_DSDT_MAG_VIB_NV = 34551 	# GL/glext.h:2266
GL_HILO16_NV = 34552 	# GL/glext.h:2267
GL_SIGNED_HILO_NV = 34553 	# GL/glext.h:2268
GL_SIGNED_HILO16_NV = 34554 	# GL/glext.h:2269
GL_SIGNED_RGBA_NV = 34555 	# GL/glext.h:2270
GL_SIGNED_RGBA8_NV = 34556 	# GL/glext.h:2271
GL_SIGNED_RGB_NV = 34558 	# GL/glext.h:2272
GL_SIGNED_RGB8_NV = 34559 	# GL/glext.h:2273
GL_SIGNED_LUMINANCE_NV = 34561 	# GL/glext.h:2274
GL_SIGNED_LUMINANCE8_NV = 34562 	# GL/glext.h:2275
GL_SIGNED_LUMINANCE_ALPHA_NV = 34563 	# GL/glext.h:2276
GL_SIGNED_LUMINANCE8_ALPHA8_NV = 34564 	# GL/glext.h:2277
GL_SIGNED_ALPHA_NV = 34565 	# GL/glext.h:2278
GL_SIGNED_ALPHA8_NV = 34566 	# GL/glext.h:2279
GL_SIGNED_INTENSITY_NV = 34567 	# GL/glext.h:2280
GL_SIGNED_INTENSITY8_NV = 34568 	# GL/glext.h:2281
GL_DSDT8_NV = 34569 	# GL/glext.h:2282
GL_DSDT8_MAG8_NV = 34570 	# GL/glext.h:2283
GL_DSDT8_MAG8_INTENSITY8_NV = 34571 	# GL/glext.h:2284
GL_SIGNED_RGB_UNSIGNED_ALPHA_NV = 34572 	# GL/glext.h:2285
GL_SIGNED_RGB8_UNSIGNED_ALPHA8_NV = 34573 	# GL/glext.h:2286
GL_HI_SCALE_NV = 34574 	# GL/glext.h:2287
GL_LO_SCALE_NV = 34575 	# GL/glext.h:2288
GL_DS_SCALE_NV = 34576 	# GL/glext.h:2289
GL_DT_SCALE_NV = 34577 	# GL/glext.h:2290
GL_MAGNITUDE_SCALE_NV = 34578 	# GL/glext.h:2291
GL_VIBRANCE_SCALE_NV = 34579 	# GL/glext.h:2292
GL_HI_BIAS_NV = 34580 	# GL/glext.h:2293
GL_LO_BIAS_NV = 34581 	# GL/glext.h:2294
GL_DS_BIAS_NV = 34582 	# GL/glext.h:2295
GL_DT_BIAS_NV = 34583 	# GL/glext.h:2296
GL_MAGNITUDE_BIAS_NV = 34584 	# GL/glext.h:2297
GL_VIBRANCE_BIAS_NV = 34585 	# GL/glext.h:2298
GL_TEXTURE_BORDER_VALUES_NV = 34586 	# GL/glext.h:2299
GL_TEXTURE_HI_SIZE_NV = 34587 	# GL/glext.h:2300
GL_TEXTURE_LO_SIZE_NV = 34588 	# GL/glext.h:2301
GL_TEXTURE_DS_SIZE_NV = 34589 	# GL/glext.h:2302
GL_TEXTURE_DT_SIZE_NV = 34590 	# GL/glext.h:2303
GL_TEXTURE_MAG_SIZE_NV = 34591 	# GL/glext.h:2304
# NV_texture_shader2 (GL/glext.h:2307)
GL_DOT_PRODUCT_TEXTURE_3D_NV = 34543 	# GL/glext.h:2308
# NV_vertex_array_range2 (GL/glext.h:2311)
GL_VERTEX_ARRAY_RANGE_WITHOUT_FLUSH_NV = 34099 	# GL/glext.h:2312
# NV_vertex_program (GL/glext.h:2315)
GL_VERTEX_PROGRAM_NV = 34336 	# GL/glext.h:2316
GL_VERTEX_STATE_PROGRAM_NV = 34337 	# GL/glext.h:2317
GL_ATTRIB_ARRAY_SIZE_NV = 34339 	# GL/glext.h:2318
GL_ATTRIB_ARRAY_STRIDE_NV = 34340 	# GL/glext.h:2319
GL_ATTRIB_ARRAY_TYPE_NV = 34341 	# GL/glext.h:2320
GL_CURRENT_ATTRIB_NV = 34342 	# GL/glext.h:2321
GL_PROGRAM_LENGTH_NV = 34343 	# GL/glext.h:2322
GL_PROGRAM_STRING_NV = 34344 	# GL/glext.h:2323
GL_MODELVIEW_PROJECTION_NV = 34345 	# GL/glext.h:2324
GL_IDENTITY_NV = 34346 	# GL/glext.h:2325
GL_INVERSE_NV = 34347 	# GL/glext.h:2326
GL_TRANSPOSE_NV = 34348 	# GL/glext.h:2327
GL_INVERSE_TRANSPOSE_NV = 34349 	# GL/glext.h:2328
GL_MAX_TRACK_MATRIX_STACK_DEPTH_NV = 34350 	# GL/glext.h:2329
GL_MAX_TRACK_MATRICES_NV = 34351 	# GL/glext.h:2330
GL_MATRIX0_NV = 34352 	# GL/glext.h:2331
GL_MATRIX1_NV = 34353 	# GL/glext.h:2332
GL_MATRIX2_NV = 34354 	# GL/glext.h:2333
GL_MATRIX3_NV = 34355 	# GL/glext.h:2334
GL_MATRIX4_NV = 34356 	# GL/glext.h:2335
GL_MATRIX5_NV = 34357 	# GL/glext.h:2336
GL_MATRIX6_NV = 34358 	# GL/glext.h:2337
GL_MATRIX7_NV = 34359 	# GL/glext.h:2338
GL_CURRENT_MATRIX_STACK_DEPTH_NV = 34368 	# GL/glext.h:2339
GL_CURRENT_MATRIX_NV = 34369 	# GL/glext.h:2340
GL_VERTEX_PROGRAM_POINT_SIZE_NV = 34370 	# GL/glext.h:2341
GL_VERTEX_PROGRAM_TWO_SIDE_NV = 34371 	# GL/glext.h:2342
GL_PROGRAM_PARAMETER_NV = 34372 	# GL/glext.h:2343
GL_ATTRIB_ARRAY_POINTER_NV = 34373 	# GL/glext.h:2344
GL_PROGRAM_TARGET_NV = 34374 	# GL/glext.h:2345
GL_PROGRAM_RESIDENT_NV = 34375 	# GL/glext.h:2346
GL_TRACK_MATRIX_NV = 34376 	# GL/glext.h:2347
GL_TRACK_MATRIX_TRANSFORM_NV = 34377 	# GL/glext.h:2348
GL_VERTEX_PROGRAM_BINDING_NV = 34378 	# GL/glext.h:2349
GL_PROGRAM_ERROR_POSITION_NV = 34379 	# GL/glext.h:2350
GL_VERTEX_ATTRIB_ARRAY0_NV = 34384 	# GL/glext.h:2351
GL_VERTEX_ATTRIB_ARRAY1_NV = 34385 	# GL/glext.h:2352
GL_VERTEX_ATTRIB_ARRAY2_NV = 34386 	# GL/glext.h:2353
GL_VERTEX_ATTRIB_ARRAY3_NV = 34387 	# GL/glext.h:2354
GL_VERTEX_ATTRIB_ARRAY4_NV = 34388 	# GL/glext.h:2355
GL_VERTEX_ATTRIB_ARRAY5_NV = 34389 	# GL/glext.h:2356
GL_VERTEX_ATTRIB_ARRAY6_NV = 34390 	# GL/glext.h:2357
GL_VERTEX_ATTRIB_ARRAY7_NV = 34391 	# GL/glext.h:2358
GL_VERTEX_ATTRIB_ARRAY8_NV = 34392 	# GL/glext.h:2359
GL_VERTEX_ATTRIB_ARRAY9_NV = 34393 	# GL/glext.h:2360
GL_VERTEX_ATTRIB_ARRAY10_NV = 34394 	# GL/glext.h:2361
GL_VERTEX_ATTRIB_ARRAY11_NV = 34395 	# GL/glext.h:2362
GL_VERTEX_ATTRIB_ARRAY12_NV = 34396 	# GL/glext.h:2363
GL_VERTEX_ATTRIB_ARRAY13_NV = 34397 	# GL/glext.h:2364
GL_VERTEX_ATTRIB_ARRAY14_NV = 34398 	# GL/glext.h:2365
GL_VERTEX_ATTRIB_ARRAY15_NV = 34399 	# GL/glext.h:2366
GL_MAP1_VERTEX_ATTRIB0_4_NV = 34400 	# GL/glext.h:2367
GL_MAP1_VERTEX_ATTRIB1_4_NV = 34401 	# GL/glext.h:2368
GL_MAP1_VERTEX_ATTRIB2_4_NV = 34402 	# GL/glext.h:2369
GL_MAP1_VERTEX_ATTRIB3_4_NV = 34403 	# GL/glext.h:2370
GL_MAP1_VERTEX_ATTRIB4_4_NV = 34404 	# GL/glext.h:2371
GL_MAP1_VERTEX_ATTRIB5_4_NV = 34405 	# GL/glext.h:2372
GL_MAP1_VERTEX_ATTRIB6_4_NV = 34406 	# GL/glext.h:2373
GL_MAP1_VERTEX_ATTRIB7_4_NV = 34407 	# GL/glext.h:2374
GL_MAP1_VERTEX_ATTRIB8_4_NV = 34408 	# GL/glext.h:2375
GL_MAP1_VERTEX_ATTRIB9_4_NV = 34409 	# GL/glext.h:2376
GL_MAP1_VERTEX_ATTRIB10_4_NV = 34410 	# GL/glext.h:2377
GL_MAP1_VERTEX_ATTRIB11_4_NV = 34411 	# GL/glext.h:2378
GL_MAP1_VERTEX_ATTRIB12_4_NV = 34412 	# GL/glext.h:2379
GL_MAP1_VERTEX_ATTRIB13_4_NV = 34413 	# GL/glext.h:2380
GL_MAP1_VERTEX_ATTRIB14_4_NV = 34414 	# GL/glext.h:2381
GL_MAP1_VERTEX_ATTRIB15_4_NV = 34415 	# GL/glext.h:2382
GL_MAP2_VERTEX_ATTRIB0_4_NV = 34416 	# GL/glext.h:2383
GL_MAP2_VERTEX_ATTRIB1_4_NV = 34417 	# GL/glext.h:2384
GL_MAP2_VERTEX_ATTRIB2_4_NV = 34418 	# GL/glext.h:2385
GL_MAP2_VERTEX_ATTRIB3_4_NV = 34419 	# GL/glext.h:2386
GL_MAP2_VERTEX_ATTRIB4_4_NV = 34420 	# GL/glext.h:2387
GL_MAP2_VERTEX_ATTRIB5_4_NV = 34421 	# GL/glext.h:2388
GL_MAP2_VERTEX_ATTRIB6_4_NV = 34422 	# GL/glext.h:2389
GL_MAP2_VERTEX_ATTRIB7_4_NV = 34423 	# GL/glext.h:2390
GL_MAP2_VERTEX_ATTRIB8_4_NV = 34424 	# GL/glext.h:2391
GL_MAP2_VERTEX_ATTRIB9_4_NV = 34425 	# GL/glext.h:2392
GL_MAP2_VERTEX_ATTRIB10_4_NV = 34426 	# GL/glext.h:2393
GL_MAP2_VERTEX_ATTRIB11_4_NV = 34427 	# GL/glext.h:2394
GL_MAP2_VERTEX_ATTRIB12_4_NV = 34428 	# GL/glext.h:2395
GL_MAP2_VERTEX_ATTRIB13_4_NV = 34429 	# GL/glext.h:2396
GL_MAP2_VERTEX_ATTRIB14_4_NV = 34430 	# GL/glext.h:2397
GL_MAP2_VERTEX_ATTRIB15_4_NV = 34431 	# GL/glext.h:2398
# SGIX_texture_coordinate_clamp (GL/glext.h:2401)
GL_TEXTURE_MAX_CLAMP_S_SGIX = 33641 	# GL/glext.h:2402
GL_TEXTURE_MAX_CLAMP_T_SGIX = 33642 	# GL/glext.h:2403
GL_TEXTURE_MAX_CLAMP_R_SGIX = 33643 	# GL/glext.h:2404
# SGIX_scalebias_hint (GL/glext.h:2407)
GL_SCALEBIAS_HINT_SGIX = 33570 	# GL/glext.h:2408
# OML_interlace (GL/glext.h:2411)
GL_INTERLACE_OML = 35200 	# GL/glext.h:2412
GL_INTERLACE_READ_OML = 35201 	# GL/glext.h:2413
# OML_subsample (GL/glext.h:2416)
GL_FORMAT_SUBSAMPLE_24_24_OML = 35202 	# GL/glext.h:2417
GL_FORMAT_SUBSAMPLE_244_244_OML = 35203 	# GL/glext.h:2418
# OML_resample (GL/glext.h:2421)
GL_PACK_RESAMPLE_OML = 35204 	# GL/glext.h:2422
GL_UNPACK_RESAMPLE_OML = 35205 	# GL/glext.h:2423
GL_RESAMPLE_REPLICATE_OML = 35206 	# GL/glext.h:2424
GL_RESAMPLE_ZERO_FILL_OML = 35207 	# GL/glext.h:2425
GL_RESAMPLE_AVERAGE_OML = 35208 	# GL/glext.h:2426
GL_RESAMPLE_DECIMATE_OML = 35209 	# GL/glext.h:2427
# NV_copy_depth_to_color (GL/glext.h:2430)
GL_DEPTH_STENCIL_TO_RGBA_NV = 34926 	# GL/glext.h:2431
GL_DEPTH_STENCIL_TO_BGRA_NV = 34927 	# GL/glext.h:2432
# ATI_envmap_bumpmap (GL/glext.h:2435)
GL_BUMP_ROT_MATRIX_ATI = 34677 	# GL/glext.h:2436
GL_BUMP_ROT_MATRIX_SIZE_ATI = 34678 	# GL/glext.h:2437
GL_BUMP_NUM_TEX_UNITS_ATI = 34679 	# GL/glext.h:2438
GL_BUMP_TEX_UNITS_ATI = 34680 	# GL/glext.h:2439
GL_DUDV_ATI = 34681 	# GL/glext.h:2440
GL_DU8DV8_ATI = 34682 	# GL/glext.h:2441
GL_BUMP_ENVMAP_ATI = 34683 	# GL/glext.h:2442
GL_BUMP_TARGET_ATI = 34684 	# GL/glext.h:2443
# ATI_fragment_shader (GL/glext.h:2446)
GL_FRAGMENT_SHADER_ATI = 35104 	# GL/glext.h:2447
GL_REG_0_ATI = 35105 	# GL/glext.h:2448
GL_REG_1_ATI = 35106 	# GL/glext.h:2449
GL_REG_2_ATI = 35107 	# GL/glext.h:2450
GL_REG_3_ATI = 35108 	# GL/glext.h:2451
GL_REG_4_ATI = 35109 	# GL/glext.h:2452
GL_REG_5_ATI = 35110 	# GL/glext.h:2453
GL_REG_6_ATI = 35111 	# GL/glext.h:2454
GL_REG_7_ATI = 35112 	# GL/glext.h:2455
GL_REG_8_ATI = 35113 	# GL/glext.h:2456
GL_REG_9_ATI = 35114 	# GL/glext.h:2457
GL_REG_10_ATI = 35115 	# GL/glext.h:2458
GL_REG_11_ATI = 35116 	# GL/glext.h:2459
GL_REG_12_ATI = 35117 	# GL/glext.h:2460
GL_REG_13_ATI = 35118 	# GL/glext.h:2461
GL_REG_14_ATI = 35119 	# GL/glext.h:2462
GL_REG_15_ATI = 35120 	# GL/glext.h:2463
GL_REG_16_ATI = 35121 	# GL/glext.h:2464
GL_REG_17_ATI = 35122 	# GL/glext.h:2465
GL_REG_18_ATI = 35123 	# GL/glext.h:2466
GL_REG_19_ATI = 35124 	# GL/glext.h:2467
GL_REG_20_ATI = 35125 	# GL/glext.h:2468
GL_REG_21_ATI = 35126 	# GL/glext.h:2469
GL_REG_22_ATI = 35127 	# GL/glext.h:2470
GL_REG_23_ATI = 35128 	# GL/glext.h:2471
GL_REG_24_ATI = 35129 	# GL/glext.h:2472
GL_REG_25_ATI = 35130 	# GL/glext.h:2473
GL_REG_26_ATI = 35131 	# GL/glext.h:2474
GL_REG_27_ATI = 35132 	# GL/glext.h:2475
GL_REG_28_ATI = 35133 	# GL/glext.h:2476
GL_REG_29_ATI = 35134 	# GL/glext.h:2477
GL_REG_30_ATI = 35135 	# GL/glext.h:2478
GL_REG_31_ATI = 35136 	# GL/glext.h:2479
GL_CON_0_ATI = 35137 	# GL/glext.h:2480
GL_CON_1_ATI = 35138 	# GL/glext.h:2481
GL_CON_2_ATI = 35139 	# GL/glext.h:2482
GL_CON_3_ATI = 35140 	# GL/glext.h:2483
GL_CON_4_ATI = 35141 	# GL/glext.h:2484
GL_CON_5_ATI = 35142 	# GL/glext.h:2485
GL_CON_6_ATI = 35143 	# GL/glext.h:2486
GL_CON_7_ATI = 35144 	# GL/glext.h:2487
GL_CON_8_ATI = 35145 	# GL/glext.h:2488
GL_CON_9_ATI = 35146 	# GL/glext.h:2489
GL_CON_10_ATI = 35147 	# GL/glext.h:2490
GL_CON_11_ATI = 35148 	# GL/glext.h:2491
GL_CON_12_ATI = 35149 	# GL/glext.h:2492
GL_CON_13_ATI = 35150 	# GL/glext.h:2493
GL_CON_14_ATI = 35151 	# GL/glext.h:2494
GL_CON_15_ATI = 35152 	# GL/glext.h:2495
GL_CON_16_ATI = 35153 	# GL/glext.h:2496
GL_CON_17_ATI = 35154 	# GL/glext.h:2497
GL_CON_18_ATI = 35155 	# GL/glext.h:2498
GL_CON_19_ATI = 35156 	# GL/glext.h:2499
GL_CON_20_ATI = 35157 	# GL/glext.h:2500
GL_CON_21_ATI = 35158 	# GL/glext.h:2501
GL_CON_22_ATI = 35159 	# GL/glext.h:2502
GL_CON_23_ATI = 35160 	# GL/glext.h:2503
GL_CON_24_ATI = 35161 	# GL/glext.h:2504
GL_CON_25_ATI = 35162 	# GL/glext.h:2505
GL_CON_26_ATI = 35163 	# GL/glext.h:2506
GL_CON_27_ATI = 35164 	# GL/glext.h:2507
GL_CON_28_ATI = 35165 	# GL/glext.h:2508
GL_CON_29_ATI = 35166 	# GL/glext.h:2509
GL_CON_30_ATI = 35167 	# GL/glext.h:2510
GL_CON_31_ATI = 35168 	# GL/glext.h:2511
GL_MOV_ATI = 35169 	# GL/glext.h:2512
GL_ADD_ATI = 35171 	# GL/glext.h:2513
GL_MUL_ATI = 35172 	# GL/glext.h:2514
GL_SUB_ATI = 35173 	# GL/glext.h:2515
GL_DOT3_ATI = 35174 	# GL/glext.h:2516
GL_DOT4_ATI = 35175 	# GL/glext.h:2517
GL_MAD_ATI = 35176 	# GL/glext.h:2518
GL_LERP_ATI = 35177 	# GL/glext.h:2519
GL_CND_ATI = 35178 	# GL/glext.h:2520
GL_CND0_ATI = 35179 	# GL/glext.h:2521
GL_DOT2_ADD_ATI = 35180 	# GL/glext.h:2522
GL_SECONDARY_INTERPOLATOR_ATI = 35181 	# GL/glext.h:2523
GL_NUM_FRAGMENT_REGISTERS_ATI = 35182 	# GL/glext.h:2524
GL_NUM_FRAGMENT_CONSTANTS_ATI = 35183 	# GL/glext.h:2525
GL_NUM_PASSES_ATI = 35184 	# GL/glext.h:2526
GL_NUM_INSTRUCTIONS_PER_PASS_ATI = 35185 	# GL/glext.h:2527
GL_NUM_INSTRUCTIONS_TOTAL_ATI = 35186 	# GL/glext.h:2528
GL_NUM_INPUT_INTERPOLATOR_COMPONENTS_ATI = 35187 	# GL/glext.h:2529
GL_NUM_LOOPBACK_COMPONENTS_ATI = 35188 	# GL/glext.h:2530
GL_COLOR_ALPHA_PAIRING_ATI = 35189 	# GL/glext.h:2531
GL_SWIZZLE_STR_ATI = 35190 	# GL/glext.h:2532
GL_SWIZZLE_STQ_ATI = 35191 	# GL/glext.h:2533
GL_SWIZZLE_STR_DR_ATI = 35192 	# GL/glext.h:2534
GL_SWIZZLE_STQ_DQ_ATI = 35193 	# GL/glext.h:2535
GL_SWIZZLE_STRQ_ATI = 35194 	# GL/glext.h:2536
GL_SWIZZLE_STRQ_DQ_ATI = 35195 	# GL/glext.h:2537
GL_RED_BIT_ATI = 1 	# GL/glext.h:2538
GL_GREEN_BIT_ATI = 2 	# GL/glext.h:2539
GL_BLUE_BIT_ATI = 4 	# GL/glext.h:2540
GL_2X_BIT_ATI = 1 	# GL/glext.h:2541
GL_4X_BIT_ATI = 2 	# GL/glext.h:2542
GL_8X_BIT_ATI = 4 	# GL/glext.h:2543
GL_HALF_BIT_ATI = 8 	# GL/glext.h:2544
GL_QUARTER_BIT_ATI = 16 	# GL/glext.h:2545
GL_EIGHTH_BIT_ATI = 32 	# GL/glext.h:2546
GL_SATURATE_BIT_ATI = 64 	# GL/glext.h:2547
GL_COMP_BIT_ATI = 2 	# GL/glext.h:2548
GL_NEGATE_BIT_ATI = 4 	# GL/glext.h:2549
GL_BIAS_BIT_ATI = 8 	# GL/glext.h:2550
# ATI_pn_triangles (GL/glext.h:2553)
GL_PN_TRIANGLES_ATI = 34800 	# GL/glext.h:2554
GL_MAX_PN_TRIANGLES_TESSELATION_LEVEL_ATI = 34801 	# GL/glext.h:2555
GL_PN_TRIANGLES_POINT_MODE_ATI = 34802 	# GL/glext.h:2556
GL_PN_TRIANGLES_NORMAL_MODE_ATI = 34803 	# GL/glext.h:2557
GL_PN_TRIANGLES_TESSELATION_LEVEL_ATI = 34804 	# GL/glext.h:2558
GL_PN_TRIANGLES_POINT_MODE_LINEAR_ATI = 34805 	# GL/glext.h:2559
GL_PN_TRIANGLES_POINT_MODE_CUBIC_ATI = 34806 	# GL/glext.h:2560
GL_PN_TRIANGLES_NORMAL_MODE_LINEAR_ATI = 34807 	# GL/glext.h:2561
GL_PN_TRIANGLES_NORMAL_MODE_QUADRATIC_ATI = 34808 	# GL/glext.h:2562
# ATI_vertex_array_object (GL/glext.h:2565)
GL_STATIC_ATI = 34656 	# GL/glext.h:2566
GL_DYNAMIC_ATI = 34657 	# GL/glext.h:2567
GL_PRESERVE_ATI = 34658 	# GL/glext.h:2568
GL_DISCARD_ATI = 34659 	# GL/glext.h:2569
GL_OBJECT_BUFFER_SIZE_ATI = 34660 	# GL/glext.h:2570
GL_OBJECT_BUFFER_USAGE_ATI = 34661 	# GL/glext.h:2571
GL_ARRAY_OBJECT_BUFFER_ATI = 34662 	# GL/glext.h:2572
GL_ARRAY_OBJECT_OFFSET_ATI = 34663 	# GL/glext.h:2573
# EXT_vertex_shader (GL/glext.h:2576)
GL_VERTEX_SHADER_EXT = 34688 	# GL/glext.h:2577
GL_VERTEX_SHADER_BINDING_EXT = 34689 	# GL/glext.h:2578
GL_OP_INDEX_EXT = 34690 	# GL/glext.h:2579
GL_OP_NEGATE_EXT = 34691 	# GL/glext.h:2580
GL_OP_DOT3_EXT = 34692 	# GL/glext.h:2581
GL_OP_DOT4_EXT = 34693 	# GL/glext.h:2582
GL_OP_MUL_EXT = 34694 	# GL/glext.h:2583
GL_OP_ADD_EXT = 34695 	# GL/glext.h:2584
GL_OP_MADD_EXT = 34696 	# GL/glext.h:2585
GL_OP_FRAC_EXT = 34697 	# GL/glext.h:2586
GL_OP_MAX_EXT = 34698 	# GL/glext.h:2587
GL_OP_MIN_EXT = 34699 	# GL/glext.h:2588
GL_OP_SET_GE_EXT = 34700 	# GL/glext.h:2589
GL_OP_SET_LT_EXT = 34701 	# GL/glext.h:2590
GL_OP_CLAMP_EXT = 34702 	# GL/glext.h:2591
GL_OP_FLOOR_EXT = 34703 	# GL/glext.h:2592
GL_OP_ROUND_EXT = 34704 	# GL/glext.h:2593
GL_OP_EXP_BASE_2_EXT = 34705 	# GL/glext.h:2594
GL_OP_LOG_BASE_2_EXT = 34706 	# GL/glext.h:2595
GL_OP_POWER_EXT = 34707 	# GL/glext.h:2596
GL_OP_RECIP_EXT = 34708 	# GL/glext.h:2597
GL_OP_RECIP_SQRT_EXT = 34709 	# GL/glext.h:2598
GL_OP_SUB_EXT = 34710 	# GL/glext.h:2599
GL_OP_CROSS_PRODUCT_EXT = 34711 	# GL/glext.h:2600
GL_OP_MULTIPLY_MATRIX_EXT = 34712 	# GL/glext.h:2601
GL_OP_MOV_EXT = 34713 	# GL/glext.h:2602
GL_OUTPUT_VERTEX_EXT = 34714 	# GL/glext.h:2603
GL_OUTPUT_COLOR0_EXT = 34715 	# GL/glext.h:2604
GL_OUTPUT_COLOR1_EXT = 34716 	# GL/glext.h:2605
GL_OUTPUT_TEXTURE_COORD0_EXT = 34717 	# GL/glext.h:2606
GL_OUTPUT_TEXTURE_COORD1_EXT = 34718 	# GL/glext.h:2607
GL_OUTPUT_TEXTURE_COORD2_EXT = 34719 	# GL/glext.h:2608
GL_OUTPUT_TEXTURE_COORD3_EXT = 34720 	# GL/glext.h:2609
GL_OUTPUT_TEXTURE_COORD4_EXT = 34721 	# GL/glext.h:2610
GL_OUTPUT_TEXTURE_COORD5_EXT = 34722 	# GL/glext.h:2611
GL_OUTPUT_TEXTURE_COORD6_EXT = 34723 	# GL/glext.h:2612
GL_OUTPUT_TEXTURE_COORD7_EXT = 34724 	# GL/glext.h:2613
GL_OUTPUT_TEXTURE_COORD8_EXT = 34725 	# GL/glext.h:2614
GL_OUTPUT_TEXTURE_COORD9_EXT = 34726 	# GL/glext.h:2615
GL_OUTPUT_TEXTURE_COORD10_EXT = 34727 	# GL/glext.h:2616
GL_OUTPUT_TEXTURE_COORD11_EXT = 34728 	# GL/glext.h:2617
GL_OUTPUT_TEXTURE_COORD12_EXT = 34729 	# GL/glext.h:2618
GL_OUTPUT_TEXTURE_COORD13_EXT = 34730 	# GL/glext.h:2619
GL_OUTPUT_TEXTURE_COORD14_EXT = 34731 	# GL/glext.h:2620
GL_OUTPUT_TEXTURE_COORD15_EXT = 34732 	# GL/glext.h:2621
GL_OUTPUT_TEXTURE_COORD16_EXT = 34733 	# GL/glext.h:2622
GL_OUTPUT_TEXTURE_COORD17_EXT = 34734 	# GL/glext.h:2623
GL_OUTPUT_TEXTURE_COORD18_EXT = 34735 	# GL/glext.h:2624
GL_OUTPUT_TEXTURE_COORD19_EXT = 34736 	# GL/glext.h:2625
GL_OUTPUT_TEXTURE_COORD20_EXT = 34737 	# GL/glext.h:2626
GL_OUTPUT_TEXTURE_COORD21_EXT = 34738 	# GL/glext.h:2627
GL_OUTPUT_TEXTURE_COORD22_EXT = 34739 	# GL/glext.h:2628
GL_OUTPUT_TEXTURE_COORD23_EXT = 34740 	# GL/glext.h:2629
GL_OUTPUT_TEXTURE_COORD24_EXT = 34741 	# GL/glext.h:2630
GL_OUTPUT_TEXTURE_COORD25_EXT = 34742 	# GL/glext.h:2631
GL_OUTPUT_TEXTURE_COORD26_EXT = 34743 	# GL/glext.h:2632
GL_OUTPUT_TEXTURE_COORD27_EXT = 34744 	# GL/glext.h:2633
GL_OUTPUT_TEXTURE_COORD28_EXT = 34745 	# GL/glext.h:2634
GL_OUTPUT_TEXTURE_COORD29_EXT = 34746 	# GL/glext.h:2635
GL_OUTPUT_TEXTURE_COORD30_EXT = 34747 	# GL/glext.h:2636
GL_OUTPUT_TEXTURE_COORD31_EXT = 34748 	# GL/glext.h:2637
GL_OUTPUT_FOG_EXT = 34749 	# GL/glext.h:2638
GL_SCALAR_EXT = 34750 	# GL/glext.h:2639
GL_VECTOR_EXT = 34751 	# GL/glext.h:2640
GL_MATRIX_EXT = 34752 	# GL/glext.h:2641
GL_VARIANT_EXT = 34753 	# GL/glext.h:2642
GL_INVARIANT_EXT = 34754 	# GL/glext.h:2643
GL_LOCAL_CONSTANT_EXT = 34755 	# GL/glext.h:2644
GL_LOCAL_EXT = 34756 	# GL/glext.h:2645
GL_MAX_VERTEX_SHADER_INSTRUCTIONS_EXT = 34757 	# GL/glext.h:2646
GL_MAX_VERTEX_SHADER_VARIANTS_EXT = 34758 	# GL/glext.h:2647
GL_MAX_VERTEX_SHADER_INVARIANTS_EXT = 34759 	# GL/glext.h:2648
GL_MAX_VERTEX_SHADER_LOCAL_CONSTANTS_EXT = 34760 	# GL/glext.h:2649
GL_MAX_VERTEX_SHADER_LOCALS_EXT = 34761 	# GL/glext.h:2650
GL_MAX_OPTIMIZED_VERTEX_SHADER_INSTRUCTIONS_EXT = 34762 	# GL/glext.h:2651
GL_MAX_OPTIMIZED_VERTEX_SHADER_VARIANTS_EXT = 34763 	# GL/glext.h:2652
GL_MAX_OPTIMIZED_VERTEX_SHADER_LOCAL_CONSTANTS_EXT = 34764 	# GL/glext.h:2653
GL_MAX_OPTIMIZED_VERTEX_SHADER_INVARIANTS_EXT = 34765 	# GL/glext.h:2654
GL_MAX_OPTIMIZED_VERTEX_SHADER_LOCALS_EXT = 34766 	# GL/glext.h:2655
GL_VERTEX_SHADER_INSTRUCTIONS_EXT = 34767 	# GL/glext.h:2656
GL_VERTEX_SHADER_VARIANTS_EXT = 34768 	# GL/glext.h:2657
GL_VERTEX_SHADER_INVARIANTS_EXT = 34769 	# GL/glext.h:2658
GL_VERTEX_SHADER_LOCAL_CONSTANTS_EXT = 34770 	# GL/glext.h:2659
GL_VERTEX_SHADER_LOCALS_EXT = 34771 	# GL/glext.h:2660
GL_VERTEX_SHADER_OPTIMIZED_EXT = 34772 	# GL/glext.h:2661
GL_X_EXT = 34773 	# GL/glext.h:2662
GL_Y_EXT = 34774 	# GL/glext.h:2663
GL_Z_EXT = 34775 	# GL/glext.h:2664
GL_W_EXT = 34776 	# GL/glext.h:2665
GL_NEGATIVE_X_EXT = 34777 	# GL/glext.h:2666
GL_NEGATIVE_Y_EXT = 34778 	# GL/glext.h:2667
GL_NEGATIVE_Z_EXT = 34779 	# GL/glext.h:2668
GL_NEGATIVE_W_EXT = 34780 	# GL/glext.h:2669
GL_ZERO_EXT = 34781 	# GL/glext.h:2670
GL_ONE_EXT = 34782 	# GL/glext.h:2671
GL_NEGATIVE_ONE_EXT = 34783 	# GL/glext.h:2672
GL_NORMALIZED_RANGE_EXT = 34784 	# GL/glext.h:2673
GL_FULL_RANGE_EXT = 34785 	# GL/glext.h:2674
GL_CURRENT_VERTEX_EXT = 34786 	# GL/glext.h:2675
GL_MVP_MATRIX_EXT = 34787 	# GL/glext.h:2676
GL_VARIANT_VALUE_EXT = 34788 	# GL/glext.h:2677
GL_VARIANT_DATATYPE_EXT = 34789 	# GL/glext.h:2678
GL_VARIANT_ARRAY_STRIDE_EXT = 34790 	# GL/glext.h:2679
GL_VARIANT_ARRAY_TYPE_EXT = 34791 	# GL/glext.h:2680
GL_VARIANT_ARRAY_EXT = 34792 	# GL/glext.h:2681
GL_VARIANT_ARRAY_POINTER_EXT = 34793 	# GL/glext.h:2682
GL_INVARIANT_VALUE_EXT = 34794 	# GL/glext.h:2683
GL_INVARIANT_DATATYPE_EXT = 34795 	# GL/glext.h:2684
GL_LOCAL_CONSTANT_VALUE_EXT = 34796 	# GL/glext.h:2685
GL_LOCAL_CONSTANT_DATATYPE_EXT = 34797 	# GL/glext.h:2686
# ATI_vertex_streams (GL/glext.h:2689)
GL_MAX_VERTEX_STREAMS_ATI = 34667 	# GL/glext.h:2690
GL_VERTEX_STREAM0_ATI = 34668 	# GL/glext.h:2691
GL_VERTEX_STREAM1_ATI = 34669 	# GL/glext.h:2692
GL_VERTEX_STREAM2_ATI = 34670 	# GL/glext.h:2693
GL_VERTEX_STREAM3_ATI = 34671 	# GL/glext.h:2694
GL_VERTEX_STREAM4_ATI = 34672 	# GL/glext.h:2695
GL_VERTEX_STREAM5_ATI = 34673 	# GL/glext.h:2696
GL_VERTEX_STREAM6_ATI = 34674 	# GL/glext.h:2697
GL_VERTEX_STREAM7_ATI = 34675 	# GL/glext.h:2698
GL_VERTEX_SOURCE_ATI = 34676 	# GL/glext.h:2699
# ATI_element_array (GL/glext.h:2702)
GL_ELEMENT_ARRAY_ATI = 34664 	# GL/glext.h:2703
GL_ELEMENT_ARRAY_TYPE_ATI = 34665 	# GL/glext.h:2704
GL_ELEMENT_ARRAY_POINTER_ATI = 34666 	# GL/glext.h:2705
# SUN_mesh_array (GL/glext.h:2708)
GL_QUAD_MESH_SUN = 34324 	# GL/glext.h:2709
GL_TRIANGLE_MESH_SUN = 34325 	# GL/glext.h:2710
# SUN_slice_accum (GL/glext.h:2713)
GL_SLICE_ACCUM_SUN = 34252 	# GL/glext.h:2714
# NV_multisample_filter_hint (GL/glext.h:2717)
GL_MULTISAMPLE_FILTER_HINT_NV = 34100 	# GL/glext.h:2718
# NV_depth_clamp (GL/glext.h:2721)
GL_DEPTH_CLAMP_NV = 34383 	# GL/glext.h:2722
# NV_occlusion_query (GL/glext.h:2725)
GL_PIXEL_COUNTER_BITS_NV = 34916 	# GL/glext.h:2726
GL_CURRENT_OCCLUSION_QUERY_ID_NV = 34917 	# GL/glext.h:2727
GL_PIXEL_COUNT_NV = 34918 	# GL/glext.h:2728
GL_PIXEL_COUNT_AVAILABLE_NV = 34919 	# GL/glext.h:2729
# NV_point_sprite (GL/glext.h:2732)
GL_POINT_SPRITE_NV = 34913 	# GL/glext.h:2733
GL_COORD_REPLACE_NV = 34914 	# GL/glext.h:2734
GL_POINT_SPRITE_R_MODE_NV = 34915 	# GL/glext.h:2735
# NV_texture_shader3 (GL/glext.h:2738)
GL_OFFSET_PROJECTIVE_TEXTURE_2D_NV = 34896 	# GL/glext.h:2739
GL_OFFSET_PROJECTIVE_TEXTURE_2D_SCALE_NV = 34897 	# GL/glext.h:2740
GL_OFFSET_PROJECTIVE_TEXTURE_RECTANGLE_NV = 34898 	# GL/glext.h:2741
GL_OFFSET_PROJECTIVE_TEXTURE_RECTANGLE_SCALE_NV = 34899 	# GL/glext.h:2742
GL_OFFSET_HILO_TEXTURE_2D_NV = 34900 	# GL/glext.h:2743
GL_OFFSET_HILO_TEXTURE_RECTANGLE_NV = 34901 	# GL/glext.h:2744
GL_OFFSET_HILO_PROJECTIVE_TEXTURE_2D_NV = 34902 	# GL/glext.h:2745
GL_OFFSET_HILO_PROJECTIVE_TEXTURE_RECTANGLE_NV = 34903 	# GL/glext.h:2746
GL_DEPENDENT_HILO_TEXTURE_2D_NV = 34904 	# GL/glext.h:2747
GL_DEPENDENT_RGB_TEXTURE_3D_NV = 34905 	# GL/glext.h:2748
GL_DEPENDENT_RGB_TEXTURE_CUBE_MAP_NV = 34906 	# GL/glext.h:2749
GL_DOT_PRODUCT_PASS_THROUGH_NV = 34907 	# GL/glext.h:2750
GL_DOT_PRODUCT_TEXTURE_1D_NV = 34908 	# GL/glext.h:2751
GL_DOT_PRODUCT_AFFINE_DEPTH_REPLACE_NV = 34909 	# GL/glext.h:2752
GL_HILO8_NV = 34910 	# GL/glext.h:2753
GL_SIGNED_HILO8_NV = 34911 	# GL/glext.h:2754
GL_FORCE_BLUE_TO_ONE_NV = 34912 	# GL/glext.h:2755
# NV_vertex_program1_1 (GL/glext.h:2758)
# EXT_shadow_funcs (GL/glext.h:2761)
# EXT_stencil_two_side (GL/glext.h:2764)
GL_STENCIL_TEST_TWO_SIDE_EXT = 35088 	# GL/glext.h:2765
GL_ACTIVE_STENCIL_FACE_EXT = 35089 	# GL/glext.h:2766
# ATI_text_fragment_shader (GL/glext.h:2769)
GL_TEXT_FRAGMENT_SHADER_ATI = 33280 	# GL/glext.h:2770
# APPLE_client_storage (GL/glext.h:2773)
GL_UNPACK_CLIENT_STORAGE_APPLE = 34226 	# GL/glext.h:2774
# APPLE_element_array (GL/glext.h:2777)
GL_ELEMENT_ARRAY_APPLE = 34664 	# GL/glext.h:2778
GL_ELEMENT_ARRAY_TYPE_APPLE = 34665 	# GL/glext.h:2779
GL_ELEMENT_ARRAY_POINTER_APPLE = 34666 	# GL/glext.h:2780
# APPLE_fence (GL/glext.h:2783)
GL_DRAW_PIXELS_APPLE = 35338 	# GL/glext.h:2784
GL_FENCE_APPLE = 35339 	# GL/glext.h:2785
# APPLE_vertex_array_object (GL/glext.h:2788)
GL_VERTEX_ARRAY_BINDING_APPLE = 34229 	# GL/glext.h:2789
# APPLE_vertex_array_range (GL/glext.h:2792)
GL_VERTEX_ARRAY_RANGE_APPLE = 34077 	# GL/glext.h:2793
GL_VERTEX_ARRAY_RANGE_LENGTH_APPLE = 34078 	# GL/glext.h:2794
GL_VERTEX_ARRAY_STORAGE_HINT_APPLE = 34079 	# GL/glext.h:2795
GL_VERTEX_ARRAY_RANGE_POINTER_APPLE = 34081 	# GL/glext.h:2796
GL_STORAGE_CACHED_APPLE = 34238 	# GL/glext.h:2797
GL_STORAGE_SHARED_APPLE = 34239 	# GL/glext.h:2798
# APPLE_ycbcr_422 (GL/glext.h:2801)
GL_YCBCR_422_APPLE = 34233 	# GL/glext.h:2802
GL_UNSIGNED_SHORT_8_8_APPLE = 34234 	# GL/glext.h:2803
GL_UNSIGNED_SHORT_8_8_REV_APPLE = 34235 	# GL/glext.h:2804
# S3_s3tc (GL/glext.h:2807)
GL_RGB_S3TC = 33696 	# GL/glext.h:2808
GL_RGB4_S3TC = 33697 	# GL/glext.h:2809
GL_RGBA_S3TC = 33698 	# GL/glext.h:2810
GL_RGBA4_S3TC = 33699 	# GL/glext.h:2811
# ATI_draw_buffers (GL/glext.h:2814)
GL_MAX_DRAW_BUFFERS_ATI = 34852 	# GL/glext.h:2815
GL_DRAW_BUFFER0_ATI = 34853 	# GL/glext.h:2816
GL_DRAW_BUFFER1_ATI = 34854 	# GL/glext.h:2817
GL_DRAW_BUFFER2_ATI = 34855 	# GL/glext.h:2818
GL_DRAW_BUFFER3_ATI = 34856 	# GL/glext.h:2819
GL_DRAW_BUFFER4_ATI = 34857 	# GL/glext.h:2820
GL_DRAW_BUFFER5_ATI = 34858 	# GL/glext.h:2821
GL_DRAW_BUFFER6_ATI = 34859 	# GL/glext.h:2822
GL_DRAW_BUFFER7_ATI = 34860 	# GL/glext.h:2823
GL_DRAW_BUFFER8_ATI = 34861 	# GL/glext.h:2824
GL_DRAW_BUFFER9_ATI = 34862 	# GL/glext.h:2825
GL_DRAW_BUFFER10_ATI = 34863 	# GL/glext.h:2826
GL_DRAW_BUFFER11_ATI = 34864 	# GL/glext.h:2827
GL_DRAW_BUFFER12_ATI = 34865 	# GL/glext.h:2828
GL_DRAW_BUFFER13_ATI = 34866 	# GL/glext.h:2829
GL_DRAW_BUFFER14_ATI = 34867 	# GL/glext.h:2830
GL_DRAW_BUFFER15_ATI = 34868 	# GL/glext.h:2831
# ATI_pixel_format_float (GL/glext.h:2834)
GL_TYPE_RGBA_FLOAT_ATI = 34848 	# GL/glext.h:2835
GL_COLOR_CLEAR_UNCLAMPED_VALUE_ATI = 34869 	# GL/glext.h:2836
# ATI_texture_env_combine3 (GL/glext.h:2839)
GL_MODULATE_ADD_ATI = 34628 	# GL/glext.h:2840
GL_MODULATE_SIGNED_ADD_ATI = 34629 	# GL/glext.h:2841
GL_MODULATE_SUBTRACT_ATI = 34630 	# GL/glext.h:2842
# ATI_texture_float (GL/glext.h:2845)
GL_RGBA_FLOAT32_ATI = 34836 	# GL/glext.h:2846
GL_RGB_FLOAT32_ATI = 34837 	# GL/glext.h:2847
GL_ALPHA_FLOAT32_ATI = 34838 	# GL/glext.h:2848
GL_INTENSITY_FLOAT32_ATI = 34839 	# GL/glext.h:2849
GL_LUMINANCE_FLOAT32_ATI = 34840 	# GL/glext.h:2850
GL_LUMINANCE_ALPHA_FLOAT32_ATI = 34841 	# GL/glext.h:2851
GL_RGBA_FLOAT16_ATI = 34842 	# GL/glext.h:2852
GL_RGB_FLOAT16_ATI = 34843 	# GL/glext.h:2853
GL_ALPHA_FLOAT16_ATI = 34844 	# GL/glext.h:2854
GL_INTENSITY_FLOAT16_ATI = 34845 	# GL/glext.h:2855
GL_LUMINANCE_FLOAT16_ATI = 34846 	# GL/glext.h:2856
GL_LUMINANCE_ALPHA_FLOAT16_ATI = 34847 	# GL/glext.h:2857
# NV_float_buffer (GL/glext.h:2860)
GL_FLOAT_R_NV = 34944 	# GL/glext.h:2861
GL_FLOAT_RG_NV = 34945 	# GL/glext.h:2862
GL_FLOAT_RGB_NV = 34946 	# GL/glext.h:2863
GL_FLOAT_RGBA_NV = 34947 	# GL/glext.h:2864
GL_FLOAT_R16_NV = 34948 	# GL/glext.h:2865
GL_FLOAT_R32_NV = 34949 	# GL/glext.h:2866
GL_FLOAT_RG16_NV = 34950 	# GL/glext.h:2867
GL_FLOAT_RG32_NV = 34951 	# GL/glext.h:2868
GL_FLOAT_RGB16_NV = 34952 	# GL/glext.h:2869
GL_FLOAT_RGB32_NV = 34953 	# GL/glext.h:2870
GL_FLOAT_RGBA16_NV = 34954 	# GL/glext.h:2871
GL_FLOAT_RGBA32_NV = 34955 	# GL/glext.h:2872
GL_TEXTURE_FLOAT_COMPONENTS_NV = 34956 	# GL/glext.h:2873
GL_FLOAT_CLEAR_COLOR_VALUE_NV = 34957 	# GL/glext.h:2874
GL_FLOAT_RGBA_MODE_NV = 34958 	# GL/glext.h:2875
# NV_fragment_program (GL/glext.h:2878)
GL_MAX_FRAGMENT_PROGRAM_LOCAL_PARAMETERS_NV = 34920 	# GL/glext.h:2879
GL_FRAGMENT_PROGRAM_NV = 34928 	# GL/glext.h:2880
GL_MAX_TEXTURE_COORDS_NV = 34929 	# GL/glext.h:2881
GL_MAX_TEXTURE_IMAGE_UNITS_NV = 34930 	# GL/glext.h:2882
GL_FRAGMENT_PROGRAM_BINDING_NV = 34931 	# GL/glext.h:2883
GL_PROGRAM_ERROR_STRING_NV = 34932 	# GL/glext.h:2884
# NV_half_float (GL/glext.h:2887)
GL_HALF_FLOAT_NV = 5131 	# GL/glext.h:2888
# NV_pixel_data_range (GL/glext.h:2891)
GL_WRITE_PIXEL_DATA_RANGE_NV = 34936 	# GL/glext.h:2892
GL_READ_PIXEL_DATA_RANGE_NV = 34937 	# GL/glext.h:2893
GL_WRITE_PIXEL_DATA_RANGE_LENGTH_NV = 34938 	# GL/glext.h:2894
GL_READ_PIXEL_DATA_RANGE_LENGTH_NV = 34939 	# GL/glext.h:2895
GL_WRITE_PIXEL_DATA_RANGE_POINTER_NV = 34940 	# GL/glext.h:2896
GL_READ_PIXEL_DATA_RANGE_POINTER_NV = 34941 	# GL/glext.h:2897
# NV_primitive_restart (GL/glext.h:2900)
GL_PRIMITIVE_RESTART_NV = 34136 	# GL/glext.h:2901
GL_PRIMITIVE_RESTART_INDEX_NV = 34137 	# GL/glext.h:2902
# NV_texture_expand_normal (GL/glext.h:2905)
GL_TEXTURE_UNSIGNED_REMAP_MODE_NV = 34959 	# GL/glext.h:2906
# NV_vertex_program2 (GL/glext.h:2909)
# ATI_map_object_buffer (GL/glext.h:2912)
# ATI_separate_stencil (GL/glext.h:2915)
GL_STENCIL_BACK_FUNC_ATI = 34816 	# GL/glext.h:2916
GL_STENCIL_BACK_FAIL_ATI = 34817 	# GL/glext.h:2917
GL_STENCIL_BACK_PASS_DEPTH_FAIL_ATI = 34818 	# GL/glext.h:2918
GL_STENCIL_BACK_PASS_DEPTH_PASS_ATI = 34819 	# GL/glext.h:2919
# ATI_vertex_attrib_array_object (GL/glext.h:2922)
# OES_read_format (GL/glext.h:2925)
GL_IMPLEMENTATION_COLOR_READ_TYPE_OES = 35738 	# GL/glext.h:2926
GL_IMPLEMENTATION_COLOR_READ_FORMAT_OES = 35739 	# GL/glext.h:2927
# EXT_depth_bounds_test (GL/glext.h:2930)
GL_DEPTH_BOUNDS_TEST_EXT = 34960 	# GL/glext.h:2931
GL_DEPTH_BOUNDS_EXT = 34961 	# GL/glext.h:2932
# EXT_texture_mirror_clamp (GL/glext.h:2935)
GL_MIRROR_CLAMP_EXT = 34626 	# GL/glext.h:2936
GL_MIRROR_CLAMP_TO_EDGE_EXT = 34627 	# GL/glext.h:2937
GL_MIRROR_CLAMP_TO_BORDER_EXT = 35090 	# GL/glext.h:2938
# EXT_blend_equation_separate (GL/glext.h:2941)
GL_BLEND_EQUATION_RGB_EXT = 32777 	# GL/glext.h:2942
GL_BLEND_EQUATION_ALPHA_EXT = 34877 	# GL/glext.h:2943
# MESA_pack_invert (GL/glext.h:2946)
GL_PACK_INVERT_MESA = 34648 	# GL/glext.h:2947
# MESA_ycbcr_texture (GL/glext.h:2950)
GL_UNSIGNED_SHORT_8_8_MESA = 34234 	# GL/glext.h:2951
GL_UNSIGNED_SHORT_8_8_REV_MESA = 34235 	# GL/glext.h:2952
GL_YCBCR_MESA = 34647 	# GL/glext.h:2953
# EXT_pixel_buffer_object (GL/glext.h:2956)
GL_PIXEL_PACK_BUFFER_EXT = 35051 	# GL/glext.h:2957
GL_PIXEL_UNPACK_BUFFER_EXT = 35052 	# GL/glext.h:2958
GL_PIXEL_PACK_BUFFER_BINDING_EXT = 35053 	# GL/glext.h:2959
GL_PIXEL_UNPACK_BUFFER_BINDING_EXT = 35055 	# GL/glext.h:2960
# NV_fragment_program_option (GL/glext.h:2963)
# NV_fragment_program2 (GL/glext.h:2966)
GL_MAX_PROGRAM_EXEC_INSTRUCTIONS_NV = 35060 	# GL/glext.h:2967
GL_MAX_PROGRAM_CALL_DEPTH_NV = 35061 	# GL/glext.h:2968
GL_MAX_PROGRAM_IF_DEPTH_NV = 35062 	# GL/glext.h:2969
GL_MAX_PROGRAM_LOOP_DEPTH_NV = 35063 	# GL/glext.h:2970
GL_MAX_PROGRAM_LOOP_COUNT_NV = 35064 	# GL/glext.h:2971
# NV_vertex_program2_option (GL/glext.h:2974)
# NV_vertex_program3 (GL/glext.h:2979)
# EXT_framebuffer_object (GL/glext.h:2983)
GL_INVALID_FRAMEBUFFER_OPERATION_EXT = 1286 	# GL/glext.h:2984
GL_MAX_RENDERBUFFER_SIZE_EXT = 34024 	# GL/glext.h:2985
GL_FRAMEBUFFER_BINDING_EXT = 36006 	# GL/glext.h:2986
GL_RENDERBUFFER_BINDING_EXT = 36007 	# GL/glext.h:2987
GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE_EXT = 36048 	# GL/glext.h:2988
GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME_EXT = 36049 	# GL/glext.h:2989
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL_EXT = 36050 	# GL/glext.h:2990
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE_EXT = 36051 	# GL/glext.h:2991
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_3D_ZOFFSET_EXT = 36052 	# GL/glext.h:2992
GL_FRAMEBUFFER_COMPLETE_EXT = 36053 	# GL/glext.h:2993
GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_EXT = 36054 	# GL/glext.h:2994
GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_EXT = 36055 	# GL/glext.h:2995
GL_FRAMEBUFFER_INCOMPLETE_DUPLICATE_ATTACHMENT_EXT = 36056 	# GL/glext.h:2996
GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_EXT = 36057 	# GL/glext.h:2997
GL_FRAMEBUFFER_INCOMPLETE_FORMATS_EXT = 36058 	# GL/glext.h:2998
GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER_EXT = 36059 	# GL/glext.h:2999
GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER_EXT = 36060 	# GL/glext.h:3000
GL_FRAMEBUFFER_UNSUPPORTED_EXT = 36061 	# GL/glext.h:3001
GL_MAX_COLOR_ATTACHMENTS_EXT = 36063 	# GL/glext.h:3002
GL_COLOR_ATTACHMENT0_EXT = 36064 	# GL/glext.h:3003
GL_COLOR_ATTACHMENT1_EXT = 36065 	# GL/glext.h:3004
GL_COLOR_ATTACHMENT2_EXT = 36066 	# GL/glext.h:3005
GL_COLOR_ATTACHMENT3_EXT = 36067 	# GL/glext.h:3006
GL_COLOR_ATTACHMENT4_EXT = 36068 	# GL/glext.h:3007
GL_COLOR_ATTACHMENT5_EXT = 36069 	# GL/glext.h:3008
GL_COLOR_ATTACHMENT6_EXT = 36070 	# GL/glext.h:3009
GL_COLOR_ATTACHMENT7_EXT = 36071 	# GL/glext.h:3010
GL_COLOR_ATTACHMENT8_EXT = 36072 	# GL/glext.h:3011
GL_COLOR_ATTACHMENT9_EXT = 36073 	# GL/glext.h:3012
GL_COLOR_ATTACHMENT10_EXT = 36074 	# GL/glext.h:3013
GL_COLOR_ATTACHMENT11_EXT = 36075 	# GL/glext.h:3014
GL_COLOR_ATTACHMENT12_EXT = 36076 	# GL/glext.h:3015
GL_COLOR_ATTACHMENT13_EXT = 36077 	# GL/glext.h:3016
GL_COLOR_ATTACHMENT14_EXT = 36078 	# GL/glext.h:3017
GL_COLOR_ATTACHMENT15_EXT = 36079 	# GL/glext.h:3018
GL_DEPTH_ATTACHMENT_EXT = 36096 	# GL/glext.h:3019
GL_STENCIL_ATTACHMENT_EXT = 36128 	# GL/glext.h:3020
GL_FRAMEBUFFER_EXT = 36160 	# GL/glext.h:3021
GL_RENDERBUFFER_EXT = 36161 	# GL/glext.h:3022
GL_RENDERBUFFER_WIDTH_EXT = 36162 	# GL/glext.h:3023
GL_RENDERBUFFER_HEIGHT_EXT = 36163 	# GL/glext.h:3024
GL_RENDERBUFFER_INTERNAL_FORMAT_EXT = 36164 	# GL/glext.h:3025
GL_STENCIL_INDEX1_EXT = 36166 	# GL/glext.h:3026
GL_STENCIL_INDEX4_EXT = 36167 	# GL/glext.h:3027
GL_STENCIL_INDEX8_EXT = 36168 	# GL/glext.h:3028
GL_STENCIL_INDEX16_EXT = 36169 	# GL/glext.h:3029
GL_RENDERBUFFER_RED_SIZE_EXT = 36176 	# GL/glext.h:3030
GL_RENDERBUFFER_GREEN_SIZE_EXT = 36177 	# GL/glext.h:3031
GL_RENDERBUFFER_BLUE_SIZE_EXT = 36178 	# GL/glext.h:3032
GL_RENDERBUFFER_ALPHA_SIZE_EXT = 36179 	# GL/glext.h:3033
GL_RENDERBUFFER_DEPTH_SIZE_EXT = 36180 	# GL/glext.h:3034
GL_RENDERBUFFER_STENCIL_SIZE_EXT = 36181 	# GL/glext.h:3035
# GREMEDY_string_marker (GL/glext.h:3038)
# VERSION_2_0 (GL/glext.h:3045)
GLchar = c_char 	# GL/glext.h:3047
# VERSION_1_5 (GL/glext.h:3050)
GLintptr = c_ptrdiff_t 	# GL/glext.h:3052
GLsizeiptr = c_ptrdiff_t 	# GL/glext.h:3053
# ARB_vertex_buffer_object (GL/glext.h:3056)
GLintptrARB = c_ptrdiff_t 	# GL/glext.h:3058
GLsizeiptrARB = c_ptrdiff_t 	# GL/glext.h:3059
# ARB_shader_objects (GL/glext.h:3062)
GLcharARB = c_char 	# GL/glext.h:3064
GLhandleARB = c_uint 	# GL/glext.h:3065
# ARB_half_float_pixel (GL/glext.h:3069)
GLhalfARB = c_ushort 	# GL/glext.h:3070
# NV_half_float (GL/glext.h:3073)
GLhalfNV = c_ushort 	# GL/glext.h:3074
# VERSION_1_2 (GL/glext.h:3077)
GL_VERSION_1_2 = 1 	# GL/glext.h:3078
GLclampf = c_float 	# /usr/include/GL/gl.h:64
# GL/glext.h:3080
glBlendColor = _link_function('glBlendColor', None, [GLclampf, GLclampf, GLclampf, GLclampf], 'VERSION_1_2')

GLenum = c_uint 	# /usr/include/GL/gl.h:53
# GL/glext.h:3081
glBlendEquation = _link_function('glBlendEquation', None, [GLenum], 'VERSION_1_2')

GLuint = c_uint 	# /usr/include/GL/gl.h:62
GLsizei = c_int 	# /usr/include/GL/gl.h:59
GLvoid = None 	# /usr/include/GL/gl.h:67
# GL/glext.h:3082
glDrawRangeElements = _link_function('glDrawRangeElements', None, [GLenum, GLuint, GLuint, GLsizei, GLenum, POINTER(GLvoid)], 'VERSION_1_2')

# GL/glext.h:3083
glColorTable = _link_function('glColorTable', None, [GLenum, GLenum, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'VERSION_1_2')

GLfloat = c_float 	# /usr/include/GL/gl.h:63
# GL/glext.h:3084
glColorTableParameterfv = _link_function('glColorTableParameterfv', None, [GLenum, GLenum, POINTER(GLfloat)], 'VERSION_1_2')

GLint = c_int 	# /usr/include/GL/gl.h:58
# GL/glext.h:3085
glColorTableParameteriv = _link_function('glColorTableParameteriv', None, [GLenum, GLenum, POINTER(GLint)], 'VERSION_1_2')

# GL/glext.h:3086
glCopyColorTable = _link_function('glCopyColorTable', None, [GLenum, GLenum, GLint, GLint, GLsizei], 'VERSION_1_2')

# GL/glext.h:3087
glGetColorTable = _link_function('glGetColorTable', None, [GLenum, GLenum, GLenum, POINTER(GLvoid)], 'VERSION_1_2')

# GL/glext.h:3088
glGetColorTableParameterfv = _link_function('glGetColorTableParameterfv', None, [GLenum, GLenum, POINTER(GLfloat)], 'VERSION_1_2')

# GL/glext.h:3089
glGetColorTableParameteriv = _link_function('glGetColorTableParameteriv', None, [GLenum, GLenum, POINTER(GLint)], 'VERSION_1_2')

# GL/glext.h:3090
glColorSubTable = _link_function('glColorSubTable', None, [GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'VERSION_1_2')

# GL/glext.h:3091
glCopyColorSubTable = _link_function('glCopyColorSubTable', None, [GLenum, GLsizei, GLint, GLint, GLsizei], 'VERSION_1_2')

# GL/glext.h:3092
glConvolutionFilter1D = _link_function('glConvolutionFilter1D', None, [GLenum, GLenum, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'VERSION_1_2')

# GL/glext.h:3093
glConvolutionFilter2D = _link_function('glConvolutionFilter2D', None, [GLenum, GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'VERSION_1_2')

# GL/glext.h:3094
glConvolutionParameterf = _link_function('glConvolutionParameterf', None, [GLenum, GLenum, GLfloat], 'VERSION_1_2')

# GL/glext.h:3095
glConvolutionParameterfv = _link_function('glConvolutionParameterfv', None, [GLenum, GLenum, POINTER(GLfloat)], 'VERSION_1_2')

# GL/glext.h:3096
glConvolutionParameteri = _link_function('glConvolutionParameteri', None, [GLenum, GLenum, GLint], 'VERSION_1_2')

# GL/glext.h:3097
glConvolutionParameteriv = _link_function('glConvolutionParameteriv', None, [GLenum, GLenum, POINTER(GLint)], 'VERSION_1_2')

# GL/glext.h:3098
glCopyConvolutionFilter1D = _link_function('glCopyConvolutionFilter1D', None, [GLenum, GLenum, GLint, GLint, GLsizei], 'VERSION_1_2')

# GL/glext.h:3099
glCopyConvolutionFilter2D = _link_function('glCopyConvolutionFilter2D', None, [GLenum, GLenum, GLint, GLint, GLsizei, GLsizei], 'VERSION_1_2')

# GL/glext.h:3100
glGetConvolutionFilter = _link_function('glGetConvolutionFilter', None, [GLenum, GLenum, GLenum, POINTER(GLvoid)], 'VERSION_1_2')

# GL/glext.h:3101
glGetConvolutionParameterfv = _link_function('glGetConvolutionParameterfv', None, [GLenum, GLenum, POINTER(GLfloat)], 'VERSION_1_2')

# GL/glext.h:3102
glGetConvolutionParameteriv = _link_function('glGetConvolutionParameteriv', None, [GLenum, GLenum, POINTER(GLint)], 'VERSION_1_2')

# GL/glext.h:3103
glGetSeparableFilter = _link_function('glGetSeparableFilter', None, [GLenum, GLenum, GLenum, POINTER(GLvoid), POINTER(GLvoid), POINTER(GLvoid)], 'VERSION_1_2')

# GL/glext.h:3104
glSeparableFilter2D = _link_function('glSeparableFilter2D', None, [GLenum, GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid), POINTER(GLvoid)], 'VERSION_1_2')

GLboolean = c_ubyte 	# /usr/include/GL/gl.h:54
# GL/glext.h:3105
glGetHistogram = _link_function('glGetHistogram', None, [GLenum, GLboolean, GLenum, GLenum, POINTER(GLvoid)], 'VERSION_1_2')

# GL/glext.h:3106
glGetHistogramParameterfv = _link_function('glGetHistogramParameterfv', None, [GLenum, GLenum, POINTER(GLfloat)], 'VERSION_1_2')

# GL/glext.h:3107
glGetHistogramParameteriv = _link_function('glGetHistogramParameteriv', None, [GLenum, GLenum, POINTER(GLint)], 'VERSION_1_2')

# GL/glext.h:3108
glGetMinmax = _link_function('glGetMinmax', None, [GLenum, GLboolean, GLenum, GLenum, POINTER(GLvoid)], 'VERSION_1_2')

# GL/glext.h:3109
glGetMinmaxParameterfv = _link_function('glGetMinmaxParameterfv', None, [GLenum, GLenum, POINTER(GLfloat)], 'VERSION_1_2')

# GL/glext.h:3110
glGetMinmaxParameteriv = _link_function('glGetMinmaxParameteriv', None, [GLenum, GLenum, POINTER(GLint)], 'VERSION_1_2')

# GL/glext.h:3111
glHistogram = _link_function('glHistogram', None, [GLenum, GLsizei, GLenum, GLboolean], 'VERSION_1_2')

# GL/glext.h:3112
glMinmax = _link_function('glMinmax', None, [GLenum, GLenum, GLboolean], 'VERSION_1_2')

# GL/glext.h:3113
glResetHistogram = _link_function('glResetHistogram', None, [GLenum], 'VERSION_1_2')

# GL/glext.h:3114
glResetMinmax = _link_function('glResetMinmax', None, [GLenum], 'VERSION_1_2')

# GL/glext.h:3115
glTexImage3D = _link_function('glTexImage3D', None, [GLenum, GLint, GLint, GLsizei, GLsizei, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)], 'VERSION_1_2')

# GL/glext.h:3116
glTexSubImage3D = _link_function('glTexSubImage3D', None, [GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'VERSION_1_2')

# GL/glext.h:3117
glCopyTexSubImage3D = _link_function('glCopyTexSubImage3D', None, [GLenum, GLint, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei], 'VERSION_1_2')

PFNGLBLENDCOLORPROC = CFUNCTYPE(None, GLclampf, GLclampf, GLclampf, GLclampf) 	# GL/glext.h:3119
PFNGLBLENDEQUATIONPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:3120
PFNGLDRAWRANGEELEMENTSPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLsizei, GLenum, POINTER(GLvoid)) 	# GL/glext.h:3121
PFNGLCOLORTABLEPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:3122
PFNGLCOLORTABLEPARAMETERFVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:3123
PFNGLCOLORTABLEPARAMETERIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:3124
PFNGLCOPYCOLORTABLEPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLsizei) 	# GL/glext.h:3125
PFNGLGETCOLORTABLEPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:3126
PFNGLGETCOLORTABLEPARAMETERFVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:3127
PFNGLGETCOLORTABLEPARAMETERIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:3128
PFNGLCOLORSUBTABLEPROC = CFUNCTYPE(None, GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:3129
PFNGLCOPYCOLORSUBTABLEPROC = CFUNCTYPE(None, GLenum, GLsizei, GLint, GLint, GLsizei) 	# GL/glext.h:3130
PFNGLCONVOLUTIONFILTER1DPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:3131
PFNGLCONVOLUTIONFILTER2DPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:3132
PFNGLCONVOLUTIONPARAMETERFPROC = CFUNCTYPE(None, GLenum, GLenum, GLfloat) 	# GL/glext.h:3133
PFNGLCONVOLUTIONPARAMETERFVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:3134
PFNGLCONVOLUTIONPARAMETERIPROC = CFUNCTYPE(None, GLenum, GLenum, GLint) 	# GL/glext.h:3135
PFNGLCONVOLUTIONPARAMETERIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:3136
PFNGLCOPYCONVOLUTIONFILTER1DPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLsizei) 	# GL/glext.h:3137
PFNGLCOPYCONVOLUTIONFILTER2DPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLsizei, GLsizei) 	# GL/glext.h:3138
PFNGLGETCONVOLUTIONFILTERPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:3139
PFNGLGETCONVOLUTIONPARAMETERFVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:3140
PFNGLGETCONVOLUTIONPARAMETERIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:3141
PFNGLGETSEPARABLEFILTERPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLvoid), POINTER(GLvoid), POINTER(GLvoid)) 	# GL/glext.h:3142
PFNGLSEPARABLEFILTER2DPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid), POINTER(GLvoid)) 	# GL/glext.h:3143
PFNGLGETHISTOGRAMPROC = CFUNCTYPE(None, GLenum, GLboolean, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:3144
PFNGLGETHISTOGRAMPARAMETERFVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:3145
PFNGLGETHISTOGRAMPARAMETERIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:3146
PFNGLGETMINMAXPROC = CFUNCTYPE(None, GLenum, GLboolean, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:3147
PFNGLGETMINMAXPARAMETERFVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:3148
PFNGLGETMINMAXPARAMETERIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:3149
PFNGLHISTOGRAMPROC = CFUNCTYPE(None, GLenum, GLsizei, GLenum, GLboolean) 	# GL/glext.h:3150
PFNGLMINMAXPROC = CFUNCTYPE(None, GLenum, GLenum, GLboolean) 	# GL/glext.h:3151
PFNGLRESETHISTOGRAMPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:3152
PFNGLRESETMINMAXPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:3153
PFNGLTEXIMAGE3DPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLsizei, GLsizei, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:3154
PFNGLTEXSUBIMAGE3DPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:3155
PFNGLCOPYTEXSUBIMAGE3DPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei) 	# GL/glext.h:3156
# VERSION_1_3 (GL/glext.h:3159)
GL_VERSION_1_3 = 1 	# GL/glext.h:3160
# GL/glext.h:3162
glActiveTexture = _link_function('glActiveTexture', None, [GLenum], 'VERSION_1_3')

# GL/glext.h:3163
glClientActiveTexture = _link_function('glClientActiveTexture', None, [GLenum], 'VERSION_1_3')

GLdouble = c_double 	# /usr/include/GL/gl.h:65
# GL/glext.h:3164
glMultiTexCoord1d = _link_function('glMultiTexCoord1d', None, [GLenum, GLdouble], 'VERSION_1_3')

# GL/glext.h:3165
glMultiTexCoord1dv = _link_function('glMultiTexCoord1dv', None, [GLenum, POINTER(GLdouble)], 'VERSION_1_3')

# GL/glext.h:3166
glMultiTexCoord1f = _link_function('glMultiTexCoord1f', None, [GLenum, GLfloat], 'VERSION_1_3')

# GL/glext.h:3167
glMultiTexCoord1fv = _link_function('glMultiTexCoord1fv', None, [GLenum, POINTER(GLfloat)], 'VERSION_1_3')

# GL/glext.h:3168
glMultiTexCoord1i = _link_function('glMultiTexCoord1i', None, [GLenum, GLint], 'VERSION_1_3')

# GL/glext.h:3169
glMultiTexCoord1iv = _link_function('glMultiTexCoord1iv', None, [GLenum, POINTER(GLint)], 'VERSION_1_3')

GLshort = c_short 	# /usr/include/GL/gl.h:57
# GL/glext.h:3170
glMultiTexCoord1s = _link_function('glMultiTexCoord1s', None, [GLenum, GLshort], 'VERSION_1_3')

# GL/glext.h:3171
glMultiTexCoord1sv = _link_function('glMultiTexCoord1sv', None, [GLenum, POINTER(GLshort)], 'VERSION_1_3')

# GL/glext.h:3172
glMultiTexCoord2d = _link_function('glMultiTexCoord2d', None, [GLenum, GLdouble, GLdouble], 'VERSION_1_3')

# GL/glext.h:3173
glMultiTexCoord2dv = _link_function('glMultiTexCoord2dv', None, [GLenum, POINTER(GLdouble)], 'VERSION_1_3')

# GL/glext.h:3174
glMultiTexCoord2f = _link_function('glMultiTexCoord2f', None, [GLenum, GLfloat, GLfloat], 'VERSION_1_3')

# GL/glext.h:3175
glMultiTexCoord2fv = _link_function('glMultiTexCoord2fv', None, [GLenum, POINTER(GLfloat)], 'VERSION_1_3')

# GL/glext.h:3176
glMultiTexCoord2i = _link_function('glMultiTexCoord2i', None, [GLenum, GLint, GLint], 'VERSION_1_3')

# GL/glext.h:3177
glMultiTexCoord2iv = _link_function('glMultiTexCoord2iv', None, [GLenum, POINTER(GLint)], 'VERSION_1_3')

# GL/glext.h:3178
glMultiTexCoord2s = _link_function('glMultiTexCoord2s', None, [GLenum, GLshort, GLshort], 'VERSION_1_3')

# GL/glext.h:3179
glMultiTexCoord2sv = _link_function('glMultiTexCoord2sv', None, [GLenum, POINTER(GLshort)], 'VERSION_1_3')

# GL/glext.h:3180
glMultiTexCoord3d = _link_function('glMultiTexCoord3d', None, [GLenum, GLdouble, GLdouble, GLdouble], 'VERSION_1_3')

# GL/glext.h:3181
glMultiTexCoord3dv = _link_function('glMultiTexCoord3dv', None, [GLenum, POINTER(GLdouble)], 'VERSION_1_3')

# GL/glext.h:3182
glMultiTexCoord3f = _link_function('glMultiTexCoord3f', None, [GLenum, GLfloat, GLfloat, GLfloat], 'VERSION_1_3')

# GL/glext.h:3183
glMultiTexCoord3fv = _link_function('glMultiTexCoord3fv', None, [GLenum, POINTER(GLfloat)], 'VERSION_1_3')

# GL/glext.h:3184
glMultiTexCoord3i = _link_function('glMultiTexCoord3i', None, [GLenum, GLint, GLint, GLint], 'VERSION_1_3')

# GL/glext.h:3185
glMultiTexCoord3iv = _link_function('glMultiTexCoord3iv', None, [GLenum, POINTER(GLint)], 'VERSION_1_3')

# GL/glext.h:3186
glMultiTexCoord3s = _link_function('glMultiTexCoord3s', None, [GLenum, GLshort, GLshort, GLshort], 'VERSION_1_3')

# GL/glext.h:3187
glMultiTexCoord3sv = _link_function('glMultiTexCoord3sv', None, [GLenum, POINTER(GLshort)], 'VERSION_1_3')

# GL/glext.h:3188
glMultiTexCoord4d = _link_function('glMultiTexCoord4d', None, [GLenum, GLdouble, GLdouble, GLdouble, GLdouble], 'VERSION_1_3')

# GL/glext.h:3189
glMultiTexCoord4dv = _link_function('glMultiTexCoord4dv', None, [GLenum, POINTER(GLdouble)], 'VERSION_1_3')

# GL/glext.h:3190
glMultiTexCoord4f = _link_function('glMultiTexCoord4f', None, [GLenum, GLfloat, GLfloat, GLfloat, GLfloat], 'VERSION_1_3')

# GL/glext.h:3191
glMultiTexCoord4fv = _link_function('glMultiTexCoord4fv', None, [GLenum, POINTER(GLfloat)], 'VERSION_1_3')

# GL/glext.h:3192
glMultiTexCoord4i = _link_function('glMultiTexCoord4i', None, [GLenum, GLint, GLint, GLint, GLint], 'VERSION_1_3')

# GL/glext.h:3193
glMultiTexCoord4iv = _link_function('glMultiTexCoord4iv', None, [GLenum, POINTER(GLint)], 'VERSION_1_3')

# GL/glext.h:3194
glMultiTexCoord4s = _link_function('glMultiTexCoord4s', None, [GLenum, GLshort, GLshort, GLshort, GLshort], 'VERSION_1_3')

# GL/glext.h:3195
glMultiTexCoord4sv = _link_function('glMultiTexCoord4sv', None, [GLenum, POINTER(GLshort)], 'VERSION_1_3')

# GL/glext.h:3196
glLoadTransposeMatrixf = _link_function('glLoadTransposeMatrixf', None, [POINTER(GLfloat)], 'VERSION_1_3')

# GL/glext.h:3197
glLoadTransposeMatrixd = _link_function('glLoadTransposeMatrixd', None, [POINTER(GLdouble)], 'VERSION_1_3')

# GL/glext.h:3198
glMultTransposeMatrixf = _link_function('glMultTransposeMatrixf', None, [POINTER(GLfloat)], 'VERSION_1_3')

# GL/glext.h:3199
glMultTransposeMatrixd = _link_function('glMultTransposeMatrixd', None, [POINTER(GLdouble)], 'VERSION_1_3')

# GL/glext.h:3200
glSampleCoverage = _link_function('glSampleCoverage', None, [GLclampf, GLboolean], 'VERSION_1_3')

# GL/glext.h:3201
glCompressedTexImage3D = _link_function('glCompressedTexImage3D', None, [GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)], 'VERSION_1_3')

# GL/glext.h:3202
glCompressedTexImage2D = _link_function('glCompressedTexImage2D', None, [GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)], 'VERSION_1_3')

# GL/glext.h:3203
glCompressedTexImage1D = _link_function('glCompressedTexImage1D', None, [GLenum, GLint, GLenum, GLsizei, GLint, GLsizei, POINTER(GLvoid)], 'VERSION_1_3')

# GL/glext.h:3204
glCompressedTexSubImage3D = _link_function('glCompressedTexSubImage3D', None, [GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)], 'VERSION_1_3')

# GL/glext.h:3205
glCompressedTexSubImage2D = _link_function('glCompressedTexSubImage2D', None, [GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)], 'VERSION_1_3')

# GL/glext.h:3206
glCompressedTexSubImage1D = _link_function('glCompressedTexSubImage1D', None, [GLenum, GLint, GLint, GLsizei, GLenum, GLsizei, POINTER(GLvoid)], 'VERSION_1_3')

# GL/glext.h:3207
glGetCompressedTexImage = _link_function('glGetCompressedTexImage', None, [GLenum, GLint, POINTER(GLvoid)], 'VERSION_1_3')

PFNGLACTIVETEXTUREPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:3209
PFNGLCLIENTACTIVETEXTUREPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:3210
PFNGLMULTITEXCOORD1DPROC = CFUNCTYPE(None, GLenum, GLdouble) 	# GL/glext.h:3211
PFNGLMULTITEXCOORD1DVPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:3212
PFNGLMULTITEXCOORD1FPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:3213
PFNGLMULTITEXCOORD1FVPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:3214
PFNGLMULTITEXCOORD1IPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:3215
PFNGLMULTITEXCOORD1IVPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:3216
PFNGLMULTITEXCOORD1SPROC = CFUNCTYPE(None, GLenum, GLshort) 	# GL/glext.h:3217
PFNGLMULTITEXCOORD1SVPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:3218
PFNGLMULTITEXCOORD2DPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble) 	# GL/glext.h:3219
PFNGLMULTITEXCOORD2DVPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:3220
PFNGLMULTITEXCOORD2FPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat) 	# GL/glext.h:3221
PFNGLMULTITEXCOORD2FVPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:3222
PFNGLMULTITEXCOORD2IPROC = CFUNCTYPE(None, GLenum, GLint, GLint) 	# GL/glext.h:3223
PFNGLMULTITEXCOORD2IVPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:3224
PFNGLMULTITEXCOORD2SPROC = CFUNCTYPE(None, GLenum, GLshort, GLshort) 	# GL/glext.h:3225
PFNGLMULTITEXCOORD2SVPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:3226
PFNGLMULTITEXCOORD3DPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:3227
PFNGLMULTITEXCOORD3DVPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:3228
PFNGLMULTITEXCOORD3FPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:3229
PFNGLMULTITEXCOORD3FVPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:3230
PFNGLMULTITEXCOORD3IPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint) 	# GL/glext.h:3231
PFNGLMULTITEXCOORD3IVPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:3232
PFNGLMULTITEXCOORD3SPROC = CFUNCTYPE(None, GLenum, GLshort, GLshort, GLshort) 	# GL/glext.h:3233
PFNGLMULTITEXCOORD3SVPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:3234
PFNGLMULTITEXCOORD4DPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:3235
PFNGLMULTITEXCOORD4DVPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:3236
PFNGLMULTITEXCOORD4FPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:3237
PFNGLMULTITEXCOORD4FVPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:3238
PFNGLMULTITEXCOORD4IPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint) 	# GL/glext.h:3239
PFNGLMULTITEXCOORD4IVPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:3240
PFNGLMULTITEXCOORD4SPROC = CFUNCTYPE(None, GLenum, GLshort, GLshort, GLshort, GLshort) 	# GL/glext.h:3241
PFNGLMULTITEXCOORD4SVPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:3242
PFNGLLOADTRANSPOSEMATRIXFPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:3243
PFNGLLOADTRANSPOSEMATRIXDPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:3244
PFNGLMULTTRANSPOSEMATRIXFPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:3245
PFNGLMULTTRANSPOSEMATRIXDPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:3246
PFNGLSAMPLECOVERAGEPROC = CFUNCTYPE(None, GLclampf, GLboolean) 	# GL/glext.h:3247
PFNGLCOMPRESSEDTEXIMAGE3DPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:3248
PFNGLCOMPRESSEDTEXIMAGE2DPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:3249
PFNGLCOMPRESSEDTEXIMAGE1DPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLsizei, GLint, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:3250
PFNGLCOMPRESSEDTEXSUBIMAGE3DPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:3251
PFNGLCOMPRESSEDTEXSUBIMAGE2DPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:3252
PFNGLCOMPRESSEDTEXSUBIMAGE1DPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLsizei, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:3253
PFNGLGETCOMPRESSEDTEXIMAGEPROC = CFUNCTYPE(None, GLenum, GLint, POINTER(GLvoid)) 	# GL/glext.h:3254
# VERSION_1_4 (GL/glext.h:3257)
GL_VERSION_1_4 = 1 	# GL/glext.h:3258
# GL/glext.h:3260
glBlendFuncSeparate = _link_function('glBlendFuncSeparate', None, [GLenum, GLenum, GLenum, GLenum], 'VERSION_1_4')

# GL/glext.h:3261
glFogCoordf = _link_function('glFogCoordf', None, [GLfloat], 'VERSION_1_4')

# GL/glext.h:3262
glFogCoordfv = _link_function('glFogCoordfv', None, [POINTER(GLfloat)], 'VERSION_1_4')

# GL/glext.h:3263
glFogCoordd = _link_function('glFogCoordd', None, [GLdouble], 'VERSION_1_4')

# GL/glext.h:3264
glFogCoorddv = _link_function('glFogCoorddv', None, [POINTER(GLdouble)], 'VERSION_1_4')

# GL/glext.h:3265
glFogCoordPointer = _link_function('glFogCoordPointer', None, [GLenum, GLsizei, POINTER(GLvoid)], 'VERSION_1_4')

# GL/glext.h:3266
glMultiDrawArrays = _link_function('glMultiDrawArrays', None, [GLenum, POINTER(GLint), POINTER(GLsizei), GLsizei], 'VERSION_1_4')

# GL/glext.h:3267
glMultiDrawElements = _link_function('glMultiDrawElements', None, [GLenum, POINTER(GLsizei), GLenum, POINTER(POINTER(GLvoid)), GLsizei], 'VERSION_1_4')

# GL/glext.h:3268
glPointParameterf = _link_function('glPointParameterf', None, [GLenum, GLfloat], 'VERSION_1_4')

# GL/glext.h:3269
glPointParameterfv = _link_function('glPointParameterfv', None, [GLenum, POINTER(GLfloat)], 'VERSION_1_4')

# GL/glext.h:3270
glPointParameteri = _link_function('glPointParameteri', None, [GLenum, GLint], 'VERSION_1_4')

# GL/glext.h:3271
glPointParameteriv = _link_function('glPointParameteriv', None, [GLenum, POINTER(GLint)], 'VERSION_1_4')

GLbyte = c_char 	# /usr/include/GL/gl.h:56
# GL/glext.h:3272
glSecondaryColor3b = _link_function('glSecondaryColor3b', None, [GLbyte, GLbyte, GLbyte], 'VERSION_1_4')

# GL/glext.h:3273
glSecondaryColor3bv = _link_function('glSecondaryColor3bv', None, [POINTER(GLbyte)], 'VERSION_1_4')

# GL/glext.h:3274
glSecondaryColor3d = _link_function('glSecondaryColor3d', None, [GLdouble, GLdouble, GLdouble], 'VERSION_1_4')

# GL/glext.h:3275
glSecondaryColor3dv = _link_function('glSecondaryColor3dv', None, [POINTER(GLdouble)], 'VERSION_1_4')

# GL/glext.h:3276
glSecondaryColor3f = _link_function('glSecondaryColor3f', None, [GLfloat, GLfloat, GLfloat], 'VERSION_1_4')

# GL/glext.h:3277
glSecondaryColor3fv = _link_function('glSecondaryColor3fv', None, [POINTER(GLfloat)], 'VERSION_1_4')

# GL/glext.h:3278
glSecondaryColor3i = _link_function('glSecondaryColor3i', None, [GLint, GLint, GLint], 'VERSION_1_4')

# GL/glext.h:3279
glSecondaryColor3iv = _link_function('glSecondaryColor3iv', None, [POINTER(GLint)], 'VERSION_1_4')

# GL/glext.h:3280
glSecondaryColor3s = _link_function('glSecondaryColor3s', None, [GLshort, GLshort, GLshort], 'VERSION_1_4')

# GL/glext.h:3281
glSecondaryColor3sv = _link_function('glSecondaryColor3sv', None, [POINTER(GLshort)], 'VERSION_1_4')

GLubyte = c_ubyte 	# /usr/include/GL/gl.h:60
# GL/glext.h:3282
glSecondaryColor3ub = _link_function('glSecondaryColor3ub', None, [GLubyte, GLubyte, GLubyte], 'VERSION_1_4')

# GL/glext.h:3283
glSecondaryColor3ubv = _link_function('glSecondaryColor3ubv', None, [POINTER(GLubyte)], 'VERSION_1_4')

# GL/glext.h:3284
glSecondaryColor3ui = _link_function('glSecondaryColor3ui', None, [GLuint, GLuint, GLuint], 'VERSION_1_4')

# GL/glext.h:3285
glSecondaryColor3uiv = _link_function('glSecondaryColor3uiv', None, [POINTER(GLuint)], 'VERSION_1_4')

GLushort = c_ushort 	# /usr/include/GL/gl.h:61
# GL/glext.h:3286
glSecondaryColor3us = _link_function('glSecondaryColor3us', None, [GLushort, GLushort, GLushort], 'VERSION_1_4')

# GL/glext.h:3287
glSecondaryColor3usv = _link_function('glSecondaryColor3usv', None, [POINTER(GLushort)], 'VERSION_1_4')

# GL/glext.h:3288
glSecondaryColorPointer = _link_function('glSecondaryColorPointer', None, [GLint, GLenum, GLsizei, POINTER(GLvoid)], 'VERSION_1_4')

# GL/glext.h:3289
glWindowPos2d = _link_function('glWindowPos2d', None, [GLdouble, GLdouble], 'VERSION_1_4')

# GL/glext.h:3290
glWindowPos2dv = _link_function('glWindowPos2dv', None, [POINTER(GLdouble)], 'VERSION_1_4')

# GL/glext.h:3291
glWindowPos2f = _link_function('glWindowPos2f', None, [GLfloat, GLfloat], 'VERSION_1_4')

# GL/glext.h:3292
glWindowPos2fv = _link_function('glWindowPos2fv', None, [POINTER(GLfloat)], 'VERSION_1_4')

# GL/glext.h:3293
glWindowPos2i = _link_function('glWindowPos2i', None, [GLint, GLint], 'VERSION_1_4')

# GL/glext.h:3294
glWindowPos2iv = _link_function('glWindowPos2iv', None, [POINTER(GLint)], 'VERSION_1_4')

# GL/glext.h:3295
glWindowPos2s = _link_function('glWindowPos2s', None, [GLshort, GLshort], 'VERSION_1_4')

# GL/glext.h:3296
glWindowPos2sv = _link_function('glWindowPos2sv', None, [POINTER(GLshort)], 'VERSION_1_4')

# GL/glext.h:3297
glWindowPos3d = _link_function('glWindowPos3d', None, [GLdouble, GLdouble, GLdouble], 'VERSION_1_4')

# GL/glext.h:3298
glWindowPos3dv = _link_function('glWindowPos3dv', None, [POINTER(GLdouble)], 'VERSION_1_4')

# GL/glext.h:3299
glWindowPos3f = _link_function('glWindowPos3f', None, [GLfloat, GLfloat, GLfloat], 'VERSION_1_4')

# GL/glext.h:3300
glWindowPos3fv = _link_function('glWindowPos3fv', None, [POINTER(GLfloat)], 'VERSION_1_4')

# GL/glext.h:3301
glWindowPos3i = _link_function('glWindowPos3i', None, [GLint, GLint, GLint], 'VERSION_1_4')

# GL/glext.h:3302
glWindowPos3iv = _link_function('glWindowPos3iv', None, [POINTER(GLint)], 'VERSION_1_4')

# GL/glext.h:3303
glWindowPos3s = _link_function('glWindowPos3s', None, [GLshort, GLshort, GLshort], 'VERSION_1_4')

# GL/glext.h:3304
glWindowPos3sv = _link_function('glWindowPos3sv', None, [POINTER(GLshort)], 'VERSION_1_4')

PFNGLBLENDFUNCSEPARATEPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:3306
PFNGLFOGCOORDFPROC = CFUNCTYPE(None, GLfloat) 	# GL/glext.h:3307
PFNGLFOGCOORDFVPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:3308
PFNGLFOGCOORDDPROC = CFUNCTYPE(None, GLdouble) 	# GL/glext.h:3309
PFNGLFOGCOORDDVPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:3310
PFNGLFOGCOORDPOINTERPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:3311
PFNGLMULTIDRAWARRAYSPROC = CFUNCTYPE(None, GLenum, POINTER(GLint), POINTER(GLsizei), GLsizei) 	# GL/glext.h:3312
PFNGLMULTIDRAWELEMENTSPROC = CFUNCTYPE(None, GLenum, POINTER(GLsizei), GLenum, POINTER(POINTER(GLvoid)), GLsizei) 	# GL/glext.h:3313
PFNGLPOINTPARAMETERFPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:3314
PFNGLPOINTPARAMETERFVPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:3315
PFNGLPOINTPARAMETERIPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:3316
PFNGLPOINTPARAMETERIVPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:3317
PFNGLSECONDARYCOLOR3BPROC = CFUNCTYPE(None, GLbyte, GLbyte, GLbyte) 	# GL/glext.h:3318
PFNGLSECONDARYCOLOR3BVPROC = CFUNCTYPE(None, POINTER(GLbyte)) 	# GL/glext.h:3319
PFNGLSECONDARYCOLOR3DPROC = CFUNCTYPE(None, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:3320
PFNGLSECONDARYCOLOR3DVPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:3321
PFNGLSECONDARYCOLOR3FPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:3322
PFNGLSECONDARYCOLOR3FVPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:3323
PFNGLSECONDARYCOLOR3IPROC = CFUNCTYPE(None, GLint, GLint, GLint) 	# GL/glext.h:3324
PFNGLSECONDARYCOLOR3IVPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:3325
PFNGLSECONDARYCOLOR3SPROC = CFUNCTYPE(None, GLshort, GLshort, GLshort) 	# GL/glext.h:3326
PFNGLSECONDARYCOLOR3SVPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:3327
PFNGLSECONDARYCOLOR3UBPROC = CFUNCTYPE(None, GLubyte, GLubyte, GLubyte) 	# GL/glext.h:3328
PFNGLSECONDARYCOLOR3UBVPROC = CFUNCTYPE(None, POINTER(GLubyte)) 	# GL/glext.h:3329
PFNGLSECONDARYCOLOR3UIPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint) 	# GL/glext.h:3330
PFNGLSECONDARYCOLOR3UIVPROC = CFUNCTYPE(None, POINTER(GLuint)) 	# GL/glext.h:3331
PFNGLSECONDARYCOLOR3USPROC = CFUNCTYPE(None, GLushort, GLushort, GLushort) 	# GL/glext.h:3332
PFNGLSECONDARYCOLOR3USVPROC = CFUNCTYPE(None, POINTER(GLushort)) 	# GL/glext.h:3333
PFNGLSECONDARYCOLORPOINTERPROC = CFUNCTYPE(None, GLint, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:3334
PFNGLWINDOWPOS2DPROC = CFUNCTYPE(None, GLdouble, GLdouble) 	# GL/glext.h:3335
PFNGLWINDOWPOS2DVPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:3336
PFNGLWINDOWPOS2FPROC = CFUNCTYPE(None, GLfloat, GLfloat) 	# GL/glext.h:3337
PFNGLWINDOWPOS2FVPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:3338
PFNGLWINDOWPOS2IPROC = CFUNCTYPE(None, GLint, GLint) 	# GL/glext.h:3339
PFNGLWINDOWPOS2IVPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:3340
PFNGLWINDOWPOS2SPROC = CFUNCTYPE(None, GLshort, GLshort) 	# GL/glext.h:3341
PFNGLWINDOWPOS2SVPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:3342
PFNGLWINDOWPOS3DPROC = CFUNCTYPE(None, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:3343
PFNGLWINDOWPOS3DVPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:3344
PFNGLWINDOWPOS3FPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:3345
PFNGLWINDOWPOS3FVPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:3346
PFNGLWINDOWPOS3IPROC = CFUNCTYPE(None, GLint, GLint, GLint) 	# GL/glext.h:3347
PFNGLWINDOWPOS3IVPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:3348
PFNGLWINDOWPOS3SPROC = CFUNCTYPE(None, GLshort, GLshort, GLshort) 	# GL/glext.h:3349
PFNGLWINDOWPOS3SVPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:3350
# VERSION_1_5 (GL/glext.h:3353)
GL_VERSION_1_5 = 1 	# GL/glext.h:3354
# GL/glext.h:3356
glGenQueries = _link_function('glGenQueries', None, [GLsizei, POINTER(GLuint)], 'VERSION_1_5')

# GL/glext.h:3357
glDeleteQueries = _link_function('glDeleteQueries', None, [GLsizei, POINTER(GLuint)], 'VERSION_1_5')

# GL/glext.h:3358
glIsQuery = _link_function('glIsQuery', GLboolean, [GLuint], 'VERSION_1_5')

# GL/glext.h:3359
glBeginQuery = _link_function('glBeginQuery', None, [GLenum, GLuint], 'VERSION_1_5')

# GL/glext.h:3360
glEndQuery = _link_function('glEndQuery', None, [GLenum], 'VERSION_1_5')

# GL/glext.h:3361
glGetQueryiv = _link_function('glGetQueryiv', None, [GLenum, GLenum, POINTER(GLint)], 'VERSION_1_5')

# GL/glext.h:3362
glGetQueryObjectiv = _link_function('glGetQueryObjectiv', None, [GLuint, GLenum, POINTER(GLint)], 'VERSION_1_5')

# GL/glext.h:3363
glGetQueryObjectuiv = _link_function('glGetQueryObjectuiv', None, [GLuint, GLenum, POINTER(GLuint)], 'VERSION_1_5')

# GL/glext.h:3364
glBindBuffer = _link_function('glBindBuffer', None, [GLenum, GLuint], 'VERSION_1_5')

# GL/glext.h:3365
glDeleteBuffers = _link_function('glDeleteBuffers', None, [GLsizei, POINTER(GLuint)], 'VERSION_1_5')

# GL/glext.h:3366
glGenBuffers = _link_function('glGenBuffers', None, [GLsizei, POINTER(GLuint)], 'VERSION_1_5')

# GL/glext.h:3367
glIsBuffer = _link_function('glIsBuffer', GLboolean, [GLuint], 'VERSION_1_5')

# GL/glext.h:3368
glBufferData = _link_function('glBufferData', None, [GLenum, GLsizeiptr, POINTER(GLvoid), GLenum], 'VERSION_1_5')

# GL/glext.h:3369
glBufferSubData = _link_function('glBufferSubData', None, [GLenum, GLintptr, GLsizeiptr, POINTER(GLvoid)], 'VERSION_1_5')

# GL/glext.h:3370
glGetBufferSubData = _link_function('glGetBufferSubData', None, [GLenum, GLintptr, GLsizeiptr, POINTER(GLvoid)], 'VERSION_1_5')

# GL/glext.h:3371
glMapBuffer = _link_function('glMapBuffer', POINTER(GLvoid), [GLenum, GLenum], 'VERSION_1_5')

# GL/glext.h:3372
glUnmapBuffer = _link_function('glUnmapBuffer', GLboolean, [GLenum], 'VERSION_1_5')

# GL/glext.h:3373
glGetBufferParameteriv = _link_function('glGetBufferParameteriv', None, [GLenum, GLenum, POINTER(GLint)], 'VERSION_1_5')

# GL/glext.h:3374
glGetBufferPointerv = _link_function('glGetBufferPointerv', None, [GLenum, GLenum, POINTER(POINTER(GLvoid))], 'VERSION_1_5')

PFNGLGENQUERIESPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:3376
PFNGLDELETEQUERIESPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:3377
PFNGLISQUERYPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:3378
PFNGLBEGINQUERYPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:3379
PFNGLENDQUERYPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:3380
PFNGLGETQUERYIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:3381
PFNGLGETQUERYOBJECTIVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:3382
PFNGLGETQUERYOBJECTUIVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLuint)) 	# GL/glext.h:3383
PFNGLBINDBUFFERPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:3384
PFNGLDELETEBUFFERSPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:3385
PFNGLGENBUFFERSPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:3386
PFNGLISBUFFERPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:3387
PFNGLBUFFERDATAPROC = CFUNCTYPE(None, GLenum, GLsizeiptr, POINTER(GLvoid), GLenum) 	# GL/glext.h:3388
PFNGLBUFFERSUBDATAPROC = CFUNCTYPE(None, GLenum, GLintptr, GLsizeiptr, POINTER(GLvoid)) 	# GL/glext.h:3389
PFNGLGETBUFFERSUBDATAPROC = CFUNCTYPE(None, GLenum, GLintptr, GLsizeiptr, POINTER(GLvoid)) 	# GL/glext.h:3390
PFNGLMAPBUFFERPROC = CFUNCTYPE(POINTER(GLvoid), GLenum, GLenum) 	# GL/glext.h:3391
PFNGLUNMAPBUFFERPROC = CFUNCTYPE(GLboolean, GLenum) 	# GL/glext.h:3392
PFNGLGETBUFFERPARAMETERIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:3393
PFNGLGETBUFFERPOINTERVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:3394
# VERSION_2_0 (GL/glext.h:3397)
GL_VERSION_2_0 = 1 	# GL/glext.h:3398
# GL/glext.h:3400
glBlendEquationSeparate = _link_function('glBlendEquationSeparate', None, [GLenum, GLenum], 'VERSION_2_0')

# GL/glext.h:3401
glDrawBuffers = _link_function('glDrawBuffers', None, [GLsizei, POINTER(GLenum)], 'VERSION_2_0')

# GL/glext.h:3402
glStencilOpSeparate = _link_function('glStencilOpSeparate', None, [GLenum, GLenum, GLenum, GLenum], 'VERSION_2_0')

# GL/glext.h:3403
glStencilFuncSeparate = _link_function('glStencilFuncSeparate', None, [GLenum, GLenum, GLint, GLuint], 'VERSION_2_0')

# GL/glext.h:3404
glStencilMaskSeparate = _link_function('glStencilMaskSeparate', None, [GLenum, GLuint], 'VERSION_2_0')

# GL/glext.h:3405
glAttachShader = _link_function('glAttachShader', None, [GLuint, GLuint], 'VERSION_2_0')

# GL/glext.h:3406
glBindAttribLocation = _link_function('glBindAttribLocation', None, [GLuint, GLuint, POINTER(GLchar)], 'VERSION_2_0')

# GL/glext.h:3407
glCompileShader = _link_function('glCompileShader', None, [GLuint], 'VERSION_2_0')

# GL/glext.h:3408
glCreateProgram = _link_function('glCreateProgram', GLuint, [], 'VERSION_2_0')

# GL/glext.h:3409
glCreateShader = _link_function('glCreateShader', GLuint, [GLenum], 'VERSION_2_0')

# GL/glext.h:3410
glDeleteProgram = _link_function('glDeleteProgram', None, [GLuint], 'VERSION_2_0')

# GL/glext.h:3411
glDeleteShader = _link_function('glDeleteShader', None, [GLuint], 'VERSION_2_0')

# GL/glext.h:3412
glDetachShader = _link_function('glDetachShader', None, [GLuint, GLuint], 'VERSION_2_0')

# GL/glext.h:3413
glDisableVertexAttribArray = _link_function('glDisableVertexAttribArray', None, [GLuint], 'VERSION_2_0')

# GL/glext.h:3414
glEnableVertexAttribArray = _link_function('glEnableVertexAttribArray', None, [GLuint], 'VERSION_2_0')

# GL/glext.h:3415
glGetActiveAttrib = _link_function('glGetActiveAttrib', None, [GLuint, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLint), POINTER(GLenum), POINTER(GLchar)], 'VERSION_2_0')

# GL/glext.h:3416
glGetActiveUniform = _link_function('glGetActiveUniform', None, [GLuint, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLint), POINTER(GLenum), POINTER(GLchar)], 'VERSION_2_0')

# GL/glext.h:3417
glGetAttachedShaders = _link_function('glGetAttachedShaders', None, [GLuint, GLsizei, POINTER(GLsizei), POINTER(GLuint)], 'VERSION_2_0')

# GL/glext.h:3418
glGetAttribLocation = _link_function('glGetAttribLocation', GLint, [GLuint, POINTER(GLchar)], 'VERSION_2_0')

# GL/glext.h:3419
glGetProgramiv = _link_function('glGetProgramiv', None, [GLuint, GLenum, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:3420
glGetProgramInfoLog = _link_function('glGetProgramInfoLog', None, [GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)], 'VERSION_2_0')

# GL/glext.h:3421
glGetShaderiv = _link_function('glGetShaderiv', None, [GLuint, GLenum, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:3422
glGetShaderInfoLog = _link_function('glGetShaderInfoLog', None, [GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)], 'VERSION_2_0')

# GL/glext.h:3423
glGetShaderSource = _link_function('glGetShaderSource', None, [GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)], 'VERSION_2_0')

# GL/glext.h:3424
glGetUniformLocation = _link_function('glGetUniformLocation', GLint, [GLuint, POINTER(GLchar)], 'VERSION_2_0')

# GL/glext.h:3425
glGetUniformfv = _link_function('glGetUniformfv', None, [GLuint, GLint, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:3426
glGetUniformiv = _link_function('glGetUniformiv', None, [GLuint, GLint, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:3427
glGetVertexAttribdv = _link_function('glGetVertexAttribdv', None, [GLuint, GLenum, POINTER(GLdouble)], 'VERSION_2_0')

# GL/glext.h:3428
glGetVertexAttribfv = _link_function('glGetVertexAttribfv', None, [GLuint, GLenum, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:3429
glGetVertexAttribiv = _link_function('glGetVertexAttribiv', None, [GLuint, GLenum, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:3430
glGetVertexAttribPointerv = _link_function('glGetVertexAttribPointerv', None, [GLuint, GLenum, POINTER(POINTER(GLvoid))], 'VERSION_2_0')

# GL/glext.h:3431
glIsProgram = _link_function('glIsProgram', GLboolean, [GLuint], 'VERSION_2_0')

# GL/glext.h:3432
glIsShader = _link_function('glIsShader', GLboolean, [GLuint], 'VERSION_2_0')

# GL/glext.h:3433
glLinkProgram = _link_function('glLinkProgram', None, [GLuint], 'VERSION_2_0')

# GL/glext.h:3434
glShaderSource = _link_function('glShaderSource', None, [GLuint, GLsizei, POINTER(POINTER(GLchar)), POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:3435
glUseProgram = _link_function('glUseProgram', None, [GLuint], 'VERSION_2_0')

# GL/glext.h:3436
glUniform1f = _link_function('glUniform1f', None, [GLint, GLfloat], 'VERSION_2_0')

# GL/glext.h:3437
glUniform2f = _link_function('glUniform2f', None, [GLint, GLfloat, GLfloat], 'VERSION_2_0')

# GL/glext.h:3438
glUniform3f = _link_function('glUniform3f', None, [GLint, GLfloat, GLfloat, GLfloat], 'VERSION_2_0')

# GL/glext.h:3439
glUniform4f = _link_function('glUniform4f', None, [GLint, GLfloat, GLfloat, GLfloat, GLfloat], 'VERSION_2_0')

# GL/glext.h:3440
glUniform1i = _link_function('glUniform1i', None, [GLint, GLint], 'VERSION_2_0')

# GL/glext.h:3441
glUniform2i = _link_function('glUniform2i', None, [GLint, GLint, GLint], 'VERSION_2_0')

# GL/glext.h:3442
glUniform3i = _link_function('glUniform3i', None, [GLint, GLint, GLint, GLint], 'VERSION_2_0')

# GL/glext.h:3443
glUniform4i = _link_function('glUniform4i', None, [GLint, GLint, GLint, GLint, GLint], 'VERSION_2_0')

# GL/glext.h:3444
glUniform1fv = _link_function('glUniform1fv', None, [GLint, GLsizei, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:3445
glUniform2fv = _link_function('glUniform2fv', None, [GLint, GLsizei, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:3446
glUniform3fv = _link_function('glUniform3fv', None, [GLint, GLsizei, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:3447
glUniform4fv = _link_function('glUniform4fv', None, [GLint, GLsizei, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:3448
glUniform1iv = _link_function('glUniform1iv', None, [GLint, GLsizei, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:3449
glUniform2iv = _link_function('glUniform2iv', None, [GLint, GLsizei, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:3450
glUniform3iv = _link_function('glUniform3iv', None, [GLint, GLsizei, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:3451
glUniform4iv = _link_function('glUniform4iv', None, [GLint, GLsizei, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:3452
glUniformMatrix2fv = _link_function('glUniformMatrix2fv', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:3453
glUniformMatrix3fv = _link_function('glUniformMatrix3fv', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:3454
glUniformMatrix4fv = _link_function('glUniformMatrix4fv', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:3455
glValidateProgram = _link_function('glValidateProgram', None, [GLuint], 'VERSION_2_0')

# GL/glext.h:3456
glVertexAttrib1d = _link_function('glVertexAttrib1d', None, [GLuint, GLdouble], 'VERSION_2_0')

# GL/glext.h:3457
glVertexAttrib1dv = _link_function('glVertexAttrib1dv', None, [GLuint, POINTER(GLdouble)], 'VERSION_2_0')

# GL/glext.h:3458
glVertexAttrib1f = _link_function('glVertexAttrib1f', None, [GLuint, GLfloat], 'VERSION_2_0')

# GL/glext.h:3459
glVertexAttrib1fv = _link_function('glVertexAttrib1fv', None, [GLuint, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:3460
glVertexAttrib1s = _link_function('glVertexAttrib1s', None, [GLuint, GLshort], 'VERSION_2_0')

# GL/glext.h:3461
glVertexAttrib1sv = _link_function('glVertexAttrib1sv', None, [GLuint, POINTER(GLshort)], 'VERSION_2_0')

# GL/glext.h:3462
glVertexAttrib2d = _link_function('glVertexAttrib2d', None, [GLuint, GLdouble, GLdouble], 'VERSION_2_0')

# GL/glext.h:3463
glVertexAttrib2dv = _link_function('glVertexAttrib2dv', None, [GLuint, POINTER(GLdouble)], 'VERSION_2_0')

# GL/glext.h:3464
glVertexAttrib2f = _link_function('glVertexAttrib2f', None, [GLuint, GLfloat, GLfloat], 'VERSION_2_0')

# GL/glext.h:3465
glVertexAttrib2fv = _link_function('glVertexAttrib2fv', None, [GLuint, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:3466
glVertexAttrib2s = _link_function('glVertexAttrib2s', None, [GLuint, GLshort, GLshort], 'VERSION_2_0')

# GL/glext.h:3467
glVertexAttrib2sv = _link_function('glVertexAttrib2sv', None, [GLuint, POINTER(GLshort)], 'VERSION_2_0')

# GL/glext.h:3468
glVertexAttrib3d = _link_function('glVertexAttrib3d', None, [GLuint, GLdouble, GLdouble, GLdouble], 'VERSION_2_0')

# GL/glext.h:3469
glVertexAttrib3dv = _link_function('glVertexAttrib3dv', None, [GLuint, POINTER(GLdouble)], 'VERSION_2_0')

# GL/glext.h:3470
glVertexAttrib3f = _link_function('glVertexAttrib3f', None, [GLuint, GLfloat, GLfloat, GLfloat], 'VERSION_2_0')

# GL/glext.h:3471
glVertexAttrib3fv = _link_function('glVertexAttrib3fv', None, [GLuint, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:3472
glVertexAttrib3s = _link_function('glVertexAttrib3s', None, [GLuint, GLshort, GLshort, GLshort], 'VERSION_2_0')

# GL/glext.h:3473
glVertexAttrib3sv = _link_function('glVertexAttrib3sv', None, [GLuint, POINTER(GLshort)], 'VERSION_2_0')

# GL/glext.h:3474
glVertexAttrib4Nbv = _link_function('glVertexAttrib4Nbv', None, [GLuint, POINTER(GLbyte)], 'VERSION_2_0')

# GL/glext.h:3475
glVertexAttrib4Niv = _link_function('glVertexAttrib4Niv', None, [GLuint, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:3476
glVertexAttrib4Nsv = _link_function('glVertexAttrib4Nsv', None, [GLuint, POINTER(GLshort)], 'VERSION_2_0')

# GL/glext.h:3477
glVertexAttrib4Nub = _link_function('glVertexAttrib4Nub', None, [GLuint, GLubyte, GLubyte, GLubyte, GLubyte], 'VERSION_2_0')

# GL/glext.h:3478
glVertexAttrib4Nubv = _link_function('glVertexAttrib4Nubv', None, [GLuint, POINTER(GLubyte)], 'VERSION_2_0')

# GL/glext.h:3479
glVertexAttrib4Nuiv = _link_function('glVertexAttrib4Nuiv', None, [GLuint, POINTER(GLuint)], 'VERSION_2_0')

# GL/glext.h:3480
glVertexAttrib4Nusv = _link_function('glVertexAttrib4Nusv', None, [GLuint, POINTER(GLushort)], 'VERSION_2_0')

# GL/glext.h:3481
glVertexAttrib4bv = _link_function('glVertexAttrib4bv', None, [GLuint, POINTER(GLbyte)], 'VERSION_2_0')

# GL/glext.h:3482
glVertexAttrib4d = _link_function('glVertexAttrib4d', None, [GLuint, GLdouble, GLdouble, GLdouble, GLdouble], 'VERSION_2_0')

# GL/glext.h:3483
glVertexAttrib4dv = _link_function('glVertexAttrib4dv', None, [GLuint, POINTER(GLdouble)], 'VERSION_2_0')

# GL/glext.h:3484
glVertexAttrib4f = _link_function('glVertexAttrib4f', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat], 'VERSION_2_0')

# GL/glext.h:3485
glVertexAttrib4fv = _link_function('glVertexAttrib4fv', None, [GLuint, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:3486
glVertexAttrib4iv = _link_function('glVertexAttrib4iv', None, [GLuint, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:3487
glVertexAttrib4s = _link_function('glVertexAttrib4s', None, [GLuint, GLshort, GLshort, GLshort, GLshort], 'VERSION_2_0')

# GL/glext.h:3488
glVertexAttrib4sv = _link_function('glVertexAttrib4sv', None, [GLuint, POINTER(GLshort)], 'VERSION_2_0')

# GL/glext.h:3489
glVertexAttrib4ubv = _link_function('glVertexAttrib4ubv', None, [GLuint, POINTER(GLubyte)], 'VERSION_2_0')

# GL/glext.h:3490
glVertexAttrib4uiv = _link_function('glVertexAttrib4uiv', None, [GLuint, POINTER(GLuint)], 'VERSION_2_0')

# GL/glext.h:3491
glVertexAttrib4usv = _link_function('glVertexAttrib4usv', None, [GLuint, POINTER(GLushort)], 'VERSION_2_0')

# GL/glext.h:3492
glVertexAttribPointer = _link_function('glVertexAttribPointer', None, [GLuint, GLint, GLenum, GLboolean, GLsizei, POINTER(GLvoid)], 'VERSION_2_0')

PFNGLBLENDEQUATIONSEPARATEPROC = CFUNCTYPE(None, GLenum, GLenum) 	# GL/glext.h:3494
PFNGLDRAWBUFFERSPROC = CFUNCTYPE(None, GLsizei, POINTER(GLenum)) 	# GL/glext.h:3495
PFNGLSTENCILOPSEPARATEPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:3496
PFNGLSTENCILFUNCSEPARATEPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLuint) 	# GL/glext.h:3497
PFNGLSTENCILMASKSEPARATEPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:3498
PFNGLATTACHSHADERPROC = CFUNCTYPE(None, GLuint, GLuint) 	# GL/glext.h:3499
PFNGLBINDATTRIBLOCATIONPROC = CFUNCTYPE(None, GLuint, GLuint, POINTER(GLchar)) 	# GL/glext.h:3500
PFNGLCOMPILESHADERPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:3501
PFNGLCREATEPROGRAMPROC = CFUNCTYPE(GLuint) 	# GL/glext.h:3502
PFNGLCREATESHADERPROC = CFUNCTYPE(GLuint, GLenum) 	# GL/glext.h:3503
PFNGLDELETEPROGRAMPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:3504
PFNGLDELETESHADERPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:3505
PFNGLDETACHSHADERPROC = CFUNCTYPE(None, GLuint, GLuint) 	# GL/glext.h:3506
PFNGLDISABLEVERTEXATTRIBARRAYPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:3507
PFNGLENABLEVERTEXATTRIBARRAYPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:3508
PFNGLGETACTIVEATTRIBPROC = CFUNCTYPE(None, GLuint, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLint), POINTER(GLenum), POINTER(GLchar)) 	# GL/glext.h:3509
PFNGLGETACTIVEUNIFORMPROC = CFUNCTYPE(None, GLuint, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLint), POINTER(GLenum), POINTER(GLchar)) 	# GL/glext.h:3510
PFNGLGETATTACHEDSHADERSPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLuint)) 	# GL/glext.h:3511
PFNGLGETATTRIBLOCATIONPROC = CFUNCTYPE(GLint, GLuint, POINTER(GLchar)) 	# GL/glext.h:3512
PFNGLGETPROGRAMIVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:3513
PFNGLGETPROGRAMINFOLOGPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)) 	# GL/glext.h:3514
PFNGLGETSHADERIVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:3515
PFNGLGETSHADERINFOLOGPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)) 	# GL/glext.h:3516
PFNGLGETSHADERSOURCEPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)) 	# GL/glext.h:3517
PFNGLGETUNIFORMLOCATIONPROC = CFUNCTYPE(GLint, GLuint, POINTER(GLchar)) 	# GL/glext.h:3518
PFNGLGETUNIFORMFVPROC = CFUNCTYPE(None, GLuint, GLint, POINTER(GLfloat)) 	# GL/glext.h:3519
PFNGLGETUNIFORMIVPROC = CFUNCTYPE(None, GLuint, GLint, POINTER(GLint)) 	# GL/glext.h:3520
PFNGLGETVERTEXATTRIBDVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLdouble)) 	# GL/glext.h:3521
PFNGLGETVERTEXATTRIBFVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:3522
PFNGLGETVERTEXATTRIBIVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:3523
PFNGLGETVERTEXATTRIBPOINTERVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:3524
PFNGLISPROGRAMPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:3525
PFNGLISSHADERPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:3526
PFNGLLINKPROGRAMPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:3527
PFNGLSHADERSOURCEPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(POINTER(GLchar)), POINTER(GLint)) 	# GL/glext.h:3528
PFNGLUSEPROGRAMPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:3529
PFNGLUNIFORM1FPROC = CFUNCTYPE(None, GLint, GLfloat) 	# GL/glext.h:3530
PFNGLUNIFORM2FPROC = CFUNCTYPE(None, GLint, GLfloat, GLfloat) 	# GL/glext.h:3531
PFNGLUNIFORM3FPROC = CFUNCTYPE(None, GLint, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:3532
PFNGLUNIFORM4FPROC = CFUNCTYPE(None, GLint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:3533
PFNGLUNIFORM1IPROC = CFUNCTYPE(None, GLint, GLint) 	# GL/glext.h:3534
PFNGLUNIFORM2IPROC = CFUNCTYPE(None, GLint, GLint, GLint) 	# GL/glext.h:3535
PFNGLUNIFORM3IPROC = CFUNCTYPE(None, GLint, GLint, GLint, GLint) 	# GL/glext.h:3536
PFNGLUNIFORM4IPROC = CFUNCTYPE(None, GLint, GLint, GLint, GLint, GLint) 	# GL/glext.h:3537
PFNGLUNIFORM1FVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:3538
PFNGLUNIFORM2FVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:3539
PFNGLUNIFORM3FVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:3540
PFNGLUNIFORM4FVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:3541
PFNGLUNIFORM1IVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:3542
PFNGLUNIFORM2IVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:3543
PFNGLUNIFORM3IVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:3544
PFNGLUNIFORM4IVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:3545
PFNGLUNIFORMMATRIX2FVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:3546
PFNGLUNIFORMMATRIX3FVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:3547
PFNGLUNIFORMMATRIX4FVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:3548
PFNGLVALIDATEPROGRAMPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:3549
PFNGLVERTEXATTRIB1DPROC = CFUNCTYPE(None, GLuint, GLdouble) 	# GL/glext.h:3550
PFNGLVERTEXATTRIB1DVPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:3551
PFNGLVERTEXATTRIB1FPROC = CFUNCTYPE(None, GLuint, GLfloat) 	# GL/glext.h:3552
PFNGLVERTEXATTRIB1FVPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:3553
PFNGLVERTEXATTRIB1SPROC = CFUNCTYPE(None, GLuint, GLshort) 	# GL/glext.h:3554
PFNGLVERTEXATTRIB1SVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:3555
PFNGLVERTEXATTRIB2DPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble) 	# GL/glext.h:3556
PFNGLVERTEXATTRIB2DVPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:3557
PFNGLVERTEXATTRIB2FPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat) 	# GL/glext.h:3558
PFNGLVERTEXATTRIB2FVPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:3559
PFNGLVERTEXATTRIB2SPROC = CFUNCTYPE(None, GLuint, GLshort, GLshort) 	# GL/glext.h:3560
PFNGLVERTEXATTRIB2SVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:3561
PFNGLVERTEXATTRIB3DPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:3562
PFNGLVERTEXATTRIB3DVPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:3563
PFNGLVERTEXATTRIB3FPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:3564
PFNGLVERTEXATTRIB3FVPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:3565
PFNGLVERTEXATTRIB3SPROC = CFUNCTYPE(None, GLuint, GLshort, GLshort, GLshort) 	# GL/glext.h:3566
PFNGLVERTEXATTRIB3SVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:3567
PFNGLVERTEXATTRIB4NBVPROC = CFUNCTYPE(None, GLuint, POINTER(GLbyte)) 	# GL/glext.h:3568
PFNGLVERTEXATTRIB4NIVPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:3569
PFNGLVERTEXATTRIB4NSVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:3570
PFNGLVERTEXATTRIB4NUBPROC = CFUNCTYPE(None, GLuint, GLubyte, GLubyte, GLubyte, GLubyte) 	# GL/glext.h:3571
PFNGLVERTEXATTRIB4NUBVPROC = CFUNCTYPE(None, GLuint, POINTER(GLubyte)) 	# GL/glext.h:3572
PFNGLVERTEXATTRIB4NUIVPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:3573
PFNGLVERTEXATTRIB4NUSVPROC = CFUNCTYPE(None, GLuint, POINTER(GLushort)) 	# GL/glext.h:3574
PFNGLVERTEXATTRIB4BVPROC = CFUNCTYPE(None, GLuint, POINTER(GLbyte)) 	# GL/glext.h:3575
PFNGLVERTEXATTRIB4DPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:3576
PFNGLVERTEXATTRIB4DVPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:3577
PFNGLVERTEXATTRIB4FPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:3578
PFNGLVERTEXATTRIB4FVPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:3579
PFNGLVERTEXATTRIB4IVPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:3580
PFNGLVERTEXATTRIB4SPROC = CFUNCTYPE(None, GLuint, GLshort, GLshort, GLshort, GLshort) 	# GL/glext.h:3581
PFNGLVERTEXATTRIB4SVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:3582
PFNGLVERTEXATTRIB4UBVPROC = CFUNCTYPE(None, GLuint, POINTER(GLubyte)) 	# GL/glext.h:3583
PFNGLVERTEXATTRIB4UIVPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:3584
PFNGLVERTEXATTRIB4USVPROC = CFUNCTYPE(None, GLuint, POINTER(GLushort)) 	# GL/glext.h:3585
PFNGLVERTEXATTRIBPOINTERPROC = CFUNCTYPE(None, GLuint, GLint, GLenum, GLboolean, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:3586
# ARB_multitexture (GL/glext.h:3589)
GL_ARB_multitexture = 1 	# GL/glext.h:3590
# GL/glext.h:3592
glActiveTextureARB = _link_function('glActiveTextureARB', None, [GLenum], 'ARB_multitexture')

# GL/glext.h:3593
glClientActiveTextureARB = _link_function('glClientActiveTextureARB', None, [GLenum], 'ARB_multitexture')

# GL/glext.h:3594
glMultiTexCoord1dARB = _link_function('glMultiTexCoord1dARB', None, [GLenum, GLdouble], 'ARB_multitexture')

# GL/glext.h:3595
glMultiTexCoord1dvARB = _link_function('glMultiTexCoord1dvARB', None, [GLenum, POINTER(GLdouble)], 'ARB_multitexture')

# GL/glext.h:3596
glMultiTexCoord1fARB = _link_function('glMultiTexCoord1fARB', None, [GLenum, GLfloat], 'ARB_multitexture')

# GL/glext.h:3597
glMultiTexCoord1fvARB = _link_function('glMultiTexCoord1fvARB', None, [GLenum, POINTER(GLfloat)], 'ARB_multitexture')

# GL/glext.h:3598
glMultiTexCoord1iARB = _link_function('glMultiTexCoord1iARB', None, [GLenum, GLint], 'ARB_multitexture')

# GL/glext.h:3599
glMultiTexCoord1ivARB = _link_function('glMultiTexCoord1ivARB', None, [GLenum, POINTER(GLint)], 'ARB_multitexture')

# GL/glext.h:3600
glMultiTexCoord1sARB = _link_function('glMultiTexCoord1sARB', None, [GLenum, GLshort], 'ARB_multitexture')

# GL/glext.h:3601
glMultiTexCoord1svARB = _link_function('glMultiTexCoord1svARB', None, [GLenum, POINTER(GLshort)], 'ARB_multitexture')

# GL/glext.h:3602
glMultiTexCoord2dARB = _link_function('glMultiTexCoord2dARB', None, [GLenum, GLdouble, GLdouble], 'ARB_multitexture')

# GL/glext.h:3603
glMultiTexCoord2dvARB = _link_function('glMultiTexCoord2dvARB', None, [GLenum, POINTER(GLdouble)], 'ARB_multitexture')

# GL/glext.h:3604
glMultiTexCoord2fARB = _link_function('glMultiTexCoord2fARB', None, [GLenum, GLfloat, GLfloat], 'ARB_multitexture')

# GL/glext.h:3605
glMultiTexCoord2fvARB = _link_function('glMultiTexCoord2fvARB', None, [GLenum, POINTER(GLfloat)], 'ARB_multitexture')

# GL/glext.h:3606
glMultiTexCoord2iARB = _link_function('glMultiTexCoord2iARB', None, [GLenum, GLint, GLint], 'ARB_multitexture')

# GL/glext.h:3607
glMultiTexCoord2ivARB = _link_function('glMultiTexCoord2ivARB', None, [GLenum, POINTER(GLint)], 'ARB_multitexture')

# GL/glext.h:3608
glMultiTexCoord2sARB = _link_function('glMultiTexCoord2sARB', None, [GLenum, GLshort, GLshort], 'ARB_multitexture')

# GL/glext.h:3609
glMultiTexCoord2svARB = _link_function('glMultiTexCoord2svARB', None, [GLenum, POINTER(GLshort)], 'ARB_multitexture')

# GL/glext.h:3610
glMultiTexCoord3dARB = _link_function('glMultiTexCoord3dARB', None, [GLenum, GLdouble, GLdouble, GLdouble], 'ARB_multitexture')

# GL/glext.h:3611
glMultiTexCoord3dvARB = _link_function('glMultiTexCoord3dvARB', None, [GLenum, POINTER(GLdouble)], 'ARB_multitexture')

# GL/glext.h:3612
glMultiTexCoord3fARB = _link_function('glMultiTexCoord3fARB', None, [GLenum, GLfloat, GLfloat, GLfloat], 'ARB_multitexture')

# GL/glext.h:3613
glMultiTexCoord3fvARB = _link_function('glMultiTexCoord3fvARB', None, [GLenum, POINTER(GLfloat)], 'ARB_multitexture')

# GL/glext.h:3614
glMultiTexCoord3iARB = _link_function('glMultiTexCoord3iARB', None, [GLenum, GLint, GLint, GLint], 'ARB_multitexture')

# GL/glext.h:3615
glMultiTexCoord3ivARB = _link_function('glMultiTexCoord3ivARB', None, [GLenum, POINTER(GLint)], 'ARB_multitexture')

# GL/glext.h:3616
glMultiTexCoord3sARB = _link_function('glMultiTexCoord3sARB', None, [GLenum, GLshort, GLshort, GLshort], 'ARB_multitexture')

# GL/glext.h:3617
glMultiTexCoord3svARB = _link_function('glMultiTexCoord3svARB', None, [GLenum, POINTER(GLshort)], 'ARB_multitexture')

# GL/glext.h:3618
glMultiTexCoord4dARB = _link_function('glMultiTexCoord4dARB', None, [GLenum, GLdouble, GLdouble, GLdouble, GLdouble], 'ARB_multitexture')

# GL/glext.h:3619
glMultiTexCoord4dvARB = _link_function('glMultiTexCoord4dvARB', None, [GLenum, POINTER(GLdouble)], 'ARB_multitexture')

# GL/glext.h:3620
glMultiTexCoord4fARB = _link_function('glMultiTexCoord4fARB', None, [GLenum, GLfloat, GLfloat, GLfloat, GLfloat], 'ARB_multitexture')

# GL/glext.h:3621
glMultiTexCoord4fvARB = _link_function('glMultiTexCoord4fvARB', None, [GLenum, POINTER(GLfloat)], 'ARB_multitexture')

# GL/glext.h:3622
glMultiTexCoord4iARB = _link_function('glMultiTexCoord4iARB', None, [GLenum, GLint, GLint, GLint, GLint], 'ARB_multitexture')

# GL/glext.h:3623
glMultiTexCoord4ivARB = _link_function('glMultiTexCoord4ivARB', None, [GLenum, POINTER(GLint)], 'ARB_multitexture')

# GL/glext.h:3624
glMultiTexCoord4sARB = _link_function('glMultiTexCoord4sARB', None, [GLenum, GLshort, GLshort, GLshort, GLshort], 'ARB_multitexture')

# GL/glext.h:3625
glMultiTexCoord4svARB = _link_function('glMultiTexCoord4svARB', None, [GLenum, POINTER(GLshort)], 'ARB_multitexture')

PFNGLACTIVETEXTUREARBPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:3627
PFNGLCLIENTACTIVETEXTUREARBPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:3628
PFNGLMULTITEXCOORD1DARBPROC = CFUNCTYPE(None, GLenum, GLdouble) 	# GL/glext.h:3629
PFNGLMULTITEXCOORD1DVARBPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:3630
PFNGLMULTITEXCOORD1FARBPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:3631
PFNGLMULTITEXCOORD1FVARBPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:3632
PFNGLMULTITEXCOORD1IARBPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:3633
PFNGLMULTITEXCOORD1IVARBPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:3634
PFNGLMULTITEXCOORD1SARBPROC = CFUNCTYPE(None, GLenum, GLshort) 	# GL/glext.h:3635
PFNGLMULTITEXCOORD1SVARBPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:3636
PFNGLMULTITEXCOORD2DARBPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble) 	# GL/glext.h:3637
PFNGLMULTITEXCOORD2DVARBPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:3638
PFNGLMULTITEXCOORD2FARBPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat) 	# GL/glext.h:3639
PFNGLMULTITEXCOORD2FVARBPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:3640
PFNGLMULTITEXCOORD2IARBPROC = CFUNCTYPE(None, GLenum, GLint, GLint) 	# GL/glext.h:3641
PFNGLMULTITEXCOORD2IVARBPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:3642
PFNGLMULTITEXCOORD2SARBPROC = CFUNCTYPE(None, GLenum, GLshort, GLshort) 	# GL/glext.h:3643
PFNGLMULTITEXCOORD2SVARBPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:3644
PFNGLMULTITEXCOORD3DARBPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:3645
PFNGLMULTITEXCOORD3DVARBPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:3646
PFNGLMULTITEXCOORD3FARBPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:3647
PFNGLMULTITEXCOORD3FVARBPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:3648
PFNGLMULTITEXCOORD3IARBPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint) 	# GL/glext.h:3649
PFNGLMULTITEXCOORD3IVARBPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:3650
PFNGLMULTITEXCOORD3SARBPROC = CFUNCTYPE(None, GLenum, GLshort, GLshort, GLshort) 	# GL/glext.h:3651
PFNGLMULTITEXCOORD3SVARBPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:3652
PFNGLMULTITEXCOORD4DARBPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:3653
PFNGLMULTITEXCOORD4DVARBPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:3654
PFNGLMULTITEXCOORD4FARBPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:3655
PFNGLMULTITEXCOORD4FVARBPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:3656
PFNGLMULTITEXCOORD4IARBPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint) 	# GL/glext.h:3657
PFNGLMULTITEXCOORD4IVARBPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:3658
PFNGLMULTITEXCOORD4SARBPROC = CFUNCTYPE(None, GLenum, GLshort, GLshort, GLshort, GLshort) 	# GL/glext.h:3659
PFNGLMULTITEXCOORD4SVARBPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:3660
# ARB_transpose_matrix (GL/glext.h:3663)
GL_ARB_transpose_matrix = 1 	# GL/glext.h:3664
# GL/glext.h:3666
glLoadTransposeMatrixfARB = _link_function('glLoadTransposeMatrixfARB', None, [POINTER(GLfloat)], 'ARB_transpose_matrix')

# GL/glext.h:3667
glLoadTransposeMatrixdARB = _link_function('glLoadTransposeMatrixdARB', None, [POINTER(GLdouble)], 'ARB_transpose_matrix')

# GL/glext.h:3668
glMultTransposeMatrixfARB = _link_function('glMultTransposeMatrixfARB', None, [POINTER(GLfloat)], 'ARB_transpose_matrix')

# GL/glext.h:3669
glMultTransposeMatrixdARB = _link_function('glMultTransposeMatrixdARB', None, [POINTER(GLdouble)], 'ARB_transpose_matrix')

PFNGLLOADTRANSPOSEMATRIXFARBPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:3671
PFNGLLOADTRANSPOSEMATRIXDARBPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:3672
PFNGLMULTTRANSPOSEMATRIXFARBPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:3673
PFNGLMULTTRANSPOSEMATRIXDARBPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:3674
# ARB_multisample (GL/glext.h:3677)
GL_ARB_multisample = 1 	# GL/glext.h:3678
# GL/glext.h:3680
glSampleCoverageARB = _link_function('glSampleCoverageARB', None, [GLclampf, GLboolean], 'ARB_multisample')

PFNGLSAMPLECOVERAGEARBPROC = CFUNCTYPE(None, GLclampf, GLboolean) 	# GL/glext.h:3682
# ARB_texture_env_add (GL/glext.h:3685)
GL_ARB_texture_env_add = 1 	# GL/glext.h:3686
# ARB_texture_cube_map (GL/glext.h:3689)
GL_ARB_texture_cube_map = 1 	# GL/glext.h:3690
# ARB_texture_compression (GL/glext.h:3693)
GL_ARB_texture_compression = 1 	# GL/glext.h:3694
# GL/glext.h:3696
glCompressedTexImage3DARB = _link_function('glCompressedTexImage3DARB', None, [GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)], 'ARB_texture_compression')

# GL/glext.h:3697
glCompressedTexImage2DARB = _link_function('glCompressedTexImage2DARB', None, [GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)], 'ARB_texture_compression')

# GL/glext.h:3698
glCompressedTexImage1DARB = _link_function('glCompressedTexImage1DARB', None, [GLenum, GLint, GLenum, GLsizei, GLint, GLsizei, POINTER(GLvoid)], 'ARB_texture_compression')

# GL/glext.h:3699
glCompressedTexSubImage3DARB = _link_function('glCompressedTexSubImage3DARB', None, [GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)], 'ARB_texture_compression')

# GL/glext.h:3700
glCompressedTexSubImage2DARB = _link_function('glCompressedTexSubImage2DARB', None, [GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)], 'ARB_texture_compression')

# GL/glext.h:3701
glCompressedTexSubImage1DARB = _link_function('glCompressedTexSubImage1DARB', None, [GLenum, GLint, GLint, GLsizei, GLenum, GLsizei, POINTER(GLvoid)], 'ARB_texture_compression')

# GL/glext.h:3702
glGetCompressedTexImageARB = _link_function('glGetCompressedTexImageARB', None, [GLenum, GLint, POINTER(GLvoid)], 'ARB_texture_compression')

PFNGLCOMPRESSEDTEXIMAGE3DARBPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:3704
PFNGLCOMPRESSEDTEXIMAGE2DARBPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:3705
PFNGLCOMPRESSEDTEXIMAGE1DARBPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLsizei, GLint, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:3706
PFNGLCOMPRESSEDTEXSUBIMAGE3DARBPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:3707
PFNGLCOMPRESSEDTEXSUBIMAGE2DARBPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:3708
PFNGLCOMPRESSEDTEXSUBIMAGE1DARBPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLsizei, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:3709
PFNGLGETCOMPRESSEDTEXIMAGEARBPROC = CFUNCTYPE(None, GLenum, GLint, POINTER(GLvoid)) 	# GL/glext.h:3710
# ARB_texture_border_clamp (GL/glext.h:3713)
GL_ARB_texture_border_clamp = 1 	# GL/glext.h:3714
# ARB_point_parameters (GL/glext.h:3717)
GL_ARB_point_parameters = 1 	# GL/glext.h:3718
# GL/glext.h:3720
glPointParameterfARB = _link_function('glPointParameterfARB', None, [GLenum, GLfloat], 'ARB_point_parameters')

# GL/glext.h:3721
glPointParameterfvARB = _link_function('glPointParameterfvARB', None, [GLenum, POINTER(GLfloat)], 'ARB_point_parameters')

PFNGLPOINTPARAMETERFARBPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:3723
PFNGLPOINTPARAMETERFVARBPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:3724
# ARB_vertex_blend (GL/glext.h:3727)
GL_ARB_vertex_blend = 1 	# GL/glext.h:3728
# GL/glext.h:3730
glWeightbvARB = _link_function('glWeightbvARB', None, [GLint, POINTER(GLbyte)], 'ARB_vertex_blend')

# GL/glext.h:3731
glWeightsvARB = _link_function('glWeightsvARB', None, [GLint, POINTER(GLshort)], 'ARB_vertex_blend')

# GL/glext.h:3732
glWeightivARB = _link_function('glWeightivARB', None, [GLint, POINTER(GLint)], 'ARB_vertex_blend')

# GL/glext.h:3733
glWeightfvARB = _link_function('glWeightfvARB', None, [GLint, POINTER(GLfloat)], 'ARB_vertex_blend')

# GL/glext.h:3734
glWeightdvARB = _link_function('glWeightdvARB', None, [GLint, POINTER(GLdouble)], 'ARB_vertex_blend')

# GL/glext.h:3735
glWeightubvARB = _link_function('glWeightubvARB', None, [GLint, POINTER(GLubyte)], 'ARB_vertex_blend')

# GL/glext.h:3736
glWeightusvARB = _link_function('glWeightusvARB', None, [GLint, POINTER(GLushort)], 'ARB_vertex_blend')

# GL/glext.h:3737
glWeightuivARB = _link_function('glWeightuivARB', None, [GLint, POINTER(GLuint)], 'ARB_vertex_blend')

# GL/glext.h:3738
glWeightPointerARB = _link_function('glWeightPointerARB', None, [GLint, GLenum, GLsizei, POINTER(GLvoid)], 'ARB_vertex_blend')

# GL/glext.h:3739
glVertexBlendARB = _link_function('glVertexBlendARB', None, [GLint], 'ARB_vertex_blend')

PFNGLWEIGHTBVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLbyte)) 	# GL/glext.h:3741
PFNGLWEIGHTSVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLshort)) 	# GL/glext.h:3742
PFNGLWEIGHTIVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLint)) 	# GL/glext.h:3743
PFNGLWEIGHTFVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLfloat)) 	# GL/glext.h:3744
PFNGLWEIGHTDVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLdouble)) 	# GL/glext.h:3745
PFNGLWEIGHTUBVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLubyte)) 	# GL/glext.h:3746
PFNGLWEIGHTUSVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLushort)) 	# GL/glext.h:3747
PFNGLWEIGHTUIVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLuint)) 	# GL/glext.h:3748
PFNGLWEIGHTPOINTERARBPROC = CFUNCTYPE(None, GLint, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:3749
PFNGLVERTEXBLENDARBPROC = CFUNCTYPE(None, GLint) 	# GL/glext.h:3750
# ARB_matrix_palette (GL/glext.h:3753)
GL_ARB_matrix_palette = 1 	# GL/glext.h:3754
# GL/glext.h:3756
glCurrentPaletteMatrixARB = _link_function('glCurrentPaletteMatrixARB', None, [GLint], 'ARB_matrix_palette')

# GL/glext.h:3757
glMatrixIndexubvARB = _link_function('glMatrixIndexubvARB', None, [GLint, POINTER(GLubyte)], 'ARB_matrix_palette')

# GL/glext.h:3758
glMatrixIndexusvARB = _link_function('glMatrixIndexusvARB', None, [GLint, POINTER(GLushort)], 'ARB_matrix_palette')

# GL/glext.h:3759
glMatrixIndexuivARB = _link_function('glMatrixIndexuivARB', None, [GLint, POINTER(GLuint)], 'ARB_matrix_palette')

# GL/glext.h:3760
glMatrixIndexPointerARB = _link_function('glMatrixIndexPointerARB', None, [GLint, GLenum, GLsizei, POINTER(GLvoid)], 'ARB_matrix_palette')

PFNGLCURRENTPALETTEMATRIXARBPROC = CFUNCTYPE(None, GLint) 	# GL/glext.h:3762
PFNGLMATRIXINDEXUBVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLubyte)) 	# GL/glext.h:3763
PFNGLMATRIXINDEXUSVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLushort)) 	# GL/glext.h:3764
PFNGLMATRIXINDEXUIVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLuint)) 	# GL/glext.h:3765
PFNGLMATRIXINDEXPOINTERARBPROC = CFUNCTYPE(None, GLint, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:3766
# ARB_texture_env_combine (GL/glext.h:3769)
GL_ARB_texture_env_combine = 1 	# GL/glext.h:3770
# ARB_texture_env_crossbar (GL/glext.h:3773)
GL_ARB_texture_env_crossbar = 1 	# GL/glext.h:3774
# ARB_texture_env_dot3 (GL/glext.h:3777)
GL_ARB_texture_env_dot3 = 1 	# GL/glext.h:3778
# ARB_texture_mirrored_repeat (GL/glext.h:3781)
GL_ARB_texture_mirrored_repeat = 1 	# GL/glext.h:3782
# ARB_depth_texture (GL/glext.h:3785)
GL_ARB_depth_texture = 1 	# GL/glext.h:3786
# ARB_shadow (GL/glext.h:3789)
GL_ARB_shadow = 1 	# GL/glext.h:3790
# ARB_shadow_ambient (GL/glext.h:3793)
GL_ARB_shadow_ambient = 1 	# GL/glext.h:3794
# ARB_window_pos (GL/glext.h:3797)
GL_ARB_window_pos = 1 	# GL/glext.h:3798
# GL/glext.h:3800
glWindowPos2dARB = _link_function('glWindowPos2dARB', None, [GLdouble, GLdouble], 'ARB_window_pos')

# GL/glext.h:3801
glWindowPos2dvARB = _link_function('glWindowPos2dvARB', None, [POINTER(GLdouble)], 'ARB_window_pos')

# GL/glext.h:3802
glWindowPos2fARB = _link_function('glWindowPos2fARB', None, [GLfloat, GLfloat], 'ARB_window_pos')

# GL/glext.h:3803
glWindowPos2fvARB = _link_function('glWindowPos2fvARB', None, [POINTER(GLfloat)], 'ARB_window_pos')

# GL/glext.h:3804
glWindowPos2iARB = _link_function('glWindowPos2iARB', None, [GLint, GLint], 'ARB_window_pos')

# GL/glext.h:3805
glWindowPos2ivARB = _link_function('glWindowPos2ivARB', None, [POINTER(GLint)], 'ARB_window_pos')

# GL/glext.h:3806
glWindowPos2sARB = _link_function('glWindowPos2sARB', None, [GLshort, GLshort], 'ARB_window_pos')

# GL/glext.h:3807
glWindowPos2svARB = _link_function('glWindowPos2svARB', None, [POINTER(GLshort)], 'ARB_window_pos')

# GL/glext.h:3808
glWindowPos3dARB = _link_function('glWindowPos3dARB', None, [GLdouble, GLdouble, GLdouble], 'ARB_window_pos')

# GL/glext.h:3809
glWindowPos3dvARB = _link_function('glWindowPos3dvARB', None, [POINTER(GLdouble)], 'ARB_window_pos')

# GL/glext.h:3810
glWindowPos3fARB = _link_function('glWindowPos3fARB', None, [GLfloat, GLfloat, GLfloat], 'ARB_window_pos')

# GL/glext.h:3811
glWindowPos3fvARB = _link_function('glWindowPos3fvARB', None, [POINTER(GLfloat)], 'ARB_window_pos')

# GL/glext.h:3812
glWindowPos3iARB = _link_function('glWindowPos3iARB', None, [GLint, GLint, GLint], 'ARB_window_pos')

# GL/glext.h:3813
glWindowPos3ivARB = _link_function('glWindowPos3ivARB', None, [POINTER(GLint)], 'ARB_window_pos')

# GL/glext.h:3814
glWindowPos3sARB = _link_function('glWindowPos3sARB', None, [GLshort, GLshort, GLshort], 'ARB_window_pos')

# GL/glext.h:3815
glWindowPos3svARB = _link_function('glWindowPos3svARB', None, [POINTER(GLshort)], 'ARB_window_pos')

PFNGLWINDOWPOS2DARBPROC = CFUNCTYPE(None, GLdouble, GLdouble) 	# GL/glext.h:3817
PFNGLWINDOWPOS2DVARBPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:3818
PFNGLWINDOWPOS2FARBPROC = CFUNCTYPE(None, GLfloat, GLfloat) 	# GL/glext.h:3819
PFNGLWINDOWPOS2FVARBPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:3820
PFNGLWINDOWPOS2IARBPROC = CFUNCTYPE(None, GLint, GLint) 	# GL/glext.h:3821
PFNGLWINDOWPOS2IVARBPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:3822
PFNGLWINDOWPOS2SARBPROC = CFUNCTYPE(None, GLshort, GLshort) 	# GL/glext.h:3823
PFNGLWINDOWPOS2SVARBPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:3824
PFNGLWINDOWPOS3DARBPROC = CFUNCTYPE(None, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:3825
PFNGLWINDOWPOS3DVARBPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:3826
PFNGLWINDOWPOS3FARBPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:3827
PFNGLWINDOWPOS3FVARBPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:3828
PFNGLWINDOWPOS3IARBPROC = CFUNCTYPE(None, GLint, GLint, GLint) 	# GL/glext.h:3829
PFNGLWINDOWPOS3IVARBPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:3830
PFNGLWINDOWPOS3SARBPROC = CFUNCTYPE(None, GLshort, GLshort, GLshort) 	# GL/glext.h:3831
PFNGLWINDOWPOS3SVARBPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:3832
# ARB_vertex_program (GL/glext.h:3835)
GL_ARB_vertex_program = 1 	# GL/glext.h:3836
# GL/glext.h:3838
glVertexAttrib1dARB = _link_function('glVertexAttrib1dARB', None, [GLuint, GLdouble], 'ARB_vertex_program')

# GL/glext.h:3839
glVertexAttrib1dvARB = _link_function('glVertexAttrib1dvARB', None, [GLuint, POINTER(GLdouble)], 'ARB_vertex_program')

# GL/glext.h:3840
glVertexAttrib1fARB = _link_function('glVertexAttrib1fARB', None, [GLuint, GLfloat], 'ARB_vertex_program')

# GL/glext.h:3841
glVertexAttrib1fvARB = _link_function('glVertexAttrib1fvARB', None, [GLuint, POINTER(GLfloat)], 'ARB_vertex_program')

# GL/glext.h:3842
glVertexAttrib1sARB = _link_function('glVertexAttrib1sARB', None, [GLuint, GLshort], 'ARB_vertex_program')

# GL/glext.h:3843
glVertexAttrib1svARB = _link_function('glVertexAttrib1svARB', None, [GLuint, POINTER(GLshort)], 'ARB_vertex_program')

# GL/glext.h:3844
glVertexAttrib2dARB = _link_function('glVertexAttrib2dARB', None, [GLuint, GLdouble, GLdouble], 'ARB_vertex_program')

# GL/glext.h:3845
glVertexAttrib2dvARB = _link_function('glVertexAttrib2dvARB', None, [GLuint, POINTER(GLdouble)], 'ARB_vertex_program')

# GL/glext.h:3846
glVertexAttrib2fARB = _link_function('glVertexAttrib2fARB', None, [GLuint, GLfloat, GLfloat], 'ARB_vertex_program')

# GL/glext.h:3847
glVertexAttrib2fvARB = _link_function('glVertexAttrib2fvARB', None, [GLuint, POINTER(GLfloat)], 'ARB_vertex_program')

# GL/glext.h:3848
glVertexAttrib2sARB = _link_function('glVertexAttrib2sARB', None, [GLuint, GLshort, GLshort], 'ARB_vertex_program')

# GL/glext.h:3849
glVertexAttrib2svARB = _link_function('glVertexAttrib2svARB', None, [GLuint, POINTER(GLshort)], 'ARB_vertex_program')

# GL/glext.h:3850
glVertexAttrib3dARB = _link_function('glVertexAttrib3dARB', None, [GLuint, GLdouble, GLdouble, GLdouble], 'ARB_vertex_program')

# GL/glext.h:3851
glVertexAttrib3dvARB = _link_function('glVertexAttrib3dvARB', None, [GLuint, POINTER(GLdouble)], 'ARB_vertex_program')

# GL/glext.h:3852
glVertexAttrib3fARB = _link_function('glVertexAttrib3fARB', None, [GLuint, GLfloat, GLfloat, GLfloat], 'ARB_vertex_program')

# GL/glext.h:3853
glVertexAttrib3fvARB = _link_function('glVertexAttrib3fvARB', None, [GLuint, POINTER(GLfloat)], 'ARB_vertex_program')

# GL/glext.h:3854
glVertexAttrib3sARB = _link_function('glVertexAttrib3sARB', None, [GLuint, GLshort, GLshort, GLshort], 'ARB_vertex_program')

# GL/glext.h:3855
glVertexAttrib3svARB = _link_function('glVertexAttrib3svARB', None, [GLuint, POINTER(GLshort)], 'ARB_vertex_program')

# GL/glext.h:3856
glVertexAttrib4NbvARB = _link_function('glVertexAttrib4NbvARB', None, [GLuint, POINTER(GLbyte)], 'ARB_vertex_program')

# GL/glext.h:3857
glVertexAttrib4NivARB = _link_function('glVertexAttrib4NivARB', None, [GLuint, POINTER(GLint)], 'ARB_vertex_program')

# GL/glext.h:3858
glVertexAttrib4NsvARB = _link_function('glVertexAttrib4NsvARB', None, [GLuint, POINTER(GLshort)], 'ARB_vertex_program')

# GL/glext.h:3859
glVertexAttrib4NubARB = _link_function('glVertexAttrib4NubARB', None, [GLuint, GLubyte, GLubyte, GLubyte, GLubyte], 'ARB_vertex_program')

# GL/glext.h:3860
glVertexAttrib4NubvARB = _link_function('glVertexAttrib4NubvARB', None, [GLuint, POINTER(GLubyte)], 'ARB_vertex_program')

# GL/glext.h:3861
glVertexAttrib4NuivARB = _link_function('glVertexAttrib4NuivARB', None, [GLuint, POINTER(GLuint)], 'ARB_vertex_program')

# GL/glext.h:3862
glVertexAttrib4NusvARB = _link_function('glVertexAttrib4NusvARB', None, [GLuint, POINTER(GLushort)], 'ARB_vertex_program')

# GL/glext.h:3863
glVertexAttrib4bvARB = _link_function('glVertexAttrib4bvARB', None, [GLuint, POINTER(GLbyte)], 'ARB_vertex_program')

# GL/glext.h:3864
glVertexAttrib4dARB = _link_function('glVertexAttrib4dARB', None, [GLuint, GLdouble, GLdouble, GLdouble, GLdouble], 'ARB_vertex_program')

# GL/glext.h:3865
glVertexAttrib4dvARB = _link_function('glVertexAttrib4dvARB', None, [GLuint, POINTER(GLdouble)], 'ARB_vertex_program')

# GL/glext.h:3866
glVertexAttrib4fARB = _link_function('glVertexAttrib4fARB', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat], 'ARB_vertex_program')

# GL/glext.h:3867
glVertexAttrib4fvARB = _link_function('glVertexAttrib4fvARB', None, [GLuint, POINTER(GLfloat)], 'ARB_vertex_program')

# GL/glext.h:3868
glVertexAttrib4ivARB = _link_function('glVertexAttrib4ivARB', None, [GLuint, POINTER(GLint)], 'ARB_vertex_program')

# GL/glext.h:3869
glVertexAttrib4sARB = _link_function('glVertexAttrib4sARB', None, [GLuint, GLshort, GLshort, GLshort, GLshort], 'ARB_vertex_program')

# GL/glext.h:3870
glVertexAttrib4svARB = _link_function('glVertexAttrib4svARB', None, [GLuint, POINTER(GLshort)], 'ARB_vertex_program')

# GL/glext.h:3871
glVertexAttrib4ubvARB = _link_function('glVertexAttrib4ubvARB', None, [GLuint, POINTER(GLubyte)], 'ARB_vertex_program')

# GL/glext.h:3872
glVertexAttrib4uivARB = _link_function('glVertexAttrib4uivARB', None, [GLuint, POINTER(GLuint)], 'ARB_vertex_program')

# GL/glext.h:3873
glVertexAttrib4usvARB = _link_function('glVertexAttrib4usvARB', None, [GLuint, POINTER(GLushort)], 'ARB_vertex_program')

# GL/glext.h:3874
glVertexAttribPointerARB = _link_function('glVertexAttribPointerARB', None, [GLuint, GLint, GLenum, GLboolean, GLsizei, POINTER(GLvoid)], 'ARB_vertex_program')

# GL/glext.h:3875
glEnableVertexAttribArrayARB = _link_function('glEnableVertexAttribArrayARB', None, [GLuint], 'ARB_vertex_program')

# GL/glext.h:3876
glDisableVertexAttribArrayARB = _link_function('glDisableVertexAttribArrayARB', None, [GLuint], 'ARB_vertex_program')

# GL/glext.h:3877
glProgramStringARB = _link_function('glProgramStringARB', None, [GLenum, GLenum, GLsizei, POINTER(GLvoid)], 'ARB_vertex_program')

# GL/glext.h:3878
glBindProgramARB = _link_function('glBindProgramARB', None, [GLenum, GLuint], 'ARB_vertex_program')

# GL/glext.h:3879
glDeleteProgramsARB = _link_function('glDeleteProgramsARB', None, [GLsizei, POINTER(GLuint)], 'ARB_vertex_program')

# GL/glext.h:3880
glGenProgramsARB = _link_function('glGenProgramsARB', None, [GLsizei, POINTER(GLuint)], 'ARB_vertex_program')

# GL/glext.h:3881
glProgramEnvParameter4dARB = _link_function('glProgramEnvParameter4dARB', None, [GLenum, GLuint, GLdouble, GLdouble, GLdouble, GLdouble], 'ARB_vertex_program')

# GL/glext.h:3882
glProgramEnvParameter4dvARB = _link_function('glProgramEnvParameter4dvARB', None, [GLenum, GLuint, POINTER(GLdouble)], 'ARB_vertex_program')

# GL/glext.h:3883
glProgramEnvParameter4fARB = _link_function('glProgramEnvParameter4fARB', None, [GLenum, GLuint, GLfloat, GLfloat, GLfloat, GLfloat], 'ARB_vertex_program')

# GL/glext.h:3884
glProgramEnvParameter4fvARB = _link_function('glProgramEnvParameter4fvARB', None, [GLenum, GLuint, POINTER(GLfloat)], 'ARB_vertex_program')

# GL/glext.h:3885
glProgramLocalParameter4dARB = _link_function('glProgramLocalParameter4dARB', None, [GLenum, GLuint, GLdouble, GLdouble, GLdouble, GLdouble], 'ARB_vertex_program')

# GL/glext.h:3886
glProgramLocalParameter4dvARB = _link_function('glProgramLocalParameter4dvARB', None, [GLenum, GLuint, POINTER(GLdouble)], 'ARB_vertex_program')

# GL/glext.h:3887
glProgramLocalParameter4fARB = _link_function('glProgramLocalParameter4fARB', None, [GLenum, GLuint, GLfloat, GLfloat, GLfloat, GLfloat], 'ARB_vertex_program')

# GL/glext.h:3888
glProgramLocalParameter4fvARB = _link_function('glProgramLocalParameter4fvARB', None, [GLenum, GLuint, POINTER(GLfloat)], 'ARB_vertex_program')

# GL/glext.h:3889
glGetProgramEnvParameterdvARB = _link_function('glGetProgramEnvParameterdvARB', None, [GLenum, GLuint, POINTER(GLdouble)], 'ARB_vertex_program')

# GL/glext.h:3890
glGetProgramEnvParameterfvARB = _link_function('glGetProgramEnvParameterfvARB', None, [GLenum, GLuint, POINTER(GLfloat)], 'ARB_vertex_program')

# GL/glext.h:3891
glGetProgramLocalParameterdvARB = _link_function('glGetProgramLocalParameterdvARB', None, [GLenum, GLuint, POINTER(GLdouble)], 'ARB_vertex_program')

# GL/glext.h:3892
glGetProgramLocalParameterfvARB = _link_function('glGetProgramLocalParameterfvARB', None, [GLenum, GLuint, POINTER(GLfloat)], 'ARB_vertex_program')

# GL/glext.h:3893
glGetProgramivARB = _link_function('glGetProgramivARB', None, [GLenum, GLenum, POINTER(GLint)], 'ARB_vertex_program')

# GL/glext.h:3894
glGetProgramStringARB = _link_function('glGetProgramStringARB', None, [GLenum, GLenum, POINTER(GLvoid)], 'ARB_vertex_program')

# GL/glext.h:3895
glGetVertexAttribdvARB = _link_function('glGetVertexAttribdvARB', None, [GLuint, GLenum, POINTER(GLdouble)], 'ARB_vertex_program')

# GL/glext.h:3896
glGetVertexAttribfvARB = _link_function('glGetVertexAttribfvARB', None, [GLuint, GLenum, POINTER(GLfloat)], 'ARB_vertex_program')

# GL/glext.h:3897
glGetVertexAttribivARB = _link_function('glGetVertexAttribivARB', None, [GLuint, GLenum, POINTER(GLint)], 'ARB_vertex_program')

# GL/glext.h:3898
glGetVertexAttribPointervARB = _link_function('glGetVertexAttribPointervARB', None, [GLuint, GLenum, POINTER(POINTER(GLvoid))], 'ARB_vertex_program')

# GL/glext.h:3899
glIsProgramARB = _link_function('glIsProgramARB', GLboolean, [GLuint], 'ARB_vertex_program')

PFNGLVERTEXATTRIB1DARBPROC = CFUNCTYPE(None, GLuint, GLdouble) 	# GL/glext.h:3901
PFNGLVERTEXATTRIB1DVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:3902
PFNGLVERTEXATTRIB1FARBPROC = CFUNCTYPE(None, GLuint, GLfloat) 	# GL/glext.h:3903
PFNGLVERTEXATTRIB1FVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:3904
PFNGLVERTEXATTRIB1SARBPROC = CFUNCTYPE(None, GLuint, GLshort) 	# GL/glext.h:3905
PFNGLVERTEXATTRIB1SVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:3906
PFNGLVERTEXATTRIB2DARBPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble) 	# GL/glext.h:3907
PFNGLVERTEXATTRIB2DVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:3908
PFNGLVERTEXATTRIB2FARBPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat) 	# GL/glext.h:3909
PFNGLVERTEXATTRIB2FVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:3910
PFNGLVERTEXATTRIB2SARBPROC = CFUNCTYPE(None, GLuint, GLshort, GLshort) 	# GL/glext.h:3911
PFNGLVERTEXATTRIB2SVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:3912
PFNGLVERTEXATTRIB3DARBPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:3913
PFNGLVERTEXATTRIB3DVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:3914
PFNGLVERTEXATTRIB3FARBPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:3915
PFNGLVERTEXATTRIB3FVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:3916
PFNGLVERTEXATTRIB3SARBPROC = CFUNCTYPE(None, GLuint, GLshort, GLshort, GLshort) 	# GL/glext.h:3917
PFNGLVERTEXATTRIB3SVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:3918
PFNGLVERTEXATTRIB4NBVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLbyte)) 	# GL/glext.h:3919
PFNGLVERTEXATTRIB4NIVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:3920
PFNGLVERTEXATTRIB4NSVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:3921
PFNGLVERTEXATTRIB4NUBARBPROC = CFUNCTYPE(None, GLuint, GLubyte, GLubyte, GLubyte, GLubyte) 	# GL/glext.h:3922
PFNGLVERTEXATTRIB4NUBVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLubyte)) 	# GL/glext.h:3923
PFNGLVERTEXATTRIB4NUIVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:3924
PFNGLVERTEXATTRIB4NUSVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLushort)) 	# GL/glext.h:3925
PFNGLVERTEXATTRIB4BVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLbyte)) 	# GL/glext.h:3926
PFNGLVERTEXATTRIB4DARBPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:3927
PFNGLVERTEXATTRIB4DVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:3928
PFNGLVERTEXATTRIB4FARBPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:3929
PFNGLVERTEXATTRIB4FVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:3930
PFNGLVERTEXATTRIB4IVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:3931
PFNGLVERTEXATTRIB4SARBPROC = CFUNCTYPE(None, GLuint, GLshort, GLshort, GLshort, GLshort) 	# GL/glext.h:3932
PFNGLVERTEXATTRIB4SVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:3933
PFNGLVERTEXATTRIB4UBVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLubyte)) 	# GL/glext.h:3934
PFNGLVERTEXATTRIB4UIVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:3935
PFNGLVERTEXATTRIB4USVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLushort)) 	# GL/glext.h:3936
PFNGLVERTEXATTRIBPOINTERARBPROC = CFUNCTYPE(None, GLuint, GLint, GLenum, GLboolean, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:3937
PFNGLENABLEVERTEXATTRIBARRAYARBPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:3938
PFNGLDISABLEVERTEXATTRIBARRAYARBPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:3939
PFNGLPROGRAMSTRINGARBPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:3940
PFNGLBINDPROGRAMARBPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:3941
PFNGLDELETEPROGRAMSARBPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:3942
PFNGLGENPROGRAMSARBPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:3943
PFNGLPROGRAMENVPARAMETER4DARBPROC = CFUNCTYPE(None, GLenum, GLuint, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:3944
PFNGLPROGRAMENVPARAMETER4DVARBPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLdouble)) 	# GL/glext.h:3945
PFNGLPROGRAMENVPARAMETER4FARBPROC = CFUNCTYPE(None, GLenum, GLuint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:3946
PFNGLPROGRAMENVPARAMETER4FVARBPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLfloat)) 	# GL/glext.h:3947
PFNGLPROGRAMLOCALPARAMETER4DARBPROC = CFUNCTYPE(None, GLenum, GLuint, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:3948
PFNGLPROGRAMLOCALPARAMETER4DVARBPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLdouble)) 	# GL/glext.h:3949
PFNGLPROGRAMLOCALPARAMETER4FARBPROC = CFUNCTYPE(None, GLenum, GLuint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:3950
PFNGLPROGRAMLOCALPARAMETER4FVARBPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLfloat)) 	# GL/glext.h:3951
PFNGLGETPROGRAMENVPARAMETERDVARBPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLdouble)) 	# GL/glext.h:3952
PFNGLGETPROGRAMENVPARAMETERFVARBPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLfloat)) 	# GL/glext.h:3953
PFNGLGETPROGRAMLOCALPARAMETERDVARBPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLdouble)) 	# GL/glext.h:3954
PFNGLGETPROGRAMLOCALPARAMETERFVARBPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLfloat)) 	# GL/glext.h:3955
PFNGLGETPROGRAMIVARBPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:3956
PFNGLGETPROGRAMSTRINGARBPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:3957
PFNGLGETVERTEXATTRIBDVARBPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLdouble)) 	# GL/glext.h:3958
PFNGLGETVERTEXATTRIBFVARBPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:3959
PFNGLGETVERTEXATTRIBIVARBPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:3960
PFNGLGETVERTEXATTRIBPOINTERVARBPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:3961
PFNGLISPROGRAMARBPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:3962
# ARB_fragment_program (GL/glext.h:3965)
GL_ARB_fragment_program = 1 	# GL/glext.h:3966
# ARB_vertex_buffer_object (GL/glext.h:3970)
GL_ARB_vertex_buffer_object = 1 	# GL/glext.h:3971
# GL/glext.h:3973
glBindBufferARB = _link_function('glBindBufferARB', None, [GLenum, GLuint], 'ARB_vertex_buffer_object')

# GL/glext.h:3974
glDeleteBuffersARB = _link_function('glDeleteBuffersARB', None, [GLsizei, POINTER(GLuint)], 'ARB_vertex_buffer_object')

# GL/glext.h:3975
glGenBuffersARB = _link_function('glGenBuffersARB', None, [GLsizei, POINTER(GLuint)], 'ARB_vertex_buffer_object')

# GL/glext.h:3976
glIsBufferARB = _link_function('glIsBufferARB', GLboolean, [GLuint], 'ARB_vertex_buffer_object')

# GL/glext.h:3977
glBufferDataARB = _link_function('glBufferDataARB', None, [GLenum, GLsizeiptrARB, POINTER(GLvoid), GLenum], 'ARB_vertex_buffer_object')

# GL/glext.h:3978
glBufferSubDataARB = _link_function('glBufferSubDataARB', None, [GLenum, GLintptrARB, GLsizeiptrARB, POINTER(GLvoid)], 'ARB_vertex_buffer_object')

# GL/glext.h:3979
glGetBufferSubDataARB = _link_function('glGetBufferSubDataARB', None, [GLenum, GLintptrARB, GLsizeiptrARB, POINTER(GLvoid)], 'ARB_vertex_buffer_object')

# GL/glext.h:3980
glMapBufferARB = _link_function('glMapBufferARB', POINTER(GLvoid), [GLenum, GLenum], 'ARB_vertex_buffer_object')

# GL/glext.h:3981
glUnmapBufferARB = _link_function('glUnmapBufferARB', GLboolean, [GLenum], 'ARB_vertex_buffer_object')

# GL/glext.h:3982
glGetBufferParameterivARB = _link_function('glGetBufferParameterivARB', None, [GLenum, GLenum, POINTER(GLint)], 'ARB_vertex_buffer_object')

# GL/glext.h:3983
glGetBufferPointervARB = _link_function('glGetBufferPointervARB', None, [GLenum, GLenum, POINTER(POINTER(GLvoid))], 'ARB_vertex_buffer_object')

PFNGLBINDBUFFERARBPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:3985
PFNGLDELETEBUFFERSARBPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:3986
PFNGLGENBUFFERSARBPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:3987
PFNGLISBUFFERARBPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:3988
PFNGLBUFFERDATAARBPROC = CFUNCTYPE(None, GLenum, GLsizeiptrARB, POINTER(GLvoid), GLenum) 	# GL/glext.h:3989
PFNGLBUFFERSUBDATAARBPROC = CFUNCTYPE(None, GLenum, GLintptrARB, GLsizeiptrARB, POINTER(GLvoid)) 	# GL/glext.h:3990
PFNGLGETBUFFERSUBDATAARBPROC = CFUNCTYPE(None, GLenum, GLintptrARB, GLsizeiptrARB, POINTER(GLvoid)) 	# GL/glext.h:3991
PFNGLMAPBUFFERARBPROC = CFUNCTYPE(POINTER(GLvoid), GLenum, GLenum) 	# GL/glext.h:3992
PFNGLUNMAPBUFFERARBPROC = CFUNCTYPE(GLboolean, GLenum) 	# GL/glext.h:3993
PFNGLGETBUFFERPARAMETERIVARBPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:3994
PFNGLGETBUFFERPOINTERVARBPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:3995
# ARB_occlusion_query (GL/glext.h:3998)
GL_ARB_occlusion_query = 1 	# GL/glext.h:3999
# GL/glext.h:4001
glGenQueriesARB = _link_function('glGenQueriesARB', None, [GLsizei, POINTER(GLuint)], 'ARB_occlusion_query')

# GL/glext.h:4002
glDeleteQueriesARB = _link_function('glDeleteQueriesARB', None, [GLsizei, POINTER(GLuint)], 'ARB_occlusion_query')

# GL/glext.h:4003
glIsQueryARB = _link_function('glIsQueryARB', GLboolean, [GLuint], 'ARB_occlusion_query')

# GL/glext.h:4004
glBeginQueryARB = _link_function('glBeginQueryARB', None, [GLenum, GLuint], 'ARB_occlusion_query')

# GL/glext.h:4005
glEndQueryARB = _link_function('glEndQueryARB', None, [GLenum], 'ARB_occlusion_query')

# GL/glext.h:4006
glGetQueryivARB = _link_function('glGetQueryivARB', None, [GLenum, GLenum, POINTER(GLint)], 'ARB_occlusion_query')

# GL/glext.h:4007
glGetQueryObjectivARB = _link_function('glGetQueryObjectivARB', None, [GLuint, GLenum, POINTER(GLint)], 'ARB_occlusion_query')

# GL/glext.h:4008
glGetQueryObjectuivARB = _link_function('glGetQueryObjectuivARB', None, [GLuint, GLenum, POINTER(GLuint)], 'ARB_occlusion_query')

PFNGLGENQUERIESARBPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:4010
PFNGLDELETEQUERIESARBPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:4011
PFNGLISQUERYARBPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:4012
PFNGLBEGINQUERYARBPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:4013
PFNGLENDQUERYARBPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:4014
PFNGLGETQUERYIVARBPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:4015
PFNGLGETQUERYOBJECTIVARBPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:4016
PFNGLGETQUERYOBJECTUIVARBPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLuint)) 	# GL/glext.h:4017
# ARB_shader_objects (GL/glext.h:4020)
GL_ARB_shader_objects = 1 	# GL/glext.h:4021
# GL/glext.h:4023
glDeleteObjectARB = _link_function('glDeleteObjectARB', None, [GLhandleARB], 'ARB_shader_objects')

# GL/glext.h:4024
glGetHandleARB = _link_function('glGetHandleARB', GLhandleARB, [GLenum], 'ARB_shader_objects')

# GL/glext.h:4025
glDetachObjectARB = _link_function('glDetachObjectARB', None, [GLhandleARB, GLhandleARB], 'ARB_shader_objects')

# GL/glext.h:4026
glCreateShaderObjectARB = _link_function('glCreateShaderObjectARB', GLhandleARB, [GLenum], 'ARB_shader_objects')

# GL/glext.h:4027
glShaderSourceARB = _link_function('glShaderSourceARB', None, [GLhandleARB, GLsizei, POINTER(POINTER(GLcharARB)), POINTER(GLint)], 'ARB_shader_objects')

# GL/glext.h:4028
glCompileShaderARB = _link_function('glCompileShaderARB', None, [GLhandleARB], 'ARB_shader_objects')

# GL/glext.h:4029
glCreateProgramObjectARB = _link_function('glCreateProgramObjectARB', GLhandleARB, [], 'ARB_shader_objects')

# GL/glext.h:4030
glAttachObjectARB = _link_function('glAttachObjectARB', None, [GLhandleARB, GLhandleARB], 'ARB_shader_objects')

# GL/glext.h:4031
glLinkProgramARB = _link_function('glLinkProgramARB', None, [GLhandleARB], 'ARB_shader_objects')

# GL/glext.h:4032
glUseProgramObjectARB = _link_function('glUseProgramObjectARB', None, [GLhandleARB], 'ARB_shader_objects')

# GL/glext.h:4033
glValidateProgramARB = _link_function('glValidateProgramARB', None, [GLhandleARB], 'ARB_shader_objects')

# GL/glext.h:4034
glUniform1fARB = _link_function('glUniform1fARB', None, [GLint, GLfloat], 'ARB_shader_objects')

# GL/glext.h:4035
glUniform2fARB = _link_function('glUniform2fARB', None, [GLint, GLfloat, GLfloat], 'ARB_shader_objects')

# GL/glext.h:4036
glUniform3fARB = _link_function('glUniform3fARB', None, [GLint, GLfloat, GLfloat, GLfloat], 'ARB_shader_objects')

# GL/glext.h:4037
glUniform4fARB = _link_function('glUniform4fARB', None, [GLint, GLfloat, GLfloat, GLfloat, GLfloat], 'ARB_shader_objects')

# GL/glext.h:4038
glUniform1iARB = _link_function('glUniform1iARB', None, [GLint, GLint], 'ARB_shader_objects')

# GL/glext.h:4039
glUniform2iARB = _link_function('glUniform2iARB', None, [GLint, GLint, GLint], 'ARB_shader_objects')

# GL/glext.h:4040
glUniform3iARB = _link_function('glUniform3iARB', None, [GLint, GLint, GLint, GLint], 'ARB_shader_objects')

# GL/glext.h:4041
glUniform4iARB = _link_function('glUniform4iARB', None, [GLint, GLint, GLint, GLint, GLint], 'ARB_shader_objects')

# GL/glext.h:4042
glUniform1fvARB = _link_function('glUniform1fvARB', None, [GLint, GLsizei, POINTER(GLfloat)], 'ARB_shader_objects')

# GL/glext.h:4043
glUniform2fvARB = _link_function('glUniform2fvARB', None, [GLint, GLsizei, POINTER(GLfloat)], 'ARB_shader_objects')

# GL/glext.h:4044
glUniform3fvARB = _link_function('glUniform3fvARB', None, [GLint, GLsizei, POINTER(GLfloat)], 'ARB_shader_objects')

# GL/glext.h:4045
glUniform4fvARB = _link_function('glUniform4fvARB', None, [GLint, GLsizei, POINTER(GLfloat)], 'ARB_shader_objects')

# GL/glext.h:4046
glUniform1ivARB = _link_function('glUniform1ivARB', None, [GLint, GLsizei, POINTER(GLint)], 'ARB_shader_objects')

# GL/glext.h:4047
glUniform2ivARB = _link_function('glUniform2ivARB', None, [GLint, GLsizei, POINTER(GLint)], 'ARB_shader_objects')

# GL/glext.h:4048
glUniform3ivARB = _link_function('glUniform3ivARB', None, [GLint, GLsizei, POINTER(GLint)], 'ARB_shader_objects')

# GL/glext.h:4049
glUniform4ivARB = _link_function('glUniform4ivARB', None, [GLint, GLsizei, POINTER(GLint)], 'ARB_shader_objects')

# GL/glext.h:4050
glUniformMatrix2fvARB = _link_function('glUniformMatrix2fvARB', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'ARB_shader_objects')

# GL/glext.h:4051
glUniformMatrix3fvARB = _link_function('glUniformMatrix3fvARB', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'ARB_shader_objects')

# GL/glext.h:4052
glUniformMatrix4fvARB = _link_function('glUniformMatrix4fvARB', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'ARB_shader_objects')

# GL/glext.h:4053
glGetObjectParameterfvARB = _link_function('glGetObjectParameterfvARB', None, [GLhandleARB, GLenum, POINTER(GLfloat)], 'ARB_shader_objects')

# GL/glext.h:4054
glGetObjectParameterivARB = _link_function('glGetObjectParameterivARB', None, [GLhandleARB, GLenum, POINTER(GLint)], 'ARB_shader_objects')

# GL/glext.h:4055
glGetInfoLogARB = _link_function('glGetInfoLogARB', None, [GLhandleARB, GLsizei, POINTER(GLsizei), POINTER(GLcharARB)], 'ARB_shader_objects')

# GL/glext.h:4056
glGetAttachedObjectsARB = _link_function('glGetAttachedObjectsARB', None, [GLhandleARB, GLsizei, POINTER(GLsizei), POINTER(GLhandleARB)], 'ARB_shader_objects')

# GL/glext.h:4057
glGetUniformLocationARB = _link_function('glGetUniformLocationARB', GLint, [GLhandleARB, POINTER(GLcharARB)], 'ARB_shader_objects')

# GL/glext.h:4058
glGetActiveUniformARB = _link_function('glGetActiveUniformARB', None, [GLhandleARB, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLint), POINTER(GLenum), POINTER(GLcharARB)], 'ARB_shader_objects')

# GL/glext.h:4059
glGetUniformfvARB = _link_function('glGetUniformfvARB', None, [GLhandleARB, GLint, POINTER(GLfloat)], 'ARB_shader_objects')

# GL/glext.h:4060
glGetUniformivARB = _link_function('glGetUniformivARB', None, [GLhandleARB, GLint, POINTER(GLint)], 'ARB_shader_objects')

# GL/glext.h:4061
glGetShaderSourceARB = _link_function('glGetShaderSourceARB', None, [GLhandleARB, GLsizei, POINTER(GLsizei), POINTER(GLcharARB)], 'ARB_shader_objects')

PFNGLDELETEOBJECTARBPROC = CFUNCTYPE(None, GLhandleARB) 	# GL/glext.h:4063
PFNGLGETHANDLEARBPROC = CFUNCTYPE(GLhandleARB, GLenum) 	# GL/glext.h:4064
PFNGLDETACHOBJECTARBPROC = CFUNCTYPE(None, GLhandleARB, GLhandleARB) 	# GL/glext.h:4065
PFNGLCREATESHADEROBJECTARBPROC = CFUNCTYPE(GLhandleARB, GLenum) 	# GL/glext.h:4066
PFNGLSHADERSOURCEARBPROC = CFUNCTYPE(None, GLhandleARB, GLsizei, POINTER(POINTER(GLcharARB)), POINTER(GLint)) 	# GL/glext.h:4067
PFNGLCOMPILESHADERARBPROC = CFUNCTYPE(None, GLhandleARB) 	# GL/glext.h:4068
PFNGLCREATEPROGRAMOBJECTARBPROC = CFUNCTYPE(GLhandleARB) 	# GL/glext.h:4069
PFNGLATTACHOBJECTARBPROC = CFUNCTYPE(None, GLhandleARB, GLhandleARB) 	# GL/glext.h:4070
PFNGLLINKPROGRAMARBPROC = CFUNCTYPE(None, GLhandleARB) 	# GL/glext.h:4071
PFNGLUSEPROGRAMOBJECTARBPROC = CFUNCTYPE(None, GLhandleARB) 	# GL/glext.h:4072
PFNGLVALIDATEPROGRAMARBPROC = CFUNCTYPE(None, GLhandleARB) 	# GL/glext.h:4073
PFNGLUNIFORM1FARBPROC = CFUNCTYPE(None, GLint, GLfloat) 	# GL/glext.h:4074
PFNGLUNIFORM2FARBPROC = CFUNCTYPE(None, GLint, GLfloat, GLfloat) 	# GL/glext.h:4075
PFNGLUNIFORM3FARBPROC = CFUNCTYPE(None, GLint, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:4076
PFNGLUNIFORM4FARBPROC = CFUNCTYPE(None, GLint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:4077
PFNGLUNIFORM1IARBPROC = CFUNCTYPE(None, GLint, GLint) 	# GL/glext.h:4078
PFNGLUNIFORM2IARBPROC = CFUNCTYPE(None, GLint, GLint, GLint) 	# GL/glext.h:4079
PFNGLUNIFORM3IARBPROC = CFUNCTYPE(None, GLint, GLint, GLint, GLint) 	# GL/glext.h:4080
PFNGLUNIFORM4IARBPROC = CFUNCTYPE(None, GLint, GLint, GLint, GLint, GLint) 	# GL/glext.h:4081
PFNGLUNIFORM1FVARBPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:4082
PFNGLUNIFORM2FVARBPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:4083
PFNGLUNIFORM3FVARBPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:4084
PFNGLUNIFORM4FVARBPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:4085
PFNGLUNIFORM1IVARBPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:4086
PFNGLUNIFORM2IVARBPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:4087
PFNGLUNIFORM3IVARBPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:4088
PFNGLUNIFORM4IVARBPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:4089
PFNGLUNIFORMMATRIX2FVARBPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:4090
PFNGLUNIFORMMATRIX3FVARBPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:4091
PFNGLUNIFORMMATRIX4FVARBPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:4092
PFNGLGETOBJECTPARAMETERFVARBPROC = CFUNCTYPE(None, GLhandleARB, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4093
PFNGLGETOBJECTPARAMETERIVARBPROC = CFUNCTYPE(None, GLhandleARB, GLenum, POINTER(GLint)) 	# GL/glext.h:4094
PFNGLGETINFOLOGARBPROC = CFUNCTYPE(None, GLhandleARB, GLsizei, POINTER(GLsizei), POINTER(GLcharARB)) 	# GL/glext.h:4095
PFNGLGETATTACHEDOBJECTSARBPROC = CFUNCTYPE(None, GLhandleARB, GLsizei, POINTER(GLsizei), POINTER(GLhandleARB)) 	# GL/glext.h:4096
PFNGLGETUNIFORMLOCATIONARBPROC = CFUNCTYPE(GLint, GLhandleARB, POINTER(GLcharARB)) 	# GL/glext.h:4097
PFNGLGETACTIVEUNIFORMARBPROC = CFUNCTYPE(None, GLhandleARB, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLint), POINTER(GLenum), POINTER(GLcharARB)) 	# GL/glext.h:4098
PFNGLGETUNIFORMFVARBPROC = CFUNCTYPE(None, GLhandleARB, GLint, POINTER(GLfloat)) 	# GL/glext.h:4099
PFNGLGETUNIFORMIVARBPROC = CFUNCTYPE(None, GLhandleARB, GLint, POINTER(GLint)) 	# GL/glext.h:4100
PFNGLGETSHADERSOURCEARBPROC = CFUNCTYPE(None, GLhandleARB, GLsizei, POINTER(GLsizei), POINTER(GLcharARB)) 	# GL/glext.h:4101
# ARB_vertex_shader (GL/glext.h:4104)
GL_ARB_vertex_shader = 1 	# GL/glext.h:4105
# GL/glext.h:4107
glBindAttribLocationARB = _link_function('glBindAttribLocationARB', None, [GLhandleARB, GLuint, POINTER(GLcharARB)], 'ARB_vertex_shader')

# GL/glext.h:4108
glGetActiveAttribARB = _link_function('glGetActiveAttribARB', None, [GLhandleARB, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLint), POINTER(GLenum), POINTER(GLcharARB)], 'ARB_vertex_shader')

# GL/glext.h:4109
glGetAttribLocationARB = _link_function('glGetAttribLocationARB', GLint, [GLhandleARB, POINTER(GLcharARB)], 'ARB_vertex_shader')

PFNGLBINDATTRIBLOCATIONARBPROC = CFUNCTYPE(None, GLhandleARB, GLuint, POINTER(GLcharARB)) 	# GL/glext.h:4111
PFNGLGETACTIVEATTRIBARBPROC = CFUNCTYPE(None, GLhandleARB, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLint), POINTER(GLenum), POINTER(GLcharARB)) 	# GL/glext.h:4112
PFNGLGETATTRIBLOCATIONARBPROC = CFUNCTYPE(GLint, GLhandleARB, POINTER(GLcharARB)) 	# GL/glext.h:4113
# ARB_fragment_shader (GL/glext.h:4116)
GL_ARB_fragment_shader = 1 	# GL/glext.h:4117
# ARB_shading_language_100 (GL/glext.h:4120)
GL_ARB_shading_language_100 = 1 	# GL/glext.h:4121
# ARB_texture_non_power_of_two (GL/glext.h:4124)
GL_ARB_texture_non_power_of_two = 1 	# GL/glext.h:4125
# ARB_point_sprite (GL/glext.h:4128)
GL_ARB_point_sprite = 1 	# GL/glext.h:4129
# ARB_fragment_program_shadow (GL/glext.h:4132)
GL_ARB_fragment_program_shadow = 1 	# GL/glext.h:4133
# ARB_draw_buffers (GL/glext.h:4136)
GL_ARB_draw_buffers = 1 	# GL/glext.h:4137
# GL/glext.h:4139
glDrawBuffersARB = _link_function('glDrawBuffersARB', None, [GLsizei, POINTER(GLenum)], 'ARB_draw_buffers')

PFNGLDRAWBUFFERSARBPROC = CFUNCTYPE(None, GLsizei, POINTER(GLenum)) 	# GL/glext.h:4141
# ARB_texture_rectangle (GL/glext.h:4144)
GL_ARB_texture_rectangle = 1 	# GL/glext.h:4145
# ARB_color_buffer_float (GL/glext.h:4148)
GL_ARB_color_buffer_float = 1 	# GL/glext.h:4149
# GL/glext.h:4151
glClampColorARB = _link_function('glClampColorARB', None, [GLenum, GLenum], 'ARB_color_buffer_float')

PFNGLCLAMPCOLORARBPROC = CFUNCTYPE(None, GLenum, GLenum) 	# GL/glext.h:4153
# ARB_half_float_pixel (GL/glext.h:4156)
GL_ARB_half_float_pixel = 1 	# GL/glext.h:4157
# ARB_texture_float (GL/glext.h:4160)
GL_ARB_texture_float = 1 	# GL/glext.h:4161
# ARB_pixel_buffer_object (GL/glext.h:4164)
GL_ARB_pixel_buffer_object = 1 	# GL/glext.h:4165
# EXT_abgr (GL/glext.h:4168)
GL_EXT_abgr = 1 	# GL/glext.h:4169
# EXT_blend_color (GL/glext.h:4172)
GL_EXT_blend_color = 1 	# GL/glext.h:4173
# GL/glext.h:4175
glBlendColorEXT = _link_function('glBlendColorEXT', None, [GLclampf, GLclampf, GLclampf, GLclampf], 'EXT_blend_color')

PFNGLBLENDCOLOREXTPROC = CFUNCTYPE(None, GLclampf, GLclampf, GLclampf, GLclampf) 	# GL/glext.h:4177
# EXT_polygon_offset (GL/glext.h:4180)
GL_EXT_polygon_offset = 1 	# GL/glext.h:4181
# GL/glext.h:4183
glPolygonOffsetEXT = _link_function('glPolygonOffsetEXT', None, [GLfloat, GLfloat], 'EXT_polygon_offset')

PFNGLPOLYGONOFFSETEXTPROC = CFUNCTYPE(None, GLfloat, GLfloat) 	# GL/glext.h:4185
# EXT_texture (GL/glext.h:4188)
GL_EXT_texture = 1 	# GL/glext.h:4189
# EXT_texture3D (GL/glext.h:4192)
GL_EXT_texture3D = 1 	# GL/glext.h:4193
# GL/glext.h:4195
glTexImage3DEXT = _link_function('glTexImage3DEXT', None, [GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)], 'EXT_texture3D')

# GL/glext.h:4196
glTexSubImage3DEXT = _link_function('glTexSubImage3DEXT', None, [GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_texture3D')

PFNGLTEXIMAGE3DEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:4198
PFNGLTEXSUBIMAGE3DEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:4199
# SGIS_texture_filter4 (GL/glext.h:4202)
GL_SGIS_texture_filter4 = 1 	# GL/glext.h:4203
# GL/glext.h:4205
glGetTexFilterFuncSGIS = _link_function('glGetTexFilterFuncSGIS', None, [GLenum, GLenum, POINTER(GLfloat)], 'SGIS_texture_filter4')

# GL/glext.h:4206
glTexFilterFuncSGIS = _link_function('glTexFilterFuncSGIS', None, [GLenum, GLenum, GLsizei, POINTER(GLfloat)], 'SGIS_texture_filter4')

PFNGLGETTEXFILTERFUNCSGISPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4208
PFNGLTEXFILTERFUNCSGISPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:4209
# EXT_subtexture (GL/glext.h:4212)
GL_EXT_subtexture = 1 	# GL/glext.h:4213
# GL/glext.h:4215
glTexSubImage1DEXT = _link_function('glTexSubImage1DEXT', None, [GLenum, GLint, GLint, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_subtexture')

# GL/glext.h:4216
glTexSubImage2DEXT = _link_function('glTexSubImage2DEXT', None, [GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_subtexture')

PFNGLTEXSUBIMAGE1DEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:4218
PFNGLTEXSUBIMAGE2DEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:4219
# EXT_copy_texture (GL/glext.h:4222)
GL_EXT_copy_texture = 1 	# GL/glext.h:4223
# GL/glext.h:4225
glCopyTexImage1DEXT = _link_function('glCopyTexImage1DEXT', None, [GLenum, GLint, GLenum, GLint, GLint, GLsizei, GLint], 'EXT_copy_texture')

# GL/glext.h:4226
glCopyTexImage2DEXT = _link_function('glCopyTexImage2DEXT', None, [GLenum, GLint, GLenum, GLint, GLint, GLsizei, GLsizei, GLint], 'EXT_copy_texture')

# GL/glext.h:4227
glCopyTexSubImage1DEXT = _link_function('glCopyTexSubImage1DEXT', None, [GLenum, GLint, GLint, GLint, GLint, GLsizei], 'EXT_copy_texture')

# GL/glext.h:4228
glCopyTexSubImage2DEXT = _link_function('glCopyTexSubImage2DEXT', None, [GLenum, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei], 'EXT_copy_texture')

# GL/glext.h:4229
glCopyTexSubImage3DEXT = _link_function('glCopyTexSubImage3DEXT', None, [GLenum, GLint, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei], 'EXT_copy_texture')

PFNGLCOPYTEXIMAGE1DEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLint, GLint, GLsizei, GLint) 	# GL/glext.h:4231
PFNGLCOPYTEXIMAGE2DEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLint, GLint, GLsizei, GLsizei, GLint) 	# GL/glext.h:4232
PFNGLCOPYTEXSUBIMAGE1DEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint, GLsizei) 	# GL/glext.h:4233
PFNGLCOPYTEXSUBIMAGE2DEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei) 	# GL/glext.h:4234
PFNGLCOPYTEXSUBIMAGE3DEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei) 	# GL/glext.h:4235
# EXT_histogram (GL/glext.h:4238)
GL_EXT_histogram = 1 	# GL/glext.h:4239
# GL/glext.h:4241
glGetHistogramEXT = _link_function('glGetHistogramEXT', None, [GLenum, GLboolean, GLenum, GLenum, POINTER(GLvoid)], 'EXT_histogram')

# GL/glext.h:4242
glGetHistogramParameterfvEXT = _link_function('glGetHistogramParameterfvEXT', None, [GLenum, GLenum, POINTER(GLfloat)], 'EXT_histogram')

# GL/glext.h:4243
glGetHistogramParameterivEXT = _link_function('glGetHistogramParameterivEXT', None, [GLenum, GLenum, POINTER(GLint)], 'EXT_histogram')

# GL/glext.h:4244
glGetMinmaxEXT = _link_function('glGetMinmaxEXT', None, [GLenum, GLboolean, GLenum, GLenum, POINTER(GLvoid)], 'EXT_histogram')

# GL/glext.h:4245
glGetMinmaxParameterfvEXT = _link_function('glGetMinmaxParameterfvEXT', None, [GLenum, GLenum, POINTER(GLfloat)], 'EXT_histogram')

# GL/glext.h:4246
glGetMinmaxParameterivEXT = _link_function('glGetMinmaxParameterivEXT', None, [GLenum, GLenum, POINTER(GLint)], 'EXT_histogram')

# GL/glext.h:4247
glHistogramEXT = _link_function('glHistogramEXT', None, [GLenum, GLsizei, GLenum, GLboolean], 'EXT_histogram')

# GL/glext.h:4248
glMinmaxEXT = _link_function('glMinmaxEXT', None, [GLenum, GLenum, GLboolean], 'EXT_histogram')

# GL/glext.h:4249
glResetHistogramEXT = _link_function('glResetHistogramEXT', None, [GLenum], 'EXT_histogram')

# GL/glext.h:4250
glResetMinmaxEXT = _link_function('glResetMinmaxEXT', None, [GLenum], 'EXT_histogram')

PFNGLGETHISTOGRAMEXTPROC = CFUNCTYPE(None, GLenum, GLboolean, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:4252
PFNGLGETHISTOGRAMPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4253
PFNGLGETHISTOGRAMPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:4254
PFNGLGETMINMAXEXTPROC = CFUNCTYPE(None, GLenum, GLboolean, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:4255
PFNGLGETMINMAXPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4256
PFNGLGETMINMAXPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:4257
PFNGLHISTOGRAMEXTPROC = CFUNCTYPE(None, GLenum, GLsizei, GLenum, GLboolean) 	# GL/glext.h:4258
PFNGLMINMAXEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLboolean) 	# GL/glext.h:4259
PFNGLRESETHISTOGRAMEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:4260
PFNGLRESETMINMAXEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:4261
# EXT_convolution (GL/glext.h:4264)
GL_EXT_convolution = 1 	# GL/glext.h:4265
# GL/glext.h:4267
glConvolutionFilter1DEXT = _link_function('glConvolutionFilter1DEXT', None, [GLenum, GLenum, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_convolution')

# GL/glext.h:4268
glConvolutionFilter2DEXT = _link_function('glConvolutionFilter2DEXT', None, [GLenum, GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_convolution')

# GL/glext.h:4269
glConvolutionParameterfEXT = _link_function('glConvolutionParameterfEXT', None, [GLenum, GLenum, GLfloat], 'EXT_convolution')

# GL/glext.h:4270
glConvolutionParameterfvEXT = _link_function('glConvolutionParameterfvEXT', None, [GLenum, GLenum, POINTER(GLfloat)], 'EXT_convolution')

# GL/glext.h:4271
glConvolutionParameteriEXT = _link_function('glConvolutionParameteriEXT', None, [GLenum, GLenum, GLint], 'EXT_convolution')

# GL/glext.h:4272
glConvolutionParameterivEXT = _link_function('glConvolutionParameterivEXT', None, [GLenum, GLenum, POINTER(GLint)], 'EXT_convolution')

# GL/glext.h:4273
glCopyConvolutionFilter1DEXT = _link_function('glCopyConvolutionFilter1DEXT', None, [GLenum, GLenum, GLint, GLint, GLsizei], 'EXT_convolution')

# GL/glext.h:4274
glCopyConvolutionFilter2DEXT = _link_function('glCopyConvolutionFilter2DEXT', None, [GLenum, GLenum, GLint, GLint, GLsizei, GLsizei], 'EXT_convolution')

# GL/glext.h:4275
glGetConvolutionFilterEXT = _link_function('glGetConvolutionFilterEXT', None, [GLenum, GLenum, GLenum, POINTER(GLvoid)], 'EXT_convolution')

# GL/glext.h:4276
glGetConvolutionParameterfvEXT = _link_function('glGetConvolutionParameterfvEXT', None, [GLenum, GLenum, POINTER(GLfloat)], 'EXT_convolution')

# GL/glext.h:4277
glGetConvolutionParameterivEXT = _link_function('glGetConvolutionParameterivEXT', None, [GLenum, GLenum, POINTER(GLint)], 'EXT_convolution')

# GL/glext.h:4278
glGetSeparableFilterEXT = _link_function('glGetSeparableFilterEXT', None, [GLenum, GLenum, GLenum, POINTER(GLvoid), POINTER(GLvoid), POINTER(GLvoid)], 'EXT_convolution')

# GL/glext.h:4279
glSeparableFilter2DEXT = _link_function('glSeparableFilter2DEXT', None, [GLenum, GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid), POINTER(GLvoid)], 'EXT_convolution')

PFNGLCONVOLUTIONFILTER1DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:4281
PFNGLCONVOLUTIONFILTER2DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:4282
PFNGLCONVOLUTIONPARAMETERFEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLfloat) 	# GL/glext.h:4283
PFNGLCONVOLUTIONPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4284
PFNGLCONVOLUTIONPARAMETERIEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint) 	# GL/glext.h:4285
PFNGLCONVOLUTIONPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:4286
PFNGLCOPYCONVOLUTIONFILTER1DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLsizei) 	# GL/glext.h:4287
PFNGLCOPYCONVOLUTIONFILTER2DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLsizei, GLsizei) 	# GL/glext.h:4288
PFNGLGETCONVOLUTIONFILTEREXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:4289
PFNGLGETCONVOLUTIONPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4290
PFNGLGETCONVOLUTIONPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:4291
PFNGLGETSEPARABLEFILTEREXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLvoid), POINTER(GLvoid), POINTER(GLvoid)) 	# GL/glext.h:4292
PFNGLSEPARABLEFILTER2DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid), POINTER(GLvoid)) 	# GL/glext.h:4293
# EXT_color_matrix (GL/glext.h:4296)
GL_EXT_color_matrix = 1 	# GL/glext.h:4297
# SGI_color_table (GL/glext.h:4300)
GL_SGI_color_table = 1 	# GL/glext.h:4301
# GL/glext.h:4303
glColorTableSGI = _link_function('glColorTableSGI', None, [GLenum, GLenum, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'SGI_color_table')

# GL/glext.h:4304
glColorTableParameterfvSGI = _link_function('glColorTableParameterfvSGI', None, [GLenum, GLenum, POINTER(GLfloat)], 'SGI_color_table')

# GL/glext.h:4305
glColorTableParameterivSGI = _link_function('glColorTableParameterivSGI', None, [GLenum, GLenum, POINTER(GLint)], 'SGI_color_table')

# GL/glext.h:4306
glCopyColorTableSGI = _link_function('glCopyColorTableSGI', None, [GLenum, GLenum, GLint, GLint, GLsizei], 'SGI_color_table')

# GL/glext.h:4307
glGetColorTableSGI = _link_function('glGetColorTableSGI', None, [GLenum, GLenum, GLenum, POINTER(GLvoid)], 'SGI_color_table')

# GL/glext.h:4308
glGetColorTableParameterfvSGI = _link_function('glGetColorTableParameterfvSGI', None, [GLenum, GLenum, POINTER(GLfloat)], 'SGI_color_table')

# GL/glext.h:4309
glGetColorTableParameterivSGI = _link_function('glGetColorTableParameterivSGI', None, [GLenum, GLenum, POINTER(GLint)], 'SGI_color_table')

PFNGLCOLORTABLESGIPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:4311
PFNGLCOLORTABLEPARAMETERFVSGIPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4312
PFNGLCOLORTABLEPARAMETERIVSGIPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:4313
PFNGLCOPYCOLORTABLESGIPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLsizei) 	# GL/glext.h:4314
PFNGLGETCOLORTABLESGIPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:4315
PFNGLGETCOLORTABLEPARAMETERFVSGIPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4316
PFNGLGETCOLORTABLEPARAMETERIVSGIPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:4317
# SGIX_pixel_texture (GL/glext.h:4320)
GL_SGIX_pixel_texture = 1 	# GL/glext.h:4321
# GL/glext.h:4323
glPixelTexGenSGIX = _link_function('glPixelTexGenSGIX', None, [GLenum], 'SGIX_pixel_texture')

PFNGLPIXELTEXGENSGIXPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:4325
# SGIS_pixel_texture (GL/glext.h:4328)
GL_SGIS_pixel_texture = 1 	# GL/glext.h:4329
# GL/glext.h:4331
glPixelTexGenParameteriSGIS = _link_function('glPixelTexGenParameteriSGIS', None, [GLenum, GLint], 'SGIS_pixel_texture')

# GL/glext.h:4332
glPixelTexGenParameterivSGIS = _link_function('glPixelTexGenParameterivSGIS', None, [GLenum, POINTER(GLint)], 'SGIS_pixel_texture')

# GL/glext.h:4333
glPixelTexGenParameterfSGIS = _link_function('glPixelTexGenParameterfSGIS', None, [GLenum, GLfloat], 'SGIS_pixel_texture')

# GL/glext.h:4334
glPixelTexGenParameterfvSGIS = _link_function('glPixelTexGenParameterfvSGIS', None, [GLenum, POINTER(GLfloat)], 'SGIS_pixel_texture')

# GL/glext.h:4335
glGetPixelTexGenParameterivSGIS = _link_function('glGetPixelTexGenParameterivSGIS', None, [GLenum, POINTER(GLint)], 'SGIS_pixel_texture')

# GL/glext.h:4336
glGetPixelTexGenParameterfvSGIS = _link_function('glGetPixelTexGenParameterfvSGIS', None, [GLenum, POINTER(GLfloat)], 'SGIS_pixel_texture')

PFNGLPIXELTEXGENPARAMETERISGISPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:4338
PFNGLPIXELTEXGENPARAMETERIVSGISPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:4339
PFNGLPIXELTEXGENPARAMETERFSGISPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:4340
PFNGLPIXELTEXGENPARAMETERFVSGISPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4341
PFNGLGETPIXELTEXGENPARAMETERIVSGISPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:4342
PFNGLGETPIXELTEXGENPARAMETERFVSGISPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4343
# SGIS_texture4D (GL/glext.h:4346)
GL_SGIS_texture4D = 1 	# GL/glext.h:4347
# GL/glext.h:4349
glTexImage4DSGIS = _link_function('glTexImage4DSGIS', None, [GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)], 'SGIS_texture4D')

# GL/glext.h:4350
glTexSubImage4DSGIS = _link_function('glTexSubImage4DSGIS', None, [GLenum, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'SGIS_texture4D')

PFNGLTEXIMAGE4DSGISPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:4352
PFNGLTEXSUBIMAGE4DSGISPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:4353
# SGI_texture_color_table (GL/glext.h:4356)
GL_SGI_texture_color_table = 1 	# GL/glext.h:4357
# EXT_cmyka (GL/glext.h:4360)
GL_EXT_cmyka = 1 	# GL/glext.h:4361
# EXT_texture_object (GL/glext.h:4364)
GL_EXT_texture_object = 1 	# GL/glext.h:4365
# GL/glext.h:4367
glAreTexturesResidentEXT = _link_function('glAreTexturesResidentEXT', GLboolean, [GLsizei, POINTER(GLuint), POINTER(GLboolean)], 'EXT_texture_object')

# GL/glext.h:4368
glBindTextureEXT = _link_function('glBindTextureEXT', None, [GLenum, GLuint], 'EXT_texture_object')

# GL/glext.h:4369
glDeleteTexturesEXT = _link_function('glDeleteTexturesEXT', None, [GLsizei, POINTER(GLuint)], 'EXT_texture_object')

# GL/glext.h:4370
glGenTexturesEXT = _link_function('glGenTexturesEXT', None, [GLsizei, POINTER(GLuint)], 'EXT_texture_object')

# GL/glext.h:4371
glIsTextureEXT = _link_function('glIsTextureEXT', GLboolean, [GLuint], 'EXT_texture_object')

# GL/glext.h:4372
glPrioritizeTexturesEXT = _link_function('glPrioritizeTexturesEXT', None, [GLsizei, POINTER(GLuint), POINTER(GLclampf)], 'EXT_texture_object')

PFNGLARETEXTURESRESIDENTEXTPROC = CFUNCTYPE(GLboolean, GLsizei, POINTER(GLuint), POINTER(GLboolean)) 	# GL/glext.h:4374
PFNGLBINDTEXTUREEXTPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:4375
PFNGLDELETETEXTURESEXTPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:4376
PFNGLGENTEXTURESEXTPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:4377
PFNGLISTEXTUREEXTPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:4378
PFNGLPRIORITIZETEXTURESEXTPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint), POINTER(GLclampf)) 	# GL/glext.h:4379
# SGIS_detail_texture (GL/glext.h:4382)
GL_SGIS_detail_texture = 1 	# GL/glext.h:4383
# GL/glext.h:4385
glDetailTexFuncSGIS = _link_function('glDetailTexFuncSGIS', None, [GLenum, GLsizei, POINTER(GLfloat)], 'SGIS_detail_texture')

# GL/glext.h:4386
glGetDetailTexFuncSGIS = _link_function('glGetDetailTexFuncSGIS', None, [GLenum, POINTER(GLfloat)], 'SGIS_detail_texture')

PFNGLDETAILTEXFUNCSGISPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:4388
PFNGLGETDETAILTEXFUNCSGISPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4389
# SGIS_sharpen_texture (GL/glext.h:4392)
GL_SGIS_sharpen_texture = 1 	# GL/glext.h:4393
# GL/glext.h:4395
glSharpenTexFuncSGIS = _link_function('glSharpenTexFuncSGIS', None, [GLenum, GLsizei, POINTER(GLfloat)], 'SGIS_sharpen_texture')

# GL/glext.h:4396
glGetSharpenTexFuncSGIS = _link_function('glGetSharpenTexFuncSGIS', None, [GLenum, POINTER(GLfloat)], 'SGIS_sharpen_texture')

PFNGLSHARPENTEXFUNCSGISPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:4398
PFNGLGETSHARPENTEXFUNCSGISPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4399
# EXT_packed_pixels (GL/glext.h:4402)
GL_EXT_packed_pixels = 1 	# GL/glext.h:4403
# SGIS_texture_lod (GL/glext.h:4406)
GL_SGIS_texture_lod = 1 	# GL/glext.h:4407
# SGIS_multisample (GL/glext.h:4410)
GL_SGIS_multisample = 1 	# GL/glext.h:4411
# GL/glext.h:4413
glSampleMaskSGIS = _link_function('glSampleMaskSGIS', None, [GLclampf, GLboolean], 'SGIS_multisample')

# GL/glext.h:4414
glSamplePatternSGIS = _link_function('glSamplePatternSGIS', None, [GLenum], 'SGIS_multisample')

PFNGLSAMPLEMASKSGISPROC = CFUNCTYPE(None, GLclampf, GLboolean) 	# GL/glext.h:4416
PFNGLSAMPLEPATTERNSGISPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:4417
# EXT_rescale_normal (GL/glext.h:4420)
GL_EXT_rescale_normal = 1 	# GL/glext.h:4421
# EXT_vertex_array (GL/glext.h:4424)
GL_EXT_vertex_array = 1 	# GL/glext.h:4425
# GL/glext.h:4427
glArrayElementEXT = _link_function('glArrayElementEXT', None, [GLint], 'EXT_vertex_array')

# GL/glext.h:4428
glColorPointerEXT = _link_function('glColorPointerEXT', None, [GLint, GLenum, GLsizei, GLsizei, POINTER(GLvoid)], 'EXT_vertex_array')

# GL/glext.h:4429
glDrawArraysEXT = _link_function('glDrawArraysEXT', None, [GLenum, GLint, GLsizei], 'EXT_vertex_array')

# GL/glext.h:4430
glEdgeFlagPointerEXT = _link_function('glEdgeFlagPointerEXT', None, [GLsizei, GLsizei, POINTER(GLboolean)], 'EXT_vertex_array')

# GL/glext.h:4431
glGetPointervEXT = _link_function('glGetPointervEXT', None, [GLenum, POINTER(POINTER(GLvoid))], 'EXT_vertex_array')

# GL/glext.h:4432
glIndexPointerEXT = _link_function('glIndexPointerEXT', None, [GLenum, GLsizei, GLsizei, POINTER(GLvoid)], 'EXT_vertex_array')

# GL/glext.h:4433
glNormalPointerEXT = _link_function('glNormalPointerEXT', None, [GLenum, GLsizei, GLsizei, POINTER(GLvoid)], 'EXT_vertex_array')

# GL/glext.h:4434
glTexCoordPointerEXT = _link_function('glTexCoordPointerEXT', None, [GLint, GLenum, GLsizei, GLsizei, POINTER(GLvoid)], 'EXT_vertex_array')

# GL/glext.h:4435
glVertexPointerEXT = _link_function('glVertexPointerEXT', None, [GLint, GLenum, GLsizei, GLsizei, POINTER(GLvoid)], 'EXT_vertex_array')

PFNGLARRAYELEMENTEXTPROC = CFUNCTYPE(None, GLint) 	# GL/glext.h:4437
PFNGLCOLORPOINTEREXTPROC = CFUNCTYPE(None, GLint, GLenum, GLsizei, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:4438
PFNGLDRAWARRAYSEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLsizei) 	# GL/glext.h:4439
PFNGLEDGEFLAGPOINTEREXTPROC = CFUNCTYPE(None, GLsizei, GLsizei, POINTER(GLboolean)) 	# GL/glext.h:4440
PFNGLGETPOINTERVEXTPROC = CFUNCTYPE(None, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:4441
PFNGLINDEXPOINTEREXTPROC = CFUNCTYPE(None, GLenum, GLsizei, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:4442
PFNGLNORMALPOINTEREXTPROC = CFUNCTYPE(None, GLenum, GLsizei, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:4443
PFNGLTEXCOORDPOINTEREXTPROC = CFUNCTYPE(None, GLint, GLenum, GLsizei, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:4444
PFNGLVERTEXPOINTEREXTPROC = CFUNCTYPE(None, GLint, GLenum, GLsizei, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:4445
# EXT_misc_attribute (GL/glext.h:4448)
GL_EXT_misc_attribute = 1 	# GL/glext.h:4449
# SGIS_generate_mipmap (GL/glext.h:4452)
GL_SGIS_generate_mipmap = 1 	# GL/glext.h:4453
# SGIX_clipmap (GL/glext.h:4456)
GL_SGIX_clipmap = 1 	# GL/glext.h:4457
# SGIX_shadow (GL/glext.h:4460)
GL_SGIX_shadow = 1 	# GL/glext.h:4461
# SGIS_texture_edge_clamp (GL/glext.h:4464)
GL_SGIS_texture_edge_clamp = 1 	# GL/glext.h:4465
# SGIS_texture_border_clamp (GL/glext.h:4468)
GL_SGIS_texture_border_clamp = 1 	# GL/glext.h:4469
# EXT_blend_minmax (GL/glext.h:4472)
GL_EXT_blend_minmax = 1 	# GL/glext.h:4473
# GL/glext.h:4475
glBlendEquationEXT = _link_function('glBlendEquationEXT', None, [GLenum], 'EXT_blend_minmax')

PFNGLBLENDEQUATIONEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:4477
# EXT_blend_subtract (GL/glext.h:4480)
GL_EXT_blend_subtract = 1 	# GL/glext.h:4481
# EXT_blend_logic_op (GL/glext.h:4484)
GL_EXT_blend_logic_op = 1 	# GL/glext.h:4485
# SGIX_interlace (GL/glext.h:4488)
GL_SGIX_interlace = 1 	# GL/glext.h:4489
# SGIX_pixel_tiles (GL/glext.h:4492)
GL_SGIX_pixel_tiles = 1 	# GL/glext.h:4493
# SGIX_texture_select (GL/glext.h:4496)
GL_SGIX_texture_select = 1 	# GL/glext.h:4497
# SGIX_sprite (GL/glext.h:4500)
GL_SGIX_sprite = 1 	# GL/glext.h:4501
# GL/glext.h:4503
glSpriteParameterfSGIX = _link_function('glSpriteParameterfSGIX', None, [GLenum, GLfloat], 'SGIX_sprite')

# GL/glext.h:4504
glSpriteParameterfvSGIX = _link_function('glSpriteParameterfvSGIX', None, [GLenum, POINTER(GLfloat)], 'SGIX_sprite')

# GL/glext.h:4505
glSpriteParameteriSGIX = _link_function('glSpriteParameteriSGIX', None, [GLenum, GLint], 'SGIX_sprite')

# GL/glext.h:4506
glSpriteParameterivSGIX = _link_function('glSpriteParameterivSGIX', None, [GLenum, POINTER(GLint)], 'SGIX_sprite')

PFNGLSPRITEPARAMETERFSGIXPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:4508
PFNGLSPRITEPARAMETERFVSGIXPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4509
PFNGLSPRITEPARAMETERISGIXPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:4510
PFNGLSPRITEPARAMETERIVSGIXPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:4511
# SGIX_texture_multi_buffer (GL/glext.h:4514)
GL_SGIX_texture_multi_buffer = 1 	# GL/glext.h:4515
# EXT_point_parameters (GL/glext.h:4518)
GL_EXT_point_parameters = 1 	# GL/glext.h:4519
# GL/glext.h:4521
glPointParameterfEXT = _link_function('glPointParameterfEXT', None, [GLenum, GLfloat], 'EXT_point_parameters')

# GL/glext.h:4522
glPointParameterfvEXT = _link_function('glPointParameterfvEXT', None, [GLenum, POINTER(GLfloat)], 'EXT_point_parameters')

PFNGLPOINTPARAMETERFEXTPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:4524
PFNGLPOINTPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4525
# SGIS_point_parameters (GL/glext.h:4528)
GL_SGIS_point_parameters = 1 	# GL/glext.h:4529
# GL/glext.h:4531
glPointParameterfSGIS = _link_function('glPointParameterfSGIS', None, [GLenum, GLfloat], 'SGIS_point_parameters')

# GL/glext.h:4532
glPointParameterfvSGIS = _link_function('glPointParameterfvSGIS', None, [GLenum, POINTER(GLfloat)], 'SGIS_point_parameters')

PFNGLPOINTPARAMETERFSGISPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:4534
PFNGLPOINTPARAMETERFVSGISPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4535
# SGIX_instruments (GL/glext.h:4538)
GL_SGIX_instruments = 1 	# GL/glext.h:4539
# GL/glext.h:4541
glGetInstrumentsSGIX = _link_function('glGetInstrumentsSGIX', GLint, [], 'SGIX_instruments')

# GL/glext.h:4542
glInstrumentsBufferSGIX = _link_function('glInstrumentsBufferSGIX', None, [GLsizei, POINTER(GLint)], 'SGIX_instruments')

# GL/glext.h:4543
glPollInstrumentsSGIX = _link_function('glPollInstrumentsSGIX', GLint, [POINTER(GLint)], 'SGIX_instruments')

# GL/glext.h:4544
glReadInstrumentsSGIX = _link_function('glReadInstrumentsSGIX', None, [GLint], 'SGIX_instruments')

# GL/glext.h:4545
glStartInstrumentsSGIX = _link_function('glStartInstrumentsSGIX', None, [], 'SGIX_instruments')

# GL/glext.h:4546
glStopInstrumentsSGIX = _link_function('glStopInstrumentsSGIX', None, [GLint], 'SGIX_instruments')

PFNGLGETINSTRUMENTSSGIXPROC = CFUNCTYPE(GLint) 	# GL/glext.h:4548
PFNGLINSTRUMENTSBUFFERSGIXPROC = CFUNCTYPE(None, GLsizei, POINTER(GLint)) 	# GL/glext.h:4549
PFNGLPOLLINSTRUMENTSSGIXPROC = CFUNCTYPE(GLint, POINTER(GLint)) 	# GL/glext.h:4550
PFNGLREADINSTRUMENTSSGIXPROC = CFUNCTYPE(None, GLint) 	# GL/glext.h:4551
PFNGLSTARTINSTRUMENTSSGIXPROC = CFUNCTYPE(None) 	# GL/glext.h:4552
PFNGLSTOPINSTRUMENTSSGIXPROC = CFUNCTYPE(None, GLint) 	# GL/glext.h:4553
# SGIX_texture_scale_bias (GL/glext.h:4556)
GL_SGIX_texture_scale_bias = 1 	# GL/glext.h:4557
# SGIX_framezoom (GL/glext.h:4560)
GL_SGIX_framezoom = 1 	# GL/glext.h:4561
# GL/glext.h:4563
glFrameZoomSGIX = _link_function('glFrameZoomSGIX', None, [GLint], 'SGIX_framezoom')

PFNGLFRAMEZOOMSGIXPROC = CFUNCTYPE(None, GLint) 	# GL/glext.h:4565
# SGIX_tag_sample_buffer (GL/glext.h:4568)
GL_SGIX_tag_sample_buffer = 1 	# GL/glext.h:4569
# GL/glext.h:4571
glTagSampleBufferSGIX = _link_function('glTagSampleBufferSGIX', None, [], 'SGIX_tag_sample_buffer')

PFNGLTAGSAMPLEBUFFERSGIXPROC = CFUNCTYPE(None) 	# GL/glext.h:4573
# SGIX_polynomial_ffd (GL/glext.h:4576)
GL_SGIX_polynomial_ffd = 1 	# GL/glext.h:4577
# GL/glext.h:4579
glDeformationMap3dSGIX = _link_function('glDeformationMap3dSGIX', None, [GLenum, GLdouble, GLdouble, GLint, GLint, GLdouble, GLdouble, GLint, GLint, GLdouble, GLdouble, GLint, GLint, POINTER(GLdouble)], 'SGIX_polynomial_ffd')

# GL/glext.h:4580
glDeformationMap3fSGIX = _link_function('glDeformationMap3fSGIX', None, [GLenum, GLfloat, GLfloat, GLint, GLint, GLfloat, GLfloat, GLint, GLint, GLfloat, GLfloat, GLint, GLint, POINTER(GLfloat)], 'SGIX_polynomial_ffd')

GLbitfield = c_uint 	# /usr/include/GL/gl.h:55
# GL/glext.h:4581
glDeformSGIX = _link_function('glDeformSGIX', None, [GLbitfield], 'SGIX_polynomial_ffd')

# GL/glext.h:4582
glLoadIdentityDeformationMapSGIX = _link_function('glLoadIdentityDeformationMapSGIX', None, [GLbitfield], 'SGIX_polynomial_ffd')

PFNGLDEFORMATIONMAP3DSGIXPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble, GLint, GLint, GLdouble, GLdouble, GLint, GLint, GLdouble, GLdouble, GLint, GLint, POINTER(GLdouble)) 	# GL/glext.h:4584
PFNGLDEFORMATIONMAP3FSGIXPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat, GLint, GLint, GLfloat, GLfloat, GLint, GLint, GLfloat, GLfloat, GLint, GLint, POINTER(GLfloat)) 	# GL/glext.h:4585
PFNGLDEFORMSGIXPROC = CFUNCTYPE(None, GLbitfield) 	# GL/glext.h:4586
PFNGLLOADIDENTITYDEFORMATIONMAPSGIXPROC = CFUNCTYPE(None, GLbitfield) 	# GL/glext.h:4587
# SGIX_reference_plane (GL/glext.h:4590)
GL_SGIX_reference_plane = 1 	# GL/glext.h:4591
# GL/glext.h:4593
glReferencePlaneSGIX = _link_function('glReferencePlaneSGIX', None, [POINTER(GLdouble)], 'SGIX_reference_plane')

PFNGLREFERENCEPLANESGIXPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:4595
# SGIX_flush_raster (GL/glext.h:4598)
GL_SGIX_flush_raster = 1 	# GL/glext.h:4599
# GL/glext.h:4601
glFlushRasterSGIX = _link_function('glFlushRasterSGIX', None, [], 'SGIX_flush_raster')

PFNGLFLUSHRASTERSGIXPROC = CFUNCTYPE(None) 	# GL/glext.h:4603
# SGIX_depth_texture (GL/glext.h:4606)
GL_SGIX_depth_texture = 1 	# GL/glext.h:4607
# SGIS_fog_function (GL/glext.h:4610)
GL_SGIS_fog_function = 1 	# GL/glext.h:4611
# GL/glext.h:4613
glFogFuncSGIS = _link_function('glFogFuncSGIS', None, [GLsizei, POINTER(GLfloat)], 'SGIS_fog_function')

# GL/glext.h:4614
glGetFogFuncSGIS = _link_function('glGetFogFuncSGIS', None, [POINTER(GLfloat)], 'SGIS_fog_function')

PFNGLFOGFUNCSGISPROC = CFUNCTYPE(None, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:4616
PFNGLGETFOGFUNCSGISPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:4617
# SGIX_fog_offset (GL/glext.h:4620)
GL_SGIX_fog_offset = 1 	# GL/glext.h:4621
# HP_image_transform (GL/glext.h:4624)
GL_HP_image_transform = 1 	# GL/glext.h:4625
# GL/glext.h:4627
glImageTransformParameteriHP = _link_function('glImageTransformParameteriHP', None, [GLenum, GLenum, GLint], 'HP_image_transform')

# GL/glext.h:4628
glImageTransformParameterfHP = _link_function('glImageTransformParameterfHP', None, [GLenum, GLenum, GLfloat], 'HP_image_transform')

# GL/glext.h:4629
glImageTransformParameterivHP = _link_function('glImageTransformParameterivHP', None, [GLenum, GLenum, POINTER(GLint)], 'HP_image_transform')

# GL/glext.h:4630
glImageTransformParameterfvHP = _link_function('glImageTransformParameterfvHP', None, [GLenum, GLenum, POINTER(GLfloat)], 'HP_image_transform')

# GL/glext.h:4631
glGetImageTransformParameterivHP = _link_function('glGetImageTransformParameterivHP', None, [GLenum, GLenum, POINTER(GLint)], 'HP_image_transform')

# GL/glext.h:4632
glGetImageTransformParameterfvHP = _link_function('glGetImageTransformParameterfvHP', None, [GLenum, GLenum, POINTER(GLfloat)], 'HP_image_transform')

PFNGLIMAGETRANSFORMPARAMETERIHPPROC = CFUNCTYPE(None, GLenum, GLenum, GLint) 	# GL/glext.h:4634
PFNGLIMAGETRANSFORMPARAMETERFHPPROC = CFUNCTYPE(None, GLenum, GLenum, GLfloat) 	# GL/glext.h:4635
PFNGLIMAGETRANSFORMPARAMETERIVHPPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:4636
PFNGLIMAGETRANSFORMPARAMETERFVHPPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4637
PFNGLGETIMAGETRANSFORMPARAMETERIVHPPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:4638
PFNGLGETIMAGETRANSFORMPARAMETERFVHPPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4639
# HP_convolution_border_modes (GL/glext.h:4642)
GL_HP_convolution_border_modes = 1 	# GL/glext.h:4643
# SGIX_texture_add_env (GL/glext.h:4646)
GL_SGIX_texture_add_env = 1 	# GL/glext.h:4647
# EXT_color_subtable (GL/glext.h:4650)
GL_EXT_color_subtable = 1 	# GL/glext.h:4651
# GL/glext.h:4653
glColorSubTableEXT = _link_function('glColorSubTableEXT', None, [GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_color_subtable')

# GL/glext.h:4654
glCopyColorSubTableEXT = _link_function('glCopyColorSubTableEXT', None, [GLenum, GLsizei, GLint, GLint, GLsizei], 'EXT_color_subtable')

PFNGLCOLORSUBTABLEEXTPROC = CFUNCTYPE(None, GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:4656
PFNGLCOPYCOLORSUBTABLEEXTPROC = CFUNCTYPE(None, GLenum, GLsizei, GLint, GLint, GLsizei) 	# GL/glext.h:4657
# PGI_vertex_hints (GL/glext.h:4660)
GL_PGI_vertex_hints = 1 	# GL/glext.h:4661
# PGI_misc_hints (GL/glext.h:4664)
GL_PGI_misc_hints = 1 	# GL/glext.h:4665
# GL/glext.h:4667
glHintPGI = _link_function('glHintPGI', None, [GLenum, GLint], 'PGI_misc_hints')

PFNGLHINTPGIPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:4669
# EXT_paletted_texture (GL/glext.h:4672)
GL_EXT_paletted_texture = 1 	# GL/glext.h:4673
# GL/glext.h:4675
glColorTableEXT = _link_function('glColorTableEXT', None, [GLenum, GLenum, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_paletted_texture')

# GL/glext.h:4676
glGetColorTableEXT = _link_function('glGetColorTableEXT', None, [GLenum, GLenum, GLenum, POINTER(GLvoid)], 'EXT_paletted_texture')

# GL/glext.h:4677
glGetColorTableParameterivEXT = _link_function('glGetColorTableParameterivEXT', None, [GLenum, GLenum, POINTER(GLint)], 'EXT_paletted_texture')

# GL/glext.h:4678
glGetColorTableParameterfvEXT = _link_function('glGetColorTableParameterfvEXT', None, [GLenum, GLenum, POINTER(GLfloat)], 'EXT_paletted_texture')

PFNGLCOLORTABLEEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:4680
PFNGLGETCOLORTABLEEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:4681
PFNGLGETCOLORTABLEPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:4682
PFNGLGETCOLORTABLEPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4683
# EXT_clip_volume_hint (GL/glext.h:4686)
GL_EXT_clip_volume_hint = 1 	# GL/glext.h:4687
# SGIX_list_priority (GL/glext.h:4690)
GL_SGIX_list_priority = 1 	# GL/glext.h:4691
# GL/glext.h:4693
glGetListParameterfvSGIX = _link_function('glGetListParameterfvSGIX', None, [GLuint, GLenum, POINTER(GLfloat)], 'SGIX_list_priority')

# GL/glext.h:4694
glGetListParameterivSGIX = _link_function('glGetListParameterivSGIX', None, [GLuint, GLenum, POINTER(GLint)], 'SGIX_list_priority')

# GL/glext.h:4695
glListParameterfSGIX = _link_function('glListParameterfSGIX', None, [GLuint, GLenum, GLfloat], 'SGIX_list_priority')

# GL/glext.h:4696
glListParameterfvSGIX = _link_function('glListParameterfvSGIX', None, [GLuint, GLenum, POINTER(GLfloat)], 'SGIX_list_priority')

# GL/glext.h:4697
glListParameteriSGIX = _link_function('glListParameteriSGIX', None, [GLuint, GLenum, GLint], 'SGIX_list_priority')

# GL/glext.h:4698
glListParameterivSGIX = _link_function('glListParameterivSGIX', None, [GLuint, GLenum, POINTER(GLint)], 'SGIX_list_priority')

PFNGLGETLISTPARAMETERFVSGIXPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4700
PFNGLGETLISTPARAMETERIVSGIXPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:4701
PFNGLLISTPARAMETERFSGIXPROC = CFUNCTYPE(None, GLuint, GLenum, GLfloat) 	# GL/glext.h:4702
PFNGLLISTPARAMETERFVSGIXPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4703
PFNGLLISTPARAMETERISGIXPROC = CFUNCTYPE(None, GLuint, GLenum, GLint) 	# GL/glext.h:4704
PFNGLLISTPARAMETERIVSGIXPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:4705
# SGIX_ir_instrument1 (GL/glext.h:4708)
GL_SGIX_ir_instrument1 = 1 	# GL/glext.h:4709
# SGIX_calligraphic_fragment (GL/glext.h:4712)
GL_SGIX_calligraphic_fragment = 1 	# GL/glext.h:4713
# SGIX_texture_lod_bias (GL/glext.h:4716)
GL_SGIX_texture_lod_bias = 1 	# GL/glext.h:4717
# SGIX_shadow_ambient (GL/glext.h:4720)
GL_SGIX_shadow_ambient = 1 	# GL/glext.h:4721
# EXT_index_texture (GL/glext.h:4724)
GL_EXT_index_texture = 1 	# GL/glext.h:4725
# EXT_index_material (GL/glext.h:4728)
GL_EXT_index_material = 1 	# GL/glext.h:4729
# GL/glext.h:4731
glIndexMaterialEXT = _link_function('glIndexMaterialEXT', None, [GLenum, GLenum], 'EXT_index_material')

PFNGLINDEXMATERIALEXTPROC = CFUNCTYPE(None, GLenum, GLenum) 	# GL/glext.h:4733
# EXT_index_func (GL/glext.h:4736)
GL_EXT_index_func = 1 	# GL/glext.h:4737
# GL/glext.h:4739
glIndexFuncEXT = _link_function('glIndexFuncEXT', None, [GLenum, GLclampf], 'EXT_index_func')

PFNGLINDEXFUNCEXTPROC = CFUNCTYPE(None, GLenum, GLclampf) 	# GL/glext.h:4741
# EXT_index_array_formats (GL/glext.h:4744)
GL_EXT_index_array_formats = 1 	# GL/glext.h:4745
# EXT_compiled_vertex_array (GL/glext.h:4748)
GL_EXT_compiled_vertex_array = 1 	# GL/glext.h:4749
# GL/glext.h:4751
glLockArraysEXT = _link_function('glLockArraysEXT', None, [GLint, GLsizei], 'EXT_compiled_vertex_array')

# GL/glext.h:4752
glUnlockArraysEXT = _link_function('glUnlockArraysEXT', None, [], 'EXT_compiled_vertex_array')

PFNGLLOCKARRAYSEXTPROC = CFUNCTYPE(None, GLint, GLsizei) 	# GL/glext.h:4754
PFNGLUNLOCKARRAYSEXTPROC = CFUNCTYPE(None) 	# GL/glext.h:4755
# EXT_cull_vertex (GL/glext.h:4758)
GL_EXT_cull_vertex = 1 	# GL/glext.h:4759
# GL/glext.h:4761
glCullParameterdvEXT = _link_function('glCullParameterdvEXT', None, [GLenum, POINTER(GLdouble)], 'EXT_cull_vertex')

# GL/glext.h:4762
glCullParameterfvEXT = _link_function('glCullParameterfvEXT', None, [GLenum, POINTER(GLfloat)], 'EXT_cull_vertex')

PFNGLCULLPARAMETERDVEXTPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:4764
PFNGLCULLPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4765
# SGIX_ycrcb (GL/glext.h:4768)
GL_SGIX_ycrcb = 1 	# GL/glext.h:4769
# SGIX_fragment_lighting (GL/glext.h:4772)
GL_SGIX_fragment_lighting = 1 	# GL/glext.h:4773
# GL/glext.h:4775
glFragmentColorMaterialSGIX = _link_function('glFragmentColorMaterialSGIX', None, [GLenum, GLenum], 'SGIX_fragment_lighting')

# GL/glext.h:4776
glFragmentLightfSGIX = _link_function('glFragmentLightfSGIX', None, [GLenum, GLenum, GLfloat], 'SGIX_fragment_lighting')

# GL/glext.h:4777
glFragmentLightfvSGIX = _link_function('glFragmentLightfvSGIX', None, [GLenum, GLenum, POINTER(GLfloat)], 'SGIX_fragment_lighting')

# GL/glext.h:4778
glFragmentLightiSGIX = _link_function('glFragmentLightiSGIX', None, [GLenum, GLenum, GLint], 'SGIX_fragment_lighting')

# GL/glext.h:4779
glFragmentLightivSGIX = _link_function('glFragmentLightivSGIX', None, [GLenum, GLenum, POINTER(GLint)], 'SGIX_fragment_lighting')

# GL/glext.h:4780
glFragmentLightModelfSGIX = _link_function('glFragmentLightModelfSGIX', None, [GLenum, GLfloat], 'SGIX_fragment_lighting')

# GL/glext.h:4781
glFragmentLightModelfvSGIX = _link_function('glFragmentLightModelfvSGIX', None, [GLenum, POINTER(GLfloat)], 'SGIX_fragment_lighting')

# GL/glext.h:4782
glFragmentLightModeliSGIX = _link_function('glFragmentLightModeliSGIX', None, [GLenum, GLint], 'SGIX_fragment_lighting')

# GL/glext.h:4783
glFragmentLightModelivSGIX = _link_function('glFragmentLightModelivSGIX', None, [GLenum, POINTER(GLint)], 'SGIX_fragment_lighting')

# GL/glext.h:4784
glFragmentMaterialfSGIX = _link_function('glFragmentMaterialfSGIX', None, [GLenum, GLenum, GLfloat], 'SGIX_fragment_lighting')

# GL/glext.h:4785
glFragmentMaterialfvSGIX = _link_function('glFragmentMaterialfvSGIX', None, [GLenum, GLenum, POINTER(GLfloat)], 'SGIX_fragment_lighting')

# GL/glext.h:4786
glFragmentMaterialiSGIX = _link_function('glFragmentMaterialiSGIX', None, [GLenum, GLenum, GLint], 'SGIX_fragment_lighting')

# GL/glext.h:4787
glFragmentMaterialivSGIX = _link_function('glFragmentMaterialivSGIX', None, [GLenum, GLenum, POINTER(GLint)], 'SGIX_fragment_lighting')

# GL/glext.h:4788
glGetFragmentLightfvSGIX = _link_function('glGetFragmentLightfvSGIX', None, [GLenum, GLenum, POINTER(GLfloat)], 'SGIX_fragment_lighting')

# GL/glext.h:4789
glGetFragmentLightivSGIX = _link_function('glGetFragmentLightivSGIX', None, [GLenum, GLenum, POINTER(GLint)], 'SGIX_fragment_lighting')

# GL/glext.h:4790
glGetFragmentMaterialfvSGIX = _link_function('glGetFragmentMaterialfvSGIX', None, [GLenum, GLenum, POINTER(GLfloat)], 'SGIX_fragment_lighting')

# GL/glext.h:4791
glGetFragmentMaterialivSGIX = _link_function('glGetFragmentMaterialivSGIX', None, [GLenum, GLenum, POINTER(GLint)], 'SGIX_fragment_lighting')

# GL/glext.h:4792
glLightEnviSGIX = _link_function('glLightEnviSGIX', None, [GLenum, GLint], 'SGIX_fragment_lighting')

PFNGLFRAGMENTCOLORMATERIALSGIXPROC = CFUNCTYPE(None, GLenum, GLenum) 	# GL/glext.h:4794
PFNGLFRAGMENTLIGHTFSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, GLfloat) 	# GL/glext.h:4795
PFNGLFRAGMENTLIGHTFVSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4796
PFNGLFRAGMENTLIGHTISGIXPROC = CFUNCTYPE(None, GLenum, GLenum, GLint) 	# GL/glext.h:4797
PFNGLFRAGMENTLIGHTIVSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:4798
PFNGLFRAGMENTLIGHTMODELFSGIXPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:4799
PFNGLFRAGMENTLIGHTMODELFVSGIXPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4800
PFNGLFRAGMENTLIGHTMODELISGIXPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:4801
PFNGLFRAGMENTLIGHTMODELIVSGIXPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:4802
PFNGLFRAGMENTMATERIALFSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, GLfloat) 	# GL/glext.h:4803
PFNGLFRAGMENTMATERIALFVSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4804
PFNGLFRAGMENTMATERIALISGIXPROC = CFUNCTYPE(None, GLenum, GLenum, GLint) 	# GL/glext.h:4805
PFNGLFRAGMENTMATERIALIVSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:4806
PFNGLGETFRAGMENTLIGHTFVSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4807
PFNGLGETFRAGMENTLIGHTIVSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:4808
PFNGLGETFRAGMENTMATERIALFVSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4809
PFNGLGETFRAGMENTMATERIALIVSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:4810
PFNGLLIGHTENVISGIXPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:4811
# IBM_rasterpos_clip (GL/glext.h:4814)
GL_IBM_rasterpos_clip = 1 	# GL/glext.h:4815
# HP_texture_lighting (GL/glext.h:4818)
GL_HP_texture_lighting = 1 	# GL/glext.h:4819
# EXT_draw_range_elements (GL/glext.h:4822)
GL_EXT_draw_range_elements = 1 	# GL/glext.h:4823
# GL/glext.h:4825
glDrawRangeElementsEXT = _link_function('glDrawRangeElementsEXT', None, [GLenum, GLuint, GLuint, GLsizei, GLenum, POINTER(GLvoid)], 'EXT_draw_range_elements')

PFNGLDRAWRANGEELEMENTSEXTPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLsizei, GLenum, POINTER(GLvoid)) 	# GL/glext.h:4827
# WIN_phong_shading (GL/glext.h:4830)
GL_WIN_phong_shading = 1 	# GL/glext.h:4831
# WIN_specular_fog (GL/glext.h:4834)
GL_WIN_specular_fog = 1 	# GL/glext.h:4835
# EXT_light_texture (GL/glext.h:4838)
GL_EXT_light_texture = 1 	# GL/glext.h:4839
# GL/glext.h:4841
glApplyTextureEXT = _link_function('glApplyTextureEXT', None, [GLenum], 'EXT_light_texture')

# GL/glext.h:4842
glTextureLightEXT = _link_function('glTextureLightEXT', None, [GLenum], 'EXT_light_texture')

# GL/glext.h:4843
glTextureMaterialEXT = _link_function('glTextureMaterialEXT', None, [GLenum, GLenum], 'EXT_light_texture')

PFNGLAPPLYTEXTUREEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:4845
PFNGLTEXTURELIGHTEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:4846
PFNGLTEXTUREMATERIALEXTPROC = CFUNCTYPE(None, GLenum, GLenum) 	# GL/glext.h:4847
# SGIX_blend_alpha_minmax (GL/glext.h:4850)
GL_SGIX_blend_alpha_minmax = 1 	# GL/glext.h:4851
# EXT_bgra (GL/glext.h:4854)
GL_EXT_bgra = 1 	# GL/glext.h:4855
# SGIX_async (GL/glext.h:4858)
GL_SGIX_async = 1 	# GL/glext.h:4859
# GL/glext.h:4861
glAsyncMarkerSGIX = _link_function('glAsyncMarkerSGIX', None, [GLuint], 'SGIX_async')

# GL/glext.h:4862
glFinishAsyncSGIX = _link_function('glFinishAsyncSGIX', GLint, [POINTER(GLuint)], 'SGIX_async')

# GL/glext.h:4863
glPollAsyncSGIX = _link_function('glPollAsyncSGIX', GLint, [POINTER(GLuint)], 'SGIX_async')

# GL/glext.h:4864
glGenAsyncMarkersSGIX = _link_function('glGenAsyncMarkersSGIX', GLuint, [GLsizei], 'SGIX_async')

# GL/glext.h:4865
glDeleteAsyncMarkersSGIX = _link_function('glDeleteAsyncMarkersSGIX', None, [GLuint, GLsizei], 'SGIX_async')

# GL/glext.h:4866
glIsAsyncMarkerSGIX = _link_function('glIsAsyncMarkerSGIX', GLboolean, [GLuint], 'SGIX_async')

PFNGLASYNCMARKERSGIXPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:4868
PFNGLFINISHASYNCSGIXPROC = CFUNCTYPE(GLint, POINTER(GLuint)) 	# GL/glext.h:4869
PFNGLPOLLASYNCSGIXPROC = CFUNCTYPE(GLint, POINTER(GLuint)) 	# GL/glext.h:4870
PFNGLGENASYNCMARKERSSGIXPROC = CFUNCTYPE(GLuint, GLsizei) 	# GL/glext.h:4871
PFNGLDELETEASYNCMARKERSSGIXPROC = CFUNCTYPE(None, GLuint, GLsizei) 	# GL/glext.h:4872
PFNGLISASYNCMARKERSGIXPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:4873
# SGIX_async_pixel (GL/glext.h:4876)
GL_SGIX_async_pixel = 1 	# GL/glext.h:4877
# SGIX_async_histogram (GL/glext.h:4880)
GL_SGIX_async_histogram = 1 	# GL/glext.h:4881
# INTEL_parallel_arrays (GL/glext.h:4884)
GL_INTEL_parallel_arrays = 1 	# GL/glext.h:4885
# GL/glext.h:4887
glVertexPointervINTEL = _link_function('glVertexPointervINTEL', None, [GLint, GLenum, POINTER(POINTER(GLvoid))], 'INTEL_parallel_arrays')

# GL/glext.h:4888
glNormalPointervINTEL = _link_function('glNormalPointervINTEL', None, [GLenum, POINTER(POINTER(GLvoid))], 'INTEL_parallel_arrays')

# GL/glext.h:4889
glColorPointervINTEL = _link_function('glColorPointervINTEL', None, [GLint, GLenum, POINTER(POINTER(GLvoid))], 'INTEL_parallel_arrays')

# GL/glext.h:4890
glTexCoordPointervINTEL = _link_function('glTexCoordPointervINTEL', None, [GLint, GLenum, POINTER(POINTER(GLvoid))], 'INTEL_parallel_arrays')

PFNGLVERTEXPOINTERVINTELPROC = CFUNCTYPE(None, GLint, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:4892
PFNGLNORMALPOINTERVINTELPROC = CFUNCTYPE(None, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:4893
PFNGLCOLORPOINTERVINTELPROC = CFUNCTYPE(None, GLint, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:4894
PFNGLTEXCOORDPOINTERVINTELPROC = CFUNCTYPE(None, GLint, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:4895
# HP_occlusion_test (GL/glext.h:4898)
GL_HP_occlusion_test = 1 	# GL/glext.h:4899
# EXT_pixel_transform (GL/glext.h:4902)
GL_EXT_pixel_transform = 1 	# GL/glext.h:4903
# GL/glext.h:4905
glPixelTransformParameteriEXT = _link_function('glPixelTransformParameteriEXT', None, [GLenum, GLenum, GLint], 'EXT_pixel_transform')

# GL/glext.h:4906
glPixelTransformParameterfEXT = _link_function('glPixelTransformParameterfEXT', None, [GLenum, GLenum, GLfloat], 'EXT_pixel_transform')

# GL/glext.h:4907
glPixelTransformParameterivEXT = _link_function('glPixelTransformParameterivEXT', None, [GLenum, GLenum, POINTER(GLint)], 'EXT_pixel_transform')

# GL/glext.h:4908
glPixelTransformParameterfvEXT = _link_function('glPixelTransformParameterfvEXT', None, [GLenum, GLenum, POINTER(GLfloat)], 'EXT_pixel_transform')

PFNGLPIXELTRANSFORMPARAMETERIEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint) 	# GL/glext.h:4910
PFNGLPIXELTRANSFORMPARAMETERFEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLfloat) 	# GL/glext.h:4911
PFNGLPIXELTRANSFORMPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:4912
PFNGLPIXELTRANSFORMPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4913
# EXT_pixel_transform_color_table (GL/glext.h:4916)
GL_EXT_pixel_transform_color_table = 1 	# GL/glext.h:4917
# EXT_shared_texture_palette (GL/glext.h:4920)
GL_EXT_shared_texture_palette = 1 	# GL/glext.h:4921
# EXT_separate_specular_color (GL/glext.h:4924)
GL_EXT_separate_specular_color = 1 	# GL/glext.h:4925
# EXT_secondary_color (GL/glext.h:4928)
GL_EXT_secondary_color = 1 	# GL/glext.h:4929
# GL/glext.h:4931
glSecondaryColor3bEXT = _link_function('glSecondaryColor3bEXT', None, [GLbyte, GLbyte, GLbyte], 'EXT_secondary_color')

# GL/glext.h:4932
glSecondaryColor3bvEXT = _link_function('glSecondaryColor3bvEXT', None, [POINTER(GLbyte)], 'EXT_secondary_color')

# GL/glext.h:4933
glSecondaryColor3dEXT = _link_function('glSecondaryColor3dEXT', None, [GLdouble, GLdouble, GLdouble], 'EXT_secondary_color')

# GL/glext.h:4934
glSecondaryColor3dvEXT = _link_function('glSecondaryColor3dvEXT', None, [POINTER(GLdouble)], 'EXT_secondary_color')

# GL/glext.h:4935
glSecondaryColor3fEXT = _link_function('glSecondaryColor3fEXT', None, [GLfloat, GLfloat, GLfloat], 'EXT_secondary_color')

# GL/glext.h:4936
glSecondaryColor3fvEXT = _link_function('glSecondaryColor3fvEXT', None, [POINTER(GLfloat)], 'EXT_secondary_color')

# GL/glext.h:4937
glSecondaryColor3iEXT = _link_function('glSecondaryColor3iEXT', None, [GLint, GLint, GLint], 'EXT_secondary_color')

# GL/glext.h:4938
glSecondaryColor3ivEXT = _link_function('glSecondaryColor3ivEXT', None, [POINTER(GLint)], 'EXT_secondary_color')

# GL/glext.h:4939
glSecondaryColor3sEXT = _link_function('glSecondaryColor3sEXT', None, [GLshort, GLshort, GLshort], 'EXT_secondary_color')

# GL/glext.h:4940
glSecondaryColor3svEXT = _link_function('glSecondaryColor3svEXT', None, [POINTER(GLshort)], 'EXT_secondary_color')

# GL/glext.h:4941
glSecondaryColor3ubEXT = _link_function('glSecondaryColor3ubEXT', None, [GLubyte, GLubyte, GLubyte], 'EXT_secondary_color')

# GL/glext.h:4942
glSecondaryColor3ubvEXT = _link_function('glSecondaryColor3ubvEXT', None, [POINTER(GLubyte)], 'EXT_secondary_color')

# GL/glext.h:4943
glSecondaryColor3uiEXT = _link_function('glSecondaryColor3uiEXT', None, [GLuint, GLuint, GLuint], 'EXT_secondary_color')

# GL/glext.h:4944
glSecondaryColor3uivEXT = _link_function('glSecondaryColor3uivEXT', None, [POINTER(GLuint)], 'EXT_secondary_color')

# GL/glext.h:4945
glSecondaryColor3usEXT = _link_function('glSecondaryColor3usEXT', None, [GLushort, GLushort, GLushort], 'EXT_secondary_color')

# GL/glext.h:4946
glSecondaryColor3usvEXT = _link_function('glSecondaryColor3usvEXT', None, [POINTER(GLushort)], 'EXT_secondary_color')

# GL/glext.h:4947
glSecondaryColorPointerEXT = _link_function('glSecondaryColorPointerEXT', None, [GLint, GLenum, GLsizei, POINTER(GLvoid)], 'EXT_secondary_color')

PFNGLSECONDARYCOLOR3BEXTPROC = CFUNCTYPE(None, GLbyte, GLbyte, GLbyte) 	# GL/glext.h:4949
PFNGLSECONDARYCOLOR3BVEXTPROC = CFUNCTYPE(None, POINTER(GLbyte)) 	# GL/glext.h:4950
PFNGLSECONDARYCOLOR3DEXTPROC = CFUNCTYPE(None, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:4951
PFNGLSECONDARYCOLOR3DVEXTPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:4952
PFNGLSECONDARYCOLOR3FEXTPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:4953
PFNGLSECONDARYCOLOR3FVEXTPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:4954
PFNGLSECONDARYCOLOR3IEXTPROC = CFUNCTYPE(None, GLint, GLint, GLint) 	# GL/glext.h:4955
PFNGLSECONDARYCOLOR3IVEXTPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:4956
PFNGLSECONDARYCOLOR3SEXTPROC = CFUNCTYPE(None, GLshort, GLshort, GLshort) 	# GL/glext.h:4957
PFNGLSECONDARYCOLOR3SVEXTPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:4958
PFNGLSECONDARYCOLOR3UBEXTPROC = CFUNCTYPE(None, GLubyte, GLubyte, GLubyte) 	# GL/glext.h:4959
PFNGLSECONDARYCOLOR3UBVEXTPROC = CFUNCTYPE(None, POINTER(GLubyte)) 	# GL/glext.h:4960
PFNGLSECONDARYCOLOR3UIEXTPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint) 	# GL/glext.h:4961
PFNGLSECONDARYCOLOR3UIVEXTPROC = CFUNCTYPE(None, POINTER(GLuint)) 	# GL/glext.h:4962
PFNGLSECONDARYCOLOR3USEXTPROC = CFUNCTYPE(None, GLushort, GLushort, GLushort) 	# GL/glext.h:4963
PFNGLSECONDARYCOLOR3USVEXTPROC = CFUNCTYPE(None, POINTER(GLushort)) 	# GL/glext.h:4964
PFNGLSECONDARYCOLORPOINTEREXTPROC = CFUNCTYPE(None, GLint, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:4965
# EXT_texture_perturb_normal (GL/glext.h:4968)
GL_EXT_texture_perturb_normal = 1 	# GL/glext.h:4969
# GL/glext.h:4971
glTextureNormalEXT = _link_function('glTextureNormalEXT', None, [GLenum], 'EXT_texture_perturb_normal')

PFNGLTEXTURENORMALEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:4973
# EXT_multi_draw_arrays (GL/glext.h:4976)
GL_EXT_multi_draw_arrays = 1 	# GL/glext.h:4977
# GL/glext.h:4979
glMultiDrawArraysEXT = _link_function('glMultiDrawArraysEXT', None, [GLenum, POINTER(GLint), POINTER(GLsizei), GLsizei], 'EXT_multi_draw_arrays')

# GL/glext.h:4980
glMultiDrawElementsEXT = _link_function('glMultiDrawElementsEXT', None, [GLenum, POINTER(GLsizei), GLenum, POINTER(POINTER(GLvoid)), GLsizei], 'EXT_multi_draw_arrays')

PFNGLMULTIDRAWARRAYSEXTPROC = CFUNCTYPE(None, GLenum, POINTER(GLint), POINTER(GLsizei), GLsizei) 	# GL/glext.h:4982
PFNGLMULTIDRAWELEMENTSEXTPROC = CFUNCTYPE(None, GLenum, POINTER(GLsizei), GLenum, POINTER(POINTER(GLvoid)), GLsizei) 	# GL/glext.h:4983
# EXT_fog_coord (GL/glext.h:4986)
GL_EXT_fog_coord = 1 	# GL/glext.h:4987
# GL/glext.h:4989
glFogCoordfEXT = _link_function('glFogCoordfEXT', None, [GLfloat], 'EXT_fog_coord')

# GL/glext.h:4990
glFogCoordfvEXT = _link_function('glFogCoordfvEXT', None, [POINTER(GLfloat)], 'EXT_fog_coord')

# GL/glext.h:4991
glFogCoorddEXT = _link_function('glFogCoorddEXT', None, [GLdouble], 'EXT_fog_coord')

# GL/glext.h:4992
glFogCoorddvEXT = _link_function('glFogCoorddvEXT', None, [POINTER(GLdouble)], 'EXT_fog_coord')

# GL/glext.h:4993
glFogCoordPointerEXT = _link_function('glFogCoordPointerEXT', None, [GLenum, GLsizei, POINTER(GLvoid)], 'EXT_fog_coord')

PFNGLFOGCOORDFEXTPROC = CFUNCTYPE(None, GLfloat) 	# GL/glext.h:4995
PFNGLFOGCOORDFVEXTPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:4996
PFNGLFOGCOORDDEXTPROC = CFUNCTYPE(None, GLdouble) 	# GL/glext.h:4997
PFNGLFOGCOORDDVEXTPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:4998
PFNGLFOGCOORDPOINTEREXTPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:4999
# REND_screen_coordinates (GL/glext.h:5002)
GL_REND_screen_coordinates = 1 	# GL/glext.h:5003
# EXT_coordinate_frame (GL/glext.h:5006)
GL_EXT_coordinate_frame = 1 	# GL/glext.h:5007
# GL/glext.h:5009
glTangent3bEXT = _link_function('glTangent3bEXT', None, [GLbyte, GLbyte, GLbyte], 'EXT_coordinate_frame')

# GL/glext.h:5010
glTangent3bvEXT = _link_function('glTangent3bvEXT', None, [POINTER(GLbyte)], 'EXT_coordinate_frame')

# GL/glext.h:5011
glTangent3dEXT = _link_function('glTangent3dEXT', None, [GLdouble, GLdouble, GLdouble], 'EXT_coordinate_frame')

# GL/glext.h:5012
glTangent3dvEXT = _link_function('glTangent3dvEXT', None, [POINTER(GLdouble)], 'EXT_coordinate_frame')

# GL/glext.h:5013
glTangent3fEXT = _link_function('glTangent3fEXT', None, [GLfloat, GLfloat, GLfloat], 'EXT_coordinate_frame')

# GL/glext.h:5014
glTangent3fvEXT = _link_function('glTangent3fvEXT', None, [POINTER(GLfloat)], 'EXT_coordinate_frame')

# GL/glext.h:5015
glTangent3iEXT = _link_function('glTangent3iEXT', None, [GLint, GLint, GLint], 'EXT_coordinate_frame')

# GL/glext.h:5016
glTangent3ivEXT = _link_function('glTangent3ivEXT', None, [POINTER(GLint)], 'EXT_coordinate_frame')

# GL/glext.h:5017
glTangent3sEXT = _link_function('glTangent3sEXT', None, [GLshort, GLshort, GLshort], 'EXT_coordinate_frame')

# GL/glext.h:5018
glTangent3svEXT = _link_function('glTangent3svEXT', None, [POINTER(GLshort)], 'EXT_coordinate_frame')

# GL/glext.h:5019
glBinormal3bEXT = _link_function('glBinormal3bEXT', None, [GLbyte, GLbyte, GLbyte], 'EXT_coordinate_frame')

# GL/glext.h:5020
glBinormal3bvEXT = _link_function('glBinormal3bvEXT', None, [POINTER(GLbyte)], 'EXT_coordinate_frame')

# GL/glext.h:5021
glBinormal3dEXT = _link_function('glBinormal3dEXT', None, [GLdouble, GLdouble, GLdouble], 'EXT_coordinate_frame')

# GL/glext.h:5022
glBinormal3dvEXT = _link_function('glBinormal3dvEXT', None, [POINTER(GLdouble)], 'EXT_coordinate_frame')

# GL/glext.h:5023
glBinormal3fEXT = _link_function('glBinormal3fEXT', None, [GLfloat, GLfloat, GLfloat], 'EXT_coordinate_frame')

# GL/glext.h:5024
glBinormal3fvEXT = _link_function('glBinormal3fvEXT', None, [POINTER(GLfloat)], 'EXT_coordinate_frame')

# GL/glext.h:5025
glBinormal3iEXT = _link_function('glBinormal3iEXT', None, [GLint, GLint, GLint], 'EXT_coordinate_frame')

# GL/glext.h:5026
glBinormal3ivEXT = _link_function('glBinormal3ivEXT', None, [POINTER(GLint)], 'EXT_coordinate_frame')

# GL/glext.h:5027
glBinormal3sEXT = _link_function('glBinormal3sEXT', None, [GLshort, GLshort, GLshort], 'EXT_coordinate_frame')

# GL/glext.h:5028
glBinormal3svEXT = _link_function('glBinormal3svEXT', None, [POINTER(GLshort)], 'EXT_coordinate_frame')

# GL/glext.h:5029
glTangentPointerEXT = _link_function('glTangentPointerEXT', None, [GLenum, GLsizei, POINTER(GLvoid)], 'EXT_coordinate_frame')

# GL/glext.h:5030
glBinormalPointerEXT = _link_function('glBinormalPointerEXT', None, [GLenum, GLsizei, POINTER(GLvoid)], 'EXT_coordinate_frame')

PFNGLTANGENT3BEXTPROC = CFUNCTYPE(None, GLbyte, GLbyte, GLbyte) 	# GL/glext.h:5032
PFNGLTANGENT3BVEXTPROC = CFUNCTYPE(None, POINTER(GLbyte)) 	# GL/glext.h:5033
PFNGLTANGENT3DEXTPROC = CFUNCTYPE(None, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:5034
PFNGLTANGENT3DVEXTPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:5035
PFNGLTANGENT3FEXTPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5036
PFNGLTANGENT3FVEXTPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:5037
PFNGLTANGENT3IEXTPROC = CFUNCTYPE(None, GLint, GLint, GLint) 	# GL/glext.h:5038
PFNGLTANGENT3IVEXTPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:5039
PFNGLTANGENT3SEXTPROC = CFUNCTYPE(None, GLshort, GLshort, GLshort) 	# GL/glext.h:5040
PFNGLTANGENT3SVEXTPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:5041
PFNGLBINORMAL3BEXTPROC = CFUNCTYPE(None, GLbyte, GLbyte, GLbyte) 	# GL/glext.h:5042
PFNGLBINORMAL3BVEXTPROC = CFUNCTYPE(None, POINTER(GLbyte)) 	# GL/glext.h:5043
PFNGLBINORMAL3DEXTPROC = CFUNCTYPE(None, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:5044
PFNGLBINORMAL3DVEXTPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:5045
PFNGLBINORMAL3FEXTPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5046
PFNGLBINORMAL3FVEXTPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:5047
PFNGLBINORMAL3IEXTPROC = CFUNCTYPE(None, GLint, GLint, GLint) 	# GL/glext.h:5048
PFNGLBINORMAL3IVEXTPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:5049
PFNGLBINORMAL3SEXTPROC = CFUNCTYPE(None, GLshort, GLshort, GLshort) 	# GL/glext.h:5050
PFNGLBINORMAL3SVEXTPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:5051
PFNGLTANGENTPOINTEREXTPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:5052
PFNGLBINORMALPOINTEREXTPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:5053
# EXT_texture_env_combine (GL/glext.h:5056)
GL_EXT_texture_env_combine = 1 	# GL/glext.h:5057
# APPLE_specular_vector (GL/glext.h:5060)
GL_APPLE_specular_vector = 1 	# GL/glext.h:5061
# APPLE_transform_hint (GL/glext.h:5064)
GL_APPLE_transform_hint = 1 	# GL/glext.h:5065
# SGIX_fog_scale (GL/glext.h:5068)
GL_SGIX_fog_scale = 1 	# GL/glext.h:5069
# SUNX_constant_data (GL/glext.h:5072)
GL_SUNX_constant_data = 1 	# GL/glext.h:5073
# GL/glext.h:5075
glFinishTextureSUNX = _link_function('glFinishTextureSUNX', None, [], 'SUNX_constant_data')

PFNGLFINISHTEXTURESUNXPROC = CFUNCTYPE(None) 	# GL/glext.h:5077
# SUN_global_alpha (GL/glext.h:5080)
GL_SUN_global_alpha = 1 	# GL/glext.h:5081
# GL/glext.h:5083
glGlobalAlphaFactorbSUN = _link_function('glGlobalAlphaFactorbSUN', None, [GLbyte], 'SUN_global_alpha')

# GL/glext.h:5084
glGlobalAlphaFactorsSUN = _link_function('glGlobalAlphaFactorsSUN', None, [GLshort], 'SUN_global_alpha')

# GL/glext.h:5085
glGlobalAlphaFactoriSUN = _link_function('glGlobalAlphaFactoriSUN', None, [GLint], 'SUN_global_alpha')

# GL/glext.h:5086
glGlobalAlphaFactorfSUN = _link_function('glGlobalAlphaFactorfSUN', None, [GLfloat], 'SUN_global_alpha')

# GL/glext.h:5087
glGlobalAlphaFactordSUN = _link_function('glGlobalAlphaFactordSUN', None, [GLdouble], 'SUN_global_alpha')

# GL/glext.h:5088
glGlobalAlphaFactorubSUN = _link_function('glGlobalAlphaFactorubSUN', None, [GLubyte], 'SUN_global_alpha')

# GL/glext.h:5089
glGlobalAlphaFactorusSUN = _link_function('glGlobalAlphaFactorusSUN', None, [GLushort], 'SUN_global_alpha')

# GL/glext.h:5090
glGlobalAlphaFactoruiSUN = _link_function('glGlobalAlphaFactoruiSUN', None, [GLuint], 'SUN_global_alpha')

PFNGLGLOBALALPHAFACTORBSUNPROC = CFUNCTYPE(None, GLbyte) 	# GL/glext.h:5092
PFNGLGLOBALALPHAFACTORSSUNPROC = CFUNCTYPE(None, GLshort) 	# GL/glext.h:5093
PFNGLGLOBALALPHAFACTORISUNPROC = CFUNCTYPE(None, GLint) 	# GL/glext.h:5094
PFNGLGLOBALALPHAFACTORFSUNPROC = CFUNCTYPE(None, GLfloat) 	# GL/glext.h:5095
PFNGLGLOBALALPHAFACTORDSUNPROC = CFUNCTYPE(None, GLdouble) 	# GL/glext.h:5096
PFNGLGLOBALALPHAFACTORUBSUNPROC = CFUNCTYPE(None, GLubyte) 	# GL/glext.h:5097
PFNGLGLOBALALPHAFACTORUSSUNPROC = CFUNCTYPE(None, GLushort) 	# GL/glext.h:5098
PFNGLGLOBALALPHAFACTORUISUNPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:5099
# SUN_triangle_list (GL/glext.h:5102)
GL_SUN_triangle_list = 1 	# GL/glext.h:5103
# GL/glext.h:5105
glReplacementCodeuiSUN = _link_function('glReplacementCodeuiSUN', None, [GLuint], 'SUN_triangle_list')

# GL/glext.h:5106
glReplacementCodeusSUN = _link_function('glReplacementCodeusSUN', None, [GLushort], 'SUN_triangle_list')

# GL/glext.h:5107
glReplacementCodeubSUN = _link_function('glReplacementCodeubSUN', None, [GLubyte], 'SUN_triangle_list')

# GL/glext.h:5108
glReplacementCodeuivSUN = _link_function('glReplacementCodeuivSUN', None, [POINTER(GLuint)], 'SUN_triangle_list')

# GL/glext.h:5109
glReplacementCodeusvSUN = _link_function('glReplacementCodeusvSUN', None, [POINTER(GLushort)], 'SUN_triangle_list')

# GL/glext.h:5110
glReplacementCodeubvSUN = _link_function('glReplacementCodeubvSUN', None, [POINTER(GLubyte)], 'SUN_triangle_list')

# GL/glext.h:5111
glReplacementCodePointerSUN = _link_function('glReplacementCodePointerSUN', None, [GLenum, GLsizei, POINTER(POINTER(GLvoid))], 'SUN_triangle_list')

PFNGLREPLACEMENTCODEUISUNPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:5113
PFNGLREPLACEMENTCODEUSSUNPROC = CFUNCTYPE(None, GLushort) 	# GL/glext.h:5114
PFNGLREPLACEMENTCODEUBSUNPROC = CFUNCTYPE(None, GLubyte) 	# GL/glext.h:5115
PFNGLREPLACEMENTCODEUIVSUNPROC = CFUNCTYPE(None, POINTER(GLuint)) 	# GL/glext.h:5116
PFNGLREPLACEMENTCODEUSVSUNPROC = CFUNCTYPE(None, POINTER(GLushort)) 	# GL/glext.h:5117
PFNGLREPLACEMENTCODEUBVSUNPROC = CFUNCTYPE(None, POINTER(GLubyte)) 	# GL/glext.h:5118
PFNGLREPLACEMENTCODEPOINTERSUNPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(POINTER(GLvoid))) 	# GL/glext.h:5119
# SUN_vertex (GL/glext.h:5122)
GL_SUN_vertex = 1 	# GL/glext.h:5123
# GL/glext.h:5125
glColor4ubVertex2fSUN = _link_function('glColor4ubVertex2fSUN', None, [GLubyte, GLubyte, GLubyte, GLubyte, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5126
glColor4ubVertex2fvSUN = _link_function('glColor4ubVertex2fvSUN', None, [POINTER(GLubyte), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5127
glColor4ubVertex3fSUN = _link_function('glColor4ubVertex3fSUN', None, [GLubyte, GLubyte, GLubyte, GLubyte, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5128
glColor4ubVertex3fvSUN = _link_function('glColor4ubVertex3fvSUN', None, [POINTER(GLubyte), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5129
glColor3fVertex3fSUN = _link_function('glColor3fVertex3fSUN', None, [GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5130
glColor3fVertex3fvSUN = _link_function('glColor3fVertex3fvSUN', None, [POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5131
glNormal3fVertex3fSUN = _link_function('glNormal3fVertex3fSUN', None, [GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5132
glNormal3fVertex3fvSUN = _link_function('glNormal3fVertex3fvSUN', None, [POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5133
glColor4fNormal3fVertex3fSUN = _link_function('glColor4fNormal3fVertex3fSUN', None, [GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5134
glColor4fNormal3fVertex3fvSUN = _link_function('glColor4fNormal3fVertex3fvSUN', None, [POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5135
glTexCoord2fVertex3fSUN = _link_function('glTexCoord2fVertex3fSUN', None, [GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5136
glTexCoord2fVertex3fvSUN = _link_function('glTexCoord2fVertex3fvSUN', None, [POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5137
glTexCoord4fVertex4fSUN = _link_function('glTexCoord4fVertex4fSUN', None, [GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5138
glTexCoord4fVertex4fvSUN = _link_function('glTexCoord4fVertex4fvSUN', None, [POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5139
glTexCoord2fColor4ubVertex3fSUN = _link_function('glTexCoord2fColor4ubVertex3fSUN', None, [GLfloat, GLfloat, GLubyte, GLubyte, GLubyte, GLubyte, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5140
glTexCoord2fColor4ubVertex3fvSUN = _link_function('glTexCoord2fColor4ubVertex3fvSUN', None, [POINTER(GLfloat), POINTER(GLubyte), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5141
glTexCoord2fColor3fVertex3fSUN = _link_function('glTexCoord2fColor3fVertex3fSUN', None, [GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5142
glTexCoord2fColor3fVertex3fvSUN = _link_function('glTexCoord2fColor3fVertex3fvSUN', None, [POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5143
glTexCoord2fNormal3fVertex3fSUN = _link_function('glTexCoord2fNormal3fVertex3fSUN', None, [GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5144
glTexCoord2fNormal3fVertex3fvSUN = _link_function('glTexCoord2fNormal3fVertex3fvSUN', None, [POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5145
glTexCoord2fColor4fNormal3fVertex3fSUN = _link_function('glTexCoord2fColor4fNormal3fVertex3fSUN', None, [GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5146
glTexCoord2fColor4fNormal3fVertex3fvSUN = _link_function('glTexCoord2fColor4fNormal3fVertex3fvSUN', None, [POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5147
glTexCoord4fColor4fNormal3fVertex4fSUN = _link_function('glTexCoord4fColor4fNormal3fVertex4fSUN', None, [GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5148
glTexCoord4fColor4fNormal3fVertex4fvSUN = _link_function('glTexCoord4fColor4fNormal3fVertex4fvSUN', None, [POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5149
glReplacementCodeuiVertex3fSUN = _link_function('glReplacementCodeuiVertex3fSUN', None, [GLuint, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5150
glReplacementCodeuiVertex3fvSUN = _link_function('glReplacementCodeuiVertex3fvSUN', None, [POINTER(GLuint), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5151
glReplacementCodeuiColor4ubVertex3fSUN = _link_function('glReplacementCodeuiColor4ubVertex3fSUN', None, [GLuint, GLubyte, GLubyte, GLubyte, GLubyte, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5152
glReplacementCodeuiColor4ubVertex3fvSUN = _link_function('glReplacementCodeuiColor4ubVertex3fvSUN', None, [POINTER(GLuint), POINTER(GLubyte), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5153
glReplacementCodeuiColor3fVertex3fSUN = _link_function('glReplacementCodeuiColor3fVertex3fSUN', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5154
glReplacementCodeuiColor3fVertex3fvSUN = _link_function('glReplacementCodeuiColor3fVertex3fvSUN', None, [POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5155
glReplacementCodeuiNormal3fVertex3fSUN = _link_function('glReplacementCodeuiNormal3fVertex3fSUN', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5156
glReplacementCodeuiNormal3fVertex3fvSUN = _link_function('glReplacementCodeuiNormal3fVertex3fvSUN', None, [POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5157
glReplacementCodeuiColor4fNormal3fVertex3fSUN = _link_function('glReplacementCodeuiColor4fNormal3fVertex3fSUN', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5158
glReplacementCodeuiColor4fNormal3fVertex3fvSUN = _link_function('glReplacementCodeuiColor4fNormal3fVertex3fvSUN', None, [POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5159
glReplacementCodeuiTexCoord2fVertex3fSUN = _link_function('glReplacementCodeuiTexCoord2fVertex3fSUN', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5160
glReplacementCodeuiTexCoord2fVertex3fvSUN = _link_function('glReplacementCodeuiTexCoord2fVertex3fvSUN', None, [POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5161
glReplacementCodeuiTexCoord2fNormal3fVertex3fSUN = _link_function('glReplacementCodeuiTexCoord2fNormal3fVertex3fSUN', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5162
glReplacementCodeuiTexCoord2fNormal3fVertex3fvSUN = _link_function('glReplacementCodeuiTexCoord2fNormal3fVertex3fvSUN', None, [POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5163
glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fSUN = _link_function('glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fSUN', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5164
glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fvSUN = _link_function('glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fvSUN', None, [POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

PFNGLCOLOR4UBVERTEX2FSUNPROC = CFUNCTYPE(None, GLubyte, GLubyte, GLubyte, GLubyte, GLfloat, GLfloat) 	# GL/glext.h:5166
PFNGLCOLOR4UBVERTEX2FVSUNPROC = CFUNCTYPE(None, POINTER(GLubyte), POINTER(GLfloat)) 	# GL/glext.h:5167
PFNGLCOLOR4UBVERTEX3FSUNPROC = CFUNCTYPE(None, GLubyte, GLubyte, GLubyte, GLubyte, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5168
PFNGLCOLOR4UBVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLubyte), POINTER(GLfloat)) 	# GL/glext.h:5169
PFNGLCOLOR3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5170
PFNGLCOLOR3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:5171
PFNGLNORMAL3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5172
PFNGLNORMAL3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:5173
PFNGLCOLOR4FNORMAL3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5174
PFNGLCOLOR4FNORMAL3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:5175
PFNGLTEXCOORD2FVERTEX3FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5176
PFNGLTEXCOORD2FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:5177
PFNGLTEXCOORD4FVERTEX4FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5178
PFNGLTEXCOORD4FVERTEX4FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:5179
PFNGLTEXCOORD2FCOLOR4UBVERTEX3FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLubyte, GLubyte, GLubyte, GLubyte, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5180
PFNGLTEXCOORD2FCOLOR4UBVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLubyte), POINTER(GLfloat)) 	# GL/glext.h:5181
PFNGLTEXCOORD2FCOLOR3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5182
PFNGLTEXCOORD2FCOLOR3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:5183
PFNGLTEXCOORD2FNORMAL3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5184
PFNGLTEXCOORD2FNORMAL3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:5185
PFNGLTEXCOORD2FCOLOR4FNORMAL3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5186
PFNGLTEXCOORD2FCOLOR4FNORMAL3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:5187
PFNGLTEXCOORD4FCOLOR4FNORMAL3FVERTEX4FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5188
PFNGLTEXCOORD4FCOLOR4FNORMAL3FVERTEX4FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:5189
PFNGLREPLACEMENTCODEUIVERTEX3FSUNPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5190
PFNGLREPLACEMENTCODEUIVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLuint), POINTER(GLfloat)) 	# GL/glext.h:5191
PFNGLREPLACEMENTCODEUICOLOR4UBVERTEX3FSUNPROC = CFUNCTYPE(None, GLuint, GLubyte, GLubyte, GLubyte, GLubyte, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5192
PFNGLREPLACEMENTCODEUICOLOR4UBVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLuint), POINTER(GLubyte), POINTER(GLfloat)) 	# GL/glext.h:5193
PFNGLREPLACEMENTCODEUICOLOR3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5194
PFNGLREPLACEMENTCODEUICOLOR3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:5195
PFNGLREPLACEMENTCODEUINORMAL3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5196
PFNGLREPLACEMENTCODEUINORMAL3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:5197
PFNGLREPLACEMENTCODEUICOLOR4FNORMAL3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5198
PFNGLREPLACEMENTCODEUICOLOR4FNORMAL3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:5199
PFNGLREPLACEMENTCODEUITEXCOORD2FVERTEX3FSUNPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5200
PFNGLREPLACEMENTCODEUITEXCOORD2FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:5201
PFNGLREPLACEMENTCODEUITEXCOORD2FNORMAL3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5202
PFNGLREPLACEMENTCODEUITEXCOORD2FNORMAL3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:5203
PFNGLREPLACEMENTCODEUITEXCOORD2FCOLOR4FNORMAL3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5204
PFNGLREPLACEMENTCODEUITEXCOORD2FCOLOR4FNORMAL3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:5205
# EXT_blend_func_separate (GL/glext.h:5208)
GL_EXT_blend_func_separate = 1 	# GL/glext.h:5209
# GL/glext.h:5211
glBlendFuncSeparateEXT = _link_function('glBlendFuncSeparateEXT', None, [GLenum, GLenum, GLenum, GLenum], 'EXT_blend_func_separate')

PFNGLBLENDFUNCSEPARATEEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:5213
# INGR_blend_func_separate (GL/glext.h:5216)
GL_INGR_blend_func_separate = 1 	# GL/glext.h:5217
# GL/glext.h:5219
glBlendFuncSeparateINGR = _link_function('glBlendFuncSeparateINGR', None, [GLenum, GLenum, GLenum, GLenum], 'INGR_blend_func_separate')

PFNGLBLENDFUNCSEPARATEINGRPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:5221
# INGR_color_clamp (GL/glext.h:5224)
GL_INGR_color_clamp = 1 	# GL/glext.h:5225
# INGR_interlace_read (GL/glext.h:5228)
GL_INGR_interlace_read = 1 	# GL/glext.h:5229
# EXT_stencil_wrap (GL/glext.h:5232)
GL_EXT_stencil_wrap = 1 	# GL/glext.h:5233
# EXT_422_pixels (GL/glext.h:5236)
GL_EXT_422_pixels = 1 	# GL/glext.h:5237
# NV_texgen_reflection (GL/glext.h:5240)
GL_NV_texgen_reflection = 1 	# GL/glext.h:5241
# SUN_convolution_border_modes (GL/glext.h:5244)
GL_SUN_convolution_border_modes = 1 	# GL/glext.h:5245
# EXT_texture_env_add (GL/glext.h:5248)
GL_EXT_texture_env_add = 1 	# GL/glext.h:5249
# EXT_texture_lod_bias (GL/glext.h:5252)
GL_EXT_texture_lod_bias = 1 	# GL/glext.h:5253
# EXT_texture_filter_anisotropic (GL/glext.h:5256)
GL_EXT_texture_filter_anisotropic = 1 	# GL/glext.h:5257
# EXT_vertex_weighting (GL/glext.h:5260)
GL_EXT_vertex_weighting = 1 	# GL/glext.h:5261
# GL/glext.h:5263
glVertexWeightfEXT = _link_function('glVertexWeightfEXT', None, [GLfloat], 'EXT_vertex_weighting')

# GL/glext.h:5264
glVertexWeightfvEXT = _link_function('glVertexWeightfvEXT', None, [POINTER(GLfloat)], 'EXT_vertex_weighting')

# GL/glext.h:5265
glVertexWeightPointerEXT = _link_function('glVertexWeightPointerEXT', None, [GLsizei, GLenum, GLsizei, POINTER(GLvoid)], 'EXT_vertex_weighting')

PFNGLVERTEXWEIGHTFEXTPROC = CFUNCTYPE(None, GLfloat) 	# GL/glext.h:5267
PFNGLVERTEXWEIGHTFVEXTPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:5268
PFNGLVERTEXWEIGHTPOINTEREXTPROC = CFUNCTYPE(None, GLsizei, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:5269
# NV_light_max_exponent (GL/glext.h:5272)
GL_NV_light_max_exponent = 1 	# GL/glext.h:5273
# NV_vertex_array_range (GL/glext.h:5276)
GL_NV_vertex_array_range = 1 	# GL/glext.h:5277
# GL/glext.h:5279
glFlushVertexArrayRangeNV = _link_function('glFlushVertexArrayRangeNV', None, [], 'NV_vertex_array_range')

# GL/glext.h:5280
glVertexArrayRangeNV = _link_function('glVertexArrayRangeNV', None, [GLsizei, POINTER(GLvoid)], 'NV_vertex_array_range')

PFNGLFLUSHVERTEXARRAYRANGENVPROC = CFUNCTYPE(None) 	# GL/glext.h:5282
PFNGLVERTEXARRAYRANGENVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:5283
# NV_register_combiners (GL/glext.h:5286)
GL_NV_register_combiners = 1 	# GL/glext.h:5287
# GL/glext.h:5289
glCombinerParameterfvNV = _link_function('glCombinerParameterfvNV', None, [GLenum, POINTER(GLfloat)], 'NV_register_combiners')

# GL/glext.h:5290
glCombinerParameterfNV = _link_function('glCombinerParameterfNV', None, [GLenum, GLfloat], 'NV_register_combiners')

# GL/glext.h:5291
glCombinerParameterivNV = _link_function('glCombinerParameterivNV', None, [GLenum, POINTER(GLint)], 'NV_register_combiners')

# GL/glext.h:5292
glCombinerParameteriNV = _link_function('glCombinerParameteriNV', None, [GLenum, GLint], 'NV_register_combiners')

# GL/glext.h:5293
glCombinerInputNV = _link_function('glCombinerInputNV', None, [GLenum, GLenum, GLenum, GLenum, GLenum, GLenum], 'NV_register_combiners')

# GL/glext.h:5294
glCombinerOutputNV = _link_function('glCombinerOutputNV', None, [GLenum, GLenum, GLenum, GLenum, GLenum, GLenum, GLenum, GLboolean, GLboolean, GLboolean], 'NV_register_combiners')

# GL/glext.h:5295
glFinalCombinerInputNV = _link_function('glFinalCombinerInputNV', None, [GLenum, GLenum, GLenum, GLenum], 'NV_register_combiners')

# GL/glext.h:5296
glGetCombinerInputParameterfvNV = _link_function('glGetCombinerInputParameterfvNV', None, [GLenum, GLenum, GLenum, GLenum, POINTER(GLfloat)], 'NV_register_combiners')

# GL/glext.h:5297
glGetCombinerInputParameterivNV = _link_function('glGetCombinerInputParameterivNV', None, [GLenum, GLenum, GLenum, GLenum, POINTER(GLint)], 'NV_register_combiners')

# GL/glext.h:5298
glGetCombinerOutputParameterfvNV = _link_function('glGetCombinerOutputParameterfvNV', None, [GLenum, GLenum, GLenum, POINTER(GLfloat)], 'NV_register_combiners')

# GL/glext.h:5299
glGetCombinerOutputParameterivNV = _link_function('glGetCombinerOutputParameterivNV', None, [GLenum, GLenum, GLenum, POINTER(GLint)], 'NV_register_combiners')

# GL/glext.h:5300
glGetFinalCombinerInputParameterfvNV = _link_function('glGetFinalCombinerInputParameterfvNV', None, [GLenum, GLenum, POINTER(GLfloat)], 'NV_register_combiners')

# GL/glext.h:5301
glGetFinalCombinerInputParameterivNV = _link_function('glGetFinalCombinerInputParameterivNV', None, [GLenum, GLenum, POINTER(GLint)], 'NV_register_combiners')

PFNGLCOMBINERPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5303
PFNGLCOMBINERPARAMETERFNVPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:5304
PFNGLCOMBINERPARAMETERIVNVPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:5305
PFNGLCOMBINERPARAMETERINVPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:5306
PFNGLCOMBINERINPUTNVPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:5307
PFNGLCOMBINEROUTPUTNVPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum, GLenum, GLenum, GLenum, GLboolean, GLboolean, GLboolean) 	# GL/glext.h:5308
PFNGLFINALCOMBINERINPUTNVPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:5309
PFNGLGETCOMBINERINPUTPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5310
PFNGLGETCOMBINERINPUTPARAMETERIVNVPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5311
PFNGLGETCOMBINEROUTPUTPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5312
PFNGLGETCOMBINEROUTPUTPARAMETERIVNVPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5313
PFNGLGETFINALCOMBINERINPUTPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5314
PFNGLGETFINALCOMBINERINPUTPARAMETERIVNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5315
# NV_fog_distance (GL/glext.h:5318)
GL_NV_fog_distance = 1 	# GL/glext.h:5319
# NV_texgen_emboss (GL/glext.h:5322)
GL_NV_texgen_emboss = 1 	# GL/glext.h:5323
# NV_blend_square (GL/glext.h:5326)
GL_NV_blend_square = 1 	# GL/glext.h:5327
# NV_texture_env_combine4 (GL/glext.h:5330)
GL_NV_texture_env_combine4 = 1 	# GL/glext.h:5331
# MESA_resize_buffers (GL/glext.h:5334)
GL_MESA_resize_buffers = 1 	# GL/glext.h:5335
# GL/glext.h:5337
glResizeBuffersMESA = _link_function('glResizeBuffersMESA', None, [], 'MESA_resize_buffers')

PFNGLRESIZEBUFFERSMESAPROC = CFUNCTYPE(None) 	# GL/glext.h:5339
# MESA_window_pos (GL/glext.h:5342)
GL_MESA_window_pos = 1 	# GL/glext.h:5343
# GL/glext.h:5345
glWindowPos2dMESA = _link_function('glWindowPos2dMESA', None, [GLdouble, GLdouble], 'MESA_window_pos')

# GL/glext.h:5346
glWindowPos2dvMESA = _link_function('glWindowPos2dvMESA', None, [POINTER(GLdouble)], 'MESA_window_pos')

# GL/glext.h:5347
glWindowPos2fMESA = _link_function('glWindowPos2fMESA', None, [GLfloat, GLfloat], 'MESA_window_pos')

# GL/glext.h:5348
glWindowPos2fvMESA = _link_function('glWindowPos2fvMESA', None, [POINTER(GLfloat)], 'MESA_window_pos')

# GL/glext.h:5349
glWindowPos2iMESA = _link_function('glWindowPos2iMESA', None, [GLint, GLint], 'MESA_window_pos')

# GL/glext.h:5350
glWindowPos2ivMESA = _link_function('glWindowPos2ivMESA', None, [POINTER(GLint)], 'MESA_window_pos')

# GL/glext.h:5351
glWindowPos2sMESA = _link_function('glWindowPos2sMESA', None, [GLshort, GLshort], 'MESA_window_pos')

# GL/glext.h:5352
glWindowPos2svMESA = _link_function('glWindowPos2svMESA', None, [POINTER(GLshort)], 'MESA_window_pos')

# GL/glext.h:5353
glWindowPos3dMESA = _link_function('glWindowPos3dMESA', None, [GLdouble, GLdouble, GLdouble], 'MESA_window_pos')

# GL/glext.h:5354
glWindowPos3dvMESA = _link_function('glWindowPos3dvMESA', None, [POINTER(GLdouble)], 'MESA_window_pos')

# GL/glext.h:5355
glWindowPos3fMESA = _link_function('glWindowPos3fMESA', None, [GLfloat, GLfloat, GLfloat], 'MESA_window_pos')

# GL/glext.h:5356
glWindowPos3fvMESA = _link_function('glWindowPos3fvMESA', None, [POINTER(GLfloat)], 'MESA_window_pos')

# GL/glext.h:5357
glWindowPos3iMESA = _link_function('glWindowPos3iMESA', None, [GLint, GLint, GLint], 'MESA_window_pos')

# GL/glext.h:5358
glWindowPos3ivMESA = _link_function('glWindowPos3ivMESA', None, [POINTER(GLint)], 'MESA_window_pos')

# GL/glext.h:5359
glWindowPos3sMESA = _link_function('glWindowPos3sMESA', None, [GLshort, GLshort, GLshort], 'MESA_window_pos')

# GL/glext.h:5360
glWindowPos3svMESA = _link_function('glWindowPos3svMESA', None, [POINTER(GLshort)], 'MESA_window_pos')

# GL/glext.h:5361
glWindowPos4dMESA = _link_function('glWindowPos4dMESA', None, [GLdouble, GLdouble, GLdouble, GLdouble], 'MESA_window_pos')

# GL/glext.h:5362
glWindowPos4dvMESA = _link_function('glWindowPos4dvMESA', None, [POINTER(GLdouble)], 'MESA_window_pos')

# GL/glext.h:5363
glWindowPos4fMESA = _link_function('glWindowPos4fMESA', None, [GLfloat, GLfloat, GLfloat, GLfloat], 'MESA_window_pos')

# GL/glext.h:5364
glWindowPos4fvMESA = _link_function('glWindowPos4fvMESA', None, [POINTER(GLfloat)], 'MESA_window_pos')

# GL/glext.h:5365
glWindowPos4iMESA = _link_function('glWindowPos4iMESA', None, [GLint, GLint, GLint, GLint], 'MESA_window_pos')

# GL/glext.h:5366
glWindowPos4ivMESA = _link_function('glWindowPos4ivMESA', None, [POINTER(GLint)], 'MESA_window_pos')

# GL/glext.h:5367
glWindowPos4sMESA = _link_function('glWindowPos4sMESA', None, [GLshort, GLshort, GLshort, GLshort], 'MESA_window_pos')

# GL/glext.h:5368
glWindowPos4svMESA = _link_function('glWindowPos4svMESA', None, [POINTER(GLshort)], 'MESA_window_pos')

PFNGLWINDOWPOS2DMESAPROC = CFUNCTYPE(None, GLdouble, GLdouble) 	# GL/glext.h:5370
PFNGLWINDOWPOS2DVMESAPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:5371
PFNGLWINDOWPOS2FMESAPROC = CFUNCTYPE(None, GLfloat, GLfloat) 	# GL/glext.h:5372
PFNGLWINDOWPOS2FVMESAPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:5373
PFNGLWINDOWPOS2IMESAPROC = CFUNCTYPE(None, GLint, GLint) 	# GL/glext.h:5374
PFNGLWINDOWPOS2IVMESAPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:5375
PFNGLWINDOWPOS2SMESAPROC = CFUNCTYPE(None, GLshort, GLshort) 	# GL/glext.h:5376
PFNGLWINDOWPOS2SVMESAPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:5377
PFNGLWINDOWPOS3DMESAPROC = CFUNCTYPE(None, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:5378
PFNGLWINDOWPOS3DVMESAPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:5379
PFNGLWINDOWPOS3FMESAPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5380
PFNGLWINDOWPOS3FVMESAPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:5381
PFNGLWINDOWPOS3IMESAPROC = CFUNCTYPE(None, GLint, GLint, GLint) 	# GL/glext.h:5382
PFNGLWINDOWPOS3IVMESAPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:5383
PFNGLWINDOWPOS3SMESAPROC = CFUNCTYPE(None, GLshort, GLshort, GLshort) 	# GL/glext.h:5384
PFNGLWINDOWPOS3SVMESAPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:5385
PFNGLWINDOWPOS4DMESAPROC = CFUNCTYPE(None, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:5386
PFNGLWINDOWPOS4DVMESAPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:5387
PFNGLWINDOWPOS4FMESAPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5388
PFNGLWINDOWPOS4FVMESAPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:5389
PFNGLWINDOWPOS4IMESAPROC = CFUNCTYPE(None, GLint, GLint, GLint, GLint) 	# GL/glext.h:5390
PFNGLWINDOWPOS4IVMESAPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:5391
PFNGLWINDOWPOS4SMESAPROC = CFUNCTYPE(None, GLshort, GLshort, GLshort, GLshort) 	# GL/glext.h:5392
PFNGLWINDOWPOS4SVMESAPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:5393
# IBM_cull_vertex (GL/glext.h:5396)
GL_IBM_cull_vertex = 1 	# GL/glext.h:5397
# IBM_multimode_draw_arrays (GL/glext.h:5400)
GL_IBM_multimode_draw_arrays = 1 	# GL/glext.h:5401
# GL/glext.h:5403
glMultiModeDrawArraysIBM = _link_function('glMultiModeDrawArraysIBM', None, [POINTER(GLenum), POINTER(GLint), POINTER(GLsizei), GLsizei, GLint], 'IBM_multimode_draw_arrays')

# GL/glext.h:5404
glMultiModeDrawElementsIBM = _link_function('glMultiModeDrawElementsIBM', None, [POINTER(GLenum), POINTER(GLsizei), GLenum, POINTER(POINTER(GLvoid)), GLsizei, GLint], 'IBM_multimode_draw_arrays')

PFNGLMULTIMODEDRAWARRAYSIBMPROC = CFUNCTYPE(None, POINTER(GLenum), POINTER(GLint), POINTER(GLsizei), GLsizei, GLint) 	# GL/glext.h:5406
PFNGLMULTIMODEDRAWELEMENTSIBMPROC = CFUNCTYPE(None, POINTER(GLenum), POINTER(GLsizei), GLenum, POINTER(POINTER(GLvoid)), GLsizei, GLint) 	# GL/glext.h:5407
# IBM_vertex_array_lists (GL/glext.h:5410)
GL_IBM_vertex_array_lists = 1 	# GL/glext.h:5411
# GL/glext.h:5413
glColorPointerListIBM = _link_function('glColorPointerListIBM', None, [GLint, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint], 'IBM_vertex_array_lists')

# GL/glext.h:5414
glSecondaryColorPointerListIBM = _link_function('glSecondaryColorPointerListIBM', None, [GLint, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint], 'IBM_vertex_array_lists')

# GL/glext.h:5415
glEdgeFlagPointerListIBM = _link_function('glEdgeFlagPointerListIBM', None, [GLint, POINTER(POINTER(GLboolean)), GLint], 'IBM_vertex_array_lists')

# GL/glext.h:5416
glFogCoordPointerListIBM = _link_function('glFogCoordPointerListIBM', None, [GLenum, GLint, POINTER(POINTER(GLvoid)), GLint], 'IBM_vertex_array_lists')

# GL/glext.h:5417
glIndexPointerListIBM = _link_function('glIndexPointerListIBM', None, [GLenum, GLint, POINTER(POINTER(GLvoid)), GLint], 'IBM_vertex_array_lists')

# GL/glext.h:5418
glNormalPointerListIBM = _link_function('glNormalPointerListIBM', None, [GLenum, GLint, POINTER(POINTER(GLvoid)), GLint], 'IBM_vertex_array_lists')

# GL/glext.h:5419
glTexCoordPointerListIBM = _link_function('glTexCoordPointerListIBM', None, [GLint, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint], 'IBM_vertex_array_lists')

# GL/glext.h:5420
glVertexPointerListIBM = _link_function('glVertexPointerListIBM', None, [GLint, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint], 'IBM_vertex_array_lists')

PFNGLCOLORPOINTERLISTIBMPROC = CFUNCTYPE(None, GLint, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint) 	# GL/glext.h:5422
PFNGLSECONDARYCOLORPOINTERLISTIBMPROC = CFUNCTYPE(None, GLint, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint) 	# GL/glext.h:5423
PFNGLEDGEFLAGPOINTERLISTIBMPROC = CFUNCTYPE(None, GLint, POINTER(POINTER(GLboolean)), GLint) 	# GL/glext.h:5424
PFNGLFOGCOORDPOINTERLISTIBMPROC = CFUNCTYPE(None, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint) 	# GL/glext.h:5425
PFNGLINDEXPOINTERLISTIBMPROC = CFUNCTYPE(None, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint) 	# GL/glext.h:5426
PFNGLNORMALPOINTERLISTIBMPROC = CFUNCTYPE(None, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint) 	# GL/glext.h:5427
PFNGLTEXCOORDPOINTERLISTIBMPROC = CFUNCTYPE(None, GLint, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint) 	# GL/glext.h:5428
PFNGLVERTEXPOINTERLISTIBMPROC = CFUNCTYPE(None, GLint, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint) 	# GL/glext.h:5429
# SGIX_subsample (GL/glext.h:5432)
GL_SGIX_subsample = 1 	# GL/glext.h:5433
# SGIX_ycrcba (GL/glext.h:5436)
GL_SGIX_ycrcba = 1 	# GL/glext.h:5437
# SGIX_ycrcb_subsample (GL/glext.h:5440)
GL_SGIX_ycrcb_subsample = 1 	# GL/glext.h:5441
# SGIX_depth_pass_instrument (GL/glext.h:5444)
GL_SGIX_depth_pass_instrument = 1 	# GL/glext.h:5445
# 3DFX_texture_compression_FXT1 (GL/glext.h:5448)
GL_3DFX_texture_compression_FXT1 = 1 	# GL/glext.h:5449
# 3DFX_multisample (GL/glext.h:5452)
GL_3DFX_multisample = 1 	# GL/glext.h:5453
# 3DFX_tbuffer (GL/glext.h:5456)
GL_3DFX_tbuffer = 1 	# GL/glext.h:5457
# GL/glext.h:5459
glTbufferMask3DFX = _link_function('glTbufferMask3DFX', None, [GLuint], '3DFX_tbuffer')

PFNGLTBUFFERMASK3DFXPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:5461
# EXT_multisample (GL/glext.h:5464)
GL_EXT_multisample = 1 	# GL/glext.h:5465
# GL/glext.h:5467
glSampleMaskEXT = _link_function('glSampleMaskEXT', None, [GLclampf, GLboolean], 'EXT_multisample')

# GL/glext.h:5468
glSamplePatternEXT = _link_function('glSamplePatternEXT', None, [GLenum], 'EXT_multisample')

PFNGLSAMPLEMASKEXTPROC = CFUNCTYPE(None, GLclampf, GLboolean) 	# GL/glext.h:5470
PFNGLSAMPLEPATTERNEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:5471
# SGIX_vertex_preclip (GL/glext.h:5474)
GL_SGIX_vertex_preclip = 1 	# GL/glext.h:5475
# SGIX_convolution_accuracy (GL/glext.h:5478)
GL_SGIX_convolution_accuracy = 1 	# GL/glext.h:5479
# SGIX_resample (GL/glext.h:5482)
GL_SGIX_resample = 1 	# GL/glext.h:5483
# SGIS_point_line_texgen (GL/glext.h:5486)
GL_SGIS_point_line_texgen = 1 	# GL/glext.h:5487
# SGIS_texture_color_mask (GL/glext.h:5490)
GL_SGIS_texture_color_mask = 1 	# GL/glext.h:5491
# GL/glext.h:5493
glTextureColorMaskSGIS = _link_function('glTextureColorMaskSGIS', None, [GLboolean, GLboolean, GLboolean, GLboolean], 'SGIS_texture_color_mask')

PFNGLTEXTURECOLORMASKSGISPROC = CFUNCTYPE(None, GLboolean, GLboolean, GLboolean, GLboolean) 	# GL/glext.h:5495
# SGIX_igloo_interface (GL/glext.h:5498)
GL_SGIX_igloo_interface = 1 	# GL/glext.h:5499
# GL/glext.h:5501
glIglooInterfaceSGIX = _link_function('glIglooInterfaceSGIX', None, [GLenum, POINTER(GLvoid)], 'SGIX_igloo_interface')

PFNGLIGLOOINTERFACESGIXPROC = CFUNCTYPE(None, GLenum, POINTER(GLvoid)) 	# GL/glext.h:5503
# EXT_texture_env_dot3 (GL/glext.h:5506)
GL_EXT_texture_env_dot3 = 1 	# GL/glext.h:5507
# ATI_texture_mirror_once (GL/glext.h:5510)
GL_ATI_texture_mirror_once = 1 	# GL/glext.h:5511
# NV_fence (GL/glext.h:5514)
GL_NV_fence = 1 	# GL/glext.h:5515
# GL/glext.h:5517
glDeleteFencesNV = _link_function('glDeleteFencesNV', None, [GLsizei, POINTER(GLuint)], 'NV_fence')

# GL/glext.h:5518
glGenFencesNV = _link_function('glGenFencesNV', None, [GLsizei, POINTER(GLuint)], 'NV_fence')

# GL/glext.h:5519
glIsFenceNV = _link_function('glIsFenceNV', GLboolean, [GLuint], 'NV_fence')

# GL/glext.h:5520
glTestFenceNV = _link_function('glTestFenceNV', GLboolean, [GLuint], 'NV_fence')

# GL/glext.h:5521
glGetFenceivNV = _link_function('glGetFenceivNV', None, [GLuint, GLenum, POINTER(GLint)], 'NV_fence')

# GL/glext.h:5522
glFinishFenceNV = _link_function('glFinishFenceNV', None, [GLuint], 'NV_fence')

# GL/glext.h:5523
glSetFenceNV = _link_function('glSetFenceNV', None, [GLuint, GLenum], 'NV_fence')

PFNGLDELETEFENCESNVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:5525
PFNGLGENFENCESNVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:5526
PFNGLISFENCENVPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:5527
PFNGLTESTFENCENVPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:5528
PFNGLGETFENCEIVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:5529
PFNGLFINISHFENCENVPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:5530
PFNGLSETFENCENVPROC = CFUNCTYPE(None, GLuint, GLenum) 	# GL/glext.h:5531
# NV_evaluators (GL/glext.h:5534)
GL_NV_evaluators = 1 	# GL/glext.h:5535
# GL/glext.h:5537
glMapControlPointsNV = _link_function('glMapControlPointsNV', None, [GLenum, GLuint, GLenum, GLsizei, GLsizei, GLint, GLint, GLboolean, POINTER(GLvoid)], 'NV_evaluators')

# GL/glext.h:5538
glMapParameterivNV = _link_function('glMapParameterivNV', None, [GLenum, GLenum, POINTER(GLint)], 'NV_evaluators')

# GL/glext.h:5539
glMapParameterfvNV = _link_function('glMapParameterfvNV', None, [GLenum, GLenum, POINTER(GLfloat)], 'NV_evaluators')

# GL/glext.h:5540
glGetMapControlPointsNV = _link_function('glGetMapControlPointsNV', None, [GLenum, GLuint, GLenum, GLsizei, GLsizei, GLboolean, POINTER(GLvoid)], 'NV_evaluators')

# GL/glext.h:5541
glGetMapParameterivNV = _link_function('glGetMapParameterivNV', None, [GLenum, GLenum, POINTER(GLint)], 'NV_evaluators')

# GL/glext.h:5542
glGetMapParameterfvNV = _link_function('glGetMapParameterfvNV', None, [GLenum, GLenum, POINTER(GLfloat)], 'NV_evaluators')

# GL/glext.h:5543
glGetMapAttribParameterivNV = _link_function('glGetMapAttribParameterivNV', None, [GLenum, GLuint, GLenum, POINTER(GLint)], 'NV_evaluators')

# GL/glext.h:5544
glGetMapAttribParameterfvNV = _link_function('glGetMapAttribParameterfvNV', None, [GLenum, GLuint, GLenum, POINTER(GLfloat)], 'NV_evaluators')

# GL/glext.h:5545
glEvalMapsNV = _link_function('glEvalMapsNV', None, [GLenum, GLenum], 'NV_evaluators')

PFNGLMAPCONTROLPOINTSNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLenum, GLsizei, GLsizei, GLint, GLint, GLboolean, POINTER(GLvoid)) 	# GL/glext.h:5547
PFNGLMAPPARAMETERIVNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5548
PFNGLMAPPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5549
PFNGLGETMAPCONTROLPOINTSNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLenum, GLsizei, GLsizei, GLboolean, POINTER(GLvoid)) 	# GL/glext.h:5550
PFNGLGETMAPPARAMETERIVNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5551
PFNGLGETMAPPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5552
PFNGLGETMAPATTRIBPARAMETERIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:5553
PFNGLGETMAPATTRIBPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5554
PFNGLEVALMAPSNVPROC = CFUNCTYPE(None, GLenum, GLenum) 	# GL/glext.h:5555
# NV_packed_depth_stencil (GL/glext.h:5558)
GL_NV_packed_depth_stencil = 1 	# GL/glext.h:5559
# NV_register_combiners2 (GL/glext.h:5562)
GL_NV_register_combiners2 = 1 	# GL/glext.h:5563
# GL/glext.h:5565
glCombinerStageParameterfvNV = _link_function('glCombinerStageParameterfvNV', None, [GLenum, GLenum, POINTER(GLfloat)], 'NV_register_combiners2')

# GL/glext.h:5566
glGetCombinerStageParameterfvNV = _link_function('glGetCombinerStageParameterfvNV', None, [GLenum, GLenum, POINTER(GLfloat)], 'NV_register_combiners2')

PFNGLCOMBINERSTAGEPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5568
PFNGLGETCOMBINERSTAGEPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5569
# NV_texture_compression_vtc (GL/glext.h:5572)
GL_NV_texture_compression_vtc = 1 	# GL/glext.h:5573
# NV_texture_rectangle (GL/glext.h:5576)
GL_NV_texture_rectangle = 1 	# GL/glext.h:5577
# NV_texture_shader (GL/glext.h:5580)
GL_NV_texture_shader = 1 	# GL/glext.h:5581
# NV_texture_shader2 (GL/glext.h:5584)
GL_NV_texture_shader2 = 1 	# GL/glext.h:5585
# NV_vertex_array_range2 (GL/glext.h:5588)
GL_NV_vertex_array_range2 = 1 	# GL/glext.h:5589
# NV_vertex_program (GL/glext.h:5592)
GL_NV_vertex_program = 1 	# GL/glext.h:5593
# GL/glext.h:5595
glAreProgramsResidentNV = _link_function('glAreProgramsResidentNV', GLboolean, [GLsizei, POINTER(GLuint), POINTER(GLboolean)], 'NV_vertex_program')

# GL/glext.h:5596
glBindProgramNV = _link_function('glBindProgramNV', None, [GLenum, GLuint], 'NV_vertex_program')

# GL/glext.h:5597
glDeleteProgramsNV = _link_function('glDeleteProgramsNV', None, [GLsizei, POINTER(GLuint)], 'NV_vertex_program')

# GL/glext.h:5598
glExecuteProgramNV = _link_function('glExecuteProgramNV', None, [GLenum, GLuint, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:5599
glGenProgramsNV = _link_function('glGenProgramsNV', None, [GLsizei, POINTER(GLuint)], 'NV_vertex_program')

# GL/glext.h:5600
glGetProgramParameterdvNV = _link_function('glGetProgramParameterdvNV', None, [GLenum, GLuint, GLenum, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:5601
glGetProgramParameterfvNV = _link_function('glGetProgramParameterfvNV', None, [GLenum, GLuint, GLenum, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:5602
glGetProgramivNV = _link_function('glGetProgramivNV', None, [GLuint, GLenum, POINTER(GLint)], 'NV_vertex_program')

# GL/glext.h:5603
glGetProgramStringNV = _link_function('glGetProgramStringNV', None, [GLuint, GLenum, POINTER(GLubyte)], 'NV_vertex_program')

# GL/glext.h:5604
glGetTrackMatrixivNV = _link_function('glGetTrackMatrixivNV', None, [GLenum, GLuint, GLenum, POINTER(GLint)], 'NV_vertex_program')

# GL/glext.h:5605
glGetVertexAttribdvNV = _link_function('glGetVertexAttribdvNV', None, [GLuint, GLenum, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:5606
glGetVertexAttribfvNV = _link_function('glGetVertexAttribfvNV', None, [GLuint, GLenum, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:5607
glGetVertexAttribivNV = _link_function('glGetVertexAttribivNV', None, [GLuint, GLenum, POINTER(GLint)], 'NV_vertex_program')

# GL/glext.h:5608
glGetVertexAttribPointervNV = _link_function('glGetVertexAttribPointervNV', None, [GLuint, GLenum, POINTER(POINTER(GLvoid))], 'NV_vertex_program')

# GL/glext.h:5609
glIsProgramNV = _link_function('glIsProgramNV', GLboolean, [GLuint], 'NV_vertex_program')

# GL/glext.h:5610
glLoadProgramNV = _link_function('glLoadProgramNV', None, [GLenum, GLuint, GLsizei, POINTER(GLubyte)], 'NV_vertex_program')

# GL/glext.h:5611
glProgramParameter4dNV = _link_function('glProgramParameter4dNV', None, [GLenum, GLuint, GLdouble, GLdouble, GLdouble, GLdouble], 'NV_vertex_program')

# GL/glext.h:5612
glProgramParameter4dvNV = _link_function('glProgramParameter4dvNV', None, [GLenum, GLuint, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:5613
glProgramParameter4fNV = _link_function('glProgramParameter4fNV', None, [GLenum, GLuint, GLfloat, GLfloat, GLfloat, GLfloat], 'NV_vertex_program')

# GL/glext.h:5614
glProgramParameter4fvNV = _link_function('glProgramParameter4fvNV', None, [GLenum, GLuint, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:5615
glProgramParameters4dvNV = _link_function('glProgramParameters4dvNV', None, [GLenum, GLuint, GLuint, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:5616
glProgramParameters4fvNV = _link_function('glProgramParameters4fvNV', None, [GLenum, GLuint, GLuint, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:5617
glRequestResidentProgramsNV = _link_function('glRequestResidentProgramsNV', None, [GLsizei, POINTER(GLuint)], 'NV_vertex_program')

# GL/glext.h:5618
glTrackMatrixNV = _link_function('glTrackMatrixNV', None, [GLenum, GLuint, GLenum, GLenum], 'NV_vertex_program')

# GL/glext.h:5619
glVertexAttribPointerNV = _link_function('glVertexAttribPointerNV', None, [GLuint, GLint, GLenum, GLsizei, POINTER(GLvoid)], 'NV_vertex_program')

# GL/glext.h:5620
glVertexAttrib1dNV = _link_function('glVertexAttrib1dNV', None, [GLuint, GLdouble], 'NV_vertex_program')

# GL/glext.h:5621
glVertexAttrib1dvNV = _link_function('glVertexAttrib1dvNV', None, [GLuint, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:5622
glVertexAttrib1fNV = _link_function('glVertexAttrib1fNV', None, [GLuint, GLfloat], 'NV_vertex_program')

# GL/glext.h:5623
glVertexAttrib1fvNV = _link_function('glVertexAttrib1fvNV', None, [GLuint, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:5624
glVertexAttrib1sNV = _link_function('glVertexAttrib1sNV', None, [GLuint, GLshort], 'NV_vertex_program')

# GL/glext.h:5625
glVertexAttrib1svNV = _link_function('glVertexAttrib1svNV', None, [GLuint, POINTER(GLshort)], 'NV_vertex_program')

# GL/glext.h:5626
glVertexAttrib2dNV = _link_function('glVertexAttrib2dNV', None, [GLuint, GLdouble, GLdouble], 'NV_vertex_program')

# GL/glext.h:5627
glVertexAttrib2dvNV = _link_function('glVertexAttrib2dvNV', None, [GLuint, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:5628
glVertexAttrib2fNV = _link_function('glVertexAttrib2fNV', None, [GLuint, GLfloat, GLfloat], 'NV_vertex_program')

# GL/glext.h:5629
glVertexAttrib2fvNV = _link_function('glVertexAttrib2fvNV', None, [GLuint, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:5630
glVertexAttrib2sNV = _link_function('glVertexAttrib2sNV', None, [GLuint, GLshort, GLshort], 'NV_vertex_program')

# GL/glext.h:5631
glVertexAttrib2svNV = _link_function('glVertexAttrib2svNV', None, [GLuint, POINTER(GLshort)], 'NV_vertex_program')

# GL/glext.h:5632
glVertexAttrib3dNV = _link_function('glVertexAttrib3dNV', None, [GLuint, GLdouble, GLdouble, GLdouble], 'NV_vertex_program')

# GL/glext.h:5633
glVertexAttrib3dvNV = _link_function('glVertexAttrib3dvNV', None, [GLuint, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:5634
glVertexAttrib3fNV = _link_function('glVertexAttrib3fNV', None, [GLuint, GLfloat, GLfloat, GLfloat], 'NV_vertex_program')

# GL/glext.h:5635
glVertexAttrib3fvNV = _link_function('glVertexAttrib3fvNV', None, [GLuint, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:5636
glVertexAttrib3sNV = _link_function('glVertexAttrib3sNV', None, [GLuint, GLshort, GLshort, GLshort], 'NV_vertex_program')

# GL/glext.h:5637
glVertexAttrib3svNV = _link_function('glVertexAttrib3svNV', None, [GLuint, POINTER(GLshort)], 'NV_vertex_program')

# GL/glext.h:5638
glVertexAttrib4dNV = _link_function('glVertexAttrib4dNV', None, [GLuint, GLdouble, GLdouble, GLdouble, GLdouble], 'NV_vertex_program')

# GL/glext.h:5639
glVertexAttrib4dvNV = _link_function('glVertexAttrib4dvNV', None, [GLuint, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:5640
glVertexAttrib4fNV = _link_function('glVertexAttrib4fNV', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat], 'NV_vertex_program')

# GL/glext.h:5641
glVertexAttrib4fvNV = _link_function('glVertexAttrib4fvNV', None, [GLuint, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:5642
glVertexAttrib4sNV = _link_function('glVertexAttrib4sNV', None, [GLuint, GLshort, GLshort, GLshort, GLshort], 'NV_vertex_program')

# GL/glext.h:5643
glVertexAttrib4svNV = _link_function('glVertexAttrib4svNV', None, [GLuint, POINTER(GLshort)], 'NV_vertex_program')

# GL/glext.h:5644
glVertexAttrib4ubNV = _link_function('glVertexAttrib4ubNV', None, [GLuint, GLubyte, GLubyte, GLubyte, GLubyte], 'NV_vertex_program')

# GL/glext.h:5645
glVertexAttrib4ubvNV = _link_function('glVertexAttrib4ubvNV', None, [GLuint, POINTER(GLubyte)], 'NV_vertex_program')

# GL/glext.h:5646
glVertexAttribs1dvNV = _link_function('glVertexAttribs1dvNV', None, [GLuint, GLsizei, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:5647
glVertexAttribs1fvNV = _link_function('glVertexAttribs1fvNV', None, [GLuint, GLsizei, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:5648
glVertexAttribs1svNV = _link_function('glVertexAttribs1svNV', None, [GLuint, GLsizei, POINTER(GLshort)], 'NV_vertex_program')

# GL/glext.h:5649
glVertexAttribs2dvNV = _link_function('glVertexAttribs2dvNV', None, [GLuint, GLsizei, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:5650
glVertexAttribs2fvNV = _link_function('glVertexAttribs2fvNV', None, [GLuint, GLsizei, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:5651
glVertexAttribs2svNV = _link_function('glVertexAttribs2svNV', None, [GLuint, GLsizei, POINTER(GLshort)], 'NV_vertex_program')

# GL/glext.h:5652
glVertexAttribs3dvNV = _link_function('glVertexAttribs3dvNV', None, [GLuint, GLsizei, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:5653
glVertexAttribs3fvNV = _link_function('glVertexAttribs3fvNV', None, [GLuint, GLsizei, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:5654
glVertexAttribs3svNV = _link_function('glVertexAttribs3svNV', None, [GLuint, GLsizei, POINTER(GLshort)], 'NV_vertex_program')

# GL/glext.h:5655
glVertexAttribs4dvNV = _link_function('glVertexAttribs4dvNV', None, [GLuint, GLsizei, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:5656
glVertexAttribs4fvNV = _link_function('glVertexAttribs4fvNV', None, [GLuint, GLsizei, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:5657
glVertexAttribs4svNV = _link_function('glVertexAttribs4svNV', None, [GLuint, GLsizei, POINTER(GLshort)], 'NV_vertex_program')

# GL/glext.h:5658
glVertexAttribs4ubvNV = _link_function('glVertexAttribs4ubvNV', None, [GLuint, GLsizei, POINTER(GLubyte)], 'NV_vertex_program')

PFNGLAREPROGRAMSRESIDENTNVPROC = CFUNCTYPE(GLboolean, GLsizei, POINTER(GLuint), POINTER(GLboolean)) 	# GL/glext.h:5660
PFNGLBINDPROGRAMNVPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:5661
PFNGLDELETEPROGRAMSNVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:5662
PFNGLEXECUTEPROGRAMNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLfloat)) 	# GL/glext.h:5663
PFNGLGENPROGRAMSNVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:5664
PFNGLGETPROGRAMPARAMETERDVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLenum, POINTER(GLdouble)) 	# GL/glext.h:5665
PFNGLGETPROGRAMPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5666
PFNGLGETPROGRAMIVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:5667
PFNGLGETPROGRAMSTRINGNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLubyte)) 	# GL/glext.h:5668
PFNGLGETTRACKMATRIXIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:5669
PFNGLGETVERTEXATTRIBDVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLdouble)) 	# GL/glext.h:5670
PFNGLGETVERTEXATTRIBFVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5671
PFNGLGETVERTEXATTRIBIVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:5672
PFNGLGETVERTEXATTRIBPOINTERVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:5673
PFNGLISPROGRAMNVPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:5674
PFNGLLOADPROGRAMNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLsizei, POINTER(GLubyte)) 	# GL/glext.h:5675
PFNGLPROGRAMPARAMETER4DNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:5676
PFNGLPROGRAMPARAMETER4DVNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLdouble)) 	# GL/glext.h:5677
PFNGLPROGRAMPARAMETER4FNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5678
PFNGLPROGRAMPARAMETER4FVNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLfloat)) 	# GL/glext.h:5679
PFNGLPROGRAMPARAMETERS4DVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, POINTER(GLdouble)) 	# GL/glext.h:5680
PFNGLPROGRAMPARAMETERS4FVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, POINTER(GLfloat)) 	# GL/glext.h:5681
PFNGLREQUESTRESIDENTPROGRAMSNVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:5682
PFNGLTRACKMATRIXNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLenum, GLenum) 	# GL/glext.h:5683
PFNGLVERTEXATTRIBPOINTERNVPROC = CFUNCTYPE(None, GLuint, GLint, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:5684
PFNGLVERTEXATTRIB1DNVPROC = CFUNCTYPE(None, GLuint, GLdouble) 	# GL/glext.h:5685
PFNGLVERTEXATTRIB1DVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:5686
PFNGLVERTEXATTRIB1FNVPROC = CFUNCTYPE(None, GLuint, GLfloat) 	# GL/glext.h:5687
PFNGLVERTEXATTRIB1FVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:5688
PFNGLVERTEXATTRIB1SNVPROC = CFUNCTYPE(None, GLuint, GLshort) 	# GL/glext.h:5689
PFNGLVERTEXATTRIB1SVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:5690
PFNGLVERTEXATTRIB2DNVPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble) 	# GL/glext.h:5691
PFNGLVERTEXATTRIB2DVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:5692
PFNGLVERTEXATTRIB2FNVPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat) 	# GL/glext.h:5693
PFNGLVERTEXATTRIB2FVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:5694
PFNGLVERTEXATTRIB2SNVPROC = CFUNCTYPE(None, GLuint, GLshort, GLshort) 	# GL/glext.h:5695
PFNGLVERTEXATTRIB2SVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:5696
PFNGLVERTEXATTRIB3DNVPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:5697
PFNGLVERTEXATTRIB3DVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:5698
PFNGLVERTEXATTRIB3FNVPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5699
PFNGLVERTEXATTRIB3FVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:5700
PFNGLVERTEXATTRIB3SNVPROC = CFUNCTYPE(None, GLuint, GLshort, GLshort, GLshort) 	# GL/glext.h:5701
PFNGLVERTEXATTRIB3SVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:5702
PFNGLVERTEXATTRIB4DNVPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:5703
PFNGLVERTEXATTRIB4DVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:5704
PFNGLVERTEXATTRIB4FNVPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5705
PFNGLVERTEXATTRIB4FVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:5706
PFNGLVERTEXATTRIB4SNVPROC = CFUNCTYPE(None, GLuint, GLshort, GLshort, GLshort, GLshort) 	# GL/glext.h:5707
PFNGLVERTEXATTRIB4SVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:5708
PFNGLVERTEXATTRIB4UBNVPROC = CFUNCTYPE(None, GLuint, GLubyte, GLubyte, GLubyte, GLubyte) 	# GL/glext.h:5709
PFNGLVERTEXATTRIB4UBVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLubyte)) 	# GL/glext.h:5710
PFNGLVERTEXATTRIBS1DVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLdouble)) 	# GL/glext.h:5711
PFNGLVERTEXATTRIBS1FVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:5712
PFNGLVERTEXATTRIBS1SVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLshort)) 	# GL/glext.h:5713
PFNGLVERTEXATTRIBS2DVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLdouble)) 	# GL/glext.h:5714
PFNGLVERTEXATTRIBS2FVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:5715
PFNGLVERTEXATTRIBS2SVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLshort)) 	# GL/glext.h:5716
PFNGLVERTEXATTRIBS3DVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLdouble)) 	# GL/glext.h:5717
PFNGLVERTEXATTRIBS3FVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:5718
PFNGLVERTEXATTRIBS3SVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLshort)) 	# GL/glext.h:5719
PFNGLVERTEXATTRIBS4DVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLdouble)) 	# GL/glext.h:5720
PFNGLVERTEXATTRIBS4FVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:5721
PFNGLVERTEXATTRIBS4SVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLshort)) 	# GL/glext.h:5722
PFNGLVERTEXATTRIBS4UBVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLubyte)) 	# GL/glext.h:5723
# SGIX_texture_coordinate_clamp (GL/glext.h:5726)
GL_SGIX_texture_coordinate_clamp = 1 	# GL/glext.h:5727
# SGIX_scalebias_hint (GL/glext.h:5730)
GL_SGIX_scalebias_hint = 1 	# GL/glext.h:5731
# OML_interlace (GL/glext.h:5734)
GL_OML_interlace = 1 	# GL/glext.h:5735
# OML_subsample (GL/glext.h:5738)
GL_OML_subsample = 1 	# GL/glext.h:5739
# OML_resample (GL/glext.h:5742)
GL_OML_resample = 1 	# GL/glext.h:5743
# NV_copy_depth_to_color (GL/glext.h:5746)
GL_NV_copy_depth_to_color = 1 	# GL/glext.h:5747
# ATI_envmap_bumpmap (GL/glext.h:5750)
GL_ATI_envmap_bumpmap = 1 	# GL/glext.h:5751
# GL/glext.h:5753
glTexBumpParameterivATI = _link_function('glTexBumpParameterivATI', None, [GLenum, POINTER(GLint)], 'ATI_envmap_bumpmap')

# GL/glext.h:5754
glTexBumpParameterfvATI = _link_function('glTexBumpParameterfvATI', None, [GLenum, POINTER(GLfloat)], 'ATI_envmap_bumpmap')

# GL/glext.h:5755
glGetTexBumpParameterivATI = _link_function('glGetTexBumpParameterivATI', None, [GLenum, POINTER(GLint)], 'ATI_envmap_bumpmap')

# GL/glext.h:5756
glGetTexBumpParameterfvATI = _link_function('glGetTexBumpParameterfvATI', None, [GLenum, POINTER(GLfloat)], 'ATI_envmap_bumpmap')

PFNGLTEXBUMPPARAMETERIVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:5758
PFNGLTEXBUMPPARAMETERFVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5759
PFNGLGETTEXBUMPPARAMETERIVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:5760
PFNGLGETTEXBUMPPARAMETERFVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5761
# ATI_fragment_shader (GL/glext.h:5764)
GL_ATI_fragment_shader = 1 	# GL/glext.h:5765
# GL/glext.h:5767
glGenFragmentShadersATI = _link_function('glGenFragmentShadersATI', GLuint, [GLuint], 'ATI_fragment_shader')

# GL/glext.h:5768
glBindFragmentShaderATI = _link_function('glBindFragmentShaderATI', None, [GLuint], 'ATI_fragment_shader')

# GL/glext.h:5769
glDeleteFragmentShaderATI = _link_function('glDeleteFragmentShaderATI', None, [GLuint], 'ATI_fragment_shader')

# GL/glext.h:5770
glBeginFragmentShaderATI = _link_function('glBeginFragmentShaderATI', None, [], 'ATI_fragment_shader')

# GL/glext.h:5771
glEndFragmentShaderATI = _link_function('glEndFragmentShaderATI', None, [], 'ATI_fragment_shader')

# GL/glext.h:5772
glPassTexCoordATI = _link_function('glPassTexCoordATI', None, [GLuint, GLuint, GLenum], 'ATI_fragment_shader')

# GL/glext.h:5773
glSampleMapATI = _link_function('glSampleMapATI', None, [GLuint, GLuint, GLenum], 'ATI_fragment_shader')

# GL/glext.h:5774
glColorFragmentOp1ATI = _link_function('glColorFragmentOp1ATI', None, [GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint], 'ATI_fragment_shader')

# GL/glext.h:5775
glColorFragmentOp2ATI = _link_function('glColorFragmentOp2ATI', None, [GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint], 'ATI_fragment_shader')

# GL/glext.h:5776
glColorFragmentOp3ATI = _link_function('glColorFragmentOp3ATI', None, [GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint], 'ATI_fragment_shader')

# GL/glext.h:5777
glAlphaFragmentOp1ATI = _link_function('glAlphaFragmentOp1ATI', None, [GLenum, GLuint, GLuint, GLuint, GLuint, GLuint], 'ATI_fragment_shader')

# GL/glext.h:5778
glAlphaFragmentOp2ATI = _link_function('glAlphaFragmentOp2ATI', None, [GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint], 'ATI_fragment_shader')

# GL/glext.h:5779
glAlphaFragmentOp3ATI = _link_function('glAlphaFragmentOp3ATI', None, [GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint], 'ATI_fragment_shader')

# GL/glext.h:5780
glSetFragmentShaderConstantATI = _link_function('glSetFragmentShaderConstantATI', None, [GLuint, POINTER(GLfloat)], 'ATI_fragment_shader')

PFNGLGENFRAGMENTSHADERSATIPROC = CFUNCTYPE(GLuint, GLuint) 	# GL/glext.h:5782
PFNGLBINDFRAGMENTSHADERATIPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:5783
PFNGLDELETEFRAGMENTSHADERATIPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:5784
PFNGLBEGINFRAGMENTSHADERATIPROC = CFUNCTYPE(None) 	# GL/glext.h:5785
PFNGLENDFRAGMENTSHADERATIPROC = CFUNCTYPE(None) 	# GL/glext.h:5786
PFNGLPASSTEXCOORDATIPROC = CFUNCTYPE(None, GLuint, GLuint, GLenum) 	# GL/glext.h:5787
PFNGLSAMPLEMAPATIPROC = CFUNCTYPE(None, GLuint, GLuint, GLenum) 	# GL/glext.h:5788
PFNGLCOLORFRAGMENTOP1ATIPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:5789
PFNGLCOLORFRAGMENTOP2ATIPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:5790
PFNGLCOLORFRAGMENTOP3ATIPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:5791
PFNGLALPHAFRAGMENTOP1ATIPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:5792
PFNGLALPHAFRAGMENTOP2ATIPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:5793
PFNGLALPHAFRAGMENTOP3ATIPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:5794
PFNGLSETFRAGMENTSHADERCONSTANTATIPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:5795
# ATI_pn_triangles (GL/glext.h:5798)
GL_ATI_pn_triangles = 1 	# GL/glext.h:5799
# GL/glext.h:5801
glPNTrianglesiATI = _link_function('glPNTrianglesiATI', None, [GLenum, GLint], 'ATI_pn_triangles')

# GL/glext.h:5802
glPNTrianglesfATI = _link_function('glPNTrianglesfATI', None, [GLenum, GLfloat], 'ATI_pn_triangles')

PFNGLPNTRIANGLESIATIPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:5804
PFNGLPNTRIANGLESFATIPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:5805
# ATI_vertex_array_object (GL/glext.h:5808)
GL_ATI_vertex_array_object = 1 	# GL/glext.h:5809
# GL/glext.h:5811
glNewObjectBufferATI = _link_function('glNewObjectBufferATI', GLuint, [GLsizei, POINTER(GLvoid), GLenum], 'ATI_vertex_array_object')

# GL/glext.h:5812
glIsObjectBufferATI = _link_function('glIsObjectBufferATI', GLboolean, [GLuint], 'ATI_vertex_array_object')

# GL/glext.h:5813
glUpdateObjectBufferATI = _link_function('glUpdateObjectBufferATI', None, [GLuint, GLuint, GLsizei, POINTER(GLvoid), GLenum], 'ATI_vertex_array_object')

# GL/glext.h:5814
glGetObjectBufferfvATI = _link_function('glGetObjectBufferfvATI', None, [GLuint, GLenum, POINTER(GLfloat)], 'ATI_vertex_array_object')

# GL/glext.h:5815
glGetObjectBufferivATI = _link_function('glGetObjectBufferivATI', None, [GLuint, GLenum, POINTER(GLint)], 'ATI_vertex_array_object')

# GL/glext.h:5816
glFreeObjectBufferATI = _link_function('glFreeObjectBufferATI', None, [GLuint], 'ATI_vertex_array_object')

# GL/glext.h:5817
glArrayObjectATI = _link_function('glArrayObjectATI', None, [GLenum, GLint, GLenum, GLsizei, GLuint, GLuint], 'ATI_vertex_array_object')

# GL/glext.h:5818
glGetArrayObjectfvATI = _link_function('glGetArrayObjectfvATI', None, [GLenum, GLenum, POINTER(GLfloat)], 'ATI_vertex_array_object')

# GL/glext.h:5819
glGetArrayObjectivATI = _link_function('glGetArrayObjectivATI', None, [GLenum, GLenum, POINTER(GLint)], 'ATI_vertex_array_object')

# GL/glext.h:5820
glVariantArrayObjectATI = _link_function('glVariantArrayObjectATI', None, [GLuint, GLenum, GLsizei, GLuint, GLuint], 'ATI_vertex_array_object')

# GL/glext.h:5821
glGetVariantArrayObjectfvATI = _link_function('glGetVariantArrayObjectfvATI', None, [GLuint, GLenum, POINTER(GLfloat)], 'ATI_vertex_array_object')

# GL/glext.h:5822
glGetVariantArrayObjectivATI = _link_function('glGetVariantArrayObjectivATI', None, [GLuint, GLenum, POINTER(GLint)], 'ATI_vertex_array_object')

PFNGLNEWOBJECTBUFFERATIPROC = CFUNCTYPE(GLuint, GLsizei, POINTER(GLvoid), GLenum) 	# GL/glext.h:5824
PFNGLISOBJECTBUFFERATIPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:5825
PFNGLUPDATEOBJECTBUFFERATIPROC = CFUNCTYPE(None, GLuint, GLuint, GLsizei, POINTER(GLvoid), GLenum) 	# GL/glext.h:5826
PFNGLGETOBJECTBUFFERFVATIPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5827
PFNGLGETOBJECTBUFFERIVATIPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:5828
PFNGLFREEOBJECTBUFFERATIPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:5829
PFNGLARRAYOBJECTATIPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLsizei, GLuint, GLuint) 	# GL/glext.h:5830
PFNGLGETARRAYOBJECTFVATIPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5831
PFNGLGETARRAYOBJECTIVATIPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5832
PFNGLVARIANTARRAYOBJECTATIPROC = CFUNCTYPE(None, GLuint, GLenum, GLsizei, GLuint, GLuint) 	# GL/glext.h:5833
PFNGLGETVARIANTARRAYOBJECTFVATIPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5834
PFNGLGETVARIANTARRAYOBJECTIVATIPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:5835
# EXT_vertex_shader (GL/glext.h:5838)
GL_EXT_vertex_shader = 1 	# GL/glext.h:5839
# GL/glext.h:5841
glBeginVertexShaderEXT = _link_function('glBeginVertexShaderEXT', None, [], 'EXT_vertex_shader')

# GL/glext.h:5842
glEndVertexShaderEXT = _link_function('glEndVertexShaderEXT', None, [], 'EXT_vertex_shader')

# GL/glext.h:5843
glBindVertexShaderEXT = _link_function('glBindVertexShaderEXT', None, [GLuint], 'EXT_vertex_shader')

# GL/glext.h:5844
glGenVertexShadersEXT = _link_function('glGenVertexShadersEXT', GLuint, [GLuint], 'EXT_vertex_shader')

# GL/glext.h:5845
glDeleteVertexShaderEXT = _link_function('glDeleteVertexShaderEXT', None, [GLuint], 'EXT_vertex_shader')

# GL/glext.h:5846
glShaderOp1EXT = _link_function('glShaderOp1EXT', None, [GLenum, GLuint, GLuint], 'EXT_vertex_shader')

# GL/glext.h:5847
glShaderOp2EXT = _link_function('glShaderOp2EXT', None, [GLenum, GLuint, GLuint, GLuint], 'EXT_vertex_shader')

# GL/glext.h:5848
glShaderOp3EXT = _link_function('glShaderOp3EXT', None, [GLenum, GLuint, GLuint, GLuint, GLuint], 'EXT_vertex_shader')

# GL/glext.h:5849
glSwizzleEXT = _link_function('glSwizzleEXT', None, [GLuint, GLuint, GLenum, GLenum, GLenum, GLenum], 'EXT_vertex_shader')

# GL/glext.h:5850
glWriteMaskEXT = _link_function('glWriteMaskEXT', None, [GLuint, GLuint, GLenum, GLenum, GLenum, GLenum], 'EXT_vertex_shader')

# GL/glext.h:5851
glInsertComponentEXT = _link_function('glInsertComponentEXT', None, [GLuint, GLuint, GLuint], 'EXT_vertex_shader')

# GL/glext.h:5852
glExtractComponentEXT = _link_function('glExtractComponentEXT', None, [GLuint, GLuint, GLuint], 'EXT_vertex_shader')

# GL/glext.h:5853
glGenSymbolsEXT = _link_function('glGenSymbolsEXT', GLuint, [GLenum, GLenum, GLenum, GLuint], 'EXT_vertex_shader')

# GL/glext.h:5854
glSetInvariantEXT = _link_function('glSetInvariantEXT', None, [GLuint, GLenum, POINTER(GLvoid)], 'EXT_vertex_shader')

# GL/glext.h:5855
glSetLocalConstantEXT = _link_function('glSetLocalConstantEXT', None, [GLuint, GLenum, POINTER(GLvoid)], 'EXT_vertex_shader')

# GL/glext.h:5856
glVariantbvEXT = _link_function('glVariantbvEXT', None, [GLuint, POINTER(GLbyte)], 'EXT_vertex_shader')

# GL/glext.h:5857
glVariantsvEXT = _link_function('glVariantsvEXT', None, [GLuint, POINTER(GLshort)], 'EXT_vertex_shader')

# GL/glext.h:5858
glVariantivEXT = _link_function('glVariantivEXT', None, [GLuint, POINTER(GLint)], 'EXT_vertex_shader')

# GL/glext.h:5859
glVariantfvEXT = _link_function('glVariantfvEXT', None, [GLuint, POINTER(GLfloat)], 'EXT_vertex_shader')

# GL/glext.h:5860
glVariantdvEXT = _link_function('glVariantdvEXT', None, [GLuint, POINTER(GLdouble)], 'EXT_vertex_shader')

# GL/glext.h:5861
glVariantubvEXT = _link_function('glVariantubvEXT', None, [GLuint, POINTER(GLubyte)], 'EXT_vertex_shader')

# GL/glext.h:5862
glVariantusvEXT = _link_function('glVariantusvEXT', None, [GLuint, POINTER(GLushort)], 'EXT_vertex_shader')

# GL/glext.h:5863
glVariantuivEXT = _link_function('glVariantuivEXT', None, [GLuint, POINTER(GLuint)], 'EXT_vertex_shader')

# GL/glext.h:5864
glVariantPointerEXT = _link_function('glVariantPointerEXT', None, [GLuint, GLenum, GLuint, POINTER(GLvoid)], 'EXT_vertex_shader')

# GL/glext.h:5865
glEnableVariantClientStateEXT = _link_function('glEnableVariantClientStateEXT', None, [GLuint], 'EXT_vertex_shader')

# GL/glext.h:5866
glDisableVariantClientStateEXT = _link_function('glDisableVariantClientStateEXT', None, [GLuint], 'EXT_vertex_shader')

# GL/glext.h:5867
glBindLightParameterEXT = _link_function('glBindLightParameterEXT', GLuint, [GLenum, GLenum], 'EXT_vertex_shader')

# GL/glext.h:5868
glBindMaterialParameterEXT = _link_function('glBindMaterialParameterEXT', GLuint, [GLenum, GLenum], 'EXT_vertex_shader')

# GL/glext.h:5869
glBindTexGenParameterEXT = _link_function('glBindTexGenParameterEXT', GLuint, [GLenum, GLenum, GLenum], 'EXT_vertex_shader')

# GL/glext.h:5870
glBindTextureUnitParameterEXT = _link_function('glBindTextureUnitParameterEXT', GLuint, [GLenum, GLenum], 'EXT_vertex_shader')

# GL/glext.h:5871
glBindParameterEXT = _link_function('glBindParameterEXT', GLuint, [GLenum], 'EXT_vertex_shader')

# GL/glext.h:5872
glIsVariantEnabledEXT = _link_function('glIsVariantEnabledEXT', GLboolean, [GLuint, GLenum], 'EXT_vertex_shader')

# GL/glext.h:5873
glGetVariantBooleanvEXT = _link_function('glGetVariantBooleanvEXT', None, [GLuint, GLenum, POINTER(GLboolean)], 'EXT_vertex_shader')

# GL/glext.h:5874
glGetVariantIntegervEXT = _link_function('glGetVariantIntegervEXT', None, [GLuint, GLenum, POINTER(GLint)], 'EXT_vertex_shader')

# GL/glext.h:5875
glGetVariantFloatvEXT = _link_function('glGetVariantFloatvEXT', None, [GLuint, GLenum, POINTER(GLfloat)], 'EXT_vertex_shader')

# GL/glext.h:5876
glGetVariantPointervEXT = _link_function('glGetVariantPointervEXT', None, [GLuint, GLenum, POINTER(POINTER(GLvoid))], 'EXT_vertex_shader')

# GL/glext.h:5877
glGetInvariantBooleanvEXT = _link_function('glGetInvariantBooleanvEXT', None, [GLuint, GLenum, POINTER(GLboolean)], 'EXT_vertex_shader')

# GL/glext.h:5878
glGetInvariantIntegervEXT = _link_function('glGetInvariantIntegervEXT', None, [GLuint, GLenum, POINTER(GLint)], 'EXT_vertex_shader')

# GL/glext.h:5879
glGetInvariantFloatvEXT = _link_function('glGetInvariantFloatvEXT', None, [GLuint, GLenum, POINTER(GLfloat)], 'EXT_vertex_shader')

# GL/glext.h:5880
glGetLocalConstantBooleanvEXT = _link_function('glGetLocalConstantBooleanvEXT', None, [GLuint, GLenum, POINTER(GLboolean)], 'EXT_vertex_shader')

# GL/glext.h:5881
glGetLocalConstantIntegervEXT = _link_function('glGetLocalConstantIntegervEXT', None, [GLuint, GLenum, POINTER(GLint)], 'EXT_vertex_shader')

# GL/glext.h:5882
glGetLocalConstantFloatvEXT = _link_function('glGetLocalConstantFloatvEXT', None, [GLuint, GLenum, POINTER(GLfloat)], 'EXT_vertex_shader')

PFNGLBEGINVERTEXSHADEREXTPROC = CFUNCTYPE(None) 	# GL/glext.h:5884
PFNGLENDVERTEXSHADEREXTPROC = CFUNCTYPE(None) 	# GL/glext.h:5885
PFNGLBINDVERTEXSHADEREXTPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:5886
PFNGLGENVERTEXSHADERSEXTPROC = CFUNCTYPE(GLuint, GLuint) 	# GL/glext.h:5887
PFNGLDELETEVERTEXSHADEREXTPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:5888
PFNGLSHADEROP1EXTPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint) 	# GL/glext.h:5889
PFNGLSHADEROP2EXTPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint) 	# GL/glext.h:5890
PFNGLSHADEROP3EXTPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:5891
PFNGLSWIZZLEEXTPROC = CFUNCTYPE(None, GLuint, GLuint, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:5892
PFNGLWRITEMASKEXTPROC = CFUNCTYPE(None, GLuint, GLuint, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:5893
PFNGLINSERTCOMPONENTEXTPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint) 	# GL/glext.h:5894
PFNGLEXTRACTCOMPONENTEXTPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint) 	# GL/glext.h:5895
PFNGLGENSYMBOLSEXTPROC = CFUNCTYPE(GLuint, GLenum, GLenum, GLenum, GLuint) 	# GL/glext.h:5896
PFNGLSETINVARIANTEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLvoid)) 	# GL/glext.h:5897
PFNGLSETLOCALCONSTANTEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLvoid)) 	# GL/glext.h:5898
PFNGLVARIANTBVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLbyte)) 	# GL/glext.h:5899
PFNGLVARIANTSVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:5900
PFNGLVARIANTIVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:5901
PFNGLVARIANTFVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:5902
PFNGLVARIANTDVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:5903
PFNGLVARIANTUBVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLubyte)) 	# GL/glext.h:5904
PFNGLVARIANTUSVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLushort)) 	# GL/glext.h:5905
PFNGLVARIANTUIVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:5906
PFNGLVARIANTPOINTEREXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, POINTER(GLvoid)) 	# GL/glext.h:5907
PFNGLENABLEVARIANTCLIENTSTATEEXTPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:5908
PFNGLDISABLEVARIANTCLIENTSTATEEXTPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:5909
PFNGLBINDLIGHTPARAMETEREXTPROC = CFUNCTYPE(GLuint, GLenum, GLenum) 	# GL/glext.h:5910
PFNGLBINDMATERIALPARAMETEREXTPROC = CFUNCTYPE(GLuint, GLenum, GLenum) 	# GL/glext.h:5911
PFNGLBINDTEXGENPARAMETEREXTPROC = CFUNCTYPE(GLuint, GLenum, GLenum, GLenum) 	# GL/glext.h:5912
PFNGLBINDTEXTUREUNITPARAMETEREXTPROC = CFUNCTYPE(GLuint, GLenum, GLenum) 	# GL/glext.h:5913
PFNGLBINDPARAMETEREXTPROC = CFUNCTYPE(GLuint, GLenum) 	# GL/glext.h:5914
PFNGLISVARIANTENABLEDEXTPROC = CFUNCTYPE(GLboolean, GLuint, GLenum) 	# GL/glext.h:5915
PFNGLGETVARIANTBOOLEANVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLboolean)) 	# GL/glext.h:5916
PFNGLGETVARIANTINTEGERVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:5917
PFNGLGETVARIANTFLOATVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5918
PFNGLGETVARIANTPOINTERVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:5919
PFNGLGETINVARIANTBOOLEANVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLboolean)) 	# GL/glext.h:5920
PFNGLGETINVARIANTINTEGERVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:5921
PFNGLGETINVARIANTFLOATVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5922
PFNGLGETLOCALCONSTANTBOOLEANVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLboolean)) 	# GL/glext.h:5923
PFNGLGETLOCALCONSTANTINTEGERVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:5924
PFNGLGETLOCALCONSTANTFLOATVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5925
# ATI_vertex_streams (GL/glext.h:5928)
GL_ATI_vertex_streams = 1 	# GL/glext.h:5929
# GL/glext.h:5931
glVertexStream1sATI = _link_function('glVertexStream1sATI', None, [GLenum, GLshort], 'ATI_vertex_streams')

# GL/glext.h:5932
glVertexStream1svATI = _link_function('glVertexStream1svATI', None, [GLenum, POINTER(GLshort)], 'ATI_vertex_streams')

# GL/glext.h:5933
glVertexStream1iATI = _link_function('glVertexStream1iATI', None, [GLenum, GLint], 'ATI_vertex_streams')

# GL/glext.h:5934
glVertexStream1ivATI = _link_function('glVertexStream1ivATI', None, [GLenum, POINTER(GLint)], 'ATI_vertex_streams')

# GL/glext.h:5935
glVertexStream1fATI = _link_function('glVertexStream1fATI', None, [GLenum, GLfloat], 'ATI_vertex_streams')

# GL/glext.h:5936
glVertexStream1fvATI = _link_function('glVertexStream1fvATI', None, [GLenum, POINTER(GLfloat)], 'ATI_vertex_streams')

# GL/glext.h:5937
glVertexStream1dATI = _link_function('glVertexStream1dATI', None, [GLenum, GLdouble], 'ATI_vertex_streams')

# GL/glext.h:5938
glVertexStream1dvATI = _link_function('glVertexStream1dvATI', None, [GLenum, POINTER(GLdouble)], 'ATI_vertex_streams')

# GL/glext.h:5939
glVertexStream2sATI = _link_function('glVertexStream2sATI', None, [GLenum, GLshort, GLshort], 'ATI_vertex_streams')

# GL/glext.h:5940
glVertexStream2svATI = _link_function('glVertexStream2svATI', None, [GLenum, POINTER(GLshort)], 'ATI_vertex_streams')

# GL/glext.h:5941
glVertexStream2iATI = _link_function('glVertexStream2iATI', None, [GLenum, GLint, GLint], 'ATI_vertex_streams')

# GL/glext.h:5942
glVertexStream2ivATI = _link_function('glVertexStream2ivATI', None, [GLenum, POINTER(GLint)], 'ATI_vertex_streams')

# GL/glext.h:5943
glVertexStream2fATI = _link_function('glVertexStream2fATI', None, [GLenum, GLfloat, GLfloat], 'ATI_vertex_streams')

# GL/glext.h:5944
glVertexStream2fvATI = _link_function('glVertexStream2fvATI', None, [GLenum, POINTER(GLfloat)], 'ATI_vertex_streams')

# GL/glext.h:5945
glVertexStream2dATI = _link_function('glVertexStream2dATI', None, [GLenum, GLdouble, GLdouble], 'ATI_vertex_streams')

# GL/glext.h:5946
glVertexStream2dvATI = _link_function('glVertexStream2dvATI', None, [GLenum, POINTER(GLdouble)], 'ATI_vertex_streams')

# GL/glext.h:5947
glVertexStream3sATI = _link_function('glVertexStream3sATI', None, [GLenum, GLshort, GLshort, GLshort], 'ATI_vertex_streams')

# GL/glext.h:5948
glVertexStream3svATI = _link_function('glVertexStream3svATI', None, [GLenum, POINTER(GLshort)], 'ATI_vertex_streams')

# GL/glext.h:5949
glVertexStream3iATI = _link_function('glVertexStream3iATI', None, [GLenum, GLint, GLint, GLint], 'ATI_vertex_streams')

# GL/glext.h:5950
glVertexStream3ivATI = _link_function('glVertexStream3ivATI', None, [GLenum, POINTER(GLint)], 'ATI_vertex_streams')

# GL/glext.h:5951
glVertexStream3fATI = _link_function('glVertexStream3fATI', None, [GLenum, GLfloat, GLfloat, GLfloat], 'ATI_vertex_streams')

# GL/glext.h:5952
glVertexStream3fvATI = _link_function('glVertexStream3fvATI', None, [GLenum, POINTER(GLfloat)], 'ATI_vertex_streams')

# GL/glext.h:5953
glVertexStream3dATI = _link_function('glVertexStream3dATI', None, [GLenum, GLdouble, GLdouble, GLdouble], 'ATI_vertex_streams')

# GL/glext.h:5954
glVertexStream3dvATI = _link_function('glVertexStream3dvATI', None, [GLenum, POINTER(GLdouble)], 'ATI_vertex_streams')

# GL/glext.h:5955
glVertexStream4sATI = _link_function('glVertexStream4sATI', None, [GLenum, GLshort, GLshort, GLshort, GLshort], 'ATI_vertex_streams')

# GL/glext.h:5956
glVertexStream4svATI = _link_function('glVertexStream4svATI', None, [GLenum, POINTER(GLshort)], 'ATI_vertex_streams')

# GL/glext.h:5957
glVertexStream4iATI = _link_function('glVertexStream4iATI', None, [GLenum, GLint, GLint, GLint, GLint], 'ATI_vertex_streams')

# GL/glext.h:5958
glVertexStream4ivATI = _link_function('glVertexStream4ivATI', None, [GLenum, POINTER(GLint)], 'ATI_vertex_streams')

# GL/glext.h:5959
glVertexStream4fATI = _link_function('glVertexStream4fATI', None, [GLenum, GLfloat, GLfloat, GLfloat, GLfloat], 'ATI_vertex_streams')

# GL/glext.h:5960
glVertexStream4fvATI = _link_function('glVertexStream4fvATI', None, [GLenum, POINTER(GLfloat)], 'ATI_vertex_streams')

# GL/glext.h:5961
glVertexStream4dATI = _link_function('glVertexStream4dATI', None, [GLenum, GLdouble, GLdouble, GLdouble, GLdouble], 'ATI_vertex_streams')

# GL/glext.h:5962
glVertexStream4dvATI = _link_function('glVertexStream4dvATI', None, [GLenum, POINTER(GLdouble)], 'ATI_vertex_streams')

# GL/glext.h:5963
glNormalStream3bATI = _link_function('glNormalStream3bATI', None, [GLenum, GLbyte, GLbyte, GLbyte], 'ATI_vertex_streams')

# GL/glext.h:5964
glNormalStream3bvATI = _link_function('glNormalStream3bvATI', None, [GLenum, POINTER(GLbyte)], 'ATI_vertex_streams')

# GL/glext.h:5965
glNormalStream3sATI = _link_function('glNormalStream3sATI', None, [GLenum, GLshort, GLshort, GLshort], 'ATI_vertex_streams')

# GL/glext.h:5966
glNormalStream3svATI = _link_function('glNormalStream3svATI', None, [GLenum, POINTER(GLshort)], 'ATI_vertex_streams')

# GL/glext.h:5967
glNormalStream3iATI = _link_function('glNormalStream3iATI', None, [GLenum, GLint, GLint, GLint], 'ATI_vertex_streams')

# GL/glext.h:5968
glNormalStream3ivATI = _link_function('glNormalStream3ivATI', None, [GLenum, POINTER(GLint)], 'ATI_vertex_streams')

# GL/glext.h:5969
glNormalStream3fATI = _link_function('glNormalStream3fATI', None, [GLenum, GLfloat, GLfloat, GLfloat], 'ATI_vertex_streams')

# GL/glext.h:5970
glNormalStream3fvATI = _link_function('glNormalStream3fvATI', None, [GLenum, POINTER(GLfloat)], 'ATI_vertex_streams')

# GL/glext.h:5971
glNormalStream3dATI = _link_function('glNormalStream3dATI', None, [GLenum, GLdouble, GLdouble, GLdouble], 'ATI_vertex_streams')

# GL/glext.h:5972
glNormalStream3dvATI = _link_function('glNormalStream3dvATI', None, [GLenum, POINTER(GLdouble)], 'ATI_vertex_streams')

# GL/glext.h:5973
glClientActiveVertexStreamATI = _link_function('glClientActiveVertexStreamATI', None, [GLenum], 'ATI_vertex_streams')

# GL/glext.h:5974
glVertexBlendEnviATI = _link_function('glVertexBlendEnviATI', None, [GLenum, GLint], 'ATI_vertex_streams')

# GL/glext.h:5975
glVertexBlendEnvfATI = _link_function('glVertexBlendEnvfATI', None, [GLenum, GLfloat], 'ATI_vertex_streams')

PFNGLVERTEXSTREAM1SATIPROC = CFUNCTYPE(None, GLenum, GLshort) 	# GL/glext.h:5977
PFNGLVERTEXSTREAM1SVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:5978
PFNGLVERTEXSTREAM1IATIPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:5979
PFNGLVERTEXSTREAM1IVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:5980
PFNGLVERTEXSTREAM1FATIPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:5981
PFNGLVERTEXSTREAM1FVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5982
PFNGLVERTEXSTREAM1DATIPROC = CFUNCTYPE(None, GLenum, GLdouble) 	# GL/glext.h:5983
PFNGLVERTEXSTREAM1DVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:5984
PFNGLVERTEXSTREAM2SATIPROC = CFUNCTYPE(None, GLenum, GLshort, GLshort) 	# GL/glext.h:5985
PFNGLVERTEXSTREAM2SVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:5986
PFNGLVERTEXSTREAM2IATIPROC = CFUNCTYPE(None, GLenum, GLint, GLint) 	# GL/glext.h:5987
PFNGLVERTEXSTREAM2IVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:5988
PFNGLVERTEXSTREAM2FATIPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat) 	# GL/glext.h:5989
PFNGLVERTEXSTREAM2FVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5990
PFNGLVERTEXSTREAM2DATIPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble) 	# GL/glext.h:5991
PFNGLVERTEXSTREAM2DVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:5992
PFNGLVERTEXSTREAM3SATIPROC = CFUNCTYPE(None, GLenum, GLshort, GLshort, GLshort) 	# GL/glext.h:5993
PFNGLVERTEXSTREAM3SVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:5994
PFNGLVERTEXSTREAM3IATIPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint) 	# GL/glext.h:5995
PFNGLVERTEXSTREAM3IVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:5996
PFNGLVERTEXSTREAM3FATIPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5997
PFNGLVERTEXSTREAM3FVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5998
PFNGLVERTEXSTREAM3DATIPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:5999
PFNGLVERTEXSTREAM3DVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:6000
PFNGLVERTEXSTREAM4SATIPROC = CFUNCTYPE(None, GLenum, GLshort, GLshort, GLshort, GLshort) 	# GL/glext.h:6001
PFNGLVERTEXSTREAM4SVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:6002
PFNGLVERTEXSTREAM4IATIPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint) 	# GL/glext.h:6003
PFNGLVERTEXSTREAM4IVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:6004
PFNGLVERTEXSTREAM4FATIPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6005
PFNGLVERTEXSTREAM4FVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6006
PFNGLVERTEXSTREAM4DATIPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:6007
PFNGLVERTEXSTREAM4DVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:6008
PFNGLNORMALSTREAM3BATIPROC = CFUNCTYPE(None, GLenum, GLbyte, GLbyte, GLbyte) 	# GL/glext.h:6009
PFNGLNORMALSTREAM3BVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLbyte)) 	# GL/glext.h:6010
PFNGLNORMALSTREAM3SATIPROC = CFUNCTYPE(None, GLenum, GLshort, GLshort, GLshort) 	# GL/glext.h:6011
PFNGLNORMALSTREAM3SVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:6012
PFNGLNORMALSTREAM3IATIPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint) 	# GL/glext.h:6013
PFNGLNORMALSTREAM3IVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:6014
PFNGLNORMALSTREAM3FATIPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6015
PFNGLNORMALSTREAM3FVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6016
PFNGLNORMALSTREAM3DATIPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:6017
PFNGLNORMALSTREAM3DVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:6018
PFNGLCLIENTACTIVEVERTEXSTREAMATIPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:6019
PFNGLVERTEXBLENDENVIATIPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:6020
PFNGLVERTEXBLENDENVFATIPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:6021
# ATI_element_array (GL/glext.h:6024)
GL_ATI_element_array = 1 	# GL/glext.h:6025
# GL/glext.h:6027
glElementPointerATI = _link_function('glElementPointerATI', None, [GLenum, POINTER(GLvoid)], 'ATI_element_array')

# GL/glext.h:6028
glDrawElementArrayATI = _link_function('glDrawElementArrayATI', None, [GLenum, GLsizei], 'ATI_element_array')

# GL/glext.h:6029
glDrawRangeElementArrayATI = _link_function('glDrawRangeElementArrayATI', None, [GLenum, GLuint, GLuint, GLsizei], 'ATI_element_array')

PFNGLELEMENTPOINTERATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLvoid)) 	# GL/glext.h:6031
PFNGLDRAWELEMENTARRAYATIPROC = CFUNCTYPE(None, GLenum, GLsizei) 	# GL/glext.h:6032
PFNGLDRAWRANGEELEMENTARRAYATIPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLsizei) 	# GL/glext.h:6033
# SUN_mesh_array (GL/glext.h:6036)
GL_SUN_mesh_array = 1 	# GL/glext.h:6037
# GL/glext.h:6039
glDrawMeshArraysSUN = _link_function('glDrawMeshArraysSUN', None, [GLenum, GLint, GLsizei, GLsizei], 'SUN_mesh_array')

PFNGLDRAWMESHARRAYSSUNPROC = CFUNCTYPE(None, GLenum, GLint, GLsizei, GLsizei) 	# GL/glext.h:6041
# SUN_slice_accum (GL/glext.h:6044)
GL_SUN_slice_accum = 1 	# GL/glext.h:6045
# NV_multisample_filter_hint (GL/glext.h:6048)
GL_NV_multisample_filter_hint = 1 	# GL/glext.h:6049
# NV_depth_clamp (GL/glext.h:6052)
GL_NV_depth_clamp = 1 	# GL/glext.h:6053
# NV_occlusion_query (GL/glext.h:6056)
GL_NV_occlusion_query = 1 	# GL/glext.h:6057
# GL/glext.h:6059
glGenOcclusionQueriesNV = _link_function('glGenOcclusionQueriesNV', None, [GLsizei, POINTER(GLuint)], 'NV_occlusion_query')

# GL/glext.h:6060
glDeleteOcclusionQueriesNV = _link_function('glDeleteOcclusionQueriesNV', None, [GLsizei, POINTER(GLuint)], 'NV_occlusion_query')

# GL/glext.h:6061
glIsOcclusionQueryNV = _link_function('glIsOcclusionQueryNV', GLboolean, [GLuint], 'NV_occlusion_query')

# GL/glext.h:6062
glBeginOcclusionQueryNV = _link_function('glBeginOcclusionQueryNV', None, [GLuint], 'NV_occlusion_query')

# GL/glext.h:6063
glEndOcclusionQueryNV = _link_function('glEndOcclusionQueryNV', None, [], 'NV_occlusion_query')

# GL/glext.h:6064
glGetOcclusionQueryivNV = _link_function('glGetOcclusionQueryivNV', None, [GLuint, GLenum, POINTER(GLint)], 'NV_occlusion_query')

# GL/glext.h:6065
glGetOcclusionQueryuivNV = _link_function('glGetOcclusionQueryuivNV', None, [GLuint, GLenum, POINTER(GLuint)], 'NV_occlusion_query')

PFNGLGENOCCLUSIONQUERIESNVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6067
PFNGLDELETEOCCLUSIONQUERIESNVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6068
PFNGLISOCCLUSIONQUERYNVPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:6069
PFNGLBEGINOCCLUSIONQUERYNVPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6070
PFNGLENDOCCLUSIONQUERYNVPROC = CFUNCTYPE(None) 	# GL/glext.h:6071
PFNGLGETOCCLUSIONQUERYIVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:6072
PFNGLGETOCCLUSIONQUERYUIVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLuint)) 	# GL/glext.h:6073
# NV_point_sprite (GL/glext.h:6076)
GL_NV_point_sprite = 1 	# GL/glext.h:6077
# GL/glext.h:6079
glPointParameteriNV = _link_function('glPointParameteriNV', None, [GLenum, GLint], 'NV_point_sprite')

# GL/glext.h:6080
glPointParameterivNV = _link_function('glPointParameterivNV', None, [GLenum, POINTER(GLint)], 'NV_point_sprite')

PFNGLPOINTPARAMETERINVPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:6082
PFNGLPOINTPARAMETERIVNVPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:6083
# NV_texture_shader3 (GL/glext.h:6086)
GL_NV_texture_shader3 = 1 	# GL/glext.h:6087
# NV_vertex_program1_1 (GL/glext.h:6090)
GL_NV_vertex_program1_1 = 1 	# GL/glext.h:6091
# EXT_shadow_funcs (GL/glext.h:6094)
GL_EXT_shadow_funcs = 1 	# GL/glext.h:6095
# EXT_stencil_two_side (GL/glext.h:6098)
GL_EXT_stencil_two_side = 1 	# GL/glext.h:6099
# GL/glext.h:6101
glActiveStencilFaceEXT = _link_function('glActiveStencilFaceEXT', None, [GLenum], 'EXT_stencil_two_side')

PFNGLACTIVESTENCILFACEEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:6103
# ATI_text_fragment_shader (GL/glext.h:6106)
GL_ATI_text_fragment_shader = 1 	# GL/glext.h:6107
# APPLE_client_storage (GL/glext.h:6110)
GL_APPLE_client_storage = 1 	# GL/glext.h:6111
# APPLE_element_array (GL/glext.h:6114)
GL_APPLE_element_array = 1 	# GL/glext.h:6115
# GL/glext.h:6117
glElementPointerAPPLE = _link_function('glElementPointerAPPLE', None, [GLenum, POINTER(GLvoid)], 'APPLE_element_array')

# GL/glext.h:6118
glDrawElementArrayAPPLE = _link_function('glDrawElementArrayAPPLE', None, [GLenum, GLint, GLsizei], 'APPLE_element_array')

# GL/glext.h:6119
glDrawRangeElementArrayAPPLE = _link_function('glDrawRangeElementArrayAPPLE', None, [GLenum, GLuint, GLuint, GLint, GLsizei], 'APPLE_element_array')

# GL/glext.h:6120
glMultiDrawElementArrayAPPLE = _link_function('glMultiDrawElementArrayAPPLE', None, [GLenum, POINTER(GLint), POINTER(GLsizei), GLsizei], 'APPLE_element_array')

# GL/glext.h:6121
glMultiDrawRangeElementArrayAPPLE = _link_function('glMultiDrawRangeElementArrayAPPLE', None, [GLenum, GLuint, GLuint, POINTER(GLint), POINTER(GLsizei), GLsizei], 'APPLE_element_array')

PFNGLELEMENTPOINTERAPPLEPROC = CFUNCTYPE(None, GLenum, POINTER(GLvoid)) 	# GL/glext.h:6123
PFNGLDRAWELEMENTARRAYAPPLEPROC = CFUNCTYPE(None, GLenum, GLint, GLsizei) 	# GL/glext.h:6124
PFNGLDRAWRANGEELEMENTARRAYAPPLEPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLint, GLsizei) 	# GL/glext.h:6125
PFNGLMULTIDRAWELEMENTARRAYAPPLEPROC = CFUNCTYPE(None, GLenum, POINTER(GLint), POINTER(GLsizei), GLsizei) 	# GL/glext.h:6126
PFNGLMULTIDRAWRANGEELEMENTARRAYAPPLEPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, POINTER(GLint), POINTER(GLsizei), GLsizei) 	# GL/glext.h:6127
# APPLE_fence (GL/glext.h:6130)
GL_APPLE_fence = 1 	# GL/glext.h:6131
# GL/glext.h:6133
glGenFencesAPPLE = _link_function('glGenFencesAPPLE', None, [GLsizei, POINTER(GLuint)], 'APPLE_fence')

# GL/glext.h:6134
glDeleteFencesAPPLE = _link_function('glDeleteFencesAPPLE', None, [GLsizei, POINTER(GLuint)], 'APPLE_fence')

# GL/glext.h:6135
glSetFenceAPPLE = _link_function('glSetFenceAPPLE', None, [GLuint], 'APPLE_fence')

# GL/glext.h:6136
glIsFenceAPPLE = _link_function('glIsFenceAPPLE', GLboolean, [GLuint], 'APPLE_fence')

# GL/glext.h:6137
glTestFenceAPPLE = _link_function('glTestFenceAPPLE', GLboolean, [GLuint], 'APPLE_fence')

# GL/glext.h:6138
glFinishFenceAPPLE = _link_function('glFinishFenceAPPLE', None, [GLuint], 'APPLE_fence')

# GL/glext.h:6139
glTestObjectAPPLE = _link_function('glTestObjectAPPLE', GLboolean, [GLenum, GLuint], 'APPLE_fence')

# GL/glext.h:6140
glFinishObjectAPPLE = _link_function('glFinishObjectAPPLE', None, [GLenum, GLint], 'APPLE_fence')

PFNGLGENFENCESAPPLEPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6142
PFNGLDELETEFENCESAPPLEPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6143
PFNGLSETFENCEAPPLEPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6144
PFNGLISFENCEAPPLEPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:6145
PFNGLTESTFENCEAPPLEPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:6146
PFNGLFINISHFENCEAPPLEPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6147
PFNGLTESTOBJECTAPPLEPROC = CFUNCTYPE(GLboolean, GLenum, GLuint) 	# GL/glext.h:6148
PFNGLFINISHOBJECTAPPLEPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:6149
# APPLE_vertex_array_object (GL/glext.h:6152)
GL_APPLE_vertex_array_object = 1 	# GL/glext.h:6153
# GL/glext.h:6155
glBindVertexArrayAPPLE = _link_function('glBindVertexArrayAPPLE', None, [GLuint], 'APPLE_vertex_array_object')

# GL/glext.h:6156
glDeleteVertexArraysAPPLE = _link_function('glDeleteVertexArraysAPPLE', None, [GLsizei, POINTER(GLuint)], 'APPLE_vertex_array_object')

# GL/glext.h:6157
glGenVertexArraysAPPLE = _link_function('glGenVertexArraysAPPLE', None, [GLsizei, POINTER(GLuint)], 'APPLE_vertex_array_object')

# GL/glext.h:6158
glIsVertexArrayAPPLE = _link_function('glIsVertexArrayAPPLE', GLboolean, [GLuint], 'APPLE_vertex_array_object')

PFNGLBINDVERTEXARRAYAPPLEPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6160
PFNGLDELETEVERTEXARRAYSAPPLEPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6161
PFNGLGENVERTEXARRAYSAPPLEPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6162
PFNGLISVERTEXARRAYAPPLEPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:6163
# APPLE_vertex_array_range (GL/glext.h:6166)
GL_APPLE_vertex_array_range = 1 	# GL/glext.h:6167
# GL/glext.h:6169
glVertexArrayRangeAPPLE = _link_function('glVertexArrayRangeAPPLE', None, [GLsizei, POINTER(GLvoid)], 'APPLE_vertex_array_range')

# GL/glext.h:6170
glFlushVertexArrayRangeAPPLE = _link_function('glFlushVertexArrayRangeAPPLE', None, [GLsizei, POINTER(GLvoid)], 'APPLE_vertex_array_range')

# GL/glext.h:6171
glVertexArrayParameteriAPPLE = _link_function('glVertexArrayParameteriAPPLE', None, [GLenum, GLint], 'APPLE_vertex_array_range')

PFNGLVERTEXARRAYRANGEAPPLEPROC = CFUNCTYPE(None, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:6173
PFNGLFLUSHVERTEXARRAYRANGEAPPLEPROC = CFUNCTYPE(None, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:6174
PFNGLVERTEXARRAYPARAMETERIAPPLEPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:6175
# APPLE_ycbcr_422 (GL/glext.h:6178)
GL_APPLE_ycbcr_422 = 1 	# GL/glext.h:6179
# S3_s3tc (GL/glext.h:6182)
GL_S3_s3tc = 1 	# GL/glext.h:6183
# ATI_draw_buffers (GL/glext.h:6186)
GL_ATI_draw_buffers = 1 	# GL/glext.h:6187
# GL/glext.h:6189
glDrawBuffersATI = _link_function('glDrawBuffersATI', None, [GLsizei, POINTER(GLenum)], 'ATI_draw_buffers')

PFNGLDRAWBUFFERSATIPROC = CFUNCTYPE(None, GLsizei, POINTER(GLenum)) 	# GL/glext.h:6191
# ATI_pixel_format_float (GL/glext.h:6194)
GL_ATI_pixel_format_float = 1 	# GL/glext.h:6195
# ATI_texture_env_combine3 (GL/glext.h:6201)
GL_ATI_texture_env_combine3 = 1 	# GL/glext.h:6202
# ATI_texture_float (GL/glext.h:6205)
GL_ATI_texture_float = 1 	# GL/glext.h:6206
# NV_float_buffer (GL/glext.h:6209)
GL_NV_float_buffer = 1 	# GL/glext.h:6210
# NV_fragment_program (GL/glext.h:6213)
GL_NV_fragment_program = 1 	# GL/glext.h:6214
# GL/glext.h:6217
glProgramNamedParameter4fNV = _link_function('glProgramNamedParameter4fNV', None, [GLuint, GLsizei, POINTER(GLubyte), GLfloat, GLfloat, GLfloat, GLfloat], 'NV_fragment_program')

# GL/glext.h:6218
glProgramNamedParameter4dNV = _link_function('glProgramNamedParameter4dNV', None, [GLuint, GLsizei, POINTER(GLubyte), GLdouble, GLdouble, GLdouble, GLdouble], 'NV_fragment_program')

# GL/glext.h:6219
glProgramNamedParameter4fvNV = _link_function('glProgramNamedParameter4fvNV', None, [GLuint, GLsizei, POINTER(GLubyte), POINTER(GLfloat)], 'NV_fragment_program')

# GL/glext.h:6220
glProgramNamedParameter4dvNV = _link_function('glProgramNamedParameter4dvNV', None, [GLuint, GLsizei, POINTER(GLubyte), POINTER(GLdouble)], 'NV_fragment_program')

# GL/glext.h:6221
glGetProgramNamedParameterfvNV = _link_function('glGetProgramNamedParameterfvNV', None, [GLuint, GLsizei, POINTER(GLubyte), POINTER(GLfloat)], 'NV_fragment_program')

# GL/glext.h:6222
glGetProgramNamedParameterdvNV = _link_function('glGetProgramNamedParameterdvNV', None, [GLuint, GLsizei, POINTER(GLubyte), POINTER(GLdouble)], 'NV_fragment_program')

PFNGLPROGRAMNAMEDPARAMETER4FNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLubyte), GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6224
PFNGLPROGRAMNAMEDPARAMETER4DNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLubyte), GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:6225
PFNGLPROGRAMNAMEDPARAMETER4FVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLubyte), POINTER(GLfloat)) 	# GL/glext.h:6226
PFNGLPROGRAMNAMEDPARAMETER4DVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLubyte), POINTER(GLdouble)) 	# GL/glext.h:6227
PFNGLGETPROGRAMNAMEDPARAMETERFVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLubyte), POINTER(GLfloat)) 	# GL/glext.h:6228
PFNGLGETPROGRAMNAMEDPARAMETERDVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLubyte), POINTER(GLdouble)) 	# GL/glext.h:6229
# NV_half_float (GL/glext.h:6232)
GL_NV_half_float = 1 	# GL/glext.h:6233
# GL/glext.h:6235
glVertex2hNV = _link_function('glVertex2hNV', None, [GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:6236
glVertex2hvNV = _link_function('glVertex2hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6237
glVertex3hNV = _link_function('glVertex3hNV', None, [GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:6238
glVertex3hvNV = _link_function('glVertex3hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6239
glVertex4hNV = _link_function('glVertex4hNV', None, [GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:6240
glVertex4hvNV = _link_function('glVertex4hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6241
glNormal3hNV = _link_function('glNormal3hNV', None, [GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:6242
glNormal3hvNV = _link_function('glNormal3hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6243
glColor3hNV = _link_function('glColor3hNV', None, [GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:6244
glColor3hvNV = _link_function('glColor3hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6245
glColor4hNV = _link_function('glColor4hNV', None, [GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:6246
glColor4hvNV = _link_function('glColor4hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6247
glTexCoord1hNV = _link_function('glTexCoord1hNV', None, [GLhalfNV], 'NV_half_float')

# GL/glext.h:6248
glTexCoord1hvNV = _link_function('glTexCoord1hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6249
glTexCoord2hNV = _link_function('glTexCoord2hNV', None, [GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:6250
glTexCoord2hvNV = _link_function('glTexCoord2hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6251
glTexCoord3hNV = _link_function('glTexCoord3hNV', None, [GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:6252
glTexCoord3hvNV = _link_function('glTexCoord3hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6253
glTexCoord4hNV = _link_function('glTexCoord4hNV', None, [GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:6254
glTexCoord4hvNV = _link_function('glTexCoord4hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6255
glMultiTexCoord1hNV = _link_function('glMultiTexCoord1hNV', None, [GLenum, GLhalfNV], 'NV_half_float')

# GL/glext.h:6256
glMultiTexCoord1hvNV = _link_function('glMultiTexCoord1hvNV', None, [GLenum, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6257
glMultiTexCoord2hNV = _link_function('glMultiTexCoord2hNV', None, [GLenum, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:6258
glMultiTexCoord2hvNV = _link_function('glMultiTexCoord2hvNV', None, [GLenum, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6259
glMultiTexCoord3hNV = _link_function('glMultiTexCoord3hNV', None, [GLenum, GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:6260
glMultiTexCoord3hvNV = _link_function('glMultiTexCoord3hvNV', None, [GLenum, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6261
glMultiTexCoord4hNV = _link_function('glMultiTexCoord4hNV', None, [GLenum, GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:6262
glMultiTexCoord4hvNV = _link_function('glMultiTexCoord4hvNV', None, [GLenum, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6263
glFogCoordhNV = _link_function('glFogCoordhNV', None, [GLhalfNV], 'NV_half_float')

# GL/glext.h:6264
glFogCoordhvNV = _link_function('glFogCoordhvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6265
glSecondaryColor3hNV = _link_function('glSecondaryColor3hNV', None, [GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:6266
glSecondaryColor3hvNV = _link_function('glSecondaryColor3hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6267
glVertexWeighthNV = _link_function('glVertexWeighthNV', None, [GLhalfNV], 'NV_half_float')

# GL/glext.h:6268
glVertexWeighthvNV = _link_function('glVertexWeighthvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6269
glVertexAttrib1hNV = _link_function('glVertexAttrib1hNV', None, [GLuint, GLhalfNV], 'NV_half_float')

# GL/glext.h:6270
glVertexAttrib1hvNV = _link_function('glVertexAttrib1hvNV', None, [GLuint, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6271
glVertexAttrib2hNV = _link_function('glVertexAttrib2hNV', None, [GLuint, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:6272
glVertexAttrib2hvNV = _link_function('glVertexAttrib2hvNV', None, [GLuint, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6273
glVertexAttrib3hNV = _link_function('glVertexAttrib3hNV', None, [GLuint, GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:6274
glVertexAttrib3hvNV = _link_function('glVertexAttrib3hvNV', None, [GLuint, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6275
glVertexAttrib4hNV = _link_function('glVertexAttrib4hNV', None, [GLuint, GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:6276
glVertexAttrib4hvNV = _link_function('glVertexAttrib4hvNV', None, [GLuint, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6277
glVertexAttribs1hvNV = _link_function('glVertexAttribs1hvNV', None, [GLuint, GLsizei, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6278
glVertexAttribs2hvNV = _link_function('glVertexAttribs2hvNV', None, [GLuint, GLsizei, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6279
glVertexAttribs3hvNV = _link_function('glVertexAttribs3hvNV', None, [GLuint, GLsizei, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6280
glVertexAttribs4hvNV = _link_function('glVertexAttribs4hvNV', None, [GLuint, GLsizei, POINTER(GLhalfNV)], 'NV_half_float')

PFNGLVERTEX2HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV) 	# GL/glext.h:6282
PFNGLVERTEX2HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:6283
PFNGLVERTEX3HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:6284
PFNGLVERTEX3HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:6285
PFNGLVERTEX4HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:6286
PFNGLVERTEX4HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:6287
PFNGLNORMAL3HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:6288
PFNGLNORMAL3HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:6289
PFNGLCOLOR3HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:6290
PFNGLCOLOR3HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:6291
PFNGLCOLOR4HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:6292
PFNGLCOLOR4HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:6293
PFNGLTEXCOORD1HNVPROC = CFUNCTYPE(None, GLhalfNV) 	# GL/glext.h:6294
PFNGLTEXCOORD1HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:6295
PFNGLTEXCOORD2HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV) 	# GL/glext.h:6296
PFNGLTEXCOORD2HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:6297
PFNGLTEXCOORD3HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:6298
PFNGLTEXCOORD3HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:6299
PFNGLTEXCOORD4HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:6300
PFNGLTEXCOORD4HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:6301
PFNGLMULTITEXCOORD1HNVPROC = CFUNCTYPE(None, GLenum, GLhalfNV) 	# GL/glext.h:6302
PFNGLMULTITEXCOORD1HVNVPROC = CFUNCTYPE(None, GLenum, POINTER(GLhalfNV)) 	# GL/glext.h:6303
PFNGLMULTITEXCOORD2HNVPROC = CFUNCTYPE(None, GLenum, GLhalfNV, GLhalfNV) 	# GL/glext.h:6304
PFNGLMULTITEXCOORD2HVNVPROC = CFUNCTYPE(None, GLenum, POINTER(GLhalfNV)) 	# GL/glext.h:6305
PFNGLMULTITEXCOORD3HNVPROC = CFUNCTYPE(None, GLenum, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:6306
PFNGLMULTITEXCOORD3HVNVPROC = CFUNCTYPE(None, GLenum, POINTER(GLhalfNV)) 	# GL/glext.h:6307
PFNGLMULTITEXCOORD4HNVPROC = CFUNCTYPE(None, GLenum, GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:6308
PFNGLMULTITEXCOORD4HVNVPROC = CFUNCTYPE(None, GLenum, POINTER(GLhalfNV)) 	# GL/glext.h:6309
PFNGLFOGCOORDHNVPROC = CFUNCTYPE(None, GLhalfNV) 	# GL/glext.h:6310
PFNGLFOGCOORDHVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:6311
PFNGLSECONDARYCOLOR3HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:6312
PFNGLSECONDARYCOLOR3HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:6313
PFNGLVERTEXWEIGHTHNVPROC = CFUNCTYPE(None, GLhalfNV) 	# GL/glext.h:6314
PFNGLVERTEXWEIGHTHVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:6315
PFNGLVERTEXATTRIB1HNVPROC = CFUNCTYPE(None, GLuint, GLhalfNV) 	# GL/glext.h:6316
PFNGLVERTEXATTRIB1HVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLhalfNV)) 	# GL/glext.h:6317
PFNGLVERTEXATTRIB2HNVPROC = CFUNCTYPE(None, GLuint, GLhalfNV, GLhalfNV) 	# GL/glext.h:6318
PFNGLVERTEXATTRIB2HVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLhalfNV)) 	# GL/glext.h:6319
PFNGLVERTEXATTRIB3HNVPROC = CFUNCTYPE(None, GLuint, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:6320
PFNGLVERTEXATTRIB3HVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLhalfNV)) 	# GL/glext.h:6321
PFNGLVERTEXATTRIB4HNVPROC = CFUNCTYPE(None, GLuint, GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:6322
PFNGLVERTEXATTRIB4HVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLhalfNV)) 	# GL/glext.h:6323
PFNGLVERTEXATTRIBS1HVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLhalfNV)) 	# GL/glext.h:6324
PFNGLVERTEXATTRIBS2HVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLhalfNV)) 	# GL/glext.h:6325
PFNGLVERTEXATTRIBS3HVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLhalfNV)) 	# GL/glext.h:6326
PFNGLVERTEXATTRIBS4HVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLhalfNV)) 	# GL/glext.h:6327
# NV_pixel_data_range (GL/glext.h:6330)
GL_NV_pixel_data_range = 1 	# GL/glext.h:6331
# GL/glext.h:6333
glPixelDataRangeNV = _link_function('glPixelDataRangeNV', None, [GLenum, GLsizei, POINTER(GLvoid)], 'NV_pixel_data_range')

# GL/glext.h:6334
glFlushPixelDataRangeNV = _link_function('glFlushPixelDataRangeNV', None, [GLenum], 'NV_pixel_data_range')

PFNGLPIXELDATARANGENVPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:6336
PFNGLFLUSHPIXELDATARANGENVPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:6337
# NV_primitive_restart (GL/glext.h:6340)
GL_NV_primitive_restart = 1 	# GL/glext.h:6341
# GL/glext.h:6343
glPrimitiveRestartNV = _link_function('glPrimitiveRestartNV', None, [], 'NV_primitive_restart')

# GL/glext.h:6344
glPrimitiveRestartIndexNV = _link_function('glPrimitiveRestartIndexNV', None, [GLuint], 'NV_primitive_restart')

PFNGLPRIMITIVERESTARTNVPROC = CFUNCTYPE(None) 	# GL/glext.h:6346
PFNGLPRIMITIVERESTARTINDEXNVPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6347
# NV_texture_expand_normal (GL/glext.h:6350)
GL_NV_texture_expand_normal = 1 	# GL/glext.h:6351
# NV_vertex_program2 (GL/glext.h:6354)
GL_NV_vertex_program2 = 1 	# GL/glext.h:6355
# ATI_map_object_buffer (GL/glext.h:6358)
GL_ATI_map_object_buffer = 1 	# GL/glext.h:6359
# GL/glext.h:6361
glMapObjectBufferATI = _link_function('glMapObjectBufferATI', POINTER(GLvoid), [GLuint], 'ATI_map_object_buffer')

# GL/glext.h:6362
glUnmapObjectBufferATI = _link_function('glUnmapObjectBufferATI', None, [GLuint], 'ATI_map_object_buffer')

PFNGLMAPOBJECTBUFFERATIPROC = CFUNCTYPE(POINTER(GLvoid), GLuint) 	# GL/glext.h:6364
PFNGLUNMAPOBJECTBUFFERATIPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6365
# ATI_separate_stencil (GL/glext.h:6368)
GL_ATI_separate_stencil = 1 	# GL/glext.h:6369
# GL/glext.h:6371
glStencilOpSeparateATI = _link_function('glStencilOpSeparateATI', None, [GLenum, GLenum, GLenum, GLenum], 'ATI_separate_stencil')

# GL/glext.h:6372
glStencilFuncSeparateATI = _link_function('glStencilFuncSeparateATI', None, [GLenum, GLenum, GLint, GLuint], 'ATI_separate_stencil')

PFNGLSTENCILOPSEPARATEATIPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:6374
PFNGLSTENCILFUNCSEPARATEATIPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLuint) 	# GL/glext.h:6375
# ATI_vertex_attrib_array_object (GL/glext.h:6378)
GL_ATI_vertex_attrib_array_object = 1 	# GL/glext.h:6379
# GL/glext.h:6381
glVertexAttribArrayObjectATI = _link_function('glVertexAttribArrayObjectATI', None, [GLuint, GLint, GLenum, GLboolean, GLsizei, GLuint, GLuint], 'ATI_vertex_attrib_array_object')

# GL/glext.h:6382
glGetVertexAttribArrayObjectfvATI = _link_function('glGetVertexAttribArrayObjectfvATI', None, [GLuint, GLenum, POINTER(GLfloat)], 'ATI_vertex_attrib_array_object')

# GL/glext.h:6383
glGetVertexAttribArrayObjectivATI = _link_function('glGetVertexAttribArrayObjectivATI', None, [GLuint, GLenum, POINTER(GLint)], 'ATI_vertex_attrib_array_object')

PFNGLVERTEXATTRIBARRAYOBJECTATIPROC = CFUNCTYPE(None, GLuint, GLint, GLenum, GLboolean, GLsizei, GLuint, GLuint) 	# GL/glext.h:6385
PFNGLGETVERTEXATTRIBARRAYOBJECTFVATIPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6386
PFNGLGETVERTEXATTRIBARRAYOBJECTIVATIPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:6387
# OES_read_format (GL/glext.h:6390)
GL_OES_read_format = 1 	# GL/glext.h:6391
# EXT_depth_bounds_test (GL/glext.h:6394)
GL_EXT_depth_bounds_test = 1 	# GL/glext.h:6395
GLclampd = c_double 	# /usr/include/GL/gl.h:66
# GL/glext.h:6397
glDepthBoundsEXT = _link_function('glDepthBoundsEXT', None, [GLclampd, GLclampd], 'EXT_depth_bounds_test')

PFNGLDEPTHBOUNDSEXTPROC = CFUNCTYPE(None, GLclampd, GLclampd) 	# GL/glext.h:6399
# EXT_texture_mirror_clamp (GL/glext.h:6402)
GL_EXT_texture_mirror_clamp = 1 	# GL/glext.h:6403
# EXT_blend_equation_separate (GL/glext.h:6406)
GL_EXT_blend_equation_separate = 1 	# GL/glext.h:6407
# GL/glext.h:6409
glBlendEquationSeparateEXT = _link_function('glBlendEquationSeparateEXT', None, [GLenum, GLenum], 'EXT_blend_equation_separate')

PFNGLBLENDEQUATIONSEPARATEEXTPROC = CFUNCTYPE(None, GLenum, GLenum) 	# GL/glext.h:6411
# MESA_pack_invert (GL/glext.h:6414)
GL_MESA_pack_invert = 1 	# GL/glext.h:6415
# MESA_ycbcr_texture (GL/glext.h:6418)
GL_MESA_ycbcr_texture = 1 	# GL/glext.h:6419
# EXT_pixel_buffer_object (GL/glext.h:6422)
GL_EXT_pixel_buffer_object = 1 	# GL/glext.h:6423
# NV_fragment_program_option (GL/glext.h:6426)
GL_NV_fragment_program_option = 1 	# GL/glext.h:6427
# NV_fragment_program2 (GL/glext.h:6430)
GL_NV_fragment_program2 = 1 	# GL/glext.h:6431
# NV_vertex_program2_option (GL/glext.h:6434)
GL_NV_vertex_program2_option = 1 	# GL/glext.h:6435
# NV_vertex_program3 (GL/glext.h:6438)
GL_NV_vertex_program3 = 1 	# GL/glext.h:6439
# EXT_framebuffer_object (GL/glext.h:6442)
GL_EXT_framebuffer_object = 1 	# GL/glext.h:6443
# GL/glext.h:6445
glIsRenderbufferEXT = _link_function('glIsRenderbufferEXT', GLboolean, [GLuint], 'EXT_framebuffer_object')

# GL/glext.h:6446
glBindRenderbufferEXT = _link_function('glBindRenderbufferEXT', None, [GLenum, GLuint], 'EXT_framebuffer_object')

# GL/glext.h:6447
glDeleteRenderbuffersEXT = _link_function('glDeleteRenderbuffersEXT', None, [GLsizei, POINTER(GLuint)], 'EXT_framebuffer_object')

# GL/glext.h:6448
glGenRenderbuffersEXT = _link_function('glGenRenderbuffersEXT', None, [GLsizei, POINTER(GLuint)], 'EXT_framebuffer_object')

# GL/glext.h:6449
glRenderbufferStorageEXT = _link_function('glRenderbufferStorageEXT', None, [GLenum, GLenum, GLsizei, GLsizei], 'EXT_framebuffer_object')

# GL/glext.h:6450
glGetRenderbufferParameterivEXT = _link_function('glGetRenderbufferParameterivEXT', None, [GLenum, GLenum, POINTER(GLint)], 'EXT_framebuffer_object')

# GL/glext.h:6451
glIsFramebufferEXT = _link_function('glIsFramebufferEXT', GLboolean, [GLuint], 'EXT_framebuffer_object')

# GL/glext.h:6452
glBindFramebufferEXT = _link_function('glBindFramebufferEXT', None, [GLenum, GLuint], 'EXT_framebuffer_object')

# GL/glext.h:6453
glDeleteFramebuffersEXT = _link_function('glDeleteFramebuffersEXT', None, [GLsizei, POINTER(GLuint)], 'EXT_framebuffer_object')

# GL/glext.h:6454
glGenFramebuffersEXT = _link_function('glGenFramebuffersEXT', None, [GLsizei, POINTER(GLuint)], 'EXT_framebuffer_object')

# GL/glext.h:6455
glCheckFramebufferStatusEXT = _link_function('glCheckFramebufferStatusEXT', GLenum, [GLenum], 'EXT_framebuffer_object')

# GL/glext.h:6456
glFramebufferTexture1DEXT = _link_function('glFramebufferTexture1DEXT', None, [GLenum, GLenum, GLenum, GLuint, GLint], 'EXT_framebuffer_object')

# GL/glext.h:6457
glFramebufferTexture2DEXT = _link_function('glFramebufferTexture2DEXT', None, [GLenum, GLenum, GLenum, GLuint, GLint], 'EXT_framebuffer_object')

# GL/glext.h:6458
glFramebufferTexture3DEXT = _link_function('glFramebufferTexture3DEXT', None, [GLenum, GLenum, GLenum, GLuint, GLint, GLint], 'EXT_framebuffer_object')

# GL/glext.h:6459
glFramebufferRenderbufferEXT = _link_function('glFramebufferRenderbufferEXT', None, [GLenum, GLenum, GLenum, GLuint], 'EXT_framebuffer_object')

# GL/glext.h:6460
glGetFramebufferAttachmentParameterivEXT = _link_function('glGetFramebufferAttachmentParameterivEXT', None, [GLenum, GLenum, GLenum, POINTER(GLint)], 'EXT_framebuffer_object')

# GL/glext.h:6461
glGenerateMipmapEXT = _link_function('glGenerateMipmapEXT', None, [GLenum], 'EXT_framebuffer_object')

PFNGLISRENDERBUFFEREXTPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:6463
PFNGLBINDRENDERBUFFEREXTPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:6464
PFNGLDELETERENDERBUFFERSEXTPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6465
PFNGLGENRENDERBUFFERSEXTPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6466
PFNGLRENDERBUFFERSTORAGEEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLsizei) 	# GL/glext.h:6467
PFNGLGETRENDERBUFFERPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:6468
PFNGLISFRAMEBUFFEREXTPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:6469
PFNGLBINDFRAMEBUFFEREXTPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:6470
PFNGLDELETEFRAMEBUFFERSEXTPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6471
PFNGLGENFRAMEBUFFERSEXTPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6472
PFNGLCHECKFRAMEBUFFERSTATUSEXTPROC = CFUNCTYPE(GLenum, GLenum) 	# GL/glext.h:6473
PFNGLFRAMEBUFFERTEXTURE1DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLuint, GLint) 	# GL/glext.h:6474
PFNGLFRAMEBUFFERTEXTURE2DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLuint, GLint) 	# GL/glext.h:6475
PFNGLFRAMEBUFFERTEXTURE3DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLuint, GLint, GLint) 	# GL/glext.h:6476
PFNGLFRAMEBUFFERRENDERBUFFEREXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLuint) 	# GL/glext.h:6477
PFNGLGETFRAMEBUFFERATTACHMENTPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:6478
PFNGLGENERATEMIPMAPEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:6479
# GREMEDY_string_marker (GL/glext.h:6482)
GL_GREMEDY_string_marker = 1 	# GL/glext.h:6483
# GL/glext.h:6485
glStringMarkerGREMEDY = _link_function('glStringMarkerGREMEDY', None, [GLsizei, POINTER(GLvoid)], 'GREMEDY_string_marker')

PFNGLSTRINGMARKERGREMEDYPROC = CFUNCTYPE(None, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:6487
# GLEXT_LEGACY (/usr/include/GL/gl.h:1633)

__all__ = ['GL_GLEXT_VERSION', 'GL_UNSIGNED_BYTE_3_3_2',
'GL_UNSIGNED_SHORT_4_4_4_4', 'GL_UNSIGNED_SHORT_5_5_5_1',
'GL_UNSIGNED_INT_8_8_8_8', 'GL_UNSIGNED_INT_10_10_10_2', 'GL_RESCALE_NORMAL',
'GL_TEXTURE_BINDING_3D', 'GL_PACK_SKIP_IMAGES', 'GL_PACK_IMAGE_HEIGHT',
'GL_UNPACK_SKIP_IMAGES', 'GL_UNPACK_IMAGE_HEIGHT', 'GL_TEXTURE_3D',
'GL_PROXY_TEXTURE_3D', 'GL_TEXTURE_DEPTH', 'GL_TEXTURE_WRAP_R',
'GL_MAX_3D_TEXTURE_SIZE', 'GL_UNSIGNED_BYTE_2_3_3_REV',
'GL_UNSIGNED_SHORT_5_6_5', 'GL_UNSIGNED_SHORT_5_6_5_REV',
'GL_UNSIGNED_SHORT_4_4_4_4_REV', 'GL_UNSIGNED_SHORT_1_5_5_5_REV',
'GL_UNSIGNED_INT_8_8_8_8_REV', 'GL_UNSIGNED_INT_2_10_10_10_REV', 'GL_BGR',
'GL_BGRA', 'GL_MAX_ELEMENTS_VERTICES', 'GL_MAX_ELEMENTS_INDICES',
'GL_CLAMP_TO_EDGE', 'GL_TEXTURE_MIN_LOD', 'GL_TEXTURE_MAX_LOD',
'GL_TEXTURE_BASE_LEVEL', 'GL_TEXTURE_MAX_LEVEL',
'GL_LIGHT_MODEL_COLOR_CONTROL', 'GL_SINGLE_COLOR',
'GL_SEPARATE_SPECULAR_COLOR', 'GL_SMOOTH_POINT_SIZE_RANGE',
'GL_SMOOTH_POINT_SIZE_GRANULARITY', 'GL_SMOOTH_LINE_WIDTH_RANGE',
'GL_SMOOTH_LINE_WIDTH_GRANULARITY', 'GL_ALIASED_POINT_SIZE_RANGE',
'GL_ALIASED_LINE_WIDTH_RANGE', 'GL_CONSTANT_COLOR',
'GL_ONE_MINUS_CONSTANT_COLOR', 'GL_CONSTANT_ALPHA',
'GL_ONE_MINUS_CONSTANT_ALPHA', 'GL_BLEND_COLOR', 'GL_FUNC_ADD', 'GL_MIN',
'GL_MAX', 'GL_BLEND_EQUATION', 'GL_FUNC_SUBTRACT', 'GL_FUNC_REVERSE_SUBTRACT',
'GL_CONVOLUTION_1D', 'GL_CONVOLUTION_2D', 'GL_SEPARABLE_2D',
'GL_CONVOLUTION_BORDER_MODE', 'GL_CONVOLUTION_FILTER_SCALE',
'GL_CONVOLUTION_FILTER_BIAS', 'GL_REDUCE', 'GL_CONVOLUTION_FORMAT',
'GL_CONVOLUTION_WIDTH', 'GL_CONVOLUTION_HEIGHT', 'GL_MAX_CONVOLUTION_WIDTH',
'GL_MAX_CONVOLUTION_HEIGHT', 'GL_POST_CONVOLUTION_RED_SCALE',
'GL_POST_CONVOLUTION_GREEN_SCALE', 'GL_POST_CONVOLUTION_BLUE_SCALE',
'GL_POST_CONVOLUTION_ALPHA_SCALE', 'GL_POST_CONVOLUTION_RED_BIAS',
'GL_POST_CONVOLUTION_GREEN_BIAS', 'GL_POST_CONVOLUTION_BLUE_BIAS',
'GL_POST_CONVOLUTION_ALPHA_BIAS', 'GL_HISTOGRAM', 'GL_PROXY_HISTOGRAM',
'GL_HISTOGRAM_WIDTH', 'GL_HISTOGRAM_FORMAT', 'GL_HISTOGRAM_RED_SIZE',
'GL_HISTOGRAM_GREEN_SIZE', 'GL_HISTOGRAM_BLUE_SIZE',
'GL_HISTOGRAM_ALPHA_SIZE', 'GL_HISTOGRAM_LUMINANCE_SIZE', 'GL_HISTOGRAM_SINK',
'GL_MINMAX', 'GL_MINMAX_FORMAT', 'GL_MINMAX_SINK', 'GL_TABLE_TOO_LARGE',
'GL_COLOR_MATRIX', 'GL_COLOR_MATRIX_STACK_DEPTH',
'GL_MAX_COLOR_MATRIX_STACK_DEPTH', 'GL_POST_COLOR_MATRIX_RED_SCALE',
'GL_POST_COLOR_MATRIX_GREEN_SCALE', 'GL_POST_COLOR_MATRIX_BLUE_SCALE',
'GL_POST_COLOR_MATRIX_ALPHA_SCALE', 'GL_POST_COLOR_MATRIX_RED_BIAS',
'GL_POST_COLOR_MATRIX_GREEN_BIAS', 'GL_POST_COLOR_MATRIX_BLUE_BIAS',
'GL_POST_COLOR_MATRIX_ALPHA_BIAS', 'GL_COLOR_TABLE',
'GL_POST_CONVOLUTION_COLOR_TABLE', 'GL_POST_COLOR_MATRIX_COLOR_TABLE',
'GL_PROXY_COLOR_TABLE', 'GL_PROXY_POST_CONVOLUTION_COLOR_TABLE',
'GL_PROXY_POST_COLOR_MATRIX_COLOR_TABLE', 'GL_COLOR_TABLE_SCALE',
'GL_COLOR_TABLE_BIAS', 'GL_COLOR_TABLE_FORMAT', 'GL_COLOR_TABLE_WIDTH',
'GL_COLOR_TABLE_RED_SIZE', 'GL_COLOR_TABLE_GREEN_SIZE',
'GL_COLOR_TABLE_BLUE_SIZE', 'GL_COLOR_TABLE_ALPHA_SIZE',
'GL_COLOR_TABLE_LUMINANCE_SIZE', 'GL_COLOR_TABLE_INTENSITY_SIZE',
'GL_CONSTANT_BORDER', 'GL_REPLICATE_BORDER', 'GL_CONVOLUTION_BORDER_COLOR',
'GL_TEXTURE0', 'GL_TEXTURE1', 'GL_TEXTURE2', 'GL_TEXTURE3', 'GL_TEXTURE4',
'GL_TEXTURE5', 'GL_TEXTURE6', 'GL_TEXTURE7', 'GL_TEXTURE8', 'GL_TEXTURE9',
'GL_TEXTURE10', 'GL_TEXTURE11', 'GL_TEXTURE12', 'GL_TEXTURE13',
'GL_TEXTURE14', 'GL_TEXTURE15', 'GL_TEXTURE16', 'GL_TEXTURE17',
'GL_TEXTURE18', 'GL_TEXTURE19', 'GL_TEXTURE20', 'GL_TEXTURE21',
'GL_TEXTURE22', 'GL_TEXTURE23', 'GL_TEXTURE24', 'GL_TEXTURE25',
'GL_TEXTURE26', 'GL_TEXTURE27', 'GL_TEXTURE28', 'GL_TEXTURE29',
'GL_TEXTURE30', 'GL_TEXTURE31', 'GL_ACTIVE_TEXTURE',
'GL_CLIENT_ACTIVE_TEXTURE', 'GL_MAX_TEXTURE_UNITS',
'GL_TRANSPOSE_MODELVIEW_MATRIX', 'GL_TRANSPOSE_PROJECTION_MATRIX',
'GL_TRANSPOSE_TEXTURE_MATRIX', 'GL_TRANSPOSE_COLOR_MATRIX', 'GL_MULTISAMPLE',
'GL_SAMPLE_ALPHA_TO_COVERAGE', 'GL_SAMPLE_ALPHA_TO_ONE', 'GL_SAMPLE_COVERAGE',
'GL_SAMPLE_BUFFERS', 'GL_SAMPLES', 'GL_SAMPLE_COVERAGE_VALUE',
'GL_SAMPLE_COVERAGE_INVERT', 'GL_MULTISAMPLE_BIT', 'GL_NORMAL_MAP',
'GL_REFLECTION_MAP', 'GL_TEXTURE_CUBE_MAP', 'GL_TEXTURE_BINDING_CUBE_MAP',
'GL_TEXTURE_CUBE_MAP_POSITIVE_X', 'GL_TEXTURE_CUBE_MAP_NEGATIVE_X',
'GL_TEXTURE_CUBE_MAP_POSITIVE_Y', 'GL_TEXTURE_CUBE_MAP_NEGATIVE_Y',
'GL_TEXTURE_CUBE_MAP_POSITIVE_Z', 'GL_TEXTURE_CUBE_MAP_NEGATIVE_Z',
'GL_PROXY_TEXTURE_CUBE_MAP', 'GL_MAX_CUBE_MAP_TEXTURE_SIZE',
'GL_COMPRESSED_ALPHA', 'GL_COMPRESSED_LUMINANCE',
'GL_COMPRESSED_LUMINANCE_ALPHA', 'GL_COMPRESSED_INTENSITY',
'GL_COMPRESSED_RGB', 'GL_COMPRESSED_RGBA', 'GL_TEXTURE_COMPRESSION_HINT',
'GL_TEXTURE_COMPRESSED_IMAGE_SIZE', 'GL_TEXTURE_COMPRESSED',
'GL_NUM_COMPRESSED_TEXTURE_FORMATS', 'GL_COMPRESSED_TEXTURE_FORMATS',
'GL_CLAMP_TO_BORDER', 'GL_COMBINE', 'GL_COMBINE_RGB', 'GL_COMBINE_ALPHA',
'GL_SOURCE0_RGB', 'GL_SOURCE1_RGB', 'GL_SOURCE2_RGB', 'GL_SOURCE0_ALPHA',
'GL_SOURCE1_ALPHA', 'GL_SOURCE2_ALPHA', 'GL_OPERAND0_RGB', 'GL_OPERAND1_RGB',
'GL_OPERAND2_RGB', 'GL_OPERAND0_ALPHA', 'GL_OPERAND1_ALPHA',
'GL_OPERAND2_ALPHA', 'GL_RGB_SCALE', 'GL_ADD_SIGNED', 'GL_INTERPOLATE',
'GL_SUBTRACT', 'GL_CONSTANT', 'GL_PRIMARY_COLOR', 'GL_PREVIOUS',
'GL_DOT3_RGB', 'GL_DOT3_RGBA', 'GL_BLEND_DST_RGB', 'GL_BLEND_SRC_RGB',
'GL_BLEND_DST_ALPHA', 'GL_BLEND_SRC_ALPHA', 'GL_POINT_SIZE_MIN',
'GL_POINT_SIZE_MAX', 'GL_POINT_FADE_THRESHOLD_SIZE',
'GL_POINT_DISTANCE_ATTENUATION', 'GL_GENERATE_MIPMAP',
'GL_GENERATE_MIPMAP_HINT', 'GL_DEPTH_COMPONENT16', 'GL_DEPTH_COMPONENT24',
'GL_DEPTH_COMPONENT32', 'GL_MIRRORED_REPEAT', 'GL_FOG_COORDINATE_SOURCE',
'GL_FOG_COORDINATE', 'GL_FRAGMENT_DEPTH', 'GL_CURRENT_FOG_COORDINATE',
'GL_FOG_COORDINATE_ARRAY_TYPE', 'GL_FOG_COORDINATE_ARRAY_STRIDE',
'GL_FOG_COORDINATE_ARRAY_POINTER', 'GL_FOG_COORDINATE_ARRAY', 'GL_COLOR_SUM',
'GL_CURRENT_SECONDARY_COLOR', 'GL_SECONDARY_COLOR_ARRAY_SIZE',
'GL_SECONDARY_COLOR_ARRAY_TYPE', 'GL_SECONDARY_COLOR_ARRAY_STRIDE',
'GL_SECONDARY_COLOR_ARRAY_POINTER', 'GL_SECONDARY_COLOR_ARRAY',
'GL_MAX_TEXTURE_LOD_BIAS', 'GL_TEXTURE_FILTER_CONTROL', 'GL_TEXTURE_LOD_BIAS',
'GL_INCR_WRAP', 'GL_DECR_WRAP', 'GL_TEXTURE_DEPTH_SIZE',
'GL_DEPTH_TEXTURE_MODE', 'GL_TEXTURE_COMPARE_MODE', 'GL_TEXTURE_COMPARE_FUNC',
'GL_COMPARE_R_TO_TEXTURE', 'GL_BUFFER_SIZE', 'GL_BUFFER_USAGE',
'GL_QUERY_COUNTER_BITS', 'GL_CURRENT_QUERY', 'GL_QUERY_RESULT',
'GL_QUERY_RESULT_AVAILABLE', 'GL_ARRAY_BUFFER', 'GL_ELEMENT_ARRAY_BUFFER',
'GL_ARRAY_BUFFER_BINDING', 'GL_ELEMENT_ARRAY_BUFFER_BINDING',
'GL_VERTEX_ARRAY_BUFFER_BINDING', 'GL_NORMAL_ARRAY_BUFFER_BINDING',
'GL_COLOR_ARRAY_BUFFER_BINDING', 'GL_INDEX_ARRAY_BUFFER_BINDING',
'GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING', 'GL_EDGE_FLAG_ARRAY_BUFFER_BINDING',
'GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING',
'GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING', 'GL_WEIGHT_ARRAY_BUFFER_BINDING',
'GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING', 'GL_READ_ONLY', 'GL_WRITE_ONLY',
'GL_READ_WRITE', 'GL_BUFFER_ACCESS', 'GL_BUFFER_MAPPED',
'GL_BUFFER_MAP_POINTER', 'GL_STREAM_DRAW', 'GL_STREAM_READ', 'GL_STREAM_COPY',
'GL_STATIC_DRAW', 'GL_STATIC_READ', 'GL_STATIC_COPY', 'GL_DYNAMIC_DRAW',
'GL_DYNAMIC_READ', 'GL_DYNAMIC_COPY', 'GL_SAMPLES_PASSED', 'GL_FOG_COORD_SRC',
'GL_FOG_COORD', 'GL_CURRENT_FOG_COORD', 'GL_FOG_COORD_ARRAY_TYPE',
'GL_FOG_COORD_ARRAY_STRIDE', 'GL_FOG_COORD_ARRAY_POINTER',
'GL_FOG_COORD_ARRAY', 'GL_FOG_COORD_ARRAY_BUFFER_BINDING', 'GL_SRC0_RGB',
'GL_SRC1_RGB', 'GL_SRC2_RGB', 'GL_SRC0_ALPHA', 'GL_SRC1_ALPHA',
'GL_SRC2_ALPHA', 'GL_BLEND_EQUATION_RGB', 'GL_VERTEX_ATTRIB_ARRAY_ENABLED',
'GL_VERTEX_ATTRIB_ARRAY_SIZE', 'GL_VERTEX_ATTRIB_ARRAY_STRIDE',
'GL_VERTEX_ATTRIB_ARRAY_TYPE', 'GL_CURRENT_VERTEX_ATTRIB',
'GL_VERTEX_PROGRAM_POINT_SIZE', 'GL_VERTEX_PROGRAM_TWO_SIDE',
'GL_VERTEX_ATTRIB_ARRAY_POINTER', 'GL_STENCIL_BACK_FUNC',
'GL_STENCIL_BACK_FAIL', 'GL_STENCIL_BACK_PASS_DEPTH_FAIL',
'GL_STENCIL_BACK_PASS_DEPTH_PASS', 'GL_MAX_DRAW_BUFFERS', 'GL_DRAW_BUFFER0',
'GL_DRAW_BUFFER1', 'GL_DRAW_BUFFER2', 'GL_DRAW_BUFFER3', 'GL_DRAW_BUFFER4',
'GL_DRAW_BUFFER5', 'GL_DRAW_BUFFER6', 'GL_DRAW_BUFFER7', 'GL_DRAW_BUFFER8',
'GL_DRAW_BUFFER9', 'GL_DRAW_BUFFER10', 'GL_DRAW_BUFFER11', 'GL_DRAW_BUFFER12',
'GL_DRAW_BUFFER13', 'GL_DRAW_BUFFER14', 'GL_DRAW_BUFFER15',
'GL_BLEND_EQUATION_ALPHA', 'GL_POINT_SPRITE', 'GL_COORD_REPLACE',
'GL_MAX_VERTEX_ATTRIBS', 'GL_VERTEX_ATTRIB_ARRAY_NORMALIZED',
'GL_MAX_TEXTURE_COORDS', 'GL_MAX_TEXTURE_IMAGE_UNITS', 'GL_FRAGMENT_SHADER',
'GL_VERTEX_SHADER', 'GL_MAX_FRAGMENT_UNIFORM_COMPONENTS',
'GL_MAX_VERTEX_UNIFORM_COMPONENTS', 'GL_MAX_VARYING_FLOATS',
'GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS', 'GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS',
'GL_SHADER_TYPE', 'GL_FLOAT_VEC2', 'GL_FLOAT_VEC3', 'GL_FLOAT_VEC4',
'GL_INT_VEC2', 'GL_INT_VEC3', 'GL_INT_VEC4', 'GL_BOOL', 'GL_BOOL_VEC2',
'GL_BOOL_VEC3', 'GL_BOOL_VEC4', 'GL_FLOAT_MAT2', 'GL_FLOAT_MAT3',
'GL_FLOAT_MAT4', 'GL_SAMPLER_1D', 'GL_SAMPLER_2D', 'GL_SAMPLER_3D',
'GL_SAMPLER_CUBE', 'GL_SAMPLER_1D_SHADOW', 'GL_SAMPLER_2D_SHADOW',
'GL_DELETE_STATUS', 'GL_COMPILE_STATUS', 'GL_LINK_STATUS',
'GL_VALIDATE_STATUS', 'GL_INFO_LOG_LENGTH', 'GL_ATTACHED_SHADERS',
'GL_ACTIVE_UNIFORMS', 'GL_ACTIVE_UNIFORM_MAX_LENGTH',
'GL_SHADER_SOURCE_LENGTH', 'GL_ACTIVE_ATTRIBUTES',
'GL_ACTIVE_ATTRIBUTE_MAX_LENGTH', 'GL_FRAGMENT_SHADER_DERIVATIVE_HINT',
'GL_SHADING_LANGUAGE_VERSION', 'GL_CURRENT_PROGRAM',
'GL_POINT_SPRITE_COORD_ORIGIN', 'GL_LOWER_LEFT', 'GL_UPPER_LEFT',
'GL_STENCIL_BACK_REF', 'GL_STENCIL_BACK_VALUE_MASK',
'GL_STENCIL_BACK_WRITEMASK', 'GL_TEXTURE0_ARB', 'GL_TEXTURE1_ARB',
'GL_TEXTURE2_ARB', 'GL_TEXTURE3_ARB', 'GL_TEXTURE4_ARB', 'GL_TEXTURE5_ARB',
'GL_TEXTURE6_ARB', 'GL_TEXTURE7_ARB', 'GL_TEXTURE8_ARB', 'GL_TEXTURE9_ARB',
'GL_TEXTURE10_ARB', 'GL_TEXTURE11_ARB', 'GL_TEXTURE12_ARB',
'GL_TEXTURE13_ARB', 'GL_TEXTURE14_ARB', 'GL_TEXTURE15_ARB',
'GL_TEXTURE16_ARB', 'GL_TEXTURE17_ARB', 'GL_TEXTURE18_ARB',
'GL_TEXTURE19_ARB', 'GL_TEXTURE20_ARB', 'GL_TEXTURE21_ARB',
'GL_TEXTURE22_ARB', 'GL_TEXTURE23_ARB', 'GL_TEXTURE24_ARB',
'GL_TEXTURE25_ARB', 'GL_TEXTURE26_ARB', 'GL_TEXTURE27_ARB',
'GL_TEXTURE28_ARB', 'GL_TEXTURE29_ARB', 'GL_TEXTURE30_ARB',
'GL_TEXTURE31_ARB', 'GL_ACTIVE_TEXTURE_ARB', 'GL_CLIENT_ACTIVE_TEXTURE_ARB',
'GL_MAX_TEXTURE_UNITS_ARB', 'GL_TRANSPOSE_MODELVIEW_MATRIX_ARB',
'GL_TRANSPOSE_PROJECTION_MATRIX_ARB', 'GL_TRANSPOSE_TEXTURE_MATRIX_ARB',
'GL_TRANSPOSE_COLOR_MATRIX_ARB', 'GL_MULTISAMPLE_ARB',
'GL_SAMPLE_ALPHA_TO_COVERAGE_ARB', 'GL_SAMPLE_ALPHA_TO_ONE_ARB',
'GL_SAMPLE_COVERAGE_ARB', 'GL_SAMPLE_BUFFERS_ARB', 'GL_SAMPLES_ARB',
'GL_SAMPLE_COVERAGE_VALUE_ARB', 'GL_SAMPLE_COVERAGE_INVERT_ARB',
'GL_MULTISAMPLE_BIT_ARB', 'GL_NORMAL_MAP_ARB', 'GL_REFLECTION_MAP_ARB',
'GL_TEXTURE_CUBE_MAP_ARB', 'GL_TEXTURE_BINDING_CUBE_MAP_ARB',
'GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB', 'GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB',
'GL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB', 'GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB',
'GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB', 'GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB',
'GL_PROXY_TEXTURE_CUBE_MAP_ARB', 'GL_MAX_CUBE_MAP_TEXTURE_SIZE_ARB',
'GL_COMPRESSED_ALPHA_ARB', 'GL_COMPRESSED_LUMINANCE_ARB',
'GL_COMPRESSED_LUMINANCE_ALPHA_ARB', 'GL_COMPRESSED_INTENSITY_ARB',
'GL_COMPRESSED_RGB_ARB', 'GL_COMPRESSED_RGBA_ARB',
'GL_TEXTURE_COMPRESSION_HINT_ARB', 'GL_TEXTURE_COMPRESSED_IMAGE_SIZE_ARB',
'GL_TEXTURE_COMPRESSED_ARB', 'GL_NUM_COMPRESSED_TEXTURE_FORMATS_ARB',
'GL_COMPRESSED_TEXTURE_FORMATS_ARB', 'GL_CLAMP_TO_BORDER_ARB',
'GL_POINT_SIZE_MIN_ARB', 'GL_POINT_SIZE_MAX_ARB',
'GL_POINT_FADE_THRESHOLD_SIZE_ARB', 'GL_POINT_DISTANCE_ATTENUATION_ARB',
'GL_MAX_VERTEX_UNITS_ARB', 'GL_ACTIVE_VERTEX_UNITS_ARB',
'GL_WEIGHT_SUM_UNITY_ARB', 'GL_VERTEX_BLEND_ARB', 'GL_CURRENT_WEIGHT_ARB',
'GL_WEIGHT_ARRAY_TYPE_ARB', 'GL_WEIGHT_ARRAY_STRIDE_ARB',
'GL_WEIGHT_ARRAY_SIZE_ARB', 'GL_WEIGHT_ARRAY_POINTER_ARB',
'GL_WEIGHT_ARRAY_ARB', 'GL_MODELVIEW0_ARB', 'GL_MODELVIEW1_ARB',
'GL_MODELVIEW2_ARB', 'GL_MODELVIEW3_ARB', 'GL_MODELVIEW4_ARB',
'GL_MODELVIEW5_ARB', 'GL_MODELVIEW6_ARB', 'GL_MODELVIEW7_ARB',
'GL_MODELVIEW8_ARB', 'GL_MODELVIEW9_ARB', 'GL_MODELVIEW10_ARB',
'GL_MODELVIEW11_ARB', 'GL_MODELVIEW12_ARB', 'GL_MODELVIEW13_ARB',
'GL_MODELVIEW14_ARB', 'GL_MODELVIEW15_ARB', 'GL_MODELVIEW16_ARB',
'GL_MODELVIEW17_ARB', 'GL_MODELVIEW18_ARB', 'GL_MODELVIEW19_ARB',
'GL_MODELVIEW20_ARB', 'GL_MODELVIEW21_ARB', 'GL_MODELVIEW22_ARB',
'GL_MODELVIEW23_ARB', 'GL_MODELVIEW24_ARB', 'GL_MODELVIEW25_ARB',
'GL_MODELVIEW26_ARB', 'GL_MODELVIEW27_ARB', 'GL_MODELVIEW28_ARB',
'GL_MODELVIEW29_ARB', 'GL_MODELVIEW30_ARB', 'GL_MODELVIEW31_ARB',
'GL_MATRIX_PALETTE_ARB', 'GL_MAX_MATRIX_PALETTE_STACK_DEPTH_ARB',
'GL_MAX_PALETTE_MATRICES_ARB', 'GL_CURRENT_PALETTE_MATRIX_ARB',
'GL_MATRIX_INDEX_ARRAY_ARB', 'GL_CURRENT_MATRIX_INDEX_ARB',
'GL_MATRIX_INDEX_ARRAY_SIZE_ARB', 'GL_MATRIX_INDEX_ARRAY_TYPE_ARB',
'GL_MATRIX_INDEX_ARRAY_STRIDE_ARB', 'GL_MATRIX_INDEX_ARRAY_POINTER_ARB',
'GL_COMBINE_ARB', 'GL_COMBINE_RGB_ARB', 'GL_COMBINE_ALPHA_ARB',
'GL_SOURCE0_RGB_ARB', 'GL_SOURCE1_RGB_ARB', 'GL_SOURCE2_RGB_ARB',
'GL_SOURCE0_ALPHA_ARB', 'GL_SOURCE1_ALPHA_ARB', 'GL_SOURCE2_ALPHA_ARB',
'GL_OPERAND0_RGB_ARB', 'GL_OPERAND1_RGB_ARB', 'GL_OPERAND2_RGB_ARB',
'GL_OPERAND0_ALPHA_ARB', 'GL_OPERAND1_ALPHA_ARB', 'GL_OPERAND2_ALPHA_ARB',
'GL_RGB_SCALE_ARB', 'GL_ADD_SIGNED_ARB', 'GL_INTERPOLATE_ARB',
'GL_SUBTRACT_ARB', 'GL_CONSTANT_ARB', 'GL_PRIMARY_COLOR_ARB',
'GL_PREVIOUS_ARB', 'GL_DOT3_RGB_ARB', 'GL_DOT3_RGBA_ARB',
'GL_MIRRORED_REPEAT_ARB', 'GL_DEPTH_COMPONENT16_ARB',
'GL_DEPTH_COMPONENT24_ARB', 'GL_DEPTH_COMPONENT32_ARB',
'GL_TEXTURE_DEPTH_SIZE_ARB', 'GL_DEPTH_TEXTURE_MODE_ARB',
'GL_TEXTURE_COMPARE_MODE_ARB', 'GL_TEXTURE_COMPARE_FUNC_ARB',
'GL_COMPARE_R_TO_TEXTURE_ARB', 'GL_TEXTURE_COMPARE_FAIL_VALUE_ARB',
'GL_COLOR_SUM_ARB', 'GL_VERTEX_PROGRAM_ARB',
'GL_VERTEX_ATTRIB_ARRAY_ENABLED_ARB', 'GL_VERTEX_ATTRIB_ARRAY_SIZE_ARB',
'GL_VERTEX_ATTRIB_ARRAY_STRIDE_ARB', 'GL_VERTEX_ATTRIB_ARRAY_TYPE_ARB',
'GL_CURRENT_VERTEX_ATTRIB_ARB', 'GL_PROGRAM_LENGTH_ARB',
'GL_PROGRAM_STRING_ARB', 'GL_MAX_PROGRAM_MATRIX_STACK_DEPTH_ARB',
'GL_MAX_PROGRAM_MATRICES_ARB', 'GL_CURRENT_MATRIX_STACK_DEPTH_ARB',
'GL_CURRENT_MATRIX_ARB', 'GL_VERTEX_PROGRAM_POINT_SIZE_ARB',
'GL_VERTEX_PROGRAM_TWO_SIDE_ARB', 'GL_VERTEX_ATTRIB_ARRAY_POINTER_ARB',
'GL_PROGRAM_ERROR_POSITION_ARB', 'GL_PROGRAM_BINDING_ARB',
'GL_MAX_VERTEX_ATTRIBS_ARB', 'GL_VERTEX_ATTRIB_ARRAY_NORMALIZED_ARB',
'GL_PROGRAM_ERROR_STRING_ARB', 'GL_PROGRAM_FORMAT_ASCII_ARB',
'GL_PROGRAM_FORMAT_ARB', 'GL_PROGRAM_INSTRUCTIONS_ARB',
'GL_MAX_PROGRAM_INSTRUCTIONS_ARB', 'GL_PROGRAM_NATIVE_INSTRUCTIONS_ARB',
'GL_MAX_PROGRAM_NATIVE_INSTRUCTIONS_ARB', 'GL_PROGRAM_TEMPORARIES_ARB',
'GL_MAX_PROGRAM_TEMPORARIES_ARB', 'GL_PROGRAM_NATIVE_TEMPORARIES_ARB',
'GL_MAX_PROGRAM_NATIVE_TEMPORARIES_ARB', 'GL_PROGRAM_PARAMETERS_ARB',
'GL_MAX_PROGRAM_PARAMETERS_ARB', 'GL_PROGRAM_NATIVE_PARAMETERS_ARB',
'GL_MAX_PROGRAM_NATIVE_PARAMETERS_ARB', 'GL_PROGRAM_ATTRIBS_ARB',
'GL_MAX_PROGRAM_ATTRIBS_ARB', 'GL_PROGRAM_NATIVE_ATTRIBS_ARB',
'GL_MAX_PROGRAM_NATIVE_ATTRIBS_ARB', 'GL_PROGRAM_ADDRESS_REGISTERS_ARB',
'GL_MAX_PROGRAM_ADDRESS_REGISTERS_ARB',
'GL_PROGRAM_NATIVE_ADDRESS_REGISTERS_ARB',
'GL_MAX_PROGRAM_NATIVE_ADDRESS_REGISTERS_ARB',
'GL_MAX_PROGRAM_LOCAL_PARAMETERS_ARB', 'GL_MAX_PROGRAM_ENV_PARAMETERS_ARB',
'GL_PROGRAM_UNDER_NATIVE_LIMITS_ARB', 'GL_TRANSPOSE_CURRENT_MATRIX_ARB',
'GL_MATRIX0_ARB', 'GL_MATRIX1_ARB', 'GL_MATRIX2_ARB', 'GL_MATRIX3_ARB',
'GL_MATRIX4_ARB', 'GL_MATRIX5_ARB', 'GL_MATRIX6_ARB', 'GL_MATRIX7_ARB',
'GL_MATRIX8_ARB', 'GL_MATRIX9_ARB', 'GL_MATRIX10_ARB', 'GL_MATRIX11_ARB',
'GL_MATRIX12_ARB', 'GL_MATRIX13_ARB', 'GL_MATRIX14_ARB', 'GL_MATRIX15_ARB',
'GL_MATRIX16_ARB', 'GL_MATRIX17_ARB', 'GL_MATRIX18_ARB', 'GL_MATRIX19_ARB',
'GL_MATRIX20_ARB', 'GL_MATRIX21_ARB', 'GL_MATRIX22_ARB', 'GL_MATRIX23_ARB',
'GL_MATRIX24_ARB', 'GL_MATRIX25_ARB', 'GL_MATRIX26_ARB', 'GL_MATRIX27_ARB',
'GL_MATRIX28_ARB', 'GL_MATRIX29_ARB', 'GL_MATRIX30_ARB', 'GL_MATRIX31_ARB',
'GL_FRAGMENT_PROGRAM_ARB', 'GL_PROGRAM_ALU_INSTRUCTIONS_ARB',
'GL_PROGRAM_TEX_INSTRUCTIONS_ARB', 'GL_PROGRAM_TEX_INDIRECTIONS_ARB',
'GL_PROGRAM_NATIVE_ALU_INSTRUCTIONS_ARB',
'GL_PROGRAM_NATIVE_TEX_INSTRUCTIONS_ARB',
'GL_PROGRAM_NATIVE_TEX_INDIRECTIONS_ARB',
'GL_MAX_PROGRAM_ALU_INSTRUCTIONS_ARB', 'GL_MAX_PROGRAM_TEX_INSTRUCTIONS_ARB',
'GL_MAX_PROGRAM_TEX_INDIRECTIONS_ARB',
'GL_MAX_PROGRAM_NATIVE_ALU_INSTRUCTIONS_ARB',
'GL_MAX_PROGRAM_NATIVE_TEX_INSTRUCTIONS_ARB',
'GL_MAX_PROGRAM_NATIVE_TEX_INDIRECTIONS_ARB', 'GL_MAX_TEXTURE_COORDS_ARB',
'GL_MAX_TEXTURE_IMAGE_UNITS_ARB', 'GL_BUFFER_SIZE_ARB', 'GL_BUFFER_USAGE_ARB',
'GL_ARRAY_BUFFER_ARB', 'GL_ELEMENT_ARRAY_BUFFER_ARB',
'GL_ARRAY_BUFFER_BINDING_ARB', 'GL_ELEMENT_ARRAY_BUFFER_BINDING_ARB',
'GL_VERTEX_ARRAY_BUFFER_BINDING_ARB', 'GL_NORMAL_ARRAY_BUFFER_BINDING_ARB',
'GL_COLOR_ARRAY_BUFFER_BINDING_ARB', 'GL_INDEX_ARRAY_BUFFER_BINDING_ARB',
'GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING_ARB',
'GL_EDGE_FLAG_ARRAY_BUFFER_BINDING_ARB',
'GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING_ARB',
'GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING_ARB',
'GL_WEIGHT_ARRAY_BUFFER_BINDING_ARB',
'GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING_ARB', 'GL_READ_ONLY_ARB',
'GL_WRITE_ONLY_ARB', 'GL_READ_WRITE_ARB', 'GL_BUFFER_ACCESS_ARB',
'GL_BUFFER_MAPPED_ARB', 'GL_BUFFER_MAP_POINTER_ARB', 'GL_STREAM_DRAW_ARB',
'GL_STREAM_READ_ARB', 'GL_STREAM_COPY_ARB', 'GL_STATIC_DRAW_ARB',
'GL_STATIC_READ_ARB', 'GL_STATIC_COPY_ARB', 'GL_DYNAMIC_DRAW_ARB',
'GL_DYNAMIC_READ_ARB', 'GL_DYNAMIC_COPY_ARB', 'GL_QUERY_COUNTER_BITS_ARB',
'GL_CURRENT_QUERY_ARB', 'GL_QUERY_RESULT_ARB',
'GL_QUERY_RESULT_AVAILABLE_ARB', 'GL_SAMPLES_PASSED_ARB',
'GL_PROGRAM_OBJECT_ARB', 'GL_SHADER_OBJECT_ARB', 'GL_OBJECT_TYPE_ARB',
'GL_OBJECT_SUBTYPE_ARB', 'GL_FLOAT_VEC2_ARB', 'GL_FLOAT_VEC3_ARB',
'GL_FLOAT_VEC4_ARB', 'GL_INT_VEC2_ARB', 'GL_INT_VEC3_ARB', 'GL_INT_VEC4_ARB',
'GL_BOOL_ARB', 'GL_BOOL_VEC2_ARB', 'GL_BOOL_VEC3_ARB', 'GL_BOOL_VEC4_ARB',
'GL_FLOAT_MAT2_ARB', 'GL_FLOAT_MAT3_ARB', 'GL_FLOAT_MAT4_ARB',
'GL_SAMPLER_1D_ARB', 'GL_SAMPLER_2D_ARB', 'GL_SAMPLER_3D_ARB',
'GL_SAMPLER_CUBE_ARB', 'GL_SAMPLER_1D_SHADOW_ARB', 'GL_SAMPLER_2D_SHADOW_ARB',
'GL_SAMPLER_2D_RECT_ARB', 'GL_SAMPLER_2D_RECT_SHADOW_ARB',
'GL_OBJECT_DELETE_STATUS_ARB', 'GL_OBJECT_COMPILE_STATUS_ARB',
'GL_OBJECT_LINK_STATUS_ARB', 'GL_OBJECT_VALIDATE_STATUS_ARB',
'GL_OBJECT_INFO_LOG_LENGTH_ARB', 'GL_OBJECT_ATTACHED_OBJECTS_ARB',
'GL_OBJECT_ACTIVE_UNIFORMS_ARB', 'GL_OBJECT_ACTIVE_UNIFORM_MAX_LENGTH_ARB',
'GL_OBJECT_SHADER_SOURCE_LENGTH_ARB', 'GL_VERTEX_SHADER_ARB',
'GL_MAX_VERTEX_UNIFORM_COMPONENTS_ARB', 'GL_MAX_VARYING_FLOATS_ARB',
'GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS_ARB',
'GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS_ARB', 'GL_OBJECT_ACTIVE_ATTRIBUTES_ARB',
'GL_OBJECT_ACTIVE_ATTRIBUTE_MAX_LENGTH_ARB', 'GL_FRAGMENT_SHADER_ARB',
'GL_MAX_FRAGMENT_UNIFORM_COMPONENTS_ARB',
'GL_FRAGMENT_SHADER_DERIVATIVE_HINT_ARB', 'GL_SHADING_LANGUAGE_VERSION_ARB',
'GL_POINT_SPRITE_ARB', 'GL_COORD_REPLACE_ARB', 'GL_MAX_DRAW_BUFFERS_ARB',
'GL_DRAW_BUFFER0_ARB', 'GL_DRAW_BUFFER1_ARB', 'GL_DRAW_BUFFER2_ARB',
'GL_DRAW_BUFFER3_ARB', 'GL_DRAW_BUFFER4_ARB', 'GL_DRAW_BUFFER5_ARB',
'GL_DRAW_BUFFER6_ARB', 'GL_DRAW_BUFFER7_ARB', 'GL_DRAW_BUFFER8_ARB',
'GL_DRAW_BUFFER9_ARB', 'GL_DRAW_BUFFER10_ARB', 'GL_DRAW_BUFFER11_ARB',
'GL_DRAW_BUFFER12_ARB', 'GL_DRAW_BUFFER13_ARB', 'GL_DRAW_BUFFER14_ARB',
'GL_DRAW_BUFFER15_ARB', 'GL_TEXTURE_RECTANGLE_ARB',
'GL_TEXTURE_BINDING_RECTANGLE_ARB', 'GL_PROXY_TEXTURE_RECTANGLE_ARB',
'GL_MAX_RECTANGLE_TEXTURE_SIZE_ARB', 'GL_RGBA_FLOAT_MODE_ARB',
'GL_CLAMP_VERTEX_COLOR_ARB', 'GL_CLAMP_FRAGMENT_COLOR_ARB',
'GL_CLAMP_READ_COLOR_ARB', 'GL_FIXED_ONLY_ARB', 'GL_HALF_FLOAT_ARB',
'GL_TEXTURE_RED_TYPE_ARB', 'GL_TEXTURE_GREEN_TYPE_ARB',
'GL_TEXTURE_BLUE_TYPE_ARB', 'GL_TEXTURE_ALPHA_TYPE_ARB',
'GL_TEXTURE_LUMINANCE_TYPE_ARB', 'GL_TEXTURE_INTENSITY_TYPE_ARB',
'GL_TEXTURE_DEPTH_TYPE_ARB', 'GL_UNSIGNED_NORMALIZED_ARB', 'GL_RGBA32F_ARB',
'GL_RGB32F_ARB', 'GL_ALPHA32F_ARB', 'GL_INTENSITY32F_ARB',
'GL_LUMINANCE32F_ARB', 'GL_LUMINANCE_ALPHA32F_ARB', 'GL_RGBA16F_ARB',
'GL_RGB16F_ARB', 'GL_ALPHA16F_ARB', 'GL_INTENSITY16F_ARB',
'GL_LUMINANCE16F_ARB', 'GL_LUMINANCE_ALPHA16F_ARB',
'GL_PIXEL_PACK_BUFFER_ARB', 'GL_PIXEL_UNPACK_BUFFER_ARB',
'GL_PIXEL_PACK_BUFFER_BINDING_ARB', 'GL_PIXEL_UNPACK_BUFFER_BINDING_ARB',
'GL_ABGR_EXT', 'GL_CONSTANT_COLOR_EXT', 'GL_ONE_MINUS_CONSTANT_COLOR_EXT',
'GL_CONSTANT_ALPHA_EXT', 'GL_ONE_MINUS_CONSTANT_ALPHA_EXT',
'GL_BLEND_COLOR_EXT', 'GL_POLYGON_OFFSET_EXT', 'GL_POLYGON_OFFSET_FACTOR_EXT',
'GL_POLYGON_OFFSET_BIAS_EXT', 'GL_ALPHA4_EXT', 'GL_ALPHA8_EXT',
'GL_ALPHA12_EXT', 'GL_ALPHA16_EXT', 'GL_LUMINANCE4_EXT', 'GL_LUMINANCE8_EXT',
'GL_LUMINANCE12_EXT', 'GL_LUMINANCE16_EXT', 'GL_LUMINANCE4_ALPHA4_EXT',
'GL_LUMINANCE6_ALPHA2_EXT', 'GL_LUMINANCE8_ALPHA8_EXT',
'GL_LUMINANCE12_ALPHA4_EXT', 'GL_LUMINANCE12_ALPHA12_EXT',
'GL_LUMINANCE16_ALPHA16_EXT', 'GL_INTENSITY_EXT', 'GL_INTENSITY4_EXT',
'GL_INTENSITY8_EXT', 'GL_INTENSITY12_EXT', 'GL_INTENSITY16_EXT',
'GL_RGB2_EXT', 'GL_RGB4_EXT', 'GL_RGB5_EXT', 'GL_RGB8_EXT', 'GL_RGB10_EXT',
'GL_RGB12_EXT', 'GL_RGB16_EXT', 'GL_RGBA2_EXT', 'GL_RGBA4_EXT',
'GL_RGB5_A1_EXT', 'GL_RGBA8_EXT', 'GL_RGB10_A2_EXT', 'GL_RGBA12_EXT',
'GL_RGBA16_EXT', 'GL_TEXTURE_RED_SIZE_EXT', 'GL_TEXTURE_GREEN_SIZE_EXT',
'GL_TEXTURE_BLUE_SIZE_EXT', 'GL_TEXTURE_ALPHA_SIZE_EXT',
'GL_TEXTURE_LUMINANCE_SIZE_EXT', 'GL_TEXTURE_INTENSITY_SIZE_EXT',
'GL_REPLACE_EXT', 'GL_PROXY_TEXTURE_1D_EXT', 'GL_PROXY_TEXTURE_2D_EXT',
'GL_TEXTURE_TOO_LARGE_EXT', 'GL_PACK_SKIP_IMAGES_EXT',
'GL_PACK_IMAGE_HEIGHT_EXT', 'GL_UNPACK_SKIP_IMAGES_EXT',
'GL_UNPACK_IMAGE_HEIGHT_EXT', 'GL_TEXTURE_3D_EXT', 'GL_PROXY_TEXTURE_3D_EXT',
'GL_TEXTURE_DEPTH_EXT', 'GL_TEXTURE_WRAP_R_EXT', 'GL_MAX_3D_TEXTURE_SIZE_EXT',
'GL_FILTER4_SGIS', 'GL_TEXTURE_FILTER4_SIZE_SGIS', 'GL_HISTOGRAM_EXT',
'GL_PROXY_HISTOGRAM_EXT', 'GL_HISTOGRAM_WIDTH_EXT', 'GL_HISTOGRAM_FORMAT_EXT',
'GL_HISTOGRAM_RED_SIZE_EXT', 'GL_HISTOGRAM_GREEN_SIZE_EXT',
'GL_HISTOGRAM_BLUE_SIZE_EXT', 'GL_HISTOGRAM_ALPHA_SIZE_EXT',
'GL_HISTOGRAM_LUMINANCE_SIZE_EXT', 'GL_HISTOGRAM_SINK_EXT', 'GL_MINMAX_EXT',
'GL_MINMAX_FORMAT_EXT', 'GL_MINMAX_SINK_EXT', 'GL_TABLE_TOO_LARGE_EXT',
'GL_CONVOLUTION_1D_EXT', 'GL_CONVOLUTION_2D_EXT', 'GL_SEPARABLE_2D_EXT',
'GL_CONVOLUTION_BORDER_MODE_EXT', 'GL_CONVOLUTION_FILTER_SCALE_EXT',
'GL_CONVOLUTION_FILTER_BIAS_EXT', 'GL_REDUCE_EXT',
'GL_CONVOLUTION_FORMAT_EXT', 'GL_CONVOLUTION_WIDTH_EXT',
'GL_CONVOLUTION_HEIGHT_EXT', 'GL_MAX_CONVOLUTION_WIDTH_EXT',
'GL_MAX_CONVOLUTION_HEIGHT_EXT', 'GL_POST_CONVOLUTION_RED_SCALE_EXT',
'GL_POST_CONVOLUTION_GREEN_SCALE_EXT', 'GL_POST_CONVOLUTION_BLUE_SCALE_EXT',
'GL_POST_CONVOLUTION_ALPHA_SCALE_EXT', 'GL_POST_CONVOLUTION_RED_BIAS_EXT',
'GL_POST_CONVOLUTION_GREEN_BIAS_EXT', 'GL_POST_CONVOLUTION_BLUE_BIAS_EXT',
'GL_POST_CONVOLUTION_ALPHA_BIAS_EXT', 'GL_COLOR_MATRIX_SGI',
'GL_COLOR_MATRIX_STACK_DEPTH_SGI', 'GL_MAX_COLOR_MATRIX_STACK_DEPTH_SGI',
'GL_POST_COLOR_MATRIX_RED_SCALE_SGI', 'GL_POST_COLOR_MATRIX_GREEN_SCALE_SGI',
'GL_POST_COLOR_MATRIX_BLUE_SCALE_SGI', 'GL_POST_COLOR_MATRIX_ALPHA_SCALE_SGI',
'GL_POST_COLOR_MATRIX_RED_BIAS_SGI', 'GL_POST_COLOR_MATRIX_GREEN_BIAS_SGI',
'GL_POST_COLOR_MATRIX_BLUE_BIAS_SGI', 'GL_POST_COLOR_MATRIX_ALPHA_BIAS_SGI',
'GL_COLOR_TABLE_SGI', 'GL_POST_CONVOLUTION_COLOR_TABLE_SGI',
'GL_POST_COLOR_MATRIX_COLOR_TABLE_SGI', 'GL_PROXY_COLOR_TABLE_SGI',
'GL_PROXY_POST_CONVOLUTION_COLOR_TABLE_SGI',
'GL_PROXY_POST_COLOR_MATRIX_COLOR_TABLE_SGI', 'GL_COLOR_TABLE_SCALE_SGI',
'GL_COLOR_TABLE_BIAS_SGI', 'GL_COLOR_TABLE_FORMAT_SGI',
'GL_COLOR_TABLE_WIDTH_SGI', 'GL_COLOR_TABLE_RED_SIZE_SGI',
'GL_COLOR_TABLE_GREEN_SIZE_SGI', 'GL_COLOR_TABLE_BLUE_SIZE_SGI',
'GL_COLOR_TABLE_ALPHA_SIZE_SGI', 'GL_COLOR_TABLE_LUMINANCE_SIZE_SGI',
'GL_COLOR_TABLE_INTENSITY_SIZE_SGI', 'GL_PIXEL_TEXTURE_SGIS',
'GL_PIXEL_FRAGMENT_RGB_SOURCE_SGIS', 'GL_PIXEL_FRAGMENT_ALPHA_SOURCE_SGIS',
'GL_PIXEL_GROUP_COLOR_SGIS', 'GL_PIXEL_TEX_GEN_SGIX',
'GL_PIXEL_TEX_GEN_MODE_SGIX', 'GL_PACK_SKIP_VOLUMES_SGIS',
'GL_PACK_IMAGE_DEPTH_SGIS', 'GL_UNPACK_SKIP_VOLUMES_SGIS',
'GL_UNPACK_IMAGE_DEPTH_SGIS', 'GL_TEXTURE_4D_SGIS',
'GL_PROXY_TEXTURE_4D_SGIS', 'GL_TEXTURE_4DSIZE_SGIS',
'GL_TEXTURE_WRAP_Q_SGIS', 'GL_MAX_4D_TEXTURE_SIZE_SGIS',
'GL_TEXTURE_4D_BINDING_SGIS', 'GL_TEXTURE_COLOR_TABLE_SGI',
'GL_PROXY_TEXTURE_COLOR_TABLE_SGI', 'GL_CMYK_EXT', 'GL_CMYKA_EXT',
'GL_PACK_CMYK_HINT_EXT', 'GL_UNPACK_CMYK_HINT_EXT', 'GL_TEXTURE_PRIORITY_EXT',
'GL_TEXTURE_RESIDENT_EXT', 'GL_TEXTURE_1D_BINDING_EXT',
'GL_TEXTURE_2D_BINDING_EXT', 'GL_TEXTURE_3D_BINDING_EXT',
'GL_DETAIL_TEXTURE_2D_SGIS', 'GL_DETAIL_TEXTURE_2D_BINDING_SGIS',
'GL_LINEAR_DETAIL_SGIS', 'GL_LINEAR_DETAIL_ALPHA_SGIS',
'GL_LINEAR_DETAIL_COLOR_SGIS', 'GL_DETAIL_TEXTURE_LEVEL_SGIS',
'GL_DETAIL_TEXTURE_MODE_SGIS', 'GL_DETAIL_TEXTURE_FUNC_POINTS_SGIS',
'GL_LINEAR_SHARPEN_SGIS', 'GL_LINEAR_SHARPEN_ALPHA_SGIS',
'GL_LINEAR_SHARPEN_COLOR_SGIS', 'GL_SHARPEN_TEXTURE_FUNC_POINTS_SGIS',
'GL_UNSIGNED_BYTE_3_3_2_EXT', 'GL_UNSIGNED_SHORT_4_4_4_4_EXT',
'GL_UNSIGNED_SHORT_5_5_5_1_EXT', 'GL_UNSIGNED_INT_8_8_8_8_EXT',
'GL_UNSIGNED_INT_10_10_10_2_EXT', 'GL_TEXTURE_MIN_LOD_SGIS',
'GL_TEXTURE_MAX_LOD_SGIS', 'GL_TEXTURE_BASE_LEVEL_SGIS',
'GL_TEXTURE_MAX_LEVEL_SGIS', 'GL_MULTISAMPLE_SGIS',
'GL_SAMPLE_ALPHA_TO_MASK_SGIS', 'GL_SAMPLE_ALPHA_TO_ONE_SGIS',
'GL_SAMPLE_MASK_SGIS', 'GL_1PASS_SGIS', 'GL_2PASS_0_SGIS', 'GL_2PASS_1_SGIS',
'GL_4PASS_0_SGIS', 'GL_4PASS_1_SGIS', 'GL_4PASS_2_SGIS', 'GL_4PASS_3_SGIS',
'GL_SAMPLE_BUFFERS_SGIS', 'GL_SAMPLES_SGIS', 'GL_SAMPLE_MASK_VALUE_SGIS',
'GL_SAMPLE_MASK_INVERT_SGIS', 'GL_SAMPLE_PATTERN_SGIS',
'GL_RESCALE_NORMAL_EXT', 'GL_VERTEX_ARRAY_EXT', 'GL_NORMAL_ARRAY_EXT',
'GL_COLOR_ARRAY_EXT', 'GL_INDEX_ARRAY_EXT', 'GL_TEXTURE_COORD_ARRAY_EXT',
'GL_EDGE_FLAG_ARRAY_EXT', 'GL_VERTEX_ARRAY_SIZE_EXT',
'GL_VERTEX_ARRAY_TYPE_EXT', 'GL_VERTEX_ARRAY_STRIDE_EXT',
'GL_VERTEX_ARRAY_COUNT_EXT', 'GL_NORMAL_ARRAY_TYPE_EXT',
'GL_NORMAL_ARRAY_STRIDE_EXT', 'GL_NORMAL_ARRAY_COUNT_EXT',
'GL_COLOR_ARRAY_SIZE_EXT', 'GL_COLOR_ARRAY_TYPE_EXT',
'GL_COLOR_ARRAY_STRIDE_EXT', 'GL_COLOR_ARRAY_COUNT_EXT',
'GL_INDEX_ARRAY_TYPE_EXT', 'GL_INDEX_ARRAY_STRIDE_EXT',
'GL_INDEX_ARRAY_COUNT_EXT', 'GL_TEXTURE_COORD_ARRAY_SIZE_EXT',
'GL_TEXTURE_COORD_ARRAY_TYPE_EXT', 'GL_TEXTURE_COORD_ARRAY_STRIDE_EXT',
'GL_TEXTURE_COORD_ARRAY_COUNT_EXT', 'GL_EDGE_FLAG_ARRAY_STRIDE_EXT',
'GL_EDGE_FLAG_ARRAY_COUNT_EXT', 'GL_VERTEX_ARRAY_POINTER_EXT',
'GL_NORMAL_ARRAY_POINTER_EXT', 'GL_COLOR_ARRAY_POINTER_EXT',
'GL_INDEX_ARRAY_POINTER_EXT', 'GL_TEXTURE_COORD_ARRAY_POINTER_EXT',
'GL_EDGE_FLAG_ARRAY_POINTER_EXT', 'GL_GENERATE_MIPMAP_SGIS',
'GL_GENERATE_MIPMAP_HINT_SGIS', 'GL_LINEAR_CLIPMAP_LINEAR_SGIX',
'GL_TEXTURE_CLIPMAP_CENTER_SGIX', 'GL_TEXTURE_CLIPMAP_FRAME_SGIX',
'GL_TEXTURE_CLIPMAP_OFFSET_SGIX', 'GL_TEXTURE_CLIPMAP_VIRTUAL_DEPTH_SGIX',
'GL_TEXTURE_CLIPMAP_LOD_OFFSET_SGIX', 'GL_TEXTURE_CLIPMAP_DEPTH_SGIX',
'GL_MAX_CLIPMAP_DEPTH_SGIX', 'GL_MAX_CLIPMAP_VIRTUAL_DEPTH_SGIX',
'GL_NEAREST_CLIPMAP_NEAREST_SGIX', 'GL_NEAREST_CLIPMAP_LINEAR_SGIX',
'GL_LINEAR_CLIPMAP_NEAREST_SGIX', 'GL_TEXTURE_COMPARE_SGIX',
'GL_TEXTURE_COMPARE_OPERATOR_SGIX', 'GL_TEXTURE_LEQUAL_R_SGIX',
'GL_TEXTURE_GEQUAL_R_SGIX', 'GL_CLAMP_TO_EDGE_SGIS',
'GL_CLAMP_TO_BORDER_SGIS', 'GL_FUNC_ADD_EXT', 'GL_MIN_EXT', 'GL_MAX_EXT',
'GL_BLEND_EQUATION_EXT', 'GL_FUNC_SUBTRACT_EXT',
'GL_FUNC_REVERSE_SUBTRACT_EXT', 'GL_INTERLACE_SGIX',
'GL_PIXEL_TILE_BEST_ALIGNMENT_SGIX', 'GL_PIXEL_TILE_CACHE_INCREMENT_SGIX',
'GL_PIXEL_TILE_WIDTH_SGIX', 'GL_PIXEL_TILE_HEIGHT_SGIX',
'GL_PIXEL_TILE_GRID_WIDTH_SGIX', 'GL_PIXEL_TILE_GRID_HEIGHT_SGIX',
'GL_PIXEL_TILE_GRID_DEPTH_SGIX', 'GL_PIXEL_TILE_CACHE_SIZE_SGIX',
'GL_DUAL_ALPHA4_SGIS', 'GL_DUAL_ALPHA8_SGIS', 'GL_DUAL_ALPHA12_SGIS',
'GL_DUAL_ALPHA16_SGIS', 'GL_DUAL_LUMINANCE4_SGIS', 'GL_DUAL_LUMINANCE8_SGIS',
'GL_DUAL_LUMINANCE12_SGIS', 'GL_DUAL_LUMINANCE16_SGIS',
'GL_DUAL_INTENSITY4_SGIS', 'GL_DUAL_INTENSITY8_SGIS',
'GL_DUAL_INTENSITY12_SGIS', 'GL_DUAL_INTENSITY16_SGIS',
'GL_DUAL_LUMINANCE_ALPHA4_SGIS', 'GL_DUAL_LUMINANCE_ALPHA8_SGIS',
'GL_QUAD_ALPHA4_SGIS', 'GL_QUAD_ALPHA8_SGIS', 'GL_QUAD_LUMINANCE4_SGIS',
'GL_QUAD_LUMINANCE8_SGIS', 'GL_QUAD_INTENSITY4_SGIS',
'GL_QUAD_INTENSITY8_SGIS', 'GL_DUAL_TEXTURE_SELECT_SGIS',
'GL_QUAD_TEXTURE_SELECT_SGIS', 'GL_SPRITE_SGIX', 'GL_SPRITE_MODE_SGIX',
'GL_SPRITE_AXIS_SGIX', 'GL_SPRITE_TRANSLATION_SGIX', 'GL_SPRITE_AXIAL_SGIX',
'GL_SPRITE_OBJECT_ALIGNED_SGIX', 'GL_SPRITE_EYE_ALIGNED_SGIX',
'GL_TEXTURE_MULTI_BUFFER_HINT_SGIX', 'GL_POINT_SIZE_MIN_EXT',
'GL_POINT_SIZE_MAX_EXT', 'GL_POINT_FADE_THRESHOLD_SIZE_EXT',
'GL_DISTANCE_ATTENUATION_EXT', 'GL_POINT_SIZE_MIN_SGIS',
'GL_POINT_SIZE_MAX_SGIS', 'GL_POINT_FADE_THRESHOLD_SIZE_SGIS',
'GL_DISTANCE_ATTENUATION_SGIS', 'GL_INSTRUMENT_BUFFER_POINTER_SGIX',
'GL_INSTRUMENT_MEASUREMENTS_SGIX', 'GL_POST_TEXTURE_FILTER_BIAS_SGIX',
'GL_POST_TEXTURE_FILTER_SCALE_SGIX', 'GL_POST_TEXTURE_FILTER_BIAS_RANGE_SGIX',
'GL_POST_TEXTURE_FILTER_SCALE_RANGE_SGIX', 'GL_FRAMEZOOM_SGIX',
'GL_FRAMEZOOM_FACTOR_SGIX', 'GL_MAX_FRAMEZOOM_FACTOR_SGIX',
'GL_TEXTURE_DEFORMATION_BIT_SGIX', 'GL_GEOMETRY_DEFORMATION_BIT_SGIX',
'GL_GEOMETRY_DEFORMATION_SGIX', 'GL_TEXTURE_DEFORMATION_SGIX',
'GL_DEFORMATIONS_MASK_SGIX', 'GL_MAX_DEFORMATION_ORDER_SGIX',
'GL_REFERENCE_PLANE_SGIX', 'GL_REFERENCE_PLANE_EQUATION_SGIX',
'GL_DEPTH_COMPONENT16_SGIX', 'GL_DEPTH_COMPONENT24_SGIX',
'GL_DEPTH_COMPONENT32_SGIX', 'GL_FOG_FUNC_SGIS', 'GL_FOG_FUNC_POINTS_SGIS',
'GL_MAX_FOG_FUNC_POINTS_SGIS', 'GL_FOG_OFFSET_SGIX',
'GL_FOG_OFFSET_VALUE_SGIX', 'GL_IMAGE_SCALE_X_HP', 'GL_IMAGE_SCALE_Y_HP',
'GL_IMAGE_TRANSLATE_X_HP', 'GL_IMAGE_TRANSLATE_Y_HP',
'GL_IMAGE_ROTATE_ANGLE_HP', 'GL_IMAGE_ROTATE_ORIGIN_X_HP',
'GL_IMAGE_ROTATE_ORIGIN_Y_HP', 'GL_IMAGE_MAG_FILTER_HP',
'GL_IMAGE_MIN_FILTER_HP', 'GL_IMAGE_CUBIC_WEIGHT_HP', 'GL_CUBIC_HP',
'GL_AVERAGE_HP', 'GL_IMAGE_TRANSFORM_2D_HP',
'GL_POST_IMAGE_TRANSFORM_COLOR_TABLE_HP',
'GL_PROXY_POST_IMAGE_TRANSFORM_COLOR_TABLE_HP', 'GL_IGNORE_BORDER_HP',
'GL_CONSTANT_BORDER_HP', 'GL_REPLICATE_BORDER_HP',
'GL_CONVOLUTION_BORDER_COLOR_HP', 'GL_TEXTURE_ENV_BIAS_SGIX',
'GL_VERTEX_DATA_HINT_PGI', 'GL_VERTEX_CONSISTENT_HINT_PGI',
'GL_MATERIAL_SIDE_HINT_PGI', 'GL_MAX_VERTEX_HINT_PGI', 'GL_COLOR3_BIT_PGI',
'GL_COLOR4_BIT_PGI', 'GL_EDGEFLAG_BIT_PGI', 'GL_INDEX_BIT_PGI',
'GL_MAT_AMBIENT_BIT_PGI', 'GL_MAT_AMBIENT_AND_DIFFUSE_BIT_PGI',
'GL_MAT_DIFFUSE_BIT_PGI', 'GL_MAT_EMISSION_BIT_PGI',
'GL_MAT_COLOR_INDEXES_BIT_PGI', 'GL_MAT_SHININESS_BIT_PGI',
'GL_MAT_SPECULAR_BIT_PGI', 'GL_NORMAL_BIT_PGI', 'GL_TEXCOORD1_BIT_PGI',
'GL_TEXCOORD2_BIT_PGI', 'GL_TEXCOORD3_BIT_PGI', 'GL_TEXCOORD4_BIT_PGI',
'GL_VERTEX23_BIT_PGI', 'GL_VERTEX4_BIT_PGI',
'GL_PREFER_DOUBLEBUFFER_HINT_PGI', 'GL_CONSERVE_MEMORY_HINT_PGI',
'GL_RECLAIM_MEMORY_HINT_PGI', 'GL_NATIVE_GRAPHICS_HANDLE_PGI',
'GL_NATIVE_GRAPHICS_BEGIN_HINT_PGI', 'GL_NATIVE_GRAPHICS_END_HINT_PGI',
'GL_ALWAYS_FAST_HINT_PGI', 'GL_ALWAYS_SOFT_HINT_PGI',
'GL_ALLOW_DRAW_OBJ_HINT_PGI', 'GL_ALLOW_DRAW_WIN_HINT_PGI',
'GL_ALLOW_DRAW_FRG_HINT_PGI', 'GL_ALLOW_DRAW_MEM_HINT_PGI',
'GL_STRICT_DEPTHFUNC_HINT_PGI', 'GL_STRICT_LIGHTING_HINT_PGI',
'GL_STRICT_SCISSOR_HINT_PGI', 'GL_FULL_STIPPLE_HINT_PGI',
'GL_CLIP_NEAR_HINT_PGI', 'GL_CLIP_FAR_HINT_PGI', 'GL_WIDE_LINE_HINT_PGI',
'GL_BACK_NORMALS_HINT_PGI', 'GL_COLOR_INDEX1_EXT', 'GL_COLOR_INDEX2_EXT',
'GL_COLOR_INDEX4_EXT', 'GL_COLOR_INDEX8_EXT', 'GL_COLOR_INDEX12_EXT',
'GL_COLOR_INDEX16_EXT', 'GL_TEXTURE_INDEX_SIZE_EXT',
'GL_CLIP_VOLUME_CLIPPING_HINT_EXT', 'GL_LIST_PRIORITY_SGIX',
'GL_IR_INSTRUMENT1_SGIX', 'GL_CALLIGRAPHIC_FRAGMENT_SGIX',
'GL_TEXTURE_LOD_BIAS_S_SGIX', 'GL_TEXTURE_LOD_BIAS_T_SGIX',
'GL_TEXTURE_LOD_BIAS_R_SGIX', 'GL_SHADOW_AMBIENT_SGIX',
'GL_INDEX_MATERIAL_EXT', 'GL_INDEX_MATERIAL_PARAMETER_EXT',
'GL_INDEX_MATERIAL_FACE_EXT', 'GL_INDEX_TEST_EXT', 'GL_INDEX_TEST_FUNC_EXT',
'GL_INDEX_TEST_REF_EXT', 'GL_IUI_V2F_EXT', 'GL_IUI_V3F_EXT',
'GL_IUI_N3F_V2F_EXT', 'GL_IUI_N3F_V3F_EXT', 'GL_T2F_IUI_V2F_EXT',
'GL_T2F_IUI_V3F_EXT', 'GL_T2F_IUI_N3F_V2F_EXT', 'GL_T2F_IUI_N3F_V3F_EXT',
'GL_ARRAY_ELEMENT_LOCK_FIRST_EXT', 'GL_ARRAY_ELEMENT_LOCK_COUNT_EXT',
'GL_CULL_VERTEX_EXT', 'GL_CULL_VERTEX_EYE_POSITION_EXT',
'GL_CULL_VERTEX_OBJECT_POSITION_EXT', 'GL_YCRCB_422_SGIX',
'GL_YCRCB_444_SGIX', 'GL_FRAGMENT_LIGHTING_SGIX',
'GL_FRAGMENT_COLOR_MATERIAL_SGIX', 'GL_FRAGMENT_COLOR_MATERIAL_FACE_SGIX',
'GL_FRAGMENT_COLOR_MATERIAL_PARAMETER_SGIX', 'GL_MAX_FRAGMENT_LIGHTS_SGIX',
'GL_MAX_ACTIVE_LIGHTS_SGIX', 'GL_CURRENT_RASTER_NORMAL_SGIX',
'GL_LIGHT_ENV_MODE_SGIX', 'GL_FRAGMENT_LIGHT_MODEL_LOCAL_VIEWER_SGIX',
'GL_FRAGMENT_LIGHT_MODEL_TWO_SIDE_SGIX',
'GL_FRAGMENT_LIGHT_MODEL_AMBIENT_SGIX',
'GL_FRAGMENT_LIGHT_MODEL_NORMAL_INTERPOLATION_SGIX',
'GL_FRAGMENT_LIGHT0_SGIX', 'GL_FRAGMENT_LIGHT1_SGIX',
'GL_FRAGMENT_LIGHT2_SGIX', 'GL_FRAGMENT_LIGHT3_SGIX',
'GL_FRAGMENT_LIGHT4_SGIX', 'GL_FRAGMENT_LIGHT5_SGIX',
'GL_FRAGMENT_LIGHT6_SGIX', 'GL_FRAGMENT_LIGHT7_SGIX',
'GL_RASTER_POSITION_UNCLIPPED_IBM', 'GL_TEXTURE_LIGHTING_MODE_HP',
'GL_TEXTURE_POST_SPECULAR_HP', 'GL_TEXTURE_PRE_SPECULAR_HP',
'GL_MAX_ELEMENTS_VERTICES_EXT', 'GL_MAX_ELEMENTS_INDICES_EXT', 'GL_PHONG_WIN',
'GL_PHONG_HINT_WIN', 'GL_FOG_SPECULAR_TEXTURE_WIN',
'GL_FRAGMENT_MATERIAL_EXT', 'GL_FRAGMENT_NORMAL_EXT', 'GL_FRAGMENT_COLOR_EXT',
'GL_ATTENUATION_EXT', 'GL_SHADOW_ATTENUATION_EXT',
'GL_TEXTURE_APPLICATION_MODE_EXT', 'GL_TEXTURE_LIGHT_EXT',
'GL_TEXTURE_MATERIAL_FACE_EXT', 'GL_TEXTURE_MATERIAL_PARAMETER_EXT',
'GL_ALPHA_MIN_SGIX', 'GL_ALPHA_MAX_SGIX', 'GL_PIXEL_TEX_GEN_Q_CEILING_SGIX',
'GL_PIXEL_TEX_GEN_Q_ROUND_SGIX', 'GL_PIXEL_TEX_GEN_Q_FLOOR_SGIX',
'GL_PIXEL_TEX_GEN_ALPHA_REPLACE_SGIX',
'GL_PIXEL_TEX_GEN_ALPHA_NO_REPLACE_SGIX', 'GL_PIXEL_TEX_GEN_ALPHA_LS_SGIX',
'GL_PIXEL_TEX_GEN_ALPHA_MS_SGIX', 'GL_BGR_EXT', 'GL_BGRA_EXT',
'GL_ASYNC_MARKER_SGIX', 'GL_ASYNC_TEX_IMAGE_SGIX',
'GL_ASYNC_DRAW_PIXELS_SGIX', 'GL_ASYNC_READ_PIXELS_SGIX',
'GL_MAX_ASYNC_TEX_IMAGE_SGIX', 'GL_MAX_ASYNC_DRAW_PIXELS_SGIX',
'GL_MAX_ASYNC_READ_PIXELS_SGIX', 'GL_ASYNC_HISTOGRAM_SGIX',
'GL_MAX_ASYNC_HISTOGRAM_SGIX', 'GL_PARALLEL_ARRAYS_INTEL',
'GL_VERTEX_ARRAY_PARALLEL_POINTERS_INTEL',
'GL_NORMAL_ARRAY_PARALLEL_POINTERS_INTEL',
'GL_COLOR_ARRAY_PARALLEL_POINTERS_INTEL',
'GL_TEXTURE_COORD_ARRAY_PARALLEL_POINTERS_INTEL', 'GL_OCCLUSION_TEST_HP',
'GL_OCCLUSION_TEST_RESULT_HP', 'GL_PIXEL_TRANSFORM_2D_EXT',
'GL_PIXEL_MAG_FILTER_EXT', 'GL_PIXEL_MIN_FILTER_EXT',
'GL_PIXEL_CUBIC_WEIGHT_EXT', 'GL_CUBIC_EXT', 'GL_AVERAGE_EXT',
'GL_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT',
'GL_MAX_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT',
'GL_PIXEL_TRANSFORM_2D_MATRIX_EXT', 'GL_SHARED_TEXTURE_PALETTE_EXT',
'GL_LIGHT_MODEL_COLOR_CONTROL_EXT', 'GL_SINGLE_COLOR_EXT',
'GL_SEPARATE_SPECULAR_COLOR_EXT', 'GL_COLOR_SUM_EXT',
'GL_CURRENT_SECONDARY_COLOR_EXT', 'GL_SECONDARY_COLOR_ARRAY_SIZE_EXT',
'GL_SECONDARY_COLOR_ARRAY_TYPE_EXT', 'GL_SECONDARY_COLOR_ARRAY_STRIDE_EXT',
'GL_SECONDARY_COLOR_ARRAY_POINTER_EXT', 'GL_SECONDARY_COLOR_ARRAY_EXT',
'GL_PERTURB_EXT', 'GL_TEXTURE_NORMAL_EXT', 'GL_FOG_COORDINATE_SOURCE_EXT',
'GL_FOG_COORDINATE_EXT', 'GL_FRAGMENT_DEPTH_EXT',
'GL_CURRENT_FOG_COORDINATE_EXT', 'GL_FOG_COORDINATE_ARRAY_TYPE_EXT',
'GL_FOG_COORDINATE_ARRAY_STRIDE_EXT', 'GL_FOG_COORDINATE_ARRAY_POINTER_EXT',
'GL_FOG_COORDINATE_ARRAY_EXT', 'GL_SCREEN_COORDINATES_REND',
'GL_INVERTED_SCREEN_W_REND', 'GL_TANGENT_ARRAY_EXT', 'GL_BINORMAL_ARRAY_EXT',
'GL_CURRENT_TANGENT_EXT', 'GL_CURRENT_BINORMAL_EXT',
'GL_TANGENT_ARRAY_TYPE_EXT', 'GL_TANGENT_ARRAY_STRIDE_EXT',
'GL_BINORMAL_ARRAY_TYPE_EXT', 'GL_BINORMAL_ARRAY_STRIDE_EXT',
'GL_TANGENT_ARRAY_POINTER_EXT', 'GL_BINORMAL_ARRAY_POINTER_EXT',
'GL_MAP1_TANGENT_EXT', 'GL_MAP2_TANGENT_EXT', 'GL_MAP1_BINORMAL_EXT',
'GL_MAP2_BINORMAL_EXT', 'GL_COMBINE_EXT', 'GL_COMBINE_RGB_EXT',
'GL_COMBINE_ALPHA_EXT', 'GL_RGB_SCALE_EXT', 'GL_ADD_SIGNED_EXT',
'GL_INTERPOLATE_EXT', 'GL_CONSTANT_EXT', 'GL_PRIMARY_COLOR_EXT',
'GL_PREVIOUS_EXT', 'GL_SOURCE0_RGB_EXT', 'GL_SOURCE1_RGB_EXT',
'GL_SOURCE2_RGB_EXT', 'GL_SOURCE0_ALPHA_EXT', 'GL_SOURCE1_ALPHA_EXT',
'GL_SOURCE2_ALPHA_EXT', 'GL_OPERAND0_RGB_EXT', 'GL_OPERAND1_RGB_EXT',
'GL_OPERAND2_RGB_EXT', 'GL_OPERAND0_ALPHA_EXT', 'GL_OPERAND1_ALPHA_EXT',
'GL_OPERAND2_ALPHA_EXT', 'GL_LIGHT_MODEL_SPECULAR_VECTOR_APPLE',
'GL_TRANSFORM_HINT_APPLE', 'GL_FOG_SCALE_SGIX', 'GL_FOG_SCALE_VALUE_SGIX',
'GL_UNPACK_CONSTANT_DATA_SUNX', 'GL_TEXTURE_CONSTANT_DATA_SUNX',
'GL_GLOBAL_ALPHA_SUN', 'GL_GLOBAL_ALPHA_FACTOR_SUN', 'GL_RESTART_SUN',
'GL_REPLACE_MIDDLE_SUN', 'GL_REPLACE_OLDEST_SUN', 'GL_TRIANGLE_LIST_SUN',
'GL_REPLACEMENT_CODE_SUN', 'GL_REPLACEMENT_CODE_ARRAY_SUN',
'GL_REPLACEMENT_CODE_ARRAY_TYPE_SUN', 'GL_REPLACEMENT_CODE_ARRAY_STRIDE_SUN',
'GL_REPLACEMENT_CODE_ARRAY_POINTER_SUN', 'GL_R1UI_V3F_SUN',
'GL_R1UI_C4UB_V3F_SUN', 'GL_R1UI_C3F_V3F_SUN', 'GL_R1UI_N3F_V3F_SUN',
'GL_R1UI_C4F_N3F_V3F_SUN', 'GL_R1UI_T2F_V3F_SUN', 'GL_R1UI_T2F_N3F_V3F_SUN',
'GL_R1UI_T2F_C4F_N3F_V3F_SUN', 'GL_BLEND_DST_RGB_EXT', 'GL_BLEND_SRC_RGB_EXT',
'GL_BLEND_DST_ALPHA_EXT', 'GL_BLEND_SRC_ALPHA_EXT', 'GL_RED_MIN_CLAMP_INGR',
'GL_GREEN_MIN_CLAMP_INGR', 'GL_BLUE_MIN_CLAMP_INGR',
'GL_ALPHA_MIN_CLAMP_INGR', 'GL_RED_MAX_CLAMP_INGR', 'GL_GREEN_MAX_CLAMP_INGR',
'GL_BLUE_MAX_CLAMP_INGR', 'GL_ALPHA_MAX_CLAMP_INGR', 'GL_INTERLACE_READ_INGR',
'GL_INCR_WRAP_EXT', 'GL_DECR_WRAP_EXT', 'GL_422_EXT', 'GL_422_REV_EXT',
'GL_422_AVERAGE_EXT', 'GL_422_REV_AVERAGE_EXT', 'GL_NORMAL_MAP_NV',
'GL_REFLECTION_MAP_NV', 'GL_NORMAL_MAP_EXT', 'GL_REFLECTION_MAP_EXT',
'GL_TEXTURE_CUBE_MAP_EXT', 'GL_TEXTURE_BINDING_CUBE_MAP_EXT',
'GL_TEXTURE_CUBE_MAP_POSITIVE_X_EXT', 'GL_TEXTURE_CUBE_MAP_NEGATIVE_X_EXT',
'GL_TEXTURE_CUBE_MAP_POSITIVE_Y_EXT', 'GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_EXT',
'GL_TEXTURE_CUBE_MAP_POSITIVE_Z_EXT', 'GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_EXT',
'GL_PROXY_TEXTURE_CUBE_MAP_EXT', 'GL_MAX_CUBE_MAP_TEXTURE_SIZE_EXT',
'GL_WRAP_BORDER_SUN', 'GL_MAX_TEXTURE_LOD_BIAS_EXT',
'GL_TEXTURE_FILTER_CONTROL_EXT', 'GL_TEXTURE_LOD_BIAS_EXT',
'GL_TEXTURE_MAX_ANISOTROPY_EXT', 'GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT',
'GL_MODELVIEW0_STACK_DEPTH_EXT', 'GL_MODELVIEW1_STACK_DEPTH_EXT',
'GL_MODELVIEW0_MATRIX_EXT', 'GL_MODELVIEW1_MATRIX_EXT',
'GL_VERTEX_WEIGHTING_EXT', 'GL_MODELVIEW0_EXT', 'GL_MODELVIEW1_EXT',
'GL_CURRENT_VERTEX_WEIGHT_EXT', 'GL_VERTEX_WEIGHT_ARRAY_EXT',
'GL_VERTEX_WEIGHT_ARRAY_SIZE_EXT', 'GL_VERTEX_WEIGHT_ARRAY_TYPE_EXT',
'GL_VERTEX_WEIGHT_ARRAY_STRIDE_EXT', 'GL_VERTEX_WEIGHT_ARRAY_POINTER_EXT',
'GL_MAX_SHININESS_NV', 'GL_MAX_SPOT_EXPONENT_NV', 'GL_VERTEX_ARRAY_RANGE_NV',
'GL_VERTEX_ARRAY_RANGE_LENGTH_NV', 'GL_VERTEX_ARRAY_RANGE_VALID_NV',
'GL_MAX_VERTEX_ARRAY_RANGE_ELEMENT_NV', 'GL_VERTEX_ARRAY_RANGE_POINTER_NV',
'GL_REGISTER_COMBINERS_NV', 'GL_VARIABLE_A_NV', 'GL_VARIABLE_B_NV',
'GL_VARIABLE_C_NV', 'GL_VARIABLE_D_NV', 'GL_VARIABLE_E_NV',
'GL_VARIABLE_F_NV', 'GL_VARIABLE_G_NV', 'GL_CONSTANT_COLOR0_NV',
'GL_CONSTANT_COLOR1_NV', 'GL_PRIMARY_COLOR_NV', 'GL_SECONDARY_COLOR_NV',
'GL_SPARE0_NV', 'GL_SPARE1_NV', 'GL_DISCARD_NV', 'GL_E_TIMES_F_NV',
'GL_SPARE0_PLUS_SECONDARY_COLOR_NV', 'GL_UNSIGNED_IDENTITY_NV',
'GL_UNSIGNED_INVERT_NV', 'GL_EXPAND_NORMAL_NV', 'GL_EXPAND_NEGATE_NV',
'GL_HALF_BIAS_NORMAL_NV', 'GL_HALF_BIAS_NEGATE_NV', 'GL_SIGNED_IDENTITY_NV',
'GL_SIGNED_NEGATE_NV', 'GL_SCALE_BY_TWO_NV', 'GL_SCALE_BY_FOUR_NV',
'GL_SCALE_BY_ONE_HALF_NV', 'GL_BIAS_BY_NEGATIVE_ONE_HALF_NV',
'GL_COMBINER_INPUT_NV', 'GL_COMBINER_MAPPING_NV',
'GL_COMBINER_COMPONENT_USAGE_NV', 'GL_COMBINER_AB_DOT_PRODUCT_NV',
'GL_COMBINER_CD_DOT_PRODUCT_NV', 'GL_COMBINER_MUX_SUM_NV',
'GL_COMBINER_SCALE_NV', 'GL_COMBINER_BIAS_NV', 'GL_COMBINER_AB_OUTPUT_NV',
'GL_COMBINER_CD_OUTPUT_NV', 'GL_COMBINER_SUM_OUTPUT_NV',
'GL_MAX_GENERAL_COMBINERS_NV', 'GL_NUM_GENERAL_COMBINERS_NV',
'GL_COLOR_SUM_CLAMP_NV', 'GL_COMBINER0_NV', 'GL_COMBINER1_NV',
'GL_COMBINER2_NV', 'GL_COMBINER3_NV', 'GL_COMBINER4_NV', 'GL_COMBINER5_NV',
'GL_COMBINER6_NV', 'GL_COMBINER7_NV', 'GL_FOG_DISTANCE_MODE_NV',
'GL_EYE_RADIAL_NV', 'GL_EYE_PLANE_ABSOLUTE_NV', 'GL_EMBOSS_LIGHT_NV',
'GL_EMBOSS_CONSTANT_NV', 'GL_EMBOSS_MAP_NV', 'GL_COMBINE4_NV',
'GL_SOURCE3_RGB_NV', 'GL_SOURCE3_ALPHA_NV', 'GL_OPERAND3_RGB_NV',
'GL_OPERAND3_ALPHA_NV', 'GL_COMPRESSED_RGB_S3TC_DXT1_EXT',
'GL_COMPRESSED_RGBA_S3TC_DXT1_EXT', 'GL_COMPRESSED_RGBA_S3TC_DXT3_EXT',
'GL_COMPRESSED_RGBA_S3TC_DXT5_EXT', 'GL_CULL_VERTEX_IBM',
'GL_VERTEX_ARRAY_LIST_IBM', 'GL_NORMAL_ARRAY_LIST_IBM',
'GL_COLOR_ARRAY_LIST_IBM', 'GL_INDEX_ARRAY_LIST_IBM',
'GL_TEXTURE_COORD_ARRAY_LIST_IBM', 'GL_EDGE_FLAG_ARRAY_LIST_IBM',
'GL_FOG_COORDINATE_ARRAY_LIST_IBM', 'GL_SECONDARY_COLOR_ARRAY_LIST_IBM',
'GL_VERTEX_ARRAY_LIST_STRIDE_IBM', 'GL_NORMAL_ARRAY_LIST_STRIDE_IBM',
'GL_COLOR_ARRAY_LIST_STRIDE_IBM', 'GL_INDEX_ARRAY_LIST_STRIDE_IBM',
'GL_TEXTURE_COORD_ARRAY_LIST_STRIDE_IBM',
'GL_EDGE_FLAG_ARRAY_LIST_STRIDE_IBM',
'GL_FOG_COORDINATE_ARRAY_LIST_STRIDE_IBM',
'GL_SECONDARY_COLOR_ARRAY_LIST_STRIDE_IBM', 'GL_PACK_SUBSAMPLE_RATE_SGIX',
'GL_UNPACK_SUBSAMPLE_RATE_SGIX', 'GL_PIXEL_SUBSAMPLE_4444_SGIX',
'GL_PIXEL_SUBSAMPLE_2424_SGIX', 'GL_PIXEL_SUBSAMPLE_4242_SGIX',
'GL_YCRCB_SGIX', 'GL_YCRCBA_SGIX', 'GL_DEPTH_PASS_INSTRUMENT_SGIX',
'GL_DEPTH_PASS_INSTRUMENT_COUNTERS_SGIX', 'GL_DEPTH_PASS_INSTRUMENT_MAX_SGIX',
'GL_COMPRESSED_RGB_FXT1_3DFX', 'GL_COMPRESSED_RGBA_FXT1_3DFX',
'GL_MULTISAMPLE_3DFX', 'GL_SAMPLE_BUFFERS_3DFX', 'GL_SAMPLES_3DFX',
'GL_MULTISAMPLE_BIT_3DFX', 'GL_MULTISAMPLE_EXT',
'GL_SAMPLE_ALPHA_TO_MASK_EXT', 'GL_SAMPLE_ALPHA_TO_ONE_EXT',
'GL_SAMPLE_MASK_EXT', 'GL_1PASS_EXT', 'GL_2PASS_0_EXT', 'GL_2PASS_1_EXT',
'GL_4PASS_0_EXT', 'GL_4PASS_1_EXT', 'GL_4PASS_2_EXT', 'GL_4PASS_3_EXT',
'GL_SAMPLE_BUFFERS_EXT', 'GL_SAMPLES_EXT', 'GL_SAMPLE_MASK_VALUE_EXT',
'GL_SAMPLE_MASK_INVERT_EXT', 'GL_SAMPLE_PATTERN_EXT',
'GL_MULTISAMPLE_BIT_EXT', 'GL_VERTEX_PRECLIP_SGIX',
'GL_VERTEX_PRECLIP_HINT_SGIX', 'GL_CONVOLUTION_HINT_SGIX',
'GL_PACK_RESAMPLE_SGIX', 'GL_UNPACK_RESAMPLE_SGIX',
'GL_RESAMPLE_REPLICATE_SGIX', 'GL_RESAMPLE_ZERO_FILL_SGIX',
'GL_RESAMPLE_DECIMATE_SGIX', 'GL_EYE_DISTANCE_TO_POINT_SGIS',
'GL_OBJECT_DISTANCE_TO_POINT_SGIS', 'GL_EYE_DISTANCE_TO_LINE_SGIS',
'GL_OBJECT_DISTANCE_TO_LINE_SGIS', 'GL_EYE_POINT_SGIS',
'GL_OBJECT_POINT_SGIS', 'GL_EYE_LINE_SGIS', 'GL_OBJECT_LINE_SGIS',
'GL_TEXTURE_COLOR_WRITEMASK_SGIS', 'GL_DOT3_RGB_EXT', 'GL_DOT3_RGBA_EXT',
'GL_MIRROR_CLAMP_ATI', 'GL_MIRROR_CLAMP_TO_EDGE_ATI', 'GL_ALL_COMPLETED_NV',
'GL_FENCE_STATUS_NV', 'GL_FENCE_CONDITION_NV', 'GL_MIRRORED_REPEAT_IBM',
'GL_EVAL_2D_NV', 'GL_EVAL_TRIANGULAR_2D_NV', 'GL_MAP_TESSELLATION_NV',
'GL_MAP_ATTRIB_U_ORDER_NV', 'GL_MAP_ATTRIB_V_ORDER_NV',
'GL_EVAL_FRACTIONAL_TESSELLATION_NV', 'GL_EVAL_VERTEX_ATTRIB0_NV',
'GL_EVAL_VERTEX_ATTRIB1_NV', 'GL_EVAL_VERTEX_ATTRIB2_NV',
'GL_EVAL_VERTEX_ATTRIB3_NV', 'GL_EVAL_VERTEX_ATTRIB4_NV',
'GL_EVAL_VERTEX_ATTRIB5_NV', 'GL_EVAL_VERTEX_ATTRIB6_NV',
'GL_EVAL_VERTEX_ATTRIB7_NV', 'GL_EVAL_VERTEX_ATTRIB8_NV',
'GL_EVAL_VERTEX_ATTRIB9_NV', 'GL_EVAL_VERTEX_ATTRIB10_NV',
'GL_EVAL_VERTEX_ATTRIB11_NV', 'GL_EVAL_VERTEX_ATTRIB12_NV',
'GL_EVAL_VERTEX_ATTRIB13_NV', 'GL_EVAL_VERTEX_ATTRIB14_NV',
'GL_EVAL_VERTEX_ATTRIB15_NV', 'GL_MAX_MAP_TESSELLATION_NV',
'GL_MAX_RATIONAL_EVAL_ORDER_NV', 'GL_DEPTH_STENCIL_NV',
'GL_UNSIGNED_INT_24_8_NV', 'GL_PER_STAGE_CONSTANTS_NV',
'GL_TEXTURE_RECTANGLE_NV', 'GL_TEXTURE_BINDING_RECTANGLE_NV',
'GL_PROXY_TEXTURE_RECTANGLE_NV', 'GL_MAX_RECTANGLE_TEXTURE_SIZE_NV',
'GL_OFFSET_TEXTURE_RECTANGLE_NV', 'GL_OFFSET_TEXTURE_RECTANGLE_SCALE_NV',
'GL_DOT_PRODUCT_TEXTURE_RECTANGLE_NV',
'GL_RGBA_UNSIGNED_DOT_PRODUCT_MAPPING_NV', 'GL_UNSIGNED_INT_S8_S8_8_8_NV',
'GL_UNSIGNED_INT_8_8_S8_S8_REV_NV', 'GL_DSDT_MAG_INTENSITY_NV',
'GL_SHADER_CONSISTENT_NV', 'GL_TEXTURE_SHADER_NV', 'GL_SHADER_OPERATION_NV',
'GL_CULL_MODES_NV', 'GL_OFFSET_TEXTURE_MATRIX_NV',
'GL_OFFSET_TEXTURE_SCALE_NV', 'GL_OFFSET_TEXTURE_BIAS_NV',
'GL_OFFSET_TEXTURE_2D_MATRIX_NV', 'GL_OFFSET_TEXTURE_2D_SCALE_NV',
'GL_OFFSET_TEXTURE_2D_BIAS_NV', 'GL_PREVIOUS_TEXTURE_INPUT_NV',
'GL_CONST_EYE_NV', 'GL_PASS_THROUGH_NV', 'GL_CULL_FRAGMENT_NV',
'GL_OFFSET_TEXTURE_2D_NV', 'GL_DEPENDENT_AR_TEXTURE_2D_NV',
'GL_DEPENDENT_GB_TEXTURE_2D_NV', 'GL_DOT_PRODUCT_NV',
'GL_DOT_PRODUCT_DEPTH_REPLACE_NV', 'GL_DOT_PRODUCT_TEXTURE_2D_NV',
'GL_DOT_PRODUCT_TEXTURE_CUBE_MAP_NV', 'GL_DOT_PRODUCT_DIFFUSE_CUBE_MAP_NV',
'GL_DOT_PRODUCT_REFLECT_CUBE_MAP_NV',
'GL_DOT_PRODUCT_CONST_EYE_REFLECT_CUBE_MAP_NV', 'GL_HILO_NV', 'GL_DSDT_NV',
'GL_DSDT_MAG_NV', 'GL_DSDT_MAG_VIB_NV', 'GL_HILO16_NV', 'GL_SIGNED_HILO_NV',
'GL_SIGNED_HILO16_NV', 'GL_SIGNED_RGBA_NV', 'GL_SIGNED_RGBA8_NV',
'GL_SIGNED_RGB_NV', 'GL_SIGNED_RGB8_NV', 'GL_SIGNED_LUMINANCE_NV',
'GL_SIGNED_LUMINANCE8_NV', 'GL_SIGNED_LUMINANCE_ALPHA_NV',
'GL_SIGNED_LUMINANCE8_ALPHA8_NV', 'GL_SIGNED_ALPHA_NV', 'GL_SIGNED_ALPHA8_NV',
'GL_SIGNED_INTENSITY_NV', 'GL_SIGNED_INTENSITY8_NV', 'GL_DSDT8_NV',
'GL_DSDT8_MAG8_NV', 'GL_DSDT8_MAG8_INTENSITY8_NV',
'GL_SIGNED_RGB_UNSIGNED_ALPHA_NV', 'GL_SIGNED_RGB8_UNSIGNED_ALPHA8_NV',
'GL_HI_SCALE_NV', 'GL_LO_SCALE_NV', 'GL_DS_SCALE_NV', 'GL_DT_SCALE_NV',
'GL_MAGNITUDE_SCALE_NV', 'GL_VIBRANCE_SCALE_NV', 'GL_HI_BIAS_NV',
'GL_LO_BIAS_NV', 'GL_DS_BIAS_NV', 'GL_DT_BIAS_NV', 'GL_MAGNITUDE_BIAS_NV',
'GL_VIBRANCE_BIAS_NV', 'GL_TEXTURE_BORDER_VALUES_NV', 'GL_TEXTURE_HI_SIZE_NV',
'GL_TEXTURE_LO_SIZE_NV', 'GL_TEXTURE_DS_SIZE_NV', 'GL_TEXTURE_DT_SIZE_NV',
'GL_TEXTURE_MAG_SIZE_NV', 'GL_DOT_PRODUCT_TEXTURE_3D_NV',
'GL_VERTEX_ARRAY_RANGE_WITHOUT_FLUSH_NV', 'GL_VERTEX_PROGRAM_NV',
'GL_VERTEX_STATE_PROGRAM_NV', 'GL_ATTRIB_ARRAY_SIZE_NV',
'GL_ATTRIB_ARRAY_STRIDE_NV', 'GL_ATTRIB_ARRAY_TYPE_NV',
'GL_CURRENT_ATTRIB_NV', 'GL_PROGRAM_LENGTH_NV', 'GL_PROGRAM_STRING_NV',
'GL_MODELVIEW_PROJECTION_NV', 'GL_IDENTITY_NV', 'GL_INVERSE_NV',
'GL_TRANSPOSE_NV', 'GL_INVERSE_TRANSPOSE_NV',
'GL_MAX_TRACK_MATRIX_STACK_DEPTH_NV', 'GL_MAX_TRACK_MATRICES_NV',
'GL_MATRIX0_NV', 'GL_MATRIX1_NV', 'GL_MATRIX2_NV', 'GL_MATRIX3_NV',
'GL_MATRIX4_NV', 'GL_MATRIX5_NV', 'GL_MATRIX6_NV', 'GL_MATRIX7_NV',
'GL_CURRENT_MATRIX_STACK_DEPTH_NV', 'GL_CURRENT_MATRIX_NV',
'GL_VERTEX_PROGRAM_POINT_SIZE_NV', 'GL_VERTEX_PROGRAM_TWO_SIDE_NV',
'GL_PROGRAM_PARAMETER_NV', 'GL_ATTRIB_ARRAY_POINTER_NV',
'GL_PROGRAM_TARGET_NV', 'GL_PROGRAM_RESIDENT_NV', 'GL_TRACK_MATRIX_NV',
'GL_TRACK_MATRIX_TRANSFORM_NV', 'GL_VERTEX_PROGRAM_BINDING_NV',
'GL_PROGRAM_ERROR_POSITION_NV', 'GL_VERTEX_ATTRIB_ARRAY0_NV',
'GL_VERTEX_ATTRIB_ARRAY1_NV', 'GL_VERTEX_ATTRIB_ARRAY2_NV',
'GL_VERTEX_ATTRIB_ARRAY3_NV', 'GL_VERTEX_ATTRIB_ARRAY4_NV',
'GL_VERTEX_ATTRIB_ARRAY5_NV', 'GL_VERTEX_ATTRIB_ARRAY6_NV',
'GL_VERTEX_ATTRIB_ARRAY7_NV', 'GL_VERTEX_ATTRIB_ARRAY8_NV',
'GL_VERTEX_ATTRIB_ARRAY9_NV', 'GL_VERTEX_ATTRIB_ARRAY10_NV',
'GL_VERTEX_ATTRIB_ARRAY11_NV', 'GL_VERTEX_ATTRIB_ARRAY12_NV',
'GL_VERTEX_ATTRIB_ARRAY13_NV', 'GL_VERTEX_ATTRIB_ARRAY14_NV',
'GL_VERTEX_ATTRIB_ARRAY15_NV', 'GL_MAP1_VERTEX_ATTRIB0_4_NV',
'GL_MAP1_VERTEX_ATTRIB1_4_NV', 'GL_MAP1_VERTEX_ATTRIB2_4_NV',
'GL_MAP1_VERTEX_ATTRIB3_4_NV', 'GL_MAP1_VERTEX_ATTRIB4_4_NV',
'GL_MAP1_VERTEX_ATTRIB5_4_NV', 'GL_MAP1_VERTEX_ATTRIB6_4_NV',
'GL_MAP1_VERTEX_ATTRIB7_4_NV', 'GL_MAP1_VERTEX_ATTRIB8_4_NV',
'GL_MAP1_VERTEX_ATTRIB9_4_NV', 'GL_MAP1_VERTEX_ATTRIB10_4_NV',
'GL_MAP1_VERTEX_ATTRIB11_4_NV', 'GL_MAP1_VERTEX_ATTRIB12_4_NV',
'GL_MAP1_VERTEX_ATTRIB13_4_NV', 'GL_MAP1_VERTEX_ATTRIB14_4_NV',
'GL_MAP1_VERTEX_ATTRIB15_4_NV', 'GL_MAP2_VERTEX_ATTRIB0_4_NV',
'GL_MAP2_VERTEX_ATTRIB1_4_NV', 'GL_MAP2_VERTEX_ATTRIB2_4_NV',
'GL_MAP2_VERTEX_ATTRIB3_4_NV', 'GL_MAP2_VERTEX_ATTRIB4_4_NV',
'GL_MAP2_VERTEX_ATTRIB5_4_NV', 'GL_MAP2_VERTEX_ATTRIB6_4_NV',
'GL_MAP2_VERTEX_ATTRIB7_4_NV', 'GL_MAP2_VERTEX_ATTRIB8_4_NV',
'GL_MAP2_VERTEX_ATTRIB9_4_NV', 'GL_MAP2_VERTEX_ATTRIB10_4_NV',
'GL_MAP2_VERTEX_ATTRIB11_4_NV', 'GL_MAP2_VERTEX_ATTRIB12_4_NV',
'GL_MAP2_VERTEX_ATTRIB13_4_NV', 'GL_MAP2_VERTEX_ATTRIB14_4_NV',
'GL_MAP2_VERTEX_ATTRIB15_4_NV', 'GL_TEXTURE_MAX_CLAMP_S_SGIX',
'GL_TEXTURE_MAX_CLAMP_T_SGIX', 'GL_TEXTURE_MAX_CLAMP_R_SGIX',
'GL_SCALEBIAS_HINT_SGIX', 'GL_INTERLACE_OML', 'GL_INTERLACE_READ_OML',
'GL_FORMAT_SUBSAMPLE_24_24_OML', 'GL_FORMAT_SUBSAMPLE_244_244_OML',
'GL_PACK_RESAMPLE_OML', 'GL_UNPACK_RESAMPLE_OML', 'GL_RESAMPLE_REPLICATE_OML',
'GL_RESAMPLE_ZERO_FILL_OML', 'GL_RESAMPLE_AVERAGE_OML',
'GL_RESAMPLE_DECIMATE_OML', 'GL_DEPTH_STENCIL_TO_RGBA_NV',
'GL_DEPTH_STENCIL_TO_BGRA_NV', 'GL_BUMP_ROT_MATRIX_ATI',
'GL_BUMP_ROT_MATRIX_SIZE_ATI', 'GL_BUMP_NUM_TEX_UNITS_ATI',
'GL_BUMP_TEX_UNITS_ATI', 'GL_DUDV_ATI', 'GL_DU8DV8_ATI', 'GL_BUMP_ENVMAP_ATI',
'GL_BUMP_TARGET_ATI', 'GL_FRAGMENT_SHADER_ATI', 'GL_REG_0_ATI',
'GL_REG_1_ATI', 'GL_REG_2_ATI', 'GL_REG_3_ATI', 'GL_REG_4_ATI',
'GL_REG_5_ATI', 'GL_REG_6_ATI', 'GL_REG_7_ATI', 'GL_REG_8_ATI',
'GL_REG_9_ATI', 'GL_REG_10_ATI', 'GL_REG_11_ATI', 'GL_REG_12_ATI',
'GL_REG_13_ATI', 'GL_REG_14_ATI', 'GL_REG_15_ATI', 'GL_REG_16_ATI',
'GL_REG_17_ATI', 'GL_REG_18_ATI', 'GL_REG_19_ATI', 'GL_REG_20_ATI',
'GL_REG_21_ATI', 'GL_REG_22_ATI', 'GL_REG_23_ATI', 'GL_REG_24_ATI',
'GL_REG_25_ATI', 'GL_REG_26_ATI', 'GL_REG_27_ATI', 'GL_REG_28_ATI',
'GL_REG_29_ATI', 'GL_REG_30_ATI', 'GL_REG_31_ATI', 'GL_CON_0_ATI',
'GL_CON_1_ATI', 'GL_CON_2_ATI', 'GL_CON_3_ATI', 'GL_CON_4_ATI',
'GL_CON_5_ATI', 'GL_CON_6_ATI', 'GL_CON_7_ATI', 'GL_CON_8_ATI',
'GL_CON_9_ATI', 'GL_CON_10_ATI', 'GL_CON_11_ATI', 'GL_CON_12_ATI',
'GL_CON_13_ATI', 'GL_CON_14_ATI', 'GL_CON_15_ATI', 'GL_CON_16_ATI',
'GL_CON_17_ATI', 'GL_CON_18_ATI', 'GL_CON_19_ATI', 'GL_CON_20_ATI',
'GL_CON_21_ATI', 'GL_CON_22_ATI', 'GL_CON_23_ATI', 'GL_CON_24_ATI',
'GL_CON_25_ATI', 'GL_CON_26_ATI', 'GL_CON_27_ATI', 'GL_CON_28_ATI',
'GL_CON_29_ATI', 'GL_CON_30_ATI', 'GL_CON_31_ATI', 'GL_MOV_ATI', 'GL_ADD_ATI',
'GL_MUL_ATI', 'GL_SUB_ATI', 'GL_DOT3_ATI', 'GL_DOT4_ATI', 'GL_MAD_ATI',
'GL_LERP_ATI', 'GL_CND_ATI', 'GL_CND0_ATI', 'GL_DOT2_ADD_ATI',
'GL_SECONDARY_INTERPOLATOR_ATI', 'GL_NUM_FRAGMENT_REGISTERS_ATI',
'GL_NUM_FRAGMENT_CONSTANTS_ATI', 'GL_NUM_PASSES_ATI',
'GL_NUM_INSTRUCTIONS_PER_PASS_ATI', 'GL_NUM_INSTRUCTIONS_TOTAL_ATI',
'GL_NUM_INPUT_INTERPOLATOR_COMPONENTS_ATI', 'GL_NUM_LOOPBACK_COMPONENTS_ATI',
'GL_COLOR_ALPHA_PAIRING_ATI', 'GL_SWIZZLE_STR_ATI', 'GL_SWIZZLE_STQ_ATI',
'GL_SWIZZLE_STR_DR_ATI', 'GL_SWIZZLE_STQ_DQ_ATI', 'GL_SWIZZLE_STRQ_ATI',
'GL_SWIZZLE_STRQ_DQ_ATI', 'GL_RED_BIT_ATI', 'GL_GREEN_BIT_ATI',
'GL_BLUE_BIT_ATI', 'GL_2X_BIT_ATI', 'GL_4X_BIT_ATI', 'GL_8X_BIT_ATI',
'GL_HALF_BIT_ATI', 'GL_QUARTER_BIT_ATI', 'GL_EIGHTH_BIT_ATI',
'GL_SATURATE_BIT_ATI', 'GL_COMP_BIT_ATI', 'GL_NEGATE_BIT_ATI',
'GL_BIAS_BIT_ATI', 'GL_PN_TRIANGLES_ATI',
'GL_MAX_PN_TRIANGLES_TESSELATION_LEVEL_ATI', 'GL_PN_TRIANGLES_POINT_MODE_ATI',
'GL_PN_TRIANGLES_NORMAL_MODE_ATI', 'GL_PN_TRIANGLES_TESSELATION_LEVEL_ATI',
'GL_PN_TRIANGLES_POINT_MODE_LINEAR_ATI',
'GL_PN_TRIANGLES_POINT_MODE_CUBIC_ATI',
'GL_PN_TRIANGLES_NORMAL_MODE_LINEAR_ATI',
'GL_PN_TRIANGLES_NORMAL_MODE_QUADRATIC_ATI', 'GL_STATIC_ATI',
'GL_DYNAMIC_ATI', 'GL_PRESERVE_ATI', 'GL_DISCARD_ATI',
'GL_OBJECT_BUFFER_SIZE_ATI', 'GL_OBJECT_BUFFER_USAGE_ATI',
'GL_ARRAY_OBJECT_BUFFER_ATI', 'GL_ARRAY_OBJECT_OFFSET_ATI',
'GL_VERTEX_SHADER_EXT', 'GL_VERTEX_SHADER_BINDING_EXT', 'GL_OP_INDEX_EXT',
'GL_OP_NEGATE_EXT', 'GL_OP_DOT3_EXT', 'GL_OP_DOT4_EXT', 'GL_OP_MUL_EXT',
'GL_OP_ADD_EXT', 'GL_OP_MADD_EXT', 'GL_OP_FRAC_EXT', 'GL_OP_MAX_EXT',
'GL_OP_MIN_EXT', 'GL_OP_SET_GE_EXT', 'GL_OP_SET_LT_EXT', 'GL_OP_CLAMP_EXT',
'GL_OP_FLOOR_EXT', 'GL_OP_ROUND_EXT', 'GL_OP_EXP_BASE_2_EXT',
'GL_OP_LOG_BASE_2_EXT', 'GL_OP_POWER_EXT', 'GL_OP_RECIP_EXT',
'GL_OP_RECIP_SQRT_EXT', 'GL_OP_SUB_EXT', 'GL_OP_CROSS_PRODUCT_EXT',
'GL_OP_MULTIPLY_MATRIX_EXT', 'GL_OP_MOV_EXT', 'GL_OUTPUT_VERTEX_EXT',
'GL_OUTPUT_COLOR0_EXT', 'GL_OUTPUT_COLOR1_EXT',
'GL_OUTPUT_TEXTURE_COORD0_EXT', 'GL_OUTPUT_TEXTURE_COORD1_EXT',
'GL_OUTPUT_TEXTURE_COORD2_EXT', 'GL_OUTPUT_TEXTURE_COORD3_EXT',
'GL_OUTPUT_TEXTURE_COORD4_EXT', 'GL_OUTPUT_TEXTURE_COORD5_EXT',
'GL_OUTPUT_TEXTURE_COORD6_EXT', 'GL_OUTPUT_TEXTURE_COORD7_EXT',
'GL_OUTPUT_TEXTURE_COORD8_EXT', 'GL_OUTPUT_TEXTURE_COORD9_EXT',
'GL_OUTPUT_TEXTURE_COORD10_EXT', 'GL_OUTPUT_TEXTURE_COORD11_EXT',
'GL_OUTPUT_TEXTURE_COORD12_EXT', 'GL_OUTPUT_TEXTURE_COORD13_EXT',
'GL_OUTPUT_TEXTURE_COORD14_EXT', 'GL_OUTPUT_TEXTURE_COORD15_EXT',
'GL_OUTPUT_TEXTURE_COORD16_EXT', 'GL_OUTPUT_TEXTURE_COORD17_EXT',
'GL_OUTPUT_TEXTURE_COORD18_EXT', 'GL_OUTPUT_TEXTURE_COORD19_EXT',
'GL_OUTPUT_TEXTURE_COORD20_EXT', 'GL_OUTPUT_TEXTURE_COORD21_EXT',
'GL_OUTPUT_TEXTURE_COORD22_EXT', 'GL_OUTPUT_TEXTURE_COORD23_EXT',
'GL_OUTPUT_TEXTURE_COORD24_EXT', 'GL_OUTPUT_TEXTURE_COORD25_EXT',
'GL_OUTPUT_TEXTURE_COORD26_EXT', 'GL_OUTPUT_TEXTURE_COORD27_EXT',
'GL_OUTPUT_TEXTURE_COORD28_EXT', 'GL_OUTPUT_TEXTURE_COORD29_EXT',
'GL_OUTPUT_TEXTURE_COORD30_EXT', 'GL_OUTPUT_TEXTURE_COORD31_EXT',
'GL_OUTPUT_FOG_EXT', 'GL_SCALAR_EXT', 'GL_VECTOR_EXT', 'GL_MATRIX_EXT',
'GL_VARIANT_EXT', 'GL_INVARIANT_EXT', 'GL_LOCAL_CONSTANT_EXT', 'GL_LOCAL_EXT',
'GL_MAX_VERTEX_SHADER_INSTRUCTIONS_EXT', 'GL_MAX_VERTEX_SHADER_VARIANTS_EXT',
'GL_MAX_VERTEX_SHADER_INVARIANTS_EXT',
'GL_MAX_VERTEX_SHADER_LOCAL_CONSTANTS_EXT', 'GL_MAX_VERTEX_SHADER_LOCALS_EXT',
'GL_MAX_OPTIMIZED_VERTEX_SHADER_INSTRUCTIONS_EXT',
'GL_MAX_OPTIMIZED_VERTEX_SHADER_VARIANTS_EXT',
'GL_MAX_OPTIMIZED_VERTEX_SHADER_LOCAL_CONSTANTS_EXT',
'GL_MAX_OPTIMIZED_VERTEX_SHADER_INVARIANTS_EXT',
'GL_MAX_OPTIMIZED_VERTEX_SHADER_LOCALS_EXT',
'GL_VERTEX_SHADER_INSTRUCTIONS_EXT', 'GL_VERTEX_SHADER_VARIANTS_EXT',
'GL_VERTEX_SHADER_INVARIANTS_EXT', 'GL_VERTEX_SHADER_LOCAL_CONSTANTS_EXT',
'GL_VERTEX_SHADER_LOCALS_EXT', 'GL_VERTEX_SHADER_OPTIMIZED_EXT', 'GL_X_EXT',
'GL_Y_EXT', 'GL_Z_EXT', 'GL_W_EXT', 'GL_NEGATIVE_X_EXT', 'GL_NEGATIVE_Y_EXT',
'GL_NEGATIVE_Z_EXT', 'GL_NEGATIVE_W_EXT', 'GL_ZERO_EXT', 'GL_ONE_EXT',
'GL_NEGATIVE_ONE_EXT', 'GL_NORMALIZED_RANGE_EXT', 'GL_FULL_RANGE_EXT',
'GL_CURRENT_VERTEX_EXT', 'GL_MVP_MATRIX_EXT', 'GL_VARIANT_VALUE_EXT',
'GL_VARIANT_DATATYPE_EXT', 'GL_VARIANT_ARRAY_STRIDE_EXT',
'GL_VARIANT_ARRAY_TYPE_EXT', 'GL_VARIANT_ARRAY_EXT',
'GL_VARIANT_ARRAY_POINTER_EXT', 'GL_INVARIANT_VALUE_EXT',
'GL_INVARIANT_DATATYPE_EXT', 'GL_LOCAL_CONSTANT_VALUE_EXT',
'GL_LOCAL_CONSTANT_DATATYPE_EXT', 'GL_MAX_VERTEX_STREAMS_ATI',
'GL_VERTEX_STREAM0_ATI', 'GL_VERTEX_STREAM1_ATI', 'GL_VERTEX_STREAM2_ATI',
'GL_VERTEX_STREAM3_ATI', 'GL_VERTEX_STREAM4_ATI', 'GL_VERTEX_STREAM5_ATI',
'GL_VERTEX_STREAM6_ATI', 'GL_VERTEX_STREAM7_ATI', 'GL_VERTEX_SOURCE_ATI',
'GL_ELEMENT_ARRAY_ATI', 'GL_ELEMENT_ARRAY_TYPE_ATI',
'GL_ELEMENT_ARRAY_POINTER_ATI', 'GL_QUAD_MESH_SUN', 'GL_TRIANGLE_MESH_SUN',
'GL_SLICE_ACCUM_SUN', 'GL_MULTISAMPLE_FILTER_HINT_NV', 'GL_DEPTH_CLAMP_NV',
'GL_PIXEL_COUNTER_BITS_NV', 'GL_CURRENT_OCCLUSION_QUERY_ID_NV',
'GL_PIXEL_COUNT_NV', 'GL_PIXEL_COUNT_AVAILABLE_NV', 'GL_POINT_SPRITE_NV',
'GL_COORD_REPLACE_NV', 'GL_POINT_SPRITE_R_MODE_NV',
'GL_OFFSET_PROJECTIVE_TEXTURE_2D_NV',
'GL_OFFSET_PROJECTIVE_TEXTURE_2D_SCALE_NV',
'GL_OFFSET_PROJECTIVE_TEXTURE_RECTANGLE_NV',
'GL_OFFSET_PROJECTIVE_TEXTURE_RECTANGLE_SCALE_NV',
'GL_OFFSET_HILO_TEXTURE_2D_NV', 'GL_OFFSET_HILO_TEXTURE_RECTANGLE_NV',
'GL_OFFSET_HILO_PROJECTIVE_TEXTURE_2D_NV',
'GL_OFFSET_HILO_PROJECTIVE_TEXTURE_RECTANGLE_NV',
'GL_DEPENDENT_HILO_TEXTURE_2D_NV', 'GL_DEPENDENT_RGB_TEXTURE_3D_NV',
'GL_DEPENDENT_RGB_TEXTURE_CUBE_MAP_NV', 'GL_DOT_PRODUCT_PASS_THROUGH_NV',
'GL_DOT_PRODUCT_TEXTURE_1D_NV', 'GL_DOT_PRODUCT_AFFINE_DEPTH_REPLACE_NV',
'GL_HILO8_NV', 'GL_SIGNED_HILO8_NV', 'GL_FORCE_BLUE_TO_ONE_NV',
'GL_STENCIL_TEST_TWO_SIDE_EXT', 'GL_ACTIVE_STENCIL_FACE_EXT',
'GL_TEXT_FRAGMENT_SHADER_ATI', 'GL_UNPACK_CLIENT_STORAGE_APPLE',
'GL_ELEMENT_ARRAY_APPLE', 'GL_ELEMENT_ARRAY_TYPE_APPLE',
'GL_ELEMENT_ARRAY_POINTER_APPLE', 'GL_DRAW_PIXELS_APPLE', 'GL_FENCE_APPLE',
'GL_VERTEX_ARRAY_BINDING_APPLE', 'GL_VERTEX_ARRAY_RANGE_APPLE',
'GL_VERTEX_ARRAY_RANGE_LENGTH_APPLE', 'GL_VERTEX_ARRAY_STORAGE_HINT_APPLE',
'GL_VERTEX_ARRAY_RANGE_POINTER_APPLE', 'GL_STORAGE_CACHED_APPLE',
'GL_STORAGE_SHARED_APPLE', 'GL_YCBCR_422_APPLE',
'GL_UNSIGNED_SHORT_8_8_APPLE', 'GL_UNSIGNED_SHORT_8_8_REV_APPLE',
'GL_RGB_S3TC', 'GL_RGB4_S3TC', 'GL_RGBA_S3TC', 'GL_RGBA4_S3TC',
'GL_MAX_DRAW_BUFFERS_ATI', 'GL_DRAW_BUFFER0_ATI', 'GL_DRAW_BUFFER1_ATI',
'GL_DRAW_BUFFER2_ATI', 'GL_DRAW_BUFFER3_ATI', 'GL_DRAW_BUFFER4_ATI',
'GL_DRAW_BUFFER5_ATI', 'GL_DRAW_BUFFER6_ATI', 'GL_DRAW_BUFFER7_ATI',
'GL_DRAW_BUFFER8_ATI', 'GL_DRAW_BUFFER9_ATI', 'GL_DRAW_BUFFER10_ATI',
'GL_DRAW_BUFFER11_ATI', 'GL_DRAW_BUFFER12_ATI', 'GL_DRAW_BUFFER13_ATI',
'GL_DRAW_BUFFER14_ATI', 'GL_DRAW_BUFFER15_ATI', 'GL_TYPE_RGBA_FLOAT_ATI',
'GL_COLOR_CLEAR_UNCLAMPED_VALUE_ATI', 'GL_MODULATE_ADD_ATI',
'GL_MODULATE_SIGNED_ADD_ATI', 'GL_MODULATE_SUBTRACT_ATI',
'GL_RGBA_FLOAT32_ATI', 'GL_RGB_FLOAT32_ATI', 'GL_ALPHA_FLOAT32_ATI',
'GL_INTENSITY_FLOAT32_ATI', 'GL_LUMINANCE_FLOAT32_ATI',
'GL_LUMINANCE_ALPHA_FLOAT32_ATI', 'GL_RGBA_FLOAT16_ATI', 'GL_RGB_FLOAT16_ATI',
'GL_ALPHA_FLOAT16_ATI', 'GL_INTENSITY_FLOAT16_ATI',
'GL_LUMINANCE_FLOAT16_ATI', 'GL_LUMINANCE_ALPHA_FLOAT16_ATI', 'GL_FLOAT_R_NV',
'GL_FLOAT_RG_NV', 'GL_FLOAT_RGB_NV', 'GL_FLOAT_RGBA_NV', 'GL_FLOAT_R16_NV',
'GL_FLOAT_R32_NV', 'GL_FLOAT_RG16_NV', 'GL_FLOAT_RG32_NV',
'GL_FLOAT_RGB16_NV', 'GL_FLOAT_RGB32_NV', 'GL_FLOAT_RGBA16_NV',
'GL_FLOAT_RGBA32_NV', 'GL_TEXTURE_FLOAT_COMPONENTS_NV',
'GL_FLOAT_CLEAR_COLOR_VALUE_NV', 'GL_FLOAT_RGBA_MODE_NV',
'GL_MAX_FRAGMENT_PROGRAM_LOCAL_PARAMETERS_NV', 'GL_FRAGMENT_PROGRAM_NV',
'GL_MAX_TEXTURE_COORDS_NV', 'GL_MAX_TEXTURE_IMAGE_UNITS_NV',
'GL_FRAGMENT_PROGRAM_BINDING_NV', 'GL_PROGRAM_ERROR_STRING_NV',
'GL_HALF_FLOAT_NV', 'GL_WRITE_PIXEL_DATA_RANGE_NV',
'GL_READ_PIXEL_DATA_RANGE_NV', 'GL_WRITE_PIXEL_DATA_RANGE_LENGTH_NV',
'GL_READ_PIXEL_DATA_RANGE_LENGTH_NV', 'GL_WRITE_PIXEL_DATA_RANGE_POINTER_NV',
'GL_READ_PIXEL_DATA_RANGE_POINTER_NV', 'GL_PRIMITIVE_RESTART_NV',
'GL_PRIMITIVE_RESTART_INDEX_NV', 'GL_TEXTURE_UNSIGNED_REMAP_MODE_NV',
'GL_STENCIL_BACK_FUNC_ATI', 'GL_STENCIL_BACK_FAIL_ATI',
'GL_STENCIL_BACK_PASS_DEPTH_FAIL_ATI', 'GL_STENCIL_BACK_PASS_DEPTH_PASS_ATI',
'GL_IMPLEMENTATION_COLOR_READ_TYPE_OES',
'GL_IMPLEMENTATION_COLOR_READ_FORMAT_OES', 'GL_DEPTH_BOUNDS_TEST_EXT',
'GL_DEPTH_BOUNDS_EXT', 'GL_MIRROR_CLAMP_EXT', 'GL_MIRROR_CLAMP_TO_EDGE_EXT',
'GL_MIRROR_CLAMP_TO_BORDER_EXT', 'GL_BLEND_EQUATION_RGB_EXT',
'GL_BLEND_EQUATION_ALPHA_EXT', 'GL_PACK_INVERT_MESA',
'GL_UNSIGNED_SHORT_8_8_MESA', 'GL_UNSIGNED_SHORT_8_8_REV_MESA',
'GL_YCBCR_MESA', 'GL_PIXEL_PACK_BUFFER_EXT', 'GL_PIXEL_UNPACK_BUFFER_EXT',
'GL_PIXEL_PACK_BUFFER_BINDING_EXT', 'GL_PIXEL_UNPACK_BUFFER_BINDING_EXT',
'GL_MAX_PROGRAM_EXEC_INSTRUCTIONS_NV', 'GL_MAX_PROGRAM_CALL_DEPTH_NV',
'GL_MAX_PROGRAM_IF_DEPTH_NV', 'GL_MAX_PROGRAM_LOOP_DEPTH_NV',
'GL_MAX_PROGRAM_LOOP_COUNT_NV', 'GL_INVALID_FRAMEBUFFER_OPERATION_EXT',
'GL_MAX_RENDERBUFFER_SIZE_EXT', 'GL_FRAMEBUFFER_BINDING_EXT',
'GL_RENDERBUFFER_BINDING_EXT', 'GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE_EXT',
'GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME_EXT',
'GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL_EXT',
'GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE_EXT',
'GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_3D_ZOFFSET_EXT',
'GL_FRAMEBUFFER_COMPLETE_EXT', 'GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_EXT',
'GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_EXT',
'GL_FRAMEBUFFER_INCOMPLETE_DUPLICATE_ATTACHMENT_EXT',
'GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_EXT',
'GL_FRAMEBUFFER_INCOMPLETE_FORMATS_EXT',
'GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER_EXT',
'GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER_EXT', 'GL_FRAMEBUFFER_UNSUPPORTED_EXT',
'GL_MAX_COLOR_ATTACHMENTS_EXT', 'GL_COLOR_ATTACHMENT0_EXT',
'GL_COLOR_ATTACHMENT1_EXT', 'GL_COLOR_ATTACHMENT2_EXT',
'GL_COLOR_ATTACHMENT3_EXT', 'GL_COLOR_ATTACHMENT4_EXT',
'GL_COLOR_ATTACHMENT5_EXT', 'GL_COLOR_ATTACHMENT6_EXT',
'GL_COLOR_ATTACHMENT7_EXT', 'GL_COLOR_ATTACHMENT8_EXT',
'GL_COLOR_ATTACHMENT9_EXT', 'GL_COLOR_ATTACHMENT10_EXT',
'GL_COLOR_ATTACHMENT11_EXT', 'GL_COLOR_ATTACHMENT12_EXT',
'GL_COLOR_ATTACHMENT13_EXT', 'GL_COLOR_ATTACHMENT14_EXT',
'GL_COLOR_ATTACHMENT15_EXT', 'GL_DEPTH_ATTACHMENT_EXT',
'GL_STENCIL_ATTACHMENT_EXT', 'GL_FRAMEBUFFER_EXT', 'GL_RENDERBUFFER_EXT',
'GL_RENDERBUFFER_WIDTH_EXT', 'GL_RENDERBUFFER_HEIGHT_EXT',
'GL_RENDERBUFFER_INTERNAL_FORMAT_EXT', 'GL_STENCIL_INDEX1_EXT',
'GL_STENCIL_INDEX4_EXT', 'GL_STENCIL_INDEX8_EXT', 'GL_STENCIL_INDEX16_EXT',
'GL_RENDERBUFFER_RED_SIZE_EXT', 'GL_RENDERBUFFER_GREEN_SIZE_EXT',
'GL_RENDERBUFFER_BLUE_SIZE_EXT', 'GL_RENDERBUFFER_ALPHA_SIZE_EXT',
'GL_RENDERBUFFER_DEPTH_SIZE_EXT', 'GL_RENDERBUFFER_STENCIL_SIZE_EXT',
'GLchar', 'GLintptr', 'GLsizeiptr', 'GLintptrARB', 'GLsizeiptrARB',
'GLcharARB', 'GLhandleARB', 'GLhalfARB', 'GLhalfNV', 'GL_VERSION_1_2',
'glBlendColor', 'glBlendEquation', 'glDrawRangeElements', 'glColorTable',
'glColorTableParameterfv', 'glColorTableParameteriv', 'glCopyColorTable',
'glGetColorTable', 'glGetColorTableParameterfv', 'glGetColorTableParameteriv',
'glColorSubTable', 'glCopyColorSubTable', 'glConvolutionFilter1D',
'glConvolutionFilter2D', 'glConvolutionParameterf',
'glConvolutionParameterfv', 'glConvolutionParameteri',
'glConvolutionParameteriv', 'glCopyConvolutionFilter1D',
'glCopyConvolutionFilter2D', 'glGetConvolutionFilter',
'glGetConvolutionParameterfv', 'glGetConvolutionParameteriv',
'glGetSeparableFilter', 'glSeparableFilter2D', 'glGetHistogram',
'glGetHistogramParameterfv', 'glGetHistogramParameteriv', 'glGetMinmax',
'glGetMinmaxParameterfv', 'glGetMinmaxParameteriv', 'glHistogram', 'glMinmax',
'glResetHistogram', 'glResetMinmax', 'glTexImage3D', 'glTexSubImage3D',
'glCopyTexSubImage3D', 'PFNGLBLENDCOLORPROC', 'PFNGLBLENDEQUATIONPROC',
'PFNGLDRAWRANGEELEMENTSPROC', 'PFNGLCOLORTABLEPROC',
'PFNGLCOLORTABLEPARAMETERFVPROC', 'PFNGLCOLORTABLEPARAMETERIVPROC',
'PFNGLCOPYCOLORTABLEPROC', 'PFNGLGETCOLORTABLEPROC',
'PFNGLGETCOLORTABLEPARAMETERFVPROC', 'PFNGLGETCOLORTABLEPARAMETERIVPROC',
'PFNGLCOLORSUBTABLEPROC', 'PFNGLCOPYCOLORSUBTABLEPROC',
'PFNGLCONVOLUTIONFILTER1DPROC', 'PFNGLCONVOLUTIONFILTER2DPROC',
'PFNGLCONVOLUTIONPARAMETERFPROC', 'PFNGLCONVOLUTIONPARAMETERFVPROC',
'PFNGLCONVOLUTIONPARAMETERIPROC', 'PFNGLCONVOLUTIONPARAMETERIVPROC',
'PFNGLCOPYCONVOLUTIONFILTER1DPROC', 'PFNGLCOPYCONVOLUTIONFILTER2DPROC',
'PFNGLGETCONVOLUTIONFILTERPROC', 'PFNGLGETCONVOLUTIONPARAMETERFVPROC',
'PFNGLGETCONVOLUTIONPARAMETERIVPROC', 'PFNGLGETSEPARABLEFILTERPROC',
'PFNGLSEPARABLEFILTER2DPROC', 'PFNGLGETHISTOGRAMPROC',
'PFNGLGETHISTOGRAMPARAMETERFVPROC', 'PFNGLGETHISTOGRAMPARAMETERIVPROC',
'PFNGLGETMINMAXPROC', 'PFNGLGETMINMAXPARAMETERFVPROC',
'PFNGLGETMINMAXPARAMETERIVPROC', 'PFNGLHISTOGRAMPROC', 'PFNGLMINMAXPROC',
'PFNGLRESETHISTOGRAMPROC', 'PFNGLRESETMINMAXPROC', 'PFNGLTEXIMAGE3DPROC',
'PFNGLTEXSUBIMAGE3DPROC', 'PFNGLCOPYTEXSUBIMAGE3DPROC', 'GL_VERSION_1_3',
'glActiveTexture', 'glClientActiveTexture', 'glMultiTexCoord1d',
'glMultiTexCoord1dv', 'glMultiTexCoord1f', 'glMultiTexCoord1fv',
'glMultiTexCoord1i', 'glMultiTexCoord1iv', 'glMultiTexCoord1s',
'glMultiTexCoord1sv', 'glMultiTexCoord2d', 'glMultiTexCoord2dv',
'glMultiTexCoord2f', 'glMultiTexCoord2fv', 'glMultiTexCoord2i',
'glMultiTexCoord2iv', 'glMultiTexCoord2s', 'glMultiTexCoord2sv',
'glMultiTexCoord3d', 'glMultiTexCoord3dv', 'glMultiTexCoord3f',
'glMultiTexCoord3fv', 'glMultiTexCoord3i', 'glMultiTexCoord3iv',
'glMultiTexCoord3s', 'glMultiTexCoord3sv', 'glMultiTexCoord4d',
'glMultiTexCoord4dv', 'glMultiTexCoord4f', 'glMultiTexCoord4fv',
'glMultiTexCoord4i', 'glMultiTexCoord4iv', 'glMultiTexCoord4s',
'glMultiTexCoord4sv', 'glLoadTransposeMatrixf', 'glLoadTransposeMatrixd',
'glMultTransposeMatrixf', 'glMultTransposeMatrixd', 'glSampleCoverage',
'glCompressedTexImage3D', 'glCompressedTexImage2D', 'glCompressedTexImage1D',
'glCompressedTexSubImage3D', 'glCompressedTexSubImage2D',
'glCompressedTexSubImage1D', 'glGetCompressedTexImage',
'PFNGLACTIVETEXTUREPROC', 'PFNGLCLIENTACTIVETEXTUREPROC',
'PFNGLMULTITEXCOORD1DPROC', 'PFNGLMULTITEXCOORD1DVPROC',
'PFNGLMULTITEXCOORD1FPROC', 'PFNGLMULTITEXCOORD1FVPROC',
'PFNGLMULTITEXCOORD1IPROC', 'PFNGLMULTITEXCOORD1IVPROC',
'PFNGLMULTITEXCOORD1SPROC', 'PFNGLMULTITEXCOORD1SVPROC',
'PFNGLMULTITEXCOORD2DPROC', 'PFNGLMULTITEXCOORD2DVPROC',
'PFNGLMULTITEXCOORD2FPROC', 'PFNGLMULTITEXCOORD2FVPROC',
'PFNGLMULTITEXCOORD2IPROC', 'PFNGLMULTITEXCOORD2IVPROC',
'PFNGLMULTITEXCOORD2SPROC', 'PFNGLMULTITEXCOORD2SVPROC',
'PFNGLMULTITEXCOORD3DPROC', 'PFNGLMULTITEXCOORD3DVPROC',
'PFNGLMULTITEXCOORD3FPROC', 'PFNGLMULTITEXCOORD3FVPROC',
'PFNGLMULTITEXCOORD3IPROC', 'PFNGLMULTITEXCOORD3IVPROC',
'PFNGLMULTITEXCOORD3SPROC', 'PFNGLMULTITEXCOORD3SVPROC',
'PFNGLMULTITEXCOORD4DPROC', 'PFNGLMULTITEXCOORD4DVPROC',
'PFNGLMULTITEXCOORD4FPROC', 'PFNGLMULTITEXCOORD4FVPROC',
'PFNGLMULTITEXCOORD4IPROC', 'PFNGLMULTITEXCOORD4IVPROC',
'PFNGLMULTITEXCOORD4SPROC', 'PFNGLMULTITEXCOORD4SVPROC',
'PFNGLLOADTRANSPOSEMATRIXFPROC', 'PFNGLLOADTRANSPOSEMATRIXDPROC',
'PFNGLMULTTRANSPOSEMATRIXFPROC', 'PFNGLMULTTRANSPOSEMATRIXDPROC',
'PFNGLSAMPLECOVERAGEPROC', 'PFNGLCOMPRESSEDTEXIMAGE3DPROC',
'PFNGLCOMPRESSEDTEXIMAGE2DPROC', 'PFNGLCOMPRESSEDTEXIMAGE1DPROC',
'PFNGLCOMPRESSEDTEXSUBIMAGE3DPROC', 'PFNGLCOMPRESSEDTEXSUBIMAGE2DPROC',
'PFNGLCOMPRESSEDTEXSUBIMAGE1DPROC', 'PFNGLGETCOMPRESSEDTEXIMAGEPROC',
'GL_VERSION_1_4', 'glBlendFuncSeparate', 'glFogCoordf', 'glFogCoordfv',
'glFogCoordd', 'glFogCoorddv', 'glFogCoordPointer', 'glMultiDrawArrays',
'glMultiDrawElements', 'glPointParameterf', 'glPointParameterfv',
'glPointParameteri', 'glPointParameteriv', 'glSecondaryColor3b',
'glSecondaryColor3bv', 'glSecondaryColor3d', 'glSecondaryColor3dv',
'glSecondaryColor3f', 'glSecondaryColor3fv', 'glSecondaryColor3i',
'glSecondaryColor3iv', 'glSecondaryColor3s', 'glSecondaryColor3sv',
'glSecondaryColor3ub', 'glSecondaryColor3ubv', 'glSecondaryColor3ui',
'glSecondaryColor3uiv', 'glSecondaryColor3us', 'glSecondaryColor3usv',
'glSecondaryColorPointer', 'glWindowPos2d', 'glWindowPos2dv', 'glWindowPos2f',
'glWindowPos2fv', 'glWindowPos2i', 'glWindowPos2iv', 'glWindowPos2s',
'glWindowPos2sv', 'glWindowPos3d', 'glWindowPos3dv', 'glWindowPos3f',
'glWindowPos3fv', 'glWindowPos3i', 'glWindowPos3iv', 'glWindowPos3s',
'glWindowPos3sv', 'PFNGLBLENDFUNCSEPARATEPROC', 'PFNGLFOGCOORDFPROC',
'PFNGLFOGCOORDFVPROC', 'PFNGLFOGCOORDDPROC', 'PFNGLFOGCOORDDVPROC',
'PFNGLFOGCOORDPOINTERPROC', 'PFNGLMULTIDRAWARRAYSPROC',
'PFNGLMULTIDRAWELEMENTSPROC', 'PFNGLPOINTPARAMETERFPROC',
'PFNGLPOINTPARAMETERFVPROC', 'PFNGLPOINTPARAMETERIPROC',
'PFNGLPOINTPARAMETERIVPROC', 'PFNGLSECONDARYCOLOR3BPROC',
'PFNGLSECONDARYCOLOR3BVPROC', 'PFNGLSECONDARYCOLOR3DPROC',
'PFNGLSECONDARYCOLOR3DVPROC', 'PFNGLSECONDARYCOLOR3FPROC',
'PFNGLSECONDARYCOLOR3FVPROC', 'PFNGLSECONDARYCOLOR3IPROC',
'PFNGLSECONDARYCOLOR3IVPROC', 'PFNGLSECONDARYCOLOR3SPROC',
'PFNGLSECONDARYCOLOR3SVPROC', 'PFNGLSECONDARYCOLOR3UBPROC',
'PFNGLSECONDARYCOLOR3UBVPROC', 'PFNGLSECONDARYCOLOR3UIPROC',
'PFNGLSECONDARYCOLOR3UIVPROC', 'PFNGLSECONDARYCOLOR3USPROC',
'PFNGLSECONDARYCOLOR3USVPROC', 'PFNGLSECONDARYCOLORPOINTERPROC',
'PFNGLWINDOWPOS2DPROC', 'PFNGLWINDOWPOS2DVPROC', 'PFNGLWINDOWPOS2FPROC',
'PFNGLWINDOWPOS2FVPROC', 'PFNGLWINDOWPOS2IPROC', 'PFNGLWINDOWPOS2IVPROC',
'PFNGLWINDOWPOS2SPROC', 'PFNGLWINDOWPOS2SVPROC', 'PFNGLWINDOWPOS3DPROC',
'PFNGLWINDOWPOS3DVPROC', 'PFNGLWINDOWPOS3FPROC', 'PFNGLWINDOWPOS3FVPROC',
'PFNGLWINDOWPOS3IPROC', 'PFNGLWINDOWPOS3IVPROC', 'PFNGLWINDOWPOS3SPROC',
'PFNGLWINDOWPOS3SVPROC', 'GL_VERSION_1_5', 'glGenQueries', 'glDeleteQueries',
'glIsQuery', 'glBeginQuery', 'glEndQuery', 'glGetQueryiv',
'glGetQueryObjectiv', 'glGetQueryObjectuiv', 'glBindBuffer',
'glDeleteBuffers', 'glGenBuffers', 'glIsBuffer', 'glBufferData',
'glBufferSubData', 'glGetBufferSubData', 'glMapBuffer', 'glUnmapBuffer',
'glGetBufferParameteriv', 'glGetBufferPointerv', 'PFNGLGENQUERIESPROC',
'PFNGLDELETEQUERIESPROC', 'PFNGLISQUERYPROC', 'PFNGLBEGINQUERYPROC',
'PFNGLENDQUERYPROC', 'PFNGLGETQUERYIVPROC', 'PFNGLGETQUERYOBJECTIVPROC',
'PFNGLGETQUERYOBJECTUIVPROC', 'PFNGLBINDBUFFERPROC', 'PFNGLDELETEBUFFERSPROC',
'PFNGLGENBUFFERSPROC', 'PFNGLISBUFFERPROC', 'PFNGLBUFFERDATAPROC',
'PFNGLBUFFERSUBDATAPROC', 'PFNGLGETBUFFERSUBDATAPROC', 'PFNGLMAPBUFFERPROC',
'PFNGLUNMAPBUFFERPROC', 'PFNGLGETBUFFERPARAMETERIVPROC',
'PFNGLGETBUFFERPOINTERVPROC', 'GL_VERSION_2_0', 'glBlendEquationSeparate',
'glDrawBuffers', 'glStencilOpSeparate', 'glStencilFuncSeparate',
'glStencilMaskSeparate', 'glAttachShader', 'glBindAttribLocation',
'glCompileShader', 'glCreateProgram', 'glCreateShader', 'glDeleteProgram',
'glDeleteShader', 'glDetachShader', 'glDisableVertexAttribArray',
'glEnableVertexAttribArray', 'glGetActiveAttrib', 'glGetActiveUniform',
'glGetAttachedShaders', 'glGetAttribLocation', 'glGetProgramiv',
'glGetProgramInfoLog', 'glGetShaderiv', 'glGetShaderInfoLog',
'glGetShaderSource', 'glGetUniformLocation', 'glGetUniformfv',
'glGetUniformiv', 'glGetVertexAttribdv', 'glGetVertexAttribfv',
'glGetVertexAttribiv', 'glGetVertexAttribPointerv', 'glIsProgram',
'glIsShader', 'glLinkProgram', 'glShaderSource', 'glUseProgram',
'glUniform1f', 'glUniform2f', 'glUniform3f', 'glUniform4f', 'glUniform1i',
'glUniform2i', 'glUniform3i', 'glUniform4i', 'glUniform1fv', 'glUniform2fv',
'glUniform3fv', 'glUniform4fv', 'glUniform1iv', 'glUniform2iv',
'glUniform3iv', 'glUniform4iv', 'glUniformMatrix2fv', 'glUniformMatrix3fv',
'glUniformMatrix4fv', 'glValidateProgram', 'glVertexAttrib1d',
'glVertexAttrib1dv', 'glVertexAttrib1f', 'glVertexAttrib1fv',
'glVertexAttrib1s', 'glVertexAttrib1sv', 'glVertexAttrib2d',
'glVertexAttrib2dv', 'glVertexAttrib2f', 'glVertexAttrib2fv',
'glVertexAttrib2s', 'glVertexAttrib2sv', 'glVertexAttrib3d',
'glVertexAttrib3dv', 'glVertexAttrib3f', 'glVertexAttrib3fv',
'glVertexAttrib3s', 'glVertexAttrib3sv', 'glVertexAttrib4Nbv',
'glVertexAttrib4Niv', 'glVertexAttrib4Nsv', 'glVertexAttrib4Nub',
'glVertexAttrib4Nubv', 'glVertexAttrib4Nuiv', 'glVertexAttrib4Nusv',
'glVertexAttrib4bv', 'glVertexAttrib4d', 'glVertexAttrib4dv',
'glVertexAttrib4f', 'glVertexAttrib4fv', 'glVertexAttrib4iv',
'glVertexAttrib4s', 'glVertexAttrib4sv', 'glVertexAttrib4ubv',
'glVertexAttrib4uiv', 'glVertexAttrib4usv', 'glVertexAttribPointer',
'PFNGLBLENDEQUATIONSEPARATEPROC', 'PFNGLDRAWBUFFERSPROC',
'PFNGLSTENCILOPSEPARATEPROC', 'PFNGLSTENCILFUNCSEPARATEPROC',
'PFNGLSTENCILMASKSEPARATEPROC', 'PFNGLATTACHSHADERPROC',
'PFNGLBINDATTRIBLOCATIONPROC', 'PFNGLCOMPILESHADERPROC',
'PFNGLCREATEPROGRAMPROC', 'PFNGLCREATESHADERPROC', 'PFNGLDELETEPROGRAMPROC',
'PFNGLDELETESHADERPROC', 'PFNGLDETACHSHADERPROC',
'PFNGLDISABLEVERTEXATTRIBARRAYPROC', 'PFNGLENABLEVERTEXATTRIBARRAYPROC',
'PFNGLGETACTIVEATTRIBPROC', 'PFNGLGETACTIVEUNIFORMPROC',
'PFNGLGETATTACHEDSHADERSPROC', 'PFNGLGETATTRIBLOCATIONPROC',
'PFNGLGETPROGRAMIVPROC', 'PFNGLGETPROGRAMINFOLOGPROC', 'PFNGLGETSHADERIVPROC',
'PFNGLGETSHADERINFOLOGPROC', 'PFNGLGETSHADERSOURCEPROC',
'PFNGLGETUNIFORMLOCATIONPROC', 'PFNGLGETUNIFORMFVPROC',
'PFNGLGETUNIFORMIVPROC', 'PFNGLGETVERTEXATTRIBDVPROC',
'PFNGLGETVERTEXATTRIBFVPROC', 'PFNGLGETVERTEXATTRIBIVPROC',
'PFNGLGETVERTEXATTRIBPOINTERVPROC', 'PFNGLISPROGRAMPROC', 'PFNGLISSHADERPROC',
'PFNGLLINKPROGRAMPROC', 'PFNGLSHADERSOURCEPROC', 'PFNGLUSEPROGRAMPROC',
'PFNGLUNIFORM1FPROC', 'PFNGLUNIFORM2FPROC', 'PFNGLUNIFORM3FPROC',
'PFNGLUNIFORM4FPROC', 'PFNGLUNIFORM1IPROC', 'PFNGLUNIFORM2IPROC',
'PFNGLUNIFORM3IPROC', 'PFNGLUNIFORM4IPROC', 'PFNGLUNIFORM1FVPROC',
'PFNGLUNIFORM2FVPROC', 'PFNGLUNIFORM3FVPROC', 'PFNGLUNIFORM4FVPROC',
'PFNGLUNIFORM1IVPROC', 'PFNGLUNIFORM2IVPROC', 'PFNGLUNIFORM3IVPROC',
'PFNGLUNIFORM4IVPROC', 'PFNGLUNIFORMMATRIX2FVPROC',
'PFNGLUNIFORMMATRIX3FVPROC', 'PFNGLUNIFORMMATRIX4FVPROC',
'PFNGLVALIDATEPROGRAMPROC', 'PFNGLVERTEXATTRIB1DPROC',
'PFNGLVERTEXATTRIB1DVPROC', 'PFNGLVERTEXATTRIB1FPROC',
'PFNGLVERTEXATTRIB1FVPROC', 'PFNGLVERTEXATTRIB1SPROC',
'PFNGLVERTEXATTRIB1SVPROC', 'PFNGLVERTEXATTRIB2DPROC',
'PFNGLVERTEXATTRIB2DVPROC', 'PFNGLVERTEXATTRIB2FPROC',
'PFNGLVERTEXATTRIB2FVPROC', 'PFNGLVERTEXATTRIB2SPROC',
'PFNGLVERTEXATTRIB2SVPROC', 'PFNGLVERTEXATTRIB3DPROC',
'PFNGLVERTEXATTRIB3DVPROC', 'PFNGLVERTEXATTRIB3FPROC',
'PFNGLVERTEXATTRIB3FVPROC', 'PFNGLVERTEXATTRIB3SPROC',
'PFNGLVERTEXATTRIB3SVPROC', 'PFNGLVERTEXATTRIB4NBVPROC',
'PFNGLVERTEXATTRIB4NIVPROC', 'PFNGLVERTEXATTRIB4NSVPROC',
'PFNGLVERTEXATTRIB4NUBPROC', 'PFNGLVERTEXATTRIB4NUBVPROC',
'PFNGLVERTEXATTRIB4NUIVPROC', 'PFNGLVERTEXATTRIB4NUSVPROC',
'PFNGLVERTEXATTRIB4BVPROC', 'PFNGLVERTEXATTRIB4DPROC',
'PFNGLVERTEXATTRIB4DVPROC', 'PFNGLVERTEXATTRIB4FPROC',
'PFNGLVERTEXATTRIB4FVPROC', 'PFNGLVERTEXATTRIB4IVPROC',
'PFNGLVERTEXATTRIB4SPROC', 'PFNGLVERTEXATTRIB4SVPROC',
'PFNGLVERTEXATTRIB4UBVPROC', 'PFNGLVERTEXATTRIB4UIVPROC',
'PFNGLVERTEXATTRIB4USVPROC', 'PFNGLVERTEXATTRIBPOINTERPROC',
'GL_ARB_multitexture', 'glActiveTextureARB', 'glClientActiveTextureARB',
'glMultiTexCoord1dARB', 'glMultiTexCoord1dvARB', 'glMultiTexCoord1fARB',
'glMultiTexCoord1fvARB', 'glMultiTexCoord1iARB', 'glMultiTexCoord1ivARB',
'glMultiTexCoord1sARB', 'glMultiTexCoord1svARB', 'glMultiTexCoord2dARB',
'glMultiTexCoord2dvARB', 'glMultiTexCoord2fARB', 'glMultiTexCoord2fvARB',
'glMultiTexCoord2iARB', 'glMultiTexCoord2ivARB', 'glMultiTexCoord2sARB',
'glMultiTexCoord2svARB', 'glMultiTexCoord3dARB', 'glMultiTexCoord3dvARB',
'glMultiTexCoord3fARB', 'glMultiTexCoord3fvARB', 'glMultiTexCoord3iARB',
'glMultiTexCoord3ivARB', 'glMultiTexCoord3sARB', 'glMultiTexCoord3svARB',
'glMultiTexCoord4dARB', 'glMultiTexCoord4dvARB', 'glMultiTexCoord4fARB',
'glMultiTexCoord4fvARB', 'glMultiTexCoord4iARB', 'glMultiTexCoord4ivARB',
'glMultiTexCoord4sARB', 'glMultiTexCoord4svARB', 'PFNGLACTIVETEXTUREARBPROC',
'PFNGLCLIENTACTIVETEXTUREARBPROC', 'PFNGLMULTITEXCOORD1DARBPROC',
'PFNGLMULTITEXCOORD1DVARBPROC', 'PFNGLMULTITEXCOORD1FARBPROC',
'PFNGLMULTITEXCOORD1FVARBPROC', 'PFNGLMULTITEXCOORD1IARBPROC',
'PFNGLMULTITEXCOORD1IVARBPROC', 'PFNGLMULTITEXCOORD1SARBPROC',
'PFNGLMULTITEXCOORD1SVARBPROC', 'PFNGLMULTITEXCOORD2DARBPROC',
'PFNGLMULTITEXCOORD2DVARBPROC', 'PFNGLMULTITEXCOORD2FARBPROC',
'PFNGLMULTITEXCOORD2FVARBPROC', 'PFNGLMULTITEXCOORD2IARBPROC',
'PFNGLMULTITEXCOORD2IVARBPROC', 'PFNGLMULTITEXCOORD2SARBPROC',
'PFNGLMULTITEXCOORD2SVARBPROC', 'PFNGLMULTITEXCOORD3DARBPROC',
'PFNGLMULTITEXCOORD3DVARBPROC', 'PFNGLMULTITEXCOORD3FARBPROC',
'PFNGLMULTITEXCOORD3FVARBPROC', 'PFNGLMULTITEXCOORD3IARBPROC',
'PFNGLMULTITEXCOORD3IVARBPROC', 'PFNGLMULTITEXCOORD3SARBPROC',
'PFNGLMULTITEXCOORD3SVARBPROC', 'PFNGLMULTITEXCOORD4DARBPROC',
'PFNGLMULTITEXCOORD4DVARBPROC', 'PFNGLMULTITEXCOORD4FARBPROC',
'PFNGLMULTITEXCOORD4FVARBPROC', 'PFNGLMULTITEXCOORD4IARBPROC',
'PFNGLMULTITEXCOORD4IVARBPROC', 'PFNGLMULTITEXCOORD4SARBPROC',
'PFNGLMULTITEXCOORD4SVARBPROC', 'GL_ARB_transpose_matrix',
'glLoadTransposeMatrixfARB', 'glLoadTransposeMatrixdARB',
'glMultTransposeMatrixfARB', 'glMultTransposeMatrixdARB',
'PFNGLLOADTRANSPOSEMATRIXFARBPROC', 'PFNGLLOADTRANSPOSEMATRIXDARBPROC',
'PFNGLMULTTRANSPOSEMATRIXFARBPROC', 'PFNGLMULTTRANSPOSEMATRIXDARBPROC',
'GL_ARB_multisample', 'glSampleCoverageARB', 'PFNGLSAMPLECOVERAGEARBPROC',
'GL_ARB_texture_env_add', 'GL_ARB_texture_cube_map',
'GL_ARB_texture_compression', 'glCompressedTexImage3DARB',
'glCompressedTexImage2DARB', 'glCompressedTexImage1DARB',
'glCompressedTexSubImage3DARB', 'glCompressedTexSubImage2DARB',
'glCompressedTexSubImage1DARB', 'glGetCompressedTexImageARB',
'PFNGLCOMPRESSEDTEXIMAGE3DARBPROC', 'PFNGLCOMPRESSEDTEXIMAGE2DARBPROC',
'PFNGLCOMPRESSEDTEXIMAGE1DARBPROC', 'PFNGLCOMPRESSEDTEXSUBIMAGE3DARBPROC',
'PFNGLCOMPRESSEDTEXSUBIMAGE2DARBPROC', 'PFNGLCOMPRESSEDTEXSUBIMAGE1DARBPROC',
'PFNGLGETCOMPRESSEDTEXIMAGEARBPROC', 'GL_ARB_texture_border_clamp',
'GL_ARB_point_parameters', 'glPointParameterfARB', 'glPointParameterfvARB',
'PFNGLPOINTPARAMETERFARBPROC', 'PFNGLPOINTPARAMETERFVARBPROC',
'GL_ARB_vertex_blend', 'glWeightbvARB', 'glWeightsvARB', 'glWeightivARB',
'glWeightfvARB', 'glWeightdvARB', 'glWeightubvARB', 'glWeightusvARB',
'glWeightuivARB', 'glWeightPointerARB', 'glVertexBlendARB',
'PFNGLWEIGHTBVARBPROC', 'PFNGLWEIGHTSVARBPROC', 'PFNGLWEIGHTIVARBPROC',
'PFNGLWEIGHTFVARBPROC', 'PFNGLWEIGHTDVARBPROC', 'PFNGLWEIGHTUBVARBPROC',
'PFNGLWEIGHTUSVARBPROC', 'PFNGLWEIGHTUIVARBPROC', 'PFNGLWEIGHTPOINTERARBPROC',
'PFNGLVERTEXBLENDARBPROC', 'GL_ARB_matrix_palette',
'glCurrentPaletteMatrixARB', 'glMatrixIndexubvARB', 'glMatrixIndexusvARB',
'glMatrixIndexuivARB', 'glMatrixIndexPointerARB',
'PFNGLCURRENTPALETTEMATRIXARBPROC', 'PFNGLMATRIXINDEXUBVARBPROC',
'PFNGLMATRIXINDEXUSVARBPROC', 'PFNGLMATRIXINDEXUIVARBPROC',
'PFNGLMATRIXINDEXPOINTERARBPROC', 'GL_ARB_texture_env_combine',
'GL_ARB_texture_env_crossbar', 'GL_ARB_texture_env_dot3',
'GL_ARB_texture_mirrored_repeat', 'GL_ARB_depth_texture', 'GL_ARB_shadow',
'GL_ARB_shadow_ambient', 'GL_ARB_window_pos', 'glWindowPos2dARB',
'glWindowPos2dvARB', 'glWindowPos2fARB', 'glWindowPos2fvARB',
'glWindowPos2iARB', 'glWindowPos2ivARB', 'glWindowPos2sARB',
'glWindowPos2svARB', 'glWindowPos3dARB', 'glWindowPos3dvARB',
'glWindowPos3fARB', 'glWindowPos3fvARB', 'glWindowPos3iARB',
'glWindowPos3ivARB', 'glWindowPos3sARB', 'glWindowPos3svARB',
'PFNGLWINDOWPOS2DARBPROC', 'PFNGLWINDOWPOS2DVARBPROC',
'PFNGLWINDOWPOS2FARBPROC', 'PFNGLWINDOWPOS2FVARBPROC',
'PFNGLWINDOWPOS2IARBPROC', 'PFNGLWINDOWPOS2IVARBPROC',
'PFNGLWINDOWPOS2SARBPROC', 'PFNGLWINDOWPOS2SVARBPROC',
'PFNGLWINDOWPOS3DARBPROC', 'PFNGLWINDOWPOS3DVARBPROC',
'PFNGLWINDOWPOS3FARBPROC', 'PFNGLWINDOWPOS3FVARBPROC',
'PFNGLWINDOWPOS3IARBPROC', 'PFNGLWINDOWPOS3IVARBPROC',
'PFNGLWINDOWPOS3SARBPROC', 'PFNGLWINDOWPOS3SVARBPROC',
'GL_ARB_vertex_program', 'glVertexAttrib1dARB', 'glVertexAttrib1dvARB',
'glVertexAttrib1fARB', 'glVertexAttrib1fvARB', 'glVertexAttrib1sARB',
'glVertexAttrib1svARB', 'glVertexAttrib2dARB', 'glVertexAttrib2dvARB',
'glVertexAttrib2fARB', 'glVertexAttrib2fvARB', 'glVertexAttrib2sARB',
'glVertexAttrib2svARB', 'glVertexAttrib3dARB', 'glVertexAttrib3dvARB',
'glVertexAttrib3fARB', 'glVertexAttrib3fvARB', 'glVertexAttrib3sARB',
'glVertexAttrib3svARB', 'glVertexAttrib4NbvARB', 'glVertexAttrib4NivARB',
'glVertexAttrib4NsvARB', 'glVertexAttrib4NubARB', 'glVertexAttrib4NubvARB',
'glVertexAttrib4NuivARB', 'glVertexAttrib4NusvARB', 'glVertexAttrib4bvARB',
'glVertexAttrib4dARB', 'glVertexAttrib4dvARB', 'glVertexAttrib4fARB',
'glVertexAttrib4fvARB', 'glVertexAttrib4ivARB', 'glVertexAttrib4sARB',
'glVertexAttrib4svARB', 'glVertexAttrib4ubvARB', 'glVertexAttrib4uivARB',
'glVertexAttrib4usvARB', 'glVertexAttribPointerARB',
'glEnableVertexAttribArrayARB', 'glDisableVertexAttribArrayARB',
'glProgramStringARB', 'glBindProgramARB', 'glDeleteProgramsARB',
'glGenProgramsARB', 'glProgramEnvParameter4dARB',
'glProgramEnvParameter4dvARB', 'glProgramEnvParameter4fARB',
'glProgramEnvParameter4fvARB', 'glProgramLocalParameter4dARB',
'glProgramLocalParameter4dvARB', 'glProgramLocalParameter4fARB',
'glProgramLocalParameter4fvARB', 'glGetProgramEnvParameterdvARB',
'glGetProgramEnvParameterfvARB', 'glGetProgramLocalParameterdvARB',
'glGetProgramLocalParameterfvARB', 'glGetProgramivARB',
'glGetProgramStringARB', 'glGetVertexAttribdvARB', 'glGetVertexAttribfvARB',
'glGetVertexAttribivARB', 'glGetVertexAttribPointervARB', 'glIsProgramARB',
'PFNGLVERTEXATTRIB1DARBPROC', 'PFNGLVERTEXATTRIB1DVARBPROC',
'PFNGLVERTEXATTRIB1FARBPROC', 'PFNGLVERTEXATTRIB1FVARBPROC',
'PFNGLVERTEXATTRIB1SARBPROC', 'PFNGLVERTEXATTRIB1SVARBPROC',
'PFNGLVERTEXATTRIB2DARBPROC', 'PFNGLVERTEXATTRIB2DVARBPROC',
'PFNGLVERTEXATTRIB2FARBPROC', 'PFNGLVERTEXATTRIB2FVARBPROC',
'PFNGLVERTEXATTRIB2SARBPROC', 'PFNGLVERTEXATTRIB2SVARBPROC',
'PFNGLVERTEXATTRIB3DARBPROC', 'PFNGLVERTEXATTRIB3DVARBPROC',
'PFNGLVERTEXATTRIB3FARBPROC', 'PFNGLVERTEXATTRIB3FVARBPROC',
'PFNGLVERTEXATTRIB3SARBPROC', 'PFNGLVERTEXATTRIB3SVARBPROC',
'PFNGLVERTEXATTRIB4NBVARBPROC', 'PFNGLVERTEXATTRIB4NIVARBPROC',
'PFNGLVERTEXATTRIB4NSVARBPROC', 'PFNGLVERTEXATTRIB4NUBARBPROC',
'PFNGLVERTEXATTRIB4NUBVARBPROC', 'PFNGLVERTEXATTRIB4NUIVARBPROC',
'PFNGLVERTEXATTRIB4NUSVARBPROC', 'PFNGLVERTEXATTRIB4BVARBPROC',
'PFNGLVERTEXATTRIB4DARBPROC', 'PFNGLVERTEXATTRIB4DVARBPROC',
'PFNGLVERTEXATTRIB4FARBPROC', 'PFNGLVERTEXATTRIB4FVARBPROC',
'PFNGLVERTEXATTRIB4IVARBPROC', 'PFNGLVERTEXATTRIB4SARBPROC',
'PFNGLVERTEXATTRIB4SVARBPROC', 'PFNGLVERTEXATTRIB4UBVARBPROC',
'PFNGLVERTEXATTRIB4UIVARBPROC', 'PFNGLVERTEXATTRIB4USVARBPROC',
'PFNGLVERTEXATTRIBPOINTERARBPROC', 'PFNGLENABLEVERTEXATTRIBARRAYARBPROC',
'PFNGLDISABLEVERTEXATTRIBARRAYARBPROC', 'PFNGLPROGRAMSTRINGARBPROC',
'PFNGLBINDPROGRAMARBPROC', 'PFNGLDELETEPROGRAMSARBPROC',
'PFNGLGENPROGRAMSARBPROC', 'PFNGLPROGRAMENVPARAMETER4DARBPROC',
'PFNGLPROGRAMENVPARAMETER4DVARBPROC', 'PFNGLPROGRAMENVPARAMETER4FARBPROC',
'PFNGLPROGRAMENVPARAMETER4FVARBPROC', 'PFNGLPROGRAMLOCALPARAMETER4DARBPROC',
'PFNGLPROGRAMLOCALPARAMETER4DVARBPROC', 'PFNGLPROGRAMLOCALPARAMETER4FARBPROC',
'PFNGLPROGRAMLOCALPARAMETER4FVARBPROC',
'PFNGLGETPROGRAMENVPARAMETERDVARBPROC',
'PFNGLGETPROGRAMENVPARAMETERFVARBPROC',
'PFNGLGETPROGRAMLOCALPARAMETERDVARBPROC',
'PFNGLGETPROGRAMLOCALPARAMETERFVARBPROC', 'PFNGLGETPROGRAMIVARBPROC',
'PFNGLGETPROGRAMSTRINGARBPROC', 'PFNGLGETVERTEXATTRIBDVARBPROC',
'PFNGLGETVERTEXATTRIBFVARBPROC', 'PFNGLGETVERTEXATTRIBIVARBPROC',
'PFNGLGETVERTEXATTRIBPOINTERVARBPROC', 'PFNGLISPROGRAMARBPROC',
'GL_ARB_fragment_program', 'GL_ARB_vertex_buffer_object', 'glBindBufferARB',
'glDeleteBuffersARB', 'glGenBuffersARB', 'glIsBufferARB', 'glBufferDataARB',
'glBufferSubDataARB', 'glGetBufferSubDataARB', 'glMapBufferARB',
'glUnmapBufferARB', 'glGetBufferParameterivARB', 'glGetBufferPointervARB',
'PFNGLBINDBUFFERARBPROC', 'PFNGLDELETEBUFFERSARBPROC',
'PFNGLGENBUFFERSARBPROC', 'PFNGLISBUFFERARBPROC', 'PFNGLBUFFERDATAARBPROC',
'PFNGLBUFFERSUBDATAARBPROC', 'PFNGLGETBUFFERSUBDATAARBPROC',
'PFNGLMAPBUFFERARBPROC', 'PFNGLUNMAPBUFFERARBPROC',
'PFNGLGETBUFFERPARAMETERIVARBPROC', 'PFNGLGETBUFFERPOINTERVARBPROC',
'GL_ARB_occlusion_query', 'glGenQueriesARB', 'glDeleteQueriesARB',
'glIsQueryARB', 'glBeginQueryARB', 'glEndQueryARB', 'glGetQueryivARB',
'glGetQueryObjectivARB', 'glGetQueryObjectuivARB', 'PFNGLGENQUERIESARBPROC',
'PFNGLDELETEQUERIESARBPROC', 'PFNGLISQUERYARBPROC', 'PFNGLBEGINQUERYARBPROC',
'PFNGLENDQUERYARBPROC', 'PFNGLGETQUERYIVARBPROC',
'PFNGLGETQUERYOBJECTIVARBPROC', 'PFNGLGETQUERYOBJECTUIVARBPROC',
'GL_ARB_shader_objects', 'glDeleteObjectARB', 'glGetHandleARB',
'glDetachObjectARB', 'glCreateShaderObjectARB', 'glShaderSourceARB',
'glCompileShaderARB', 'glCreateProgramObjectARB', 'glAttachObjectARB',
'glLinkProgramARB', 'glUseProgramObjectARB', 'glValidateProgramARB',
'glUniform1fARB', 'glUniform2fARB', 'glUniform3fARB', 'glUniform4fARB',
'glUniform1iARB', 'glUniform2iARB', 'glUniform3iARB', 'glUniform4iARB',
'glUniform1fvARB', 'glUniform2fvARB', 'glUniform3fvARB', 'glUniform4fvARB',
'glUniform1ivARB', 'glUniform2ivARB', 'glUniform3ivARB', 'glUniform4ivARB',
'glUniformMatrix2fvARB', 'glUniformMatrix3fvARB', 'glUniformMatrix4fvARB',
'glGetObjectParameterfvARB', 'glGetObjectParameterivARB', 'glGetInfoLogARB',
'glGetAttachedObjectsARB', 'glGetUniformLocationARB', 'glGetActiveUniformARB',
'glGetUniformfvARB', 'glGetUniformivARB', 'glGetShaderSourceARB',
'PFNGLDELETEOBJECTARBPROC', 'PFNGLGETHANDLEARBPROC',
'PFNGLDETACHOBJECTARBPROC', 'PFNGLCREATESHADEROBJECTARBPROC',
'PFNGLSHADERSOURCEARBPROC', 'PFNGLCOMPILESHADERARBPROC',
'PFNGLCREATEPROGRAMOBJECTARBPROC', 'PFNGLATTACHOBJECTARBPROC',
'PFNGLLINKPROGRAMARBPROC', 'PFNGLUSEPROGRAMOBJECTARBPROC',
'PFNGLVALIDATEPROGRAMARBPROC', 'PFNGLUNIFORM1FARBPROC',
'PFNGLUNIFORM2FARBPROC', 'PFNGLUNIFORM3FARBPROC', 'PFNGLUNIFORM4FARBPROC',
'PFNGLUNIFORM1IARBPROC', 'PFNGLUNIFORM2IARBPROC', 'PFNGLUNIFORM3IARBPROC',
'PFNGLUNIFORM4IARBPROC', 'PFNGLUNIFORM1FVARBPROC', 'PFNGLUNIFORM2FVARBPROC',
'PFNGLUNIFORM3FVARBPROC', 'PFNGLUNIFORM4FVARBPROC', 'PFNGLUNIFORM1IVARBPROC',
'PFNGLUNIFORM2IVARBPROC', 'PFNGLUNIFORM3IVARBPROC', 'PFNGLUNIFORM4IVARBPROC',
'PFNGLUNIFORMMATRIX2FVARBPROC', 'PFNGLUNIFORMMATRIX3FVARBPROC',
'PFNGLUNIFORMMATRIX4FVARBPROC', 'PFNGLGETOBJECTPARAMETERFVARBPROC',
'PFNGLGETOBJECTPARAMETERIVARBPROC', 'PFNGLGETINFOLOGARBPROC',
'PFNGLGETATTACHEDOBJECTSARBPROC', 'PFNGLGETUNIFORMLOCATIONARBPROC',
'PFNGLGETACTIVEUNIFORMARBPROC', 'PFNGLGETUNIFORMFVARBPROC',
'PFNGLGETUNIFORMIVARBPROC', 'PFNGLGETSHADERSOURCEARBPROC',
'GL_ARB_vertex_shader', 'glBindAttribLocationARB', 'glGetActiveAttribARB',
'glGetAttribLocationARB', 'PFNGLBINDATTRIBLOCATIONARBPROC',
'PFNGLGETACTIVEATTRIBARBPROC', 'PFNGLGETATTRIBLOCATIONARBPROC',
'GL_ARB_fragment_shader', 'GL_ARB_shading_language_100',
'GL_ARB_texture_non_power_of_two', 'GL_ARB_point_sprite',
'GL_ARB_fragment_program_shadow', 'GL_ARB_draw_buffers', 'glDrawBuffersARB',
'PFNGLDRAWBUFFERSARBPROC', 'GL_ARB_texture_rectangle',
'GL_ARB_color_buffer_float', 'glClampColorARB', 'PFNGLCLAMPCOLORARBPROC',
'GL_ARB_half_float_pixel', 'GL_ARB_texture_float',
'GL_ARB_pixel_buffer_object', 'GL_EXT_abgr', 'GL_EXT_blend_color',
'glBlendColorEXT', 'PFNGLBLENDCOLOREXTPROC', 'GL_EXT_polygon_offset',
'glPolygonOffsetEXT', 'PFNGLPOLYGONOFFSETEXTPROC', 'GL_EXT_texture',
'GL_EXT_texture3D', 'glTexImage3DEXT', 'glTexSubImage3DEXT',
'PFNGLTEXIMAGE3DEXTPROC', 'PFNGLTEXSUBIMAGE3DEXTPROC',
'GL_SGIS_texture_filter4', 'glGetTexFilterFuncSGIS', 'glTexFilterFuncSGIS',
'PFNGLGETTEXFILTERFUNCSGISPROC', 'PFNGLTEXFILTERFUNCSGISPROC',
'GL_EXT_subtexture', 'glTexSubImage1DEXT', 'glTexSubImage2DEXT',
'PFNGLTEXSUBIMAGE1DEXTPROC', 'PFNGLTEXSUBIMAGE2DEXTPROC',
'GL_EXT_copy_texture', 'glCopyTexImage1DEXT', 'glCopyTexImage2DEXT',
'glCopyTexSubImage1DEXT', 'glCopyTexSubImage2DEXT', 'glCopyTexSubImage3DEXT',
'PFNGLCOPYTEXIMAGE1DEXTPROC', 'PFNGLCOPYTEXIMAGE2DEXTPROC',
'PFNGLCOPYTEXSUBIMAGE1DEXTPROC', 'PFNGLCOPYTEXSUBIMAGE2DEXTPROC',
'PFNGLCOPYTEXSUBIMAGE3DEXTPROC', 'GL_EXT_histogram', 'glGetHistogramEXT',
'glGetHistogramParameterfvEXT', 'glGetHistogramParameterivEXT',
'glGetMinmaxEXT', 'glGetMinmaxParameterfvEXT', 'glGetMinmaxParameterivEXT',
'glHistogramEXT', 'glMinmaxEXT', 'glResetHistogramEXT', 'glResetMinmaxEXT',
'PFNGLGETHISTOGRAMEXTPROC', 'PFNGLGETHISTOGRAMPARAMETERFVEXTPROC',
'PFNGLGETHISTOGRAMPARAMETERIVEXTPROC', 'PFNGLGETMINMAXEXTPROC',
'PFNGLGETMINMAXPARAMETERFVEXTPROC', 'PFNGLGETMINMAXPARAMETERIVEXTPROC',
'PFNGLHISTOGRAMEXTPROC', 'PFNGLMINMAXEXTPROC', 'PFNGLRESETHISTOGRAMEXTPROC',
'PFNGLRESETMINMAXEXTPROC', 'GL_EXT_convolution', 'glConvolutionFilter1DEXT',
'glConvolutionFilter2DEXT', 'glConvolutionParameterfEXT',
'glConvolutionParameterfvEXT', 'glConvolutionParameteriEXT',
'glConvolutionParameterivEXT', 'glCopyConvolutionFilter1DEXT',
'glCopyConvolutionFilter2DEXT', 'glGetConvolutionFilterEXT',
'glGetConvolutionParameterfvEXT', 'glGetConvolutionParameterivEXT',
'glGetSeparableFilterEXT', 'glSeparableFilter2DEXT',
'PFNGLCONVOLUTIONFILTER1DEXTPROC', 'PFNGLCONVOLUTIONFILTER2DEXTPROC',
'PFNGLCONVOLUTIONPARAMETERFEXTPROC', 'PFNGLCONVOLUTIONPARAMETERFVEXTPROC',
'PFNGLCONVOLUTIONPARAMETERIEXTPROC', 'PFNGLCONVOLUTIONPARAMETERIVEXTPROC',
'PFNGLCOPYCONVOLUTIONFILTER1DEXTPROC', 'PFNGLCOPYCONVOLUTIONFILTER2DEXTPROC',
'PFNGLGETCONVOLUTIONFILTEREXTPROC', 'PFNGLGETCONVOLUTIONPARAMETERFVEXTPROC',
'PFNGLGETCONVOLUTIONPARAMETERIVEXTPROC', 'PFNGLGETSEPARABLEFILTEREXTPROC',
'PFNGLSEPARABLEFILTER2DEXTPROC', 'GL_EXT_color_matrix', 'GL_SGI_color_table',
'glColorTableSGI', 'glColorTableParameterfvSGI', 'glColorTableParameterivSGI',
'glCopyColorTableSGI', 'glGetColorTableSGI', 'glGetColorTableParameterfvSGI',
'glGetColorTableParameterivSGI', 'PFNGLCOLORTABLESGIPROC',
'PFNGLCOLORTABLEPARAMETERFVSGIPROC', 'PFNGLCOLORTABLEPARAMETERIVSGIPROC',
'PFNGLCOPYCOLORTABLESGIPROC', 'PFNGLGETCOLORTABLESGIPROC',
'PFNGLGETCOLORTABLEPARAMETERFVSGIPROC',
'PFNGLGETCOLORTABLEPARAMETERIVSGIPROC', 'GL_SGIX_pixel_texture',
'glPixelTexGenSGIX', 'PFNGLPIXELTEXGENSGIXPROC', 'GL_SGIS_pixel_texture',
'glPixelTexGenParameteriSGIS', 'glPixelTexGenParameterivSGIS',
'glPixelTexGenParameterfSGIS', 'glPixelTexGenParameterfvSGIS',
'glGetPixelTexGenParameterivSGIS', 'glGetPixelTexGenParameterfvSGIS',
'PFNGLPIXELTEXGENPARAMETERISGISPROC', 'PFNGLPIXELTEXGENPARAMETERIVSGISPROC',
'PFNGLPIXELTEXGENPARAMETERFSGISPROC', 'PFNGLPIXELTEXGENPARAMETERFVSGISPROC',
'PFNGLGETPIXELTEXGENPARAMETERIVSGISPROC',
'PFNGLGETPIXELTEXGENPARAMETERFVSGISPROC', 'GL_SGIS_texture4D',
'glTexImage4DSGIS', 'glTexSubImage4DSGIS', 'PFNGLTEXIMAGE4DSGISPROC',
'PFNGLTEXSUBIMAGE4DSGISPROC', 'GL_SGI_texture_color_table', 'GL_EXT_cmyka',
'GL_EXT_texture_object', 'glAreTexturesResidentEXT', 'glBindTextureEXT',
'glDeleteTexturesEXT', 'glGenTexturesEXT', 'glIsTextureEXT',
'glPrioritizeTexturesEXT', 'PFNGLARETEXTURESRESIDENTEXTPROC',
'PFNGLBINDTEXTUREEXTPROC', 'PFNGLDELETETEXTURESEXTPROC',
'PFNGLGENTEXTURESEXTPROC', 'PFNGLISTEXTUREEXTPROC',
'PFNGLPRIORITIZETEXTURESEXTPROC', 'GL_SGIS_detail_texture',
'glDetailTexFuncSGIS', 'glGetDetailTexFuncSGIS', 'PFNGLDETAILTEXFUNCSGISPROC',
'PFNGLGETDETAILTEXFUNCSGISPROC', 'GL_SGIS_sharpen_texture',
'glSharpenTexFuncSGIS', 'glGetSharpenTexFuncSGIS',
'PFNGLSHARPENTEXFUNCSGISPROC', 'PFNGLGETSHARPENTEXFUNCSGISPROC',
'GL_EXT_packed_pixels', 'GL_SGIS_texture_lod', 'GL_SGIS_multisample',
'glSampleMaskSGIS', 'glSamplePatternSGIS', 'PFNGLSAMPLEMASKSGISPROC',
'PFNGLSAMPLEPATTERNSGISPROC', 'GL_EXT_rescale_normal', 'GL_EXT_vertex_array',
'glArrayElementEXT', 'glColorPointerEXT', 'glDrawArraysEXT',
'glEdgeFlagPointerEXT', 'glGetPointervEXT', 'glIndexPointerEXT',
'glNormalPointerEXT', 'glTexCoordPointerEXT', 'glVertexPointerEXT',
'PFNGLARRAYELEMENTEXTPROC', 'PFNGLCOLORPOINTEREXTPROC',
'PFNGLDRAWARRAYSEXTPROC', 'PFNGLEDGEFLAGPOINTEREXTPROC',
'PFNGLGETPOINTERVEXTPROC', 'PFNGLINDEXPOINTEREXTPROC',
'PFNGLNORMALPOINTEREXTPROC', 'PFNGLTEXCOORDPOINTEREXTPROC',
'PFNGLVERTEXPOINTEREXTPROC', 'GL_EXT_misc_attribute',
'GL_SGIS_generate_mipmap', 'GL_SGIX_clipmap', 'GL_SGIX_shadow',
'GL_SGIS_texture_edge_clamp', 'GL_SGIS_texture_border_clamp',
'GL_EXT_blend_minmax', 'glBlendEquationEXT', 'PFNGLBLENDEQUATIONEXTPROC',
'GL_EXT_blend_subtract', 'GL_EXT_blend_logic_op', 'GL_SGIX_interlace',
'GL_SGIX_pixel_tiles', 'GL_SGIX_texture_select', 'GL_SGIX_sprite',
'glSpriteParameterfSGIX', 'glSpriteParameterfvSGIX', 'glSpriteParameteriSGIX',
'glSpriteParameterivSGIX', 'PFNGLSPRITEPARAMETERFSGIXPROC',
'PFNGLSPRITEPARAMETERFVSGIXPROC', 'PFNGLSPRITEPARAMETERISGIXPROC',
'PFNGLSPRITEPARAMETERIVSGIXPROC', 'GL_SGIX_texture_multi_buffer',
'GL_EXT_point_parameters', 'glPointParameterfEXT', 'glPointParameterfvEXT',
'PFNGLPOINTPARAMETERFEXTPROC', 'PFNGLPOINTPARAMETERFVEXTPROC',
'GL_SGIS_point_parameters', 'glPointParameterfSGIS', 'glPointParameterfvSGIS',
'PFNGLPOINTPARAMETERFSGISPROC', 'PFNGLPOINTPARAMETERFVSGISPROC',
'GL_SGIX_instruments', 'glGetInstrumentsSGIX', 'glInstrumentsBufferSGIX',
'glPollInstrumentsSGIX', 'glReadInstrumentsSGIX', 'glStartInstrumentsSGIX',
'glStopInstrumentsSGIX', 'PFNGLGETINSTRUMENTSSGIXPROC',
'PFNGLINSTRUMENTSBUFFERSGIXPROC', 'PFNGLPOLLINSTRUMENTSSGIXPROC',
'PFNGLREADINSTRUMENTSSGIXPROC', 'PFNGLSTARTINSTRUMENTSSGIXPROC',
'PFNGLSTOPINSTRUMENTSSGIXPROC', 'GL_SGIX_texture_scale_bias',
'GL_SGIX_framezoom', 'glFrameZoomSGIX', 'PFNGLFRAMEZOOMSGIXPROC',
'GL_SGIX_tag_sample_buffer', 'glTagSampleBufferSGIX',
'PFNGLTAGSAMPLEBUFFERSGIXPROC', 'GL_SGIX_polynomial_ffd',
'glDeformationMap3dSGIX', 'glDeformationMap3fSGIX', 'glDeformSGIX',
'glLoadIdentityDeformationMapSGIX', 'PFNGLDEFORMATIONMAP3DSGIXPROC',
'PFNGLDEFORMATIONMAP3FSGIXPROC', 'PFNGLDEFORMSGIXPROC',
'PFNGLLOADIDENTITYDEFORMATIONMAPSGIXPROC', 'GL_SGIX_reference_plane',
'glReferencePlaneSGIX', 'PFNGLREFERENCEPLANESGIXPROC', 'GL_SGIX_flush_raster',
'glFlushRasterSGIX', 'PFNGLFLUSHRASTERSGIXPROC', 'GL_SGIX_depth_texture',
'GL_SGIS_fog_function', 'glFogFuncSGIS', 'glGetFogFuncSGIS',
'PFNGLFOGFUNCSGISPROC', 'PFNGLGETFOGFUNCSGISPROC', 'GL_SGIX_fog_offset',
'GL_HP_image_transform', 'glImageTransformParameteriHP',
'glImageTransformParameterfHP', 'glImageTransformParameterivHP',
'glImageTransformParameterfvHP', 'glGetImageTransformParameterivHP',
'glGetImageTransformParameterfvHP', 'PFNGLIMAGETRANSFORMPARAMETERIHPPROC',
'PFNGLIMAGETRANSFORMPARAMETERFHPPROC', 'PFNGLIMAGETRANSFORMPARAMETERIVHPPROC',
'PFNGLIMAGETRANSFORMPARAMETERFVHPPROC',
'PFNGLGETIMAGETRANSFORMPARAMETERIVHPPROC',
'PFNGLGETIMAGETRANSFORMPARAMETERFVHPPROC', 'GL_HP_convolution_border_modes',
'GL_SGIX_texture_add_env', 'GL_EXT_color_subtable', 'glColorSubTableEXT',
'glCopyColorSubTableEXT', 'PFNGLCOLORSUBTABLEEXTPROC',
'PFNGLCOPYCOLORSUBTABLEEXTPROC', 'GL_PGI_vertex_hints', 'GL_PGI_misc_hints',
'glHintPGI', 'PFNGLHINTPGIPROC', 'GL_EXT_paletted_texture', 'glColorTableEXT',
'glGetColorTableEXT', 'glGetColorTableParameterivEXT',
'glGetColorTableParameterfvEXT', 'PFNGLCOLORTABLEEXTPROC',
'PFNGLGETCOLORTABLEEXTPROC', 'PFNGLGETCOLORTABLEPARAMETERIVEXTPROC',
'PFNGLGETCOLORTABLEPARAMETERFVEXTPROC', 'GL_EXT_clip_volume_hint',
'GL_SGIX_list_priority', 'glGetListParameterfvSGIX',
'glGetListParameterivSGIX', 'glListParameterfSGIX', 'glListParameterfvSGIX',
'glListParameteriSGIX', 'glListParameterivSGIX',
'PFNGLGETLISTPARAMETERFVSGIXPROC', 'PFNGLGETLISTPARAMETERIVSGIXPROC',
'PFNGLLISTPARAMETERFSGIXPROC', 'PFNGLLISTPARAMETERFVSGIXPROC',
'PFNGLLISTPARAMETERISGIXPROC', 'PFNGLLISTPARAMETERIVSGIXPROC',
'GL_SGIX_ir_instrument1', 'GL_SGIX_calligraphic_fragment',
'GL_SGIX_texture_lod_bias', 'GL_SGIX_shadow_ambient', 'GL_EXT_index_texture',
'GL_EXT_index_material', 'glIndexMaterialEXT', 'PFNGLINDEXMATERIALEXTPROC',
'GL_EXT_index_func', 'glIndexFuncEXT', 'PFNGLINDEXFUNCEXTPROC',
'GL_EXT_index_array_formats', 'GL_EXT_compiled_vertex_array',
'glLockArraysEXT', 'glUnlockArraysEXT', 'PFNGLLOCKARRAYSEXTPROC',
'PFNGLUNLOCKARRAYSEXTPROC', 'GL_EXT_cull_vertex', 'glCullParameterdvEXT',
'glCullParameterfvEXT', 'PFNGLCULLPARAMETERDVEXTPROC',
'PFNGLCULLPARAMETERFVEXTPROC', 'GL_SGIX_ycrcb', 'GL_SGIX_fragment_lighting',
'glFragmentColorMaterialSGIX', 'glFragmentLightfSGIX',
'glFragmentLightfvSGIX', 'glFragmentLightiSGIX', 'glFragmentLightivSGIX',
'glFragmentLightModelfSGIX', 'glFragmentLightModelfvSGIX',
'glFragmentLightModeliSGIX', 'glFragmentLightModelivSGIX',
'glFragmentMaterialfSGIX', 'glFragmentMaterialfvSGIX',
'glFragmentMaterialiSGIX', 'glFragmentMaterialivSGIX',
'glGetFragmentLightfvSGIX', 'glGetFragmentLightivSGIX',
'glGetFragmentMaterialfvSGIX', 'glGetFragmentMaterialivSGIX',
'glLightEnviSGIX', 'PFNGLFRAGMENTCOLORMATERIALSGIXPROC',
'PFNGLFRAGMENTLIGHTFSGIXPROC', 'PFNGLFRAGMENTLIGHTFVSGIXPROC',
'PFNGLFRAGMENTLIGHTISGIXPROC', 'PFNGLFRAGMENTLIGHTIVSGIXPROC',
'PFNGLFRAGMENTLIGHTMODELFSGIXPROC', 'PFNGLFRAGMENTLIGHTMODELFVSGIXPROC',
'PFNGLFRAGMENTLIGHTMODELISGIXPROC', 'PFNGLFRAGMENTLIGHTMODELIVSGIXPROC',
'PFNGLFRAGMENTMATERIALFSGIXPROC', 'PFNGLFRAGMENTMATERIALFVSGIXPROC',
'PFNGLFRAGMENTMATERIALISGIXPROC', 'PFNGLFRAGMENTMATERIALIVSGIXPROC',
'PFNGLGETFRAGMENTLIGHTFVSGIXPROC', 'PFNGLGETFRAGMENTLIGHTIVSGIXPROC',
'PFNGLGETFRAGMENTMATERIALFVSGIXPROC', 'PFNGLGETFRAGMENTMATERIALIVSGIXPROC',
'PFNGLLIGHTENVISGIXPROC', 'GL_IBM_rasterpos_clip', 'GL_HP_texture_lighting',
'GL_EXT_draw_range_elements', 'glDrawRangeElementsEXT',
'PFNGLDRAWRANGEELEMENTSEXTPROC', 'GL_WIN_phong_shading',
'GL_WIN_specular_fog', 'GL_EXT_light_texture', 'glApplyTextureEXT',
'glTextureLightEXT', 'glTextureMaterialEXT', 'PFNGLAPPLYTEXTUREEXTPROC',
'PFNGLTEXTURELIGHTEXTPROC', 'PFNGLTEXTUREMATERIALEXTPROC',
'GL_SGIX_blend_alpha_minmax', 'GL_EXT_bgra', 'GL_SGIX_async',
'glAsyncMarkerSGIX', 'glFinishAsyncSGIX', 'glPollAsyncSGIX',
'glGenAsyncMarkersSGIX', 'glDeleteAsyncMarkersSGIX', 'glIsAsyncMarkerSGIX',
'PFNGLASYNCMARKERSGIXPROC', 'PFNGLFINISHASYNCSGIXPROC',
'PFNGLPOLLASYNCSGIXPROC', 'PFNGLGENASYNCMARKERSSGIXPROC',
'PFNGLDELETEASYNCMARKERSSGIXPROC', 'PFNGLISASYNCMARKERSGIXPROC',
'GL_SGIX_async_pixel', 'GL_SGIX_async_histogram', 'GL_INTEL_parallel_arrays',
'glVertexPointervINTEL', 'glNormalPointervINTEL', 'glColorPointervINTEL',
'glTexCoordPointervINTEL', 'PFNGLVERTEXPOINTERVINTELPROC',
'PFNGLNORMALPOINTERVINTELPROC', 'PFNGLCOLORPOINTERVINTELPROC',
'PFNGLTEXCOORDPOINTERVINTELPROC', 'GL_HP_occlusion_test',
'GL_EXT_pixel_transform', 'glPixelTransformParameteriEXT',
'glPixelTransformParameterfEXT', 'glPixelTransformParameterivEXT',
'glPixelTransformParameterfvEXT', 'PFNGLPIXELTRANSFORMPARAMETERIEXTPROC',
'PFNGLPIXELTRANSFORMPARAMETERFEXTPROC',
'PFNGLPIXELTRANSFORMPARAMETERIVEXTPROC',
'PFNGLPIXELTRANSFORMPARAMETERFVEXTPROC', 'GL_EXT_pixel_transform_color_table',
'GL_EXT_shared_texture_palette', 'GL_EXT_separate_specular_color',
'GL_EXT_secondary_color', 'glSecondaryColor3bEXT', 'glSecondaryColor3bvEXT',
'glSecondaryColor3dEXT', 'glSecondaryColor3dvEXT', 'glSecondaryColor3fEXT',
'glSecondaryColor3fvEXT', 'glSecondaryColor3iEXT', 'glSecondaryColor3ivEXT',
'glSecondaryColor3sEXT', 'glSecondaryColor3svEXT', 'glSecondaryColor3ubEXT',
'glSecondaryColor3ubvEXT', 'glSecondaryColor3uiEXT',
'glSecondaryColor3uivEXT', 'glSecondaryColor3usEXT',
'glSecondaryColor3usvEXT', 'glSecondaryColorPointerEXT',
'PFNGLSECONDARYCOLOR3BEXTPROC', 'PFNGLSECONDARYCOLOR3BVEXTPROC',
'PFNGLSECONDARYCOLOR3DEXTPROC', 'PFNGLSECONDARYCOLOR3DVEXTPROC',
'PFNGLSECONDARYCOLOR3FEXTPROC', 'PFNGLSECONDARYCOLOR3FVEXTPROC',
'PFNGLSECONDARYCOLOR3IEXTPROC', 'PFNGLSECONDARYCOLOR3IVEXTPROC',
'PFNGLSECONDARYCOLOR3SEXTPROC', 'PFNGLSECONDARYCOLOR3SVEXTPROC',
'PFNGLSECONDARYCOLOR3UBEXTPROC', 'PFNGLSECONDARYCOLOR3UBVEXTPROC',
'PFNGLSECONDARYCOLOR3UIEXTPROC', 'PFNGLSECONDARYCOLOR3UIVEXTPROC',
'PFNGLSECONDARYCOLOR3USEXTPROC', 'PFNGLSECONDARYCOLOR3USVEXTPROC',
'PFNGLSECONDARYCOLORPOINTEREXTPROC', 'GL_EXT_texture_perturb_normal',
'glTextureNormalEXT', 'PFNGLTEXTURENORMALEXTPROC', 'GL_EXT_multi_draw_arrays',
'glMultiDrawArraysEXT', 'glMultiDrawElementsEXT',
'PFNGLMULTIDRAWARRAYSEXTPROC', 'PFNGLMULTIDRAWELEMENTSEXTPROC',
'GL_EXT_fog_coord', 'glFogCoordfEXT', 'glFogCoordfvEXT', 'glFogCoorddEXT',
'glFogCoorddvEXT', 'glFogCoordPointerEXT', 'PFNGLFOGCOORDFEXTPROC',
'PFNGLFOGCOORDFVEXTPROC', 'PFNGLFOGCOORDDEXTPROC', 'PFNGLFOGCOORDDVEXTPROC',
'PFNGLFOGCOORDPOINTEREXTPROC', 'GL_REND_screen_coordinates',
'GL_EXT_coordinate_frame', 'glTangent3bEXT', 'glTangent3bvEXT',
'glTangent3dEXT', 'glTangent3dvEXT', 'glTangent3fEXT', 'glTangent3fvEXT',
'glTangent3iEXT', 'glTangent3ivEXT', 'glTangent3sEXT', 'glTangent3svEXT',
'glBinormal3bEXT', 'glBinormal3bvEXT', 'glBinormal3dEXT', 'glBinormal3dvEXT',
'glBinormal3fEXT', 'glBinormal3fvEXT', 'glBinormal3iEXT', 'glBinormal3ivEXT',
'glBinormal3sEXT', 'glBinormal3svEXT', 'glTangentPointerEXT',
'glBinormalPointerEXT', 'PFNGLTANGENT3BEXTPROC', 'PFNGLTANGENT3BVEXTPROC',
'PFNGLTANGENT3DEXTPROC', 'PFNGLTANGENT3DVEXTPROC', 'PFNGLTANGENT3FEXTPROC',
'PFNGLTANGENT3FVEXTPROC', 'PFNGLTANGENT3IEXTPROC', 'PFNGLTANGENT3IVEXTPROC',
'PFNGLTANGENT3SEXTPROC', 'PFNGLTANGENT3SVEXTPROC', 'PFNGLBINORMAL3BEXTPROC',
'PFNGLBINORMAL3BVEXTPROC', 'PFNGLBINORMAL3DEXTPROC',
'PFNGLBINORMAL3DVEXTPROC', 'PFNGLBINORMAL3FEXTPROC',
'PFNGLBINORMAL3FVEXTPROC', 'PFNGLBINORMAL3IEXTPROC',
'PFNGLBINORMAL3IVEXTPROC', 'PFNGLBINORMAL3SEXTPROC',
'PFNGLBINORMAL3SVEXTPROC', 'PFNGLTANGENTPOINTEREXTPROC',
'PFNGLBINORMALPOINTEREXTPROC', 'GL_EXT_texture_env_combine',
'GL_APPLE_specular_vector', 'GL_APPLE_transform_hint', 'GL_SGIX_fog_scale',
'GL_SUNX_constant_data', 'glFinishTextureSUNX', 'PFNGLFINISHTEXTURESUNXPROC',
'GL_SUN_global_alpha', 'glGlobalAlphaFactorbSUN', 'glGlobalAlphaFactorsSUN',
'glGlobalAlphaFactoriSUN', 'glGlobalAlphaFactorfSUN',
'glGlobalAlphaFactordSUN', 'glGlobalAlphaFactorubSUN',
'glGlobalAlphaFactorusSUN', 'glGlobalAlphaFactoruiSUN',
'PFNGLGLOBALALPHAFACTORBSUNPROC', 'PFNGLGLOBALALPHAFACTORSSUNPROC',
'PFNGLGLOBALALPHAFACTORISUNPROC', 'PFNGLGLOBALALPHAFACTORFSUNPROC',
'PFNGLGLOBALALPHAFACTORDSUNPROC', 'PFNGLGLOBALALPHAFACTORUBSUNPROC',
'PFNGLGLOBALALPHAFACTORUSSUNPROC', 'PFNGLGLOBALALPHAFACTORUISUNPROC',
'GL_SUN_triangle_list', 'glReplacementCodeuiSUN', 'glReplacementCodeusSUN',
'glReplacementCodeubSUN', 'glReplacementCodeuivSUN',
'glReplacementCodeusvSUN', 'glReplacementCodeubvSUN',
'glReplacementCodePointerSUN', 'PFNGLREPLACEMENTCODEUISUNPROC',
'PFNGLREPLACEMENTCODEUSSUNPROC', 'PFNGLREPLACEMENTCODEUBSUNPROC',
'PFNGLREPLACEMENTCODEUIVSUNPROC', 'PFNGLREPLACEMENTCODEUSVSUNPROC',
'PFNGLREPLACEMENTCODEUBVSUNPROC', 'PFNGLREPLACEMENTCODEPOINTERSUNPROC',
'GL_SUN_vertex', 'glColor4ubVertex2fSUN', 'glColor4ubVertex2fvSUN',
'glColor4ubVertex3fSUN', 'glColor4ubVertex3fvSUN', 'glColor3fVertex3fSUN',
'glColor3fVertex3fvSUN', 'glNormal3fVertex3fSUN', 'glNormal3fVertex3fvSUN',
'glColor4fNormal3fVertex3fSUN', 'glColor4fNormal3fVertex3fvSUN',
'glTexCoord2fVertex3fSUN', 'glTexCoord2fVertex3fvSUN',
'glTexCoord4fVertex4fSUN', 'glTexCoord4fVertex4fvSUN',
'glTexCoord2fColor4ubVertex3fSUN', 'glTexCoord2fColor4ubVertex3fvSUN',
'glTexCoord2fColor3fVertex3fSUN', 'glTexCoord2fColor3fVertex3fvSUN',
'glTexCoord2fNormal3fVertex3fSUN', 'glTexCoord2fNormal3fVertex3fvSUN',
'glTexCoord2fColor4fNormal3fVertex3fSUN',
'glTexCoord2fColor4fNormal3fVertex3fvSUN',
'glTexCoord4fColor4fNormal3fVertex4fSUN',
'glTexCoord4fColor4fNormal3fVertex4fvSUN', 'glReplacementCodeuiVertex3fSUN',
'glReplacementCodeuiVertex3fvSUN', 'glReplacementCodeuiColor4ubVertex3fSUN',
'glReplacementCodeuiColor4ubVertex3fvSUN',
'glReplacementCodeuiColor3fVertex3fSUN',
'glReplacementCodeuiColor3fVertex3fvSUN',
'glReplacementCodeuiNormal3fVertex3fSUN',
'glReplacementCodeuiNormal3fVertex3fvSUN',
'glReplacementCodeuiColor4fNormal3fVertex3fSUN',
'glReplacementCodeuiColor4fNormal3fVertex3fvSUN',
'glReplacementCodeuiTexCoord2fVertex3fSUN',
'glReplacementCodeuiTexCoord2fVertex3fvSUN',
'glReplacementCodeuiTexCoord2fNormal3fVertex3fSUN',
'glReplacementCodeuiTexCoord2fNormal3fVertex3fvSUN',
'glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fSUN',
'glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fvSUN',
'PFNGLCOLOR4UBVERTEX2FSUNPROC', 'PFNGLCOLOR4UBVERTEX2FVSUNPROC',
'PFNGLCOLOR4UBVERTEX3FSUNPROC', 'PFNGLCOLOR4UBVERTEX3FVSUNPROC',
'PFNGLCOLOR3FVERTEX3FSUNPROC', 'PFNGLCOLOR3FVERTEX3FVSUNPROC',
'PFNGLNORMAL3FVERTEX3FSUNPROC', 'PFNGLNORMAL3FVERTEX3FVSUNPROC',
'PFNGLCOLOR4FNORMAL3FVERTEX3FSUNPROC', 'PFNGLCOLOR4FNORMAL3FVERTEX3FVSUNPROC',
'PFNGLTEXCOORD2FVERTEX3FSUNPROC', 'PFNGLTEXCOORD2FVERTEX3FVSUNPROC',
'PFNGLTEXCOORD4FVERTEX4FSUNPROC', 'PFNGLTEXCOORD4FVERTEX4FVSUNPROC',
'PFNGLTEXCOORD2FCOLOR4UBVERTEX3FSUNPROC',
'PFNGLTEXCOORD2FCOLOR4UBVERTEX3FVSUNPROC',
'PFNGLTEXCOORD2FCOLOR3FVERTEX3FSUNPROC',
'PFNGLTEXCOORD2FCOLOR3FVERTEX3FVSUNPROC',
'PFNGLTEXCOORD2FNORMAL3FVERTEX3FSUNPROC',
'PFNGLTEXCOORD2FNORMAL3FVERTEX3FVSUNPROC',
'PFNGLTEXCOORD2FCOLOR4FNORMAL3FVERTEX3FSUNPROC',
'PFNGLTEXCOORD2FCOLOR4FNORMAL3FVERTEX3FVSUNPROC',
'PFNGLTEXCOORD4FCOLOR4FNORMAL3FVERTEX4FSUNPROC',
'PFNGLTEXCOORD4FCOLOR4FNORMAL3FVERTEX4FVSUNPROC',
'PFNGLREPLACEMENTCODEUIVERTEX3FSUNPROC',
'PFNGLREPLACEMENTCODEUIVERTEX3FVSUNPROC',
'PFNGLREPLACEMENTCODEUICOLOR4UBVERTEX3FSUNPROC',
'PFNGLREPLACEMENTCODEUICOLOR4UBVERTEX3FVSUNPROC',
'PFNGLREPLACEMENTCODEUICOLOR3FVERTEX3FSUNPROC',
'PFNGLREPLACEMENTCODEUICOLOR3FVERTEX3FVSUNPROC',
'PFNGLREPLACEMENTCODEUINORMAL3FVERTEX3FSUNPROC',
'PFNGLREPLACEMENTCODEUINORMAL3FVERTEX3FVSUNPROC',
'PFNGLREPLACEMENTCODEUICOLOR4FNORMAL3FVERTEX3FSUNPROC',
'PFNGLREPLACEMENTCODEUICOLOR4FNORMAL3FVERTEX3FVSUNPROC',
'PFNGLREPLACEMENTCODEUITEXCOORD2FVERTEX3FSUNPROC',
'PFNGLREPLACEMENTCODEUITEXCOORD2FVERTEX3FVSUNPROC',
'PFNGLREPLACEMENTCODEUITEXCOORD2FNORMAL3FVERTEX3FSUNPROC',
'PFNGLREPLACEMENTCODEUITEXCOORD2FNORMAL3FVERTEX3FVSUNPROC',
'PFNGLREPLACEMENTCODEUITEXCOORD2FCOLOR4FNORMAL3FVERTEX3FSUNPROC',
'PFNGLREPLACEMENTCODEUITEXCOORD2FCOLOR4FNORMAL3FVERTEX3FVSUNPROC',
'GL_EXT_blend_func_separate', 'glBlendFuncSeparateEXT',
'PFNGLBLENDFUNCSEPARATEEXTPROC', 'GL_INGR_blend_func_separate',
'glBlendFuncSeparateINGR', 'PFNGLBLENDFUNCSEPARATEINGRPROC',
'GL_INGR_color_clamp', 'GL_INGR_interlace_read', 'GL_EXT_stencil_wrap',
'GL_EXT_422_pixels', 'GL_NV_texgen_reflection',
'GL_SUN_convolution_border_modes', 'GL_EXT_texture_env_add',
'GL_EXT_texture_lod_bias', 'GL_EXT_texture_filter_anisotropic',
'GL_EXT_vertex_weighting', 'glVertexWeightfEXT', 'glVertexWeightfvEXT',
'glVertexWeightPointerEXT', 'PFNGLVERTEXWEIGHTFEXTPROC',
'PFNGLVERTEXWEIGHTFVEXTPROC', 'PFNGLVERTEXWEIGHTPOINTEREXTPROC',
'GL_NV_light_max_exponent', 'GL_NV_vertex_array_range',
'glFlushVertexArrayRangeNV', 'glVertexArrayRangeNV',
'PFNGLFLUSHVERTEXARRAYRANGENVPROC', 'PFNGLVERTEXARRAYRANGENVPROC',
'GL_NV_register_combiners', 'glCombinerParameterfvNV',
'glCombinerParameterfNV', 'glCombinerParameterivNV', 'glCombinerParameteriNV',
'glCombinerInputNV', 'glCombinerOutputNV', 'glFinalCombinerInputNV',
'glGetCombinerInputParameterfvNV', 'glGetCombinerInputParameterivNV',
'glGetCombinerOutputParameterfvNV', 'glGetCombinerOutputParameterivNV',
'glGetFinalCombinerInputParameterfvNV',
'glGetFinalCombinerInputParameterivNV', 'PFNGLCOMBINERPARAMETERFVNVPROC',
'PFNGLCOMBINERPARAMETERFNVPROC', 'PFNGLCOMBINERPARAMETERIVNVPROC',
'PFNGLCOMBINERPARAMETERINVPROC', 'PFNGLCOMBINERINPUTNVPROC',
'PFNGLCOMBINEROUTPUTNVPROC', 'PFNGLFINALCOMBINERINPUTNVPROC',
'PFNGLGETCOMBINERINPUTPARAMETERFVNVPROC',
'PFNGLGETCOMBINERINPUTPARAMETERIVNVPROC',
'PFNGLGETCOMBINEROUTPUTPARAMETERFVNVPROC',
'PFNGLGETCOMBINEROUTPUTPARAMETERIVNVPROC',
'PFNGLGETFINALCOMBINERINPUTPARAMETERFVNVPROC',
'PFNGLGETFINALCOMBINERINPUTPARAMETERIVNVPROC', 'GL_NV_fog_distance',
'GL_NV_texgen_emboss', 'GL_NV_blend_square', 'GL_NV_texture_env_combine4',
'GL_MESA_resize_buffers', 'glResizeBuffersMESA', 'PFNGLRESIZEBUFFERSMESAPROC',
'GL_MESA_window_pos', 'glWindowPos2dMESA', 'glWindowPos2dvMESA',
'glWindowPos2fMESA', 'glWindowPos2fvMESA', 'glWindowPos2iMESA',
'glWindowPos2ivMESA', 'glWindowPos2sMESA', 'glWindowPos2svMESA',
'glWindowPos3dMESA', 'glWindowPos3dvMESA', 'glWindowPos3fMESA',
'glWindowPos3fvMESA', 'glWindowPos3iMESA', 'glWindowPos3ivMESA',
'glWindowPos3sMESA', 'glWindowPos3svMESA', 'glWindowPos4dMESA',
'glWindowPos4dvMESA', 'glWindowPos4fMESA', 'glWindowPos4fvMESA',
'glWindowPos4iMESA', 'glWindowPos4ivMESA', 'glWindowPos4sMESA',
'glWindowPos4svMESA', 'PFNGLWINDOWPOS2DMESAPROC', 'PFNGLWINDOWPOS2DVMESAPROC',
'PFNGLWINDOWPOS2FMESAPROC', 'PFNGLWINDOWPOS2FVMESAPROC',
'PFNGLWINDOWPOS2IMESAPROC', 'PFNGLWINDOWPOS2IVMESAPROC',
'PFNGLWINDOWPOS2SMESAPROC', 'PFNGLWINDOWPOS2SVMESAPROC',
'PFNGLWINDOWPOS3DMESAPROC', 'PFNGLWINDOWPOS3DVMESAPROC',
'PFNGLWINDOWPOS3FMESAPROC', 'PFNGLWINDOWPOS3FVMESAPROC',
'PFNGLWINDOWPOS3IMESAPROC', 'PFNGLWINDOWPOS3IVMESAPROC',
'PFNGLWINDOWPOS3SMESAPROC', 'PFNGLWINDOWPOS3SVMESAPROC',
'PFNGLWINDOWPOS4DMESAPROC', 'PFNGLWINDOWPOS4DVMESAPROC',
'PFNGLWINDOWPOS4FMESAPROC', 'PFNGLWINDOWPOS4FVMESAPROC',
'PFNGLWINDOWPOS4IMESAPROC', 'PFNGLWINDOWPOS4IVMESAPROC',
'PFNGLWINDOWPOS4SMESAPROC', 'PFNGLWINDOWPOS4SVMESAPROC', 'GL_IBM_cull_vertex',
'GL_IBM_multimode_draw_arrays', 'glMultiModeDrawArraysIBM',
'glMultiModeDrawElementsIBM', 'PFNGLMULTIMODEDRAWARRAYSIBMPROC',
'PFNGLMULTIMODEDRAWELEMENTSIBMPROC', 'GL_IBM_vertex_array_lists',
'glColorPointerListIBM', 'glSecondaryColorPointerListIBM',
'glEdgeFlagPointerListIBM', 'glFogCoordPointerListIBM',
'glIndexPointerListIBM', 'glNormalPointerListIBM', 'glTexCoordPointerListIBM',
'glVertexPointerListIBM', 'PFNGLCOLORPOINTERLISTIBMPROC',
'PFNGLSECONDARYCOLORPOINTERLISTIBMPROC', 'PFNGLEDGEFLAGPOINTERLISTIBMPROC',
'PFNGLFOGCOORDPOINTERLISTIBMPROC', 'PFNGLINDEXPOINTERLISTIBMPROC',
'PFNGLNORMALPOINTERLISTIBMPROC', 'PFNGLTEXCOORDPOINTERLISTIBMPROC',
'PFNGLVERTEXPOINTERLISTIBMPROC', 'GL_SGIX_subsample', 'GL_SGIX_ycrcba',
'GL_SGIX_ycrcb_subsample', 'GL_SGIX_depth_pass_instrument',
'GL_3DFX_texture_compression_FXT1', 'GL_3DFX_multisample', 'GL_3DFX_tbuffer',
'glTbufferMask3DFX', 'PFNGLTBUFFERMASK3DFXPROC', 'GL_EXT_multisample',
'glSampleMaskEXT', 'glSamplePatternEXT', 'PFNGLSAMPLEMASKEXTPROC',
'PFNGLSAMPLEPATTERNEXTPROC', 'GL_SGIX_vertex_preclip',
'GL_SGIX_convolution_accuracy', 'GL_SGIX_resample',
'GL_SGIS_point_line_texgen', 'GL_SGIS_texture_color_mask',
'glTextureColorMaskSGIS', 'PFNGLTEXTURECOLORMASKSGISPROC',
'GL_SGIX_igloo_interface', 'glIglooInterfaceSGIX',
'PFNGLIGLOOINTERFACESGIXPROC', 'GL_EXT_texture_env_dot3',
'GL_ATI_texture_mirror_once', 'GL_NV_fence', 'glDeleteFencesNV',
'glGenFencesNV', 'glIsFenceNV', 'glTestFenceNV', 'glGetFenceivNV',
'glFinishFenceNV', 'glSetFenceNV', 'PFNGLDELETEFENCESNVPROC',
'PFNGLGENFENCESNVPROC', 'PFNGLISFENCENVPROC', 'PFNGLTESTFENCENVPROC',
'PFNGLGETFENCEIVNVPROC', 'PFNGLFINISHFENCENVPROC', 'PFNGLSETFENCENVPROC',
'GL_NV_evaluators', 'glMapControlPointsNV', 'glMapParameterivNV',
'glMapParameterfvNV', 'glGetMapControlPointsNV', 'glGetMapParameterivNV',
'glGetMapParameterfvNV', 'glGetMapAttribParameterivNV',
'glGetMapAttribParameterfvNV', 'glEvalMapsNV', 'PFNGLMAPCONTROLPOINTSNVPROC',
'PFNGLMAPPARAMETERIVNVPROC', 'PFNGLMAPPARAMETERFVNVPROC',
'PFNGLGETMAPCONTROLPOINTSNVPROC', 'PFNGLGETMAPPARAMETERIVNVPROC',
'PFNGLGETMAPPARAMETERFVNVPROC', 'PFNGLGETMAPATTRIBPARAMETERIVNVPROC',
'PFNGLGETMAPATTRIBPARAMETERFVNVPROC', 'PFNGLEVALMAPSNVPROC',
'GL_NV_packed_depth_stencil', 'GL_NV_register_combiners2',
'glCombinerStageParameterfvNV', 'glGetCombinerStageParameterfvNV',
'PFNGLCOMBINERSTAGEPARAMETERFVNVPROC',
'PFNGLGETCOMBINERSTAGEPARAMETERFVNVPROC', 'GL_NV_texture_compression_vtc',
'GL_NV_texture_rectangle', 'GL_NV_texture_shader', 'GL_NV_texture_shader2',
'GL_NV_vertex_array_range2', 'GL_NV_vertex_program',
'glAreProgramsResidentNV', 'glBindProgramNV', 'glDeleteProgramsNV',
'glExecuteProgramNV', 'glGenProgramsNV', 'glGetProgramParameterdvNV',
'glGetProgramParameterfvNV', 'glGetProgramivNV', 'glGetProgramStringNV',
'glGetTrackMatrixivNV', 'glGetVertexAttribdvNV', 'glGetVertexAttribfvNV',
'glGetVertexAttribivNV', 'glGetVertexAttribPointervNV', 'glIsProgramNV',
'glLoadProgramNV', 'glProgramParameter4dNV', 'glProgramParameter4dvNV',
'glProgramParameter4fNV', 'glProgramParameter4fvNV',
'glProgramParameters4dvNV', 'glProgramParameters4fvNV',
'glRequestResidentProgramsNV', 'glTrackMatrixNV', 'glVertexAttribPointerNV',
'glVertexAttrib1dNV', 'glVertexAttrib1dvNV', 'glVertexAttrib1fNV',
'glVertexAttrib1fvNV', 'glVertexAttrib1sNV', 'glVertexAttrib1svNV',
'glVertexAttrib2dNV', 'glVertexAttrib2dvNV', 'glVertexAttrib2fNV',
'glVertexAttrib2fvNV', 'glVertexAttrib2sNV', 'glVertexAttrib2svNV',
'glVertexAttrib3dNV', 'glVertexAttrib3dvNV', 'glVertexAttrib3fNV',
'glVertexAttrib3fvNV', 'glVertexAttrib3sNV', 'glVertexAttrib3svNV',
'glVertexAttrib4dNV', 'glVertexAttrib4dvNV', 'glVertexAttrib4fNV',
'glVertexAttrib4fvNV', 'glVertexAttrib4sNV', 'glVertexAttrib4svNV',
'glVertexAttrib4ubNV', 'glVertexAttrib4ubvNV', 'glVertexAttribs1dvNV',
'glVertexAttribs1fvNV', 'glVertexAttribs1svNV', 'glVertexAttribs2dvNV',
'glVertexAttribs2fvNV', 'glVertexAttribs2svNV', 'glVertexAttribs3dvNV',
'glVertexAttribs3fvNV', 'glVertexAttribs3svNV', 'glVertexAttribs4dvNV',
'glVertexAttribs4fvNV', 'glVertexAttribs4svNV', 'glVertexAttribs4ubvNV',
'PFNGLAREPROGRAMSRESIDENTNVPROC', 'PFNGLBINDPROGRAMNVPROC',
'PFNGLDELETEPROGRAMSNVPROC', 'PFNGLEXECUTEPROGRAMNVPROC',
'PFNGLGENPROGRAMSNVPROC', 'PFNGLGETPROGRAMPARAMETERDVNVPROC',
'PFNGLGETPROGRAMPARAMETERFVNVPROC', 'PFNGLGETPROGRAMIVNVPROC',
'PFNGLGETPROGRAMSTRINGNVPROC', 'PFNGLGETTRACKMATRIXIVNVPROC',
'PFNGLGETVERTEXATTRIBDVNVPROC', 'PFNGLGETVERTEXATTRIBFVNVPROC',
'PFNGLGETVERTEXATTRIBIVNVPROC', 'PFNGLGETVERTEXATTRIBPOINTERVNVPROC',
'PFNGLISPROGRAMNVPROC', 'PFNGLLOADPROGRAMNVPROC',
'PFNGLPROGRAMPARAMETER4DNVPROC', 'PFNGLPROGRAMPARAMETER4DVNVPROC',
'PFNGLPROGRAMPARAMETER4FNVPROC', 'PFNGLPROGRAMPARAMETER4FVNVPROC',
'PFNGLPROGRAMPARAMETERS4DVNVPROC', 'PFNGLPROGRAMPARAMETERS4FVNVPROC',
'PFNGLREQUESTRESIDENTPROGRAMSNVPROC', 'PFNGLTRACKMATRIXNVPROC',
'PFNGLVERTEXATTRIBPOINTERNVPROC', 'PFNGLVERTEXATTRIB1DNVPROC',
'PFNGLVERTEXATTRIB1DVNVPROC', 'PFNGLVERTEXATTRIB1FNVPROC',
'PFNGLVERTEXATTRIB1FVNVPROC', 'PFNGLVERTEXATTRIB1SNVPROC',
'PFNGLVERTEXATTRIB1SVNVPROC', 'PFNGLVERTEXATTRIB2DNVPROC',
'PFNGLVERTEXATTRIB2DVNVPROC', 'PFNGLVERTEXATTRIB2FNVPROC',
'PFNGLVERTEXATTRIB2FVNVPROC', 'PFNGLVERTEXATTRIB2SNVPROC',
'PFNGLVERTEXATTRIB2SVNVPROC', 'PFNGLVERTEXATTRIB3DNVPROC',
'PFNGLVERTEXATTRIB3DVNVPROC', 'PFNGLVERTEXATTRIB3FNVPROC',
'PFNGLVERTEXATTRIB3FVNVPROC', 'PFNGLVERTEXATTRIB3SNVPROC',
'PFNGLVERTEXATTRIB3SVNVPROC', 'PFNGLVERTEXATTRIB4DNVPROC',
'PFNGLVERTEXATTRIB4DVNVPROC', 'PFNGLVERTEXATTRIB4FNVPROC',
'PFNGLVERTEXATTRIB4FVNVPROC', 'PFNGLVERTEXATTRIB4SNVPROC',
'PFNGLVERTEXATTRIB4SVNVPROC', 'PFNGLVERTEXATTRIB4UBNVPROC',
'PFNGLVERTEXATTRIB4UBVNVPROC', 'PFNGLVERTEXATTRIBS1DVNVPROC',
'PFNGLVERTEXATTRIBS1FVNVPROC', 'PFNGLVERTEXATTRIBS1SVNVPROC',
'PFNGLVERTEXATTRIBS2DVNVPROC', 'PFNGLVERTEXATTRIBS2FVNVPROC',
'PFNGLVERTEXATTRIBS2SVNVPROC', 'PFNGLVERTEXATTRIBS3DVNVPROC',
'PFNGLVERTEXATTRIBS3FVNVPROC', 'PFNGLVERTEXATTRIBS3SVNVPROC',
'PFNGLVERTEXATTRIBS4DVNVPROC', 'PFNGLVERTEXATTRIBS4FVNVPROC',
'PFNGLVERTEXATTRIBS4SVNVPROC', 'PFNGLVERTEXATTRIBS4UBVNVPROC',
'GL_SGIX_texture_coordinate_clamp', 'GL_SGIX_scalebias_hint',
'GL_OML_interlace', 'GL_OML_subsample', 'GL_OML_resample',
'GL_NV_copy_depth_to_color', 'GL_ATI_envmap_bumpmap',
'glTexBumpParameterivATI', 'glTexBumpParameterfvATI',
'glGetTexBumpParameterivATI', 'glGetTexBumpParameterfvATI',
'PFNGLTEXBUMPPARAMETERIVATIPROC', 'PFNGLTEXBUMPPARAMETERFVATIPROC',
'PFNGLGETTEXBUMPPARAMETERIVATIPROC', 'PFNGLGETTEXBUMPPARAMETERFVATIPROC',
'GL_ATI_fragment_shader', 'glGenFragmentShadersATI',
'glBindFragmentShaderATI', 'glDeleteFragmentShaderATI',
'glBeginFragmentShaderATI', 'glEndFragmentShaderATI', 'glPassTexCoordATI',
'glSampleMapATI', 'glColorFragmentOp1ATI', 'glColorFragmentOp2ATI',
'glColorFragmentOp3ATI', 'glAlphaFragmentOp1ATI', 'glAlphaFragmentOp2ATI',
'glAlphaFragmentOp3ATI', 'glSetFragmentShaderConstantATI',
'PFNGLGENFRAGMENTSHADERSATIPROC', 'PFNGLBINDFRAGMENTSHADERATIPROC',
'PFNGLDELETEFRAGMENTSHADERATIPROC', 'PFNGLBEGINFRAGMENTSHADERATIPROC',
'PFNGLENDFRAGMENTSHADERATIPROC', 'PFNGLPASSTEXCOORDATIPROC',
'PFNGLSAMPLEMAPATIPROC', 'PFNGLCOLORFRAGMENTOP1ATIPROC',
'PFNGLCOLORFRAGMENTOP2ATIPROC', 'PFNGLCOLORFRAGMENTOP3ATIPROC',
'PFNGLALPHAFRAGMENTOP1ATIPROC', 'PFNGLALPHAFRAGMENTOP2ATIPROC',
'PFNGLALPHAFRAGMENTOP3ATIPROC', 'PFNGLSETFRAGMENTSHADERCONSTANTATIPROC',
'GL_ATI_pn_triangles', 'glPNTrianglesiATI', 'glPNTrianglesfATI',
'PFNGLPNTRIANGLESIATIPROC', 'PFNGLPNTRIANGLESFATIPROC',
'GL_ATI_vertex_array_object', 'glNewObjectBufferATI', 'glIsObjectBufferATI',
'glUpdateObjectBufferATI', 'glGetObjectBufferfvATI', 'glGetObjectBufferivATI',
'glFreeObjectBufferATI', 'glArrayObjectATI', 'glGetArrayObjectfvATI',
'glGetArrayObjectivATI', 'glVariantArrayObjectATI',
'glGetVariantArrayObjectfvATI', 'glGetVariantArrayObjectivATI',
'PFNGLNEWOBJECTBUFFERATIPROC', 'PFNGLISOBJECTBUFFERATIPROC',
'PFNGLUPDATEOBJECTBUFFERATIPROC', 'PFNGLGETOBJECTBUFFERFVATIPROC',
'PFNGLGETOBJECTBUFFERIVATIPROC', 'PFNGLFREEOBJECTBUFFERATIPROC',
'PFNGLARRAYOBJECTATIPROC', 'PFNGLGETARRAYOBJECTFVATIPROC',
'PFNGLGETARRAYOBJECTIVATIPROC', 'PFNGLVARIANTARRAYOBJECTATIPROC',
'PFNGLGETVARIANTARRAYOBJECTFVATIPROC', 'PFNGLGETVARIANTARRAYOBJECTIVATIPROC',
'GL_EXT_vertex_shader', 'glBeginVertexShaderEXT', 'glEndVertexShaderEXT',
'glBindVertexShaderEXT', 'glGenVertexShadersEXT', 'glDeleteVertexShaderEXT',
'glShaderOp1EXT', 'glShaderOp2EXT', 'glShaderOp3EXT', 'glSwizzleEXT',
'glWriteMaskEXT', 'glInsertComponentEXT', 'glExtractComponentEXT',
'glGenSymbolsEXT', 'glSetInvariantEXT', 'glSetLocalConstantEXT',
'glVariantbvEXT', 'glVariantsvEXT', 'glVariantivEXT', 'glVariantfvEXT',
'glVariantdvEXT', 'glVariantubvEXT', 'glVariantusvEXT', 'glVariantuivEXT',
'glVariantPointerEXT', 'glEnableVariantClientStateEXT',
'glDisableVariantClientStateEXT', 'glBindLightParameterEXT',
'glBindMaterialParameterEXT', 'glBindTexGenParameterEXT',
'glBindTextureUnitParameterEXT', 'glBindParameterEXT',
'glIsVariantEnabledEXT', 'glGetVariantBooleanvEXT', 'glGetVariantIntegervEXT',
'glGetVariantFloatvEXT', 'glGetVariantPointervEXT',
'glGetInvariantBooleanvEXT', 'glGetInvariantIntegervEXT',
'glGetInvariantFloatvEXT', 'glGetLocalConstantBooleanvEXT',
'glGetLocalConstantIntegervEXT', 'glGetLocalConstantFloatvEXT',
'PFNGLBEGINVERTEXSHADEREXTPROC', 'PFNGLENDVERTEXSHADEREXTPROC',
'PFNGLBINDVERTEXSHADEREXTPROC', 'PFNGLGENVERTEXSHADERSEXTPROC',
'PFNGLDELETEVERTEXSHADEREXTPROC', 'PFNGLSHADEROP1EXTPROC',
'PFNGLSHADEROP2EXTPROC', 'PFNGLSHADEROP3EXTPROC', 'PFNGLSWIZZLEEXTPROC',
'PFNGLWRITEMASKEXTPROC', 'PFNGLINSERTCOMPONENTEXTPROC',
'PFNGLEXTRACTCOMPONENTEXTPROC', 'PFNGLGENSYMBOLSEXTPROC',
'PFNGLSETINVARIANTEXTPROC', 'PFNGLSETLOCALCONSTANTEXTPROC',
'PFNGLVARIANTBVEXTPROC', 'PFNGLVARIANTSVEXTPROC', 'PFNGLVARIANTIVEXTPROC',
'PFNGLVARIANTFVEXTPROC', 'PFNGLVARIANTDVEXTPROC', 'PFNGLVARIANTUBVEXTPROC',
'PFNGLVARIANTUSVEXTPROC', 'PFNGLVARIANTUIVEXTPROC',
'PFNGLVARIANTPOINTEREXTPROC', 'PFNGLENABLEVARIANTCLIENTSTATEEXTPROC',
'PFNGLDISABLEVARIANTCLIENTSTATEEXTPROC', 'PFNGLBINDLIGHTPARAMETEREXTPROC',
'PFNGLBINDMATERIALPARAMETEREXTPROC', 'PFNGLBINDTEXGENPARAMETEREXTPROC',
'PFNGLBINDTEXTUREUNITPARAMETEREXTPROC', 'PFNGLBINDPARAMETEREXTPROC',
'PFNGLISVARIANTENABLEDEXTPROC', 'PFNGLGETVARIANTBOOLEANVEXTPROC',
'PFNGLGETVARIANTINTEGERVEXTPROC', 'PFNGLGETVARIANTFLOATVEXTPROC',
'PFNGLGETVARIANTPOINTERVEXTPROC', 'PFNGLGETINVARIANTBOOLEANVEXTPROC',
'PFNGLGETINVARIANTINTEGERVEXTPROC', 'PFNGLGETINVARIANTFLOATVEXTPROC',
'PFNGLGETLOCALCONSTANTBOOLEANVEXTPROC',
'PFNGLGETLOCALCONSTANTINTEGERVEXTPROC', 'PFNGLGETLOCALCONSTANTFLOATVEXTPROC',
'GL_ATI_vertex_streams', 'glVertexStream1sATI', 'glVertexStream1svATI',
'glVertexStream1iATI', 'glVertexStream1ivATI', 'glVertexStream1fATI',
'glVertexStream1fvATI', 'glVertexStream1dATI', 'glVertexStream1dvATI',
'glVertexStream2sATI', 'glVertexStream2svATI', 'glVertexStream2iATI',
'glVertexStream2ivATI', 'glVertexStream2fATI', 'glVertexStream2fvATI',
'glVertexStream2dATI', 'glVertexStream2dvATI', 'glVertexStream3sATI',
'glVertexStream3svATI', 'glVertexStream3iATI', 'glVertexStream3ivATI',
'glVertexStream3fATI', 'glVertexStream3fvATI', 'glVertexStream3dATI',
'glVertexStream3dvATI', 'glVertexStream4sATI', 'glVertexStream4svATI',
'glVertexStream4iATI', 'glVertexStream4ivATI', 'glVertexStream4fATI',
'glVertexStream4fvATI', 'glVertexStream4dATI', 'glVertexStream4dvATI',
'glNormalStream3bATI', 'glNormalStream3bvATI', 'glNormalStream3sATI',
'glNormalStream3svATI', 'glNormalStream3iATI', 'glNormalStream3ivATI',
'glNormalStream3fATI', 'glNormalStream3fvATI', 'glNormalStream3dATI',
'glNormalStream3dvATI', 'glClientActiveVertexStreamATI',
'glVertexBlendEnviATI', 'glVertexBlendEnvfATI', 'PFNGLVERTEXSTREAM1SATIPROC',
'PFNGLVERTEXSTREAM1SVATIPROC', 'PFNGLVERTEXSTREAM1IATIPROC',
'PFNGLVERTEXSTREAM1IVATIPROC', 'PFNGLVERTEXSTREAM1FATIPROC',
'PFNGLVERTEXSTREAM1FVATIPROC', 'PFNGLVERTEXSTREAM1DATIPROC',
'PFNGLVERTEXSTREAM1DVATIPROC', 'PFNGLVERTEXSTREAM2SATIPROC',
'PFNGLVERTEXSTREAM2SVATIPROC', 'PFNGLVERTEXSTREAM2IATIPROC',
'PFNGLVERTEXSTREAM2IVATIPROC', 'PFNGLVERTEXSTREAM2FATIPROC',
'PFNGLVERTEXSTREAM2FVATIPROC', 'PFNGLVERTEXSTREAM2DATIPROC',
'PFNGLVERTEXSTREAM2DVATIPROC', 'PFNGLVERTEXSTREAM3SATIPROC',
'PFNGLVERTEXSTREAM3SVATIPROC', 'PFNGLVERTEXSTREAM3IATIPROC',
'PFNGLVERTEXSTREAM3IVATIPROC', 'PFNGLVERTEXSTREAM3FATIPROC',
'PFNGLVERTEXSTREAM3FVATIPROC', 'PFNGLVERTEXSTREAM3DATIPROC',
'PFNGLVERTEXSTREAM3DVATIPROC', 'PFNGLVERTEXSTREAM4SATIPROC',
'PFNGLVERTEXSTREAM4SVATIPROC', 'PFNGLVERTEXSTREAM4IATIPROC',
'PFNGLVERTEXSTREAM4IVATIPROC', 'PFNGLVERTEXSTREAM4FATIPROC',
'PFNGLVERTEXSTREAM4FVATIPROC', 'PFNGLVERTEXSTREAM4DATIPROC',
'PFNGLVERTEXSTREAM4DVATIPROC', 'PFNGLNORMALSTREAM3BATIPROC',
'PFNGLNORMALSTREAM3BVATIPROC', 'PFNGLNORMALSTREAM3SATIPROC',
'PFNGLNORMALSTREAM3SVATIPROC', 'PFNGLNORMALSTREAM3IATIPROC',
'PFNGLNORMALSTREAM3IVATIPROC', 'PFNGLNORMALSTREAM3FATIPROC',
'PFNGLNORMALSTREAM3FVATIPROC', 'PFNGLNORMALSTREAM3DATIPROC',
'PFNGLNORMALSTREAM3DVATIPROC', 'PFNGLCLIENTACTIVEVERTEXSTREAMATIPROC',
'PFNGLVERTEXBLENDENVIATIPROC', 'PFNGLVERTEXBLENDENVFATIPROC',
'GL_ATI_element_array', 'glElementPointerATI', 'glDrawElementArrayATI',
'glDrawRangeElementArrayATI', 'PFNGLELEMENTPOINTERATIPROC',
'PFNGLDRAWELEMENTARRAYATIPROC', 'PFNGLDRAWRANGEELEMENTARRAYATIPROC',
'GL_SUN_mesh_array', 'glDrawMeshArraysSUN', 'PFNGLDRAWMESHARRAYSSUNPROC',
'GL_SUN_slice_accum', 'GL_NV_multisample_filter_hint', 'GL_NV_depth_clamp',
'GL_NV_occlusion_query', 'glGenOcclusionQueriesNV',
'glDeleteOcclusionQueriesNV', 'glIsOcclusionQueryNV',
'glBeginOcclusionQueryNV', 'glEndOcclusionQueryNV', 'glGetOcclusionQueryivNV',
'glGetOcclusionQueryuivNV', 'PFNGLGENOCCLUSIONQUERIESNVPROC',
'PFNGLDELETEOCCLUSIONQUERIESNVPROC', 'PFNGLISOCCLUSIONQUERYNVPROC',
'PFNGLBEGINOCCLUSIONQUERYNVPROC', 'PFNGLENDOCCLUSIONQUERYNVPROC',
'PFNGLGETOCCLUSIONQUERYIVNVPROC', 'PFNGLGETOCCLUSIONQUERYUIVNVPROC',
'GL_NV_point_sprite', 'glPointParameteriNV', 'glPointParameterivNV',
'PFNGLPOINTPARAMETERINVPROC', 'PFNGLPOINTPARAMETERIVNVPROC',
'GL_NV_texture_shader3', 'GL_NV_vertex_program1_1', 'GL_EXT_shadow_funcs',
'GL_EXT_stencil_two_side', 'glActiveStencilFaceEXT',
'PFNGLACTIVESTENCILFACEEXTPROC', 'GL_ATI_text_fragment_shader',
'GL_APPLE_client_storage', 'GL_APPLE_element_array', 'glElementPointerAPPLE',
'glDrawElementArrayAPPLE', 'glDrawRangeElementArrayAPPLE',
'glMultiDrawElementArrayAPPLE', 'glMultiDrawRangeElementArrayAPPLE',
'PFNGLELEMENTPOINTERAPPLEPROC', 'PFNGLDRAWELEMENTARRAYAPPLEPROC',
'PFNGLDRAWRANGEELEMENTARRAYAPPLEPROC', 'PFNGLMULTIDRAWELEMENTARRAYAPPLEPROC',
'PFNGLMULTIDRAWRANGEELEMENTARRAYAPPLEPROC', 'GL_APPLE_fence',
'glGenFencesAPPLE', 'glDeleteFencesAPPLE', 'glSetFenceAPPLE',
'glIsFenceAPPLE', 'glTestFenceAPPLE', 'glFinishFenceAPPLE',
'glTestObjectAPPLE', 'glFinishObjectAPPLE', 'PFNGLGENFENCESAPPLEPROC',
'PFNGLDELETEFENCESAPPLEPROC', 'PFNGLSETFENCEAPPLEPROC',
'PFNGLISFENCEAPPLEPROC', 'PFNGLTESTFENCEAPPLEPROC',
'PFNGLFINISHFENCEAPPLEPROC', 'PFNGLTESTOBJECTAPPLEPROC',
'PFNGLFINISHOBJECTAPPLEPROC', 'GL_APPLE_vertex_array_object',
'glBindVertexArrayAPPLE', 'glDeleteVertexArraysAPPLE',
'glGenVertexArraysAPPLE', 'glIsVertexArrayAPPLE',
'PFNGLBINDVERTEXARRAYAPPLEPROC', 'PFNGLDELETEVERTEXARRAYSAPPLEPROC',
'PFNGLGENVERTEXARRAYSAPPLEPROC', 'PFNGLISVERTEXARRAYAPPLEPROC',
'GL_APPLE_vertex_array_range', 'glVertexArrayRangeAPPLE',
'glFlushVertexArrayRangeAPPLE', 'glVertexArrayParameteriAPPLE',
'PFNGLVERTEXARRAYRANGEAPPLEPROC', 'PFNGLFLUSHVERTEXARRAYRANGEAPPLEPROC',
'PFNGLVERTEXARRAYPARAMETERIAPPLEPROC', 'GL_APPLE_ycbcr_422', 'GL_S3_s3tc',
'GL_ATI_draw_buffers', 'glDrawBuffersATI', 'PFNGLDRAWBUFFERSATIPROC',
'GL_ATI_pixel_format_float', 'GL_ATI_texture_env_combine3',
'GL_ATI_texture_float', 'GL_NV_float_buffer', 'GL_NV_fragment_program',
'glProgramNamedParameter4fNV', 'glProgramNamedParameter4dNV',
'glProgramNamedParameter4fvNV', 'glProgramNamedParameter4dvNV',
'glGetProgramNamedParameterfvNV', 'glGetProgramNamedParameterdvNV',
'PFNGLPROGRAMNAMEDPARAMETER4FNVPROC', 'PFNGLPROGRAMNAMEDPARAMETER4DNVPROC',
'PFNGLPROGRAMNAMEDPARAMETER4FVNVPROC', 'PFNGLPROGRAMNAMEDPARAMETER4DVNVPROC',
'PFNGLGETPROGRAMNAMEDPARAMETERFVNVPROC',
'PFNGLGETPROGRAMNAMEDPARAMETERDVNVPROC', 'GL_NV_half_float', 'glVertex2hNV',
'glVertex2hvNV', 'glVertex3hNV', 'glVertex3hvNV', 'glVertex4hNV',
'glVertex4hvNV', 'glNormal3hNV', 'glNormal3hvNV', 'glColor3hNV',
'glColor3hvNV', 'glColor4hNV', 'glColor4hvNV', 'glTexCoord1hNV',
'glTexCoord1hvNV', 'glTexCoord2hNV', 'glTexCoord2hvNV', 'glTexCoord3hNV',
'glTexCoord3hvNV', 'glTexCoord4hNV', 'glTexCoord4hvNV', 'glMultiTexCoord1hNV',
'glMultiTexCoord1hvNV', 'glMultiTexCoord2hNV', 'glMultiTexCoord2hvNV',
'glMultiTexCoord3hNV', 'glMultiTexCoord3hvNV', 'glMultiTexCoord4hNV',
'glMultiTexCoord4hvNV', 'glFogCoordhNV', 'glFogCoordhvNV',
'glSecondaryColor3hNV', 'glSecondaryColor3hvNV', 'glVertexWeighthNV',
'glVertexWeighthvNV', 'glVertexAttrib1hNV', 'glVertexAttrib1hvNV',
'glVertexAttrib2hNV', 'glVertexAttrib2hvNV', 'glVertexAttrib3hNV',
'glVertexAttrib3hvNV', 'glVertexAttrib4hNV', 'glVertexAttrib4hvNV',
'glVertexAttribs1hvNV', 'glVertexAttribs2hvNV', 'glVertexAttribs3hvNV',
'glVertexAttribs4hvNV', 'PFNGLVERTEX2HNVPROC', 'PFNGLVERTEX2HVNVPROC',
'PFNGLVERTEX3HNVPROC', 'PFNGLVERTEX3HVNVPROC', 'PFNGLVERTEX4HNVPROC',
'PFNGLVERTEX4HVNVPROC', 'PFNGLNORMAL3HNVPROC', 'PFNGLNORMAL3HVNVPROC',
'PFNGLCOLOR3HNVPROC', 'PFNGLCOLOR3HVNVPROC', 'PFNGLCOLOR4HNVPROC',
'PFNGLCOLOR4HVNVPROC', 'PFNGLTEXCOORD1HNVPROC', 'PFNGLTEXCOORD1HVNVPROC',
'PFNGLTEXCOORD2HNVPROC', 'PFNGLTEXCOORD2HVNVPROC', 'PFNGLTEXCOORD3HNVPROC',
'PFNGLTEXCOORD3HVNVPROC', 'PFNGLTEXCOORD4HNVPROC', 'PFNGLTEXCOORD4HVNVPROC',
'PFNGLMULTITEXCOORD1HNVPROC', 'PFNGLMULTITEXCOORD1HVNVPROC',
'PFNGLMULTITEXCOORD2HNVPROC', 'PFNGLMULTITEXCOORD2HVNVPROC',
'PFNGLMULTITEXCOORD3HNVPROC', 'PFNGLMULTITEXCOORD3HVNVPROC',
'PFNGLMULTITEXCOORD4HNVPROC', 'PFNGLMULTITEXCOORD4HVNVPROC',
'PFNGLFOGCOORDHNVPROC', 'PFNGLFOGCOORDHVNVPROC',
'PFNGLSECONDARYCOLOR3HNVPROC', 'PFNGLSECONDARYCOLOR3HVNVPROC',
'PFNGLVERTEXWEIGHTHNVPROC', 'PFNGLVERTEXWEIGHTHVNVPROC',
'PFNGLVERTEXATTRIB1HNVPROC', 'PFNGLVERTEXATTRIB1HVNVPROC',
'PFNGLVERTEXATTRIB2HNVPROC', 'PFNGLVERTEXATTRIB2HVNVPROC',
'PFNGLVERTEXATTRIB3HNVPROC', 'PFNGLVERTEXATTRIB3HVNVPROC',
'PFNGLVERTEXATTRIB4HNVPROC', 'PFNGLVERTEXATTRIB4HVNVPROC',
'PFNGLVERTEXATTRIBS1HVNVPROC', 'PFNGLVERTEXATTRIBS2HVNVPROC',
'PFNGLVERTEXATTRIBS3HVNVPROC', 'PFNGLVERTEXATTRIBS4HVNVPROC',
'GL_NV_pixel_data_range', 'glPixelDataRangeNV', 'glFlushPixelDataRangeNV',
'PFNGLPIXELDATARANGENVPROC', 'PFNGLFLUSHPIXELDATARANGENVPROC',
'GL_NV_primitive_restart', 'glPrimitiveRestartNV',
'glPrimitiveRestartIndexNV', 'PFNGLPRIMITIVERESTARTNVPROC',
'PFNGLPRIMITIVERESTARTINDEXNVPROC', 'GL_NV_texture_expand_normal',
'GL_NV_vertex_program2', 'GL_ATI_map_object_buffer', 'glMapObjectBufferATI',
'glUnmapObjectBufferATI', 'PFNGLMAPOBJECTBUFFERATIPROC',
'PFNGLUNMAPOBJECTBUFFERATIPROC', 'GL_ATI_separate_stencil',
'glStencilOpSeparateATI', 'glStencilFuncSeparateATI',
'PFNGLSTENCILOPSEPARATEATIPROC', 'PFNGLSTENCILFUNCSEPARATEATIPROC',
'GL_ATI_vertex_attrib_array_object', 'glVertexAttribArrayObjectATI',
'glGetVertexAttribArrayObjectfvATI', 'glGetVertexAttribArrayObjectivATI',
'PFNGLVERTEXATTRIBARRAYOBJECTATIPROC',
'PFNGLGETVERTEXATTRIBARRAYOBJECTFVATIPROC',
'PFNGLGETVERTEXATTRIBARRAYOBJECTIVATIPROC', 'GL_OES_read_format',
'GL_EXT_depth_bounds_test', 'glDepthBoundsEXT', 'PFNGLDEPTHBOUNDSEXTPROC',
'GL_EXT_texture_mirror_clamp', 'GL_EXT_blend_equation_separate',
'glBlendEquationSeparateEXT', 'PFNGLBLENDEQUATIONSEPARATEEXTPROC',
'GL_MESA_pack_invert', 'GL_MESA_ycbcr_texture', 'GL_EXT_pixel_buffer_object',
'GL_NV_fragment_program_option', 'GL_NV_fragment_program2',
'GL_NV_vertex_program2_option', 'GL_NV_vertex_program3',
'GL_EXT_framebuffer_object', 'glIsRenderbufferEXT', 'glBindRenderbufferEXT',
'glDeleteRenderbuffersEXT', 'glGenRenderbuffersEXT',
'glRenderbufferStorageEXT', 'glGetRenderbufferParameterivEXT',
'glIsFramebufferEXT', 'glBindFramebufferEXT', 'glDeleteFramebuffersEXT',
'glGenFramebuffersEXT', 'glCheckFramebufferStatusEXT',
'glFramebufferTexture1DEXT', 'glFramebufferTexture2DEXT',
'glFramebufferTexture3DEXT', 'glFramebufferRenderbufferEXT',
'glGetFramebufferAttachmentParameterivEXT', 'glGenerateMipmapEXT',
'PFNGLISRENDERBUFFEREXTPROC', 'PFNGLBINDRENDERBUFFEREXTPROC',
'PFNGLDELETERENDERBUFFERSEXTPROC', 'PFNGLGENRENDERBUFFERSEXTPROC',
'PFNGLRENDERBUFFERSTORAGEEXTPROC', 'PFNGLGETRENDERBUFFERPARAMETERIVEXTPROC',
'PFNGLISFRAMEBUFFEREXTPROC', 'PFNGLBINDFRAMEBUFFEREXTPROC',
'PFNGLDELETEFRAMEBUFFERSEXTPROC', 'PFNGLGENFRAMEBUFFERSEXTPROC',
'PFNGLCHECKFRAMEBUFFERSTATUSEXTPROC', 'PFNGLFRAMEBUFFERTEXTURE1DEXTPROC',
'PFNGLFRAMEBUFFERTEXTURE2DEXTPROC', 'PFNGLFRAMEBUFFERTEXTURE3DEXTPROC',
'PFNGLFRAMEBUFFERRENDERBUFFEREXTPROC',
'PFNGLGETFRAMEBUFFERATTACHMENTPARAMETERIVEXTPROC',
'PFNGLGENERATEMIPMAPEXTPROC', 'GL_GREMEDY_string_marker',
'glStringMarkerGREMEDY', 'PFNGLSTRINGMARKERGREMEDYPROC']
# END GENERATED CONTENT (do not edit above this line)



