# ----------------------------------------------------------------------------
# pyglet
# Copyright (c) 2006-2007 Alex Holkner
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions 
# are met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above copyright 
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#  * Neither the name of the pyglet nor the names of its
#    contributors may be used to endorse or promote products
#    derived from this software without specific prior written
#    permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# ----------------------------------------------------------------------------
'''Wrapper for http://developer.download.nvidia.com/opengl/includes/glext.h

Generated by tools/gengl.py.
Do not modify this file.
'''

__docformat__ = 'restructuredtext'
__version__ = '$Id: glext_nv.py 878 2007-06-09 04:58:51Z Alex.Holkner $'

from ctypes import *
from pyglet.gl.lib import link_GL as _link_function
from pyglet.gl.lib import c_ptrdiff_t

# BEGIN GENERATED CONTENT (do not edit below this line)

# This content is generated by tools/gengl.py.
# Wrapper for http://developer.download.nvidia.com/opengl/includes/glext.h


# GLEXT_LEGACY (/usr/include/GL/gl.h:70)
GL_GLEXT_VERSION = 28 	# GL/glext.h:74
# VERSION_1_2 (GL/glext.h:76)
GL_UNSIGNED_BYTE_3_3_2 = 32818 	# GL/glext.h:77
GL_UNSIGNED_SHORT_4_4_4_4 = 32819 	# GL/glext.h:78
GL_UNSIGNED_SHORT_5_5_5_1 = 32820 	# GL/glext.h:79
GL_UNSIGNED_INT_8_8_8_8 = 32821 	# GL/glext.h:80
GL_UNSIGNED_INT_10_10_10_2 = 32822 	# GL/glext.h:81
GL_RESCALE_NORMAL = 32826 	# GL/glext.h:82
GL_TEXTURE_BINDING_3D = 32874 	# GL/glext.h:83
GL_PACK_SKIP_IMAGES = 32875 	# GL/glext.h:84
GL_PACK_IMAGE_HEIGHT = 32876 	# GL/glext.h:85
GL_UNPACK_SKIP_IMAGES = 32877 	# GL/glext.h:86
GL_UNPACK_IMAGE_HEIGHT = 32878 	# GL/glext.h:87
GL_TEXTURE_3D = 32879 	# GL/glext.h:88
GL_PROXY_TEXTURE_3D = 32880 	# GL/glext.h:89
GL_TEXTURE_DEPTH = 32881 	# GL/glext.h:90
GL_TEXTURE_WRAP_R = 32882 	# GL/glext.h:91
GL_MAX_3D_TEXTURE_SIZE = 32883 	# GL/glext.h:92
GL_UNSIGNED_BYTE_2_3_3_REV = 33634 	# GL/glext.h:93
GL_UNSIGNED_SHORT_5_6_5 = 33635 	# GL/glext.h:94
GL_UNSIGNED_SHORT_5_6_5_REV = 33636 	# GL/glext.h:95
GL_UNSIGNED_SHORT_4_4_4_4_REV = 33637 	# GL/glext.h:96
GL_UNSIGNED_SHORT_1_5_5_5_REV = 33638 	# GL/glext.h:97
GL_UNSIGNED_INT_8_8_8_8_REV = 33639 	# GL/glext.h:98
GL_UNSIGNED_INT_2_10_10_10_REV = 33640 	# GL/glext.h:99
GL_BGR = 32992 	# GL/glext.h:100
GL_BGRA = 32993 	# GL/glext.h:101
GL_MAX_ELEMENTS_VERTICES = 33000 	# GL/glext.h:102
GL_MAX_ELEMENTS_INDICES = 33001 	# GL/glext.h:103
GL_CLAMP_TO_EDGE = 33071 	# GL/glext.h:104
GL_TEXTURE_MIN_LOD = 33082 	# GL/glext.h:105
GL_TEXTURE_MAX_LOD = 33083 	# GL/glext.h:106
GL_TEXTURE_BASE_LEVEL = 33084 	# GL/glext.h:107
GL_TEXTURE_MAX_LEVEL = 33085 	# GL/glext.h:108
GL_LIGHT_MODEL_COLOR_CONTROL = 33272 	# GL/glext.h:109
GL_SINGLE_COLOR = 33273 	# GL/glext.h:110
GL_SEPARATE_SPECULAR_COLOR = 33274 	# GL/glext.h:111
GL_SMOOTH_POINT_SIZE_RANGE = 2834 	# GL/glext.h:112
GL_SMOOTH_POINT_SIZE_GRANULARITY = 2835 	# GL/glext.h:113
GL_SMOOTH_LINE_WIDTH_RANGE = 2850 	# GL/glext.h:114
GL_SMOOTH_LINE_WIDTH_GRANULARITY = 2851 	# GL/glext.h:115
GL_ALIASED_POINT_SIZE_RANGE = 33901 	# GL/glext.h:116
GL_ALIASED_LINE_WIDTH_RANGE = 33902 	# GL/glext.h:117
# ARB_imaging (GL/glext.h:120)
GL_CONSTANT_COLOR = 32769 	# GL/glext.h:121
GL_ONE_MINUS_CONSTANT_COLOR = 32770 	# GL/glext.h:122
GL_CONSTANT_ALPHA = 32771 	# GL/glext.h:123
GL_ONE_MINUS_CONSTANT_ALPHA = 32772 	# GL/glext.h:124
GL_BLEND_COLOR = 32773 	# GL/glext.h:125
GL_FUNC_ADD = 32774 	# GL/glext.h:126
GL_MIN = 32775 	# GL/glext.h:127
GL_MAX = 32776 	# GL/glext.h:128
GL_BLEND_EQUATION = 32777 	# GL/glext.h:129
GL_FUNC_SUBTRACT = 32778 	# GL/glext.h:130
GL_FUNC_REVERSE_SUBTRACT = 32779 	# GL/glext.h:131
GL_CONVOLUTION_1D = 32784 	# GL/glext.h:132
GL_CONVOLUTION_2D = 32785 	# GL/glext.h:133
GL_SEPARABLE_2D = 32786 	# GL/glext.h:134
GL_CONVOLUTION_BORDER_MODE = 32787 	# GL/glext.h:135
GL_CONVOLUTION_FILTER_SCALE = 32788 	# GL/glext.h:136
GL_CONVOLUTION_FILTER_BIAS = 32789 	# GL/glext.h:137
GL_REDUCE = 32790 	# GL/glext.h:138
GL_CONVOLUTION_FORMAT = 32791 	# GL/glext.h:139
GL_CONVOLUTION_WIDTH = 32792 	# GL/glext.h:140
GL_CONVOLUTION_HEIGHT = 32793 	# GL/glext.h:141
GL_MAX_CONVOLUTION_WIDTH = 32794 	# GL/glext.h:142
GL_MAX_CONVOLUTION_HEIGHT = 32795 	# GL/glext.h:143
GL_POST_CONVOLUTION_RED_SCALE = 32796 	# GL/glext.h:144
GL_POST_CONVOLUTION_GREEN_SCALE = 32797 	# GL/glext.h:145
GL_POST_CONVOLUTION_BLUE_SCALE = 32798 	# GL/glext.h:146
GL_POST_CONVOLUTION_ALPHA_SCALE = 32799 	# GL/glext.h:147
GL_POST_CONVOLUTION_RED_BIAS = 32800 	# GL/glext.h:148
GL_POST_CONVOLUTION_GREEN_BIAS = 32801 	# GL/glext.h:149
GL_POST_CONVOLUTION_BLUE_BIAS = 32802 	# GL/glext.h:150
GL_POST_CONVOLUTION_ALPHA_BIAS = 32803 	# GL/glext.h:151
GL_HISTOGRAM = 32804 	# GL/glext.h:152
GL_PROXY_HISTOGRAM = 32805 	# GL/glext.h:153
GL_HISTOGRAM_WIDTH = 32806 	# GL/glext.h:154
GL_HISTOGRAM_FORMAT = 32807 	# GL/glext.h:155
GL_HISTOGRAM_RED_SIZE = 32808 	# GL/glext.h:156
GL_HISTOGRAM_GREEN_SIZE = 32809 	# GL/glext.h:157
GL_HISTOGRAM_BLUE_SIZE = 32810 	# GL/glext.h:158
GL_HISTOGRAM_ALPHA_SIZE = 32811 	# GL/glext.h:159
GL_HISTOGRAM_LUMINANCE_SIZE = 32812 	# GL/glext.h:160
GL_HISTOGRAM_SINK = 32813 	# GL/glext.h:161
GL_MINMAX = 32814 	# GL/glext.h:162
GL_MINMAX_FORMAT = 32815 	# GL/glext.h:163
GL_MINMAX_SINK = 32816 	# GL/glext.h:164
GL_TABLE_TOO_LARGE = 32817 	# GL/glext.h:165
GL_COLOR_MATRIX = 32945 	# GL/glext.h:166
GL_COLOR_MATRIX_STACK_DEPTH = 32946 	# GL/glext.h:167
GL_MAX_COLOR_MATRIX_STACK_DEPTH = 32947 	# GL/glext.h:168
GL_POST_COLOR_MATRIX_RED_SCALE = 32948 	# GL/glext.h:169
GL_POST_COLOR_MATRIX_GREEN_SCALE = 32949 	# GL/glext.h:170
GL_POST_COLOR_MATRIX_BLUE_SCALE = 32950 	# GL/glext.h:171
GL_POST_COLOR_MATRIX_ALPHA_SCALE = 32951 	# GL/glext.h:172
GL_POST_COLOR_MATRIX_RED_BIAS = 32952 	# GL/glext.h:173
GL_POST_COLOR_MATRIX_GREEN_BIAS = 32953 	# GL/glext.h:174
GL_POST_COLOR_MATRIX_BLUE_BIAS = 32954 	# GL/glext.h:175
GL_POST_COLOR_MATRIX_ALPHA_BIAS = 32955 	# GL/glext.h:176
GL_COLOR_TABLE = 32976 	# GL/glext.h:177
GL_POST_CONVOLUTION_COLOR_TABLE = 32977 	# GL/glext.h:178
GL_POST_COLOR_MATRIX_COLOR_TABLE = 32978 	# GL/glext.h:179
GL_PROXY_COLOR_TABLE = 32979 	# GL/glext.h:180
GL_PROXY_POST_CONVOLUTION_COLOR_TABLE = 32980 	# GL/glext.h:181
GL_PROXY_POST_COLOR_MATRIX_COLOR_TABLE = 32981 	# GL/glext.h:182
GL_COLOR_TABLE_SCALE = 32982 	# GL/glext.h:183
GL_COLOR_TABLE_BIAS = 32983 	# GL/glext.h:184
GL_COLOR_TABLE_FORMAT = 32984 	# GL/glext.h:185
GL_COLOR_TABLE_WIDTH = 32985 	# GL/glext.h:186
GL_COLOR_TABLE_RED_SIZE = 32986 	# GL/glext.h:187
GL_COLOR_TABLE_GREEN_SIZE = 32987 	# GL/glext.h:188
GL_COLOR_TABLE_BLUE_SIZE = 32988 	# GL/glext.h:189
GL_COLOR_TABLE_ALPHA_SIZE = 32989 	# GL/glext.h:190
GL_COLOR_TABLE_LUMINANCE_SIZE = 32990 	# GL/glext.h:191
GL_COLOR_TABLE_INTENSITY_SIZE = 32991 	# GL/glext.h:192
GL_CONSTANT_BORDER = 33105 	# GL/glext.h:193
GL_REPLICATE_BORDER = 33107 	# GL/glext.h:194
GL_CONVOLUTION_BORDER_COLOR = 33108 	# GL/glext.h:195
# VERSION_1_3 (GL/glext.h:198)
GL_TEXTURE0 = 33984 	# GL/glext.h:199
GL_TEXTURE1 = 33985 	# GL/glext.h:200
GL_TEXTURE2 = 33986 	# GL/glext.h:201
GL_TEXTURE3 = 33987 	# GL/glext.h:202
GL_TEXTURE4 = 33988 	# GL/glext.h:203
GL_TEXTURE5 = 33989 	# GL/glext.h:204
GL_TEXTURE6 = 33990 	# GL/glext.h:205
GL_TEXTURE7 = 33991 	# GL/glext.h:206
GL_TEXTURE8 = 33992 	# GL/glext.h:207
GL_TEXTURE9 = 33993 	# GL/glext.h:208
GL_TEXTURE10 = 33994 	# GL/glext.h:209
GL_TEXTURE11 = 33995 	# GL/glext.h:210
GL_TEXTURE12 = 33996 	# GL/glext.h:211
GL_TEXTURE13 = 33997 	# GL/glext.h:212
GL_TEXTURE14 = 33998 	# GL/glext.h:213
GL_TEXTURE15 = 33999 	# GL/glext.h:214
GL_TEXTURE16 = 34000 	# GL/glext.h:215
GL_TEXTURE17 = 34001 	# GL/glext.h:216
GL_TEXTURE18 = 34002 	# GL/glext.h:217
GL_TEXTURE19 = 34003 	# GL/glext.h:218
GL_TEXTURE20 = 34004 	# GL/glext.h:219
GL_TEXTURE21 = 34005 	# GL/glext.h:220
GL_TEXTURE22 = 34006 	# GL/glext.h:221
GL_TEXTURE23 = 34007 	# GL/glext.h:222
GL_TEXTURE24 = 34008 	# GL/glext.h:223
GL_TEXTURE25 = 34009 	# GL/glext.h:224
GL_TEXTURE26 = 34010 	# GL/glext.h:225
GL_TEXTURE27 = 34011 	# GL/glext.h:226
GL_TEXTURE28 = 34012 	# GL/glext.h:227
GL_TEXTURE29 = 34013 	# GL/glext.h:228
GL_TEXTURE30 = 34014 	# GL/glext.h:229
GL_TEXTURE31 = 34015 	# GL/glext.h:230
GL_ACTIVE_TEXTURE = 34016 	# GL/glext.h:231
GL_CLIENT_ACTIVE_TEXTURE = 34017 	# GL/glext.h:232
GL_MAX_TEXTURE_UNITS = 34018 	# GL/glext.h:233
GL_TRANSPOSE_MODELVIEW_MATRIX = 34019 	# GL/glext.h:234
GL_TRANSPOSE_PROJECTION_MATRIX = 34020 	# GL/glext.h:235
GL_TRANSPOSE_TEXTURE_MATRIX = 34021 	# GL/glext.h:236
GL_TRANSPOSE_COLOR_MATRIX = 34022 	# GL/glext.h:237
GL_MULTISAMPLE = 32925 	# GL/glext.h:238
GL_SAMPLE_ALPHA_TO_COVERAGE = 32926 	# GL/glext.h:239
GL_SAMPLE_ALPHA_TO_ONE = 32927 	# GL/glext.h:240
GL_SAMPLE_COVERAGE = 32928 	# GL/glext.h:241
GL_SAMPLE_BUFFERS = 32936 	# GL/glext.h:242
GL_SAMPLES = 32937 	# GL/glext.h:243
GL_SAMPLE_COVERAGE_VALUE = 32938 	# GL/glext.h:244
GL_SAMPLE_COVERAGE_INVERT = 32939 	# GL/glext.h:245
GL_MULTISAMPLE_BIT = 536870912 	# GL/glext.h:246
GL_NORMAL_MAP = 34065 	# GL/glext.h:247
GL_REFLECTION_MAP = 34066 	# GL/glext.h:248
GL_TEXTURE_CUBE_MAP = 34067 	# GL/glext.h:249
GL_TEXTURE_BINDING_CUBE_MAP = 34068 	# GL/glext.h:250
GL_TEXTURE_CUBE_MAP_POSITIVE_X = 34069 	# GL/glext.h:251
GL_TEXTURE_CUBE_MAP_NEGATIVE_X = 34070 	# GL/glext.h:252
GL_TEXTURE_CUBE_MAP_POSITIVE_Y = 34071 	# GL/glext.h:253
GL_TEXTURE_CUBE_MAP_NEGATIVE_Y = 34072 	# GL/glext.h:254
GL_TEXTURE_CUBE_MAP_POSITIVE_Z = 34073 	# GL/glext.h:255
GL_TEXTURE_CUBE_MAP_NEGATIVE_Z = 34074 	# GL/glext.h:256
GL_PROXY_TEXTURE_CUBE_MAP = 34075 	# GL/glext.h:257
GL_MAX_CUBE_MAP_TEXTURE_SIZE = 34076 	# GL/glext.h:258
GL_COMPRESSED_ALPHA = 34025 	# GL/glext.h:259
GL_COMPRESSED_LUMINANCE = 34026 	# GL/glext.h:260
GL_COMPRESSED_LUMINANCE_ALPHA = 34027 	# GL/glext.h:261
GL_COMPRESSED_INTENSITY = 34028 	# GL/glext.h:262
GL_COMPRESSED_RGB = 34029 	# GL/glext.h:263
GL_COMPRESSED_RGBA = 34030 	# GL/glext.h:264
GL_TEXTURE_COMPRESSION_HINT = 34031 	# GL/glext.h:265
GL_TEXTURE_COMPRESSED_IMAGE_SIZE = 34464 	# GL/glext.h:266
GL_TEXTURE_COMPRESSED = 34465 	# GL/glext.h:267
GL_NUM_COMPRESSED_TEXTURE_FORMATS = 34466 	# GL/glext.h:268
GL_COMPRESSED_TEXTURE_FORMATS = 34467 	# GL/glext.h:269
GL_CLAMP_TO_BORDER = 33069 	# GL/glext.h:270
GL_COMBINE = 34160 	# GL/glext.h:271
GL_COMBINE_RGB = 34161 	# GL/glext.h:272
GL_COMBINE_ALPHA = 34162 	# GL/glext.h:273
GL_SOURCE0_RGB = 34176 	# GL/glext.h:274
GL_SOURCE1_RGB = 34177 	# GL/glext.h:275
GL_SOURCE2_RGB = 34178 	# GL/glext.h:276
GL_SOURCE0_ALPHA = 34184 	# GL/glext.h:277
GL_SOURCE1_ALPHA = 34185 	# GL/glext.h:278
GL_SOURCE2_ALPHA = 34186 	# GL/glext.h:279
GL_OPERAND0_RGB = 34192 	# GL/glext.h:280
GL_OPERAND1_RGB = 34193 	# GL/glext.h:281
GL_OPERAND2_RGB = 34194 	# GL/glext.h:282
GL_OPERAND0_ALPHA = 34200 	# GL/glext.h:283
GL_OPERAND1_ALPHA = 34201 	# GL/glext.h:284
GL_OPERAND2_ALPHA = 34202 	# GL/glext.h:285
GL_RGB_SCALE = 34163 	# GL/glext.h:286
GL_ADD_SIGNED = 34164 	# GL/glext.h:287
GL_INTERPOLATE = 34165 	# GL/glext.h:288
GL_SUBTRACT = 34023 	# GL/glext.h:289
GL_CONSTANT = 34166 	# GL/glext.h:290
GL_PRIMARY_COLOR = 34167 	# GL/glext.h:291
GL_PREVIOUS = 34168 	# GL/glext.h:292
GL_DOT3_RGB = 34478 	# GL/glext.h:293
GL_DOT3_RGBA = 34479 	# GL/glext.h:294
# VERSION_1_4 (GL/glext.h:297)
GL_BLEND_DST_RGB = 32968 	# GL/glext.h:298
GL_BLEND_SRC_RGB = 32969 	# GL/glext.h:299
GL_BLEND_DST_ALPHA = 32970 	# GL/glext.h:300
GL_BLEND_SRC_ALPHA = 32971 	# GL/glext.h:301
GL_POINT_SIZE_MIN = 33062 	# GL/glext.h:302
GL_POINT_SIZE_MAX = 33063 	# GL/glext.h:303
GL_POINT_FADE_THRESHOLD_SIZE = 33064 	# GL/glext.h:304
GL_POINT_DISTANCE_ATTENUATION = 33065 	# GL/glext.h:305
GL_GENERATE_MIPMAP = 33169 	# GL/glext.h:306
GL_GENERATE_MIPMAP_HINT = 33170 	# GL/glext.h:307
GL_DEPTH_COMPONENT16 = 33189 	# GL/glext.h:308
GL_DEPTH_COMPONENT24 = 33190 	# GL/glext.h:309
GL_DEPTH_COMPONENT32 = 33191 	# GL/glext.h:310
GL_MIRRORED_REPEAT = 33648 	# GL/glext.h:311
GL_FOG_COORDINATE_SOURCE = 33872 	# GL/glext.h:312
GL_FOG_COORDINATE = 33873 	# GL/glext.h:313
GL_FRAGMENT_DEPTH = 33874 	# GL/glext.h:314
GL_CURRENT_FOG_COORDINATE = 33875 	# GL/glext.h:315
GL_FOG_COORDINATE_ARRAY_TYPE = 33876 	# GL/glext.h:316
GL_FOG_COORDINATE_ARRAY_STRIDE = 33877 	# GL/glext.h:317
GL_FOG_COORDINATE_ARRAY_POINTER = 33878 	# GL/glext.h:318
GL_FOG_COORDINATE_ARRAY = 33879 	# GL/glext.h:319
GL_COLOR_SUM = 33880 	# GL/glext.h:320
GL_CURRENT_SECONDARY_COLOR = 33881 	# GL/glext.h:321
GL_SECONDARY_COLOR_ARRAY_SIZE = 33882 	# GL/glext.h:322
GL_SECONDARY_COLOR_ARRAY_TYPE = 33883 	# GL/glext.h:323
GL_SECONDARY_COLOR_ARRAY_STRIDE = 33884 	# GL/glext.h:324
GL_SECONDARY_COLOR_ARRAY_POINTER = 33885 	# GL/glext.h:325
GL_SECONDARY_COLOR_ARRAY = 33886 	# GL/glext.h:326
GL_MAX_TEXTURE_LOD_BIAS = 34045 	# GL/glext.h:327
GL_TEXTURE_FILTER_CONTROL = 34048 	# GL/glext.h:328
GL_TEXTURE_LOD_BIAS = 34049 	# GL/glext.h:329
GL_INCR_WRAP = 34055 	# GL/glext.h:330
GL_DECR_WRAP = 34056 	# GL/glext.h:331
GL_TEXTURE_DEPTH_SIZE = 34890 	# GL/glext.h:332
GL_DEPTH_TEXTURE_MODE = 34891 	# GL/glext.h:333
GL_TEXTURE_COMPARE_MODE = 34892 	# GL/glext.h:334
GL_TEXTURE_COMPARE_FUNC = 34893 	# GL/glext.h:335
GL_COMPARE_R_TO_TEXTURE = 34894 	# GL/glext.h:336
# VERSION_1_5 (GL/glext.h:339)
GL_BUFFER_SIZE = 34660 	# GL/glext.h:340
GL_BUFFER_USAGE = 34661 	# GL/glext.h:341
GL_QUERY_COUNTER_BITS = 34916 	# GL/glext.h:342
GL_CURRENT_QUERY = 34917 	# GL/glext.h:343
GL_QUERY_RESULT = 34918 	# GL/glext.h:344
GL_QUERY_RESULT_AVAILABLE = 34919 	# GL/glext.h:345
GL_ARRAY_BUFFER = 34962 	# GL/glext.h:346
GL_ELEMENT_ARRAY_BUFFER = 34963 	# GL/glext.h:347
GL_ARRAY_BUFFER_BINDING = 34964 	# GL/glext.h:348
GL_ELEMENT_ARRAY_BUFFER_BINDING = 34965 	# GL/glext.h:349
GL_VERTEX_ARRAY_BUFFER_BINDING = 34966 	# GL/glext.h:350
GL_NORMAL_ARRAY_BUFFER_BINDING = 34967 	# GL/glext.h:351
GL_COLOR_ARRAY_BUFFER_BINDING = 34968 	# GL/glext.h:352
GL_INDEX_ARRAY_BUFFER_BINDING = 34969 	# GL/glext.h:353
GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING = 34970 	# GL/glext.h:354
GL_EDGE_FLAG_ARRAY_BUFFER_BINDING = 34971 	# GL/glext.h:355
GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING = 34972 	# GL/glext.h:356
GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING = 34973 	# GL/glext.h:357
GL_WEIGHT_ARRAY_BUFFER_BINDING = 34974 	# GL/glext.h:358
GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING = 34975 	# GL/glext.h:359
GL_READ_ONLY = 35000 	# GL/glext.h:360
GL_WRITE_ONLY = 35001 	# GL/glext.h:361
GL_READ_WRITE = 35002 	# GL/glext.h:362
GL_BUFFER_ACCESS = 35003 	# GL/glext.h:363
GL_BUFFER_MAPPED = 35004 	# GL/glext.h:364
GL_BUFFER_MAP_POINTER = 35005 	# GL/glext.h:365
GL_STREAM_DRAW = 35040 	# GL/glext.h:366
GL_STREAM_READ = 35041 	# GL/glext.h:367
GL_STREAM_COPY = 35042 	# GL/glext.h:368
GL_STATIC_DRAW = 35044 	# GL/glext.h:369
GL_STATIC_READ = 35045 	# GL/glext.h:370
GL_STATIC_COPY = 35046 	# GL/glext.h:371
GL_DYNAMIC_DRAW = 35048 	# GL/glext.h:372
GL_DYNAMIC_READ = 35049 	# GL/glext.h:373
GL_DYNAMIC_COPY = 35050 	# GL/glext.h:374
GL_SAMPLES_PASSED = 35092 	# GL/glext.h:375
GL_FOG_COORD_SRC = 33872 	# GL/glext.h:376
GL_FOG_COORD = 33873 	# GL/glext.h:377
GL_CURRENT_FOG_COORD = 33875 	# GL/glext.h:378
GL_FOG_COORD_ARRAY_TYPE = 33876 	# GL/glext.h:379
GL_FOG_COORD_ARRAY_STRIDE = 33877 	# GL/glext.h:380
GL_FOG_COORD_ARRAY_POINTER = 33878 	# GL/glext.h:381
GL_FOG_COORD_ARRAY = 33879 	# GL/glext.h:382
GL_FOG_COORD_ARRAY_BUFFER_BINDING = 34973 	# GL/glext.h:383
GL_SRC0_RGB = 34176 	# GL/glext.h:384
GL_SRC1_RGB = 34177 	# GL/glext.h:385
GL_SRC2_RGB = 34178 	# GL/glext.h:386
GL_SRC0_ALPHA = 34184 	# GL/glext.h:387
GL_SRC1_ALPHA = 34185 	# GL/glext.h:388
GL_SRC2_ALPHA = 34186 	# GL/glext.h:389
# VERSION_2_0 (GL/glext.h:392)
GL_BLEND_EQUATION_RGB = 32777 	# GL/glext.h:393
GL_VERTEX_ATTRIB_ARRAY_ENABLED = 34338 	# GL/glext.h:394
GL_VERTEX_ATTRIB_ARRAY_SIZE = 34339 	# GL/glext.h:395
GL_VERTEX_ATTRIB_ARRAY_STRIDE = 34340 	# GL/glext.h:396
GL_VERTEX_ATTRIB_ARRAY_TYPE = 34341 	# GL/glext.h:397
GL_CURRENT_VERTEX_ATTRIB = 34342 	# GL/glext.h:398
GL_VERTEX_PROGRAM_POINT_SIZE = 34370 	# GL/glext.h:399
GL_VERTEX_PROGRAM_TWO_SIDE = 34371 	# GL/glext.h:400
GL_VERTEX_ATTRIB_ARRAY_POINTER = 34373 	# GL/glext.h:401
GL_STENCIL_BACK_FUNC = 34816 	# GL/glext.h:402
GL_STENCIL_BACK_FAIL = 34817 	# GL/glext.h:403
GL_STENCIL_BACK_PASS_DEPTH_FAIL = 34818 	# GL/glext.h:404
GL_STENCIL_BACK_PASS_DEPTH_PASS = 34819 	# GL/glext.h:405
GL_MAX_DRAW_BUFFERS = 34852 	# GL/glext.h:406
GL_DRAW_BUFFER0 = 34853 	# GL/glext.h:407
GL_DRAW_BUFFER1 = 34854 	# GL/glext.h:408
GL_DRAW_BUFFER2 = 34855 	# GL/glext.h:409
GL_DRAW_BUFFER3 = 34856 	# GL/glext.h:410
GL_DRAW_BUFFER4 = 34857 	# GL/glext.h:411
GL_DRAW_BUFFER5 = 34858 	# GL/glext.h:412
GL_DRAW_BUFFER6 = 34859 	# GL/glext.h:413
GL_DRAW_BUFFER7 = 34860 	# GL/glext.h:414
GL_DRAW_BUFFER8 = 34861 	# GL/glext.h:415
GL_DRAW_BUFFER9 = 34862 	# GL/glext.h:416
GL_DRAW_BUFFER10 = 34863 	# GL/glext.h:417
GL_DRAW_BUFFER11 = 34864 	# GL/glext.h:418
GL_DRAW_BUFFER12 = 34865 	# GL/glext.h:419
GL_DRAW_BUFFER13 = 34866 	# GL/glext.h:420
GL_DRAW_BUFFER14 = 34867 	# GL/glext.h:421
GL_DRAW_BUFFER15 = 34868 	# GL/glext.h:422
GL_BLEND_EQUATION_ALPHA = 34877 	# GL/glext.h:423
GL_POINT_SPRITE = 34913 	# GL/glext.h:424
GL_COORD_REPLACE = 34914 	# GL/glext.h:425
GL_MAX_VERTEX_ATTRIBS = 34921 	# GL/glext.h:426
GL_VERTEX_ATTRIB_ARRAY_NORMALIZED = 34922 	# GL/glext.h:427
GL_MAX_TEXTURE_COORDS = 34929 	# GL/glext.h:428
GL_MAX_TEXTURE_IMAGE_UNITS = 34930 	# GL/glext.h:429
GL_FRAGMENT_SHADER = 35632 	# GL/glext.h:430
GL_VERTEX_SHADER = 35633 	# GL/glext.h:431
GL_MAX_FRAGMENT_UNIFORM_COMPONENTS = 35657 	# GL/glext.h:432
GL_MAX_VERTEX_UNIFORM_COMPONENTS = 35658 	# GL/glext.h:433
GL_MAX_VARYING_FLOATS = 35659 	# GL/glext.h:434
GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS = 35660 	# GL/glext.h:435
GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS = 35661 	# GL/glext.h:436
GL_SHADER_TYPE = 35663 	# GL/glext.h:437
GL_FLOAT_VEC2 = 35664 	# GL/glext.h:438
GL_FLOAT_VEC3 = 35665 	# GL/glext.h:439
GL_FLOAT_VEC4 = 35666 	# GL/glext.h:440
GL_INT_VEC2 = 35667 	# GL/glext.h:441
GL_INT_VEC3 = 35668 	# GL/glext.h:442
GL_INT_VEC4 = 35669 	# GL/glext.h:443
GL_BOOL = 35670 	# GL/glext.h:444
GL_BOOL_VEC2 = 35671 	# GL/glext.h:445
GL_BOOL_VEC3 = 35672 	# GL/glext.h:446
GL_BOOL_VEC4 = 35673 	# GL/glext.h:447
GL_FLOAT_MAT2 = 35674 	# GL/glext.h:448
GL_FLOAT_MAT3 = 35675 	# GL/glext.h:449
GL_FLOAT_MAT4 = 35676 	# GL/glext.h:450
GL_SAMPLER_1D = 35677 	# GL/glext.h:451
GL_SAMPLER_2D = 35678 	# GL/glext.h:452
GL_SAMPLER_3D = 35679 	# GL/glext.h:453
GL_SAMPLER_CUBE = 35680 	# GL/glext.h:454
GL_SAMPLER_1D_SHADOW = 35681 	# GL/glext.h:455
GL_SAMPLER_2D_SHADOW = 35682 	# GL/glext.h:456
GL_DELETE_STATUS = 35712 	# GL/glext.h:457
GL_COMPILE_STATUS = 35713 	# GL/glext.h:458
GL_LINK_STATUS = 35714 	# GL/glext.h:459
GL_VALIDATE_STATUS = 35715 	# GL/glext.h:460
GL_INFO_LOG_LENGTH = 35716 	# GL/glext.h:461
GL_ATTACHED_SHADERS = 35717 	# GL/glext.h:462
GL_ACTIVE_UNIFORMS = 35718 	# GL/glext.h:463
GL_ACTIVE_UNIFORM_MAX_LENGTH = 35719 	# GL/glext.h:464
GL_SHADER_SOURCE_LENGTH = 35720 	# GL/glext.h:465
GL_ACTIVE_ATTRIBUTES = 35721 	# GL/glext.h:466
GL_ACTIVE_ATTRIBUTE_MAX_LENGTH = 35722 	# GL/glext.h:467
GL_FRAGMENT_SHADER_DERIVATIVE_HINT = 35723 	# GL/glext.h:468
GL_SHADING_LANGUAGE_VERSION = 35724 	# GL/glext.h:469
GL_CURRENT_PROGRAM = 35725 	# GL/glext.h:470
GL_POINT_SPRITE_COORD_ORIGIN = 36000 	# GL/glext.h:471
GL_LOWER_LEFT = 36001 	# GL/glext.h:472
GL_UPPER_LEFT = 36002 	# GL/glext.h:473
GL_STENCIL_BACK_REF = 36003 	# GL/glext.h:474
GL_STENCIL_BACK_VALUE_MASK = 36004 	# GL/glext.h:475
GL_STENCIL_BACK_WRITEMASK = 36005 	# GL/glext.h:476
# VERSION_2_1 (GL/glext.h:479)
GL_PIXEL_PACK_BUFFER = 35051 	# GL/glext.h:480
GL_PIXEL_UNPACK_BUFFER = 35052 	# GL/glext.h:481
GL_PIXEL_PACK_BUFFER_BINDING = 35053 	# GL/glext.h:482
GL_PIXEL_UNPACK_BUFFER_BINDING = 35055 	# GL/glext.h:483
GL_SRGB = 35904 	# GL/glext.h:484
GL_SRGB8 = 35905 	# GL/glext.h:485
GL_SRGB_ALPHA = 35906 	# GL/glext.h:486
GL_SRGB8_ALPHA8 = 35907 	# GL/glext.h:487
GL_SLUMINANCE_ALPHA = 35908 	# GL/glext.h:488
GL_SLUMINANCE8_ALPHA8 = 35909 	# GL/glext.h:489
GL_SLUMINANCE = 35910 	# GL/glext.h:490
GL_SLUMINANCE8 = 35911 	# GL/glext.h:491
GL_COMPRESSED_SRGB = 35912 	# GL/glext.h:492
GL_COMPRESSED_SRGB_ALPHA = 35913 	# GL/glext.h:493
GL_COMPRESSED_SLUMINANCE = 35914 	# GL/glext.h:494
GL_COMPRESSED_SLUMINANCE_ALPHA = 35915 	# GL/glext.h:495
GL_FLOAT_MAT2x3 = 35685 	# GL/glext.h:496
GL_FLOAT_MAT2x4 = 35686 	# GL/glext.h:497
GL_FLOAT_MAT3x2 = 35687 	# GL/glext.h:498
GL_FLOAT_MAT3x4 = 35688 	# GL/glext.h:499
GL_FLOAT_MAT4x2 = 35689 	# GL/glext.h:500
GL_FLOAT_MAT4x3 = 35690 	# GL/glext.h:501
GL_CURRENT_RASTER_SECONDARY_COLOR = 33887 	# GL/glext.h:502
# ARB_multitexture (GL/glext.h:505)
GL_TEXTURE0_ARB = 33984 	# GL/glext.h:506
GL_TEXTURE1_ARB = 33985 	# GL/glext.h:507
GL_TEXTURE2_ARB = 33986 	# GL/glext.h:508
GL_TEXTURE3_ARB = 33987 	# GL/glext.h:509
GL_TEXTURE4_ARB = 33988 	# GL/glext.h:510
GL_TEXTURE5_ARB = 33989 	# GL/glext.h:511
GL_TEXTURE6_ARB = 33990 	# GL/glext.h:512
GL_TEXTURE7_ARB = 33991 	# GL/glext.h:513
GL_TEXTURE8_ARB = 33992 	# GL/glext.h:514
GL_TEXTURE9_ARB = 33993 	# GL/glext.h:515
GL_TEXTURE10_ARB = 33994 	# GL/glext.h:516
GL_TEXTURE11_ARB = 33995 	# GL/glext.h:517
GL_TEXTURE12_ARB = 33996 	# GL/glext.h:518
GL_TEXTURE13_ARB = 33997 	# GL/glext.h:519
GL_TEXTURE14_ARB = 33998 	# GL/glext.h:520
GL_TEXTURE15_ARB = 33999 	# GL/glext.h:521
GL_TEXTURE16_ARB = 34000 	# GL/glext.h:522
GL_TEXTURE17_ARB = 34001 	# GL/glext.h:523
GL_TEXTURE18_ARB = 34002 	# GL/glext.h:524
GL_TEXTURE19_ARB = 34003 	# GL/glext.h:525
GL_TEXTURE20_ARB = 34004 	# GL/glext.h:526
GL_TEXTURE21_ARB = 34005 	# GL/glext.h:527
GL_TEXTURE22_ARB = 34006 	# GL/glext.h:528
GL_TEXTURE23_ARB = 34007 	# GL/glext.h:529
GL_TEXTURE24_ARB = 34008 	# GL/glext.h:530
GL_TEXTURE25_ARB = 34009 	# GL/glext.h:531
GL_TEXTURE26_ARB = 34010 	# GL/glext.h:532
GL_TEXTURE27_ARB = 34011 	# GL/glext.h:533
GL_TEXTURE28_ARB = 34012 	# GL/glext.h:534
GL_TEXTURE29_ARB = 34013 	# GL/glext.h:535
GL_TEXTURE30_ARB = 34014 	# GL/glext.h:536
GL_TEXTURE31_ARB = 34015 	# GL/glext.h:537
GL_ACTIVE_TEXTURE_ARB = 34016 	# GL/glext.h:538
GL_CLIENT_ACTIVE_TEXTURE_ARB = 34017 	# GL/glext.h:539
GL_MAX_TEXTURE_UNITS_ARB = 34018 	# GL/glext.h:540
# ARB_transpose_matrix (GL/glext.h:543)
GL_TRANSPOSE_MODELVIEW_MATRIX_ARB = 34019 	# GL/glext.h:544
GL_TRANSPOSE_PROJECTION_MATRIX_ARB = 34020 	# GL/glext.h:545
GL_TRANSPOSE_TEXTURE_MATRIX_ARB = 34021 	# GL/glext.h:546
GL_TRANSPOSE_COLOR_MATRIX_ARB = 34022 	# GL/glext.h:547
# ARB_multisample (GL/glext.h:550)
GL_MULTISAMPLE_ARB = 32925 	# GL/glext.h:551
GL_SAMPLE_ALPHA_TO_COVERAGE_ARB = 32926 	# GL/glext.h:552
GL_SAMPLE_ALPHA_TO_ONE_ARB = 32927 	# GL/glext.h:553
GL_SAMPLE_COVERAGE_ARB = 32928 	# GL/glext.h:554
GL_SAMPLE_BUFFERS_ARB = 32936 	# GL/glext.h:555
GL_SAMPLES_ARB = 32937 	# GL/glext.h:556
GL_SAMPLE_COVERAGE_VALUE_ARB = 32938 	# GL/glext.h:557
GL_SAMPLE_COVERAGE_INVERT_ARB = 32939 	# GL/glext.h:558
GL_MULTISAMPLE_BIT_ARB = 536870912 	# GL/glext.h:559
# ARB_texture_env_add (GL/glext.h:562)
# ARB_texture_cube_map (GL/glext.h:565)
GL_NORMAL_MAP_ARB = 34065 	# GL/glext.h:566
GL_REFLECTION_MAP_ARB = 34066 	# GL/glext.h:567
GL_TEXTURE_CUBE_MAP_ARB = 34067 	# GL/glext.h:568
GL_TEXTURE_BINDING_CUBE_MAP_ARB = 34068 	# GL/glext.h:569
GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB = 34069 	# GL/glext.h:570
GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB = 34070 	# GL/glext.h:571
GL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB = 34071 	# GL/glext.h:572
GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB = 34072 	# GL/glext.h:573
GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB = 34073 	# GL/glext.h:574
GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB = 34074 	# GL/glext.h:575
GL_PROXY_TEXTURE_CUBE_MAP_ARB = 34075 	# GL/glext.h:576
GL_MAX_CUBE_MAP_TEXTURE_SIZE_ARB = 34076 	# GL/glext.h:577
# ARB_texture_compression (GL/glext.h:580)
GL_COMPRESSED_ALPHA_ARB = 34025 	# GL/glext.h:581
GL_COMPRESSED_LUMINANCE_ARB = 34026 	# GL/glext.h:582
GL_COMPRESSED_LUMINANCE_ALPHA_ARB = 34027 	# GL/glext.h:583
GL_COMPRESSED_INTENSITY_ARB = 34028 	# GL/glext.h:584
GL_COMPRESSED_RGB_ARB = 34029 	# GL/glext.h:585
GL_COMPRESSED_RGBA_ARB = 34030 	# GL/glext.h:586
GL_TEXTURE_COMPRESSION_HINT_ARB = 34031 	# GL/glext.h:587
GL_TEXTURE_COMPRESSED_IMAGE_SIZE_ARB = 34464 	# GL/glext.h:588
GL_TEXTURE_COMPRESSED_ARB = 34465 	# GL/glext.h:589
GL_NUM_COMPRESSED_TEXTURE_FORMATS_ARB = 34466 	# GL/glext.h:590
GL_COMPRESSED_TEXTURE_FORMATS_ARB = 34467 	# GL/glext.h:591
# ARB_texture_border_clamp (GL/glext.h:594)
GL_CLAMP_TO_BORDER_ARB = 33069 	# GL/glext.h:595
# ARB_point_parameters (GL/glext.h:598)
GL_POINT_SIZE_MIN_ARB = 33062 	# GL/glext.h:599
GL_POINT_SIZE_MAX_ARB = 33063 	# GL/glext.h:600
GL_POINT_FADE_THRESHOLD_SIZE_ARB = 33064 	# GL/glext.h:601
GL_POINT_DISTANCE_ATTENUATION_ARB = 33065 	# GL/glext.h:602
# ARB_vertex_blend (GL/glext.h:605)
GL_MAX_VERTEX_UNITS_ARB = 34468 	# GL/glext.h:606
GL_ACTIVE_VERTEX_UNITS_ARB = 34469 	# GL/glext.h:607
GL_WEIGHT_SUM_UNITY_ARB = 34470 	# GL/glext.h:608
GL_VERTEX_BLEND_ARB = 34471 	# GL/glext.h:609
GL_CURRENT_WEIGHT_ARB = 34472 	# GL/glext.h:610
GL_WEIGHT_ARRAY_TYPE_ARB = 34473 	# GL/glext.h:611
GL_WEIGHT_ARRAY_STRIDE_ARB = 34474 	# GL/glext.h:612
GL_WEIGHT_ARRAY_SIZE_ARB = 34475 	# GL/glext.h:613
GL_WEIGHT_ARRAY_POINTER_ARB = 34476 	# GL/glext.h:614
GL_WEIGHT_ARRAY_ARB = 34477 	# GL/glext.h:615
GL_MODELVIEW0_ARB = 5888 	# GL/glext.h:616
GL_MODELVIEW1_ARB = 34058 	# GL/glext.h:617
GL_MODELVIEW2_ARB = 34594 	# GL/glext.h:618
GL_MODELVIEW3_ARB = 34595 	# GL/glext.h:619
GL_MODELVIEW4_ARB = 34596 	# GL/glext.h:620
GL_MODELVIEW5_ARB = 34597 	# GL/glext.h:621
GL_MODELVIEW6_ARB = 34598 	# GL/glext.h:622
GL_MODELVIEW7_ARB = 34599 	# GL/glext.h:623
GL_MODELVIEW8_ARB = 34600 	# GL/glext.h:624
GL_MODELVIEW9_ARB = 34601 	# GL/glext.h:625
GL_MODELVIEW10_ARB = 34602 	# GL/glext.h:626
GL_MODELVIEW11_ARB = 34603 	# GL/glext.h:627
GL_MODELVIEW12_ARB = 34604 	# GL/glext.h:628
GL_MODELVIEW13_ARB = 34605 	# GL/glext.h:629
GL_MODELVIEW14_ARB = 34606 	# GL/glext.h:630
GL_MODELVIEW15_ARB = 34607 	# GL/glext.h:631
GL_MODELVIEW16_ARB = 34608 	# GL/glext.h:632
GL_MODELVIEW17_ARB = 34609 	# GL/glext.h:633
GL_MODELVIEW18_ARB = 34610 	# GL/glext.h:634
GL_MODELVIEW19_ARB = 34611 	# GL/glext.h:635
GL_MODELVIEW20_ARB = 34612 	# GL/glext.h:636
GL_MODELVIEW21_ARB = 34613 	# GL/glext.h:637
GL_MODELVIEW22_ARB = 34614 	# GL/glext.h:638
GL_MODELVIEW23_ARB = 34615 	# GL/glext.h:639
GL_MODELVIEW24_ARB = 34616 	# GL/glext.h:640
GL_MODELVIEW25_ARB = 34617 	# GL/glext.h:641
GL_MODELVIEW26_ARB = 34618 	# GL/glext.h:642
GL_MODELVIEW27_ARB = 34619 	# GL/glext.h:643
GL_MODELVIEW28_ARB = 34620 	# GL/glext.h:644
GL_MODELVIEW29_ARB = 34621 	# GL/glext.h:645
GL_MODELVIEW30_ARB = 34622 	# GL/glext.h:646
GL_MODELVIEW31_ARB = 34623 	# GL/glext.h:647
# ARB_matrix_palette (GL/glext.h:650)
GL_MATRIX_PALETTE_ARB = 34880 	# GL/glext.h:651
GL_MAX_MATRIX_PALETTE_STACK_DEPTH_ARB = 34881 	# GL/glext.h:652
GL_MAX_PALETTE_MATRICES_ARB = 34882 	# GL/glext.h:653
GL_CURRENT_PALETTE_MATRIX_ARB = 34883 	# GL/glext.h:654
GL_MATRIX_INDEX_ARRAY_ARB = 34884 	# GL/glext.h:655
GL_CURRENT_MATRIX_INDEX_ARB = 34885 	# GL/glext.h:656
GL_MATRIX_INDEX_ARRAY_SIZE_ARB = 34886 	# GL/glext.h:657
GL_MATRIX_INDEX_ARRAY_TYPE_ARB = 34887 	# GL/glext.h:658
GL_MATRIX_INDEX_ARRAY_STRIDE_ARB = 34888 	# GL/glext.h:659
GL_MATRIX_INDEX_ARRAY_POINTER_ARB = 34889 	# GL/glext.h:660
# ARB_texture_env_combine (GL/glext.h:663)
GL_COMBINE_ARB = 34160 	# GL/glext.h:664
GL_COMBINE_RGB_ARB = 34161 	# GL/glext.h:665
GL_COMBINE_ALPHA_ARB = 34162 	# GL/glext.h:666
GL_SOURCE0_RGB_ARB = 34176 	# GL/glext.h:667
GL_SOURCE1_RGB_ARB = 34177 	# GL/glext.h:668
GL_SOURCE2_RGB_ARB = 34178 	# GL/glext.h:669
GL_SOURCE0_ALPHA_ARB = 34184 	# GL/glext.h:670
GL_SOURCE1_ALPHA_ARB = 34185 	# GL/glext.h:671
GL_SOURCE2_ALPHA_ARB = 34186 	# GL/glext.h:672
GL_OPERAND0_RGB_ARB = 34192 	# GL/glext.h:673
GL_OPERAND1_RGB_ARB = 34193 	# GL/glext.h:674
GL_OPERAND2_RGB_ARB = 34194 	# GL/glext.h:675
GL_OPERAND0_ALPHA_ARB = 34200 	# GL/glext.h:676
GL_OPERAND1_ALPHA_ARB = 34201 	# GL/glext.h:677
GL_OPERAND2_ALPHA_ARB = 34202 	# GL/glext.h:678
GL_RGB_SCALE_ARB = 34163 	# GL/glext.h:679
GL_ADD_SIGNED_ARB = 34164 	# GL/glext.h:680
GL_INTERPOLATE_ARB = 34165 	# GL/glext.h:681
GL_SUBTRACT_ARB = 34023 	# GL/glext.h:682
GL_CONSTANT_ARB = 34166 	# GL/glext.h:683
GL_PRIMARY_COLOR_ARB = 34167 	# GL/glext.h:684
GL_PREVIOUS_ARB = 34168 	# GL/glext.h:685
# ARB_texture_env_crossbar (GL/glext.h:688)
# ARB_texture_env_dot3 (GL/glext.h:691)
GL_DOT3_RGB_ARB = 34478 	# GL/glext.h:692
GL_DOT3_RGBA_ARB = 34479 	# GL/glext.h:693
# ARB_texture_mirrored_repeat (GL/glext.h:696)
GL_MIRRORED_REPEAT_ARB = 33648 	# GL/glext.h:697
# ARB_depth_texture (GL/glext.h:700)
GL_DEPTH_COMPONENT16_ARB = 33189 	# GL/glext.h:701
GL_DEPTH_COMPONENT24_ARB = 33190 	# GL/glext.h:702
GL_DEPTH_COMPONENT32_ARB = 33191 	# GL/glext.h:703
GL_TEXTURE_DEPTH_SIZE_ARB = 34890 	# GL/glext.h:704
GL_DEPTH_TEXTURE_MODE_ARB = 34891 	# GL/glext.h:705
# ARB_shadow (GL/glext.h:708)
GL_TEXTURE_COMPARE_MODE_ARB = 34892 	# GL/glext.h:709
GL_TEXTURE_COMPARE_FUNC_ARB = 34893 	# GL/glext.h:710
GL_COMPARE_R_TO_TEXTURE_ARB = 34894 	# GL/glext.h:711
# ARB_shadow_ambient (GL/glext.h:714)
GL_TEXTURE_COMPARE_FAIL_VALUE_ARB = 32959 	# GL/glext.h:715
# ARB_window_pos (GL/glext.h:718)
# ARB_vertex_program (GL/glext.h:721)
GL_COLOR_SUM_ARB = 33880 	# GL/glext.h:722
GL_VERTEX_PROGRAM_ARB = 34336 	# GL/glext.h:723
GL_VERTEX_ATTRIB_ARRAY_ENABLED_ARB = 34338 	# GL/glext.h:724
GL_VERTEX_ATTRIB_ARRAY_SIZE_ARB = 34339 	# GL/glext.h:725
GL_VERTEX_ATTRIB_ARRAY_STRIDE_ARB = 34340 	# GL/glext.h:726
GL_VERTEX_ATTRIB_ARRAY_TYPE_ARB = 34341 	# GL/glext.h:727
GL_CURRENT_VERTEX_ATTRIB_ARB = 34342 	# GL/glext.h:728
GL_PROGRAM_LENGTH_ARB = 34343 	# GL/glext.h:729
GL_PROGRAM_STRING_ARB = 34344 	# GL/glext.h:730
GL_MAX_PROGRAM_MATRIX_STACK_DEPTH_ARB = 34350 	# GL/glext.h:731
GL_MAX_PROGRAM_MATRICES_ARB = 34351 	# GL/glext.h:732
GL_CURRENT_MATRIX_STACK_DEPTH_ARB = 34368 	# GL/glext.h:733
GL_CURRENT_MATRIX_ARB = 34369 	# GL/glext.h:734
GL_VERTEX_PROGRAM_POINT_SIZE_ARB = 34370 	# GL/glext.h:735
GL_VERTEX_PROGRAM_TWO_SIDE_ARB = 34371 	# GL/glext.h:736
GL_VERTEX_ATTRIB_ARRAY_POINTER_ARB = 34373 	# GL/glext.h:737
GL_PROGRAM_ERROR_POSITION_ARB = 34379 	# GL/glext.h:738
GL_PROGRAM_BINDING_ARB = 34423 	# GL/glext.h:739
GL_MAX_VERTEX_ATTRIBS_ARB = 34921 	# GL/glext.h:740
GL_VERTEX_ATTRIB_ARRAY_NORMALIZED_ARB = 34922 	# GL/glext.h:741
GL_PROGRAM_ERROR_STRING_ARB = 34932 	# GL/glext.h:742
GL_PROGRAM_FORMAT_ASCII_ARB = 34933 	# GL/glext.h:743
GL_PROGRAM_FORMAT_ARB = 34934 	# GL/glext.h:744
GL_PROGRAM_INSTRUCTIONS_ARB = 34976 	# GL/glext.h:745
GL_MAX_PROGRAM_INSTRUCTIONS_ARB = 34977 	# GL/glext.h:746
GL_PROGRAM_NATIVE_INSTRUCTIONS_ARB = 34978 	# GL/glext.h:747
GL_MAX_PROGRAM_NATIVE_INSTRUCTIONS_ARB = 34979 	# GL/glext.h:748
GL_PROGRAM_TEMPORARIES_ARB = 34980 	# GL/glext.h:749
GL_MAX_PROGRAM_TEMPORARIES_ARB = 34981 	# GL/glext.h:750
GL_PROGRAM_NATIVE_TEMPORARIES_ARB = 34982 	# GL/glext.h:751
GL_MAX_PROGRAM_NATIVE_TEMPORARIES_ARB = 34983 	# GL/glext.h:752
GL_PROGRAM_PARAMETERS_ARB = 34984 	# GL/glext.h:753
GL_MAX_PROGRAM_PARAMETERS_ARB = 34985 	# GL/glext.h:754
GL_PROGRAM_NATIVE_PARAMETERS_ARB = 34986 	# GL/glext.h:755
GL_MAX_PROGRAM_NATIVE_PARAMETERS_ARB = 34987 	# GL/glext.h:756
GL_PROGRAM_ATTRIBS_ARB = 34988 	# GL/glext.h:757
GL_MAX_PROGRAM_ATTRIBS_ARB = 34989 	# GL/glext.h:758
GL_PROGRAM_NATIVE_ATTRIBS_ARB = 34990 	# GL/glext.h:759
GL_MAX_PROGRAM_NATIVE_ATTRIBS_ARB = 34991 	# GL/glext.h:760
GL_PROGRAM_ADDRESS_REGISTERS_ARB = 34992 	# GL/glext.h:761
GL_MAX_PROGRAM_ADDRESS_REGISTERS_ARB = 34993 	# GL/glext.h:762
GL_PROGRAM_NATIVE_ADDRESS_REGISTERS_ARB = 34994 	# GL/glext.h:763
GL_MAX_PROGRAM_NATIVE_ADDRESS_REGISTERS_ARB = 34995 	# GL/glext.h:764
GL_MAX_PROGRAM_LOCAL_PARAMETERS_ARB = 34996 	# GL/glext.h:765
GL_MAX_PROGRAM_ENV_PARAMETERS_ARB = 34997 	# GL/glext.h:766
GL_PROGRAM_UNDER_NATIVE_LIMITS_ARB = 34998 	# GL/glext.h:767
GL_TRANSPOSE_CURRENT_MATRIX_ARB = 34999 	# GL/glext.h:768
GL_MATRIX0_ARB = 35008 	# GL/glext.h:769
GL_MATRIX1_ARB = 35009 	# GL/glext.h:770
GL_MATRIX2_ARB = 35010 	# GL/glext.h:771
GL_MATRIX3_ARB = 35011 	# GL/glext.h:772
GL_MATRIX4_ARB = 35012 	# GL/glext.h:773
GL_MATRIX5_ARB = 35013 	# GL/glext.h:774
GL_MATRIX6_ARB = 35014 	# GL/glext.h:775
GL_MATRIX7_ARB = 35015 	# GL/glext.h:776
GL_MATRIX8_ARB = 35016 	# GL/glext.h:777
GL_MATRIX9_ARB = 35017 	# GL/glext.h:778
GL_MATRIX10_ARB = 35018 	# GL/glext.h:779
GL_MATRIX11_ARB = 35019 	# GL/glext.h:780
GL_MATRIX12_ARB = 35020 	# GL/glext.h:781
GL_MATRIX13_ARB = 35021 	# GL/glext.h:782
GL_MATRIX14_ARB = 35022 	# GL/glext.h:783
GL_MATRIX15_ARB = 35023 	# GL/glext.h:784
GL_MATRIX16_ARB = 35024 	# GL/glext.h:785
GL_MATRIX17_ARB = 35025 	# GL/glext.h:786
GL_MATRIX18_ARB = 35026 	# GL/glext.h:787
GL_MATRIX19_ARB = 35027 	# GL/glext.h:788
GL_MATRIX20_ARB = 35028 	# GL/glext.h:789
GL_MATRIX21_ARB = 35029 	# GL/glext.h:790
GL_MATRIX22_ARB = 35030 	# GL/glext.h:791
GL_MATRIX23_ARB = 35031 	# GL/glext.h:792
GL_MATRIX24_ARB = 35032 	# GL/glext.h:793
GL_MATRIX25_ARB = 35033 	# GL/glext.h:794
GL_MATRIX26_ARB = 35034 	# GL/glext.h:795
GL_MATRIX27_ARB = 35035 	# GL/glext.h:796
GL_MATRIX28_ARB = 35036 	# GL/glext.h:797
GL_MATRIX29_ARB = 35037 	# GL/glext.h:798
GL_MATRIX30_ARB = 35038 	# GL/glext.h:799
GL_MATRIX31_ARB = 35039 	# GL/glext.h:800
# ARB_fragment_program (GL/glext.h:803)
GL_FRAGMENT_PROGRAM_ARB = 34820 	# GL/glext.h:804
GL_PROGRAM_ALU_INSTRUCTIONS_ARB = 34821 	# GL/glext.h:805
GL_PROGRAM_TEX_INSTRUCTIONS_ARB = 34822 	# GL/glext.h:806
GL_PROGRAM_TEX_INDIRECTIONS_ARB = 34823 	# GL/glext.h:807
GL_PROGRAM_NATIVE_ALU_INSTRUCTIONS_ARB = 34824 	# GL/glext.h:808
GL_PROGRAM_NATIVE_TEX_INSTRUCTIONS_ARB = 34825 	# GL/glext.h:809
GL_PROGRAM_NATIVE_TEX_INDIRECTIONS_ARB = 34826 	# GL/glext.h:810
GL_MAX_PROGRAM_ALU_INSTRUCTIONS_ARB = 34827 	# GL/glext.h:811
GL_MAX_PROGRAM_TEX_INSTRUCTIONS_ARB = 34828 	# GL/glext.h:812
GL_MAX_PROGRAM_TEX_INDIRECTIONS_ARB = 34829 	# GL/glext.h:813
GL_MAX_PROGRAM_NATIVE_ALU_INSTRUCTIONS_ARB = 34830 	# GL/glext.h:814
GL_MAX_PROGRAM_NATIVE_TEX_INSTRUCTIONS_ARB = 34831 	# GL/glext.h:815
GL_MAX_PROGRAM_NATIVE_TEX_INDIRECTIONS_ARB = 34832 	# GL/glext.h:816
GL_MAX_TEXTURE_COORDS_ARB = 34929 	# GL/glext.h:817
GL_MAX_TEXTURE_IMAGE_UNITS_ARB = 34930 	# GL/glext.h:818
# ARB_vertex_buffer_object (GL/glext.h:821)
GL_BUFFER_SIZE_ARB = 34660 	# GL/glext.h:822
GL_BUFFER_USAGE_ARB = 34661 	# GL/glext.h:823
GL_ARRAY_BUFFER_ARB = 34962 	# GL/glext.h:824
GL_ELEMENT_ARRAY_BUFFER_ARB = 34963 	# GL/glext.h:825
GL_ARRAY_BUFFER_BINDING_ARB = 34964 	# GL/glext.h:826
GL_ELEMENT_ARRAY_BUFFER_BINDING_ARB = 34965 	# GL/glext.h:827
GL_VERTEX_ARRAY_BUFFER_BINDING_ARB = 34966 	# GL/glext.h:828
GL_NORMAL_ARRAY_BUFFER_BINDING_ARB = 34967 	# GL/glext.h:829
GL_COLOR_ARRAY_BUFFER_BINDING_ARB = 34968 	# GL/glext.h:830
GL_INDEX_ARRAY_BUFFER_BINDING_ARB = 34969 	# GL/glext.h:831
GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING_ARB = 34970 	# GL/glext.h:832
GL_EDGE_FLAG_ARRAY_BUFFER_BINDING_ARB = 34971 	# GL/glext.h:833
GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING_ARB = 34972 	# GL/glext.h:834
GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING_ARB = 34973 	# GL/glext.h:835
GL_WEIGHT_ARRAY_BUFFER_BINDING_ARB = 34974 	# GL/glext.h:836
GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING_ARB = 34975 	# GL/glext.h:837
GL_READ_ONLY_ARB = 35000 	# GL/glext.h:838
GL_WRITE_ONLY_ARB = 35001 	# GL/glext.h:839
GL_READ_WRITE_ARB = 35002 	# GL/glext.h:840
GL_BUFFER_ACCESS_ARB = 35003 	# GL/glext.h:841
GL_BUFFER_MAPPED_ARB = 35004 	# GL/glext.h:842
GL_BUFFER_MAP_POINTER_ARB = 35005 	# GL/glext.h:843
GL_STREAM_DRAW_ARB = 35040 	# GL/glext.h:844
GL_STREAM_READ_ARB = 35041 	# GL/glext.h:845
GL_STREAM_COPY_ARB = 35042 	# GL/glext.h:846
GL_STATIC_DRAW_ARB = 35044 	# GL/glext.h:847
GL_STATIC_READ_ARB = 35045 	# GL/glext.h:848
GL_STATIC_COPY_ARB = 35046 	# GL/glext.h:849
GL_DYNAMIC_DRAW_ARB = 35048 	# GL/glext.h:850
GL_DYNAMIC_READ_ARB = 35049 	# GL/glext.h:851
GL_DYNAMIC_COPY_ARB = 35050 	# GL/glext.h:852
# ARB_occlusion_query (GL/glext.h:855)
GL_QUERY_COUNTER_BITS_ARB = 34916 	# GL/glext.h:856
GL_CURRENT_QUERY_ARB = 34917 	# GL/glext.h:857
GL_QUERY_RESULT_ARB = 34918 	# GL/glext.h:858
GL_QUERY_RESULT_AVAILABLE_ARB = 34919 	# GL/glext.h:859
GL_SAMPLES_PASSED_ARB = 35092 	# GL/glext.h:860
# ARB_shader_objects (GL/glext.h:863)
GL_PROGRAM_OBJECT_ARB = 35648 	# GL/glext.h:864
GL_SHADER_OBJECT_ARB = 35656 	# GL/glext.h:865
GL_OBJECT_TYPE_ARB = 35662 	# GL/glext.h:866
GL_OBJECT_SUBTYPE_ARB = 35663 	# GL/glext.h:867
GL_FLOAT_VEC2_ARB = 35664 	# GL/glext.h:868
GL_FLOAT_VEC3_ARB = 35665 	# GL/glext.h:869
GL_FLOAT_VEC4_ARB = 35666 	# GL/glext.h:870
GL_INT_VEC2_ARB = 35667 	# GL/glext.h:871
GL_INT_VEC3_ARB = 35668 	# GL/glext.h:872
GL_INT_VEC4_ARB = 35669 	# GL/glext.h:873
GL_BOOL_ARB = 35670 	# GL/glext.h:874
GL_BOOL_VEC2_ARB = 35671 	# GL/glext.h:875
GL_BOOL_VEC3_ARB = 35672 	# GL/glext.h:876
GL_BOOL_VEC4_ARB = 35673 	# GL/glext.h:877
GL_FLOAT_MAT2_ARB = 35674 	# GL/glext.h:878
GL_FLOAT_MAT3_ARB = 35675 	# GL/glext.h:879
GL_FLOAT_MAT4_ARB = 35676 	# GL/glext.h:880
GL_SAMPLER_1D_ARB = 35677 	# GL/glext.h:881
GL_SAMPLER_2D_ARB = 35678 	# GL/glext.h:882
GL_SAMPLER_3D_ARB = 35679 	# GL/glext.h:883
GL_SAMPLER_CUBE_ARB = 35680 	# GL/glext.h:884
GL_SAMPLER_1D_SHADOW_ARB = 35681 	# GL/glext.h:885
GL_SAMPLER_2D_SHADOW_ARB = 35682 	# GL/glext.h:886
GL_SAMPLER_2D_RECT_ARB = 35683 	# GL/glext.h:887
GL_SAMPLER_2D_RECT_SHADOW_ARB = 35684 	# GL/glext.h:888
GL_OBJECT_DELETE_STATUS_ARB = 35712 	# GL/glext.h:889
GL_OBJECT_COMPILE_STATUS_ARB = 35713 	# GL/glext.h:890
GL_OBJECT_LINK_STATUS_ARB = 35714 	# GL/glext.h:891
GL_OBJECT_VALIDATE_STATUS_ARB = 35715 	# GL/glext.h:892
GL_OBJECT_INFO_LOG_LENGTH_ARB = 35716 	# GL/glext.h:893
GL_OBJECT_ATTACHED_OBJECTS_ARB = 35717 	# GL/glext.h:894
GL_OBJECT_ACTIVE_UNIFORMS_ARB = 35718 	# GL/glext.h:895
GL_OBJECT_ACTIVE_UNIFORM_MAX_LENGTH_ARB = 35719 	# GL/glext.h:896
GL_OBJECT_SHADER_SOURCE_LENGTH_ARB = 35720 	# GL/glext.h:897
# ARB_vertex_shader (GL/glext.h:900)
GL_VERTEX_SHADER_ARB = 35633 	# GL/glext.h:901
GL_MAX_VERTEX_UNIFORM_COMPONENTS_ARB = 35658 	# GL/glext.h:902
GL_MAX_VARYING_FLOATS_ARB = 35659 	# GL/glext.h:903
GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS_ARB = 35660 	# GL/glext.h:904
GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS_ARB = 35661 	# GL/glext.h:905
GL_OBJECT_ACTIVE_ATTRIBUTES_ARB = 35721 	# GL/glext.h:906
GL_OBJECT_ACTIVE_ATTRIBUTE_MAX_LENGTH_ARB = 35722 	# GL/glext.h:907
# ARB_fragment_shader (GL/glext.h:910)
GL_FRAGMENT_SHADER_ARB = 35632 	# GL/glext.h:911
GL_MAX_FRAGMENT_UNIFORM_COMPONENTS_ARB = 35657 	# GL/glext.h:912
GL_FRAGMENT_SHADER_DERIVATIVE_HINT_ARB = 35723 	# GL/glext.h:913
# ARB_shading_language_100 (GL/glext.h:916)
GL_SHADING_LANGUAGE_VERSION_ARB = 35724 	# GL/glext.h:917
# ARB_texture_non_power_of_two (GL/glext.h:920)
# ARB_point_sprite (GL/glext.h:923)
GL_POINT_SPRITE_ARB = 34913 	# GL/glext.h:924
GL_COORD_REPLACE_ARB = 34914 	# GL/glext.h:925
# ARB_fragment_program_shadow (GL/glext.h:928)
# ARB_draw_buffers (GL/glext.h:931)
GL_MAX_DRAW_BUFFERS_ARB = 34852 	# GL/glext.h:932
GL_DRAW_BUFFER0_ARB = 34853 	# GL/glext.h:933
GL_DRAW_BUFFER1_ARB = 34854 	# GL/glext.h:934
GL_DRAW_BUFFER2_ARB = 34855 	# GL/glext.h:935
GL_DRAW_BUFFER3_ARB = 34856 	# GL/glext.h:936
GL_DRAW_BUFFER4_ARB = 34857 	# GL/glext.h:937
GL_DRAW_BUFFER5_ARB = 34858 	# GL/glext.h:938
GL_DRAW_BUFFER6_ARB = 34859 	# GL/glext.h:939
GL_DRAW_BUFFER7_ARB = 34860 	# GL/glext.h:940
GL_DRAW_BUFFER8_ARB = 34861 	# GL/glext.h:941
GL_DRAW_BUFFER9_ARB = 34862 	# GL/glext.h:942
GL_DRAW_BUFFER10_ARB = 34863 	# GL/glext.h:943
GL_DRAW_BUFFER11_ARB = 34864 	# GL/glext.h:944
GL_DRAW_BUFFER12_ARB = 34865 	# GL/glext.h:945
GL_DRAW_BUFFER13_ARB = 34866 	# GL/glext.h:946
GL_DRAW_BUFFER14_ARB = 34867 	# GL/glext.h:947
GL_DRAW_BUFFER15_ARB = 34868 	# GL/glext.h:948
# ARB_texture_rectangle (GL/glext.h:951)
GL_TEXTURE_RECTANGLE_ARB = 34037 	# GL/glext.h:952
GL_TEXTURE_BINDING_RECTANGLE_ARB = 34038 	# GL/glext.h:953
GL_PROXY_TEXTURE_RECTANGLE_ARB = 34039 	# GL/glext.h:954
GL_MAX_RECTANGLE_TEXTURE_SIZE_ARB = 34040 	# GL/glext.h:955
# ARB_color_buffer_float (GL/glext.h:958)
GL_RGBA_FLOAT_MODE_ARB = 34848 	# GL/glext.h:959
GL_CLAMP_VERTEX_COLOR_ARB = 35098 	# GL/glext.h:960
GL_CLAMP_FRAGMENT_COLOR_ARB = 35099 	# GL/glext.h:961
GL_CLAMP_READ_COLOR_ARB = 35100 	# GL/glext.h:962
GL_FIXED_ONLY_ARB = 35101 	# GL/glext.h:963
# ARB_half_float_pixel (GL/glext.h:966)
GL_HALF_FLOAT_ARB = 5131 	# GL/glext.h:967
# ARB_texture_float (GL/glext.h:970)
GL_TEXTURE_RED_TYPE_ARB = 35856 	# GL/glext.h:971
GL_TEXTURE_GREEN_TYPE_ARB = 35857 	# GL/glext.h:972
GL_TEXTURE_BLUE_TYPE_ARB = 35858 	# GL/glext.h:973
GL_TEXTURE_ALPHA_TYPE_ARB = 35859 	# GL/glext.h:974
GL_TEXTURE_LUMINANCE_TYPE_ARB = 35860 	# GL/glext.h:975
GL_TEXTURE_INTENSITY_TYPE_ARB = 35861 	# GL/glext.h:976
GL_TEXTURE_DEPTH_TYPE_ARB = 35862 	# GL/glext.h:977
GL_UNSIGNED_NORMALIZED_ARB = 35863 	# GL/glext.h:978
GL_RGBA32F_ARB = 34836 	# GL/glext.h:979
GL_RGB32F_ARB = 34837 	# GL/glext.h:980
GL_ALPHA32F_ARB = 34838 	# GL/glext.h:981
GL_INTENSITY32F_ARB = 34839 	# GL/glext.h:982
GL_LUMINANCE32F_ARB = 34840 	# GL/glext.h:983
GL_LUMINANCE_ALPHA32F_ARB = 34841 	# GL/glext.h:984
GL_RGBA16F_ARB = 34842 	# GL/glext.h:985
GL_RGB16F_ARB = 34843 	# GL/glext.h:986
GL_ALPHA16F_ARB = 34844 	# GL/glext.h:987
GL_INTENSITY16F_ARB = 34845 	# GL/glext.h:988
GL_LUMINANCE16F_ARB = 34846 	# GL/glext.h:989
GL_LUMINANCE_ALPHA16F_ARB = 34847 	# GL/glext.h:990
# ARB_pixel_buffer_object (GL/glext.h:993)
GL_PIXEL_PACK_BUFFER_ARB = 35051 	# GL/glext.h:994
GL_PIXEL_UNPACK_BUFFER_ARB = 35052 	# GL/glext.h:995
GL_PIXEL_PACK_BUFFER_BINDING_ARB = 35053 	# GL/glext.h:996
GL_PIXEL_UNPACK_BUFFER_BINDING_ARB = 35055 	# GL/glext.h:997
# EXT_abgr (GL/glext.h:1000)
GL_ABGR_EXT = 32768 	# GL/glext.h:1001
# EXT_blend_color (GL/glext.h:1004)
GL_CONSTANT_COLOR_EXT = 32769 	# GL/glext.h:1005
GL_ONE_MINUS_CONSTANT_COLOR_EXT = 32770 	# GL/glext.h:1006
GL_CONSTANT_ALPHA_EXT = 32771 	# GL/glext.h:1007
GL_ONE_MINUS_CONSTANT_ALPHA_EXT = 32772 	# GL/glext.h:1008
GL_BLEND_COLOR_EXT = 32773 	# GL/glext.h:1009
# EXT_polygon_offset (GL/glext.h:1012)
GL_POLYGON_OFFSET_EXT = 32823 	# GL/glext.h:1013
GL_POLYGON_OFFSET_FACTOR_EXT = 32824 	# GL/glext.h:1014
GL_POLYGON_OFFSET_BIAS_EXT = 32825 	# GL/glext.h:1015
# EXT_texture (GL/glext.h:1018)
GL_ALPHA4_EXT = 32827 	# GL/glext.h:1019
GL_ALPHA8_EXT = 32828 	# GL/glext.h:1020
GL_ALPHA12_EXT = 32829 	# GL/glext.h:1021
GL_ALPHA16_EXT = 32830 	# GL/glext.h:1022
GL_LUMINANCE4_EXT = 32831 	# GL/glext.h:1023
GL_LUMINANCE8_EXT = 32832 	# GL/glext.h:1024
GL_LUMINANCE12_EXT = 32833 	# GL/glext.h:1025
GL_LUMINANCE16_EXT = 32834 	# GL/glext.h:1026
GL_LUMINANCE4_ALPHA4_EXT = 32835 	# GL/glext.h:1027
GL_LUMINANCE6_ALPHA2_EXT = 32836 	# GL/glext.h:1028
GL_LUMINANCE8_ALPHA8_EXT = 32837 	# GL/glext.h:1029
GL_LUMINANCE12_ALPHA4_EXT = 32838 	# GL/glext.h:1030
GL_LUMINANCE12_ALPHA12_EXT = 32839 	# GL/glext.h:1031
GL_LUMINANCE16_ALPHA16_EXT = 32840 	# GL/glext.h:1032
GL_INTENSITY_EXT = 32841 	# GL/glext.h:1033
GL_INTENSITY4_EXT = 32842 	# GL/glext.h:1034
GL_INTENSITY8_EXT = 32843 	# GL/glext.h:1035
GL_INTENSITY12_EXT = 32844 	# GL/glext.h:1036
GL_INTENSITY16_EXT = 32845 	# GL/glext.h:1037
GL_RGB2_EXT = 32846 	# GL/glext.h:1038
GL_RGB4_EXT = 32847 	# GL/glext.h:1039
GL_RGB5_EXT = 32848 	# GL/glext.h:1040
GL_RGB8_EXT = 32849 	# GL/glext.h:1041
GL_RGB10_EXT = 32850 	# GL/glext.h:1042
GL_RGB12_EXT = 32851 	# GL/glext.h:1043
GL_RGB16_EXT = 32852 	# GL/glext.h:1044
GL_RGBA2_EXT = 32853 	# GL/glext.h:1045
GL_RGBA4_EXT = 32854 	# GL/glext.h:1046
GL_RGB5_A1_EXT = 32855 	# GL/glext.h:1047
GL_RGBA8_EXT = 32856 	# GL/glext.h:1048
GL_RGB10_A2_EXT = 32857 	# GL/glext.h:1049
GL_RGBA12_EXT = 32858 	# GL/glext.h:1050
GL_RGBA16_EXT = 32859 	# GL/glext.h:1051
GL_TEXTURE_RED_SIZE_EXT = 32860 	# GL/glext.h:1052
GL_TEXTURE_GREEN_SIZE_EXT = 32861 	# GL/glext.h:1053
GL_TEXTURE_BLUE_SIZE_EXT = 32862 	# GL/glext.h:1054
GL_TEXTURE_ALPHA_SIZE_EXT = 32863 	# GL/glext.h:1055
GL_TEXTURE_LUMINANCE_SIZE_EXT = 32864 	# GL/glext.h:1056
GL_TEXTURE_INTENSITY_SIZE_EXT = 32865 	# GL/glext.h:1057
GL_REPLACE_EXT = 32866 	# GL/glext.h:1058
GL_PROXY_TEXTURE_1D_EXT = 32867 	# GL/glext.h:1059
GL_PROXY_TEXTURE_2D_EXT = 32868 	# GL/glext.h:1060
GL_TEXTURE_TOO_LARGE_EXT = 32869 	# GL/glext.h:1061
# EXT_texture3D (GL/glext.h:1064)
GL_PACK_SKIP_IMAGES_EXT = 32875 	# GL/glext.h:1065
GL_PACK_IMAGE_HEIGHT_EXT = 32876 	# GL/glext.h:1066
GL_UNPACK_SKIP_IMAGES_EXT = 32877 	# GL/glext.h:1067
GL_UNPACK_IMAGE_HEIGHT_EXT = 32878 	# GL/glext.h:1068
GL_TEXTURE_3D_EXT = 32879 	# GL/glext.h:1069
GL_PROXY_TEXTURE_3D_EXT = 32880 	# GL/glext.h:1070
GL_TEXTURE_DEPTH_EXT = 32881 	# GL/glext.h:1071
GL_TEXTURE_WRAP_R_EXT = 32882 	# GL/glext.h:1072
GL_MAX_3D_TEXTURE_SIZE_EXT = 32883 	# GL/glext.h:1073
# SGIS_texture_filter4 (GL/glext.h:1076)
GL_FILTER4_SGIS = 33094 	# GL/glext.h:1077
GL_TEXTURE_FILTER4_SIZE_SGIS = 33095 	# GL/glext.h:1078
# EXT_subtexture (GL/glext.h:1081)
# EXT_copy_texture (GL/glext.h:1084)
# EXT_histogram (GL/glext.h:1087)
GL_HISTOGRAM_EXT = 32804 	# GL/glext.h:1088
GL_PROXY_HISTOGRAM_EXT = 32805 	# GL/glext.h:1089
GL_HISTOGRAM_WIDTH_EXT = 32806 	# GL/glext.h:1090
GL_HISTOGRAM_FORMAT_EXT = 32807 	# GL/glext.h:1091
GL_HISTOGRAM_RED_SIZE_EXT = 32808 	# GL/glext.h:1092
GL_HISTOGRAM_GREEN_SIZE_EXT = 32809 	# GL/glext.h:1093
GL_HISTOGRAM_BLUE_SIZE_EXT = 32810 	# GL/glext.h:1094
GL_HISTOGRAM_ALPHA_SIZE_EXT = 32811 	# GL/glext.h:1095
GL_HISTOGRAM_LUMINANCE_SIZE_EXT = 32812 	# GL/glext.h:1096
GL_HISTOGRAM_SINK_EXT = 32813 	# GL/glext.h:1097
GL_MINMAX_EXT = 32814 	# GL/glext.h:1098
GL_MINMAX_FORMAT_EXT = 32815 	# GL/glext.h:1099
GL_MINMAX_SINK_EXT = 32816 	# GL/glext.h:1100
GL_TABLE_TOO_LARGE_EXT = 32817 	# GL/glext.h:1101
# EXT_convolution (GL/glext.h:1104)
GL_CONVOLUTION_1D_EXT = 32784 	# GL/glext.h:1105
GL_CONVOLUTION_2D_EXT = 32785 	# GL/glext.h:1106
GL_SEPARABLE_2D_EXT = 32786 	# GL/glext.h:1107
GL_CONVOLUTION_BORDER_MODE_EXT = 32787 	# GL/glext.h:1108
GL_CONVOLUTION_FILTER_SCALE_EXT = 32788 	# GL/glext.h:1109
GL_CONVOLUTION_FILTER_BIAS_EXT = 32789 	# GL/glext.h:1110
GL_REDUCE_EXT = 32790 	# GL/glext.h:1111
GL_CONVOLUTION_FORMAT_EXT = 32791 	# GL/glext.h:1112
GL_CONVOLUTION_WIDTH_EXT = 32792 	# GL/glext.h:1113
GL_CONVOLUTION_HEIGHT_EXT = 32793 	# GL/glext.h:1114
GL_MAX_CONVOLUTION_WIDTH_EXT = 32794 	# GL/glext.h:1115
GL_MAX_CONVOLUTION_HEIGHT_EXT = 32795 	# GL/glext.h:1116
GL_POST_CONVOLUTION_RED_SCALE_EXT = 32796 	# GL/glext.h:1117
GL_POST_CONVOLUTION_GREEN_SCALE_EXT = 32797 	# GL/glext.h:1118
GL_POST_CONVOLUTION_BLUE_SCALE_EXT = 32798 	# GL/glext.h:1119
GL_POST_CONVOLUTION_ALPHA_SCALE_EXT = 32799 	# GL/glext.h:1120
GL_POST_CONVOLUTION_RED_BIAS_EXT = 32800 	# GL/glext.h:1121
GL_POST_CONVOLUTION_GREEN_BIAS_EXT = 32801 	# GL/glext.h:1122
GL_POST_CONVOLUTION_BLUE_BIAS_EXT = 32802 	# GL/glext.h:1123
GL_POST_CONVOLUTION_ALPHA_BIAS_EXT = 32803 	# GL/glext.h:1124
# SGI_color_matrix (GL/glext.h:1127)
GL_COLOR_MATRIX_SGI = 32945 	# GL/glext.h:1128
GL_COLOR_MATRIX_STACK_DEPTH_SGI = 32946 	# GL/glext.h:1129
GL_MAX_COLOR_MATRIX_STACK_DEPTH_SGI = 32947 	# GL/glext.h:1130
GL_POST_COLOR_MATRIX_RED_SCALE_SGI = 32948 	# GL/glext.h:1131
GL_POST_COLOR_MATRIX_GREEN_SCALE_SGI = 32949 	# GL/glext.h:1132
GL_POST_COLOR_MATRIX_BLUE_SCALE_SGI = 32950 	# GL/glext.h:1133
GL_POST_COLOR_MATRIX_ALPHA_SCALE_SGI = 32951 	# GL/glext.h:1134
GL_POST_COLOR_MATRIX_RED_BIAS_SGI = 32952 	# GL/glext.h:1135
GL_POST_COLOR_MATRIX_GREEN_BIAS_SGI = 32953 	# GL/glext.h:1136
GL_POST_COLOR_MATRIX_BLUE_BIAS_SGI = 32954 	# GL/glext.h:1137
GL_POST_COLOR_MATRIX_ALPHA_BIAS_SGI = 32955 	# GL/glext.h:1138
# SGI_color_table (GL/glext.h:1141)
GL_COLOR_TABLE_SGI = 32976 	# GL/glext.h:1142
GL_POST_CONVOLUTION_COLOR_TABLE_SGI = 32977 	# GL/glext.h:1143
GL_POST_COLOR_MATRIX_COLOR_TABLE_SGI = 32978 	# GL/glext.h:1144
GL_PROXY_COLOR_TABLE_SGI = 32979 	# GL/glext.h:1145
GL_PROXY_POST_CONVOLUTION_COLOR_TABLE_SGI = 32980 	# GL/glext.h:1146
GL_PROXY_POST_COLOR_MATRIX_COLOR_TABLE_SGI = 32981 	# GL/glext.h:1147
GL_COLOR_TABLE_SCALE_SGI = 32982 	# GL/glext.h:1148
GL_COLOR_TABLE_BIAS_SGI = 32983 	# GL/glext.h:1149
GL_COLOR_TABLE_FORMAT_SGI = 32984 	# GL/glext.h:1150
GL_COLOR_TABLE_WIDTH_SGI = 32985 	# GL/glext.h:1151
GL_COLOR_TABLE_RED_SIZE_SGI = 32986 	# GL/glext.h:1152
GL_COLOR_TABLE_GREEN_SIZE_SGI = 32987 	# GL/glext.h:1153
GL_COLOR_TABLE_BLUE_SIZE_SGI = 32988 	# GL/glext.h:1154
GL_COLOR_TABLE_ALPHA_SIZE_SGI = 32989 	# GL/glext.h:1155
GL_COLOR_TABLE_LUMINANCE_SIZE_SGI = 32990 	# GL/glext.h:1156
GL_COLOR_TABLE_INTENSITY_SIZE_SGI = 32991 	# GL/glext.h:1157
# SGIS_pixel_texture (GL/glext.h:1160)
GL_PIXEL_TEXTURE_SGIS = 33619 	# GL/glext.h:1161
GL_PIXEL_FRAGMENT_RGB_SOURCE_SGIS = 33620 	# GL/glext.h:1162
GL_PIXEL_FRAGMENT_ALPHA_SOURCE_SGIS = 33621 	# GL/glext.h:1163
GL_PIXEL_GROUP_COLOR_SGIS = 33622 	# GL/glext.h:1164
# SGIX_pixel_texture (GL/glext.h:1167)
GL_PIXEL_TEX_GEN_SGIX = 33081 	# GL/glext.h:1168
GL_PIXEL_TEX_GEN_MODE_SGIX = 33579 	# GL/glext.h:1169
# SGIS_texture4D (GL/glext.h:1172)
GL_PACK_SKIP_VOLUMES_SGIS = 33072 	# GL/glext.h:1173
GL_PACK_IMAGE_DEPTH_SGIS = 33073 	# GL/glext.h:1174
GL_UNPACK_SKIP_VOLUMES_SGIS = 33074 	# GL/glext.h:1175
GL_UNPACK_IMAGE_DEPTH_SGIS = 33075 	# GL/glext.h:1176
GL_TEXTURE_4D_SGIS = 33076 	# GL/glext.h:1177
GL_PROXY_TEXTURE_4D_SGIS = 33077 	# GL/glext.h:1178
GL_TEXTURE_4DSIZE_SGIS = 33078 	# GL/glext.h:1179
GL_TEXTURE_WRAP_Q_SGIS = 33079 	# GL/glext.h:1180
GL_MAX_4D_TEXTURE_SIZE_SGIS = 33080 	# GL/glext.h:1181
GL_TEXTURE_4D_BINDING_SGIS = 33103 	# GL/glext.h:1182
# SGI_texture_color_table (GL/glext.h:1185)
GL_TEXTURE_COLOR_TABLE_SGI = 32956 	# GL/glext.h:1186
GL_PROXY_TEXTURE_COLOR_TABLE_SGI = 32957 	# GL/glext.h:1187
# EXT_cmyka (GL/glext.h:1190)
GL_CMYK_EXT = 32780 	# GL/glext.h:1191
GL_CMYKA_EXT = 32781 	# GL/glext.h:1192
GL_PACK_CMYK_HINT_EXT = 32782 	# GL/glext.h:1193
GL_UNPACK_CMYK_HINT_EXT = 32783 	# GL/glext.h:1194
# EXT_texture_object (GL/glext.h:1197)
GL_TEXTURE_PRIORITY_EXT = 32870 	# GL/glext.h:1198
GL_TEXTURE_RESIDENT_EXT = 32871 	# GL/glext.h:1199
GL_TEXTURE_1D_BINDING_EXT = 32872 	# GL/glext.h:1200
GL_TEXTURE_2D_BINDING_EXT = 32873 	# GL/glext.h:1201
GL_TEXTURE_3D_BINDING_EXT = 32874 	# GL/glext.h:1202
# SGIS_detail_texture (GL/glext.h:1205)
GL_DETAIL_TEXTURE_2D_SGIS = 32917 	# GL/glext.h:1206
GL_DETAIL_TEXTURE_2D_BINDING_SGIS = 32918 	# GL/glext.h:1207
GL_LINEAR_DETAIL_SGIS = 32919 	# GL/glext.h:1208
GL_LINEAR_DETAIL_ALPHA_SGIS = 32920 	# GL/glext.h:1209
GL_LINEAR_DETAIL_COLOR_SGIS = 32921 	# GL/glext.h:1210
GL_DETAIL_TEXTURE_LEVEL_SGIS = 32922 	# GL/glext.h:1211
GL_DETAIL_TEXTURE_MODE_SGIS = 32923 	# GL/glext.h:1212
GL_DETAIL_TEXTURE_FUNC_POINTS_SGIS = 32924 	# GL/glext.h:1213
# SGIS_sharpen_texture (GL/glext.h:1216)
GL_LINEAR_SHARPEN_SGIS = 32941 	# GL/glext.h:1217
GL_LINEAR_SHARPEN_ALPHA_SGIS = 32942 	# GL/glext.h:1218
GL_LINEAR_SHARPEN_COLOR_SGIS = 32943 	# GL/glext.h:1219
GL_SHARPEN_TEXTURE_FUNC_POINTS_SGIS = 32944 	# GL/glext.h:1220
# EXT_packed_pixels (GL/glext.h:1223)
GL_UNSIGNED_BYTE_3_3_2_EXT = 32818 	# GL/glext.h:1224
GL_UNSIGNED_SHORT_4_4_4_4_EXT = 32819 	# GL/glext.h:1225
GL_UNSIGNED_SHORT_5_5_5_1_EXT = 32820 	# GL/glext.h:1226
GL_UNSIGNED_INT_8_8_8_8_EXT = 32821 	# GL/glext.h:1227
GL_UNSIGNED_INT_10_10_10_2_EXT = 32822 	# GL/glext.h:1228
# SGIS_texture_lod (GL/glext.h:1231)
GL_TEXTURE_MIN_LOD_SGIS = 33082 	# GL/glext.h:1232
GL_TEXTURE_MAX_LOD_SGIS = 33083 	# GL/glext.h:1233
GL_TEXTURE_BASE_LEVEL_SGIS = 33084 	# GL/glext.h:1234
GL_TEXTURE_MAX_LEVEL_SGIS = 33085 	# GL/glext.h:1235
# SGIS_multisample (GL/glext.h:1238)
GL_MULTISAMPLE_SGIS = 32925 	# GL/glext.h:1239
GL_SAMPLE_ALPHA_TO_MASK_SGIS = 32926 	# GL/glext.h:1240
GL_SAMPLE_ALPHA_TO_ONE_SGIS = 32927 	# GL/glext.h:1241
GL_SAMPLE_MASK_SGIS = 32928 	# GL/glext.h:1242
GL_1PASS_SGIS = 32929 	# GL/glext.h:1243
GL_2PASS_0_SGIS = 32930 	# GL/glext.h:1244
GL_2PASS_1_SGIS = 32931 	# GL/glext.h:1245
GL_4PASS_0_SGIS = 32932 	# GL/glext.h:1246
GL_4PASS_1_SGIS = 32933 	# GL/glext.h:1247
GL_4PASS_2_SGIS = 32934 	# GL/glext.h:1248
GL_4PASS_3_SGIS = 32935 	# GL/glext.h:1249
GL_SAMPLE_BUFFERS_SGIS = 32936 	# GL/glext.h:1250
GL_SAMPLES_SGIS = 32937 	# GL/glext.h:1251
GL_SAMPLE_MASK_VALUE_SGIS = 32938 	# GL/glext.h:1252
GL_SAMPLE_MASK_INVERT_SGIS = 32939 	# GL/glext.h:1253
GL_SAMPLE_PATTERN_SGIS = 32940 	# GL/glext.h:1254
# EXT_rescale_normal (GL/glext.h:1257)
GL_RESCALE_NORMAL_EXT = 32826 	# GL/glext.h:1258
# EXT_vertex_array (GL/glext.h:1261)
GL_VERTEX_ARRAY_EXT = 32884 	# GL/glext.h:1262
GL_NORMAL_ARRAY_EXT = 32885 	# GL/glext.h:1263
GL_COLOR_ARRAY_EXT = 32886 	# GL/glext.h:1264
GL_INDEX_ARRAY_EXT = 32887 	# GL/glext.h:1265
GL_TEXTURE_COORD_ARRAY_EXT = 32888 	# GL/glext.h:1266
GL_EDGE_FLAG_ARRAY_EXT = 32889 	# GL/glext.h:1267
GL_VERTEX_ARRAY_SIZE_EXT = 32890 	# GL/glext.h:1268
GL_VERTEX_ARRAY_TYPE_EXT = 32891 	# GL/glext.h:1269
GL_VERTEX_ARRAY_STRIDE_EXT = 32892 	# GL/glext.h:1270
GL_VERTEX_ARRAY_COUNT_EXT = 32893 	# GL/glext.h:1271
GL_NORMAL_ARRAY_TYPE_EXT = 32894 	# GL/glext.h:1272
GL_NORMAL_ARRAY_STRIDE_EXT = 32895 	# GL/glext.h:1273
GL_NORMAL_ARRAY_COUNT_EXT = 32896 	# GL/glext.h:1274
GL_COLOR_ARRAY_SIZE_EXT = 32897 	# GL/glext.h:1275
GL_COLOR_ARRAY_TYPE_EXT = 32898 	# GL/glext.h:1276
GL_COLOR_ARRAY_STRIDE_EXT = 32899 	# GL/glext.h:1277
GL_COLOR_ARRAY_COUNT_EXT = 32900 	# GL/glext.h:1278
GL_INDEX_ARRAY_TYPE_EXT = 32901 	# GL/glext.h:1279
GL_INDEX_ARRAY_STRIDE_EXT = 32902 	# GL/glext.h:1280
GL_INDEX_ARRAY_COUNT_EXT = 32903 	# GL/glext.h:1281
GL_TEXTURE_COORD_ARRAY_SIZE_EXT = 32904 	# GL/glext.h:1282
GL_TEXTURE_COORD_ARRAY_TYPE_EXT = 32905 	# GL/glext.h:1283
GL_TEXTURE_COORD_ARRAY_STRIDE_EXT = 32906 	# GL/glext.h:1284
GL_TEXTURE_COORD_ARRAY_COUNT_EXT = 32907 	# GL/glext.h:1285
GL_EDGE_FLAG_ARRAY_STRIDE_EXT = 32908 	# GL/glext.h:1286
GL_EDGE_FLAG_ARRAY_COUNT_EXT = 32909 	# GL/glext.h:1287
GL_VERTEX_ARRAY_POINTER_EXT = 32910 	# GL/glext.h:1288
GL_NORMAL_ARRAY_POINTER_EXT = 32911 	# GL/glext.h:1289
GL_COLOR_ARRAY_POINTER_EXT = 32912 	# GL/glext.h:1290
GL_INDEX_ARRAY_POINTER_EXT = 32913 	# GL/glext.h:1291
GL_TEXTURE_COORD_ARRAY_POINTER_EXT = 32914 	# GL/glext.h:1292
GL_EDGE_FLAG_ARRAY_POINTER_EXT = 32915 	# GL/glext.h:1293
# EXT_misc_attribute (GL/glext.h:1296)
# SGIS_generate_mipmap (GL/glext.h:1299)
GL_GENERATE_MIPMAP_SGIS = 33169 	# GL/glext.h:1300
GL_GENERATE_MIPMAP_HINT_SGIS = 33170 	# GL/glext.h:1301
# SGIX_clipmap (GL/glext.h:1304)
GL_LINEAR_CLIPMAP_LINEAR_SGIX = 33136 	# GL/glext.h:1305
GL_TEXTURE_CLIPMAP_CENTER_SGIX = 33137 	# GL/glext.h:1306
GL_TEXTURE_CLIPMAP_FRAME_SGIX = 33138 	# GL/glext.h:1307
GL_TEXTURE_CLIPMAP_OFFSET_SGIX = 33139 	# GL/glext.h:1308
GL_TEXTURE_CLIPMAP_VIRTUAL_DEPTH_SGIX = 33140 	# GL/glext.h:1309
GL_TEXTURE_CLIPMAP_LOD_OFFSET_SGIX = 33141 	# GL/glext.h:1310
GL_TEXTURE_CLIPMAP_DEPTH_SGIX = 33142 	# GL/glext.h:1311
GL_MAX_CLIPMAP_DEPTH_SGIX = 33143 	# GL/glext.h:1312
GL_MAX_CLIPMAP_VIRTUAL_DEPTH_SGIX = 33144 	# GL/glext.h:1313
GL_NEAREST_CLIPMAP_NEAREST_SGIX = 33869 	# GL/glext.h:1314
GL_NEAREST_CLIPMAP_LINEAR_SGIX = 33870 	# GL/glext.h:1315
GL_LINEAR_CLIPMAP_NEAREST_SGIX = 33871 	# GL/glext.h:1316
# SGIX_shadow (GL/glext.h:1319)
GL_TEXTURE_COMPARE_SGIX = 33178 	# GL/glext.h:1320
GL_TEXTURE_COMPARE_OPERATOR_SGIX = 33179 	# GL/glext.h:1321
GL_TEXTURE_LEQUAL_R_SGIX = 33180 	# GL/glext.h:1322
GL_TEXTURE_GEQUAL_R_SGIX = 33181 	# GL/glext.h:1323
# SGIS_texture_edge_clamp (GL/glext.h:1326)
GL_CLAMP_TO_EDGE_SGIS = 33071 	# GL/glext.h:1327
# SGIS_texture_border_clamp (GL/glext.h:1330)
GL_CLAMP_TO_BORDER_SGIS = 33069 	# GL/glext.h:1331
# EXT_blend_minmax (GL/glext.h:1334)
GL_FUNC_ADD_EXT = 32774 	# GL/glext.h:1335
GL_MIN_EXT = 32775 	# GL/glext.h:1336
GL_MAX_EXT = 32776 	# GL/glext.h:1337
GL_BLEND_EQUATION_EXT = 32777 	# GL/glext.h:1338
# EXT_blend_subtract (GL/glext.h:1341)
GL_FUNC_SUBTRACT_EXT = 32778 	# GL/glext.h:1342
GL_FUNC_REVERSE_SUBTRACT_EXT = 32779 	# GL/glext.h:1343
# EXT_blend_logic_op (GL/glext.h:1346)
# SGIX_interlace (GL/glext.h:1349)
GL_INTERLACE_SGIX = 32916 	# GL/glext.h:1350
# SGIX_pixel_tiles (GL/glext.h:1353)
GL_PIXEL_TILE_BEST_ALIGNMENT_SGIX = 33086 	# GL/glext.h:1354
GL_PIXEL_TILE_CACHE_INCREMENT_SGIX = 33087 	# GL/glext.h:1355
GL_PIXEL_TILE_WIDTH_SGIX = 33088 	# GL/glext.h:1356
GL_PIXEL_TILE_HEIGHT_SGIX = 33089 	# GL/glext.h:1357
GL_PIXEL_TILE_GRID_WIDTH_SGIX = 33090 	# GL/glext.h:1358
GL_PIXEL_TILE_GRID_HEIGHT_SGIX = 33091 	# GL/glext.h:1359
GL_PIXEL_TILE_GRID_DEPTH_SGIX = 33092 	# GL/glext.h:1360
GL_PIXEL_TILE_CACHE_SIZE_SGIX = 33093 	# GL/glext.h:1361
# SGIS_texture_select (GL/glext.h:1364)
GL_DUAL_ALPHA4_SGIS = 33040 	# GL/glext.h:1365
GL_DUAL_ALPHA8_SGIS = 33041 	# GL/glext.h:1366
GL_DUAL_ALPHA12_SGIS = 33042 	# GL/glext.h:1367
GL_DUAL_ALPHA16_SGIS = 33043 	# GL/glext.h:1368
GL_DUAL_LUMINANCE4_SGIS = 33044 	# GL/glext.h:1369
GL_DUAL_LUMINANCE8_SGIS = 33045 	# GL/glext.h:1370
GL_DUAL_LUMINANCE12_SGIS = 33046 	# GL/glext.h:1371
GL_DUAL_LUMINANCE16_SGIS = 33047 	# GL/glext.h:1372
GL_DUAL_INTENSITY4_SGIS = 33048 	# GL/glext.h:1373
GL_DUAL_INTENSITY8_SGIS = 33049 	# GL/glext.h:1374
GL_DUAL_INTENSITY12_SGIS = 33050 	# GL/glext.h:1375
GL_DUAL_INTENSITY16_SGIS = 33051 	# GL/glext.h:1376
GL_DUAL_LUMINANCE_ALPHA4_SGIS = 33052 	# GL/glext.h:1377
GL_DUAL_LUMINANCE_ALPHA8_SGIS = 33053 	# GL/glext.h:1378
GL_QUAD_ALPHA4_SGIS = 33054 	# GL/glext.h:1379
GL_QUAD_ALPHA8_SGIS = 33055 	# GL/glext.h:1380
GL_QUAD_LUMINANCE4_SGIS = 33056 	# GL/glext.h:1381
GL_QUAD_LUMINANCE8_SGIS = 33057 	# GL/glext.h:1382
GL_QUAD_INTENSITY4_SGIS = 33058 	# GL/glext.h:1383
GL_QUAD_INTENSITY8_SGIS = 33059 	# GL/glext.h:1384
GL_DUAL_TEXTURE_SELECT_SGIS = 33060 	# GL/glext.h:1385
GL_QUAD_TEXTURE_SELECT_SGIS = 33061 	# GL/glext.h:1386
# SGIX_sprite (GL/glext.h:1389)
GL_SPRITE_SGIX = 33096 	# GL/glext.h:1390
GL_SPRITE_MODE_SGIX = 33097 	# GL/glext.h:1391
GL_SPRITE_AXIS_SGIX = 33098 	# GL/glext.h:1392
GL_SPRITE_TRANSLATION_SGIX = 33099 	# GL/glext.h:1393
GL_SPRITE_AXIAL_SGIX = 33100 	# GL/glext.h:1394
GL_SPRITE_OBJECT_ALIGNED_SGIX = 33101 	# GL/glext.h:1395
GL_SPRITE_EYE_ALIGNED_SGIX = 33102 	# GL/glext.h:1396
# SGIX_texture_multi_buffer (GL/glext.h:1399)
GL_TEXTURE_MULTI_BUFFER_HINT_SGIX = 33070 	# GL/glext.h:1400
# EXT_point_parameters (GL/glext.h:1403)
GL_POINT_SIZE_MIN_EXT = 33062 	# GL/glext.h:1404
GL_POINT_SIZE_MAX_EXT = 33063 	# GL/glext.h:1405
GL_POINT_FADE_THRESHOLD_SIZE_EXT = 33064 	# GL/glext.h:1406
GL_DISTANCE_ATTENUATION_EXT = 33065 	# GL/glext.h:1407
# SGIS_point_parameters (GL/glext.h:1410)
GL_POINT_SIZE_MIN_SGIS = 33062 	# GL/glext.h:1411
GL_POINT_SIZE_MAX_SGIS = 33063 	# GL/glext.h:1412
GL_POINT_FADE_THRESHOLD_SIZE_SGIS = 33064 	# GL/glext.h:1413
GL_DISTANCE_ATTENUATION_SGIS = 33065 	# GL/glext.h:1414
# SGIX_instruments (GL/glext.h:1417)
GL_INSTRUMENT_BUFFER_POINTER_SGIX = 33152 	# GL/glext.h:1418
GL_INSTRUMENT_MEASUREMENTS_SGIX = 33153 	# GL/glext.h:1419
# SGIX_texture_scale_bias (GL/glext.h:1422)
GL_POST_TEXTURE_FILTER_BIAS_SGIX = 33145 	# GL/glext.h:1423
GL_POST_TEXTURE_FILTER_SCALE_SGIX = 33146 	# GL/glext.h:1424
GL_POST_TEXTURE_FILTER_BIAS_RANGE_SGIX = 33147 	# GL/glext.h:1425
GL_POST_TEXTURE_FILTER_SCALE_RANGE_SGIX = 33148 	# GL/glext.h:1426
# SGIX_framezoom (GL/glext.h:1429)
GL_FRAMEZOOM_SGIX = 33163 	# GL/glext.h:1430
GL_FRAMEZOOM_FACTOR_SGIX = 33164 	# GL/glext.h:1431
GL_MAX_FRAMEZOOM_FACTOR_SGIX = 33165 	# GL/glext.h:1432
# SGIX_tag_sample_buffer (GL/glext.h:1435)
# FfdMaskSGIX (GL/glext.h:1438)
GL_TEXTURE_DEFORMATION_BIT_SGIX = 1 	# GL/glext.h:1439
GL_GEOMETRY_DEFORMATION_BIT_SGIX = 2 	# GL/glext.h:1440
# SGIX_polynomial_ffd (GL/glext.h:1443)
GL_GEOMETRY_DEFORMATION_SGIX = 33172 	# GL/glext.h:1444
GL_TEXTURE_DEFORMATION_SGIX = 33173 	# GL/glext.h:1445
GL_DEFORMATIONS_MASK_SGIX = 33174 	# GL/glext.h:1446
GL_MAX_DEFORMATION_ORDER_SGIX = 33175 	# GL/glext.h:1447
# SGIX_reference_plane (GL/glext.h:1450)
GL_REFERENCE_PLANE_SGIX = 33149 	# GL/glext.h:1451
GL_REFERENCE_PLANE_EQUATION_SGIX = 33150 	# GL/glext.h:1452
# SGIX_flush_raster (GL/glext.h:1455)
# SGIX_depth_texture (GL/glext.h:1458)
GL_DEPTH_COMPONENT16_SGIX = 33189 	# GL/glext.h:1459
GL_DEPTH_COMPONENT24_SGIX = 33190 	# GL/glext.h:1460
GL_DEPTH_COMPONENT32_SGIX = 33191 	# GL/glext.h:1461
# SGIS_fog_function (GL/glext.h:1464)
GL_FOG_FUNC_SGIS = 33066 	# GL/glext.h:1465
GL_FOG_FUNC_POINTS_SGIS = 33067 	# GL/glext.h:1466
GL_MAX_FOG_FUNC_POINTS_SGIS = 33068 	# GL/glext.h:1467
# SGIX_fog_offset (GL/glext.h:1470)
GL_FOG_OFFSET_SGIX = 33176 	# GL/glext.h:1471
GL_FOG_OFFSET_VALUE_SGIX = 33177 	# GL/glext.h:1472
# HP_image_transform (GL/glext.h:1475)
GL_IMAGE_SCALE_X_HP = 33109 	# GL/glext.h:1476
GL_IMAGE_SCALE_Y_HP = 33110 	# GL/glext.h:1477
GL_IMAGE_TRANSLATE_X_HP = 33111 	# GL/glext.h:1478
GL_IMAGE_TRANSLATE_Y_HP = 33112 	# GL/glext.h:1479
GL_IMAGE_ROTATE_ANGLE_HP = 33113 	# GL/glext.h:1480
GL_IMAGE_ROTATE_ORIGIN_X_HP = 33114 	# GL/glext.h:1481
GL_IMAGE_ROTATE_ORIGIN_Y_HP = 33115 	# GL/glext.h:1482
GL_IMAGE_MAG_FILTER_HP = 33116 	# GL/glext.h:1483
GL_IMAGE_MIN_FILTER_HP = 33117 	# GL/glext.h:1484
GL_IMAGE_CUBIC_WEIGHT_HP = 33118 	# GL/glext.h:1485
GL_CUBIC_HP = 33119 	# GL/glext.h:1486
GL_AVERAGE_HP = 33120 	# GL/glext.h:1487
GL_IMAGE_TRANSFORM_2D_HP = 33121 	# GL/glext.h:1488
GL_POST_IMAGE_TRANSFORM_COLOR_TABLE_HP = 33122 	# GL/glext.h:1489
GL_PROXY_POST_IMAGE_TRANSFORM_COLOR_TABLE_HP = 33123 	# GL/glext.h:1490
# HP_convolution_border_modes (GL/glext.h:1493)
GL_IGNORE_BORDER_HP = 33104 	# GL/glext.h:1494
GL_CONSTANT_BORDER_HP = 33105 	# GL/glext.h:1495
GL_REPLICATE_BORDER_HP = 33107 	# GL/glext.h:1496
GL_CONVOLUTION_BORDER_COLOR_HP = 33108 	# GL/glext.h:1497
# INGR_palette_buffer (GL/glext.h:1500)
# SGIX_texture_add_env (GL/glext.h:1503)
GL_TEXTURE_ENV_BIAS_SGIX = 32958 	# GL/glext.h:1504
# EXT_color_subtable (GL/glext.h:1507)
# PGI_vertex_hints (GL/glext.h:1510)
GL_VERTEX_DATA_HINT_PGI = 107050 	# GL/glext.h:1511
GL_VERTEX_CONSISTENT_HINT_PGI = 107051 	# GL/glext.h:1512
GL_MATERIAL_SIDE_HINT_PGI = 107052 	# GL/glext.h:1513
GL_MAX_VERTEX_HINT_PGI = 107053 	# GL/glext.h:1514
GL_COLOR3_BIT_PGI = 65536 	# GL/glext.h:1515
GL_COLOR4_BIT_PGI = 131072 	# GL/glext.h:1516
GL_EDGEFLAG_BIT_PGI = 262144 	# GL/glext.h:1517
GL_INDEX_BIT_PGI = 524288 	# GL/glext.h:1518
GL_MAT_AMBIENT_BIT_PGI = 1048576 	# GL/glext.h:1519
GL_MAT_AMBIENT_AND_DIFFUSE_BIT_PGI = 2097152 	# GL/glext.h:1520
GL_MAT_DIFFUSE_BIT_PGI = 4194304 	# GL/glext.h:1521
GL_MAT_EMISSION_BIT_PGI = 8388608 	# GL/glext.h:1522
GL_MAT_COLOR_INDEXES_BIT_PGI = 16777216 	# GL/glext.h:1523
GL_MAT_SHININESS_BIT_PGI = 33554432 	# GL/glext.h:1524
GL_MAT_SPECULAR_BIT_PGI = 67108864 	# GL/glext.h:1525
GL_NORMAL_BIT_PGI = 134217728 	# GL/glext.h:1526
GL_TEXCOORD1_BIT_PGI = 268435456 	# GL/glext.h:1527
GL_TEXCOORD2_BIT_PGI = 536870912 	# GL/glext.h:1528
GL_TEXCOORD3_BIT_PGI = 1073741824 	# GL/glext.h:1529
GL_TEXCOORD4_BIT_PGI = 2147483648 	# GL/glext.h:1530
GL_VERTEX23_BIT_PGI = 4 	# GL/glext.h:1531
GL_VERTEX4_BIT_PGI = 8 	# GL/glext.h:1532
# PGI_misc_hints (GL/glext.h:1535)
GL_PREFER_DOUBLEBUFFER_HINT_PGI = 107000 	# GL/glext.h:1536
GL_CONSERVE_MEMORY_HINT_PGI = 107005 	# GL/glext.h:1537
GL_RECLAIM_MEMORY_HINT_PGI = 107006 	# GL/glext.h:1538
GL_NATIVE_GRAPHICS_HANDLE_PGI = 107010 	# GL/glext.h:1539
GL_NATIVE_GRAPHICS_BEGIN_HINT_PGI = 107011 	# GL/glext.h:1540
GL_NATIVE_GRAPHICS_END_HINT_PGI = 107012 	# GL/glext.h:1541
GL_ALWAYS_FAST_HINT_PGI = 107020 	# GL/glext.h:1542
GL_ALWAYS_SOFT_HINT_PGI = 107021 	# GL/glext.h:1543
GL_ALLOW_DRAW_OBJ_HINT_PGI = 107022 	# GL/glext.h:1544
GL_ALLOW_DRAW_WIN_HINT_PGI = 107023 	# GL/glext.h:1545
GL_ALLOW_DRAW_FRG_HINT_PGI = 107024 	# GL/glext.h:1546
GL_ALLOW_DRAW_MEM_HINT_PGI = 107025 	# GL/glext.h:1547
GL_STRICT_DEPTHFUNC_HINT_PGI = 107030 	# GL/glext.h:1548
GL_STRICT_LIGHTING_HINT_PGI = 107031 	# GL/glext.h:1549
GL_STRICT_SCISSOR_HINT_PGI = 107032 	# GL/glext.h:1550
GL_FULL_STIPPLE_HINT_PGI = 107033 	# GL/glext.h:1551
GL_CLIP_NEAR_HINT_PGI = 107040 	# GL/glext.h:1552
GL_CLIP_FAR_HINT_PGI = 107041 	# GL/glext.h:1553
GL_WIDE_LINE_HINT_PGI = 107042 	# GL/glext.h:1554
GL_BACK_NORMALS_HINT_PGI = 107043 	# GL/glext.h:1555
# EXT_paletted_texture (GL/glext.h:1558)
GL_COLOR_INDEX1_EXT = 32994 	# GL/glext.h:1559
GL_COLOR_INDEX2_EXT = 32995 	# GL/glext.h:1560
GL_COLOR_INDEX4_EXT = 32996 	# GL/glext.h:1561
GL_COLOR_INDEX8_EXT = 32997 	# GL/glext.h:1562
GL_COLOR_INDEX12_EXT = 32998 	# GL/glext.h:1563
GL_COLOR_INDEX16_EXT = 32999 	# GL/glext.h:1564
GL_TEXTURE_INDEX_SIZE_EXT = 33005 	# GL/glext.h:1565
# EXT_clip_volume_hint (GL/glext.h:1568)
GL_CLIP_VOLUME_CLIPPING_HINT_EXT = 33008 	# GL/glext.h:1569
# SGIX_list_priority (GL/glext.h:1572)
GL_LIST_PRIORITY_SGIX = 33154 	# GL/glext.h:1573
# SGIX_ir_instrument1 (GL/glext.h:1576)
GL_IR_INSTRUMENT1_SGIX = 33151 	# GL/glext.h:1577
# SGIX_calligraphic_fragment (GL/glext.h:1580)
GL_CALLIGRAPHIC_FRAGMENT_SGIX = 33155 	# GL/glext.h:1581
# SGIX_texture_lod_bias (GL/glext.h:1584)
GL_TEXTURE_LOD_BIAS_S_SGIX = 33166 	# GL/glext.h:1585
GL_TEXTURE_LOD_BIAS_T_SGIX = 33167 	# GL/glext.h:1586
GL_TEXTURE_LOD_BIAS_R_SGIX = 33168 	# GL/glext.h:1587
# SGIX_shadow_ambient (GL/glext.h:1590)
GL_SHADOW_AMBIENT_SGIX = 32959 	# GL/glext.h:1591
# EXT_index_texture (GL/glext.h:1594)
# EXT_index_material (GL/glext.h:1597)
GL_INDEX_MATERIAL_EXT = 33208 	# GL/glext.h:1598
GL_INDEX_MATERIAL_PARAMETER_EXT = 33209 	# GL/glext.h:1599
GL_INDEX_MATERIAL_FACE_EXT = 33210 	# GL/glext.h:1600
# EXT_index_func (GL/glext.h:1603)
GL_INDEX_TEST_EXT = 33205 	# GL/glext.h:1604
GL_INDEX_TEST_FUNC_EXT = 33206 	# GL/glext.h:1605
GL_INDEX_TEST_REF_EXT = 33207 	# GL/glext.h:1606
# EXT_index_array_formats (GL/glext.h:1609)
GL_IUI_V2F_EXT = 33197 	# GL/glext.h:1610
GL_IUI_V3F_EXT = 33198 	# GL/glext.h:1611
GL_IUI_N3F_V2F_EXT = 33199 	# GL/glext.h:1612
GL_IUI_N3F_V3F_EXT = 33200 	# GL/glext.h:1613
GL_T2F_IUI_V2F_EXT = 33201 	# GL/glext.h:1614
GL_T2F_IUI_V3F_EXT = 33202 	# GL/glext.h:1615
GL_T2F_IUI_N3F_V2F_EXT = 33203 	# GL/glext.h:1616
GL_T2F_IUI_N3F_V3F_EXT = 33204 	# GL/glext.h:1617
# EXT_compiled_vertex_array (GL/glext.h:1620)
GL_ARRAY_ELEMENT_LOCK_FIRST_EXT = 33192 	# GL/glext.h:1621
GL_ARRAY_ELEMENT_LOCK_COUNT_EXT = 33193 	# GL/glext.h:1622
# EXT_cull_vertex (GL/glext.h:1625)
GL_CULL_VERTEX_EXT = 33194 	# GL/glext.h:1626
GL_CULL_VERTEX_EYE_POSITION_EXT = 33195 	# GL/glext.h:1627
GL_CULL_VERTEX_OBJECT_POSITION_EXT = 33196 	# GL/glext.h:1628
# SGIX_ycrcb (GL/glext.h:1631)
GL_YCRCB_422_SGIX = 33211 	# GL/glext.h:1632
GL_YCRCB_444_SGIX = 33212 	# GL/glext.h:1633
# SGIX_fragment_lighting (GL/glext.h:1636)
GL_FRAGMENT_LIGHTING_SGIX = 33792 	# GL/glext.h:1637
GL_FRAGMENT_COLOR_MATERIAL_SGIX = 33793 	# GL/glext.h:1638
GL_FRAGMENT_COLOR_MATERIAL_FACE_SGIX = 33794 	# GL/glext.h:1639
GL_FRAGMENT_COLOR_MATERIAL_PARAMETER_SGIX = 33795 	# GL/glext.h:1640
GL_MAX_FRAGMENT_LIGHTS_SGIX = 33796 	# GL/glext.h:1641
GL_MAX_ACTIVE_LIGHTS_SGIX = 33797 	# GL/glext.h:1642
GL_CURRENT_RASTER_NORMAL_SGIX = 33798 	# GL/glext.h:1643
GL_LIGHT_ENV_MODE_SGIX = 33799 	# GL/glext.h:1644
GL_FRAGMENT_LIGHT_MODEL_LOCAL_VIEWER_SGIX = 33800 	# GL/glext.h:1645
GL_FRAGMENT_LIGHT_MODEL_TWO_SIDE_SGIX = 33801 	# GL/glext.h:1646
GL_FRAGMENT_LIGHT_MODEL_AMBIENT_SGIX = 33802 	# GL/glext.h:1647
GL_FRAGMENT_LIGHT_MODEL_NORMAL_INTERPOLATION_SGIX = 33803 	# GL/glext.h:1648
GL_FRAGMENT_LIGHT0_SGIX = 33804 	# GL/glext.h:1649
GL_FRAGMENT_LIGHT1_SGIX = 33805 	# GL/glext.h:1650
GL_FRAGMENT_LIGHT2_SGIX = 33806 	# GL/glext.h:1651
GL_FRAGMENT_LIGHT3_SGIX = 33807 	# GL/glext.h:1652
GL_FRAGMENT_LIGHT4_SGIX = 33808 	# GL/glext.h:1653
GL_FRAGMENT_LIGHT5_SGIX = 33809 	# GL/glext.h:1654
GL_FRAGMENT_LIGHT6_SGIX = 33810 	# GL/glext.h:1655
GL_FRAGMENT_LIGHT7_SGIX = 33811 	# GL/glext.h:1656
# IBM_rasterpos_clip (GL/glext.h:1659)
GL_RASTER_POSITION_UNCLIPPED_IBM = 103010 	# GL/glext.h:1660
# HP_texture_lighting (GL/glext.h:1663)
GL_TEXTURE_LIGHTING_MODE_HP = 33127 	# GL/glext.h:1664
GL_TEXTURE_POST_SPECULAR_HP = 33128 	# GL/glext.h:1665
GL_TEXTURE_PRE_SPECULAR_HP = 33129 	# GL/glext.h:1666
# EXT_draw_range_elements (GL/glext.h:1669)
GL_MAX_ELEMENTS_VERTICES_EXT = 33000 	# GL/glext.h:1670
GL_MAX_ELEMENTS_INDICES_EXT = 33001 	# GL/glext.h:1671
# WIN_phong_shading (GL/glext.h:1674)
GL_PHONG_WIN = 33002 	# GL/glext.h:1675
GL_PHONG_HINT_WIN = 33003 	# GL/glext.h:1676
# WIN_specular_fog (GL/glext.h:1679)
GL_FOG_SPECULAR_TEXTURE_WIN = 33004 	# GL/glext.h:1680
# EXT_light_texture (GL/glext.h:1683)
GL_FRAGMENT_MATERIAL_EXT = 33609 	# GL/glext.h:1684
GL_FRAGMENT_NORMAL_EXT = 33610 	# GL/glext.h:1685
GL_FRAGMENT_COLOR_EXT = 33612 	# GL/glext.h:1686
GL_ATTENUATION_EXT = 33613 	# GL/glext.h:1687
GL_SHADOW_ATTENUATION_EXT = 33614 	# GL/glext.h:1688
GL_TEXTURE_APPLICATION_MODE_EXT = 33615 	# GL/glext.h:1689
GL_TEXTURE_LIGHT_EXT = 33616 	# GL/glext.h:1690
GL_TEXTURE_MATERIAL_FACE_EXT = 33617 	# GL/glext.h:1691
GL_TEXTURE_MATERIAL_PARAMETER_EXT = 33618 	# GL/glext.h:1692
# SGIX_blend_alpha_minmax (GL/glext.h:1696)
GL_ALPHA_MIN_SGIX = 33568 	# GL/glext.h:1697
GL_ALPHA_MAX_SGIX = 33569 	# GL/glext.h:1698
# SGIX_impact_pixel_texture (GL/glext.h:1701)
GL_PIXEL_TEX_GEN_Q_CEILING_SGIX = 33156 	# GL/glext.h:1702
GL_PIXEL_TEX_GEN_Q_ROUND_SGIX = 33157 	# GL/glext.h:1703
GL_PIXEL_TEX_GEN_Q_FLOOR_SGIX = 33158 	# GL/glext.h:1704
GL_PIXEL_TEX_GEN_ALPHA_REPLACE_SGIX = 33159 	# GL/glext.h:1705
GL_PIXEL_TEX_GEN_ALPHA_NO_REPLACE_SGIX = 33160 	# GL/glext.h:1706
GL_PIXEL_TEX_GEN_ALPHA_LS_SGIX = 33161 	# GL/glext.h:1707
GL_PIXEL_TEX_GEN_ALPHA_MS_SGIX = 33162 	# GL/glext.h:1708
# EXT_bgra (GL/glext.h:1711)
GL_BGR_EXT = 32992 	# GL/glext.h:1712
GL_BGRA_EXT = 32993 	# GL/glext.h:1713
# SGIX_async (GL/glext.h:1716)
GL_ASYNC_MARKER_SGIX = 33577 	# GL/glext.h:1717
# SGIX_async_pixel (GL/glext.h:1720)
GL_ASYNC_TEX_IMAGE_SGIX = 33628 	# GL/glext.h:1721
GL_ASYNC_DRAW_PIXELS_SGIX = 33629 	# GL/glext.h:1722
GL_ASYNC_READ_PIXELS_SGIX = 33630 	# GL/glext.h:1723
GL_MAX_ASYNC_TEX_IMAGE_SGIX = 33631 	# GL/glext.h:1724
GL_MAX_ASYNC_DRAW_PIXELS_SGIX = 33632 	# GL/glext.h:1725
GL_MAX_ASYNC_READ_PIXELS_SGIX = 33633 	# GL/glext.h:1726
# SGIX_async_histogram (GL/glext.h:1729)
GL_ASYNC_HISTOGRAM_SGIX = 33580 	# GL/glext.h:1730
GL_MAX_ASYNC_HISTOGRAM_SGIX = 33581 	# GL/glext.h:1731
# INTEL_texture_scissor (GL/glext.h:1734)
# INTEL_parallel_arrays (GL/glext.h:1737)
GL_PARALLEL_ARRAYS_INTEL = 33780 	# GL/glext.h:1738
GL_VERTEX_ARRAY_PARALLEL_POINTERS_INTEL = 33781 	# GL/glext.h:1739
GL_NORMAL_ARRAY_PARALLEL_POINTERS_INTEL = 33782 	# GL/glext.h:1740
GL_COLOR_ARRAY_PARALLEL_POINTERS_INTEL = 33783 	# GL/glext.h:1741
GL_TEXTURE_COORD_ARRAY_PARALLEL_POINTERS_INTEL = 33784 	# GL/glext.h:1742
# HP_occlusion_test (GL/glext.h:1745)
GL_OCCLUSION_TEST_HP = 33125 	# GL/glext.h:1746
GL_OCCLUSION_TEST_RESULT_HP = 33126 	# GL/glext.h:1747
# EXT_pixel_transform (GL/glext.h:1750)
GL_PIXEL_TRANSFORM_2D_EXT = 33584 	# GL/glext.h:1751
GL_PIXEL_MAG_FILTER_EXT = 33585 	# GL/glext.h:1752
GL_PIXEL_MIN_FILTER_EXT = 33586 	# GL/glext.h:1753
GL_PIXEL_CUBIC_WEIGHT_EXT = 33587 	# GL/glext.h:1754
GL_CUBIC_EXT = 33588 	# GL/glext.h:1755
GL_AVERAGE_EXT = 33589 	# GL/glext.h:1756
GL_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT = 33590 	# GL/glext.h:1757
GL_MAX_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT = 33591 	# GL/glext.h:1758
GL_PIXEL_TRANSFORM_2D_MATRIX_EXT = 33592 	# GL/glext.h:1759
# EXT_pixel_transform_color_table (GL/glext.h:1762)
# EXT_shared_texture_palette (GL/glext.h:1765)
GL_SHARED_TEXTURE_PALETTE_EXT = 33275 	# GL/glext.h:1766
# EXT_separate_specular_color (GL/glext.h:1769)
GL_LIGHT_MODEL_COLOR_CONTROL_EXT = 33272 	# GL/glext.h:1770
GL_SINGLE_COLOR_EXT = 33273 	# GL/glext.h:1771
GL_SEPARATE_SPECULAR_COLOR_EXT = 33274 	# GL/glext.h:1772
# EXT_secondary_color (GL/glext.h:1775)
GL_COLOR_SUM_EXT = 33880 	# GL/glext.h:1776
GL_CURRENT_SECONDARY_COLOR_EXT = 33881 	# GL/glext.h:1777
GL_SECONDARY_COLOR_ARRAY_SIZE_EXT = 33882 	# GL/glext.h:1778
GL_SECONDARY_COLOR_ARRAY_TYPE_EXT = 33883 	# GL/glext.h:1779
GL_SECONDARY_COLOR_ARRAY_STRIDE_EXT = 33884 	# GL/glext.h:1780
GL_SECONDARY_COLOR_ARRAY_POINTER_EXT = 33885 	# GL/glext.h:1781
GL_SECONDARY_COLOR_ARRAY_EXT = 33886 	# GL/glext.h:1782
# EXT_texture_perturb_normal (GL/glext.h:1785)
GL_PERTURB_EXT = 34222 	# GL/glext.h:1786
GL_TEXTURE_NORMAL_EXT = 34223 	# GL/glext.h:1787
# EXT_multi_draw_arrays (GL/glext.h:1790)
# EXT_fog_coord (GL/glext.h:1793)
GL_FOG_COORDINATE_SOURCE_EXT = 33872 	# GL/glext.h:1794
GL_FOG_COORDINATE_EXT = 33873 	# GL/glext.h:1795
GL_FRAGMENT_DEPTH_EXT = 33874 	# GL/glext.h:1796
GL_CURRENT_FOG_COORDINATE_EXT = 33875 	# GL/glext.h:1797
GL_FOG_COORDINATE_ARRAY_TYPE_EXT = 33876 	# GL/glext.h:1798
GL_FOG_COORDINATE_ARRAY_STRIDE_EXT = 33877 	# GL/glext.h:1799
GL_FOG_COORDINATE_ARRAY_POINTER_EXT = 33878 	# GL/glext.h:1800
GL_FOG_COORDINATE_ARRAY_EXT = 33879 	# GL/glext.h:1801
# REND_screen_coordinates (GL/glext.h:1804)
GL_SCREEN_COORDINATES_REND = 33936 	# GL/glext.h:1805
GL_INVERTED_SCREEN_W_REND = 33937 	# GL/glext.h:1806
# EXT_coordinate_frame (GL/glext.h:1809)
GL_TANGENT_ARRAY_EXT = 33849 	# GL/glext.h:1810
GL_BINORMAL_ARRAY_EXT = 33850 	# GL/glext.h:1811
GL_CURRENT_TANGENT_EXT = 33851 	# GL/glext.h:1812
GL_CURRENT_BINORMAL_EXT = 33852 	# GL/glext.h:1813
GL_TANGENT_ARRAY_TYPE_EXT = 33854 	# GL/glext.h:1814
GL_TANGENT_ARRAY_STRIDE_EXT = 33855 	# GL/glext.h:1815
GL_BINORMAL_ARRAY_TYPE_EXT = 33856 	# GL/glext.h:1816
GL_BINORMAL_ARRAY_STRIDE_EXT = 33857 	# GL/glext.h:1817
GL_TANGENT_ARRAY_POINTER_EXT = 33858 	# GL/glext.h:1818
GL_BINORMAL_ARRAY_POINTER_EXT = 33859 	# GL/glext.h:1819
GL_MAP1_TANGENT_EXT = 33860 	# GL/glext.h:1820
GL_MAP2_TANGENT_EXT = 33861 	# GL/glext.h:1821
GL_MAP1_BINORMAL_EXT = 33862 	# GL/glext.h:1822
GL_MAP2_BINORMAL_EXT = 33863 	# GL/glext.h:1823
# EXT_texture_env_combine (GL/glext.h:1826)
GL_COMBINE_EXT = 34160 	# GL/glext.h:1827
GL_COMBINE_RGB_EXT = 34161 	# GL/glext.h:1828
GL_COMBINE_ALPHA_EXT = 34162 	# GL/glext.h:1829
GL_RGB_SCALE_EXT = 34163 	# GL/glext.h:1830
GL_ADD_SIGNED_EXT = 34164 	# GL/glext.h:1831
GL_INTERPOLATE_EXT = 34165 	# GL/glext.h:1832
GL_CONSTANT_EXT = 34166 	# GL/glext.h:1833
GL_PRIMARY_COLOR_EXT = 34167 	# GL/glext.h:1834
GL_PREVIOUS_EXT = 34168 	# GL/glext.h:1835
GL_SOURCE0_RGB_EXT = 34176 	# GL/glext.h:1836
GL_SOURCE1_RGB_EXT = 34177 	# GL/glext.h:1837
GL_SOURCE2_RGB_EXT = 34178 	# GL/glext.h:1838
GL_SOURCE0_ALPHA_EXT = 34184 	# GL/glext.h:1839
GL_SOURCE1_ALPHA_EXT = 34185 	# GL/glext.h:1840
GL_SOURCE2_ALPHA_EXT = 34186 	# GL/glext.h:1841
GL_OPERAND0_RGB_EXT = 34192 	# GL/glext.h:1842
GL_OPERAND1_RGB_EXT = 34193 	# GL/glext.h:1843
GL_OPERAND2_RGB_EXT = 34194 	# GL/glext.h:1844
GL_OPERAND0_ALPHA_EXT = 34200 	# GL/glext.h:1845
GL_OPERAND1_ALPHA_EXT = 34201 	# GL/glext.h:1846
GL_OPERAND2_ALPHA_EXT = 34202 	# GL/glext.h:1847
# APPLE_specular_vector (GL/glext.h:1850)
GL_LIGHT_MODEL_SPECULAR_VECTOR_APPLE = 34224 	# GL/glext.h:1851
# APPLE_transform_hint (GL/glext.h:1854)
GL_TRANSFORM_HINT_APPLE = 34225 	# GL/glext.h:1855
# SGIX_fog_scale (GL/glext.h:1858)
GL_FOG_SCALE_SGIX = 33276 	# GL/glext.h:1859
GL_FOG_SCALE_VALUE_SGIX = 33277 	# GL/glext.h:1860
# SUNX_constant_data (GL/glext.h:1863)
GL_UNPACK_CONSTANT_DATA_SUNX = 33237 	# GL/glext.h:1864
GL_TEXTURE_CONSTANT_DATA_SUNX = 33238 	# GL/glext.h:1865
# SUN_global_alpha (GL/glext.h:1868)
GL_GLOBAL_ALPHA_SUN = 33241 	# GL/glext.h:1869
GL_GLOBAL_ALPHA_FACTOR_SUN = 33242 	# GL/glext.h:1870
# SUN_triangle_list (GL/glext.h:1873)
GL_RESTART_SUN = 1 	# GL/glext.h:1874
GL_REPLACE_MIDDLE_SUN = 2 	# GL/glext.h:1875
GL_REPLACE_OLDEST_SUN = 3 	# GL/glext.h:1876
GL_TRIANGLE_LIST_SUN = 33239 	# GL/glext.h:1877
GL_REPLACEMENT_CODE_SUN = 33240 	# GL/glext.h:1878
GL_REPLACEMENT_CODE_ARRAY_SUN = 34240 	# GL/glext.h:1879
GL_REPLACEMENT_CODE_ARRAY_TYPE_SUN = 34241 	# GL/glext.h:1880
GL_REPLACEMENT_CODE_ARRAY_STRIDE_SUN = 34242 	# GL/glext.h:1881
GL_REPLACEMENT_CODE_ARRAY_POINTER_SUN = 34243 	# GL/glext.h:1882
GL_R1UI_V3F_SUN = 34244 	# GL/glext.h:1883
GL_R1UI_C4UB_V3F_SUN = 34245 	# GL/glext.h:1884
GL_R1UI_C3F_V3F_SUN = 34246 	# GL/glext.h:1885
GL_R1UI_N3F_V3F_SUN = 34247 	# GL/glext.h:1886
GL_R1UI_C4F_N3F_V3F_SUN = 34248 	# GL/glext.h:1887
GL_R1UI_T2F_V3F_SUN = 34249 	# GL/glext.h:1888
GL_R1UI_T2F_N3F_V3F_SUN = 34250 	# GL/glext.h:1889
GL_R1UI_T2F_C4F_N3F_V3F_SUN = 34251 	# GL/glext.h:1890
# SUN_vertex (GL/glext.h:1893)
# EXT_blend_func_separate (GL/glext.h:1896)
GL_BLEND_DST_RGB_EXT = 32968 	# GL/glext.h:1897
GL_BLEND_SRC_RGB_EXT = 32969 	# GL/glext.h:1898
GL_BLEND_DST_ALPHA_EXT = 32970 	# GL/glext.h:1899
GL_BLEND_SRC_ALPHA_EXT = 32971 	# GL/glext.h:1900
# INGR_color_clamp (GL/glext.h:1903)
GL_RED_MIN_CLAMP_INGR = 34144 	# GL/glext.h:1904
GL_GREEN_MIN_CLAMP_INGR = 34145 	# GL/glext.h:1905
GL_BLUE_MIN_CLAMP_INGR = 34146 	# GL/glext.h:1906
GL_ALPHA_MIN_CLAMP_INGR = 34147 	# GL/glext.h:1907
GL_RED_MAX_CLAMP_INGR = 34148 	# GL/glext.h:1908
GL_GREEN_MAX_CLAMP_INGR = 34149 	# GL/glext.h:1909
GL_BLUE_MAX_CLAMP_INGR = 34150 	# GL/glext.h:1910
GL_ALPHA_MAX_CLAMP_INGR = 34151 	# GL/glext.h:1911
# INGR_interlace_read (GL/glext.h:1914)
GL_INTERLACE_READ_INGR = 34152 	# GL/glext.h:1915
# EXT_stencil_wrap (GL/glext.h:1918)
GL_INCR_WRAP_EXT = 34055 	# GL/glext.h:1919
GL_DECR_WRAP_EXT = 34056 	# GL/glext.h:1920
# EXT_422_pixels (GL/glext.h:1923)
GL_422_EXT = 32972 	# GL/glext.h:1924
GL_422_REV_EXT = 32973 	# GL/glext.h:1925
GL_422_AVERAGE_EXT = 32974 	# GL/glext.h:1926
GL_422_REV_AVERAGE_EXT = 32975 	# GL/glext.h:1927
# NV_texgen_reflection (GL/glext.h:1930)
GL_NORMAL_MAP_NV = 34065 	# GL/glext.h:1931
GL_REFLECTION_MAP_NV = 34066 	# GL/glext.h:1932
# EXT_texture_cube_map (GL/glext.h:1935)
GL_NORMAL_MAP_EXT = 34065 	# GL/glext.h:1936
GL_REFLECTION_MAP_EXT = 34066 	# GL/glext.h:1937
GL_TEXTURE_CUBE_MAP_EXT = 34067 	# GL/glext.h:1938
GL_TEXTURE_BINDING_CUBE_MAP_EXT = 34068 	# GL/glext.h:1939
GL_TEXTURE_CUBE_MAP_POSITIVE_X_EXT = 34069 	# GL/glext.h:1940
GL_TEXTURE_CUBE_MAP_NEGATIVE_X_EXT = 34070 	# GL/glext.h:1941
GL_TEXTURE_CUBE_MAP_POSITIVE_Y_EXT = 34071 	# GL/glext.h:1942
GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_EXT = 34072 	# GL/glext.h:1943
GL_TEXTURE_CUBE_MAP_POSITIVE_Z_EXT = 34073 	# GL/glext.h:1944
GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_EXT = 34074 	# GL/glext.h:1945
GL_PROXY_TEXTURE_CUBE_MAP_EXT = 34075 	# GL/glext.h:1946
GL_MAX_CUBE_MAP_TEXTURE_SIZE_EXT = 34076 	# GL/glext.h:1947
# SUN_convolution_border_modes (GL/glext.h:1950)
GL_WRAP_BORDER_SUN = 33236 	# GL/glext.h:1951
# EXT_texture_env_add (GL/glext.h:1954)
# EXT_texture_lod_bias (GL/glext.h:1957)
GL_MAX_TEXTURE_LOD_BIAS_EXT = 34045 	# GL/glext.h:1958
GL_TEXTURE_FILTER_CONTROL_EXT = 34048 	# GL/glext.h:1959
GL_TEXTURE_LOD_BIAS_EXT = 34049 	# GL/glext.h:1960
# EXT_texture_filter_anisotropic (GL/glext.h:1963)
GL_TEXTURE_MAX_ANISOTROPY_EXT = 34046 	# GL/glext.h:1964
GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT = 34047 	# GL/glext.h:1965
# EXT_vertex_weighting (GL/glext.h:1968)
GL_MODELVIEW0_STACK_DEPTH_EXT = 0 	# GL/glext.h:1969
GL_MODELVIEW1_STACK_DEPTH_EXT = 34050 	# GL/glext.h:1970
GL_MODELVIEW0_MATRIX_EXT = 0 	# GL/glext.h:1971
GL_MODELVIEW1_MATRIX_EXT = 34054 	# GL/glext.h:1972
GL_VERTEX_WEIGHTING_EXT = 34057 	# GL/glext.h:1973
GL_MODELVIEW0_EXT = 0 	# GL/glext.h:1974
GL_MODELVIEW1_EXT = 34058 	# GL/glext.h:1975
GL_CURRENT_VERTEX_WEIGHT_EXT = 34059 	# GL/glext.h:1976
GL_VERTEX_WEIGHT_ARRAY_EXT = 34060 	# GL/glext.h:1977
GL_VERTEX_WEIGHT_ARRAY_SIZE_EXT = 34061 	# GL/glext.h:1978
GL_VERTEX_WEIGHT_ARRAY_TYPE_EXT = 34062 	# GL/glext.h:1979
GL_VERTEX_WEIGHT_ARRAY_STRIDE_EXT = 34063 	# GL/glext.h:1980
GL_VERTEX_WEIGHT_ARRAY_POINTER_EXT = 34064 	# GL/glext.h:1981
# NV_light_max_exponent (GL/glext.h:1984)
GL_MAX_SHININESS_NV = 34052 	# GL/glext.h:1985
GL_MAX_SPOT_EXPONENT_NV = 34053 	# GL/glext.h:1986
# NV_vertex_array_range (GL/glext.h:1989)
GL_VERTEX_ARRAY_RANGE_NV = 34077 	# GL/glext.h:1990
GL_VERTEX_ARRAY_RANGE_LENGTH_NV = 34078 	# GL/glext.h:1991
GL_VERTEX_ARRAY_RANGE_VALID_NV = 34079 	# GL/glext.h:1992
GL_MAX_VERTEX_ARRAY_RANGE_ELEMENT_NV = 34080 	# GL/glext.h:1993
GL_VERTEX_ARRAY_RANGE_POINTER_NV = 34081 	# GL/glext.h:1994
# NV_register_combiners (GL/glext.h:1997)
GL_REGISTER_COMBINERS_NV = 34082 	# GL/glext.h:1998
GL_VARIABLE_A_NV = 34083 	# GL/glext.h:1999
GL_VARIABLE_B_NV = 34084 	# GL/glext.h:2000
GL_VARIABLE_C_NV = 34085 	# GL/glext.h:2001
GL_VARIABLE_D_NV = 34086 	# GL/glext.h:2002
GL_VARIABLE_E_NV = 34087 	# GL/glext.h:2003
GL_VARIABLE_F_NV = 34088 	# GL/glext.h:2004
GL_VARIABLE_G_NV = 34089 	# GL/glext.h:2005
GL_CONSTANT_COLOR0_NV = 34090 	# GL/glext.h:2006
GL_CONSTANT_COLOR1_NV = 34091 	# GL/glext.h:2007
GL_PRIMARY_COLOR_NV = 34092 	# GL/glext.h:2008
GL_SECONDARY_COLOR_NV = 34093 	# GL/glext.h:2009
GL_SPARE0_NV = 34094 	# GL/glext.h:2010
GL_SPARE1_NV = 34095 	# GL/glext.h:2011
GL_DISCARD_NV = 34096 	# GL/glext.h:2012
GL_E_TIMES_F_NV = 34097 	# GL/glext.h:2013
GL_SPARE0_PLUS_SECONDARY_COLOR_NV = 34098 	# GL/glext.h:2014
GL_UNSIGNED_IDENTITY_NV = 34102 	# GL/glext.h:2015
GL_UNSIGNED_INVERT_NV = 34103 	# GL/glext.h:2016
GL_EXPAND_NORMAL_NV = 34104 	# GL/glext.h:2017
GL_EXPAND_NEGATE_NV = 34105 	# GL/glext.h:2018
GL_HALF_BIAS_NORMAL_NV = 34106 	# GL/glext.h:2019
GL_HALF_BIAS_NEGATE_NV = 34107 	# GL/glext.h:2020
GL_SIGNED_IDENTITY_NV = 34108 	# GL/glext.h:2021
GL_SIGNED_NEGATE_NV = 34109 	# GL/glext.h:2022
GL_SCALE_BY_TWO_NV = 34110 	# GL/glext.h:2023
GL_SCALE_BY_FOUR_NV = 34111 	# GL/glext.h:2024
GL_SCALE_BY_ONE_HALF_NV = 34112 	# GL/glext.h:2025
GL_BIAS_BY_NEGATIVE_ONE_HALF_NV = 34113 	# GL/glext.h:2026
GL_COMBINER_INPUT_NV = 34114 	# GL/glext.h:2027
GL_COMBINER_MAPPING_NV = 34115 	# GL/glext.h:2028
GL_COMBINER_COMPONENT_USAGE_NV = 34116 	# GL/glext.h:2029
GL_COMBINER_AB_DOT_PRODUCT_NV = 34117 	# GL/glext.h:2030
GL_COMBINER_CD_DOT_PRODUCT_NV = 34118 	# GL/glext.h:2031
GL_COMBINER_MUX_SUM_NV = 34119 	# GL/glext.h:2032
GL_COMBINER_SCALE_NV = 34120 	# GL/glext.h:2033
GL_COMBINER_BIAS_NV = 34121 	# GL/glext.h:2034
GL_COMBINER_AB_OUTPUT_NV = 34122 	# GL/glext.h:2035
GL_COMBINER_CD_OUTPUT_NV = 34123 	# GL/glext.h:2036
GL_COMBINER_SUM_OUTPUT_NV = 34124 	# GL/glext.h:2037
GL_MAX_GENERAL_COMBINERS_NV = 34125 	# GL/glext.h:2038
GL_NUM_GENERAL_COMBINERS_NV = 34126 	# GL/glext.h:2039
GL_COLOR_SUM_CLAMP_NV = 34127 	# GL/glext.h:2040
GL_COMBINER0_NV = 34128 	# GL/glext.h:2041
GL_COMBINER1_NV = 34129 	# GL/glext.h:2042
GL_COMBINER2_NV = 34130 	# GL/glext.h:2043
GL_COMBINER3_NV = 34131 	# GL/glext.h:2044
GL_COMBINER4_NV = 34132 	# GL/glext.h:2045
GL_COMBINER5_NV = 34133 	# GL/glext.h:2046
GL_COMBINER6_NV = 34134 	# GL/glext.h:2047
GL_COMBINER7_NV = 34135 	# GL/glext.h:2048
# NV_fog_distance (GL/glext.h:2056)
GL_FOG_DISTANCE_MODE_NV = 34138 	# GL/glext.h:2057
GL_EYE_RADIAL_NV = 34139 	# GL/glext.h:2058
GL_EYE_PLANE_ABSOLUTE_NV = 34140 	# GL/glext.h:2059
# NV_texgen_emboss (GL/glext.h:2063)
GL_EMBOSS_LIGHT_NV = 34141 	# GL/glext.h:2064
GL_EMBOSS_CONSTANT_NV = 34142 	# GL/glext.h:2065
GL_EMBOSS_MAP_NV = 34143 	# GL/glext.h:2066
# NV_blend_square (GL/glext.h:2069)
# NV_texture_env_combine4 (GL/glext.h:2072)
GL_COMBINE4_NV = 34051 	# GL/glext.h:2073
GL_SOURCE3_RGB_NV = 34179 	# GL/glext.h:2074
GL_SOURCE3_ALPHA_NV = 34187 	# GL/glext.h:2075
GL_OPERAND3_RGB_NV = 34195 	# GL/glext.h:2076
GL_OPERAND3_ALPHA_NV = 34203 	# GL/glext.h:2077
# MESA_resize_buffers (GL/glext.h:2080)
# MESA_window_pos (GL/glext.h:2083)
# EXT_texture_compression_s3tc (GL/glext.h:2086)
GL_COMPRESSED_RGB_S3TC_DXT1_EXT = 33776 	# GL/glext.h:2087
GL_COMPRESSED_RGBA_S3TC_DXT1_EXT = 33777 	# GL/glext.h:2088
GL_COMPRESSED_RGBA_S3TC_DXT3_EXT = 33778 	# GL/glext.h:2089
GL_COMPRESSED_RGBA_S3TC_DXT5_EXT = 33779 	# GL/glext.h:2090
# IBM_cull_vertex (GL/glext.h:2093)
GL_CULL_VERTEX_IBM = 103050 	# GL/glext.h:2094
# IBM_multimode_draw_arrays (GL/glext.h:2097)
# IBM_vertex_array_lists (GL/glext.h:2100)
GL_VERTEX_ARRAY_LIST_IBM = 103070 	# GL/glext.h:2101
GL_NORMAL_ARRAY_LIST_IBM = 103071 	# GL/glext.h:2102
GL_COLOR_ARRAY_LIST_IBM = 103072 	# GL/glext.h:2103
GL_INDEX_ARRAY_LIST_IBM = 103073 	# GL/glext.h:2104
GL_TEXTURE_COORD_ARRAY_LIST_IBM = 103074 	# GL/glext.h:2105
GL_EDGE_FLAG_ARRAY_LIST_IBM = 103075 	# GL/glext.h:2106
GL_FOG_COORDINATE_ARRAY_LIST_IBM = 103076 	# GL/glext.h:2107
GL_SECONDARY_COLOR_ARRAY_LIST_IBM = 103077 	# GL/glext.h:2108
GL_VERTEX_ARRAY_LIST_STRIDE_IBM = 103080 	# GL/glext.h:2109
GL_NORMAL_ARRAY_LIST_STRIDE_IBM = 103081 	# GL/glext.h:2110
GL_COLOR_ARRAY_LIST_STRIDE_IBM = 103082 	# GL/glext.h:2111
GL_INDEX_ARRAY_LIST_STRIDE_IBM = 103083 	# GL/glext.h:2112
GL_TEXTURE_COORD_ARRAY_LIST_STRIDE_IBM = 103084 	# GL/glext.h:2113
GL_EDGE_FLAG_ARRAY_LIST_STRIDE_IBM = 103085 	# GL/glext.h:2114
GL_FOG_COORDINATE_ARRAY_LIST_STRIDE_IBM = 103086 	# GL/glext.h:2115
GL_SECONDARY_COLOR_ARRAY_LIST_STRIDE_IBM = 103087 	# GL/glext.h:2116
# SGIX_subsample (GL/glext.h:2119)
GL_PACK_SUBSAMPLE_RATE_SGIX = 34208 	# GL/glext.h:2120
GL_UNPACK_SUBSAMPLE_RATE_SGIX = 34209 	# GL/glext.h:2121
GL_PIXEL_SUBSAMPLE_4444_SGIX = 34210 	# GL/glext.h:2122
GL_PIXEL_SUBSAMPLE_2424_SGIX = 34211 	# GL/glext.h:2123
GL_PIXEL_SUBSAMPLE_4242_SGIX = 34212 	# GL/glext.h:2124
# SGIX_ycrcb_subsample (GL/glext.h:2127)
# SGIX_ycrcba (GL/glext.h:2130)
GL_YCRCB_SGIX = 33560 	# GL/glext.h:2131
GL_YCRCBA_SGIX = 33561 	# GL/glext.h:2132
# SGI_depth_pass_instrument (GL/glext.h:2135)
GL_DEPTH_PASS_INSTRUMENT_SGIX = 33552 	# GL/glext.h:2136
GL_DEPTH_PASS_INSTRUMENT_COUNTERS_SGIX = 33553 	# GL/glext.h:2137
GL_DEPTH_PASS_INSTRUMENT_MAX_SGIX = 33554 	# GL/glext.h:2138
# 3DFX_texture_compression_FXT1 (GL/glext.h:2141)
GL_COMPRESSED_RGB_FXT1_3DFX = 34480 	# GL/glext.h:2142
GL_COMPRESSED_RGBA_FXT1_3DFX = 34481 	# GL/glext.h:2143
# 3DFX_multisample (GL/glext.h:2146)
GL_MULTISAMPLE_3DFX = 34482 	# GL/glext.h:2147
GL_SAMPLE_BUFFERS_3DFX = 34483 	# GL/glext.h:2148
GL_SAMPLES_3DFX = 34484 	# GL/glext.h:2149
GL_MULTISAMPLE_BIT_3DFX = 536870912 	# GL/glext.h:2150
# 3DFX_tbuffer (GL/glext.h:2153)
# EXT_multisample (GL/glext.h:2156)
GL_MULTISAMPLE_EXT = 32925 	# GL/glext.h:2157
GL_SAMPLE_ALPHA_TO_MASK_EXT = 32926 	# GL/glext.h:2158
GL_SAMPLE_ALPHA_TO_ONE_EXT = 32927 	# GL/glext.h:2159
GL_SAMPLE_MASK_EXT = 32928 	# GL/glext.h:2160
GL_1PASS_EXT = 32929 	# GL/glext.h:2161
GL_2PASS_0_EXT = 32930 	# GL/glext.h:2162
GL_2PASS_1_EXT = 32931 	# GL/glext.h:2163
GL_4PASS_0_EXT = 32932 	# GL/glext.h:2164
GL_4PASS_1_EXT = 32933 	# GL/glext.h:2165
GL_4PASS_2_EXT = 32934 	# GL/glext.h:2166
GL_4PASS_3_EXT = 32935 	# GL/glext.h:2167
GL_SAMPLE_BUFFERS_EXT = 32936 	# GL/glext.h:2168
GL_SAMPLES_EXT = 32937 	# GL/glext.h:2169
GL_SAMPLE_MASK_VALUE_EXT = 32938 	# GL/glext.h:2170
GL_SAMPLE_MASK_INVERT_EXT = 32939 	# GL/glext.h:2171
GL_SAMPLE_PATTERN_EXT = 32940 	# GL/glext.h:2172
GL_MULTISAMPLE_BIT_EXT = 536870912 	# GL/glext.h:2173
# SGIX_vertex_preclip (GL/glext.h:2176)
GL_VERTEX_PRECLIP_SGIX = 33774 	# GL/glext.h:2177
GL_VERTEX_PRECLIP_HINT_SGIX = 33775 	# GL/glext.h:2178
# SGIX_convolution_accuracy (GL/glext.h:2181)
GL_CONVOLUTION_HINT_SGIX = 33558 	# GL/glext.h:2182
# SGIX_resample (GL/glext.h:2185)
GL_PACK_RESAMPLE_SGIX = 33836 	# GL/glext.h:2186
GL_UNPACK_RESAMPLE_SGIX = 33837 	# GL/glext.h:2187
GL_RESAMPLE_REPLICATE_SGIX = 33838 	# GL/glext.h:2188
GL_RESAMPLE_ZERO_FILL_SGIX = 33839 	# GL/glext.h:2189
GL_RESAMPLE_DECIMATE_SGIX = 33840 	# GL/glext.h:2190
# SGIS_point_line_texgen (GL/glext.h:2193)
GL_EYE_DISTANCE_TO_POINT_SGIS = 33264 	# GL/glext.h:2194
GL_OBJECT_DISTANCE_TO_POINT_SGIS = 33265 	# GL/glext.h:2195
GL_EYE_DISTANCE_TO_LINE_SGIS = 33266 	# GL/glext.h:2196
GL_OBJECT_DISTANCE_TO_LINE_SGIS = 33267 	# GL/glext.h:2197
GL_EYE_POINT_SGIS = 33268 	# GL/glext.h:2198
GL_OBJECT_POINT_SGIS = 33269 	# GL/glext.h:2199
GL_EYE_LINE_SGIS = 33270 	# GL/glext.h:2200
GL_OBJECT_LINE_SGIS = 33271 	# GL/glext.h:2201
# SGIS_texture_color_mask (GL/glext.h:2204)
GL_TEXTURE_COLOR_WRITEMASK_SGIS = 33263 	# GL/glext.h:2205
# EXT_texture_env_dot3 (GL/glext.h:2208)
GL_DOT3_RGB_EXT = 34624 	# GL/glext.h:2209
GL_DOT3_RGBA_EXT = 34625 	# GL/glext.h:2210
# ATI_texture_mirror_once (GL/glext.h:2213)
GL_MIRROR_CLAMP_ATI = 34626 	# GL/glext.h:2214
GL_MIRROR_CLAMP_TO_EDGE_ATI = 34627 	# GL/glext.h:2215
# NV_fence (GL/glext.h:2218)
GL_ALL_COMPLETED_NV = 34034 	# GL/glext.h:2219
GL_FENCE_STATUS_NV = 34035 	# GL/glext.h:2220
GL_FENCE_CONDITION_NV = 34036 	# GL/glext.h:2221
# IBM_texture_mirrored_repeat (GL/glext.h:2224)
GL_MIRRORED_REPEAT_IBM = 33648 	# GL/glext.h:2225
# NV_evaluators (GL/glext.h:2228)
GL_EVAL_2D_NV = 34496 	# GL/glext.h:2229
GL_EVAL_TRIANGULAR_2D_NV = 34497 	# GL/glext.h:2230
GL_MAP_TESSELLATION_NV = 34498 	# GL/glext.h:2231
GL_MAP_ATTRIB_U_ORDER_NV = 34499 	# GL/glext.h:2232
GL_MAP_ATTRIB_V_ORDER_NV = 34500 	# GL/glext.h:2233
GL_EVAL_FRACTIONAL_TESSELLATION_NV = 34501 	# GL/glext.h:2234
GL_EVAL_VERTEX_ATTRIB0_NV = 34502 	# GL/glext.h:2235
GL_EVAL_VERTEX_ATTRIB1_NV = 34503 	# GL/glext.h:2236
GL_EVAL_VERTEX_ATTRIB2_NV = 34504 	# GL/glext.h:2237
GL_EVAL_VERTEX_ATTRIB3_NV = 34505 	# GL/glext.h:2238
GL_EVAL_VERTEX_ATTRIB4_NV = 34506 	# GL/glext.h:2239
GL_EVAL_VERTEX_ATTRIB5_NV = 34507 	# GL/glext.h:2240
GL_EVAL_VERTEX_ATTRIB6_NV = 34508 	# GL/glext.h:2241
GL_EVAL_VERTEX_ATTRIB7_NV = 34509 	# GL/glext.h:2242
GL_EVAL_VERTEX_ATTRIB8_NV = 34510 	# GL/glext.h:2243
GL_EVAL_VERTEX_ATTRIB9_NV = 34511 	# GL/glext.h:2244
GL_EVAL_VERTEX_ATTRIB10_NV = 34512 	# GL/glext.h:2245
GL_EVAL_VERTEX_ATTRIB11_NV = 34513 	# GL/glext.h:2246
GL_EVAL_VERTEX_ATTRIB12_NV = 34514 	# GL/glext.h:2247
GL_EVAL_VERTEX_ATTRIB13_NV = 34515 	# GL/glext.h:2248
GL_EVAL_VERTEX_ATTRIB14_NV = 34516 	# GL/glext.h:2249
GL_EVAL_VERTEX_ATTRIB15_NV = 34517 	# GL/glext.h:2250
GL_MAX_MAP_TESSELLATION_NV = 34518 	# GL/glext.h:2251
GL_MAX_RATIONAL_EVAL_ORDER_NV = 34519 	# GL/glext.h:2252
# NV_packed_depth_stencil (GL/glext.h:2255)
GL_DEPTH_STENCIL_NV = 34041 	# GL/glext.h:2256
GL_UNSIGNED_INT_24_8_NV = 34042 	# GL/glext.h:2257
# NV_register_combiners2 (GL/glext.h:2260)
GL_PER_STAGE_CONSTANTS_NV = 34101 	# GL/glext.h:2261
# NV_texture_compression_vtc (GL/glext.h:2264)
# NV_texture_rectangle (GL/glext.h:2267)
GL_TEXTURE_RECTANGLE_NV = 34037 	# GL/glext.h:2268
GL_TEXTURE_BINDING_RECTANGLE_NV = 34038 	# GL/glext.h:2269
GL_PROXY_TEXTURE_RECTANGLE_NV = 34039 	# GL/glext.h:2270
GL_MAX_RECTANGLE_TEXTURE_SIZE_NV = 34040 	# GL/glext.h:2271
# NV_texture_shader (GL/glext.h:2274)
GL_OFFSET_TEXTURE_RECTANGLE_NV = 34380 	# GL/glext.h:2275
GL_OFFSET_TEXTURE_RECTANGLE_SCALE_NV = 34381 	# GL/glext.h:2276
GL_DOT_PRODUCT_TEXTURE_RECTANGLE_NV = 34382 	# GL/glext.h:2277
GL_RGBA_UNSIGNED_DOT_PRODUCT_MAPPING_NV = 34521 	# GL/glext.h:2278
GL_UNSIGNED_INT_S8_S8_8_8_NV = 34522 	# GL/glext.h:2279
GL_UNSIGNED_INT_8_8_S8_S8_REV_NV = 34523 	# GL/glext.h:2280
GL_DSDT_MAG_INTENSITY_NV = 34524 	# GL/glext.h:2281
GL_SHADER_CONSISTENT_NV = 34525 	# GL/glext.h:2282
GL_TEXTURE_SHADER_NV = 34526 	# GL/glext.h:2283
GL_SHADER_OPERATION_NV = 34527 	# GL/glext.h:2284
GL_CULL_MODES_NV = 34528 	# GL/glext.h:2285
GL_OFFSET_TEXTURE_MATRIX_NV = 34529 	# GL/glext.h:2286
GL_OFFSET_TEXTURE_SCALE_NV = 34530 	# GL/glext.h:2287
GL_OFFSET_TEXTURE_BIAS_NV = 34531 	# GL/glext.h:2288
GL_OFFSET_TEXTURE_2D_MATRIX_NV = 34529 	# GL/glext.h:2289
GL_OFFSET_TEXTURE_2D_SCALE_NV = 34530 	# GL/glext.h:2290
GL_OFFSET_TEXTURE_2D_BIAS_NV = 34531 	# GL/glext.h:2291
GL_PREVIOUS_TEXTURE_INPUT_NV = 34532 	# GL/glext.h:2292
GL_CONST_EYE_NV = 34533 	# GL/glext.h:2293
GL_PASS_THROUGH_NV = 34534 	# GL/glext.h:2294
GL_CULL_FRAGMENT_NV = 34535 	# GL/glext.h:2295
GL_OFFSET_TEXTURE_2D_NV = 34536 	# GL/glext.h:2296
GL_DEPENDENT_AR_TEXTURE_2D_NV = 34537 	# GL/glext.h:2297
GL_DEPENDENT_GB_TEXTURE_2D_NV = 34538 	# GL/glext.h:2298
GL_DOT_PRODUCT_NV = 34540 	# GL/glext.h:2299
GL_DOT_PRODUCT_DEPTH_REPLACE_NV = 34541 	# GL/glext.h:2300
GL_DOT_PRODUCT_TEXTURE_2D_NV = 34542 	# GL/glext.h:2301
GL_DOT_PRODUCT_TEXTURE_CUBE_MAP_NV = 34544 	# GL/glext.h:2302
GL_DOT_PRODUCT_DIFFUSE_CUBE_MAP_NV = 34545 	# GL/glext.h:2303
GL_DOT_PRODUCT_REFLECT_CUBE_MAP_NV = 34546 	# GL/glext.h:2304
GL_DOT_PRODUCT_CONST_EYE_REFLECT_CUBE_MAP_NV = 34547 	# GL/glext.h:2305
GL_HILO_NV = 34548 	# GL/glext.h:2306
GL_DSDT_NV = 34549 	# GL/glext.h:2307
GL_DSDT_MAG_NV = 34550 	# GL/glext.h:2308
GL_DSDT_MAG_VIB_NV = 34551 	# GL/glext.h:2309
GL_HILO16_NV = 34552 	# GL/glext.h:2310
GL_SIGNED_HILO_NV = 34553 	# GL/glext.h:2311
GL_SIGNED_HILO16_NV = 34554 	# GL/glext.h:2312
GL_SIGNED_RGBA_NV = 34555 	# GL/glext.h:2313
GL_SIGNED_RGBA8_NV = 34556 	# GL/glext.h:2314
GL_SIGNED_RGB_NV = 34558 	# GL/glext.h:2315
GL_SIGNED_RGB8_NV = 34559 	# GL/glext.h:2316
GL_SIGNED_LUMINANCE_NV = 34561 	# GL/glext.h:2317
GL_SIGNED_LUMINANCE8_NV = 34562 	# GL/glext.h:2318
GL_SIGNED_LUMINANCE_ALPHA_NV = 34563 	# GL/glext.h:2319
GL_SIGNED_LUMINANCE8_ALPHA8_NV = 34564 	# GL/glext.h:2320
GL_SIGNED_ALPHA_NV = 34565 	# GL/glext.h:2321
GL_SIGNED_ALPHA8_NV = 34566 	# GL/glext.h:2322
GL_SIGNED_INTENSITY_NV = 34567 	# GL/glext.h:2323
GL_SIGNED_INTENSITY8_NV = 34568 	# GL/glext.h:2324
GL_DSDT8_NV = 34569 	# GL/glext.h:2325
GL_DSDT8_MAG8_NV = 34570 	# GL/glext.h:2326
GL_DSDT8_MAG8_INTENSITY8_NV = 34571 	# GL/glext.h:2327
GL_SIGNED_RGB_UNSIGNED_ALPHA_NV = 34572 	# GL/glext.h:2328
GL_SIGNED_RGB8_UNSIGNED_ALPHA8_NV = 34573 	# GL/glext.h:2329
GL_HI_SCALE_NV = 34574 	# GL/glext.h:2330
GL_LO_SCALE_NV = 34575 	# GL/glext.h:2331
GL_DS_SCALE_NV = 34576 	# GL/glext.h:2332
GL_DT_SCALE_NV = 34577 	# GL/glext.h:2333
GL_MAGNITUDE_SCALE_NV = 34578 	# GL/glext.h:2334
GL_VIBRANCE_SCALE_NV = 34579 	# GL/glext.h:2335
GL_HI_BIAS_NV = 34580 	# GL/glext.h:2336
GL_LO_BIAS_NV = 34581 	# GL/glext.h:2337
GL_DS_BIAS_NV = 34582 	# GL/glext.h:2338
GL_DT_BIAS_NV = 34583 	# GL/glext.h:2339
GL_MAGNITUDE_BIAS_NV = 34584 	# GL/glext.h:2340
GL_VIBRANCE_BIAS_NV = 34585 	# GL/glext.h:2341
GL_TEXTURE_BORDER_VALUES_NV = 34586 	# GL/glext.h:2342
GL_TEXTURE_HI_SIZE_NV = 34587 	# GL/glext.h:2343
GL_TEXTURE_LO_SIZE_NV = 34588 	# GL/glext.h:2344
GL_TEXTURE_DS_SIZE_NV = 34589 	# GL/glext.h:2345
GL_TEXTURE_DT_SIZE_NV = 34590 	# GL/glext.h:2346
GL_TEXTURE_MAG_SIZE_NV = 34591 	# GL/glext.h:2347
# NV_texture_shader2 (GL/glext.h:2350)
GL_DOT_PRODUCT_TEXTURE_3D_NV = 34543 	# GL/glext.h:2351
# NV_vertex_array_range2 (GL/glext.h:2354)
GL_VERTEX_ARRAY_RANGE_WITHOUT_FLUSH_NV = 34099 	# GL/glext.h:2355
# NV_vertex_program (GL/glext.h:2358)
GL_VERTEX_PROGRAM_NV = 34336 	# GL/glext.h:2359
GL_VERTEX_STATE_PROGRAM_NV = 34337 	# GL/glext.h:2360
GL_ATTRIB_ARRAY_SIZE_NV = 34339 	# GL/glext.h:2361
GL_ATTRIB_ARRAY_STRIDE_NV = 34340 	# GL/glext.h:2362
GL_ATTRIB_ARRAY_TYPE_NV = 34341 	# GL/glext.h:2363
GL_CURRENT_ATTRIB_NV = 34342 	# GL/glext.h:2364
GL_PROGRAM_LENGTH_NV = 34343 	# GL/glext.h:2365
GL_PROGRAM_STRING_NV = 34344 	# GL/glext.h:2366
GL_MODELVIEW_PROJECTION_NV = 34345 	# GL/glext.h:2367
GL_IDENTITY_NV = 34346 	# GL/glext.h:2368
GL_INVERSE_NV = 34347 	# GL/glext.h:2369
GL_TRANSPOSE_NV = 34348 	# GL/glext.h:2370
GL_INVERSE_TRANSPOSE_NV = 34349 	# GL/glext.h:2371
GL_MAX_TRACK_MATRIX_STACK_DEPTH_NV = 34350 	# GL/glext.h:2372
GL_MAX_TRACK_MATRICES_NV = 34351 	# GL/glext.h:2373
GL_MATRIX0_NV = 34352 	# GL/glext.h:2374
GL_MATRIX1_NV = 34353 	# GL/glext.h:2375
GL_MATRIX2_NV = 34354 	# GL/glext.h:2376
GL_MATRIX3_NV = 34355 	# GL/glext.h:2377
GL_MATRIX4_NV = 34356 	# GL/glext.h:2378
GL_MATRIX5_NV = 34357 	# GL/glext.h:2379
GL_MATRIX6_NV = 34358 	# GL/glext.h:2380
GL_MATRIX7_NV = 34359 	# GL/glext.h:2381
GL_CURRENT_MATRIX_STACK_DEPTH_NV = 34368 	# GL/glext.h:2382
GL_CURRENT_MATRIX_NV = 34369 	# GL/glext.h:2383
GL_VERTEX_PROGRAM_POINT_SIZE_NV = 34370 	# GL/glext.h:2384
GL_VERTEX_PROGRAM_TWO_SIDE_NV = 34371 	# GL/glext.h:2385
GL_PROGRAM_PARAMETER_NV = 34372 	# GL/glext.h:2386
GL_ATTRIB_ARRAY_POINTER_NV = 34373 	# GL/glext.h:2387
GL_PROGRAM_TARGET_NV = 34374 	# GL/glext.h:2388
GL_PROGRAM_RESIDENT_NV = 34375 	# GL/glext.h:2389
GL_TRACK_MATRIX_NV = 34376 	# GL/glext.h:2390
GL_TRACK_MATRIX_TRANSFORM_NV = 34377 	# GL/glext.h:2391
GL_VERTEX_PROGRAM_BINDING_NV = 34378 	# GL/glext.h:2392
GL_PROGRAM_ERROR_POSITION_NV = 34379 	# GL/glext.h:2393
GL_VERTEX_ATTRIB_ARRAY0_NV = 34384 	# GL/glext.h:2394
GL_VERTEX_ATTRIB_ARRAY1_NV = 34385 	# GL/glext.h:2395
GL_VERTEX_ATTRIB_ARRAY2_NV = 34386 	# GL/glext.h:2396
GL_VERTEX_ATTRIB_ARRAY3_NV = 34387 	# GL/glext.h:2397
GL_VERTEX_ATTRIB_ARRAY4_NV = 34388 	# GL/glext.h:2398
GL_VERTEX_ATTRIB_ARRAY5_NV = 34389 	# GL/glext.h:2399
GL_VERTEX_ATTRIB_ARRAY6_NV = 34390 	# GL/glext.h:2400
GL_VERTEX_ATTRIB_ARRAY7_NV = 34391 	# GL/glext.h:2401
GL_VERTEX_ATTRIB_ARRAY8_NV = 34392 	# GL/glext.h:2402
GL_VERTEX_ATTRIB_ARRAY9_NV = 34393 	# GL/glext.h:2403
GL_VERTEX_ATTRIB_ARRAY10_NV = 34394 	# GL/glext.h:2404
GL_VERTEX_ATTRIB_ARRAY11_NV = 34395 	# GL/glext.h:2405
GL_VERTEX_ATTRIB_ARRAY12_NV = 34396 	# GL/glext.h:2406
GL_VERTEX_ATTRIB_ARRAY13_NV = 34397 	# GL/glext.h:2407
GL_VERTEX_ATTRIB_ARRAY14_NV = 34398 	# GL/glext.h:2408
GL_VERTEX_ATTRIB_ARRAY15_NV = 34399 	# GL/glext.h:2409
GL_MAP1_VERTEX_ATTRIB0_4_NV = 34400 	# GL/glext.h:2410
GL_MAP1_VERTEX_ATTRIB1_4_NV = 34401 	# GL/glext.h:2411
GL_MAP1_VERTEX_ATTRIB2_4_NV = 34402 	# GL/glext.h:2412
GL_MAP1_VERTEX_ATTRIB3_4_NV = 34403 	# GL/glext.h:2413
GL_MAP1_VERTEX_ATTRIB4_4_NV = 34404 	# GL/glext.h:2414
GL_MAP1_VERTEX_ATTRIB5_4_NV = 34405 	# GL/glext.h:2415
GL_MAP1_VERTEX_ATTRIB6_4_NV = 34406 	# GL/glext.h:2416
GL_MAP1_VERTEX_ATTRIB7_4_NV = 34407 	# GL/glext.h:2417
GL_MAP1_VERTEX_ATTRIB8_4_NV = 34408 	# GL/glext.h:2418
GL_MAP1_VERTEX_ATTRIB9_4_NV = 34409 	# GL/glext.h:2419
GL_MAP1_VERTEX_ATTRIB10_4_NV = 34410 	# GL/glext.h:2420
GL_MAP1_VERTEX_ATTRIB11_4_NV = 34411 	# GL/glext.h:2421
GL_MAP1_VERTEX_ATTRIB12_4_NV = 34412 	# GL/glext.h:2422
GL_MAP1_VERTEX_ATTRIB13_4_NV = 34413 	# GL/glext.h:2423
GL_MAP1_VERTEX_ATTRIB14_4_NV = 34414 	# GL/glext.h:2424
GL_MAP1_VERTEX_ATTRIB15_4_NV = 34415 	# GL/glext.h:2425
GL_MAP2_VERTEX_ATTRIB0_4_NV = 34416 	# GL/glext.h:2426
GL_MAP2_VERTEX_ATTRIB1_4_NV = 34417 	# GL/glext.h:2427
GL_MAP2_VERTEX_ATTRIB2_4_NV = 34418 	# GL/glext.h:2428
GL_MAP2_VERTEX_ATTRIB3_4_NV = 34419 	# GL/glext.h:2429
GL_MAP2_VERTEX_ATTRIB4_4_NV = 34420 	# GL/glext.h:2430
GL_MAP2_VERTEX_ATTRIB5_4_NV = 34421 	# GL/glext.h:2431
GL_MAP2_VERTEX_ATTRIB6_4_NV = 34422 	# GL/glext.h:2432
GL_MAP2_VERTEX_ATTRIB7_4_NV = 34423 	# GL/glext.h:2433
GL_MAP2_VERTEX_ATTRIB8_4_NV = 34424 	# GL/glext.h:2434
GL_MAP2_VERTEX_ATTRIB9_4_NV = 34425 	# GL/glext.h:2435
GL_MAP2_VERTEX_ATTRIB10_4_NV = 34426 	# GL/glext.h:2436
GL_MAP2_VERTEX_ATTRIB11_4_NV = 34427 	# GL/glext.h:2437
GL_MAP2_VERTEX_ATTRIB12_4_NV = 34428 	# GL/glext.h:2438
GL_MAP2_VERTEX_ATTRIB13_4_NV = 34429 	# GL/glext.h:2439
GL_MAP2_VERTEX_ATTRIB14_4_NV = 34430 	# GL/glext.h:2440
GL_MAP2_VERTEX_ATTRIB15_4_NV = 34431 	# GL/glext.h:2441
# SGIX_texture_coordinate_clamp (GL/glext.h:2444)
GL_TEXTURE_MAX_CLAMP_S_SGIX = 33641 	# GL/glext.h:2445
GL_TEXTURE_MAX_CLAMP_T_SGIX = 33642 	# GL/glext.h:2446
GL_TEXTURE_MAX_CLAMP_R_SGIX = 33643 	# GL/glext.h:2447
# SGIX_scalebias_hint (GL/glext.h:2450)
GL_SCALEBIAS_HINT_SGIX = 33570 	# GL/glext.h:2451
# OML_interlace (GL/glext.h:2454)
GL_INTERLACE_OML = 35200 	# GL/glext.h:2455
GL_INTERLACE_READ_OML = 35201 	# GL/glext.h:2456
# OML_subsample (GL/glext.h:2459)
GL_FORMAT_SUBSAMPLE_24_24_OML = 35202 	# GL/glext.h:2460
GL_FORMAT_SUBSAMPLE_244_244_OML = 35203 	# GL/glext.h:2461
# OML_resample (GL/glext.h:2464)
GL_PACK_RESAMPLE_OML = 35204 	# GL/glext.h:2465
GL_UNPACK_RESAMPLE_OML = 35205 	# GL/glext.h:2466
GL_RESAMPLE_REPLICATE_OML = 35206 	# GL/glext.h:2467
GL_RESAMPLE_ZERO_FILL_OML = 35207 	# GL/glext.h:2468
GL_RESAMPLE_AVERAGE_OML = 35208 	# GL/glext.h:2469
GL_RESAMPLE_DECIMATE_OML = 35209 	# GL/glext.h:2470
# NV_copy_depth_to_color (GL/glext.h:2473)
GL_DEPTH_STENCIL_TO_RGBA_NV = 34926 	# GL/glext.h:2474
GL_DEPTH_STENCIL_TO_BGRA_NV = 34927 	# GL/glext.h:2475
# ATI_envmap_bumpmap (GL/glext.h:2478)
GL_BUMP_ROT_MATRIX_ATI = 34677 	# GL/glext.h:2479
GL_BUMP_ROT_MATRIX_SIZE_ATI = 34678 	# GL/glext.h:2480
GL_BUMP_NUM_TEX_UNITS_ATI = 34679 	# GL/glext.h:2481
GL_BUMP_TEX_UNITS_ATI = 34680 	# GL/glext.h:2482
GL_DUDV_ATI = 34681 	# GL/glext.h:2483
GL_DU8DV8_ATI = 34682 	# GL/glext.h:2484
GL_BUMP_ENVMAP_ATI = 34683 	# GL/glext.h:2485
GL_BUMP_TARGET_ATI = 34684 	# GL/glext.h:2486
# ATI_fragment_shader (GL/glext.h:2489)
GL_FRAGMENT_SHADER_ATI = 35104 	# GL/glext.h:2490
GL_REG_0_ATI = 35105 	# GL/glext.h:2491
GL_REG_1_ATI = 35106 	# GL/glext.h:2492
GL_REG_2_ATI = 35107 	# GL/glext.h:2493
GL_REG_3_ATI = 35108 	# GL/glext.h:2494
GL_REG_4_ATI = 35109 	# GL/glext.h:2495
GL_REG_5_ATI = 35110 	# GL/glext.h:2496
GL_REG_6_ATI = 35111 	# GL/glext.h:2497
GL_REG_7_ATI = 35112 	# GL/glext.h:2498
GL_REG_8_ATI = 35113 	# GL/glext.h:2499
GL_REG_9_ATI = 35114 	# GL/glext.h:2500
GL_REG_10_ATI = 35115 	# GL/glext.h:2501
GL_REG_11_ATI = 35116 	# GL/glext.h:2502
GL_REG_12_ATI = 35117 	# GL/glext.h:2503
GL_REG_13_ATI = 35118 	# GL/glext.h:2504
GL_REG_14_ATI = 35119 	# GL/glext.h:2505
GL_REG_15_ATI = 35120 	# GL/glext.h:2506
GL_REG_16_ATI = 35121 	# GL/glext.h:2507
GL_REG_17_ATI = 35122 	# GL/glext.h:2508
GL_REG_18_ATI = 35123 	# GL/glext.h:2509
GL_REG_19_ATI = 35124 	# GL/glext.h:2510
GL_REG_20_ATI = 35125 	# GL/glext.h:2511
GL_REG_21_ATI = 35126 	# GL/glext.h:2512
GL_REG_22_ATI = 35127 	# GL/glext.h:2513
GL_REG_23_ATI = 35128 	# GL/glext.h:2514
GL_REG_24_ATI = 35129 	# GL/glext.h:2515
GL_REG_25_ATI = 35130 	# GL/glext.h:2516
GL_REG_26_ATI = 35131 	# GL/glext.h:2517
GL_REG_27_ATI = 35132 	# GL/glext.h:2518
GL_REG_28_ATI = 35133 	# GL/glext.h:2519
GL_REG_29_ATI = 35134 	# GL/glext.h:2520
GL_REG_30_ATI = 35135 	# GL/glext.h:2521
GL_REG_31_ATI = 35136 	# GL/glext.h:2522
GL_CON_0_ATI = 35137 	# GL/glext.h:2523
GL_CON_1_ATI = 35138 	# GL/glext.h:2524
GL_CON_2_ATI = 35139 	# GL/glext.h:2525
GL_CON_3_ATI = 35140 	# GL/glext.h:2526
GL_CON_4_ATI = 35141 	# GL/glext.h:2527
GL_CON_5_ATI = 35142 	# GL/glext.h:2528
GL_CON_6_ATI = 35143 	# GL/glext.h:2529
GL_CON_7_ATI = 35144 	# GL/glext.h:2530
GL_CON_8_ATI = 35145 	# GL/glext.h:2531
GL_CON_9_ATI = 35146 	# GL/glext.h:2532
GL_CON_10_ATI = 35147 	# GL/glext.h:2533
GL_CON_11_ATI = 35148 	# GL/glext.h:2534
GL_CON_12_ATI = 35149 	# GL/glext.h:2535
GL_CON_13_ATI = 35150 	# GL/glext.h:2536
GL_CON_14_ATI = 35151 	# GL/glext.h:2537
GL_CON_15_ATI = 35152 	# GL/glext.h:2538
GL_CON_16_ATI = 35153 	# GL/glext.h:2539
GL_CON_17_ATI = 35154 	# GL/glext.h:2540
GL_CON_18_ATI = 35155 	# GL/glext.h:2541
GL_CON_19_ATI = 35156 	# GL/glext.h:2542
GL_CON_20_ATI = 35157 	# GL/glext.h:2543
GL_CON_21_ATI = 35158 	# GL/glext.h:2544
GL_CON_22_ATI = 35159 	# GL/glext.h:2545
GL_CON_23_ATI = 35160 	# GL/glext.h:2546
GL_CON_24_ATI = 35161 	# GL/glext.h:2547
GL_CON_25_ATI = 35162 	# GL/glext.h:2548
GL_CON_26_ATI = 35163 	# GL/glext.h:2549
GL_CON_27_ATI = 35164 	# GL/glext.h:2550
GL_CON_28_ATI = 35165 	# GL/glext.h:2551
GL_CON_29_ATI = 35166 	# GL/glext.h:2552
GL_CON_30_ATI = 35167 	# GL/glext.h:2553
GL_CON_31_ATI = 35168 	# GL/glext.h:2554
GL_MOV_ATI = 35169 	# GL/glext.h:2555
GL_ADD_ATI = 35171 	# GL/glext.h:2556
GL_MUL_ATI = 35172 	# GL/glext.h:2557
GL_SUB_ATI = 35173 	# GL/glext.h:2558
GL_DOT3_ATI = 35174 	# GL/glext.h:2559
GL_DOT4_ATI = 35175 	# GL/glext.h:2560
GL_MAD_ATI = 35176 	# GL/glext.h:2561
GL_LERP_ATI = 35177 	# GL/glext.h:2562
GL_CND_ATI = 35178 	# GL/glext.h:2563
GL_CND0_ATI = 35179 	# GL/glext.h:2564
GL_DOT2_ADD_ATI = 35180 	# GL/glext.h:2565
GL_SECONDARY_INTERPOLATOR_ATI = 35181 	# GL/glext.h:2566
GL_NUM_FRAGMENT_REGISTERS_ATI = 35182 	# GL/glext.h:2567
GL_NUM_FRAGMENT_CONSTANTS_ATI = 35183 	# GL/glext.h:2568
GL_NUM_PASSES_ATI = 35184 	# GL/glext.h:2569
GL_NUM_INSTRUCTIONS_PER_PASS_ATI = 35185 	# GL/glext.h:2570
GL_NUM_INSTRUCTIONS_TOTAL_ATI = 35186 	# GL/glext.h:2571
GL_NUM_INPUT_INTERPOLATOR_COMPONENTS_ATI = 35187 	# GL/glext.h:2572
GL_NUM_LOOPBACK_COMPONENTS_ATI = 35188 	# GL/glext.h:2573
GL_COLOR_ALPHA_PAIRING_ATI = 35189 	# GL/glext.h:2574
GL_SWIZZLE_STR_ATI = 35190 	# GL/glext.h:2575
GL_SWIZZLE_STQ_ATI = 35191 	# GL/glext.h:2576
GL_SWIZZLE_STR_DR_ATI = 35192 	# GL/glext.h:2577
GL_SWIZZLE_STQ_DQ_ATI = 35193 	# GL/glext.h:2578
GL_SWIZZLE_STRQ_ATI = 35194 	# GL/glext.h:2579
GL_SWIZZLE_STRQ_DQ_ATI = 35195 	# GL/glext.h:2580
GL_RED_BIT_ATI = 1 	# GL/glext.h:2581
GL_GREEN_BIT_ATI = 2 	# GL/glext.h:2582
GL_BLUE_BIT_ATI = 4 	# GL/glext.h:2583
GL_2X_BIT_ATI = 1 	# GL/glext.h:2584
GL_4X_BIT_ATI = 2 	# GL/glext.h:2585
GL_8X_BIT_ATI = 4 	# GL/glext.h:2586
GL_HALF_BIT_ATI = 8 	# GL/glext.h:2587
GL_QUARTER_BIT_ATI = 16 	# GL/glext.h:2588
GL_EIGHTH_BIT_ATI = 32 	# GL/glext.h:2589
GL_SATURATE_BIT_ATI = 64 	# GL/glext.h:2590
GL_COMP_BIT_ATI = 2 	# GL/glext.h:2591
GL_NEGATE_BIT_ATI = 4 	# GL/glext.h:2592
GL_BIAS_BIT_ATI = 8 	# GL/glext.h:2593
# ATI_pn_triangles (GL/glext.h:2596)
GL_PN_TRIANGLES_ATI = 34800 	# GL/glext.h:2597
GL_MAX_PN_TRIANGLES_TESSELATION_LEVEL_ATI = 34801 	# GL/glext.h:2598
GL_PN_TRIANGLES_POINT_MODE_ATI = 34802 	# GL/glext.h:2599
GL_PN_TRIANGLES_NORMAL_MODE_ATI = 34803 	# GL/glext.h:2600
GL_PN_TRIANGLES_TESSELATION_LEVEL_ATI = 34804 	# GL/glext.h:2601
GL_PN_TRIANGLES_POINT_MODE_LINEAR_ATI = 34805 	# GL/glext.h:2602
GL_PN_TRIANGLES_POINT_MODE_CUBIC_ATI = 34806 	# GL/glext.h:2603
GL_PN_TRIANGLES_NORMAL_MODE_LINEAR_ATI = 34807 	# GL/glext.h:2604
GL_PN_TRIANGLES_NORMAL_MODE_QUADRATIC_ATI = 34808 	# GL/glext.h:2605
# ATI_vertex_array_object (GL/glext.h:2608)
GL_STATIC_ATI = 34656 	# GL/glext.h:2609
GL_DYNAMIC_ATI = 34657 	# GL/glext.h:2610
GL_PRESERVE_ATI = 34658 	# GL/glext.h:2611
GL_DISCARD_ATI = 34659 	# GL/glext.h:2612
GL_OBJECT_BUFFER_SIZE_ATI = 34660 	# GL/glext.h:2613
GL_OBJECT_BUFFER_USAGE_ATI = 34661 	# GL/glext.h:2614
GL_ARRAY_OBJECT_BUFFER_ATI = 34662 	# GL/glext.h:2615
GL_ARRAY_OBJECT_OFFSET_ATI = 34663 	# GL/glext.h:2616
# EXT_vertex_shader (GL/glext.h:2619)
GL_VERTEX_SHADER_EXT = 34688 	# GL/glext.h:2620
GL_VERTEX_SHADER_BINDING_EXT = 34689 	# GL/glext.h:2621
GL_OP_INDEX_EXT = 34690 	# GL/glext.h:2622
GL_OP_NEGATE_EXT = 34691 	# GL/glext.h:2623
GL_OP_DOT3_EXT = 34692 	# GL/glext.h:2624
GL_OP_DOT4_EXT = 34693 	# GL/glext.h:2625
GL_OP_MUL_EXT = 34694 	# GL/glext.h:2626
GL_OP_ADD_EXT = 34695 	# GL/glext.h:2627
GL_OP_MADD_EXT = 34696 	# GL/glext.h:2628
GL_OP_FRAC_EXT = 34697 	# GL/glext.h:2629
GL_OP_MAX_EXT = 34698 	# GL/glext.h:2630
GL_OP_MIN_EXT = 34699 	# GL/glext.h:2631
GL_OP_SET_GE_EXT = 34700 	# GL/glext.h:2632
GL_OP_SET_LT_EXT = 34701 	# GL/glext.h:2633
GL_OP_CLAMP_EXT = 34702 	# GL/glext.h:2634
GL_OP_FLOOR_EXT = 34703 	# GL/glext.h:2635
GL_OP_ROUND_EXT = 34704 	# GL/glext.h:2636
GL_OP_EXP_BASE_2_EXT = 34705 	# GL/glext.h:2637
GL_OP_LOG_BASE_2_EXT = 34706 	# GL/glext.h:2638
GL_OP_POWER_EXT = 34707 	# GL/glext.h:2639
GL_OP_RECIP_EXT = 34708 	# GL/glext.h:2640
GL_OP_RECIP_SQRT_EXT = 34709 	# GL/glext.h:2641
GL_OP_SUB_EXT = 34710 	# GL/glext.h:2642
GL_OP_CROSS_PRODUCT_EXT = 34711 	# GL/glext.h:2643
GL_OP_MULTIPLY_MATRIX_EXT = 34712 	# GL/glext.h:2644
GL_OP_MOV_EXT = 34713 	# GL/glext.h:2645
GL_OUTPUT_VERTEX_EXT = 34714 	# GL/glext.h:2646
GL_OUTPUT_COLOR0_EXT = 34715 	# GL/glext.h:2647
GL_OUTPUT_COLOR1_EXT = 34716 	# GL/glext.h:2648
GL_OUTPUT_TEXTURE_COORD0_EXT = 34717 	# GL/glext.h:2649
GL_OUTPUT_TEXTURE_COORD1_EXT = 34718 	# GL/glext.h:2650
GL_OUTPUT_TEXTURE_COORD2_EXT = 34719 	# GL/glext.h:2651
GL_OUTPUT_TEXTURE_COORD3_EXT = 34720 	# GL/glext.h:2652
GL_OUTPUT_TEXTURE_COORD4_EXT = 34721 	# GL/glext.h:2653
GL_OUTPUT_TEXTURE_COORD5_EXT = 34722 	# GL/glext.h:2654
GL_OUTPUT_TEXTURE_COORD6_EXT = 34723 	# GL/glext.h:2655
GL_OUTPUT_TEXTURE_COORD7_EXT = 34724 	# GL/glext.h:2656
GL_OUTPUT_TEXTURE_COORD8_EXT = 34725 	# GL/glext.h:2657
GL_OUTPUT_TEXTURE_COORD9_EXT = 34726 	# GL/glext.h:2658
GL_OUTPUT_TEXTURE_COORD10_EXT = 34727 	# GL/glext.h:2659
GL_OUTPUT_TEXTURE_COORD11_EXT = 34728 	# GL/glext.h:2660
GL_OUTPUT_TEXTURE_COORD12_EXT = 34729 	# GL/glext.h:2661
GL_OUTPUT_TEXTURE_COORD13_EXT = 34730 	# GL/glext.h:2662
GL_OUTPUT_TEXTURE_COORD14_EXT = 34731 	# GL/glext.h:2663
GL_OUTPUT_TEXTURE_COORD15_EXT = 34732 	# GL/glext.h:2664
GL_OUTPUT_TEXTURE_COORD16_EXT = 34733 	# GL/glext.h:2665
GL_OUTPUT_TEXTURE_COORD17_EXT = 34734 	# GL/glext.h:2666
GL_OUTPUT_TEXTURE_COORD18_EXT = 34735 	# GL/glext.h:2667
GL_OUTPUT_TEXTURE_COORD19_EXT = 34736 	# GL/glext.h:2668
GL_OUTPUT_TEXTURE_COORD20_EXT = 34737 	# GL/glext.h:2669
GL_OUTPUT_TEXTURE_COORD21_EXT = 34738 	# GL/glext.h:2670
GL_OUTPUT_TEXTURE_COORD22_EXT = 34739 	# GL/glext.h:2671
GL_OUTPUT_TEXTURE_COORD23_EXT = 34740 	# GL/glext.h:2672
GL_OUTPUT_TEXTURE_COORD24_EXT = 34741 	# GL/glext.h:2673
GL_OUTPUT_TEXTURE_COORD25_EXT = 34742 	# GL/glext.h:2674
GL_OUTPUT_TEXTURE_COORD26_EXT = 34743 	# GL/glext.h:2675
GL_OUTPUT_TEXTURE_COORD27_EXT = 34744 	# GL/glext.h:2676
GL_OUTPUT_TEXTURE_COORD28_EXT = 34745 	# GL/glext.h:2677
GL_OUTPUT_TEXTURE_COORD29_EXT = 34746 	# GL/glext.h:2678
GL_OUTPUT_TEXTURE_COORD30_EXT = 34747 	# GL/glext.h:2679
GL_OUTPUT_TEXTURE_COORD31_EXT = 34748 	# GL/glext.h:2680
GL_OUTPUT_FOG_EXT = 34749 	# GL/glext.h:2681
GL_SCALAR_EXT = 34750 	# GL/glext.h:2682
GL_VECTOR_EXT = 34751 	# GL/glext.h:2683
GL_MATRIX_EXT = 34752 	# GL/glext.h:2684
GL_VARIANT_EXT = 34753 	# GL/glext.h:2685
GL_INVARIANT_EXT = 34754 	# GL/glext.h:2686
GL_LOCAL_CONSTANT_EXT = 34755 	# GL/glext.h:2687
GL_LOCAL_EXT = 34756 	# GL/glext.h:2688
GL_MAX_VERTEX_SHADER_INSTRUCTIONS_EXT = 34757 	# GL/glext.h:2689
GL_MAX_VERTEX_SHADER_VARIANTS_EXT = 34758 	# GL/glext.h:2690
GL_MAX_VERTEX_SHADER_INVARIANTS_EXT = 34759 	# GL/glext.h:2691
GL_MAX_VERTEX_SHADER_LOCAL_CONSTANTS_EXT = 34760 	# GL/glext.h:2692
GL_MAX_VERTEX_SHADER_LOCALS_EXT = 34761 	# GL/glext.h:2693
GL_MAX_OPTIMIZED_VERTEX_SHADER_INSTRUCTIONS_EXT = 34762 	# GL/glext.h:2694
GL_MAX_OPTIMIZED_VERTEX_SHADER_VARIANTS_EXT = 34763 	# GL/glext.h:2695
GL_MAX_OPTIMIZED_VERTEX_SHADER_LOCAL_CONSTANTS_EXT = 34764 	# GL/glext.h:2696
GL_MAX_OPTIMIZED_VERTEX_SHADER_INVARIANTS_EXT = 34765 	# GL/glext.h:2697
GL_MAX_OPTIMIZED_VERTEX_SHADER_LOCALS_EXT = 34766 	# GL/glext.h:2698
GL_VERTEX_SHADER_INSTRUCTIONS_EXT = 34767 	# GL/glext.h:2699
GL_VERTEX_SHADER_VARIANTS_EXT = 34768 	# GL/glext.h:2700
GL_VERTEX_SHADER_INVARIANTS_EXT = 34769 	# GL/glext.h:2701
GL_VERTEX_SHADER_LOCAL_CONSTANTS_EXT = 34770 	# GL/glext.h:2702
GL_VERTEX_SHADER_LOCALS_EXT = 34771 	# GL/glext.h:2703
GL_VERTEX_SHADER_OPTIMIZED_EXT = 34772 	# GL/glext.h:2704
GL_X_EXT = 34773 	# GL/glext.h:2705
GL_Y_EXT = 34774 	# GL/glext.h:2706
GL_Z_EXT = 34775 	# GL/glext.h:2707
GL_W_EXT = 34776 	# GL/glext.h:2708
GL_NEGATIVE_X_EXT = 34777 	# GL/glext.h:2709
GL_NEGATIVE_Y_EXT = 34778 	# GL/glext.h:2710
GL_NEGATIVE_Z_EXT = 34779 	# GL/glext.h:2711
GL_NEGATIVE_W_EXT = 34780 	# GL/glext.h:2712
GL_ZERO_EXT = 34781 	# GL/glext.h:2713
GL_ONE_EXT = 34782 	# GL/glext.h:2714
GL_NEGATIVE_ONE_EXT = 34783 	# GL/glext.h:2715
GL_NORMALIZED_RANGE_EXT = 34784 	# GL/glext.h:2716
GL_FULL_RANGE_EXT = 34785 	# GL/glext.h:2717
GL_CURRENT_VERTEX_EXT = 34786 	# GL/glext.h:2718
GL_MVP_MATRIX_EXT = 34787 	# GL/glext.h:2719
GL_VARIANT_VALUE_EXT = 34788 	# GL/glext.h:2720
GL_VARIANT_DATATYPE_EXT = 34789 	# GL/glext.h:2721
GL_VARIANT_ARRAY_STRIDE_EXT = 34790 	# GL/glext.h:2722
GL_VARIANT_ARRAY_TYPE_EXT = 34791 	# GL/glext.h:2723
GL_VARIANT_ARRAY_EXT = 34792 	# GL/glext.h:2724
GL_VARIANT_ARRAY_POINTER_EXT = 34793 	# GL/glext.h:2725
GL_INVARIANT_VALUE_EXT = 34794 	# GL/glext.h:2726
GL_INVARIANT_DATATYPE_EXT = 34795 	# GL/glext.h:2727
GL_LOCAL_CONSTANT_VALUE_EXT = 34796 	# GL/glext.h:2728
GL_LOCAL_CONSTANT_DATATYPE_EXT = 34797 	# GL/glext.h:2729
# ATI_vertex_streams (GL/glext.h:2732)
GL_MAX_VERTEX_STREAMS_ATI = 34667 	# GL/glext.h:2733
GL_VERTEX_STREAM0_ATI = 34668 	# GL/glext.h:2734
GL_VERTEX_STREAM1_ATI = 34669 	# GL/glext.h:2735
GL_VERTEX_STREAM2_ATI = 34670 	# GL/glext.h:2736
GL_VERTEX_STREAM3_ATI = 34671 	# GL/glext.h:2737
GL_VERTEX_STREAM4_ATI = 34672 	# GL/glext.h:2738
GL_VERTEX_STREAM5_ATI = 34673 	# GL/glext.h:2739
GL_VERTEX_STREAM6_ATI = 34674 	# GL/glext.h:2740
GL_VERTEX_STREAM7_ATI = 34675 	# GL/glext.h:2741
GL_VERTEX_SOURCE_ATI = 34676 	# GL/glext.h:2742
# ATI_element_array (GL/glext.h:2745)
GL_ELEMENT_ARRAY_ATI = 34664 	# GL/glext.h:2746
GL_ELEMENT_ARRAY_TYPE_ATI = 34665 	# GL/glext.h:2747
GL_ELEMENT_ARRAY_POINTER_ATI = 34666 	# GL/glext.h:2748
# SUN_mesh_array (GL/glext.h:2751)
GL_QUAD_MESH_SUN = 34324 	# GL/glext.h:2752
GL_TRIANGLE_MESH_SUN = 34325 	# GL/glext.h:2753
# SUN_slice_accum (GL/glext.h:2756)
GL_SLICE_ACCUM_SUN = 34252 	# GL/glext.h:2757
# NV_multisample_filter_hint (GL/glext.h:2760)
GL_MULTISAMPLE_FILTER_HINT_NV = 34100 	# GL/glext.h:2761
# NV_depth_clamp (GL/glext.h:2764)
GL_DEPTH_CLAMP_NV = 34383 	# GL/glext.h:2765
# NV_occlusion_query (GL/glext.h:2768)
GL_PIXEL_COUNTER_BITS_NV = 34916 	# GL/glext.h:2769
GL_CURRENT_OCCLUSION_QUERY_ID_NV = 34917 	# GL/glext.h:2770
GL_PIXEL_COUNT_NV = 34918 	# GL/glext.h:2771
GL_PIXEL_COUNT_AVAILABLE_NV = 34919 	# GL/glext.h:2772
# NV_point_sprite (GL/glext.h:2775)
GL_POINT_SPRITE_NV = 34913 	# GL/glext.h:2776
GL_COORD_REPLACE_NV = 34914 	# GL/glext.h:2777
GL_POINT_SPRITE_R_MODE_NV = 34915 	# GL/glext.h:2778
# NV_texture_shader3 (GL/glext.h:2781)
GL_OFFSET_PROJECTIVE_TEXTURE_2D_NV = 34896 	# GL/glext.h:2782
GL_OFFSET_PROJECTIVE_TEXTURE_2D_SCALE_NV = 34897 	# GL/glext.h:2783
GL_OFFSET_PROJECTIVE_TEXTURE_RECTANGLE_NV = 34898 	# GL/glext.h:2784
GL_OFFSET_PROJECTIVE_TEXTURE_RECTANGLE_SCALE_NV = 34899 	# GL/glext.h:2785
GL_OFFSET_HILO_TEXTURE_2D_NV = 34900 	# GL/glext.h:2786
GL_OFFSET_HILO_TEXTURE_RECTANGLE_NV = 34901 	# GL/glext.h:2787
GL_OFFSET_HILO_PROJECTIVE_TEXTURE_2D_NV = 34902 	# GL/glext.h:2788
GL_OFFSET_HILO_PROJECTIVE_TEXTURE_RECTANGLE_NV = 34903 	# GL/glext.h:2789
GL_DEPENDENT_HILO_TEXTURE_2D_NV = 34904 	# GL/glext.h:2790
GL_DEPENDENT_RGB_TEXTURE_3D_NV = 34905 	# GL/glext.h:2791
GL_DEPENDENT_RGB_TEXTURE_CUBE_MAP_NV = 34906 	# GL/glext.h:2792
GL_DOT_PRODUCT_PASS_THROUGH_NV = 34907 	# GL/glext.h:2793
GL_DOT_PRODUCT_TEXTURE_1D_NV = 34908 	# GL/glext.h:2794
GL_DOT_PRODUCT_AFFINE_DEPTH_REPLACE_NV = 34909 	# GL/glext.h:2795
GL_HILO8_NV = 34910 	# GL/glext.h:2796
GL_SIGNED_HILO8_NV = 34911 	# GL/glext.h:2797
GL_FORCE_BLUE_TO_ONE_NV = 34912 	# GL/glext.h:2798
# NV_vertex_program1_1 (GL/glext.h:2801)
# EXT_shadow_funcs (GL/glext.h:2804)
# EXT_stencil_two_side (GL/glext.h:2807)
GL_STENCIL_TEST_TWO_SIDE_EXT = 35088 	# GL/glext.h:2808
GL_ACTIVE_STENCIL_FACE_EXT = 35089 	# GL/glext.h:2809
# ATI_text_fragment_shader (GL/glext.h:2812)
GL_TEXT_FRAGMENT_SHADER_ATI = 33280 	# GL/glext.h:2813
# APPLE_client_storage (GL/glext.h:2816)
GL_UNPACK_CLIENT_STORAGE_APPLE = 34226 	# GL/glext.h:2817
# APPLE_element_array (GL/glext.h:2820)
GL_ELEMENT_ARRAY_APPLE = 34664 	# GL/glext.h:2821
GL_ELEMENT_ARRAY_TYPE_APPLE = 34665 	# GL/glext.h:2822
GL_ELEMENT_ARRAY_POINTER_APPLE = 34666 	# GL/glext.h:2823
# APPLE_fence (GL/glext.h:2826)
GL_DRAW_PIXELS_APPLE = 35338 	# GL/glext.h:2827
GL_FENCE_APPLE = 35339 	# GL/glext.h:2828
# APPLE_vertex_array_object (GL/glext.h:2831)
GL_VERTEX_ARRAY_BINDING_APPLE = 34229 	# GL/glext.h:2832
# APPLE_vertex_array_range (GL/glext.h:2835)
GL_VERTEX_ARRAY_RANGE_APPLE = 34077 	# GL/glext.h:2836
GL_VERTEX_ARRAY_RANGE_LENGTH_APPLE = 34078 	# GL/glext.h:2837
GL_VERTEX_ARRAY_STORAGE_HINT_APPLE = 34079 	# GL/glext.h:2838
GL_VERTEX_ARRAY_RANGE_POINTER_APPLE = 34081 	# GL/glext.h:2839
GL_STORAGE_CACHED_APPLE = 34238 	# GL/glext.h:2840
GL_STORAGE_SHARED_APPLE = 34239 	# GL/glext.h:2841
# APPLE_ycbcr_422 (GL/glext.h:2844)
GL_YCBCR_422_APPLE = 34233 	# GL/glext.h:2845
GL_UNSIGNED_SHORT_8_8_APPLE = 34234 	# GL/glext.h:2846
GL_UNSIGNED_SHORT_8_8_REV_APPLE = 34235 	# GL/glext.h:2847
# S3_s3tc (GL/glext.h:2850)
GL_RGB_S3TC = 33696 	# GL/glext.h:2851
GL_RGB4_S3TC = 33697 	# GL/glext.h:2852
GL_RGBA_S3TC = 33698 	# GL/glext.h:2853
GL_RGBA4_S3TC = 33699 	# GL/glext.h:2854
# ATI_draw_buffers (GL/glext.h:2857)
GL_MAX_DRAW_BUFFERS_ATI = 34852 	# GL/glext.h:2858
GL_DRAW_BUFFER0_ATI = 34853 	# GL/glext.h:2859
GL_DRAW_BUFFER1_ATI = 34854 	# GL/glext.h:2860
GL_DRAW_BUFFER2_ATI = 34855 	# GL/glext.h:2861
GL_DRAW_BUFFER3_ATI = 34856 	# GL/glext.h:2862
GL_DRAW_BUFFER4_ATI = 34857 	# GL/glext.h:2863
GL_DRAW_BUFFER5_ATI = 34858 	# GL/glext.h:2864
GL_DRAW_BUFFER6_ATI = 34859 	# GL/glext.h:2865
GL_DRAW_BUFFER7_ATI = 34860 	# GL/glext.h:2866
GL_DRAW_BUFFER8_ATI = 34861 	# GL/glext.h:2867
GL_DRAW_BUFFER9_ATI = 34862 	# GL/glext.h:2868
GL_DRAW_BUFFER10_ATI = 34863 	# GL/glext.h:2869
GL_DRAW_BUFFER11_ATI = 34864 	# GL/glext.h:2870
GL_DRAW_BUFFER12_ATI = 34865 	# GL/glext.h:2871
GL_DRAW_BUFFER13_ATI = 34866 	# GL/glext.h:2872
GL_DRAW_BUFFER14_ATI = 34867 	# GL/glext.h:2873
GL_DRAW_BUFFER15_ATI = 34868 	# GL/glext.h:2874
# ATI_pixel_format_float (GL/glext.h:2877)
GL_TYPE_RGBA_FLOAT_ATI = 34848 	# GL/glext.h:2878
GL_COLOR_CLEAR_UNCLAMPED_VALUE_ATI = 34869 	# GL/glext.h:2879
# ATI_texture_env_combine3 (GL/glext.h:2882)
GL_MODULATE_ADD_ATI = 34628 	# GL/glext.h:2883
GL_MODULATE_SIGNED_ADD_ATI = 34629 	# GL/glext.h:2884
GL_MODULATE_SUBTRACT_ATI = 34630 	# GL/glext.h:2885
# ATI_texture_float (GL/glext.h:2888)
GL_RGBA_FLOAT32_ATI = 34836 	# GL/glext.h:2889
GL_RGB_FLOAT32_ATI = 34837 	# GL/glext.h:2890
GL_ALPHA_FLOAT32_ATI = 34838 	# GL/glext.h:2891
GL_INTENSITY_FLOAT32_ATI = 34839 	# GL/glext.h:2892
GL_LUMINANCE_FLOAT32_ATI = 34840 	# GL/glext.h:2893
GL_LUMINANCE_ALPHA_FLOAT32_ATI = 34841 	# GL/glext.h:2894
GL_RGBA_FLOAT16_ATI = 34842 	# GL/glext.h:2895
GL_RGB_FLOAT16_ATI = 34843 	# GL/glext.h:2896
GL_ALPHA_FLOAT16_ATI = 34844 	# GL/glext.h:2897
GL_INTENSITY_FLOAT16_ATI = 34845 	# GL/glext.h:2898
GL_LUMINANCE_FLOAT16_ATI = 34846 	# GL/glext.h:2899
GL_LUMINANCE_ALPHA_FLOAT16_ATI = 34847 	# GL/glext.h:2900
# NV_float_buffer (GL/glext.h:2903)
GL_FLOAT_R_NV = 34944 	# GL/glext.h:2904
GL_FLOAT_RG_NV = 34945 	# GL/glext.h:2905
GL_FLOAT_RGB_NV = 34946 	# GL/glext.h:2906
GL_FLOAT_RGBA_NV = 34947 	# GL/glext.h:2907
GL_FLOAT_R16_NV = 34948 	# GL/glext.h:2908
GL_FLOAT_R32_NV = 34949 	# GL/glext.h:2909
GL_FLOAT_RG16_NV = 34950 	# GL/glext.h:2910
GL_FLOAT_RG32_NV = 34951 	# GL/glext.h:2911
GL_FLOAT_RGB16_NV = 34952 	# GL/glext.h:2912
GL_FLOAT_RGB32_NV = 34953 	# GL/glext.h:2913
GL_FLOAT_RGBA16_NV = 34954 	# GL/glext.h:2914
GL_FLOAT_RGBA32_NV = 34955 	# GL/glext.h:2915
GL_TEXTURE_FLOAT_COMPONENTS_NV = 34956 	# GL/glext.h:2916
GL_FLOAT_CLEAR_COLOR_VALUE_NV = 34957 	# GL/glext.h:2917
GL_FLOAT_RGBA_MODE_NV = 34958 	# GL/glext.h:2918
# NV_fragment_program (GL/glext.h:2921)
GL_MAX_FRAGMENT_PROGRAM_LOCAL_PARAMETERS_NV = 34920 	# GL/glext.h:2922
GL_FRAGMENT_PROGRAM_NV = 34928 	# GL/glext.h:2923
GL_MAX_TEXTURE_COORDS_NV = 34929 	# GL/glext.h:2924
GL_MAX_TEXTURE_IMAGE_UNITS_NV = 34930 	# GL/glext.h:2925
GL_FRAGMENT_PROGRAM_BINDING_NV = 34931 	# GL/glext.h:2926
GL_PROGRAM_ERROR_STRING_NV = 34932 	# GL/glext.h:2927
# NV_half_float (GL/glext.h:2930)
GL_HALF_FLOAT_NV = 5131 	# GL/glext.h:2931
# NV_pixel_data_range (GL/glext.h:2934)
GL_WRITE_PIXEL_DATA_RANGE_NV = 34936 	# GL/glext.h:2935
GL_READ_PIXEL_DATA_RANGE_NV = 34937 	# GL/glext.h:2936
GL_WRITE_PIXEL_DATA_RANGE_LENGTH_NV = 34938 	# GL/glext.h:2937
GL_READ_PIXEL_DATA_RANGE_LENGTH_NV = 34939 	# GL/glext.h:2938
GL_WRITE_PIXEL_DATA_RANGE_POINTER_NV = 34940 	# GL/glext.h:2939
GL_READ_PIXEL_DATA_RANGE_POINTER_NV = 34941 	# GL/glext.h:2940
# NV_primitive_restart (GL/glext.h:2943)
GL_PRIMITIVE_RESTART_NV = 34136 	# GL/glext.h:2944
GL_PRIMITIVE_RESTART_INDEX_NV = 34137 	# GL/glext.h:2945
# NV_texture_expand_normal (GL/glext.h:2948)
GL_TEXTURE_UNSIGNED_REMAP_MODE_NV = 34959 	# GL/glext.h:2949
# NV_vertex_program2 (GL/glext.h:2952)
# ATI_map_object_buffer (GL/glext.h:2955)
# ATI_separate_stencil (GL/glext.h:2958)
GL_STENCIL_BACK_FUNC_ATI = 34816 	# GL/glext.h:2959
GL_STENCIL_BACK_FAIL_ATI = 34817 	# GL/glext.h:2960
GL_STENCIL_BACK_PASS_DEPTH_FAIL_ATI = 34818 	# GL/glext.h:2961
GL_STENCIL_BACK_PASS_DEPTH_PASS_ATI = 34819 	# GL/glext.h:2962
# ATI_vertex_attrib_array_object (GL/glext.h:2965)
# OES_read_format (GL/glext.h:2968)
GL_IMPLEMENTATION_COLOR_READ_TYPE_OES = 35738 	# GL/glext.h:2969
GL_IMPLEMENTATION_COLOR_READ_FORMAT_OES = 35739 	# GL/glext.h:2970
# EXT_depth_bounds_test (GL/glext.h:2973)
GL_DEPTH_BOUNDS_TEST_EXT = 34960 	# GL/glext.h:2974
GL_DEPTH_BOUNDS_EXT = 34961 	# GL/glext.h:2975
# EXT_texture_mirror_clamp (GL/glext.h:2978)
GL_MIRROR_CLAMP_EXT = 34626 	# GL/glext.h:2979
GL_MIRROR_CLAMP_TO_EDGE_EXT = 34627 	# GL/glext.h:2980
GL_MIRROR_CLAMP_TO_BORDER_EXT = 35090 	# GL/glext.h:2981
# EXT_blend_equation_separate (GL/glext.h:2984)
GL_BLEND_EQUATION_RGB_EXT = 32777 	# GL/glext.h:2985
GL_BLEND_EQUATION_ALPHA_EXT = 34877 	# GL/glext.h:2986
# MESA_pack_invert (GL/glext.h:2989)
GL_PACK_INVERT_MESA = 34648 	# GL/glext.h:2990
# MESA_ycbcr_texture (GL/glext.h:2993)
GL_UNSIGNED_SHORT_8_8_MESA = 34234 	# GL/glext.h:2994
GL_UNSIGNED_SHORT_8_8_REV_MESA = 34235 	# GL/glext.h:2995
GL_YCBCR_MESA = 34647 	# GL/glext.h:2996
# EXT_pixel_buffer_object (GL/glext.h:2999)
GL_PIXEL_PACK_BUFFER_EXT = 35051 	# GL/glext.h:3000
GL_PIXEL_UNPACK_BUFFER_EXT = 35052 	# GL/glext.h:3001
GL_PIXEL_PACK_BUFFER_BINDING_EXT = 35053 	# GL/glext.h:3002
GL_PIXEL_UNPACK_BUFFER_BINDING_EXT = 35055 	# GL/glext.h:3003
# NV_fragment_program_option (GL/glext.h:3006)
# NV_fragment_program2 (GL/glext.h:3009)
GL_MAX_PROGRAM_EXEC_INSTRUCTIONS_NV = 35060 	# GL/glext.h:3010
GL_MAX_PROGRAM_CALL_DEPTH_NV = 35061 	# GL/glext.h:3011
GL_MAX_PROGRAM_IF_DEPTH_NV = 35062 	# GL/glext.h:3012
GL_MAX_PROGRAM_LOOP_DEPTH_NV = 35063 	# GL/glext.h:3013
GL_MAX_PROGRAM_LOOP_COUNT_NV = 35064 	# GL/glext.h:3014
# NV_vertex_program2_option (GL/glext.h:3017)
# NV_vertex_program3 (GL/glext.h:3022)
# EXT_framebuffer_object (GL/glext.h:3026)
GL_INVALID_FRAMEBUFFER_OPERATION_EXT = 1286 	# GL/glext.h:3027
GL_MAX_RENDERBUFFER_SIZE_EXT = 34024 	# GL/glext.h:3028
GL_FRAMEBUFFER_BINDING_EXT = 36006 	# GL/glext.h:3029
GL_RENDERBUFFER_BINDING_EXT = 36007 	# GL/glext.h:3030
GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE_EXT = 36048 	# GL/glext.h:3031
GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME_EXT = 36049 	# GL/glext.h:3032
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL_EXT = 36050 	# GL/glext.h:3033
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE_EXT = 36051 	# GL/glext.h:3034
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_3D_ZOFFSET_EXT = 36052 	# GL/glext.h:3035
GL_FRAMEBUFFER_COMPLETE_EXT = 36053 	# GL/glext.h:3036
GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_EXT = 36054 	# GL/glext.h:3037
GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_EXT = 36055 	# GL/glext.h:3038
GL_FRAMEBUFFER_INCOMPLETE_DUPLICATE_ATTACHMENT_EXT = 36056 	# GL/glext.h:3039
GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_EXT = 36057 	# GL/glext.h:3040
GL_FRAMEBUFFER_INCOMPLETE_FORMATS_EXT = 36058 	# GL/glext.h:3041
GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER_EXT = 36059 	# GL/glext.h:3042
GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER_EXT = 36060 	# GL/glext.h:3043
GL_FRAMEBUFFER_UNSUPPORTED_EXT = 36061 	# GL/glext.h:3044
GL_MAX_COLOR_ATTACHMENTS_EXT = 36063 	# GL/glext.h:3045
GL_COLOR_ATTACHMENT0_EXT = 36064 	# GL/glext.h:3046
GL_COLOR_ATTACHMENT1_EXT = 36065 	# GL/glext.h:3047
GL_COLOR_ATTACHMENT2_EXT = 36066 	# GL/glext.h:3048
GL_COLOR_ATTACHMENT3_EXT = 36067 	# GL/glext.h:3049
GL_COLOR_ATTACHMENT4_EXT = 36068 	# GL/glext.h:3050
GL_COLOR_ATTACHMENT5_EXT = 36069 	# GL/glext.h:3051
GL_COLOR_ATTACHMENT6_EXT = 36070 	# GL/glext.h:3052
GL_COLOR_ATTACHMENT7_EXT = 36071 	# GL/glext.h:3053
GL_COLOR_ATTACHMENT8_EXT = 36072 	# GL/glext.h:3054
GL_COLOR_ATTACHMENT9_EXT = 36073 	# GL/glext.h:3055
GL_COLOR_ATTACHMENT10_EXT = 36074 	# GL/glext.h:3056
GL_COLOR_ATTACHMENT11_EXT = 36075 	# GL/glext.h:3057
GL_COLOR_ATTACHMENT12_EXT = 36076 	# GL/glext.h:3058
GL_COLOR_ATTACHMENT13_EXT = 36077 	# GL/glext.h:3059
GL_COLOR_ATTACHMENT14_EXT = 36078 	# GL/glext.h:3060
GL_COLOR_ATTACHMENT15_EXT = 36079 	# GL/glext.h:3061
GL_DEPTH_ATTACHMENT_EXT = 36096 	# GL/glext.h:3062
GL_STENCIL_ATTACHMENT_EXT = 36128 	# GL/glext.h:3063
GL_FRAMEBUFFER_EXT = 36160 	# GL/glext.h:3064
GL_RENDERBUFFER_EXT = 36161 	# GL/glext.h:3065
GL_RENDERBUFFER_WIDTH_EXT = 36162 	# GL/glext.h:3066
GL_RENDERBUFFER_HEIGHT_EXT = 36163 	# GL/glext.h:3067
GL_RENDERBUFFER_INTERNAL_FORMAT_EXT = 36164 	# GL/glext.h:3068
GL_STENCIL_INDEX_EXT = 36165 	# GL/glext.h:3069
GL_STENCIL_INDEX1_EXT = 36166 	# GL/glext.h:3070
GL_STENCIL_INDEX4_EXT = 36167 	# GL/glext.h:3071
GL_STENCIL_INDEX8_EXT = 36168 	# GL/glext.h:3072
GL_STENCIL_INDEX16_EXT = 36169 	# GL/glext.h:3073
GL_RENDERBUFFER_RED_SIZE_EXT = 36176 	# GL/glext.h:3074
GL_RENDERBUFFER_GREEN_SIZE_EXT = 36177 	# GL/glext.h:3075
GL_RENDERBUFFER_BLUE_SIZE_EXT = 36178 	# GL/glext.h:3076
GL_RENDERBUFFER_ALPHA_SIZE_EXT = 36179 	# GL/glext.h:3077
GL_RENDERBUFFER_DEPTH_SIZE_EXT = 36180 	# GL/glext.h:3078
GL_RENDERBUFFER_STENCIL_SIZE_EXT = 36181 	# GL/glext.h:3079
# GREMEDY_string_marker (GL/glext.h:3082)
# EXT_Cg_shader (GL/glext.h:3085)
GL_CG_VERTEX_SHADER_EXT = 35086 	# GL/glext.h:3086
GL_CG_FRAGMENT_SHADER_EXT = 35087 	# GL/glext.h:3087
# EXT_timer_query (GL/glext.h:3090)
GL_TIME_ELAPSED_EXT = 35007 	# GL/glext.h:3091
# EXT_texture_buffer_object (GL/glext.h:3094)
GL_TEXTURE_BUFFER_EXT = 35882 	# GL/glext.h:3095
GL_MAX_TEXTURE_BUFFER_SIZE_EXT = 35883 	# GL/glext.h:3096
GL_TEXTURE_BINDING_BUFFER_EXT = 35884 	# GL/glext.h:3097
GL_TEXTURE_BUFFER_DATA_STORE_BINDING_EXT = 35885 	# GL/glext.h:3098
GL_TEXTURE_BUFFER_FORMAT_EXT = 35886 	# GL/glext.h:3099
# EXT_gpu_shader4 (GL/glext.h:3102)
GL_SAMPLER_1D_ARRAY_EXT = 36288 	# GL/glext.h:3103
GL_SAMPLER_2D_ARRAY_EXT = 36289 	# GL/glext.h:3104
GL_SAMPLER_BUFFER_EXT = 36290 	# GL/glext.h:3105
GL_SAMPLER_1D_ARRAY_SHADOW_EXT = 36291 	# GL/glext.h:3106
GL_SAMPLER_2D_ARRAY_SHADOW_EXT = 36292 	# GL/glext.h:3107
GL_SAMPLER_CUBE_SHADOW_EXT = 36293 	# GL/glext.h:3108
GL_UNSIGNED_INT_VEC2_EXT = 36294 	# GL/glext.h:3109
GL_UNSIGNED_INT_VEC3_EXT = 36295 	# GL/glext.h:3110
GL_UNSIGNED_INT_VEC4_EXT = 36296 	# GL/glext.h:3111
GL_INT_SAMPLER_1D_EXT = 36297 	# GL/glext.h:3112
GL_INT_SAMPLER_2D_EXT = 36298 	# GL/glext.h:3113
GL_INT_SAMPLER_3D_EXT = 36299 	# GL/glext.h:3114
GL_INT_SAMPLER_CUBE_EXT = 36300 	# GL/glext.h:3115
GL_INT_SAMPLER_2D_RECT_EXT = 36301 	# GL/glext.h:3116
GL_INT_SAMPLER_1D_ARRAY_EXT = 36302 	# GL/glext.h:3117
GL_INT_SAMPLER_2D_ARRAY_EXT = 36303 	# GL/glext.h:3118
GL_INT_SAMPLER_BUFFER_EXT = 36304 	# GL/glext.h:3119
GL_UNSIGNED_INT_SAMPLER_1D_EXT = 36305 	# GL/glext.h:3120
GL_UNSIGNED_INT_SAMPLER_2D_EXT = 36306 	# GL/glext.h:3121
GL_UNSIGNED_INT_SAMPLER_3D_EXT = 36307 	# GL/glext.h:3122
GL_UNSIGNED_INT_SAMPLER_CUBE_EXT = 36308 	# GL/glext.h:3123
GL_UNSIGNED_INT_SAMPLER_2D_RECT_EXT = 36309 	# GL/glext.h:3124
GL_UNSIGNED_INT_SAMPLER_1D_ARRAY_EXT = 36310 	# GL/glext.h:3125
GL_UNSIGNED_INT_SAMPLER_2D_ARRAY_EXT = 36311 	# GL/glext.h:3126
GL_UNSIGNED_INT_SAMPLER_BUFFER_EXT = 36312 	# GL/glext.h:3127
GL_VERTEX_ATTRIB_ARRAY_INTEGER_EXT = 35069 	# GL/glext.h:3128
# EXT_geometry_shader4 (GL/glext.h:3131)
GL_GEOMETRY_SHADER_EXT = 36313 	# GL/glext.h:3132
GL_MAX_GEOMETRY_VARYING_COMPONENTS_EXT = 36317 	# GL/glext.h:3133
GL_MAX_VERTEX_VARYING_COMPONENTS_EXT = 36318 	# GL/glext.h:3134
GL_MAX_VARYING_COMPONENTS_EXT = 35659 	# GL/glext.h:3135
GL_MAX_GEOMETRY_UNIFORM_COMPONENTS_EXT = 36319 	# GL/glext.h:3136
GL_MAX_GEOMETRY_OUTPUT_VERTICES_EXT = 36320 	# GL/glext.h:3137
GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS_EXT = 36321 	# GL/glext.h:3138
GL_GEOMETRY_VERTICES_OUT_EXT = 36314 	# GL/glext.h:3139
GL_GEOMETRY_INPUT_TYPE_EXT = 36315 	# GL/glext.h:3140
GL_GEOMETRY_OUTPUT_TYPE_EXT = 36316 	# GL/glext.h:3141
GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS_EXT = 35881 	# GL/glext.h:3142
GL_LINES_ADJACENCY_EXT = 10 	# GL/glext.h:3143
GL_LINE_STRIP_ADJACENCY_EXT = 11 	# GL/glext.h:3144
GL_TRIANGLES_ADJACENCY_EXT = 12 	# GL/glext.h:3145
GL_TRIANGLE_STRIP_ADJACENCY_EXT = 13 	# GL/glext.h:3146
GL_FRAMEBUFFER_ATTACHMENT_LAYERED_EXT = 36263 	# GL/glext.h:3147
GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS_EXT = 36264 	# GL/glext.h:3148
GL_FRAMEBUFFER_INCOMPLETE_LAYER_COUNT_EXT = 36265 	# GL/glext.h:3149
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER_EXT = 36052 	# GL/glext.h:3150
GL_PROGRAM_POINT_SIZE_EXT = 34370 	# GL/glext.h:3151
# EXT_bindable_uniform (GL/glext.h:3154)
GL_MAX_VERTEX_BINDABLE_UNIFORMS_EXT = 36322 	# GL/glext.h:3155
GL_MAX_FRAGMENT_BINDABLE_UNIFORMS_EXT = 36323 	# GL/glext.h:3156
GL_MAX_GEOMETRY_BINDABLE_UNIFORMS_EXT = 36324 	# GL/glext.h:3157
GL_MAX_BINDABLE_UNIFORM_SIZE_EXT = 36333 	# GL/glext.h:3158
GL_UNIFORM_BUFFER_EXT = 36334 	# GL/glext.h:3159
GL_UNIFORM_BUFFER_BINDING_EXT = 36335 	# GL/glext.h:3160
# EXT_framebuffer_sRGB (GL/glext.h:3163)
GL_FRAMEBUFFER_SRGB_EXT = 36281 	# GL/glext.h:3164
GL_FRAMEBUFFER_SRGB_CAPABLE_EXT = 36282 	# GL/glext.h:3165
# EXT_texture_shared_exponent (GL/glext.h:3168)
GL_RGB9_E5_EXT = 35901 	# GL/glext.h:3169
GL_UNSIGNED_INT_5_9_9_9_REV_EXT = 35902 	# GL/glext.h:3170
GL_TEXTURE_SHARED_SIZE_EXT = 35903 	# GL/glext.h:3171
# EXT_packed_float (GL/glext.h:3174)
GL_R11F_G11F_B10F_EXT = 35898 	# GL/glext.h:3175
GL_UNSIGNED_INT_10F_11F_11F_REV_EXT = 35899 	# GL/glext.h:3176
GL_RGBA_SIGNED_COMPONENTS_EXT = 35900 	# GL/glext.h:3177
# EXT_texture_array (GL/glext.h:3180)
GL_TEXTURE_1D_ARRAY_EXT = 35864 	# GL/glext.h:3181
GL_PROXY_TEXTURE_1D_ARRAY_EXT = 35865 	# GL/glext.h:3182
GL_TEXTURE_2D_ARRAY_EXT = 35866 	# GL/glext.h:3183
GL_PROXY_TEXTURE_2D_ARRAY_EXT = 35867 	# GL/glext.h:3184
GL_TEXTURE_BINDING_1D_ARRAY_EXT = 35868 	# GL/glext.h:3185
GL_TEXTURE_BINDING_2D_ARRAY_EXT = 35869 	# GL/glext.h:3186
GL_MAX_ARRAY_TEXTURE_LAYERS_EXT = 35071 	# GL/glext.h:3187
GL_COMPARE_REF_DEPTH_TO_TEXTURE_EXT = 34894 	# GL/glext.h:3188
# EXT_texture_integer (GL/glext.h:3192)
GL_RGBA32UI_EXT = 36208 	# GL/glext.h:3193
GL_RGB32UI_EXT = 36209 	# GL/glext.h:3194
GL_ALPHA32UI_EXT = 36210 	# GL/glext.h:3195
GL_INTENSITY32UI_EXT = 36211 	# GL/glext.h:3196
GL_LUMINANCE32UI_EXT = 36212 	# GL/glext.h:3197
GL_LUMINANCE_ALPHA32UI_EXT = 36213 	# GL/glext.h:3198
GL_RGBA16UI_EXT = 36214 	# GL/glext.h:3199
GL_RGB16UI_EXT = 36215 	# GL/glext.h:3200
GL_ALPHA16UI_EXT = 36216 	# GL/glext.h:3201
GL_INTENSITY16UI_EXT = 36217 	# GL/glext.h:3202
GL_LUMINANCE16UI_EXT = 36218 	# GL/glext.h:3203
GL_LUMINANCE_ALPHA16UI_EXT = 36219 	# GL/glext.h:3204
GL_RGBA8UI_EXT = 36220 	# GL/glext.h:3205
GL_RGB8UI_EXT = 36221 	# GL/glext.h:3206
GL_ALPHA8UI_EXT = 36222 	# GL/glext.h:3207
GL_INTENSITY8UI_EXT = 36223 	# GL/glext.h:3208
GL_LUMINANCE8UI_EXT = 36224 	# GL/glext.h:3209
GL_LUMINANCE_ALPHA8UI_EXT = 36225 	# GL/glext.h:3210
GL_RGBA32I_EXT = 36226 	# GL/glext.h:3211
GL_RGB32I_EXT = 36227 	# GL/glext.h:3212
GL_ALPHA32I_EXT = 36228 	# GL/glext.h:3213
GL_INTENSITY32I_EXT = 36229 	# GL/glext.h:3214
GL_LUMINANCE32I_EXT = 36230 	# GL/glext.h:3215
GL_LUMINANCE_ALPHA32I_EXT = 36231 	# GL/glext.h:3216
GL_RGBA16I_EXT = 36232 	# GL/glext.h:3217
GL_RGB16I_EXT = 36233 	# GL/glext.h:3218
GL_ALPHA16I_EXT = 36234 	# GL/glext.h:3219
GL_INTENSITY16I_EXT = 36235 	# GL/glext.h:3220
GL_LUMINANCE16I_EXT = 36236 	# GL/glext.h:3221
GL_LUMINANCE_ALPHA16I_EXT = 36237 	# GL/glext.h:3222
GL_RGBA8I_EXT = 36238 	# GL/glext.h:3223
GL_RGB8I_EXT = 36239 	# GL/glext.h:3224
GL_ALPHA8I_EXT = 36240 	# GL/glext.h:3225
GL_INTENSITY8I_EXT = 36241 	# GL/glext.h:3226
GL_LUMINANCE8I_EXT = 36242 	# GL/glext.h:3227
GL_LUMINANCE_ALPHA8I_EXT = 36243 	# GL/glext.h:3228
GL_RED_INTEGER_EXT = 36244 	# GL/glext.h:3229
GL_GREEN_INTEGER_EXT = 36245 	# GL/glext.h:3230
GL_BLUE_INTEGER_EXT = 36246 	# GL/glext.h:3231
GL_ALPHA_INTEGER_EXT = 36247 	# GL/glext.h:3232
GL_RGB_INTEGER_EXT = 36248 	# GL/glext.h:3233
GL_RGBA_INTEGER_EXT = 36249 	# GL/glext.h:3234
GL_BGR_INTEGER_EXT = 36250 	# GL/glext.h:3235
GL_BGRA_INTEGER_EXT = 36251 	# GL/glext.h:3236
GL_LUMINANCE_INTEGER_EXT = 36252 	# GL/glext.h:3237
GL_LUMINANCE_ALPHA_INTEGER_EXT = 36253 	# GL/glext.h:3238
GL_RGBA_INTEGER_MODE_EXT = 36254 	# GL/glext.h:3239
# NV_depth_buffer_float (GL/glext.h:3242)
GL_DEPTH_COMPONENT32F_NV = 36267 	# GL/glext.h:3243
GL_DEPTH32F_STENCIL8_NV = 36268 	# GL/glext.h:3244
GL_FLOAT_32_UNSIGNED_INT_24_8_REV_NV = 36269 	# GL/glext.h:3245
GL_DEPTH_BUFFER_FLOAT_MODE_NV = 36271 	# GL/glext.h:3246
# EXT_texture_compression_latc (GL/glext.h:3249)
GL_COMPRESSED_LUMINANCE_LATC1_EXT = 35952 	# GL/glext.h:3250
GL_COMPRESSED_SIGNED_LUMINANCE_LATC1_EXT = 35953 	# GL/glext.h:3251
GL_COMPRESSED_LUMINANCE_ALPHA_LATC2_EXT = 35954 	# GL/glext.h:3252
GL_COMPRESSED_SIGNED_LUMINANCE_ALPHA_LATC2_EXT = 35955 	# GL/glext.h:3253
# NV_transform_feedback (GL/glext.h:3256)
GL_BACK_PRIMARY_COLOR_NV = 35959 	# GL/glext.h:3257
GL_BACK_SECONDARY_COLOR_NV = 35960 	# GL/glext.h:3258
GL_TEXTURE_COORD_NV = 35961 	# GL/glext.h:3259
GL_CLIP_DISTANCE_NV = 35962 	# GL/glext.h:3260
GL_VERTEX_ID_NV = 35963 	# GL/glext.h:3261
GL_PRIMITIVE_ID_NV = 35964 	# GL/glext.h:3262
GL_GENERIC_ATTRIB_NV = 35965 	# GL/glext.h:3263
GL_TRANSFORM_FEEDBACK_ATTRIBS_NV = 35966 	# GL/glext.h:3264
GL_TRANSFORM_FEEDBACK_BUFFER_MODE_NV = 35967 	# GL/glext.h:3265
GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS_NV = 35968 	# GL/glext.h:3266
GL_ACTIVE_VARYINGS_NV = 35969 	# GL/glext.h:3267
GL_ACTIVE_VARYING_MAX_LENGTH_NV = 35970 	# GL/glext.h:3268
GL_TRANSFORM_FEEDBACK_VARYINGS_NV = 35971 	# GL/glext.h:3269
GL_TRANSFORM_FEEDBACK_BUFFER_START_NV = 35972 	# GL/glext.h:3270
GL_TRANSFORM_FEEDBACK_BUFFER_SIZE_NV = 35973 	# GL/glext.h:3271
GL_TRANSFORM_FEEDBACK_RECORD_NV = 35974 	# GL/glext.h:3272
GL_PRIMITIVES_GENERATED_NV = 35975 	# GL/glext.h:3273
GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN_NV = 35976 	# GL/glext.h:3274
GL_RASTERIZER_DISCARD_NV = 35977 	# GL/glext.h:3275
GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_ATTRIBS_NV = 35978 	# GL/glext.h:3276
GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS_NV = 35979 	# GL/glext.h:3277
GL_INTERLEAVED_ATTRIBS_NV = 35980 	# GL/glext.h:3278
GL_SEPARATE_ATTRIBS_NV = 35981 	# GL/glext.h:3279
GL_TRANSFORM_FEEDBACK_BUFFER_NV = 35982 	# GL/glext.h:3280
GL_TRANSFORM_FEEDBACK_BUFFER_BINDING_NV = 35983 	# GL/glext.h:3281
# NV_geometry_program4 (GL/glext.h:3284)
GL_GEOMETRY_PROGRAM_NV = 35878 	# GL/glext.h:3285
GL_MAX_PROGRAM_OUTPUT_VERTICES_NV = 35879 	# GL/glext.h:3286
GL_MAX_PROGRAM_TOTAL_OUTPUT_COMPONENTS_NV = 35880 	# GL/glext.h:3287
# NV_gpu_program4 (GL/glext.h:3290)
GL_MIN_PROGRAM_TEXEL_OFFSET_NV = 35076 	# GL/glext.h:3291
GL_MAX_PROGRAM_TEXEL_OFFSET_NV = 35077 	# GL/glext.h:3292
GL_PROGRAM_ATTRIB_COMPONENTS_NV = 35078 	# GL/glext.h:3293
GL_PROGRAM_RESULT_COMPONENTS_NV = 35079 	# GL/glext.h:3294
GL_MAX_PROGRAM_ATTRIB_COMPONENTS_NV = 35080 	# GL/glext.h:3295
GL_MAX_PROGRAM_RESULT_COMPONENTS_NV = 35081 	# GL/glext.h:3296
GL_MAX_PROGRAM_GENERIC_ATTRIBS_NV = 36261 	# GL/glext.h:3297
GL_MAX_PROGRAM_GENERIC_RESULTS_NV = 36262 	# GL/glext.h:3298
# NV_framebuffer_multisample_coverage (GL/glext.h:3301)
GL_RENDERBUFFER_COVERAGE_SAMPLES_NV = 36011 	# GL/glext.h:3302
GL_RENDERBUFFER_COLOR_SAMPLES_NV = 36368 	# GL/glext.h:3303
GL_MAX_RENDERBUFFER_COVERAGE_SAMPLES_NV = 36183 	# GL/glext.h:3304
GL_MAX_RENDERBUFFER_COLOR_SAMPLES_NV = 36369 	# GL/glext.h:3305
GL_MAX_MULTISAMPLE_COVERAGE_MODES_NV = 36370 	# GL/glext.h:3306
GL_MULTISAMPLE_COVERAGE_MODES_NV = 36371 	# GL/glext.h:3307
# EXT_framebuffer_multisample (GL/glext.h:3310)
GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE_EXT = 36182 	# GL/glext.h:3311
GL_MAX_SAMPLES_EXT = 36183 	# GL/glext.h:3312
GL_RENDERBUFFER_SAMPLES_EXT = 36011 	# GL/glext.h:3313
# EXT_framebuffer_blit (GL/glext.h:3316)
GL_READ_FRAMEBUFFER_EXT = 36008 	# GL/glext.h:3317
GL_DRAW_FRAMEBUFFER_EXT = 36009 	# GL/glext.h:3318
GL_DRAW_FRAMEBUFFER_BINDING_EXT = 36006 	# GL/glext.h:3319
GL_READ_FRAMEBUFFER_BINDING_EXT = 36010 	# GL/glext.h:3320
# EXT_texture_compression_rgtc (GL/glext.h:3323)
GL_COMPRESSED_RED_RGTC1_EXT = 36283 	# GL/glext.h:3324
GL_COMPRESSED_SIGNED_RED_RGTC1_EXT = 36284 	# GL/glext.h:3325
GL_COMPRESSED_RED_GREEN_RGTC2_EXT = 36285 	# GL/glext.h:3326
GL_COMPRESSED_SIGNED_RED_GREEN_RGTC2_EXT = 36286 	# GL/glext.h:3327
# VERSION_2_0 (GL/glext.h:3333)
GLchar = c_char 	# GL/glext.h:3335
# VERSION_1_5 (GL/glext.h:3338)
GLintptr = c_ptrdiff_t 	# GL/glext.h:3340
GLsizeiptr = c_ptrdiff_t 	# GL/glext.h:3341
# ARB_vertex_buffer_object (GL/glext.h:3344)
GLintptrARB = c_ptrdiff_t 	# GL/glext.h:3346
GLsizeiptrARB = c_ptrdiff_t 	# GL/glext.h:3347
# ARB_shader_objects (GL/glext.h:3350)
GLcharARB = c_char 	# GL/glext.h:3352
GLhandleARB = c_uint 	# GL/glext.h:3353
# ARB_half_float_pixel (GL/glext.h:3357)
GLhalfARB = c_ushort 	# GL/glext.h:3358
# NV_half_float (GL/glext.h:3361)
GLhalfNV = c_ushort 	# GL/glext.h:3362
# EXT_timer_query (GL/glext.h:3365)
GLint64EXT = c_longlong 	# GL/glext.h:3366
GLuint64EXT = c_ulonglong 	# GL/glext.h:3367
# VERSION_1_2 (GL/glext.h:3370)
GL_VERSION_1_2 = 1 	# GL/glext.h:3371
GLclampf = c_float 	# /usr/include/GL/gl.h:64
# GL/glext.h:3373
glBlendColor = _link_function('glBlendColor', None, [GLclampf, GLclampf, GLclampf, GLclampf], 'VERSION_1_2')

GLenum = c_uint 	# /usr/include/GL/gl.h:53
# GL/glext.h:3374
glBlendEquation = _link_function('glBlendEquation', None, [GLenum], 'VERSION_1_2')

GLuint = c_uint 	# /usr/include/GL/gl.h:62
GLsizei = c_int 	# /usr/include/GL/gl.h:59
GLvoid = None 	# /usr/include/GL/gl.h:67
# GL/glext.h:3375
glDrawRangeElements = _link_function('glDrawRangeElements', None, [GLenum, GLuint, GLuint, GLsizei, GLenum, POINTER(GLvoid)], 'VERSION_1_2')

# GL/glext.h:3376
glColorTable = _link_function('glColorTable', None, [GLenum, GLenum, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'VERSION_1_2')

GLfloat = c_float 	# /usr/include/GL/gl.h:63
# GL/glext.h:3377
glColorTableParameterfv = _link_function('glColorTableParameterfv', None, [GLenum, GLenum, POINTER(GLfloat)], 'VERSION_1_2')

GLint = c_int 	# /usr/include/GL/gl.h:58
# GL/glext.h:3378
glColorTableParameteriv = _link_function('glColorTableParameteriv', None, [GLenum, GLenum, POINTER(GLint)], 'VERSION_1_2')

# GL/glext.h:3379
glCopyColorTable = _link_function('glCopyColorTable', None, [GLenum, GLenum, GLint, GLint, GLsizei], 'VERSION_1_2')

# GL/glext.h:3380
glGetColorTable = _link_function('glGetColorTable', None, [GLenum, GLenum, GLenum, POINTER(GLvoid)], 'VERSION_1_2')

# GL/glext.h:3381
glGetColorTableParameterfv = _link_function('glGetColorTableParameterfv', None, [GLenum, GLenum, POINTER(GLfloat)], 'VERSION_1_2')

# GL/glext.h:3382
glGetColorTableParameteriv = _link_function('glGetColorTableParameteriv', None, [GLenum, GLenum, POINTER(GLint)], 'VERSION_1_2')

# GL/glext.h:3383
glColorSubTable = _link_function('glColorSubTable', None, [GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'VERSION_1_2')

# GL/glext.h:3384
glCopyColorSubTable = _link_function('glCopyColorSubTable', None, [GLenum, GLsizei, GLint, GLint, GLsizei], 'VERSION_1_2')

# GL/glext.h:3385
glConvolutionFilter1D = _link_function('glConvolutionFilter1D', None, [GLenum, GLenum, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'VERSION_1_2')

# GL/glext.h:3386
glConvolutionFilter2D = _link_function('glConvolutionFilter2D', None, [GLenum, GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'VERSION_1_2')

# GL/glext.h:3387
glConvolutionParameterf = _link_function('glConvolutionParameterf', None, [GLenum, GLenum, GLfloat], 'VERSION_1_2')

# GL/glext.h:3388
glConvolutionParameterfv = _link_function('glConvolutionParameterfv', None, [GLenum, GLenum, POINTER(GLfloat)], 'VERSION_1_2')

# GL/glext.h:3389
glConvolutionParameteri = _link_function('glConvolutionParameteri', None, [GLenum, GLenum, GLint], 'VERSION_1_2')

# GL/glext.h:3390
glConvolutionParameteriv = _link_function('glConvolutionParameteriv', None, [GLenum, GLenum, POINTER(GLint)], 'VERSION_1_2')

# GL/glext.h:3391
glCopyConvolutionFilter1D = _link_function('glCopyConvolutionFilter1D', None, [GLenum, GLenum, GLint, GLint, GLsizei], 'VERSION_1_2')

# GL/glext.h:3392
glCopyConvolutionFilter2D = _link_function('glCopyConvolutionFilter2D', None, [GLenum, GLenum, GLint, GLint, GLsizei, GLsizei], 'VERSION_1_2')

# GL/glext.h:3393
glGetConvolutionFilter = _link_function('glGetConvolutionFilter', None, [GLenum, GLenum, GLenum, POINTER(GLvoid)], 'VERSION_1_2')

# GL/glext.h:3394
glGetConvolutionParameterfv = _link_function('glGetConvolutionParameterfv', None, [GLenum, GLenum, POINTER(GLfloat)], 'VERSION_1_2')

# GL/glext.h:3395
glGetConvolutionParameteriv = _link_function('glGetConvolutionParameteriv', None, [GLenum, GLenum, POINTER(GLint)], 'VERSION_1_2')

# GL/glext.h:3396
glGetSeparableFilter = _link_function('glGetSeparableFilter', None, [GLenum, GLenum, GLenum, POINTER(GLvoid), POINTER(GLvoid), POINTER(GLvoid)], 'VERSION_1_2')

# GL/glext.h:3397
glSeparableFilter2D = _link_function('glSeparableFilter2D', None, [GLenum, GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid), POINTER(GLvoid)], 'VERSION_1_2')

GLboolean = c_ubyte 	# /usr/include/GL/gl.h:54
# GL/glext.h:3398
glGetHistogram = _link_function('glGetHistogram', None, [GLenum, GLboolean, GLenum, GLenum, POINTER(GLvoid)], 'VERSION_1_2')

# GL/glext.h:3399
glGetHistogramParameterfv = _link_function('glGetHistogramParameterfv', None, [GLenum, GLenum, POINTER(GLfloat)], 'VERSION_1_2')

# GL/glext.h:3400
glGetHistogramParameteriv = _link_function('glGetHistogramParameteriv', None, [GLenum, GLenum, POINTER(GLint)], 'VERSION_1_2')

# GL/glext.h:3401
glGetMinmax = _link_function('glGetMinmax', None, [GLenum, GLboolean, GLenum, GLenum, POINTER(GLvoid)], 'VERSION_1_2')

# GL/glext.h:3402
glGetMinmaxParameterfv = _link_function('glGetMinmaxParameterfv', None, [GLenum, GLenum, POINTER(GLfloat)], 'VERSION_1_2')

# GL/glext.h:3403
glGetMinmaxParameteriv = _link_function('glGetMinmaxParameteriv', None, [GLenum, GLenum, POINTER(GLint)], 'VERSION_1_2')

# GL/glext.h:3404
glHistogram = _link_function('glHistogram', None, [GLenum, GLsizei, GLenum, GLboolean], 'VERSION_1_2')

# GL/glext.h:3405
glMinmax = _link_function('glMinmax', None, [GLenum, GLenum, GLboolean], 'VERSION_1_2')

# GL/glext.h:3406
glResetHistogram = _link_function('glResetHistogram', None, [GLenum], 'VERSION_1_2')

# GL/glext.h:3407
glResetMinmax = _link_function('glResetMinmax', None, [GLenum], 'VERSION_1_2')

# GL/glext.h:3408
glTexImage3D = _link_function('glTexImage3D', None, [GLenum, GLint, GLint, GLsizei, GLsizei, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)], 'VERSION_1_2')

# GL/glext.h:3409
glTexSubImage3D = _link_function('glTexSubImage3D', None, [GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'VERSION_1_2')

# GL/glext.h:3410
glCopyTexSubImage3D = _link_function('glCopyTexSubImage3D', None, [GLenum, GLint, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei], 'VERSION_1_2')

PFNGLBLENDCOLORPROC = CFUNCTYPE(None, GLclampf, GLclampf, GLclampf, GLclampf) 	# GL/glext.h:3412
PFNGLBLENDEQUATIONPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:3413
PFNGLDRAWRANGEELEMENTSPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLsizei, GLenum, POINTER(GLvoid)) 	# GL/glext.h:3414
PFNGLCOLORTABLEPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:3415
PFNGLCOLORTABLEPARAMETERFVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:3416
PFNGLCOLORTABLEPARAMETERIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:3417
PFNGLCOPYCOLORTABLEPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLsizei) 	# GL/glext.h:3418
PFNGLGETCOLORTABLEPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:3419
PFNGLGETCOLORTABLEPARAMETERFVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:3420
PFNGLGETCOLORTABLEPARAMETERIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:3421
PFNGLCOLORSUBTABLEPROC = CFUNCTYPE(None, GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:3422
PFNGLCOPYCOLORSUBTABLEPROC = CFUNCTYPE(None, GLenum, GLsizei, GLint, GLint, GLsizei) 	# GL/glext.h:3423
PFNGLCONVOLUTIONFILTER1DPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:3424
PFNGLCONVOLUTIONFILTER2DPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:3425
PFNGLCONVOLUTIONPARAMETERFPROC = CFUNCTYPE(None, GLenum, GLenum, GLfloat) 	# GL/glext.h:3426
PFNGLCONVOLUTIONPARAMETERFVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:3427
PFNGLCONVOLUTIONPARAMETERIPROC = CFUNCTYPE(None, GLenum, GLenum, GLint) 	# GL/glext.h:3428
PFNGLCONVOLUTIONPARAMETERIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:3429
PFNGLCOPYCONVOLUTIONFILTER1DPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLsizei) 	# GL/glext.h:3430
PFNGLCOPYCONVOLUTIONFILTER2DPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLsizei, GLsizei) 	# GL/glext.h:3431
PFNGLGETCONVOLUTIONFILTERPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:3432
PFNGLGETCONVOLUTIONPARAMETERFVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:3433
PFNGLGETCONVOLUTIONPARAMETERIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:3434
PFNGLGETSEPARABLEFILTERPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLvoid), POINTER(GLvoid), POINTER(GLvoid)) 	# GL/glext.h:3435
PFNGLSEPARABLEFILTER2DPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid), POINTER(GLvoid)) 	# GL/glext.h:3436
PFNGLGETHISTOGRAMPROC = CFUNCTYPE(None, GLenum, GLboolean, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:3437
PFNGLGETHISTOGRAMPARAMETERFVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:3438
PFNGLGETHISTOGRAMPARAMETERIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:3439
PFNGLGETMINMAXPROC = CFUNCTYPE(None, GLenum, GLboolean, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:3440
PFNGLGETMINMAXPARAMETERFVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:3441
PFNGLGETMINMAXPARAMETERIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:3442
PFNGLHISTOGRAMPROC = CFUNCTYPE(None, GLenum, GLsizei, GLenum, GLboolean) 	# GL/glext.h:3443
PFNGLMINMAXPROC = CFUNCTYPE(None, GLenum, GLenum, GLboolean) 	# GL/glext.h:3444
PFNGLRESETHISTOGRAMPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:3445
PFNGLRESETMINMAXPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:3446
PFNGLTEXIMAGE3DPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLsizei, GLsizei, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:3447
PFNGLTEXSUBIMAGE3DPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:3448
PFNGLCOPYTEXSUBIMAGE3DPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei) 	# GL/glext.h:3449
# VERSION_1_3 (GL/glext.h:3452)
GL_VERSION_1_3 = 1 	# GL/glext.h:3453
# GL/glext.h:3455
glActiveTexture = _link_function('glActiveTexture', None, [GLenum], 'VERSION_1_3')

# GL/glext.h:3456
glClientActiveTexture = _link_function('glClientActiveTexture', None, [GLenum], 'VERSION_1_3')

GLdouble = c_double 	# /usr/include/GL/gl.h:65
# GL/glext.h:3457
glMultiTexCoord1d = _link_function('glMultiTexCoord1d', None, [GLenum, GLdouble], 'VERSION_1_3')

# GL/glext.h:3458
glMultiTexCoord1dv = _link_function('glMultiTexCoord1dv', None, [GLenum, POINTER(GLdouble)], 'VERSION_1_3')

# GL/glext.h:3459
glMultiTexCoord1f = _link_function('glMultiTexCoord1f', None, [GLenum, GLfloat], 'VERSION_1_3')

# GL/glext.h:3460
glMultiTexCoord1fv = _link_function('glMultiTexCoord1fv', None, [GLenum, POINTER(GLfloat)], 'VERSION_1_3')

# GL/glext.h:3461
glMultiTexCoord1i = _link_function('glMultiTexCoord1i', None, [GLenum, GLint], 'VERSION_1_3')

# GL/glext.h:3462
glMultiTexCoord1iv = _link_function('glMultiTexCoord1iv', None, [GLenum, POINTER(GLint)], 'VERSION_1_3')

GLshort = c_short 	# /usr/include/GL/gl.h:57
# GL/glext.h:3463
glMultiTexCoord1s = _link_function('glMultiTexCoord1s', None, [GLenum, GLshort], 'VERSION_1_3')

# GL/glext.h:3464
glMultiTexCoord1sv = _link_function('glMultiTexCoord1sv', None, [GLenum, POINTER(GLshort)], 'VERSION_1_3')

# GL/glext.h:3465
glMultiTexCoord2d = _link_function('glMultiTexCoord2d', None, [GLenum, GLdouble, GLdouble], 'VERSION_1_3')

# GL/glext.h:3466
glMultiTexCoord2dv = _link_function('glMultiTexCoord2dv', None, [GLenum, POINTER(GLdouble)], 'VERSION_1_3')

# GL/glext.h:3467
glMultiTexCoord2f = _link_function('glMultiTexCoord2f', None, [GLenum, GLfloat, GLfloat], 'VERSION_1_3')

# GL/glext.h:3468
glMultiTexCoord2fv = _link_function('glMultiTexCoord2fv', None, [GLenum, POINTER(GLfloat)], 'VERSION_1_3')

# GL/glext.h:3469
glMultiTexCoord2i = _link_function('glMultiTexCoord2i', None, [GLenum, GLint, GLint], 'VERSION_1_3')

# GL/glext.h:3470
glMultiTexCoord2iv = _link_function('glMultiTexCoord2iv', None, [GLenum, POINTER(GLint)], 'VERSION_1_3')

# GL/glext.h:3471
glMultiTexCoord2s = _link_function('glMultiTexCoord2s', None, [GLenum, GLshort, GLshort], 'VERSION_1_3')

# GL/glext.h:3472
glMultiTexCoord2sv = _link_function('glMultiTexCoord2sv', None, [GLenum, POINTER(GLshort)], 'VERSION_1_3')

# GL/glext.h:3473
glMultiTexCoord3d = _link_function('glMultiTexCoord3d', None, [GLenum, GLdouble, GLdouble, GLdouble], 'VERSION_1_3')

# GL/glext.h:3474
glMultiTexCoord3dv = _link_function('glMultiTexCoord3dv', None, [GLenum, POINTER(GLdouble)], 'VERSION_1_3')

# GL/glext.h:3475
glMultiTexCoord3f = _link_function('glMultiTexCoord3f', None, [GLenum, GLfloat, GLfloat, GLfloat], 'VERSION_1_3')

# GL/glext.h:3476
glMultiTexCoord3fv = _link_function('glMultiTexCoord3fv', None, [GLenum, POINTER(GLfloat)], 'VERSION_1_3')

# GL/glext.h:3477
glMultiTexCoord3i = _link_function('glMultiTexCoord3i', None, [GLenum, GLint, GLint, GLint], 'VERSION_1_3')

# GL/glext.h:3478
glMultiTexCoord3iv = _link_function('glMultiTexCoord3iv', None, [GLenum, POINTER(GLint)], 'VERSION_1_3')

# GL/glext.h:3479
glMultiTexCoord3s = _link_function('glMultiTexCoord3s', None, [GLenum, GLshort, GLshort, GLshort], 'VERSION_1_3')

# GL/glext.h:3480
glMultiTexCoord3sv = _link_function('glMultiTexCoord3sv', None, [GLenum, POINTER(GLshort)], 'VERSION_1_3')

# GL/glext.h:3481
glMultiTexCoord4d = _link_function('glMultiTexCoord4d', None, [GLenum, GLdouble, GLdouble, GLdouble, GLdouble], 'VERSION_1_3')

# GL/glext.h:3482
glMultiTexCoord4dv = _link_function('glMultiTexCoord4dv', None, [GLenum, POINTER(GLdouble)], 'VERSION_1_3')

# GL/glext.h:3483
glMultiTexCoord4f = _link_function('glMultiTexCoord4f', None, [GLenum, GLfloat, GLfloat, GLfloat, GLfloat], 'VERSION_1_3')

# GL/glext.h:3484
glMultiTexCoord4fv = _link_function('glMultiTexCoord4fv', None, [GLenum, POINTER(GLfloat)], 'VERSION_1_3')

# GL/glext.h:3485
glMultiTexCoord4i = _link_function('glMultiTexCoord4i', None, [GLenum, GLint, GLint, GLint, GLint], 'VERSION_1_3')

# GL/glext.h:3486
glMultiTexCoord4iv = _link_function('glMultiTexCoord4iv', None, [GLenum, POINTER(GLint)], 'VERSION_1_3')

# GL/glext.h:3487
glMultiTexCoord4s = _link_function('glMultiTexCoord4s', None, [GLenum, GLshort, GLshort, GLshort, GLshort], 'VERSION_1_3')

# GL/glext.h:3488
glMultiTexCoord4sv = _link_function('glMultiTexCoord4sv', None, [GLenum, POINTER(GLshort)], 'VERSION_1_3')

# GL/glext.h:3489
glLoadTransposeMatrixf = _link_function('glLoadTransposeMatrixf', None, [POINTER(GLfloat)], 'VERSION_1_3')

# GL/glext.h:3490
glLoadTransposeMatrixd = _link_function('glLoadTransposeMatrixd', None, [POINTER(GLdouble)], 'VERSION_1_3')

# GL/glext.h:3491
glMultTransposeMatrixf = _link_function('glMultTransposeMatrixf', None, [POINTER(GLfloat)], 'VERSION_1_3')

# GL/glext.h:3492
glMultTransposeMatrixd = _link_function('glMultTransposeMatrixd', None, [POINTER(GLdouble)], 'VERSION_1_3')

# GL/glext.h:3493
glSampleCoverage = _link_function('glSampleCoverage', None, [GLclampf, GLboolean], 'VERSION_1_3')

# GL/glext.h:3494
glCompressedTexImage3D = _link_function('glCompressedTexImage3D', None, [GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)], 'VERSION_1_3')

# GL/glext.h:3495
glCompressedTexImage2D = _link_function('glCompressedTexImage2D', None, [GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)], 'VERSION_1_3')

# GL/glext.h:3496
glCompressedTexImage1D = _link_function('glCompressedTexImage1D', None, [GLenum, GLint, GLenum, GLsizei, GLint, GLsizei, POINTER(GLvoid)], 'VERSION_1_3')

# GL/glext.h:3497
glCompressedTexSubImage3D = _link_function('glCompressedTexSubImage3D', None, [GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)], 'VERSION_1_3')

# GL/glext.h:3498
glCompressedTexSubImage2D = _link_function('glCompressedTexSubImage2D', None, [GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)], 'VERSION_1_3')

# GL/glext.h:3499
glCompressedTexSubImage1D = _link_function('glCompressedTexSubImage1D', None, [GLenum, GLint, GLint, GLsizei, GLenum, GLsizei, POINTER(GLvoid)], 'VERSION_1_3')

# GL/glext.h:3500
glGetCompressedTexImage = _link_function('glGetCompressedTexImage', None, [GLenum, GLint, POINTER(GLvoid)], 'VERSION_1_3')

PFNGLACTIVETEXTUREPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:3502
PFNGLCLIENTACTIVETEXTUREPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:3503
PFNGLMULTITEXCOORD1DPROC = CFUNCTYPE(None, GLenum, GLdouble) 	# GL/glext.h:3504
PFNGLMULTITEXCOORD1DVPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:3505
PFNGLMULTITEXCOORD1FPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:3506
PFNGLMULTITEXCOORD1FVPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:3507
PFNGLMULTITEXCOORD1IPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:3508
PFNGLMULTITEXCOORD1IVPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:3509
PFNGLMULTITEXCOORD1SPROC = CFUNCTYPE(None, GLenum, GLshort) 	# GL/glext.h:3510
PFNGLMULTITEXCOORD1SVPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:3511
PFNGLMULTITEXCOORD2DPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble) 	# GL/glext.h:3512
PFNGLMULTITEXCOORD2DVPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:3513
PFNGLMULTITEXCOORD2FPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat) 	# GL/glext.h:3514
PFNGLMULTITEXCOORD2FVPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:3515
PFNGLMULTITEXCOORD2IPROC = CFUNCTYPE(None, GLenum, GLint, GLint) 	# GL/glext.h:3516
PFNGLMULTITEXCOORD2IVPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:3517
PFNGLMULTITEXCOORD2SPROC = CFUNCTYPE(None, GLenum, GLshort, GLshort) 	# GL/glext.h:3518
PFNGLMULTITEXCOORD2SVPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:3519
PFNGLMULTITEXCOORD3DPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:3520
PFNGLMULTITEXCOORD3DVPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:3521
PFNGLMULTITEXCOORD3FPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:3522
PFNGLMULTITEXCOORD3FVPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:3523
PFNGLMULTITEXCOORD3IPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint) 	# GL/glext.h:3524
PFNGLMULTITEXCOORD3IVPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:3525
PFNGLMULTITEXCOORD3SPROC = CFUNCTYPE(None, GLenum, GLshort, GLshort, GLshort) 	# GL/glext.h:3526
PFNGLMULTITEXCOORD3SVPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:3527
PFNGLMULTITEXCOORD4DPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:3528
PFNGLMULTITEXCOORD4DVPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:3529
PFNGLMULTITEXCOORD4FPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:3530
PFNGLMULTITEXCOORD4FVPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:3531
PFNGLMULTITEXCOORD4IPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint) 	# GL/glext.h:3532
PFNGLMULTITEXCOORD4IVPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:3533
PFNGLMULTITEXCOORD4SPROC = CFUNCTYPE(None, GLenum, GLshort, GLshort, GLshort, GLshort) 	# GL/glext.h:3534
PFNGLMULTITEXCOORD4SVPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:3535
PFNGLLOADTRANSPOSEMATRIXFPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:3536
PFNGLLOADTRANSPOSEMATRIXDPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:3537
PFNGLMULTTRANSPOSEMATRIXFPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:3538
PFNGLMULTTRANSPOSEMATRIXDPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:3539
PFNGLSAMPLECOVERAGEPROC = CFUNCTYPE(None, GLclampf, GLboolean) 	# GL/glext.h:3540
PFNGLCOMPRESSEDTEXIMAGE3DPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:3541
PFNGLCOMPRESSEDTEXIMAGE2DPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:3542
PFNGLCOMPRESSEDTEXIMAGE1DPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLsizei, GLint, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:3543
PFNGLCOMPRESSEDTEXSUBIMAGE3DPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:3544
PFNGLCOMPRESSEDTEXSUBIMAGE2DPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:3545
PFNGLCOMPRESSEDTEXSUBIMAGE1DPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLsizei, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:3546
PFNGLGETCOMPRESSEDTEXIMAGEPROC = CFUNCTYPE(None, GLenum, GLint, POINTER(GLvoid)) 	# GL/glext.h:3547
# VERSION_1_4 (GL/glext.h:3550)
GL_VERSION_1_4 = 1 	# GL/glext.h:3551
# GL/glext.h:3553
glBlendFuncSeparate = _link_function('glBlendFuncSeparate', None, [GLenum, GLenum, GLenum, GLenum], 'VERSION_1_4')

# GL/glext.h:3554
glFogCoordf = _link_function('glFogCoordf', None, [GLfloat], 'VERSION_1_4')

# GL/glext.h:3555
glFogCoordfv = _link_function('glFogCoordfv', None, [POINTER(GLfloat)], 'VERSION_1_4')

# GL/glext.h:3556
glFogCoordd = _link_function('glFogCoordd', None, [GLdouble], 'VERSION_1_4')

# GL/glext.h:3557
glFogCoorddv = _link_function('glFogCoorddv', None, [POINTER(GLdouble)], 'VERSION_1_4')

# GL/glext.h:3558
glFogCoordPointer = _link_function('glFogCoordPointer', None, [GLenum, GLsizei, POINTER(GLvoid)], 'VERSION_1_4')

# GL/glext.h:3559
glMultiDrawArrays = _link_function('glMultiDrawArrays', None, [GLenum, POINTER(GLint), POINTER(GLsizei), GLsizei], 'VERSION_1_4')

# GL/glext.h:3560
glMultiDrawElements = _link_function('glMultiDrawElements', None, [GLenum, POINTER(GLsizei), GLenum, POINTER(POINTER(GLvoid)), GLsizei], 'VERSION_1_4')

# GL/glext.h:3561
glPointParameterf = _link_function('glPointParameterf', None, [GLenum, GLfloat], 'VERSION_1_4')

# GL/glext.h:3562
glPointParameterfv = _link_function('glPointParameterfv', None, [GLenum, POINTER(GLfloat)], 'VERSION_1_4')

# GL/glext.h:3563
glPointParameteri = _link_function('glPointParameteri', None, [GLenum, GLint], 'VERSION_1_4')

# GL/glext.h:3564
glPointParameteriv = _link_function('glPointParameteriv', None, [GLenum, POINTER(GLint)], 'VERSION_1_4')

GLbyte = c_char 	# /usr/include/GL/gl.h:56
# GL/glext.h:3565
glSecondaryColor3b = _link_function('glSecondaryColor3b', None, [GLbyte, GLbyte, GLbyte], 'VERSION_1_4')

# GL/glext.h:3566
glSecondaryColor3bv = _link_function('glSecondaryColor3bv', None, [POINTER(GLbyte)], 'VERSION_1_4')

# GL/glext.h:3567
glSecondaryColor3d = _link_function('glSecondaryColor3d', None, [GLdouble, GLdouble, GLdouble], 'VERSION_1_4')

# GL/glext.h:3568
glSecondaryColor3dv = _link_function('glSecondaryColor3dv', None, [POINTER(GLdouble)], 'VERSION_1_4')

# GL/glext.h:3569
glSecondaryColor3f = _link_function('glSecondaryColor3f', None, [GLfloat, GLfloat, GLfloat], 'VERSION_1_4')

# GL/glext.h:3570
glSecondaryColor3fv = _link_function('glSecondaryColor3fv', None, [POINTER(GLfloat)], 'VERSION_1_4')

# GL/glext.h:3571
glSecondaryColor3i = _link_function('glSecondaryColor3i', None, [GLint, GLint, GLint], 'VERSION_1_4')

# GL/glext.h:3572
glSecondaryColor3iv = _link_function('glSecondaryColor3iv', None, [POINTER(GLint)], 'VERSION_1_4')

# GL/glext.h:3573
glSecondaryColor3s = _link_function('glSecondaryColor3s', None, [GLshort, GLshort, GLshort], 'VERSION_1_4')

# GL/glext.h:3574
glSecondaryColor3sv = _link_function('glSecondaryColor3sv', None, [POINTER(GLshort)], 'VERSION_1_4')

GLubyte = c_ubyte 	# /usr/include/GL/gl.h:60
# GL/glext.h:3575
glSecondaryColor3ub = _link_function('glSecondaryColor3ub', None, [GLubyte, GLubyte, GLubyte], 'VERSION_1_4')

# GL/glext.h:3576
glSecondaryColor3ubv = _link_function('glSecondaryColor3ubv', None, [POINTER(GLubyte)], 'VERSION_1_4')

# GL/glext.h:3577
glSecondaryColor3ui = _link_function('glSecondaryColor3ui', None, [GLuint, GLuint, GLuint], 'VERSION_1_4')

# GL/glext.h:3578
glSecondaryColor3uiv = _link_function('glSecondaryColor3uiv', None, [POINTER(GLuint)], 'VERSION_1_4')

GLushort = c_ushort 	# /usr/include/GL/gl.h:61
# GL/glext.h:3579
glSecondaryColor3us = _link_function('glSecondaryColor3us', None, [GLushort, GLushort, GLushort], 'VERSION_1_4')

# GL/glext.h:3580
glSecondaryColor3usv = _link_function('glSecondaryColor3usv', None, [POINTER(GLushort)], 'VERSION_1_4')

# GL/glext.h:3581
glSecondaryColorPointer = _link_function('glSecondaryColorPointer', None, [GLint, GLenum, GLsizei, POINTER(GLvoid)], 'VERSION_1_4')

# GL/glext.h:3582
glWindowPos2d = _link_function('glWindowPos2d', None, [GLdouble, GLdouble], 'VERSION_1_4')

# GL/glext.h:3583
glWindowPos2dv = _link_function('glWindowPos2dv', None, [POINTER(GLdouble)], 'VERSION_1_4')

# GL/glext.h:3584
glWindowPos2f = _link_function('glWindowPos2f', None, [GLfloat, GLfloat], 'VERSION_1_4')

# GL/glext.h:3585
glWindowPos2fv = _link_function('glWindowPos2fv', None, [POINTER(GLfloat)], 'VERSION_1_4')

# GL/glext.h:3586
glWindowPos2i = _link_function('glWindowPos2i', None, [GLint, GLint], 'VERSION_1_4')

# GL/glext.h:3587
glWindowPos2iv = _link_function('glWindowPos2iv', None, [POINTER(GLint)], 'VERSION_1_4')

# GL/glext.h:3588
glWindowPos2s = _link_function('glWindowPos2s', None, [GLshort, GLshort], 'VERSION_1_4')

# GL/glext.h:3589
glWindowPos2sv = _link_function('glWindowPos2sv', None, [POINTER(GLshort)], 'VERSION_1_4')

# GL/glext.h:3590
glWindowPos3d = _link_function('glWindowPos3d', None, [GLdouble, GLdouble, GLdouble], 'VERSION_1_4')

# GL/glext.h:3591
glWindowPos3dv = _link_function('glWindowPos3dv', None, [POINTER(GLdouble)], 'VERSION_1_4')

# GL/glext.h:3592
glWindowPos3f = _link_function('glWindowPos3f', None, [GLfloat, GLfloat, GLfloat], 'VERSION_1_4')

# GL/glext.h:3593
glWindowPos3fv = _link_function('glWindowPos3fv', None, [POINTER(GLfloat)], 'VERSION_1_4')

# GL/glext.h:3594
glWindowPos3i = _link_function('glWindowPos3i', None, [GLint, GLint, GLint], 'VERSION_1_4')

# GL/glext.h:3595
glWindowPos3iv = _link_function('glWindowPos3iv', None, [POINTER(GLint)], 'VERSION_1_4')

# GL/glext.h:3596
glWindowPos3s = _link_function('glWindowPos3s', None, [GLshort, GLshort, GLshort], 'VERSION_1_4')

# GL/glext.h:3597
glWindowPos3sv = _link_function('glWindowPos3sv', None, [POINTER(GLshort)], 'VERSION_1_4')

PFNGLBLENDFUNCSEPARATEPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:3599
PFNGLFOGCOORDFPROC = CFUNCTYPE(None, GLfloat) 	# GL/glext.h:3600
PFNGLFOGCOORDFVPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:3601
PFNGLFOGCOORDDPROC = CFUNCTYPE(None, GLdouble) 	# GL/glext.h:3602
PFNGLFOGCOORDDVPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:3603
PFNGLFOGCOORDPOINTERPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:3604
PFNGLMULTIDRAWARRAYSPROC = CFUNCTYPE(None, GLenum, POINTER(GLint), POINTER(GLsizei), GLsizei) 	# GL/glext.h:3605
PFNGLMULTIDRAWELEMENTSPROC = CFUNCTYPE(None, GLenum, POINTER(GLsizei), GLenum, POINTER(POINTER(GLvoid)), GLsizei) 	# GL/glext.h:3606
PFNGLPOINTPARAMETERFPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:3607
PFNGLPOINTPARAMETERFVPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:3608
PFNGLPOINTPARAMETERIPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:3609
PFNGLPOINTPARAMETERIVPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:3610
PFNGLSECONDARYCOLOR3BPROC = CFUNCTYPE(None, GLbyte, GLbyte, GLbyte) 	# GL/glext.h:3611
PFNGLSECONDARYCOLOR3BVPROC = CFUNCTYPE(None, POINTER(GLbyte)) 	# GL/glext.h:3612
PFNGLSECONDARYCOLOR3DPROC = CFUNCTYPE(None, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:3613
PFNGLSECONDARYCOLOR3DVPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:3614
PFNGLSECONDARYCOLOR3FPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:3615
PFNGLSECONDARYCOLOR3FVPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:3616
PFNGLSECONDARYCOLOR3IPROC = CFUNCTYPE(None, GLint, GLint, GLint) 	# GL/glext.h:3617
PFNGLSECONDARYCOLOR3IVPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:3618
PFNGLSECONDARYCOLOR3SPROC = CFUNCTYPE(None, GLshort, GLshort, GLshort) 	# GL/glext.h:3619
PFNGLSECONDARYCOLOR3SVPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:3620
PFNGLSECONDARYCOLOR3UBPROC = CFUNCTYPE(None, GLubyte, GLubyte, GLubyte) 	# GL/glext.h:3621
PFNGLSECONDARYCOLOR3UBVPROC = CFUNCTYPE(None, POINTER(GLubyte)) 	# GL/glext.h:3622
PFNGLSECONDARYCOLOR3UIPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint) 	# GL/glext.h:3623
PFNGLSECONDARYCOLOR3UIVPROC = CFUNCTYPE(None, POINTER(GLuint)) 	# GL/glext.h:3624
PFNGLSECONDARYCOLOR3USPROC = CFUNCTYPE(None, GLushort, GLushort, GLushort) 	# GL/glext.h:3625
PFNGLSECONDARYCOLOR3USVPROC = CFUNCTYPE(None, POINTER(GLushort)) 	# GL/glext.h:3626
PFNGLSECONDARYCOLORPOINTERPROC = CFUNCTYPE(None, GLint, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:3627
PFNGLWINDOWPOS2DPROC = CFUNCTYPE(None, GLdouble, GLdouble) 	# GL/glext.h:3628
PFNGLWINDOWPOS2DVPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:3629
PFNGLWINDOWPOS2FPROC = CFUNCTYPE(None, GLfloat, GLfloat) 	# GL/glext.h:3630
PFNGLWINDOWPOS2FVPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:3631
PFNGLWINDOWPOS2IPROC = CFUNCTYPE(None, GLint, GLint) 	# GL/glext.h:3632
PFNGLWINDOWPOS2IVPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:3633
PFNGLWINDOWPOS2SPROC = CFUNCTYPE(None, GLshort, GLshort) 	# GL/glext.h:3634
PFNGLWINDOWPOS2SVPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:3635
PFNGLWINDOWPOS3DPROC = CFUNCTYPE(None, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:3636
PFNGLWINDOWPOS3DVPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:3637
PFNGLWINDOWPOS3FPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:3638
PFNGLWINDOWPOS3FVPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:3639
PFNGLWINDOWPOS3IPROC = CFUNCTYPE(None, GLint, GLint, GLint) 	# GL/glext.h:3640
PFNGLWINDOWPOS3IVPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:3641
PFNGLWINDOWPOS3SPROC = CFUNCTYPE(None, GLshort, GLshort, GLshort) 	# GL/glext.h:3642
PFNGLWINDOWPOS3SVPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:3643
# VERSION_1_5 (GL/glext.h:3646)
GL_VERSION_1_5 = 1 	# GL/glext.h:3647
# GL/glext.h:3649
glGenQueries = _link_function('glGenQueries', None, [GLsizei, POINTER(GLuint)], 'VERSION_1_5')

# GL/glext.h:3650
glDeleteQueries = _link_function('glDeleteQueries', None, [GLsizei, POINTER(GLuint)], 'VERSION_1_5')

# GL/glext.h:3651
glIsQuery = _link_function('glIsQuery', GLboolean, [GLuint], 'VERSION_1_5')

# GL/glext.h:3652
glBeginQuery = _link_function('glBeginQuery', None, [GLenum, GLuint], 'VERSION_1_5')

# GL/glext.h:3653
glEndQuery = _link_function('glEndQuery', None, [GLenum], 'VERSION_1_5')

# GL/glext.h:3654
glGetQueryiv = _link_function('glGetQueryiv', None, [GLenum, GLenum, POINTER(GLint)], 'VERSION_1_5')

# GL/glext.h:3655
glGetQueryObjectiv = _link_function('glGetQueryObjectiv', None, [GLuint, GLenum, POINTER(GLint)], 'VERSION_1_5')

# GL/glext.h:3656
glGetQueryObjectuiv = _link_function('glGetQueryObjectuiv', None, [GLuint, GLenum, POINTER(GLuint)], 'VERSION_1_5')

# GL/glext.h:3657
glBindBuffer = _link_function('glBindBuffer', None, [GLenum, GLuint], 'VERSION_1_5')

# GL/glext.h:3658
glDeleteBuffers = _link_function('glDeleteBuffers', None, [GLsizei, POINTER(GLuint)], 'VERSION_1_5')

# GL/glext.h:3659
glGenBuffers = _link_function('glGenBuffers', None, [GLsizei, POINTER(GLuint)], 'VERSION_1_5')

# GL/glext.h:3660
glIsBuffer = _link_function('glIsBuffer', GLboolean, [GLuint], 'VERSION_1_5')

# GL/glext.h:3661
glBufferData = _link_function('glBufferData', None, [GLenum, GLsizeiptr, POINTER(GLvoid), GLenum], 'VERSION_1_5')

# GL/glext.h:3662
glBufferSubData = _link_function('glBufferSubData', None, [GLenum, GLintptr, GLsizeiptr, POINTER(GLvoid)], 'VERSION_1_5')

# GL/glext.h:3663
glGetBufferSubData = _link_function('glGetBufferSubData', None, [GLenum, GLintptr, GLsizeiptr, POINTER(GLvoid)], 'VERSION_1_5')

# GL/glext.h:3664
glMapBuffer = _link_function('glMapBuffer', POINTER(GLvoid), [GLenum, GLenum], 'VERSION_1_5')

# GL/glext.h:3665
glUnmapBuffer = _link_function('glUnmapBuffer', GLboolean, [GLenum], 'VERSION_1_5')

# GL/glext.h:3666
glGetBufferParameteriv = _link_function('glGetBufferParameteriv', None, [GLenum, GLenum, POINTER(GLint)], 'VERSION_1_5')

# GL/glext.h:3667
glGetBufferPointerv = _link_function('glGetBufferPointerv', None, [GLenum, GLenum, POINTER(POINTER(GLvoid))], 'VERSION_1_5')

PFNGLGENQUERIESPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:3669
PFNGLDELETEQUERIESPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:3670
PFNGLISQUERYPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:3671
PFNGLBEGINQUERYPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:3672
PFNGLENDQUERYPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:3673
PFNGLGETQUERYIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:3674
PFNGLGETQUERYOBJECTIVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:3675
PFNGLGETQUERYOBJECTUIVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLuint)) 	# GL/glext.h:3676
PFNGLBINDBUFFERPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:3677
PFNGLDELETEBUFFERSPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:3678
PFNGLGENBUFFERSPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:3679
PFNGLISBUFFERPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:3680
PFNGLBUFFERDATAPROC = CFUNCTYPE(None, GLenum, GLsizeiptr, POINTER(GLvoid), GLenum) 	# GL/glext.h:3681
PFNGLBUFFERSUBDATAPROC = CFUNCTYPE(None, GLenum, GLintptr, GLsizeiptr, POINTER(GLvoid)) 	# GL/glext.h:3682
PFNGLGETBUFFERSUBDATAPROC = CFUNCTYPE(None, GLenum, GLintptr, GLsizeiptr, POINTER(GLvoid)) 	# GL/glext.h:3683
PFNGLMAPBUFFERPROC = CFUNCTYPE(POINTER(GLvoid), GLenum, GLenum) 	# GL/glext.h:3684
PFNGLUNMAPBUFFERPROC = CFUNCTYPE(GLboolean, GLenum) 	# GL/glext.h:3685
PFNGLGETBUFFERPARAMETERIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:3686
PFNGLGETBUFFERPOINTERVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:3687
# VERSION_2_0 (GL/glext.h:3690)
GL_VERSION_2_0 = 1 	# GL/glext.h:3691
# GL/glext.h:3693
glBlendEquationSeparate = _link_function('glBlendEquationSeparate', None, [GLenum, GLenum], 'VERSION_2_0')

# GL/glext.h:3694
glDrawBuffers = _link_function('glDrawBuffers', None, [GLsizei, POINTER(GLenum)], 'VERSION_2_0')

# GL/glext.h:3695
glStencilOpSeparate = _link_function('glStencilOpSeparate', None, [GLenum, GLenum, GLenum, GLenum], 'VERSION_2_0')

# GL/glext.h:3696
glStencilFuncSeparate = _link_function('glStencilFuncSeparate', None, [GLenum, GLenum, GLint, GLuint], 'VERSION_2_0')

# GL/glext.h:3697
glStencilMaskSeparate = _link_function('glStencilMaskSeparate', None, [GLenum, GLuint], 'VERSION_2_0')

# GL/glext.h:3698
glAttachShader = _link_function('glAttachShader', None, [GLuint, GLuint], 'VERSION_2_0')

# GL/glext.h:3699
glBindAttribLocation = _link_function('glBindAttribLocation', None, [GLuint, GLuint, POINTER(GLchar)], 'VERSION_2_0')

# GL/glext.h:3700
glCompileShader = _link_function('glCompileShader', None, [GLuint], 'VERSION_2_0')

# GL/glext.h:3701
glCreateProgram = _link_function('glCreateProgram', GLuint, [], 'VERSION_2_0')

# GL/glext.h:3702
glCreateShader = _link_function('glCreateShader', GLuint, [GLenum], 'VERSION_2_0')

# GL/glext.h:3703
glDeleteProgram = _link_function('glDeleteProgram', None, [GLuint], 'VERSION_2_0')

# GL/glext.h:3704
glDeleteShader = _link_function('glDeleteShader', None, [GLuint], 'VERSION_2_0')

# GL/glext.h:3705
glDetachShader = _link_function('glDetachShader', None, [GLuint, GLuint], 'VERSION_2_0')

# GL/glext.h:3706
glDisableVertexAttribArray = _link_function('glDisableVertexAttribArray', None, [GLuint], 'VERSION_2_0')

# GL/glext.h:3707
glEnableVertexAttribArray = _link_function('glEnableVertexAttribArray', None, [GLuint], 'VERSION_2_0')

# GL/glext.h:3708
glGetActiveAttrib = _link_function('glGetActiveAttrib', None, [GLuint, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLint), POINTER(GLenum), POINTER(GLchar)], 'VERSION_2_0')

# GL/glext.h:3709
glGetActiveUniform = _link_function('glGetActiveUniform', None, [GLuint, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLint), POINTER(GLenum), POINTER(GLchar)], 'VERSION_2_0')

# GL/glext.h:3710
glGetAttachedShaders = _link_function('glGetAttachedShaders', None, [GLuint, GLsizei, POINTER(GLsizei), POINTER(GLuint)], 'VERSION_2_0')

# GL/glext.h:3711
glGetAttribLocation = _link_function('glGetAttribLocation', GLint, [GLuint, POINTER(GLchar)], 'VERSION_2_0')

# GL/glext.h:3712
glGetProgramiv = _link_function('glGetProgramiv', None, [GLuint, GLenum, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:3713
glGetProgramInfoLog = _link_function('glGetProgramInfoLog', None, [GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)], 'VERSION_2_0')

# GL/glext.h:3714
glGetShaderiv = _link_function('glGetShaderiv', None, [GLuint, GLenum, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:3715
glGetShaderInfoLog = _link_function('glGetShaderInfoLog', None, [GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)], 'VERSION_2_0')

# GL/glext.h:3716
glGetShaderSource = _link_function('glGetShaderSource', None, [GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)], 'VERSION_2_0')

# GL/glext.h:3717
glGetUniformLocation = _link_function('glGetUniformLocation', GLint, [GLuint, POINTER(GLchar)], 'VERSION_2_0')

# GL/glext.h:3718
glGetUniformfv = _link_function('glGetUniformfv', None, [GLuint, GLint, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:3719
glGetUniformiv = _link_function('glGetUniformiv', None, [GLuint, GLint, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:3720
glGetVertexAttribdv = _link_function('glGetVertexAttribdv', None, [GLuint, GLenum, POINTER(GLdouble)], 'VERSION_2_0')

# GL/glext.h:3721
glGetVertexAttribfv = _link_function('glGetVertexAttribfv', None, [GLuint, GLenum, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:3722
glGetVertexAttribiv = _link_function('glGetVertexAttribiv', None, [GLuint, GLenum, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:3723
glGetVertexAttribPointerv = _link_function('glGetVertexAttribPointerv', None, [GLuint, GLenum, POINTER(POINTER(GLvoid))], 'VERSION_2_0')

# GL/glext.h:3724
glIsProgram = _link_function('glIsProgram', GLboolean, [GLuint], 'VERSION_2_0')

# GL/glext.h:3725
glIsShader = _link_function('glIsShader', GLboolean, [GLuint], 'VERSION_2_0')

# GL/glext.h:3726
glLinkProgram = _link_function('glLinkProgram', None, [GLuint], 'VERSION_2_0')

# GL/glext.h:3727
glShaderSource = _link_function('glShaderSource', None, [GLuint, GLsizei, POINTER(POINTER(GLchar)), POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:3728
glUseProgram = _link_function('glUseProgram', None, [GLuint], 'VERSION_2_0')

# GL/glext.h:3729
glUniform1f = _link_function('glUniform1f', None, [GLint, GLfloat], 'VERSION_2_0')

# GL/glext.h:3730
glUniform2f = _link_function('glUniform2f', None, [GLint, GLfloat, GLfloat], 'VERSION_2_0')

# GL/glext.h:3731
glUniform3f = _link_function('glUniform3f', None, [GLint, GLfloat, GLfloat, GLfloat], 'VERSION_2_0')

# GL/glext.h:3732
glUniform4f = _link_function('glUniform4f', None, [GLint, GLfloat, GLfloat, GLfloat, GLfloat], 'VERSION_2_0')

# GL/glext.h:3733
glUniform1i = _link_function('glUniform1i', None, [GLint, GLint], 'VERSION_2_0')

# GL/glext.h:3734
glUniform2i = _link_function('glUniform2i', None, [GLint, GLint, GLint], 'VERSION_2_0')

# GL/glext.h:3735
glUniform3i = _link_function('glUniform3i', None, [GLint, GLint, GLint, GLint], 'VERSION_2_0')

# GL/glext.h:3736
glUniform4i = _link_function('glUniform4i', None, [GLint, GLint, GLint, GLint, GLint], 'VERSION_2_0')

# GL/glext.h:3737
glUniform1fv = _link_function('glUniform1fv', None, [GLint, GLsizei, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:3738
glUniform2fv = _link_function('glUniform2fv', None, [GLint, GLsizei, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:3739
glUniform3fv = _link_function('glUniform3fv', None, [GLint, GLsizei, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:3740
glUniform4fv = _link_function('glUniform4fv', None, [GLint, GLsizei, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:3741
glUniform1iv = _link_function('glUniform1iv', None, [GLint, GLsizei, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:3742
glUniform2iv = _link_function('glUniform2iv', None, [GLint, GLsizei, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:3743
glUniform3iv = _link_function('glUniform3iv', None, [GLint, GLsizei, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:3744
glUniform4iv = _link_function('glUniform4iv', None, [GLint, GLsizei, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:3745
glUniformMatrix2fv = _link_function('glUniformMatrix2fv', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:3746
glUniformMatrix3fv = _link_function('glUniformMatrix3fv', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:3747
glUniformMatrix4fv = _link_function('glUniformMatrix4fv', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:3748
glValidateProgram = _link_function('glValidateProgram', None, [GLuint], 'VERSION_2_0')

# GL/glext.h:3749
glVertexAttrib1d = _link_function('glVertexAttrib1d', None, [GLuint, GLdouble], 'VERSION_2_0')

# GL/glext.h:3750
glVertexAttrib1dv = _link_function('glVertexAttrib1dv', None, [GLuint, POINTER(GLdouble)], 'VERSION_2_0')

# GL/glext.h:3751
glVertexAttrib1f = _link_function('glVertexAttrib1f', None, [GLuint, GLfloat], 'VERSION_2_0')

# GL/glext.h:3752
glVertexAttrib1fv = _link_function('glVertexAttrib1fv', None, [GLuint, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:3753
glVertexAttrib1s = _link_function('glVertexAttrib1s', None, [GLuint, GLshort], 'VERSION_2_0')

# GL/glext.h:3754
glVertexAttrib1sv = _link_function('glVertexAttrib1sv', None, [GLuint, POINTER(GLshort)], 'VERSION_2_0')

# GL/glext.h:3755
glVertexAttrib2d = _link_function('glVertexAttrib2d', None, [GLuint, GLdouble, GLdouble], 'VERSION_2_0')

# GL/glext.h:3756
glVertexAttrib2dv = _link_function('glVertexAttrib2dv', None, [GLuint, POINTER(GLdouble)], 'VERSION_2_0')

# GL/glext.h:3757
glVertexAttrib2f = _link_function('glVertexAttrib2f', None, [GLuint, GLfloat, GLfloat], 'VERSION_2_0')

# GL/glext.h:3758
glVertexAttrib2fv = _link_function('glVertexAttrib2fv', None, [GLuint, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:3759
glVertexAttrib2s = _link_function('glVertexAttrib2s', None, [GLuint, GLshort, GLshort], 'VERSION_2_0')

# GL/glext.h:3760
glVertexAttrib2sv = _link_function('glVertexAttrib2sv', None, [GLuint, POINTER(GLshort)], 'VERSION_2_0')

# GL/glext.h:3761
glVertexAttrib3d = _link_function('glVertexAttrib3d', None, [GLuint, GLdouble, GLdouble, GLdouble], 'VERSION_2_0')

# GL/glext.h:3762
glVertexAttrib3dv = _link_function('glVertexAttrib3dv', None, [GLuint, POINTER(GLdouble)], 'VERSION_2_0')

# GL/glext.h:3763
glVertexAttrib3f = _link_function('glVertexAttrib3f', None, [GLuint, GLfloat, GLfloat, GLfloat], 'VERSION_2_0')

# GL/glext.h:3764
glVertexAttrib3fv = _link_function('glVertexAttrib3fv', None, [GLuint, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:3765
glVertexAttrib3s = _link_function('glVertexAttrib3s', None, [GLuint, GLshort, GLshort, GLshort], 'VERSION_2_0')

# GL/glext.h:3766
glVertexAttrib3sv = _link_function('glVertexAttrib3sv', None, [GLuint, POINTER(GLshort)], 'VERSION_2_0')

# GL/glext.h:3767
glVertexAttrib4Nbv = _link_function('glVertexAttrib4Nbv', None, [GLuint, POINTER(GLbyte)], 'VERSION_2_0')

# GL/glext.h:3768
glVertexAttrib4Niv = _link_function('glVertexAttrib4Niv', None, [GLuint, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:3769
glVertexAttrib4Nsv = _link_function('glVertexAttrib4Nsv', None, [GLuint, POINTER(GLshort)], 'VERSION_2_0')

# GL/glext.h:3770
glVertexAttrib4Nub = _link_function('glVertexAttrib4Nub', None, [GLuint, GLubyte, GLubyte, GLubyte, GLubyte], 'VERSION_2_0')

# GL/glext.h:3771
glVertexAttrib4Nubv = _link_function('glVertexAttrib4Nubv', None, [GLuint, POINTER(GLubyte)], 'VERSION_2_0')

# GL/glext.h:3772
glVertexAttrib4Nuiv = _link_function('glVertexAttrib4Nuiv', None, [GLuint, POINTER(GLuint)], 'VERSION_2_0')

# GL/glext.h:3773
glVertexAttrib4Nusv = _link_function('glVertexAttrib4Nusv', None, [GLuint, POINTER(GLushort)], 'VERSION_2_0')

# GL/glext.h:3774
glVertexAttrib4bv = _link_function('glVertexAttrib4bv', None, [GLuint, POINTER(GLbyte)], 'VERSION_2_0')

# GL/glext.h:3775
glVertexAttrib4d = _link_function('glVertexAttrib4d', None, [GLuint, GLdouble, GLdouble, GLdouble, GLdouble], 'VERSION_2_0')

# GL/glext.h:3776
glVertexAttrib4dv = _link_function('glVertexAttrib4dv', None, [GLuint, POINTER(GLdouble)], 'VERSION_2_0')

# GL/glext.h:3777
glVertexAttrib4f = _link_function('glVertexAttrib4f', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat], 'VERSION_2_0')

# GL/glext.h:3778
glVertexAttrib4fv = _link_function('glVertexAttrib4fv', None, [GLuint, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:3779
glVertexAttrib4iv = _link_function('glVertexAttrib4iv', None, [GLuint, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:3780
glVertexAttrib4s = _link_function('glVertexAttrib4s', None, [GLuint, GLshort, GLshort, GLshort, GLshort], 'VERSION_2_0')

# GL/glext.h:3781
glVertexAttrib4sv = _link_function('glVertexAttrib4sv', None, [GLuint, POINTER(GLshort)], 'VERSION_2_0')

# GL/glext.h:3782
glVertexAttrib4ubv = _link_function('glVertexAttrib4ubv', None, [GLuint, POINTER(GLubyte)], 'VERSION_2_0')

# GL/glext.h:3783
glVertexAttrib4uiv = _link_function('glVertexAttrib4uiv', None, [GLuint, POINTER(GLuint)], 'VERSION_2_0')

# GL/glext.h:3784
glVertexAttrib4usv = _link_function('glVertexAttrib4usv', None, [GLuint, POINTER(GLushort)], 'VERSION_2_0')

# GL/glext.h:3785
glVertexAttribPointer = _link_function('glVertexAttribPointer', None, [GLuint, GLint, GLenum, GLboolean, GLsizei, POINTER(GLvoid)], 'VERSION_2_0')

PFNGLBLENDEQUATIONSEPARATEPROC = CFUNCTYPE(None, GLenum, GLenum) 	# GL/glext.h:3787
PFNGLDRAWBUFFERSPROC = CFUNCTYPE(None, GLsizei, POINTER(GLenum)) 	# GL/glext.h:3788
PFNGLSTENCILOPSEPARATEPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:3789
PFNGLSTENCILFUNCSEPARATEPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLuint) 	# GL/glext.h:3790
PFNGLSTENCILMASKSEPARATEPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:3791
PFNGLATTACHSHADERPROC = CFUNCTYPE(None, GLuint, GLuint) 	# GL/glext.h:3792
PFNGLBINDATTRIBLOCATIONPROC = CFUNCTYPE(None, GLuint, GLuint, POINTER(GLchar)) 	# GL/glext.h:3793
PFNGLCOMPILESHADERPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:3794
PFNGLCREATEPROGRAMPROC = CFUNCTYPE(GLuint) 	# GL/glext.h:3795
PFNGLCREATESHADERPROC = CFUNCTYPE(GLuint, GLenum) 	# GL/glext.h:3796
PFNGLDELETEPROGRAMPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:3797
PFNGLDELETESHADERPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:3798
PFNGLDETACHSHADERPROC = CFUNCTYPE(None, GLuint, GLuint) 	# GL/glext.h:3799
PFNGLDISABLEVERTEXATTRIBARRAYPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:3800
PFNGLENABLEVERTEXATTRIBARRAYPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:3801
PFNGLGETACTIVEATTRIBPROC = CFUNCTYPE(None, GLuint, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLint), POINTER(GLenum), POINTER(GLchar)) 	# GL/glext.h:3802
PFNGLGETACTIVEUNIFORMPROC = CFUNCTYPE(None, GLuint, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLint), POINTER(GLenum), POINTER(GLchar)) 	# GL/glext.h:3803
PFNGLGETATTACHEDSHADERSPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLuint)) 	# GL/glext.h:3804
PFNGLGETATTRIBLOCATIONPROC = CFUNCTYPE(GLint, GLuint, POINTER(GLchar)) 	# GL/glext.h:3805
PFNGLGETPROGRAMIVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:3806
PFNGLGETPROGRAMINFOLOGPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)) 	# GL/glext.h:3807
PFNGLGETSHADERIVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:3808
PFNGLGETSHADERINFOLOGPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)) 	# GL/glext.h:3809
PFNGLGETSHADERSOURCEPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)) 	# GL/glext.h:3810
PFNGLGETUNIFORMLOCATIONPROC = CFUNCTYPE(GLint, GLuint, POINTER(GLchar)) 	# GL/glext.h:3811
PFNGLGETUNIFORMFVPROC = CFUNCTYPE(None, GLuint, GLint, POINTER(GLfloat)) 	# GL/glext.h:3812
PFNGLGETUNIFORMIVPROC = CFUNCTYPE(None, GLuint, GLint, POINTER(GLint)) 	# GL/glext.h:3813
PFNGLGETVERTEXATTRIBDVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLdouble)) 	# GL/glext.h:3814
PFNGLGETVERTEXATTRIBFVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:3815
PFNGLGETVERTEXATTRIBIVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:3816
PFNGLGETVERTEXATTRIBPOINTERVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:3817
PFNGLISPROGRAMPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:3818
PFNGLISSHADERPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:3819
PFNGLLINKPROGRAMPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:3820
PFNGLSHADERSOURCEPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(POINTER(GLchar)), POINTER(GLint)) 	# GL/glext.h:3821
PFNGLUSEPROGRAMPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:3822
PFNGLUNIFORM1FPROC = CFUNCTYPE(None, GLint, GLfloat) 	# GL/glext.h:3823
PFNGLUNIFORM2FPROC = CFUNCTYPE(None, GLint, GLfloat, GLfloat) 	# GL/glext.h:3824
PFNGLUNIFORM3FPROC = CFUNCTYPE(None, GLint, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:3825
PFNGLUNIFORM4FPROC = CFUNCTYPE(None, GLint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:3826
PFNGLUNIFORM1IPROC = CFUNCTYPE(None, GLint, GLint) 	# GL/glext.h:3827
PFNGLUNIFORM2IPROC = CFUNCTYPE(None, GLint, GLint, GLint) 	# GL/glext.h:3828
PFNGLUNIFORM3IPROC = CFUNCTYPE(None, GLint, GLint, GLint, GLint) 	# GL/glext.h:3829
PFNGLUNIFORM4IPROC = CFUNCTYPE(None, GLint, GLint, GLint, GLint, GLint) 	# GL/glext.h:3830
PFNGLUNIFORM1FVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:3831
PFNGLUNIFORM2FVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:3832
PFNGLUNIFORM3FVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:3833
PFNGLUNIFORM4FVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:3834
PFNGLUNIFORM1IVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:3835
PFNGLUNIFORM2IVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:3836
PFNGLUNIFORM3IVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:3837
PFNGLUNIFORM4IVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:3838
PFNGLUNIFORMMATRIX2FVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:3839
PFNGLUNIFORMMATRIX3FVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:3840
PFNGLUNIFORMMATRIX4FVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:3841
PFNGLVALIDATEPROGRAMPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:3842
PFNGLVERTEXATTRIB1DPROC = CFUNCTYPE(None, GLuint, GLdouble) 	# GL/glext.h:3843
PFNGLVERTEXATTRIB1DVPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:3844
PFNGLVERTEXATTRIB1FPROC = CFUNCTYPE(None, GLuint, GLfloat) 	# GL/glext.h:3845
PFNGLVERTEXATTRIB1FVPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:3846
PFNGLVERTEXATTRIB1SPROC = CFUNCTYPE(None, GLuint, GLshort) 	# GL/glext.h:3847
PFNGLVERTEXATTRIB1SVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:3848
PFNGLVERTEXATTRIB2DPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble) 	# GL/glext.h:3849
PFNGLVERTEXATTRIB2DVPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:3850
PFNGLVERTEXATTRIB2FPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat) 	# GL/glext.h:3851
PFNGLVERTEXATTRIB2FVPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:3852
PFNGLVERTEXATTRIB2SPROC = CFUNCTYPE(None, GLuint, GLshort, GLshort) 	# GL/glext.h:3853
PFNGLVERTEXATTRIB2SVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:3854
PFNGLVERTEXATTRIB3DPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:3855
PFNGLVERTEXATTRIB3DVPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:3856
PFNGLVERTEXATTRIB3FPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:3857
PFNGLVERTEXATTRIB3FVPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:3858
PFNGLVERTEXATTRIB3SPROC = CFUNCTYPE(None, GLuint, GLshort, GLshort, GLshort) 	# GL/glext.h:3859
PFNGLVERTEXATTRIB3SVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:3860
PFNGLVERTEXATTRIB4NBVPROC = CFUNCTYPE(None, GLuint, POINTER(GLbyte)) 	# GL/glext.h:3861
PFNGLVERTEXATTRIB4NIVPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:3862
PFNGLVERTEXATTRIB4NSVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:3863
PFNGLVERTEXATTRIB4NUBPROC = CFUNCTYPE(None, GLuint, GLubyte, GLubyte, GLubyte, GLubyte) 	# GL/glext.h:3864
PFNGLVERTEXATTRIB4NUBVPROC = CFUNCTYPE(None, GLuint, POINTER(GLubyte)) 	# GL/glext.h:3865
PFNGLVERTEXATTRIB4NUIVPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:3866
PFNGLVERTEXATTRIB4NUSVPROC = CFUNCTYPE(None, GLuint, POINTER(GLushort)) 	# GL/glext.h:3867
PFNGLVERTEXATTRIB4BVPROC = CFUNCTYPE(None, GLuint, POINTER(GLbyte)) 	# GL/glext.h:3868
PFNGLVERTEXATTRIB4DPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:3869
PFNGLVERTEXATTRIB4DVPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:3870
PFNGLVERTEXATTRIB4FPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:3871
PFNGLVERTEXATTRIB4FVPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:3872
PFNGLVERTEXATTRIB4IVPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:3873
PFNGLVERTEXATTRIB4SPROC = CFUNCTYPE(None, GLuint, GLshort, GLshort, GLshort, GLshort) 	# GL/glext.h:3874
PFNGLVERTEXATTRIB4SVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:3875
PFNGLVERTEXATTRIB4UBVPROC = CFUNCTYPE(None, GLuint, POINTER(GLubyte)) 	# GL/glext.h:3876
PFNGLVERTEXATTRIB4UIVPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:3877
PFNGLVERTEXATTRIB4USVPROC = CFUNCTYPE(None, GLuint, POINTER(GLushort)) 	# GL/glext.h:3878
PFNGLVERTEXATTRIBPOINTERPROC = CFUNCTYPE(None, GLuint, GLint, GLenum, GLboolean, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:3879
# VERSION_2_1 (GL/glext.h:3882)
GL_VERSION_2_1 = 1 	# GL/glext.h:3883
# GL/glext.h:3885
glUniformMatrix2x3fv = _link_function('glUniformMatrix2x3fv', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'VERSION_2_1')

# GL/glext.h:3886
glUniformMatrix3x2fv = _link_function('glUniformMatrix3x2fv', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'VERSION_2_1')

# GL/glext.h:3887
glUniformMatrix2x4fv = _link_function('glUniformMatrix2x4fv', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'VERSION_2_1')

# GL/glext.h:3888
glUniformMatrix4x2fv = _link_function('glUniformMatrix4x2fv', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'VERSION_2_1')

# GL/glext.h:3889
glUniformMatrix3x4fv = _link_function('glUniformMatrix3x4fv', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'VERSION_2_1')

# GL/glext.h:3890
glUniformMatrix4x3fv = _link_function('glUniformMatrix4x3fv', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'VERSION_2_1')

PFNGLUNIFORMMATRIX2X3FVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:3892
PFNGLUNIFORMMATRIX3X2FVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:3893
PFNGLUNIFORMMATRIX2X4FVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:3894
PFNGLUNIFORMMATRIX4X2FVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:3895
PFNGLUNIFORMMATRIX3X4FVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:3896
PFNGLUNIFORMMATRIX4X3FVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:3897
# ARB_multitexture (GL/glext.h:3900)
GL_ARB_multitexture = 1 	# GL/glext.h:3901
# GL/glext.h:3903
glActiveTextureARB = _link_function('glActiveTextureARB', None, [GLenum], 'ARB_multitexture')

# GL/glext.h:3904
glClientActiveTextureARB = _link_function('glClientActiveTextureARB', None, [GLenum], 'ARB_multitexture')

# GL/glext.h:3905
glMultiTexCoord1dARB = _link_function('glMultiTexCoord1dARB', None, [GLenum, GLdouble], 'ARB_multitexture')

# GL/glext.h:3906
glMultiTexCoord1dvARB = _link_function('glMultiTexCoord1dvARB', None, [GLenum, POINTER(GLdouble)], 'ARB_multitexture')

# GL/glext.h:3907
glMultiTexCoord1fARB = _link_function('glMultiTexCoord1fARB', None, [GLenum, GLfloat], 'ARB_multitexture')

# GL/glext.h:3908
glMultiTexCoord1fvARB = _link_function('glMultiTexCoord1fvARB', None, [GLenum, POINTER(GLfloat)], 'ARB_multitexture')

# GL/glext.h:3909
glMultiTexCoord1iARB = _link_function('glMultiTexCoord1iARB', None, [GLenum, GLint], 'ARB_multitexture')

# GL/glext.h:3910
glMultiTexCoord1ivARB = _link_function('glMultiTexCoord1ivARB', None, [GLenum, POINTER(GLint)], 'ARB_multitexture')

# GL/glext.h:3911
glMultiTexCoord1sARB = _link_function('glMultiTexCoord1sARB', None, [GLenum, GLshort], 'ARB_multitexture')

# GL/glext.h:3912
glMultiTexCoord1svARB = _link_function('glMultiTexCoord1svARB', None, [GLenum, POINTER(GLshort)], 'ARB_multitexture')

# GL/glext.h:3913
glMultiTexCoord2dARB = _link_function('glMultiTexCoord2dARB', None, [GLenum, GLdouble, GLdouble], 'ARB_multitexture')

# GL/glext.h:3914
glMultiTexCoord2dvARB = _link_function('glMultiTexCoord2dvARB', None, [GLenum, POINTER(GLdouble)], 'ARB_multitexture')

# GL/glext.h:3915
glMultiTexCoord2fARB = _link_function('glMultiTexCoord2fARB', None, [GLenum, GLfloat, GLfloat], 'ARB_multitexture')

# GL/glext.h:3916
glMultiTexCoord2fvARB = _link_function('glMultiTexCoord2fvARB', None, [GLenum, POINTER(GLfloat)], 'ARB_multitexture')

# GL/glext.h:3917
glMultiTexCoord2iARB = _link_function('glMultiTexCoord2iARB', None, [GLenum, GLint, GLint], 'ARB_multitexture')

# GL/glext.h:3918
glMultiTexCoord2ivARB = _link_function('glMultiTexCoord2ivARB', None, [GLenum, POINTER(GLint)], 'ARB_multitexture')

# GL/glext.h:3919
glMultiTexCoord2sARB = _link_function('glMultiTexCoord2sARB', None, [GLenum, GLshort, GLshort], 'ARB_multitexture')

# GL/glext.h:3920
glMultiTexCoord2svARB = _link_function('glMultiTexCoord2svARB', None, [GLenum, POINTER(GLshort)], 'ARB_multitexture')

# GL/glext.h:3921
glMultiTexCoord3dARB = _link_function('glMultiTexCoord3dARB', None, [GLenum, GLdouble, GLdouble, GLdouble], 'ARB_multitexture')

# GL/glext.h:3922
glMultiTexCoord3dvARB = _link_function('glMultiTexCoord3dvARB', None, [GLenum, POINTER(GLdouble)], 'ARB_multitexture')

# GL/glext.h:3923
glMultiTexCoord3fARB = _link_function('glMultiTexCoord3fARB', None, [GLenum, GLfloat, GLfloat, GLfloat], 'ARB_multitexture')

# GL/glext.h:3924
glMultiTexCoord3fvARB = _link_function('glMultiTexCoord3fvARB', None, [GLenum, POINTER(GLfloat)], 'ARB_multitexture')

# GL/glext.h:3925
glMultiTexCoord3iARB = _link_function('glMultiTexCoord3iARB', None, [GLenum, GLint, GLint, GLint], 'ARB_multitexture')

# GL/glext.h:3926
glMultiTexCoord3ivARB = _link_function('glMultiTexCoord3ivARB', None, [GLenum, POINTER(GLint)], 'ARB_multitexture')

# GL/glext.h:3927
glMultiTexCoord3sARB = _link_function('glMultiTexCoord3sARB', None, [GLenum, GLshort, GLshort, GLshort], 'ARB_multitexture')

# GL/glext.h:3928
glMultiTexCoord3svARB = _link_function('glMultiTexCoord3svARB', None, [GLenum, POINTER(GLshort)], 'ARB_multitexture')

# GL/glext.h:3929
glMultiTexCoord4dARB = _link_function('glMultiTexCoord4dARB', None, [GLenum, GLdouble, GLdouble, GLdouble, GLdouble], 'ARB_multitexture')

# GL/glext.h:3930
glMultiTexCoord4dvARB = _link_function('glMultiTexCoord4dvARB', None, [GLenum, POINTER(GLdouble)], 'ARB_multitexture')

# GL/glext.h:3931
glMultiTexCoord4fARB = _link_function('glMultiTexCoord4fARB', None, [GLenum, GLfloat, GLfloat, GLfloat, GLfloat], 'ARB_multitexture')

# GL/glext.h:3932
glMultiTexCoord4fvARB = _link_function('glMultiTexCoord4fvARB', None, [GLenum, POINTER(GLfloat)], 'ARB_multitexture')

# GL/glext.h:3933
glMultiTexCoord4iARB = _link_function('glMultiTexCoord4iARB', None, [GLenum, GLint, GLint, GLint, GLint], 'ARB_multitexture')

# GL/glext.h:3934
glMultiTexCoord4ivARB = _link_function('glMultiTexCoord4ivARB', None, [GLenum, POINTER(GLint)], 'ARB_multitexture')

# GL/glext.h:3935
glMultiTexCoord4sARB = _link_function('glMultiTexCoord4sARB', None, [GLenum, GLshort, GLshort, GLshort, GLshort], 'ARB_multitexture')

# GL/glext.h:3936
glMultiTexCoord4svARB = _link_function('glMultiTexCoord4svARB', None, [GLenum, POINTER(GLshort)], 'ARB_multitexture')

PFNGLACTIVETEXTUREARBPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:3938
PFNGLCLIENTACTIVETEXTUREARBPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:3939
PFNGLMULTITEXCOORD1DARBPROC = CFUNCTYPE(None, GLenum, GLdouble) 	# GL/glext.h:3940
PFNGLMULTITEXCOORD1DVARBPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:3941
PFNGLMULTITEXCOORD1FARBPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:3942
PFNGLMULTITEXCOORD1FVARBPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:3943
PFNGLMULTITEXCOORD1IARBPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:3944
PFNGLMULTITEXCOORD1IVARBPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:3945
PFNGLMULTITEXCOORD1SARBPROC = CFUNCTYPE(None, GLenum, GLshort) 	# GL/glext.h:3946
PFNGLMULTITEXCOORD1SVARBPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:3947
PFNGLMULTITEXCOORD2DARBPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble) 	# GL/glext.h:3948
PFNGLMULTITEXCOORD2DVARBPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:3949
PFNGLMULTITEXCOORD2FARBPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat) 	# GL/glext.h:3950
PFNGLMULTITEXCOORD2FVARBPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:3951
PFNGLMULTITEXCOORD2IARBPROC = CFUNCTYPE(None, GLenum, GLint, GLint) 	# GL/glext.h:3952
PFNGLMULTITEXCOORD2IVARBPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:3953
PFNGLMULTITEXCOORD2SARBPROC = CFUNCTYPE(None, GLenum, GLshort, GLshort) 	# GL/glext.h:3954
PFNGLMULTITEXCOORD2SVARBPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:3955
PFNGLMULTITEXCOORD3DARBPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:3956
PFNGLMULTITEXCOORD3DVARBPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:3957
PFNGLMULTITEXCOORD3FARBPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:3958
PFNGLMULTITEXCOORD3FVARBPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:3959
PFNGLMULTITEXCOORD3IARBPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint) 	# GL/glext.h:3960
PFNGLMULTITEXCOORD3IVARBPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:3961
PFNGLMULTITEXCOORD3SARBPROC = CFUNCTYPE(None, GLenum, GLshort, GLshort, GLshort) 	# GL/glext.h:3962
PFNGLMULTITEXCOORD3SVARBPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:3963
PFNGLMULTITEXCOORD4DARBPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:3964
PFNGLMULTITEXCOORD4DVARBPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:3965
PFNGLMULTITEXCOORD4FARBPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:3966
PFNGLMULTITEXCOORD4FVARBPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:3967
PFNGLMULTITEXCOORD4IARBPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint) 	# GL/glext.h:3968
PFNGLMULTITEXCOORD4IVARBPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:3969
PFNGLMULTITEXCOORD4SARBPROC = CFUNCTYPE(None, GLenum, GLshort, GLshort, GLshort, GLshort) 	# GL/glext.h:3970
PFNGLMULTITEXCOORD4SVARBPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:3971
# ARB_transpose_matrix (GL/glext.h:3974)
GL_ARB_transpose_matrix = 1 	# GL/glext.h:3975
# GL/glext.h:3977
glLoadTransposeMatrixfARB = _link_function('glLoadTransposeMatrixfARB', None, [POINTER(GLfloat)], 'ARB_transpose_matrix')

# GL/glext.h:3978
glLoadTransposeMatrixdARB = _link_function('glLoadTransposeMatrixdARB', None, [POINTER(GLdouble)], 'ARB_transpose_matrix')

# GL/glext.h:3979
glMultTransposeMatrixfARB = _link_function('glMultTransposeMatrixfARB', None, [POINTER(GLfloat)], 'ARB_transpose_matrix')

# GL/glext.h:3980
glMultTransposeMatrixdARB = _link_function('glMultTransposeMatrixdARB', None, [POINTER(GLdouble)], 'ARB_transpose_matrix')

PFNGLLOADTRANSPOSEMATRIXFARBPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:3982
PFNGLLOADTRANSPOSEMATRIXDARBPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:3983
PFNGLMULTTRANSPOSEMATRIXFARBPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:3984
PFNGLMULTTRANSPOSEMATRIXDARBPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:3985
# ARB_multisample (GL/glext.h:3988)
GL_ARB_multisample = 1 	# GL/glext.h:3989
# GL/glext.h:3991
glSampleCoverageARB = _link_function('glSampleCoverageARB', None, [GLclampf, GLboolean], 'ARB_multisample')

PFNGLSAMPLECOVERAGEARBPROC = CFUNCTYPE(None, GLclampf, GLboolean) 	# GL/glext.h:3993
# ARB_texture_env_add (GL/glext.h:3996)
GL_ARB_texture_env_add = 1 	# GL/glext.h:3997
# ARB_texture_cube_map (GL/glext.h:4000)
GL_ARB_texture_cube_map = 1 	# GL/glext.h:4001
# ARB_texture_compression (GL/glext.h:4004)
GL_ARB_texture_compression = 1 	# GL/glext.h:4005
# GL/glext.h:4007
glCompressedTexImage3DARB = _link_function('glCompressedTexImage3DARB', None, [GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)], 'ARB_texture_compression')

# GL/glext.h:4008
glCompressedTexImage2DARB = _link_function('glCompressedTexImage2DARB', None, [GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)], 'ARB_texture_compression')

# GL/glext.h:4009
glCompressedTexImage1DARB = _link_function('glCompressedTexImage1DARB', None, [GLenum, GLint, GLenum, GLsizei, GLint, GLsizei, POINTER(GLvoid)], 'ARB_texture_compression')

# GL/glext.h:4010
glCompressedTexSubImage3DARB = _link_function('glCompressedTexSubImage3DARB', None, [GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)], 'ARB_texture_compression')

# GL/glext.h:4011
glCompressedTexSubImage2DARB = _link_function('glCompressedTexSubImage2DARB', None, [GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)], 'ARB_texture_compression')

# GL/glext.h:4012
glCompressedTexSubImage1DARB = _link_function('glCompressedTexSubImage1DARB', None, [GLenum, GLint, GLint, GLsizei, GLenum, GLsizei, POINTER(GLvoid)], 'ARB_texture_compression')

# GL/glext.h:4013
glGetCompressedTexImageARB = _link_function('glGetCompressedTexImageARB', None, [GLenum, GLint, POINTER(GLvoid)], 'ARB_texture_compression')

PFNGLCOMPRESSEDTEXIMAGE3DARBPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:4015
PFNGLCOMPRESSEDTEXIMAGE2DARBPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:4016
PFNGLCOMPRESSEDTEXIMAGE1DARBPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLsizei, GLint, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:4017
PFNGLCOMPRESSEDTEXSUBIMAGE3DARBPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:4018
PFNGLCOMPRESSEDTEXSUBIMAGE2DARBPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:4019
PFNGLCOMPRESSEDTEXSUBIMAGE1DARBPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLsizei, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:4020
PFNGLGETCOMPRESSEDTEXIMAGEARBPROC = CFUNCTYPE(None, GLenum, GLint, POINTER(GLvoid)) 	# GL/glext.h:4021
# ARB_texture_border_clamp (GL/glext.h:4024)
GL_ARB_texture_border_clamp = 1 	# GL/glext.h:4025
# ARB_point_parameters (GL/glext.h:4028)
GL_ARB_point_parameters = 1 	# GL/glext.h:4029
# GL/glext.h:4031
glPointParameterfARB = _link_function('glPointParameterfARB', None, [GLenum, GLfloat], 'ARB_point_parameters')

# GL/glext.h:4032
glPointParameterfvARB = _link_function('glPointParameterfvARB', None, [GLenum, POINTER(GLfloat)], 'ARB_point_parameters')

PFNGLPOINTPARAMETERFARBPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:4034
PFNGLPOINTPARAMETERFVARBPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4035
# ARB_vertex_blend (GL/glext.h:4038)
GL_ARB_vertex_blend = 1 	# GL/glext.h:4039
# GL/glext.h:4041
glWeightbvARB = _link_function('glWeightbvARB', None, [GLint, POINTER(GLbyte)], 'ARB_vertex_blend')

# GL/glext.h:4042
glWeightsvARB = _link_function('glWeightsvARB', None, [GLint, POINTER(GLshort)], 'ARB_vertex_blend')

# GL/glext.h:4043
glWeightivARB = _link_function('glWeightivARB', None, [GLint, POINTER(GLint)], 'ARB_vertex_blend')

# GL/glext.h:4044
glWeightfvARB = _link_function('glWeightfvARB', None, [GLint, POINTER(GLfloat)], 'ARB_vertex_blend')

# GL/glext.h:4045
glWeightdvARB = _link_function('glWeightdvARB', None, [GLint, POINTER(GLdouble)], 'ARB_vertex_blend')

# GL/glext.h:4046
glWeightubvARB = _link_function('glWeightubvARB', None, [GLint, POINTER(GLubyte)], 'ARB_vertex_blend')

# GL/glext.h:4047
glWeightusvARB = _link_function('glWeightusvARB', None, [GLint, POINTER(GLushort)], 'ARB_vertex_blend')

# GL/glext.h:4048
glWeightuivARB = _link_function('glWeightuivARB', None, [GLint, POINTER(GLuint)], 'ARB_vertex_blend')

# GL/glext.h:4049
glWeightPointerARB = _link_function('glWeightPointerARB', None, [GLint, GLenum, GLsizei, POINTER(GLvoid)], 'ARB_vertex_blend')

# GL/glext.h:4050
glVertexBlendARB = _link_function('glVertexBlendARB', None, [GLint], 'ARB_vertex_blend')

PFNGLWEIGHTBVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLbyte)) 	# GL/glext.h:4052
PFNGLWEIGHTSVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLshort)) 	# GL/glext.h:4053
PFNGLWEIGHTIVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLint)) 	# GL/glext.h:4054
PFNGLWEIGHTFVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLfloat)) 	# GL/glext.h:4055
PFNGLWEIGHTDVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLdouble)) 	# GL/glext.h:4056
PFNGLWEIGHTUBVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLubyte)) 	# GL/glext.h:4057
PFNGLWEIGHTUSVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLushort)) 	# GL/glext.h:4058
PFNGLWEIGHTUIVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLuint)) 	# GL/glext.h:4059
PFNGLWEIGHTPOINTERARBPROC = CFUNCTYPE(None, GLint, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:4060
PFNGLVERTEXBLENDARBPROC = CFUNCTYPE(None, GLint) 	# GL/glext.h:4061
# ARB_matrix_palette (GL/glext.h:4064)
GL_ARB_matrix_palette = 1 	# GL/glext.h:4065
# GL/glext.h:4067
glCurrentPaletteMatrixARB = _link_function('glCurrentPaletteMatrixARB', None, [GLint], 'ARB_matrix_palette')

# GL/glext.h:4068
glMatrixIndexubvARB = _link_function('glMatrixIndexubvARB', None, [GLint, POINTER(GLubyte)], 'ARB_matrix_palette')

# GL/glext.h:4069
glMatrixIndexusvARB = _link_function('glMatrixIndexusvARB', None, [GLint, POINTER(GLushort)], 'ARB_matrix_palette')

# GL/glext.h:4070
glMatrixIndexuivARB = _link_function('glMatrixIndexuivARB', None, [GLint, POINTER(GLuint)], 'ARB_matrix_palette')

# GL/glext.h:4071
glMatrixIndexPointerARB = _link_function('glMatrixIndexPointerARB', None, [GLint, GLenum, GLsizei, POINTER(GLvoid)], 'ARB_matrix_palette')

PFNGLCURRENTPALETTEMATRIXARBPROC = CFUNCTYPE(None, GLint) 	# GL/glext.h:4073
PFNGLMATRIXINDEXUBVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLubyte)) 	# GL/glext.h:4074
PFNGLMATRIXINDEXUSVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLushort)) 	# GL/glext.h:4075
PFNGLMATRIXINDEXUIVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLuint)) 	# GL/glext.h:4076
PFNGLMATRIXINDEXPOINTERARBPROC = CFUNCTYPE(None, GLint, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:4077
# ARB_texture_env_combine (GL/glext.h:4080)
GL_ARB_texture_env_combine = 1 	# GL/glext.h:4081
# ARB_texture_env_crossbar (GL/glext.h:4084)
GL_ARB_texture_env_crossbar = 1 	# GL/glext.h:4085
# ARB_texture_env_dot3 (GL/glext.h:4088)
GL_ARB_texture_env_dot3 = 1 	# GL/glext.h:4089
# ARB_texture_mirrored_repeat (GL/glext.h:4092)
GL_ARB_texture_mirrored_repeat = 1 	# GL/glext.h:4093
# ARB_depth_texture (GL/glext.h:4096)
GL_ARB_depth_texture = 1 	# GL/glext.h:4097
# ARB_shadow (GL/glext.h:4100)
GL_ARB_shadow = 1 	# GL/glext.h:4101
# ARB_shadow_ambient (GL/glext.h:4104)
GL_ARB_shadow_ambient = 1 	# GL/glext.h:4105
# ARB_window_pos (GL/glext.h:4108)
GL_ARB_window_pos = 1 	# GL/glext.h:4109
# GL/glext.h:4111
glWindowPos2dARB = _link_function('glWindowPos2dARB', None, [GLdouble, GLdouble], 'ARB_window_pos')

# GL/glext.h:4112
glWindowPos2dvARB = _link_function('glWindowPos2dvARB', None, [POINTER(GLdouble)], 'ARB_window_pos')

# GL/glext.h:4113
glWindowPos2fARB = _link_function('glWindowPos2fARB', None, [GLfloat, GLfloat], 'ARB_window_pos')

# GL/glext.h:4114
glWindowPos2fvARB = _link_function('glWindowPos2fvARB', None, [POINTER(GLfloat)], 'ARB_window_pos')

# GL/glext.h:4115
glWindowPos2iARB = _link_function('glWindowPos2iARB', None, [GLint, GLint], 'ARB_window_pos')

# GL/glext.h:4116
glWindowPos2ivARB = _link_function('glWindowPos2ivARB', None, [POINTER(GLint)], 'ARB_window_pos')

# GL/glext.h:4117
glWindowPos2sARB = _link_function('glWindowPos2sARB', None, [GLshort, GLshort], 'ARB_window_pos')

# GL/glext.h:4118
glWindowPos2svARB = _link_function('glWindowPos2svARB', None, [POINTER(GLshort)], 'ARB_window_pos')

# GL/glext.h:4119
glWindowPos3dARB = _link_function('glWindowPos3dARB', None, [GLdouble, GLdouble, GLdouble], 'ARB_window_pos')

# GL/glext.h:4120
glWindowPos3dvARB = _link_function('glWindowPos3dvARB', None, [POINTER(GLdouble)], 'ARB_window_pos')

# GL/glext.h:4121
glWindowPos3fARB = _link_function('glWindowPos3fARB', None, [GLfloat, GLfloat, GLfloat], 'ARB_window_pos')

# GL/glext.h:4122
glWindowPos3fvARB = _link_function('glWindowPos3fvARB', None, [POINTER(GLfloat)], 'ARB_window_pos')

# GL/glext.h:4123
glWindowPos3iARB = _link_function('glWindowPos3iARB', None, [GLint, GLint, GLint], 'ARB_window_pos')

# GL/glext.h:4124
glWindowPos3ivARB = _link_function('glWindowPos3ivARB', None, [POINTER(GLint)], 'ARB_window_pos')

# GL/glext.h:4125
glWindowPos3sARB = _link_function('glWindowPos3sARB', None, [GLshort, GLshort, GLshort], 'ARB_window_pos')

# GL/glext.h:4126
glWindowPos3svARB = _link_function('glWindowPos3svARB', None, [POINTER(GLshort)], 'ARB_window_pos')

PFNGLWINDOWPOS2DARBPROC = CFUNCTYPE(None, GLdouble, GLdouble) 	# GL/glext.h:4128
PFNGLWINDOWPOS2DVARBPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:4129
PFNGLWINDOWPOS2FARBPROC = CFUNCTYPE(None, GLfloat, GLfloat) 	# GL/glext.h:4130
PFNGLWINDOWPOS2FVARBPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:4131
PFNGLWINDOWPOS2IARBPROC = CFUNCTYPE(None, GLint, GLint) 	# GL/glext.h:4132
PFNGLWINDOWPOS2IVARBPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:4133
PFNGLWINDOWPOS2SARBPROC = CFUNCTYPE(None, GLshort, GLshort) 	# GL/glext.h:4134
PFNGLWINDOWPOS2SVARBPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:4135
PFNGLWINDOWPOS3DARBPROC = CFUNCTYPE(None, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:4136
PFNGLWINDOWPOS3DVARBPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:4137
PFNGLWINDOWPOS3FARBPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:4138
PFNGLWINDOWPOS3FVARBPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:4139
PFNGLWINDOWPOS3IARBPROC = CFUNCTYPE(None, GLint, GLint, GLint) 	# GL/glext.h:4140
PFNGLWINDOWPOS3IVARBPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:4141
PFNGLWINDOWPOS3SARBPROC = CFUNCTYPE(None, GLshort, GLshort, GLshort) 	# GL/glext.h:4142
PFNGLWINDOWPOS3SVARBPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:4143
# ARB_vertex_program (GL/glext.h:4146)
GL_ARB_vertex_program = 1 	# GL/glext.h:4147
# GL/glext.h:4149
glVertexAttrib1dARB = _link_function('glVertexAttrib1dARB', None, [GLuint, GLdouble], 'ARB_vertex_program')

# GL/glext.h:4150
glVertexAttrib1dvARB = _link_function('glVertexAttrib1dvARB', None, [GLuint, POINTER(GLdouble)], 'ARB_vertex_program')

# GL/glext.h:4151
glVertexAttrib1fARB = _link_function('glVertexAttrib1fARB', None, [GLuint, GLfloat], 'ARB_vertex_program')

# GL/glext.h:4152
glVertexAttrib1fvARB = _link_function('glVertexAttrib1fvARB', None, [GLuint, POINTER(GLfloat)], 'ARB_vertex_program')

# GL/glext.h:4153
glVertexAttrib1sARB = _link_function('glVertexAttrib1sARB', None, [GLuint, GLshort], 'ARB_vertex_program')

# GL/glext.h:4154
glVertexAttrib1svARB = _link_function('glVertexAttrib1svARB', None, [GLuint, POINTER(GLshort)], 'ARB_vertex_program')

# GL/glext.h:4155
glVertexAttrib2dARB = _link_function('glVertexAttrib2dARB', None, [GLuint, GLdouble, GLdouble], 'ARB_vertex_program')

# GL/glext.h:4156
glVertexAttrib2dvARB = _link_function('glVertexAttrib2dvARB', None, [GLuint, POINTER(GLdouble)], 'ARB_vertex_program')

# GL/glext.h:4157
glVertexAttrib2fARB = _link_function('glVertexAttrib2fARB', None, [GLuint, GLfloat, GLfloat], 'ARB_vertex_program')

# GL/glext.h:4158
glVertexAttrib2fvARB = _link_function('glVertexAttrib2fvARB', None, [GLuint, POINTER(GLfloat)], 'ARB_vertex_program')

# GL/glext.h:4159
glVertexAttrib2sARB = _link_function('glVertexAttrib2sARB', None, [GLuint, GLshort, GLshort], 'ARB_vertex_program')

# GL/glext.h:4160
glVertexAttrib2svARB = _link_function('glVertexAttrib2svARB', None, [GLuint, POINTER(GLshort)], 'ARB_vertex_program')

# GL/glext.h:4161
glVertexAttrib3dARB = _link_function('glVertexAttrib3dARB', None, [GLuint, GLdouble, GLdouble, GLdouble], 'ARB_vertex_program')

# GL/glext.h:4162
glVertexAttrib3dvARB = _link_function('glVertexAttrib3dvARB', None, [GLuint, POINTER(GLdouble)], 'ARB_vertex_program')

# GL/glext.h:4163
glVertexAttrib3fARB = _link_function('glVertexAttrib3fARB', None, [GLuint, GLfloat, GLfloat, GLfloat], 'ARB_vertex_program')

# GL/glext.h:4164
glVertexAttrib3fvARB = _link_function('glVertexAttrib3fvARB', None, [GLuint, POINTER(GLfloat)], 'ARB_vertex_program')

# GL/glext.h:4165
glVertexAttrib3sARB = _link_function('glVertexAttrib3sARB', None, [GLuint, GLshort, GLshort, GLshort], 'ARB_vertex_program')

# GL/glext.h:4166
glVertexAttrib3svARB = _link_function('glVertexAttrib3svARB', None, [GLuint, POINTER(GLshort)], 'ARB_vertex_program')

# GL/glext.h:4167
glVertexAttrib4NbvARB = _link_function('glVertexAttrib4NbvARB', None, [GLuint, POINTER(GLbyte)], 'ARB_vertex_program')

# GL/glext.h:4168
glVertexAttrib4NivARB = _link_function('glVertexAttrib4NivARB', None, [GLuint, POINTER(GLint)], 'ARB_vertex_program')

# GL/glext.h:4169
glVertexAttrib4NsvARB = _link_function('glVertexAttrib4NsvARB', None, [GLuint, POINTER(GLshort)], 'ARB_vertex_program')

# GL/glext.h:4170
glVertexAttrib4NubARB = _link_function('glVertexAttrib4NubARB', None, [GLuint, GLubyte, GLubyte, GLubyte, GLubyte], 'ARB_vertex_program')

# GL/glext.h:4171
glVertexAttrib4NubvARB = _link_function('glVertexAttrib4NubvARB', None, [GLuint, POINTER(GLubyte)], 'ARB_vertex_program')

# GL/glext.h:4172
glVertexAttrib4NuivARB = _link_function('glVertexAttrib4NuivARB', None, [GLuint, POINTER(GLuint)], 'ARB_vertex_program')

# GL/glext.h:4173
glVertexAttrib4NusvARB = _link_function('glVertexAttrib4NusvARB', None, [GLuint, POINTER(GLushort)], 'ARB_vertex_program')

# GL/glext.h:4174
glVertexAttrib4bvARB = _link_function('glVertexAttrib4bvARB', None, [GLuint, POINTER(GLbyte)], 'ARB_vertex_program')

# GL/glext.h:4175
glVertexAttrib4dARB = _link_function('glVertexAttrib4dARB', None, [GLuint, GLdouble, GLdouble, GLdouble, GLdouble], 'ARB_vertex_program')

# GL/glext.h:4176
glVertexAttrib4dvARB = _link_function('glVertexAttrib4dvARB', None, [GLuint, POINTER(GLdouble)], 'ARB_vertex_program')

# GL/glext.h:4177
glVertexAttrib4fARB = _link_function('glVertexAttrib4fARB', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat], 'ARB_vertex_program')

# GL/glext.h:4178
glVertexAttrib4fvARB = _link_function('glVertexAttrib4fvARB', None, [GLuint, POINTER(GLfloat)], 'ARB_vertex_program')

# GL/glext.h:4179
glVertexAttrib4ivARB = _link_function('glVertexAttrib4ivARB', None, [GLuint, POINTER(GLint)], 'ARB_vertex_program')

# GL/glext.h:4180
glVertexAttrib4sARB = _link_function('glVertexAttrib4sARB', None, [GLuint, GLshort, GLshort, GLshort, GLshort], 'ARB_vertex_program')

# GL/glext.h:4181
glVertexAttrib4svARB = _link_function('glVertexAttrib4svARB', None, [GLuint, POINTER(GLshort)], 'ARB_vertex_program')

# GL/glext.h:4182
glVertexAttrib4ubvARB = _link_function('glVertexAttrib4ubvARB', None, [GLuint, POINTER(GLubyte)], 'ARB_vertex_program')

# GL/glext.h:4183
glVertexAttrib4uivARB = _link_function('glVertexAttrib4uivARB', None, [GLuint, POINTER(GLuint)], 'ARB_vertex_program')

# GL/glext.h:4184
glVertexAttrib4usvARB = _link_function('glVertexAttrib4usvARB', None, [GLuint, POINTER(GLushort)], 'ARB_vertex_program')

# GL/glext.h:4185
glVertexAttribPointerARB = _link_function('glVertexAttribPointerARB', None, [GLuint, GLint, GLenum, GLboolean, GLsizei, POINTER(GLvoid)], 'ARB_vertex_program')

# GL/glext.h:4186
glEnableVertexAttribArrayARB = _link_function('glEnableVertexAttribArrayARB', None, [GLuint], 'ARB_vertex_program')

# GL/glext.h:4187
glDisableVertexAttribArrayARB = _link_function('glDisableVertexAttribArrayARB', None, [GLuint], 'ARB_vertex_program')

# GL/glext.h:4188
glProgramStringARB = _link_function('glProgramStringARB', None, [GLenum, GLenum, GLsizei, POINTER(GLvoid)], 'ARB_vertex_program')

# GL/glext.h:4189
glBindProgramARB = _link_function('glBindProgramARB', None, [GLenum, GLuint], 'ARB_vertex_program')

# GL/glext.h:4190
glDeleteProgramsARB = _link_function('glDeleteProgramsARB', None, [GLsizei, POINTER(GLuint)], 'ARB_vertex_program')

# GL/glext.h:4191
glGenProgramsARB = _link_function('glGenProgramsARB', None, [GLsizei, POINTER(GLuint)], 'ARB_vertex_program')

# GL/glext.h:4192
glProgramEnvParameter4dARB = _link_function('glProgramEnvParameter4dARB', None, [GLenum, GLuint, GLdouble, GLdouble, GLdouble, GLdouble], 'ARB_vertex_program')

# GL/glext.h:4193
glProgramEnvParameter4dvARB = _link_function('glProgramEnvParameter4dvARB', None, [GLenum, GLuint, POINTER(GLdouble)], 'ARB_vertex_program')

# GL/glext.h:4194
glProgramEnvParameter4fARB = _link_function('glProgramEnvParameter4fARB', None, [GLenum, GLuint, GLfloat, GLfloat, GLfloat, GLfloat], 'ARB_vertex_program')

# GL/glext.h:4195
glProgramEnvParameter4fvARB = _link_function('glProgramEnvParameter4fvARB', None, [GLenum, GLuint, POINTER(GLfloat)], 'ARB_vertex_program')

# GL/glext.h:4196
glProgramLocalParameter4dARB = _link_function('glProgramLocalParameter4dARB', None, [GLenum, GLuint, GLdouble, GLdouble, GLdouble, GLdouble], 'ARB_vertex_program')

# GL/glext.h:4197
glProgramLocalParameter4dvARB = _link_function('glProgramLocalParameter4dvARB', None, [GLenum, GLuint, POINTER(GLdouble)], 'ARB_vertex_program')

# GL/glext.h:4198
glProgramLocalParameter4fARB = _link_function('glProgramLocalParameter4fARB', None, [GLenum, GLuint, GLfloat, GLfloat, GLfloat, GLfloat], 'ARB_vertex_program')

# GL/glext.h:4199
glProgramLocalParameter4fvARB = _link_function('glProgramLocalParameter4fvARB', None, [GLenum, GLuint, POINTER(GLfloat)], 'ARB_vertex_program')

# GL/glext.h:4200
glGetProgramEnvParameterdvARB = _link_function('glGetProgramEnvParameterdvARB', None, [GLenum, GLuint, POINTER(GLdouble)], 'ARB_vertex_program')

# GL/glext.h:4201
glGetProgramEnvParameterfvARB = _link_function('glGetProgramEnvParameterfvARB', None, [GLenum, GLuint, POINTER(GLfloat)], 'ARB_vertex_program')

# GL/glext.h:4202
glGetProgramLocalParameterdvARB = _link_function('glGetProgramLocalParameterdvARB', None, [GLenum, GLuint, POINTER(GLdouble)], 'ARB_vertex_program')

# GL/glext.h:4203
glGetProgramLocalParameterfvARB = _link_function('glGetProgramLocalParameterfvARB', None, [GLenum, GLuint, POINTER(GLfloat)], 'ARB_vertex_program')

# GL/glext.h:4204
glGetProgramivARB = _link_function('glGetProgramivARB', None, [GLenum, GLenum, POINTER(GLint)], 'ARB_vertex_program')

# GL/glext.h:4205
glGetProgramStringARB = _link_function('glGetProgramStringARB', None, [GLenum, GLenum, POINTER(GLvoid)], 'ARB_vertex_program')

# GL/glext.h:4206
glGetVertexAttribdvARB = _link_function('glGetVertexAttribdvARB', None, [GLuint, GLenum, POINTER(GLdouble)], 'ARB_vertex_program')

# GL/glext.h:4207
glGetVertexAttribfvARB = _link_function('glGetVertexAttribfvARB', None, [GLuint, GLenum, POINTER(GLfloat)], 'ARB_vertex_program')

# GL/glext.h:4208
glGetVertexAttribivARB = _link_function('glGetVertexAttribivARB', None, [GLuint, GLenum, POINTER(GLint)], 'ARB_vertex_program')

# GL/glext.h:4209
glGetVertexAttribPointervARB = _link_function('glGetVertexAttribPointervARB', None, [GLuint, GLenum, POINTER(POINTER(GLvoid))], 'ARB_vertex_program')

# GL/glext.h:4210
glIsProgramARB = _link_function('glIsProgramARB', GLboolean, [GLuint], 'ARB_vertex_program')

PFNGLVERTEXATTRIB1DARBPROC = CFUNCTYPE(None, GLuint, GLdouble) 	# GL/glext.h:4212
PFNGLVERTEXATTRIB1DVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:4213
PFNGLVERTEXATTRIB1FARBPROC = CFUNCTYPE(None, GLuint, GLfloat) 	# GL/glext.h:4214
PFNGLVERTEXATTRIB1FVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:4215
PFNGLVERTEXATTRIB1SARBPROC = CFUNCTYPE(None, GLuint, GLshort) 	# GL/glext.h:4216
PFNGLVERTEXATTRIB1SVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:4217
PFNGLVERTEXATTRIB2DARBPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble) 	# GL/glext.h:4218
PFNGLVERTEXATTRIB2DVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:4219
PFNGLVERTEXATTRIB2FARBPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat) 	# GL/glext.h:4220
PFNGLVERTEXATTRIB2FVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:4221
PFNGLVERTEXATTRIB2SARBPROC = CFUNCTYPE(None, GLuint, GLshort, GLshort) 	# GL/glext.h:4222
PFNGLVERTEXATTRIB2SVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:4223
PFNGLVERTEXATTRIB3DARBPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:4224
PFNGLVERTEXATTRIB3DVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:4225
PFNGLVERTEXATTRIB3FARBPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:4226
PFNGLVERTEXATTRIB3FVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:4227
PFNGLVERTEXATTRIB3SARBPROC = CFUNCTYPE(None, GLuint, GLshort, GLshort, GLshort) 	# GL/glext.h:4228
PFNGLVERTEXATTRIB3SVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:4229
PFNGLVERTEXATTRIB4NBVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLbyte)) 	# GL/glext.h:4230
PFNGLVERTEXATTRIB4NIVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:4231
PFNGLVERTEXATTRIB4NSVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:4232
PFNGLVERTEXATTRIB4NUBARBPROC = CFUNCTYPE(None, GLuint, GLubyte, GLubyte, GLubyte, GLubyte) 	# GL/glext.h:4233
PFNGLVERTEXATTRIB4NUBVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLubyte)) 	# GL/glext.h:4234
PFNGLVERTEXATTRIB4NUIVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:4235
PFNGLVERTEXATTRIB4NUSVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLushort)) 	# GL/glext.h:4236
PFNGLVERTEXATTRIB4BVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLbyte)) 	# GL/glext.h:4237
PFNGLVERTEXATTRIB4DARBPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:4238
PFNGLVERTEXATTRIB4DVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:4239
PFNGLVERTEXATTRIB4FARBPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:4240
PFNGLVERTEXATTRIB4FVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:4241
PFNGLVERTEXATTRIB4IVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:4242
PFNGLVERTEXATTRIB4SARBPROC = CFUNCTYPE(None, GLuint, GLshort, GLshort, GLshort, GLshort) 	# GL/glext.h:4243
PFNGLVERTEXATTRIB4SVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:4244
PFNGLVERTEXATTRIB4UBVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLubyte)) 	# GL/glext.h:4245
PFNGLVERTEXATTRIB4UIVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:4246
PFNGLVERTEXATTRIB4USVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLushort)) 	# GL/glext.h:4247
PFNGLVERTEXATTRIBPOINTERARBPROC = CFUNCTYPE(None, GLuint, GLint, GLenum, GLboolean, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:4248
PFNGLENABLEVERTEXATTRIBARRAYARBPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:4249
PFNGLDISABLEVERTEXATTRIBARRAYARBPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:4250
PFNGLPROGRAMSTRINGARBPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:4251
PFNGLBINDPROGRAMARBPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:4252
PFNGLDELETEPROGRAMSARBPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:4253
PFNGLGENPROGRAMSARBPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:4254
PFNGLPROGRAMENVPARAMETER4DARBPROC = CFUNCTYPE(None, GLenum, GLuint, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:4255
PFNGLPROGRAMENVPARAMETER4DVARBPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLdouble)) 	# GL/glext.h:4256
PFNGLPROGRAMENVPARAMETER4FARBPROC = CFUNCTYPE(None, GLenum, GLuint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:4257
PFNGLPROGRAMENVPARAMETER4FVARBPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLfloat)) 	# GL/glext.h:4258
PFNGLPROGRAMLOCALPARAMETER4DARBPROC = CFUNCTYPE(None, GLenum, GLuint, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:4259
PFNGLPROGRAMLOCALPARAMETER4DVARBPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLdouble)) 	# GL/glext.h:4260
PFNGLPROGRAMLOCALPARAMETER4FARBPROC = CFUNCTYPE(None, GLenum, GLuint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:4261
PFNGLPROGRAMLOCALPARAMETER4FVARBPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLfloat)) 	# GL/glext.h:4262
PFNGLGETPROGRAMENVPARAMETERDVARBPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLdouble)) 	# GL/glext.h:4263
PFNGLGETPROGRAMENVPARAMETERFVARBPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLfloat)) 	# GL/glext.h:4264
PFNGLGETPROGRAMLOCALPARAMETERDVARBPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLdouble)) 	# GL/glext.h:4265
PFNGLGETPROGRAMLOCALPARAMETERFVARBPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLfloat)) 	# GL/glext.h:4266
PFNGLGETPROGRAMIVARBPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:4267
PFNGLGETPROGRAMSTRINGARBPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:4268
PFNGLGETVERTEXATTRIBDVARBPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLdouble)) 	# GL/glext.h:4269
PFNGLGETVERTEXATTRIBFVARBPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4270
PFNGLGETVERTEXATTRIBIVARBPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:4271
PFNGLGETVERTEXATTRIBPOINTERVARBPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:4272
PFNGLISPROGRAMARBPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:4273
# ARB_fragment_program (GL/glext.h:4276)
GL_ARB_fragment_program = 1 	# GL/glext.h:4277
# ARB_vertex_buffer_object (GL/glext.h:4281)
GL_ARB_vertex_buffer_object = 1 	# GL/glext.h:4282
# GL/glext.h:4284
glBindBufferARB = _link_function('glBindBufferARB', None, [GLenum, GLuint], 'ARB_vertex_buffer_object')

# GL/glext.h:4285
glDeleteBuffersARB = _link_function('glDeleteBuffersARB', None, [GLsizei, POINTER(GLuint)], 'ARB_vertex_buffer_object')

# GL/glext.h:4286
glGenBuffersARB = _link_function('glGenBuffersARB', None, [GLsizei, POINTER(GLuint)], 'ARB_vertex_buffer_object')

# GL/glext.h:4287
glIsBufferARB = _link_function('glIsBufferARB', GLboolean, [GLuint], 'ARB_vertex_buffer_object')

# GL/glext.h:4288
glBufferDataARB = _link_function('glBufferDataARB', None, [GLenum, GLsizeiptrARB, POINTER(GLvoid), GLenum], 'ARB_vertex_buffer_object')

# GL/glext.h:4289
glBufferSubDataARB = _link_function('glBufferSubDataARB', None, [GLenum, GLintptrARB, GLsizeiptrARB, POINTER(GLvoid)], 'ARB_vertex_buffer_object')

# GL/glext.h:4290
glGetBufferSubDataARB = _link_function('glGetBufferSubDataARB', None, [GLenum, GLintptrARB, GLsizeiptrARB, POINTER(GLvoid)], 'ARB_vertex_buffer_object')

# GL/glext.h:4291
glMapBufferARB = _link_function('glMapBufferARB', POINTER(GLvoid), [GLenum, GLenum], 'ARB_vertex_buffer_object')

# GL/glext.h:4292
glUnmapBufferARB = _link_function('glUnmapBufferARB', GLboolean, [GLenum], 'ARB_vertex_buffer_object')

# GL/glext.h:4293
glGetBufferParameterivARB = _link_function('glGetBufferParameterivARB', None, [GLenum, GLenum, POINTER(GLint)], 'ARB_vertex_buffer_object')

# GL/glext.h:4294
glGetBufferPointervARB = _link_function('glGetBufferPointervARB', None, [GLenum, GLenum, POINTER(POINTER(GLvoid))], 'ARB_vertex_buffer_object')

PFNGLBINDBUFFERARBPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:4296
PFNGLDELETEBUFFERSARBPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:4297
PFNGLGENBUFFERSARBPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:4298
PFNGLISBUFFERARBPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:4299
PFNGLBUFFERDATAARBPROC = CFUNCTYPE(None, GLenum, GLsizeiptrARB, POINTER(GLvoid), GLenum) 	# GL/glext.h:4300
PFNGLBUFFERSUBDATAARBPROC = CFUNCTYPE(None, GLenum, GLintptrARB, GLsizeiptrARB, POINTER(GLvoid)) 	# GL/glext.h:4301
PFNGLGETBUFFERSUBDATAARBPROC = CFUNCTYPE(None, GLenum, GLintptrARB, GLsizeiptrARB, POINTER(GLvoid)) 	# GL/glext.h:4302
PFNGLMAPBUFFERARBPROC = CFUNCTYPE(POINTER(GLvoid), GLenum, GLenum) 	# GL/glext.h:4303
PFNGLUNMAPBUFFERARBPROC = CFUNCTYPE(GLboolean, GLenum) 	# GL/glext.h:4304
PFNGLGETBUFFERPARAMETERIVARBPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:4305
PFNGLGETBUFFERPOINTERVARBPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:4306
# ARB_occlusion_query (GL/glext.h:4309)
GL_ARB_occlusion_query = 1 	# GL/glext.h:4310
# GL/glext.h:4312
glGenQueriesARB = _link_function('glGenQueriesARB', None, [GLsizei, POINTER(GLuint)], 'ARB_occlusion_query')

# GL/glext.h:4313
glDeleteQueriesARB = _link_function('glDeleteQueriesARB', None, [GLsizei, POINTER(GLuint)], 'ARB_occlusion_query')

# GL/glext.h:4314
glIsQueryARB = _link_function('glIsQueryARB', GLboolean, [GLuint], 'ARB_occlusion_query')

# GL/glext.h:4315
glBeginQueryARB = _link_function('glBeginQueryARB', None, [GLenum, GLuint], 'ARB_occlusion_query')

# GL/glext.h:4316
glEndQueryARB = _link_function('glEndQueryARB', None, [GLenum], 'ARB_occlusion_query')

# GL/glext.h:4317
glGetQueryivARB = _link_function('glGetQueryivARB', None, [GLenum, GLenum, POINTER(GLint)], 'ARB_occlusion_query')

# GL/glext.h:4318
glGetQueryObjectivARB = _link_function('glGetQueryObjectivARB', None, [GLuint, GLenum, POINTER(GLint)], 'ARB_occlusion_query')

# GL/glext.h:4319
glGetQueryObjectuivARB = _link_function('glGetQueryObjectuivARB', None, [GLuint, GLenum, POINTER(GLuint)], 'ARB_occlusion_query')

PFNGLGENQUERIESARBPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:4321
PFNGLDELETEQUERIESARBPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:4322
PFNGLISQUERYARBPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:4323
PFNGLBEGINQUERYARBPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:4324
PFNGLENDQUERYARBPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:4325
PFNGLGETQUERYIVARBPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:4326
PFNGLGETQUERYOBJECTIVARBPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:4327
PFNGLGETQUERYOBJECTUIVARBPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLuint)) 	# GL/glext.h:4328
# ARB_shader_objects (GL/glext.h:4331)
GL_ARB_shader_objects = 1 	# GL/glext.h:4332
# GL/glext.h:4334
glDeleteObjectARB = _link_function('glDeleteObjectARB', None, [GLhandleARB], 'ARB_shader_objects')

# GL/glext.h:4335
glGetHandleARB = _link_function('glGetHandleARB', GLhandleARB, [GLenum], 'ARB_shader_objects')

# GL/glext.h:4336
glDetachObjectARB = _link_function('glDetachObjectARB', None, [GLhandleARB, GLhandleARB], 'ARB_shader_objects')

# GL/glext.h:4337
glCreateShaderObjectARB = _link_function('glCreateShaderObjectARB', GLhandleARB, [GLenum], 'ARB_shader_objects')

# GL/glext.h:4338
glShaderSourceARB = _link_function('glShaderSourceARB', None, [GLhandleARB, GLsizei, POINTER(POINTER(GLcharARB)), POINTER(GLint)], 'ARB_shader_objects')

# GL/glext.h:4339
glCompileShaderARB = _link_function('glCompileShaderARB', None, [GLhandleARB], 'ARB_shader_objects')

# GL/glext.h:4340
glCreateProgramObjectARB = _link_function('glCreateProgramObjectARB', GLhandleARB, [], 'ARB_shader_objects')

# GL/glext.h:4341
glAttachObjectARB = _link_function('glAttachObjectARB', None, [GLhandleARB, GLhandleARB], 'ARB_shader_objects')

# GL/glext.h:4342
glLinkProgramARB = _link_function('glLinkProgramARB', None, [GLhandleARB], 'ARB_shader_objects')

# GL/glext.h:4343
glUseProgramObjectARB = _link_function('glUseProgramObjectARB', None, [GLhandleARB], 'ARB_shader_objects')

# GL/glext.h:4344
glValidateProgramARB = _link_function('glValidateProgramARB', None, [GLhandleARB], 'ARB_shader_objects')

# GL/glext.h:4345
glUniform1fARB = _link_function('glUniform1fARB', None, [GLint, GLfloat], 'ARB_shader_objects')

# GL/glext.h:4346
glUniform2fARB = _link_function('glUniform2fARB', None, [GLint, GLfloat, GLfloat], 'ARB_shader_objects')

# GL/glext.h:4347
glUniform3fARB = _link_function('glUniform3fARB', None, [GLint, GLfloat, GLfloat, GLfloat], 'ARB_shader_objects')

# GL/glext.h:4348
glUniform4fARB = _link_function('glUniform4fARB', None, [GLint, GLfloat, GLfloat, GLfloat, GLfloat], 'ARB_shader_objects')

# GL/glext.h:4349
glUniform1iARB = _link_function('glUniform1iARB', None, [GLint, GLint], 'ARB_shader_objects')

# GL/glext.h:4350
glUniform2iARB = _link_function('glUniform2iARB', None, [GLint, GLint, GLint], 'ARB_shader_objects')

# GL/glext.h:4351
glUniform3iARB = _link_function('glUniform3iARB', None, [GLint, GLint, GLint, GLint], 'ARB_shader_objects')

# GL/glext.h:4352
glUniform4iARB = _link_function('glUniform4iARB', None, [GLint, GLint, GLint, GLint, GLint], 'ARB_shader_objects')

# GL/glext.h:4353
glUniform1fvARB = _link_function('glUniform1fvARB', None, [GLint, GLsizei, POINTER(GLfloat)], 'ARB_shader_objects')

# GL/glext.h:4354
glUniform2fvARB = _link_function('glUniform2fvARB', None, [GLint, GLsizei, POINTER(GLfloat)], 'ARB_shader_objects')

# GL/glext.h:4355
glUniform3fvARB = _link_function('glUniform3fvARB', None, [GLint, GLsizei, POINTER(GLfloat)], 'ARB_shader_objects')

# GL/glext.h:4356
glUniform4fvARB = _link_function('glUniform4fvARB', None, [GLint, GLsizei, POINTER(GLfloat)], 'ARB_shader_objects')

# GL/glext.h:4357
glUniform1ivARB = _link_function('glUniform1ivARB', None, [GLint, GLsizei, POINTER(GLint)], 'ARB_shader_objects')

# GL/glext.h:4358
glUniform2ivARB = _link_function('glUniform2ivARB', None, [GLint, GLsizei, POINTER(GLint)], 'ARB_shader_objects')

# GL/glext.h:4359
glUniform3ivARB = _link_function('glUniform3ivARB', None, [GLint, GLsizei, POINTER(GLint)], 'ARB_shader_objects')

# GL/glext.h:4360
glUniform4ivARB = _link_function('glUniform4ivARB', None, [GLint, GLsizei, POINTER(GLint)], 'ARB_shader_objects')

# GL/glext.h:4361
glUniformMatrix2fvARB = _link_function('glUniformMatrix2fvARB', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'ARB_shader_objects')

# GL/glext.h:4362
glUniformMatrix3fvARB = _link_function('glUniformMatrix3fvARB', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'ARB_shader_objects')

# GL/glext.h:4363
glUniformMatrix4fvARB = _link_function('glUniformMatrix4fvARB', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'ARB_shader_objects')

# GL/glext.h:4364
glGetObjectParameterfvARB = _link_function('glGetObjectParameterfvARB', None, [GLhandleARB, GLenum, POINTER(GLfloat)], 'ARB_shader_objects')

# GL/glext.h:4365
glGetObjectParameterivARB = _link_function('glGetObjectParameterivARB', None, [GLhandleARB, GLenum, POINTER(GLint)], 'ARB_shader_objects')

# GL/glext.h:4366
glGetInfoLogARB = _link_function('glGetInfoLogARB', None, [GLhandleARB, GLsizei, POINTER(GLsizei), POINTER(GLcharARB)], 'ARB_shader_objects')

# GL/glext.h:4367
glGetAttachedObjectsARB = _link_function('glGetAttachedObjectsARB', None, [GLhandleARB, GLsizei, POINTER(GLsizei), POINTER(GLhandleARB)], 'ARB_shader_objects')

# GL/glext.h:4368
glGetUniformLocationARB = _link_function('glGetUniformLocationARB', GLint, [GLhandleARB, POINTER(GLcharARB)], 'ARB_shader_objects')

# GL/glext.h:4369
glGetActiveUniformARB = _link_function('glGetActiveUniformARB', None, [GLhandleARB, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLint), POINTER(GLenum), POINTER(GLcharARB)], 'ARB_shader_objects')

# GL/glext.h:4370
glGetUniformfvARB = _link_function('glGetUniformfvARB', None, [GLhandleARB, GLint, POINTER(GLfloat)], 'ARB_shader_objects')

# GL/glext.h:4371
glGetUniformivARB = _link_function('glGetUniformivARB', None, [GLhandleARB, GLint, POINTER(GLint)], 'ARB_shader_objects')

# GL/glext.h:4372
glGetShaderSourceARB = _link_function('glGetShaderSourceARB', None, [GLhandleARB, GLsizei, POINTER(GLsizei), POINTER(GLcharARB)], 'ARB_shader_objects')

PFNGLDELETEOBJECTARBPROC = CFUNCTYPE(None, GLhandleARB) 	# GL/glext.h:4374
PFNGLGETHANDLEARBPROC = CFUNCTYPE(GLhandleARB, GLenum) 	# GL/glext.h:4375
PFNGLDETACHOBJECTARBPROC = CFUNCTYPE(None, GLhandleARB, GLhandleARB) 	# GL/glext.h:4376
PFNGLCREATESHADEROBJECTARBPROC = CFUNCTYPE(GLhandleARB, GLenum) 	# GL/glext.h:4377
PFNGLSHADERSOURCEARBPROC = CFUNCTYPE(None, GLhandleARB, GLsizei, POINTER(POINTER(GLcharARB)), POINTER(GLint)) 	# GL/glext.h:4378
PFNGLCOMPILESHADERARBPROC = CFUNCTYPE(None, GLhandleARB) 	# GL/glext.h:4379
PFNGLCREATEPROGRAMOBJECTARBPROC = CFUNCTYPE(GLhandleARB) 	# GL/glext.h:4380
PFNGLATTACHOBJECTARBPROC = CFUNCTYPE(None, GLhandleARB, GLhandleARB) 	# GL/glext.h:4381
PFNGLLINKPROGRAMARBPROC = CFUNCTYPE(None, GLhandleARB) 	# GL/glext.h:4382
PFNGLUSEPROGRAMOBJECTARBPROC = CFUNCTYPE(None, GLhandleARB) 	# GL/glext.h:4383
PFNGLVALIDATEPROGRAMARBPROC = CFUNCTYPE(None, GLhandleARB) 	# GL/glext.h:4384
PFNGLUNIFORM1FARBPROC = CFUNCTYPE(None, GLint, GLfloat) 	# GL/glext.h:4385
PFNGLUNIFORM2FARBPROC = CFUNCTYPE(None, GLint, GLfloat, GLfloat) 	# GL/glext.h:4386
PFNGLUNIFORM3FARBPROC = CFUNCTYPE(None, GLint, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:4387
PFNGLUNIFORM4FARBPROC = CFUNCTYPE(None, GLint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:4388
PFNGLUNIFORM1IARBPROC = CFUNCTYPE(None, GLint, GLint) 	# GL/glext.h:4389
PFNGLUNIFORM2IARBPROC = CFUNCTYPE(None, GLint, GLint, GLint) 	# GL/glext.h:4390
PFNGLUNIFORM3IARBPROC = CFUNCTYPE(None, GLint, GLint, GLint, GLint) 	# GL/glext.h:4391
PFNGLUNIFORM4IARBPROC = CFUNCTYPE(None, GLint, GLint, GLint, GLint, GLint) 	# GL/glext.h:4392
PFNGLUNIFORM1FVARBPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:4393
PFNGLUNIFORM2FVARBPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:4394
PFNGLUNIFORM3FVARBPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:4395
PFNGLUNIFORM4FVARBPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:4396
PFNGLUNIFORM1IVARBPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:4397
PFNGLUNIFORM2IVARBPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:4398
PFNGLUNIFORM3IVARBPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:4399
PFNGLUNIFORM4IVARBPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:4400
PFNGLUNIFORMMATRIX2FVARBPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:4401
PFNGLUNIFORMMATRIX3FVARBPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:4402
PFNGLUNIFORMMATRIX4FVARBPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:4403
PFNGLGETOBJECTPARAMETERFVARBPROC = CFUNCTYPE(None, GLhandleARB, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4404
PFNGLGETOBJECTPARAMETERIVARBPROC = CFUNCTYPE(None, GLhandleARB, GLenum, POINTER(GLint)) 	# GL/glext.h:4405
PFNGLGETINFOLOGARBPROC = CFUNCTYPE(None, GLhandleARB, GLsizei, POINTER(GLsizei), POINTER(GLcharARB)) 	# GL/glext.h:4406
PFNGLGETATTACHEDOBJECTSARBPROC = CFUNCTYPE(None, GLhandleARB, GLsizei, POINTER(GLsizei), POINTER(GLhandleARB)) 	# GL/glext.h:4407
PFNGLGETUNIFORMLOCATIONARBPROC = CFUNCTYPE(GLint, GLhandleARB, POINTER(GLcharARB)) 	# GL/glext.h:4408
PFNGLGETACTIVEUNIFORMARBPROC = CFUNCTYPE(None, GLhandleARB, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLint), POINTER(GLenum), POINTER(GLcharARB)) 	# GL/glext.h:4409
PFNGLGETUNIFORMFVARBPROC = CFUNCTYPE(None, GLhandleARB, GLint, POINTER(GLfloat)) 	# GL/glext.h:4410
PFNGLGETUNIFORMIVARBPROC = CFUNCTYPE(None, GLhandleARB, GLint, POINTER(GLint)) 	# GL/glext.h:4411
PFNGLGETSHADERSOURCEARBPROC = CFUNCTYPE(None, GLhandleARB, GLsizei, POINTER(GLsizei), POINTER(GLcharARB)) 	# GL/glext.h:4412
# ARB_vertex_shader (GL/glext.h:4415)
GL_ARB_vertex_shader = 1 	# GL/glext.h:4416
# GL/glext.h:4418
glBindAttribLocationARB = _link_function('glBindAttribLocationARB', None, [GLhandleARB, GLuint, POINTER(GLcharARB)], 'ARB_vertex_shader')

# GL/glext.h:4419
glGetActiveAttribARB = _link_function('glGetActiveAttribARB', None, [GLhandleARB, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLint), POINTER(GLenum), POINTER(GLcharARB)], 'ARB_vertex_shader')

# GL/glext.h:4420
glGetAttribLocationARB = _link_function('glGetAttribLocationARB', GLint, [GLhandleARB, POINTER(GLcharARB)], 'ARB_vertex_shader')

PFNGLBINDATTRIBLOCATIONARBPROC = CFUNCTYPE(None, GLhandleARB, GLuint, POINTER(GLcharARB)) 	# GL/glext.h:4422
PFNGLGETACTIVEATTRIBARBPROC = CFUNCTYPE(None, GLhandleARB, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLint), POINTER(GLenum), POINTER(GLcharARB)) 	# GL/glext.h:4423
PFNGLGETATTRIBLOCATIONARBPROC = CFUNCTYPE(GLint, GLhandleARB, POINTER(GLcharARB)) 	# GL/glext.h:4424
# ARB_fragment_shader (GL/glext.h:4427)
GL_ARB_fragment_shader = 1 	# GL/glext.h:4428
# ARB_shading_language_100 (GL/glext.h:4431)
GL_ARB_shading_language_100 = 1 	# GL/glext.h:4432
# ARB_texture_non_power_of_two (GL/glext.h:4435)
GL_ARB_texture_non_power_of_two = 1 	# GL/glext.h:4436
# ARB_point_sprite (GL/glext.h:4439)
GL_ARB_point_sprite = 1 	# GL/glext.h:4440
# ARB_fragment_program_shadow (GL/glext.h:4443)
GL_ARB_fragment_program_shadow = 1 	# GL/glext.h:4444
# ARB_draw_buffers (GL/glext.h:4447)
GL_ARB_draw_buffers = 1 	# GL/glext.h:4448
# GL/glext.h:4450
glDrawBuffersARB = _link_function('glDrawBuffersARB', None, [GLsizei, POINTER(GLenum)], 'ARB_draw_buffers')

PFNGLDRAWBUFFERSARBPROC = CFUNCTYPE(None, GLsizei, POINTER(GLenum)) 	# GL/glext.h:4452
# ARB_texture_rectangle (GL/glext.h:4455)
GL_ARB_texture_rectangle = 1 	# GL/glext.h:4456
# ARB_color_buffer_float (GL/glext.h:4459)
GL_ARB_color_buffer_float = 1 	# GL/glext.h:4460
# GL/glext.h:4462
glClampColorARB = _link_function('glClampColorARB', None, [GLenum, GLenum], 'ARB_color_buffer_float')

PFNGLCLAMPCOLORARBPROC = CFUNCTYPE(None, GLenum, GLenum) 	# GL/glext.h:4464
# ARB_half_float_pixel (GL/glext.h:4467)
GL_ARB_half_float_pixel = 1 	# GL/glext.h:4468
# ARB_texture_float (GL/glext.h:4471)
GL_ARB_texture_float = 1 	# GL/glext.h:4472
# ARB_pixel_buffer_object (GL/glext.h:4475)
GL_ARB_pixel_buffer_object = 1 	# GL/glext.h:4476
# EXT_abgr (GL/glext.h:4479)
GL_EXT_abgr = 1 	# GL/glext.h:4480
# EXT_blend_color (GL/glext.h:4483)
GL_EXT_blend_color = 1 	# GL/glext.h:4484
# GL/glext.h:4486
glBlendColorEXT = _link_function('glBlendColorEXT', None, [GLclampf, GLclampf, GLclampf, GLclampf], 'EXT_blend_color')

PFNGLBLENDCOLOREXTPROC = CFUNCTYPE(None, GLclampf, GLclampf, GLclampf, GLclampf) 	# GL/glext.h:4488
# EXT_polygon_offset (GL/glext.h:4491)
GL_EXT_polygon_offset = 1 	# GL/glext.h:4492
# GL/glext.h:4494
glPolygonOffsetEXT = _link_function('glPolygonOffsetEXT', None, [GLfloat, GLfloat], 'EXT_polygon_offset')

PFNGLPOLYGONOFFSETEXTPROC = CFUNCTYPE(None, GLfloat, GLfloat) 	# GL/glext.h:4496
# EXT_texture (GL/glext.h:4499)
GL_EXT_texture = 1 	# GL/glext.h:4500
# EXT_texture3D (GL/glext.h:4503)
GL_EXT_texture3D = 1 	# GL/glext.h:4504
# GL/glext.h:4506
glTexImage3DEXT = _link_function('glTexImage3DEXT', None, [GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)], 'EXT_texture3D')

# GL/glext.h:4507
glTexSubImage3DEXT = _link_function('glTexSubImage3DEXT', None, [GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_texture3D')

PFNGLTEXIMAGE3DEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:4509
PFNGLTEXSUBIMAGE3DEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:4510
# SGIS_texture_filter4 (GL/glext.h:4513)
GL_SGIS_texture_filter4 = 1 	# GL/glext.h:4514
# GL/glext.h:4516
glGetTexFilterFuncSGIS = _link_function('glGetTexFilterFuncSGIS', None, [GLenum, GLenum, POINTER(GLfloat)], 'SGIS_texture_filter4')

# GL/glext.h:4517
glTexFilterFuncSGIS = _link_function('glTexFilterFuncSGIS', None, [GLenum, GLenum, GLsizei, POINTER(GLfloat)], 'SGIS_texture_filter4')

PFNGLGETTEXFILTERFUNCSGISPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4519
PFNGLTEXFILTERFUNCSGISPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:4520
# EXT_subtexture (GL/glext.h:4523)
GL_EXT_subtexture = 1 	# GL/glext.h:4524
# GL/glext.h:4526
glTexSubImage1DEXT = _link_function('glTexSubImage1DEXT', None, [GLenum, GLint, GLint, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_subtexture')

# GL/glext.h:4527
glTexSubImage2DEXT = _link_function('glTexSubImage2DEXT', None, [GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_subtexture')

PFNGLTEXSUBIMAGE1DEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:4529
PFNGLTEXSUBIMAGE2DEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:4530
# EXT_copy_texture (GL/glext.h:4533)
GL_EXT_copy_texture = 1 	# GL/glext.h:4534
# GL/glext.h:4536
glCopyTexImage1DEXT = _link_function('glCopyTexImage1DEXT', None, [GLenum, GLint, GLenum, GLint, GLint, GLsizei, GLint], 'EXT_copy_texture')

# GL/glext.h:4537
glCopyTexImage2DEXT = _link_function('glCopyTexImage2DEXT', None, [GLenum, GLint, GLenum, GLint, GLint, GLsizei, GLsizei, GLint], 'EXT_copy_texture')

# GL/glext.h:4538
glCopyTexSubImage1DEXT = _link_function('glCopyTexSubImage1DEXT', None, [GLenum, GLint, GLint, GLint, GLint, GLsizei], 'EXT_copy_texture')

# GL/glext.h:4539
glCopyTexSubImage2DEXT = _link_function('glCopyTexSubImage2DEXT', None, [GLenum, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei], 'EXT_copy_texture')

# GL/glext.h:4540
glCopyTexSubImage3DEXT = _link_function('glCopyTexSubImage3DEXT', None, [GLenum, GLint, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei], 'EXT_copy_texture')

PFNGLCOPYTEXIMAGE1DEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLint, GLint, GLsizei, GLint) 	# GL/glext.h:4542
PFNGLCOPYTEXIMAGE2DEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLint, GLint, GLsizei, GLsizei, GLint) 	# GL/glext.h:4543
PFNGLCOPYTEXSUBIMAGE1DEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint, GLsizei) 	# GL/glext.h:4544
PFNGLCOPYTEXSUBIMAGE2DEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei) 	# GL/glext.h:4545
PFNGLCOPYTEXSUBIMAGE3DEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei) 	# GL/glext.h:4546
# EXT_histogram (GL/glext.h:4549)
GL_EXT_histogram = 1 	# GL/glext.h:4550
# GL/glext.h:4552
glGetHistogramEXT = _link_function('glGetHistogramEXT', None, [GLenum, GLboolean, GLenum, GLenum, POINTER(GLvoid)], 'EXT_histogram')

# GL/glext.h:4553
glGetHistogramParameterfvEXT = _link_function('glGetHistogramParameterfvEXT', None, [GLenum, GLenum, POINTER(GLfloat)], 'EXT_histogram')

# GL/glext.h:4554
glGetHistogramParameterivEXT = _link_function('glGetHistogramParameterivEXT', None, [GLenum, GLenum, POINTER(GLint)], 'EXT_histogram')

# GL/glext.h:4555
glGetMinmaxEXT = _link_function('glGetMinmaxEXT', None, [GLenum, GLboolean, GLenum, GLenum, POINTER(GLvoid)], 'EXT_histogram')

# GL/glext.h:4556
glGetMinmaxParameterfvEXT = _link_function('glGetMinmaxParameterfvEXT', None, [GLenum, GLenum, POINTER(GLfloat)], 'EXT_histogram')

# GL/glext.h:4557
glGetMinmaxParameterivEXT = _link_function('glGetMinmaxParameterivEXT', None, [GLenum, GLenum, POINTER(GLint)], 'EXT_histogram')

# GL/glext.h:4558
glHistogramEXT = _link_function('glHistogramEXT', None, [GLenum, GLsizei, GLenum, GLboolean], 'EXT_histogram')

# GL/glext.h:4559
glMinmaxEXT = _link_function('glMinmaxEXT', None, [GLenum, GLenum, GLboolean], 'EXT_histogram')

# GL/glext.h:4560
glResetHistogramEXT = _link_function('glResetHistogramEXT', None, [GLenum], 'EXT_histogram')

# GL/glext.h:4561
glResetMinmaxEXT = _link_function('glResetMinmaxEXT', None, [GLenum], 'EXT_histogram')

PFNGLGETHISTOGRAMEXTPROC = CFUNCTYPE(None, GLenum, GLboolean, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:4563
PFNGLGETHISTOGRAMPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4564
PFNGLGETHISTOGRAMPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:4565
PFNGLGETMINMAXEXTPROC = CFUNCTYPE(None, GLenum, GLboolean, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:4566
PFNGLGETMINMAXPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4567
PFNGLGETMINMAXPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:4568
PFNGLHISTOGRAMEXTPROC = CFUNCTYPE(None, GLenum, GLsizei, GLenum, GLboolean) 	# GL/glext.h:4569
PFNGLMINMAXEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLboolean) 	# GL/glext.h:4570
PFNGLRESETHISTOGRAMEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:4571
PFNGLRESETMINMAXEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:4572
# EXT_convolution (GL/glext.h:4575)
GL_EXT_convolution = 1 	# GL/glext.h:4576
# GL/glext.h:4578
glConvolutionFilter1DEXT = _link_function('glConvolutionFilter1DEXT', None, [GLenum, GLenum, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_convolution')

# GL/glext.h:4579
glConvolutionFilter2DEXT = _link_function('glConvolutionFilter2DEXT', None, [GLenum, GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_convolution')

# GL/glext.h:4580
glConvolutionParameterfEXT = _link_function('glConvolutionParameterfEXT', None, [GLenum, GLenum, GLfloat], 'EXT_convolution')

# GL/glext.h:4581
glConvolutionParameterfvEXT = _link_function('glConvolutionParameterfvEXT', None, [GLenum, GLenum, POINTER(GLfloat)], 'EXT_convolution')

# GL/glext.h:4582
glConvolutionParameteriEXT = _link_function('glConvolutionParameteriEXT', None, [GLenum, GLenum, GLint], 'EXT_convolution')

# GL/glext.h:4583
glConvolutionParameterivEXT = _link_function('glConvolutionParameterivEXT', None, [GLenum, GLenum, POINTER(GLint)], 'EXT_convolution')

# GL/glext.h:4584
glCopyConvolutionFilter1DEXT = _link_function('glCopyConvolutionFilter1DEXT', None, [GLenum, GLenum, GLint, GLint, GLsizei], 'EXT_convolution')

# GL/glext.h:4585
glCopyConvolutionFilter2DEXT = _link_function('glCopyConvolutionFilter2DEXT', None, [GLenum, GLenum, GLint, GLint, GLsizei, GLsizei], 'EXT_convolution')

# GL/glext.h:4586
glGetConvolutionFilterEXT = _link_function('glGetConvolutionFilterEXT', None, [GLenum, GLenum, GLenum, POINTER(GLvoid)], 'EXT_convolution')

# GL/glext.h:4587
glGetConvolutionParameterfvEXT = _link_function('glGetConvolutionParameterfvEXT', None, [GLenum, GLenum, POINTER(GLfloat)], 'EXT_convolution')

# GL/glext.h:4588
glGetConvolutionParameterivEXT = _link_function('glGetConvolutionParameterivEXT', None, [GLenum, GLenum, POINTER(GLint)], 'EXT_convolution')

# GL/glext.h:4589
glGetSeparableFilterEXT = _link_function('glGetSeparableFilterEXT', None, [GLenum, GLenum, GLenum, POINTER(GLvoid), POINTER(GLvoid), POINTER(GLvoid)], 'EXT_convolution')

# GL/glext.h:4590
glSeparableFilter2DEXT = _link_function('glSeparableFilter2DEXT', None, [GLenum, GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid), POINTER(GLvoid)], 'EXT_convolution')

PFNGLCONVOLUTIONFILTER1DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:4592
PFNGLCONVOLUTIONFILTER2DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:4593
PFNGLCONVOLUTIONPARAMETERFEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLfloat) 	# GL/glext.h:4594
PFNGLCONVOLUTIONPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4595
PFNGLCONVOLUTIONPARAMETERIEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint) 	# GL/glext.h:4596
PFNGLCONVOLUTIONPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:4597
PFNGLCOPYCONVOLUTIONFILTER1DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLsizei) 	# GL/glext.h:4598
PFNGLCOPYCONVOLUTIONFILTER2DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLsizei, GLsizei) 	# GL/glext.h:4599
PFNGLGETCONVOLUTIONFILTEREXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:4600
PFNGLGETCONVOLUTIONPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4601
PFNGLGETCONVOLUTIONPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:4602
PFNGLGETSEPARABLEFILTEREXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLvoid), POINTER(GLvoid), POINTER(GLvoid)) 	# GL/glext.h:4603
PFNGLSEPARABLEFILTER2DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid), POINTER(GLvoid)) 	# GL/glext.h:4604
# EXT_color_matrix (GL/glext.h:4607)
GL_EXT_color_matrix = 1 	# GL/glext.h:4608
# SGI_color_table (GL/glext.h:4611)
GL_SGI_color_table = 1 	# GL/glext.h:4612
# GL/glext.h:4614
glColorTableSGI = _link_function('glColorTableSGI', None, [GLenum, GLenum, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'SGI_color_table')

# GL/glext.h:4615
glColorTableParameterfvSGI = _link_function('glColorTableParameterfvSGI', None, [GLenum, GLenum, POINTER(GLfloat)], 'SGI_color_table')

# GL/glext.h:4616
glColorTableParameterivSGI = _link_function('glColorTableParameterivSGI', None, [GLenum, GLenum, POINTER(GLint)], 'SGI_color_table')

# GL/glext.h:4617
glCopyColorTableSGI = _link_function('glCopyColorTableSGI', None, [GLenum, GLenum, GLint, GLint, GLsizei], 'SGI_color_table')

# GL/glext.h:4618
glGetColorTableSGI = _link_function('glGetColorTableSGI', None, [GLenum, GLenum, GLenum, POINTER(GLvoid)], 'SGI_color_table')

# GL/glext.h:4619
glGetColorTableParameterfvSGI = _link_function('glGetColorTableParameterfvSGI', None, [GLenum, GLenum, POINTER(GLfloat)], 'SGI_color_table')

# GL/glext.h:4620
glGetColorTableParameterivSGI = _link_function('glGetColorTableParameterivSGI', None, [GLenum, GLenum, POINTER(GLint)], 'SGI_color_table')

PFNGLCOLORTABLESGIPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:4622
PFNGLCOLORTABLEPARAMETERFVSGIPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4623
PFNGLCOLORTABLEPARAMETERIVSGIPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:4624
PFNGLCOPYCOLORTABLESGIPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLsizei) 	# GL/glext.h:4625
PFNGLGETCOLORTABLESGIPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:4626
PFNGLGETCOLORTABLEPARAMETERFVSGIPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4627
PFNGLGETCOLORTABLEPARAMETERIVSGIPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:4628
# SGIX_pixel_texture (GL/glext.h:4631)
GL_SGIX_pixel_texture = 1 	# GL/glext.h:4632
# GL/glext.h:4634
glPixelTexGenSGIX = _link_function('glPixelTexGenSGIX', None, [GLenum], 'SGIX_pixel_texture')

PFNGLPIXELTEXGENSGIXPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:4636
# SGIS_pixel_texture (GL/glext.h:4639)
GL_SGIS_pixel_texture = 1 	# GL/glext.h:4640
# GL/glext.h:4642
glPixelTexGenParameteriSGIS = _link_function('glPixelTexGenParameteriSGIS', None, [GLenum, GLint], 'SGIS_pixel_texture')

# GL/glext.h:4643
glPixelTexGenParameterivSGIS = _link_function('glPixelTexGenParameterivSGIS', None, [GLenum, POINTER(GLint)], 'SGIS_pixel_texture')

# GL/glext.h:4644
glPixelTexGenParameterfSGIS = _link_function('glPixelTexGenParameterfSGIS', None, [GLenum, GLfloat], 'SGIS_pixel_texture')

# GL/glext.h:4645
glPixelTexGenParameterfvSGIS = _link_function('glPixelTexGenParameterfvSGIS', None, [GLenum, POINTER(GLfloat)], 'SGIS_pixel_texture')

# GL/glext.h:4646
glGetPixelTexGenParameterivSGIS = _link_function('glGetPixelTexGenParameterivSGIS', None, [GLenum, POINTER(GLint)], 'SGIS_pixel_texture')

# GL/glext.h:4647
glGetPixelTexGenParameterfvSGIS = _link_function('glGetPixelTexGenParameterfvSGIS', None, [GLenum, POINTER(GLfloat)], 'SGIS_pixel_texture')

PFNGLPIXELTEXGENPARAMETERISGISPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:4649
PFNGLPIXELTEXGENPARAMETERIVSGISPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:4650
PFNGLPIXELTEXGENPARAMETERFSGISPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:4651
PFNGLPIXELTEXGENPARAMETERFVSGISPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4652
PFNGLGETPIXELTEXGENPARAMETERIVSGISPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:4653
PFNGLGETPIXELTEXGENPARAMETERFVSGISPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4654
# SGIS_texture4D (GL/glext.h:4657)
GL_SGIS_texture4D = 1 	# GL/glext.h:4658
# GL/glext.h:4660
glTexImage4DSGIS = _link_function('glTexImage4DSGIS', None, [GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)], 'SGIS_texture4D')

# GL/glext.h:4661
glTexSubImage4DSGIS = _link_function('glTexSubImage4DSGIS', None, [GLenum, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'SGIS_texture4D')

PFNGLTEXIMAGE4DSGISPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:4663
PFNGLTEXSUBIMAGE4DSGISPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:4664
# SGI_texture_color_table (GL/glext.h:4667)
GL_SGI_texture_color_table = 1 	# GL/glext.h:4668
# EXT_cmyka (GL/glext.h:4671)
GL_EXT_cmyka = 1 	# GL/glext.h:4672
# EXT_texture_object (GL/glext.h:4675)
GL_EXT_texture_object = 1 	# GL/glext.h:4676
# GL/glext.h:4678
glAreTexturesResidentEXT = _link_function('glAreTexturesResidentEXT', GLboolean, [GLsizei, POINTER(GLuint), POINTER(GLboolean)], 'EXT_texture_object')

# GL/glext.h:4679
glBindTextureEXT = _link_function('glBindTextureEXT', None, [GLenum, GLuint], 'EXT_texture_object')

# GL/glext.h:4680
glDeleteTexturesEXT = _link_function('glDeleteTexturesEXT', None, [GLsizei, POINTER(GLuint)], 'EXT_texture_object')

# GL/glext.h:4681
glGenTexturesEXT = _link_function('glGenTexturesEXT', None, [GLsizei, POINTER(GLuint)], 'EXT_texture_object')

# GL/glext.h:4682
glIsTextureEXT = _link_function('glIsTextureEXT', GLboolean, [GLuint], 'EXT_texture_object')

# GL/glext.h:4683
glPrioritizeTexturesEXT = _link_function('glPrioritizeTexturesEXT', None, [GLsizei, POINTER(GLuint), POINTER(GLclampf)], 'EXT_texture_object')

PFNGLARETEXTURESRESIDENTEXTPROC = CFUNCTYPE(GLboolean, GLsizei, POINTER(GLuint), POINTER(GLboolean)) 	# GL/glext.h:4685
PFNGLBINDTEXTUREEXTPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:4686
PFNGLDELETETEXTURESEXTPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:4687
PFNGLGENTEXTURESEXTPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:4688
PFNGLISTEXTUREEXTPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:4689
PFNGLPRIORITIZETEXTURESEXTPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint), POINTER(GLclampf)) 	# GL/glext.h:4690
# SGIS_detail_texture (GL/glext.h:4693)
GL_SGIS_detail_texture = 1 	# GL/glext.h:4694
# GL/glext.h:4696
glDetailTexFuncSGIS = _link_function('glDetailTexFuncSGIS', None, [GLenum, GLsizei, POINTER(GLfloat)], 'SGIS_detail_texture')

# GL/glext.h:4697
glGetDetailTexFuncSGIS = _link_function('glGetDetailTexFuncSGIS', None, [GLenum, POINTER(GLfloat)], 'SGIS_detail_texture')

PFNGLDETAILTEXFUNCSGISPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:4699
PFNGLGETDETAILTEXFUNCSGISPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4700
# SGIS_sharpen_texture (GL/glext.h:4703)
GL_SGIS_sharpen_texture = 1 	# GL/glext.h:4704
# GL/glext.h:4706
glSharpenTexFuncSGIS = _link_function('glSharpenTexFuncSGIS', None, [GLenum, GLsizei, POINTER(GLfloat)], 'SGIS_sharpen_texture')

# GL/glext.h:4707
glGetSharpenTexFuncSGIS = _link_function('glGetSharpenTexFuncSGIS', None, [GLenum, POINTER(GLfloat)], 'SGIS_sharpen_texture')

PFNGLSHARPENTEXFUNCSGISPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:4709
PFNGLGETSHARPENTEXFUNCSGISPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4710
# EXT_packed_pixels (GL/glext.h:4713)
GL_EXT_packed_pixels = 1 	# GL/glext.h:4714
# SGIS_texture_lod (GL/glext.h:4717)
GL_SGIS_texture_lod = 1 	# GL/glext.h:4718
# SGIS_multisample (GL/glext.h:4721)
GL_SGIS_multisample = 1 	# GL/glext.h:4722
# GL/glext.h:4724
glSampleMaskSGIS = _link_function('glSampleMaskSGIS', None, [GLclampf, GLboolean], 'SGIS_multisample')

# GL/glext.h:4725
glSamplePatternSGIS = _link_function('glSamplePatternSGIS', None, [GLenum], 'SGIS_multisample')

PFNGLSAMPLEMASKSGISPROC = CFUNCTYPE(None, GLclampf, GLboolean) 	# GL/glext.h:4727
PFNGLSAMPLEPATTERNSGISPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:4728
# EXT_rescale_normal (GL/glext.h:4731)
GL_EXT_rescale_normal = 1 	# GL/glext.h:4732
# EXT_vertex_array (GL/glext.h:4735)
GL_EXT_vertex_array = 1 	# GL/glext.h:4736
# GL/glext.h:4738
glArrayElementEXT = _link_function('glArrayElementEXT', None, [GLint], 'EXT_vertex_array')

# GL/glext.h:4739
glColorPointerEXT = _link_function('glColorPointerEXT', None, [GLint, GLenum, GLsizei, GLsizei, POINTER(GLvoid)], 'EXT_vertex_array')

# GL/glext.h:4740
glDrawArraysEXT = _link_function('glDrawArraysEXT', None, [GLenum, GLint, GLsizei], 'EXT_vertex_array')

# GL/glext.h:4741
glEdgeFlagPointerEXT = _link_function('glEdgeFlagPointerEXT', None, [GLsizei, GLsizei, POINTER(GLboolean)], 'EXT_vertex_array')

# GL/glext.h:4742
glGetPointervEXT = _link_function('glGetPointervEXT', None, [GLenum, POINTER(POINTER(GLvoid))], 'EXT_vertex_array')

# GL/glext.h:4743
glIndexPointerEXT = _link_function('glIndexPointerEXT', None, [GLenum, GLsizei, GLsizei, POINTER(GLvoid)], 'EXT_vertex_array')

# GL/glext.h:4744
glNormalPointerEXT = _link_function('glNormalPointerEXT', None, [GLenum, GLsizei, GLsizei, POINTER(GLvoid)], 'EXT_vertex_array')

# GL/glext.h:4745
glTexCoordPointerEXT = _link_function('glTexCoordPointerEXT', None, [GLint, GLenum, GLsizei, GLsizei, POINTER(GLvoid)], 'EXT_vertex_array')

# GL/glext.h:4746
glVertexPointerEXT = _link_function('glVertexPointerEXT', None, [GLint, GLenum, GLsizei, GLsizei, POINTER(GLvoid)], 'EXT_vertex_array')

PFNGLARRAYELEMENTEXTPROC = CFUNCTYPE(None, GLint) 	# GL/glext.h:4748
PFNGLCOLORPOINTEREXTPROC = CFUNCTYPE(None, GLint, GLenum, GLsizei, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:4749
PFNGLDRAWARRAYSEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLsizei) 	# GL/glext.h:4750
PFNGLEDGEFLAGPOINTEREXTPROC = CFUNCTYPE(None, GLsizei, GLsizei, POINTER(GLboolean)) 	# GL/glext.h:4751
PFNGLGETPOINTERVEXTPROC = CFUNCTYPE(None, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:4752
PFNGLINDEXPOINTEREXTPROC = CFUNCTYPE(None, GLenum, GLsizei, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:4753
PFNGLNORMALPOINTEREXTPROC = CFUNCTYPE(None, GLenum, GLsizei, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:4754
PFNGLTEXCOORDPOINTEREXTPROC = CFUNCTYPE(None, GLint, GLenum, GLsizei, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:4755
PFNGLVERTEXPOINTEREXTPROC = CFUNCTYPE(None, GLint, GLenum, GLsizei, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:4756
# EXT_misc_attribute (GL/glext.h:4759)
GL_EXT_misc_attribute = 1 	# GL/glext.h:4760
# SGIS_generate_mipmap (GL/glext.h:4763)
GL_SGIS_generate_mipmap = 1 	# GL/glext.h:4764
# SGIX_clipmap (GL/glext.h:4767)
GL_SGIX_clipmap = 1 	# GL/glext.h:4768
# SGIX_shadow (GL/glext.h:4771)
GL_SGIX_shadow = 1 	# GL/glext.h:4772
# SGIS_texture_edge_clamp (GL/glext.h:4775)
GL_SGIS_texture_edge_clamp = 1 	# GL/glext.h:4776
# SGIS_texture_border_clamp (GL/glext.h:4779)
GL_SGIS_texture_border_clamp = 1 	# GL/glext.h:4780
# EXT_blend_minmax (GL/glext.h:4783)
GL_EXT_blend_minmax = 1 	# GL/glext.h:4784
# GL/glext.h:4786
glBlendEquationEXT = _link_function('glBlendEquationEXT', None, [GLenum], 'EXT_blend_minmax')

PFNGLBLENDEQUATIONEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:4788
# EXT_blend_subtract (GL/glext.h:4791)
GL_EXT_blend_subtract = 1 	# GL/glext.h:4792
# EXT_blend_logic_op (GL/glext.h:4795)
GL_EXT_blend_logic_op = 1 	# GL/glext.h:4796
# SGIX_interlace (GL/glext.h:4799)
GL_SGIX_interlace = 1 	# GL/glext.h:4800
# SGIX_pixel_tiles (GL/glext.h:4803)
GL_SGIX_pixel_tiles = 1 	# GL/glext.h:4804
# SGIX_texture_select (GL/glext.h:4807)
GL_SGIX_texture_select = 1 	# GL/glext.h:4808
# SGIX_sprite (GL/glext.h:4811)
GL_SGIX_sprite = 1 	# GL/glext.h:4812
# GL/glext.h:4814
glSpriteParameterfSGIX = _link_function('glSpriteParameterfSGIX', None, [GLenum, GLfloat], 'SGIX_sprite')

# GL/glext.h:4815
glSpriteParameterfvSGIX = _link_function('glSpriteParameterfvSGIX', None, [GLenum, POINTER(GLfloat)], 'SGIX_sprite')

# GL/glext.h:4816
glSpriteParameteriSGIX = _link_function('glSpriteParameteriSGIX', None, [GLenum, GLint], 'SGIX_sprite')

# GL/glext.h:4817
glSpriteParameterivSGIX = _link_function('glSpriteParameterivSGIX', None, [GLenum, POINTER(GLint)], 'SGIX_sprite')

PFNGLSPRITEPARAMETERFSGIXPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:4819
PFNGLSPRITEPARAMETERFVSGIXPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4820
PFNGLSPRITEPARAMETERISGIXPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:4821
PFNGLSPRITEPARAMETERIVSGIXPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:4822
# SGIX_texture_multi_buffer (GL/glext.h:4825)
GL_SGIX_texture_multi_buffer = 1 	# GL/glext.h:4826
# EXT_point_parameters (GL/glext.h:4829)
GL_EXT_point_parameters = 1 	# GL/glext.h:4830
# GL/glext.h:4832
glPointParameterfEXT = _link_function('glPointParameterfEXT', None, [GLenum, GLfloat], 'EXT_point_parameters')

# GL/glext.h:4833
glPointParameterfvEXT = _link_function('glPointParameterfvEXT', None, [GLenum, POINTER(GLfloat)], 'EXT_point_parameters')

PFNGLPOINTPARAMETERFEXTPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:4835
PFNGLPOINTPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4836
# SGIS_point_parameters (GL/glext.h:4839)
GL_SGIS_point_parameters = 1 	# GL/glext.h:4840
# GL/glext.h:4842
glPointParameterfSGIS = _link_function('glPointParameterfSGIS', None, [GLenum, GLfloat], 'SGIS_point_parameters')

# GL/glext.h:4843
glPointParameterfvSGIS = _link_function('glPointParameterfvSGIS', None, [GLenum, POINTER(GLfloat)], 'SGIS_point_parameters')

PFNGLPOINTPARAMETERFSGISPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:4845
PFNGLPOINTPARAMETERFVSGISPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4846
# SGIX_instruments (GL/glext.h:4849)
GL_SGIX_instruments = 1 	# GL/glext.h:4850
# GL/glext.h:4852
glGetInstrumentsSGIX = _link_function('glGetInstrumentsSGIX', GLint, [], 'SGIX_instruments')

# GL/glext.h:4853
glInstrumentsBufferSGIX = _link_function('glInstrumentsBufferSGIX', None, [GLsizei, POINTER(GLint)], 'SGIX_instruments')

# GL/glext.h:4854
glPollInstrumentsSGIX = _link_function('glPollInstrumentsSGIX', GLint, [POINTER(GLint)], 'SGIX_instruments')

# GL/glext.h:4855
glReadInstrumentsSGIX = _link_function('glReadInstrumentsSGIX', None, [GLint], 'SGIX_instruments')

# GL/glext.h:4856
glStartInstrumentsSGIX = _link_function('glStartInstrumentsSGIX', None, [], 'SGIX_instruments')

# GL/glext.h:4857
glStopInstrumentsSGIX = _link_function('glStopInstrumentsSGIX', None, [GLint], 'SGIX_instruments')

PFNGLGETINSTRUMENTSSGIXPROC = CFUNCTYPE(GLint) 	# GL/glext.h:4859
PFNGLINSTRUMENTSBUFFERSGIXPROC = CFUNCTYPE(None, GLsizei, POINTER(GLint)) 	# GL/glext.h:4860
PFNGLPOLLINSTRUMENTSSGIXPROC = CFUNCTYPE(GLint, POINTER(GLint)) 	# GL/glext.h:4861
PFNGLREADINSTRUMENTSSGIXPROC = CFUNCTYPE(None, GLint) 	# GL/glext.h:4862
PFNGLSTARTINSTRUMENTSSGIXPROC = CFUNCTYPE(None) 	# GL/glext.h:4863
PFNGLSTOPINSTRUMENTSSGIXPROC = CFUNCTYPE(None, GLint) 	# GL/glext.h:4864
# SGIX_texture_scale_bias (GL/glext.h:4867)
GL_SGIX_texture_scale_bias = 1 	# GL/glext.h:4868
# SGIX_framezoom (GL/glext.h:4871)
GL_SGIX_framezoom = 1 	# GL/glext.h:4872
# GL/glext.h:4874
glFrameZoomSGIX = _link_function('glFrameZoomSGIX', None, [GLint], 'SGIX_framezoom')

PFNGLFRAMEZOOMSGIXPROC = CFUNCTYPE(None, GLint) 	# GL/glext.h:4876
# SGIX_tag_sample_buffer (GL/glext.h:4879)
GL_SGIX_tag_sample_buffer = 1 	# GL/glext.h:4880
# GL/glext.h:4882
glTagSampleBufferSGIX = _link_function('glTagSampleBufferSGIX', None, [], 'SGIX_tag_sample_buffer')

PFNGLTAGSAMPLEBUFFERSGIXPROC = CFUNCTYPE(None) 	# GL/glext.h:4884
# SGIX_polynomial_ffd (GL/glext.h:4887)
GL_SGIX_polynomial_ffd = 1 	# GL/glext.h:4888
# GL/glext.h:4890
glDeformationMap3dSGIX = _link_function('glDeformationMap3dSGIX', None, [GLenum, GLdouble, GLdouble, GLint, GLint, GLdouble, GLdouble, GLint, GLint, GLdouble, GLdouble, GLint, GLint, POINTER(GLdouble)], 'SGIX_polynomial_ffd')

# GL/glext.h:4891
glDeformationMap3fSGIX = _link_function('glDeformationMap3fSGIX', None, [GLenum, GLfloat, GLfloat, GLint, GLint, GLfloat, GLfloat, GLint, GLint, GLfloat, GLfloat, GLint, GLint, POINTER(GLfloat)], 'SGIX_polynomial_ffd')

GLbitfield = c_uint 	# /usr/include/GL/gl.h:55
# GL/glext.h:4892
glDeformSGIX = _link_function('glDeformSGIX', None, [GLbitfield], 'SGIX_polynomial_ffd')

# GL/glext.h:4893
glLoadIdentityDeformationMapSGIX = _link_function('glLoadIdentityDeformationMapSGIX', None, [GLbitfield], 'SGIX_polynomial_ffd')

PFNGLDEFORMATIONMAP3DSGIXPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble, GLint, GLint, GLdouble, GLdouble, GLint, GLint, GLdouble, GLdouble, GLint, GLint, POINTER(GLdouble)) 	# GL/glext.h:4895
PFNGLDEFORMATIONMAP3FSGIXPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat, GLint, GLint, GLfloat, GLfloat, GLint, GLint, GLfloat, GLfloat, GLint, GLint, POINTER(GLfloat)) 	# GL/glext.h:4896
PFNGLDEFORMSGIXPROC = CFUNCTYPE(None, GLbitfield) 	# GL/glext.h:4897
PFNGLLOADIDENTITYDEFORMATIONMAPSGIXPROC = CFUNCTYPE(None, GLbitfield) 	# GL/glext.h:4898
# SGIX_reference_plane (GL/glext.h:4901)
GL_SGIX_reference_plane = 1 	# GL/glext.h:4902
# GL/glext.h:4904
glReferencePlaneSGIX = _link_function('glReferencePlaneSGIX', None, [POINTER(GLdouble)], 'SGIX_reference_plane')

PFNGLREFERENCEPLANESGIXPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:4906
# SGIX_flush_raster (GL/glext.h:4909)
GL_SGIX_flush_raster = 1 	# GL/glext.h:4910
# GL/glext.h:4912
glFlushRasterSGIX = _link_function('glFlushRasterSGIX', None, [], 'SGIX_flush_raster')

PFNGLFLUSHRASTERSGIXPROC = CFUNCTYPE(None) 	# GL/glext.h:4914
# SGIX_depth_texture (GL/glext.h:4917)
GL_SGIX_depth_texture = 1 	# GL/glext.h:4918
# SGIS_fog_function (GL/glext.h:4921)
GL_SGIS_fog_function = 1 	# GL/glext.h:4922
# GL/glext.h:4924
glFogFuncSGIS = _link_function('glFogFuncSGIS', None, [GLsizei, POINTER(GLfloat)], 'SGIS_fog_function')

# GL/glext.h:4925
glGetFogFuncSGIS = _link_function('glGetFogFuncSGIS', None, [POINTER(GLfloat)], 'SGIS_fog_function')

PFNGLFOGFUNCSGISPROC = CFUNCTYPE(None, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:4927
PFNGLGETFOGFUNCSGISPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:4928
# SGIX_fog_offset (GL/glext.h:4931)
GL_SGIX_fog_offset = 1 	# GL/glext.h:4932
# HP_image_transform (GL/glext.h:4935)
GL_HP_image_transform = 1 	# GL/glext.h:4936
# GL/glext.h:4938
glImageTransformParameteriHP = _link_function('glImageTransformParameteriHP', None, [GLenum, GLenum, GLint], 'HP_image_transform')

# GL/glext.h:4939
glImageTransformParameterfHP = _link_function('glImageTransformParameterfHP', None, [GLenum, GLenum, GLfloat], 'HP_image_transform')

# GL/glext.h:4940
glImageTransformParameterivHP = _link_function('glImageTransformParameterivHP', None, [GLenum, GLenum, POINTER(GLint)], 'HP_image_transform')

# GL/glext.h:4941
glImageTransformParameterfvHP = _link_function('glImageTransformParameterfvHP', None, [GLenum, GLenum, POINTER(GLfloat)], 'HP_image_transform')

# GL/glext.h:4942
glGetImageTransformParameterivHP = _link_function('glGetImageTransformParameterivHP', None, [GLenum, GLenum, POINTER(GLint)], 'HP_image_transform')

# GL/glext.h:4943
glGetImageTransformParameterfvHP = _link_function('glGetImageTransformParameterfvHP', None, [GLenum, GLenum, POINTER(GLfloat)], 'HP_image_transform')

PFNGLIMAGETRANSFORMPARAMETERIHPPROC = CFUNCTYPE(None, GLenum, GLenum, GLint) 	# GL/glext.h:4945
PFNGLIMAGETRANSFORMPARAMETERFHPPROC = CFUNCTYPE(None, GLenum, GLenum, GLfloat) 	# GL/glext.h:4946
PFNGLIMAGETRANSFORMPARAMETERIVHPPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:4947
PFNGLIMAGETRANSFORMPARAMETERFVHPPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4948
PFNGLGETIMAGETRANSFORMPARAMETERIVHPPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:4949
PFNGLGETIMAGETRANSFORMPARAMETERFVHPPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4950
# HP_convolution_border_modes (GL/glext.h:4953)
GL_HP_convolution_border_modes = 1 	# GL/glext.h:4954
# SGIX_texture_add_env (GL/glext.h:4957)
GL_SGIX_texture_add_env = 1 	# GL/glext.h:4958
# EXT_color_subtable (GL/glext.h:4961)
GL_EXT_color_subtable = 1 	# GL/glext.h:4962
# GL/glext.h:4964
glColorSubTableEXT = _link_function('glColorSubTableEXT', None, [GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_color_subtable')

# GL/glext.h:4965
glCopyColorSubTableEXT = _link_function('glCopyColorSubTableEXT', None, [GLenum, GLsizei, GLint, GLint, GLsizei], 'EXT_color_subtable')

PFNGLCOLORSUBTABLEEXTPROC = CFUNCTYPE(None, GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:4967
PFNGLCOPYCOLORSUBTABLEEXTPROC = CFUNCTYPE(None, GLenum, GLsizei, GLint, GLint, GLsizei) 	# GL/glext.h:4968
# PGI_vertex_hints (GL/glext.h:4971)
GL_PGI_vertex_hints = 1 	# GL/glext.h:4972
# PGI_misc_hints (GL/glext.h:4975)
GL_PGI_misc_hints = 1 	# GL/glext.h:4976
# GL/glext.h:4978
glHintPGI = _link_function('glHintPGI', None, [GLenum, GLint], 'PGI_misc_hints')

PFNGLHINTPGIPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:4980
# EXT_paletted_texture (GL/glext.h:4983)
GL_EXT_paletted_texture = 1 	# GL/glext.h:4984
# GL/glext.h:4986
glColorTableEXT = _link_function('glColorTableEXT', None, [GLenum, GLenum, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_paletted_texture')

# GL/glext.h:4987
glGetColorTableEXT = _link_function('glGetColorTableEXT', None, [GLenum, GLenum, GLenum, POINTER(GLvoid)], 'EXT_paletted_texture')

# GL/glext.h:4988
glGetColorTableParameterivEXT = _link_function('glGetColorTableParameterivEXT', None, [GLenum, GLenum, POINTER(GLint)], 'EXT_paletted_texture')

# GL/glext.h:4989
glGetColorTableParameterfvEXT = _link_function('glGetColorTableParameterfvEXT', None, [GLenum, GLenum, POINTER(GLfloat)], 'EXT_paletted_texture')

PFNGLCOLORTABLEEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:4991
PFNGLGETCOLORTABLEEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:4992
PFNGLGETCOLORTABLEPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:4993
PFNGLGETCOLORTABLEPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:4994
# EXT_clip_volume_hint (GL/glext.h:4997)
GL_EXT_clip_volume_hint = 1 	# GL/glext.h:4998
# SGIX_list_priority (GL/glext.h:5001)
GL_SGIX_list_priority = 1 	# GL/glext.h:5002
# GL/glext.h:5004
glGetListParameterfvSGIX = _link_function('glGetListParameterfvSGIX', None, [GLuint, GLenum, POINTER(GLfloat)], 'SGIX_list_priority')

# GL/glext.h:5005
glGetListParameterivSGIX = _link_function('glGetListParameterivSGIX', None, [GLuint, GLenum, POINTER(GLint)], 'SGIX_list_priority')

# GL/glext.h:5006
glListParameterfSGIX = _link_function('glListParameterfSGIX', None, [GLuint, GLenum, GLfloat], 'SGIX_list_priority')

# GL/glext.h:5007
glListParameterfvSGIX = _link_function('glListParameterfvSGIX', None, [GLuint, GLenum, POINTER(GLfloat)], 'SGIX_list_priority')

# GL/glext.h:5008
glListParameteriSGIX = _link_function('glListParameteriSGIX', None, [GLuint, GLenum, GLint], 'SGIX_list_priority')

# GL/glext.h:5009
glListParameterivSGIX = _link_function('glListParameterivSGIX', None, [GLuint, GLenum, POINTER(GLint)], 'SGIX_list_priority')

PFNGLGETLISTPARAMETERFVSGIXPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5011
PFNGLGETLISTPARAMETERIVSGIXPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:5012
PFNGLLISTPARAMETERFSGIXPROC = CFUNCTYPE(None, GLuint, GLenum, GLfloat) 	# GL/glext.h:5013
PFNGLLISTPARAMETERFVSGIXPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5014
PFNGLLISTPARAMETERISGIXPROC = CFUNCTYPE(None, GLuint, GLenum, GLint) 	# GL/glext.h:5015
PFNGLLISTPARAMETERIVSGIXPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:5016
# SGIX_ir_instrument1 (GL/glext.h:5019)
GL_SGIX_ir_instrument1 = 1 	# GL/glext.h:5020
# SGIX_calligraphic_fragment (GL/glext.h:5023)
GL_SGIX_calligraphic_fragment = 1 	# GL/glext.h:5024
# SGIX_texture_lod_bias (GL/glext.h:5027)
GL_SGIX_texture_lod_bias = 1 	# GL/glext.h:5028
# SGIX_shadow_ambient (GL/glext.h:5031)
GL_SGIX_shadow_ambient = 1 	# GL/glext.h:5032
# EXT_index_texture (GL/glext.h:5035)
GL_EXT_index_texture = 1 	# GL/glext.h:5036
# EXT_index_material (GL/glext.h:5039)
GL_EXT_index_material = 1 	# GL/glext.h:5040
# GL/glext.h:5042
glIndexMaterialEXT = _link_function('glIndexMaterialEXT', None, [GLenum, GLenum], 'EXT_index_material')

PFNGLINDEXMATERIALEXTPROC = CFUNCTYPE(None, GLenum, GLenum) 	# GL/glext.h:5044
# EXT_index_func (GL/glext.h:5047)
GL_EXT_index_func = 1 	# GL/glext.h:5048
# GL/glext.h:5050
glIndexFuncEXT = _link_function('glIndexFuncEXT', None, [GLenum, GLclampf], 'EXT_index_func')

PFNGLINDEXFUNCEXTPROC = CFUNCTYPE(None, GLenum, GLclampf) 	# GL/glext.h:5052
# EXT_index_array_formats (GL/glext.h:5055)
GL_EXT_index_array_formats = 1 	# GL/glext.h:5056
# EXT_compiled_vertex_array (GL/glext.h:5059)
GL_EXT_compiled_vertex_array = 1 	# GL/glext.h:5060
# GL/glext.h:5062
glLockArraysEXT = _link_function('glLockArraysEXT', None, [GLint, GLsizei], 'EXT_compiled_vertex_array')

# GL/glext.h:5063
glUnlockArraysEXT = _link_function('glUnlockArraysEXT', None, [], 'EXT_compiled_vertex_array')

PFNGLLOCKARRAYSEXTPROC = CFUNCTYPE(None, GLint, GLsizei) 	# GL/glext.h:5065
PFNGLUNLOCKARRAYSEXTPROC = CFUNCTYPE(None) 	# GL/glext.h:5066
# EXT_cull_vertex (GL/glext.h:5069)
GL_EXT_cull_vertex = 1 	# GL/glext.h:5070
# GL/glext.h:5072
glCullParameterdvEXT = _link_function('glCullParameterdvEXT', None, [GLenum, POINTER(GLdouble)], 'EXT_cull_vertex')

# GL/glext.h:5073
glCullParameterfvEXT = _link_function('glCullParameterfvEXT', None, [GLenum, POINTER(GLfloat)], 'EXT_cull_vertex')

PFNGLCULLPARAMETERDVEXTPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:5075
PFNGLCULLPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5076
# SGIX_ycrcb (GL/glext.h:5079)
GL_SGIX_ycrcb = 1 	# GL/glext.h:5080
# SGIX_fragment_lighting (GL/glext.h:5083)
GL_SGIX_fragment_lighting = 1 	# GL/glext.h:5084
# GL/glext.h:5086
glFragmentColorMaterialSGIX = _link_function('glFragmentColorMaterialSGIX', None, [GLenum, GLenum], 'SGIX_fragment_lighting')

# GL/glext.h:5087
glFragmentLightfSGIX = _link_function('glFragmentLightfSGIX', None, [GLenum, GLenum, GLfloat], 'SGIX_fragment_lighting')

# GL/glext.h:5088
glFragmentLightfvSGIX = _link_function('glFragmentLightfvSGIX', None, [GLenum, GLenum, POINTER(GLfloat)], 'SGIX_fragment_lighting')

# GL/glext.h:5089
glFragmentLightiSGIX = _link_function('glFragmentLightiSGIX', None, [GLenum, GLenum, GLint], 'SGIX_fragment_lighting')

# GL/glext.h:5090
glFragmentLightivSGIX = _link_function('glFragmentLightivSGIX', None, [GLenum, GLenum, POINTER(GLint)], 'SGIX_fragment_lighting')

# GL/glext.h:5091
glFragmentLightModelfSGIX = _link_function('glFragmentLightModelfSGIX', None, [GLenum, GLfloat], 'SGIX_fragment_lighting')

# GL/glext.h:5092
glFragmentLightModelfvSGIX = _link_function('glFragmentLightModelfvSGIX', None, [GLenum, POINTER(GLfloat)], 'SGIX_fragment_lighting')

# GL/glext.h:5093
glFragmentLightModeliSGIX = _link_function('glFragmentLightModeliSGIX', None, [GLenum, GLint], 'SGIX_fragment_lighting')

# GL/glext.h:5094
glFragmentLightModelivSGIX = _link_function('glFragmentLightModelivSGIX', None, [GLenum, POINTER(GLint)], 'SGIX_fragment_lighting')

# GL/glext.h:5095
glFragmentMaterialfSGIX = _link_function('glFragmentMaterialfSGIX', None, [GLenum, GLenum, GLfloat], 'SGIX_fragment_lighting')

# GL/glext.h:5096
glFragmentMaterialfvSGIX = _link_function('glFragmentMaterialfvSGIX', None, [GLenum, GLenum, POINTER(GLfloat)], 'SGIX_fragment_lighting')

# GL/glext.h:5097
glFragmentMaterialiSGIX = _link_function('glFragmentMaterialiSGIX', None, [GLenum, GLenum, GLint], 'SGIX_fragment_lighting')

# GL/glext.h:5098
glFragmentMaterialivSGIX = _link_function('glFragmentMaterialivSGIX', None, [GLenum, GLenum, POINTER(GLint)], 'SGIX_fragment_lighting')

# GL/glext.h:5099
glGetFragmentLightfvSGIX = _link_function('glGetFragmentLightfvSGIX', None, [GLenum, GLenum, POINTER(GLfloat)], 'SGIX_fragment_lighting')

# GL/glext.h:5100
glGetFragmentLightivSGIX = _link_function('glGetFragmentLightivSGIX', None, [GLenum, GLenum, POINTER(GLint)], 'SGIX_fragment_lighting')

# GL/glext.h:5101
glGetFragmentMaterialfvSGIX = _link_function('glGetFragmentMaterialfvSGIX', None, [GLenum, GLenum, POINTER(GLfloat)], 'SGIX_fragment_lighting')

# GL/glext.h:5102
glGetFragmentMaterialivSGIX = _link_function('glGetFragmentMaterialivSGIX', None, [GLenum, GLenum, POINTER(GLint)], 'SGIX_fragment_lighting')

# GL/glext.h:5103
glLightEnviSGIX = _link_function('glLightEnviSGIX', None, [GLenum, GLint], 'SGIX_fragment_lighting')

PFNGLFRAGMENTCOLORMATERIALSGIXPROC = CFUNCTYPE(None, GLenum, GLenum) 	# GL/glext.h:5105
PFNGLFRAGMENTLIGHTFSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, GLfloat) 	# GL/glext.h:5106
PFNGLFRAGMENTLIGHTFVSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5107
PFNGLFRAGMENTLIGHTISGIXPROC = CFUNCTYPE(None, GLenum, GLenum, GLint) 	# GL/glext.h:5108
PFNGLFRAGMENTLIGHTIVSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5109
PFNGLFRAGMENTLIGHTMODELFSGIXPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:5110
PFNGLFRAGMENTLIGHTMODELFVSGIXPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5111
PFNGLFRAGMENTLIGHTMODELISGIXPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:5112
PFNGLFRAGMENTLIGHTMODELIVSGIXPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:5113
PFNGLFRAGMENTMATERIALFSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, GLfloat) 	# GL/glext.h:5114
PFNGLFRAGMENTMATERIALFVSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5115
PFNGLFRAGMENTMATERIALISGIXPROC = CFUNCTYPE(None, GLenum, GLenum, GLint) 	# GL/glext.h:5116
PFNGLFRAGMENTMATERIALIVSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5117
PFNGLGETFRAGMENTLIGHTFVSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5118
PFNGLGETFRAGMENTLIGHTIVSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5119
PFNGLGETFRAGMENTMATERIALFVSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5120
PFNGLGETFRAGMENTMATERIALIVSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5121
PFNGLLIGHTENVISGIXPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:5122
# IBM_rasterpos_clip (GL/glext.h:5125)
GL_IBM_rasterpos_clip = 1 	# GL/glext.h:5126
# HP_texture_lighting (GL/glext.h:5129)
GL_HP_texture_lighting = 1 	# GL/glext.h:5130
# EXT_draw_range_elements (GL/glext.h:5133)
GL_EXT_draw_range_elements = 1 	# GL/glext.h:5134
# GL/glext.h:5136
glDrawRangeElementsEXT = _link_function('glDrawRangeElementsEXT', None, [GLenum, GLuint, GLuint, GLsizei, GLenum, POINTER(GLvoid)], 'EXT_draw_range_elements')

PFNGLDRAWRANGEELEMENTSEXTPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLsizei, GLenum, POINTER(GLvoid)) 	# GL/glext.h:5138
# WIN_phong_shading (GL/glext.h:5141)
GL_WIN_phong_shading = 1 	# GL/glext.h:5142
# WIN_specular_fog (GL/glext.h:5145)
GL_WIN_specular_fog = 1 	# GL/glext.h:5146
# EXT_light_texture (GL/glext.h:5149)
GL_EXT_light_texture = 1 	# GL/glext.h:5150
# GL/glext.h:5152
glApplyTextureEXT = _link_function('glApplyTextureEXT', None, [GLenum], 'EXT_light_texture')

# GL/glext.h:5153
glTextureLightEXT = _link_function('glTextureLightEXT', None, [GLenum], 'EXT_light_texture')

# GL/glext.h:5154
glTextureMaterialEXT = _link_function('glTextureMaterialEXT', None, [GLenum, GLenum], 'EXT_light_texture')

PFNGLAPPLYTEXTUREEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:5156
PFNGLTEXTURELIGHTEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:5157
PFNGLTEXTUREMATERIALEXTPROC = CFUNCTYPE(None, GLenum, GLenum) 	# GL/glext.h:5158
# SGIX_blend_alpha_minmax (GL/glext.h:5161)
GL_SGIX_blend_alpha_minmax = 1 	# GL/glext.h:5162
# EXT_bgra (GL/glext.h:5165)
GL_EXT_bgra = 1 	# GL/glext.h:5166
# SGIX_async (GL/glext.h:5169)
GL_SGIX_async = 1 	# GL/glext.h:5170
# GL/glext.h:5172
glAsyncMarkerSGIX = _link_function('glAsyncMarkerSGIX', None, [GLuint], 'SGIX_async')

# GL/glext.h:5173
glFinishAsyncSGIX = _link_function('glFinishAsyncSGIX', GLint, [POINTER(GLuint)], 'SGIX_async')

# GL/glext.h:5174
glPollAsyncSGIX = _link_function('glPollAsyncSGIX', GLint, [POINTER(GLuint)], 'SGIX_async')

# GL/glext.h:5175
glGenAsyncMarkersSGIX = _link_function('glGenAsyncMarkersSGIX', GLuint, [GLsizei], 'SGIX_async')

# GL/glext.h:5176
glDeleteAsyncMarkersSGIX = _link_function('glDeleteAsyncMarkersSGIX', None, [GLuint, GLsizei], 'SGIX_async')

# GL/glext.h:5177
glIsAsyncMarkerSGIX = _link_function('glIsAsyncMarkerSGIX', GLboolean, [GLuint], 'SGIX_async')

PFNGLASYNCMARKERSGIXPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:5179
PFNGLFINISHASYNCSGIXPROC = CFUNCTYPE(GLint, POINTER(GLuint)) 	# GL/glext.h:5180
PFNGLPOLLASYNCSGIXPROC = CFUNCTYPE(GLint, POINTER(GLuint)) 	# GL/glext.h:5181
PFNGLGENASYNCMARKERSSGIXPROC = CFUNCTYPE(GLuint, GLsizei) 	# GL/glext.h:5182
PFNGLDELETEASYNCMARKERSSGIXPROC = CFUNCTYPE(None, GLuint, GLsizei) 	# GL/glext.h:5183
PFNGLISASYNCMARKERSGIXPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:5184
# SGIX_async_pixel (GL/glext.h:5187)
GL_SGIX_async_pixel = 1 	# GL/glext.h:5188
# SGIX_async_histogram (GL/glext.h:5191)
GL_SGIX_async_histogram = 1 	# GL/glext.h:5192
# INTEL_parallel_arrays (GL/glext.h:5195)
GL_INTEL_parallel_arrays = 1 	# GL/glext.h:5196
# GL/glext.h:5198
glVertexPointervINTEL = _link_function('glVertexPointervINTEL', None, [GLint, GLenum, POINTER(POINTER(GLvoid))], 'INTEL_parallel_arrays')

# GL/glext.h:5199
glNormalPointervINTEL = _link_function('glNormalPointervINTEL', None, [GLenum, POINTER(POINTER(GLvoid))], 'INTEL_parallel_arrays')

# GL/glext.h:5200
glColorPointervINTEL = _link_function('glColorPointervINTEL', None, [GLint, GLenum, POINTER(POINTER(GLvoid))], 'INTEL_parallel_arrays')

# GL/glext.h:5201
glTexCoordPointervINTEL = _link_function('glTexCoordPointervINTEL', None, [GLint, GLenum, POINTER(POINTER(GLvoid))], 'INTEL_parallel_arrays')

PFNGLVERTEXPOINTERVINTELPROC = CFUNCTYPE(None, GLint, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:5203
PFNGLNORMALPOINTERVINTELPROC = CFUNCTYPE(None, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:5204
PFNGLCOLORPOINTERVINTELPROC = CFUNCTYPE(None, GLint, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:5205
PFNGLTEXCOORDPOINTERVINTELPROC = CFUNCTYPE(None, GLint, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:5206
# HP_occlusion_test (GL/glext.h:5209)
GL_HP_occlusion_test = 1 	# GL/glext.h:5210
# EXT_pixel_transform (GL/glext.h:5213)
GL_EXT_pixel_transform = 1 	# GL/glext.h:5214
# GL/glext.h:5216
glPixelTransformParameteriEXT = _link_function('glPixelTransformParameteriEXT', None, [GLenum, GLenum, GLint], 'EXT_pixel_transform')

# GL/glext.h:5217
glPixelTransformParameterfEXT = _link_function('glPixelTransformParameterfEXT', None, [GLenum, GLenum, GLfloat], 'EXT_pixel_transform')

# GL/glext.h:5218
glPixelTransformParameterivEXT = _link_function('glPixelTransformParameterivEXT', None, [GLenum, GLenum, POINTER(GLint)], 'EXT_pixel_transform')

# GL/glext.h:5219
glPixelTransformParameterfvEXT = _link_function('glPixelTransformParameterfvEXT', None, [GLenum, GLenum, POINTER(GLfloat)], 'EXT_pixel_transform')

PFNGLPIXELTRANSFORMPARAMETERIEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint) 	# GL/glext.h:5221
PFNGLPIXELTRANSFORMPARAMETERFEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLfloat) 	# GL/glext.h:5222
PFNGLPIXELTRANSFORMPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5223
PFNGLPIXELTRANSFORMPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5224
# EXT_pixel_transform_color_table (GL/glext.h:5227)
GL_EXT_pixel_transform_color_table = 1 	# GL/glext.h:5228
# EXT_shared_texture_palette (GL/glext.h:5231)
GL_EXT_shared_texture_palette = 1 	# GL/glext.h:5232
# EXT_separate_specular_color (GL/glext.h:5235)
GL_EXT_separate_specular_color = 1 	# GL/glext.h:5236
# EXT_secondary_color (GL/glext.h:5239)
GL_EXT_secondary_color = 1 	# GL/glext.h:5240
# GL/glext.h:5242
glSecondaryColor3bEXT = _link_function('glSecondaryColor3bEXT', None, [GLbyte, GLbyte, GLbyte], 'EXT_secondary_color')

# GL/glext.h:5243
glSecondaryColor3bvEXT = _link_function('glSecondaryColor3bvEXT', None, [POINTER(GLbyte)], 'EXT_secondary_color')

# GL/glext.h:5244
glSecondaryColor3dEXT = _link_function('glSecondaryColor3dEXT', None, [GLdouble, GLdouble, GLdouble], 'EXT_secondary_color')

# GL/glext.h:5245
glSecondaryColor3dvEXT = _link_function('glSecondaryColor3dvEXT', None, [POINTER(GLdouble)], 'EXT_secondary_color')

# GL/glext.h:5246
glSecondaryColor3fEXT = _link_function('glSecondaryColor3fEXT', None, [GLfloat, GLfloat, GLfloat], 'EXT_secondary_color')

# GL/glext.h:5247
glSecondaryColor3fvEXT = _link_function('glSecondaryColor3fvEXT', None, [POINTER(GLfloat)], 'EXT_secondary_color')

# GL/glext.h:5248
glSecondaryColor3iEXT = _link_function('glSecondaryColor3iEXT', None, [GLint, GLint, GLint], 'EXT_secondary_color')

# GL/glext.h:5249
glSecondaryColor3ivEXT = _link_function('glSecondaryColor3ivEXT', None, [POINTER(GLint)], 'EXT_secondary_color')

# GL/glext.h:5250
glSecondaryColor3sEXT = _link_function('glSecondaryColor3sEXT', None, [GLshort, GLshort, GLshort], 'EXT_secondary_color')

# GL/glext.h:5251
glSecondaryColor3svEXT = _link_function('glSecondaryColor3svEXT', None, [POINTER(GLshort)], 'EXT_secondary_color')

# GL/glext.h:5252
glSecondaryColor3ubEXT = _link_function('glSecondaryColor3ubEXT', None, [GLubyte, GLubyte, GLubyte], 'EXT_secondary_color')

# GL/glext.h:5253
glSecondaryColor3ubvEXT = _link_function('glSecondaryColor3ubvEXT', None, [POINTER(GLubyte)], 'EXT_secondary_color')

# GL/glext.h:5254
glSecondaryColor3uiEXT = _link_function('glSecondaryColor3uiEXT', None, [GLuint, GLuint, GLuint], 'EXT_secondary_color')

# GL/glext.h:5255
glSecondaryColor3uivEXT = _link_function('glSecondaryColor3uivEXT', None, [POINTER(GLuint)], 'EXT_secondary_color')

# GL/glext.h:5256
glSecondaryColor3usEXT = _link_function('glSecondaryColor3usEXT', None, [GLushort, GLushort, GLushort], 'EXT_secondary_color')

# GL/glext.h:5257
glSecondaryColor3usvEXT = _link_function('glSecondaryColor3usvEXT', None, [POINTER(GLushort)], 'EXT_secondary_color')

# GL/glext.h:5258
glSecondaryColorPointerEXT = _link_function('glSecondaryColorPointerEXT', None, [GLint, GLenum, GLsizei, POINTER(GLvoid)], 'EXT_secondary_color')

PFNGLSECONDARYCOLOR3BEXTPROC = CFUNCTYPE(None, GLbyte, GLbyte, GLbyte) 	# GL/glext.h:5260
PFNGLSECONDARYCOLOR3BVEXTPROC = CFUNCTYPE(None, POINTER(GLbyte)) 	# GL/glext.h:5261
PFNGLSECONDARYCOLOR3DEXTPROC = CFUNCTYPE(None, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:5262
PFNGLSECONDARYCOLOR3DVEXTPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:5263
PFNGLSECONDARYCOLOR3FEXTPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5264
PFNGLSECONDARYCOLOR3FVEXTPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:5265
PFNGLSECONDARYCOLOR3IEXTPROC = CFUNCTYPE(None, GLint, GLint, GLint) 	# GL/glext.h:5266
PFNGLSECONDARYCOLOR3IVEXTPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:5267
PFNGLSECONDARYCOLOR3SEXTPROC = CFUNCTYPE(None, GLshort, GLshort, GLshort) 	# GL/glext.h:5268
PFNGLSECONDARYCOLOR3SVEXTPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:5269
PFNGLSECONDARYCOLOR3UBEXTPROC = CFUNCTYPE(None, GLubyte, GLubyte, GLubyte) 	# GL/glext.h:5270
PFNGLSECONDARYCOLOR3UBVEXTPROC = CFUNCTYPE(None, POINTER(GLubyte)) 	# GL/glext.h:5271
PFNGLSECONDARYCOLOR3UIEXTPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint) 	# GL/glext.h:5272
PFNGLSECONDARYCOLOR3UIVEXTPROC = CFUNCTYPE(None, POINTER(GLuint)) 	# GL/glext.h:5273
PFNGLSECONDARYCOLOR3USEXTPROC = CFUNCTYPE(None, GLushort, GLushort, GLushort) 	# GL/glext.h:5274
PFNGLSECONDARYCOLOR3USVEXTPROC = CFUNCTYPE(None, POINTER(GLushort)) 	# GL/glext.h:5275
PFNGLSECONDARYCOLORPOINTEREXTPROC = CFUNCTYPE(None, GLint, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:5276
# EXT_texture_perturb_normal (GL/glext.h:5279)
GL_EXT_texture_perturb_normal = 1 	# GL/glext.h:5280
# GL/glext.h:5282
glTextureNormalEXT = _link_function('glTextureNormalEXT', None, [GLenum], 'EXT_texture_perturb_normal')

PFNGLTEXTURENORMALEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:5284
# EXT_multi_draw_arrays (GL/glext.h:5287)
GL_EXT_multi_draw_arrays = 1 	# GL/glext.h:5288
# GL/glext.h:5290
glMultiDrawArraysEXT = _link_function('glMultiDrawArraysEXT', None, [GLenum, POINTER(GLint), POINTER(GLsizei), GLsizei], 'EXT_multi_draw_arrays')

# GL/glext.h:5291
glMultiDrawElementsEXT = _link_function('glMultiDrawElementsEXT', None, [GLenum, POINTER(GLsizei), GLenum, POINTER(POINTER(GLvoid)), GLsizei], 'EXT_multi_draw_arrays')

PFNGLMULTIDRAWARRAYSEXTPROC = CFUNCTYPE(None, GLenum, POINTER(GLint), POINTER(GLsizei), GLsizei) 	# GL/glext.h:5293
PFNGLMULTIDRAWELEMENTSEXTPROC = CFUNCTYPE(None, GLenum, POINTER(GLsizei), GLenum, POINTER(POINTER(GLvoid)), GLsizei) 	# GL/glext.h:5294
# EXT_fog_coord (GL/glext.h:5297)
GL_EXT_fog_coord = 1 	# GL/glext.h:5298
# GL/glext.h:5300
glFogCoordfEXT = _link_function('glFogCoordfEXT', None, [GLfloat], 'EXT_fog_coord')

# GL/glext.h:5301
glFogCoordfvEXT = _link_function('glFogCoordfvEXT', None, [POINTER(GLfloat)], 'EXT_fog_coord')

# GL/glext.h:5302
glFogCoorddEXT = _link_function('glFogCoorddEXT', None, [GLdouble], 'EXT_fog_coord')

# GL/glext.h:5303
glFogCoorddvEXT = _link_function('glFogCoorddvEXT', None, [POINTER(GLdouble)], 'EXT_fog_coord')

# GL/glext.h:5304
glFogCoordPointerEXT = _link_function('glFogCoordPointerEXT', None, [GLenum, GLsizei, POINTER(GLvoid)], 'EXT_fog_coord')

PFNGLFOGCOORDFEXTPROC = CFUNCTYPE(None, GLfloat) 	# GL/glext.h:5306
PFNGLFOGCOORDFVEXTPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:5307
PFNGLFOGCOORDDEXTPROC = CFUNCTYPE(None, GLdouble) 	# GL/glext.h:5308
PFNGLFOGCOORDDVEXTPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:5309
PFNGLFOGCOORDPOINTEREXTPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:5310
# REND_screen_coordinates (GL/glext.h:5313)
GL_REND_screen_coordinates = 1 	# GL/glext.h:5314
# EXT_coordinate_frame (GL/glext.h:5317)
GL_EXT_coordinate_frame = 1 	# GL/glext.h:5318
# GL/glext.h:5320
glTangent3bEXT = _link_function('glTangent3bEXT', None, [GLbyte, GLbyte, GLbyte], 'EXT_coordinate_frame')

# GL/glext.h:5321
glTangent3bvEXT = _link_function('glTangent3bvEXT', None, [POINTER(GLbyte)], 'EXT_coordinate_frame')

# GL/glext.h:5322
glTangent3dEXT = _link_function('glTangent3dEXT', None, [GLdouble, GLdouble, GLdouble], 'EXT_coordinate_frame')

# GL/glext.h:5323
glTangent3dvEXT = _link_function('glTangent3dvEXT', None, [POINTER(GLdouble)], 'EXT_coordinate_frame')

# GL/glext.h:5324
glTangent3fEXT = _link_function('glTangent3fEXT', None, [GLfloat, GLfloat, GLfloat], 'EXT_coordinate_frame')

# GL/glext.h:5325
glTangent3fvEXT = _link_function('glTangent3fvEXT', None, [POINTER(GLfloat)], 'EXT_coordinate_frame')

# GL/glext.h:5326
glTangent3iEXT = _link_function('glTangent3iEXT', None, [GLint, GLint, GLint], 'EXT_coordinate_frame')

# GL/glext.h:5327
glTangent3ivEXT = _link_function('glTangent3ivEXT', None, [POINTER(GLint)], 'EXT_coordinate_frame')

# GL/glext.h:5328
glTangent3sEXT = _link_function('glTangent3sEXT', None, [GLshort, GLshort, GLshort], 'EXT_coordinate_frame')

# GL/glext.h:5329
glTangent3svEXT = _link_function('glTangent3svEXT', None, [POINTER(GLshort)], 'EXT_coordinate_frame')

# GL/glext.h:5330
glBinormal3bEXT = _link_function('glBinormal3bEXT', None, [GLbyte, GLbyte, GLbyte], 'EXT_coordinate_frame')

# GL/glext.h:5331
glBinormal3bvEXT = _link_function('glBinormal3bvEXT', None, [POINTER(GLbyte)], 'EXT_coordinate_frame')

# GL/glext.h:5332
glBinormal3dEXT = _link_function('glBinormal3dEXT', None, [GLdouble, GLdouble, GLdouble], 'EXT_coordinate_frame')

# GL/glext.h:5333
glBinormal3dvEXT = _link_function('glBinormal3dvEXT', None, [POINTER(GLdouble)], 'EXT_coordinate_frame')

# GL/glext.h:5334
glBinormal3fEXT = _link_function('glBinormal3fEXT', None, [GLfloat, GLfloat, GLfloat], 'EXT_coordinate_frame')

# GL/glext.h:5335
glBinormal3fvEXT = _link_function('glBinormal3fvEXT', None, [POINTER(GLfloat)], 'EXT_coordinate_frame')

# GL/glext.h:5336
glBinormal3iEXT = _link_function('glBinormal3iEXT', None, [GLint, GLint, GLint], 'EXT_coordinate_frame')

# GL/glext.h:5337
glBinormal3ivEXT = _link_function('glBinormal3ivEXT', None, [POINTER(GLint)], 'EXT_coordinate_frame')

# GL/glext.h:5338
glBinormal3sEXT = _link_function('glBinormal3sEXT', None, [GLshort, GLshort, GLshort], 'EXT_coordinate_frame')

# GL/glext.h:5339
glBinormal3svEXT = _link_function('glBinormal3svEXT', None, [POINTER(GLshort)], 'EXT_coordinate_frame')

# GL/glext.h:5340
glTangentPointerEXT = _link_function('glTangentPointerEXT', None, [GLenum, GLsizei, POINTER(GLvoid)], 'EXT_coordinate_frame')

# GL/glext.h:5341
glBinormalPointerEXT = _link_function('glBinormalPointerEXT', None, [GLenum, GLsizei, POINTER(GLvoid)], 'EXT_coordinate_frame')

PFNGLTANGENT3BEXTPROC = CFUNCTYPE(None, GLbyte, GLbyte, GLbyte) 	# GL/glext.h:5343
PFNGLTANGENT3BVEXTPROC = CFUNCTYPE(None, POINTER(GLbyte)) 	# GL/glext.h:5344
PFNGLTANGENT3DEXTPROC = CFUNCTYPE(None, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:5345
PFNGLTANGENT3DVEXTPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:5346
PFNGLTANGENT3FEXTPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5347
PFNGLTANGENT3FVEXTPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:5348
PFNGLTANGENT3IEXTPROC = CFUNCTYPE(None, GLint, GLint, GLint) 	# GL/glext.h:5349
PFNGLTANGENT3IVEXTPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:5350
PFNGLTANGENT3SEXTPROC = CFUNCTYPE(None, GLshort, GLshort, GLshort) 	# GL/glext.h:5351
PFNGLTANGENT3SVEXTPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:5352
PFNGLBINORMAL3BEXTPROC = CFUNCTYPE(None, GLbyte, GLbyte, GLbyte) 	# GL/glext.h:5353
PFNGLBINORMAL3BVEXTPROC = CFUNCTYPE(None, POINTER(GLbyte)) 	# GL/glext.h:5354
PFNGLBINORMAL3DEXTPROC = CFUNCTYPE(None, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:5355
PFNGLBINORMAL3DVEXTPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:5356
PFNGLBINORMAL3FEXTPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5357
PFNGLBINORMAL3FVEXTPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:5358
PFNGLBINORMAL3IEXTPROC = CFUNCTYPE(None, GLint, GLint, GLint) 	# GL/glext.h:5359
PFNGLBINORMAL3IVEXTPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:5360
PFNGLBINORMAL3SEXTPROC = CFUNCTYPE(None, GLshort, GLshort, GLshort) 	# GL/glext.h:5361
PFNGLBINORMAL3SVEXTPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:5362
PFNGLTANGENTPOINTEREXTPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:5363
PFNGLBINORMALPOINTEREXTPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:5364
# EXT_texture_env_combine (GL/glext.h:5367)
GL_EXT_texture_env_combine = 1 	# GL/glext.h:5368
# APPLE_specular_vector (GL/glext.h:5371)
GL_APPLE_specular_vector = 1 	# GL/glext.h:5372
# APPLE_transform_hint (GL/glext.h:5375)
GL_APPLE_transform_hint = 1 	# GL/glext.h:5376
# SGIX_fog_scale (GL/glext.h:5379)
GL_SGIX_fog_scale = 1 	# GL/glext.h:5380
# SUNX_constant_data (GL/glext.h:5383)
GL_SUNX_constant_data = 1 	# GL/glext.h:5384
# GL/glext.h:5386
glFinishTextureSUNX = _link_function('glFinishTextureSUNX', None, [], 'SUNX_constant_data')

PFNGLFINISHTEXTURESUNXPROC = CFUNCTYPE(None) 	# GL/glext.h:5388
# SUN_global_alpha (GL/glext.h:5391)
GL_SUN_global_alpha = 1 	# GL/glext.h:5392
# GL/glext.h:5394
glGlobalAlphaFactorbSUN = _link_function('glGlobalAlphaFactorbSUN', None, [GLbyte], 'SUN_global_alpha')

# GL/glext.h:5395
glGlobalAlphaFactorsSUN = _link_function('glGlobalAlphaFactorsSUN', None, [GLshort], 'SUN_global_alpha')

# GL/glext.h:5396
glGlobalAlphaFactoriSUN = _link_function('glGlobalAlphaFactoriSUN', None, [GLint], 'SUN_global_alpha')

# GL/glext.h:5397
glGlobalAlphaFactorfSUN = _link_function('glGlobalAlphaFactorfSUN', None, [GLfloat], 'SUN_global_alpha')

# GL/glext.h:5398
glGlobalAlphaFactordSUN = _link_function('glGlobalAlphaFactordSUN', None, [GLdouble], 'SUN_global_alpha')

# GL/glext.h:5399
glGlobalAlphaFactorubSUN = _link_function('glGlobalAlphaFactorubSUN', None, [GLubyte], 'SUN_global_alpha')

# GL/glext.h:5400
glGlobalAlphaFactorusSUN = _link_function('glGlobalAlphaFactorusSUN', None, [GLushort], 'SUN_global_alpha')

# GL/glext.h:5401
glGlobalAlphaFactoruiSUN = _link_function('glGlobalAlphaFactoruiSUN', None, [GLuint], 'SUN_global_alpha')

PFNGLGLOBALALPHAFACTORBSUNPROC = CFUNCTYPE(None, GLbyte) 	# GL/glext.h:5403
PFNGLGLOBALALPHAFACTORSSUNPROC = CFUNCTYPE(None, GLshort) 	# GL/glext.h:5404
PFNGLGLOBALALPHAFACTORISUNPROC = CFUNCTYPE(None, GLint) 	# GL/glext.h:5405
PFNGLGLOBALALPHAFACTORFSUNPROC = CFUNCTYPE(None, GLfloat) 	# GL/glext.h:5406
PFNGLGLOBALALPHAFACTORDSUNPROC = CFUNCTYPE(None, GLdouble) 	# GL/glext.h:5407
PFNGLGLOBALALPHAFACTORUBSUNPROC = CFUNCTYPE(None, GLubyte) 	# GL/glext.h:5408
PFNGLGLOBALALPHAFACTORUSSUNPROC = CFUNCTYPE(None, GLushort) 	# GL/glext.h:5409
PFNGLGLOBALALPHAFACTORUISUNPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:5410
# SUN_triangle_list (GL/glext.h:5413)
GL_SUN_triangle_list = 1 	# GL/glext.h:5414
# GL/glext.h:5416
glReplacementCodeuiSUN = _link_function('glReplacementCodeuiSUN', None, [GLuint], 'SUN_triangle_list')

# GL/glext.h:5417
glReplacementCodeusSUN = _link_function('glReplacementCodeusSUN', None, [GLushort], 'SUN_triangle_list')

# GL/glext.h:5418
glReplacementCodeubSUN = _link_function('glReplacementCodeubSUN', None, [GLubyte], 'SUN_triangle_list')

# GL/glext.h:5419
glReplacementCodeuivSUN = _link_function('glReplacementCodeuivSUN', None, [POINTER(GLuint)], 'SUN_triangle_list')

# GL/glext.h:5420
glReplacementCodeusvSUN = _link_function('glReplacementCodeusvSUN', None, [POINTER(GLushort)], 'SUN_triangle_list')

# GL/glext.h:5421
glReplacementCodeubvSUN = _link_function('glReplacementCodeubvSUN', None, [POINTER(GLubyte)], 'SUN_triangle_list')

# GL/glext.h:5422
glReplacementCodePointerSUN = _link_function('glReplacementCodePointerSUN', None, [GLenum, GLsizei, POINTER(POINTER(GLvoid))], 'SUN_triangle_list')

PFNGLREPLACEMENTCODEUISUNPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:5424
PFNGLREPLACEMENTCODEUSSUNPROC = CFUNCTYPE(None, GLushort) 	# GL/glext.h:5425
PFNGLREPLACEMENTCODEUBSUNPROC = CFUNCTYPE(None, GLubyte) 	# GL/glext.h:5426
PFNGLREPLACEMENTCODEUIVSUNPROC = CFUNCTYPE(None, POINTER(GLuint)) 	# GL/glext.h:5427
PFNGLREPLACEMENTCODEUSVSUNPROC = CFUNCTYPE(None, POINTER(GLushort)) 	# GL/glext.h:5428
PFNGLREPLACEMENTCODEUBVSUNPROC = CFUNCTYPE(None, POINTER(GLubyte)) 	# GL/glext.h:5429
PFNGLREPLACEMENTCODEPOINTERSUNPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(POINTER(GLvoid))) 	# GL/glext.h:5430
# SUN_vertex (GL/glext.h:5433)
GL_SUN_vertex = 1 	# GL/glext.h:5434
# GL/glext.h:5436
glColor4ubVertex2fSUN = _link_function('glColor4ubVertex2fSUN', None, [GLubyte, GLubyte, GLubyte, GLubyte, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5437
glColor4ubVertex2fvSUN = _link_function('glColor4ubVertex2fvSUN', None, [POINTER(GLubyte), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5438
glColor4ubVertex3fSUN = _link_function('glColor4ubVertex3fSUN', None, [GLubyte, GLubyte, GLubyte, GLubyte, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5439
glColor4ubVertex3fvSUN = _link_function('glColor4ubVertex3fvSUN', None, [POINTER(GLubyte), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5440
glColor3fVertex3fSUN = _link_function('glColor3fVertex3fSUN', None, [GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5441
glColor3fVertex3fvSUN = _link_function('glColor3fVertex3fvSUN', None, [POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5442
glNormal3fVertex3fSUN = _link_function('glNormal3fVertex3fSUN', None, [GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5443
glNormal3fVertex3fvSUN = _link_function('glNormal3fVertex3fvSUN', None, [POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5444
glColor4fNormal3fVertex3fSUN = _link_function('glColor4fNormal3fVertex3fSUN', None, [GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5445
glColor4fNormal3fVertex3fvSUN = _link_function('glColor4fNormal3fVertex3fvSUN', None, [POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5446
glTexCoord2fVertex3fSUN = _link_function('glTexCoord2fVertex3fSUN', None, [GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5447
glTexCoord2fVertex3fvSUN = _link_function('glTexCoord2fVertex3fvSUN', None, [POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5448
glTexCoord4fVertex4fSUN = _link_function('glTexCoord4fVertex4fSUN', None, [GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5449
glTexCoord4fVertex4fvSUN = _link_function('glTexCoord4fVertex4fvSUN', None, [POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5450
glTexCoord2fColor4ubVertex3fSUN = _link_function('glTexCoord2fColor4ubVertex3fSUN', None, [GLfloat, GLfloat, GLubyte, GLubyte, GLubyte, GLubyte, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5451
glTexCoord2fColor4ubVertex3fvSUN = _link_function('glTexCoord2fColor4ubVertex3fvSUN', None, [POINTER(GLfloat), POINTER(GLubyte), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5452
glTexCoord2fColor3fVertex3fSUN = _link_function('glTexCoord2fColor3fVertex3fSUN', None, [GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5453
glTexCoord2fColor3fVertex3fvSUN = _link_function('glTexCoord2fColor3fVertex3fvSUN', None, [POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5454
glTexCoord2fNormal3fVertex3fSUN = _link_function('glTexCoord2fNormal3fVertex3fSUN', None, [GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5455
glTexCoord2fNormal3fVertex3fvSUN = _link_function('glTexCoord2fNormal3fVertex3fvSUN', None, [POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5456
glTexCoord2fColor4fNormal3fVertex3fSUN = _link_function('glTexCoord2fColor4fNormal3fVertex3fSUN', None, [GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5457
glTexCoord2fColor4fNormal3fVertex3fvSUN = _link_function('glTexCoord2fColor4fNormal3fVertex3fvSUN', None, [POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5458
glTexCoord4fColor4fNormal3fVertex4fSUN = _link_function('glTexCoord4fColor4fNormal3fVertex4fSUN', None, [GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5459
glTexCoord4fColor4fNormal3fVertex4fvSUN = _link_function('glTexCoord4fColor4fNormal3fVertex4fvSUN', None, [POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5460
glReplacementCodeuiVertex3fSUN = _link_function('glReplacementCodeuiVertex3fSUN', None, [GLuint, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5461
glReplacementCodeuiVertex3fvSUN = _link_function('glReplacementCodeuiVertex3fvSUN', None, [POINTER(GLuint), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5462
glReplacementCodeuiColor4ubVertex3fSUN = _link_function('glReplacementCodeuiColor4ubVertex3fSUN', None, [GLuint, GLubyte, GLubyte, GLubyte, GLubyte, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5463
glReplacementCodeuiColor4ubVertex3fvSUN = _link_function('glReplacementCodeuiColor4ubVertex3fvSUN', None, [POINTER(GLuint), POINTER(GLubyte), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5464
glReplacementCodeuiColor3fVertex3fSUN = _link_function('glReplacementCodeuiColor3fVertex3fSUN', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5465
glReplacementCodeuiColor3fVertex3fvSUN = _link_function('glReplacementCodeuiColor3fVertex3fvSUN', None, [POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5466
glReplacementCodeuiNormal3fVertex3fSUN = _link_function('glReplacementCodeuiNormal3fVertex3fSUN', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5467
glReplacementCodeuiNormal3fVertex3fvSUN = _link_function('glReplacementCodeuiNormal3fVertex3fvSUN', None, [POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5468
glReplacementCodeuiColor4fNormal3fVertex3fSUN = _link_function('glReplacementCodeuiColor4fNormal3fVertex3fSUN', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5469
glReplacementCodeuiColor4fNormal3fVertex3fvSUN = _link_function('glReplacementCodeuiColor4fNormal3fVertex3fvSUN', None, [POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5470
glReplacementCodeuiTexCoord2fVertex3fSUN = _link_function('glReplacementCodeuiTexCoord2fVertex3fSUN', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5471
glReplacementCodeuiTexCoord2fVertex3fvSUN = _link_function('glReplacementCodeuiTexCoord2fVertex3fvSUN', None, [POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5472
glReplacementCodeuiTexCoord2fNormal3fVertex3fSUN = _link_function('glReplacementCodeuiTexCoord2fNormal3fVertex3fSUN', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5473
glReplacementCodeuiTexCoord2fNormal3fVertex3fvSUN = _link_function('glReplacementCodeuiTexCoord2fNormal3fVertex3fvSUN', None, [POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:5474
glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fSUN = _link_function('glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fSUN', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:5475
glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fvSUN = _link_function('glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fvSUN', None, [POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

PFNGLCOLOR4UBVERTEX2FSUNPROC = CFUNCTYPE(None, GLubyte, GLubyte, GLubyte, GLubyte, GLfloat, GLfloat) 	# GL/glext.h:5477
PFNGLCOLOR4UBVERTEX2FVSUNPROC = CFUNCTYPE(None, POINTER(GLubyte), POINTER(GLfloat)) 	# GL/glext.h:5478
PFNGLCOLOR4UBVERTEX3FSUNPROC = CFUNCTYPE(None, GLubyte, GLubyte, GLubyte, GLubyte, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5479
PFNGLCOLOR4UBVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLubyte), POINTER(GLfloat)) 	# GL/glext.h:5480
PFNGLCOLOR3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5481
PFNGLCOLOR3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:5482
PFNGLNORMAL3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5483
PFNGLNORMAL3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:5484
PFNGLCOLOR4FNORMAL3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5485
PFNGLCOLOR4FNORMAL3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:5486
PFNGLTEXCOORD2FVERTEX3FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5487
PFNGLTEXCOORD2FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:5488
PFNGLTEXCOORD4FVERTEX4FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5489
PFNGLTEXCOORD4FVERTEX4FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:5490
PFNGLTEXCOORD2FCOLOR4UBVERTEX3FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLubyte, GLubyte, GLubyte, GLubyte, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5491
PFNGLTEXCOORD2FCOLOR4UBVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLubyte), POINTER(GLfloat)) 	# GL/glext.h:5492
PFNGLTEXCOORD2FCOLOR3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5493
PFNGLTEXCOORD2FCOLOR3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:5494
PFNGLTEXCOORD2FNORMAL3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5495
PFNGLTEXCOORD2FNORMAL3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:5496
PFNGLTEXCOORD2FCOLOR4FNORMAL3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5497
PFNGLTEXCOORD2FCOLOR4FNORMAL3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:5498
PFNGLTEXCOORD4FCOLOR4FNORMAL3FVERTEX4FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5499
PFNGLTEXCOORD4FCOLOR4FNORMAL3FVERTEX4FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:5500
PFNGLREPLACEMENTCODEUIVERTEX3FSUNPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5501
PFNGLREPLACEMENTCODEUIVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLuint), POINTER(GLfloat)) 	# GL/glext.h:5502
PFNGLREPLACEMENTCODEUICOLOR4UBVERTEX3FSUNPROC = CFUNCTYPE(None, GLuint, GLubyte, GLubyte, GLubyte, GLubyte, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5503
PFNGLREPLACEMENTCODEUICOLOR4UBVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLuint), POINTER(GLubyte), POINTER(GLfloat)) 	# GL/glext.h:5504
PFNGLREPLACEMENTCODEUICOLOR3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5505
PFNGLREPLACEMENTCODEUICOLOR3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:5506
PFNGLREPLACEMENTCODEUINORMAL3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5507
PFNGLREPLACEMENTCODEUINORMAL3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:5508
PFNGLREPLACEMENTCODEUICOLOR4FNORMAL3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5509
PFNGLREPLACEMENTCODEUICOLOR4FNORMAL3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:5510
PFNGLREPLACEMENTCODEUITEXCOORD2FVERTEX3FSUNPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5511
PFNGLREPLACEMENTCODEUITEXCOORD2FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:5512
PFNGLREPLACEMENTCODEUITEXCOORD2FNORMAL3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5513
PFNGLREPLACEMENTCODEUITEXCOORD2FNORMAL3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:5514
PFNGLREPLACEMENTCODEUITEXCOORD2FCOLOR4FNORMAL3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5515
PFNGLREPLACEMENTCODEUITEXCOORD2FCOLOR4FNORMAL3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:5516
# EXT_blend_func_separate (GL/glext.h:5519)
GL_EXT_blend_func_separate = 1 	# GL/glext.h:5520
# GL/glext.h:5522
glBlendFuncSeparateEXT = _link_function('glBlendFuncSeparateEXT', None, [GLenum, GLenum, GLenum, GLenum], 'EXT_blend_func_separate')

PFNGLBLENDFUNCSEPARATEEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:5524
# INGR_blend_func_separate (GL/glext.h:5527)
GL_INGR_blend_func_separate = 1 	# GL/glext.h:5528
# GL/glext.h:5530
glBlendFuncSeparateINGR = _link_function('glBlendFuncSeparateINGR', None, [GLenum, GLenum, GLenum, GLenum], 'INGR_blend_func_separate')

PFNGLBLENDFUNCSEPARATEINGRPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:5532
# INGR_color_clamp (GL/glext.h:5535)
GL_INGR_color_clamp = 1 	# GL/glext.h:5536
# INGR_interlace_read (GL/glext.h:5539)
GL_INGR_interlace_read = 1 	# GL/glext.h:5540
# EXT_stencil_wrap (GL/glext.h:5543)
GL_EXT_stencil_wrap = 1 	# GL/glext.h:5544
# EXT_422_pixels (GL/glext.h:5547)
GL_EXT_422_pixels = 1 	# GL/glext.h:5548
# NV_texgen_reflection (GL/glext.h:5551)
GL_NV_texgen_reflection = 1 	# GL/glext.h:5552
# SUN_convolution_border_modes (GL/glext.h:5555)
GL_SUN_convolution_border_modes = 1 	# GL/glext.h:5556
# EXT_texture_env_add (GL/glext.h:5559)
GL_EXT_texture_env_add = 1 	# GL/glext.h:5560
# EXT_texture_lod_bias (GL/glext.h:5563)
GL_EXT_texture_lod_bias = 1 	# GL/glext.h:5564
# EXT_texture_filter_anisotropic (GL/glext.h:5567)
GL_EXT_texture_filter_anisotropic = 1 	# GL/glext.h:5568
# EXT_vertex_weighting (GL/glext.h:5571)
GL_EXT_vertex_weighting = 1 	# GL/glext.h:5572
# GL/glext.h:5574
glVertexWeightfEXT = _link_function('glVertexWeightfEXT', None, [GLfloat], 'EXT_vertex_weighting')

# GL/glext.h:5575
glVertexWeightfvEXT = _link_function('glVertexWeightfvEXT', None, [POINTER(GLfloat)], 'EXT_vertex_weighting')

# GL/glext.h:5576
glVertexWeightPointerEXT = _link_function('glVertexWeightPointerEXT', None, [GLsizei, GLenum, GLsizei, POINTER(GLvoid)], 'EXT_vertex_weighting')

PFNGLVERTEXWEIGHTFEXTPROC = CFUNCTYPE(None, GLfloat) 	# GL/glext.h:5578
PFNGLVERTEXWEIGHTFVEXTPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:5579
PFNGLVERTEXWEIGHTPOINTEREXTPROC = CFUNCTYPE(None, GLsizei, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:5580
# NV_light_max_exponent (GL/glext.h:5583)
GL_NV_light_max_exponent = 1 	# GL/glext.h:5584
# NV_vertex_array_range (GL/glext.h:5587)
GL_NV_vertex_array_range = 1 	# GL/glext.h:5588
# GL/glext.h:5590
glFlushVertexArrayRangeNV = _link_function('glFlushVertexArrayRangeNV', None, [], 'NV_vertex_array_range')

# GL/glext.h:5591
glVertexArrayRangeNV = _link_function('glVertexArrayRangeNV', None, [GLsizei, POINTER(GLvoid)], 'NV_vertex_array_range')

PFNGLFLUSHVERTEXARRAYRANGENVPROC = CFUNCTYPE(None) 	# GL/glext.h:5593
PFNGLVERTEXARRAYRANGENVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:5594
# NV_register_combiners (GL/glext.h:5597)
GL_NV_register_combiners = 1 	# GL/glext.h:5598
# GL/glext.h:5600
glCombinerParameterfvNV = _link_function('glCombinerParameterfvNV', None, [GLenum, POINTER(GLfloat)], 'NV_register_combiners')

# GL/glext.h:5601
glCombinerParameterfNV = _link_function('glCombinerParameterfNV', None, [GLenum, GLfloat], 'NV_register_combiners')

# GL/glext.h:5602
glCombinerParameterivNV = _link_function('glCombinerParameterivNV', None, [GLenum, POINTER(GLint)], 'NV_register_combiners')

# GL/glext.h:5603
glCombinerParameteriNV = _link_function('glCombinerParameteriNV', None, [GLenum, GLint], 'NV_register_combiners')

# GL/glext.h:5604
glCombinerInputNV = _link_function('glCombinerInputNV', None, [GLenum, GLenum, GLenum, GLenum, GLenum, GLenum], 'NV_register_combiners')

# GL/glext.h:5605
glCombinerOutputNV = _link_function('glCombinerOutputNV', None, [GLenum, GLenum, GLenum, GLenum, GLenum, GLenum, GLenum, GLboolean, GLboolean, GLboolean], 'NV_register_combiners')

# GL/glext.h:5606
glFinalCombinerInputNV = _link_function('glFinalCombinerInputNV', None, [GLenum, GLenum, GLenum, GLenum], 'NV_register_combiners')

# GL/glext.h:5607
glGetCombinerInputParameterfvNV = _link_function('glGetCombinerInputParameterfvNV', None, [GLenum, GLenum, GLenum, GLenum, POINTER(GLfloat)], 'NV_register_combiners')

# GL/glext.h:5608
glGetCombinerInputParameterivNV = _link_function('glGetCombinerInputParameterivNV', None, [GLenum, GLenum, GLenum, GLenum, POINTER(GLint)], 'NV_register_combiners')

# GL/glext.h:5609
glGetCombinerOutputParameterfvNV = _link_function('glGetCombinerOutputParameterfvNV', None, [GLenum, GLenum, GLenum, POINTER(GLfloat)], 'NV_register_combiners')

# GL/glext.h:5610
glGetCombinerOutputParameterivNV = _link_function('glGetCombinerOutputParameterivNV', None, [GLenum, GLenum, GLenum, POINTER(GLint)], 'NV_register_combiners')

# GL/glext.h:5611
glGetFinalCombinerInputParameterfvNV = _link_function('glGetFinalCombinerInputParameterfvNV', None, [GLenum, GLenum, POINTER(GLfloat)], 'NV_register_combiners')

# GL/glext.h:5612
glGetFinalCombinerInputParameterivNV = _link_function('glGetFinalCombinerInputParameterivNV', None, [GLenum, GLenum, POINTER(GLint)], 'NV_register_combiners')

PFNGLCOMBINERPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5614
PFNGLCOMBINERPARAMETERFNVPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:5615
PFNGLCOMBINERPARAMETERIVNVPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:5616
PFNGLCOMBINERPARAMETERINVPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:5617
PFNGLCOMBINERINPUTNVPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:5618
PFNGLCOMBINEROUTPUTNVPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum, GLenum, GLenum, GLenum, GLboolean, GLboolean, GLboolean) 	# GL/glext.h:5619
PFNGLFINALCOMBINERINPUTNVPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:5620
PFNGLGETCOMBINERINPUTPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5621
PFNGLGETCOMBINERINPUTPARAMETERIVNVPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5622
PFNGLGETCOMBINEROUTPUTPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5623
PFNGLGETCOMBINEROUTPUTPARAMETERIVNVPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5624
PFNGLGETFINALCOMBINERINPUTPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5625
PFNGLGETFINALCOMBINERINPUTPARAMETERIVNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5626
# NV_fog_distance (GL/glext.h:5629)
GL_NV_fog_distance = 1 	# GL/glext.h:5630
# NV_texgen_emboss (GL/glext.h:5633)
GL_NV_texgen_emboss = 1 	# GL/glext.h:5634
# NV_blend_square (GL/glext.h:5637)
GL_NV_blend_square = 1 	# GL/glext.h:5638
# NV_texture_env_combine4 (GL/glext.h:5641)
GL_NV_texture_env_combine4 = 1 	# GL/glext.h:5642
# MESA_resize_buffers (GL/glext.h:5645)
GL_MESA_resize_buffers = 1 	# GL/glext.h:5646
# GL/glext.h:5648
glResizeBuffersMESA = _link_function('glResizeBuffersMESA', None, [], 'MESA_resize_buffers')

PFNGLRESIZEBUFFERSMESAPROC = CFUNCTYPE(None) 	# GL/glext.h:5650
# MESA_window_pos (GL/glext.h:5653)
GL_MESA_window_pos = 1 	# GL/glext.h:5654
# GL/glext.h:5656
glWindowPos2dMESA = _link_function('glWindowPos2dMESA', None, [GLdouble, GLdouble], 'MESA_window_pos')

# GL/glext.h:5657
glWindowPos2dvMESA = _link_function('glWindowPos2dvMESA', None, [POINTER(GLdouble)], 'MESA_window_pos')

# GL/glext.h:5658
glWindowPos2fMESA = _link_function('glWindowPos2fMESA', None, [GLfloat, GLfloat], 'MESA_window_pos')

# GL/glext.h:5659
glWindowPos2fvMESA = _link_function('glWindowPos2fvMESA', None, [POINTER(GLfloat)], 'MESA_window_pos')

# GL/glext.h:5660
glWindowPos2iMESA = _link_function('glWindowPos2iMESA', None, [GLint, GLint], 'MESA_window_pos')

# GL/glext.h:5661
glWindowPos2ivMESA = _link_function('glWindowPos2ivMESA', None, [POINTER(GLint)], 'MESA_window_pos')

# GL/glext.h:5662
glWindowPos2sMESA = _link_function('glWindowPos2sMESA', None, [GLshort, GLshort], 'MESA_window_pos')

# GL/glext.h:5663
glWindowPos2svMESA = _link_function('glWindowPos2svMESA', None, [POINTER(GLshort)], 'MESA_window_pos')

# GL/glext.h:5664
glWindowPos3dMESA = _link_function('glWindowPos3dMESA', None, [GLdouble, GLdouble, GLdouble], 'MESA_window_pos')

# GL/glext.h:5665
glWindowPos3dvMESA = _link_function('glWindowPos3dvMESA', None, [POINTER(GLdouble)], 'MESA_window_pos')

# GL/glext.h:5666
glWindowPos3fMESA = _link_function('glWindowPos3fMESA', None, [GLfloat, GLfloat, GLfloat], 'MESA_window_pos')

# GL/glext.h:5667
glWindowPos3fvMESA = _link_function('glWindowPos3fvMESA', None, [POINTER(GLfloat)], 'MESA_window_pos')

# GL/glext.h:5668
glWindowPos3iMESA = _link_function('glWindowPos3iMESA', None, [GLint, GLint, GLint], 'MESA_window_pos')

# GL/glext.h:5669
glWindowPos3ivMESA = _link_function('glWindowPos3ivMESA', None, [POINTER(GLint)], 'MESA_window_pos')

# GL/glext.h:5670
glWindowPos3sMESA = _link_function('glWindowPos3sMESA', None, [GLshort, GLshort, GLshort], 'MESA_window_pos')

# GL/glext.h:5671
glWindowPos3svMESA = _link_function('glWindowPos3svMESA', None, [POINTER(GLshort)], 'MESA_window_pos')

# GL/glext.h:5672
glWindowPos4dMESA = _link_function('glWindowPos4dMESA', None, [GLdouble, GLdouble, GLdouble, GLdouble], 'MESA_window_pos')

# GL/glext.h:5673
glWindowPos4dvMESA = _link_function('glWindowPos4dvMESA', None, [POINTER(GLdouble)], 'MESA_window_pos')

# GL/glext.h:5674
glWindowPos4fMESA = _link_function('glWindowPos4fMESA', None, [GLfloat, GLfloat, GLfloat, GLfloat], 'MESA_window_pos')

# GL/glext.h:5675
glWindowPos4fvMESA = _link_function('glWindowPos4fvMESA', None, [POINTER(GLfloat)], 'MESA_window_pos')

# GL/glext.h:5676
glWindowPos4iMESA = _link_function('glWindowPos4iMESA', None, [GLint, GLint, GLint, GLint], 'MESA_window_pos')

# GL/glext.h:5677
glWindowPos4ivMESA = _link_function('glWindowPos4ivMESA', None, [POINTER(GLint)], 'MESA_window_pos')

# GL/glext.h:5678
glWindowPos4sMESA = _link_function('glWindowPos4sMESA', None, [GLshort, GLshort, GLshort, GLshort], 'MESA_window_pos')

# GL/glext.h:5679
glWindowPos4svMESA = _link_function('glWindowPos4svMESA', None, [POINTER(GLshort)], 'MESA_window_pos')

PFNGLWINDOWPOS2DMESAPROC = CFUNCTYPE(None, GLdouble, GLdouble) 	# GL/glext.h:5681
PFNGLWINDOWPOS2DVMESAPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:5682
PFNGLWINDOWPOS2FMESAPROC = CFUNCTYPE(None, GLfloat, GLfloat) 	# GL/glext.h:5683
PFNGLWINDOWPOS2FVMESAPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:5684
PFNGLWINDOWPOS2IMESAPROC = CFUNCTYPE(None, GLint, GLint) 	# GL/glext.h:5685
PFNGLWINDOWPOS2IVMESAPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:5686
PFNGLWINDOWPOS2SMESAPROC = CFUNCTYPE(None, GLshort, GLshort) 	# GL/glext.h:5687
PFNGLWINDOWPOS2SVMESAPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:5688
PFNGLWINDOWPOS3DMESAPROC = CFUNCTYPE(None, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:5689
PFNGLWINDOWPOS3DVMESAPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:5690
PFNGLWINDOWPOS3FMESAPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5691
PFNGLWINDOWPOS3FVMESAPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:5692
PFNGLWINDOWPOS3IMESAPROC = CFUNCTYPE(None, GLint, GLint, GLint) 	# GL/glext.h:5693
PFNGLWINDOWPOS3IVMESAPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:5694
PFNGLWINDOWPOS3SMESAPROC = CFUNCTYPE(None, GLshort, GLshort, GLshort) 	# GL/glext.h:5695
PFNGLWINDOWPOS3SVMESAPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:5696
PFNGLWINDOWPOS4DMESAPROC = CFUNCTYPE(None, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:5697
PFNGLWINDOWPOS4DVMESAPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:5698
PFNGLWINDOWPOS4FMESAPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5699
PFNGLWINDOWPOS4FVMESAPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:5700
PFNGLWINDOWPOS4IMESAPROC = CFUNCTYPE(None, GLint, GLint, GLint, GLint) 	# GL/glext.h:5701
PFNGLWINDOWPOS4IVMESAPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:5702
PFNGLWINDOWPOS4SMESAPROC = CFUNCTYPE(None, GLshort, GLshort, GLshort, GLshort) 	# GL/glext.h:5703
PFNGLWINDOWPOS4SVMESAPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:5704
# EXT_texture_compression_s3tc (GL/glext.h:5707)
GL_EXT_texture_compression_s3tc = 1 	# GL/glext.h:5708
# IBM_cull_vertex (GL/glext.h:5711)
GL_IBM_cull_vertex = 1 	# GL/glext.h:5712
# IBM_multimode_draw_arrays (GL/glext.h:5715)
GL_IBM_multimode_draw_arrays = 1 	# GL/glext.h:5716
# GL/glext.h:5718
glMultiModeDrawArraysIBM = _link_function('glMultiModeDrawArraysIBM', None, [POINTER(GLenum), POINTER(GLint), POINTER(GLsizei), GLsizei, GLint], 'IBM_multimode_draw_arrays')

# GL/glext.h:5719
glMultiModeDrawElementsIBM = _link_function('glMultiModeDrawElementsIBM', None, [POINTER(GLenum), POINTER(GLsizei), GLenum, POINTER(POINTER(GLvoid)), GLsizei, GLint], 'IBM_multimode_draw_arrays')

PFNGLMULTIMODEDRAWARRAYSIBMPROC = CFUNCTYPE(None, POINTER(GLenum), POINTER(GLint), POINTER(GLsizei), GLsizei, GLint) 	# GL/glext.h:5721
PFNGLMULTIMODEDRAWELEMENTSIBMPROC = CFUNCTYPE(None, POINTER(GLenum), POINTER(GLsizei), GLenum, POINTER(POINTER(GLvoid)), GLsizei, GLint) 	# GL/glext.h:5722
# IBM_vertex_array_lists (GL/glext.h:5725)
GL_IBM_vertex_array_lists = 1 	# GL/glext.h:5726
# GL/glext.h:5728
glColorPointerListIBM = _link_function('glColorPointerListIBM', None, [GLint, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint], 'IBM_vertex_array_lists')

# GL/glext.h:5729
glSecondaryColorPointerListIBM = _link_function('glSecondaryColorPointerListIBM', None, [GLint, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint], 'IBM_vertex_array_lists')

# GL/glext.h:5730
glEdgeFlagPointerListIBM = _link_function('glEdgeFlagPointerListIBM', None, [GLint, POINTER(POINTER(GLboolean)), GLint], 'IBM_vertex_array_lists')

# GL/glext.h:5731
glFogCoordPointerListIBM = _link_function('glFogCoordPointerListIBM', None, [GLenum, GLint, POINTER(POINTER(GLvoid)), GLint], 'IBM_vertex_array_lists')

# GL/glext.h:5732
glIndexPointerListIBM = _link_function('glIndexPointerListIBM', None, [GLenum, GLint, POINTER(POINTER(GLvoid)), GLint], 'IBM_vertex_array_lists')

# GL/glext.h:5733
glNormalPointerListIBM = _link_function('glNormalPointerListIBM', None, [GLenum, GLint, POINTER(POINTER(GLvoid)), GLint], 'IBM_vertex_array_lists')

# GL/glext.h:5734
glTexCoordPointerListIBM = _link_function('glTexCoordPointerListIBM', None, [GLint, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint], 'IBM_vertex_array_lists')

# GL/glext.h:5735
glVertexPointerListIBM = _link_function('glVertexPointerListIBM', None, [GLint, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint], 'IBM_vertex_array_lists')

PFNGLCOLORPOINTERLISTIBMPROC = CFUNCTYPE(None, GLint, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint) 	# GL/glext.h:5737
PFNGLSECONDARYCOLORPOINTERLISTIBMPROC = CFUNCTYPE(None, GLint, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint) 	# GL/glext.h:5738
PFNGLEDGEFLAGPOINTERLISTIBMPROC = CFUNCTYPE(None, GLint, POINTER(POINTER(GLboolean)), GLint) 	# GL/glext.h:5739
PFNGLFOGCOORDPOINTERLISTIBMPROC = CFUNCTYPE(None, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint) 	# GL/glext.h:5740
PFNGLINDEXPOINTERLISTIBMPROC = CFUNCTYPE(None, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint) 	# GL/glext.h:5741
PFNGLNORMALPOINTERLISTIBMPROC = CFUNCTYPE(None, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint) 	# GL/glext.h:5742
PFNGLTEXCOORDPOINTERLISTIBMPROC = CFUNCTYPE(None, GLint, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint) 	# GL/glext.h:5743
PFNGLVERTEXPOINTERLISTIBMPROC = CFUNCTYPE(None, GLint, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint) 	# GL/glext.h:5744
# SGIX_subsample (GL/glext.h:5747)
GL_SGIX_subsample = 1 	# GL/glext.h:5748
# SGIX_ycrcba (GL/glext.h:5751)
GL_SGIX_ycrcba = 1 	# GL/glext.h:5752
# SGIX_ycrcb_subsample (GL/glext.h:5755)
GL_SGIX_ycrcb_subsample = 1 	# GL/glext.h:5756
# SGIX_depth_pass_instrument (GL/glext.h:5759)
GL_SGIX_depth_pass_instrument = 1 	# GL/glext.h:5760
# 3DFX_texture_compression_FXT1 (GL/glext.h:5763)
GL_3DFX_texture_compression_FXT1 = 1 	# GL/glext.h:5764
# 3DFX_multisample (GL/glext.h:5767)
GL_3DFX_multisample = 1 	# GL/glext.h:5768
# 3DFX_tbuffer (GL/glext.h:5771)
GL_3DFX_tbuffer = 1 	# GL/glext.h:5772
# GL/glext.h:5774
glTbufferMask3DFX = _link_function('glTbufferMask3DFX', None, [GLuint], '3DFX_tbuffer')

PFNGLTBUFFERMASK3DFXPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:5776
# EXT_multisample (GL/glext.h:5779)
GL_EXT_multisample = 1 	# GL/glext.h:5780
# GL/glext.h:5782
glSampleMaskEXT = _link_function('glSampleMaskEXT', None, [GLclampf, GLboolean], 'EXT_multisample')

# GL/glext.h:5783
glSamplePatternEXT = _link_function('glSamplePatternEXT', None, [GLenum], 'EXT_multisample')

PFNGLSAMPLEMASKEXTPROC = CFUNCTYPE(None, GLclampf, GLboolean) 	# GL/glext.h:5785
PFNGLSAMPLEPATTERNEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:5786
# SGIX_vertex_preclip (GL/glext.h:5789)
GL_SGIX_vertex_preclip = 1 	# GL/glext.h:5790
# SGIX_convolution_accuracy (GL/glext.h:5793)
GL_SGIX_convolution_accuracy = 1 	# GL/glext.h:5794
# SGIX_resample (GL/glext.h:5797)
GL_SGIX_resample = 1 	# GL/glext.h:5798
# SGIS_point_line_texgen (GL/glext.h:5801)
GL_SGIS_point_line_texgen = 1 	# GL/glext.h:5802
# SGIS_texture_color_mask (GL/glext.h:5805)
GL_SGIS_texture_color_mask = 1 	# GL/glext.h:5806
# GL/glext.h:5808
glTextureColorMaskSGIS = _link_function('glTextureColorMaskSGIS', None, [GLboolean, GLboolean, GLboolean, GLboolean], 'SGIS_texture_color_mask')

PFNGLTEXTURECOLORMASKSGISPROC = CFUNCTYPE(None, GLboolean, GLboolean, GLboolean, GLboolean) 	# GL/glext.h:5810
# SGIX_igloo_interface (GL/glext.h:5813)
GL_SGIX_igloo_interface = 1 	# GL/glext.h:5814
# GL/glext.h:5816
glIglooInterfaceSGIX = _link_function('glIglooInterfaceSGIX', None, [GLenum, POINTER(GLvoid)], 'SGIX_igloo_interface')

PFNGLIGLOOINTERFACESGIXPROC = CFUNCTYPE(None, GLenum, POINTER(GLvoid)) 	# GL/glext.h:5818
# EXT_texture_env_dot3 (GL/glext.h:5821)
GL_EXT_texture_env_dot3 = 1 	# GL/glext.h:5822
# ATI_texture_mirror_once (GL/glext.h:5825)
GL_ATI_texture_mirror_once = 1 	# GL/glext.h:5826
# NV_fence (GL/glext.h:5829)
GL_NV_fence = 1 	# GL/glext.h:5830
# GL/glext.h:5832
glDeleteFencesNV = _link_function('glDeleteFencesNV', None, [GLsizei, POINTER(GLuint)], 'NV_fence')

# GL/glext.h:5833
glGenFencesNV = _link_function('glGenFencesNV', None, [GLsizei, POINTER(GLuint)], 'NV_fence')

# GL/glext.h:5834
glIsFenceNV = _link_function('glIsFenceNV', GLboolean, [GLuint], 'NV_fence')

# GL/glext.h:5835
glTestFenceNV = _link_function('glTestFenceNV', GLboolean, [GLuint], 'NV_fence')

# GL/glext.h:5836
glGetFenceivNV = _link_function('glGetFenceivNV', None, [GLuint, GLenum, POINTER(GLint)], 'NV_fence')

# GL/glext.h:5837
glFinishFenceNV = _link_function('glFinishFenceNV', None, [GLuint], 'NV_fence')

# GL/glext.h:5838
glSetFenceNV = _link_function('glSetFenceNV', None, [GLuint, GLenum], 'NV_fence')

PFNGLDELETEFENCESNVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:5840
PFNGLGENFENCESNVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:5841
PFNGLISFENCENVPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:5842
PFNGLTESTFENCENVPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:5843
PFNGLGETFENCEIVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:5844
PFNGLFINISHFENCENVPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:5845
PFNGLSETFENCENVPROC = CFUNCTYPE(None, GLuint, GLenum) 	# GL/glext.h:5846
# NV_evaluators (GL/glext.h:5849)
GL_NV_evaluators = 1 	# GL/glext.h:5850
# GL/glext.h:5852
glMapControlPointsNV = _link_function('glMapControlPointsNV', None, [GLenum, GLuint, GLenum, GLsizei, GLsizei, GLint, GLint, GLboolean, POINTER(GLvoid)], 'NV_evaluators')

# GL/glext.h:5853
glMapParameterivNV = _link_function('glMapParameterivNV', None, [GLenum, GLenum, POINTER(GLint)], 'NV_evaluators')

# GL/glext.h:5854
glMapParameterfvNV = _link_function('glMapParameterfvNV', None, [GLenum, GLenum, POINTER(GLfloat)], 'NV_evaluators')

# GL/glext.h:5855
glGetMapControlPointsNV = _link_function('glGetMapControlPointsNV', None, [GLenum, GLuint, GLenum, GLsizei, GLsizei, GLboolean, POINTER(GLvoid)], 'NV_evaluators')

# GL/glext.h:5856
glGetMapParameterivNV = _link_function('glGetMapParameterivNV', None, [GLenum, GLenum, POINTER(GLint)], 'NV_evaluators')

# GL/glext.h:5857
glGetMapParameterfvNV = _link_function('glGetMapParameterfvNV', None, [GLenum, GLenum, POINTER(GLfloat)], 'NV_evaluators')

# GL/glext.h:5858
glGetMapAttribParameterivNV = _link_function('glGetMapAttribParameterivNV', None, [GLenum, GLuint, GLenum, POINTER(GLint)], 'NV_evaluators')

# GL/glext.h:5859
glGetMapAttribParameterfvNV = _link_function('glGetMapAttribParameterfvNV', None, [GLenum, GLuint, GLenum, POINTER(GLfloat)], 'NV_evaluators')

# GL/glext.h:5860
glEvalMapsNV = _link_function('glEvalMapsNV', None, [GLenum, GLenum], 'NV_evaluators')

PFNGLMAPCONTROLPOINTSNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLenum, GLsizei, GLsizei, GLint, GLint, GLboolean, POINTER(GLvoid)) 	# GL/glext.h:5862
PFNGLMAPPARAMETERIVNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5863
PFNGLMAPPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5864
PFNGLGETMAPCONTROLPOINTSNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLenum, GLsizei, GLsizei, GLboolean, POINTER(GLvoid)) 	# GL/glext.h:5865
PFNGLGETMAPPARAMETERIVNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5866
PFNGLGETMAPPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5867
PFNGLGETMAPATTRIBPARAMETERIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:5868
PFNGLGETMAPATTRIBPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5869
PFNGLEVALMAPSNVPROC = CFUNCTYPE(None, GLenum, GLenum) 	# GL/glext.h:5870
# NV_packed_depth_stencil (GL/glext.h:5873)
GL_NV_packed_depth_stencil = 1 	# GL/glext.h:5874
# NV_register_combiners2 (GL/glext.h:5877)
GL_NV_register_combiners2 = 1 	# GL/glext.h:5878
# GL/glext.h:5880
glCombinerStageParameterfvNV = _link_function('glCombinerStageParameterfvNV', None, [GLenum, GLenum, POINTER(GLfloat)], 'NV_register_combiners2')

# GL/glext.h:5881
glGetCombinerStageParameterfvNV = _link_function('glGetCombinerStageParameterfvNV', None, [GLenum, GLenum, POINTER(GLfloat)], 'NV_register_combiners2')

PFNGLCOMBINERSTAGEPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5883
PFNGLGETCOMBINERSTAGEPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5884
# NV_texture_compression_vtc (GL/glext.h:5887)
GL_NV_texture_compression_vtc = 1 	# GL/glext.h:5888
# NV_texture_rectangle (GL/glext.h:5891)
GL_NV_texture_rectangle = 1 	# GL/glext.h:5892
# NV_texture_shader (GL/glext.h:5895)
GL_NV_texture_shader = 1 	# GL/glext.h:5896
# NV_texture_shader2 (GL/glext.h:5899)
GL_NV_texture_shader2 = 1 	# GL/glext.h:5900
# NV_vertex_array_range2 (GL/glext.h:5903)
GL_NV_vertex_array_range2 = 1 	# GL/glext.h:5904
# NV_vertex_program (GL/glext.h:5907)
GL_NV_vertex_program = 1 	# GL/glext.h:5908
# GL/glext.h:5910
glAreProgramsResidentNV = _link_function('glAreProgramsResidentNV', GLboolean, [GLsizei, POINTER(GLuint), POINTER(GLboolean)], 'NV_vertex_program')

# GL/glext.h:5911
glBindProgramNV = _link_function('glBindProgramNV', None, [GLenum, GLuint], 'NV_vertex_program')

# GL/glext.h:5912
glDeleteProgramsNV = _link_function('glDeleteProgramsNV', None, [GLsizei, POINTER(GLuint)], 'NV_vertex_program')

# GL/glext.h:5913
glExecuteProgramNV = _link_function('glExecuteProgramNV', None, [GLenum, GLuint, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:5914
glGenProgramsNV = _link_function('glGenProgramsNV', None, [GLsizei, POINTER(GLuint)], 'NV_vertex_program')

# GL/glext.h:5915
glGetProgramParameterdvNV = _link_function('glGetProgramParameterdvNV', None, [GLenum, GLuint, GLenum, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:5916
glGetProgramParameterfvNV = _link_function('glGetProgramParameterfvNV', None, [GLenum, GLuint, GLenum, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:5917
glGetProgramivNV = _link_function('glGetProgramivNV', None, [GLuint, GLenum, POINTER(GLint)], 'NV_vertex_program')

# GL/glext.h:5918
glGetProgramStringNV = _link_function('glGetProgramStringNV', None, [GLuint, GLenum, POINTER(GLubyte)], 'NV_vertex_program')

# GL/glext.h:5919
glGetTrackMatrixivNV = _link_function('glGetTrackMatrixivNV', None, [GLenum, GLuint, GLenum, POINTER(GLint)], 'NV_vertex_program')

# GL/glext.h:5920
glGetVertexAttribdvNV = _link_function('glGetVertexAttribdvNV', None, [GLuint, GLenum, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:5921
glGetVertexAttribfvNV = _link_function('glGetVertexAttribfvNV', None, [GLuint, GLenum, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:5922
glGetVertexAttribivNV = _link_function('glGetVertexAttribivNV', None, [GLuint, GLenum, POINTER(GLint)], 'NV_vertex_program')

# GL/glext.h:5923
glGetVertexAttribPointervNV = _link_function('glGetVertexAttribPointervNV', None, [GLuint, GLenum, POINTER(POINTER(GLvoid))], 'NV_vertex_program')

# GL/glext.h:5924
glIsProgramNV = _link_function('glIsProgramNV', GLboolean, [GLuint], 'NV_vertex_program')

# GL/glext.h:5925
glLoadProgramNV = _link_function('glLoadProgramNV', None, [GLenum, GLuint, GLsizei, POINTER(GLubyte)], 'NV_vertex_program')

# GL/glext.h:5926
glProgramParameter4dNV = _link_function('glProgramParameter4dNV', None, [GLenum, GLuint, GLdouble, GLdouble, GLdouble, GLdouble], 'NV_vertex_program')

# GL/glext.h:5927
glProgramParameter4dvNV = _link_function('glProgramParameter4dvNV', None, [GLenum, GLuint, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:5928
glProgramParameter4fNV = _link_function('glProgramParameter4fNV', None, [GLenum, GLuint, GLfloat, GLfloat, GLfloat, GLfloat], 'NV_vertex_program')

# GL/glext.h:5929
glProgramParameter4fvNV = _link_function('glProgramParameter4fvNV', None, [GLenum, GLuint, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:5930
glProgramParameters4dvNV = _link_function('glProgramParameters4dvNV', None, [GLenum, GLuint, GLuint, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:5931
glProgramParameters4fvNV = _link_function('glProgramParameters4fvNV', None, [GLenum, GLuint, GLuint, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:5932
glRequestResidentProgramsNV = _link_function('glRequestResidentProgramsNV', None, [GLsizei, POINTER(GLuint)], 'NV_vertex_program')

# GL/glext.h:5933
glTrackMatrixNV = _link_function('glTrackMatrixNV', None, [GLenum, GLuint, GLenum, GLenum], 'NV_vertex_program')

# GL/glext.h:5934
glVertexAttribPointerNV = _link_function('glVertexAttribPointerNV', None, [GLuint, GLint, GLenum, GLsizei, POINTER(GLvoid)], 'NV_vertex_program')

# GL/glext.h:5935
glVertexAttrib1dNV = _link_function('glVertexAttrib1dNV', None, [GLuint, GLdouble], 'NV_vertex_program')

# GL/glext.h:5936
glVertexAttrib1dvNV = _link_function('glVertexAttrib1dvNV', None, [GLuint, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:5937
glVertexAttrib1fNV = _link_function('glVertexAttrib1fNV', None, [GLuint, GLfloat], 'NV_vertex_program')

# GL/glext.h:5938
glVertexAttrib1fvNV = _link_function('glVertexAttrib1fvNV', None, [GLuint, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:5939
glVertexAttrib1sNV = _link_function('glVertexAttrib1sNV', None, [GLuint, GLshort], 'NV_vertex_program')

# GL/glext.h:5940
glVertexAttrib1svNV = _link_function('glVertexAttrib1svNV', None, [GLuint, POINTER(GLshort)], 'NV_vertex_program')

# GL/glext.h:5941
glVertexAttrib2dNV = _link_function('glVertexAttrib2dNV', None, [GLuint, GLdouble, GLdouble], 'NV_vertex_program')

# GL/glext.h:5942
glVertexAttrib2dvNV = _link_function('glVertexAttrib2dvNV', None, [GLuint, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:5943
glVertexAttrib2fNV = _link_function('glVertexAttrib2fNV', None, [GLuint, GLfloat, GLfloat], 'NV_vertex_program')

# GL/glext.h:5944
glVertexAttrib2fvNV = _link_function('glVertexAttrib2fvNV', None, [GLuint, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:5945
glVertexAttrib2sNV = _link_function('glVertexAttrib2sNV', None, [GLuint, GLshort, GLshort], 'NV_vertex_program')

# GL/glext.h:5946
glVertexAttrib2svNV = _link_function('glVertexAttrib2svNV', None, [GLuint, POINTER(GLshort)], 'NV_vertex_program')

# GL/glext.h:5947
glVertexAttrib3dNV = _link_function('glVertexAttrib3dNV', None, [GLuint, GLdouble, GLdouble, GLdouble], 'NV_vertex_program')

# GL/glext.h:5948
glVertexAttrib3dvNV = _link_function('glVertexAttrib3dvNV', None, [GLuint, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:5949
glVertexAttrib3fNV = _link_function('glVertexAttrib3fNV', None, [GLuint, GLfloat, GLfloat, GLfloat], 'NV_vertex_program')

# GL/glext.h:5950
glVertexAttrib3fvNV = _link_function('glVertexAttrib3fvNV', None, [GLuint, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:5951
glVertexAttrib3sNV = _link_function('glVertexAttrib3sNV', None, [GLuint, GLshort, GLshort, GLshort], 'NV_vertex_program')

# GL/glext.h:5952
glVertexAttrib3svNV = _link_function('glVertexAttrib3svNV', None, [GLuint, POINTER(GLshort)], 'NV_vertex_program')

# GL/glext.h:5953
glVertexAttrib4dNV = _link_function('glVertexAttrib4dNV', None, [GLuint, GLdouble, GLdouble, GLdouble, GLdouble], 'NV_vertex_program')

# GL/glext.h:5954
glVertexAttrib4dvNV = _link_function('glVertexAttrib4dvNV', None, [GLuint, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:5955
glVertexAttrib4fNV = _link_function('glVertexAttrib4fNV', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat], 'NV_vertex_program')

# GL/glext.h:5956
glVertexAttrib4fvNV = _link_function('glVertexAttrib4fvNV', None, [GLuint, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:5957
glVertexAttrib4sNV = _link_function('glVertexAttrib4sNV', None, [GLuint, GLshort, GLshort, GLshort, GLshort], 'NV_vertex_program')

# GL/glext.h:5958
glVertexAttrib4svNV = _link_function('glVertexAttrib4svNV', None, [GLuint, POINTER(GLshort)], 'NV_vertex_program')

# GL/glext.h:5959
glVertexAttrib4ubNV = _link_function('glVertexAttrib4ubNV', None, [GLuint, GLubyte, GLubyte, GLubyte, GLubyte], 'NV_vertex_program')

# GL/glext.h:5960
glVertexAttrib4ubvNV = _link_function('glVertexAttrib4ubvNV', None, [GLuint, POINTER(GLubyte)], 'NV_vertex_program')

# GL/glext.h:5961
glVertexAttribs1dvNV = _link_function('glVertexAttribs1dvNV', None, [GLuint, GLsizei, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:5962
glVertexAttribs1fvNV = _link_function('glVertexAttribs1fvNV', None, [GLuint, GLsizei, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:5963
glVertexAttribs1svNV = _link_function('glVertexAttribs1svNV', None, [GLuint, GLsizei, POINTER(GLshort)], 'NV_vertex_program')

# GL/glext.h:5964
glVertexAttribs2dvNV = _link_function('glVertexAttribs2dvNV', None, [GLuint, GLsizei, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:5965
glVertexAttribs2fvNV = _link_function('glVertexAttribs2fvNV', None, [GLuint, GLsizei, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:5966
glVertexAttribs2svNV = _link_function('glVertexAttribs2svNV', None, [GLuint, GLsizei, POINTER(GLshort)], 'NV_vertex_program')

# GL/glext.h:5967
glVertexAttribs3dvNV = _link_function('glVertexAttribs3dvNV', None, [GLuint, GLsizei, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:5968
glVertexAttribs3fvNV = _link_function('glVertexAttribs3fvNV', None, [GLuint, GLsizei, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:5969
glVertexAttribs3svNV = _link_function('glVertexAttribs3svNV', None, [GLuint, GLsizei, POINTER(GLshort)], 'NV_vertex_program')

# GL/glext.h:5970
glVertexAttribs4dvNV = _link_function('glVertexAttribs4dvNV', None, [GLuint, GLsizei, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:5971
glVertexAttribs4fvNV = _link_function('glVertexAttribs4fvNV', None, [GLuint, GLsizei, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:5972
glVertexAttribs4svNV = _link_function('glVertexAttribs4svNV', None, [GLuint, GLsizei, POINTER(GLshort)], 'NV_vertex_program')

# GL/glext.h:5973
glVertexAttribs4ubvNV = _link_function('glVertexAttribs4ubvNV', None, [GLuint, GLsizei, POINTER(GLubyte)], 'NV_vertex_program')

PFNGLAREPROGRAMSRESIDENTNVPROC = CFUNCTYPE(GLboolean, GLsizei, POINTER(GLuint), POINTER(GLboolean)) 	# GL/glext.h:5975
PFNGLBINDPROGRAMNVPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:5976
PFNGLDELETEPROGRAMSNVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:5977
PFNGLEXECUTEPROGRAMNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLfloat)) 	# GL/glext.h:5978
PFNGLGENPROGRAMSNVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:5979
PFNGLGETPROGRAMPARAMETERDVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLenum, POINTER(GLdouble)) 	# GL/glext.h:5980
PFNGLGETPROGRAMPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5981
PFNGLGETPROGRAMIVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:5982
PFNGLGETPROGRAMSTRINGNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLubyte)) 	# GL/glext.h:5983
PFNGLGETTRACKMATRIXIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:5984
PFNGLGETVERTEXATTRIBDVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLdouble)) 	# GL/glext.h:5985
PFNGLGETVERTEXATTRIBFVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5986
PFNGLGETVERTEXATTRIBIVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:5987
PFNGLGETVERTEXATTRIBPOINTERVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:5988
PFNGLISPROGRAMNVPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:5989
PFNGLLOADPROGRAMNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLsizei, POINTER(GLubyte)) 	# GL/glext.h:5990
PFNGLPROGRAMPARAMETER4DNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:5991
PFNGLPROGRAMPARAMETER4DVNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLdouble)) 	# GL/glext.h:5992
PFNGLPROGRAMPARAMETER4FNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5993
PFNGLPROGRAMPARAMETER4FVNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLfloat)) 	# GL/glext.h:5994
PFNGLPROGRAMPARAMETERS4DVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, POINTER(GLdouble)) 	# GL/glext.h:5995
PFNGLPROGRAMPARAMETERS4FVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, POINTER(GLfloat)) 	# GL/glext.h:5996
PFNGLREQUESTRESIDENTPROGRAMSNVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:5997
PFNGLTRACKMATRIXNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLenum, GLenum) 	# GL/glext.h:5998
PFNGLVERTEXATTRIBPOINTERNVPROC = CFUNCTYPE(None, GLuint, GLint, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:5999
PFNGLVERTEXATTRIB1DNVPROC = CFUNCTYPE(None, GLuint, GLdouble) 	# GL/glext.h:6000
PFNGLVERTEXATTRIB1DVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:6001
PFNGLVERTEXATTRIB1FNVPROC = CFUNCTYPE(None, GLuint, GLfloat) 	# GL/glext.h:6002
PFNGLVERTEXATTRIB1FVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:6003
PFNGLVERTEXATTRIB1SNVPROC = CFUNCTYPE(None, GLuint, GLshort) 	# GL/glext.h:6004
PFNGLVERTEXATTRIB1SVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:6005
PFNGLVERTEXATTRIB2DNVPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble) 	# GL/glext.h:6006
PFNGLVERTEXATTRIB2DVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:6007
PFNGLVERTEXATTRIB2FNVPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat) 	# GL/glext.h:6008
PFNGLVERTEXATTRIB2FVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:6009
PFNGLVERTEXATTRIB2SNVPROC = CFUNCTYPE(None, GLuint, GLshort, GLshort) 	# GL/glext.h:6010
PFNGLVERTEXATTRIB2SVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:6011
PFNGLVERTEXATTRIB3DNVPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:6012
PFNGLVERTEXATTRIB3DVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:6013
PFNGLVERTEXATTRIB3FNVPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6014
PFNGLVERTEXATTRIB3FVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:6015
PFNGLVERTEXATTRIB3SNVPROC = CFUNCTYPE(None, GLuint, GLshort, GLshort, GLshort) 	# GL/glext.h:6016
PFNGLVERTEXATTRIB3SVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:6017
PFNGLVERTEXATTRIB4DNVPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:6018
PFNGLVERTEXATTRIB4DVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:6019
PFNGLVERTEXATTRIB4FNVPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6020
PFNGLVERTEXATTRIB4FVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:6021
PFNGLVERTEXATTRIB4SNVPROC = CFUNCTYPE(None, GLuint, GLshort, GLshort, GLshort, GLshort) 	# GL/glext.h:6022
PFNGLVERTEXATTRIB4SVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:6023
PFNGLVERTEXATTRIB4UBNVPROC = CFUNCTYPE(None, GLuint, GLubyte, GLubyte, GLubyte, GLubyte) 	# GL/glext.h:6024
PFNGLVERTEXATTRIB4UBVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLubyte)) 	# GL/glext.h:6025
PFNGLVERTEXATTRIBS1DVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLdouble)) 	# GL/glext.h:6026
PFNGLVERTEXATTRIBS1FVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:6027
PFNGLVERTEXATTRIBS1SVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLshort)) 	# GL/glext.h:6028
PFNGLVERTEXATTRIBS2DVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLdouble)) 	# GL/glext.h:6029
PFNGLVERTEXATTRIBS2FVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:6030
PFNGLVERTEXATTRIBS2SVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLshort)) 	# GL/glext.h:6031
PFNGLVERTEXATTRIBS3DVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLdouble)) 	# GL/glext.h:6032
PFNGLVERTEXATTRIBS3FVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:6033
PFNGLVERTEXATTRIBS3SVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLshort)) 	# GL/glext.h:6034
PFNGLVERTEXATTRIBS4DVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLdouble)) 	# GL/glext.h:6035
PFNGLVERTEXATTRIBS4FVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:6036
PFNGLVERTEXATTRIBS4SVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLshort)) 	# GL/glext.h:6037
PFNGLVERTEXATTRIBS4UBVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLubyte)) 	# GL/glext.h:6038
# SGIX_texture_coordinate_clamp (GL/glext.h:6041)
GL_SGIX_texture_coordinate_clamp = 1 	# GL/glext.h:6042
# SGIX_scalebias_hint (GL/glext.h:6045)
GL_SGIX_scalebias_hint = 1 	# GL/glext.h:6046
# OML_interlace (GL/glext.h:6049)
GL_OML_interlace = 1 	# GL/glext.h:6050
# OML_subsample (GL/glext.h:6053)
GL_OML_subsample = 1 	# GL/glext.h:6054
# OML_resample (GL/glext.h:6057)
GL_OML_resample = 1 	# GL/glext.h:6058
# NV_copy_depth_to_color (GL/glext.h:6061)
GL_NV_copy_depth_to_color = 1 	# GL/glext.h:6062
# ATI_envmap_bumpmap (GL/glext.h:6065)
GL_ATI_envmap_bumpmap = 1 	# GL/glext.h:6066
# GL/glext.h:6068
glTexBumpParameterivATI = _link_function('glTexBumpParameterivATI', None, [GLenum, POINTER(GLint)], 'ATI_envmap_bumpmap')

# GL/glext.h:6069
glTexBumpParameterfvATI = _link_function('glTexBumpParameterfvATI', None, [GLenum, POINTER(GLfloat)], 'ATI_envmap_bumpmap')

# GL/glext.h:6070
glGetTexBumpParameterivATI = _link_function('glGetTexBumpParameterivATI', None, [GLenum, POINTER(GLint)], 'ATI_envmap_bumpmap')

# GL/glext.h:6071
glGetTexBumpParameterfvATI = _link_function('glGetTexBumpParameterfvATI', None, [GLenum, POINTER(GLfloat)], 'ATI_envmap_bumpmap')

PFNGLTEXBUMPPARAMETERIVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:6073
PFNGLTEXBUMPPARAMETERFVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6074
PFNGLGETTEXBUMPPARAMETERIVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:6075
PFNGLGETTEXBUMPPARAMETERFVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6076
# ATI_fragment_shader (GL/glext.h:6079)
GL_ATI_fragment_shader = 1 	# GL/glext.h:6080
# GL/glext.h:6082
glGenFragmentShadersATI = _link_function('glGenFragmentShadersATI', GLuint, [GLuint], 'ATI_fragment_shader')

# GL/glext.h:6083
glBindFragmentShaderATI = _link_function('glBindFragmentShaderATI', None, [GLuint], 'ATI_fragment_shader')

# GL/glext.h:6084
glDeleteFragmentShaderATI = _link_function('glDeleteFragmentShaderATI', None, [GLuint], 'ATI_fragment_shader')

# GL/glext.h:6085
glBeginFragmentShaderATI = _link_function('glBeginFragmentShaderATI', None, [], 'ATI_fragment_shader')

# GL/glext.h:6086
glEndFragmentShaderATI = _link_function('glEndFragmentShaderATI', None, [], 'ATI_fragment_shader')

# GL/glext.h:6087
glPassTexCoordATI = _link_function('glPassTexCoordATI', None, [GLuint, GLuint, GLenum], 'ATI_fragment_shader')

# GL/glext.h:6088
glSampleMapATI = _link_function('glSampleMapATI', None, [GLuint, GLuint, GLenum], 'ATI_fragment_shader')

# GL/glext.h:6089
glColorFragmentOp1ATI = _link_function('glColorFragmentOp1ATI', None, [GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint], 'ATI_fragment_shader')

# GL/glext.h:6090
glColorFragmentOp2ATI = _link_function('glColorFragmentOp2ATI', None, [GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint], 'ATI_fragment_shader')

# GL/glext.h:6091
glColorFragmentOp3ATI = _link_function('glColorFragmentOp3ATI', None, [GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint], 'ATI_fragment_shader')

# GL/glext.h:6092
glAlphaFragmentOp1ATI = _link_function('glAlphaFragmentOp1ATI', None, [GLenum, GLuint, GLuint, GLuint, GLuint, GLuint], 'ATI_fragment_shader')

# GL/glext.h:6093
glAlphaFragmentOp2ATI = _link_function('glAlphaFragmentOp2ATI', None, [GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint], 'ATI_fragment_shader')

# GL/glext.h:6094
glAlphaFragmentOp3ATI = _link_function('glAlphaFragmentOp3ATI', None, [GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint], 'ATI_fragment_shader')

# GL/glext.h:6095
glSetFragmentShaderConstantATI = _link_function('glSetFragmentShaderConstantATI', None, [GLuint, POINTER(GLfloat)], 'ATI_fragment_shader')

PFNGLGENFRAGMENTSHADERSATIPROC = CFUNCTYPE(GLuint, GLuint) 	# GL/glext.h:6097
PFNGLBINDFRAGMENTSHADERATIPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6098
PFNGLDELETEFRAGMENTSHADERATIPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6099
PFNGLBEGINFRAGMENTSHADERATIPROC = CFUNCTYPE(None) 	# GL/glext.h:6100
PFNGLENDFRAGMENTSHADERATIPROC = CFUNCTYPE(None) 	# GL/glext.h:6101
PFNGLPASSTEXCOORDATIPROC = CFUNCTYPE(None, GLuint, GLuint, GLenum) 	# GL/glext.h:6102
PFNGLSAMPLEMAPATIPROC = CFUNCTYPE(None, GLuint, GLuint, GLenum) 	# GL/glext.h:6103
PFNGLCOLORFRAGMENTOP1ATIPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:6104
PFNGLCOLORFRAGMENTOP2ATIPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:6105
PFNGLCOLORFRAGMENTOP3ATIPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:6106
PFNGLALPHAFRAGMENTOP1ATIPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:6107
PFNGLALPHAFRAGMENTOP2ATIPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:6108
PFNGLALPHAFRAGMENTOP3ATIPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:6109
PFNGLSETFRAGMENTSHADERCONSTANTATIPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:6110
# ATI_pn_triangles (GL/glext.h:6113)
GL_ATI_pn_triangles = 1 	# GL/glext.h:6114
# GL/glext.h:6116
glPNTrianglesiATI = _link_function('glPNTrianglesiATI', None, [GLenum, GLint], 'ATI_pn_triangles')

# GL/glext.h:6117
glPNTrianglesfATI = _link_function('glPNTrianglesfATI', None, [GLenum, GLfloat], 'ATI_pn_triangles')

PFNGLPNTRIANGLESIATIPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:6119
PFNGLPNTRIANGLESFATIPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:6120
# ATI_vertex_array_object (GL/glext.h:6123)
GL_ATI_vertex_array_object = 1 	# GL/glext.h:6124
# GL/glext.h:6126
glNewObjectBufferATI = _link_function('glNewObjectBufferATI', GLuint, [GLsizei, POINTER(GLvoid), GLenum], 'ATI_vertex_array_object')

# GL/glext.h:6127
glIsObjectBufferATI = _link_function('glIsObjectBufferATI', GLboolean, [GLuint], 'ATI_vertex_array_object')

# GL/glext.h:6128
glUpdateObjectBufferATI = _link_function('glUpdateObjectBufferATI', None, [GLuint, GLuint, GLsizei, POINTER(GLvoid), GLenum], 'ATI_vertex_array_object')

# GL/glext.h:6129
glGetObjectBufferfvATI = _link_function('glGetObjectBufferfvATI', None, [GLuint, GLenum, POINTER(GLfloat)], 'ATI_vertex_array_object')

# GL/glext.h:6130
glGetObjectBufferivATI = _link_function('glGetObjectBufferivATI', None, [GLuint, GLenum, POINTER(GLint)], 'ATI_vertex_array_object')

# GL/glext.h:6131
glFreeObjectBufferATI = _link_function('glFreeObjectBufferATI', None, [GLuint], 'ATI_vertex_array_object')

# GL/glext.h:6132
glArrayObjectATI = _link_function('glArrayObjectATI', None, [GLenum, GLint, GLenum, GLsizei, GLuint, GLuint], 'ATI_vertex_array_object')

# GL/glext.h:6133
glGetArrayObjectfvATI = _link_function('glGetArrayObjectfvATI', None, [GLenum, GLenum, POINTER(GLfloat)], 'ATI_vertex_array_object')

# GL/glext.h:6134
glGetArrayObjectivATI = _link_function('glGetArrayObjectivATI', None, [GLenum, GLenum, POINTER(GLint)], 'ATI_vertex_array_object')

# GL/glext.h:6135
glVariantArrayObjectATI = _link_function('glVariantArrayObjectATI', None, [GLuint, GLenum, GLsizei, GLuint, GLuint], 'ATI_vertex_array_object')

# GL/glext.h:6136
glGetVariantArrayObjectfvATI = _link_function('glGetVariantArrayObjectfvATI', None, [GLuint, GLenum, POINTER(GLfloat)], 'ATI_vertex_array_object')

# GL/glext.h:6137
glGetVariantArrayObjectivATI = _link_function('glGetVariantArrayObjectivATI', None, [GLuint, GLenum, POINTER(GLint)], 'ATI_vertex_array_object')

PFNGLNEWOBJECTBUFFERATIPROC = CFUNCTYPE(GLuint, GLsizei, POINTER(GLvoid), GLenum) 	# GL/glext.h:6139
PFNGLISOBJECTBUFFERATIPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:6140
PFNGLUPDATEOBJECTBUFFERATIPROC = CFUNCTYPE(None, GLuint, GLuint, GLsizei, POINTER(GLvoid), GLenum) 	# GL/glext.h:6141
PFNGLGETOBJECTBUFFERFVATIPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6142
PFNGLGETOBJECTBUFFERIVATIPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:6143
PFNGLFREEOBJECTBUFFERATIPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6144
PFNGLARRAYOBJECTATIPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLsizei, GLuint, GLuint) 	# GL/glext.h:6145
PFNGLGETARRAYOBJECTFVATIPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6146
PFNGLGETARRAYOBJECTIVATIPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:6147
PFNGLVARIANTARRAYOBJECTATIPROC = CFUNCTYPE(None, GLuint, GLenum, GLsizei, GLuint, GLuint) 	# GL/glext.h:6148
PFNGLGETVARIANTARRAYOBJECTFVATIPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6149
PFNGLGETVARIANTARRAYOBJECTIVATIPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:6150
# EXT_vertex_shader (GL/glext.h:6153)
GL_EXT_vertex_shader = 1 	# GL/glext.h:6154
# GL/glext.h:6156
glBeginVertexShaderEXT = _link_function('glBeginVertexShaderEXT', None, [], 'EXT_vertex_shader')

# GL/glext.h:6157
glEndVertexShaderEXT = _link_function('glEndVertexShaderEXT', None, [], 'EXT_vertex_shader')

# GL/glext.h:6158
glBindVertexShaderEXT = _link_function('glBindVertexShaderEXT', None, [GLuint], 'EXT_vertex_shader')

# GL/glext.h:6159
glGenVertexShadersEXT = _link_function('glGenVertexShadersEXT', GLuint, [GLuint], 'EXT_vertex_shader')

# GL/glext.h:6160
glDeleteVertexShaderEXT = _link_function('glDeleteVertexShaderEXT', None, [GLuint], 'EXT_vertex_shader')

# GL/glext.h:6161
glShaderOp1EXT = _link_function('glShaderOp1EXT', None, [GLenum, GLuint, GLuint], 'EXT_vertex_shader')

# GL/glext.h:6162
glShaderOp2EXT = _link_function('glShaderOp2EXT', None, [GLenum, GLuint, GLuint, GLuint], 'EXT_vertex_shader')

# GL/glext.h:6163
glShaderOp3EXT = _link_function('glShaderOp3EXT', None, [GLenum, GLuint, GLuint, GLuint, GLuint], 'EXT_vertex_shader')

# GL/glext.h:6164
glSwizzleEXT = _link_function('glSwizzleEXT', None, [GLuint, GLuint, GLenum, GLenum, GLenum, GLenum], 'EXT_vertex_shader')

# GL/glext.h:6165
glWriteMaskEXT = _link_function('glWriteMaskEXT', None, [GLuint, GLuint, GLenum, GLenum, GLenum, GLenum], 'EXT_vertex_shader')

# GL/glext.h:6166
glInsertComponentEXT = _link_function('glInsertComponentEXT', None, [GLuint, GLuint, GLuint], 'EXT_vertex_shader')

# GL/glext.h:6167
glExtractComponentEXT = _link_function('glExtractComponentEXT', None, [GLuint, GLuint, GLuint], 'EXT_vertex_shader')

# GL/glext.h:6168
glGenSymbolsEXT = _link_function('glGenSymbolsEXT', GLuint, [GLenum, GLenum, GLenum, GLuint], 'EXT_vertex_shader')

# GL/glext.h:6169
glSetInvariantEXT = _link_function('glSetInvariantEXT', None, [GLuint, GLenum, POINTER(GLvoid)], 'EXT_vertex_shader')

# GL/glext.h:6170
glSetLocalConstantEXT = _link_function('glSetLocalConstantEXT', None, [GLuint, GLenum, POINTER(GLvoid)], 'EXT_vertex_shader')

# GL/glext.h:6171
glVariantbvEXT = _link_function('glVariantbvEXT', None, [GLuint, POINTER(GLbyte)], 'EXT_vertex_shader')

# GL/glext.h:6172
glVariantsvEXT = _link_function('glVariantsvEXT', None, [GLuint, POINTER(GLshort)], 'EXT_vertex_shader')

# GL/glext.h:6173
glVariantivEXT = _link_function('glVariantivEXT', None, [GLuint, POINTER(GLint)], 'EXT_vertex_shader')

# GL/glext.h:6174
glVariantfvEXT = _link_function('glVariantfvEXT', None, [GLuint, POINTER(GLfloat)], 'EXT_vertex_shader')

# GL/glext.h:6175
glVariantdvEXT = _link_function('glVariantdvEXT', None, [GLuint, POINTER(GLdouble)], 'EXT_vertex_shader')

# GL/glext.h:6176
glVariantubvEXT = _link_function('glVariantubvEXT', None, [GLuint, POINTER(GLubyte)], 'EXT_vertex_shader')

# GL/glext.h:6177
glVariantusvEXT = _link_function('glVariantusvEXT', None, [GLuint, POINTER(GLushort)], 'EXT_vertex_shader')

# GL/glext.h:6178
glVariantuivEXT = _link_function('glVariantuivEXT', None, [GLuint, POINTER(GLuint)], 'EXT_vertex_shader')

# GL/glext.h:6179
glVariantPointerEXT = _link_function('glVariantPointerEXT', None, [GLuint, GLenum, GLuint, POINTER(GLvoid)], 'EXT_vertex_shader')

# GL/glext.h:6180
glEnableVariantClientStateEXT = _link_function('glEnableVariantClientStateEXT', None, [GLuint], 'EXT_vertex_shader')

# GL/glext.h:6181
glDisableVariantClientStateEXT = _link_function('glDisableVariantClientStateEXT', None, [GLuint], 'EXT_vertex_shader')

# GL/glext.h:6182
glBindLightParameterEXT = _link_function('glBindLightParameterEXT', GLuint, [GLenum, GLenum], 'EXT_vertex_shader')

# GL/glext.h:6183
glBindMaterialParameterEXT = _link_function('glBindMaterialParameterEXT', GLuint, [GLenum, GLenum], 'EXT_vertex_shader')

# GL/glext.h:6184
glBindTexGenParameterEXT = _link_function('glBindTexGenParameterEXT', GLuint, [GLenum, GLenum, GLenum], 'EXT_vertex_shader')

# GL/glext.h:6185
glBindTextureUnitParameterEXT = _link_function('glBindTextureUnitParameterEXT', GLuint, [GLenum, GLenum], 'EXT_vertex_shader')

# GL/glext.h:6186
glBindParameterEXT = _link_function('glBindParameterEXT', GLuint, [GLenum], 'EXT_vertex_shader')

# GL/glext.h:6187
glIsVariantEnabledEXT = _link_function('glIsVariantEnabledEXT', GLboolean, [GLuint, GLenum], 'EXT_vertex_shader')

# GL/glext.h:6188
glGetVariantBooleanvEXT = _link_function('glGetVariantBooleanvEXT', None, [GLuint, GLenum, POINTER(GLboolean)], 'EXT_vertex_shader')

# GL/glext.h:6189
glGetVariantIntegervEXT = _link_function('glGetVariantIntegervEXT', None, [GLuint, GLenum, POINTER(GLint)], 'EXT_vertex_shader')

# GL/glext.h:6190
glGetVariantFloatvEXT = _link_function('glGetVariantFloatvEXT', None, [GLuint, GLenum, POINTER(GLfloat)], 'EXT_vertex_shader')

# GL/glext.h:6191
glGetVariantPointervEXT = _link_function('glGetVariantPointervEXT', None, [GLuint, GLenum, POINTER(POINTER(GLvoid))], 'EXT_vertex_shader')

# GL/glext.h:6192
glGetInvariantBooleanvEXT = _link_function('glGetInvariantBooleanvEXT', None, [GLuint, GLenum, POINTER(GLboolean)], 'EXT_vertex_shader')

# GL/glext.h:6193
glGetInvariantIntegervEXT = _link_function('glGetInvariantIntegervEXT', None, [GLuint, GLenum, POINTER(GLint)], 'EXT_vertex_shader')

# GL/glext.h:6194
glGetInvariantFloatvEXT = _link_function('glGetInvariantFloatvEXT', None, [GLuint, GLenum, POINTER(GLfloat)], 'EXT_vertex_shader')

# GL/glext.h:6195
glGetLocalConstantBooleanvEXT = _link_function('glGetLocalConstantBooleanvEXT', None, [GLuint, GLenum, POINTER(GLboolean)], 'EXT_vertex_shader')

# GL/glext.h:6196
glGetLocalConstantIntegervEXT = _link_function('glGetLocalConstantIntegervEXT', None, [GLuint, GLenum, POINTER(GLint)], 'EXT_vertex_shader')

# GL/glext.h:6197
glGetLocalConstantFloatvEXT = _link_function('glGetLocalConstantFloatvEXT', None, [GLuint, GLenum, POINTER(GLfloat)], 'EXT_vertex_shader')

PFNGLBEGINVERTEXSHADEREXTPROC = CFUNCTYPE(None) 	# GL/glext.h:6199
PFNGLENDVERTEXSHADEREXTPROC = CFUNCTYPE(None) 	# GL/glext.h:6200
PFNGLBINDVERTEXSHADEREXTPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6201
PFNGLGENVERTEXSHADERSEXTPROC = CFUNCTYPE(GLuint, GLuint) 	# GL/glext.h:6202
PFNGLDELETEVERTEXSHADEREXTPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6203
PFNGLSHADEROP1EXTPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint) 	# GL/glext.h:6204
PFNGLSHADEROP2EXTPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint) 	# GL/glext.h:6205
PFNGLSHADEROP3EXTPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:6206
PFNGLSWIZZLEEXTPROC = CFUNCTYPE(None, GLuint, GLuint, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:6207
PFNGLWRITEMASKEXTPROC = CFUNCTYPE(None, GLuint, GLuint, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:6208
PFNGLINSERTCOMPONENTEXTPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint) 	# GL/glext.h:6209
PFNGLEXTRACTCOMPONENTEXTPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint) 	# GL/glext.h:6210
PFNGLGENSYMBOLSEXTPROC = CFUNCTYPE(GLuint, GLenum, GLenum, GLenum, GLuint) 	# GL/glext.h:6211
PFNGLSETINVARIANTEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLvoid)) 	# GL/glext.h:6212
PFNGLSETLOCALCONSTANTEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLvoid)) 	# GL/glext.h:6213
PFNGLVARIANTBVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLbyte)) 	# GL/glext.h:6214
PFNGLVARIANTSVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:6215
PFNGLVARIANTIVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:6216
PFNGLVARIANTFVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:6217
PFNGLVARIANTDVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:6218
PFNGLVARIANTUBVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLubyte)) 	# GL/glext.h:6219
PFNGLVARIANTUSVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLushort)) 	# GL/glext.h:6220
PFNGLVARIANTUIVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:6221
PFNGLVARIANTPOINTEREXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, POINTER(GLvoid)) 	# GL/glext.h:6222
PFNGLENABLEVARIANTCLIENTSTATEEXTPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6223
PFNGLDISABLEVARIANTCLIENTSTATEEXTPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6224
PFNGLBINDLIGHTPARAMETEREXTPROC = CFUNCTYPE(GLuint, GLenum, GLenum) 	# GL/glext.h:6225
PFNGLBINDMATERIALPARAMETEREXTPROC = CFUNCTYPE(GLuint, GLenum, GLenum) 	# GL/glext.h:6226
PFNGLBINDTEXGENPARAMETEREXTPROC = CFUNCTYPE(GLuint, GLenum, GLenum, GLenum) 	# GL/glext.h:6227
PFNGLBINDTEXTUREUNITPARAMETEREXTPROC = CFUNCTYPE(GLuint, GLenum, GLenum) 	# GL/glext.h:6228
PFNGLBINDPARAMETEREXTPROC = CFUNCTYPE(GLuint, GLenum) 	# GL/glext.h:6229
PFNGLISVARIANTENABLEDEXTPROC = CFUNCTYPE(GLboolean, GLuint, GLenum) 	# GL/glext.h:6230
PFNGLGETVARIANTBOOLEANVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLboolean)) 	# GL/glext.h:6231
PFNGLGETVARIANTINTEGERVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:6232
PFNGLGETVARIANTFLOATVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6233
PFNGLGETVARIANTPOINTERVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:6234
PFNGLGETINVARIANTBOOLEANVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLboolean)) 	# GL/glext.h:6235
PFNGLGETINVARIANTINTEGERVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:6236
PFNGLGETINVARIANTFLOATVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6237
PFNGLGETLOCALCONSTANTBOOLEANVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLboolean)) 	# GL/glext.h:6238
PFNGLGETLOCALCONSTANTINTEGERVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:6239
PFNGLGETLOCALCONSTANTFLOATVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6240
# ATI_vertex_streams (GL/glext.h:6243)
GL_ATI_vertex_streams = 1 	# GL/glext.h:6244
# GL/glext.h:6246
glVertexStream1sATI = _link_function('glVertexStream1sATI', None, [GLenum, GLshort], 'ATI_vertex_streams')

# GL/glext.h:6247
glVertexStream1svATI = _link_function('glVertexStream1svATI', None, [GLenum, POINTER(GLshort)], 'ATI_vertex_streams')

# GL/glext.h:6248
glVertexStream1iATI = _link_function('glVertexStream1iATI', None, [GLenum, GLint], 'ATI_vertex_streams')

# GL/glext.h:6249
glVertexStream1ivATI = _link_function('glVertexStream1ivATI', None, [GLenum, POINTER(GLint)], 'ATI_vertex_streams')

# GL/glext.h:6250
glVertexStream1fATI = _link_function('glVertexStream1fATI', None, [GLenum, GLfloat], 'ATI_vertex_streams')

# GL/glext.h:6251
glVertexStream1fvATI = _link_function('glVertexStream1fvATI', None, [GLenum, POINTER(GLfloat)], 'ATI_vertex_streams')

# GL/glext.h:6252
glVertexStream1dATI = _link_function('glVertexStream1dATI', None, [GLenum, GLdouble], 'ATI_vertex_streams')

# GL/glext.h:6253
glVertexStream1dvATI = _link_function('glVertexStream1dvATI', None, [GLenum, POINTER(GLdouble)], 'ATI_vertex_streams')

# GL/glext.h:6254
glVertexStream2sATI = _link_function('glVertexStream2sATI', None, [GLenum, GLshort, GLshort], 'ATI_vertex_streams')

# GL/glext.h:6255
glVertexStream2svATI = _link_function('glVertexStream2svATI', None, [GLenum, POINTER(GLshort)], 'ATI_vertex_streams')

# GL/glext.h:6256
glVertexStream2iATI = _link_function('glVertexStream2iATI', None, [GLenum, GLint, GLint], 'ATI_vertex_streams')

# GL/glext.h:6257
glVertexStream2ivATI = _link_function('glVertexStream2ivATI', None, [GLenum, POINTER(GLint)], 'ATI_vertex_streams')

# GL/glext.h:6258
glVertexStream2fATI = _link_function('glVertexStream2fATI', None, [GLenum, GLfloat, GLfloat], 'ATI_vertex_streams')

# GL/glext.h:6259
glVertexStream2fvATI = _link_function('glVertexStream2fvATI', None, [GLenum, POINTER(GLfloat)], 'ATI_vertex_streams')

# GL/glext.h:6260
glVertexStream2dATI = _link_function('glVertexStream2dATI', None, [GLenum, GLdouble, GLdouble], 'ATI_vertex_streams')

# GL/glext.h:6261
glVertexStream2dvATI = _link_function('glVertexStream2dvATI', None, [GLenum, POINTER(GLdouble)], 'ATI_vertex_streams')

# GL/glext.h:6262
glVertexStream3sATI = _link_function('glVertexStream3sATI', None, [GLenum, GLshort, GLshort, GLshort], 'ATI_vertex_streams')

# GL/glext.h:6263
glVertexStream3svATI = _link_function('glVertexStream3svATI', None, [GLenum, POINTER(GLshort)], 'ATI_vertex_streams')

# GL/glext.h:6264
glVertexStream3iATI = _link_function('glVertexStream3iATI', None, [GLenum, GLint, GLint, GLint], 'ATI_vertex_streams')

# GL/glext.h:6265
glVertexStream3ivATI = _link_function('glVertexStream3ivATI', None, [GLenum, POINTER(GLint)], 'ATI_vertex_streams')

# GL/glext.h:6266
glVertexStream3fATI = _link_function('glVertexStream3fATI', None, [GLenum, GLfloat, GLfloat, GLfloat], 'ATI_vertex_streams')

# GL/glext.h:6267
glVertexStream3fvATI = _link_function('glVertexStream3fvATI', None, [GLenum, POINTER(GLfloat)], 'ATI_vertex_streams')

# GL/glext.h:6268
glVertexStream3dATI = _link_function('glVertexStream3dATI', None, [GLenum, GLdouble, GLdouble, GLdouble], 'ATI_vertex_streams')

# GL/glext.h:6269
glVertexStream3dvATI = _link_function('glVertexStream3dvATI', None, [GLenum, POINTER(GLdouble)], 'ATI_vertex_streams')

# GL/glext.h:6270
glVertexStream4sATI = _link_function('glVertexStream4sATI', None, [GLenum, GLshort, GLshort, GLshort, GLshort], 'ATI_vertex_streams')

# GL/glext.h:6271
glVertexStream4svATI = _link_function('glVertexStream4svATI', None, [GLenum, POINTER(GLshort)], 'ATI_vertex_streams')

# GL/glext.h:6272
glVertexStream4iATI = _link_function('glVertexStream4iATI', None, [GLenum, GLint, GLint, GLint, GLint], 'ATI_vertex_streams')

# GL/glext.h:6273
glVertexStream4ivATI = _link_function('glVertexStream4ivATI', None, [GLenum, POINTER(GLint)], 'ATI_vertex_streams')

# GL/glext.h:6274
glVertexStream4fATI = _link_function('glVertexStream4fATI', None, [GLenum, GLfloat, GLfloat, GLfloat, GLfloat], 'ATI_vertex_streams')

# GL/glext.h:6275
glVertexStream4fvATI = _link_function('glVertexStream4fvATI', None, [GLenum, POINTER(GLfloat)], 'ATI_vertex_streams')

# GL/glext.h:6276
glVertexStream4dATI = _link_function('glVertexStream4dATI', None, [GLenum, GLdouble, GLdouble, GLdouble, GLdouble], 'ATI_vertex_streams')

# GL/glext.h:6277
glVertexStream4dvATI = _link_function('glVertexStream4dvATI', None, [GLenum, POINTER(GLdouble)], 'ATI_vertex_streams')

# GL/glext.h:6278
glNormalStream3bATI = _link_function('glNormalStream3bATI', None, [GLenum, GLbyte, GLbyte, GLbyte], 'ATI_vertex_streams')

# GL/glext.h:6279
glNormalStream3bvATI = _link_function('glNormalStream3bvATI', None, [GLenum, POINTER(GLbyte)], 'ATI_vertex_streams')

# GL/glext.h:6280
glNormalStream3sATI = _link_function('glNormalStream3sATI', None, [GLenum, GLshort, GLshort, GLshort], 'ATI_vertex_streams')

# GL/glext.h:6281
glNormalStream3svATI = _link_function('glNormalStream3svATI', None, [GLenum, POINTER(GLshort)], 'ATI_vertex_streams')

# GL/glext.h:6282
glNormalStream3iATI = _link_function('glNormalStream3iATI', None, [GLenum, GLint, GLint, GLint], 'ATI_vertex_streams')

# GL/glext.h:6283
glNormalStream3ivATI = _link_function('glNormalStream3ivATI', None, [GLenum, POINTER(GLint)], 'ATI_vertex_streams')

# GL/glext.h:6284
glNormalStream3fATI = _link_function('glNormalStream3fATI', None, [GLenum, GLfloat, GLfloat, GLfloat], 'ATI_vertex_streams')

# GL/glext.h:6285
glNormalStream3fvATI = _link_function('glNormalStream3fvATI', None, [GLenum, POINTER(GLfloat)], 'ATI_vertex_streams')

# GL/glext.h:6286
glNormalStream3dATI = _link_function('glNormalStream3dATI', None, [GLenum, GLdouble, GLdouble, GLdouble], 'ATI_vertex_streams')

# GL/glext.h:6287
glNormalStream3dvATI = _link_function('glNormalStream3dvATI', None, [GLenum, POINTER(GLdouble)], 'ATI_vertex_streams')

# GL/glext.h:6288
glClientActiveVertexStreamATI = _link_function('glClientActiveVertexStreamATI', None, [GLenum], 'ATI_vertex_streams')

# GL/glext.h:6289
glVertexBlendEnviATI = _link_function('glVertexBlendEnviATI', None, [GLenum, GLint], 'ATI_vertex_streams')

# GL/glext.h:6290
glVertexBlendEnvfATI = _link_function('glVertexBlendEnvfATI', None, [GLenum, GLfloat], 'ATI_vertex_streams')

PFNGLVERTEXSTREAM1SATIPROC = CFUNCTYPE(None, GLenum, GLshort) 	# GL/glext.h:6292
PFNGLVERTEXSTREAM1SVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:6293
PFNGLVERTEXSTREAM1IATIPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:6294
PFNGLVERTEXSTREAM1IVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:6295
PFNGLVERTEXSTREAM1FATIPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:6296
PFNGLVERTEXSTREAM1FVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6297
PFNGLVERTEXSTREAM1DATIPROC = CFUNCTYPE(None, GLenum, GLdouble) 	# GL/glext.h:6298
PFNGLVERTEXSTREAM1DVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:6299
PFNGLVERTEXSTREAM2SATIPROC = CFUNCTYPE(None, GLenum, GLshort, GLshort) 	# GL/glext.h:6300
PFNGLVERTEXSTREAM2SVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:6301
PFNGLVERTEXSTREAM2IATIPROC = CFUNCTYPE(None, GLenum, GLint, GLint) 	# GL/glext.h:6302
PFNGLVERTEXSTREAM2IVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:6303
PFNGLVERTEXSTREAM2FATIPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat) 	# GL/glext.h:6304
PFNGLVERTEXSTREAM2FVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6305
PFNGLVERTEXSTREAM2DATIPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble) 	# GL/glext.h:6306
PFNGLVERTEXSTREAM2DVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:6307
PFNGLVERTEXSTREAM3SATIPROC = CFUNCTYPE(None, GLenum, GLshort, GLshort, GLshort) 	# GL/glext.h:6308
PFNGLVERTEXSTREAM3SVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:6309
PFNGLVERTEXSTREAM3IATIPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint) 	# GL/glext.h:6310
PFNGLVERTEXSTREAM3IVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:6311
PFNGLVERTEXSTREAM3FATIPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6312
PFNGLVERTEXSTREAM3FVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6313
PFNGLVERTEXSTREAM3DATIPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:6314
PFNGLVERTEXSTREAM3DVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:6315
PFNGLVERTEXSTREAM4SATIPROC = CFUNCTYPE(None, GLenum, GLshort, GLshort, GLshort, GLshort) 	# GL/glext.h:6316
PFNGLVERTEXSTREAM4SVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:6317
PFNGLVERTEXSTREAM4IATIPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint) 	# GL/glext.h:6318
PFNGLVERTEXSTREAM4IVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:6319
PFNGLVERTEXSTREAM4FATIPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6320
PFNGLVERTEXSTREAM4FVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6321
PFNGLVERTEXSTREAM4DATIPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:6322
PFNGLVERTEXSTREAM4DVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:6323
PFNGLNORMALSTREAM3BATIPROC = CFUNCTYPE(None, GLenum, GLbyte, GLbyte, GLbyte) 	# GL/glext.h:6324
PFNGLNORMALSTREAM3BVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLbyte)) 	# GL/glext.h:6325
PFNGLNORMALSTREAM3SATIPROC = CFUNCTYPE(None, GLenum, GLshort, GLshort, GLshort) 	# GL/glext.h:6326
PFNGLNORMALSTREAM3SVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:6327
PFNGLNORMALSTREAM3IATIPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint) 	# GL/glext.h:6328
PFNGLNORMALSTREAM3IVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:6329
PFNGLNORMALSTREAM3FATIPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6330
PFNGLNORMALSTREAM3FVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6331
PFNGLNORMALSTREAM3DATIPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:6332
PFNGLNORMALSTREAM3DVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:6333
PFNGLCLIENTACTIVEVERTEXSTREAMATIPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:6334
PFNGLVERTEXBLENDENVIATIPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:6335
PFNGLVERTEXBLENDENVFATIPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:6336
# ATI_element_array (GL/glext.h:6339)
GL_ATI_element_array = 1 	# GL/glext.h:6340
# GL/glext.h:6342
glElementPointerATI = _link_function('glElementPointerATI', None, [GLenum, POINTER(GLvoid)], 'ATI_element_array')

# GL/glext.h:6343
glDrawElementArrayATI = _link_function('glDrawElementArrayATI', None, [GLenum, GLsizei], 'ATI_element_array')

# GL/glext.h:6344
glDrawRangeElementArrayATI = _link_function('glDrawRangeElementArrayATI', None, [GLenum, GLuint, GLuint, GLsizei], 'ATI_element_array')

PFNGLELEMENTPOINTERATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLvoid)) 	# GL/glext.h:6346
PFNGLDRAWELEMENTARRAYATIPROC = CFUNCTYPE(None, GLenum, GLsizei) 	# GL/glext.h:6347
PFNGLDRAWRANGEELEMENTARRAYATIPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLsizei) 	# GL/glext.h:6348
# SUN_mesh_array (GL/glext.h:6351)
GL_SUN_mesh_array = 1 	# GL/glext.h:6352
# GL/glext.h:6354
glDrawMeshArraysSUN = _link_function('glDrawMeshArraysSUN', None, [GLenum, GLint, GLsizei, GLsizei], 'SUN_mesh_array')

PFNGLDRAWMESHARRAYSSUNPROC = CFUNCTYPE(None, GLenum, GLint, GLsizei, GLsizei) 	# GL/glext.h:6356
# SUN_slice_accum (GL/glext.h:6359)
GL_SUN_slice_accum = 1 	# GL/glext.h:6360
# NV_multisample_filter_hint (GL/glext.h:6363)
GL_NV_multisample_filter_hint = 1 	# GL/glext.h:6364
# NV_depth_clamp (GL/glext.h:6367)
GL_NV_depth_clamp = 1 	# GL/glext.h:6368
# NV_occlusion_query (GL/glext.h:6371)
GL_NV_occlusion_query = 1 	# GL/glext.h:6372
# GL/glext.h:6374
glGenOcclusionQueriesNV = _link_function('glGenOcclusionQueriesNV', None, [GLsizei, POINTER(GLuint)], 'NV_occlusion_query')

# GL/glext.h:6375
glDeleteOcclusionQueriesNV = _link_function('glDeleteOcclusionQueriesNV', None, [GLsizei, POINTER(GLuint)], 'NV_occlusion_query')

# GL/glext.h:6376
glIsOcclusionQueryNV = _link_function('glIsOcclusionQueryNV', GLboolean, [GLuint], 'NV_occlusion_query')

# GL/glext.h:6377
glBeginOcclusionQueryNV = _link_function('glBeginOcclusionQueryNV', None, [GLuint], 'NV_occlusion_query')

# GL/glext.h:6378
glEndOcclusionQueryNV = _link_function('glEndOcclusionQueryNV', None, [], 'NV_occlusion_query')

# GL/glext.h:6379
glGetOcclusionQueryivNV = _link_function('glGetOcclusionQueryivNV', None, [GLuint, GLenum, POINTER(GLint)], 'NV_occlusion_query')

# GL/glext.h:6380
glGetOcclusionQueryuivNV = _link_function('glGetOcclusionQueryuivNV', None, [GLuint, GLenum, POINTER(GLuint)], 'NV_occlusion_query')

PFNGLGENOCCLUSIONQUERIESNVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6382
PFNGLDELETEOCCLUSIONQUERIESNVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6383
PFNGLISOCCLUSIONQUERYNVPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:6384
PFNGLBEGINOCCLUSIONQUERYNVPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6385
PFNGLENDOCCLUSIONQUERYNVPROC = CFUNCTYPE(None) 	# GL/glext.h:6386
PFNGLGETOCCLUSIONQUERYIVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:6387
PFNGLGETOCCLUSIONQUERYUIVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLuint)) 	# GL/glext.h:6388
# NV_point_sprite (GL/glext.h:6391)
GL_NV_point_sprite = 1 	# GL/glext.h:6392
# GL/glext.h:6394
glPointParameteriNV = _link_function('glPointParameteriNV', None, [GLenum, GLint], 'NV_point_sprite')

# GL/glext.h:6395
glPointParameterivNV = _link_function('glPointParameterivNV', None, [GLenum, POINTER(GLint)], 'NV_point_sprite')

PFNGLPOINTPARAMETERINVPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:6397
PFNGLPOINTPARAMETERIVNVPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:6398
# NV_texture_shader3 (GL/glext.h:6401)
GL_NV_texture_shader3 = 1 	# GL/glext.h:6402
# NV_vertex_program1_1 (GL/glext.h:6405)
GL_NV_vertex_program1_1 = 1 	# GL/glext.h:6406
# EXT_shadow_funcs (GL/glext.h:6409)
GL_EXT_shadow_funcs = 1 	# GL/glext.h:6410
# EXT_stencil_two_side (GL/glext.h:6413)
GL_EXT_stencil_two_side = 1 	# GL/glext.h:6414
# GL/glext.h:6416
glActiveStencilFaceEXT = _link_function('glActiveStencilFaceEXT', None, [GLenum], 'EXT_stencil_two_side')

PFNGLACTIVESTENCILFACEEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:6418
# ATI_text_fragment_shader (GL/glext.h:6421)
GL_ATI_text_fragment_shader = 1 	# GL/glext.h:6422
# APPLE_client_storage (GL/glext.h:6425)
GL_APPLE_client_storage = 1 	# GL/glext.h:6426
# APPLE_element_array (GL/glext.h:6429)
GL_APPLE_element_array = 1 	# GL/glext.h:6430
# GL/glext.h:6432
glElementPointerAPPLE = _link_function('glElementPointerAPPLE', None, [GLenum, POINTER(GLvoid)], 'APPLE_element_array')

# GL/glext.h:6433
glDrawElementArrayAPPLE = _link_function('glDrawElementArrayAPPLE', None, [GLenum, GLint, GLsizei], 'APPLE_element_array')

# GL/glext.h:6434
glDrawRangeElementArrayAPPLE = _link_function('glDrawRangeElementArrayAPPLE', None, [GLenum, GLuint, GLuint, GLint, GLsizei], 'APPLE_element_array')

# GL/glext.h:6435
glMultiDrawElementArrayAPPLE = _link_function('glMultiDrawElementArrayAPPLE', None, [GLenum, POINTER(GLint), POINTER(GLsizei), GLsizei], 'APPLE_element_array')

# GL/glext.h:6436
glMultiDrawRangeElementArrayAPPLE = _link_function('glMultiDrawRangeElementArrayAPPLE', None, [GLenum, GLuint, GLuint, POINTER(GLint), POINTER(GLsizei), GLsizei], 'APPLE_element_array')

PFNGLELEMENTPOINTERAPPLEPROC = CFUNCTYPE(None, GLenum, POINTER(GLvoid)) 	# GL/glext.h:6438
PFNGLDRAWELEMENTARRAYAPPLEPROC = CFUNCTYPE(None, GLenum, GLint, GLsizei) 	# GL/glext.h:6439
PFNGLDRAWRANGEELEMENTARRAYAPPLEPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLint, GLsizei) 	# GL/glext.h:6440
PFNGLMULTIDRAWELEMENTARRAYAPPLEPROC = CFUNCTYPE(None, GLenum, POINTER(GLint), POINTER(GLsizei), GLsizei) 	# GL/glext.h:6441
PFNGLMULTIDRAWRANGEELEMENTARRAYAPPLEPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, POINTER(GLint), POINTER(GLsizei), GLsizei) 	# GL/glext.h:6442
# APPLE_fence (GL/glext.h:6445)
GL_APPLE_fence = 1 	# GL/glext.h:6446
# GL/glext.h:6448
glGenFencesAPPLE = _link_function('glGenFencesAPPLE', None, [GLsizei, POINTER(GLuint)], 'APPLE_fence')

# GL/glext.h:6449
glDeleteFencesAPPLE = _link_function('glDeleteFencesAPPLE', None, [GLsizei, POINTER(GLuint)], 'APPLE_fence')

# GL/glext.h:6450
glSetFenceAPPLE = _link_function('glSetFenceAPPLE', None, [GLuint], 'APPLE_fence')

# GL/glext.h:6451
glIsFenceAPPLE = _link_function('glIsFenceAPPLE', GLboolean, [GLuint], 'APPLE_fence')

# GL/glext.h:6452
glTestFenceAPPLE = _link_function('glTestFenceAPPLE', GLboolean, [GLuint], 'APPLE_fence')

# GL/glext.h:6453
glFinishFenceAPPLE = _link_function('glFinishFenceAPPLE', None, [GLuint], 'APPLE_fence')

# GL/glext.h:6454
glTestObjectAPPLE = _link_function('glTestObjectAPPLE', GLboolean, [GLenum, GLuint], 'APPLE_fence')

# GL/glext.h:6455
glFinishObjectAPPLE = _link_function('glFinishObjectAPPLE', None, [GLenum, GLint], 'APPLE_fence')

PFNGLGENFENCESAPPLEPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6457
PFNGLDELETEFENCESAPPLEPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6458
PFNGLSETFENCEAPPLEPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6459
PFNGLISFENCEAPPLEPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:6460
PFNGLTESTFENCEAPPLEPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:6461
PFNGLFINISHFENCEAPPLEPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6462
PFNGLTESTOBJECTAPPLEPROC = CFUNCTYPE(GLboolean, GLenum, GLuint) 	# GL/glext.h:6463
PFNGLFINISHOBJECTAPPLEPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:6464
# APPLE_vertex_array_object (GL/glext.h:6467)
GL_APPLE_vertex_array_object = 1 	# GL/glext.h:6468
# GL/glext.h:6470
glBindVertexArrayAPPLE = _link_function('glBindVertexArrayAPPLE', None, [GLuint], 'APPLE_vertex_array_object')

# GL/glext.h:6471
glDeleteVertexArraysAPPLE = _link_function('glDeleteVertexArraysAPPLE', None, [GLsizei, POINTER(GLuint)], 'APPLE_vertex_array_object')

# GL/glext.h:6472
glGenVertexArraysAPPLE = _link_function('glGenVertexArraysAPPLE', None, [GLsizei, POINTER(GLuint)], 'APPLE_vertex_array_object')

# GL/glext.h:6473
glIsVertexArrayAPPLE = _link_function('glIsVertexArrayAPPLE', GLboolean, [GLuint], 'APPLE_vertex_array_object')

PFNGLBINDVERTEXARRAYAPPLEPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6475
PFNGLDELETEVERTEXARRAYSAPPLEPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6476
PFNGLGENVERTEXARRAYSAPPLEPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6477
PFNGLISVERTEXARRAYAPPLEPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:6478
# APPLE_vertex_array_range (GL/glext.h:6481)
GL_APPLE_vertex_array_range = 1 	# GL/glext.h:6482
# GL/glext.h:6484
glVertexArrayRangeAPPLE = _link_function('glVertexArrayRangeAPPLE', None, [GLsizei, POINTER(GLvoid)], 'APPLE_vertex_array_range')

# GL/glext.h:6485
glFlushVertexArrayRangeAPPLE = _link_function('glFlushVertexArrayRangeAPPLE', None, [GLsizei, POINTER(GLvoid)], 'APPLE_vertex_array_range')

# GL/glext.h:6486
glVertexArrayParameteriAPPLE = _link_function('glVertexArrayParameteriAPPLE', None, [GLenum, GLint], 'APPLE_vertex_array_range')

PFNGLVERTEXARRAYRANGEAPPLEPROC = CFUNCTYPE(None, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:6488
PFNGLFLUSHVERTEXARRAYRANGEAPPLEPROC = CFUNCTYPE(None, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:6489
PFNGLVERTEXARRAYPARAMETERIAPPLEPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:6490
# APPLE_ycbcr_422 (GL/glext.h:6493)
GL_APPLE_ycbcr_422 = 1 	# GL/glext.h:6494
# S3_s3tc (GL/glext.h:6497)
GL_S3_s3tc = 1 	# GL/glext.h:6498
# ATI_draw_buffers (GL/glext.h:6501)
GL_ATI_draw_buffers = 1 	# GL/glext.h:6502
# GL/glext.h:6504
glDrawBuffersATI = _link_function('glDrawBuffersATI', None, [GLsizei, POINTER(GLenum)], 'ATI_draw_buffers')

PFNGLDRAWBUFFERSATIPROC = CFUNCTYPE(None, GLsizei, POINTER(GLenum)) 	# GL/glext.h:6506
# ATI_pixel_format_float (GL/glext.h:6509)
GL_ATI_pixel_format_float = 1 	# GL/glext.h:6510
# ATI_texture_env_combine3 (GL/glext.h:6516)
GL_ATI_texture_env_combine3 = 1 	# GL/glext.h:6517
# ATI_texture_float (GL/glext.h:6520)
GL_ATI_texture_float = 1 	# GL/glext.h:6521
# NV_float_buffer (GL/glext.h:6524)
GL_NV_float_buffer = 1 	# GL/glext.h:6525
# NV_fragment_program (GL/glext.h:6528)
GL_NV_fragment_program = 1 	# GL/glext.h:6529
# GL/glext.h:6532
glProgramNamedParameter4fNV = _link_function('glProgramNamedParameter4fNV', None, [GLuint, GLsizei, POINTER(GLubyte), GLfloat, GLfloat, GLfloat, GLfloat], 'NV_fragment_program')

# GL/glext.h:6533
glProgramNamedParameter4dNV = _link_function('glProgramNamedParameter4dNV', None, [GLuint, GLsizei, POINTER(GLubyte), GLdouble, GLdouble, GLdouble, GLdouble], 'NV_fragment_program')

# GL/glext.h:6534
glProgramNamedParameter4fvNV = _link_function('glProgramNamedParameter4fvNV', None, [GLuint, GLsizei, POINTER(GLubyte), POINTER(GLfloat)], 'NV_fragment_program')

# GL/glext.h:6535
glProgramNamedParameter4dvNV = _link_function('glProgramNamedParameter4dvNV', None, [GLuint, GLsizei, POINTER(GLubyte), POINTER(GLdouble)], 'NV_fragment_program')

# GL/glext.h:6536
glGetProgramNamedParameterfvNV = _link_function('glGetProgramNamedParameterfvNV', None, [GLuint, GLsizei, POINTER(GLubyte), POINTER(GLfloat)], 'NV_fragment_program')

# GL/glext.h:6537
glGetProgramNamedParameterdvNV = _link_function('glGetProgramNamedParameterdvNV', None, [GLuint, GLsizei, POINTER(GLubyte), POINTER(GLdouble)], 'NV_fragment_program')

PFNGLPROGRAMNAMEDPARAMETER4FNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLubyte), GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6539
PFNGLPROGRAMNAMEDPARAMETER4DNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLubyte), GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:6540
PFNGLPROGRAMNAMEDPARAMETER4FVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLubyte), POINTER(GLfloat)) 	# GL/glext.h:6541
PFNGLPROGRAMNAMEDPARAMETER4DVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLubyte), POINTER(GLdouble)) 	# GL/glext.h:6542
PFNGLGETPROGRAMNAMEDPARAMETERFVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLubyte), POINTER(GLfloat)) 	# GL/glext.h:6543
PFNGLGETPROGRAMNAMEDPARAMETERDVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLubyte), POINTER(GLdouble)) 	# GL/glext.h:6544
# NV_half_float (GL/glext.h:6547)
GL_NV_half_float = 1 	# GL/glext.h:6548
# GL/glext.h:6550
glVertex2hNV = _link_function('glVertex2hNV', None, [GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:6551
glVertex2hvNV = _link_function('glVertex2hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6552
glVertex3hNV = _link_function('glVertex3hNV', None, [GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:6553
glVertex3hvNV = _link_function('glVertex3hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6554
glVertex4hNV = _link_function('glVertex4hNV', None, [GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:6555
glVertex4hvNV = _link_function('glVertex4hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6556
glNormal3hNV = _link_function('glNormal3hNV', None, [GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:6557
glNormal3hvNV = _link_function('glNormal3hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6558
glColor3hNV = _link_function('glColor3hNV', None, [GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:6559
glColor3hvNV = _link_function('glColor3hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6560
glColor4hNV = _link_function('glColor4hNV', None, [GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:6561
glColor4hvNV = _link_function('glColor4hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6562
glTexCoord1hNV = _link_function('glTexCoord1hNV', None, [GLhalfNV], 'NV_half_float')

# GL/glext.h:6563
glTexCoord1hvNV = _link_function('glTexCoord1hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6564
glTexCoord2hNV = _link_function('glTexCoord2hNV', None, [GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:6565
glTexCoord2hvNV = _link_function('glTexCoord2hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6566
glTexCoord3hNV = _link_function('glTexCoord3hNV', None, [GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:6567
glTexCoord3hvNV = _link_function('glTexCoord3hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6568
glTexCoord4hNV = _link_function('glTexCoord4hNV', None, [GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:6569
glTexCoord4hvNV = _link_function('glTexCoord4hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6570
glMultiTexCoord1hNV = _link_function('glMultiTexCoord1hNV', None, [GLenum, GLhalfNV], 'NV_half_float')

# GL/glext.h:6571
glMultiTexCoord1hvNV = _link_function('glMultiTexCoord1hvNV', None, [GLenum, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6572
glMultiTexCoord2hNV = _link_function('glMultiTexCoord2hNV', None, [GLenum, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:6573
glMultiTexCoord2hvNV = _link_function('glMultiTexCoord2hvNV', None, [GLenum, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6574
glMultiTexCoord3hNV = _link_function('glMultiTexCoord3hNV', None, [GLenum, GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:6575
glMultiTexCoord3hvNV = _link_function('glMultiTexCoord3hvNV', None, [GLenum, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6576
glMultiTexCoord4hNV = _link_function('glMultiTexCoord4hNV', None, [GLenum, GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:6577
glMultiTexCoord4hvNV = _link_function('glMultiTexCoord4hvNV', None, [GLenum, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6578
glFogCoordhNV = _link_function('glFogCoordhNV', None, [GLhalfNV], 'NV_half_float')

# GL/glext.h:6579
glFogCoordhvNV = _link_function('glFogCoordhvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6580
glSecondaryColor3hNV = _link_function('glSecondaryColor3hNV', None, [GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:6581
glSecondaryColor3hvNV = _link_function('glSecondaryColor3hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6582
glVertexWeighthNV = _link_function('glVertexWeighthNV', None, [GLhalfNV], 'NV_half_float')

# GL/glext.h:6583
glVertexWeighthvNV = _link_function('glVertexWeighthvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6584
glVertexAttrib1hNV = _link_function('glVertexAttrib1hNV', None, [GLuint, GLhalfNV], 'NV_half_float')

# GL/glext.h:6585
glVertexAttrib1hvNV = _link_function('glVertexAttrib1hvNV', None, [GLuint, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6586
glVertexAttrib2hNV = _link_function('glVertexAttrib2hNV', None, [GLuint, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:6587
glVertexAttrib2hvNV = _link_function('glVertexAttrib2hvNV', None, [GLuint, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6588
glVertexAttrib3hNV = _link_function('glVertexAttrib3hNV', None, [GLuint, GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:6589
glVertexAttrib3hvNV = _link_function('glVertexAttrib3hvNV', None, [GLuint, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6590
glVertexAttrib4hNV = _link_function('glVertexAttrib4hNV', None, [GLuint, GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:6591
glVertexAttrib4hvNV = _link_function('glVertexAttrib4hvNV', None, [GLuint, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6592
glVertexAttribs1hvNV = _link_function('glVertexAttribs1hvNV', None, [GLuint, GLsizei, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6593
glVertexAttribs2hvNV = _link_function('glVertexAttribs2hvNV', None, [GLuint, GLsizei, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6594
glVertexAttribs3hvNV = _link_function('glVertexAttribs3hvNV', None, [GLuint, GLsizei, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:6595
glVertexAttribs4hvNV = _link_function('glVertexAttribs4hvNV', None, [GLuint, GLsizei, POINTER(GLhalfNV)], 'NV_half_float')

PFNGLVERTEX2HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV) 	# GL/glext.h:6597
PFNGLVERTEX2HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:6598
PFNGLVERTEX3HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:6599
PFNGLVERTEX3HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:6600
PFNGLVERTEX4HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:6601
PFNGLVERTEX4HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:6602
PFNGLNORMAL3HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:6603
PFNGLNORMAL3HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:6604
PFNGLCOLOR3HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:6605
PFNGLCOLOR3HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:6606
PFNGLCOLOR4HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:6607
PFNGLCOLOR4HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:6608
PFNGLTEXCOORD1HNVPROC = CFUNCTYPE(None, GLhalfNV) 	# GL/glext.h:6609
PFNGLTEXCOORD1HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:6610
PFNGLTEXCOORD2HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV) 	# GL/glext.h:6611
PFNGLTEXCOORD2HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:6612
PFNGLTEXCOORD3HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:6613
PFNGLTEXCOORD3HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:6614
PFNGLTEXCOORD4HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:6615
PFNGLTEXCOORD4HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:6616
PFNGLMULTITEXCOORD1HNVPROC = CFUNCTYPE(None, GLenum, GLhalfNV) 	# GL/glext.h:6617
PFNGLMULTITEXCOORD1HVNVPROC = CFUNCTYPE(None, GLenum, POINTER(GLhalfNV)) 	# GL/glext.h:6618
PFNGLMULTITEXCOORD2HNVPROC = CFUNCTYPE(None, GLenum, GLhalfNV, GLhalfNV) 	# GL/glext.h:6619
PFNGLMULTITEXCOORD2HVNVPROC = CFUNCTYPE(None, GLenum, POINTER(GLhalfNV)) 	# GL/glext.h:6620
PFNGLMULTITEXCOORD3HNVPROC = CFUNCTYPE(None, GLenum, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:6621
PFNGLMULTITEXCOORD3HVNVPROC = CFUNCTYPE(None, GLenum, POINTER(GLhalfNV)) 	# GL/glext.h:6622
PFNGLMULTITEXCOORD4HNVPROC = CFUNCTYPE(None, GLenum, GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:6623
PFNGLMULTITEXCOORD4HVNVPROC = CFUNCTYPE(None, GLenum, POINTER(GLhalfNV)) 	# GL/glext.h:6624
PFNGLFOGCOORDHNVPROC = CFUNCTYPE(None, GLhalfNV) 	# GL/glext.h:6625
PFNGLFOGCOORDHVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:6626
PFNGLSECONDARYCOLOR3HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:6627
PFNGLSECONDARYCOLOR3HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:6628
PFNGLVERTEXWEIGHTHNVPROC = CFUNCTYPE(None, GLhalfNV) 	# GL/glext.h:6629
PFNGLVERTEXWEIGHTHVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:6630
PFNGLVERTEXATTRIB1HNVPROC = CFUNCTYPE(None, GLuint, GLhalfNV) 	# GL/glext.h:6631
PFNGLVERTEXATTRIB1HVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLhalfNV)) 	# GL/glext.h:6632
PFNGLVERTEXATTRIB2HNVPROC = CFUNCTYPE(None, GLuint, GLhalfNV, GLhalfNV) 	# GL/glext.h:6633
PFNGLVERTEXATTRIB2HVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLhalfNV)) 	# GL/glext.h:6634
PFNGLVERTEXATTRIB3HNVPROC = CFUNCTYPE(None, GLuint, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:6635
PFNGLVERTEXATTRIB3HVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLhalfNV)) 	# GL/glext.h:6636
PFNGLVERTEXATTRIB4HNVPROC = CFUNCTYPE(None, GLuint, GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:6637
PFNGLVERTEXATTRIB4HVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLhalfNV)) 	# GL/glext.h:6638
PFNGLVERTEXATTRIBS1HVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLhalfNV)) 	# GL/glext.h:6639
PFNGLVERTEXATTRIBS2HVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLhalfNV)) 	# GL/glext.h:6640
PFNGLVERTEXATTRIBS3HVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLhalfNV)) 	# GL/glext.h:6641
PFNGLVERTEXATTRIBS4HVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLhalfNV)) 	# GL/glext.h:6642
# NV_pixel_data_range (GL/glext.h:6645)
GL_NV_pixel_data_range = 1 	# GL/glext.h:6646
# GL/glext.h:6648
glPixelDataRangeNV = _link_function('glPixelDataRangeNV', None, [GLenum, GLsizei, POINTER(GLvoid)], 'NV_pixel_data_range')

# GL/glext.h:6649
glFlushPixelDataRangeNV = _link_function('glFlushPixelDataRangeNV', None, [GLenum], 'NV_pixel_data_range')

PFNGLPIXELDATARANGENVPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:6651
PFNGLFLUSHPIXELDATARANGENVPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:6652
# NV_primitive_restart (GL/glext.h:6655)
GL_NV_primitive_restart = 1 	# GL/glext.h:6656
# GL/glext.h:6658
glPrimitiveRestartNV = _link_function('glPrimitiveRestartNV', None, [], 'NV_primitive_restart')

# GL/glext.h:6659
glPrimitiveRestartIndexNV = _link_function('glPrimitiveRestartIndexNV', None, [GLuint], 'NV_primitive_restart')

PFNGLPRIMITIVERESTARTNVPROC = CFUNCTYPE(None) 	# GL/glext.h:6661
PFNGLPRIMITIVERESTARTINDEXNVPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6662
# NV_texture_expand_normal (GL/glext.h:6665)
GL_NV_texture_expand_normal = 1 	# GL/glext.h:6666
# NV_vertex_program2 (GL/glext.h:6669)
GL_NV_vertex_program2 = 1 	# GL/glext.h:6670
# ATI_map_object_buffer (GL/glext.h:6673)
GL_ATI_map_object_buffer = 1 	# GL/glext.h:6674
# GL/glext.h:6676
glMapObjectBufferATI = _link_function('glMapObjectBufferATI', POINTER(GLvoid), [GLuint], 'ATI_map_object_buffer')

# GL/glext.h:6677
glUnmapObjectBufferATI = _link_function('glUnmapObjectBufferATI', None, [GLuint], 'ATI_map_object_buffer')

PFNGLMAPOBJECTBUFFERATIPROC = CFUNCTYPE(POINTER(GLvoid), GLuint) 	# GL/glext.h:6679
PFNGLUNMAPOBJECTBUFFERATIPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6680
# ATI_separate_stencil (GL/glext.h:6683)
GL_ATI_separate_stencil = 1 	# GL/glext.h:6684
# GL/glext.h:6686
glStencilOpSeparateATI = _link_function('glStencilOpSeparateATI', None, [GLenum, GLenum, GLenum, GLenum], 'ATI_separate_stencil')

# GL/glext.h:6687
glStencilFuncSeparateATI = _link_function('glStencilFuncSeparateATI', None, [GLenum, GLenum, GLint, GLuint], 'ATI_separate_stencil')

PFNGLSTENCILOPSEPARATEATIPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:6689
PFNGLSTENCILFUNCSEPARATEATIPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLuint) 	# GL/glext.h:6690
# ATI_vertex_attrib_array_object (GL/glext.h:6693)
GL_ATI_vertex_attrib_array_object = 1 	# GL/glext.h:6694
# GL/glext.h:6696
glVertexAttribArrayObjectATI = _link_function('glVertexAttribArrayObjectATI', None, [GLuint, GLint, GLenum, GLboolean, GLsizei, GLuint, GLuint], 'ATI_vertex_attrib_array_object')

# GL/glext.h:6697
glGetVertexAttribArrayObjectfvATI = _link_function('glGetVertexAttribArrayObjectfvATI', None, [GLuint, GLenum, POINTER(GLfloat)], 'ATI_vertex_attrib_array_object')

# GL/glext.h:6698
glGetVertexAttribArrayObjectivATI = _link_function('glGetVertexAttribArrayObjectivATI', None, [GLuint, GLenum, POINTER(GLint)], 'ATI_vertex_attrib_array_object')

PFNGLVERTEXATTRIBARRAYOBJECTATIPROC = CFUNCTYPE(None, GLuint, GLint, GLenum, GLboolean, GLsizei, GLuint, GLuint) 	# GL/glext.h:6700
PFNGLGETVERTEXATTRIBARRAYOBJECTFVATIPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6701
PFNGLGETVERTEXATTRIBARRAYOBJECTIVATIPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:6702
# OES_read_format (GL/glext.h:6705)
GL_OES_read_format = 1 	# GL/glext.h:6706
# EXT_depth_bounds_test (GL/glext.h:6709)
GL_EXT_depth_bounds_test = 1 	# GL/glext.h:6710
GLclampd = c_double 	# /usr/include/GL/gl.h:66
# GL/glext.h:6712
glDepthBoundsEXT = _link_function('glDepthBoundsEXT', None, [GLclampd, GLclampd], 'EXT_depth_bounds_test')

PFNGLDEPTHBOUNDSEXTPROC = CFUNCTYPE(None, GLclampd, GLclampd) 	# GL/glext.h:6714
# EXT_texture_mirror_clamp (GL/glext.h:6717)
GL_EXT_texture_mirror_clamp = 1 	# GL/glext.h:6718
# EXT_blend_equation_separate (GL/glext.h:6721)
GL_EXT_blend_equation_separate = 1 	# GL/glext.h:6722
# GL/glext.h:6724
glBlendEquationSeparateEXT = _link_function('glBlendEquationSeparateEXT', None, [GLenum, GLenum], 'EXT_blend_equation_separate')

PFNGLBLENDEQUATIONSEPARATEEXTPROC = CFUNCTYPE(None, GLenum, GLenum) 	# GL/glext.h:6726
# MESA_pack_invert (GL/glext.h:6729)
GL_MESA_pack_invert = 1 	# GL/glext.h:6730
# MESA_ycbcr_texture (GL/glext.h:6733)
GL_MESA_ycbcr_texture = 1 	# GL/glext.h:6734
# EXT_pixel_buffer_object (GL/glext.h:6737)
GL_EXT_pixel_buffer_object = 1 	# GL/glext.h:6738
# NV_fragment_program_option (GL/glext.h:6741)
GL_NV_fragment_program_option = 1 	# GL/glext.h:6742
# NV_fragment_program2 (GL/glext.h:6745)
GL_NV_fragment_program2 = 1 	# GL/glext.h:6746
# NV_vertex_program2_option (GL/glext.h:6749)
GL_NV_vertex_program2_option = 1 	# GL/glext.h:6750
# NV_vertex_program3 (GL/glext.h:6753)
GL_NV_vertex_program3 = 1 	# GL/glext.h:6754
# EXT_framebuffer_object (GL/glext.h:6757)
GL_EXT_framebuffer_object = 1 	# GL/glext.h:6758
# GL/glext.h:6760
glIsRenderbufferEXT = _link_function('glIsRenderbufferEXT', GLboolean, [GLuint], 'EXT_framebuffer_object')

# GL/glext.h:6761
glBindRenderbufferEXT = _link_function('glBindRenderbufferEXT', None, [GLenum, GLuint], 'EXT_framebuffer_object')

# GL/glext.h:6762
glDeleteRenderbuffersEXT = _link_function('glDeleteRenderbuffersEXT', None, [GLsizei, POINTER(GLuint)], 'EXT_framebuffer_object')

# GL/glext.h:6763
glGenRenderbuffersEXT = _link_function('glGenRenderbuffersEXT', None, [GLsizei, POINTER(GLuint)], 'EXT_framebuffer_object')

# GL/glext.h:6764
glRenderbufferStorageEXT = _link_function('glRenderbufferStorageEXT', None, [GLenum, GLenum, GLsizei, GLsizei], 'EXT_framebuffer_object')

# GL/glext.h:6765
glGetRenderbufferParameterivEXT = _link_function('glGetRenderbufferParameterivEXT', None, [GLenum, GLenum, POINTER(GLint)], 'EXT_framebuffer_object')

# GL/glext.h:6766
glIsFramebufferEXT = _link_function('glIsFramebufferEXT', GLboolean, [GLuint], 'EXT_framebuffer_object')

# GL/glext.h:6767
glBindFramebufferEXT = _link_function('glBindFramebufferEXT', None, [GLenum, GLuint], 'EXT_framebuffer_object')

# GL/glext.h:6768
glDeleteFramebuffersEXT = _link_function('glDeleteFramebuffersEXT', None, [GLsizei, POINTER(GLuint)], 'EXT_framebuffer_object')

# GL/glext.h:6769
glGenFramebuffersEXT = _link_function('glGenFramebuffersEXT', None, [GLsizei, POINTER(GLuint)], 'EXT_framebuffer_object')

# GL/glext.h:6770
glCheckFramebufferStatusEXT = _link_function('glCheckFramebufferStatusEXT', GLenum, [GLenum], 'EXT_framebuffer_object')

# GL/glext.h:6771
glFramebufferTexture1DEXT = _link_function('glFramebufferTexture1DEXT', None, [GLenum, GLenum, GLenum, GLuint, GLint], 'EXT_framebuffer_object')

# GL/glext.h:6772
glFramebufferTexture2DEXT = _link_function('glFramebufferTexture2DEXT', None, [GLenum, GLenum, GLenum, GLuint, GLint], 'EXT_framebuffer_object')

# GL/glext.h:6773
glFramebufferTexture3DEXT = _link_function('glFramebufferTexture3DEXT', None, [GLenum, GLenum, GLenum, GLuint, GLint, GLint], 'EXT_framebuffer_object')

# GL/glext.h:6774
glFramebufferRenderbufferEXT = _link_function('glFramebufferRenderbufferEXT', None, [GLenum, GLenum, GLenum, GLuint], 'EXT_framebuffer_object')

# GL/glext.h:6775
glGetFramebufferAttachmentParameterivEXT = _link_function('glGetFramebufferAttachmentParameterivEXT', None, [GLenum, GLenum, GLenum, POINTER(GLint)], 'EXT_framebuffer_object')

# GL/glext.h:6776
glGenerateMipmapEXT = _link_function('glGenerateMipmapEXT', None, [GLenum], 'EXT_framebuffer_object')

PFNGLISRENDERBUFFEREXTPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:6778
PFNGLBINDRENDERBUFFEREXTPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:6779
PFNGLDELETERENDERBUFFERSEXTPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6780
PFNGLGENRENDERBUFFERSEXTPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6781
PFNGLRENDERBUFFERSTORAGEEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLsizei) 	# GL/glext.h:6782
PFNGLGETRENDERBUFFERPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:6783
PFNGLISFRAMEBUFFEREXTPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:6784
PFNGLBINDFRAMEBUFFEREXTPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:6785
PFNGLDELETEFRAMEBUFFERSEXTPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6786
PFNGLGENFRAMEBUFFERSEXTPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6787
PFNGLCHECKFRAMEBUFFERSTATUSEXTPROC = CFUNCTYPE(GLenum, GLenum) 	# GL/glext.h:6788
PFNGLFRAMEBUFFERTEXTURE1DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLuint, GLint) 	# GL/glext.h:6789
PFNGLFRAMEBUFFERTEXTURE2DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLuint, GLint) 	# GL/glext.h:6790
PFNGLFRAMEBUFFERTEXTURE3DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLuint, GLint, GLint) 	# GL/glext.h:6791
PFNGLFRAMEBUFFERRENDERBUFFEREXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLuint) 	# GL/glext.h:6792
PFNGLGETFRAMEBUFFERATTACHMENTPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:6793
PFNGLGENERATEMIPMAPEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:6794
# GREMEDY_string_marker (GL/glext.h:6797)
GL_GREMEDY_string_marker = 1 	# GL/glext.h:6798
# GL/glext.h:6800
glStringMarkerGREMEDY = _link_function('glStringMarkerGREMEDY', None, [GLsizei, POINTER(GLvoid)], 'GREMEDY_string_marker')

PFNGLSTRINGMARKERGREMEDYPROC = CFUNCTYPE(None, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:6802
# EXT_Cg_shader (GL/glext.h:6805)
GL_EXT_Cg_shader = 1 	# GL/glext.h:6806
# EXT_timer_query (GL/glext.h:6809)
GL_EXT_timer_query = 1 	# GL/glext.h:6810
# GL/glext.h:6812
glGetQueryObjecti64vEXT = _link_function('glGetQueryObjecti64vEXT', None, [GLuint, GLenum, POINTER(GLint64EXT)], 'EXT_timer_query')

# GL/glext.h:6813
glGetQueryObjectui64vEXT = _link_function('glGetQueryObjectui64vEXT', None, [GLuint, GLenum, POINTER(GLuint64EXT)], 'EXT_timer_query')

PFNGLGETQUERYOBJECTI64VEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint64EXT)) 	# GL/glext.h:6815
PFNGLGETQUERYOBJECTUI64VEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLuint64EXT)) 	# GL/glext.h:6816
# EXT_texture_buffer_object (GL/glext.h:6819)
GL_EXT_texture_buffer_object = 1 	# GL/glext.h:6820
# GL/glext.h:6822
glTexBufferEXT = _link_function('glTexBufferEXT', None, [GLenum, GLenum, GLuint], 'EXT_texture_buffer_object')

PFNGLTEXBUFFEREXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLuint) 	# GL/glext.h:6824
# NV_transform_feedback (GL/glext.h:6827)
GL_NV_transform_feedback = 1 	# GL/glext.h:6828
# GL/glext.h:6830
glBeginTransformFeedbackNV = _link_function('glBeginTransformFeedbackNV', None, [GLenum], 'NV_transform_feedback')

# GL/glext.h:6831
glEndTransformFeedbackNV = _link_function('glEndTransformFeedbackNV', None, [], 'NV_transform_feedback')

# GL/glext.h:6832
glTransformFeedbackAttribsNV = _link_function('glTransformFeedbackAttribsNV', None, [GLuint, POINTER(GLint), GLenum], 'NV_transform_feedback')

# GL/glext.h:6833
glBindBufferRangeNV = _link_function('glBindBufferRangeNV', None, [GLenum, GLuint, GLuint, GLintptr, GLsizeiptr], 'NV_transform_feedback')

# GL/glext.h:6834
glBindBufferOffsetNV = _link_function('glBindBufferOffsetNV', None, [GLenum, GLuint, GLuint, GLintptr], 'NV_transform_feedback')

# GL/glext.h:6835
glBindBufferBaseNV = _link_function('glBindBufferBaseNV', None, [GLenum, GLuint, GLuint], 'NV_transform_feedback')

# GL/glext.h:6836
glTransformFeedbackVaryingsNV = _link_function('glTransformFeedbackVaryingsNV', None, [GLuint, GLsizei, POINTER(GLint), GLenum], 'NV_transform_feedback')

# GL/glext.h:6837
glActiveVaryingNV = _link_function('glActiveVaryingNV', None, [GLuint, POINTER(GLchar)], 'NV_transform_feedback')

# GL/glext.h:6838
glGetVaryingLocationNV = _link_function('glGetVaryingLocationNV', GLint, [GLuint, POINTER(GLchar)], 'NV_transform_feedback')

# GL/glext.h:6839
glGetActiveVaryingNV = _link_function('glGetActiveVaryingNV', None, [GLuint, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLsizei), POINTER(GLenum), POINTER(GLchar)], 'NV_transform_feedback')

# GL/glext.h:6840
glGetTransformFeedbackVaryingNV = _link_function('glGetTransformFeedbackVaryingNV', None, [GLuint, GLuint, POINTER(GLint)], 'NV_transform_feedback')

PFNGLBEGINTRANSFORMFEEDBACKNVPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:6842
PFNGLENDTRANSFORMFEEDBACKNVPROC = CFUNCTYPE(None) 	# GL/glext.h:6843
PFNGLTRANSFORMFEEDBACKATTRIBSNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLint), GLenum) 	# GL/glext.h:6844
PFNGLBINDBUFFERRANGENVPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLintptr, GLsizeiptr) 	# GL/glext.h:6845
PFNGLBINDBUFFEROFFSETNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLintptr) 	# GL/glext.h:6846
PFNGLBINDBUFFERBASENVPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint) 	# GL/glext.h:6847
PFNGLTRANSFORMFEEDBACKVARYINGSNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLint), GLenum) 	# GL/glext.h:6848
PFNGLACTIVEVARYINGNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLchar)) 	# GL/glext.h:6849
PFNGLGETVARYINGLOCATIONNVPROC = CFUNCTYPE(GLint, GLuint, POINTER(GLchar)) 	# GL/glext.h:6850
PFNGLGETACTIVEVARYINGNVPROC = CFUNCTYPE(None, GLuint, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLsizei), POINTER(GLenum), POINTER(GLchar)) 	# GL/glext.h:6851
PFNGLGETTRANSFORMFEEDBACKVARYINGNVPROC = CFUNCTYPE(None, GLuint, GLuint, POINTER(GLint)) 	# GL/glext.h:6852
# NV_depth_buffer_float (GL/glext.h:6856)
GL_NV_depth_buffer_float = 1 	# GL/glext.h:6857
# GL/glext.h:6859
glDepthRangedNV = _link_function('glDepthRangedNV', None, [GLdouble, GLdouble], 'NV_depth_buffer_float')

# GL/glext.h:6860
glClearDepthdNV = _link_function('glClearDepthdNV', None, [GLdouble], 'NV_depth_buffer_float')

# GL/glext.h:6861
glDepthBoundsdNV = _link_function('glDepthBoundsdNV', None, [GLdouble, GLdouble], 'NV_depth_buffer_float')

PFNGLDEPTHRANGEDNVPROC = CFUNCTYPE(None, GLdouble, GLdouble) 	# GL/glext.h:6863
PFNGLCLEARDEPTHDNVPROC = CFUNCTYPE(None, GLdouble) 	# GL/glext.h:6864
PFNGLDEPTHBOUNDSDNVPROC = CFUNCTYPE(None, GLdouble, GLdouble) 	# GL/glext.h:6865
# EXT_texture_compression_latc (GL/glext.h:6868)
GL_EXT_texture_compression_latc = 1 	# GL/glext.h:6869
# EXT_framebuffer_sRGB (GL/glext.h:6872)
GL_EXT_framebuffer_sRGB = 1 	# GL/glext.h:6873
# EXT_texture_shared_exponent (GL/glext.h:6876)
GL_EXT_texture_shared_exponent = 1 	# GL/glext.h:6877
# EXT_packed_float (GL/glext.h:6880)
GL_EXT_packed_float = 1 	# GL/glext.h:6881
# EXT_texture_array (GL/glext.h:6884)
GL_EXT_texture_array = 1 	# GL/glext.h:6885
# EXT_draw_buffers2 (GL/glext.h:6888)
GL_EXT_draw_buffers2 = 1 	# GL/glext.h:6889
# GL/glext.h:6891
glColorMaskIndexedEXT = _link_function('glColorMaskIndexedEXT', None, [GLuint, GLboolean, GLboolean, GLboolean, GLboolean], 'EXT_draw_buffers2')

# GL/glext.h:6892
glGetBooleanIndexedvEXT = _link_function('glGetBooleanIndexedvEXT', None, [GLenum, GLuint, POINTER(GLboolean)], 'EXT_draw_buffers2')

# GL/glext.h:6893
glGetIntegerIndexedvEXT = _link_function('glGetIntegerIndexedvEXT', None, [GLenum, GLuint, POINTER(GLint)], 'EXT_draw_buffers2')

# GL/glext.h:6894
glEnableIndexedEXT = _link_function('glEnableIndexedEXT', None, [GLenum, GLuint], 'EXT_draw_buffers2')

# GL/glext.h:6895
glDisableIndexedEXT = _link_function('glDisableIndexedEXT', None, [GLenum, GLuint], 'EXT_draw_buffers2')

# GL/glext.h:6896
glIsEnabledIndexedEXT = _link_function('glIsEnabledIndexedEXT', GLboolean, [GLenum, GLuint], 'EXT_draw_buffers2')

PFNGLCOLORMASKINDEXEDEXTPROC = CFUNCTYPE(None, GLuint, GLboolean, GLboolean, GLboolean, GLboolean) 	# GL/glext.h:6898
PFNGLGETBOOLEANINDEXEDVEXTPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLboolean)) 	# GL/glext.h:6899
PFNGLGETINTEGERINDEXEDVEXTPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLint)) 	# GL/glext.h:6900
PFNGLENABLEINDEXEDEXTPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:6901
PFNGLDISABLEINDEXEDEXTPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:6902
PFNGLISENABLEDINDEXEDEXTPROC = CFUNCTYPE(GLboolean, GLenum, GLuint) 	# GL/glext.h:6903
# EXT_texture_integer (GL/glext.h:6906)
GL_EXT_texture_integer = 1 	# GL/glext.h:6907
# GL/glext.h:6909
glTexParameterIivEXT = _link_function('glTexParameterIivEXT', None, [GLenum, GLenum, POINTER(GLint)], 'EXT_texture_integer')

# GL/glext.h:6910
glTexParameterIuivEXT = _link_function('glTexParameterIuivEXT', None, [GLenum, GLenum, POINTER(GLuint)], 'EXT_texture_integer')

# GL/glext.h:6911
glGetTexParameterIivEXT = _link_function('glGetTexParameterIivEXT', None, [GLenum, GLenum, POINTER(GLint)], 'EXT_texture_integer')

# GL/glext.h:6912
glGetTexParameterIuivEXT = _link_function('glGetTexParameterIuivEXT', None, [GLenum, GLenum, POINTER(GLuint)], 'EXT_texture_integer')

# GL/glext.h:6913
glClearColorIiEXT = _link_function('glClearColorIiEXT', None, [GLint, GLint, GLint, GLint], 'EXT_texture_integer')

# GL/glext.h:6914
glClearColorIuiEXT = _link_function('glClearColorIuiEXT', None, [GLuint, GLuint, GLuint, GLuint], 'EXT_texture_integer')

PFNGLTEXPARAMETERIIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:6916
PFNGLTEXPARAMETERIUIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLuint)) 	# GL/glext.h:6917
PFNGLGETTEXPARAMETERIIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:6918
PFNGLGETTEXPARAMETERIUIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLuint)) 	# GL/glext.h:6919
PFNGLCLEARCOLORIIEXTPROC = CFUNCTYPE(None, GLint, GLint, GLint, GLint) 	# GL/glext.h:6920
PFNGLCLEARCOLORIUIEXTPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:6921
# EXT_bindable_uniform (GL/glext.h:6924)
GL_EXT_bindable_uniform = 1 	# GL/glext.h:6925
# GL/glext.h:6927
glUniformBufferEXT = _link_function('glUniformBufferEXT', None, [GLuint, GLint, GLuint], 'EXT_bindable_uniform')

# GL/glext.h:6928
glGetUniformBufferSizeEXT = _link_function('glGetUniformBufferSizeEXT', GLint, [GLuint, GLint], 'EXT_bindable_uniform')

# GL/glext.h:6929
glGetUniformOffsetEXT = _link_function('glGetUniformOffsetEXT', GLintptr, [GLuint, GLint], 'EXT_bindable_uniform')

PFNGLUNIFORMBUFFEREXTPROC = CFUNCTYPE(None, GLuint, GLint, GLuint) 	# GL/glext.h:6931
PFNGLGETUNIFORMBUFFERSIZEEXTPROC = CFUNCTYPE(GLint, GLuint, GLint) 	# GL/glext.h:6932
PFNGLGETUNIFORMOFFSETEXTPROC = CFUNCTYPE(GLintptr, GLuint, GLint) 	# GL/glext.h:6933
# EXT_gpu_shader4 (GL/glext.h:6936)
GL_EXT_gpu_shader4 = 1 	# GL/glext.h:6937
# GL/glext.h:6939
glGetUniformuivEXT = _link_function('glGetUniformuivEXT', None, [GLuint, GLint, POINTER(GLuint)], 'EXT_gpu_shader4')

# GL/glext.h:6940
glBindFragDataLocationEXT = _link_function('glBindFragDataLocationEXT', None, [GLuint, GLuint, POINTER(GLchar)], 'EXT_gpu_shader4')

# GL/glext.h:6941
glGetFragDataLocationEXT = _link_function('glGetFragDataLocationEXT', GLint, [GLuint, POINTER(GLchar)], 'EXT_gpu_shader4')

# GL/glext.h:6942
glUniform1uiEXT = _link_function('glUniform1uiEXT', None, [GLint, GLuint], 'EXT_gpu_shader4')

# GL/glext.h:6943
glUniform2uiEXT = _link_function('glUniform2uiEXT', None, [GLint, GLuint, GLuint], 'EXT_gpu_shader4')

# GL/glext.h:6944
glUniform3uiEXT = _link_function('glUniform3uiEXT', None, [GLint, GLuint, GLuint, GLuint], 'EXT_gpu_shader4')

# GL/glext.h:6945
glUniform4uiEXT = _link_function('glUniform4uiEXT', None, [GLint, GLuint, GLuint, GLuint, GLuint], 'EXT_gpu_shader4')

# GL/glext.h:6946
glUniform1uivEXT = _link_function('glUniform1uivEXT', None, [GLint, GLsizei, POINTER(GLuint)], 'EXT_gpu_shader4')

# GL/glext.h:6947
glUniform2uivEXT = _link_function('glUniform2uivEXT', None, [GLint, GLsizei, POINTER(GLuint)], 'EXT_gpu_shader4')

# GL/glext.h:6948
glUniform3uivEXT = _link_function('glUniform3uivEXT', None, [GLint, GLsizei, POINTER(GLuint)], 'EXT_gpu_shader4')

# GL/glext.h:6949
glUniform4uivEXT = _link_function('glUniform4uivEXT', None, [GLint, GLsizei, POINTER(GLuint)], 'EXT_gpu_shader4')

# GL/glext.h:6950
glVertexAttribI1iEXT = _link_function('glVertexAttribI1iEXT', None, [GLuint, GLint], 'EXT_gpu_shader4')

# GL/glext.h:6951
glVertexAttribI2iEXT = _link_function('glVertexAttribI2iEXT', None, [GLuint, GLint, GLint], 'EXT_gpu_shader4')

# GL/glext.h:6952
glVertexAttribI3iEXT = _link_function('glVertexAttribI3iEXT', None, [GLuint, GLint, GLint, GLint], 'EXT_gpu_shader4')

# GL/glext.h:6953
glVertexAttribI4iEXT = _link_function('glVertexAttribI4iEXT', None, [GLuint, GLint, GLint, GLint, GLint], 'EXT_gpu_shader4')

# GL/glext.h:6954
glVertexAttribI1uiEXT = _link_function('glVertexAttribI1uiEXT', None, [GLuint, GLuint], 'EXT_gpu_shader4')

# GL/glext.h:6955
glVertexAttribI2uiEXT = _link_function('glVertexAttribI2uiEXT', None, [GLuint, GLuint, GLuint], 'EXT_gpu_shader4')

# GL/glext.h:6956
glVertexAttribI3uiEXT = _link_function('glVertexAttribI3uiEXT', None, [GLuint, GLuint, GLuint, GLuint], 'EXT_gpu_shader4')

# GL/glext.h:6957
glVertexAttribI4uiEXT = _link_function('glVertexAttribI4uiEXT', None, [GLuint, GLuint, GLuint, GLuint, GLuint], 'EXT_gpu_shader4')

# GL/glext.h:6958
glVertexAttribI1ivEXT = _link_function('glVertexAttribI1ivEXT', None, [GLuint, POINTER(GLint)], 'EXT_gpu_shader4')

# GL/glext.h:6959
glVertexAttribI2ivEXT = _link_function('glVertexAttribI2ivEXT', None, [GLuint, POINTER(GLint)], 'EXT_gpu_shader4')

# GL/glext.h:6960
glVertexAttribI3ivEXT = _link_function('glVertexAttribI3ivEXT', None, [GLuint, POINTER(GLint)], 'EXT_gpu_shader4')

# GL/glext.h:6961
glVertexAttribI4ivEXT = _link_function('glVertexAttribI4ivEXT', None, [GLuint, POINTER(GLint)], 'EXT_gpu_shader4')

# GL/glext.h:6962
glVertexAttribI1uivEXT = _link_function('glVertexAttribI1uivEXT', None, [GLuint, POINTER(GLuint)], 'EXT_gpu_shader4')

# GL/glext.h:6963
glVertexAttribI2uivEXT = _link_function('glVertexAttribI2uivEXT', None, [GLuint, POINTER(GLuint)], 'EXT_gpu_shader4')

# GL/glext.h:6964
glVertexAttribI3uivEXT = _link_function('glVertexAttribI3uivEXT', None, [GLuint, POINTER(GLuint)], 'EXT_gpu_shader4')

# GL/glext.h:6965
glVertexAttribI4uivEXT = _link_function('glVertexAttribI4uivEXT', None, [GLuint, POINTER(GLuint)], 'EXT_gpu_shader4')

# GL/glext.h:6966
glVertexAttribI4bvEXT = _link_function('glVertexAttribI4bvEXT', None, [GLuint, POINTER(GLbyte)], 'EXT_gpu_shader4')

# GL/glext.h:6967
glVertexAttribI4svEXT = _link_function('glVertexAttribI4svEXT', None, [GLuint, POINTER(GLshort)], 'EXT_gpu_shader4')

# GL/glext.h:6968
glVertexAttribI4ubvEXT = _link_function('glVertexAttribI4ubvEXT', None, [GLuint, POINTER(GLubyte)], 'EXT_gpu_shader4')

# GL/glext.h:6969
glVertexAttribI4usvEXT = _link_function('glVertexAttribI4usvEXT', None, [GLuint, POINTER(GLushort)], 'EXT_gpu_shader4')

# GL/glext.h:6970
glVertexAttribIPointerEXT = _link_function('glVertexAttribIPointerEXT', None, [GLuint, GLint, GLenum, GLsizei, POINTER(GLvoid)], 'EXT_gpu_shader4')

# GL/glext.h:6971
glGetVertexAttribIivEXT = _link_function('glGetVertexAttribIivEXT', None, [GLuint, GLenum, POINTER(GLint)], 'EXT_gpu_shader4')

# GL/glext.h:6972
glGetVertexAttribIuivEXT = _link_function('glGetVertexAttribIuivEXT', None, [GLuint, GLenum, POINTER(GLuint)], 'EXT_gpu_shader4')

PFNGLGETUNIFORMUIVEXTPROC = CFUNCTYPE(None, GLuint, GLint, POINTER(GLuint)) 	# GL/glext.h:6974
PFNGLBINDFRAGDATALOCATIONEXTPROC = CFUNCTYPE(None, GLuint, GLuint, POINTER(GLchar)) 	# GL/glext.h:6975
PFNGLGETFRAGDATALOCATIONEXTPROC = CFUNCTYPE(GLint, GLuint, POINTER(GLchar)) 	# GL/glext.h:6976
PFNGLUNIFORM1UIEXTPROC = CFUNCTYPE(None, GLint, GLuint) 	# GL/glext.h:6977
PFNGLUNIFORM2UIEXTPROC = CFUNCTYPE(None, GLint, GLuint, GLuint) 	# GL/glext.h:6978
PFNGLUNIFORM3UIEXTPROC = CFUNCTYPE(None, GLint, GLuint, GLuint, GLuint) 	# GL/glext.h:6979
PFNGLUNIFORM4UIEXTPROC = CFUNCTYPE(None, GLint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:6980
PFNGLUNIFORM1UIVEXTPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6981
PFNGLUNIFORM2UIVEXTPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6982
PFNGLUNIFORM3UIVEXTPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6983
PFNGLUNIFORM4UIVEXTPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6984
PFNGLVERTEXATTRIBI1IEXTPROC = CFUNCTYPE(None, GLuint, GLint) 	# GL/glext.h:6985
PFNGLVERTEXATTRIBI2IEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLint) 	# GL/glext.h:6986
PFNGLVERTEXATTRIBI3IEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLint, GLint) 	# GL/glext.h:6987
PFNGLVERTEXATTRIBI4IEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLint, GLint, GLint) 	# GL/glext.h:6988
PFNGLVERTEXATTRIBI1UIEXTPROC = CFUNCTYPE(None, GLuint, GLuint) 	# GL/glext.h:6989
PFNGLVERTEXATTRIBI2UIEXTPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint) 	# GL/glext.h:6990
PFNGLVERTEXATTRIBI3UIEXTPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:6991
PFNGLVERTEXATTRIBI4UIEXTPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:6992
PFNGLVERTEXATTRIBI1IVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:6993
PFNGLVERTEXATTRIBI2IVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:6994
PFNGLVERTEXATTRIBI3IVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:6995
PFNGLVERTEXATTRIBI4IVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:6996
PFNGLVERTEXATTRIBI1UIVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:6997
PFNGLVERTEXATTRIBI2UIVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:6998
PFNGLVERTEXATTRIBI3UIVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:6999
PFNGLVERTEXATTRIBI4UIVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:7000
PFNGLVERTEXATTRIBI4BVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLbyte)) 	# GL/glext.h:7001
PFNGLVERTEXATTRIBI4SVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:7002
PFNGLVERTEXATTRIBI4UBVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLubyte)) 	# GL/glext.h:7003
PFNGLVERTEXATTRIBI4USVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLushort)) 	# GL/glext.h:7004
PFNGLVERTEXATTRIBIPOINTEREXTPROC = CFUNCTYPE(None, GLuint, GLint, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:7005
PFNGLGETVERTEXATTRIBIIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:7006
PFNGLGETVERTEXATTRIBIUIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLuint)) 	# GL/glext.h:7007
# EXT_geometry_shader4 (GL/glext.h:7011)
GL_EXT_geometry_shader4 = 1 	# GL/glext.h:7012
# GL/glext.h:7014
glProgramParameteriEXT = _link_function('glProgramParameteriEXT', None, [GLuint, GLenum, GLint], 'EXT_geometry_shader4')

# GL/glext.h:7015
glFramebufferTextureEXT = _link_function('glFramebufferTextureEXT', None, [GLenum, GLenum, GLuint, GLint], 'EXT_geometry_shader4')

# GL/glext.h:7016
glFramebufferTextureLayerEXT = _link_function('glFramebufferTextureLayerEXT', None, [GLenum, GLenum, GLuint, GLint, GLint], 'EXT_geometry_shader4')

# GL/glext.h:7017
glFramebufferTextureFaceEXT = _link_function('glFramebufferTextureFaceEXT', None, [GLenum, GLenum, GLuint, GLint, GLenum], 'EXT_geometry_shader4')

PFNGLPROGRAMPARAMETERIEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint) 	# GL/glext.h:7019
PFNGLFRAMEBUFFERTEXTUREEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLuint, GLint) 	# GL/glext.h:7020
PFNGLFRAMEBUFFERTEXTURELAYEREXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLuint, GLint, GLint) 	# GL/glext.h:7021
PFNGLFRAMEBUFFERTEXTUREFACEEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLuint, GLint, GLenum) 	# GL/glext.h:7022
# NV_geometry_program4 (GL/glext.h:7025)
GL_NV_geometry_program4 = 1 	# GL/glext.h:7026
# GL/glext.h:7028
glProgramVertexLimitNV = _link_function('glProgramVertexLimitNV', None, [GLenum, GLint], 'NV_geometry_program4')

PFNGLPROGRAMVERTEXLIMITNVPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:7030
# NV_gpu_program4 (GL/glext.h:7033)
GL_NV_gpu_program4 = 1 	# GL/glext.h:7034
# GL/glext.h:7036
glProgramLocalParameterI4iNV = _link_function('glProgramLocalParameterI4iNV', None, [GLenum, GLuint, GLint, GLint, GLint, GLint], 'NV_gpu_program4')

# GL/glext.h:7037
glProgramLocalParameterI4ivNV = _link_function('glProgramLocalParameterI4ivNV', None, [GLenum, GLuint, POINTER(GLint)], 'NV_gpu_program4')

# GL/glext.h:7038
glProgramLocalParametersI4ivNV = _link_function('glProgramLocalParametersI4ivNV', None, [GLenum, GLuint, GLsizei, POINTER(GLint)], 'NV_gpu_program4')

# GL/glext.h:7039
glProgramLocalParameterI4uiNV = _link_function('glProgramLocalParameterI4uiNV', None, [GLenum, GLuint, GLuint, GLuint, GLuint, GLuint], 'NV_gpu_program4')

# GL/glext.h:7040
glProgramLocalParameterI4uivNV = _link_function('glProgramLocalParameterI4uivNV', None, [GLenum, GLuint, POINTER(GLuint)], 'NV_gpu_program4')

# GL/glext.h:7041
glProgramLocalParametersI4uivNV = _link_function('glProgramLocalParametersI4uivNV', None, [GLenum, GLuint, GLsizei, POINTER(GLuint)], 'NV_gpu_program4')

# GL/glext.h:7042
glProgramEnvParameterI4iNV = _link_function('glProgramEnvParameterI4iNV', None, [GLenum, GLuint, GLint, GLint, GLint, GLint], 'NV_gpu_program4')

# GL/glext.h:7043
glProgramEnvParameterI4ivNV = _link_function('glProgramEnvParameterI4ivNV', None, [GLenum, GLuint, POINTER(GLint)], 'NV_gpu_program4')

# GL/glext.h:7044
glProgramEnvParametersI4ivNV = _link_function('glProgramEnvParametersI4ivNV', None, [GLenum, GLuint, GLsizei, POINTER(GLint)], 'NV_gpu_program4')

# GL/glext.h:7045
glProgramEnvParameterI4uiNV = _link_function('glProgramEnvParameterI4uiNV', None, [GLenum, GLuint, GLuint, GLuint, GLuint, GLuint], 'NV_gpu_program4')

# GL/glext.h:7046
glProgramEnvParameterI4uivNV = _link_function('glProgramEnvParameterI4uivNV', None, [GLenum, GLuint, POINTER(GLuint)], 'NV_gpu_program4')

# GL/glext.h:7047
glProgramEnvParametersI4uivNV = _link_function('glProgramEnvParametersI4uivNV', None, [GLenum, GLuint, GLsizei, POINTER(GLuint)], 'NV_gpu_program4')

# GL/glext.h:7048
glGetProgramLocalParameterIivNV = _link_function('glGetProgramLocalParameterIivNV', None, [GLenum, GLuint, POINTER(GLint)], 'NV_gpu_program4')

# GL/glext.h:7049
glGetProgramLocalParameterIuivNV = _link_function('glGetProgramLocalParameterIuivNV', None, [GLenum, GLuint, POINTER(GLuint)], 'NV_gpu_program4')

# GL/glext.h:7050
glGetProgramEnvParameterIivNV = _link_function('glGetProgramEnvParameterIivNV', None, [GLenum, GLuint, POINTER(GLint)], 'NV_gpu_program4')

# GL/glext.h:7051
glGetProgramEnvParameterIuivNV = _link_function('glGetProgramEnvParameterIuivNV', None, [GLenum, GLuint, POINTER(GLuint)], 'NV_gpu_program4')

PFNGLPROGRAMLOCALPARAMETERI4INVPROC = CFUNCTYPE(None, GLenum, GLuint, GLint, GLint, GLint, GLint) 	# GL/glext.h:7053
PFNGLPROGRAMLOCALPARAMETERI4IVNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLint)) 	# GL/glext.h:7054
PFNGLPROGRAMLOCALPARAMETERSI4IVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLsizei, POINTER(GLint)) 	# GL/glext.h:7055
PFNGLPROGRAMLOCALPARAMETERI4UINVPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:7056
PFNGLPROGRAMLOCALPARAMETERI4UIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLuint)) 	# GL/glext.h:7057
PFNGLPROGRAMLOCALPARAMETERSI4UIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:7058
PFNGLPROGRAMENVPARAMETERI4INVPROC = CFUNCTYPE(None, GLenum, GLuint, GLint, GLint, GLint, GLint) 	# GL/glext.h:7059
PFNGLPROGRAMENVPARAMETERI4IVNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLint)) 	# GL/glext.h:7060
PFNGLPROGRAMENVPARAMETERSI4IVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLsizei, POINTER(GLint)) 	# GL/glext.h:7061
PFNGLPROGRAMENVPARAMETERI4UINVPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:7062
PFNGLPROGRAMENVPARAMETERI4UIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLuint)) 	# GL/glext.h:7063
PFNGLPROGRAMENVPARAMETERSI4UIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:7064
PFNGLGETPROGRAMLOCALPARAMETERIIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLint)) 	# GL/glext.h:7065
PFNGLGETPROGRAMLOCALPARAMETERIUIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLuint)) 	# GL/glext.h:7066
PFNGLGETPROGRAMENVPARAMETERIIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLint)) 	# GL/glext.h:7067
PFNGLGETPROGRAMENVPARAMETERIUIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLuint)) 	# GL/glext.h:7068
# NV_parameter_buffer_object (GL/glext.h:7071)
GL_NV_parameter_buffer_object = 1 	# GL/glext.h:7072
# GL/glext.h:7074
glProgramBufferParametersfvNV = _link_function('glProgramBufferParametersfvNV', None, [GLenum, GLuint, GLuint, GLsizei, POINTER(GLfloat)], 'NV_parameter_buffer_object')

# GL/glext.h:7075
glProgramBufferParametersIivNV = _link_function('glProgramBufferParametersIivNV', None, [GLenum, GLuint, GLuint, GLsizei, POINTER(GLint)], 'NV_parameter_buffer_object')

# GL/glext.h:7076
glProgramBufferParametersIuivNV = _link_function('glProgramBufferParametersIuivNV', None, [GLenum, GLuint, GLuint, GLsizei, POINTER(GLuint)], 'NV_parameter_buffer_object')

PFNGLPROGRAMBUFFERPARAMETERSFVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:7078
PFNGLPROGRAMBUFFERPARAMETERSIIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLsizei, POINTER(GLint)) 	# GL/glext.h:7079
PFNGLPROGRAMBUFFERPARAMETERSIUIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:7080
# EXT_framebuffer_multisample (GL/glext.h:7083)
GL_EXT_framebuffer_multisample = 1 	# GL/glext.h:7084
# GL/glext.h:7086
glRenderbufferStorageMultisampleEXT = _link_function('glRenderbufferStorageMultisampleEXT', None, [GLenum, GLsizei, GLenum, GLsizei, GLsizei], 'EXT_framebuffer_multisample')

PFNGLRENDERBUFFERSTORAGEMULTISAMPLEEXTPROC = CFUNCTYPE(None, GLenum, GLsizei, GLenum, GLsizei, GLsizei) 	# GL/glext.h:7088
# NV_framebuffer_multisample_coverage (GL/glext.h:7091)
GL_NV_framebuffer_multisample_coverage = 1 	# GL/glext.h:7092
# GL/glext.h:7094
glRenderbufferStorageMultisampleCoverageNV = _link_function('glRenderbufferStorageMultisampleCoverageNV', None, [GLenum, GLsizei, GLsizei, GLenum, GLsizei, GLsizei], 'NV_framebuffer_multisample_coverage')

PFNGLRENDERBUFFERSTORAGEMULTISAMPLECOVERAGENVPROC = CFUNCTYPE(None, GLenum, GLsizei, GLsizei, GLenum, GLsizei, GLsizei) 	# GL/glext.h:7096
# EXT_framebuffer_blit (GL/glext.h:7099)
GL_EXT_framebuffer_blit = 1 	# GL/glext.h:7100
# GL/glext.h:7102
glBlitFramebufferEXT = _link_function('glBlitFramebufferEXT', None, [GLint, GLint, GLint, GLint, GLint, GLint, GLint, GLint, GLbitfield, GLenum], 'EXT_framebuffer_blit')

PFNGLBLITFRAMEBUFFEREXTPROC = CFUNCTYPE(None, GLint, GLint, GLint, GLint, GLint, GLint, GLint, GLint, GLbitfield, GLenum) 	# GL/glext.h:7104
# EXT_draw_instanced (GL/glext.h:7107)
GL_EXT_draw_instanced = 1 	# GL/glext.h:7108
# GL/glext.h:7110
glDrawArraysInstancedEXT = _link_function('glDrawArraysInstancedEXT', None, [GLenum, GLint, GLsizei, GLsizei], 'EXT_draw_instanced')

# GL/glext.h:7111
glDrawElementsInstancedEXT = _link_function('glDrawElementsInstancedEXT', None, [GLenum, GLsizei, GLenum, POINTER(GLvoid), GLsizei], 'EXT_draw_instanced')

PFNGLDRAWARRAYSINSTANCEDEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLsizei, GLsizei) 	# GL/glext.h:7113
PFNGLDRAWELEMENTSINSTANCEDEXTPROC = CFUNCTYPE(None, GLenum, GLsizei, GLenum, POINTER(GLvoid), GLsizei) 	# GL/glext.h:7114
# EXT_texture_compression_rgtc (GL/glext.h:7117)
GL_EXT_texture_compression_rgtc = 1 	# GL/glext.h:7118
# GLEXT_LEGACY (/usr/include/GL/gl.h:1633)

__all__ = ['GL_GLEXT_VERSION', 'GL_UNSIGNED_BYTE_3_3_2',
'GL_UNSIGNED_SHORT_4_4_4_4', 'GL_UNSIGNED_SHORT_5_5_5_1',
'GL_UNSIGNED_INT_8_8_8_8', 'GL_UNSIGNED_INT_10_10_10_2', 'GL_RESCALE_NORMAL',
'GL_TEXTURE_BINDING_3D', 'GL_PACK_SKIP_IMAGES', 'GL_PACK_IMAGE_HEIGHT',
'GL_UNPACK_SKIP_IMAGES', 'GL_UNPACK_IMAGE_HEIGHT', 'GL_TEXTURE_3D',
'GL_PROXY_TEXTURE_3D', 'GL_TEXTURE_DEPTH', 'GL_TEXTURE_WRAP_R',
'GL_MAX_3D_TEXTURE_SIZE', 'GL_UNSIGNED_BYTE_2_3_3_REV',
'GL_UNSIGNED_SHORT_5_6_5', 'GL_UNSIGNED_SHORT_5_6_5_REV',
'GL_UNSIGNED_SHORT_4_4_4_4_REV', 'GL_UNSIGNED_SHORT_1_5_5_5_REV',
'GL_UNSIGNED_INT_8_8_8_8_REV', 'GL_UNSIGNED_INT_2_10_10_10_REV', 'GL_BGR',
'GL_BGRA', 'GL_MAX_ELEMENTS_VERTICES', 'GL_MAX_ELEMENTS_INDICES',
'GL_CLAMP_TO_EDGE', 'GL_TEXTURE_MIN_LOD', 'GL_TEXTURE_MAX_LOD',
'GL_TEXTURE_BASE_LEVEL', 'GL_TEXTURE_MAX_LEVEL',
'GL_LIGHT_MODEL_COLOR_CONTROL', 'GL_SINGLE_COLOR',
'GL_SEPARATE_SPECULAR_COLOR', 'GL_SMOOTH_POINT_SIZE_RANGE',
'GL_SMOOTH_POINT_SIZE_GRANULARITY', 'GL_SMOOTH_LINE_WIDTH_RANGE',
'GL_SMOOTH_LINE_WIDTH_GRANULARITY', 'GL_ALIASED_POINT_SIZE_RANGE',
'GL_ALIASED_LINE_WIDTH_RANGE', 'GL_CONSTANT_COLOR',
'GL_ONE_MINUS_CONSTANT_COLOR', 'GL_CONSTANT_ALPHA',
'GL_ONE_MINUS_CONSTANT_ALPHA', 'GL_BLEND_COLOR', 'GL_FUNC_ADD', 'GL_MIN',
'GL_MAX', 'GL_BLEND_EQUATION', 'GL_FUNC_SUBTRACT', 'GL_FUNC_REVERSE_SUBTRACT',
'GL_CONVOLUTION_1D', 'GL_CONVOLUTION_2D', 'GL_SEPARABLE_2D',
'GL_CONVOLUTION_BORDER_MODE', 'GL_CONVOLUTION_FILTER_SCALE',
'GL_CONVOLUTION_FILTER_BIAS', 'GL_REDUCE', 'GL_CONVOLUTION_FORMAT',
'GL_CONVOLUTION_WIDTH', 'GL_CONVOLUTION_HEIGHT', 'GL_MAX_CONVOLUTION_WIDTH',
'GL_MAX_CONVOLUTION_HEIGHT', 'GL_POST_CONVOLUTION_RED_SCALE',
'GL_POST_CONVOLUTION_GREEN_SCALE', 'GL_POST_CONVOLUTION_BLUE_SCALE',
'GL_POST_CONVOLUTION_ALPHA_SCALE', 'GL_POST_CONVOLUTION_RED_BIAS',
'GL_POST_CONVOLUTION_GREEN_BIAS', 'GL_POST_CONVOLUTION_BLUE_BIAS',
'GL_POST_CONVOLUTION_ALPHA_BIAS', 'GL_HISTOGRAM', 'GL_PROXY_HISTOGRAM',
'GL_HISTOGRAM_WIDTH', 'GL_HISTOGRAM_FORMAT', 'GL_HISTOGRAM_RED_SIZE',
'GL_HISTOGRAM_GREEN_SIZE', 'GL_HISTOGRAM_BLUE_SIZE',
'GL_HISTOGRAM_ALPHA_SIZE', 'GL_HISTOGRAM_LUMINANCE_SIZE', 'GL_HISTOGRAM_SINK',
'GL_MINMAX', 'GL_MINMAX_FORMAT', 'GL_MINMAX_SINK', 'GL_TABLE_TOO_LARGE',
'GL_COLOR_MATRIX', 'GL_COLOR_MATRIX_STACK_DEPTH',
'GL_MAX_COLOR_MATRIX_STACK_DEPTH', 'GL_POST_COLOR_MATRIX_RED_SCALE',
'GL_POST_COLOR_MATRIX_GREEN_SCALE', 'GL_POST_COLOR_MATRIX_BLUE_SCALE',
'GL_POST_COLOR_MATRIX_ALPHA_SCALE', 'GL_POST_COLOR_MATRIX_RED_BIAS',
'GL_POST_COLOR_MATRIX_GREEN_BIAS', 'GL_POST_COLOR_MATRIX_BLUE_BIAS',
'GL_POST_COLOR_MATRIX_ALPHA_BIAS', 'GL_COLOR_TABLE',
'GL_POST_CONVOLUTION_COLOR_TABLE', 'GL_POST_COLOR_MATRIX_COLOR_TABLE',
'GL_PROXY_COLOR_TABLE', 'GL_PROXY_POST_CONVOLUTION_COLOR_TABLE',
'GL_PROXY_POST_COLOR_MATRIX_COLOR_TABLE', 'GL_COLOR_TABLE_SCALE',
'GL_COLOR_TABLE_BIAS', 'GL_COLOR_TABLE_FORMAT', 'GL_COLOR_TABLE_WIDTH',
'GL_COLOR_TABLE_RED_SIZE', 'GL_COLOR_TABLE_GREEN_SIZE',
'GL_COLOR_TABLE_BLUE_SIZE', 'GL_COLOR_TABLE_ALPHA_SIZE',
'GL_COLOR_TABLE_LUMINANCE_SIZE', 'GL_COLOR_TABLE_INTENSITY_SIZE',
'GL_CONSTANT_BORDER', 'GL_REPLICATE_BORDER', 'GL_CONVOLUTION_BORDER_COLOR',
'GL_TEXTURE0', 'GL_TEXTURE1', 'GL_TEXTURE2', 'GL_TEXTURE3', 'GL_TEXTURE4',
'GL_TEXTURE5', 'GL_TEXTURE6', 'GL_TEXTURE7', 'GL_TEXTURE8', 'GL_TEXTURE9',
'GL_TEXTURE10', 'GL_TEXTURE11', 'GL_TEXTURE12', 'GL_TEXTURE13',
'GL_TEXTURE14', 'GL_TEXTURE15', 'GL_TEXTURE16', 'GL_TEXTURE17',
'GL_TEXTURE18', 'GL_TEXTURE19', 'GL_TEXTURE20', 'GL_TEXTURE21',
'GL_TEXTURE22', 'GL_TEXTURE23', 'GL_TEXTURE24', 'GL_TEXTURE25',
'GL_TEXTURE26', 'GL_TEXTURE27', 'GL_TEXTURE28', 'GL_TEXTURE29',
'GL_TEXTURE30', 'GL_TEXTURE31', 'GL_ACTIVE_TEXTURE',
'GL_CLIENT_ACTIVE_TEXTURE', 'GL_MAX_TEXTURE_UNITS',
'GL_TRANSPOSE_MODELVIEW_MATRIX', 'GL_TRANSPOSE_PROJECTION_MATRIX',
'GL_TRANSPOSE_TEXTURE_MATRIX', 'GL_TRANSPOSE_COLOR_MATRIX', 'GL_MULTISAMPLE',
'GL_SAMPLE_ALPHA_TO_COVERAGE', 'GL_SAMPLE_ALPHA_TO_ONE', 'GL_SAMPLE_COVERAGE',
'GL_SAMPLE_BUFFERS', 'GL_SAMPLES', 'GL_SAMPLE_COVERAGE_VALUE',
'GL_SAMPLE_COVERAGE_INVERT', 'GL_MULTISAMPLE_BIT', 'GL_NORMAL_MAP',
'GL_REFLECTION_MAP', 'GL_TEXTURE_CUBE_MAP', 'GL_TEXTURE_BINDING_CUBE_MAP',
'GL_TEXTURE_CUBE_MAP_POSITIVE_X', 'GL_TEXTURE_CUBE_MAP_NEGATIVE_X',
'GL_TEXTURE_CUBE_MAP_POSITIVE_Y', 'GL_TEXTURE_CUBE_MAP_NEGATIVE_Y',
'GL_TEXTURE_CUBE_MAP_POSITIVE_Z', 'GL_TEXTURE_CUBE_MAP_NEGATIVE_Z',
'GL_PROXY_TEXTURE_CUBE_MAP', 'GL_MAX_CUBE_MAP_TEXTURE_SIZE',
'GL_COMPRESSED_ALPHA', 'GL_COMPRESSED_LUMINANCE',
'GL_COMPRESSED_LUMINANCE_ALPHA', 'GL_COMPRESSED_INTENSITY',
'GL_COMPRESSED_RGB', 'GL_COMPRESSED_RGBA', 'GL_TEXTURE_COMPRESSION_HINT',
'GL_TEXTURE_COMPRESSED_IMAGE_SIZE', 'GL_TEXTURE_COMPRESSED',
'GL_NUM_COMPRESSED_TEXTURE_FORMATS', 'GL_COMPRESSED_TEXTURE_FORMATS',
'GL_CLAMP_TO_BORDER', 'GL_COMBINE', 'GL_COMBINE_RGB', 'GL_COMBINE_ALPHA',
'GL_SOURCE0_RGB', 'GL_SOURCE1_RGB', 'GL_SOURCE2_RGB', 'GL_SOURCE0_ALPHA',
'GL_SOURCE1_ALPHA', 'GL_SOURCE2_ALPHA', 'GL_OPERAND0_RGB', 'GL_OPERAND1_RGB',
'GL_OPERAND2_RGB', 'GL_OPERAND0_ALPHA', 'GL_OPERAND1_ALPHA',
'GL_OPERAND2_ALPHA', 'GL_RGB_SCALE', 'GL_ADD_SIGNED', 'GL_INTERPOLATE',
'GL_SUBTRACT', 'GL_CONSTANT', 'GL_PRIMARY_COLOR', 'GL_PREVIOUS',
'GL_DOT3_RGB', 'GL_DOT3_RGBA', 'GL_BLEND_DST_RGB', 'GL_BLEND_SRC_RGB',
'GL_BLEND_DST_ALPHA', 'GL_BLEND_SRC_ALPHA', 'GL_POINT_SIZE_MIN',
'GL_POINT_SIZE_MAX', 'GL_POINT_FADE_THRESHOLD_SIZE',
'GL_POINT_DISTANCE_ATTENUATION', 'GL_GENERATE_MIPMAP',
'GL_GENERATE_MIPMAP_HINT', 'GL_DEPTH_COMPONENT16', 'GL_DEPTH_COMPONENT24',
'GL_DEPTH_COMPONENT32', 'GL_MIRRORED_REPEAT', 'GL_FOG_COORDINATE_SOURCE',
'GL_FOG_COORDINATE', 'GL_FRAGMENT_DEPTH', 'GL_CURRENT_FOG_COORDINATE',
'GL_FOG_COORDINATE_ARRAY_TYPE', 'GL_FOG_COORDINATE_ARRAY_STRIDE',
'GL_FOG_COORDINATE_ARRAY_POINTER', 'GL_FOG_COORDINATE_ARRAY', 'GL_COLOR_SUM',
'GL_CURRENT_SECONDARY_COLOR', 'GL_SECONDARY_COLOR_ARRAY_SIZE',
'GL_SECONDARY_COLOR_ARRAY_TYPE', 'GL_SECONDARY_COLOR_ARRAY_STRIDE',
'GL_SECONDARY_COLOR_ARRAY_POINTER', 'GL_SECONDARY_COLOR_ARRAY',
'GL_MAX_TEXTURE_LOD_BIAS', 'GL_TEXTURE_FILTER_CONTROL', 'GL_TEXTURE_LOD_BIAS',
'GL_INCR_WRAP', 'GL_DECR_WRAP', 'GL_TEXTURE_DEPTH_SIZE',
'GL_DEPTH_TEXTURE_MODE', 'GL_TEXTURE_COMPARE_MODE', 'GL_TEXTURE_COMPARE_FUNC',
'GL_COMPARE_R_TO_TEXTURE', 'GL_BUFFER_SIZE', 'GL_BUFFER_USAGE',
'GL_QUERY_COUNTER_BITS', 'GL_CURRENT_QUERY', 'GL_QUERY_RESULT',
'GL_QUERY_RESULT_AVAILABLE', 'GL_ARRAY_BUFFER', 'GL_ELEMENT_ARRAY_BUFFER',
'GL_ARRAY_BUFFER_BINDING', 'GL_ELEMENT_ARRAY_BUFFER_BINDING',
'GL_VERTEX_ARRAY_BUFFER_BINDING', 'GL_NORMAL_ARRAY_BUFFER_BINDING',
'GL_COLOR_ARRAY_BUFFER_BINDING', 'GL_INDEX_ARRAY_BUFFER_BINDING',
'GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING', 'GL_EDGE_FLAG_ARRAY_BUFFER_BINDING',
'GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING',
'GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING', 'GL_WEIGHT_ARRAY_BUFFER_BINDING',
'GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING', 'GL_READ_ONLY', 'GL_WRITE_ONLY',
'GL_READ_WRITE', 'GL_BUFFER_ACCESS', 'GL_BUFFER_MAPPED',
'GL_BUFFER_MAP_POINTER', 'GL_STREAM_DRAW', 'GL_STREAM_READ', 'GL_STREAM_COPY',
'GL_STATIC_DRAW', 'GL_STATIC_READ', 'GL_STATIC_COPY', 'GL_DYNAMIC_DRAW',
'GL_DYNAMIC_READ', 'GL_DYNAMIC_COPY', 'GL_SAMPLES_PASSED', 'GL_FOG_COORD_SRC',
'GL_FOG_COORD', 'GL_CURRENT_FOG_COORD', 'GL_FOG_COORD_ARRAY_TYPE',
'GL_FOG_COORD_ARRAY_STRIDE', 'GL_FOG_COORD_ARRAY_POINTER',
'GL_FOG_COORD_ARRAY', 'GL_FOG_COORD_ARRAY_BUFFER_BINDING', 'GL_SRC0_RGB',
'GL_SRC1_RGB', 'GL_SRC2_RGB', 'GL_SRC0_ALPHA', 'GL_SRC1_ALPHA',
'GL_SRC2_ALPHA', 'GL_BLEND_EQUATION_RGB', 'GL_VERTEX_ATTRIB_ARRAY_ENABLED',
'GL_VERTEX_ATTRIB_ARRAY_SIZE', 'GL_VERTEX_ATTRIB_ARRAY_STRIDE',
'GL_VERTEX_ATTRIB_ARRAY_TYPE', 'GL_CURRENT_VERTEX_ATTRIB',
'GL_VERTEX_PROGRAM_POINT_SIZE', 'GL_VERTEX_PROGRAM_TWO_SIDE',
'GL_VERTEX_ATTRIB_ARRAY_POINTER', 'GL_STENCIL_BACK_FUNC',
'GL_STENCIL_BACK_FAIL', 'GL_STENCIL_BACK_PASS_DEPTH_FAIL',
'GL_STENCIL_BACK_PASS_DEPTH_PASS', 'GL_MAX_DRAW_BUFFERS', 'GL_DRAW_BUFFER0',
'GL_DRAW_BUFFER1', 'GL_DRAW_BUFFER2', 'GL_DRAW_BUFFER3', 'GL_DRAW_BUFFER4',
'GL_DRAW_BUFFER5', 'GL_DRAW_BUFFER6', 'GL_DRAW_BUFFER7', 'GL_DRAW_BUFFER8',
'GL_DRAW_BUFFER9', 'GL_DRAW_BUFFER10', 'GL_DRAW_BUFFER11', 'GL_DRAW_BUFFER12',
'GL_DRAW_BUFFER13', 'GL_DRAW_BUFFER14', 'GL_DRAW_BUFFER15',
'GL_BLEND_EQUATION_ALPHA', 'GL_POINT_SPRITE', 'GL_COORD_REPLACE',
'GL_MAX_VERTEX_ATTRIBS', 'GL_VERTEX_ATTRIB_ARRAY_NORMALIZED',
'GL_MAX_TEXTURE_COORDS', 'GL_MAX_TEXTURE_IMAGE_UNITS', 'GL_FRAGMENT_SHADER',
'GL_VERTEX_SHADER', 'GL_MAX_FRAGMENT_UNIFORM_COMPONENTS',
'GL_MAX_VERTEX_UNIFORM_COMPONENTS', 'GL_MAX_VARYING_FLOATS',
'GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS', 'GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS',
'GL_SHADER_TYPE', 'GL_FLOAT_VEC2', 'GL_FLOAT_VEC3', 'GL_FLOAT_VEC4',
'GL_INT_VEC2', 'GL_INT_VEC3', 'GL_INT_VEC4', 'GL_BOOL', 'GL_BOOL_VEC2',
'GL_BOOL_VEC3', 'GL_BOOL_VEC4', 'GL_FLOAT_MAT2', 'GL_FLOAT_MAT3',
'GL_FLOAT_MAT4', 'GL_SAMPLER_1D', 'GL_SAMPLER_2D', 'GL_SAMPLER_3D',
'GL_SAMPLER_CUBE', 'GL_SAMPLER_1D_SHADOW', 'GL_SAMPLER_2D_SHADOW',
'GL_DELETE_STATUS', 'GL_COMPILE_STATUS', 'GL_LINK_STATUS',
'GL_VALIDATE_STATUS', 'GL_INFO_LOG_LENGTH', 'GL_ATTACHED_SHADERS',
'GL_ACTIVE_UNIFORMS', 'GL_ACTIVE_UNIFORM_MAX_LENGTH',
'GL_SHADER_SOURCE_LENGTH', 'GL_ACTIVE_ATTRIBUTES',
'GL_ACTIVE_ATTRIBUTE_MAX_LENGTH', 'GL_FRAGMENT_SHADER_DERIVATIVE_HINT',
'GL_SHADING_LANGUAGE_VERSION', 'GL_CURRENT_PROGRAM',
'GL_POINT_SPRITE_COORD_ORIGIN', 'GL_LOWER_LEFT', 'GL_UPPER_LEFT',
'GL_STENCIL_BACK_REF', 'GL_STENCIL_BACK_VALUE_MASK',
'GL_STENCIL_BACK_WRITEMASK', 'GL_PIXEL_PACK_BUFFER', 'GL_PIXEL_UNPACK_BUFFER',
'GL_PIXEL_PACK_BUFFER_BINDING', 'GL_PIXEL_UNPACK_BUFFER_BINDING', 'GL_SRGB',
'GL_SRGB8', 'GL_SRGB_ALPHA', 'GL_SRGB8_ALPHA8', 'GL_SLUMINANCE_ALPHA',
'GL_SLUMINANCE8_ALPHA8', 'GL_SLUMINANCE', 'GL_SLUMINANCE8',
'GL_COMPRESSED_SRGB', 'GL_COMPRESSED_SRGB_ALPHA', 'GL_COMPRESSED_SLUMINANCE',
'GL_COMPRESSED_SLUMINANCE_ALPHA', 'GL_FLOAT_MAT2x3', 'GL_FLOAT_MAT2x4',
'GL_FLOAT_MAT3x2', 'GL_FLOAT_MAT3x4', 'GL_FLOAT_MAT4x2', 'GL_FLOAT_MAT4x3',
'GL_CURRENT_RASTER_SECONDARY_COLOR', 'GL_TEXTURE0_ARB', 'GL_TEXTURE1_ARB',
'GL_TEXTURE2_ARB', 'GL_TEXTURE3_ARB', 'GL_TEXTURE4_ARB', 'GL_TEXTURE5_ARB',
'GL_TEXTURE6_ARB', 'GL_TEXTURE7_ARB', 'GL_TEXTURE8_ARB', 'GL_TEXTURE9_ARB',
'GL_TEXTURE10_ARB', 'GL_TEXTURE11_ARB', 'GL_TEXTURE12_ARB',
'GL_TEXTURE13_ARB', 'GL_TEXTURE14_ARB', 'GL_TEXTURE15_ARB',
'GL_TEXTURE16_ARB', 'GL_TEXTURE17_ARB', 'GL_TEXTURE18_ARB',
'GL_TEXTURE19_ARB', 'GL_TEXTURE20_ARB', 'GL_TEXTURE21_ARB',
'GL_TEXTURE22_ARB', 'GL_TEXTURE23_ARB', 'GL_TEXTURE24_ARB',
'GL_TEXTURE25_ARB', 'GL_TEXTURE26_ARB', 'GL_TEXTURE27_ARB',
'GL_TEXTURE28_ARB', 'GL_TEXTURE29_ARB', 'GL_TEXTURE30_ARB',
'GL_TEXTURE31_ARB', 'GL_ACTIVE_TEXTURE_ARB', 'GL_CLIENT_ACTIVE_TEXTURE_ARB',
'GL_MAX_TEXTURE_UNITS_ARB', 'GL_TRANSPOSE_MODELVIEW_MATRIX_ARB',
'GL_TRANSPOSE_PROJECTION_MATRIX_ARB', 'GL_TRANSPOSE_TEXTURE_MATRIX_ARB',
'GL_TRANSPOSE_COLOR_MATRIX_ARB', 'GL_MULTISAMPLE_ARB',
'GL_SAMPLE_ALPHA_TO_COVERAGE_ARB', 'GL_SAMPLE_ALPHA_TO_ONE_ARB',
'GL_SAMPLE_COVERAGE_ARB', 'GL_SAMPLE_BUFFERS_ARB', 'GL_SAMPLES_ARB',
'GL_SAMPLE_COVERAGE_VALUE_ARB', 'GL_SAMPLE_COVERAGE_INVERT_ARB',
'GL_MULTISAMPLE_BIT_ARB', 'GL_NORMAL_MAP_ARB', 'GL_REFLECTION_MAP_ARB',
'GL_TEXTURE_CUBE_MAP_ARB', 'GL_TEXTURE_BINDING_CUBE_MAP_ARB',
'GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB', 'GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB',
'GL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB', 'GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB',
'GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB', 'GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB',
'GL_PROXY_TEXTURE_CUBE_MAP_ARB', 'GL_MAX_CUBE_MAP_TEXTURE_SIZE_ARB',
'GL_COMPRESSED_ALPHA_ARB', 'GL_COMPRESSED_LUMINANCE_ARB',
'GL_COMPRESSED_LUMINANCE_ALPHA_ARB', 'GL_COMPRESSED_INTENSITY_ARB',
'GL_COMPRESSED_RGB_ARB', 'GL_COMPRESSED_RGBA_ARB',
'GL_TEXTURE_COMPRESSION_HINT_ARB', 'GL_TEXTURE_COMPRESSED_IMAGE_SIZE_ARB',
'GL_TEXTURE_COMPRESSED_ARB', 'GL_NUM_COMPRESSED_TEXTURE_FORMATS_ARB',
'GL_COMPRESSED_TEXTURE_FORMATS_ARB', 'GL_CLAMP_TO_BORDER_ARB',
'GL_POINT_SIZE_MIN_ARB', 'GL_POINT_SIZE_MAX_ARB',
'GL_POINT_FADE_THRESHOLD_SIZE_ARB', 'GL_POINT_DISTANCE_ATTENUATION_ARB',
'GL_MAX_VERTEX_UNITS_ARB', 'GL_ACTIVE_VERTEX_UNITS_ARB',
'GL_WEIGHT_SUM_UNITY_ARB', 'GL_VERTEX_BLEND_ARB', 'GL_CURRENT_WEIGHT_ARB',
'GL_WEIGHT_ARRAY_TYPE_ARB', 'GL_WEIGHT_ARRAY_STRIDE_ARB',
'GL_WEIGHT_ARRAY_SIZE_ARB', 'GL_WEIGHT_ARRAY_POINTER_ARB',
'GL_WEIGHT_ARRAY_ARB', 'GL_MODELVIEW0_ARB', 'GL_MODELVIEW1_ARB',
'GL_MODELVIEW2_ARB', 'GL_MODELVIEW3_ARB', 'GL_MODELVIEW4_ARB',
'GL_MODELVIEW5_ARB', 'GL_MODELVIEW6_ARB', 'GL_MODELVIEW7_ARB',
'GL_MODELVIEW8_ARB', 'GL_MODELVIEW9_ARB', 'GL_MODELVIEW10_ARB',
'GL_MODELVIEW11_ARB', 'GL_MODELVIEW12_ARB', 'GL_MODELVIEW13_ARB',
'GL_MODELVIEW14_ARB', 'GL_MODELVIEW15_ARB', 'GL_MODELVIEW16_ARB',
'GL_MODELVIEW17_ARB', 'GL_MODELVIEW18_ARB', 'GL_MODELVIEW19_ARB',
'GL_MODELVIEW20_ARB', 'GL_MODELVIEW21_ARB', 'GL_MODELVIEW22_ARB',
'GL_MODELVIEW23_ARB', 'GL_MODELVIEW24_ARB', 'GL_MODELVIEW25_ARB',
'GL_MODELVIEW26_ARB', 'GL_MODELVIEW27_ARB', 'GL_MODELVIEW28_ARB',
'GL_MODELVIEW29_ARB', 'GL_MODELVIEW30_ARB', 'GL_MODELVIEW31_ARB',
'GL_MATRIX_PALETTE_ARB', 'GL_MAX_MATRIX_PALETTE_STACK_DEPTH_ARB',
'GL_MAX_PALETTE_MATRICES_ARB', 'GL_CURRENT_PALETTE_MATRIX_ARB',
'GL_MATRIX_INDEX_ARRAY_ARB', 'GL_CURRENT_MATRIX_INDEX_ARB',
'GL_MATRIX_INDEX_ARRAY_SIZE_ARB', 'GL_MATRIX_INDEX_ARRAY_TYPE_ARB',
'GL_MATRIX_INDEX_ARRAY_STRIDE_ARB', 'GL_MATRIX_INDEX_ARRAY_POINTER_ARB',
'GL_COMBINE_ARB', 'GL_COMBINE_RGB_ARB', 'GL_COMBINE_ALPHA_ARB',
'GL_SOURCE0_RGB_ARB', 'GL_SOURCE1_RGB_ARB', 'GL_SOURCE2_RGB_ARB',
'GL_SOURCE0_ALPHA_ARB', 'GL_SOURCE1_ALPHA_ARB', 'GL_SOURCE2_ALPHA_ARB',
'GL_OPERAND0_RGB_ARB', 'GL_OPERAND1_RGB_ARB', 'GL_OPERAND2_RGB_ARB',
'GL_OPERAND0_ALPHA_ARB', 'GL_OPERAND1_ALPHA_ARB', 'GL_OPERAND2_ALPHA_ARB',
'GL_RGB_SCALE_ARB', 'GL_ADD_SIGNED_ARB', 'GL_INTERPOLATE_ARB',
'GL_SUBTRACT_ARB', 'GL_CONSTANT_ARB', 'GL_PRIMARY_COLOR_ARB',
'GL_PREVIOUS_ARB', 'GL_DOT3_RGB_ARB', 'GL_DOT3_RGBA_ARB',
'GL_MIRRORED_REPEAT_ARB', 'GL_DEPTH_COMPONENT16_ARB',
'GL_DEPTH_COMPONENT24_ARB', 'GL_DEPTH_COMPONENT32_ARB',
'GL_TEXTURE_DEPTH_SIZE_ARB', 'GL_DEPTH_TEXTURE_MODE_ARB',
'GL_TEXTURE_COMPARE_MODE_ARB', 'GL_TEXTURE_COMPARE_FUNC_ARB',
'GL_COMPARE_R_TO_TEXTURE_ARB', 'GL_TEXTURE_COMPARE_FAIL_VALUE_ARB',
'GL_COLOR_SUM_ARB', 'GL_VERTEX_PROGRAM_ARB',
'GL_VERTEX_ATTRIB_ARRAY_ENABLED_ARB', 'GL_VERTEX_ATTRIB_ARRAY_SIZE_ARB',
'GL_VERTEX_ATTRIB_ARRAY_STRIDE_ARB', 'GL_VERTEX_ATTRIB_ARRAY_TYPE_ARB',
'GL_CURRENT_VERTEX_ATTRIB_ARB', 'GL_PROGRAM_LENGTH_ARB',
'GL_PROGRAM_STRING_ARB', 'GL_MAX_PROGRAM_MATRIX_STACK_DEPTH_ARB',
'GL_MAX_PROGRAM_MATRICES_ARB', 'GL_CURRENT_MATRIX_STACK_DEPTH_ARB',
'GL_CURRENT_MATRIX_ARB', 'GL_VERTEX_PROGRAM_POINT_SIZE_ARB',
'GL_VERTEX_PROGRAM_TWO_SIDE_ARB', 'GL_VERTEX_ATTRIB_ARRAY_POINTER_ARB',
'GL_PROGRAM_ERROR_POSITION_ARB', 'GL_PROGRAM_BINDING_ARB',
'GL_MAX_VERTEX_ATTRIBS_ARB', 'GL_VERTEX_ATTRIB_ARRAY_NORMALIZED_ARB',
'GL_PROGRAM_ERROR_STRING_ARB', 'GL_PROGRAM_FORMAT_ASCII_ARB',
'GL_PROGRAM_FORMAT_ARB', 'GL_PROGRAM_INSTRUCTIONS_ARB',
'GL_MAX_PROGRAM_INSTRUCTIONS_ARB', 'GL_PROGRAM_NATIVE_INSTRUCTIONS_ARB',
'GL_MAX_PROGRAM_NATIVE_INSTRUCTIONS_ARB', 'GL_PROGRAM_TEMPORARIES_ARB',
'GL_MAX_PROGRAM_TEMPORARIES_ARB', 'GL_PROGRAM_NATIVE_TEMPORARIES_ARB',
'GL_MAX_PROGRAM_NATIVE_TEMPORARIES_ARB', 'GL_PROGRAM_PARAMETERS_ARB',
'GL_MAX_PROGRAM_PARAMETERS_ARB', 'GL_PROGRAM_NATIVE_PARAMETERS_ARB',
'GL_MAX_PROGRAM_NATIVE_PARAMETERS_ARB', 'GL_PROGRAM_ATTRIBS_ARB',
'GL_MAX_PROGRAM_ATTRIBS_ARB', 'GL_PROGRAM_NATIVE_ATTRIBS_ARB',
'GL_MAX_PROGRAM_NATIVE_ATTRIBS_ARB', 'GL_PROGRAM_ADDRESS_REGISTERS_ARB',
'GL_MAX_PROGRAM_ADDRESS_REGISTERS_ARB',
'GL_PROGRAM_NATIVE_ADDRESS_REGISTERS_ARB',
'GL_MAX_PROGRAM_NATIVE_ADDRESS_REGISTERS_ARB',
'GL_MAX_PROGRAM_LOCAL_PARAMETERS_ARB', 'GL_MAX_PROGRAM_ENV_PARAMETERS_ARB',
'GL_PROGRAM_UNDER_NATIVE_LIMITS_ARB', 'GL_TRANSPOSE_CURRENT_MATRIX_ARB',
'GL_MATRIX0_ARB', 'GL_MATRIX1_ARB', 'GL_MATRIX2_ARB', 'GL_MATRIX3_ARB',
'GL_MATRIX4_ARB', 'GL_MATRIX5_ARB', 'GL_MATRIX6_ARB', 'GL_MATRIX7_ARB',
'GL_MATRIX8_ARB', 'GL_MATRIX9_ARB', 'GL_MATRIX10_ARB', 'GL_MATRIX11_ARB',
'GL_MATRIX12_ARB', 'GL_MATRIX13_ARB', 'GL_MATRIX14_ARB', 'GL_MATRIX15_ARB',
'GL_MATRIX16_ARB', 'GL_MATRIX17_ARB', 'GL_MATRIX18_ARB', 'GL_MATRIX19_ARB',
'GL_MATRIX20_ARB', 'GL_MATRIX21_ARB', 'GL_MATRIX22_ARB', 'GL_MATRIX23_ARB',
'GL_MATRIX24_ARB', 'GL_MATRIX25_ARB', 'GL_MATRIX26_ARB', 'GL_MATRIX27_ARB',
'GL_MATRIX28_ARB', 'GL_MATRIX29_ARB', 'GL_MATRIX30_ARB', 'GL_MATRIX31_ARB',
'GL_FRAGMENT_PROGRAM_ARB', 'GL_PROGRAM_ALU_INSTRUCTIONS_ARB',
'GL_PROGRAM_TEX_INSTRUCTIONS_ARB', 'GL_PROGRAM_TEX_INDIRECTIONS_ARB',
'GL_PROGRAM_NATIVE_ALU_INSTRUCTIONS_ARB',
'GL_PROGRAM_NATIVE_TEX_INSTRUCTIONS_ARB',
'GL_PROGRAM_NATIVE_TEX_INDIRECTIONS_ARB',
'GL_MAX_PROGRAM_ALU_INSTRUCTIONS_ARB', 'GL_MAX_PROGRAM_TEX_INSTRUCTIONS_ARB',
'GL_MAX_PROGRAM_TEX_INDIRECTIONS_ARB',
'GL_MAX_PROGRAM_NATIVE_ALU_INSTRUCTIONS_ARB',
'GL_MAX_PROGRAM_NATIVE_TEX_INSTRUCTIONS_ARB',
'GL_MAX_PROGRAM_NATIVE_TEX_INDIRECTIONS_ARB', 'GL_MAX_TEXTURE_COORDS_ARB',
'GL_MAX_TEXTURE_IMAGE_UNITS_ARB', 'GL_BUFFER_SIZE_ARB', 'GL_BUFFER_USAGE_ARB',
'GL_ARRAY_BUFFER_ARB', 'GL_ELEMENT_ARRAY_BUFFER_ARB',
'GL_ARRAY_BUFFER_BINDING_ARB', 'GL_ELEMENT_ARRAY_BUFFER_BINDING_ARB',
'GL_VERTEX_ARRAY_BUFFER_BINDING_ARB', 'GL_NORMAL_ARRAY_BUFFER_BINDING_ARB',
'GL_COLOR_ARRAY_BUFFER_BINDING_ARB', 'GL_INDEX_ARRAY_BUFFER_BINDING_ARB',
'GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING_ARB',
'GL_EDGE_FLAG_ARRAY_BUFFER_BINDING_ARB',
'GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING_ARB',
'GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING_ARB',
'GL_WEIGHT_ARRAY_BUFFER_BINDING_ARB',
'GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING_ARB', 'GL_READ_ONLY_ARB',
'GL_WRITE_ONLY_ARB', 'GL_READ_WRITE_ARB', 'GL_BUFFER_ACCESS_ARB',
'GL_BUFFER_MAPPED_ARB', 'GL_BUFFER_MAP_POINTER_ARB', 'GL_STREAM_DRAW_ARB',
'GL_STREAM_READ_ARB', 'GL_STREAM_COPY_ARB', 'GL_STATIC_DRAW_ARB',
'GL_STATIC_READ_ARB', 'GL_STATIC_COPY_ARB', 'GL_DYNAMIC_DRAW_ARB',
'GL_DYNAMIC_READ_ARB', 'GL_DYNAMIC_COPY_ARB', 'GL_QUERY_COUNTER_BITS_ARB',
'GL_CURRENT_QUERY_ARB', 'GL_QUERY_RESULT_ARB',
'GL_QUERY_RESULT_AVAILABLE_ARB', 'GL_SAMPLES_PASSED_ARB',
'GL_PROGRAM_OBJECT_ARB', 'GL_SHADER_OBJECT_ARB', 'GL_OBJECT_TYPE_ARB',
'GL_OBJECT_SUBTYPE_ARB', 'GL_FLOAT_VEC2_ARB', 'GL_FLOAT_VEC3_ARB',
'GL_FLOAT_VEC4_ARB', 'GL_INT_VEC2_ARB', 'GL_INT_VEC3_ARB', 'GL_INT_VEC4_ARB',
'GL_BOOL_ARB', 'GL_BOOL_VEC2_ARB', 'GL_BOOL_VEC3_ARB', 'GL_BOOL_VEC4_ARB',
'GL_FLOAT_MAT2_ARB', 'GL_FLOAT_MAT3_ARB', 'GL_FLOAT_MAT4_ARB',
'GL_SAMPLER_1D_ARB', 'GL_SAMPLER_2D_ARB', 'GL_SAMPLER_3D_ARB',
'GL_SAMPLER_CUBE_ARB', 'GL_SAMPLER_1D_SHADOW_ARB', 'GL_SAMPLER_2D_SHADOW_ARB',
'GL_SAMPLER_2D_RECT_ARB', 'GL_SAMPLER_2D_RECT_SHADOW_ARB',
'GL_OBJECT_DELETE_STATUS_ARB', 'GL_OBJECT_COMPILE_STATUS_ARB',
'GL_OBJECT_LINK_STATUS_ARB', 'GL_OBJECT_VALIDATE_STATUS_ARB',
'GL_OBJECT_INFO_LOG_LENGTH_ARB', 'GL_OBJECT_ATTACHED_OBJECTS_ARB',
'GL_OBJECT_ACTIVE_UNIFORMS_ARB', 'GL_OBJECT_ACTIVE_UNIFORM_MAX_LENGTH_ARB',
'GL_OBJECT_SHADER_SOURCE_LENGTH_ARB', 'GL_VERTEX_SHADER_ARB',
'GL_MAX_VERTEX_UNIFORM_COMPONENTS_ARB', 'GL_MAX_VARYING_FLOATS_ARB',
'GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS_ARB',
'GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS_ARB', 'GL_OBJECT_ACTIVE_ATTRIBUTES_ARB',
'GL_OBJECT_ACTIVE_ATTRIBUTE_MAX_LENGTH_ARB', 'GL_FRAGMENT_SHADER_ARB',
'GL_MAX_FRAGMENT_UNIFORM_COMPONENTS_ARB',
'GL_FRAGMENT_SHADER_DERIVATIVE_HINT_ARB', 'GL_SHADING_LANGUAGE_VERSION_ARB',
'GL_POINT_SPRITE_ARB', 'GL_COORD_REPLACE_ARB', 'GL_MAX_DRAW_BUFFERS_ARB',
'GL_DRAW_BUFFER0_ARB', 'GL_DRAW_BUFFER1_ARB', 'GL_DRAW_BUFFER2_ARB',
'GL_DRAW_BUFFER3_ARB', 'GL_DRAW_BUFFER4_ARB', 'GL_DRAW_BUFFER5_ARB',
'GL_DRAW_BUFFER6_ARB', 'GL_DRAW_BUFFER7_ARB', 'GL_DRAW_BUFFER8_ARB',
'GL_DRAW_BUFFER9_ARB', 'GL_DRAW_BUFFER10_ARB', 'GL_DRAW_BUFFER11_ARB',
'GL_DRAW_BUFFER12_ARB', 'GL_DRAW_BUFFER13_ARB', 'GL_DRAW_BUFFER14_ARB',
'GL_DRAW_BUFFER15_ARB', 'GL_TEXTURE_RECTANGLE_ARB',
'GL_TEXTURE_BINDING_RECTANGLE_ARB', 'GL_PROXY_TEXTURE_RECTANGLE_ARB',
'GL_MAX_RECTANGLE_TEXTURE_SIZE_ARB', 'GL_RGBA_FLOAT_MODE_ARB',
'GL_CLAMP_VERTEX_COLOR_ARB', 'GL_CLAMP_FRAGMENT_COLOR_ARB',
'GL_CLAMP_READ_COLOR_ARB', 'GL_FIXED_ONLY_ARB', 'GL_HALF_FLOAT_ARB',
'GL_TEXTURE_RED_TYPE_ARB', 'GL_TEXTURE_GREEN_TYPE_ARB',
'GL_TEXTURE_BLUE_TYPE_ARB', 'GL_TEXTURE_ALPHA_TYPE_ARB',
'GL_TEXTURE_LUMINANCE_TYPE_ARB', 'GL_TEXTURE_INTENSITY_TYPE_ARB',
'GL_TEXTURE_DEPTH_TYPE_ARB', 'GL_UNSIGNED_NORMALIZED_ARB', 'GL_RGBA32F_ARB',
'GL_RGB32F_ARB', 'GL_ALPHA32F_ARB', 'GL_INTENSITY32F_ARB',
'GL_LUMINANCE32F_ARB', 'GL_LUMINANCE_ALPHA32F_ARB', 'GL_RGBA16F_ARB',
'GL_RGB16F_ARB', 'GL_ALPHA16F_ARB', 'GL_INTENSITY16F_ARB',
'GL_LUMINANCE16F_ARB', 'GL_LUMINANCE_ALPHA16F_ARB',
'GL_PIXEL_PACK_BUFFER_ARB', 'GL_PIXEL_UNPACK_BUFFER_ARB',
'GL_PIXEL_PACK_BUFFER_BINDING_ARB', 'GL_PIXEL_UNPACK_BUFFER_BINDING_ARB',
'GL_ABGR_EXT', 'GL_CONSTANT_COLOR_EXT', 'GL_ONE_MINUS_CONSTANT_COLOR_EXT',
'GL_CONSTANT_ALPHA_EXT', 'GL_ONE_MINUS_CONSTANT_ALPHA_EXT',
'GL_BLEND_COLOR_EXT', 'GL_POLYGON_OFFSET_EXT', 'GL_POLYGON_OFFSET_FACTOR_EXT',
'GL_POLYGON_OFFSET_BIAS_EXT', 'GL_ALPHA4_EXT', 'GL_ALPHA8_EXT',
'GL_ALPHA12_EXT', 'GL_ALPHA16_EXT', 'GL_LUMINANCE4_EXT', 'GL_LUMINANCE8_EXT',
'GL_LUMINANCE12_EXT', 'GL_LUMINANCE16_EXT', 'GL_LUMINANCE4_ALPHA4_EXT',
'GL_LUMINANCE6_ALPHA2_EXT', 'GL_LUMINANCE8_ALPHA8_EXT',
'GL_LUMINANCE12_ALPHA4_EXT', 'GL_LUMINANCE12_ALPHA12_EXT',
'GL_LUMINANCE16_ALPHA16_EXT', 'GL_INTENSITY_EXT', 'GL_INTENSITY4_EXT',
'GL_INTENSITY8_EXT', 'GL_INTENSITY12_EXT', 'GL_INTENSITY16_EXT',
'GL_RGB2_EXT', 'GL_RGB4_EXT', 'GL_RGB5_EXT', 'GL_RGB8_EXT', 'GL_RGB10_EXT',
'GL_RGB12_EXT', 'GL_RGB16_EXT', 'GL_RGBA2_EXT', 'GL_RGBA4_EXT',
'GL_RGB5_A1_EXT', 'GL_RGBA8_EXT', 'GL_RGB10_A2_EXT', 'GL_RGBA12_EXT',
'GL_RGBA16_EXT', 'GL_TEXTURE_RED_SIZE_EXT', 'GL_TEXTURE_GREEN_SIZE_EXT',
'GL_TEXTURE_BLUE_SIZE_EXT', 'GL_TEXTURE_ALPHA_SIZE_EXT',
'GL_TEXTURE_LUMINANCE_SIZE_EXT', 'GL_TEXTURE_INTENSITY_SIZE_EXT',
'GL_REPLACE_EXT', 'GL_PROXY_TEXTURE_1D_EXT', 'GL_PROXY_TEXTURE_2D_EXT',
'GL_TEXTURE_TOO_LARGE_EXT', 'GL_PACK_SKIP_IMAGES_EXT',
'GL_PACK_IMAGE_HEIGHT_EXT', 'GL_UNPACK_SKIP_IMAGES_EXT',
'GL_UNPACK_IMAGE_HEIGHT_EXT', 'GL_TEXTURE_3D_EXT', 'GL_PROXY_TEXTURE_3D_EXT',
'GL_TEXTURE_DEPTH_EXT', 'GL_TEXTURE_WRAP_R_EXT', 'GL_MAX_3D_TEXTURE_SIZE_EXT',
'GL_FILTER4_SGIS', 'GL_TEXTURE_FILTER4_SIZE_SGIS', 'GL_HISTOGRAM_EXT',
'GL_PROXY_HISTOGRAM_EXT', 'GL_HISTOGRAM_WIDTH_EXT', 'GL_HISTOGRAM_FORMAT_EXT',
'GL_HISTOGRAM_RED_SIZE_EXT', 'GL_HISTOGRAM_GREEN_SIZE_EXT',
'GL_HISTOGRAM_BLUE_SIZE_EXT', 'GL_HISTOGRAM_ALPHA_SIZE_EXT',
'GL_HISTOGRAM_LUMINANCE_SIZE_EXT', 'GL_HISTOGRAM_SINK_EXT', 'GL_MINMAX_EXT',
'GL_MINMAX_FORMAT_EXT', 'GL_MINMAX_SINK_EXT', 'GL_TABLE_TOO_LARGE_EXT',
'GL_CONVOLUTION_1D_EXT', 'GL_CONVOLUTION_2D_EXT', 'GL_SEPARABLE_2D_EXT',
'GL_CONVOLUTION_BORDER_MODE_EXT', 'GL_CONVOLUTION_FILTER_SCALE_EXT',
'GL_CONVOLUTION_FILTER_BIAS_EXT', 'GL_REDUCE_EXT',
'GL_CONVOLUTION_FORMAT_EXT', 'GL_CONVOLUTION_WIDTH_EXT',
'GL_CONVOLUTION_HEIGHT_EXT', 'GL_MAX_CONVOLUTION_WIDTH_EXT',
'GL_MAX_CONVOLUTION_HEIGHT_EXT', 'GL_POST_CONVOLUTION_RED_SCALE_EXT',
'GL_POST_CONVOLUTION_GREEN_SCALE_EXT', 'GL_POST_CONVOLUTION_BLUE_SCALE_EXT',
'GL_POST_CONVOLUTION_ALPHA_SCALE_EXT', 'GL_POST_CONVOLUTION_RED_BIAS_EXT',
'GL_POST_CONVOLUTION_GREEN_BIAS_EXT', 'GL_POST_CONVOLUTION_BLUE_BIAS_EXT',
'GL_POST_CONVOLUTION_ALPHA_BIAS_EXT', 'GL_COLOR_MATRIX_SGI',
'GL_COLOR_MATRIX_STACK_DEPTH_SGI', 'GL_MAX_COLOR_MATRIX_STACK_DEPTH_SGI',
'GL_POST_COLOR_MATRIX_RED_SCALE_SGI', 'GL_POST_COLOR_MATRIX_GREEN_SCALE_SGI',
'GL_POST_COLOR_MATRIX_BLUE_SCALE_SGI', 'GL_POST_COLOR_MATRIX_ALPHA_SCALE_SGI',
'GL_POST_COLOR_MATRIX_RED_BIAS_SGI', 'GL_POST_COLOR_MATRIX_GREEN_BIAS_SGI',
'GL_POST_COLOR_MATRIX_BLUE_BIAS_SGI', 'GL_POST_COLOR_MATRIX_ALPHA_BIAS_SGI',
'GL_COLOR_TABLE_SGI', 'GL_POST_CONVOLUTION_COLOR_TABLE_SGI',
'GL_POST_COLOR_MATRIX_COLOR_TABLE_SGI', 'GL_PROXY_COLOR_TABLE_SGI',
'GL_PROXY_POST_CONVOLUTION_COLOR_TABLE_SGI',
'GL_PROXY_POST_COLOR_MATRIX_COLOR_TABLE_SGI', 'GL_COLOR_TABLE_SCALE_SGI',
'GL_COLOR_TABLE_BIAS_SGI', 'GL_COLOR_TABLE_FORMAT_SGI',
'GL_COLOR_TABLE_WIDTH_SGI', 'GL_COLOR_TABLE_RED_SIZE_SGI',
'GL_COLOR_TABLE_GREEN_SIZE_SGI', 'GL_COLOR_TABLE_BLUE_SIZE_SGI',
'GL_COLOR_TABLE_ALPHA_SIZE_SGI', 'GL_COLOR_TABLE_LUMINANCE_SIZE_SGI',
'GL_COLOR_TABLE_INTENSITY_SIZE_SGI', 'GL_PIXEL_TEXTURE_SGIS',
'GL_PIXEL_FRAGMENT_RGB_SOURCE_SGIS', 'GL_PIXEL_FRAGMENT_ALPHA_SOURCE_SGIS',
'GL_PIXEL_GROUP_COLOR_SGIS', 'GL_PIXEL_TEX_GEN_SGIX',
'GL_PIXEL_TEX_GEN_MODE_SGIX', 'GL_PACK_SKIP_VOLUMES_SGIS',
'GL_PACK_IMAGE_DEPTH_SGIS', 'GL_UNPACK_SKIP_VOLUMES_SGIS',
'GL_UNPACK_IMAGE_DEPTH_SGIS', 'GL_TEXTURE_4D_SGIS',
'GL_PROXY_TEXTURE_4D_SGIS', 'GL_TEXTURE_4DSIZE_SGIS',
'GL_TEXTURE_WRAP_Q_SGIS', 'GL_MAX_4D_TEXTURE_SIZE_SGIS',
'GL_TEXTURE_4D_BINDING_SGIS', 'GL_TEXTURE_COLOR_TABLE_SGI',
'GL_PROXY_TEXTURE_COLOR_TABLE_SGI', 'GL_CMYK_EXT', 'GL_CMYKA_EXT',
'GL_PACK_CMYK_HINT_EXT', 'GL_UNPACK_CMYK_HINT_EXT', 'GL_TEXTURE_PRIORITY_EXT',
'GL_TEXTURE_RESIDENT_EXT', 'GL_TEXTURE_1D_BINDING_EXT',
'GL_TEXTURE_2D_BINDING_EXT', 'GL_TEXTURE_3D_BINDING_EXT',
'GL_DETAIL_TEXTURE_2D_SGIS', 'GL_DETAIL_TEXTURE_2D_BINDING_SGIS',
'GL_LINEAR_DETAIL_SGIS', 'GL_LINEAR_DETAIL_ALPHA_SGIS',
'GL_LINEAR_DETAIL_COLOR_SGIS', 'GL_DETAIL_TEXTURE_LEVEL_SGIS',
'GL_DETAIL_TEXTURE_MODE_SGIS', 'GL_DETAIL_TEXTURE_FUNC_POINTS_SGIS',
'GL_LINEAR_SHARPEN_SGIS', 'GL_LINEAR_SHARPEN_ALPHA_SGIS',
'GL_LINEAR_SHARPEN_COLOR_SGIS', 'GL_SHARPEN_TEXTURE_FUNC_POINTS_SGIS',
'GL_UNSIGNED_BYTE_3_3_2_EXT', 'GL_UNSIGNED_SHORT_4_4_4_4_EXT',
'GL_UNSIGNED_SHORT_5_5_5_1_EXT', 'GL_UNSIGNED_INT_8_8_8_8_EXT',
'GL_UNSIGNED_INT_10_10_10_2_EXT', 'GL_TEXTURE_MIN_LOD_SGIS',
'GL_TEXTURE_MAX_LOD_SGIS', 'GL_TEXTURE_BASE_LEVEL_SGIS',
'GL_TEXTURE_MAX_LEVEL_SGIS', 'GL_MULTISAMPLE_SGIS',
'GL_SAMPLE_ALPHA_TO_MASK_SGIS', 'GL_SAMPLE_ALPHA_TO_ONE_SGIS',
'GL_SAMPLE_MASK_SGIS', 'GL_1PASS_SGIS', 'GL_2PASS_0_SGIS', 'GL_2PASS_1_SGIS',
'GL_4PASS_0_SGIS', 'GL_4PASS_1_SGIS', 'GL_4PASS_2_SGIS', 'GL_4PASS_3_SGIS',
'GL_SAMPLE_BUFFERS_SGIS', 'GL_SAMPLES_SGIS', 'GL_SAMPLE_MASK_VALUE_SGIS',
'GL_SAMPLE_MASK_INVERT_SGIS', 'GL_SAMPLE_PATTERN_SGIS',
'GL_RESCALE_NORMAL_EXT', 'GL_VERTEX_ARRAY_EXT', 'GL_NORMAL_ARRAY_EXT',
'GL_COLOR_ARRAY_EXT', 'GL_INDEX_ARRAY_EXT', 'GL_TEXTURE_COORD_ARRAY_EXT',
'GL_EDGE_FLAG_ARRAY_EXT', 'GL_VERTEX_ARRAY_SIZE_EXT',
'GL_VERTEX_ARRAY_TYPE_EXT', 'GL_VERTEX_ARRAY_STRIDE_EXT',
'GL_VERTEX_ARRAY_COUNT_EXT', 'GL_NORMAL_ARRAY_TYPE_EXT',
'GL_NORMAL_ARRAY_STRIDE_EXT', 'GL_NORMAL_ARRAY_COUNT_EXT',
'GL_COLOR_ARRAY_SIZE_EXT', 'GL_COLOR_ARRAY_TYPE_EXT',
'GL_COLOR_ARRAY_STRIDE_EXT', 'GL_COLOR_ARRAY_COUNT_EXT',
'GL_INDEX_ARRAY_TYPE_EXT', 'GL_INDEX_ARRAY_STRIDE_EXT',
'GL_INDEX_ARRAY_COUNT_EXT', 'GL_TEXTURE_COORD_ARRAY_SIZE_EXT',
'GL_TEXTURE_COORD_ARRAY_TYPE_EXT', 'GL_TEXTURE_COORD_ARRAY_STRIDE_EXT',
'GL_TEXTURE_COORD_ARRAY_COUNT_EXT', 'GL_EDGE_FLAG_ARRAY_STRIDE_EXT',
'GL_EDGE_FLAG_ARRAY_COUNT_EXT', 'GL_VERTEX_ARRAY_POINTER_EXT',
'GL_NORMAL_ARRAY_POINTER_EXT', 'GL_COLOR_ARRAY_POINTER_EXT',
'GL_INDEX_ARRAY_POINTER_EXT', 'GL_TEXTURE_COORD_ARRAY_POINTER_EXT',
'GL_EDGE_FLAG_ARRAY_POINTER_EXT', 'GL_GENERATE_MIPMAP_SGIS',
'GL_GENERATE_MIPMAP_HINT_SGIS', 'GL_LINEAR_CLIPMAP_LINEAR_SGIX',
'GL_TEXTURE_CLIPMAP_CENTER_SGIX', 'GL_TEXTURE_CLIPMAP_FRAME_SGIX',
'GL_TEXTURE_CLIPMAP_OFFSET_SGIX', 'GL_TEXTURE_CLIPMAP_VIRTUAL_DEPTH_SGIX',
'GL_TEXTURE_CLIPMAP_LOD_OFFSET_SGIX', 'GL_TEXTURE_CLIPMAP_DEPTH_SGIX',
'GL_MAX_CLIPMAP_DEPTH_SGIX', 'GL_MAX_CLIPMAP_VIRTUAL_DEPTH_SGIX',
'GL_NEAREST_CLIPMAP_NEAREST_SGIX', 'GL_NEAREST_CLIPMAP_LINEAR_SGIX',
'GL_LINEAR_CLIPMAP_NEAREST_SGIX', 'GL_TEXTURE_COMPARE_SGIX',
'GL_TEXTURE_COMPARE_OPERATOR_SGIX', 'GL_TEXTURE_LEQUAL_R_SGIX',
'GL_TEXTURE_GEQUAL_R_SGIX', 'GL_CLAMP_TO_EDGE_SGIS',
'GL_CLAMP_TO_BORDER_SGIS', 'GL_FUNC_ADD_EXT', 'GL_MIN_EXT', 'GL_MAX_EXT',
'GL_BLEND_EQUATION_EXT', 'GL_FUNC_SUBTRACT_EXT',
'GL_FUNC_REVERSE_SUBTRACT_EXT', 'GL_INTERLACE_SGIX',
'GL_PIXEL_TILE_BEST_ALIGNMENT_SGIX', 'GL_PIXEL_TILE_CACHE_INCREMENT_SGIX',
'GL_PIXEL_TILE_WIDTH_SGIX', 'GL_PIXEL_TILE_HEIGHT_SGIX',
'GL_PIXEL_TILE_GRID_WIDTH_SGIX', 'GL_PIXEL_TILE_GRID_HEIGHT_SGIX',
'GL_PIXEL_TILE_GRID_DEPTH_SGIX', 'GL_PIXEL_TILE_CACHE_SIZE_SGIX',
'GL_DUAL_ALPHA4_SGIS', 'GL_DUAL_ALPHA8_SGIS', 'GL_DUAL_ALPHA12_SGIS',
'GL_DUAL_ALPHA16_SGIS', 'GL_DUAL_LUMINANCE4_SGIS', 'GL_DUAL_LUMINANCE8_SGIS',
'GL_DUAL_LUMINANCE12_SGIS', 'GL_DUAL_LUMINANCE16_SGIS',
'GL_DUAL_INTENSITY4_SGIS', 'GL_DUAL_INTENSITY8_SGIS',
'GL_DUAL_INTENSITY12_SGIS', 'GL_DUAL_INTENSITY16_SGIS',
'GL_DUAL_LUMINANCE_ALPHA4_SGIS', 'GL_DUAL_LUMINANCE_ALPHA8_SGIS',
'GL_QUAD_ALPHA4_SGIS', 'GL_QUAD_ALPHA8_SGIS', 'GL_QUAD_LUMINANCE4_SGIS',
'GL_QUAD_LUMINANCE8_SGIS', 'GL_QUAD_INTENSITY4_SGIS',
'GL_QUAD_INTENSITY8_SGIS', 'GL_DUAL_TEXTURE_SELECT_SGIS',
'GL_QUAD_TEXTURE_SELECT_SGIS', 'GL_SPRITE_SGIX', 'GL_SPRITE_MODE_SGIX',
'GL_SPRITE_AXIS_SGIX', 'GL_SPRITE_TRANSLATION_SGIX', 'GL_SPRITE_AXIAL_SGIX',
'GL_SPRITE_OBJECT_ALIGNED_SGIX', 'GL_SPRITE_EYE_ALIGNED_SGIX',
'GL_TEXTURE_MULTI_BUFFER_HINT_SGIX', 'GL_POINT_SIZE_MIN_EXT',
'GL_POINT_SIZE_MAX_EXT', 'GL_POINT_FADE_THRESHOLD_SIZE_EXT',
'GL_DISTANCE_ATTENUATION_EXT', 'GL_POINT_SIZE_MIN_SGIS',
'GL_POINT_SIZE_MAX_SGIS', 'GL_POINT_FADE_THRESHOLD_SIZE_SGIS',
'GL_DISTANCE_ATTENUATION_SGIS', 'GL_INSTRUMENT_BUFFER_POINTER_SGIX',
'GL_INSTRUMENT_MEASUREMENTS_SGIX', 'GL_POST_TEXTURE_FILTER_BIAS_SGIX',
'GL_POST_TEXTURE_FILTER_SCALE_SGIX', 'GL_POST_TEXTURE_FILTER_BIAS_RANGE_SGIX',
'GL_POST_TEXTURE_FILTER_SCALE_RANGE_SGIX', 'GL_FRAMEZOOM_SGIX',
'GL_FRAMEZOOM_FACTOR_SGIX', 'GL_MAX_FRAMEZOOM_FACTOR_SGIX',
'GL_TEXTURE_DEFORMATION_BIT_SGIX', 'GL_GEOMETRY_DEFORMATION_BIT_SGIX',
'GL_GEOMETRY_DEFORMATION_SGIX', 'GL_TEXTURE_DEFORMATION_SGIX',
'GL_DEFORMATIONS_MASK_SGIX', 'GL_MAX_DEFORMATION_ORDER_SGIX',
'GL_REFERENCE_PLANE_SGIX', 'GL_REFERENCE_PLANE_EQUATION_SGIX',
'GL_DEPTH_COMPONENT16_SGIX', 'GL_DEPTH_COMPONENT24_SGIX',
'GL_DEPTH_COMPONENT32_SGIX', 'GL_FOG_FUNC_SGIS', 'GL_FOG_FUNC_POINTS_SGIS',
'GL_MAX_FOG_FUNC_POINTS_SGIS', 'GL_FOG_OFFSET_SGIX',
'GL_FOG_OFFSET_VALUE_SGIX', 'GL_IMAGE_SCALE_X_HP', 'GL_IMAGE_SCALE_Y_HP',
'GL_IMAGE_TRANSLATE_X_HP', 'GL_IMAGE_TRANSLATE_Y_HP',
'GL_IMAGE_ROTATE_ANGLE_HP', 'GL_IMAGE_ROTATE_ORIGIN_X_HP',
'GL_IMAGE_ROTATE_ORIGIN_Y_HP', 'GL_IMAGE_MAG_FILTER_HP',
'GL_IMAGE_MIN_FILTER_HP', 'GL_IMAGE_CUBIC_WEIGHT_HP', 'GL_CUBIC_HP',
'GL_AVERAGE_HP', 'GL_IMAGE_TRANSFORM_2D_HP',
'GL_POST_IMAGE_TRANSFORM_COLOR_TABLE_HP',
'GL_PROXY_POST_IMAGE_TRANSFORM_COLOR_TABLE_HP', 'GL_IGNORE_BORDER_HP',
'GL_CONSTANT_BORDER_HP', 'GL_REPLICATE_BORDER_HP',
'GL_CONVOLUTION_BORDER_COLOR_HP', 'GL_TEXTURE_ENV_BIAS_SGIX',
'GL_VERTEX_DATA_HINT_PGI', 'GL_VERTEX_CONSISTENT_HINT_PGI',
'GL_MATERIAL_SIDE_HINT_PGI', 'GL_MAX_VERTEX_HINT_PGI', 'GL_COLOR3_BIT_PGI',
'GL_COLOR4_BIT_PGI', 'GL_EDGEFLAG_BIT_PGI', 'GL_INDEX_BIT_PGI',
'GL_MAT_AMBIENT_BIT_PGI', 'GL_MAT_AMBIENT_AND_DIFFUSE_BIT_PGI',
'GL_MAT_DIFFUSE_BIT_PGI', 'GL_MAT_EMISSION_BIT_PGI',
'GL_MAT_COLOR_INDEXES_BIT_PGI', 'GL_MAT_SHININESS_BIT_PGI',
'GL_MAT_SPECULAR_BIT_PGI', 'GL_NORMAL_BIT_PGI', 'GL_TEXCOORD1_BIT_PGI',
'GL_TEXCOORD2_BIT_PGI', 'GL_TEXCOORD3_BIT_PGI', 'GL_TEXCOORD4_BIT_PGI',
'GL_VERTEX23_BIT_PGI', 'GL_VERTEX4_BIT_PGI',
'GL_PREFER_DOUBLEBUFFER_HINT_PGI', 'GL_CONSERVE_MEMORY_HINT_PGI',
'GL_RECLAIM_MEMORY_HINT_PGI', 'GL_NATIVE_GRAPHICS_HANDLE_PGI',
'GL_NATIVE_GRAPHICS_BEGIN_HINT_PGI', 'GL_NATIVE_GRAPHICS_END_HINT_PGI',
'GL_ALWAYS_FAST_HINT_PGI', 'GL_ALWAYS_SOFT_HINT_PGI',
'GL_ALLOW_DRAW_OBJ_HINT_PGI', 'GL_ALLOW_DRAW_WIN_HINT_PGI',
'GL_ALLOW_DRAW_FRG_HINT_PGI', 'GL_ALLOW_DRAW_MEM_HINT_PGI',
'GL_STRICT_DEPTHFUNC_HINT_PGI', 'GL_STRICT_LIGHTING_HINT_PGI',
'GL_STRICT_SCISSOR_HINT_PGI', 'GL_FULL_STIPPLE_HINT_PGI',
'GL_CLIP_NEAR_HINT_PGI', 'GL_CLIP_FAR_HINT_PGI', 'GL_WIDE_LINE_HINT_PGI',
'GL_BACK_NORMALS_HINT_PGI', 'GL_COLOR_INDEX1_EXT', 'GL_COLOR_INDEX2_EXT',
'GL_COLOR_INDEX4_EXT', 'GL_COLOR_INDEX8_EXT', 'GL_COLOR_INDEX12_EXT',
'GL_COLOR_INDEX16_EXT', 'GL_TEXTURE_INDEX_SIZE_EXT',
'GL_CLIP_VOLUME_CLIPPING_HINT_EXT', 'GL_LIST_PRIORITY_SGIX',
'GL_IR_INSTRUMENT1_SGIX', 'GL_CALLIGRAPHIC_FRAGMENT_SGIX',
'GL_TEXTURE_LOD_BIAS_S_SGIX', 'GL_TEXTURE_LOD_BIAS_T_SGIX',
'GL_TEXTURE_LOD_BIAS_R_SGIX', 'GL_SHADOW_AMBIENT_SGIX',
'GL_INDEX_MATERIAL_EXT', 'GL_INDEX_MATERIAL_PARAMETER_EXT',
'GL_INDEX_MATERIAL_FACE_EXT', 'GL_INDEX_TEST_EXT', 'GL_INDEX_TEST_FUNC_EXT',
'GL_INDEX_TEST_REF_EXT', 'GL_IUI_V2F_EXT', 'GL_IUI_V3F_EXT',
'GL_IUI_N3F_V2F_EXT', 'GL_IUI_N3F_V3F_EXT', 'GL_T2F_IUI_V2F_EXT',
'GL_T2F_IUI_V3F_EXT', 'GL_T2F_IUI_N3F_V2F_EXT', 'GL_T2F_IUI_N3F_V3F_EXT',
'GL_ARRAY_ELEMENT_LOCK_FIRST_EXT', 'GL_ARRAY_ELEMENT_LOCK_COUNT_EXT',
'GL_CULL_VERTEX_EXT', 'GL_CULL_VERTEX_EYE_POSITION_EXT',
'GL_CULL_VERTEX_OBJECT_POSITION_EXT', 'GL_YCRCB_422_SGIX',
'GL_YCRCB_444_SGIX', 'GL_FRAGMENT_LIGHTING_SGIX',
'GL_FRAGMENT_COLOR_MATERIAL_SGIX', 'GL_FRAGMENT_COLOR_MATERIAL_FACE_SGIX',
'GL_FRAGMENT_COLOR_MATERIAL_PARAMETER_SGIX', 'GL_MAX_FRAGMENT_LIGHTS_SGIX',
'GL_MAX_ACTIVE_LIGHTS_SGIX', 'GL_CURRENT_RASTER_NORMAL_SGIX',
'GL_LIGHT_ENV_MODE_SGIX', 'GL_FRAGMENT_LIGHT_MODEL_LOCAL_VIEWER_SGIX',
'GL_FRAGMENT_LIGHT_MODEL_TWO_SIDE_SGIX',
'GL_FRAGMENT_LIGHT_MODEL_AMBIENT_SGIX',
'GL_FRAGMENT_LIGHT_MODEL_NORMAL_INTERPOLATION_SGIX',
'GL_FRAGMENT_LIGHT0_SGIX', 'GL_FRAGMENT_LIGHT1_SGIX',
'GL_FRAGMENT_LIGHT2_SGIX', 'GL_FRAGMENT_LIGHT3_SGIX',
'GL_FRAGMENT_LIGHT4_SGIX', 'GL_FRAGMENT_LIGHT5_SGIX',
'GL_FRAGMENT_LIGHT6_SGIX', 'GL_FRAGMENT_LIGHT7_SGIX',
'GL_RASTER_POSITION_UNCLIPPED_IBM', 'GL_TEXTURE_LIGHTING_MODE_HP',
'GL_TEXTURE_POST_SPECULAR_HP', 'GL_TEXTURE_PRE_SPECULAR_HP',
'GL_MAX_ELEMENTS_VERTICES_EXT', 'GL_MAX_ELEMENTS_INDICES_EXT', 'GL_PHONG_WIN',
'GL_PHONG_HINT_WIN', 'GL_FOG_SPECULAR_TEXTURE_WIN',
'GL_FRAGMENT_MATERIAL_EXT', 'GL_FRAGMENT_NORMAL_EXT', 'GL_FRAGMENT_COLOR_EXT',
'GL_ATTENUATION_EXT', 'GL_SHADOW_ATTENUATION_EXT',
'GL_TEXTURE_APPLICATION_MODE_EXT', 'GL_TEXTURE_LIGHT_EXT',
'GL_TEXTURE_MATERIAL_FACE_EXT', 'GL_TEXTURE_MATERIAL_PARAMETER_EXT',
'GL_ALPHA_MIN_SGIX', 'GL_ALPHA_MAX_SGIX', 'GL_PIXEL_TEX_GEN_Q_CEILING_SGIX',
'GL_PIXEL_TEX_GEN_Q_ROUND_SGIX', 'GL_PIXEL_TEX_GEN_Q_FLOOR_SGIX',
'GL_PIXEL_TEX_GEN_ALPHA_REPLACE_SGIX',
'GL_PIXEL_TEX_GEN_ALPHA_NO_REPLACE_SGIX', 'GL_PIXEL_TEX_GEN_ALPHA_LS_SGIX',
'GL_PIXEL_TEX_GEN_ALPHA_MS_SGIX', 'GL_BGR_EXT', 'GL_BGRA_EXT',
'GL_ASYNC_MARKER_SGIX', 'GL_ASYNC_TEX_IMAGE_SGIX',
'GL_ASYNC_DRAW_PIXELS_SGIX', 'GL_ASYNC_READ_PIXELS_SGIX',
'GL_MAX_ASYNC_TEX_IMAGE_SGIX', 'GL_MAX_ASYNC_DRAW_PIXELS_SGIX',
'GL_MAX_ASYNC_READ_PIXELS_SGIX', 'GL_ASYNC_HISTOGRAM_SGIX',
'GL_MAX_ASYNC_HISTOGRAM_SGIX', 'GL_PARALLEL_ARRAYS_INTEL',
'GL_VERTEX_ARRAY_PARALLEL_POINTERS_INTEL',
'GL_NORMAL_ARRAY_PARALLEL_POINTERS_INTEL',
'GL_COLOR_ARRAY_PARALLEL_POINTERS_INTEL',
'GL_TEXTURE_COORD_ARRAY_PARALLEL_POINTERS_INTEL', 'GL_OCCLUSION_TEST_HP',
'GL_OCCLUSION_TEST_RESULT_HP', 'GL_PIXEL_TRANSFORM_2D_EXT',
'GL_PIXEL_MAG_FILTER_EXT', 'GL_PIXEL_MIN_FILTER_EXT',
'GL_PIXEL_CUBIC_WEIGHT_EXT', 'GL_CUBIC_EXT', 'GL_AVERAGE_EXT',
'GL_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT',
'GL_MAX_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT',
'GL_PIXEL_TRANSFORM_2D_MATRIX_EXT', 'GL_SHARED_TEXTURE_PALETTE_EXT',
'GL_LIGHT_MODEL_COLOR_CONTROL_EXT', 'GL_SINGLE_COLOR_EXT',
'GL_SEPARATE_SPECULAR_COLOR_EXT', 'GL_COLOR_SUM_EXT',
'GL_CURRENT_SECONDARY_COLOR_EXT', 'GL_SECONDARY_COLOR_ARRAY_SIZE_EXT',
'GL_SECONDARY_COLOR_ARRAY_TYPE_EXT', 'GL_SECONDARY_COLOR_ARRAY_STRIDE_EXT',
'GL_SECONDARY_COLOR_ARRAY_POINTER_EXT', 'GL_SECONDARY_COLOR_ARRAY_EXT',
'GL_PERTURB_EXT', 'GL_TEXTURE_NORMAL_EXT', 'GL_FOG_COORDINATE_SOURCE_EXT',
'GL_FOG_COORDINATE_EXT', 'GL_FRAGMENT_DEPTH_EXT',
'GL_CURRENT_FOG_COORDINATE_EXT', 'GL_FOG_COORDINATE_ARRAY_TYPE_EXT',
'GL_FOG_COORDINATE_ARRAY_STRIDE_EXT', 'GL_FOG_COORDINATE_ARRAY_POINTER_EXT',
'GL_FOG_COORDINATE_ARRAY_EXT', 'GL_SCREEN_COORDINATES_REND',
'GL_INVERTED_SCREEN_W_REND', 'GL_TANGENT_ARRAY_EXT', 'GL_BINORMAL_ARRAY_EXT',
'GL_CURRENT_TANGENT_EXT', 'GL_CURRENT_BINORMAL_EXT',
'GL_TANGENT_ARRAY_TYPE_EXT', 'GL_TANGENT_ARRAY_STRIDE_EXT',
'GL_BINORMAL_ARRAY_TYPE_EXT', 'GL_BINORMAL_ARRAY_STRIDE_EXT',
'GL_TANGENT_ARRAY_POINTER_EXT', 'GL_BINORMAL_ARRAY_POINTER_EXT',
'GL_MAP1_TANGENT_EXT', 'GL_MAP2_TANGENT_EXT', 'GL_MAP1_BINORMAL_EXT',
'GL_MAP2_BINORMAL_EXT', 'GL_COMBINE_EXT', 'GL_COMBINE_RGB_EXT',
'GL_COMBINE_ALPHA_EXT', 'GL_RGB_SCALE_EXT', 'GL_ADD_SIGNED_EXT',
'GL_INTERPOLATE_EXT', 'GL_CONSTANT_EXT', 'GL_PRIMARY_COLOR_EXT',
'GL_PREVIOUS_EXT', 'GL_SOURCE0_RGB_EXT', 'GL_SOURCE1_RGB_EXT',
'GL_SOURCE2_RGB_EXT', 'GL_SOURCE0_ALPHA_EXT', 'GL_SOURCE1_ALPHA_EXT',
'GL_SOURCE2_ALPHA_EXT', 'GL_OPERAND0_RGB_EXT', 'GL_OPERAND1_RGB_EXT',
'GL_OPERAND2_RGB_EXT', 'GL_OPERAND0_ALPHA_EXT', 'GL_OPERAND1_ALPHA_EXT',
'GL_OPERAND2_ALPHA_EXT', 'GL_LIGHT_MODEL_SPECULAR_VECTOR_APPLE',
'GL_TRANSFORM_HINT_APPLE', 'GL_FOG_SCALE_SGIX', 'GL_FOG_SCALE_VALUE_SGIX',
'GL_UNPACK_CONSTANT_DATA_SUNX', 'GL_TEXTURE_CONSTANT_DATA_SUNX',
'GL_GLOBAL_ALPHA_SUN', 'GL_GLOBAL_ALPHA_FACTOR_SUN', 'GL_RESTART_SUN',
'GL_REPLACE_MIDDLE_SUN', 'GL_REPLACE_OLDEST_SUN', 'GL_TRIANGLE_LIST_SUN',
'GL_REPLACEMENT_CODE_SUN', 'GL_REPLACEMENT_CODE_ARRAY_SUN',
'GL_REPLACEMENT_CODE_ARRAY_TYPE_SUN', 'GL_REPLACEMENT_CODE_ARRAY_STRIDE_SUN',
'GL_REPLACEMENT_CODE_ARRAY_POINTER_SUN', 'GL_R1UI_V3F_SUN',
'GL_R1UI_C4UB_V3F_SUN', 'GL_R1UI_C3F_V3F_SUN', 'GL_R1UI_N3F_V3F_SUN',
'GL_R1UI_C4F_N3F_V3F_SUN', 'GL_R1UI_T2F_V3F_SUN', 'GL_R1UI_T2F_N3F_V3F_SUN',
'GL_R1UI_T2F_C4F_N3F_V3F_SUN', 'GL_BLEND_DST_RGB_EXT', 'GL_BLEND_SRC_RGB_EXT',
'GL_BLEND_DST_ALPHA_EXT', 'GL_BLEND_SRC_ALPHA_EXT', 'GL_RED_MIN_CLAMP_INGR',
'GL_GREEN_MIN_CLAMP_INGR', 'GL_BLUE_MIN_CLAMP_INGR',
'GL_ALPHA_MIN_CLAMP_INGR', 'GL_RED_MAX_CLAMP_INGR', 'GL_GREEN_MAX_CLAMP_INGR',
'GL_BLUE_MAX_CLAMP_INGR', 'GL_ALPHA_MAX_CLAMP_INGR', 'GL_INTERLACE_READ_INGR',
'GL_INCR_WRAP_EXT', 'GL_DECR_WRAP_EXT', 'GL_422_EXT', 'GL_422_REV_EXT',
'GL_422_AVERAGE_EXT', 'GL_422_REV_AVERAGE_EXT', 'GL_NORMAL_MAP_NV',
'GL_REFLECTION_MAP_NV', 'GL_NORMAL_MAP_EXT', 'GL_REFLECTION_MAP_EXT',
'GL_TEXTURE_CUBE_MAP_EXT', 'GL_TEXTURE_BINDING_CUBE_MAP_EXT',
'GL_TEXTURE_CUBE_MAP_POSITIVE_X_EXT', 'GL_TEXTURE_CUBE_MAP_NEGATIVE_X_EXT',
'GL_TEXTURE_CUBE_MAP_POSITIVE_Y_EXT', 'GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_EXT',
'GL_TEXTURE_CUBE_MAP_POSITIVE_Z_EXT', 'GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_EXT',
'GL_PROXY_TEXTURE_CUBE_MAP_EXT', 'GL_MAX_CUBE_MAP_TEXTURE_SIZE_EXT',
'GL_WRAP_BORDER_SUN', 'GL_MAX_TEXTURE_LOD_BIAS_EXT',
'GL_TEXTURE_FILTER_CONTROL_EXT', 'GL_TEXTURE_LOD_BIAS_EXT',
'GL_TEXTURE_MAX_ANISOTROPY_EXT', 'GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT',
'GL_MODELVIEW0_STACK_DEPTH_EXT', 'GL_MODELVIEW1_STACK_DEPTH_EXT',
'GL_MODELVIEW0_MATRIX_EXT', 'GL_MODELVIEW1_MATRIX_EXT',
'GL_VERTEX_WEIGHTING_EXT', 'GL_MODELVIEW0_EXT', 'GL_MODELVIEW1_EXT',
'GL_CURRENT_VERTEX_WEIGHT_EXT', 'GL_VERTEX_WEIGHT_ARRAY_EXT',
'GL_VERTEX_WEIGHT_ARRAY_SIZE_EXT', 'GL_VERTEX_WEIGHT_ARRAY_TYPE_EXT',
'GL_VERTEX_WEIGHT_ARRAY_STRIDE_EXT', 'GL_VERTEX_WEIGHT_ARRAY_POINTER_EXT',
'GL_MAX_SHININESS_NV', 'GL_MAX_SPOT_EXPONENT_NV', 'GL_VERTEX_ARRAY_RANGE_NV',
'GL_VERTEX_ARRAY_RANGE_LENGTH_NV', 'GL_VERTEX_ARRAY_RANGE_VALID_NV',
'GL_MAX_VERTEX_ARRAY_RANGE_ELEMENT_NV', 'GL_VERTEX_ARRAY_RANGE_POINTER_NV',
'GL_REGISTER_COMBINERS_NV', 'GL_VARIABLE_A_NV', 'GL_VARIABLE_B_NV',
'GL_VARIABLE_C_NV', 'GL_VARIABLE_D_NV', 'GL_VARIABLE_E_NV',
'GL_VARIABLE_F_NV', 'GL_VARIABLE_G_NV', 'GL_CONSTANT_COLOR0_NV',
'GL_CONSTANT_COLOR1_NV', 'GL_PRIMARY_COLOR_NV', 'GL_SECONDARY_COLOR_NV',
'GL_SPARE0_NV', 'GL_SPARE1_NV', 'GL_DISCARD_NV', 'GL_E_TIMES_F_NV',
'GL_SPARE0_PLUS_SECONDARY_COLOR_NV', 'GL_UNSIGNED_IDENTITY_NV',
'GL_UNSIGNED_INVERT_NV', 'GL_EXPAND_NORMAL_NV', 'GL_EXPAND_NEGATE_NV',
'GL_HALF_BIAS_NORMAL_NV', 'GL_HALF_BIAS_NEGATE_NV', 'GL_SIGNED_IDENTITY_NV',
'GL_SIGNED_NEGATE_NV', 'GL_SCALE_BY_TWO_NV', 'GL_SCALE_BY_FOUR_NV',
'GL_SCALE_BY_ONE_HALF_NV', 'GL_BIAS_BY_NEGATIVE_ONE_HALF_NV',
'GL_COMBINER_INPUT_NV', 'GL_COMBINER_MAPPING_NV',
'GL_COMBINER_COMPONENT_USAGE_NV', 'GL_COMBINER_AB_DOT_PRODUCT_NV',
'GL_COMBINER_CD_DOT_PRODUCT_NV', 'GL_COMBINER_MUX_SUM_NV',
'GL_COMBINER_SCALE_NV', 'GL_COMBINER_BIAS_NV', 'GL_COMBINER_AB_OUTPUT_NV',
'GL_COMBINER_CD_OUTPUT_NV', 'GL_COMBINER_SUM_OUTPUT_NV',
'GL_MAX_GENERAL_COMBINERS_NV', 'GL_NUM_GENERAL_COMBINERS_NV',
'GL_COLOR_SUM_CLAMP_NV', 'GL_COMBINER0_NV', 'GL_COMBINER1_NV',
'GL_COMBINER2_NV', 'GL_COMBINER3_NV', 'GL_COMBINER4_NV', 'GL_COMBINER5_NV',
'GL_COMBINER6_NV', 'GL_COMBINER7_NV', 'GL_FOG_DISTANCE_MODE_NV',
'GL_EYE_RADIAL_NV', 'GL_EYE_PLANE_ABSOLUTE_NV', 'GL_EMBOSS_LIGHT_NV',
'GL_EMBOSS_CONSTANT_NV', 'GL_EMBOSS_MAP_NV', 'GL_COMBINE4_NV',
'GL_SOURCE3_RGB_NV', 'GL_SOURCE3_ALPHA_NV', 'GL_OPERAND3_RGB_NV',
'GL_OPERAND3_ALPHA_NV', 'GL_COMPRESSED_RGB_S3TC_DXT1_EXT',
'GL_COMPRESSED_RGBA_S3TC_DXT1_EXT', 'GL_COMPRESSED_RGBA_S3TC_DXT3_EXT',
'GL_COMPRESSED_RGBA_S3TC_DXT5_EXT', 'GL_CULL_VERTEX_IBM',
'GL_VERTEX_ARRAY_LIST_IBM', 'GL_NORMAL_ARRAY_LIST_IBM',
'GL_COLOR_ARRAY_LIST_IBM', 'GL_INDEX_ARRAY_LIST_IBM',
'GL_TEXTURE_COORD_ARRAY_LIST_IBM', 'GL_EDGE_FLAG_ARRAY_LIST_IBM',
'GL_FOG_COORDINATE_ARRAY_LIST_IBM', 'GL_SECONDARY_COLOR_ARRAY_LIST_IBM',
'GL_VERTEX_ARRAY_LIST_STRIDE_IBM', 'GL_NORMAL_ARRAY_LIST_STRIDE_IBM',
'GL_COLOR_ARRAY_LIST_STRIDE_IBM', 'GL_INDEX_ARRAY_LIST_STRIDE_IBM',
'GL_TEXTURE_COORD_ARRAY_LIST_STRIDE_IBM',
'GL_EDGE_FLAG_ARRAY_LIST_STRIDE_IBM',
'GL_FOG_COORDINATE_ARRAY_LIST_STRIDE_IBM',
'GL_SECONDARY_COLOR_ARRAY_LIST_STRIDE_IBM', 'GL_PACK_SUBSAMPLE_RATE_SGIX',
'GL_UNPACK_SUBSAMPLE_RATE_SGIX', 'GL_PIXEL_SUBSAMPLE_4444_SGIX',
'GL_PIXEL_SUBSAMPLE_2424_SGIX', 'GL_PIXEL_SUBSAMPLE_4242_SGIX',
'GL_YCRCB_SGIX', 'GL_YCRCBA_SGIX', 'GL_DEPTH_PASS_INSTRUMENT_SGIX',
'GL_DEPTH_PASS_INSTRUMENT_COUNTERS_SGIX', 'GL_DEPTH_PASS_INSTRUMENT_MAX_SGIX',
'GL_COMPRESSED_RGB_FXT1_3DFX', 'GL_COMPRESSED_RGBA_FXT1_3DFX',
'GL_MULTISAMPLE_3DFX', 'GL_SAMPLE_BUFFERS_3DFX', 'GL_SAMPLES_3DFX',
'GL_MULTISAMPLE_BIT_3DFX', 'GL_MULTISAMPLE_EXT',
'GL_SAMPLE_ALPHA_TO_MASK_EXT', 'GL_SAMPLE_ALPHA_TO_ONE_EXT',
'GL_SAMPLE_MASK_EXT', 'GL_1PASS_EXT', 'GL_2PASS_0_EXT', 'GL_2PASS_1_EXT',
'GL_4PASS_0_EXT', 'GL_4PASS_1_EXT', 'GL_4PASS_2_EXT', 'GL_4PASS_3_EXT',
'GL_SAMPLE_BUFFERS_EXT', 'GL_SAMPLES_EXT', 'GL_SAMPLE_MASK_VALUE_EXT',
'GL_SAMPLE_MASK_INVERT_EXT', 'GL_SAMPLE_PATTERN_EXT',
'GL_MULTISAMPLE_BIT_EXT', 'GL_VERTEX_PRECLIP_SGIX',
'GL_VERTEX_PRECLIP_HINT_SGIX', 'GL_CONVOLUTION_HINT_SGIX',
'GL_PACK_RESAMPLE_SGIX', 'GL_UNPACK_RESAMPLE_SGIX',
'GL_RESAMPLE_REPLICATE_SGIX', 'GL_RESAMPLE_ZERO_FILL_SGIX',
'GL_RESAMPLE_DECIMATE_SGIX', 'GL_EYE_DISTANCE_TO_POINT_SGIS',
'GL_OBJECT_DISTANCE_TO_POINT_SGIS', 'GL_EYE_DISTANCE_TO_LINE_SGIS',
'GL_OBJECT_DISTANCE_TO_LINE_SGIS', 'GL_EYE_POINT_SGIS',
'GL_OBJECT_POINT_SGIS', 'GL_EYE_LINE_SGIS', 'GL_OBJECT_LINE_SGIS',
'GL_TEXTURE_COLOR_WRITEMASK_SGIS', 'GL_DOT3_RGB_EXT', 'GL_DOT3_RGBA_EXT',
'GL_MIRROR_CLAMP_ATI', 'GL_MIRROR_CLAMP_TO_EDGE_ATI', 'GL_ALL_COMPLETED_NV',
'GL_FENCE_STATUS_NV', 'GL_FENCE_CONDITION_NV', 'GL_MIRRORED_REPEAT_IBM',
'GL_EVAL_2D_NV', 'GL_EVAL_TRIANGULAR_2D_NV', 'GL_MAP_TESSELLATION_NV',
'GL_MAP_ATTRIB_U_ORDER_NV', 'GL_MAP_ATTRIB_V_ORDER_NV',
'GL_EVAL_FRACTIONAL_TESSELLATION_NV', 'GL_EVAL_VERTEX_ATTRIB0_NV',
'GL_EVAL_VERTEX_ATTRIB1_NV', 'GL_EVAL_VERTEX_ATTRIB2_NV',
'GL_EVAL_VERTEX_ATTRIB3_NV', 'GL_EVAL_VERTEX_ATTRIB4_NV',
'GL_EVAL_VERTEX_ATTRIB5_NV', 'GL_EVAL_VERTEX_ATTRIB6_NV',
'GL_EVAL_VERTEX_ATTRIB7_NV', 'GL_EVAL_VERTEX_ATTRIB8_NV',
'GL_EVAL_VERTEX_ATTRIB9_NV', 'GL_EVAL_VERTEX_ATTRIB10_NV',
'GL_EVAL_VERTEX_ATTRIB11_NV', 'GL_EVAL_VERTEX_ATTRIB12_NV',
'GL_EVAL_VERTEX_ATTRIB13_NV', 'GL_EVAL_VERTEX_ATTRIB14_NV',
'GL_EVAL_VERTEX_ATTRIB15_NV', 'GL_MAX_MAP_TESSELLATION_NV',
'GL_MAX_RATIONAL_EVAL_ORDER_NV', 'GL_DEPTH_STENCIL_NV',
'GL_UNSIGNED_INT_24_8_NV', 'GL_PER_STAGE_CONSTANTS_NV',
'GL_TEXTURE_RECTANGLE_NV', 'GL_TEXTURE_BINDING_RECTANGLE_NV',
'GL_PROXY_TEXTURE_RECTANGLE_NV', 'GL_MAX_RECTANGLE_TEXTURE_SIZE_NV',
'GL_OFFSET_TEXTURE_RECTANGLE_NV', 'GL_OFFSET_TEXTURE_RECTANGLE_SCALE_NV',
'GL_DOT_PRODUCT_TEXTURE_RECTANGLE_NV',
'GL_RGBA_UNSIGNED_DOT_PRODUCT_MAPPING_NV', 'GL_UNSIGNED_INT_S8_S8_8_8_NV',
'GL_UNSIGNED_INT_8_8_S8_S8_REV_NV', 'GL_DSDT_MAG_INTENSITY_NV',
'GL_SHADER_CONSISTENT_NV', 'GL_TEXTURE_SHADER_NV', 'GL_SHADER_OPERATION_NV',
'GL_CULL_MODES_NV', 'GL_OFFSET_TEXTURE_MATRIX_NV',
'GL_OFFSET_TEXTURE_SCALE_NV', 'GL_OFFSET_TEXTURE_BIAS_NV',
'GL_OFFSET_TEXTURE_2D_MATRIX_NV', 'GL_OFFSET_TEXTURE_2D_SCALE_NV',
'GL_OFFSET_TEXTURE_2D_BIAS_NV', 'GL_PREVIOUS_TEXTURE_INPUT_NV',
'GL_CONST_EYE_NV', 'GL_PASS_THROUGH_NV', 'GL_CULL_FRAGMENT_NV',
'GL_OFFSET_TEXTURE_2D_NV', 'GL_DEPENDENT_AR_TEXTURE_2D_NV',
'GL_DEPENDENT_GB_TEXTURE_2D_NV', 'GL_DOT_PRODUCT_NV',
'GL_DOT_PRODUCT_DEPTH_REPLACE_NV', 'GL_DOT_PRODUCT_TEXTURE_2D_NV',
'GL_DOT_PRODUCT_TEXTURE_CUBE_MAP_NV', 'GL_DOT_PRODUCT_DIFFUSE_CUBE_MAP_NV',
'GL_DOT_PRODUCT_REFLECT_CUBE_MAP_NV',
'GL_DOT_PRODUCT_CONST_EYE_REFLECT_CUBE_MAP_NV', 'GL_HILO_NV', 'GL_DSDT_NV',
'GL_DSDT_MAG_NV', 'GL_DSDT_MAG_VIB_NV', 'GL_HILO16_NV', 'GL_SIGNED_HILO_NV',
'GL_SIGNED_HILO16_NV', 'GL_SIGNED_RGBA_NV', 'GL_SIGNED_RGBA8_NV',
'GL_SIGNED_RGB_NV', 'GL_SIGNED_RGB8_NV', 'GL_SIGNED_LUMINANCE_NV',
'GL_SIGNED_LUMINANCE8_NV', 'GL_SIGNED_LUMINANCE_ALPHA_NV',
'GL_SIGNED_LUMINANCE8_ALPHA8_NV', 'GL_SIGNED_ALPHA_NV', 'GL_SIGNED_ALPHA8_NV',
'GL_SIGNED_INTENSITY_NV', 'GL_SIGNED_INTENSITY8_NV', 'GL_DSDT8_NV',
'GL_DSDT8_MAG8_NV', 'GL_DSDT8_MAG8_INTENSITY8_NV',
'GL_SIGNED_RGB_UNSIGNED_ALPHA_NV', 'GL_SIGNED_RGB8_UNSIGNED_ALPHA8_NV',
'GL_HI_SCALE_NV', 'GL_LO_SCALE_NV', 'GL_DS_SCALE_NV', 'GL_DT_SCALE_NV',
'GL_MAGNITUDE_SCALE_NV', 'GL_VIBRANCE_SCALE_NV', 'GL_HI_BIAS_NV',
'GL_LO_BIAS_NV', 'GL_DS_BIAS_NV', 'GL_DT_BIAS_NV', 'GL_MAGNITUDE_BIAS_NV',
'GL_VIBRANCE_BIAS_NV', 'GL_TEXTURE_BORDER_VALUES_NV', 'GL_TEXTURE_HI_SIZE_NV',
'GL_TEXTURE_LO_SIZE_NV', 'GL_TEXTURE_DS_SIZE_NV', 'GL_TEXTURE_DT_SIZE_NV',
'GL_TEXTURE_MAG_SIZE_NV', 'GL_DOT_PRODUCT_TEXTURE_3D_NV',
'GL_VERTEX_ARRAY_RANGE_WITHOUT_FLUSH_NV', 'GL_VERTEX_PROGRAM_NV',
'GL_VERTEX_STATE_PROGRAM_NV', 'GL_ATTRIB_ARRAY_SIZE_NV',
'GL_ATTRIB_ARRAY_STRIDE_NV', 'GL_ATTRIB_ARRAY_TYPE_NV',
'GL_CURRENT_ATTRIB_NV', 'GL_PROGRAM_LENGTH_NV', 'GL_PROGRAM_STRING_NV',
'GL_MODELVIEW_PROJECTION_NV', 'GL_IDENTITY_NV', 'GL_INVERSE_NV',
'GL_TRANSPOSE_NV', 'GL_INVERSE_TRANSPOSE_NV',
'GL_MAX_TRACK_MATRIX_STACK_DEPTH_NV', 'GL_MAX_TRACK_MATRICES_NV',
'GL_MATRIX0_NV', 'GL_MATRIX1_NV', 'GL_MATRIX2_NV', 'GL_MATRIX3_NV',
'GL_MATRIX4_NV', 'GL_MATRIX5_NV', 'GL_MATRIX6_NV', 'GL_MATRIX7_NV',
'GL_CURRENT_MATRIX_STACK_DEPTH_NV', 'GL_CURRENT_MATRIX_NV',
'GL_VERTEX_PROGRAM_POINT_SIZE_NV', 'GL_VERTEX_PROGRAM_TWO_SIDE_NV',
'GL_PROGRAM_PARAMETER_NV', 'GL_ATTRIB_ARRAY_POINTER_NV',
'GL_PROGRAM_TARGET_NV', 'GL_PROGRAM_RESIDENT_NV', 'GL_TRACK_MATRIX_NV',
'GL_TRACK_MATRIX_TRANSFORM_NV', 'GL_VERTEX_PROGRAM_BINDING_NV',
'GL_PROGRAM_ERROR_POSITION_NV', 'GL_VERTEX_ATTRIB_ARRAY0_NV',
'GL_VERTEX_ATTRIB_ARRAY1_NV', 'GL_VERTEX_ATTRIB_ARRAY2_NV',
'GL_VERTEX_ATTRIB_ARRAY3_NV', 'GL_VERTEX_ATTRIB_ARRAY4_NV',
'GL_VERTEX_ATTRIB_ARRAY5_NV', 'GL_VERTEX_ATTRIB_ARRAY6_NV',
'GL_VERTEX_ATTRIB_ARRAY7_NV', 'GL_VERTEX_ATTRIB_ARRAY8_NV',
'GL_VERTEX_ATTRIB_ARRAY9_NV', 'GL_VERTEX_ATTRIB_ARRAY10_NV',
'GL_VERTEX_ATTRIB_ARRAY11_NV', 'GL_VERTEX_ATTRIB_ARRAY12_NV',
'GL_VERTEX_ATTRIB_ARRAY13_NV', 'GL_VERTEX_ATTRIB_ARRAY14_NV',
'GL_VERTEX_ATTRIB_ARRAY15_NV', 'GL_MAP1_VERTEX_ATTRIB0_4_NV',
'GL_MAP1_VERTEX_ATTRIB1_4_NV', 'GL_MAP1_VERTEX_ATTRIB2_4_NV',
'GL_MAP1_VERTEX_ATTRIB3_4_NV', 'GL_MAP1_VERTEX_ATTRIB4_4_NV',
'GL_MAP1_VERTEX_ATTRIB5_4_NV', 'GL_MAP1_VERTEX_ATTRIB6_4_NV',
'GL_MAP1_VERTEX_ATTRIB7_4_NV', 'GL_MAP1_VERTEX_ATTRIB8_4_NV',
'GL_MAP1_VERTEX_ATTRIB9_4_NV', 'GL_MAP1_VERTEX_ATTRIB10_4_NV',
'GL_MAP1_VERTEX_ATTRIB11_4_NV', 'GL_MAP1_VERTEX_ATTRIB12_4_NV',
'GL_MAP1_VERTEX_ATTRIB13_4_NV', 'GL_MAP1_VERTEX_ATTRIB14_4_NV',
'GL_MAP1_VERTEX_ATTRIB15_4_NV', 'GL_MAP2_VERTEX_ATTRIB0_4_NV',
'GL_MAP2_VERTEX_ATTRIB1_4_NV', 'GL_MAP2_VERTEX_ATTRIB2_4_NV',
'GL_MAP2_VERTEX_ATTRIB3_4_NV', 'GL_MAP2_VERTEX_ATTRIB4_4_NV',
'GL_MAP2_VERTEX_ATTRIB5_4_NV', 'GL_MAP2_VERTEX_ATTRIB6_4_NV',
'GL_MAP2_VERTEX_ATTRIB7_4_NV', 'GL_MAP2_VERTEX_ATTRIB8_4_NV',
'GL_MAP2_VERTEX_ATTRIB9_4_NV', 'GL_MAP2_VERTEX_ATTRIB10_4_NV',
'GL_MAP2_VERTEX_ATTRIB11_4_NV', 'GL_MAP2_VERTEX_ATTRIB12_4_NV',
'GL_MAP2_VERTEX_ATTRIB13_4_NV', 'GL_MAP2_VERTEX_ATTRIB14_4_NV',
'GL_MAP2_VERTEX_ATTRIB15_4_NV', 'GL_TEXTURE_MAX_CLAMP_S_SGIX',
'GL_TEXTURE_MAX_CLAMP_T_SGIX', 'GL_TEXTURE_MAX_CLAMP_R_SGIX',
'GL_SCALEBIAS_HINT_SGIX', 'GL_INTERLACE_OML', 'GL_INTERLACE_READ_OML',
'GL_FORMAT_SUBSAMPLE_24_24_OML', 'GL_FORMAT_SUBSAMPLE_244_244_OML',
'GL_PACK_RESAMPLE_OML', 'GL_UNPACK_RESAMPLE_OML', 'GL_RESAMPLE_REPLICATE_OML',
'GL_RESAMPLE_ZERO_FILL_OML', 'GL_RESAMPLE_AVERAGE_OML',
'GL_RESAMPLE_DECIMATE_OML', 'GL_DEPTH_STENCIL_TO_RGBA_NV',
'GL_DEPTH_STENCIL_TO_BGRA_NV', 'GL_BUMP_ROT_MATRIX_ATI',
'GL_BUMP_ROT_MATRIX_SIZE_ATI', 'GL_BUMP_NUM_TEX_UNITS_ATI',
'GL_BUMP_TEX_UNITS_ATI', 'GL_DUDV_ATI', 'GL_DU8DV8_ATI', 'GL_BUMP_ENVMAP_ATI',
'GL_BUMP_TARGET_ATI', 'GL_FRAGMENT_SHADER_ATI', 'GL_REG_0_ATI',
'GL_REG_1_ATI', 'GL_REG_2_ATI', 'GL_REG_3_ATI', 'GL_REG_4_ATI',
'GL_REG_5_ATI', 'GL_REG_6_ATI', 'GL_REG_7_ATI', 'GL_REG_8_ATI',
'GL_REG_9_ATI', 'GL_REG_10_ATI', 'GL_REG_11_ATI', 'GL_REG_12_ATI',
'GL_REG_13_ATI', 'GL_REG_14_ATI', 'GL_REG_15_ATI', 'GL_REG_16_ATI',
'GL_REG_17_ATI', 'GL_REG_18_ATI', 'GL_REG_19_ATI', 'GL_REG_20_ATI',
'GL_REG_21_ATI', 'GL_REG_22_ATI', 'GL_REG_23_ATI', 'GL_REG_24_ATI',
'GL_REG_25_ATI', 'GL_REG_26_ATI', 'GL_REG_27_ATI', 'GL_REG_28_ATI',
'GL_REG_29_ATI', 'GL_REG_30_ATI', 'GL_REG_31_ATI', 'GL_CON_0_ATI',
'GL_CON_1_ATI', 'GL_CON_2_ATI', 'GL_CON_3_ATI', 'GL_CON_4_ATI',
'GL_CON_5_ATI', 'GL_CON_6_ATI', 'GL_CON_7_ATI', 'GL_CON_8_ATI',
'GL_CON_9_ATI', 'GL_CON_10_ATI', 'GL_CON_11_ATI', 'GL_CON_12_ATI',
'GL_CON_13_ATI', 'GL_CON_14_ATI', 'GL_CON_15_ATI', 'GL_CON_16_ATI',
'GL_CON_17_ATI', 'GL_CON_18_ATI', 'GL_CON_19_ATI', 'GL_CON_20_ATI',
'GL_CON_21_ATI', 'GL_CON_22_ATI', 'GL_CON_23_ATI', 'GL_CON_24_ATI',
'GL_CON_25_ATI', 'GL_CON_26_ATI', 'GL_CON_27_ATI', 'GL_CON_28_ATI',
'GL_CON_29_ATI', 'GL_CON_30_ATI', 'GL_CON_31_ATI', 'GL_MOV_ATI', 'GL_ADD_ATI',
'GL_MUL_ATI', 'GL_SUB_ATI', 'GL_DOT3_ATI', 'GL_DOT4_ATI', 'GL_MAD_ATI',
'GL_LERP_ATI', 'GL_CND_ATI', 'GL_CND0_ATI', 'GL_DOT2_ADD_ATI',
'GL_SECONDARY_INTERPOLATOR_ATI', 'GL_NUM_FRAGMENT_REGISTERS_ATI',
'GL_NUM_FRAGMENT_CONSTANTS_ATI', 'GL_NUM_PASSES_ATI',
'GL_NUM_INSTRUCTIONS_PER_PASS_ATI', 'GL_NUM_INSTRUCTIONS_TOTAL_ATI',
'GL_NUM_INPUT_INTERPOLATOR_COMPONENTS_ATI', 'GL_NUM_LOOPBACK_COMPONENTS_ATI',
'GL_COLOR_ALPHA_PAIRING_ATI', 'GL_SWIZZLE_STR_ATI', 'GL_SWIZZLE_STQ_ATI',
'GL_SWIZZLE_STR_DR_ATI', 'GL_SWIZZLE_STQ_DQ_ATI', 'GL_SWIZZLE_STRQ_ATI',
'GL_SWIZZLE_STRQ_DQ_ATI', 'GL_RED_BIT_ATI', 'GL_GREEN_BIT_ATI',
'GL_BLUE_BIT_ATI', 'GL_2X_BIT_ATI', 'GL_4X_BIT_ATI', 'GL_8X_BIT_ATI',
'GL_HALF_BIT_ATI', 'GL_QUARTER_BIT_ATI', 'GL_EIGHTH_BIT_ATI',
'GL_SATURATE_BIT_ATI', 'GL_COMP_BIT_ATI', 'GL_NEGATE_BIT_ATI',
'GL_BIAS_BIT_ATI', 'GL_PN_TRIANGLES_ATI',
'GL_MAX_PN_TRIANGLES_TESSELATION_LEVEL_ATI', 'GL_PN_TRIANGLES_POINT_MODE_ATI',
'GL_PN_TRIANGLES_NORMAL_MODE_ATI', 'GL_PN_TRIANGLES_TESSELATION_LEVEL_ATI',
'GL_PN_TRIANGLES_POINT_MODE_LINEAR_ATI',
'GL_PN_TRIANGLES_POINT_MODE_CUBIC_ATI',
'GL_PN_TRIANGLES_NORMAL_MODE_LINEAR_ATI',
'GL_PN_TRIANGLES_NORMAL_MODE_QUADRATIC_ATI', 'GL_STATIC_ATI',
'GL_DYNAMIC_ATI', 'GL_PRESERVE_ATI', 'GL_DISCARD_ATI',
'GL_OBJECT_BUFFER_SIZE_ATI', 'GL_OBJECT_BUFFER_USAGE_ATI',
'GL_ARRAY_OBJECT_BUFFER_ATI', 'GL_ARRAY_OBJECT_OFFSET_ATI',
'GL_VERTEX_SHADER_EXT', 'GL_VERTEX_SHADER_BINDING_EXT', 'GL_OP_INDEX_EXT',
'GL_OP_NEGATE_EXT', 'GL_OP_DOT3_EXT', 'GL_OP_DOT4_EXT', 'GL_OP_MUL_EXT',
'GL_OP_ADD_EXT', 'GL_OP_MADD_EXT', 'GL_OP_FRAC_EXT', 'GL_OP_MAX_EXT',
'GL_OP_MIN_EXT', 'GL_OP_SET_GE_EXT', 'GL_OP_SET_LT_EXT', 'GL_OP_CLAMP_EXT',
'GL_OP_FLOOR_EXT', 'GL_OP_ROUND_EXT', 'GL_OP_EXP_BASE_2_EXT',
'GL_OP_LOG_BASE_2_EXT', 'GL_OP_POWER_EXT', 'GL_OP_RECIP_EXT',
'GL_OP_RECIP_SQRT_EXT', 'GL_OP_SUB_EXT', 'GL_OP_CROSS_PRODUCT_EXT',
'GL_OP_MULTIPLY_MATRIX_EXT', 'GL_OP_MOV_EXT', 'GL_OUTPUT_VERTEX_EXT',
'GL_OUTPUT_COLOR0_EXT', 'GL_OUTPUT_COLOR1_EXT',
'GL_OUTPUT_TEXTURE_COORD0_EXT', 'GL_OUTPUT_TEXTURE_COORD1_EXT',
'GL_OUTPUT_TEXTURE_COORD2_EXT', 'GL_OUTPUT_TEXTURE_COORD3_EXT',
'GL_OUTPUT_TEXTURE_COORD4_EXT', 'GL_OUTPUT_TEXTURE_COORD5_EXT',
'GL_OUTPUT_TEXTURE_COORD6_EXT', 'GL_OUTPUT_TEXTURE_COORD7_EXT',
'GL_OUTPUT_TEXTURE_COORD8_EXT', 'GL_OUTPUT_TEXTURE_COORD9_EXT',
'GL_OUTPUT_TEXTURE_COORD10_EXT', 'GL_OUTPUT_TEXTURE_COORD11_EXT',
'GL_OUTPUT_TEXTURE_COORD12_EXT', 'GL_OUTPUT_TEXTURE_COORD13_EXT',
'GL_OUTPUT_TEXTURE_COORD14_EXT', 'GL_OUTPUT_TEXTURE_COORD15_EXT',
'GL_OUTPUT_TEXTURE_COORD16_EXT', 'GL_OUTPUT_TEXTURE_COORD17_EXT',
'GL_OUTPUT_TEXTURE_COORD18_EXT', 'GL_OUTPUT_TEXTURE_COORD19_EXT',
'GL_OUTPUT_TEXTURE_COORD20_EXT', 'GL_OUTPUT_TEXTURE_COORD21_EXT',
'GL_OUTPUT_TEXTURE_COORD22_EXT', 'GL_OUTPUT_TEXTURE_COORD23_EXT',
'GL_OUTPUT_TEXTURE_COORD24_EXT', 'GL_OUTPUT_TEXTURE_COORD25_EXT',
'GL_OUTPUT_TEXTURE_COORD26_EXT', 'GL_OUTPUT_TEXTURE_COORD27_EXT',
'GL_OUTPUT_TEXTURE_COORD28_EXT', 'GL_OUTPUT_TEXTURE_COORD29_EXT',
'GL_OUTPUT_TEXTURE_COORD30_EXT', 'GL_OUTPUT_TEXTURE_COORD31_EXT',
'GL_OUTPUT_FOG_EXT', 'GL_SCALAR_EXT', 'GL_VECTOR_EXT', 'GL_MATRIX_EXT',
'GL_VARIANT_EXT', 'GL_INVARIANT_EXT', 'GL_LOCAL_CONSTANT_EXT', 'GL_LOCAL_EXT',
'GL_MAX_VERTEX_SHADER_INSTRUCTIONS_EXT', 'GL_MAX_VERTEX_SHADER_VARIANTS_EXT',
'GL_MAX_VERTEX_SHADER_INVARIANTS_EXT',
'GL_MAX_VERTEX_SHADER_LOCAL_CONSTANTS_EXT', 'GL_MAX_VERTEX_SHADER_LOCALS_EXT',
'GL_MAX_OPTIMIZED_VERTEX_SHADER_INSTRUCTIONS_EXT',
'GL_MAX_OPTIMIZED_VERTEX_SHADER_VARIANTS_EXT',
'GL_MAX_OPTIMIZED_VERTEX_SHADER_LOCAL_CONSTANTS_EXT',
'GL_MAX_OPTIMIZED_VERTEX_SHADER_INVARIANTS_EXT',
'GL_MAX_OPTIMIZED_VERTEX_SHADER_LOCALS_EXT',
'GL_VERTEX_SHADER_INSTRUCTIONS_EXT', 'GL_VERTEX_SHADER_VARIANTS_EXT',
'GL_VERTEX_SHADER_INVARIANTS_EXT', 'GL_VERTEX_SHADER_LOCAL_CONSTANTS_EXT',
'GL_VERTEX_SHADER_LOCALS_EXT', 'GL_VERTEX_SHADER_OPTIMIZED_EXT', 'GL_X_EXT',
'GL_Y_EXT', 'GL_Z_EXT', 'GL_W_EXT', 'GL_NEGATIVE_X_EXT', 'GL_NEGATIVE_Y_EXT',
'GL_NEGATIVE_Z_EXT', 'GL_NEGATIVE_W_EXT', 'GL_ZERO_EXT', 'GL_ONE_EXT',
'GL_NEGATIVE_ONE_EXT', 'GL_NORMALIZED_RANGE_EXT', 'GL_FULL_RANGE_EXT',
'GL_CURRENT_VERTEX_EXT', 'GL_MVP_MATRIX_EXT', 'GL_VARIANT_VALUE_EXT',
'GL_VARIANT_DATATYPE_EXT', 'GL_VARIANT_ARRAY_STRIDE_EXT',
'GL_VARIANT_ARRAY_TYPE_EXT', 'GL_VARIANT_ARRAY_EXT',
'GL_VARIANT_ARRAY_POINTER_EXT', 'GL_INVARIANT_VALUE_EXT',
'GL_INVARIANT_DATATYPE_EXT', 'GL_LOCAL_CONSTANT_VALUE_EXT',
'GL_LOCAL_CONSTANT_DATATYPE_EXT', 'GL_MAX_VERTEX_STREAMS_ATI',
'GL_VERTEX_STREAM0_ATI', 'GL_VERTEX_STREAM1_ATI', 'GL_VERTEX_STREAM2_ATI',
'GL_VERTEX_STREAM3_ATI', 'GL_VERTEX_STREAM4_ATI', 'GL_VERTEX_STREAM5_ATI',
'GL_VERTEX_STREAM6_ATI', 'GL_VERTEX_STREAM7_ATI', 'GL_VERTEX_SOURCE_ATI',
'GL_ELEMENT_ARRAY_ATI', 'GL_ELEMENT_ARRAY_TYPE_ATI',
'GL_ELEMENT_ARRAY_POINTER_ATI', 'GL_QUAD_MESH_SUN', 'GL_TRIANGLE_MESH_SUN',
'GL_SLICE_ACCUM_SUN', 'GL_MULTISAMPLE_FILTER_HINT_NV', 'GL_DEPTH_CLAMP_NV',
'GL_PIXEL_COUNTER_BITS_NV', 'GL_CURRENT_OCCLUSION_QUERY_ID_NV',
'GL_PIXEL_COUNT_NV', 'GL_PIXEL_COUNT_AVAILABLE_NV', 'GL_POINT_SPRITE_NV',
'GL_COORD_REPLACE_NV', 'GL_POINT_SPRITE_R_MODE_NV',
'GL_OFFSET_PROJECTIVE_TEXTURE_2D_NV',
'GL_OFFSET_PROJECTIVE_TEXTURE_2D_SCALE_NV',
'GL_OFFSET_PROJECTIVE_TEXTURE_RECTANGLE_NV',
'GL_OFFSET_PROJECTIVE_TEXTURE_RECTANGLE_SCALE_NV',
'GL_OFFSET_HILO_TEXTURE_2D_NV', 'GL_OFFSET_HILO_TEXTURE_RECTANGLE_NV',
'GL_OFFSET_HILO_PROJECTIVE_TEXTURE_2D_NV',
'GL_OFFSET_HILO_PROJECTIVE_TEXTURE_RECTANGLE_NV',
'GL_DEPENDENT_HILO_TEXTURE_2D_NV', 'GL_DEPENDENT_RGB_TEXTURE_3D_NV',
'GL_DEPENDENT_RGB_TEXTURE_CUBE_MAP_NV', 'GL_DOT_PRODUCT_PASS_THROUGH_NV',
'GL_DOT_PRODUCT_TEXTURE_1D_NV', 'GL_DOT_PRODUCT_AFFINE_DEPTH_REPLACE_NV',
'GL_HILO8_NV', 'GL_SIGNED_HILO8_NV', 'GL_FORCE_BLUE_TO_ONE_NV',
'GL_STENCIL_TEST_TWO_SIDE_EXT', 'GL_ACTIVE_STENCIL_FACE_EXT',
'GL_TEXT_FRAGMENT_SHADER_ATI', 'GL_UNPACK_CLIENT_STORAGE_APPLE',
'GL_ELEMENT_ARRAY_APPLE', 'GL_ELEMENT_ARRAY_TYPE_APPLE',
'GL_ELEMENT_ARRAY_POINTER_APPLE', 'GL_DRAW_PIXELS_APPLE', 'GL_FENCE_APPLE',
'GL_VERTEX_ARRAY_BINDING_APPLE', 'GL_VERTEX_ARRAY_RANGE_APPLE',
'GL_VERTEX_ARRAY_RANGE_LENGTH_APPLE', 'GL_VERTEX_ARRAY_STORAGE_HINT_APPLE',
'GL_VERTEX_ARRAY_RANGE_POINTER_APPLE', 'GL_STORAGE_CACHED_APPLE',
'GL_STORAGE_SHARED_APPLE', 'GL_YCBCR_422_APPLE',
'GL_UNSIGNED_SHORT_8_8_APPLE', 'GL_UNSIGNED_SHORT_8_8_REV_APPLE',
'GL_RGB_S3TC', 'GL_RGB4_S3TC', 'GL_RGBA_S3TC', 'GL_RGBA4_S3TC',
'GL_MAX_DRAW_BUFFERS_ATI', 'GL_DRAW_BUFFER0_ATI', 'GL_DRAW_BUFFER1_ATI',
'GL_DRAW_BUFFER2_ATI', 'GL_DRAW_BUFFER3_ATI', 'GL_DRAW_BUFFER4_ATI',
'GL_DRAW_BUFFER5_ATI', 'GL_DRAW_BUFFER6_ATI', 'GL_DRAW_BUFFER7_ATI',
'GL_DRAW_BUFFER8_ATI', 'GL_DRAW_BUFFER9_ATI', 'GL_DRAW_BUFFER10_ATI',
'GL_DRAW_BUFFER11_ATI', 'GL_DRAW_BUFFER12_ATI', 'GL_DRAW_BUFFER13_ATI',
'GL_DRAW_BUFFER14_ATI', 'GL_DRAW_BUFFER15_ATI', 'GL_TYPE_RGBA_FLOAT_ATI',
'GL_COLOR_CLEAR_UNCLAMPED_VALUE_ATI', 'GL_MODULATE_ADD_ATI',
'GL_MODULATE_SIGNED_ADD_ATI', 'GL_MODULATE_SUBTRACT_ATI',
'GL_RGBA_FLOAT32_ATI', 'GL_RGB_FLOAT32_ATI', 'GL_ALPHA_FLOAT32_ATI',
'GL_INTENSITY_FLOAT32_ATI', 'GL_LUMINANCE_FLOAT32_ATI',
'GL_LUMINANCE_ALPHA_FLOAT32_ATI', 'GL_RGBA_FLOAT16_ATI', 'GL_RGB_FLOAT16_ATI',
'GL_ALPHA_FLOAT16_ATI', 'GL_INTENSITY_FLOAT16_ATI',
'GL_LUMINANCE_FLOAT16_ATI', 'GL_LUMINANCE_ALPHA_FLOAT16_ATI', 'GL_FLOAT_R_NV',
'GL_FLOAT_RG_NV', 'GL_FLOAT_RGB_NV', 'GL_FLOAT_RGBA_NV', 'GL_FLOAT_R16_NV',
'GL_FLOAT_R32_NV', 'GL_FLOAT_RG16_NV', 'GL_FLOAT_RG32_NV',
'GL_FLOAT_RGB16_NV', 'GL_FLOAT_RGB32_NV', 'GL_FLOAT_RGBA16_NV',
'GL_FLOAT_RGBA32_NV', 'GL_TEXTURE_FLOAT_COMPONENTS_NV',
'GL_FLOAT_CLEAR_COLOR_VALUE_NV', 'GL_FLOAT_RGBA_MODE_NV',
'GL_MAX_FRAGMENT_PROGRAM_LOCAL_PARAMETERS_NV', 'GL_FRAGMENT_PROGRAM_NV',
'GL_MAX_TEXTURE_COORDS_NV', 'GL_MAX_TEXTURE_IMAGE_UNITS_NV',
'GL_FRAGMENT_PROGRAM_BINDING_NV', 'GL_PROGRAM_ERROR_STRING_NV',
'GL_HALF_FLOAT_NV', 'GL_WRITE_PIXEL_DATA_RANGE_NV',
'GL_READ_PIXEL_DATA_RANGE_NV', 'GL_WRITE_PIXEL_DATA_RANGE_LENGTH_NV',
'GL_READ_PIXEL_DATA_RANGE_LENGTH_NV', 'GL_WRITE_PIXEL_DATA_RANGE_POINTER_NV',
'GL_READ_PIXEL_DATA_RANGE_POINTER_NV', 'GL_PRIMITIVE_RESTART_NV',
'GL_PRIMITIVE_RESTART_INDEX_NV', 'GL_TEXTURE_UNSIGNED_REMAP_MODE_NV',
'GL_STENCIL_BACK_FUNC_ATI', 'GL_STENCIL_BACK_FAIL_ATI',
'GL_STENCIL_BACK_PASS_DEPTH_FAIL_ATI', 'GL_STENCIL_BACK_PASS_DEPTH_PASS_ATI',
'GL_IMPLEMENTATION_COLOR_READ_TYPE_OES',
'GL_IMPLEMENTATION_COLOR_READ_FORMAT_OES', 'GL_DEPTH_BOUNDS_TEST_EXT',
'GL_DEPTH_BOUNDS_EXT', 'GL_MIRROR_CLAMP_EXT', 'GL_MIRROR_CLAMP_TO_EDGE_EXT',
'GL_MIRROR_CLAMP_TO_BORDER_EXT', 'GL_BLEND_EQUATION_RGB_EXT',
'GL_BLEND_EQUATION_ALPHA_EXT', 'GL_PACK_INVERT_MESA',
'GL_UNSIGNED_SHORT_8_8_MESA', 'GL_UNSIGNED_SHORT_8_8_REV_MESA',
'GL_YCBCR_MESA', 'GL_PIXEL_PACK_BUFFER_EXT', 'GL_PIXEL_UNPACK_BUFFER_EXT',
'GL_PIXEL_PACK_BUFFER_BINDING_EXT', 'GL_PIXEL_UNPACK_BUFFER_BINDING_EXT',
'GL_MAX_PROGRAM_EXEC_INSTRUCTIONS_NV', 'GL_MAX_PROGRAM_CALL_DEPTH_NV',
'GL_MAX_PROGRAM_IF_DEPTH_NV', 'GL_MAX_PROGRAM_LOOP_DEPTH_NV',
'GL_MAX_PROGRAM_LOOP_COUNT_NV', 'GL_INVALID_FRAMEBUFFER_OPERATION_EXT',
'GL_MAX_RENDERBUFFER_SIZE_EXT', 'GL_FRAMEBUFFER_BINDING_EXT',
'GL_RENDERBUFFER_BINDING_EXT', 'GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE_EXT',
'GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME_EXT',
'GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL_EXT',
'GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE_EXT',
'GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_3D_ZOFFSET_EXT',
'GL_FRAMEBUFFER_COMPLETE_EXT', 'GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_EXT',
'GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_EXT',
'GL_FRAMEBUFFER_INCOMPLETE_DUPLICATE_ATTACHMENT_EXT',
'GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_EXT',
'GL_FRAMEBUFFER_INCOMPLETE_FORMATS_EXT',
'GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER_EXT',
'GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER_EXT', 'GL_FRAMEBUFFER_UNSUPPORTED_EXT',
'GL_MAX_COLOR_ATTACHMENTS_EXT', 'GL_COLOR_ATTACHMENT0_EXT',
'GL_COLOR_ATTACHMENT1_EXT', 'GL_COLOR_ATTACHMENT2_EXT',
'GL_COLOR_ATTACHMENT3_EXT', 'GL_COLOR_ATTACHMENT4_EXT',
'GL_COLOR_ATTACHMENT5_EXT', 'GL_COLOR_ATTACHMENT6_EXT',
'GL_COLOR_ATTACHMENT7_EXT', 'GL_COLOR_ATTACHMENT8_EXT',
'GL_COLOR_ATTACHMENT9_EXT', 'GL_COLOR_ATTACHMENT10_EXT',
'GL_COLOR_ATTACHMENT11_EXT', 'GL_COLOR_ATTACHMENT12_EXT',
'GL_COLOR_ATTACHMENT13_EXT', 'GL_COLOR_ATTACHMENT14_EXT',
'GL_COLOR_ATTACHMENT15_EXT', 'GL_DEPTH_ATTACHMENT_EXT',
'GL_STENCIL_ATTACHMENT_EXT', 'GL_FRAMEBUFFER_EXT', 'GL_RENDERBUFFER_EXT',
'GL_RENDERBUFFER_WIDTH_EXT', 'GL_RENDERBUFFER_HEIGHT_EXT',
'GL_RENDERBUFFER_INTERNAL_FORMAT_EXT', 'GL_STENCIL_INDEX_EXT',
'GL_STENCIL_INDEX1_EXT', 'GL_STENCIL_INDEX4_EXT', 'GL_STENCIL_INDEX8_EXT',
'GL_STENCIL_INDEX16_EXT', 'GL_RENDERBUFFER_RED_SIZE_EXT',
'GL_RENDERBUFFER_GREEN_SIZE_EXT', 'GL_RENDERBUFFER_BLUE_SIZE_EXT',
'GL_RENDERBUFFER_ALPHA_SIZE_EXT', 'GL_RENDERBUFFER_DEPTH_SIZE_EXT',
'GL_RENDERBUFFER_STENCIL_SIZE_EXT', 'GL_CG_VERTEX_SHADER_EXT',
'GL_CG_FRAGMENT_SHADER_EXT', 'GL_TIME_ELAPSED_EXT', 'GL_TEXTURE_BUFFER_EXT',
'GL_MAX_TEXTURE_BUFFER_SIZE_EXT', 'GL_TEXTURE_BINDING_BUFFER_EXT',
'GL_TEXTURE_BUFFER_DATA_STORE_BINDING_EXT', 'GL_TEXTURE_BUFFER_FORMAT_EXT',
'GL_SAMPLER_1D_ARRAY_EXT', 'GL_SAMPLER_2D_ARRAY_EXT', 'GL_SAMPLER_BUFFER_EXT',
'GL_SAMPLER_1D_ARRAY_SHADOW_EXT', 'GL_SAMPLER_2D_ARRAY_SHADOW_EXT',
'GL_SAMPLER_CUBE_SHADOW_EXT', 'GL_UNSIGNED_INT_VEC2_EXT',
'GL_UNSIGNED_INT_VEC3_EXT', 'GL_UNSIGNED_INT_VEC4_EXT',
'GL_INT_SAMPLER_1D_EXT', 'GL_INT_SAMPLER_2D_EXT', 'GL_INT_SAMPLER_3D_EXT',
'GL_INT_SAMPLER_CUBE_EXT', 'GL_INT_SAMPLER_2D_RECT_EXT',
'GL_INT_SAMPLER_1D_ARRAY_EXT', 'GL_INT_SAMPLER_2D_ARRAY_EXT',
'GL_INT_SAMPLER_BUFFER_EXT', 'GL_UNSIGNED_INT_SAMPLER_1D_EXT',
'GL_UNSIGNED_INT_SAMPLER_2D_EXT', 'GL_UNSIGNED_INT_SAMPLER_3D_EXT',
'GL_UNSIGNED_INT_SAMPLER_CUBE_EXT', 'GL_UNSIGNED_INT_SAMPLER_2D_RECT_EXT',
'GL_UNSIGNED_INT_SAMPLER_1D_ARRAY_EXT',
'GL_UNSIGNED_INT_SAMPLER_2D_ARRAY_EXT', 'GL_UNSIGNED_INT_SAMPLER_BUFFER_EXT',
'GL_VERTEX_ATTRIB_ARRAY_INTEGER_EXT', 'GL_GEOMETRY_SHADER_EXT',
'GL_MAX_GEOMETRY_VARYING_COMPONENTS_EXT',
'GL_MAX_VERTEX_VARYING_COMPONENTS_EXT', 'GL_MAX_VARYING_COMPONENTS_EXT',
'GL_MAX_GEOMETRY_UNIFORM_COMPONENTS_EXT',
'GL_MAX_GEOMETRY_OUTPUT_VERTICES_EXT',
'GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS_EXT', 'GL_GEOMETRY_VERTICES_OUT_EXT',
'GL_GEOMETRY_INPUT_TYPE_EXT', 'GL_GEOMETRY_OUTPUT_TYPE_EXT',
'GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS_EXT', 'GL_LINES_ADJACENCY_EXT',
'GL_LINE_STRIP_ADJACENCY_EXT', 'GL_TRIANGLES_ADJACENCY_EXT',
'GL_TRIANGLE_STRIP_ADJACENCY_EXT', 'GL_FRAMEBUFFER_ATTACHMENT_LAYERED_EXT',
'GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS_EXT',
'GL_FRAMEBUFFER_INCOMPLETE_LAYER_COUNT_EXT',
'GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER_EXT', 'GL_PROGRAM_POINT_SIZE_EXT',
'GL_MAX_VERTEX_BINDABLE_UNIFORMS_EXT',
'GL_MAX_FRAGMENT_BINDABLE_UNIFORMS_EXT',
'GL_MAX_GEOMETRY_BINDABLE_UNIFORMS_EXT', 'GL_MAX_BINDABLE_UNIFORM_SIZE_EXT',
'GL_UNIFORM_BUFFER_EXT', 'GL_UNIFORM_BUFFER_BINDING_EXT',
'GL_FRAMEBUFFER_SRGB_EXT', 'GL_FRAMEBUFFER_SRGB_CAPABLE_EXT',
'GL_RGB9_E5_EXT', 'GL_UNSIGNED_INT_5_9_9_9_REV_EXT',
'GL_TEXTURE_SHARED_SIZE_EXT', 'GL_R11F_G11F_B10F_EXT',
'GL_UNSIGNED_INT_10F_11F_11F_REV_EXT', 'GL_RGBA_SIGNED_COMPONENTS_EXT',
'GL_TEXTURE_1D_ARRAY_EXT', 'GL_PROXY_TEXTURE_1D_ARRAY_EXT',
'GL_TEXTURE_2D_ARRAY_EXT', 'GL_PROXY_TEXTURE_2D_ARRAY_EXT',
'GL_TEXTURE_BINDING_1D_ARRAY_EXT', 'GL_TEXTURE_BINDING_2D_ARRAY_EXT',
'GL_MAX_ARRAY_TEXTURE_LAYERS_EXT', 'GL_COMPARE_REF_DEPTH_TO_TEXTURE_EXT',
'GL_RGBA32UI_EXT', 'GL_RGB32UI_EXT', 'GL_ALPHA32UI_EXT',
'GL_INTENSITY32UI_EXT', 'GL_LUMINANCE32UI_EXT', 'GL_LUMINANCE_ALPHA32UI_EXT',
'GL_RGBA16UI_EXT', 'GL_RGB16UI_EXT', 'GL_ALPHA16UI_EXT',
'GL_INTENSITY16UI_EXT', 'GL_LUMINANCE16UI_EXT', 'GL_LUMINANCE_ALPHA16UI_EXT',
'GL_RGBA8UI_EXT', 'GL_RGB8UI_EXT', 'GL_ALPHA8UI_EXT', 'GL_INTENSITY8UI_EXT',
'GL_LUMINANCE8UI_EXT', 'GL_LUMINANCE_ALPHA8UI_EXT', 'GL_RGBA32I_EXT',
'GL_RGB32I_EXT', 'GL_ALPHA32I_EXT', 'GL_INTENSITY32I_EXT',
'GL_LUMINANCE32I_EXT', 'GL_LUMINANCE_ALPHA32I_EXT', 'GL_RGBA16I_EXT',
'GL_RGB16I_EXT', 'GL_ALPHA16I_EXT', 'GL_INTENSITY16I_EXT',
'GL_LUMINANCE16I_EXT', 'GL_LUMINANCE_ALPHA16I_EXT', 'GL_RGBA8I_EXT',
'GL_RGB8I_EXT', 'GL_ALPHA8I_EXT', 'GL_INTENSITY8I_EXT', 'GL_LUMINANCE8I_EXT',
'GL_LUMINANCE_ALPHA8I_EXT', 'GL_RED_INTEGER_EXT', 'GL_GREEN_INTEGER_EXT',
'GL_BLUE_INTEGER_EXT', 'GL_ALPHA_INTEGER_EXT', 'GL_RGB_INTEGER_EXT',
'GL_RGBA_INTEGER_EXT', 'GL_BGR_INTEGER_EXT', 'GL_BGRA_INTEGER_EXT',
'GL_LUMINANCE_INTEGER_EXT', 'GL_LUMINANCE_ALPHA_INTEGER_EXT',
'GL_RGBA_INTEGER_MODE_EXT', 'GL_DEPTH_COMPONENT32F_NV',
'GL_DEPTH32F_STENCIL8_NV', 'GL_FLOAT_32_UNSIGNED_INT_24_8_REV_NV',
'GL_DEPTH_BUFFER_FLOAT_MODE_NV', 'GL_COMPRESSED_LUMINANCE_LATC1_EXT',
'GL_COMPRESSED_SIGNED_LUMINANCE_LATC1_EXT',
'GL_COMPRESSED_LUMINANCE_ALPHA_LATC2_EXT',
'GL_COMPRESSED_SIGNED_LUMINANCE_ALPHA_LATC2_EXT', 'GL_BACK_PRIMARY_COLOR_NV',
'GL_BACK_SECONDARY_COLOR_NV', 'GL_TEXTURE_COORD_NV', 'GL_CLIP_DISTANCE_NV',
'GL_VERTEX_ID_NV', 'GL_PRIMITIVE_ID_NV', 'GL_GENERIC_ATTRIB_NV',
'GL_TRANSFORM_FEEDBACK_ATTRIBS_NV', 'GL_TRANSFORM_FEEDBACK_BUFFER_MODE_NV',
'GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS_NV', 'GL_ACTIVE_VARYINGS_NV',
'GL_ACTIVE_VARYING_MAX_LENGTH_NV', 'GL_TRANSFORM_FEEDBACK_VARYINGS_NV',
'GL_TRANSFORM_FEEDBACK_BUFFER_START_NV',
'GL_TRANSFORM_FEEDBACK_BUFFER_SIZE_NV', 'GL_TRANSFORM_FEEDBACK_RECORD_NV',
'GL_PRIMITIVES_GENERATED_NV', 'GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN_NV',
'GL_RASTERIZER_DISCARD_NV',
'GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_ATTRIBS_NV',
'GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS_NV', 'GL_INTERLEAVED_ATTRIBS_NV',
'GL_SEPARATE_ATTRIBS_NV', 'GL_TRANSFORM_FEEDBACK_BUFFER_NV',
'GL_TRANSFORM_FEEDBACK_BUFFER_BINDING_NV', 'GL_GEOMETRY_PROGRAM_NV',
'GL_MAX_PROGRAM_OUTPUT_VERTICES_NV',
'GL_MAX_PROGRAM_TOTAL_OUTPUT_COMPONENTS_NV', 'GL_MIN_PROGRAM_TEXEL_OFFSET_NV',
'GL_MAX_PROGRAM_TEXEL_OFFSET_NV', 'GL_PROGRAM_ATTRIB_COMPONENTS_NV',
'GL_PROGRAM_RESULT_COMPONENTS_NV', 'GL_MAX_PROGRAM_ATTRIB_COMPONENTS_NV',
'GL_MAX_PROGRAM_RESULT_COMPONENTS_NV', 'GL_MAX_PROGRAM_GENERIC_ATTRIBS_NV',
'GL_MAX_PROGRAM_GENERIC_RESULTS_NV', 'GL_RENDERBUFFER_COVERAGE_SAMPLES_NV',
'GL_RENDERBUFFER_COLOR_SAMPLES_NV', 'GL_MAX_RENDERBUFFER_COVERAGE_SAMPLES_NV',
'GL_MAX_RENDERBUFFER_COLOR_SAMPLES_NV',
'GL_MAX_MULTISAMPLE_COVERAGE_MODES_NV', 'GL_MULTISAMPLE_COVERAGE_MODES_NV',
'GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE_EXT', 'GL_MAX_SAMPLES_EXT',
'GL_RENDERBUFFER_SAMPLES_EXT', 'GL_READ_FRAMEBUFFER_EXT',
'GL_DRAW_FRAMEBUFFER_EXT', 'GL_DRAW_FRAMEBUFFER_BINDING_EXT',
'GL_READ_FRAMEBUFFER_BINDING_EXT', 'GL_COMPRESSED_RED_RGTC1_EXT',
'GL_COMPRESSED_SIGNED_RED_RGTC1_EXT', 'GL_COMPRESSED_RED_GREEN_RGTC2_EXT',
'GL_COMPRESSED_SIGNED_RED_GREEN_RGTC2_EXT', 'GLchar', 'GLintptr',
'GLsizeiptr', 'GLintptrARB', 'GLsizeiptrARB', 'GLcharARB', 'GLhandleARB',
'GLhalfARB', 'GLhalfNV', 'GLint64EXT', 'GLuint64EXT', 'GL_VERSION_1_2',
'glBlendColor', 'glBlendEquation', 'glDrawRangeElements', 'glColorTable',
'glColorTableParameterfv', 'glColorTableParameteriv', 'glCopyColorTable',
'glGetColorTable', 'glGetColorTableParameterfv', 'glGetColorTableParameteriv',
'glColorSubTable', 'glCopyColorSubTable', 'glConvolutionFilter1D',
'glConvolutionFilter2D', 'glConvolutionParameterf',
'glConvolutionParameterfv', 'glConvolutionParameteri',
'glConvolutionParameteriv', 'glCopyConvolutionFilter1D',
'glCopyConvolutionFilter2D', 'glGetConvolutionFilter',
'glGetConvolutionParameterfv', 'glGetConvolutionParameteriv',
'glGetSeparableFilter', 'glSeparableFilter2D', 'glGetHistogram',
'glGetHistogramParameterfv', 'glGetHistogramParameteriv', 'glGetMinmax',
'glGetMinmaxParameterfv', 'glGetMinmaxParameteriv', 'glHistogram', 'glMinmax',
'glResetHistogram', 'glResetMinmax', 'glTexImage3D', 'glTexSubImage3D',
'glCopyTexSubImage3D', 'PFNGLBLENDCOLORPROC', 'PFNGLBLENDEQUATIONPROC',
'PFNGLDRAWRANGEELEMENTSPROC', 'PFNGLCOLORTABLEPROC',
'PFNGLCOLORTABLEPARAMETERFVPROC', 'PFNGLCOLORTABLEPARAMETERIVPROC',
'PFNGLCOPYCOLORTABLEPROC', 'PFNGLGETCOLORTABLEPROC',
'PFNGLGETCOLORTABLEPARAMETERFVPROC', 'PFNGLGETCOLORTABLEPARAMETERIVPROC',
'PFNGLCOLORSUBTABLEPROC', 'PFNGLCOPYCOLORSUBTABLEPROC',
'PFNGLCONVOLUTIONFILTER1DPROC', 'PFNGLCONVOLUTIONFILTER2DPROC',
'PFNGLCONVOLUTIONPARAMETERFPROC', 'PFNGLCONVOLUTIONPARAMETERFVPROC',
'PFNGLCONVOLUTIONPARAMETERIPROC', 'PFNGLCONVOLUTIONPARAMETERIVPROC',
'PFNGLCOPYCONVOLUTIONFILTER1DPROC', 'PFNGLCOPYCONVOLUTIONFILTER2DPROC',
'PFNGLGETCONVOLUTIONFILTERPROC', 'PFNGLGETCONVOLUTIONPARAMETERFVPROC',
'PFNGLGETCONVOLUTIONPARAMETERIVPROC', 'PFNGLGETSEPARABLEFILTERPROC',
'PFNGLSEPARABLEFILTER2DPROC', 'PFNGLGETHISTOGRAMPROC',
'PFNGLGETHISTOGRAMPARAMETERFVPROC', 'PFNGLGETHISTOGRAMPARAMETERIVPROC',
'PFNGLGETMINMAXPROC', 'PFNGLGETMINMAXPARAMETERFVPROC',
'PFNGLGETMINMAXPARAMETERIVPROC', 'PFNGLHISTOGRAMPROC', 'PFNGLMINMAXPROC',
'PFNGLRESETHISTOGRAMPROC', 'PFNGLRESETMINMAXPROC', 'PFNGLTEXIMAGE3DPROC',
'PFNGLTEXSUBIMAGE3DPROC', 'PFNGLCOPYTEXSUBIMAGE3DPROC', 'GL_VERSION_1_3',
'glActiveTexture', 'glClientActiveTexture', 'glMultiTexCoord1d',
'glMultiTexCoord1dv', 'glMultiTexCoord1f', 'glMultiTexCoord1fv',
'glMultiTexCoord1i', 'glMultiTexCoord1iv', 'glMultiTexCoord1s',
'glMultiTexCoord1sv', 'glMultiTexCoord2d', 'glMultiTexCoord2dv',
'glMultiTexCoord2f', 'glMultiTexCoord2fv', 'glMultiTexCoord2i',
'glMultiTexCoord2iv', 'glMultiTexCoord2s', 'glMultiTexCoord2sv',
'glMultiTexCoord3d', 'glMultiTexCoord3dv', 'glMultiTexCoord3f',
'glMultiTexCoord3fv', 'glMultiTexCoord3i', 'glMultiTexCoord3iv',
'glMultiTexCoord3s', 'glMultiTexCoord3sv', 'glMultiTexCoord4d',
'glMultiTexCoord4dv', 'glMultiTexCoord4f', 'glMultiTexCoord4fv',
'glMultiTexCoord4i', 'glMultiTexCoord4iv', 'glMultiTexCoord4s',
'glMultiTexCoord4sv', 'glLoadTransposeMatrixf', 'glLoadTransposeMatrixd',
'glMultTransposeMatrixf', 'glMultTransposeMatrixd', 'glSampleCoverage',
'glCompressedTexImage3D', 'glCompressedTexImage2D', 'glCompressedTexImage1D',
'glCompressedTexSubImage3D', 'glCompressedTexSubImage2D',
'glCompressedTexSubImage1D', 'glGetCompressedTexImage',
'PFNGLACTIVETEXTUREPROC', 'PFNGLCLIENTACTIVETEXTUREPROC',
'PFNGLMULTITEXCOORD1DPROC', 'PFNGLMULTITEXCOORD1DVPROC',
'PFNGLMULTITEXCOORD1FPROC', 'PFNGLMULTITEXCOORD1FVPROC',
'PFNGLMULTITEXCOORD1IPROC', 'PFNGLMULTITEXCOORD1IVPROC',
'PFNGLMULTITEXCOORD1SPROC', 'PFNGLMULTITEXCOORD1SVPROC',
'PFNGLMULTITEXCOORD2DPROC', 'PFNGLMULTITEXCOORD2DVPROC',
'PFNGLMULTITEXCOORD2FPROC', 'PFNGLMULTITEXCOORD2FVPROC',
'PFNGLMULTITEXCOORD2IPROC', 'PFNGLMULTITEXCOORD2IVPROC',
'PFNGLMULTITEXCOORD2SPROC', 'PFNGLMULTITEXCOORD2SVPROC',
'PFNGLMULTITEXCOORD3DPROC', 'PFNGLMULTITEXCOORD3DVPROC',
'PFNGLMULTITEXCOORD3FPROC', 'PFNGLMULTITEXCOORD3FVPROC',
'PFNGLMULTITEXCOORD3IPROC', 'PFNGLMULTITEXCOORD3IVPROC',
'PFNGLMULTITEXCOORD3SPROC', 'PFNGLMULTITEXCOORD3SVPROC',
'PFNGLMULTITEXCOORD4DPROC', 'PFNGLMULTITEXCOORD4DVPROC',
'PFNGLMULTITEXCOORD4FPROC', 'PFNGLMULTITEXCOORD4FVPROC',
'PFNGLMULTITEXCOORD4IPROC', 'PFNGLMULTITEXCOORD4IVPROC',
'PFNGLMULTITEXCOORD4SPROC', 'PFNGLMULTITEXCOORD4SVPROC',
'PFNGLLOADTRANSPOSEMATRIXFPROC', 'PFNGLLOADTRANSPOSEMATRIXDPROC',
'PFNGLMULTTRANSPOSEMATRIXFPROC', 'PFNGLMULTTRANSPOSEMATRIXDPROC',
'PFNGLSAMPLECOVERAGEPROC', 'PFNGLCOMPRESSEDTEXIMAGE3DPROC',
'PFNGLCOMPRESSEDTEXIMAGE2DPROC', 'PFNGLCOMPRESSEDTEXIMAGE1DPROC',
'PFNGLCOMPRESSEDTEXSUBIMAGE3DPROC', 'PFNGLCOMPRESSEDTEXSUBIMAGE2DPROC',
'PFNGLCOMPRESSEDTEXSUBIMAGE1DPROC', 'PFNGLGETCOMPRESSEDTEXIMAGEPROC',
'GL_VERSION_1_4', 'glBlendFuncSeparate', 'glFogCoordf', 'glFogCoordfv',
'glFogCoordd', 'glFogCoorddv', 'glFogCoordPointer', 'glMultiDrawArrays',
'glMultiDrawElements', 'glPointParameterf', 'glPointParameterfv',
'glPointParameteri', 'glPointParameteriv', 'glSecondaryColor3b',
'glSecondaryColor3bv', 'glSecondaryColor3d', 'glSecondaryColor3dv',
'glSecondaryColor3f', 'glSecondaryColor3fv', 'glSecondaryColor3i',
'glSecondaryColor3iv', 'glSecondaryColor3s', 'glSecondaryColor3sv',
'glSecondaryColor3ub', 'glSecondaryColor3ubv', 'glSecondaryColor3ui',
'glSecondaryColor3uiv', 'glSecondaryColor3us', 'glSecondaryColor3usv',
'glSecondaryColorPointer', 'glWindowPos2d', 'glWindowPos2dv', 'glWindowPos2f',
'glWindowPos2fv', 'glWindowPos2i', 'glWindowPos2iv', 'glWindowPos2s',
'glWindowPos2sv', 'glWindowPos3d', 'glWindowPos3dv', 'glWindowPos3f',
'glWindowPos3fv', 'glWindowPos3i', 'glWindowPos3iv', 'glWindowPos3s',
'glWindowPos3sv', 'PFNGLBLENDFUNCSEPARATEPROC', 'PFNGLFOGCOORDFPROC',
'PFNGLFOGCOORDFVPROC', 'PFNGLFOGCOORDDPROC', 'PFNGLFOGCOORDDVPROC',
'PFNGLFOGCOORDPOINTERPROC', 'PFNGLMULTIDRAWARRAYSPROC',
'PFNGLMULTIDRAWELEMENTSPROC', 'PFNGLPOINTPARAMETERFPROC',
'PFNGLPOINTPARAMETERFVPROC', 'PFNGLPOINTPARAMETERIPROC',
'PFNGLPOINTPARAMETERIVPROC', 'PFNGLSECONDARYCOLOR3BPROC',
'PFNGLSECONDARYCOLOR3BVPROC', 'PFNGLSECONDARYCOLOR3DPROC',
'PFNGLSECONDARYCOLOR3DVPROC', 'PFNGLSECONDARYCOLOR3FPROC',
'PFNGLSECONDARYCOLOR3FVPROC', 'PFNGLSECONDARYCOLOR3IPROC',
'PFNGLSECONDARYCOLOR3IVPROC', 'PFNGLSECONDARYCOLOR3SPROC',
'PFNGLSECONDARYCOLOR3SVPROC', 'PFNGLSECONDARYCOLOR3UBPROC',
'PFNGLSECONDARYCOLOR3UBVPROC', 'PFNGLSECONDARYCOLOR3UIPROC',
'PFNGLSECONDARYCOLOR3UIVPROC', 'PFNGLSECONDARYCOLOR3USPROC',
'PFNGLSECONDARYCOLOR3USVPROC', 'PFNGLSECONDARYCOLORPOINTERPROC',
'PFNGLWINDOWPOS2DPROC', 'PFNGLWINDOWPOS2DVPROC', 'PFNGLWINDOWPOS2FPROC',
'PFNGLWINDOWPOS2FVPROC', 'PFNGLWINDOWPOS2IPROC', 'PFNGLWINDOWPOS2IVPROC',
'PFNGLWINDOWPOS2SPROC', 'PFNGLWINDOWPOS2SVPROC', 'PFNGLWINDOWPOS3DPROC',
'PFNGLWINDOWPOS3DVPROC', 'PFNGLWINDOWPOS3FPROC', 'PFNGLWINDOWPOS3FVPROC',
'PFNGLWINDOWPOS3IPROC', 'PFNGLWINDOWPOS3IVPROC', 'PFNGLWINDOWPOS3SPROC',
'PFNGLWINDOWPOS3SVPROC', 'GL_VERSION_1_5', 'glGenQueries', 'glDeleteQueries',
'glIsQuery', 'glBeginQuery', 'glEndQuery', 'glGetQueryiv',
'glGetQueryObjectiv', 'glGetQueryObjectuiv', 'glBindBuffer',
'glDeleteBuffers', 'glGenBuffers', 'glIsBuffer', 'glBufferData',
'glBufferSubData', 'glGetBufferSubData', 'glMapBuffer', 'glUnmapBuffer',
'glGetBufferParameteriv', 'glGetBufferPointerv', 'PFNGLGENQUERIESPROC',
'PFNGLDELETEQUERIESPROC', 'PFNGLISQUERYPROC', 'PFNGLBEGINQUERYPROC',
'PFNGLENDQUERYPROC', 'PFNGLGETQUERYIVPROC', 'PFNGLGETQUERYOBJECTIVPROC',
'PFNGLGETQUERYOBJECTUIVPROC', 'PFNGLBINDBUFFERPROC', 'PFNGLDELETEBUFFERSPROC',
'PFNGLGENBUFFERSPROC', 'PFNGLISBUFFERPROC', 'PFNGLBUFFERDATAPROC',
'PFNGLBUFFERSUBDATAPROC', 'PFNGLGETBUFFERSUBDATAPROC', 'PFNGLMAPBUFFERPROC',
'PFNGLUNMAPBUFFERPROC', 'PFNGLGETBUFFERPARAMETERIVPROC',
'PFNGLGETBUFFERPOINTERVPROC', 'GL_VERSION_2_0', 'glBlendEquationSeparate',
'glDrawBuffers', 'glStencilOpSeparate', 'glStencilFuncSeparate',
'glStencilMaskSeparate', 'glAttachShader', 'glBindAttribLocation',
'glCompileShader', 'glCreateProgram', 'glCreateShader', 'glDeleteProgram',
'glDeleteShader', 'glDetachShader', 'glDisableVertexAttribArray',
'glEnableVertexAttribArray', 'glGetActiveAttrib', 'glGetActiveUniform',
'glGetAttachedShaders', 'glGetAttribLocation', 'glGetProgramiv',
'glGetProgramInfoLog', 'glGetShaderiv', 'glGetShaderInfoLog',
'glGetShaderSource', 'glGetUniformLocation', 'glGetUniformfv',
'glGetUniformiv', 'glGetVertexAttribdv', 'glGetVertexAttribfv',
'glGetVertexAttribiv', 'glGetVertexAttribPointerv', 'glIsProgram',
'glIsShader', 'glLinkProgram', 'glShaderSource', 'glUseProgram',
'glUniform1f', 'glUniform2f', 'glUniform3f', 'glUniform4f', 'glUniform1i',
'glUniform2i', 'glUniform3i', 'glUniform4i', 'glUniform1fv', 'glUniform2fv',
'glUniform3fv', 'glUniform4fv', 'glUniform1iv', 'glUniform2iv',
'glUniform3iv', 'glUniform4iv', 'glUniformMatrix2fv', 'glUniformMatrix3fv',
'glUniformMatrix4fv', 'glValidateProgram', 'glVertexAttrib1d',
'glVertexAttrib1dv', 'glVertexAttrib1f', 'glVertexAttrib1fv',
'glVertexAttrib1s', 'glVertexAttrib1sv', 'glVertexAttrib2d',
'glVertexAttrib2dv', 'glVertexAttrib2f', 'glVertexAttrib2fv',
'glVertexAttrib2s', 'glVertexAttrib2sv', 'glVertexAttrib3d',
'glVertexAttrib3dv', 'glVertexAttrib3f', 'glVertexAttrib3fv',
'glVertexAttrib3s', 'glVertexAttrib3sv', 'glVertexAttrib4Nbv',
'glVertexAttrib4Niv', 'glVertexAttrib4Nsv', 'glVertexAttrib4Nub',
'glVertexAttrib4Nubv', 'glVertexAttrib4Nuiv', 'glVertexAttrib4Nusv',
'glVertexAttrib4bv', 'glVertexAttrib4d', 'glVertexAttrib4dv',
'glVertexAttrib4f', 'glVertexAttrib4fv', 'glVertexAttrib4iv',
'glVertexAttrib4s', 'glVertexAttrib4sv', 'glVertexAttrib4ubv',
'glVertexAttrib4uiv', 'glVertexAttrib4usv', 'glVertexAttribPointer',
'PFNGLBLENDEQUATIONSEPARATEPROC', 'PFNGLDRAWBUFFERSPROC',
'PFNGLSTENCILOPSEPARATEPROC', 'PFNGLSTENCILFUNCSEPARATEPROC',
'PFNGLSTENCILMASKSEPARATEPROC', 'PFNGLATTACHSHADERPROC',
'PFNGLBINDATTRIBLOCATIONPROC', 'PFNGLCOMPILESHADERPROC',
'PFNGLCREATEPROGRAMPROC', 'PFNGLCREATESHADERPROC', 'PFNGLDELETEPROGRAMPROC',
'PFNGLDELETESHADERPROC', 'PFNGLDETACHSHADERPROC',
'PFNGLDISABLEVERTEXATTRIBARRAYPROC', 'PFNGLENABLEVERTEXATTRIBARRAYPROC',
'PFNGLGETACTIVEATTRIBPROC', 'PFNGLGETACTIVEUNIFORMPROC',
'PFNGLGETATTACHEDSHADERSPROC', 'PFNGLGETATTRIBLOCATIONPROC',
'PFNGLGETPROGRAMIVPROC', 'PFNGLGETPROGRAMINFOLOGPROC', 'PFNGLGETSHADERIVPROC',
'PFNGLGETSHADERINFOLOGPROC', 'PFNGLGETSHADERSOURCEPROC',
'PFNGLGETUNIFORMLOCATIONPROC', 'PFNGLGETUNIFORMFVPROC',
'PFNGLGETUNIFORMIVPROC', 'PFNGLGETVERTEXATTRIBDVPROC',
'PFNGLGETVERTEXATTRIBFVPROC', 'PFNGLGETVERTEXATTRIBIVPROC',
'PFNGLGETVERTEXATTRIBPOINTERVPROC', 'PFNGLISPROGRAMPROC', 'PFNGLISSHADERPROC',
'PFNGLLINKPROGRAMPROC', 'PFNGLSHADERSOURCEPROC', 'PFNGLUSEPROGRAMPROC',
'PFNGLUNIFORM1FPROC', 'PFNGLUNIFORM2FPROC', 'PFNGLUNIFORM3FPROC',
'PFNGLUNIFORM4FPROC', 'PFNGLUNIFORM1IPROC', 'PFNGLUNIFORM2IPROC',
'PFNGLUNIFORM3IPROC', 'PFNGLUNIFORM4IPROC', 'PFNGLUNIFORM1FVPROC',
'PFNGLUNIFORM2FVPROC', 'PFNGLUNIFORM3FVPROC', 'PFNGLUNIFORM4FVPROC',
'PFNGLUNIFORM1IVPROC', 'PFNGLUNIFORM2IVPROC', 'PFNGLUNIFORM3IVPROC',
'PFNGLUNIFORM4IVPROC', 'PFNGLUNIFORMMATRIX2FVPROC',
'PFNGLUNIFORMMATRIX3FVPROC', 'PFNGLUNIFORMMATRIX4FVPROC',
'PFNGLVALIDATEPROGRAMPROC', 'PFNGLVERTEXATTRIB1DPROC',
'PFNGLVERTEXATTRIB1DVPROC', 'PFNGLVERTEXATTRIB1FPROC',
'PFNGLVERTEXATTRIB1FVPROC', 'PFNGLVERTEXATTRIB1SPROC',
'PFNGLVERTEXATTRIB1SVPROC', 'PFNGLVERTEXATTRIB2DPROC',
'PFNGLVERTEXATTRIB2DVPROC', 'PFNGLVERTEXATTRIB2FPROC',
'PFNGLVERTEXATTRIB2FVPROC', 'PFNGLVERTEXATTRIB2SPROC',
'PFNGLVERTEXATTRIB2SVPROC', 'PFNGLVERTEXATTRIB3DPROC',
'PFNGLVERTEXATTRIB3DVPROC', 'PFNGLVERTEXATTRIB3FPROC',
'PFNGLVERTEXATTRIB3FVPROC', 'PFNGLVERTEXATTRIB3SPROC',
'PFNGLVERTEXATTRIB3SVPROC', 'PFNGLVERTEXATTRIB4NBVPROC',
'PFNGLVERTEXATTRIB4NIVPROC', 'PFNGLVERTEXATTRIB4NSVPROC',
'PFNGLVERTEXATTRIB4NUBPROC', 'PFNGLVERTEXATTRIB4NUBVPROC',
'PFNGLVERTEXATTRIB4NUIVPROC', 'PFNGLVERTEXATTRIB4NUSVPROC',
'PFNGLVERTEXATTRIB4BVPROC', 'PFNGLVERTEXATTRIB4DPROC',
'PFNGLVERTEXATTRIB4DVPROC', 'PFNGLVERTEXATTRIB4FPROC',
'PFNGLVERTEXATTRIB4FVPROC', 'PFNGLVERTEXATTRIB4IVPROC',
'PFNGLVERTEXATTRIB4SPROC', 'PFNGLVERTEXATTRIB4SVPROC',
'PFNGLVERTEXATTRIB4UBVPROC', 'PFNGLVERTEXATTRIB4UIVPROC',
'PFNGLVERTEXATTRIB4USVPROC', 'PFNGLVERTEXATTRIBPOINTERPROC', 'GL_VERSION_2_1',
'glUniformMatrix2x3fv', 'glUniformMatrix3x2fv', 'glUniformMatrix2x4fv',
'glUniformMatrix4x2fv', 'glUniformMatrix3x4fv', 'glUniformMatrix4x3fv',
'PFNGLUNIFORMMATRIX2X3FVPROC', 'PFNGLUNIFORMMATRIX3X2FVPROC',
'PFNGLUNIFORMMATRIX2X4FVPROC', 'PFNGLUNIFORMMATRIX4X2FVPROC',
'PFNGLUNIFORMMATRIX3X4FVPROC', 'PFNGLUNIFORMMATRIX4X3FVPROC',
'GL_ARB_multitexture', 'glActiveTextureARB', 'glClientActiveTextureARB',
'glMultiTexCoord1dARB', 'glMultiTexCoord1dvARB', 'glMultiTexCoord1fARB',
'glMultiTexCoord1fvARB', 'glMultiTexCoord1iARB', 'glMultiTexCoord1ivARB',
'glMultiTexCoord1sARB', 'glMultiTexCoord1svARB', 'glMultiTexCoord2dARB',
'glMultiTexCoord2dvARB', 'glMultiTexCoord2fARB', 'glMultiTexCoord2fvARB',
'glMultiTexCoord2iARB', 'glMultiTexCoord2ivARB', 'glMultiTexCoord2sARB',
'glMultiTexCoord2svARB', 'glMultiTexCoord3dARB', 'glMultiTexCoord3dvARB',
'glMultiTexCoord3fARB', 'glMultiTexCoord3fvARB', 'glMultiTexCoord3iARB',
'glMultiTexCoord3ivARB', 'glMultiTexCoord3sARB', 'glMultiTexCoord3svARB',
'glMultiTexCoord4dARB', 'glMultiTexCoord4dvARB', 'glMultiTexCoord4fARB',
'glMultiTexCoord4fvARB', 'glMultiTexCoord4iARB', 'glMultiTexCoord4ivARB',
'glMultiTexCoord4sARB', 'glMultiTexCoord4svARB', 'PFNGLACTIVETEXTUREARBPROC',
'PFNGLCLIENTACTIVETEXTUREARBPROC', 'PFNGLMULTITEXCOORD1DARBPROC',
'PFNGLMULTITEXCOORD1DVARBPROC', 'PFNGLMULTITEXCOORD1FARBPROC',
'PFNGLMULTITEXCOORD1FVARBPROC', 'PFNGLMULTITEXCOORD1IARBPROC',
'PFNGLMULTITEXCOORD1IVARBPROC', 'PFNGLMULTITEXCOORD1SARBPROC',
'PFNGLMULTITEXCOORD1SVARBPROC', 'PFNGLMULTITEXCOORD2DARBPROC',
'PFNGLMULTITEXCOORD2DVARBPROC', 'PFNGLMULTITEXCOORD2FARBPROC',
'PFNGLMULTITEXCOORD2FVARBPROC', 'PFNGLMULTITEXCOORD2IARBPROC',
'PFNGLMULTITEXCOORD2IVARBPROC', 'PFNGLMULTITEXCOORD2SARBPROC',
'PFNGLMULTITEXCOORD2SVARBPROC', 'PFNGLMULTITEXCOORD3DARBPROC',
'PFNGLMULTITEXCOORD3DVARBPROC', 'PFNGLMULTITEXCOORD3FARBPROC',
'PFNGLMULTITEXCOORD3FVARBPROC', 'PFNGLMULTITEXCOORD3IARBPROC',
'PFNGLMULTITEXCOORD3IVARBPROC', 'PFNGLMULTITEXCOORD3SARBPROC',
'PFNGLMULTITEXCOORD3SVARBPROC', 'PFNGLMULTITEXCOORD4DARBPROC',
'PFNGLMULTITEXCOORD4DVARBPROC', 'PFNGLMULTITEXCOORD4FARBPROC',
'PFNGLMULTITEXCOORD4FVARBPROC', 'PFNGLMULTITEXCOORD4IARBPROC',
'PFNGLMULTITEXCOORD4IVARBPROC', 'PFNGLMULTITEXCOORD4SARBPROC',
'PFNGLMULTITEXCOORD4SVARBPROC', 'GL_ARB_transpose_matrix',
'glLoadTransposeMatrixfARB', 'glLoadTransposeMatrixdARB',
'glMultTransposeMatrixfARB', 'glMultTransposeMatrixdARB',
'PFNGLLOADTRANSPOSEMATRIXFARBPROC', 'PFNGLLOADTRANSPOSEMATRIXDARBPROC',
'PFNGLMULTTRANSPOSEMATRIXFARBPROC', 'PFNGLMULTTRANSPOSEMATRIXDARBPROC',
'GL_ARB_multisample', 'glSampleCoverageARB', 'PFNGLSAMPLECOVERAGEARBPROC',
'GL_ARB_texture_env_add', 'GL_ARB_texture_cube_map',
'GL_ARB_texture_compression', 'glCompressedTexImage3DARB',
'glCompressedTexImage2DARB', 'glCompressedTexImage1DARB',
'glCompressedTexSubImage3DARB', 'glCompressedTexSubImage2DARB',
'glCompressedTexSubImage1DARB', 'glGetCompressedTexImageARB',
'PFNGLCOMPRESSEDTEXIMAGE3DARBPROC', 'PFNGLCOMPRESSEDTEXIMAGE2DARBPROC',
'PFNGLCOMPRESSEDTEXIMAGE1DARBPROC', 'PFNGLCOMPRESSEDTEXSUBIMAGE3DARBPROC',
'PFNGLCOMPRESSEDTEXSUBIMAGE2DARBPROC', 'PFNGLCOMPRESSEDTEXSUBIMAGE1DARBPROC',
'PFNGLGETCOMPRESSEDTEXIMAGEARBPROC', 'GL_ARB_texture_border_clamp',
'GL_ARB_point_parameters', 'glPointParameterfARB', 'glPointParameterfvARB',
'PFNGLPOINTPARAMETERFARBPROC', 'PFNGLPOINTPARAMETERFVARBPROC',
'GL_ARB_vertex_blend', 'glWeightbvARB', 'glWeightsvARB', 'glWeightivARB',
'glWeightfvARB', 'glWeightdvARB', 'glWeightubvARB', 'glWeightusvARB',
'glWeightuivARB', 'glWeightPointerARB', 'glVertexBlendARB',
'PFNGLWEIGHTBVARBPROC', 'PFNGLWEIGHTSVARBPROC', 'PFNGLWEIGHTIVARBPROC',
'PFNGLWEIGHTFVARBPROC', 'PFNGLWEIGHTDVARBPROC', 'PFNGLWEIGHTUBVARBPROC',
'PFNGLWEIGHTUSVARBPROC', 'PFNGLWEIGHTUIVARBPROC', 'PFNGLWEIGHTPOINTERARBPROC',
'PFNGLVERTEXBLENDARBPROC', 'GL_ARB_matrix_palette',
'glCurrentPaletteMatrixARB', 'glMatrixIndexubvARB', 'glMatrixIndexusvARB',
'glMatrixIndexuivARB', 'glMatrixIndexPointerARB',
'PFNGLCURRENTPALETTEMATRIXARBPROC', 'PFNGLMATRIXINDEXUBVARBPROC',
'PFNGLMATRIXINDEXUSVARBPROC', 'PFNGLMATRIXINDEXUIVARBPROC',
'PFNGLMATRIXINDEXPOINTERARBPROC', 'GL_ARB_texture_env_combine',
'GL_ARB_texture_env_crossbar', 'GL_ARB_texture_env_dot3',
'GL_ARB_texture_mirrored_repeat', 'GL_ARB_depth_texture', 'GL_ARB_shadow',
'GL_ARB_shadow_ambient', 'GL_ARB_window_pos', 'glWindowPos2dARB',
'glWindowPos2dvARB', 'glWindowPos2fARB', 'glWindowPos2fvARB',
'glWindowPos2iARB', 'glWindowPos2ivARB', 'glWindowPos2sARB',
'glWindowPos2svARB', 'glWindowPos3dARB', 'glWindowPos3dvARB',
'glWindowPos3fARB', 'glWindowPos3fvARB', 'glWindowPos3iARB',
'glWindowPos3ivARB', 'glWindowPos3sARB', 'glWindowPos3svARB',
'PFNGLWINDOWPOS2DARBPROC', 'PFNGLWINDOWPOS2DVARBPROC',
'PFNGLWINDOWPOS2FARBPROC', 'PFNGLWINDOWPOS2FVARBPROC',
'PFNGLWINDOWPOS2IARBPROC', 'PFNGLWINDOWPOS2IVARBPROC',
'PFNGLWINDOWPOS2SARBPROC', 'PFNGLWINDOWPOS2SVARBPROC',
'PFNGLWINDOWPOS3DARBPROC', 'PFNGLWINDOWPOS3DVARBPROC',
'PFNGLWINDOWPOS3FARBPROC', 'PFNGLWINDOWPOS3FVARBPROC',
'PFNGLWINDOWPOS3IARBPROC', 'PFNGLWINDOWPOS3IVARBPROC',
'PFNGLWINDOWPOS3SARBPROC', 'PFNGLWINDOWPOS3SVARBPROC',
'GL_ARB_vertex_program', 'glVertexAttrib1dARB', 'glVertexAttrib1dvARB',
'glVertexAttrib1fARB', 'glVertexAttrib1fvARB', 'glVertexAttrib1sARB',
'glVertexAttrib1svARB', 'glVertexAttrib2dARB', 'glVertexAttrib2dvARB',
'glVertexAttrib2fARB', 'glVertexAttrib2fvARB', 'glVertexAttrib2sARB',
'glVertexAttrib2svARB', 'glVertexAttrib3dARB', 'glVertexAttrib3dvARB',
'glVertexAttrib3fARB', 'glVertexAttrib3fvARB', 'glVertexAttrib3sARB',
'glVertexAttrib3svARB', 'glVertexAttrib4NbvARB', 'glVertexAttrib4NivARB',
'glVertexAttrib4NsvARB', 'glVertexAttrib4NubARB', 'glVertexAttrib4NubvARB',
'glVertexAttrib4NuivARB', 'glVertexAttrib4NusvARB', 'glVertexAttrib4bvARB',
'glVertexAttrib4dARB', 'glVertexAttrib4dvARB', 'glVertexAttrib4fARB',
'glVertexAttrib4fvARB', 'glVertexAttrib4ivARB', 'glVertexAttrib4sARB',
'glVertexAttrib4svARB', 'glVertexAttrib4ubvARB', 'glVertexAttrib4uivARB',
'glVertexAttrib4usvARB', 'glVertexAttribPointerARB',
'glEnableVertexAttribArrayARB', 'glDisableVertexAttribArrayARB',
'glProgramStringARB', 'glBindProgramARB', 'glDeleteProgramsARB',
'glGenProgramsARB', 'glProgramEnvParameter4dARB',
'glProgramEnvParameter4dvARB', 'glProgramEnvParameter4fARB',
'glProgramEnvParameter4fvARB', 'glProgramLocalParameter4dARB',
'glProgramLocalParameter4dvARB', 'glProgramLocalParameter4fARB',
'glProgramLocalParameter4fvARB', 'glGetProgramEnvParameterdvARB',
'glGetProgramEnvParameterfvARB', 'glGetProgramLocalParameterdvARB',
'glGetProgramLocalParameterfvARB', 'glGetProgramivARB',
'glGetProgramStringARB', 'glGetVertexAttribdvARB', 'glGetVertexAttribfvARB',
'glGetVertexAttribivARB', 'glGetVertexAttribPointervARB', 'glIsProgramARB',
'PFNGLVERTEXATTRIB1DARBPROC', 'PFNGLVERTEXATTRIB1DVARBPROC',
'PFNGLVERTEXATTRIB1FARBPROC', 'PFNGLVERTEXATTRIB1FVARBPROC',
'PFNGLVERTEXATTRIB1SARBPROC', 'PFNGLVERTEXATTRIB1SVARBPROC',
'PFNGLVERTEXATTRIB2DARBPROC', 'PFNGLVERTEXATTRIB2DVARBPROC',
'PFNGLVERTEXATTRIB2FARBPROC', 'PFNGLVERTEXATTRIB2FVARBPROC',
'PFNGLVERTEXATTRIB2SARBPROC', 'PFNGLVERTEXATTRIB2SVARBPROC',
'PFNGLVERTEXATTRIB3DARBPROC', 'PFNGLVERTEXATTRIB3DVARBPROC',
'PFNGLVERTEXATTRIB3FARBPROC', 'PFNGLVERTEXATTRIB3FVARBPROC',
'PFNGLVERTEXATTRIB3SARBPROC', 'PFNGLVERTEXATTRIB3SVARBPROC',
'PFNGLVERTEXATTRIB4NBVARBPROC', 'PFNGLVERTEXATTRIB4NIVARBPROC',
'PFNGLVERTEXATTRIB4NSVARBPROC', 'PFNGLVERTEXATTRIB4NUBARBPROC',
'PFNGLVERTEXATTRIB4NUBVARBPROC', 'PFNGLVERTEXATTRIB4NUIVARBPROC',
'PFNGLVERTEXATTRIB4NUSVARBPROC', 'PFNGLVERTEXATTRIB4BVARBPROC',
'PFNGLVERTEXATTRIB4DARBPROC', 'PFNGLVERTEXATTRIB4DVARBPROC',
'PFNGLVERTEXATTRIB4FARBPROC', 'PFNGLVERTEXATTRIB4FVARBPROC',
'PFNGLVERTEXATTRIB4IVARBPROC', 'PFNGLVERTEXATTRIB4SARBPROC',
'PFNGLVERTEXATTRIB4SVARBPROC', 'PFNGLVERTEXATTRIB4UBVARBPROC',
'PFNGLVERTEXATTRIB4UIVARBPROC', 'PFNGLVERTEXATTRIB4USVARBPROC',
'PFNGLVERTEXATTRIBPOINTERARBPROC', 'PFNGLENABLEVERTEXATTRIBARRAYARBPROC',
'PFNGLDISABLEVERTEXATTRIBARRAYARBPROC', 'PFNGLPROGRAMSTRINGARBPROC',
'PFNGLBINDPROGRAMARBPROC', 'PFNGLDELETEPROGRAMSARBPROC',
'PFNGLGENPROGRAMSARBPROC', 'PFNGLPROGRAMENVPARAMETER4DARBPROC',
'PFNGLPROGRAMENVPARAMETER4DVARBPROC', 'PFNGLPROGRAMENVPARAMETER4FARBPROC',
'PFNGLPROGRAMENVPARAMETER4FVARBPROC', 'PFNGLPROGRAMLOCALPARAMETER4DARBPROC',
'PFNGLPROGRAMLOCALPARAMETER4DVARBPROC', 'PFNGLPROGRAMLOCALPARAMETER4FARBPROC',
'PFNGLPROGRAMLOCALPARAMETER4FVARBPROC',
'PFNGLGETPROGRAMENVPARAMETERDVARBPROC',
'PFNGLGETPROGRAMENVPARAMETERFVARBPROC',
'PFNGLGETPROGRAMLOCALPARAMETERDVARBPROC',
'PFNGLGETPROGRAMLOCALPARAMETERFVARBPROC', 'PFNGLGETPROGRAMIVARBPROC',
'PFNGLGETPROGRAMSTRINGARBPROC', 'PFNGLGETVERTEXATTRIBDVARBPROC',
'PFNGLGETVERTEXATTRIBFVARBPROC', 'PFNGLGETVERTEXATTRIBIVARBPROC',
'PFNGLGETVERTEXATTRIBPOINTERVARBPROC', 'PFNGLISPROGRAMARBPROC',
'GL_ARB_fragment_program', 'GL_ARB_vertex_buffer_object', 'glBindBufferARB',
'glDeleteBuffersARB', 'glGenBuffersARB', 'glIsBufferARB', 'glBufferDataARB',
'glBufferSubDataARB', 'glGetBufferSubDataARB', 'glMapBufferARB',
'glUnmapBufferARB', 'glGetBufferParameterivARB', 'glGetBufferPointervARB',
'PFNGLBINDBUFFERARBPROC', 'PFNGLDELETEBUFFERSARBPROC',
'PFNGLGENBUFFERSARBPROC', 'PFNGLISBUFFERARBPROC', 'PFNGLBUFFERDATAARBPROC',
'PFNGLBUFFERSUBDATAARBPROC', 'PFNGLGETBUFFERSUBDATAARBPROC',
'PFNGLMAPBUFFERARBPROC', 'PFNGLUNMAPBUFFERARBPROC',
'PFNGLGETBUFFERPARAMETERIVARBPROC', 'PFNGLGETBUFFERPOINTERVARBPROC',
'GL_ARB_occlusion_query', 'glGenQueriesARB', 'glDeleteQueriesARB',
'glIsQueryARB', 'glBeginQueryARB', 'glEndQueryARB', 'glGetQueryivARB',
'glGetQueryObjectivARB', 'glGetQueryObjectuivARB', 'PFNGLGENQUERIESARBPROC',
'PFNGLDELETEQUERIESARBPROC', 'PFNGLISQUERYARBPROC', 'PFNGLBEGINQUERYARBPROC',
'PFNGLENDQUERYARBPROC', 'PFNGLGETQUERYIVARBPROC',
'PFNGLGETQUERYOBJECTIVARBPROC', 'PFNGLGETQUERYOBJECTUIVARBPROC',
'GL_ARB_shader_objects', 'glDeleteObjectARB', 'glGetHandleARB',
'glDetachObjectARB', 'glCreateShaderObjectARB', 'glShaderSourceARB',
'glCompileShaderARB', 'glCreateProgramObjectARB', 'glAttachObjectARB',
'glLinkProgramARB', 'glUseProgramObjectARB', 'glValidateProgramARB',
'glUniform1fARB', 'glUniform2fARB', 'glUniform3fARB', 'glUniform4fARB',
'glUniform1iARB', 'glUniform2iARB', 'glUniform3iARB', 'glUniform4iARB',
'glUniform1fvARB', 'glUniform2fvARB', 'glUniform3fvARB', 'glUniform4fvARB',
'glUniform1ivARB', 'glUniform2ivARB', 'glUniform3ivARB', 'glUniform4ivARB',
'glUniformMatrix2fvARB', 'glUniformMatrix3fvARB', 'glUniformMatrix4fvARB',
'glGetObjectParameterfvARB', 'glGetObjectParameterivARB', 'glGetInfoLogARB',
'glGetAttachedObjectsARB', 'glGetUniformLocationARB', 'glGetActiveUniformARB',
'glGetUniformfvARB', 'glGetUniformivARB', 'glGetShaderSourceARB',
'PFNGLDELETEOBJECTARBPROC', 'PFNGLGETHANDLEARBPROC',
'PFNGLDETACHOBJECTARBPROC', 'PFNGLCREATESHADEROBJECTARBPROC',
'PFNGLSHADERSOURCEARBPROC', 'PFNGLCOMPILESHADERARBPROC',
'PFNGLCREATEPROGRAMOBJECTARBPROC', 'PFNGLATTACHOBJECTARBPROC',
'PFNGLLINKPROGRAMARBPROC', 'PFNGLUSEPROGRAMOBJECTARBPROC',
'PFNGLVALIDATEPROGRAMARBPROC', 'PFNGLUNIFORM1FARBPROC',
'PFNGLUNIFORM2FARBPROC', 'PFNGLUNIFORM3FARBPROC', 'PFNGLUNIFORM4FARBPROC',
'PFNGLUNIFORM1IARBPROC', 'PFNGLUNIFORM2IARBPROC', 'PFNGLUNIFORM3IARBPROC',
'PFNGLUNIFORM4IARBPROC', 'PFNGLUNIFORM1FVARBPROC', 'PFNGLUNIFORM2FVARBPROC',
'PFNGLUNIFORM3FVARBPROC', 'PFNGLUNIFORM4FVARBPROC', 'PFNGLUNIFORM1IVARBPROC',
'PFNGLUNIFORM2IVARBPROC', 'PFNGLUNIFORM3IVARBPROC', 'PFNGLUNIFORM4IVARBPROC',
'PFNGLUNIFORMMATRIX2FVARBPROC', 'PFNGLUNIFORMMATRIX3FVARBPROC',
'PFNGLUNIFORMMATRIX4FVARBPROC', 'PFNGLGETOBJECTPARAMETERFVARBPROC',
'PFNGLGETOBJECTPARAMETERIVARBPROC', 'PFNGLGETINFOLOGARBPROC',
'PFNGLGETATTACHEDOBJECTSARBPROC', 'PFNGLGETUNIFORMLOCATIONARBPROC',
'PFNGLGETACTIVEUNIFORMARBPROC', 'PFNGLGETUNIFORMFVARBPROC',
'PFNGLGETUNIFORMIVARBPROC', 'PFNGLGETSHADERSOURCEARBPROC',
'GL_ARB_vertex_shader', 'glBindAttribLocationARB', 'glGetActiveAttribARB',
'glGetAttribLocationARB', 'PFNGLBINDATTRIBLOCATIONARBPROC',
'PFNGLGETACTIVEATTRIBARBPROC', 'PFNGLGETATTRIBLOCATIONARBPROC',
'GL_ARB_fragment_shader', 'GL_ARB_shading_language_100',
'GL_ARB_texture_non_power_of_two', 'GL_ARB_point_sprite',
'GL_ARB_fragment_program_shadow', 'GL_ARB_draw_buffers', 'glDrawBuffersARB',
'PFNGLDRAWBUFFERSARBPROC', 'GL_ARB_texture_rectangle',
'GL_ARB_color_buffer_float', 'glClampColorARB', 'PFNGLCLAMPCOLORARBPROC',
'GL_ARB_half_float_pixel', 'GL_ARB_texture_float',
'GL_ARB_pixel_buffer_object', 'GL_EXT_abgr', 'GL_EXT_blend_color',
'glBlendColorEXT', 'PFNGLBLENDCOLOREXTPROC', 'GL_EXT_polygon_offset',
'glPolygonOffsetEXT', 'PFNGLPOLYGONOFFSETEXTPROC', 'GL_EXT_texture',
'GL_EXT_texture3D', 'glTexImage3DEXT', 'glTexSubImage3DEXT',
'PFNGLTEXIMAGE3DEXTPROC', 'PFNGLTEXSUBIMAGE3DEXTPROC',
'GL_SGIS_texture_filter4', 'glGetTexFilterFuncSGIS', 'glTexFilterFuncSGIS',
'PFNGLGETTEXFILTERFUNCSGISPROC', 'PFNGLTEXFILTERFUNCSGISPROC',
'GL_EXT_subtexture', 'glTexSubImage1DEXT', 'glTexSubImage2DEXT',
'PFNGLTEXSUBIMAGE1DEXTPROC', 'PFNGLTEXSUBIMAGE2DEXTPROC',
'GL_EXT_copy_texture', 'glCopyTexImage1DEXT', 'glCopyTexImage2DEXT',
'glCopyTexSubImage1DEXT', 'glCopyTexSubImage2DEXT', 'glCopyTexSubImage3DEXT',
'PFNGLCOPYTEXIMAGE1DEXTPROC', 'PFNGLCOPYTEXIMAGE2DEXTPROC',
'PFNGLCOPYTEXSUBIMAGE1DEXTPROC', 'PFNGLCOPYTEXSUBIMAGE2DEXTPROC',
'PFNGLCOPYTEXSUBIMAGE3DEXTPROC', 'GL_EXT_histogram', 'glGetHistogramEXT',
'glGetHistogramParameterfvEXT', 'glGetHistogramParameterivEXT',
'glGetMinmaxEXT', 'glGetMinmaxParameterfvEXT', 'glGetMinmaxParameterivEXT',
'glHistogramEXT', 'glMinmaxEXT', 'glResetHistogramEXT', 'glResetMinmaxEXT',
'PFNGLGETHISTOGRAMEXTPROC', 'PFNGLGETHISTOGRAMPARAMETERFVEXTPROC',
'PFNGLGETHISTOGRAMPARAMETERIVEXTPROC', 'PFNGLGETMINMAXEXTPROC',
'PFNGLGETMINMAXPARAMETERFVEXTPROC', 'PFNGLGETMINMAXPARAMETERIVEXTPROC',
'PFNGLHISTOGRAMEXTPROC', 'PFNGLMINMAXEXTPROC', 'PFNGLRESETHISTOGRAMEXTPROC',
'PFNGLRESETMINMAXEXTPROC', 'GL_EXT_convolution', 'glConvolutionFilter1DEXT',
'glConvolutionFilter2DEXT', 'glConvolutionParameterfEXT',
'glConvolutionParameterfvEXT', 'glConvolutionParameteriEXT',
'glConvolutionParameterivEXT', 'glCopyConvolutionFilter1DEXT',
'glCopyConvolutionFilter2DEXT', 'glGetConvolutionFilterEXT',
'glGetConvolutionParameterfvEXT', 'glGetConvolutionParameterivEXT',
'glGetSeparableFilterEXT', 'glSeparableFilter2DEXT',
'PFNGLCONVOLUTIONFILTER1DEXTPROC', 'PFNGLCONVOLUTIONFILTER2DEXTPROC',
'PFNGLCONVOLUTIONPARAMETERFEXTPROC', 'PFNGLCONVOLUTIONPARAMETERFVEXTPROC',
'PFNGLCONVOLUTIONPARAMETERIEXTPROC', 'PFNGLCONVOLUTIONPARAMETERIVEXTPROC',
'PFNGLCOPYCONVOLUTIONFILTER1DEXTPROC', 'PFNGLCOPYCONVOLUTIONFILTER2DEXTPROC',
'PFNGLGETCONVOLUTIONFILTEREXTPROC', 'PFNGLGETCONVOLUTIONPARAMETERFVEXTPROC',
'PFNGLGETCONVOLUTIONPARAMETERIVEXTPROC', 'PFNGLGETSEPARABLEFILTEREXTPROC',
'PFNGLSEPARABLEFILTER2DEXTPROC', 'GL_EXT_color_matrix', 'GL_SGI_color_table',
'glColorTableSGI', 'glColorTableParameterfvSGI', 'glColorTableParameterivSGI',
'glCopyColorTableSGI', 'glGetColorTableSGI', 'glGetColorTableParameterfvSGI',
'glGetColorTableParameterivSGI', 'PFNGLCOLORTABLESGIPROC',
'PFNGLCOLORTABLEPARAMETERFVSGIPROC', 'PFNGLCOLORTABLEPARAMETERIVSGIPROC',
'PFNGLCOPYCOLORTABLESGIPROC', 'PFNGLGETCOLORTABLESGIPROC',
'PFNGLGETCOLORTABLEPARAMETERFVSGIPROC',
'PFNGLGETCOLORTABLEPARAMETERIVSGIPROC', 'GL_SGIX_pixel_texture',
'glPixelTexGenSGIX', 'PFNGLPIXELTEXGENSGIXPROC', 'GL_SGIS_pixel_texture',
'glPixelTexGenParameteriSGIS', 'glPixelTexGenParameterivSGIS',
'glPixelTexGenParameterfSGIS', 'glPixelTexGenParameterfvSGIS',
'glGetPixelTexGenParameterivSGIS', 'glGetPixelTexGenParameterfvSGIS',
'PFNGLPIXELTEXGENPARAMETERISGISPROC', 'PFNGLPIXELTEXGENPARAMETERIVSGISPROC',
'PFNGLPIXELTEXGENPARAMETERFSGISPROC', 'PFNGLPIXELTEXGENPARAMETERFVSGISPROC',
'PFNGLGETPIXELTEXGENPARAMETERIVSGISPROC',
'PFNGLGETPIXELTEXGENPARAMETERFVSGISPROC', 'GL_SGIS_texture4D',
'glTexImage4DSGIS', 'glTexSubImage4DSGIS', 'PFNGLTEXIMAGE4DSGISPROC',
'PFNGLTEXSUBIMAGE4DSGISPROC', 'GL_SGI_texture_color_table', 'GL_EXT_cmyka',
'GL_EXT_texture_object', 'glAreTexturesResidentEXT', 'glBindTextureEXT',
'glDeleteTexturesEXT', 'glGenTexturesEXT', 'glIsTextureEXT',
'glPrioritizeTexturesEXT', 'PFNGLARETEXTURESRESIDENTEXTPROC',
'PFNGLBINDTEXTUREEXTPROC', 'PFNGLDELETETEXTURESEXTPROC',
'PFNGLGENTEXTURESEXTPROC', 'PFNGLISTEXTUREEXTPROC',
'PFNGLPRIORITIZETEXTURESEXTPROC', 'GL_SGIS_detail_texture',
'glDetailTexFuncSGIS', 'glGetDetailTexFuncSGIS', 'PFNGLDETAILTEXFUNCSGISPROC',
'PFNGLGETDETAILTEXFUNCSGISPROC', 'GL_SGIS_sharpen_texture',
'glSharpenTexFuncSGIS', 'glGetSharpenTexFuncSGIS',
'PFNGLSHARPENTEXFUNCSGISPROC', 'PFNGLGETSHARPENTEXFUNCSGISPROC',
'GL_EXT_packed_pixels', 'GL_SGIS_texture_lod', 'GL_SGIS_multisample',
'glSampleMaskSGIS', 'glSamplePatternSGIS', 'PFNGLSAMPLEMASKSGISPROC',
'PFNGLSAMPLEPATTERNSGISPROC', 'GL_EXT_rescale_normal', 'GL_EXT_vertex_array',
'glArrayElementEXT', 'glColorPointerEXT', 'glDrawArraysEXT',
'glEdgeFlagPointerEXT', 'glGetPointervEXT', 'glIndexPointerEXT',
'glNormalPointerEXT', 'glTexCoordPointerEXT', 'glVertexPointerEXT',
'PFNGLARRAYELEMENTEXTPROC', 'PFNGLCOLORPOINTEREXTPROC',
'PFNGLDRAWARRAYSEXTPROC', 'PFNGLEDGEFLAGPOINTEREXTPROC',
'PFNGLGETPOINTERVEXTPROC', 'PFNGLINDEXPOINTEREXTPROC',
'PFNGLNORMALPOINTEREXTPROC', 'PFNGLTEXCOORDPOINTEREXTPROC',
'PFNGLVERTEXPOINTEREXTPROC', 'GL_EXT_misc_attribute',
'GL_SGIS_generate_mipmap', 'GL_SGIX_clipmap', 'GL_SGIX_shadow',
'GL_SGIS_texture_edge_clamp', 'GL_SGIS_texture_border_clamp',
'GL_EXT_blend_minmax', 'glBlendEquationEXT', 'PFNGLBLENDEQUATIONEXTPROC',
'GL_EXT_blend_subtract', 'GL_EXT_blend_logic_op', 'GL_SGIX_interlace',
'GL_SGIX_pixel_tiles', 'GL_SGIX_texture_select', 'GL_SGIX_sprite',
'glSpriteParameterfSGIX', 'glSpriteParameterfvSGIX', 'glSpriteParameteriSGIX',
'glSpriteParameterivSGIX', 'PFNGLSPRITEPARAMETERFSGIXPROC',
'PFNGLSPRITEPARAMETERFVSGIXPROC', 'PFNGLSPRITEPARAMETERISGIXPROC',
'PFNGLSPRITEPARAMETERIVSGIXPROC', 'GL_SGIX_texture_multi_buffer',
'GL_EXT_point_parameters', 'glPointParameterfEXT', 'glPointParameterfvEXT',
'PFNGLPOINTPARAMETERFEXTPROC', 'PFNGLPOINTPARAMETERFVEXTPROC',
'GL_SGIS_point_parameters', 'glPointParameterfSGIS', 'glPointParameterfvSGIS',
'PFNGLPOINTPARAMETERFSGISPROC', 'PFNGLPOINTPARAMETERFVSGISPROC',
'GL_SGIX_instruments', 'glGetInstrumentsSGIX', 'glInstrumentsBufferSGIX',
'glPollInstrumentsSGIX', 'glReadInstrumentsSGIX', 'glStartInstrumentsSGIX',
'glStopInstrumentsSGIX', 'PFNGLGETINSTRUMENTSSGIXPROC',
'PFNGLINSTRUMENTSBUFFERSGIXPROC', 'PFNGLPOLLINSTRUMENTSSGIXPROC',
'PFNGLREADINSTRUMENTSSGIXPROC', 'PFNGLSTARTINSTRUMENTSSGIXPROC',
'PFNGLSTOPINSTRUMENTSSGIXPROC', 'GL_SGIX_texture_scale_bias',
'GL_SGIX_framezoom', 'glFrameZoomSGIX', 'PFNGLFRAMEZOOMSGIXPROC',
'GL_SGIX_tag_sample_buffer', 'glTagSampleBufferSGIX',
'PFNGLTAGSAMPLEBUFFERSGIXPROC', 'GL_SGIX_polynomial_ffd',
'glDeformationMap3dSGIX', 'glDeformationMap3fSGIX', 'glDeformSGIX',
'glLoadIdentityDeformationMapSGIX', 'PFNGLDEFORMATIONMAP3DSGIXPROC',
'PFNGLDEFORMATIONMAP3FSGIXPROC', 'PFNGLDEFORMSGIXPROC',
'PFNGLLOADIDENTITYDEFORMATIONMAPSGIXPROC', 'GL_SGIX_reference_plane',
'glReferencePlaneSGIX', 'PFNGLREFERENCEPLANESGIXPROC', 'GL_SGIX_flush_raster',
'glFlushRasterSGIX', 'PFNGLFLUSHRASTERSGIXPROC', 'GL_SGIX_depth_texture',
'GL_SGIS_fog_function', 'glFogFuncSGIS', 'glGetFogFuncSGIS',
'PFNGLFOGFUNCSGISPROC', 'PFNGLGETFOGFUNCSGISPROC', 'GL_SGIX_fog_offset',
'GL_HP_image_transform', 'glImageTransformParameteriHP',
'glImageTransformParameterfHP', 'glImageTransformParameterivHP',
'glImageTransformParameterfvHP', 'glGetImageTransformParameterivHP',
'glGetImageTransformParameterfvHP', 'PFNGLIMAGETRANSFORMPARAMETERIHPPROC',
'PFNGLIMAGETRANSFORMPARAMETERFHPPROC', 'PFNGLIMAGETRANSFORMPARAMETERIVHPPROC',
'PFNGLIMAGETRANSFORMPARAMETERFVHPPROC',
'PFNGLGETIMAGETRANSFORMPARAMETERIVHPPROC',
'PFNGLGETIMAGETRANSFORMPARAMETERFVHPPROC', 'GL_HP_convolution_border_modes',
'GL_SGIX_texture_add_env', 'GL_EXT_color_subtable', 'glColorSubTableEXT',
'glCopyColorSubTableEXT', 'PFNGLCOLORSUBTABLEEXTPROC',
'PFNGLCOPYCOLORSUBTABLEEXTPROC', 'GL_PGI_vertex_hints', 'GL_PGI_misc_hints',
'glHintPGI', 'PFNGLHINTPGIPROC', 'GL_EXT_paletted_texture', 'glColorTableEXT',
'glGetColorTableEXT', 'glGetColorTableParameterivEXT',
'glGetColorTableParameterfvEXT', 'PFNGLCOLORTABLEEXTPROC',
'PFNGLGETCOLORTABLEEXTPROC', 'PFNGLGETCOLORTABLEPARAMETERIVEXTPROC',
'PFNGLGETCOLORTABLEPARAMETERFVEXTPROC', 'GL_EXT_clip_volume_hint',
'GL_SGIX_list_priority', 'glGetListParameterfvSGIX',
'glGetListParameterivSGIX', 'glListParameterfSGIX', 'glListParameterfvSGIX',
'glListParameteriSGIX', 'glListParameterivSGIX',
'PFNGLGETLISTPARAMETERFVSGIXPROC', 'PFNGLGETLISTPARAMETERIVSGIXPROC',
'PFNGLLISTPARAMETERFSGIXPROC', 'PFNGLLISTPARAMETERFVSGIXPROC',
'PFNGLLISTPARAMETERISGIXPROC', 'PFNGLLISTPARAMETERIVSGIXPROC',
'GL_SGIX_ir_instrument1', 'GL_SGIX_calligraphic_fragment',
'GL_SGIX_texture_lod_bias', 'GL_SGIX_shadow_ambient', 'GL_EXT_index_texture',
'GL_EXT_index_material', 'glIndexMaterialEXT', 'PFNGLINDEXMATERIALEXTPROC',
'GL_EXT_index_func', 'glIndexFuncEXT', 'PFNGLINDEXFUNCEXTPROC',
'GL_EXT_index_array_formats', 'GL_EXT_compiled_vertex_array',
'glLockArraysEXT', 'glUnlockArraysEXT', 'PFNGLLOCKARRAYSEXTPROC',
'PFNGLUNLOCKARRAYSEXTPROC', 'GL_EXT_cull_vertex', 'glCullParameterdvEXT',
'glCullParameterfvEXT', 'PFNGLCULLPARAMETERDVEXTPROC',
'PFNGLCULLPARAMETERFVEXTPROC', 'GL_SGIX_ycrcb', 'GL_SGIX_fragment_lighting',
'glFragmentColorMaterialSGIX', 'glFragmentLightfSGIX',
'glFragmentLightfvSGIX', 'glFragmentLightiSGIX', 'glFragmentLightivSGIX',
'glFragmentLightModelfSGIX', 'glFragmentLightModelfvSGIX',
'glFragmentLightModeliSGIX', 'glFragmentLightModelivSGIX',
'glFragmentMaterialfSGIX', 'glFragmentMaterialfvSGIX',
'glFragmentMaterialiSGIX', 'glFragmentMaterialivSGIX',
'glGetFragmentLightfvSGIX', 'glGetFragmentLightivSGIX',
'glGetFragmentMaterialfvSGIX', 'glGetFragmentMaterialivSGIX',
'glLightEnviSGIX', 'PFNGLFRAGMENTCOLORMATERIALSGIXPROC',
'PFNGLFRAGMENTLIGHTFSGIXPROC', 'PFNGLFRAGMENTLIGHTFVSGIXPROC',
'PFNGLFRAGMENTLIGHTISGIXPROC', 'PFNGLFRAGMENTLIGHTIVSGIXPROC',
'PFNGLFRAGMENTLIGHTMODELFSGIXPROC', 'PFNGLFRAGMENTLIGHTMODELFVSGIXPROC',
'PFNGLFRAGMENTLIGHTMODELISGIXPROC', 'PFNGLFRAGMENTLIGHTMODELIVSGIXPROC',
'PFNGLFRAGMENTMATERIALFSGIXPROC', 'PFNGLFRAGMENTMATERIALFVSGIXPROC',
'PFNGLFRAGMENTMATERIALISGIXPROC', 'PFNGLFRAGMENTMATERIALIVSGIXPROC',
'PFNGLGETFRAGMENTLIGHTFVSGIXPROC', 'PFNGLGETFRAGMENTLIGHTIVSGIXPROC',
'PFNGLGETFRAGMENTMATERIALFVSGIXPROC', 'PFNGLGETFRAGMENTMATERIALIVSGIXPROC',
'PFNGLLIGHTENVISGIXPROC', 'GL_IBM_rasterpos_clip', 'GL_HP_texture_lighting',
'GL_EXT_draw_range_elements', 'glDrawRangeElementsEXT',
'PFNGLDRAWRANGEELEMENTSEXTPROC', 'GL_WIN_phong_shading',
'GL_WIN_specular_fog', 'GL_EXT_light_texture', 'glApplyTextureEXT',
'glTextureLightEXT', 'glTextureMaterialEXT', 'PFNGLAPPLYTEXTUREEXTPROC',
'PFNGLTEXTURELIGHTEXTPROC', 'PFNGLTEXTUREMATERIALEXTPROC',
'GL_SGIX_blend_alpha_minmax', 'GL_EXT_bgra', 'GL_SGIX_async',
'glAsyncMarkerSGIX', 'glFinishAsyncSGIX', 'glPollAsyncSGIX',
'glGenAsyncMarkersSGIX', 'glDeleteAsyncMarkersSGIX', 'glIsAsyncMarkerSGIX',
'PFNGLASYNCMARKERSGIXPROC', 'PFNGLFINISHASYNCSGIXPROC',
'PFNGLPOLLASYNCSGIXPROC', 'PFNGLGENASYNCMARKERSSGIXPROC',
'PFNGLDELETEASYNCMARKERSSGIXPROC', 'PFNGLISASYNCMARKERSGIXPROC',
'GL_SGIX_async_pixel', 'GL_SGIX_async_histogram', 'GL_INTEL_parallel_arrays',
'glVertexPointervINTEL', 'glNormalPointervINTEL', 'glColorPointervINTEL',
'glTexCoordPointervINTEL', 'PFNGLVERTEXPOINTERVINTELPROC',
'PFNGLNORMALPOINTERVINTELPROC', 'PFNGLCOLORPOINTERVINTELPROC',
'PFNGLTEXCOORDPOINTERVINTELPROC', 'GL_HP_occlusion_test',
'GL_EXT_pixel_transform', 'glPixelTransformParameteriEXT',
'glPixelTransformParameterfEXT', 'glPixelTransformParameterivEXT',
'glPixelTransformParameterfvEXT', 'PFNGLPIXELTRANSFORMPARAMETERIEXTPROC',
'PFNGLPIXELTRANSFORMPARAMETERFEXTPROC',
'PFNGLPIXELTRANSFORMPARAMETERIVEXTPROC',
'PFNGLPIXELTRANSFORMPARAMETERFVEXTPROC', 'GL_EXT_pixel_transform_color_table',
'GL_EXT_shared_texture_palette', 'GL_EXT_separate_specular_color',
'GL_EXT_secondary_color', 'glSecondaryColor3bEXT', 'glSecondaryColor3bvEXT',
'glSecondaryColor3dEXT', 'glSecondaryColor3dvEXT', 'glSecondaryColor3fEXT',
'glSecondaryColor3fvEXT', 'glSecondaryColor3iEXT', 'glSecondaryColor3ivEXT',
'glSecondaryColor3sEXT', 'glSecondaryColor3svEXT', 'glSecondaryColor3ubEXT',
'glSecondaryColor3ubvEXT', 'glSecondaryColor3uiEXT',
'glSecondaryColor3uivEXT', 'glSecondaryColor3usEXT',
'glSecondaryColor3usvEXT', 'glSecondaryColorPointerEXT',
'PFNGLSECONDARYCOLOR3BEXTPROC', 'PFNGLSECONDARYCOLOR3BVEXTPROC',
'PFNGLSECONDARYCOLOR3DEXTPROC', 'PFNGLSECONDARYCOLOR3DVEXTPROC',
'PFNGLSECONDARYCOLOR3FEXTPROC', 'PFNGLSECONDARYCOLOR3FVEXTPROC',
'PFNGLSECONDARYCOLOR3IEXTPROC', 'PFNGLSECONDARYCOLOR3IVEXTPROC',
'PFNGLSECONDARYCOLOR3SEXTPROC', 'PFNGLSECONDARYCOLOR3SVEXTPROC',
'PFNGLSECONDARYCOLOR3UBEXTPROC', 'PFNGLSECONDARYCOLOR3UBVEXTPROC',
'PFNGLSECONDARYCOLOR3UIEXTPROC', 'PFNGLSECONDARYCOLOR3UIVEXTPROC',
'PFNGLSECONDARYCOLOR3USEXTPROC', 'PFNGLSECONDARYCOLOR3USVEXTPROC',
'PFNGLSECONDARYCOLORPOINTEREXTPROC', 'GL_EXT_texture_perturb_normal',
'glTextureNormalEXT', 'PFNGLTEXTURENORMALEXTPROC', 'GL_EXT_multi_draw_arrays',
'glMultiDrawArraysEXT', 'glMultiDrawElementsEXT',
'PFNGLMULTIDRAWARRAYSEXTPROC', 'PFNGLMULTIDRAWELEMENTSEXTPROC',
'GL_EXT_fog_coord', 'glFogCoordfEXT', 'glFogCoordfvEXT', 'glFogCoorddEXT',
'glFogCoorddvEXT', 'glFogCoordPointerEXT', 'PFNGLFOGCOORDFEXTPROC',
'PFNGLFOGCOORDFVEXTPROC', 'PFNGLFOGCOORDDEXTPROC', 'PFNGLFOGCOORDDVEXTPROC',
'PFNGLFOGCOORDPOINTEREXTPROC', 'GL_REND_screen_coordinates',
'GL_EXT_coordinate_frame', 'glTangent3bEXT', 'glTangent3bvEXT',
'glTangent3dEXT', 'glTangent3dvEXT', 'glTangent3fEXT', 'glTangent3fvEXT',
'glTangent3iEXT', 'glTangent3ivEXT', 'glTangent3sEXT', 'glTangent3svEXT',
'glBinormal3bEXT', 'glBinormal3bvEXT', 'glBinormal3dEXT', 'glBinormal3dvEXT',
'glBinormal3fEXT', 'glBinormal3fvEXT', 'glBinormal3iEXT', 'glBinormal3ivEXT',
'glBinormal3sEXT', 'glBinormal3svEXT', 'glTangentPointerEXT',
'glBinormalPointerEXT', 'PFNGLTANGENT3BEXTPROC', 'PFNGLTANGENT3BVEXTPROC',
'PFNGLTANGENT3DEXTPROC', 'PFNGLTANGENT3DVEXTPROC', 'PFNGLTANGENT3FEXTPROC',
'PFNGLTANGENT3FVEXTPROC', 'PFNGLTANGENT3IEXTPROC', 'PFNGLTANGENT3IVEXTPROC',
'PFNGLTANGENT3SEXTPROC', 'PFNGLTANGENT3SVEXTPROC', 'PFNGLBINORMAL3BEXTPROC',
'PFNGLBINORMAL3BVEXTPROC', 'PFNGLBINORMAL3DEXTPROC',
'PFNGLBINORMAL3DVEXTPROC', 'PFNGLBINORMAL3FEXTPROC',
'PFNGLBINORMAL3FVEXTPROC', 'PFNGLBINORMAL3IEXTPROC',
'PFNGLBINORMAL3IVEXTPROC', 'PFNGLBINORMAL3SEXTPROC',
'PFNGLBINORMAL3SVEXTPROC', 'PFNGLTANGENTPOINTEREXTPROC',
'PFNGLBINORMALPOINTEREXTPROC', 'GL_EXT_texture_env_combine',
'GL_APPLE_specular_vector', 'GL_APPLE_transform_hint', 'GL_SGIX_fog_scale',
'GL_SUNX_constant_data', 'glFinishTextureSUNX', 'PFNGLFINISHTEXTURESUNXPROC',
'GL_SUN_global_alpha', 'glGlobalAlphaFactorbSUN', 'glGlobalAlphaFactorsSUN',
'glGlobalAlphaFactoriSUN', 'glGlobalAlphaFactorfSUN',
'glGlobalAlphaFactordSUN', 'glGlobalAlphaFactorubSUN',
'glGlobalAlphaFactorusSUN', 'glGlobalAlphaFactoruiSUN',
'PFNGLGLOBALALPHAFACTORBSUNPROC', 'PFNGLGLOBALALPHAFACTORSSUNPROC',
'PFNGLGLOBALALPHAFACTORISUNPROC', 'PFNGLGLOBALALPHAFACTORFSUNPROC',
'PFNGLGLOBALALPHAFACTORDSUNPROC', 'PFNGLGLOBALALPHAFACTORUBSUNPROC',
'PFNGLGLOBALALPHAFACTORUSSUNPROC', 'PFNGLGLOBALALPHAFACTORUISUNPROC',
'GL_SUN_triangle_list', 'glReplacementCodeuiSUN', 'glReplacementCodeusSUN',
'glReplacementCodeubSUN', 'glReplacementCodeuivSUN',
'glReplacementCodeusvSUN', 'glReplacementCodeubvSUN',
'glReplacementCodePointerSUN', 'PFNGLREPLACEMENTCODEUISUNPROC',
'PFNGLREPLACEMENTCODEUSSUNPROC', 'PFNGLREPLACEMENTCODEUBSUNPROC',
'PFNGLREPLACEMENTCODEUIVSUNPROC', 'PFNGLREPLACEMENTCODEUSVSUNPROC',
'PFNGLREPLACEMENTCODEUBVSUNPROC', 'PFNGLREPLACEMENTCODEPOINTERSUNPROC',
'GL_SUN_vertex', 'glColor4ubVertex2fSUN', 'glColor4ubVertex2fvSUN',
'glColor4ubVertex3fSUN', 'glColor4ubVertex3fvSUN', 'glColor3fVertex3fSUN',
'glColor3fVertex3fvSUN', 'glNormal3fVertex3fSUN', 'glNormal3fVertex3fvSUN',
'glColor4fNormal3fVertex3fSUN', 'glColor4fNormal3fVertex3fvSUN',
'glTexCoord2fVertex3fSUN', 'glTexCoord2fVertex3fvSUN',
'glTexCoord4fVertex4fSUN', 'glTexCoord4fVertex4fvSUN',
'glTexCoord2fColor4ubVertex3fSUN', 'glTexCoord2fColor4ubVertex3fvSUN',
'glTexCoord2fColor3fVertex3fSUN', 'glTexCoord2fColor3fVertex3fvSUN',
'glTexCoord2fNormal3fVertex3fSUN', 'glTexCoord2fNormal3fVertex3fvSUN',
'glTexCoord2fColor4fNormal3fVertex3fSUN',
'glTexCoord2fColor4fNormal3fVertex3fvSUN',
'glTexCoord4fColor4fNormal3fVertex4fSUN',
'glTexCoord4fColor4fNormal3fVertex4fvSUN', 'glReplacementCodeuiVertex3fSUN',
'glReplacementCodeuiVertex3fvSUN', 'glReplacementCodeuiColor4ubVertex3fSUN',
'glReplacementCodeuiColor4ubVertex3fvSUN',
'glReplacementCodeuiColor3fVertex3fSUN',
'glReplacementCodeuiColor3fVertex3fvSUN',
'glReplacementCodeuiNormal3fVertex3fSUN',
'glReplacementCodeuiNormal3fVertex3fvSUN',
'glReplacementCodeuiColor4fNormal3fVertex3fSUN',
'glReplacementCodeuiColor4fNormal3fVertex3fvSUN',
'glReplacementCodeuiTexCoord2fVertex3fSUN',
'glReplacementCodeuiTexCoord2fVertex3fvSUN',
'glReplacementCodeuiTexCoord2fNormal3fVertex3fSUN',
'glReplacementCodeuiTexCoord2fNormal3fVertex3fvSUN',
'glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fSUN',
'glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fvSUN',
'PFNGLCOLOR4UBVERTEX2FSUNPROC', 'PFNGLCOLOR4UBVERTEX2FVSUNPROC',
'PFNGLCOLOR4UBVERTEX3FSUNPROC', 'PFNGLCOLOR4UBVERTEX3FVSUNPROC',
'PFNGLCOLOR3FVERTEX3FSUNPROC', 'PFNGLCOLOR3FVERTEX3FVSUNPROC',
'PFNGLNORMAL3FVERTEX3FSUNPROC', 'PFNGLNORMAL3FVERTEX3FVSUNPROC',
'PFNGLCOLOR4FNORMAL3FVERTEX3FSUNPROC', 'PFNGLCOLOR4FNORMAL3FVERTEX3FVSUNPROC',
'PFNGLTEXCOORD2FVERTEX3FSUNPROC', 'PFNGLTEXCOORD2FVERTEX3FVSUNPROC',
'PFNGLTEXCOORD4FVERTEX4FSUNPROC', 'PFNGLTEXCOORD4FVERTEX4FVSUNPROC',
'PFNGLTEXCOORD2FCOLOR4UBVERTEX3FSUNPROC',
'PFNGLTEXCOORD2FCOLOR4UBVERTEX3FVSUNPROC',
'PFNGLTEXCOORD2FCOLOR3FVERTEX3FSUNPROC',
'PFNGLTEXCOORD2FCOLOR3FVERTEX3FVSUNPROC',
'PFNGLTEXCOORD2FNORMAL3FVERTEX3FSUNPROC',
'PFNGLTEXCOORD2FNORMAL3FVERTEX3FVSUNPROC',
'PFNGLTEXCOORD2FCOLOR4FNORMAL3FVERTEX3FSUNPROC',
'PFNGLTEXCOORD2FCOLOR4FNORMAL3FVERTEX3FVSUNPROC',
'PFNGLTEXCOORD4FCOLOR4FNORMAL3FVERTEX4FSUNPROC',
'PFNGLTEXCOORD4FCOLOR4FNORMAL3FVERTEX4FVSUNPROC',
'PFNGLREPLACEMENTCODEUIVERTEX3FSUNPROC',
'PFNGLREPLACEMENTCODEUIVERTEX3FVSUNPROC',
'PFNGLREPLACEMENTCODEUICOLOR4UBVERTEX3FSUNPROC',
'PFNGLREPLACEMENTCODEUICOLOR4UBVERTEX3FVSUNPROC',
'PFNGLREPLACEMENTCODEUICOLOR3FVERTEX3FSUNPROC',
'PFNGLREPLACEMENTCODEUICOLOR3FVERTEX3FVSUNPROC',
'PFNGLREPLACEMENTCODEUINORMAL3FVERTEX3FSUNPROC',
'PFNGLREPLACEMENTCODEUINORMAL3FVERTEX3FVSUNPROC',
'PFNGLREPLACEMENTCODEUICOLOR4FNORMAL3FVERTEX3FSUNPROC',
'PFNGLREPLACEMENTCODEUICOLOR4FNORMAL3FVERTEX3FVSUNPROC',
'PFNGLREPLACEMENTCODEUITEXCOORD2FVERTEX3FSUNPROC',
'PFNGLREPLACEMENTCODEUITEXCOORD2FVERTEX3FVSUNPROC',
'PFNGLREPLACEMENTCODEUITEXCOORD2FNORMAL3FVERTEX3FSUNPROC',
'PFNGLREPLACEMENTCODEUITEXCOORD2FNORMAL3FVERTEX3FVSUNPROC',
'PFNGLREPLACEMENTCODEUITEXCOORD2FCOLOR4FNORMAL3FVERTEX3FSUNPROC',
'PFNGLREPLACEMENTCODEUITEXCOORD2FCOLOR4FNORMAL3FVERTEX3FVSUNPROC',
'GL_EXT_blend_func_separate', 'glBlendFuncSeparateEXT',
'PFNGLBLENDFUNCSEPARATEEXTPROC', 'GL_INGR_blend_func_separate',
'glBlendFuncSeparateINGR', 'PFNGLBLENDFUNCSEPARATEINGRPROC',
'GL_INGR_color_clamp', 'GL_INGR_interlace_read', 'GL_EXT_stencil_wrap',
'GL_EXT_422_pixels', 'GL_NV_texgen_reflection',
'GL_SUN_convolution_border_modes', 'GL_EXT_texture_env_add',
'GL_EXT_texture_lod_bias', 'GL_EXT_texture_filter_anisotropic',
'GL_EXT_vertex_weighting', 'glVertexWeightfEXT', 'glVertexWeightfvEXT',
'glVertexWeightPointerEXT', 'PFNGLVERTEXWEIGHTFEXTPROC',
'PFNGLVERTEXWEIGHTFVEXTPROC', 'PFNGLVERTEXWEIGHTPOINTEREXTPROC',
'GL_NV_light_max_exponent', 'GL_NV_vertex_array_range',
'glFlushVertexArrayRangeNV', 'glVertexArrayRangeNV',
'PFNGLFLUSHVERTEXARRAYRANGENVPROC', 'PFNGLVERTEXARRAYRANGENVPROC',
'GL_NV_register_combiners', 'glCombinerParameterfvNV',
'glCombinerParameterfNV', 'glCombinerParameterivNV', 'glCombinerParameteriNV',
'glCombinerInputNV', 'glCombinerOutputNV', 'glFinalCombinerInputNV',
'glGetCombinerInputParameterfvNV', 'glGetCombinerInputParameterivNV',
'glGetCombinerOutputParameterfvNV', 'glGetCombinerOutputParameterivNV',
'glGetFinalCombinerInputParameterfvNV',
'glGetFinalCombinerInputParameterivNV', 'PFNGLCOMBINERPARAMETERFVNVPROC',
'PFNGLCOMBINERPARAMETERFNVPROC', 'PFNGLCOMBINERPARAMETERIVNVPROC',
'PFNGLCOMBINERPARAMETERINVPROC', 'PFNGLCOMBINERINPUTNVPROC',
'PFNGLCOMBINEROUTPUTNVPROC', 'PFNGLFINALCOMBINERINPUTNVPROC',
'PFNGLGETCOMBINERINPUTPARAMETERFVNVPROC',
'PFNGLGETCOMBINERINPUTPARAMETERIVNVPROC',
'PFNGLGETCOMBINEROUTPUTPARAMETERFVNVPROC',
'PFNGLGETCOMBINEROUTPUTPARAMETERIVNVPROC',
'PFNGLGETFINALCOMBINERINPUTPARAMETERFVNVPROC',
'PFNGLGETFINALCOMBINERINPUTPARAMETERIVNVPROC', 'GL_NV_fog_distance',
'GL_NV_texgen_emboss', 'GL_NV_blend_square', 'GL_NV_texture_env_combine4',
'GL_MESA_resize_buffers', 'glResizeBuffersMESA', 'PFNGLRESIZEBUFFERSMESAPROC',
'GL_MESA_window_pos', 'glWindowPos2dMESA', 'glWindowPos2dvMESA',
'glWindowPos2fMESA', 'glWindowPos2fvMESA', 'glWindowPos2iMESA',
'glWindowPos2ivMESA', 'glWindowPos2sMESA', 'glWindowPos2svMESA',
'glWindowPos3dMESA', 'glWindowPos3dvMESA', 'glWindowPos3fMESA',
'glWindowPos3fvMESA', 'glWindowPos3iMESA', 'glWindowPos3ivMESA',
'glWindowPos3sMESA', 'glWindowPos3svMESA', 'glWindowPos4dMESA',
'glWindowPos4dvMESA', 'glWindowPos4fMESA', 'glWindowPos4fvMESA',
'glWindowPos4iMESA', 'glWindowPos4ivMESA', 'glWindowPos4sMESA',
'glWindowPos4svMESA', 'PFNGLWINDOWPOS2DMESAPROC', 'PFNGLWINDOWPOS2DVMESAPROC',
'PFNGLWINDOWPOS2FMESAPROC', 'PFNGLWINDOWPOS2FVMESAPROC',
'PFNGLWINDOWPOS2IMESAPROC', 'PFNGLWINDOWPOS2IVMESAPROC',
'PFNGLWINDOWPOS2SMESAPROC', 'PFNGLWINDOWPOS2SVMESAPROC',
'PFNGLWINDOWPOS3DMESAPROC', 'PFNGLWINDOWPOS3DVMESAPROC',
'PFNGLWINDOWPOS3FMESAPROC', 'PFNGLWINDOWPOS3FVMESAPROC',
'PFNGLWINDOWPOS3IMESAPROC', 'PFNGLWINDOWPOS3IVMESAPROC',
'PFNGLWINDOWPOS3SMESAPROC', 'PFNGLWINDOWPOS3SVMESAPROC',
'PFNGLWINDOWPOS4DMESAPROC', 'PFNGLWINDOWPOS4DVMESAPROC',
'PFNGLWINDOWPOS4FMESAPROC', 'PFNGLWINDOWPOS4FVMESAPROC',
'PFNGLWINDOWPOS4IMESAPROC', 'PFNGLWINDOWPOS4IVMESAPROC',
'PFNGLWINDOWPOS4SMESAPROC', 'PFNGLWINDOWPOS4SVMESAPROC',
'GL_EXT_texture_compression_s3tc', 'GL_IBM_cull_vertex',
'GL_IBM_multimode_draw_arrays', 'glMultiModeDrawArraysIBM',
'glMultiModeDrawElementsIBM', 'PFNGLMULTIMODEDRAWARRAYSIBMPROC',
'PFNGLMULTIMODEDRAWELEMENTSIBMPROC', 'GL_IBM_vertex_array_lists',
'glColorPointerListIBM', 'glSecondaryColorPointerListIBM',
'glEdgeFlagPointerListIBM', 'glFogCoordPointerListIBM',
'glIndexPointerListIBM', 'glNormalPointerListIBM', 'glTexCoordPointerListIBM',
'glVertexPointerListIBM', 'PFNGLCOLORPOINTERLISTIBMPROC',
'PFNGLSECONDARYCOLORPOINTERLISTIBMPROC', 'PFNGLEDGEFLAGPOINTERLISTIBMPROC',
'PFNGLFOGCOORDPOINTERLISTIBMPROC', 'PFNGLINDEXPOINTERLISTIBMPROC',
'PFNGLNORMALPOINTERLISTIBMPROC', 'PFNGLTEXCOORDPOINTERLISTIBMPROC',
'PFNGLVERTEXPOINTERLISTIBMPROC', 'GL_SGIX_subsample', 'GL_SGIX_ycrcba',
'GL_SGIX_ycrcb_subsample', 'GL_SGIX_depth_pass_instrument',
'GL_3DFX_texture_compression_FXT1', 'GL_3DFX_multisample', 'GL_3DFX_tbuffer',
'glTbufferMask3DFX', 'PFNGLTBUFFERMASK3DFXPROC', 'GL_EXT_multisample',
'glSampleMaskEXT', 'glSamplePatternEXT', 'PFNGLSAMPLEMASKEXTPROC',
'PFNGLSAMPLEPATTERNEXTPROC', 'GL_SGIX_vertex_preclip',
'GL_SGIX_convolution_accuracy', 'GL_SGIX_resample',
'GL_SGIS_point_line_texgen', 'GL_SGIS_texture_color_mask',
'glTextureColorMaskSGIS', 'PFNGLTEXTURECOLORMASKSGISPROC',
'GL_SGIX_igloo_interface', 'glIglooInterfaceSGIX',
'PFNGLIGLOOINTERFACESGIXPROC', 'GL_EXT_texture_env_dot3',
'GL_ATI_texture_mirror_once', 'GL_NV_fence', 'glDeleteFencesNV',
'glGenFencesNV', 'glIsFenceNV', 'glTestFenceNV', 'glGetFenceivNV',
'glFinishFenceNV', 'glSetFenceNV', 'PFNGLDELETEFENCESNVPROC',
'PFNGLGENFENCESNVPROC', 'PFNGLISFENCENVPROC', 'PFNGLTESTFENCENVPROC',
'PFNGLGETFENCEIVNVPROC', 'PFNGLFINISHFENCENVPROC', 'PFNGLSETFENCENVPROC',
'GL_NV_evaluators', 'glMapControlPointsNV', 'glMapParameterivNV',
'glMapParameterfvNV', 'glGetMapControlPointsNV', 'glGetMapParameterivNV',
'glGetMapParameterfvNV', 'glGetMapAttribParameterivNV',
'glGetMapAttribParameterfvNV', 'glEvalMapsNV', 'PFNGLMAPCONTROLPOINTSNVPROC',
'PFNGLMAPPARAMETERIVNVPROC', 'PFNGLMAPPARAMETERFVNVPROC',
'PFNGLGETMAPCONTROLPOINTSNVPROC', 'PFNGLGETMAPPARAMETERIVNVPROC',
'PFNGLGETMAPPARAMETERFVNVPROC', 'PFNGLGETMAPATTRIBPARAMETERIVNVPROC',
'PFNGLGETMAPATTRIBPARAMETERFVNVPROC', 'PFNGLEVALMAPSNVPROC',
'GL_NV_packed_depth_stencil', 'GL_NV_register_combiners2',
'glCombinerStageParameterfvNV', 'glGetCombinerStageParameterfvNV',
'PFNGLCOMBINERSTAGEPARAMETERFVNVPROC',
'PFNGLGETCOMBINERSTAGEPARAMETERFVNVPROC', 'GL_NV_texture_compression_vtc',
'GL_NV_texture_rectangle', 'GL_NV_texture_shader', 'GL_NV_texture_shader2',
'GL_NV_vertex_array_range2', 'GL_NV_vertex_program',
'glAreProgramsResidentNV', 'glBindProgramNV', 'glDeleteProgramsNV',
'glExecuteProgramNV', 'glGenProgramsNV', 'glGetProgramParameterdvNV',
'glGetProgramParameterfvNV', 'glGetProgramivNV', 'glGetProgramStringNV',
'glGetTrackMatrixivNV', 'glGetVertexAttribdvNV', 'glGetVertexAttribfvNV',
'glGetVertexAttribivNV', 'glGetVertexAttribPointervNV', 'glIsProgramNV',
'glLoadProgramNV', 'glProgramParameter4dNV', 'glProgramParameter4dvNV',
'glProgramParameter4fNV', 'glProgramParameter4fvNV',
'glProgramParameters4dvNV', 'glProgramParameters4fvNV',
'glRequestResidentProgramsNV', 'glTrackMatrixNV', 'glVertexAttribPointerNV',
'glVertexAttrib1dNV', 'glVertexAttrib1dvNV', 'glVertexAttrib1fNV',
'glVertexAttrib1fvNV', 'glVertexAttrib1sNV', 'glVertexAttrib1svNV',
'glVertexAttrib2dNV', 'glVertexAttrib2dvNV', 'glVertexAttrib2fNV',
'glVertexAttrib2fvNV', 'glVertexAttrib2sNV', 'glVertexAttrib2svNV',
'glVertexAttrib3dNV', 'glVertexAttrib3dvNV', 'glVertexAttrib3fNV',
'glVertexAttrib3fvNV', 'glVertexAttrib3sNV', 'glVertexAttrib3svNV',
'glVertexAttrib4dNV', 'glVertexAttrib4dvNV', 'glVertexAttrib4fNV',
'glVertexAttrib4fvNV', 'glVertexAttrib4sNV', 'glVertexAttrib4svNV',
'glVertexAttrib4ubNV', 'glVertexAttrib4ubvNV', 'glVertexAttribs1dvNV',
'glVertexAttribs1fvNV', 'glVertexAttribs1svNV', 'glVertexAttribs2dvNV',
'glVertexAttribs2fvNV', 'glVertexAttribs2svNV', 'glVertexAttribs3dvNV',
'glVertexAttribs3fvNV', 'glVertexAttribs3svNV', 'glVertexAttribs4dvNV',
'glVertexAttribs4fvNV', 'glVertexAttribs4svNV', 'glVertexAttribs4ubvNV',
'PFNGLAREPROGRAMSRESIDENTNVPROC', 'PFNGLBINDPROGRAMNVPROC',
'PFNGLDELETEPROGRAMSNVPROC', 'PFNGLEXECUTEPROGRAMNVPROC',
'PFNGLGENPROGRAMSNVPROC', 'PFNGLGETPROGRAMPARAMETERDVNVPROC',
'PFNGLGETPROGRAMPARAMETERFVNVPROC', 'PFNGLGETPROGRAMIVNVPROC',
'PFNGLGETPROGRAMSTRINGNVPROC', 'PFNGLGETTRACKMATRIXIVNVPROC',
'PFNGLGETVERTEXATTRIBDVNVPROC', 'PFNGLGETVERTEXATTRIBFVNVPROC',
'PFNGLGETVERTEXATTRIBIVNVPROC', 'PFNGLGETVERTEXATTRIBPOINTERVNVPROC',
'PFNGLISPROGRAMNVPROC', 'PFNGLLOADPROGRAMNVPROC',
'PFNGLPROGRAMPARAMETER4DNVPROC', 'PFNGLPROGRAMPARAMETER4DVNVPROC',
'PFNGLPROGRAMPARAMETER4FNVPROC', 'PFNGLPROGRAMPARAMETER4FVNVPROC',
'PFNGLPROGRAMPARAMETERS4DVNVPROC', 'PFNGLPROGRAMPARAMETERS4FVNVPROC',
'PFNGLREQUESTRESIDENTPROGRAMSNVPROC', 'PFNGLTRACKMATRIXNVPROC',
'PFNGLVERTEXATTRIBPOINTERNVPROC', 'PFNGLVERTEXATTRIB1DNVPROC',
'PFNGLVERTEXATTRIB1DVNVPROC', 'PFNGLVERTEXATTRIB1FNVPROC',
'PFNGLVERTEXATTRIB1FVNVPROC', 'PFNGLVERTEXATTRIB1SNVPROC',
'PFNGLVERTEXATTRIB1SVNVPROC', 'PFNGLVERTEXATTRIB2DNVPROC',
'PFNGLVERTEXATTRIB2DVNVPROC', 'PFNGLVERTEXATTRIB2FNVPROC',
'PFNGLVERTEXATTRIB2FVNVPROC', 'PFNGLVERTEXATTRIB2SNVPROC',
'PFNGLVERTEXATTRIB2SVNVPROC', 'PFNGLVERTEXATTRIB3DNVPROC',
'PFNGLVERTEXATTRIB3DVNVPROC', 'PFNGLVERTEXATTRIB3FNVPROC',
'PFNGLVERTEXATTRIB3FVNVPROC', 'PFNGLVERTEXATTRIB3SNVPROC',
'PFNGLVERTEXATTRIB3SVNVPROC', 'PFNGLVERTEXATTRIB4DNVPROC',
'PFNGLVERTEXATTRIB4DVNVPROC', 'PFNGLVERTEXATTRIB4FNVPROC',
'PFNGLVERTEXATTRIB4FVNVPROC', 'PFNGLVERTEXATTRIB4SNVPROC',
'PFNGLVERTEXATTRIB4SVNVPROC', 'PFNGLVERTEXATTRIB4UBNVPROC',
'PFNGLVERTEXATTRIB4UBVNVPROC', 'PFNGLVERTEXATTRIBS1DVNVPROC',
'PFNGLVERTEXATTRIBS1FVNVPROC', 'PFNGLVERTEXATTRIBS1SVNVPROC',
'PFNGLVERTEXATTRIBS2DVNVPROC', 'PFNGLVERTEXATTRIBS2FVNVPROC',
'PFNGLVERTEXATTRIBS2SVNVPROC', 'PFNGLVERTEXATTRIBS3DVNVPROC',
'PFNGLVERTEXATTRIBS3FVNVPROC', 'PFNGLVERTEXATTRIBS3SVNVPROC',
'PFNGLVERTEXATTRIBS4DVNVPROC', 'PFNGLVERTEXATTRIBS4FVNVPROC',
'PFNGLVERTEXATTRIBS4SVNVPROC', 'PFNGLVERTEXATTRIBS4UBVNVPROC',
'GL_SGIX_texture_coordinate_clamp', 'GL_SGIX_scalebias_hint',
'GL_OML_interlace', 'GL_OML_subsample', 'GL_OML_resample',
'GL_NV_copy_depth_to_color', 'GL_ATI_envmap_bumpmap',
'glTexBumpParameterivATI', 'glTexBumpParameterfvATI',
'glGetTexBumpParameterivATI', 'glGetTexBumpParameterfvATI',
'PFNGLTEXBUMPPARAMETERIVATIPROC', 'PFNGLTEXBUMPPARAMETERFVATIPROC',
'PFNGLGETTEXBUMPPARAMETERIVATIPROC', 'PFNGLGETTEXBUMPPARAMETERFVATIPROC',
'GL_ATI_fragment_shader', 'glGenFragmentShadersATI',
'glBindFragmentShaderATI', 'glDeleteFragmentShaderATI',
'glBeginFragmentShaderATI', 'glEndFragmentShaderATI', 'glPassTexCoordATI',
'glSampleMapATI', 'glColorFragmentOp1ATI', 'glColorFragmentOp2ATI',
'glColorFragmentOp3ATI', 'glAlphaFragmentOp1ATI', 'glAlphaFragmentOp2ATI',
'glAlphaFragmentOp3ATI', 'glSetFragmentShaderConstantATI',
'PFNGLGENFRAGMENTSHADERSATIPROC', 'PFNGLBINDFRAGMENTSHADERATIPROC',
'PFNGLDELETEFRAGMENTSHADERATIPROC', 'PFNGLBEGINFRAGMENTSHADERATIPROC',
'PFNGLENDFRAGMENTSHADERATIPROC', 'PFNGLPASSTEXCOORDATIPROC',
'PFNGLSAMPLEMAPATIPROC', 'PFNGLCOLORFRAGMENTOP1ATIPROC',
'PFNGLCOLORFRAGMENTOP2ATIPROC', 'PFNGLCOLORFRAGMENTOP3ATIPROC',
'PFNGLALPHAFRAGMENTOP1ATIPROC', 'PFNGLALPHAFRAGMENTOP2ATIPROC',
'PFNGLALPHAFRAGMENTOP3ATIPROC', 'PFNGLSETFRAGMENTSHADERCONSTANTATIPROC',
'GL_ATI_pn_triangles', 'glPNTrianglesiATI', 'glPNTrianglesfATI',
'PFNGLPNTRIANGLESIATIPROC', 'PFNGLPNTRIANGLESFATIPROC',
'GL_ATI_vertex_array_object', 'glNewObjectBufferATI', 'glIsObjectBufferATI',
'glUpdateObjectBufferATI', 'glGetObjectBufferfvATI', 'glGetObjectBufferivATI',
'glFreeObjectBufferATI', 'glArrayObjectATI', 'glGetArrayObjectfvATI',
'glGetArrayObjectivATI', 'glVariantArrayObjectATI',
'glGetVariantArrayObjectfvATI', 'glGetVariantArrayObjectivATI',
'PFNGLNEWOBJECTBUFFERATIPROC', 'PFNGLISOBJECTBUFFERATIPROC',
'PFNGLUPDATEOBJECTBUFFERATIPROC', 'PFNGLGETOBJECTBUFFERFVATIPROC',
'PFNGLGETOBJECTBUFFERIVATIPROC', 'PFNGLFREEOBJECTBUFFERATIPROC',
'PFNGLARRAYOBJECTATIPROC', 'PFNGLGETARRAYOBJECTFVATIPROC',
'PFNGLGETARRAYOBJECTIVATIPROC', 'PFNGLVARIANTARRAYOBJECTATIPROC',
'PFNGLGETVARIANTARRAYOBJECTFVATIPROC', 'PFNGLGETVARIANTARRAYOBJECTIVATIPROC',
'GL_EXT_vertex_shader', 'glBeginVertexShaderEXT', 'glEndVertexShaderEXT',
'glBindVertexShaderEXT', 'glGenVertexShadersEXT', 'glDeleteVertexShaderEXT',
'glShaderOp1EXT', 'glShaderOp2EXT', 'glShaderOp3EXT', 'glSwizzleEXT',
'glWriteMaskEXT', 'glInsertComponentEXT', 'glExtractComponentEXT',
'glGenSymbolsEXT', 'glSetInvariantEXT', 'glSetLocalConstantEXT',
'glVariantbvEXT', 'glVariantsvEXT', 'glVariantivEXT', 'glVariantfvEXT',
'glVariantdvEXT', 'glVariantubvEXT', 'glVariantusvEXT', 'glVariantuivEXT',
'glVariantPointerEXT', 'glEnableVariantClientStateEXT',
'glDisableVariantClientStateEXT', 'glBindLightParameterEXT',
'glBindMaterialParameterEXT', 'glBindTexGenParameterEXT',
'glBindTextureUnitParameterEXT', 'glBindParameterEXT',
'glIsVariantEnabledEXT', 'glGetVariantBooleanvEXT', 'glGetVariantIntegervEXT',
'glGetVariantFloatvEXT', 'glGetVariantPointervEXT',
'glGetInvariantBooleanvEXT', 'glGetInvariantIntegervEXT',
'glGetInvariantFloatvEXT', 'glGetLocalConstantBooleanvEXT',
'glGetLocalConstantIntegervEXT', 'glGetLocalConstantFloatvEXT',
'PFNGLBEGINVERTEXSHADEREXTPROC', 'PFNGLENDVERTEXSHADEREXTPROC',
'PFNGLBINDVERTEXSHADEREXTPROC', 'PFNGLGENVERTEXSHADERSEXTPROC',
'PFNGLDELETEVERTEXSHADEREXTPROC', 'PFNGLSHADEROP1EXTPROC',
'PFNGLSHADEROP2EXTPROC', 'PFNGLSHADEROP3EXTPROC', 'PFNGLSWIZZLEEXTPROC',
'PFNGLWRITEMASKEXTPROC', 'PFNGLINSERTCOMPONENTEXTPROC',
'PFNGLEXTRACTCOMPONENTEXTPROC', 'PFNGLGENSYMBOLSEXTPROC',
'PFNGLSETINVARIANTEXTPROC', 'PFNGLSETLOCALCONSTANTEXTPROC',
'PFNGLVARIANTBVEXTPROC', 'PFNGLVARIANTSVEXTPROC', 'PFNGLVARIANTIVEXTPROC',
'PFNGLVARIANTFVEXTPROC', 'PFNGLVARIANTDVEXTPROC', 'PFNGLVARIANTUBVEXTPROC',
'PFNGLVARIANTUSVEXTPROC', 'PFNGLVARIANTUIVEXTPROC',
'PFNGLVARIANTPOINTEREXTPROC', 'PFNGLENABLEVARIANTCLIENTSTATEEXTPROC',
'PFNGLDISABLEVARIANTCLIENTSTATEEXTPROC', 'PFNGLBINDLIGHTPARAMETEREXTPROC',
'PFNGLBINDMATERIALPARAMETEREXTPROC', 'PFNGLBINDTEXGENPARAMETEREXTPROC',
'PFNGLBINDTEXTUREUNITPARAMETEREXTPROC', 'PFNGLBINDPARAMETEREXTPROC',
'PFNGLISVARIANTENABLEDEXTPROC', 'PFNGLGETVARIANTBOOLEANVEXTPROC',
'PFNGLGETVARIANTINTEGERVEXTPROC', 'PFNGLGETVARIANTFLOATVEXTPROC',
'PFNGLGETVARIANTPOINTERVEXTPROC', 'PFNGLGETINVARIANTBOOLEANVEXTPROC',
'PFNGLGETINVARIANTINTEGERVEXTPROC', 'PFNGLGETINVARIANTFLOATVEXTPROC',
'PFNGLGETLOCALCONSTANTBOOLEANVEXTPROC',
'PFNGLGETLOCALCONSTANTINTEGERVEXTPROC', 'PFNGLGETLOCALCONSTANTFLOATVEXTPROC',
'GL_ATI_vertex_streams', 'glVertexStream1sATI', 'glVertexStream1svATI',
'glVertexStream1iATI', 'glVertexStream1ivATI', 'glVertexStream1fATI',
'glVertexStream1fvATI', 'glVertexStream1dATI', 'glVertexStream1dvATI',
'glVertexStream2sATI', 'glVertexStream2svATI', 'glVertexStream2iATI',
'glVertexStream2ivATI', 'glVertexStream2fATI', 'glVertexStream2fvATI',
'glVertexStream2dATI', 'glVertexStream2dvATI', 'glVertexStream3sATI',
'glVertexStream3svATI', 'glVertexStream3iATI', 'glVertexStream3ivATI',
'glVertexStream3fATI', 'glVertexStream3fvATI', 'glVertexStream3dATI',
'glVertexStream3dvATI', 'glVertexStream4sATI', 'glVertexStream4svATI',
'glVertexStream4iATI', 'glVertexStream4ivATI', 'glVertexStream4fATI',
'glVertexStream4fvATI', 'glVertexStream4dATI', 'glVertexStream4dvATI',
'glNormalStream3bATI', 'glNormalStream3bvATI', 'glNormalStream3sATI',
'glNormalStream3svATI', 'glNormalStream3iATI', 'glNormalStream3ivATI',
'glNormalStream3fATI', 'glNormalStream3fvATI', 'glNormalStream3dATI',
'glNormalStream3dvATI', 'glClientActiveVertexStreamATI',
'glVertexBlendEnviATI', 'glVertexBlendEnvfATI', 'PFNGLVERTEXSTREAM1SATIPROC',
'PFNGLVERTEXSTREAM1SVATIPROC', 'PFNGLVERTEXSTREAM1IATIPROC',
'PFNGLVERTEXSTREAM1IVATIPROC', 'PFNGLVERTEXSTREAM1FATIPROC',
'PFNGLVERTEXSTREAM1FVATIPROC', 'PFNGLVERTEXSTREAM1DATIPROC',
'PFNGLVERTEXSTREAM1DVATIPROC', 'PFNGLVERTEXSTREAM2SATIPROC',
'PFNGLVERTEXSTREAM2SVATIPROC', 'PFNGLVERTEXSTREAM2IATIPROC',
'PFNGLVERTEXSTREAM2IVATIPROC', 'PFNGLVERTEXSTREAM2FATIPROC',
'PFNGLVERTEXSTREAM2FVATIPROC', 'PFNGLVERTEXSTREAM2DATIPROC',
'PFNGLVERTEXSTREAM2DVATIPROC', 'PFNGLVERTEXSTREAM3SATIPROC',
'PFNGLVERTEXSTREAM3SVATIPROC', 'PFNGLVERTEXSTREAM3IATIPROC',
'PFNGLVERTEXSTREAM3IVATIPROC', 'PFNGLVERTEXSTREAM3FATIPROC',
'PFNGLVERTEXSTREAM3FVATIPROC', 'PFNGLVERTEXSTREAM3DATIPROC',
'PFNGLVERTEXSTREAM3DVATIPROC', 'PFNGLVERTEXSTREAM4SATIPROC',
'PFNGLVERTEXSTREAM4SVATIPROC', 'PFNGLVERTEXSTREAM4IATIPROC',
'PFNGLVERTEXSTREAM4IVATIPROC', 'PFNGLVERTEXSTREAM4FATIPROC',
'PFNGLVERTEXSTREAM4FVATIPROC', 'PFNGLVERTEXSTREAM4DATIPROC',
'PFNGLVERTEXSTREAM4DVATIPROC', 'PFNGLNORMALSTREAM3BATIPROC',
'PFNGLNORMALSTREAM3BVATIPROC', 'PFNGLNORMALSTREAM3SATIPROC',
'PFNGLNORMALSTREAM3SVATIPROC', 'PFNGLNORMALSTREAM3IATIPROC',
'PFNGLNORMALSTREAM3IVATIPROC', 'PFNGLNORMALSTREAM3FATIPROC',
'PFNGLNORMALSTREAM3FVATIPROC', 'PFNGLNORMALSTREAM3DATIPROC',
'PFNGLNORMALSTREAM3DVATIPROC', 'PFNGLCLIENTACTIVEVERTEXSTREAMATIPROC',
'PFNGLVERTEXBLENDENVIATIPROC', 'PFNGLVERTEXBLENDENVFATIPROC',
'GL_ATI_element_array', 'glElementPointerATI', 'glDrawElementArrayATI',
'glDrawRangeElementArrayATI', 'PFNGLELEMENTPOINTERATIPROC',
'PFNGLDRAWELEMENTARRAYATIPROC', 'PFNGLDRAWRANGEELEMENTARRAYATIPROC',
'GL_SUN_mesh_array', 'glDrawMeshArraysSUN', 'PFNGLDRAWMESHARRAYSSUNPROC',
'GL_SUN_slice_accum', 'GL_NV_multisample_filter_hint', 'GL_NV_depth_clamp',
'GL_NV_occlusion_query', 'glGenOcclusionQueriesNV',
'glDeleteOcclusionQueriesNV', 'glIsOcclusionQueryNV',
'glBeginOcclusionQueryNV', 'glEndOcclusionQueryNV', 'glGetOcclusionQueryivNV',
'glGetOcclusionQueryuivNV', 'PFNGLGENOCCLUSIONQUERIESNVPROC',
'PFNGLDELETEOCCLUSIONQUERIESNVPROC', 'PFNGLISOCCLUSIONQUERYNVPROC',
'PFNGLBEGINOCCLUSIONQUERYNVPROC', 'PFNGLENDOCCLUSIONQUERYNVPROC',
'PFNGLGETOCCLUSIONQUERYIVNVPROC', 'PFNGLGETOCCLUSIONQUERYUIVNVPROC',
'GL_NV_point_sprite', 'glPointParameteriNV', 'glPointParameterivNV',
'PFNGLPOINTPARAMETERINVPROC', 'PFNGLPOINTPARAMETERIVNVPROC',
'GL_NV_texture_shader3', 'GL_NV_vertex_program1_1', 'GL_EXT_shadow_funcs',
'GL_EXT_stencil_two_side', 'glActiveStencilFaceEXT',
'PFNGLACTIVESTENCILFACEEXTPROC', 'GL_ATI_text_fragment_shader',
'GL_APPLE_client_storage', 'GL_APPLE_element_array', 'glElementPointerAPPLE',
'glDrawElementArrayAPPLE', 'glDrawRangeElementArrayAPPLE',
'glMultiDrawElementArrayAPPLE', 'glMultiDrawRangeElementArrayAPPLE',
'PFNGLELEMENTPOINTERAPPLEPROC', 'PFNGLDRAWELEMENTARRAYAPPLEPROC',
'PFNGLDRAWRANGEELEMENTARRAYAPPLEPROC', 'PFNGLMULTIDRAWELEMENTARRAYAPPLEPROC',
'PFNGLMULTIDRAWRANGEELEMENTARRAYAPPLEPROC', 'GL_APPLE_fence',
'glGenFencesAPPLE', 'glDeleteFencesAPPLE', 'glSetFenceAPPLE',
'glIsFenceAPPLE', 'glTestFenceAPPLE', 'glFinishFenceAPPLE',
'glTestObjectAPPLE', 'glFinishObjectAPPLE', 'PFNGLGENFENCESAPPLEPROC',
'PFNGLDELETEFENCESAPPLEPROC', 'PFNGLSETFENCEAPPLEPROC',
'PFNGLISFENCEAPPLEPROC', 'PFNGLTESTFENCEAPPLEPROC',
'PFNGLFINISHFENCEAPPLEPROC', 'PFNGLTESTOBJECTAPPLEPROC',
'PFNGLFINISHOBJECTAPPLEPROC', 'GL_APPLE_vertex_array_object',
'glBindVertexArrayAPPLE', 'glDeleteVertexArraysAPPLE',
'glGenVertexArraysAPPLE', 'glIsVertexArrayAPPLE',
'PFNGLBINDVERTEXARRAYAPPLEPROC', 'PFNGLDELETEVERTEXARRAYSAPPLEPROC',
'PFNGLGENVERTEXARRAYSAPPLEPROC', 'PFNGLISVERTEXARRAYAPPLEPROC',
'GL_APPLE_vertex_array_range', 'glVertexArrayRangeAPPLE',
'glFlushVertexArrayRangeAPPLE', 'glVertexArrayParameteriAPPLE',
'PFNGLVERTEXARRAYRANGEAPPLEPROC', 'PFNGLFLUSHVERTEXARRAYRANGEAPPLEPROC',
'PFNGLVERTEXARRAYPARAMETERIAPPLEPROC', 'GL_APPLE_ycbcr_422', 'GL_S3_s3tc',
'GL_ATI_draw_buffers', 'glDrawBuffersATI', 'PFNGLDRAWBUFFERSATIPROC',
'GL_ATI_pixel_format_float', 'GL_ATI_texture_env_combine3',
'GL_ATI_texture_float', 'GL_NV_float_buffer', 'GL_NV_fragment_program',
'glProgramNamedParameter4fNV', 'glProgramNamedParameter4dNV',
'glProgramNamedParameter4fvNV', 'glProgramNamedParameter4dvNV',
'glGetProgramNamedParameterfvNV', 'glGetProgramNamedParameterdvNV',
'PFNGLPROGRAMNAMEDPARAMETER4FNVPROC', 'PFNGLPROGRAMNAMEDPARAMETER4DNVPROC',
'PFNGLPROGRAMNAMEDPARAMETER4FVNVPROC', 'PFNGLPROGRAMNAMEDPARAMETER4DVNVPROC',
'PFNGLGETPROGRAMNAMEDPARAMETERFVNVPROC',
'PFNGLGETPROGRAMNAMEDPARAMETERDVNVPROC', 'GL_NV_half_float', 'glVertex2hNV',
'glVertex2hvNV', 'glVertex3hNV', 'glVertex3hvNV', 'glVertex4hNV',
'glVertex4hvNV', 'glNormal3hNV', 'glNormal3hvNV', 'glColor3hNV',
'glColor3hvNV', 'glColor4hNV', 'glColor4hvNV', 'glTexCoord1hNV',
'glTexCoord1hvNV', 'glTexCoord2hNV', 'glTexCoord2hvNV', 'glTexCoord3hNV',
'glTexCoord3hvNV', 'glTexCoord4hNV', 'glTexCoord4hvNV', 'glMultiTexCoord1hNV',
'glMultiTexCoord1hvNV', 'glMultiTexCoord2hNV', 'glMultiTexCoord2hvNV',
'glMultiTexCoord3hNV', 'glMultiTexCoord3hvNV', 'glMultiTexCoord4hNV',
'glMultiTexCoord4hvNV', 'glFogCoordhNV', 'glFogCoordhvNV',
'glSecondaryColor3hNV', 'glSecondaryColor3hvNV', 'glVertexWeighthNV',
'glVertexWeighthvNV', 'glVertexAttrib1hNV', 'glVertexAttrib1hvNV',
'glVertexAttrib2hNV', 'glVertexAttrib2hvNV', 'glVertexAttrib3hNV',
'glVertexAttrib3hvNV', 'glVertexAttrib4hNV', 'glVertexAttrib4hvNV',
'glVertexAttribs1hvNV', 'glVertexAttribs2hvNV', 'glVertexAttribs3hvNV',
'glVertexAttribs4hvNV', 'PFNGLVERTEX2HNVPROC', 'PFNGLVERTEX2HVNVPROC',
'PFNGLVERTEX3HNVPROC', 'PFNGLVERTEX3HVNVPROC', 'PFNGLVERTEX4HNVPROC',
'PFNGLVERTEX4HVNVPROC', 'PFNGLNORMAL3HNVPROC', 'PFNGLNORMAL3HVNVPROC',
'PFNGLCOLOR3HNVPROC', 'PFNGLCOLOR3HVNVPROC', 'PFNGLCOLOR4HNVPROC',
'PFNGLCOLOR4HVNVPROC', 'PFNGLTEXCOORD1HNVPROC', 'PFNGLTEXCOORD1HVNVPROC',
'PFNGLTEXCOORD2HNVPROC', 'PFNGLTEXCOORD2HVNVPROC', 'PFNGLTEXCOORD3HNVPROC',
'PFNGLTEXCOORD3HVNVPROC', 'PFNGLTEXCOORD4HNVPROC', 'PFNGLTEXCOORD4HVNVPROC',
'PFNGLMULTITEXCOORD1HNVPROC', 'PFNGLMULTITEXCOORD1HVNVPROC',
'PFNGLMULTITEXCOORD2HNVPROC', 'PFNGLMULTITEXCOORD2HVNVPROC',
'PFNGLMULTITEXCOORD3HNVPROC', 'PFNGLMULTITEXCOORD3HVNVPROC',
'PFNGLMULTITEXCOORD4HNVPROC', 'PFNGLMULTITEXCOORD4HVNVPROC',
'PFNGLFOGCOORDHNVPROC', 'PFNGLFOGCOORDHVNVPROC',
'PFNGLSECONDARYCOLOR3HNVPROC', 'PFNGLSECONDARYCOLOR3HVNVPROC',
'PFNGLVERTEXWEIGHTHNVPROC', 'PFNGLVERTEXWEIGHTHVNVPROC',
'PFNGLVERTEXATTRIB1HNVPROC', 'PFNGLVERTEXATTRIB1HVNVPROC',
'PFNGLVERTEXATTRIB2HNVPROC', 'PFNGLVERTEXATTRIB2HVNVPROC',
'PFNGLVERTEXATTRIB3HNVPROC', 'PFNGLVERTEXATTRIB3HVNVPROC',
'PFNGLVERTEXATTRIB4HNVPROC', 'PFNGLVERTEXATTRIB4HVNVPROC',
'PFNGLVERTEXATTRIBS1HVNVPROC', 'PFNGLVERTEXATTRIBS2HVNVPROC',
'PFNGLVERTEXATTRIBS3HVNVPROC', 'PFNGLVERTEXATTRIBS4HVNVPROC',
'GL_NV_pixel_data_range', 'glPixelDataRangeNV', 'glFlushPixelDataRangeNV',
'PFNGLPIXELDATARANGENVPROC', 'PFNGLFLUSHPIXELDATARANGENVPROC',
'GL_NV_primitive_restart', 'glPrimitiveRestartNV',
'glPrimitiveRestartIndexNV', 'PFNGLPRIMITIVERESTARTNVPROC',
'PFNGLPRIMITIVERESTARTINDEXNVPROC', 'GL_NV_texture_expand_normal',
'GL_NV_vertex_program2', 'GL_ATI_map_object_buffer', 'glMapObjectBufferATI',
'glUnmapObjectBufferATI', 'PFNGLMAPOBJECTBUFFERATIPROC',
'PFNGLUNMAPOBJECTBUFFERATIPROC', 'GL_ATI_separate_stencil',
'glStencilOpSeparateATI', 'glStencilFuncSeparateATI',
'PFNGLSTENCILOPSEPARATEATIPROC', 'PFNGLSTENCILFUNCSEPARATEATIPROC',
'GL_ATI_vertex_attrib_array_object', 'glVertexAttribArrayObjectATI',
'glGetVertexAttribArrayObjectfvATI', 'glGetVertexAttribArrayObjectivATI',
'PFNGLVERTEXATTRIBARRAYOBJECTATIPROC',
'PFNGLGETVERTEXATTRIBARRAYOBJECTFVATIPROC',
'PFNGLGETVERTEXATTRIBARRAYOBJECTIVATIPROC', 'GL_OES_read_format',
'GL_EXT_depth_bounds_test', 'glDepthBoundsEXT', 'PFNGLDEPTHBOUNDSEXTPROC',
'GL_EXT_texture_mirror_clamp', 'GL_EXT_blend_equation_separate',
'glBlendEquationSeparateEXT', 'PFNGLBLENDEQUATIONSEPARATEEXTPROC',
'GL_MESA_pack_invert', 'GL_MESA_ycbcr_texture', 'GL_EXT_pixel_buffer_object',
'GL_NV_fragment_program_option', 'GL_NV_fragment_program2',
'GL_NV_vertex_program2_option', 'GL_NV_vertex_program3',
'GL_EXT_framebuffer_object', 'glIsRenderbufferEXT', 'glBindRenderbufferEXT',
'glDeleteRenderbuffersEXT', 'glGenRenderbuffersEXT',
'glRenderbufferStorageEXT', 'glGetRenderbufferParameterivEXT',
'glIsFramebufferEXT', 'glBindFramebufferEXT', 'glDeleteFramebuffersEXT',
'glGenFramebuffersEXT', 'glCheckFramebufferStatusEXT',
'glFramebufferTexture1DEXT', 'glFramebufferTexture2DEXT',
'glFramebufferTexture3DEXT', 'glFramebufferRenderbufferEXT',
'glGetFramebufferAttachmentParameterivEXT', 'glGenerateMipmapEXT',
'PFNGLISRENDERBUFFEREXTPROC', 'PFNGLBINDRENDERBUFFEREXTPROC',
'PFNGLDELETERENDERBUFFERSEXTPROC', 'PFNGLGENRENDERBUFFERSEXTPROC',
'PFNGLRENDERBUFFERSTORAGEEXTPROC', 'PFNGLGETRENDERBUFFERPARAMETERIVEXTPROC',
'PFNGLISFRAMEBUFFEREXTPROC', 'PFNGLBINDFRAMEBUFFEREXTPROC',
'PFNGLDELETEFRAMEBUFFERSEXTPROC', 'PFNGLGENFRAMEBUFFERSEXTPROC',
'PFNGLCHECKFRAMEBUFFERSTATUSEXTPROC', 'PFNGLFRAMEBUFFERTEXTURE1DEXTPROC',
'PFNGLFRAMEBUFFERTEXTURE2DEXTPROC', 'PFNGLFRAMEBUFFERTEXTURE3DEXTPROC',
'PFNGLFRAMEBUFFERRENDERBUFFEREXTPROC',
'PFNGLGETFRAMEBUFFERATTACHMENTPARAMETERIVEXTPROC',
'PFNGLGENERATEMIPMAPEXTPROC', 'GL_GREMEDY_string_marker',
'glStringMarkerGREMEDY', 'PFNGLSTRINGMARKERGREMEDYPROC', 'GL_EXT_Cg_shader',
'GL_EXT_timer_query', 'glGetQueryObjecti64vEXT', 'glGetQueryObjectui64vEXT',
'PFNGLGETQUERYOBJECTI64VEXTPROC', 'PFNGLGETQUERYOBJECTUI64VEXTPROC',
'GL_EXT_texture_buffer_object', 'glTexBufferEXT', 'PFNGLTEXBUFFEREXTPROC',
'GL_NV_transform_feedback', 'glBeginTransformFeedbackNV',
'glEndTransformFeedbackNV', 'glTransformFeedbackAttribsNV',
'glBindBufferRangeNV', 'glBindBufferOffsetNV', 'glBindBufferBaseNV',
'glTransformFeedbackVaryingsNV', 'glActiveVaryingNV',
'glGetVaryingLocationNV', 'glGetActiveVaryingNV',
'glGetTransformFeedbackVaryingNV', 'PFNGLBEGINTRANSFORMFEEDBACKNVPROC',
'PFNGLENDTRANSFORMFEEDBACKNVPROC', 'PFNGLTRANSFORMFEEDBACKATTRIBSNVPROC',
'PFNGLBINDBUFFERRANGENVPROC', 'PFNGLBINDBUFFEROFFSETNVPROC',
'PFNGLBINDBUFFERBASENVPROC', 'PFNGLTRANSFORMFEEDBACKVARYINGSNVPROC',
'PFNGLACTIVEVARYINGNVPROC', 'PFNGLGETVARYINGLOCATIONNVPROC',
'PFNGLGETACTIVEVARYINGNVPROC', 'PFNGLGETTRANSFORMFEEDBACKVARYINGNVPROC',
'GL_NV_depth_buffer_float', 'glDepthRangedNV', 'glClearDepthdNV',
'glDepthBoundsdNV', 'PFNGLDEPTHRANGEDNVPROC', 'PFNGLCLEARDEPTHDNVPROC',
'PFNGLDEPTHBOUNDSDNVPROC', 'GL_EXT_texture_compression_latc',
'GL_EXT_framebuffer_sRGB', 'GL_EXT_texture_shared_exponent',
'GL_EXT_packed_float', 'GL_EXT_texture_array', 'GL_EXT_draw_buffers2',
'glColorMaskIndexedEXT', 'glGetBooleanIndexedvEXT', 'glGetIntegerIndexedvEXT',
'glEnableIndexedEXT', 'glDisableIndexedEXT', 'glIsEnabledIndexedEXT',
'PFNGLCOLORMASKINDEXEDEXTPROC', 'PFNGLGETBOOLEANINDEXEDVEXTPROC',
'PFNGLGETINTEGERINDEXEDVEXTPROC', 'PFNGLENABLEINDEXEDEXTPROC',
'PFNGLDISABLEINDEXEDEXTPROC', 'PFNGLISENABLEDINDEXEDEXTPROC',
'GL_EXT_texture_integer', 'glTexParameterIivEXT', 'glTexParameterIuivEXT',
'glGetTexParameterIivEXT', 'glGetTexParameterIuivEXT', 'glClearColorIiEXT',
'glClearColorIuiEXT', 'PFNGLTEXPARAMETERIIVEXTPROC',
'PFNGLTEXPARAMETERIUIVEXTPROC', 'PFNGLGETTEXPARAMETERIIVEXTPROC',
'PFNGLGETTEXPARAMETERIUIVEXTPROC', 'PFNGLCLEARCOLORIIEXTPROC',
'PFNGLCLEARCOLORIUIEXTPROC', 'GL_EXT_bindable_uniform', 'glUniformBufferEXT',
'glGetUniformBufferSizeEXT', 'glGetUniformOffsetEXT',
'PFNGLUNIFORMBUFFEREXTPROC', 'PFNGLGETUNIFORMBUFFERSIZEEXTPROC',
'PFNGLGETUNIFORMOFFSETEXTPROC', 'GL_EXT_gpu_shader4', 'glGetUniformuivEXT',
'glBindFragDataLocationEXT', 'glGetFragDataLocationEXT', 'glUniform1uiEXT',
'glUniform2uiEXT', 'glUniform3uiEXT', 'glUniform4uiEXT', 'glUniform1uivEXT',
'glUniform2uivEXT', 'glUniform3uivEXT', 'glUniform4uivEXT',
'glVertexAttribI1iEXT', 'glVertexAttribI2iEXT', 'glVertexAttribI3iEXT',
'glVertexAttribI4iEXT', 'glVertexAttribI1uiEXT', 'glVertexAttribI2uiEXT',
'glVertexAttribI3uiEXT', 'glVertexAttribI4uiEXT', 'glVertexAttribI1ivEXT',
'glVertexAttribI2ivEXT', 'glVertexAttribI3ivEXT', 'glVertexAttribI4ivEXT',
'glVertexAttribI1uivEXT', 'glVertexAttribI2uivEXT', 'glVertexAttribI3uivEXT',
'glVertexAttribI4uivEXT', 'glVertexAttribI4bvEXT', 'glVertexAttribI4svEXT',
'glVertexAttribI4ubvEXT', 'glVertexAttribI4usvEXT',
'glVertexAttribIPointerEXT', 'glGetVertexAttribIivEXT',
'glGetVertexAttribIuivEXT', 'PFNGLGETUNIFORMUIVEXTPROC',
'PFNGLBINDFRAGDATALOCATIONEXTPROC', 'PFNGLGETFRAGDATALOCATIONEXTPROC',
'PFNGLUNIFORM1UIEXTPROC', 'PFNGLUNIFORM2UIEXTPROC', 'PFNGLUNIFORM3UIEXTPROC',
'PFNGLUNIFORM4UIEXTPROC', 'PFNGLUNIFORM1UIVEXTPROC',
'PFNGLUNIFORM2UIVEXTPROC', 'PFNGLUNIFORM3UIVEXTPROC',
'PFNGLUNIFORM4UIVEXTPROC', 'PFNGLVERTEXATTRIBI1IEXTPROC',
'PFNGLVERTEXATTRIBI2IEXTPROC', 'PFNGLVERTEXATTRIBI3IEXTPROC',
'PFNGLVERTEXATTRIBI4IEXTPROC', 'PFNGLVERTEXATTRIBI1UIEXTPROC',
'PFNGLVERTEXATTRIBI2UIEXTPROC', 'PFNGLVERTEXATTRIBI3UIEXTPROC',
'PFNGLVERTEXATTRIBI4UIEXTPROC', 'PFNGLVERTEXATTRIBI1IVEXTPROC',
'PFNGLVERTEXATTRIBI2IVEXTPROC', 'PFNGLVERTEXATTRIBI3IVEXTPROC',
'PFNGLVERTEXATTRIBI4IVEXTPROC', 'PFNGLVERTEXATTRIBI1UIVEXTPROC',
'PFNGLVERTEXATTRIBI2UIVEXTPROC', 'PFNGLVERTEXATTRIBI3UIVEXTPROC',
'PFNGLVERTEXATTRIBI4UIVEXTPROC', 'PFNGLVERTEXATTRIBI4BVEXTPROC',
'PFNGLVERTEXATTRIBI4SVEXTPROC', 'PFNGLVERTEXATTRIBI4UBVEXTPROC',
'PFNGLVERTEXATTRIBI4USVEXTPROC', 'PFNGLVERTEXATTRIBIPOINTEREXTPROC',
'PFNGLGETVERTEXATTRIBIIVEXTPROC', 'PFNGLGETVERTEXATTRIBIUIVEXTPROC',
'GL_EXT_geometry_shader4', 'glProgramParameteriEXT',
'glFramebufferTextureEXT', 'glFramebufferTextureLayerEXT',
'glFramebufferTextureFaceEXT', 'PFNGLPROGRAMPARAMETERIEXTPROC',
'PFNGLFRAMEBUFFERTEXTUREEXTPROC', 'PFNGLFRAMEBUFFERTEXTURELAYEREXTPROC',
'PFNGLFRAMEBUFFERTEXTUREFACEEXTPROC', 'GL_NV_geometry_program4',
'glProgramVertexLimitNV', 'PFNGLPROGRAMVERTEXLIMITNVPROC',
'GL_NV_gpu_program4', 'glProgramLocalParameterI4iNV',
'glProgramLocalParameterI4ivNV', 'glProgramLocalParametersI4ivNV',
'glProgramLocalParameterI4uiNV', 'glProgramLocalParameterI4uivNV',
'glProgramLocalParametersI4uivNV', 'glProgramEnvParameterI4iNV',
'glProgramEnvParameterI4ivNV', 'glProgramEnvParametersI4ivNV',
'glProgramEnvParameterI4uiNV', 'glProgramEnvParameterI4uivNV',
'glProgramEnvParametersI4uivNV', 'glGetProgramLocalParameterIivNV',
'glGetProgramLocalParameterIuivNV', 'glGetProgramEnvParameterIivNV',
'glGetProgramEnvParameterIuivNV', 'PFNGLPROGRAMLOCALPARAMETERI4INVPROC',
'PFNGLPROGRAMLOCALPARAMETERI4IVNVPROC',
'PFNGLPROGRAMLOCALPARAMETERSI4IVNVPROC',
'PFNGLPROGRAMLOCALPARAMETERI4UINVPROC',
'PFNGLPROGRAMLOCALPARAMETERI4UIVNVPROC',
'PFNGLPROGRAMLOCALPARAMETERSI4UIVNVPROC', 'PFNGLPROGRAMENVPARAMETERI4INVPROC',
'PFNGLPROGRAMENVPARAMETERI4IVNVPROC', 'PFNGLPROGRAMENVPARAMETERSI4IVNVPROC',
'PFNGLPROGRAMENVPARAMETERI4UINVPROC', 'PFNGLPROGRAMENVPARAMETERI4UIVNVPROC',
'PFNGLPROGRAMENVPARAMETERSI4UIVNVPROC',
'PFNGLGETPROGRAMLOCALPARAMETERIIVNVPROC',
'PFNGLGETPROGRAMLOCALPARAMETERIUIVNVPROC',
'PFNGLGETPROGRAMENVPARAMETERIIVNVPROC',
'PFNGLGETPROGRAMENVPARAMETERIUIVNVPROC', 'GL_NV_parameter_buffer_object',
'glProgramBufferParametersfvNV', 'glProgramBufferParametersIivNV',
'glProgramBufferParametersIuivNV', 'PFNGLPROGRAMBUFFERPARAMETERSFVNVPROC',
'PFNGLPROGRAMBUFFERPARAMETERSIIVNVPROC',
'PFNGLPROGRAMBUFFERPARAMETERSIUIVNVPROC', 'GL_EXT_framebuffer_multisample',
'glRenderbufferStorageMultisampleEXT',
'PFNGLRENDERBUFFERSTORAGEMULTISAMPLEEXTPROC',
'GL_NV_framebuffer_multisample_coverage',
'glRenderbufferStorageMultisampleCoverageNV',
'PFNGLRENDERBUFFERSTORAGEMULTISAMPLECOVERAGENVPROC',
'GL_EXT_framebuffer_blit', 'glBlitFramebufferEXT',
'PFNGLBLITFRAMEBUFFEREXTPROC', 'GL_EXT_draw_instanced',
'glDrawArraysInstancedEXT', 'glDrawElementsInstancedEXT',
'PFNGLDRAWARRAYSINSTANCEDEXTPROC', 'PFNGLDRAWELEMENTSINSTANCEDEXTPROC',
'GL_EXT_texture_compression_rgtc']
# END GENERATED CONTENT (do not edit above this line)



